#include <dce/pthread.h>
#include <iostream.hxx>
#include <time.h>
#include "daerpc.h"
#include "types.hxx"
#include "daepacket_edescript.hxx"
#include "daerequest.hxx"
#include "dae1srequest.hxx"
#include "daepacketheader.hxx"
#include "daepacket.hxx"
#include <dce/dce_error.h>
#include "useful_macros.hxx"
#include "misc.hxx"
#include "daepacket_debug.hxx"

DaePacket::DaePacket(DaeRequest* a_request)
{
    request = a_request;
    header.setPacketSize(request->getSendSize());
    if (debug_on(DEBUG_G_PACKET_BUILD)) { cdebug << "Formed packet, size = " << getPacketSize() << " words\n"; }
}
    
Status DaePacket::setContent(DaeRequest* a_request)
{
    request = a_request;
    header.setPacketSize(request->getSendSize());
    if (debug_on(DEBUG_G_PACKET_BUILD)) { cdebug << "Formed packet, size = " << getPacketSize() << " words\n"; }
    return Status();
}

Status DaePacket::transfer(rpc_binding_handle_t& daeBinding_h, dae_context_handle& daeContext_h)
{
    Status status;
    struct timespec rebind_time = {60, 0};
    idl_long_int packet_size = getPacketSize();
    int total_size = request->getTotalSize() + header.getHeaderSize();
    int retry_count(0), i;
    reply_t err_msg(0);
    error_status_t rpc_status;
    daeTransfer_t *transfer_buffer = new daeTransfer_t[total_size];
    NEW_CHECK(transfer_buffer);
    status = header.insert(transfer_buffer, 0);
    RETURN_IF_ERROR(status);
    status = request->insert(transfer_buffer, header.getHeaderSize());
    RETURN_IF_ERROR(status);
    while(retry_count++ < 5)
    {
    	rpc_status = rpc_s_ok;
	if (debug_on(DEBUG_G_PACKET_SEND)) 
	{ 
	    cdebug << "Sending packet size " << packet_size*sizeof(idl_long_int) << " bytes, attempt " << retry_count << "\n";
	}
    	err_msg = daeDispatchRequest(daeBinding_h, daeContext_h, total_size, &packet_size, transfer_buffer, &rpc_status);
	if (rpc_status == rpc_s_ok)
	    break;
	PRINT_RPC_ERROR(cerr, rpc_status, "DaeDispatchRequest");
	cerr << "*** Looks like the DAE Server process has restarted: hang on, I shall attempt to compensate ..." << endl;
// Now wait until we have valid binding and context handles
        while(rebind_handle(daeBinding_h, daeContext_h, daerpcif_v1_0_c_ifspec) == FALSE)
	{
	    pthread_delay_np(&rebind_time);
        }
	cerr << ">>> New DAE Server process successfully located <<<" << endl;
    }
    RETURN_RPC_STATUS(rpc_status, "daeDispatchRequest"); 
    if (strcmp((char*)err_msg, "All OK")) 
    {
	status.Append(RPC_ERROR, FCODE, "daeDispatchRequest", (char*)err_msg); 
    }
    free(err_msg);
    err_msg = 0;
    if (debug_on(DEBUG_G_PACKET_SEND)) 
    { 
	cdebug << "Received packet size " << packet_size*sizeof(idl_long_int) << " bytes\n";
    }
    if (!status.ok()) { delete []transfer_buffer; return status; }
    status = request->retrieve(transfer_buffer, 0);
    delete []transfer_buffer;
    return status;
}

int DaePacket::getPacketSize() const
{
    return header.getPacketSize();
}

void DaePacket::print(ostream& os)
{
     os << "packet\n";
}

ostream& operator<<(ostream&os, DaePacket& packet)
{
    packet.print(os);
    return os;
}
