#include <dce/pthread.h>
#include <stdio.h>
#include <iostream.hxx>
#include <string.hxx>
#include <stdarg.h>
#include "daerpc.h"
#include "types.hxx"
#include "useful_macros.hxx"
#include "misc.hxx"
#ifdef __VMS
#include <clidef.h>
#include <descrip.h>
#include <ssdef.h>
#include <lib$routines.h>
#endif

extern "C" {
#include <dce/dce_cf.h>
}
#include <dce/dce_error.h>

void print_buffer(daeTransfer_t *buffer)
{
    int i;
    cerr << "Transfer buffer returned\n";
    for(i=0; i<8; i++)
    {
        cerr << i << "=" << (unsigned)buffer[i] << "\n";
    }
    for(i=16; i<18; i++)
    {
        cerr << i << "=" << *((unsigned short*)buffer+i) << "\n";
    }
    return;
}

const char* get_hostname_c(char host_case)
{
    String* temp = new String;
    *temp = get_hostname(host_case);
    return *temp;
}


int appendToStream(ostream& stream, int nitems,  ... )
{
    int i;
    char* item;
    va_list item_list;
    va_start(item_list, nitems);
    for(int j=0; j<nitems; j++)
    {
        item = va_arg(item_list, char*);
	for(i=0; i<nitems; i++)
        {
 	    stream.put(item[i]);
        }
    } 
    va_end(item_list);
    return 0;
}
    
String get_hostname(char host_case)
{
//    unsigned long status;
    unsigned int status;
    char* name;

    dce_cf_get_host_name(&name, &status);
    if (status != dce_cf_st_ok)
    {
	cerr << "Unable to determine hostname\n";
	return String("unknown");
    }
    String host(name+6);		// skip over "hosts/" part
    free(name);
    switch(host_case)
    {
	case 'l':
	case 'L':
	    return host.lower();
	    break;
	case 'u':
	case 'U':
	    return host.upper();
	    break;
	default:
	    break;
    }
    return host;
}

ostream& operator<<(ostream& os, enum RequestType type)
{
    switch(type)
    {
	case DAE1S:
	    os << "DAE1S";
	    break;

	default:
	    os << "invalid request code (" << (int)type << ")";
	    break;
    }
    return os;
}

/*
 * returns 1 if successfully rebinds to a listening server, 0 otherwise
 * called if server died, but client still up
 */

// ch is dae_context_handle
boolean32 rebind_handle(rpc_binding_handle_t& bh, void*& ch, const rpc_if_handle_t& ih)
{
    unsigned32 status;

#ifdef DEBUG
    cerr << "*** Rebinding handle and re-establishing server context ***\n";
#endif	/* DEBUG */

    if (ch != NULL) 
    {
	rpc_ss_destroy_client_context(&ch);	// destroys it and sets it to NULL
    }
    rpc_binding_reset(bh, &status);
    PRINT_RPC_ERROR(cerr, status, "rebind_handle - cannot reset binding handle"); 
    rpc_ep_resolve_binding(bh, ih, &status);
#ifdef DEBUG
    PRINT_RPC_ERROR(cerr, status, "rebind_handle - cannot resolve endpoint"); 
#endif	/* DEBUG */
    boolean32 temp = rpc_mgmt_is_server_listening(bh, &status);
#ifdef DEBUG
    PRINT_RPC_ERROR(cerr, status, "rebind_handle - cannot check if server listening"); 
#endif	/* DEBUG */
    if (temp == TRUE)
    {
	ch = open_dae_connection(bh, &status);
	PRINT_RPC_ERROR(cerr, status, "rebind_handle - error opening new context connection");
        if ((ch == NULL) || (status != rpc_s_ok))
	{ 
	    cerr << "rebind_handle: cannot obtain new context handle" << endl;
	    temp = FALSE;
	    ch = NULL;
	}
    }
    return temp;
}

void decode_binding_handle(ostream& os, const unsigned_char_p_t string_binding)
{
    unsigned32 status;  
    unsigned_char_p_t obj_uuid, protseq, network_addr, endpoint, network_options, entry_name;

    rpc_string_binding_parse(string_binding, &obj_uuid, &protseq, 
			     &network_addr, &endpoint, &network_options, &status); 
    if (status != rpc_s_ok)
    {
	PRINT_RPC_ERROR(os, status, "Cannot decode binding handle"); 
	return;
    }
    os << "*** Description of Binding Handle ***\n";

#define DO(__descrip, __value)	\
    if (__value != 0) \
    { \
	os << __descrip << (char*)__value << "\n"; \
	rpc_string_free(&__value, &status); \
    } \
    else \
    { \
	os << __descrip << "<none>\n"; \
    }

    DO("Object UUID: ", obj_uuid);
    DO("Protocol sequence: ", protseq);
// Convert ascii string a.b.c.d into a real hostname
//    unsigned long address = inet_addr((char*)network_addr);
//    struct hostent* host = gethostbyaddr((unsigned char*)&address, 4, AF_INET);
//    host == 0 ? "not resolvable" : host->h_name
//    rpc_ns_binding_inq_entry_name(bh, rpc_c_ns_syntax_default, &entry_name, &status);
//    PRINT_RPC_ERROR(os, status, "Cannot resolve name in CDS");
//    if (status == rpc_s_ok) { DO("Entry name:", entry_name); }
    DO("Network address: ", network_addr);
    DO("Endpoint: ", endpoint);
    DO("Network options: ", network_options);
#undef DO
    return;
}

// Specify bh=NULL for current server: must give a fully bound handle
// by e.g. calling rpc_ep_resolve_binding() first
unsigned32 get_rpc_statistics(ostream& os, const rpc_binding_handle_t& bh)
{
    unsigned32 rpc_status, status;
    rpc_stats_vector_t* stats_vector;

    os <<  "*** RPC Statistics ***\n";
    rpc_mgmt_inq_stats(bh, &stats_vector, &rpc_status);
    if (rpc_status == rpc_s_ok)
    {
#define DO(__descrip, __value)	os << __descrip << stats_vector->stats[__value] << "\n"
    	DO("Number of RPC calls received = ", rpc_c_stats_calls_in);
    	DO("Number of RPC calls initiated = ", rpc_c_stats_calls_out);
    	DO("Number of network packets received = ", rpc_c_stats_pkts_in);
    	DO("Number of network packets sent = ", rpc_c_stats_pkts_out);
#undef DO
    	rpc_mgmt_stats_vector_free(&stats_vector, &status);
    }
    else
    {
	PRINT_RPC_ERROR(os, rpc_status, "Cannot obtain RPC statistics");
    }
    return rpc_status;
}

/*
 *  compare two binding handles, return 1 if same & 0 otherwise
 */
int compare_bindings(const rpc_binding_handle_t& one, const rpc_binding_handle_t& two)
{
    unsigned32 status;
    unsigned_char_p_t string_binding_one, string_binding_two;

    rpc_binding_to_string_binding(one, &string_binding_one, &status);    
    PRINT_RPC_ERROR(cerr, status, "compare bindings");
    rpc_binding_to_string_binding(two, &string_binding_two, &status);
    PRINT_RPC_ERROR(cerr, status, "compare bindings");
    int test = strcmp((char*)string_binding_one, (char*)string_binding_two);
    rpc_string_free(&string_binding_one, &status);
    PRINT_RPC_ERROR(cerr, status, "compare bindings");
    rpc_string_free(&string_binding_two, &status);
    PRINT_RPC_ERROR(cerr, status, "compare bindings");
    return(!test);
}

ostream& operator<<(ostream& os, const struct dae_context* dc)
{
    os << (dc->handle != 0 ? (char*)dc->handle : "<unknown>");
    return os;
}

#ifdef __VMS
/*
 * broadcast a message to all user son the node ... 
 * image or user must have OPER priviledge
 *
 * returns 0 on success, 1 on failure
 */

#define FAA$DESCRIPTOR(__name, __string)    \
    struct dsc$descriptor_s __name = { strlen(__string), DSC$K_DTYPE_T, DSC$K_CLASS_S, __string }


int broadcast_message(const String& message)
{
    String command = String("REPLY/NODE/BELL/ALL \"") + message + String("\"");
    FAA$DESCRIPTOR(command_string, (char*)command);
    int flags = CLI$M_AUTHPRIV|CLI$M_TRUSTED;
    int status = lib$spawn(&command_string, 0, 0, &flags, 
			   0, 0, 0, 0, 0, 0, 0, 0, 0);
    return (status == SS$_NORMAL ? 0 : 1);
}

int wait_a_bit(int secs)
{
    struct timespec delay = { secs, 0 };
    return pthread_delay_np(&delay);
}

//#define CHECK(__status)		if (__status != cma_s_dbg_ok)
void thread_info(char* descriptions[], void* handles[], int nhandles)
{
    char command[80];
// See if we are in DEC Thread METERED mode ... we can get some extra info then
    const char* cma_meter = getenv("CMA_METER");    
    int metered = ((cma_meter != NULL) && (strcmp(cma_meter, "1") == 0));

    cout << "Interpretation of thread handles (" << (metered ? "" : "non-") << "metered mode)" << endl;
    for(int i=0; i<nhandles; i++)
    {
	cout << "&" << descriptions[i] << " = " << handles[i] << endl;
	sprintf(command, "handle 0x%p", handles[i]);
	cma_debug_cmd(command);
    }

    cout << "*** Currently running threads" << endl;
    cma_debug_cmd("thread -cf");
    cout << "*** Blocked threads" << endl;
    cma_debug_cmd("thread -b");
    cout << "*** Detached threads" << endl;
    cma_debug_cmd("thread -d");
    cout << "*** Terminated threads" << endl;
    cma_debug_cmd("thread -t");

    cout << "*** Locked mutexes" << endl;
    cma_debug_cmd("mutex -lf");
// 
    cout << "*** Threads waiting to lock a mutex" << endl;
    cma_debug_cmd("mutex -qf");
    cout << "*** Condition variables with threads waiting" << endl;
    cma_debug_cmd("condition -wfq");
    cout << "***" << endl;
    return;
}

#endif
