/*
 *  This file contains the RIO interface header file
 *  that is callable from FORTRAN
 *
 * it hopes to simulate the good old RIO routines as part of DAE 1S
 *
 *  Author:   Freddie Akeroyd
 */


/*
 * Unix fortran compilers append an "_" to Fortran subroutine names
 *
 * OPEN_DAE is special in that it passes a string and so requires
 * a jacket FORTRAN routine
 */
#ifndef RIO_INTERFACE_HXX
#define RIO_INTERFACE_HXX

#ifdef __unix
#define OPEN_DAE open_dae__
#define CLOSE_DAE close_dae_
#define RESET_DAE reset_dae_
#define RIO rio_
#define RIOD riod_
#define RMEM rmem_
#define RMEMD rmemd_
#define WIO wio_
#define WIOD wiod_
#define WMEM wmem_
#define WMEMD wmemd_
#define RMEMVI rmemvi_
#define READ_CRPT read_crpt_
#else
#define OPEN_DAE open_dae_
#define CLOSE_DAE close_dae
#define RESET_DAE reset_dae
#define RIO rio
#define RIOD riod
#define RMEM rmem
#define RMEMD rmemd
#define WIO wio
#define WIOD wiod
#define WMEM wmem
#define WMEMD wmemd
#define RMEMVI rmemvi
#define READ_CRPT read_crpt
#endif	/* ifdef unix */

/*
 * bus_addr 	the multibus address
 * word		the transferred data word (16 bit)
 * device	a VMS channel number in DAE-I (ignored here, just for compatibility)
 * nwrds	the number of 16 bit words to transfer
 * wrd_length	word length of DAE memory. Could be 8, 16 or 32 bit
		for DAE-I, but only 32 bit allowed for DAE-II
 */

extern "C"
{
    fort_i4 OPEN_DAE(STRING& dae_name);
    fort_i4 open_dae_c(const char* dae_name);
    fort_i4 CLOSE_DAE();
    fort_i4 RESET_DAE(fort_i4& nn);
    fort_i4 RIO(fort_i4& bus_addr, fort_i2& word, fort_i2& device);
    fort_i4 RIOD(fort_i4& bus_addr, fort_i2* buffer, fort_i4& nwrds, fort_i2& device);
    fort_i4 RMEM(fort_i4& bus_addr, fort_i2& word, fort_i2& device);
    fort_i4 RMEMD(fort_i4& bus_addr, fort_i2* buffer, fort_i4& nwrds, fort_i2& device);
    fort_i4 WIO(fort_i4& bus_addr, fort_i2& word, fort_i2& device);
    fort_i4 WIOD(fort_i4& bus_addr, fort_i2* buffer, fort_i4& nwrds, fort_i2& device);
    fort_i4 WMEM(fort_i4& bus_addr, fort_i2& word, fort_i2& device);
    fort_i4 WMEMD(fort_i4& bus_addr, fort_i2* buffer, fort_i4& nwrds, fort_i2& device);
    fort_i4 RMEMVI(fort_i4& bus_addr, fort_i4* buffer, fort_i4& length, fort_i2& device, fort_i4& wrd_len);
    fort_i4 READ_CRPT(fort_i4*& start, fort_i4& crpt_section, fort_i4& size_in_words);
}
#endif	/* RIO_INTERFACE_HXX */
