//  Module:	SET.HXX
//  
//  Function:	Simple set template class.
//		
//		
//  Author:	R.Brodie 8/6/94
//
//
// Only a minimal subset of operations is implemented - adding more shouldn't be too hard.
// We use the SPLASH based array/list/queue to store the data, and let it worry about resizing & lazy copying.
// Elements must be of an integral type (typically an enumerated type).

#include "splash.h"
template <class ElementType> class Set
    {
    public:	Set(void);							// Create null set
		Set(const ElementType&);				
		Set<ElementType>  operator+=(const ElementType&);		// Add single member to set
		Set<ElementType> operator+=(const Set<ElementType>&);		// Compute union (+= form).
		Set<ElementType>  operator-=(const ElementType&);		// Rmove single member from set
		Set<ElementType> operator-=(const Set<ElementType>&);		// Compute (A & !B)        
		int is_member(const ElementType&) const;	    		// Membership test
		int nelements(void) const;					// Find maximum number of members
		
    private:	SPListZero<int> member_array;
    };
    
template<class ElementType>
Set<ElementType>::Set(void)
    {
    }

template<class ElementType>
Set<ElementType>::Set(const ElementType& e)
    {
    member_array[(int) e] = 1;
    }

template<class ElementType>
Set<ElementType> Set<ElementType>::operator+=(const ElementType& e)
    {
    member_array[(int) e] = 1;
    return(*this);
    }

template<class ElementType>
Set<ElementType> Set<ElementType>::operator+=(const Set<ElementType>& set_b)    
    {
    int larger_count = nelements() > set_b.nelements() ? nelements() : set_b.nelements();
    for (int i=0; i < larger_count; i++)
	member_array[i] |= set_b.member_array[i];
    return(*this);
    }

template<class ElementType>
Set<ElementType> Set<ElementType>::operator-=(const ElementType& e)
    {
    member_array[(int) e] = 0;
    return(*this);
    }

template<class ElementType>
Set<ElementType> Set<ElementType>::operator-=(const Set<ElementType>& set_b)    
    {
    int larger_count = nelements() > set_b.nelements() ? nelements() : set_b.nelements();
    for (int i=0; i < larger_count; i++)
	member_array[i] &= !set_b.member_array[i];
    return(*this);
    }

template<class ElementType>
int Set<ElementType>::is_member(const ElementType& e) const
    {
    return(member_array[(int) e]);
    }

template<class ElementType>
int Set<ElementType>::nelements(void) const
    {
    return(member_array.count());
    }

// Need an operator< to make an SPList of sets
template<class ElementType>
inline int operator<(const Set<ElementType>&a, const Set<ElementType>& b)
{
    return a.nelements() < b.nelements();
}
