//  Module:	STATUS.HXX
//  
//  Function:	Class definitions for status objects.
//		These are used to keep error context information.
//
//  Author:	R.Brodie 28/9/93.

#ifndef	    ICE_STATUS_LOADED
#define	    ICE_STATUS_LOADED 1

#define return_status(x) return(Status(x,FCODE))
#define return_VMS_status(x) return((VMSErrorObject)(x))
#define return_POSIX_status return((POSIXErrorObject)(errno))
#define return_DCE_status(x) return((DCEErrorObject)(x))  
#define return_ok return(Status())

#include <string.hxx>
#include <iostream.h>

enum error_sc_type
{
WARNING  =	0,				// These are not in ascending order. 
SUCCESS =	1,				// This is to be consistent with the definitions in <STSDEF.H> on VMS 
ERROR =		2,                              // It won't cause any portability problems, as the access function for
INFO =		3,                              // the severity code is private to STATUS. If there really is a need to
SEVERE,FATAL =	4                               // have if error.severity() < WARNING, then make severity() return a class 
};                                              // object with an operator<(error_sc_type) such that INFO<ERROR etc.


class Status;		    // Forward declaration, so that Status can friends with AbstractErrorObject.

class AbstractErrorObject
    {
    friend class Status;
    friend ostream& operator<<(ostream& stream, const AbstractErrorObject&eobj);

    protected:	int ok(void) const;     			    //  Used by Status.
		int success(void) const;                            //
                virtual int Errlevel(void) const =0;		    //  Returns sevcode, or equivalent.
    		AbstractErrorObject(int icode_i);		    // Don't allow error objects except
		virtual ~AbstractErrorObject(void);	    	    // those created as part of a list.
								     
		virtual void print(ostream& stream) const = 0;	    // Helper function for <<, see Lippman p.460.
    

    protected:	int icode;
    };
    
class IceErrorObject : public AbstractErrorObject
    {
    friend ostream& operator<<(ostream& stream, const AbstractErrorObject& eobj);
    
    public:   	IceErrorObject(int icode_i, int fcode_i, String p1_i="", String p2_i="");   // Don't allow error objects except
		~IceErrorObject(void);							    // those created as part of a list.
    		int Errlevel(void) const;		    				  
    
    protected:	void print(ostream& stream) const;	

    private:	int fcode;
		String p1;
		String p2;
    };

typedef AbstractErrorObject* ErrorPointer;			
							
#include "splash.h"

typedef SPList<ErrorPointer> ErrorPointerList;

class ErrorPointerListCounted : public ErrorPointerList
    {
    public:	int nrefs;					// A reference count.
    };

class VMSErrorObject;
class POSIXErrorObject;
class DCEErrorObject;
class IDNAErrorObject;

class Status
    {
    friend  ostream& operator<< (ostream&, const Status&);

    public:	Status(void);
		Status(int icode_i, int fcode_i, String p1_i="", String p2_i="");	
		Status(int icode_i, int fcode_i, String p1_i   , int    p2_i   );   
		Status(int icode_i, int fcode_i, int    p1_i   , String p2_i="");   
		Status(int icode_i, int fcode_i, int    p1_i   , int    p2_i   );   
		Status::Status(const Status& wrs_in);
		Status::Status(const VMSErrorObject& eobj_in);
		Status::Status(const POSIXErrorObject& eobj_in);
		Status::Status(const DCEErrorObject& eobj_in);
		Status::Status(const IDNAErrorObject& eobj_in);
		Status::operator=(const Status& s_in); 
		~Status(void);							

		Append(int icode_i, int fcode_i, String p1_i="", String p2_i="");	
		Report(void);					
		Dismiss(void);                                  // Report, Dismiss and Signal clear the error condition, 
		Signal(void);                                   // by disposing of all the Error objects in the list.
		ok(void) const; 
		success(void) const;
		Errlevel(void) const;

    private:	Reduce_Refcount(void);				// Helper for destructor and assignment operator.

    private:    ErrorPointerListCounted* elist_ptr;		// List of error objects.
    };


#endif	//  ICE_STATUS_LOADED
