/***********************************************************************
 *
 *	Dictionary Support Module Definitions.
 *
 *	$Revision: 1.4 $
 *	$Date: 1996/12/09 18:07:02 $
 *	$Author: faa $
 *
 ***********************************************************************/

/***********************************************************************
 *
 * Copyright (C) 1990, 1991, 1992, 1994, 1995 Free Software Foundation, Inc.
 * Written by Steve Byrne.
 *
 * This file is part of GNU Smalltalk.
 *
 * GNU Smalltalk is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the Free
 * Software Foundation; either version 1, or (at your option) any later 
 * version.
 * 
 * GNU Smalltalk is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or 
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
 * more details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * GNU Smalltalk; see the file COPYING.  If not, write to the Free Software
 * Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.  
 *
 ***********************************************************************/


/*
 *    Change Log
 * ============================================================================
 * Author      Date       Change 
 * sbb	     23 Jun 95	  switched to GST guard prefix
 *
 * sbb	     30 May 95	  Replaced objectClass with mst_objectClass to avoid
 *			  conflicts with X.  Boolean => mst_Boolean. Object =>
 *			  mst_Object.
 *
 * sbb	     20 Jan 95	  Added 64 bit support.
 *
 * sbb	      2 Jan 95	  Switched to the new CType model.
 *
 * sbb	     21 Aug 94	  switched to low order bit for int flagging.
 *
 * sbyrne    13 Jan 89	  Created.
 *
 */


#ifndef __GSTDICT__
#define __GSTDICT__

/* ### YUK YUK YUK -- I hate to do this, but ... I need access to this
   structure for the binary save/restore code and I'd rather not have to
   write a bunch of accessors just to get on the air.  Fix this asap. */

/*
   3                   2                   1 
 1 0 9 8 7 6 5 4 3 2 1 0 9 8 7 6 5 4 3 2 1 0 9 8 7 6 5 4 3 2 1 0
+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
|    # fixed fields                                   |p|w|F|f|1|
+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
*/

typedef struct InstanceSpecStruct {
#ifdef WORDS_BIGENDIAN
#ifdef LONG_PAD_BIT_FIELDS
  unsigned	dummy			: 32; /* unused */
#endif
  unsigned	numFixedFields		: 27;
  unsigned	isPointers 		: 1;
  unsigned	isWords			: 1;
  unsigned	isIndexable		: 1;
  unsigned	hasFinalize		: 1;
  unsigned	intMark			: 1; /* must be low order bit */
#else
  unsigned	intMark			: 1; /* must be low order bit */
  unsigned	hasFinalize		: 1;
  unsigned	isIndexable		: 1;
  unsigned	isWords			: 1;
  unsigned	isPointers 		: 1;
  unsigned	numFixedFields		: 27;
#ifdef LONG_PAD_BIT_FIELDS
  unsigned	dummy			: 32; /* unused */
#endif
#endif
} InstanceSpec;

/* Note the use of structural inheritance in C structure definitions here */
#define BEHAVIOR_HEADER \
  OBJ_HEADER; \
  OOP		superClass; \
  OOP		subClasses; \
  OOP		methodDictionary; \
  InstanceSpec	instanceSpec

typedef struct BehaviorStruct {
  BEHAVIOR_HEADER;
} *Behavior;

#define CLASS_DESCRIPTION_HEADER \
  BEHAVIOR_HEADER; \
  OOP		name; \
  OOP		comment; \
  OOP		instanceVariables; \
  OOP		category

#define cObjectAnonType		fromInt(-1)
#define cObjectCharType		fromInt(0)
#define cObjectUnsignedCharType	fromInt(1)
#define cObjectShortType	fromInt(2)
#define cObjectUnsignedShortType fromInt(3)
#define cObjectLongType		fromInt(4)
#define cObjectUnsignedLongType	fromInt(5)
#define cObjectFloatType	fromInt(6)
#define cObjectDoubleType	fromInt(7)
#define cObjectStringType	fromInt(8)


typedef struct ClassDescriptionStruct {
  CLASS_DESCRIPTION_HEADER;
} *ClassDescription;


typedef struct ClassStruct {
  CLASS_DESCRIPTION_HEADER;
  OOP		classVariables;	/* dictionary of varName, storage */
  OOP		sharedPools;
} *Class;

typedef struct MetaclassStruct {
  CLASS_DESCRIPTION_HEADER;
  OOP		instanceClass;
} *Metaclass;

typedef struct CObjectStruct {
  OBJ_HEADER;
  voidPtr	addr;
  OOP		type;
} *CObject;

typedef struct ByteArrayStruct {
  OBJ_HEADER;
  Byte          bytes[1];
} *ByteArray;

typedef struct CTypeStruct {
  OBJ_HEADER;
  OOP		cObjectType;	/* the CObject subclass to use when
				   instantiating this type */
} *CType;

extern OOP		mst_objectClass, magnitudeClass, charClass, timeClass,
			dateClass,
			numberClass, floatClass, integerClass, lookupKeyClass,
			associationClass, linkClass, processClass,
			symLinkClass, collectionClass,
			sequenceableCollectionClass, linkedListClass,
			semaphoreClass,
			arrayedCollectionClass,	arrayClass, stringClass,
			symbolClass, byteArrayClass, compiledMethodClass,
			intervalClass, orderedCollectionClass,
			sortedCollectionClass, bagClass, mappedCollectionClass,
			setClass, dictionaryClass, 
			systemDictionaryClass,
			identityDictionaryClass, undefinedObjectClass,
			booleanClass, falseClass, trueClass, 
			processorSchedulerClass, delayClass, sharedQueueClass,
			behaviorClass,
			classDescriptionClass, classClass, metaclassClass,
			smalltalkDictionary, messageClass, methodContextClass,
			blockContextClass,
			streamClass, positionableStreamClass, readStreamClass,
			writeStreamClass, readWriteStreamClass, cObjectClass,
			fileStreamClass, memoryClass,
			byteMemoryClass, wordMemoryClass, randomClass,
			cFuncDescriptorClass, tokenStreamClass,
			methodInfoClass, fileSegmentClass,
  			processorOOP, cTypeClass;

extern OOP		associationNew(OOP key, OOP value), dictionaryNew(void), associationValue(OOP associationOOP),
			superClass(OOP classOOP), validClassMethodDictionary(OOP classOOP),
			classVariableDictionary(OOP classOOP), findSharedPoolVariable(OOP classOOP, OOP symbol),
			dictionaryAt(OOP dictionaryOOP, OOP keyOOP), dictionaryAtPut(OOP dictionaryOOP, OOP keyOOP, OOP valueOOP), dictionaryAdd(OOP dictionaryOOP, OOP associationOOP),
			arrayAt(OOP arrayOOP, int index), floatNew(double f), arrayNew(int numElts), stringNew(char *s),
			indexOOP(OOP oop, int index), instVarAt(OOP oop, int index), indexStringOOP(OOP oop, int index),
			classMethodDictionary(OOP classOOP), findClass(OOP classNameOOP),
			dictionaryAssociationAt(OOP dictionaryOOP, OOP keyOOP), findClassMethod(OOP classOOP, OOP selector),
			messageNewArgs(OOP selectorOOP, OOP argsArray), messageSelector(OOP messageOOP), messageArgs(OOP messageOOP),
  			getClassSymbol(OOP classOOP), dictionaryCopy(OOP dictionaryOOP),
			instanceVariableArray(OOP classOOP), sharedPoolDictionary(OOP classOOP),
			instantiateOOPWith(OOP classOOP, long int numIndexFields), metaclassInstance(OOP metaclassOOP),
			cObjectNew(voidPtr cObjPtr), cObjectNewTyped(voidPtr cObjPtr, OOP typeOOP), cObjectSubtype(),
  			allocCObject(OOP classOOP, long unsigned int size), cTypeNew(OOP cObjectSubclassOOP), newString(int len),
  			byteArrayNew(Byte *bytes, int len), countedStringNew(char *s, int len),
			identityDictionaryAtPut(OOP identityDictionaryOOP, OOP keyOOP, OOP valueOOP);

extern mst_Object	newInstanceWith(OOP classOOP, long int numIndexFields), newInstance(OOP classOOP), instantiate(OOP classOOP),
  			instantiateWith(OOP classOOP, long int numIndexFields);

extern int		numIndexableFields(OOP oop), dictionarySize(OOP dictionaryOOP),
			oopFixedFields(OOP oop), oopSizeBytes(OOP oop);

extern mst_Boolean	isIndexable(OOP classOOP), isAKindOf(OOP memberOOP, OOP classOOP), checkIndexableBoundsOf(OOP oop, int index),
  			checkBoundsOf(OOP oop, int index), instVarAtPut(OOP oop, int index, OOP value), indexOOPPut(OOP oop, int index, OOP value),
			isPointers(OOP oop), isAClass(OOP oop), isAMetaclass(OOP oop), 
			classIsPointers(OOP classOOP);

extern Byte		*stringOOPChars(OOP stringOOP), *toCString(OOP stringOOP), *toByteArray(OOP byteArrayOOP);

extern void		indexStringOOPPut(OOP oop, int index, OOP value),
			printAssocationKey(), printOOPConstructor(OOP oop),
			addClassSymbol(), initDictionary(void), initSTDIOObjects(void),
			setOOPString(OOP stringOOP, char *s), setCObjectValue(OOP cObjOOP, voidPtr value), setComment(OOP classDescOOP, OOP commentOOP),
			setOOPBytes(OOP byteArrayOOP, Byte *bytes), nilFill(register OOP *oopPtr, register long int OOPCount), freeCObject(OOP cObjOOP),
			setAssociationValue(OOP associationOOP, OOP value), printAssociationKey(OOP associationOOP);
extern voidPtr		cObjectValue(OOP cObjOOP);

extern double		floatOOPValue(OOP floatOOP);
extern void		arrayAtPut(OOP arrayOOP, int index, OOP value);

extern void		dictInit(void); /* ### TEMP HACK ### */

#endif /* __GSTDICT__ */
