/*  Open GENIE, a program for neutron scattering data analysis & display
    ISIS facility, Rutherford Appleton Laboratory, Chilton, Didcot, OXON UK
    Copyright (C) 1996,1997

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA. */

#include "geniedefs.h"
//
// CLASS DRAWABLE_OBJECTS POINTER LIST
//
// *** include string headers
#include <gnustring.h>

#include "Drawable_objects.h"
#include "Drawable_objectsPTR_List.h"

static const char* rcsid = "@(#)$Id: Drawable_objectsPTR_List.cxx,v 1.9 1997/03/26 16:26:12 cmm Exp $";
USE_RCSID(rcsid);

//#define DEBUG_LIST

//---
// Constructor
//
Drawable_objectsPTRList::Drawable_objectsPTRList() {
	int i;

	// set the parameters
	SIZE = 20;
	INCREMENT = 10;
	LastItem = 0;

	// create a list array
	MyList = new Drawable_objectsPTR[SIZE];

	// clear it.
	for(i=0; i<SIZE;i++) {
	    MyList[i] = 0;
	    }

#ifdef DEBUG_LIST
	gerr << "Created Drawable_objectsPTRList of size " << SIZE << endl;
#endif /* DEBUG_LIST */
}

// ***
// destructor
//
Drawable_objectsPTRList::~Drawable_objectsPTRList() {
	int i;

	// ** destruct all items on the list
	for (i=0; i<LastItem; i++) {
	    delete MyList[i];
	}

	// delete the list array
	delete [] MyList;

#ifdef DEBUG_LIST
	gerr << "Destroyed Drawable_objectsPTRList containing " << length() << " objects" << endl;
#endif /* DEBUG_LIST */
}

// ***
// add an item to the list
// Return an index usable by get()
//
int Drawable_objectsPTRList::push(Drawable_objectsPTR& newPTR) {

	MyList[LastItem] = newPTR;
	LastItem++;

	if (LastItem == SIZE) 
	{
	    Drawable_objectsPTR *temp;
	    int i;

	    // increase the list size
	    SIZE += INCREMENT;

#ifdef DEBUG_LIST
	    gerr << "Growing Drawable_objectsPTRList to size " << SIZE << endl;
#endif /* DEBUG_LIST */

	    // create a temporary list
	    temp = MyList;
	    MyList =  new Drawable_objectsPTR[SIZE];

	    // copy the old list
	    for (i=0; i<LastItem; i++) {
		MyList[i] = temp[i]; }

	    // And finish off list with NULLs
	    for (i=LastItem; i<SIZE; i++) 
	    {
		MyList[i] = 0; 
	    }
	    
	    // Now free up old pointer list array
	    delete[] temp;
	}
	return (LastItem - 1);
}

// *** 
// return an item from the list
//
Drawable_objectsPTR Drawable_objectsPTRList::get(int i) {

	if (i >= LastItem) return NULL;
	if (i < 0) return NULL;
	return MyList[i];
}

// return index of item on list, -1 if not found
int Drawable_objectsPTRList::find(Drawable_objectsPTR PTR)
{
    for(int i=0; i<LastItem; i++)
    {
	if (MyList[i] == PTR)
	{
	    return i;
	}
    }
    return -1;
}

int Drawable_objectsPTRList::length() 
{
    return LastItem;
}

ostream& Drawable_objectsPTRList::identity(ostream& os, int objnum)
{
    if (objnum < 0 || objnum >= LastItem)
    {
	return os << "Invalid Object Number: " << objnum;
    }
    else
    {
	return MyList[objnum]->identity(os);
    }
}
