/*  Open GENIE, a program for neutron scattering data analysis & display
    ISIS facility, Rutherford Appleton Laboratory, Chilton, Didcot, OXON UK
    Copyright (C) 1996,1997

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA. */

//
//
// **** INCLUDE FILE FOR ALL PGPLOT ROUTINES USED ****
//
//-------
#ifndef PGINCLUDE_H
#define PGINCLUDE_H

#include <string.h>
#include <stdio.h>

#include <gnustring.h>

	struct simple { int length; char spot[512]; };

#define PGDESCRIPTOR(name,text) struct simple name;\
		name.length = (int)text.length();\
		strcpy(name.spot,text);

#define STRING simple

// See "g_types.h" for values to convert Smalltalk "undefined" to
typedef float pg_float;					// PGPLOT FORTRAN REAL DATA TYPE
typedef int pg_int;					// PGPLOT FORTRAN INTEGER DATA TYPE
typedef void (*pg_cont_func_t)(pg_int& visible, pg_float& x, pg_float& y, pg_float& z);		// Function used by PGCONX
typedef void (*pg_cont_lab_func_t)(pg_int* flags, pg_float& x_in, pg_float& y_in, pg_float& x_out, pg_float& y_out);	// Function used by PGCONLX

//
//------
// **** define PGPLOT substitutions for UNIX
// NB: any routine that calls a PGplot routine and does NOT
//     pass any strings, can directly call the PGPLOT library - 
//     these routines end in "_".
//     Any routines that pass strings call intermediate FORTRAN
//     routines and the names end in "__"

// f2c appends _ to all routines, but __ to ones already containing an "_"
#if defined(HAVE_LIBF2C)
        #define pgstart pgstart___
        #define pgbegin pgbegin___
        #define pgbox pgbox___
        #define pgcurse pgcurse___
        #define pglabel pglabel___
        #define pgmtext pgmtext___
        #define pgnumb pgnumb___
        #define pgptext pgptext___
        #define pgqinf pgqinf___
        #define pgtext pgtext___
        #define pgscrn pgscrn___
        #define pgconlx pgconlx___
	#define pgopen pgopen___
	#define gresc gresc___
#elif defined(__VMS)
	#define pgstart pgstart_
	#define pgbegin pgbegin_
	#define pgbox pgbox_
	#define pgcurse pgcurse_
	#define pglabel pglabel_
	#define pgmtext pgmtext_
	#define pgnumb pgnumb_
	#define pgptext pgptext_
	#define pgqinf pgqinf_
	#define pgtext pgtext_
	#define pgscrn pgscrn_
        #define pgconlx pgconlx_
	#define pgopen pgopen_
	#define gresc gresc_
#elif defined(__unix)
	#define pglen pglen__
	#define pgbox pgbox__
	#define pgbegin pgbegin__
	#define pgcurse pgcurse__
	#define pgnumb pgnumb__
	#define pglabel pglabel__
	#define pgmtext pgmtext__
	#define pgptext pgptext__
	#define pgstart pgstart__
	#define pgtbox pgtbox__
	#define pgtext pgtext__
	#define pgscrn pgscrn__
	#define pgqinf pgqinf__
        #define pgconlx pgconlx__
        #define pgopen pgopen__
	#define gresc gresc__
#endif

#if defined(__unix) || defined(HAVE_LIBF2C)
	#define pgadvance pgadvance_
	#define pgask pgask_
	#define pgbbuf pgbbuf_
	#define pgbin pgbin_
	#define pgconb pgconb_
	#define pgcons pgcons_
	#define pgcont pgcont_
	#define pgconx pgconx_
	#define pgdraw pgdraw_
	#define pgebuf pgebuf_
	#define pgend pgend_
	#define pgenv pgenv_
	#define pgerrx pgerrx_
	#define pgerry pgerry_
	#define pgetxt pgetxt_
	#define pgfunt pgfunt_
	#define pgfunx pgfunx_
	#define pgfuny pgfuny_
	#define pggray pggray_
	#define pghi2d pghi2d_
	#define pghist pghist_
	#define pgiden pgiden_
	#define pglcur pglcur_
	#define pgldev pgldev_
	#define pgline pgline_
	#define pgmove pgmove_
	#define pgncurse pgncurse_
	#define pgolin pgolin_
	#define pgpage pgpage_
	#define pgpaper pgpaper_
	#define pgpoint pgpoint_
	#define pgpoly pgpoly_
	#define pgqcf pgqcf_
	#define pgqch pgqch_
	#define pgqci pgqci_
	#define pgqcol pgqcol_
	#define pgqfs pgqfs_
	#define pgqls pgqls_
	#define pgqlw pgqlw_
	#define pgqvp pgqvp_
	#define pgqwin pgqwin_
	#define pgrect pgrect_
	#define pgrnd pgrnd_
	#define pgrnge pgrnge_
	#define pgscf pgscf_
	#define pgsch pgsch_
	#define pgsci pgsci_
	#define pgscr pgscr_
	#define pgshls pgshls_
	#define pgsfs pgsfs_
	#define pgsls pgsls_
	#define pgslw pgslw_
	#define pgupdt pgupdt_
	#define pgvport pgvport_
	#define pgvsize pgvsize_
	#define pgvstand pgvstand_
	#define pgwindow pgwindow_
	#define pgwnad pgwnad_
	#define grxrgb grxrgb_
	#define pgqcr pgqcr_
	#define pgcell pgcell_
	#define pgslct pgslct_
	#define pgclos pgclos_
	#define pgsubp pgsubp_
#endif

#define GRESC(str1) {\
	PGDESCRIPTOR(_str1, str1);\
	gresc(_str1);\
	}

//--------------
// *** PGxxxx routines that pass strings/characters
//     must have macros written
//
// *** MACRO FOR PGBEGIN 
#define PGBEGIN(i1, str1, i2, i3) {\
	pg_int _i1 = i1, _i2 = i2, _i3 = i3;\
	PGDESCRIPTOR(_str1, str1);\
	String one = str1;\
	pgbegin (_i1, _str1, _i2, _i3);\
	}

// *** MACRO FOR PGOPEN 
#define PGOPEN(str1, device_id) {\
	PGDESCRIPTOR(_str1, str1);\
	device_id = pgopen (_str1);\
	}

// *** MACRO FOR PGSTART
#define PGSTART(i1, str1, i2, i3, f1) {\
	pg_int _i1 = i1, _i2 = i2, _i3 = i3;\
	pg_float _f1 = f1;\
	PGDESCRIPTOR(_str1, str1);\
	String one = str1;\
	pgstart (_i1, _str1, _i2, _i3, _f1);\
	}

// *** MACRO FOR PGBOX
#define PGBOX(str1, f1, i1, str2, f2, i2) {\
	pg_int _i1 = i1, _i2 = i2;\
	pg_float _f1 = f1, _f2 = f2;\
	PGDESCRIPTOR(_str1, str1);\
	String one = str1;\
	PGDESCRIPTOR(_str2, str2);\
	String two = str2;\
	pgbox (_str1, _f1, _i1, _str2, _f2, _i2);\
	}

// *** MACRO FOR PGCURSE
#define PGCURSE(f1, f2, s1) {\
    pg_float& _f1 = f1;\
    pg_float& _f2 = f2;\
    String& str1 = s1;\
    PGDESCRIPTOR(_str1,str1);\
    pgcurse(_f1, _f2, _str1);\
    str1 = _str1.spot[0];\
}

// *** MACRO FOR PGLABEL
#define PGLABEL(str1, str2, str3) {\
	PGDESCRIPTOR(_str1,str1);\
	String one = str1;\
	PGDESCRIPTOR(_str2,str2);\
	String two = str2;\
	PGDESCRIPTOR(_str3,str3);\
	String three = str3;\
	pglabel(_str1, _str2, _str3);\
	}

// *** MACRO FOR PGLEN
#define PGLEN(i1, str1, f1, f2) {\
	pg_int _i1 = i1;\
	pg_float _f1 = f1, _f2 = f2;\
	PGDESCRIPTOR(_str1,str1);\
	String one = str1;\
	pglen(_i1, _str1, _f1, _f2);\
	}

// *** MACRO FOR PGMTEXT
#define PGMTEXT(str1, f1, f2, f3, str2) {\
	pg_float _f1 = f1, _f2 = f2;\
	pg_float _f3 = f3;\
	PGDESCRIPTOR(_str1,str1);\
	String one = str1;\
	PGDESCRIPTOR(_str2,str2);\
	String two = str2;\
	pgmtext(_str1, _f1, _f2, _f3, _str2);\
	}

// *** MACRO FOR PGNUMB
#define PGNUMB(i1, i2, i3, str1, i4) {\
	pg_int _i1 = i1, _i2 = i2;\
	pg_int _i3 = i3, _i4 = i4;\
	PGDESCRIPTOR(_str1,str1);\
	String one = str1;\
	pgnumb(_i1, _i2, _i3, _str1, _i4);\
	}

// *** MACRO FOR PGPTEXT
#define PGPTEXT(f1, f2, f3, f4, str2) {\
	pg_float _f1 = f1, _f2 = f2;\
	pg_float _f3 = f3, _f4 = f4;\
	PGDESCRIPTOR(_str2,str2);\
	String two = str2;\
	pgptext(_f1, _f2, _f3, _f4, _str2);\
	}

// *** MACRO FOR PGQINF
#define PGQINF(s1, s2, i1) {\
	pg_int& _i1 = i1;\
	String& str1 = s1;\
	String& str2 = s2;\
	PGDESCRIPTOR(_str1,str1);\
	PGDESCRIPTOR(_str2,str2);\
	pgqinf(_str1, _str2, _i1);\
	}

// *** MACRO FOR PGTBOX
#define PGTBOX(str1, f1, i1, str2, f2, i2) {\
	pg_int _i1 = i1, _i2 = i2;\
	pg_float _f1 = f1, _f2 = f2;\
	PGDESCRIPTOR(_str1,str1);\
	String one = str1;\
	PGDESCRIPTOR(_str2,str2);\
	String two = str2;\
	pgtbox(_str1, _f1, _i1, _str2, _f2, _i2);\
	}

// *** MACRO FOR PGTEXT
#define PGTEXT(f1, f2, str1) {\
	pg_float _f1 = f1, pg_float _f2 = f2;\
	PGDESCRIPTOR(_str1, str1);\
	String one = str1;\
	pgtext(_f1, _f2, _str1);\
	}

// PGSCRN
#define PGSCRN(i1, str1, i2) { \
	pg_int _i1 = i1, _i2 = i2;\
	PGDESCRIPTOR(_str1,str1);\
	String one = str1;\
	pgscrn(_i1, _str1, _i2);\
	i2 = _i2;\
	}

//
//
// *** There must be externals for all PGxxxx routines
//
//
extern "C" {
	void gresc(STRING&);
	void pgadvance();								// advance to next page
	void pgask(const pg_int&);							// set prompt mode on or off

	void pgbbuf();									// start buffering output
	void pgbegin(const pg_int&, const STRING&, const pg_int&, const pg_int&);	// open output device
	void pgstart(pg_int&, STRING&, pg_int&, pg_int&, pg_float&);			// open output device
	void pgbin(const pg_int&, const pg_float [], const pg_float [],const pg_int&);	// histogram of binned data
	void pgbox(const STRING&, const pg_float&, const pg_int&, const STRING&, 
			const pg_float&, const pg_int&);	// draw frame & axes

	void pgconb(const pg_float [],const pg_int&,const pg_int&,const pg_int&,const pg_int&,
			const pg_int&,const pg_int&,const pg_float [], const pg_int&,
			const pg_float [],const pg_float&);	// contour map 2D, blanking
	void pgcons(const pg_float [],const pg_int&,const pg_int&,const pg_int&,const pg_int&,
			const pg_int&,const pg_int&,const pg_float [],const pg_int&,const pg_float []);	// contour map 2D, fast
	void pgcont(pg_float [],pg_int&,pg_int&,pg_int&,pg_int&,pg_int&,pg_int&,pg_float [],pg_int&,pg_float []);	// contour map 2D, contour follow
	void pgconlx(const pg_float [],const pg_int&,const pg_int&,const pg_int&,const pg_int&,const pg_int&,
			const pg_int&,const pg_float&,pg_cont_lab_func_t,const STRING&,const pg_int&,
			const pg_int&);	// contour map 2D, contour follow
	void pgconx(const pg_float [],const pg_int&,const pg_int&,const pg_int&,const pg_int&,
			const pg_int&,const pg_int&,const pg_float [],const pg_int&,pg_cont_func_t);
	pg_int  pgcurse(pg_float&, pg_float&, STRING&);					// read cursor position

	void pgdraw(pg_float&, pg_float&);						// DRAWREL

	void pgebuf();									// end output buffering
	void pgend();									// terminate pgplot
	void pgenv(pg_float&, pg_float&, pg_float&, pg_float&, pg_int&, pg_int&);	// set viewport/window/axes
	void pgerrx(pg_int&, pg_float [], pg_float [], pg_float [], pg_float&);		// horizontal error bars
	void pgerry(pg_int&, pg_float [], pg_float [], pg_float [], pg_float&);		// vertical error bars
	void pgetxt();									// erase text on graphics screen

	void pgfunt(pg_float*, pg_float*, pg_int&, pg_float&, pg_float&, pg_int&);	// X = f(t), Y = g(t)
	void pgfunx(pg_float*, pg_int&, pg_float&, pg_float&, pg_int&);			// Y = f(x)
	void pgfuny(pg_float*, pg_int&, pg_float&, pg_float&, pg_int&);			// X = f(y)

	void pggray(pg_float [], pg_int&, pg_int&, pg_int&, pg_int&, pg_int&, pg_int&,pg_float&,pg_float&,pg_float []); // gray scale of 2D array

	void pghi2d(pg_float [],pg_int&,pg_int&,pg_int&,pg_int&,const pg_int&,pg_int&,pg_float [],pg_int&,pg_float&,pg_int&,pg_float []);	// X-section of 2D array
	void pghist(pg_int&, pg_float [], pg_float&, pg_float&, pg_int&, pg_int&);	// unbinned histogram

	void pgiden();									// write username/time

	void pglabel(STRING&, STRING&, STRING&);					// label axes, title
	void pglcur(pg_int&, pg_int&, pg_float [], pg_float []);			// draw line using cursor
	void pgldev();									// list available devices
	void pglen(pg_int&, STRING&, pg_float&, pg_float&);				// find string length
	void pgline(pg_int&, pg_float [], pg_float []);					// draw polyline

	void pgmove(pg_float&, pg_float&);						// MOVEABS
	void pgmtext(STRING&, pg_float&, pg_float&, pg_float&, STRING&);		// write text to viewport

	void pgncurse(pg_int&, pg_int&, pg_float [], pg_float [], pg_int&);		// mark popg_ints using cursor
	void pgnumb(pg_int&, pg_int&, pg_int&, STRING&, pg_int&);			// convert number to text

	void pgolin(pg_int&, pg_int&, pg_float [], pg_float [], pg_int&);		// mark popg_ints using cursor

	void pgpage();									// advance to next page
	void pgpaper(pg_float&, pg_float&);						// change size of paper
	void pgpoint(pg_int&, pg_float [], pg_float [], pg_int&);			// plot marker plot
	void pgpoly(pg_int&, pg_float [], pg_float []);					// draw/fill polygon
	void pgptext(pg_float&, pg_float&, pg_float&, pg_float&, STRING&);		// write text

	void pgqcf(pg_int&);								// inquire font
	void pgqch(pg_float&);								// inquire character height
	void pgqci(pg_int&);								// inquire colour
	void pgqcol(pg_int&, pg_int&);							// inquire colour capability
	void pgqfs(pg_int&);								// inquire fill area style
	void pgqinf(STRING&, STRING&, pg_int&);						// inquire general information
	void pgqls(pg_int&);								// inquire line style
	void pgqlw(pg_int&);								// inquire line width
	void pgqvp(pg_int&, pg_float&, pg_float&, pg_float&, pg_float&);		// inquire viewport size/position
	void pgqwin(pg_float&, pg_float&, pg_float&, pg_float&);			// inquire window co-ords

	void pgrect(pg_float&, pg_float&, pg_float&, pg_float&);			// draw/fill rectangle
	void pgrnd(pg_float&, pg_int&);							// round up
	void pgrnge(pg_float&, pg_float&, pg_float&, pg_float&);			// choose axes limits

	void pgscf(pg_int&);								// set character font
	void pgsch(const pg_float&);							// set character height
	void pgsci(pg_int&);								// set colour
	void pgscr(const pg_int&, const pg_float&, const pg_float&, const pg_float&);				// set colour representation
	void pgscrn(pg_int&, STRING&, pg_int&);						// set colour representation
	void pgshls(pg_int&, pg_float&, pg_float&, pg_float&);				// set colour representation
	void pgsfs(const pg_int&);							// set fill area style
	void pgsls(const pg_int&);							// set line style
	void pgslw(pg_int&);								// set line width

	void pgtbox(STRING&, pg_float&, pg_int&, STRING&, pg_float&, pg_int&);		// draw time box
	void pgtext(pg_float&, pg_float&, STRING&);					// write text

	void pgupdt();									// flush ALL output buffers

	void pgvport(pg_float&, pg_float&, pg_float&, pg_float&);			// set viewport NDC
	void pgvsize(pg_float&, pg_float&, pg_float&, pg_float&);			// set viewport INCHES
	void pgvstand();								// set standard viewport

	void pgwindow(const pg_float&, const pg_float&, const pg_float&, const pg_float&);			// set window
	void pgwnad(pg_float&, pg_float&, pg_float&, pg_float&);			// set window & viewport
	void grxrgb(const pg_float&, const pg_float&, const pg_float&, pg_float&, pg_float&, pg_float&);
	void pgqcr(const pg_int&, pg_float&, pg_float&, pg_float&);
        void pgcell(const pg_float*, const pg_int&, const pg_int&, const pg_int&, const pg_int&, const pg_int&, 
			const pg_int&, const pg_float&, const pg_float&, pg_cont_lab_func_t, const pg_int*, 
			const pg_int&, const pg_int&, const pg_int&);
	void pgslct(pg_int&);
	void pgclos();
	void pgsubp(pg_int&, pg_int&);
	pg_int pgopen(STRING&);
	}

#endif /* PGINCLUDE_H */
