/*  Open GENIE, a program for neutron scattering data analysis & display
    ISIS facility, Rutherford Appleton Laboratory, Chilton, Didcot, OXON UK
    Copyright (C) 1996,1997

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA. */

#include "geniedefs.h"
//+++
//
// CLASS: Axes
//
//
//---
#include <gnustring.h>

#include "Drawable_objects.h"
#include "Axes.h"
#include "Pginclude.h"

static const char* rcsid = "@(#)$Id: axes.cxx,v 1.9 1997/03/27 22:05:58 cmm Exp $";
USE_RCSID(rcsid);

Axes::Axes(String tt,
	   String yl,
	   String xl,
	   float tcs,
	   float cs,
	   int f,
	   float xts,
	   float yts,
	   int sh,
	   int thickness,
	   int show_x,
	   int show_y,
	   int y_numbers_horizontal,
	   int type,
           int x_label_vertical,
	   int y_label_vertical)   {

	// *** set visibility
	invisible = 0;

	// *** set the x label 
	X_label = xl;

	// *** set the y label 
	Y_label = yl;

	// *** set the title
	if (tt == " ")
		    title = TITLE;
	else
		    title = tt;

	// *** set the colour
	if (sh == -1)
		    line_colour = LINE_COLOUR;
	else
		    line_colour = sh;

	// *** set the line thickness
	if (thickness == 0)
		    line_thickness = LINE_THICKNESS;
	else
		    line_thickness = thickness;

	// *** set the text character size
	if (tcs == 0.0)
		    text_char_size = TEXT_CHAR_SIZE;
	else
		    text_char_size = tcs;

	// *** set the number character size
	if (cs == 0.0)
		    character_size = CHARACTER_SIZE;
	else
		    character_size = cs;

	// *** set the  tick spacing
	Ytick_spacing = yts;
	Xtick_spacing = xts;

	// *** set the font
	if (f == 0)
		    font = FONT;
        else
		    font = f;

	// *** get the box
	box_visible = BOX_VISIBLE;

	// *** y axis
	Y_at_X0 = Y_AT_X0;

	// *** x axis
	X_at_Y0 = X_AT_Y0;

	// *** get major tick marks
	major_tick_marks = MAJOR_TICK_MARKS;

	// *** get minor tick marks
	minor_tick_marks = MINOR_TICK_MARKS;

	// *** get inset ticks
	inset_ticks = INSET_TICKS;

	// *** x axis log/lin
	Xlinear = XLINEAR;

	// *** y axis log/lin
	Ylinear = YLINEAR;

	// *** y axis orientation
	if (y_numbers_horizontal)
	{
	    Y_orientation = 1;
	}
	else
	{
	    Y_orientation = 0;
	}

//	Y_orientation = Y_ORIENTATION;

	// *** graticule
	graticule = GRATICULE;

        // *** do we show values on axes?
        show_X_values = show_x;
        show_Y_values = show_y;

	// *** 
	X_label_vertical = x_label_vertical;
	Y_label_vertical = y_label_vertical;

	// *** fill in line type
            if (type == 0)
                    linetype = LINE_TYPE;
            else
                    linetype = type;

	}

Axes::~Axes() {
	}


void Axes::display() {

	String xparam, yparam, side;
	int zero = 0;
	float scaler = 0.8F;

// *** work out the options


	if (Y_at_X0 == 1) {
	    yparam.prepend("A");
	    };

	if (X_at_Y0 == 1) {
	    xparam.prepend("A");
	    };
	
	if (box_visible == 1) {
	    yparam.prepend("BC");
	    xparam.prepend("BC");
	    };

	if (graticule == 1) {
	    yparam.prepend("G");
	    xparam.prepend("G");
	    };

	if (inset_ticks == 0) {
	    yparam.prepend("I");
	    xparam.prepend("I");
	    };

	if (Xlinear == 0) xparam.prepend("L");
	if (Ylinear == 0) yparam.prepend("L");

	if (show_X_values)
	    xparam.prepend("N");

	if (show_Y_values)
	    yparam.prepend("N");

	if (major_tick_marks == 1) {
	    yparam.prepend("T");
	    xparam.prepend("T");
	    };

	if (minor_tick_marks == 1) {
	    yparam.prepend("S");
	    xparam.prepend("S");
	    };

	if (Y_orientation != 0) yparam.prepend("V");

// *** make the call to pgplot to draw axes

	// *** set line thickness
	pgslw(line_thickness);

	// *** set line type
	pgsls(linetype);

	// *** set colour
	pgsci(line_colour);

	// *** set font
	pgscf(font);

	// *** set char height
	pgsch(character_size);

	// *** set line style
	pgsls(linetype);

	PGBOX(xparam, Xtick_spacing, zero, yparam, Ytick_spacing, zero);

// *** title/labels for the plot
	    float disp;
	    float coord;
	    float fjust;
	    
	    // *** do title first
	    // *** set char height
	    pgsch(text_char_size);
	
	    disp = 1.0F;
	    coord = 0.5F;
	    fjust = 0.5F;
	    side = "T";

	    PGMTEXT(side, disp, coord, fjust, title);

	    // *** do X label
	    // *** set char height
	    pgsch(text_char_size * scaler);
	
	    // *** calculate offset based on numerical height

	    disp = (character_size/text_char_size) + 2.5F;
	    coord = 0.5F;
	    fjust = 0.5F;
	    side = "B";

	    PGMTEXT(side, disp, coord, fjust, X_label);

	    // *** do X label
	
	    coord = 0.5F;
    
#if 0
	    if (Y_orientation == 0) { 
		    side = "L";
		    fjust = 0.5;}
	    else {
		    side = "LV";
		    fjust = 1.0;}
#endif
	    if (!Y_label_vertical)
	    {
		side = "LV";
                fjust = 1.0F;
	    }
	    else
	    {
		side = "L";
		fjust = 0.5F;
	    }

	    PGMTEXT(side, disp, coord, fjust, Y_label);

}
