/*  Open GENIE, a program for neutron scattering data analysis & display
    ISIS facility, Rutherford Appleton Laboratory, Chilton, Didcot, OXON UK
    Copyright (C) 1996,1997

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA. */

#include "geniedefs.h"
#include "g_types.h"
#include "Pginclude.h"
#include "cell_array.h"
#include "pg_coord_trans_func.h"

static const char* rcsid = "@(#)$Id: cell_array.cxx,v 1.11 1997/04/02 15:07:50 faa Exp $";
USE_RCSID(rcsid);

CellArray::CellArray(g_real* values, int nx_values, int ny_values, g_real fg, g_real bg, int type,
			int mode, g_integer* table, int ntab, TwoDimensionalCoordinateTransformFunction* ctf) :
	TwoDimensionalPlot(values, nx_values, ny_values, ctf), m_ntab(ntab), m_type(type), m_mode(mode), m_fg((float)fg), m_bg((float)bg)
{
    m_table = new pg_int[ntab];
    for(int i=0; i<ntab; i++)
    {
	m_table[i] = table[i];
    }
}

CellArray::~CellArray()
{
    delete[] m_table;
}

void CellArray::display()
{
    displayPlot();
}


void CellArray::displayPlot()
{
    PGCoordinateTransformFunction* pg_func = (PGCoordinateTransformFunction*)this->function();
    pg_func->setPointers(0);	// set for reverse function lookup
    pg_cont_lab_func_t func = pg_func->contourLabelFunction;
	pgcell(m_values.values(), m_values.dims(0), m_values.dims(1), 1, m_values.dims(0), 1, m_values.dims(1),
		   m_fg, m_bg, func, m_table, m_ntab, m_type, m_mode);
}

// this is called by display() in CellArrayWedge
// We should be drawing into an unscaled window (i.e. world coordinates of 0.0->1.0 in both dimensions)
void CellArray::displayWedge(int mode)
{
    if (mode == 0)
	displayVerticalWedge();
    else if (mode == 1)
	displayHorizontalWedge();
}

// Colour wedge with bands drawn vertically
void CellArray::displayVerticalWedge()
{
    pgwindow(m_bg, m_fg, 0.0F, 1.0F);	// rescale current window X coords
    int nx = m_ntab;
    g_real* wx = new g_real[nx];
    pg_float* values = new pg_float[2*nx];
    g_real wy[] = { 0.0, 1.0 };
    double step = (m_fg - m_bg) / (double)(nx - 1);
// Must set up values FORTRAN array style
    for(int i=0; i<nx; i++)
    {
	wx[i] = m_bg + i * step;
	values[i] = (float)wx[i];
	values[i+nx] = (float)wx[i];
    }
    int wx_dims[] = { nx }, wy_dims[] = { 2 };
    PGCoordinateTransformFunction* pg_func;
    pg_func = new PGCoordinateTransformFunction(wx, wx_dims, 1, PointMode, wy, wy_dims, 1, PointMode);
    pg_func->setPointers(0);	// set for reverse function lookup
    pg_cont_lab_func_t func = pg_func->contourLabelFunction;
// Always draw wedge in smooth interpolation mode (1) so we get the boundaries right
	pgcell(values, nx, 2, 1, nx, 1, 2, m_fg, m_bg, func, m_table, m_ntab, m_type, 1);
    delete[] values;
    delete[] wx;
}

// Colour wedge with bands drawn horizontally
void CellArray::displayHorizontalWedge()
{
    pgwindow(0.0F, 1.0F, m_bg, m_fg);	// rescale current window X coords
    int ny = m_ntab;
    g_real* wy = new g_real[ny];
    pg_float* values = new pg_float[2*ny];
    g_real wx[] = { 0.0, 1.0 };
    double step = (m_fg - m_bg) / (double)(ny - 1);
// Must set up values FORTRAN array style
    for(int i=0; i<ny; i++)
    {
	wy[i] = m_bg + i * step;
	values[2*i] = values[2*i+1] = (float)wy[i];
    }
    int wx_dims[] = { 2 }, wy_dims[] = { ny };
    PGCoordinateTransformFunction* pg_func;
    pg_func = new PGCoordinateTransformFunction(wx, wx_dims, 1, PointMode, wy, wy_dims, 1, PointMode);
    pg_func->setPointers(0);	// set for reverse function lookup
    pg_cont_lab_func_t func = pg_func->contourLabelFunction;
// Always draw wedge in smooth interpolation mode (1) so we get the boundaries right
    pgcell(values, 2, ny, 1, 2, 1, ny, m_fg, m_bg, func, m_table, m_ntab, m_type, 1);
    delete[] values;
    delete[] wy;
}
