/*  Open GENIE, a program for neutron scattering data analysis & display
    ISIS facility, Rutherford Appleton Laboratory, Chilton, Didcot, OXON UK
    Copyright (C) 1996,1997

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA. */

#include "geniedefs.h"
#include "g_types.h"
#include "Pginclude.h"
#include "contour_plot.h"
#include "pg_coord_trans_func.h"

static const char* rcsid = "@(#)$Id: contour_plot.cxx,v 1.7 1997/04/01 18:44:45 faa Exp $";
USE_RCSID(rcsid);

ContourPlot::ContourPlot(g_real* values, int nx, int ny, g_real* contours, int ncontours,
                                TwoDimensionalCoordinateTransformFunction* ctf, int ls, int lc, float lt) : 
	TwoDimensionalPlot(values, nx, ny, ctf), m_ncontours(ncontours), m_line_style(ls)
{
    line_thickness = (int)lt;
    line_colour = lc;
    m_contours = new pg_float[ncontours];
    for(int i=0; i<ncontours; i++)
    {
	m_contours[i] = (float)contours[i];
    }
}

ContourPlot::~ContourPlot()
{
    delete []m_contours;
}

void ContourPlot::display()
{
    displayPlot();
}

void ContourPlot::displayPlot()
{
// Draw contour plot
    pgsci(line_colour);			// colour
    pgsls(m_line_style);		// line style
    pgslw(line_thickness);		// line width
// use "-m_ncontours" to force use of "pgsls" value
    PGCoordinateTransformFunction* pg_func = (PGCoordinateTransformFunction*)(this->function());
    pg_func->setPointers();
    pgconx(m_values.values(), m_values.dims(0), m_values.dims(1), 1, m_values.dims(0), 1, m_values.dims(1), 
		m_contours, -m_ncontours, pg_func->contourFunction);
}

