/*  Open GENIE, a program for neutron scattering data analysis & display
    ISIS facility, Rutherford Appleton Laboratory, Chilton, Didcot, OXON UK
    Copyright (C) 1996,1997

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA. */

#include "geniedefs.h"
// -----------------------------------------------------------------------------
// DRAW_GRATICULE.C
//
// Description: C++ code to allow creation of a Genie object-graphics graticule.
//
#include "Genie_functions.h"

// *** includes for the picture list
#include "Pictlist.h"

// *** local includes
#include "Axes.h"

static const char* rcsid = "@(#)$Id: draw_graticule.cxx,v 1.7 1997/04/08 16:32:32 cmm Exp $";
USE_RCSID(rcsid);

    void new_graticule(int colour,
		    float line_width,
		    int xlog,
		    int ylog,
		    float num_size,
		    int xnum,
		    int ynum,
		    int yhoriz,
		    int quality,
		    int type,
		    g_objectid objid) {

	Drawable_objects *PTR;
	float fzero = 0.0F;
	int width,x(0),y(0);
	int box;
	int grat;

	// *** check box visible status
	box = Axes::BOX_VISIBLE;
	Axes::BOX_VISIBLE = 0;

	// *** check graticule status
	grat = Axes::GRATICULE;
	Axes::GRATICULE = 1;

	// *** check for log axes
	if (xlog == 1) {
	    x = Axes::XLINEAR;
	    Axes::XLINEAR = 0;
	    }

	if (ylog == 1) {
	    y = Axes::YLINEAR;
	    Axes::YLINEAR = 0;
	    }

	// *** convert line width to an integer
	width = (int)line_width;

	// *** NEW a set of axes
	PTR = new Axes(" "," "," ",fzero,num_size,quality,fzero,fzero,colour,width,xnum,ynum,yhoriz,type);
	
	// *** reset log axes
	if (xlog == 1) Axes::XLINEAR = x;
	if (ylog == 1) Axes::YLINEAR = y;

	// *** reset box visible & graticule
	Axes::BOX_VISIBLE = box;
	Axes::GRATICULE = grat;

	// *** add them to the display list
	objid[ObjectPosition] = PicturePTR[Current_Picture]->disp_add(PTR);
	objid[ObjectPicture] = Current_Picture;

    }

    void draw_graticule(int colour,
                    float line_width,
                    int xlog,
                    int ylog,
		    float num_size,
		    int xnum,
		    int ynum,
	 	    int yhoriz,
		    int quality,
		    int type) {

    g_integer objid[GENIE_OBJECTID_SIZE];
    new_graticule(colour, line_width, xlog, ylog, num_size,xnum,ynum,yhoriz,
		  quality,type,objid);
    display_object(objid);

    }
