/*  Open GENIE, a program for neutron scattering data analysis & display
    ISIS facility, Rutherford Appleton Laboratory, Chilton, Didcot, OXON UK
    Copyright (C) 1996,1997

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA. */

#ifndef DRAWABLE_VALUES_H
#define DRAWABLE_VALUES_H

// Values stored in FORTRAN style (i.e. column major)
// First array element is 0
// The default setup assumes we are given "C" arrays and want to store values in "Fortran" style

class DrawableValues
{
    public:
	enum ArrayStyle { Fortran, C };

    protected:
    	pg_float* m_values;
    	int m_ndims;
    	int* m_dims_array;
    	int* m_dope_array;
	int m_npoints;
	ArrayStyle m_dims_style;		// How m_values is stored

    public:
	int setupDims(int* dims_array, int ndims, ArrayStyle dims_style = Fortran);
	int setup(float* values, int* dims_array, int ndims, ArrayStyle values_style = C, ArrayStyle dims_style = Fortran);
	int setup(double* values, int* dims_array, int ndims, ArrayStyle values_style = C, ArrayStyle dims_style = Fortran);
	DrawableValues(float* values, int dim1, ArrayStyle values_style = C, ArrayStyle dims_style = Fortran);
	DrawableValues(float* values, int dim1, int dim2, ArrayStyle values_style = C, ArrayStyle dims_style = Fortran);
	DrawableValues(float* values, int* dims_array, int ndims, ArrayStyle values_style = C, ArrayStyle dims_style = Fortran);
	DrawableValues(double* values, int dim1, ArrayStyle values_style = C, ArrayStyle dims_style = Fortran);
	DrawableValues(double* values, int dim1, int dim2, ArrayStyle values_style = C, ArrayStyle dims_style = Fortran);
	DrawableValues(double* values, int* dims_array, int ndims, ArrayStyle values_style = C, ArrayStyle dims_style = Fortran);
	DrawableValues() : m_values(0), m_ndims(0), m_dims_array(0), m_dope_array(0), m_npoints(0), m_dims_style(Fortran)  {}
	~DrawableValues();
	pg_float& operator()(int i) { return m_values[i]; }			// (m_dope_array[0] == 1)
	const pg_float& operator()(int i) const { return m_values[i]; }			// (m_dope_array[0] == 1)
 	pg_float& operator()(int i, int j) { return m_values[i*m_dope_array[0] + j*m_dope_array[1]]; }	
 	const pg_float& operator()(int i, int j) const { return m_values[i*m_dope_array[0] + j*m_dope_array[1]]; }	
	pg_float& operator()(int* pos, int npos);
	int dims(int i) { return m_dims_array[i]; }
	int ndims() const { return m_ndims; }
    	pg_float* values() { return m_values; }		// in "m_dims_style" style
	int npoints() const { return m_npoints; }
	DrawableValues slice(int dim, double val) const;	// return lower dimensional array when axis "dim" is
								// fixed at value "val" (interpolating if necessary)

    protected:
  	static pg_float* duplicate(const float* old_array, int n);
  	static pg_float* duplicate(const double* old_array, int n);
  	static pg_float* duplicate(const float* old_array, int* old_dims_array, int old_ndims, 
					ArrayStyle old_dims_style = C, ArrayStyle new_dims_style = Fortran);
  	static pg_float* duplicate(const double* old_array, int* old_dims_array, int old_ndims, 
					ArrayStyle old_dims_style = C, ArrayStyle new_dims_style = Fortran);
	static int points(int* dims_array, int ndims);
};

#endif /* DRAWABLE_VALUES_H */
