/*  Open GENIE, a program for neutron scattering data analysis & display
    ISIS facility, Rutherford Appleton Laboratory, Chilton, Didcot, OXON UK
    Copyright (C) 1996,1997

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA. */

#include "geniedefs.h"
//+++
//
// CLASS: Errors
//
//
//---

#include <string.h>

// *** include string headers
#include <gnustring.h>

#include "Errors.h"

static const char* rcsid = "@(#)$Id: errors.cxx,v 1.8 1997/04/08 16:32:37 cmm Exp $";
USE_RCSID(rcsid);

// *** constructor
Errors::Errors() {
	    }

// *** destructor
Errors::~Errors() {
	}

// *** error clearing
void Errors::clear() {
    int i;

    // *** loop through all strings
    for (i=0; i<5; i++) {
	OBJECT[i] = "";
	FAILURE[i] = "";
	SOLUTION[i] = "";
	}
    return;
    }

// *** return the error status
int Errors::status() {
    int i=0;

    if (OBJECT[0].length() > 0) i = 1;
    if (OBJECT[1].length() > 0) i = 2;
    if (OBJECT[2].length() > 0) i = 3;
    if (OBJECT[3].length() > 0) i = 4;
    if (OBJECT[4].length() > 0) i = 5;

    return i;
    }

// *** return the object name
String Errors::object(int number) {

    if (number > 0 && number <= 5)
	
	return OBJECT[number-1];

    return "";
    }

// *** return the failure
String Errors::failure(int number) {

    if (number > 0 && number <=5)
	return FAILURE[number-1];

    return "";
    }

// *** return the solution
String Errors::solution(int number) {

    if (number > 0 && number <=5)
	return SOLUTION[number-1];

    return "";
    }

// *** return the whole message
String Errors::message(int number) 
{
    if (number > 0 && number <=5)
    {	
	if ((strcmp(SOLUTION[number-1], "") != 0) && (strcmp(SOLUTION[number-1], " ") != 0))
	{ 
	    return (OBJECT[number-1]+"\n"+FAILURE[number-1]+"\n"+SOLUTION[number-1]);
        }
	else
	{
	    return (OBJECT[number-1]+"\n"+FAILURE[number-1]);
	}
    }
    return "";
}

// *** add an error to the list
void Errors::add_error(String one,
			String two,
			String three) {

    // *** trap for no error
    if (one == "") return;

    // *** move all errors down list
    for (int i=4; i>0; --i) {
	OBJECT[i] = OBJECT[i-1];
	FAILURE[i] = FAILURE[i-1];
	SOLUTION[i] = SOLUTION[i-1];
	}

    // *** add error at top
    OBJECT[0] = one;
    FAILURE[0] = two;
    SOLUTION[0] = three;
   }
