/*  Open GENIE, a program for neutron scattering data analysis & display
    ISIS facility, Rutherford Appleton Laboratory, Chilton, Didcot, OXON UK
    Copyright (C) 1996,1997

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA. */

#include "geniedefs.h"
/*
 * main file of routines for outputting G3 intermediate files
 *
 */
#ifdef HAVE_UNISTD_H
#	include <unistd.h>
#endif
#include <sys/types.h>	/* for lseek */
#include <stdio.h>
#include <string.h>
#include <ctype.h>
#include <sys/stat.h>
#ifdef _WIN32
#	include <io.h>
#	include <winsock.h>
#else
#	include <arpa/inet.h>
#	include <netinet/in.h>
#endif
#include <fcntl.h>
#include <gnustring.h>
#ifdef __VMS
#    ifdef __VMS_POSIX
#	include <vms_posix_extensions.h>
#    else
#	include <unixio.h>
#	include <unixlib.h>
#    endif
#endif
#include "genie_types.h"
#include "Genie_functions.h"
#include "fortran_types.h"

#include "g3io.h"

#ifdef __ultrix
extern "C" u_long ntohl(u_long);
#endif

#define	arrayDopeOneD(__gv,__n) \
		arrayGetDims(__gv)[0]=(__n); \
		arrayGetDims(__gv)[1]=0; \
		arrayGetDope(__gv)[0]=1; \
		arrayGetDope(__gv)[1]=0

#define	MAX(__a,__b)	((__a) < (__b) ? (__b) : (__a))
 
// #define error_add(__s1, __s2, __s3) printf("Error returned from %s: %s, %s\n", __s1, __s2, __s3)

#define faaread(__fd, __p_variable, __length, __counter) \
    if (read(__fd, (void*)__p_variable, __length) != __length) \
    { \
	char error_buffer[100]; \
	sprintf(error_buffer, "%s at line %i", __FILE__, (int)__LINE__); \
	error_add(error_buffer, "Command failed while reading from disk - file probably corrupted or not G3 type", \
		"Check file with a DIR/G3 command"); \
	return -1; \
    } \
    __counter += __length

#define faawrite(__fd, __p_variable, __length, __counter) \
    if (write(__fd, (const void*)__p_variable, __length) != __length) \
    { \
	char error_buffer[100]; \
	sprintf(error_buffer, "%s at line %i", __FILE__, (int)__LINE__); \
	error_add(error_buffer, "Command failed while writing to disk", \
		"Check file with a DIR/G3 command"); \
	return -1; \
    } \
    __counter += __length

#define faalseek(__fd, __pos, __opt) \
    if (lseek(__fd, __pos, __opt) == -1) \
    { \
	char error_buffer[100]; \
	sprintf(error_buffer, "%s at line %i", __FILE__, (int)__LINE__); \
	error_add(error_buffer, "seek failed", \
		"Check file with a DIR/G3 command"); \
	return -1; \
    }

#define RFB(__buffer, __p_variable, __length,  __nsofar, __nmax) \
    if (readFromBuffer(__buffer, __p_variable, __length) != 0) \
    { \
	char error_buffer[100]; \
	sprintf(error_buffer, "%s at line %i", __FILE__, (int)__LINE__); \
	error_add(error_buffer, "Read from buffer failed", \
		""); \
	return -1; \
    } \
    __nsofar += __length; \
    if ( __nsofar > __nmax ) \
    { \
	char error_buffer[100]; \
	sprintf(error_buffer, "%s at line %i", __FILE__, (int)__LINE__); \
	error_add(error_buffer, "Too many characters read from buffer", \
		""); \
	return -1; \
    }

#define DTOI(__p_double) 	local_to_ieee_double(__p_double)

#if 0
    if (local_to_ieee_double(__p_double) != 0) \
    { \
	char error_buffer[100]; \
	sprintf(error_buffer, "%s at line %i", __FILE__, (int)__LINE__); \
	error_add(error_buffer, "dtoi: failed to convert number from VAX to IEEE format", \
		""); \
	return -1; \
    }
#endif

#define ITOD(__p_double) ieee_to_local_double(__p_double)

#if 0
    if (ieee_to_local_double(__p_double) != 0) \
    { \
	char error_buffer[100]; \
	sprintf(error_buffer, "%s at line %i", __FILE__, (int)__LINE__); \
	error_add(error_buffer, "itod: failed to convert number from IEEE to VAX format", \
		""); \
	return -1; \
    }
#endif

/*
 * write out gv to current file position, return total number of bytes
 */
#if 0
/* return number of bytes written, 0 on error */
int writeGvar(int fd, g_variable gv, const char* marker)
{
    ssize_t nbytes = 0;
    size_t dummy = 0;
    off_t old_pos, current_pos;
    int mlen;
    mlen = (strlen(marker) + 1) * sizeof(char);	/* +1 for \0 */
    faawrite(fd, &mlen, sizeof(int), dummy);
    faawrite(fd, marker, mlen, dummy);
    old_pos = lseek(fd, 0, SEEK_CUR);
    faawrite(fd, &nbytes, sizeof(size_t), dummy);
    nbytes = 0;
    if (writeGvarOut(fd, gv, &nbytes) != 0)
    {
	return -1;
    }
    current_pos = lseek(fd, 0, SEEK_CUR);
    faalseek(fd, old_pos, SEEK_SET);
    faawrite(fd, &nbytes, sizeof(size_t), dummy);
    faalseek(fd, current_pos, SEEK_SET);
    return(nbytes);
}

/* returns -1 on error, number of bytes on success */
int readGvar(int fd, g_variable* gv, const char** marker)
{
    ssize_t nbytes, nread;
    char *cbuff, *buffer;
    int mlen;

    nread = 0;
    faaread(fd, &mlen, sizeof(int), nread);	/* length + \0 */
    cbuff = (char*)M_alloc(mlen);
    faaread(fd, cbuff, mlen, nread);
    *marker = cbuff;
    faaread(fd, &nbytes, sizeof(size_t), nread);
    buffer = (char*)M_alloc(nbytes);
    faaread(fd, buffer, nbytes, nread);
    nread = 0;
    if (readGvarIn(&buffer, gv, nbytes, &nread) != 0 ) 
    {
  	return -1;
    }
    return(nbytes);
}

/* return 0 OK, -1 on error */
int writeGvarOut(int fd, g_variable gv, ssize_t* nbytes)
{
    g_integer gi, *ibuffer;
    g_real gr, *rbuffer;
    g_variable tgv;
    String gs;
    int var_code, slen, nfields;
    long alen, i;

    var_code = (int) variableType(gv);
    faawrite(fd, &var_code, sizeof(int), *nbytes);

    switch(var_code) {

    case _Integer:
	gi = getInteger(gv);
	faawrite(fd, &gi, sizeof(g_integer), *nbytes);
	break;

    case _Real:
	gr = getReal(gv);
	ITOD(&gr);
	faawrite(fd, &gr, sizeof(g_real), *nbytes);
	break;

    case _String:    
	gs = getString(gv);
	slen = (gs.length() + 1) * sizeof(char);
	faawrite(fd, &slen, sizeof(int), *nbytes);
	faawrite(fd, (const char*)gs, slen, *nbytes);
	break;

    case _Workspace:
	nfields = wrklen(gv);
	faawrite(fd, &nfields, sizeof(int), *nbytes);

	for(i=0; i<nfields; i++)
	{
	    gs = getFieldByIndex(gv, i);
	    slen = (gs.length() + 1) * sizeof(char);
	    faawrite(fd, &slen, sizeof(int), *nbytes);
	    faawrite(fd, (const char*)gs, slen, *nbytes);

	    tgv = getElementByIndex(gv, i);
	    if (writeGvarOut(fd, tgv, nbytes) != 0)
	    {
		return -1;
	    }
	}
	break;

    case _IntegerArray:
 	alen = ArrayLength(gv);
 	faawrite(fd, &alen, sizeof(long), *nbytes);	

	ibuffer = (g_integer*) M_alloc(alen*sizeof(g_integer));
	for(i=0; i<alen; i++)
	{
	    ibuffer[i] = arrayGetInteger(gv, i);
	}
	faawrite(fd, ibuffer, alen*sizeof(g_integer), *nbytes);
	break;

    case _RealArray:
 	alen = ArrayLength(gv);
 	faawrite(fd, &alen, sizeof(long), *nbytes);	

	rbuffer = (g_real*) M_alloc(alen*sizeof(g_real));
	for(i=0; i<alen; i++)
	{
	    rbuffer[i] = arrayGetReal(gv, i);
	    ITOD(rbuffer+i);
	}
	faawrite(fd, rbuffer, alen*sizeof(g_real), *nbytes);
	break;

    case _StringArray:
 	alen = ArrayLength(gv);
 	faawrite(fd, &alen, sizeof(long), *nbytes);	

	for(i=0; i<alen; i++)
	{
	    gs = arrayGetString(gv, i);
	    slen = (gs.length() + 1) * sizeof(char);
	    faawrite(fd, &slen, sizeof(int), *nbytes);
	    faawrite(fd, (const char*)gs, slen, *nbytes);
	}
	break;

    case _WorkspaceArray:
	alen = ArrayLength(gv);
 	faawrite(fd, &alen, sizeof(long), *nbytes);	

	for(i=0; i<alen; i++)
	{
	    putWorkspace(tgv, arrayGetWorkspace(gv, i));
	    if (writeGvarOut(fd, tgv, nbytes) != 0)
	    {
		return -1;
	    }
	}
	break;

    case _Undefined:
	break;
 
    default:
	return -1;
	break;
    }
    return 0;
}

int readGvarIn(char** read_buffer, g_variable *gv, size_t nbytes, ssize_t *nread)
{
    g_integer gi, *ibuffer;
    g_real gr, *rbuffer;
    g_variable tgv;
    char *cbuff;
    int var_code, slen, nfields;
    long alen, i;

    RFB(read_buffer, &var_code, sizeof(int), *nread, nbytes);
    switch(var_code) {

    case _Integer:
    	RFB(read_buffer, &gi, sizeof(g_integer), *nread, nbytes);
	putInteger(*gv, gi);
	break;

    case _Real:
    	RFB(read_buffer, &gr, sizeof(g_real), *nread, nbytes);
	DTOI(&gr);
	putReal(*gv, gr);
	break;

    case _String:    
    	RFB(read_buffer, &slen, sizeof(int), *nread, nbytes);
	cbuff = new char[slen];
    	RFB(read_buffer, cbuff, slen, *nread, nbytes);
	putString(*gv, cbuff, slen-1);
	delete []cbuff;
	break;

    case _Workspace:
    	RFB(read_buffer, &nfields, sizeof(int), *nread, nbytes);
	*gv = newGworkspace(nfields);
	for(i=0; i<nfields; i++)
	{
    	    RFB(read_buffer, &slen, sizeof(int), *nread, nbytes);
	    cbuff = new char[slen];
    	    RFB(read_buffer, cbuff, slen, *nread, nbytes);
	    if (readGvarIn(read_buffer, &tgv, nbytes, nread) != 0)
	    {
		return -1;
	    }
	    addToWorkspace(*gv, cbuff, tgv);
	    delete[] cbuff;
	}
	break;

    case _IntegerArray:
    	RFB(read_buffer, &alen, sizeof(long), *nread, nbytes);
	ibuffer = (g_integer*) M_alloc(alen*sizeof(g_integer));
    	RFB(read_buffer, ibuffer, alen*sizeof(g_integer), *nread, nbytes);
	*gv = newGarrayInteger(alen);
	arrayDopeOneD(*gv, alen);
	for(i=0; i<alen; i++)
	{
	    arrayPutInteger(*gv, i, ibuffer[i]);
	}
	break;

    case _RealArray:
    	RFB(read_buffer, &alen, sizeof(long), *nread, nbytes);
	rbuffer = (g_real*) M_alloc(alen*sizeof(g_real));
    	RFB(read_buffer, rbuffer, alen*sizeof(g_real), *nread, nbytes);
	*gv = newGarrayReal(alen);
	arrayDopeOneD(*gv, alen);
	for(i=0; i<alen; i++)
	{
	    DTOI(rbuffer+i);
	    arrayPutReal(*gv, i, rbuffer[i]);
	}
	break;

    case _StringArray:
    	RFB(read_buffer, &alen, sizeof(long), *nread, nbytes);
	*gv = newGarrayString(alen);
	arrayDopeOneD(*gv, alen);
	for(i=0; i<alen; i++)
	{
    	    RFB(read_buffer, &slen, sizeof(int), *nread, nbytes);
	    cbuff = new char[slen];
    	    RFB(read_buffer, cbuff, slen, *nread, nbytes);
	    arrayPutString(*gv, i, cbuff, strlen(cbuff));
	    delete []cbuff;
	}
	break;

    case _WorkspaceArray:
    	RFB(read_buffer, &alen, sizeof(long), *nread, nbytes);
	*gv = newGarrayWorkspace(alen);
	arrayDopeOneD(*gv, alen);
	for(i=0; i<alen; i++)
	{
    	    if (readGvarIn(read_buffer, &tgv, nbytes, nread) != 0)
	    {
		return -1;
	    }
	    arrayPutWorkspace(*gv, i, getWorkspace(tgv));
	}
	break;

    case _Undefined:
	undefineGvar(*gv);
	break;
 
    default:
	return -1;
	break;
    }
    return 0;
}

int makeG3FileHeader(int fd)
{
    int nrec = 0;
    size_t dummy = 0;
//    off_t pos;
//    pos = lseek(fd, 0, SEEK_CUR);
    faalseek(fd, 0, SEEK_SET);
    faawrite(fd, G3_FILE_HEADER, START_OF_G3_FILE, dummy);
    faawrite(fd, &nrec, sizeof(int), dummy);
//    faalseek(fd, pos, SEEK_SET);
    return 0;
}
    
int updateG3FileHeader(int fd, int ninc)
{
    size_t dummy=0;
    int nrec;
    off_t pos;
    pos = lseek(fd, 0, SEEK_CUR);
    faalseek(fd, START_OF_G3_FILE, SEEK_SET); 	// Place after header and version info
    faaread(fd, &nrec, sizeof(int), dummy);
    nrec += ninc;
    faalseek(fd, -sizeof(int), SEEK_CUR);
    faawrite(fd, &nrec, sizeof(int), dummy);
    faalseek(fd, pos, SEEK_SET);
    return nrec;
}
    
/* 
 * read length bytes from buffer and increment buffer pointer
 */
int readFromBuffer(char** buffer, void* p_var, size_t length)
{
    memcpy(p_var, *buffer, length);
    *buffer += length;
    return 0;
}


int openG3File(const char* file, file_open_mode o_mode, int* blocks)
{
    int fd, flags;
//    size_t dummy;
    if ( (file = translatePath(file, 'c')) == NULL )
	return -1;
    switch (o_mode) {

    case OpenAppend:
	flags = O_RDWR | O_CREAT;
	fd = open(file, flags, 0644);
        faalseek(fd, START_OF_G3_FILE, SEEK_SET); 	// Place after header and version info
	if (read(fd, blocks, sizeof(int)) == sizeof(int) && *blocks > 0)
	{
	    lseek(fd, 0, SEEK_END);
	}
	else
	{
	    *blocks = 0;
	    makeG3FileHeader(fd);
	}
	break;

    case OpenNew:
	flags = O_RDWR | O_CREAT | O_TRUNC;
	fd = open(file, flags, 0644);
	*blocks = 0;
	makeG3FileHeader(fd);
	break;

    case OpenRead:
	flags = O_RDONLY;
	fd = open(file, flags, 0);
        faalseek(fd, START_OF_G3_FILE, SEEK_SET); 	// Place after header and version info
	if ((read(fd, blocks, sizeof(int)) != sizeof(int)) || (*blocks <= 0))
	{
	    return -1;
	}
	break;

    default:
	return -1;
	break;
    }
    return(fd);
}

int closeG3File(int fd)
{
    return(close(fd));
}

int dirG3File(int fd, char*** directory, size_t **sizes)
{
    int nblocks, i, mlen;
    size_t nbytes, dummy=0;
    off_t pos;
    pos = lseek(fd, 0, SEEK_CUR);
    faalseek(fd, START_OF_G3_FILE, SEEK_SET);
    faaread(fd, &nblocks, sizeof(int), dummy);
    *directory = (char**)M_alloc((nblocks+1)*sizeof(char*));
    *sizes = (size_t*)M_alloc(nblocks*sizeof(size_t));
    for(i=0; i<nblocks; i++)
    {
	faaread(fd, &mlen, sizeof(int), dummy);
	(*directory)[i] = (char*)M_alloc(mlen*sizeof(char));
	faaread(fd, (*directory)[i], mlen, dummy);
	faaread(fd, &nbytes, sizeof(size_t), dummy);
	(*sizes)[i] = nbytes;
	faalseek(fd, nbytes, SEEK_CUR);
    } 
    (*directory)[nblocks] = NULL;
    faalseek(fd, pos, SEEK_SET);
    return 0;
}
    
int seekG3FileBlock(int fd, int block)
{
    int nblocks, i, mlen;
    size_t nbytes, dummy=0;
    faalseek(fd, START_OF_G3_FILE, SEEK_SET);
    faaread(fd, &nblocks, sizeof(int), dummy);
    if ( block > nblocks || block < 0 )
    {
 	 return -1;
    }
    for(i=1; i<block; i++)
    {
	faaread(fd, &mlen, sizeof(int), dummy);
	faalseek(fd, mlen, SEEK_CUR);
	faaread(fd, &nbytes, sizeof(size_t), dummy);
	faalseek(fd, nbytes, SEEK_CUR);
    } 
    return 0;
}

int seekG3FileMarker(int fd, const char *marker)
{
    char* cbuff;
    int not_done, i, nblocks, mlen;
    size_t nbytes, dummy=0;
    faalseek(fd, START_OF_G3_FILE, SEEK_SET);
    faaread(fd, &nblocks, sizeof(int), dummy);
    nbytes = 0;
    not_done = 1;
    for(i=1; i<=nblocks && not_done; i++)
    {
	faalseek(fd, nbytes, SEEK_CUR);
	faaread(fd, &mlen, sizeof(int), dummy);
	cbuff = new char[mlen];
	faaread(fd, cbuff, mlen, dummy);
	not_done = strcmp(marker, cbuff);
	delete []cbuff;
	faaread(fd, &nbytes, sizeof(size_t), dummy);
    }
    if ( not_done ) 
    {
	return -1;
    }
    faalseek(fd, -(mlen + sizeof(int) + sizeof(size_t)), SEEK_CUR);
    return i;
}

#endif
/*
 * translate a file specification onto the form appropriate for
 * opening
 *
 * opt can be 'c' to make it return a path appropriate for a C open
 *            'f'                                         Fortran
 * returns NULL on an invalid option error
 */

// Helper routine for decc$to_vms()
static const char* action_result = 0;
static int action_routine(char* unix_file, int type)
{
    action_result = unix_file;
    return 1;
}

const char* translatePath(const char *path, char opt)
{
#if defined(__VMS_POSIX)
    int mode;
    const char *result;
    switch(opt)
    {
	case 'c':
	case 'C':
	    mode = _TO_PATHNAME;	/* files opened in C must have a POSIX path */
	    break;
	case 'f':
	case 'F':
	    mode = _TO_VMSNAME;		/* files opened in FORTRAN must have a VMS path */
	    break;
	default:
	    return(NULL);
	    break;
    }
    result = _translate(path, mode);
    return( result == NULL ? path : result );
#elif defined(__VMS)
    int status;
    const char *result;
    switch(opt)
    {
	case 'c':	// DEC C can take either style, but go for UNIX style for compatability with POSIX
	case 'C':
	    result = decc$translate_vms(path);
	    return(result == 0 || (long)result == -1 ? path : result);

	case 'f':	// FORTRAN can only cope with VMS style
	case 'F':
	    status = decc$to_vms(path, action_routine, 0, 0);
	    return(status > 0 && action_result != 0 ? action_result : path);
    }
#else
    return path;
#endif
}

void translatePath(String& path, char opt)
{
    const char *result = translatePath((const char*)path, opt);
    path = result;
}

FileType getFileType(const char* fileName)
{
// The union is here to ensure the buffer is properly aligned (the test for HDF requires this)
    union { unsigned u; unsigned char c[FILETYPE_BUFFER_LENGTH+1]; } header_buffer_union;
    unsigned char* header_buffer = header_buffer_union.c;
    char error_buffer[200];
//    fort_int *temp;
    int i, fd, nread;
    struct stat stat_buffer;

/*
 * Open the file and read in the first FILETYPE_BUFFER_LENGTH bytes - these will
 * be used to determine the type of the file
 */
#if defined(__VMS) && !defined(__VMS_POSIX)
    if ((fd = open((char*)fileName, O_RDONLY, 0, "shr=put", "ctx=stm")) < 0)
#else
	if ((fd = open(fileName, O_RDONLY, 0)) < 0)
#endif /* __VMS */
    {
	sprintf(error_buffer, "Unable to open file \"%s\" - %s", fileName, strerror(errno));
	error_add("fileType", error_buffer, "Check file exists and is readable");
 	return(FileTypeError);
    }
    if (fstat(fd, &stat_buffer) != 0)
    {
	gerr << "Unable to stat() \"" << fileName << "\": " << strerror(errno) << endl;
	close(fd);
	return FileTypeError;
    }
    if (S_ISDIR(stat_buffer.st_mode))
    {
	close(fd);
	return FileTypeDirectory;
    }
    nread = read(fd, (char*)header_buffer, FILETYPE_BUFFER_LENGTH);
    if (nread == -1)
    {
	sprintf(error_buffer, "Unable to read %i bytes from file %s - %s", (int)FILETYPE_BUFFER_LENGTH, 
		fileName, strerror(errno));
	error_add("fileType", error_buffer, "Check file exists and is readable");
	close(fd);
 	return(FileTypeError);
    }

    if (close(fd) != 0)
    {
	sprintf(error_buffer, "Unable to close file %s - %s", fileName, strerror(errno));
	error_add("fileType", error_buffer, "");
 	return(FileTypeError);
    } 
/*
 *   Now calculate file type using characters from header_buffer
 */
    if (!strncmp((char*)header_buffer, G3_FILE_HEADER_BASE, strlen(G3_FILE_HEADER_BASE)))
    {
	return(FileTypeG3Intermediate);
    }

    if ((header_buffer[80] == 1) && (header_buffer[84] == 42))
    {
	return(FileTypeG2Intermediate);
    }
/*
 * look at the "address of RUN and INST section" attribute - if there, must be an ISIS raw file
 */
    if ((header_buffer[84] == 32) && (header_buffer[88] == 126))
    {
	return(FileTypeRaw);
    }

/*
 * CRPT is like a RAW file, but is aligned differently to allow for a "status" section
 */
    if ((header_buffer[84] == 129) && (header_buffer[88] == 1))
    {
	return(FileTypeCRPT);
    }
/*
 * HDF files have magic cookie 0x0e031301 in the first 4 bytes
 */
    if (ntohl(header_buffer_union.u) == 0x0e031301)
    {
	return FileTypeHDF;
    }
/*
 * check for open genie ASCII file
 */
    if (!strncmp((char*)header_buffer, G3_ASCII_FILE_HEADER_BASE, strlen(G3_ASCII_FILE_HEADER_BASE)))
    {
	return FileTypeOpenGenieASCII;
    }
/*
 * check for ASCII data file
 */
    int is_ascii = 1;
    for(i=0; i<nread; i++)
    {
	if (!isascii(header_buffer[i]))
            is_ascii = 0;
    }
    if (is_ascii)
	return FileTypeASCII;

    return(FileTypeUnknown);
}
