/*  Open GENIE, a program for neutron scattering data analysis & display
    ISIS facility, Rutherford Appleton Laboratory, Chilton, Didcot, OXON UK
    Copyright (C) 1996,1997

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA. */

/*
 * header file for routines to output G3 intermediate file
 *
 *  Written by Freddie Akeroyd, RAL, 1994
 */
#ifndef _G3IO_H_
#define _G3IO_H_

/* 
 * keep version numbers only one digit in size and do not change G3_FILE_HEADER_BASE
 * this will ebsure START_OF_G3_FILE is always correct 
 */
#define G3IO_MAJOR_VERSION	"1"
#define G3IO_MINOR_VERSION	"0"

#define G3_ASCII_FILE_HEADER_BASE 	"# Open Genie ASCII File #"
/* 
 * G3IO version hostory:
 *     1.0    Initial release version
 */

#define G3_FILE_HEADER_BASE 	"Open Genie Intermediate file V"
static const char* G3_FILE_HEADER = G3_FILE_HEADER_BASE G3IO_MAJOR_VERSION "." G3IO_MINOR_VERSION ;
static const int START_OF_G3_VERSION = strlen(G3_FILE_HEADER_BASE) * sizeof(char);
static const int START_OF_G3_FILE = strlen(G3_FILE_HEADER) * sizeof(char);
/* 
 * BUFFER_LENGTH must be at least as long as the longest header to be read 
 * currently: G3_FILE_HEADER or G2_FILE_HEADER
 */
static const int FILETYPE_BUFFER_LENGTH = 100;
/*  
 * G2 Intermediate file header starts with 80 spaces and a few special characters
 * hopefully this will be enough for a unique signature
 */
static const char* G2_FILE_HEADER = 
    "                                                                                \001\0\0\0\052";

typedef enum {
	FileTypeError = -1,
	FileTypeUnknown = 0,
    	FileTypeRaw = 1,
    	FileTypeG2Intermediate = 2,
    	FileTypeG3Intermediate = 3,
	FileTypeCRPT = 4,
	FileTypeHDF = 5,
	FileTypeASCII = 6,		/* plain ASCII file */
	FileTypeDirectory = 7,
	FileTypeOpenGenieASCII = 8
    } FileType;
	
static const char* fileTypeNames[] = { "<unknown>", "raw", "G2 intermediate", "G3 intermediate", "CRPT", "HDF", "ASCII", "<directory>", "G3 ASCII", 0 };

typedef enum { OpenAppend = 1, OpenNew = 2, OpenRead  = 3 } file_open_mode;

/*
 * the following are used to make sure that files
 * are always output in VAX floating point style whether
 * on a VAXstation or a DECstation
 */
#include "int_convert.h"
#include "float_convert.h"

/*
 * the following is to overcome the problem of pathnames on POSIX
 * needing to be given in POSIX form for C opens and VMS form for
 * FORTRAN open. The routines will allow filenames to be given
 * in either form and convert to the one appropriate.
 */
#ifdef __VMS_POSIX
extern "C" {
char* _translate(const char *path, int mode);
	}
#endif /* ifdef __VMS_POSIX */

/* prototypes */
int writeGvar(int fd, g_variable gv, const char* marker);
int readGvar(int fd, g_variable* gv, const char** marker);
int writeGvarOut(int fd, g_variable gv, ssize_t* nbytes);
int readGvarIn(char** read_buffer, g_variable *gv, size_t nbytes, ssize_t *nread);
int updateG3FileHeader(int fd, int ninc);
int readFromBuffer(char** buffer, void* p_var, size_t length);
int openG3File(const char* file, file_open_mode o_mode, int* blocks);
int closeG3File(int fd);
int dirG3File(int fd, char*** directory, size_t **sizes);
int seekG3FileBlock(int fd, int block);
int seekG3FileMarker(int fd, const char *marker);

const char* translatePath(const char *path, char opt);
void translatePath(String& path, char opt);

FileType getFileType(const char* fileName);

#endif	/* ifdef _G3IO_H_ */
