/*  Open GENIE, a program for neutron scattering data analysis & display
    ISIS facility, Rutherford Appleton Laboratory, Chilton, Didcot, OXON UK
    Copyright (C) 1996,1997

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA. */

#include "geniedefs.h"
#include <iostream.h>

#ifdef _WIN32
#include <strstrea.h>
#else
#include <strstream.h>
#endif

#include <string.h>
#include <stdlib.h>
#include <setjmp.h>

#include "smalltalk.h"
#include "g_extensions.h"
#include "genievars.h"
#include "parsedefs.h"
#include "parse_funcs.h"
#include "smalltalk_object.h"
#include "gsignals.h"
//#include "extra_genie_commands.h"

//genie_command_t* extra_genie_command_lists = 0;

//********************************************************************************
// C++ extensions hard coded from the smalltalk internals and loaded via
// stextensions.gcl.
//
// In general these are only things which would be tricky to do in smalltalk
// and are needed by GENIE.
//********************************************************************************

static const char* rcsid = "@(#)$Id: g_extensions.cxx,v 1.32 1997/06/29 15:15:04 faa Exp $";
USE_RCSID(rcsid);

// Callin from Smalltalk
// =====================

// Return nilOOP on failure
OOP g_extensions( extension_function_code function, OOP oopParameters )
{

    OOP result(nilOOP);

    BEGINSAVEBLOCK

    switch(function)				    // NOTE: each part of the CASE is responsible for doing a return OOP;
    {
    case Extension_test:
    	gout << "Extensions test called" << endl;
    	break;

    case String_to_gvar:
	    {
            g_real realvar;
            g_integer intvar;
            GenieString numstring(oopParameters, 1);

            result = (OOP)numstring;
            istrstream iss((const char*)numstring);

            if ( numstring.contains('.') || numstring.contains('e') || numstring.contains('E') )
	        {
    		    iss >> realvar;
	    	    if( !iss.fail() ) result = (OOP)GenieReal(realvar);
	        }
	        else
	        {
		        iss >> intvar;
		        if ( !iss.fail() ) result = (OOP)GenieInteger(intvar);
	        }
  	    }
        break;

    case StreamMode:        // print a GENIE string on the standard output
        {
            GenieString  fd(oopParameters, 1);
            GenieInteger mode(oopParameters, 2);
            if (mode.defined())
            {
                switch (fd[0])
                {
                case 'o': result = (OOP)GenieInteger(gout.mode(mode)); break;
                case 'i': result = (OOP)GenieInteger(ginf.mode(mode)); break;
                case 'e': result = (OOP)GenieInteger(gerr.mode(mode)); break;
                case 'd': result = (OOP)GenieInteger(gdbg.mode(mode)); break;
                }
            }
	    else
	    {
                switch (fd[0])
                {
		case 'o': result = (OOP)GenieInteger(gout.mode()); break;
		case 'i': result = (OOP)GenieInteger(ginf.mode()); break;
		case 'e': result = (OOP)GenieInteger(gerr.mode()); break;
		case 'd': result = (OOP)GenieInteger(gdbg.mode()); break;
		}
	    }
        }
        break;

    case StringPrint:    // print a GENIE string on the standard output
        {
            GenieString gs(oopParameters, 1);
            GenieString fd(oopParameters, 2);
            if (gs.defined())
            {
                switch (fd[0])
                {
                case 'o': gout << gs << flush; break;
                case 'i': ginf << gs << flush; break;
                case 'E': gerr << gs << flush; break;
                case 'd': gdbg << gs << flush; break;
                case 'e': *g_err << gs << flush; break;  // No line numbers & logging
                }
            }
        }
        break;

    case StringRead:     // print a GENIE string on the standard output
        {
            static char buffer[SHORTCHARBUFF+1];
            gin.getline(buffer, SHORTCHARBUFF);
            result = (OOP)GenieString(buffer);
        }
        break;

    case MangleAs:
	    {
            GenieString self(oopParameters, 1);
            GenieString as(oopParameters, 2);
            char buff[SHORTCHARBUFF+3];		// add on some extra for gX and \0 etc.

	    int len = self.length();
	    if (len < SHORTCHARBUFF)
	    {
                strncpy(buff, (const char*)self, len);
		buff[len] = '\0';
                if ( as == "procedure" )
                    MangleProc(buff);
                else if ( as == "variable" )
                    MangleVar(buff);
                else if ( as == "type" )
                    MangleType(buff);
	    }
	    else
	    {
		gerr << "MangeAs: name too long" << endl;
		buff[0] = '\0';
	    }
            self = buff;
            result = (OOP)self;
        }
        break;

    case Demangle:
	    {
            GenieString self(oopParameters, 1);
            char buff[SHORTCHARBUFF+1];		// for '\0'

	    if (self.length() < SHORTCHARBUFF)
	    {
                strcpy(buff, (const char*)self);
                DeMangle(buff);
	    }
	    else
	    {
		gerr << "DeMangle: name too long" << endl;
		buff[0] = '\0';
	    }
            self = buff;
            result = (OOP)self;
  	    }
        break;

    case QuitTheLot:
        Kill_smalltalk(1);
        break;

    case LoadTheCode:
	    {
            GenieString fileName(oopParameters, 1);
            Unset_signals();		// Only works to 1 level of recursion !!
            result = (load_file(fileName, fileName, BATCH, (time_t)0) != 0 ? falseOOP : trueOOP);
            Set_signals();
  	    }
        break;

#if 0
    case AddCommandToReadline:
    {
	GenieString command(oopParameters, 1), comment(oopParameters, 2);
	
	break;
    }
#endif

    default:
    	PERROR("Undefined extension");
    }

    ENDSAVEBLOCK

    return result;
}
