/*  Open GENIE, a program for neutron scattering data analysis & display
    ISIS facility, Rutherford Appleton Laboratory, Chilton, Didcot, OXON UK
    Copyright (C) 1996,1997

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA. */

#ifndef G_MODULE
#define G_MODULE

// Forward declarations
struct STRING;
class GenieWorkspace;

#include "module.h"

#include "Pginclude.h"		// for STRING
#include "fortran_types.h"

extern Module genie_modules;

typedef enum {
    Module_test = 0,
    Module_load = 1,
    Module_execute = 2,
    Module_list = 3,
    Module_unload = 4
} module_function_code;

OOP g_module(module_function_code function, OOP arguments);

#if defined(HAVE_LIBF2C)	/* f2c mangles with "__" for names already containing an "_", else as per UNIX */
#define g3_module_get_real g3_module_get_real__
#define g3_module_put_real g3_module_put_real__
#define g3_module_get_double g3_module_get_double__
#define g3_module_put_double g3_module_put_double__
#define g3_module_get_int g3_module_get_int__
#define g3_module_put_int g3_module_put_int__
#define g3_module_get_string g3_module_get_string__
#define g3_module_put_string g3_module_put_string__
#define g3_module_string_array_alloc g3_module_string_array_alloc__
#define g3_module_string_array_add g3_module_string_array_add__
#define g3_module_string_array_free g3_module_string_array_free__
#define g3_module_get_real_array g3_module_get_real_array__
#define g3_module_put_real_array g3_module_put_real_array__
#define g3_module_put_nd_real_array g3_module_put_nd_real_array__
#define g3_module_get_double_array g3_module_get_double_array__
#define g3_module_put_double_array g3_module_put_double_array__
#define g3_module_get_int_array g3_module_get_int_array__
#define g3_module_put_int_array g3_module_put_int_array__
#define g3_module_print g3_module_print__
#define g3_module_open_stdio g3_module_open_stdio__
#define g3_module_close_stdio g3_module_close_stdio__
#define g3_module_get_version g3_module_get_version__
#define g3_module_version_ok g3_module_version_ok__
#elif defined(__unix)		/* UNIX mangles all FORTRAN names with one "_" */
#define g3_module_get_real g3_module_get_real_
#define g3_module_put_real g3_module_put_real_
#define g3_module_get_double g3_module_get_double_
#define g3_module_put_double g3_module_put_double_
#define g3_module_get_int g3_module_get_int_
#define g3_module_put_int g3_module_put_int_
#define g3_module_get_string g3_module_get_string_
#define g3_module_put_string g3_module_put_string_
#define g3_module_string_array_alloc g3_module_string_array_alloc_
#define g3_module_string_array_add g3_module_string_array_add_
#define g3_module_string_array_free g3_module_string_array_free_
#define g3_module_get_real_array g3_module_get_real_array_
#define g3_module_get_double_array g3_module_get_double_array_
#define g3_module_put_real_array g3_module_put_real_array_
#define g3_module_put_nd_real_array g3_module_put_nd_real_array_
#define g3_module_put_double_array g3_module_put_double_array_
#define g3_module_get_int_array g3_module_get_int_array_
#define g3_module_put_int_array g3_module_put_int_array_
#define g3_module_print g3_module_print_
#define g3_module_open_stdio g3_module_open_stdio_
#define g3_module_close_stdio g3_module_close_stdio_
#define g3_module_get_version g3_module_get_version_
#define g3_module_version_ok g3_module_version_ok_
#endif

extern "C"
{
    void g3_module_get_real(GenieWorkspace& pars, const STRING& name, fort_real& val);
    void g3_module_put_real(GenieWorkspace& res, const STRING& name, const fort_real& val);
    void g3_module_get_double(GenieWorkspace& pars, const STRING& name, fort_double& val);
    void g3_module_put_double(GenieWorkspace& res, const STRING& name, const fort_double& val);
    void g3_module_get_string(GenieWorkspace& pars, const STRING& name, STRING& val);
    void g3_module_put_string(GenieWorkspace& res, const STRING& name, const STRING& val);
    void g3_module_string_array_alloc(const fort_int& length, void*& handle);
    void g3_module_string_array_add(void*& handle, const fort_int& element, const STRING& val);
    void g3_module_string_array_free(GenieWorkspace& res, const STRING& name, void*& handle);
    void g3_module_get_int(GenieWorkspace& pars, const STRING& name, fort_int& val);
    void g3_module_put_int(GenieWorkspace& res, const STRING& name, const fort_int& val);
    void g3_module_get_real_array(GenieWorkspace& pars, const STRING& name, fort_real* val, fort_int& len);
    void g3_module_get_double_array(GenieWorkspace& pars, const STRING& name, fort_double* val, fort_int& len);
    void g3_module_get_int_array(GenieWorkspace& pars, const STRING& name, fort_int* val, fort_int& len);
    void g3_module_put_real_array(GenieWorkspace& pars, const STRING& name, const fort_real* val, const fort_int& len);
    void g3_module_put_double_array(GenieWorkspace& pars, const STRING& name, const fort_double* val, const fort_int& len);
    void g3_module_put_int_array(GenieWorkspace& pars, const STRING& name, const fort_int* val, const fort_int& len);
    void g3_module_print(const STRING& s, const fort_int& option);
    void g3_module_open_stdio(char* itty, fort_int& tty_len);
    void g3_module_close_stdio(fort_int& stdio_unit);

    fort_int g3_module_put_nd_real_array(GenieWorkspace& pars, const STRING& name, const fort_real* val, 
						const fort_int* dims_array, const fort_int& ndims);
    fort_int g3_module_get_version(fort_int& major, fort_int& minor);
    fort_int g3_module_version_ok(const fort_int& major, const fort_int& minor);
}

// These are only called from C++, so need to be extern "C"
// They allocate space for the array with malloc() if (*val == 0)
void g3_module_get_real_array_alloc(GenieWorkspace& pars, const STRING& name, fort_real** val, fort_int& len);
void g3_module_get_double_array_alloc(GenieWorkspace& pars, const STRING& name, fort_double** val, fort_int& len);
void g3_module_get_int_array_alloc(GenieWorkspace& pars, const STRING& name, fort_int** val, fort_int& len);

extern "C"
{
#if HAVE_LIBF2C
#       define f_map_fortran_stdio f_map_fortran_stdio__
#       define f_unmap_fortran_stdio f_unmap_fortran_stdio__
#       define module_test_function module_test_function__
#       define sample_genie_coord_tran_func sample_genie_coord_tran_func__
#elif defined(__unix)
#       define f_map_fortran_stdio f_map_fortran_stdio_
#       define f_unmap_fortran_stdio f_unmap_fortran_stdio_
#       define module_test_function module_test_function_
#       define sample_genie_coord_tran_func sample_genie_coord_tran_func_
#endif
// The following two are in "module_test_function.f"
    void module_test_function(GenieWorkspace& pars, GenieWorkspace& res);       // "module_func_t"
    int sample_genie_coord_tran_func(int* flags, const g_real* val_in, g_real* val_out);        // "coord_tran_func_t"

    void f_map_fortran_stdio(const char*, fort_int&);
    void f_unmap_fortran_stdio();
}

#endif /* G_MODULE */
