/*  Open GENIE, a program for neutron scattering data analysis & display
    ISIS facility, Rutherford Appleton Laboratory, Chilton, Didcot, OXON UK
    Copyright (C) 1996,1997

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA. */

#include "geniedefs.h"

#ifdef HAVE_UNISTD_H
#	include <unistd.h>
#endif

#include <stdio.h>
#include <math.h>
#include <string.h>
#include <gnustring.h>
#include <time.h>
#include "genie_types.h"
#include "units.h"

#define	MAX(__a, __b)	((__a) < (__b) ? (__b) : (__a))

#define DEFINED(__a)	((__a) != DBL_MAX)	/* to handle undefined real genie variables */

#include <iostream.h>

#include "smalltalk.h"
#include "genievars.h"
#include "g_testing.h"
#include "g_fortextern.h"

const char* translatePath(const char *path, char opt);

static int newGRebin(const GenieRealArray& xnew, const GenieWorkspace& oldwksp, GenieWorkspace& newwksp);

static int newGRebin(const GenieRealArray& xold, const GenieRealArray& yold, const GenieRealArray& eold,
              const GenieRealArray& xnew, GenieRealArray& ynew, GenieRealArray& enew);

static int TOFToEnergy(g_real scale, g_integer emode, g_real efixed, GenieRealArray& xarray, 
		g_integer ntc, g_real l1, g_real l2);

static int newGUnits(GenieWorkspace& worksp, g_integer iunit, g_real emin, g_real emax);
static int focusGRange(GenieWorkspace& wksp, GenieWorkspace& pars, GenieRealArray& xarray,
                g_integer unit_type, c_g_string file, g_integer start, g_integer end);


static int energyToTOF(g_real scale, g_integer emode, g_real efixed, GenieRealArray& xarray, 
		g_integer ntc, g_real l1, g_real l2);
static g_integer truncate(GenieRealArray& xarray, GenieRealArray& yarray, GenieRealArray& earray, g_integer np, g_real xmin, g_real xmax);
const char* unit_name(int iunit, char axis);
int unit_code(const char* cunit, char axis);


static OOP example1(OOP parameters);
static OOP example2(OOP parameters);
static OOP newUnits(OOP parameters);
static OOP newRebin(OOP parameters);
static OOP focusArray(OOP parameters);
static OOP focusRange(OOP parameters);

extern int associateGenieVariables(OOP oopParameters, ... );

// Return nilOOP on failure
OOP g_testing(testing_function_code function, OOP oopParameters)
{
//    if (!validParameters(oopParameters))
//    {
//	PERROR("g_testing(invalid parameters)");
//	return oopParameters; 
//    }
    OOP result(oopParameters);
    static char buffer[SHORTCHARBUFF];
#ifdef CALL_TIMING
    time_t clock;
    time(&clock);
#endif  /* CALL_TIMING */

    BEGINSAVEBLOCK

    switch(function)
    {
	case TestCall:
        {
	    GenieRealArray p1, p2;
	    associateGenieVariables(oopParameters, "p1", &p1, "p2", &p2, 0);
	    p1.inspect();
	    p2.inspect();
	    break;
	}

	case Example1:
	    result = example1(oopParameters);
	    break;

	case Example2:
	    result = example2(oopParameters);
	    break;

	case NewUnits:
	    result = newUnits(oopParameters);
       	    break;  	

	case NewRebin:
	    result = newRebin(oopParameters);
	    break;  	

	case FocusRange:
	    result = focusRange(oopParameters);
	    break;

        default:
    	    cerr << "g_testing(unknown function)" << endl;
	    result = nilOOP;
	    break;
    }
#ifdef CALL_TIMING
    sprintf(buffer, "g_testing call duration = %d\n", time(NULL) - clock);
    io_write_channel(Stderr, buffer);
#endif  /* CALL_TIMING */
//    fullGC();

    ENDSAVEBLOCK

    return result;
}                

static OOP example2(OOP parameters)
{
#if 0
    GenieRealArray a,b;
    GenieInteger num;
    GenieString junk("hello");
    associateGenieVariables(parameters, "a", &a, "b", &b, "num", &num, NULL);
    for(int i=0; i<num; i++)
    {
	a(i, 1.0);
	b(i, 1.0);
    }    
#endif
    GenieWorkspace worksp;
    worksp.associate(parameters, "WORKSP");
    cout << worksp << endl;
    worksp("TEST", 1.0);
    return parameters;
}

static OOP newRebin(OOP parameters)
{
#if 0
    GenieWorkspace oldwksp, newwksp;
    GenieRealArray xnew;
    associateGenieVariables(parameters, "OLDWKSP", &oldwksp, "NEWWKSP", &newwksp, "XNEW", &xnew, NULL);
    newGRebin(xnew, oldwksp, newwksp);
#endif
    return parameters;
}

static OOP focusRange(OOP parameters)
{
#if 0
    GenieInteger unit_type, start, end;
    GenieWorkspace wksp, pars;
    GenieRealArray xarray;
    GenieString file;
    
    associateGenieVariables(parameters, "IUNIT", &unit_type, "WKSP", &wksp, "XARRAY", &xarray,
					"START", &start, "END", &end, "PARS", &pars, "FILE", &file, NULL);

    focusGRange(wksp, pars, xarray, unit_type, (const char *)file, start, end);
#endif
    return parameters;
}

static int focusGRange(GenieWorkspace& wksp, GenieWorkspace& pars, GenieRealArray& xarray,
                g_integer unit_type, c_g_string file, g_integer start, g_integer end)
{
#if 0
    fort_int ndet, ndetmax, ntc, ntcmax, nuse, errcode;
//    int nspec = end - start + 1;
    fort_real l1, l2, delta, fi, duration, TWOTHETA;
    String comb_time, run_title, user_name, inst_name, run_no, junk;
    g_real denom, yval, emin=0.0, emax=0.0;
    GenieRealArray ttt, yarray, earray, xxarray, l1_array = pars("L1");
    GenieSpectrum wksp1(27), wksp2(27), wksp3(27);
    
    int quick, i;
    String runid(translatePath(file, 'f'));
    OPEN_DATA_FILE(runid, ntc, ndet, nuse, errcode);
    if (errcode != 0)
    {
	gerr << "Error in open_data_file" << endl;
 	return 1;
    }
    ndetmax = ndet + 1;
    ntcmax = ntc + 1;
    fort_real* user_tables = new fort_real[30];
    fort_real* x = new fort_real[ntcmax];
    fort_real* delt_work = new fort_real[ndetmax];
    fort_real* l2_work = new fort_real[ndetmax];
    fort_real* tthe_work = new fort_real[ndetmax];
    fort_int* spec_work = new fort_int[ndetmax];
    fort_int* idat = new fort_int[ntcmax];
    fort_int* idat_t = new fort_int[ntcmax];
    quick = 0;
// first spectrum
    READ_DATA(runid, errcode, start, delt_work, spec_work, tthe_work,
		l2_work, ndetmax, x, idat, ntcmax, l1, l2, TWOTHETA, delta,
		fi, run_title, duration, comb_time, user_name, inst_name, 
		run_no, user_tables, nuse, quick);
    l1 = l1_array(0);
    l2 = 0.0;
    TWOTHETA = 90.0;
    wksp1("X", GenieRealArray(x, ntcmax));
    yarray.alloc(ntcmax);
    earray.alloc(ntcmax);
    for(i=0; i<ntc; i++)
    {
	denom = x[i+1] - x[i];
// first Y is dummy - we want last to be
	yval = (g_real)idat[i+1] / denom;
	yarray(i, yval);
	earray(i, sqrt((double)idat[i+1]) / denom);
    }
    yarray(ntc, 0.0);
    earray(ntc, 0.0);
    wksp1("Y", yarray);
    wksp1("E", earray);
//    wksp("YVAL_MAX", yval_max);
    wksp1("EMODE", 0L);
    wksp1("EFIXED", 0.0);
    wksp1("L1", l1);
    wksp1("L2", l2);
    wksp1("TWOTHETA", TWOTHETA);
    wksp1("BIN_GROUPING", 0L);	// no grouping
    wksp1("NTC", ntc);
    wksp1("NUSE", nuse);
    wksp1("FI", fi);
    wksp1("HISTORY", " ");
    wksp1("TIME", comb_time);
    wksp1("TITLE", run_title);
    wksp1("UT", GenieRealArray(user_tables, nuse));
    wksp1("XLABEL", unit_name(2, 'x'));
    wksp1("YLABEL", unit_name(2, 'y'));
    wksp1("USER_NAME", user_name);
    wksp1("INST_NAME", inst_name);
    wksp1("RUN_NO", run_no);
    wksp1("FILE", runid);
    wksp1("DELTA", delta);
    wksp1("RUN_DURATION", duration);
//    junk = dec(start) + "-" + dec(end);	
    char buffer[100];
    sprintf(buffer, "Focus: %li to %li", (long)start, (long)end);
    wksp1("SPEC_NO", buffer);
    newGUnits(wksp1, unit_type, emin, emax);
    newGRebin(xarray, wksp1, wksp1);
// the rest
    quick = 1;
////// HACK!
    int j;
    for(i=start-1; i>=end; i--)
    {
    	READ_DATA(runid, errcode, start, delt_work, spec_work, tthe_work,
		l2_work, ndetmax, x, idat, ntcmax, l1, l2, TWOTHETA, delta, fi,
		run_title, duration, comb_time, user_name, inst_name, 
		run_no, user_tables, nuse, quick);
	l1 = l1_array(start-i);
    	l2 = 0.0;
    	TWOTHETA = PI / 2.0;
	wksp2("NTC", ntc);
    	wksp2("X", GenieRealArray(x, ntcmax));
    	wksp2("DELTA", delta);
    	wksp2("XLABEL", unit_name(2, 'x'));
    	wksp2("YLABEL", unit_name(2, 'y'));
    	wksp2("UT", GenieRealArray(user_tables, nuse));
	yarray.alloc(ntcmax);
	earray.alloc(ntcmax);
    	for(j=0; j<ntc; j++)
    	{	
	denom = x[j+1] - x[j];
// first Y is dummy - we want last to be
	yval = (g_real)idat[j+1] / denom;
	yarray(j, yval);
	earray(j, sqrt((double)idat[j+1]) / denom);
    	}
    	yarray(ntc, 0.0);
    	earray(ntc, 0.0);
    	wksp2("Y", yarray);
    	wksp2("E", earray);
//    wksp("YVAL_MAX", yval_max);
    	wksp2("EMODE", 0);
    	wksp2("EFIXED", 0.0);
    	wksp2("L1", l1);
    	wksp2("L2", l2);
    	wksp2("TWOTHETA", TWOTHETA);
    	wksp2("NTC", ntc);
    	wksp2("NUSE", nuse);
    	newGUnits(wksp2, unit_type, emin, emax);
    	newGRebin(xarray, wksp2, wksp2);
	wksp1 += wksp2;	
    }         
    wksp = wksp1;
    delete[] user_tables;
    delete []x;
    delete[] delt_work;
    delete[] l2_work;
    delete[] tthe_work;
    delete[] spec_work;
    delete[] idat;
    delete[] idat_t;
#endif
    return 0;
}

static OOP example1(OOP parameters)
{
#if 0
    GenieInteger one;
    GenieInteger two;    
    GenieStringArray three;
    GenieWorkspace four;
    cout << "Entering testing" << endl;
    associateGenieVariables(parameters, "one", &one, "two", &two, "three", &three, "four", &four, NULL);
    one = 3;
#if 0
    two = one + 6;
#endif
    cout << "one = " << one << endl;
    cout << "two = " << two << endl;
    cout << "three" << three << endl;
    cout << "four.x = " << four("x") << endl;
//    three(1, 5.0);
//    three(2, 7.0);
//    three(3, GenieReal(8.0));
    four("x", GenieInteger(3));
#endif
    return parameters;
}

/*
 * error codes:
 *-1 = points truncated in array
 * 0 = success
 * 1 = input unit code invalid
 * 2 = output unit code invalid
 * 3 = inelastic, not specified emode/efixed properly
 * 4 = input unit is channel number --- cannot convert
 * 5 = invalid l1, l2 or TwoTheta
 */

static OOP newUnits(OOP parameters)
{
    GenieInteger iunit;
    GenieWorkspace worksp;
    GenieReal emin, emax;
    associateGenieVariables(parameters, "IUNIT", &iunit, "WORKSP", &worksp, 
				        "EMIN", &emin, "EMAX", &emax, NULL);
    newGUnits(worksp, iunit, emin, emax);
    return parameters;
}

static int newGUnits(GenieWorkspace& worksp, g_integer iunit, g_real emin, g_real emax)
{
#if 0
    g_integer ntc = worksp("NTC");
    GenieRealArray xarray = worksp("X");
    GenieRealArray yarray = worksp("Y");
    GenieRealArray earray = worksp("E");
    c_g_string xunit_in = worksp("XLABEL");
    c_g_string yunit_in = worksp("YLABEL");
    g_real delta = worksp("DELTA");
    g_real l1 = worksp("L1");
    g_real l2 = worksp("L2");
    g_real TwoTheta = worksp("TWOTHETA");
    g_integer emode = worksp("EMODE");
    g_real efixed = worksp("EFIXED");
    g_real ymax;
    const char* xunit_out;
    const char* yunit_out;
    int i, j, error_code = 0, unit_convert = iunit;
    g_real factor, FourPi, SinTheta, ltot, step, t1, e2, t2; // ,e1
    unit_code_struct xunit_code_in, xunit_code_out, yunit_code_in,
		yunit_code_out;
//    cerr << "Starting main stuff" << endl;
//    for(i=0; i<10; i++)
//    {
//	cerr << xarray(i) << " " << yarray(i) << " " << earray(i) << endl;
//    }
//    cerr << "Delta = " << delta << endl;
/* only y can be dimensionless */
    if ( (xunit_code_in = (unit_code_struct) unit_code(xunit_in, 'x')) <= 0)
    {
	return -1;		
    }

    if ( xunit_code_in == 1 )
    {
	return -1;		
    }

    if ( (yunit_code_in = (unit_code_struct) unit_code(yunit_in, 'y')) == -1)
    {
	return -1;		
    }

    if ( unit_convert < 1 || unit_convert >= NUNIT )
    {
	printf("Error: invalid output unit code %i\n", unit_convert);
	return -1;
    }
    xunit_code_out = (unit_code_struct) iunit;
    yunit_code_out = ((yunit_code_in == 0) ? (unit_code_struct)0 : xunit_code_out);
/* 
 * deal with some easy options 
 */
    if ( xunit_code_in == xunit_code_out )
    {
//	cerr << "No change of units needed" << endl;
	return -1;
    }

/*
 * convert y and e to counts if not dimensionless
 */
    if ( yunit_code_in != 0 )
    {
	for(i=0; i<ntc; i++)
    	{
	    step = xarray(i+1) - xarray(i);
	    if DEFINED(yarray(i)) { yarray(i, yarray(i) * step); }
	    if DEFINED(earray(i)) { earray(i, earray(i) * step); }
        }
    }
    ltot = l1 + l2;
    SinTheta = sin(fabs(TwoTheta) * DEG2RAD / 2.0);
    FourPi = 4.0 * PI;
    if ( ltot == 0.0 )
    {
	printf("Error: total flight path l = 0\n");
	return -1;
    }
    if (( xunit_code_in >= 9 || xunit_code_out >= 9 ) && ( emode <= 0 || emode >= 3 || efixed <= 0.0 ))
    {
	printf("Error: emode (%i) or efixed (%g) wrongly specified\n", (int)emode, (double)efixed);
	return -1;
    }

/* 
 * now convert units to a common base of TOF in microseconds 
 * C1 and C2 defined in units.h
 */

//    cerr << "Convert 1" << endl;
    switch(xunit_code_in) {

    case Dimensionless:
	printf("Error: invalid xunit_in = %s\n", xunit_in);
	return -1;		
	break;

    case ChannelNo:
	return -1;
	break;

    case TOF:
//	for(i=0; i<ntc+1; i++)
//	{
//	    xarray(i, xarray(i) - delta);
//	}
	xarray -= delta;	
	break;

    case Wavelength:
	factor = ltot * C1;
	for(i=0; i<ntc+1; i++)
	{
	    xarray(i, xarray(i) * factor);
	}
	break;

    case dSpacing:
	factor = 2.0 * ltot * SinTheta * C1;
	for(i=0; i<ntc+1; i++)
	{
	    xarray(i, xarray(i) * factor);
	}
	break;

    case Energy:
	factor = C1 * ltot * sqrt(C2);
	for(i=0; i<ntc+1; i++)
	{
	    xarray(i, factor / sqrt(xarray(i)));
	}
	break;

    case MomentumTransfer:
	factor = FourPi * C1 * ltot * SinTheta;
	for(i=0; i<ntc+1; i++)
	{
	    xarray(i, factor / xarray(i));
	}
	break;

    case Qsquared:
	factor = FourPi * C1 * ltot * SinTheta;
	for(i=0; i<ntc+1; i++)
	{
	    xarray(i, factor / sqrt(xarray(i)));
	}
	break;

    case ReciprocalTOF:
	factor = ltot;
	for(i=0; i<ntc+1; i++)
	{
	    xarray(i, factor * xarray(i));
	}
	break;

    case DeltaE:
	energyToTOF(1.0, emode, efixed, xarray, ntc, l1, l2);
	break;

    case DeltaE_inWaveNumber:
	energyToTOF(MEV2WAVENUMBER, emode, efixed, xarray, ntc, l1, l2);
	break;

    case PrimaryWavelength:
	if ( emode == 2 )
	{
	    e2 = efixed;
	    t2 = C1 * l2 * sqrt(C2/e2);
	    factor = l1 * C1;
	    for(i=0; i<ntc+1; i++)
	    {
	    	t1 = factor * xarray(i);
		xarray(i, t1 + t2);
	    }
	}
	break;

    default:
	printf("Error: invalid xunit_in = %s\n", xunit_in);
	return -1;		
	break;
    }

//    cerr << "Convert 1 done" << endl;
/*
 * TOF cannot be -ve
 */
    for(i=0; i<ntc+1; i++)
    {
	if (xarray(i) <= 0.0)
	{
	    printf("Error: zero or negative time-of-flight\n");
	    return -1;		
	}
    }

/*
 * Now convert TOF to required units
 */
//    cerr << "Convert 2" << endl;
    switch(xunit_code_out) {

    case Dimensionless:
	error_code = 3;
	return -1;
	break;

    case ChannelNo:
	for(i=0; i<ntc+1; i++)
	{
	    xarray(i, (g_real)i + 1.0);
	}
	break;

    case TOF:
	for(i=0; i<ntc+1; i++)
	{
	    xarray(i, xarray(i) + delta);
	}
	break;

    case Wavelength:
	factor = ltot * C1;
	for(i=0; i<ntc+1; i++)
	{
	    xarray(i, xarray(i) / factor);
	}
	break;

    case dSpacing:
	factor = 2.0 * ltot * SinTheta * C1;
    	if (factor == 0.0)
    	{
	    error_code = 5;
	    printf("Error: L1+L2 or Sin(TwoTheta) is 0.0\n");
	    return -1;
    	}
//	for(i=0; i<ntc+1; i++)
//	{
//	    xarray(i, xarray(i) / factor);
//	}
	xarray /= factor;
	break;

    case Energy:
	factor = C1 * C1 * ltot * ltot * C2;
	for(i=0; i<ntc+1; i++)
	{
	    xarray(i, factor / (xarray(i) * xarray(i)));
	}
	break;

    case MomentumTransfer:
	factor = FourPi * C1 * ltot * SinTheta;
	for(i=0; i<ntc+1; i++)
	{
	    xarray(i, factor / xarray(i));
	}
	break;

    case Qsquared:
	factor = FourPi * C1 * ltot * SinTheta;
	factor = factor * factor;
	for(i=0; i<ntc+1; i++)
	{
	    xarray(i, factor / (xarray(i) * xarray(i)));
	}
	break;

    case ReciprocalTOF:
	factor = ltot;
	for(i=0; i<ntc+1; i++)
	{
	    xarray(i, xarray(i) / factor);
	}
	break;

    case DeltaE:
	TOFToEnergy(1.0, emode, efixed, xarray, ntc, l1, l2);
	break;

    case DeltaE_inWaveNumber:
	TOFToEnergy(MEV2WAVENUMBER, emode, efixed, xarray, ntc, l1, l2);
	break;


    case PrimaryWavelength:
	if ( emode == 2 )
	{
	    e2 = efixed;
	    t2 = C1 * l2 * sqrt(C2/e2);
	    factor = l1 * C1;
	    for(i=0; i<ntc+1; i++)
	    {
		t1 = xarray(i) - t2;
		xarray(i, (( t1 <= 0.0 ) ? ( -BIGVAL ) : ( t1 / factor )));
	    }
	}
	break;

    default:
	error_code = 2;
	printf("Error: invalid output unit\n");
	return -1;		
	break;
    }
//    cerr << "Convert 2 done" << endl;

/*
 * flip order 
 */
    if (xarray(0) > xarray(ntc))
    {
	for(i=0; i < ntc/2; i++)
	{
	    j = ntc - i - 1;

	    factor = yarray(i);
	    yarray(i, yarray(j));
	    yarray(j, factor);

	    factor = earray(i);
	    earray(i, earray(j));
	    earray(j, factor);
	}
	yarray(ntc, 0.0);
	earray(ntc, 0.0);
	for(i=0; i < (ntc+1)/2; i++)
	{
	    j = ntc - i ;
	    factor = xarray(i);
	    xarray(i, xarray(j));
	    xarray(j, factor);
	}
    }

//    cerr << "Flip done" << endl;
    if ( xunit_code_out == DeltaE || xunit_code_out == PrimaryWavelength )
    {
	ntc = truncate(xarray, yarray, earray, ntc+1, emin, emax) - 1;
    }

/*
 * now scale back y and e if not dimensionless
 */
    if (yunit_code_out != 0)
    {
    	for(i=0; i<ntc; i++)
    	{
	    step = xarray(i+1) - xarray(i);
	    if DEFINED(yarray(i)) { yarray(i, yarray(i) / step); }
	    if DEFINED(earray(i)) { earray(i, earray(i) / step); }
        }
    }

//    cerr << "Scaled back" << endl;

    ymax = 0.0;
    for(i=0; i<ntc; i++)
    {
	if ( DEFINED(yarray(i)) && (yarray(i) > ymax) )
    	{
	    ymax = yarray(i);
    	}
    }

    xunit_out = unit_name( (int)xunit_code_out, 'x');
    yunit_out = unit_name( (int)yunit_code_out, 'y');

    if ( (xunit_out == NULL) || (yunit_out == NULL) )
    {
	error_code = 2;
    }
//    cerr << "Replacing" << endl;

    worksp("YMAX", ymax);
    worksp("XLABEL", xunit_out);
    worksp("YLABEL", yunit_out);
#endif
    return 0;
}



/* remove all points outside range */

static g_integer truncate(GenieRealArray& xarray, GenieRealArray& yarray, GenieRealArray& earray, g_integer np, g_real xmin, g_real xmax)
{
    g_integer i,j=0;
#if 0
    for(i=0; i<np; i++)
    {
	if (xarray(i) >= xmin && xarray(i) <= xmax)
	{
	    xarray(j, xarray(i));
	    yarray(j, yarray(i));
	    earray(j, earray(i));
	    j++;
	}
    }
#endif
    return(j);	/* new np value */
}


/*
 * 0 success 
 * 1 invalid emode
 */
static int energyToTOF(g_real scale, g_integer emode, g_real efixed, GenieRealArray& xarray, 
		g_integer ntc, g_real l1, g_real l2)
{
    g_real factor, e1, e2, t1, t2;
    g_integer i;
#if 0
	if (emode == 1)
	{
	    e1 = efixed;
	    t1 = C1 * l1 * sqrt(C2/e1);
	    factor = C1 * l2 * sqrt(C2);
	    for(i=0; i<ntc+1; i++)
	    {
		e2 = e1 - xarray(i) / scale;	// xarray[i] / scale --> from whatever to meV
		t2 = factor / sqrt(e2);
		xarray(i, t1 + t2);
	    }
	}
	else if (emode == 2)
	{
	    e2 = efixed;
	    t2 = C1 * l2 * sqrt(C2/e2);
	    factor = C1 * l1 * sqrt(C2); 
	    for(i=0; i<ntc+1; i++)
	    {
		e1 = e2 + xarray(i) / scale;	// xarray[i] / scale --> from whatever to meV
		t1 = factor / sqrt(e1);
		xarray(i, t1 + t2);
	    }
	}
	else
	{
	    return 1;
	}
#endif
	return 0;
}

/*
 * 0 success
 * 1 invalid emode
 */
static int TOFToEnergy(g_real scale, g_integer emode, g_real efixed, GenieRealArray& xarray, 
		g_integer ntc, g_real l1, g_real l2)
{
    g_real factor, e1, e2, t1, t2;
    g_integer i;
#if 0
	if (emode == 1)
	{
	    e1 = efixed;
	    t1 = C1 * l1 * sqrt(C2/e1);
	    factor = C1 * C1 * l2 * l2 * C2;
	    for(i=0; i<ntc+1; i++)
	    {
		t2 = xarray(i) - t1;
		if ( t2 <= 0.0 )
		{
		    xarray(i, -BIGVAL);
		}
		else
		{
		    e2 = factor / (t2 * t2);
		    xarray(i,  (e1 - e2) * scale);	// e1 * scale --> from meV to whatever
		}
	    }
	}
	else if (emode == 2)
	{
	    e2 = efixed;
	    t2 = C1 * l2 * sqrt(C2/e2);
	    factor = C1 * C1 * l1 * l1 * C2;
	    for(i=0; i<ntc+1; i++)
	    {
		t1 = xarray(i) - t2;
		if ( t1 <= 0.0 )
		{
		    xarray(i, BIGVAL);
		}
		else
		{
		    e1 = factor / (t1 * t1);
		    xarray(i, (e1 - e2) * scale);	// e1 * scale --> from meV to whatever
		}
	    }
	}
        else 
	{
	    return 1;
	}
#endif
    return 0;
}

static int newGRebin(const GenieRealArray& xold, const GenieRealArray& yold, const GenieRealArray& eold,
              const GenieRealArray& xnew, GenieRealArray& ynew, GenieRealArray& enew)
{
#if 0
    g_integer i,iold(0),inew(0);
    g_real xo_low, xo_high, xn_low, xn_high, delta, width;
    int ntcold = xold.length() - 1;
    int ntcnew = xnew.length() - 1;
// Make sure ynew and enew are big enough
//    cout << "Params: " << ntcold << " " << ntcnew << endl;
//    cout << "Range: " << xold(iold) << " " << xnew(inew) << endl;
    ynew.alloc(xnew, 0.0);
    enew.alloc(xnew, 0.0);
    while((inew < ntcnew) && (iold < ntcold))
    {
	xo_low = xold(iold);
	xo_high = xold(iold+1);
	xn_low = xnew(inew);
	xn_high = xnew(inew+1);
	if ( xn_high <= xo_low )
	{
	    inew++;		/* old and new bins do not overlap */
	}
	else if ( xo_high <= xn_low )
	{
	    iold++;		/* old and new bins do not overlap */
	}
	else
	{
/*
 *        delta is the overlap of the bins on the x axis
 */
	    delta = MIN(xo_high, xn_high) - MAX(xo_low, xn_low);
	    width = xo_high - xo_low;
	    if ( (delta <= 0.0) || (width <= 0.0) )
	    {
		cout << "rebin error" << endl;
		return 1;	/* something wrong */
	    }
/*
 *        yold contains counts/unit time, ynew is just counts at moment
 *	  enew contains counts**2
 */
//	cout << "adding " << delta*yold(iold) << endl;
	    ynew.add(inew, delta*yold(iold));
	    enew.add(inew, delta*width*eold(iold)*eold(iold));
	    if ( xn_high > xo_high )
	    {
		iold++;
	    }
	    else
	    {
		inew++;
	    }
	}
    }
/*
 * convert back to counts/unit time
 */
    for(i=0; i<ntcnew; i++)
    {
	width = xnew(i+1)-xnew(i);
	ynew(i, ynew(i) / width);
	enew(i, sqrt(enew(i)) / width);
    }
//    cout << "ynew from rebin" << ynew << endl;
#endif
    return 0;
}

static int newGRebin(const GenieRealArray& xnew, const GenieWorkspace& oldwksp, GenieWorkspace& newwksp)
{
#if 0
    GenieRealArray ynew, enew, xwork;
    xwork.deepCopy(xnew);
    newGRebin(oldwksp("X"), oldwksp("Y"), oldwksp("E"), xwork, ynew, enew);
    newwksp("X", xwork);
    newwksp("Y", ynew);
    newwksp("E", enew);
    newwksp("NTC", xwork.length()-1);
#endif
    return 0;
}
