/*  Open GENIE, a program for neutron scattering data analysis & display
    ISIS facility, Rutherford Appleton Laboratory, Chilton, Didcot, OXON UK
    Copyright (C) 1996,1997

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA. */

#include "geniedefs.h"
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <sys/types.h>
#include "parsedefs.h"
#include "genievars.h"
#include "smalltalk_object.h"

#if HAVE_LIBTK

#include "g_tktcl.h"

/*
 * Module: 		Tk/Tcl <-> Genie interface
 * Purpose:		Define commands to allow passing of data and commands from Genie <-> TCL
 * Author:		Freddie Akeroyd, ISIS
 * Date:		September 1996
 */

static const char* rcsid = "@(#)$Id: g_tktcl.cxx,v 1.26 1997/06/24 21:27:44 faa Exp $";
USE_RCSID(rcsid);

#if defined(__VMS) && __VMS_VER < 70000000
#define strdup(__a) strcpy((char*)malloc(1+strlen(__a)), __a)
#endif

static int return_list(Tcl_Interp* interp)
{
    char *tmp = (char*)g_out->str();
    int len = strlen(tmp);
    if (len == 0)
	return TCL_OK;
    if (tmp[len-1] == '\2')
	tmp[len-1] = '\0';	// remove trailing '\2' if present
    char* bit = strtok(tmp, "\2");
    if (bit == 0)
	return TCL_OK;
    Tcl_AppendElement(interp, bit);
    while ( (bit = strtok(0, "\2")) != 0 )
    {
	Tcl_AppendElement(interp, bit);
    }
    return TCL_OK;
}

// Execute a command line in Smalltalk
/* static */ int 
smalltalk(ClientData clientData, Tcl_Interp *interp, int argc, char* argv[])
{
    int result;
    if (argc < 2)
    {
	interp->result = "Usage: smalltalk command_string ?options?";
	return TCL_ERROR; 
    }
// command string to run is in argv[1]
    if (evalSmalltalkCommand(argv[1]) != 0)
	result = TCL_ERROR;
    else
	result = TCL_OK;

    Tcl_ResetResult(interp);
    if (argc == 3 && !strcmp(argv[2], "-out"))
    {
	Tcl_SetResult(interp, (char*)g_out->str(), TCL_VOLATILE);
// bin the rest
        (void)g_inf->str();
        (void)g_err->str();
        (void)g_dbg->str();
    }
// gout contains a set of list elements separated by "\2"
    else if (argc == 3 && !strcmp(argv[2], "-list"))
    {
	result = return_list(interp);
// bin the rest
        (void)g_inf->str();
        (void)g_err->str();
        (void)g_dbg->str();
    }
    else
    {
        Tcl_AppendElement(interp, (char*)g_out->str());
        Tcl_AppendElement(interp, (char*)g_inf->str());
        Tcl_AppendElement(interp, (char*)g_err->str());
        Tcl_AppendElement(interp, (char*)g_dbg->str());
    }
    return result;
}

static int 
setinstvars(ClientData clientData, Tcl_Interp *interp, int argc, char* argv[])
{
    int result;
    if (argc != 3)
    {
	interp->result = "Usage: setinstvars oop_tag instvars_list";
	return TCL_ERROR; 
    }
    char* oop_tag = argv[1];
    char* instvars_list = argv[2];
    char** argv_list(0);
    int argc_list;
    if (Tcl_SplitList(interp, instvars_list, &argc_list, &argv_list) != TCL_OK)
	return TCL_ERROR;
    OOP oop = (OOP)(unsigned long)atol(oop_tag);
    SmalltalkObject obj(oop);
    for(int i=0; i<argc_list; i+=2)
    {
	obj.fromString(argv_list[i], argv_list[i+1]);
    }
    if (argv_list != 0)
	free((char*)argv_list);
    return TCL_OK;
}

// Execute a command line in GCL
static int 
gcl(ClientData clientData, Tcl_Interp *interp, int argc, char* argv[])
{
    int result;
    if (argc < 2)
    {
	interp->result = "Usage: gcl command_string ?options?";
	return TCL_ERROR; 
    }
// command string to run is in argv[1]
    if (evalGCLCommand(argv[1]) != 0)
	result = TCL_ERROR;
    else
        result = TCL_OK;

    Tcl_ResetResult(interp);
    if (argc == 3 && !strcmp(argv[2], "-out"))
    {
	Tcl_SetResult(interp, (char*)g_out->str(), TCL_VOLATILE);
// bin the rest
        (void)g_inf->str();
        (void)g_err->str();
        (void)g_dbg->str();
    }
    else
    {
        Tcl_AppendElement(interp, (char*)g_out->str());
        Tcl_AppendElement(interp, (char*)g_inf->str());
        Tcl_AppendElement(interp, (char*)g_err->str());
        Tcl_AppendElement(interp, (char*)g_dbg->str());
    }
    return result;
}

// Global DATA
PGTkItem* PGTkItemList[MAX_PGTKITEMS];
Tcl_Interp* PGTkDefaultInterp = 0;

// create a PGPLOT window (basically a canvas)
// Only ever gets called in the creation command
//  pgwindow pg1
// As the created object is a canvas, so all other processing done
// via canvas procedure
//
// Window names should be of the form .where.ever.pgwin1 etc., with
// pgwin1 accessed from PGPLOT by the device name /TK1
//

static Tcl_CmdDeleteProc *canvas_delete_proc = 0;

static void
pgwindow_delete_proc(ClientData clientData)
{
    int i;
    for(i=0; i<MAX_PGTKITEMS; i++)
    {
	if (PGTkItemList[i] != 0 && clientData == PGTkItemList[i]->cmdinfo.deleteData)
	    break;
    }
    if (i != MAX_PGTKITEMS)
    {
	PGTkItemList[i] = 0;
	gprintf("Deleting window %d\n", i);
    }
    if (canvas_delete_proc)
	(*canvas_delete_proc)(clientData);
}

static char* 
pgwindow_trace(ClientData clientData, Tcl_Interp* interp, char* name1, char* name2, int flags)
{
// name2 will be NULL as we are not tracing an array
    PGTkItem* item = (PGTkItem*)clientData;
    if (flags & TCL_TRACE_WRITES)
	gprintf("WRITE to %s\n", name1);
    if (flags & TCL_TRACE_READS)
	gprintf("READ of %s\n", name1);
    if (flags & TCL_TRACE_UNSETS)
	gprintf("UNSET of %s\n", name1);
    return NULL;
}

static int 
pgwindow(ClientData clientData, Tcl_Interp *interp, int argc, char* argv[])
{
    static Tcl_CmdInfo* canvas_cmdinfo = 0;
    int result;
    Tcl_ResetResult(interp);
// Do we know how to create a new canvas?
    if (canvas_cmdinfo == 0)
    {
	for(int i=0; i<MAX_PGTKITEMS; i++)
	    PGTkItemList[i] = 0;
	canvas_cmdinfo = new Tcl_CmdInfo;
	if (!Tcl_GetCommandInfo(interp, "canvas", canvas_cmdinfo))
	{
	    Tcl_AppendResult(interp, "pgwindow: \"canvas\" command not found", 0);
	    return TCL_ERROR;
  	}
	PGTkDefaultInterp = interp;
    } 
// overwrite the command name of "pgwindow" with "canvas"
//   argv[0] = Tk_GetUid("canvas");
// Create a new canvas
    result = (*(canvas_cmdinfo->proc))(canvas_cmdinfo->clientData, interp, argc, argv);
    if (result != TCL_OK)
	return result;
// Execute events generated by canvas command
    while(Tk_DoOneEvent(TK_ALL_EVENTS|TK_DONT_WAIT))
        ;
// Parse window name - the last character should be a number
    char* pgwin_name = strdup(argv[1]);
    int pgwin_number = pgwin_name[strlen(pgwin_name)-1] - '1'; 
// Find an open device slot in the table
    if (pgwin_number < 0 || pgwin_number >= MAX_PGTKITEMS)
    {
	Tcl_AppendResult(interp, "pgwindow: invalid PG window number", 0);
	return TCL_ERROR;
    }
    if (PGTkItemList[pgwin_number] != 0)
    {
	Tcl_AppendResult(interp, "pgwindow: PG window already in use", 0);
	return TCL_ERROR;
    }
// Now store the address of the new Tk widget command
    PGTkItem* item = new PGTkItem;
    item->interp = interp;
    item->opened = 0;		/* No PGOPEN yet */
    item->cmd = Tk_GetUid(pgwin_name);
//    if (Tcl_TraceVar(interp, item->cmd, TCL_GLOBAL_ONLY|TCL_TRACE_READS|TCL_TRACE_WRITES|TCL_TRACE_UNSETS, pgwindow_trace, item) != TCL_OK)
//	return TCL_ERROR;
    item->tkwin = Tk_NameToWindow(interp, item->cmd, Tk_MainWindow(interp));
    if (item->tkwin == 0)
	return TCL_ERROR;
    if (!Tcl_GetCommandInfo(interp, Tk_PathName(item->tkwin), &(item->cmdinfo)))
	return TCL_ERROR;
    canvas_delete_proc = item->cmdinfo.deleteProc;
    item->cmdinfo.deleteProc = pgwindow_delete_proc;
    if (!Tcl_SetCommandInfo(interp, Tk_PathName(item->tkwin), &(item->cmdinfo)))
	return TCL_ERROR;
    PGTkItemList[pgwin_number] = item; 
    return TCL_OK;
} 

// Files searched for in directory given by GENIE_TCL_DIR environment variable
// Last entry must be NULL
static const char* tcl_init_files[] = { "objecttags.tcl", 0, "display.tcl", "graph.tcl", "st1.tcl",
	"miscDis.tcl", "printOutput.tcl", "oneDgraph.tcl", "oneDHistogram.tcl",
	"oneDErrors.tcl", "oneDMarkers.tcl", "axes.tcl", "label.tcl", "title.tcl",
	"text.tcl", "polygon.tcl", "polyline.tcl", 0 };

/*
 * Application init for TK/TCL, called by Tk_Main
 */
int
Genie_Tcl_AppInit(Tcl_Interp* interp)
{
    if (Tcl_Init(interp) == TCL_ERROR)
    {
	return TCL_ERROR;
    }
    if (Tk_Init(interp) == TCL_ERROR)
    {
	return TCL_ERROR;
    } 
#ifdef HAVE_LIBTIX
    if (Tix_Init(interp) == TCL_ERROR)
    {
	return TCL_ERROR;
    } 
#endif /* HAVE_LIBTIX */
/*
 * Now add the new commands
 */
    if (!Tcl_CreateCommand(interp, "gcl", gcl, (ClientData)0, (Tcl_CmdDeleteProc*)0))
	return TCL_ERROR;
    if (!Tcl_CreateCommand(interp, "smalltalk", smalltalk, (ClientData)0, (Tcl_CmdDeleteProc*)0))
	return TCL_ERROR;
    if (!Tcl_CreateCommand(interp, "pgwindow", pgwindow, (ClientData)0, (Tcl_CmdDeleteProc*)0))
	return TCL_ERROR;
    if (!Tcl_CreateCommand(interp, "setinstvars", setinstvars, (ClientData)0, (Tcl_CmdDeleteProc*)0))
	return TCL_ERROR;
/*
 * now load any stuff we need
 */
    const char* genie_tcl_dir = (getenv("GENIE_TCL_DIR") != 0 ? strdup(getenv("GENIE_TCL_DIR")) : CURRENT_DIRECTORY);
    char loadpath[256];
    for(int i = 0; tcl_init_files[i] != 0; i++)
    {
	sprintf(loadpath, "%s%s%s", genie_tcl_dir, DIRECTORY_SEPARATOR, tcl_init_files[i]);
	if (Tcl_EvalFile(interp, loadpath) != TCL_OK)
	{
	    gerr << "Error executing TCL startup commands from \"" << loadpath << "\"" << endl;
	    return TCL_ERROR;
	}
    }
    return TCL_OK;
}

#endif /* HAVE_LIBTK */
