/*  Open GENIE, a program for neutron scattering data analysis & display
    ISIS facility, Rutherford Appleton Laboratory, Chilton, Didcot, OXON UK
    Copyright (C) 1996,1997

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA. */

#include "geniedefs.h"
#include <iostream.h>
#include <stdio.h>
#include <stdlib.h>

#ifdef HAVE_UNISTD_H
#	include <unistd.h>
#else
#	include "getopt.h"
#endif

#include <string.h>
#include <fcntl.h>

#if defined(__VMS) && __VMS_VER < 70000000
#define strdup(__a) strcpy((char*)malloc(1+strlen((const char*)__a)), (const char*)__a)
extern "C" int putenv(const char* var);
#endif /* defined(__VMS) && __VMS_VER < 70000000 */

#ifdef __ultrix
extern "C" int getopt(int argc, char **argv, char *optstring);	// missing prototype
#endif /* __ultrix */

#ifdef HAVE_LIBTK
#include "g_tktcl.h"
#endif

#include "smalltalk.h"
#include "forkinit.h"
#include "genie_data_access.h"

static const char* rcsid = "@(#)$Id: genie.cxx,v 1.36.2.1 1997/08/01 16:15:58 faa Exp $";
USE_RCSID(rcsid);

/*
 * Calls the top level routine in the smalltalk interpreter
 */

int
main(int argc, char* argv[], char *arge[])
{
    // I REALLYMEAN these here, don't want the fudged ones !!
#   undef cin
#   undef cout
#   undef cerr
#   undef clog

    // only display debugging output if Open GENIE is also checking module files,
    // we assume that this is a development build if this is the case.
    int screen_output = 1;				/* Default terminal style I/O on */
    int debugging_output = 0;				/* Default no debug output */

    int ch;
    while ( (ch = getopt(argc, argv, "gsydhBEcn")) != -1 )
       switch ( ch )
       {
	case 'B':
		screen_output = 0;			/* put output into a strstream for TCL */
		break;
        case 'c':
		debugging_output = 1;			/* Enable debugging output */
		break;
	}
    ch = optind;
    String temp;
//
// Look for args after flags:   genie [flags] [gcl_init_file] [smalltalk_image]
// Trap zero length strings as wel get these from VMS due to coming via
// a command file
//
    if (argv[ch] != 0)
    {
	if (strlen(argv[ch]) > 0)
	{
	    cat("GENIE_GCL_INIT=", argv[ch], temp);
	    putenv(strdup(temp));
	}
	ch++;
    }
    if (argv[ch] != 0)
    {
	if (strlen(argv[ch]) > 0)
	{
	    cat("GENIE_SMALLTALK_IMAGE=", argv[ch], temp);
	    putenv(strdup(temp));
	}
	ch++;
    }

    istream in(cin.rdbuf());
    genie_ostream out(cout, "", "", screen_output);
    genie_ostream inf(cout, BLUE_ON, BLUE_OFF, screen_output);
    genie_ostream err(cout, RED_ON, RED_OFF, screen_output);
    genie_ostream dbg(cout, PINK_ON, PINK_OFF, debugging_output && screen_output);

    FORKINIT(argv);		/* do any pre-fork initialisation. */

    // initialize a gdai with the streams for interactive GENIE
	// this GDAI will be used for GX_ access from modules etc. in
	// a running GENIE.
	This_session = new gdai("GENIE", in, out, inf, err, dbg);

    parser_main(argc, argv, arge, in, out, inf, err, dbg);

    // We never get here unless "-B" is supplied in argv[]

    // Clear out streams of init startup data
    (void)out.str();
    (void)err.str();
    (void)inf.str();
    (void)dbg.str();

    // Choose the GUI we are running here

#if HAVE_LIBTK
    // Take startup commands form file pointed to by GENIE_TCL_INIT
    int fd, tk_argc;
    char* tk_argv[3];
    const char* tk_init_file = ( getenv("GENIE_TCL_INIT") != 0 ? strdup(getenv("GENIE_TCL_INIT")) : "genieinit.tcl" );
    if ( (fd = open(tk_init_file, O_RDONLY, 0)) != -1 )
    {
	ginf << "Loading TCL commands from \"" << tk_init_file << "\"" << endl;
	close(fd);
    }
    else
    {
	ginf << "Warning - \"" << tk_init_file << "\" not present: was GENIE_TCL_INIT defined correctly?" << endl;
	tk_init_file = 0;
    }
    tk_argv[0] = argv[0];
    tk_argv[1] = (char *)tk_init_file;
    tk_argv[2] = 0;
    tk_argc = (tk_init_file != 0 ? 2 : 1);
    Tk_Main(tk_argc, tk_argv, Genie_Tcl_AppInit);
#endif /* HAVE_LIBTK */

    return 0;
}
