/*  Open GENIE, a program for neutron scattering data analysis & display
    ISIS facility, Rutherford Appleton Laboratory, Chilton, Didcot, OXON UK
    Copyright (C) 1996,1997

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA. */

#ifndef GENIE_ARRAY
#define GENIE_ARRAY

#include "genie_array_base.h"

//template <class GenieType, class CDataType, GenieVariableType GXType> class GenieArray;

template <class GenieType, class CDataType, GenieVariableType GXType>
class GenieArray : public GenieArrayBase
{
public:
// Smalltalk arrays start at 1, C arrays at 0
    typedef GenieArray<GenieType, CDataType, GXType> Array;
    typedef CDataType (*collect_t)(CDataType);
    typedef CDataType (*with_collect_t)(CDataType, CDataType);
    CDataType operator()(int i) const { return GenieType::fromOOP(indexOOP(m_value, i+1)); }
    CDataType operator[](int i) const { return GenieType::fromOOP(indexOOP(m_value, i+1)); }
    CDataType operator()(long i) const { return GenieType::fromOOP(indexOOP(m_value, i+1)); }
    CDataType operator[](long i) const { return GenieType::fromOOP(indexOOP(m_value, i+1)); }
    void operator()(int i, const CDataType& t) { indexOOPPut(m_value, i+1, GenieType::toOOP(t)); }    
    void operator()(long i, const GenieType& gt) { indexOOPPut(m_value, i+1, gt.value()); }    
    void operator()(const CDataType* ca, int len);
    ~GenieArray()  { }
    GenieArray(int dim1, int dim2 = 0);
    GenieArray(OOP oop = nilOOP) : GenieArrayBase(oop) {}
    GenieArray(const Array& ga);
    void operator=(const Array& ga);
    void collect(collect_t func);    
    void withCollect(with_collect_t func, Array& ga2, int nil_flag);    
//    int alloc(const Array& ga, CDataType fill);
    OOP become(OOP oop);
    virtual void alloc(const GenieArrayBase& ga) { become(allocOOP(ga.dims())); }
    virtual void alloc(int dim1, int dim2 = 0) { become(allocOOP(dim1, dim2)); }
    virtual void alloc(OOP a_dimsOOP) { become(allocOOP(a_dimsOOP)); }
    virtual void alloc(const int* dims_array, int ndims) { alloc(makeDimsOOP(dims_array, ndims)); }
//    void add(int dim1, CDataType val);
    ostream& print(ostream& os, const char* note=NULL) const;
    OOP shallowCopy(const GenieArray<GenieType, CDataType, GXType>& ga);
    OOP deepCopy(const GenieArray<GenieType, CDataType, GXType>& ga);
    OOP deepCopy() const { return deepCopyOOP(m_value); }
    OOP shallowCopy() const { return shallowCopyOOP(m_value); }
    bool operator==(const GenieArray<GenieType, CDataType, GXType>& ga);
    static OOP deepCopyOOP(OOP oop);
    static OOP shallowCopyOOP(OOP oop);
    static OOP allocOOP(int dim1, int dim2 = 0);
    static OOP allocOOP(OOP a_dimsOOP);
};

// Macros fo building "+=" operators
// For: 	GenieRealArray& GenieRealArray::operator+=(g_real gt) 
// Use: 	BUILD_OPEQ(GenieRealArray,+,GenieReal,g_real)

#define BUILD_OPEQ(__array_type,__op,__g_type,__type)  BUILD_OP(__array_type,__op,=,__g_type,__type)

#define BUILD_OP(__array_type,__op,__eq,__g_type,__type) \
__array_type& __array_type::operator __op##__eq(__type gt) \
{ \
    for(int i=1; i<=length(); i++) \
    { \
        indexOOPPut(m_value, i, __g_type::toOOP(__g_type::fromOOP(indexOOP(m_value, i)) __op gt)); \
    } \
    return *this; \
}

#endif /* GENIE_ARRAY */
