/*  Open GENIE, a program for neutron scattering data analysis & display
    ISIS facility, Rutherford Appleton Laboratory, Chilton, Didcot, OXON UK
    Copyright (C) 1996,1997

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA. */

#ifndef GENIE_ARRAY_BASE
#define GENIE_ARRAY_BASE

class GenieArrayBase : public GenieVariableBase
{
protected:
    int* m_dope;			// copy of dope array (for fast access and lookup)			
    int m_ndims;			// copy number of dimensions of array
    void updateDimsAndDope();		// rebuild m_dope and m_ndims from the OOPs;

public:
    enum Array_inst_vars    { MY_DIMS=1, MY_DOPE=2, MY_DATA=3 };
    GenieArrayBase(OOP oop = nilOOP);
    GenieArrayBase(OOP params, int location);
    GenieArrayBase(const GenieCalloutList& gcl, const char* name) : GenieVariableBase(gcl, name), m_dope(0), m_ndims(0) {}
    ~GenieArrayBase()  { delete[] m_dope; }
    GenieArrayBase(const GenieArrayBase& gab) : m_dope(0), m_ndims(0) { deepCopy(gab); }
    GenieArrayBase& operator=(const GenieArrayBase& gab);
    GenieArrayBase(int dim1, int dim2 = 0) : GenieVariableBase(nilOOP), m_dope(0), m_ndims(0) { setDimsAndDope(dim1, dim2); }
    virtual void setDimsAndDope(int* dims_array, int ndims);
    virtual void setDimsAndDope(int dim1, int dim2=0);
    void copyDims(const GenieArrayBase& gab);
    ostream& print(ostream& os, const char* note=NULL) const;
    OOP deepCopy() const { return deepCopyOOP(m_value); }
    OOP deepCopy(const GenieArrayBase& gab);
    OOP shallowCopy(const GenieArrayBase& gab);
    OOP shallowCopy() const { return GenieVariableBase::shallowCopy(); }
    OOP become(OOP oop);
    static OOP deepCopyOOP(OOP oop);
    OOP dims() const { return dimsOOP(m_value); }
    int dims(int i) const { return toInt(arrayAt(dims(), i)); }		// array starts at element 1
    int* dimsArray() const; 
    int dimsSize() const { return numIndexableFields(dims()); }
    virtual void alloc(int dim1, int dim2 = 0) { gerr << "ERROR: GenieArrayBase::alloc() called" << endl; }
    virtual void alloc(const GenieArrayBase& gab) { gerr << "ERROR: GenieArrayBase::alloc() called" << endl; }
    virtual void alloc(OOP a_dimsOOP) { gerr << "ERROR: GenieArrayBase::alloc() called" << endl; }
    virtual void alloc(const int* dims_array, int ndims) { gerr << "ERROR: GenieArrayBase::alloc() called" << endl; }
    virtual void init(OOP oop = nilOOP, OOP params = nilOOP, int location = 0); 
    OOP dope() const { return instVarAt(m_value, MY_DOPE); }
    int dope(int i) const { return toInt(arrayAt(dope(), i)); }
    int length() const { return numIndexableFields(m_value); }
    int writeAsciiDims(fstream& fs, int indent = 0);
    static OOP readAsciiDims(fstream& fs);
    static OOP dimsOOP(OOP oop) { return instVarAt(oop, MY_DIMS); }
    static OOP dopeOOP(OOP oop) { return instVarAt(oop, MY_DOPE); }
    static OOP makeDimsOOP(int dim1, int dim2=0);
    static OOP makeDimsOOP(const int* dims_array, int ndims);
    static OOP makeDopeOOP(const int* dims_array, int ndims);
    static OOP makeDopeOOP(int dim1, int dim2=0);
    static OOP makeDopeOOP(OOP a_dimsOOP);
    static int arraySize(OOP a_dimsOOP);
    static int arraySize(int dim1, int dim2=0) { return (dim2 == 0 ? dim1 : dim1 * dim2); }
};

ostream& operator<<(ostream& os, const GenieArrayBase& gab);

#endif /* GENIE_ARRAY_BASE */
