/*  Open GENIE, a program for neutron scattering data analysis & display
    ISIS facility, Rutherford Appleton Laboratory, Chilton, Didcot, OXON UK
    Copyright (C) 1996,1997

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA. */

/*
 *      Module:         genie_cmodule
 *      Author:         Freddie Akeroyd, ISIS
 *      Purpose:        C MODULE interface to calling user programs
 *
 */
#include "geniedefs.h"
#include "genie_cmodule.h"
#include "smalltalk.h"
#include "genievars.h"
#include "module.h"
#include "g_module.h"

static const char* rcsid = "@(#)$Id: genie_cmodule.cxx,v 1.5 1997/06/25 01:06:53 faa Exp $";
USE_RCSID(rcsid);

void cmodule_get_real(GenieWorkspace* pars, const char* name, fort_real* val)
{
    STRING name_s; name_s.length = strlen(name); strcpy(name_s.spot, name);
    g3_module_get_real(*pars, name_s, *val);
}

void cmodule_get_double(GenieWorkspace* pars, const char* name, fort_double* val)
{
    STRING name_s; name_s.length = strlen(name); strcpy(name_s.spot, name);
    g3_module_get_double(*pars, name_s, *val);
}

void cmodule_put_real(GenieWorkspace* res, const char* name, fort_real val)
{
    STRING name_s; name_s.length = strlen(name); strcpy(name_s.spot, name);
    g3_module_put_real(*res, name_s, val);
}

void cmodule_print(const char* s, fort_int option)
{
    STRING name_s; name_s.length = strlen(s); strcpy(name_s.spot, s);
    g3_module_print(name_s, option);
}

fort_int cmodule_get_version(fort_int* major, fort_int* minor)
{
    return g3_module_get_version(*major, *minor);
}

fort_int cmodule_version_ok(fort_int major, fort_int minor)
{
    return g3_module_version_ok(major, minor);
}

void cmodule_put_double(GenieWorkspace* res, const char* name, fort_double val)
{
    STRING name_s; name_s.length = strlen(name); strcpy(name_s.spot, name);
    g3_module_put_double(*res, name_s, val);
}

void cmodule_get_string(GenieWorkspace* pars, const char* name, char** val)
{
    STRING val_s; 
    STRING name_s; name_s.length = strlen(name); strcpy(name_s.spot, name);
    g3_module_get_string(*pars, name_s, val_s);
    *val = (char*)malloc(1+val_s.length);
    strcpy(*val, val_s.spot);
}

void cmodule_put_string(GenieWorkspace* res, const char* name, const char* val)
{
    STRING name_s; name_s.length = strlen(name); strcpy(name_s.spot, name);
    STRING val_s; val_s.length = strlen(val); strcpy(val_s.spot, val);
    g3_module_put_string(*res, name_s, val_s);
}

void cmodule_put_string_array(GenieWorkspace* res, const char* name, const char* val[], fort_int len)
{
    STRING val_s;
    STRING name_s; name_s.length = strlen(name); strcpy(name_s.spot, name);
    void* handle;
    g3_module_string_array_alloc(len, handle);
    for(int i=0; (i < len) && (val[i] != 0); i++)
    {
 	val_s.length = strlen(val[i]);
	strcpy(val_s.spot, val[i]);
	g3_module_string_array_add(handle, i+1, val_s);
    }
    g3_module_string_array_free(*res, name_s, handle);
}

void cmodule_get_int(GenieWorkspace* pars, const char* name, fort_int* val)
{
    STRING name_s; name_s.length = strlen(name); strcpy(name_s.spot, name);
    g3_module_get_int(*pars, name_s, *val);
}

void cmodule_put_int(GenieWorkspace* res, const char* name, fort_int val)
{
    STRING name_s; name_s.length = strlen(name); strcpy(name_s.spot, name);
    g3_module_put_int(*res, name_s, val);
}

// Allocates space for returned array with malloc()
void cmodule_get_real_array(GenieWorkspace* pars, const char* name, fort_real** val, fort_int* len)
{
    STRING name_s; name_s.length = strlen(name); strcpy(name_s.spot, name);
    g3_module_get_real_array_alloc(*pars, name_s, val, *len);
}

// Allocates space for returned array with malloc()
void cmodule_get_double_array(GenieWorkspace* pars, const char* name, fort_double** val, fort_int* len)
{
    STRING name_s; name_s.length = strlen(name); strcpy(name_s.spot, name);
    g3_module_get_double_array_alloc(*pars, name_s, val, *len);
}
    
// Allocates space for returned array with malloc()
void cmodule_get_int_array(GenieWorkspace* pars, const char* name, fort_int** val, fort_int* len)
{
    STRING name_s; name_s.length = strlen(name); strcpy(name_s.spot, name);
    g3_module_get_int_array_alloc(*pars, name_s, val, *len);
}

void cmodule_put_real_array(GenieWorkspace* pars, const char* name, const fort_real* val, fort_int len)
{
    STRING name_s; name_s.length = strlen(name); strcpy(name_s.spot, name);
    g3_module_put_real_array(*pars, name_s, val, len);
}

void cmodule_put_double_array(GenieWorkspace* pars, const char* name, const fort_double* val, fort_int len)
{
    STRING name_s; name_s.length = strlen(name); strcpy(name_s.spot, name);
    g3_module_put_double_array(*pars, name_s, val, len);
}

void cmodule_put_int_array(GenieWorkspace* pars, const char* name, const fort_int* val, fort_int len)
{
    STRING name_s; name_s.length = strlen(name); strcpy(name_s.spot, name);
    g3_module_put_int_array(*pars, name_s, val, len);
}

fort_int cmodule_put_nd_real_array(GenieWorkspace* pars, const char* name, const fort_real* val, 
	const fort_int* dims_array, fort_int ndims)
{
    STRING name_s; name_s.length = strlen(name); strcpy(name_s.spot, name);
    return g3_module_put_nd_real_array(*pars, name_s, val, dims_array, ndims);
}
