/*  Open GENIE, a program for neutron scattering data analysis & display
    ISIS facility, Rutherford Appleton Laboratory, Chilton, Didcot, OXON UK
    Copyright (C) 1996,1997

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA. */

#ifndef GENIE_FLAT_ARRAY
#define GENIE_FLAT_ARRAY

#include "genie_array_base.h"

extern "C" int oopNumFields(OOP oop);

enum FlatArrayFlag { WriteBackInternalArrays=01 };

//#define FLAT_ARRAY_POINTER			m_data
// Cast via void* to avoid alignment warning in later casts
#define FLAT_ARRAY_POINTER	( (void*)((ByteArray)m_byteArray->object)->bytes )
#define FLAT_ARRAY_DATA		( (CDataType*)FLAT_ARRAY_POINTER )

template <class GenieType, class CDataType, GenieVariableType GXType>
class GenieFlatArray : public GenieArrayBase
{
protected:
    CDataType* m_data;
    unsigned m_flags;
    OOP m_byteArray;

public:
// Remember: Smalltalk arrays start at 1, but C arrays at 0
    typedef GenieFlatArray<GenieType, CDataType, GXType> Array;
    typedef CDataType (*collect_t)(CDataType);
    typedef CDataType (*collect_ref_t)(CDataType&);
    typedef CDataType (*with_collect_t)(CDataType, CDataType);
    CDataType& operator()(int i) { return FLAT_ARRAY_DATA[i]; }
    CDataType& operator[](int i) { return FLAT_ARRAY_DATA[i]; }
    const CDataType operator()(int i) const { return FLAT_ARRAY_DATA[i]; }
    const CDataType operator[](int i) const { return FLAT_ARRAY_DATA[i]; }
    CDataType& operator()(int i, int j) { return FLAT_ARRAY_DATA[i*m_dope[0] + j*m_dope[1]]; }
    virtual void init(OOP oop = nilOOP, OOP params = nilOOP, int location = 0);
    GenieFlatArray(OOP params, int location) : m_data(0), m_flags(0), m_byteArray(nilOOP)
	 { init(arrayAt(params, location), nilOOP, 0); }
    void operator()(int i, const CDataType& t) { FLAT_ARRAY_DATA[i] = t; }
    void operator()(int i, const GenieType& gt) { FLAT_ARRAY_DATA[i] = gt; }
    ~GenieFlatArray() { }
    GenieFlatArray(int dim1, int dim2 = 0);
    GenieFlatArray(const int* dims_array, int ndims);
    GenieFlatArray(OOP oop = nilOOP);
    GenieFlatArray(const GenieCalloutList& gcl, const char* name);
    GenieFlatArray(const Array& ga);
    GenieFlatArray(const Array& ga, CDataType value);
    void operator()(const CDataType* c_array, int dim1, int dim2);
    void operator()(const CDataType* c_array, int dim1) { operator()(c_array, dim1, 0); }
    void operator=(const Array& ga);
    Array operator+(const Array& ga);
    Array operator-(const Array& ga);
    Array operator*(const Array& ga);
    Array operator/(const Array& ga);
    void operator+=(CDataType ct);
    void operator/=(CDataType ct);
    void collect(collect_t func);
    void collect(collect_ref_t func);
    void withCollect(with_collect_t func, const Array& ga, int nil_flag);
    OOP become(OOP oop);
    OOP become(const Array& ga) { return become(ga.value()); }
    virtual void alloc(const GenieArrayBase& ga) { become(allocOOP(ga.dims())); }
//    void alloc(const Array& ga, CDataType value) { alloc(ga); fill(value); } 
    virtual void alloc(int dim1, int dim2 = 0) { become(allocOOP(dim1, dim2)); }
    virtual void alloc(OOP a_dimsOOP) { become(allocOOP(a_dimsOOP)); }
    virtual void alloc(const int* dims_array, int ndims) { alloc(makeDimsOOP(dims_array, ndims)); }
    ostream& print(ostream& os, const char* note=NULL) const;
    void shallowCopy(const Array& ga);
    OOP deepCopy(const Array& ga);
    OOP deepCopy() const { return deepCopyOOP(m_value); }
    OOP shallowCopy() const { return shallowCopyOOP(m_value); }
    bool operator==(const GenieFlatArray<GenieType, CDataType, GXType>& ga);
    static OOP deepCopyOOP(OOP oop);
    static OOP shallowCopyOOP(OOP oop);
    static OOP byteArrayOOP(OOP oop) { return instVarAt(oop, MY_DATA); }
    static CDataType* bytesOOP(OOP oop) { return (CDataType*)mstGetCdata(byteArrayOOP(oop)); }
    static int lengthOOP(OOP oop) { return oopNumFields(byteArrayOOP(oop)) / sizeof(CDataType); }
    static OOP allocOOP(int dim1, int dim2 = 0);
    static OOP allocOOP(const int* dims_array, int ndims);
    static OOP allocOOP(OOP a_dimsOOP);
    static OOP makeByteArrayOOP(OOP a_dimsOOP);
    static void fillByteArray(OOP a_byteArrayOOP, CDataType value);
    OOP byteArray() { return m_byteArray; }
    const CDataType* data() const { return FLAT_ARRAY_DATA; }
    int length() const { return lengthOOP(m_value); }
    void setFlag(FlatArrayFlag flag) { m_flags |= flag; }
    void clearFlags() { m_flags = 0; }
    void fill(CDataType value, CDataType step = 0, int mode = 0);
    CDataType max(bool skip_undefined = 0);
    CDataType min(bool skip_undefined = 0);
    CDataType sum();
};

#endif /* GENIE_FLAT_ARRAY */
