/*  Open GENIE, a program for neutron scattering data analysis & display
    ISIS facility, Rutherford Appleton Laboratory, Chilton, Didcot, OXON UK
    Copyright (C) 1996,1997

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA. */

#ifndef GENIE_FLOAT
#define GENIE_FLOAT

#include "g_types.h"

class GenieFloat : public GenieVariableBase
{
protected:
    float m_float;

public:
    operator float() const { return m_float; }
    operator float&() { return m_float; }
    GenieFloat(float f) : m_float(f), GenieVariableBase(nilOOP) {}
    GenieFloat(const GenieFloat& grt) : m_float(grt), GenieVariableBase(nilOOP) {}
    GenieFloat(OOP oop = nilOOP) : m_float(fromOOP(oop)), GenieVariableBase(nilOOP) {};
    static float fromOOP(OOP oop) { return oop == nilOOP ? FLT_MAX : (float)floatOOPValue(oop); }
    static OOP toOOP(float f) { return f == FLT_MAX ? nilOOP : saveOOP(floatNew(f)); }
    static OOP deepCopyOOP(OOP oop) { return oop == nilOOP ? nilOOP : toOOP(fromOOP(oop)); }
    OOP deepCopy() const { return deepCopyOOP(m_value); }
    ostream& print(ostream& os, const char* note=NULL) const;
    ~GenieFloat() { m_value = toOOP(m_float); }
    void init(OOP oop = nilOOP, OOP params = nilOOP, int location = 0) 
	{ GenieVariableBase::init(oop, params, location); m_float = fromOOP(m_value); }
};

#endif	/* GENIE_FLOAT */
