/*  Open GENIE, a program for neutron scattering data analysis & display
    ISIS facility, Rutherford Appleton Laboratory, Chilton, Didcot, OXON UK
    Copyright (C) 1996,1997

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA. */

#ifndef GENIE_INTEGER_ARRAY
#define GENIE_INTEGER_ARRAY

#include "genie_flat_array.h"

// Typedef for doing a collect with a FORTRAN function loaded from a module
typedef g_integer (*integer_array_collect_ref_t)(g_integer&);

class GenieIntegerArray : public GenieFlatArray<GenieInteger, g_integer, GenieIntegerArrayType>
{
protected:
    int* m_int;
    long* m_long;
    typedef GenieFlatArray<GenieInteger, g_integer, GenieIntegerArrayType> BaseClass;

public:
    GenieIntegerArray(OOP params, int location) : BaseClass(params, location), m_int(0), m_long(0) {}
    GenieIntegerArray(const int* ip, int dim1, int dim2 = 0);
    GenieIntegerArray(const long* lp, int dim1, int dim2 = 0);
    virtual void init(OOP oop = nilOOP, OOP params = nilOOP, int location = 0);
    g_integer& operator()(int i) { return BaseClass::operator()(i); }
    void operator()(const int* ip, int dim1, int dim2);
    void operator()(const int* ip, int dim1) { operator()(ip, dim1, 0); }
    void operator()(const long* lp, int dim1, int dim2);
    void operator()(const long* lp, int dim1) { operator()(lp, dim1, 0); }
    GenieIntegerArray(OOP oop = nilOOP) : BaseClass(oop), m_int(0), m_long(0) {}
    GenieIntegerArray(int dim1, int dim2 = 0) : BaseClass(dim1, dim2), m_int(0), m_long(0) {}
    GenieIntegerArray(const GenieIntegerArray& gia) : BaseClass(gia), m_int(0), m_long(0) {}
    GenieIntegerArray(const GenieCalloutList& gcl, const char* name) : BaseClass(gcl, name), m_int(0), m_long(0) {}
    operator int*() { return setupInt(); }
    operator long*() { return setupLong(); }
    int* setupInt();
    long* setupLong();
    ~GenieIntegerArray();
    void removeInternalArrays();
    GenieVariableType read(IntermediateFile* gif);
    GenieVariableType write(IntermediateFile* gif);
    GenieVariableType skip(IntermediateFile* gif);
    GenieVariableType readAscii(fstream& fs);
    GenieVariableType writeAscii(fstream& fs, int indent = 0);
    OOP become(OOP oop);
    OOP become(const BaseClass& ga) { return become(ga.value()); }
    OOP become(const GenieVariable& gv);
};

#endif /* GENIE_INTEGER_ARRAY */
