/*  Open GENIE, a program for neutron scattering data analysis & display
    ISIS facility, Rutherford Appleton Laboratory, Chilton, Didcot, OXON UK
    Copyright (C) 1996,1997

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA. */

/*
 *      Module:         genie_interval
 *      Author:         Freddie Akeroyd, ISIS
 *
 */
#include "geniedefs.h"
#include "genievars.h"
#include "intermediate_file.h"
#include "genie_interval.h"

static const char* rcsid = "@(#)$Id: genie_interval.cxx,v 1.5 1997/05/06 09:57:00 cmm Exp $";
USE_RCSID(rcsid);

GenieVariableType GenieInterval::write(IntermediateFile* gif)
{
    if (gif->writeInteger(start()) && gif->writeInteger(stop()) && gif->writeInteger(step()))
    {
	return GenieIntervalType;
    }
    else
    {
        return GenieTypeError;
    }
}

GenieVariableType GenieInterval::skip(IntermediateFile* gif)
{
    if (gif->skipInteger() && gif->skipInteger() && gif->skipInteger())
    {
        return GenieIntervalType;
    }
    else
    {
        return GenieTypeError;
    }
}

GenieVariableType GenieInterval::read(IntermediateFile* gif)
{
    g3io_int_t temp1, temp2, temp3;
    if (gif->readInteger(temp1) && gif->readInteger(temp2) && gif->readInteger(temp3))
    {
	setStart(temp1);
	setStop(temp2);
	setStep(temp3);
        return GenieIntervalType;
    }
    else
    {
        return GenieTypeError;
    }
}

GenieVariableType GenieInterval::writeAscii(fstream& fs, int indent)
{
    fs << setindent(indent) << start() << " " << stop() << " " << step() << "\n";
    return GenieIntervalType;
}

GenieVariableType GenieInterval::readAscii(fstream& fs)
{
    int a_start, a_stop, a_step;
    fs >> a_start >> a_stop >> a_step;
    setStart(a_start);
    setStop(a_stop);
    setStep(a_step);
    return GenieIntervalType;
}

g_integer GenieInterval::sum()
{
    g_integer a_sum(0); 
    int a_step = step();
    for(int i=start(); i<=stop(); i+= a_step)
    {
	a_sum++;
    }
    return a_sum;
}
