/*  Open GENIE, a program for neutron scattering data analysis & display
    ISIS facility, Rutherford Appleton Laboratory, Chilton, Didcot, OXON UK
    Copyright (C) 1996,1997

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA. */

#ifndef GENIE_INTERVAL
#define GENIE_INTERVAL

/*
 * class to encapsulate a smalltalk Interval class
 */

#include "g_types.h"

class IntermediateFile;

class GenieInterval : public GenieVariableBase
{
    protected:
    	enum IntervalInstVars { IntervalStart = 1, IntervalStop = 2, IntervalStep = 3 };	// Smalltalk instance variables

    public:
	GenieInterval(OOP oop = nilOOP) : GenieVariableBase(oop) {}
 	GenieInterval(const GenieCalloutList& gcl, const char* name) : GenieVariableBase(gcl, name) {}
	GenieInterval(OOP params, int location) : GenieVariableBase(params, location) {}
	g_integer start() { return toInt(instVarAt(m_value, IntervalStart)); }
	g_integer stop() { return toInt(instVarAt(m_value, IntervalStop)); }
	g_integer step() { return toInt(instVarAt(m_value, IntervalStep)); }
	g_integer sum();
	void setStart(long val) { instVarAtPut(m_value, IntervalStart, fromInt(val)); }
	void setStop(long val) { instVarAtPut(m_value, IntervalStop, fromInt(val)); }
	void setStep(long val) { instVarAtPut(m_value, IntervalStep, fromInt(val)); }
    	GenieVariableType read(IntermediateFile* gif);
    	GenieVariableType write(IntermediateFile* gif);
    	GenieVariableType skip(IntermediateFile* gif);
    	GenieVariableType readAscii(fstream& fs);
    	GenieVariableType writeAscii(fstream& fs, int indent = 0);
};

#endif /* GENIE_INTERVAL */
