/*  Open GENIE, a program for neutron scattering data analysis & display
    ISIS facility, Rutherford Appleton Laboratory, Chilton, Didcot, OXON UK
    Copyright (C) 1996,1997

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA. */

#include "geniedefs.h"
#include <gnustring.h>

#ifdef _WIN32
#include <strstrea.h>
#else
#include <strstream.h>
#endif

#include "genie_ostream.h"


//  example usage
//
// main()
// {
//	 genie_ostream gout(cout, "(", ")"), d;
//	 d = cerr;
//
//	 d << "WOW! " << 3 << endl;
//	 gout << "Hello " << 456.9 << endl;
//
//	return 0;
// }

// normal constructor
genie_ostream::genie_ostream(ostream& s, String b, String e, int screen_output, int log) : start(b), stop(e), buffer_length(2048),
	buffer(new char[buffer_length]), sofar(buffer, buffer_length, ios::out), logging(log), screen(screen_output)
{
    init(s.rdbuf());
}


// Allows assignment of ordinary iostreams if desired
genie_ostream& genie_ostream::operator=(ostream& s)
{
	if (&s != this)
    	{
		init(s.rdbuf());
	}
	return *this;
}

genie_ostream& genie_ostream::flush()
{
    if (logging)
    {
        logit(str());
    }
    if (screen)
    {
	(void)str();			// bin ostrstream output
        ostream::flush();
    }
    return *this;
}


const char* genie_ostream::str()
{
    sofar << ends;
    sofar.seekp(ios::beg);
    return buffer;
}

genie_ostream& genie_ostream::operator<<(ostream &(*a)(ostream &))
{ 
    if (screen) 
    	ostream::operator<<(a); 
    sofar << a; 
    return *this;
}

genie_ostream& genie_ostream::operator<<(ios &(*a)(ios &))
{ 
    if (screen) 
	ostream::operator<<(a);
    sofar << a;   
    return *this; 
}
