/*  Open GENIE, a program for neutron scattering data analysis & display
    ISIS facility, Rutherford Appleton Laboratory, Chilton, Didcot, OXON UK
    Copyright (C) 1996,1997

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA. */

#ifndef __GENIE_OSTREAM__
#define __GENIE_OSTREAM__

class genie_ostream : public ostream
{

private:
    String      start, stop;
    const int 	buffer_length;
    char* 	buffer;
    ostrstream  sofar;
    int         logging, screen;    // (Was defined as bool)

    // constructors

public:
	genie_ostream(const genie_ostream& s) : buffer_length(0), buffer(0), logging(0), screen(1) {  }
	genie_ostream& operator=(const genie_ostream& s) { return *this; }

// moved this up two lines from here for Linux & gcc 2.6.3
//public:
	inline genie_ostream() : buffer_length(0), buffer(0), logging(0), screen(1) { }

	// screen_output = 1    -> echo to screen 
	genie_ostream(ostream& s, String b="", String e="", int screen_output = 1, int log = 0);

	// destructor
	inline ~genie_ostream() { delete buffer; }

	// return pointer to characters so far in strstream, and reset to to start
	const char* str();

	// switch stream on or off, for no parameter, just report current setting
	int mode( int mode = 2 ) { if (mode==2) return screen; else { screen = mode; return screen; } }

	genie_ostream &operator=(ostream& s);

    	// special flush for logging (if enabled)
    	genie_ostream& flush();

	// The member we want to redefine to bracket the output
	inline genie_ostream &operator<<(const char *a)
        { if (screen) { ostream::operator<<(start) << a << stop; }; sofar << a; return *this; }
	inline genie_ostream &operator<<(char a)
        { if (screen) { ostream::operator<<(start) << a << stop; }; sofar << a; return *this; }
	inline genie_ostream &operator<<(short a)
        { if (screen) { ostream::operator<<(start) << a << stop; }; sofar << a; return *this; }
	inline genie_ostream &operator<<(int a)
        { if (screen) { ostream::operator<<(start) << a << stop; }; sofar << a; return *this; }
	inline genie_ostream &operator<<(long a)
        { if (screen) { ostream::operator<<(start) << a << stop; }; sofar << a; return *this; }
	inline genie_ostream &operator<<(unsigned char a)
        { if (screen) { ostream::operator<<(start) << a << stop; }; sofar << a; return *this; }
	inline genie_ostream &operator<<(unsigned short a)
        { if (screen) { ostream::operator<<(start) << a << stop; }; sofar << a; return *this; }
	inline genie_ostream &operator<<(unsigned int a)
        { if (screen) { ostream::operator<<(start) << a << stop; }; sofar << a; return *this; }
	inline genie_ostream &operator<<(unsigned long a)
        { if (screen) { ostream::operator<<(start) << a << stop; }; sofar << a; return *this; }
//#ifdef __ultrix
      inline genie_ostream &operator<<(void *a)		// prototype for "const void*" not present
//#else
//      inline genie_ostream &operator<<(const void *a)
//#endif /* __ultrix */
        { if (screen) { ostream::operator<<(start) << a << stop; }; sofar << a; return *this; }
	inline genie_ostream &operator<<(streambuf *a)
        { if (screen) { ostream::operator<<(start) << a << stop; }; sofar << a; return *this; }
	inline genie_ostream &operator<<(float a)
        { if (screen) { ostream::operator<<(start) << a << stop; }; sofar << a; return *this; }
	inline genie_ostream &operator<<(double a)
        { if (screen) { ostream::operator<<(start) << a << stop; }; sofar << a; return *this; }
	inline genie_ostream &operator<<(const String& a)
        { if (screen) { ostream::operator<<(start) << a << stop; }; sofar << a; return *this; }

	// No point in bracketing the output of IO manips!
	genie_ostream &operator<<(ostream &(*a)(ostream &));
	genie_ostream &operator<<(ios &(*a)(ios &));

	// redefine write() to add start and stop sequences
	ostream& write(const char *s, size_t n)
	{
		if (screen)
		{
			ostream::write(start, start.length());
			ostream::write((const char*)s, n);
			ostream::write(stop, stop.length());
		};
		sofar.write((const char*)s, n);
		return *this;
	}

/*
   	ostream& write(const unsigned char *s, streamsize n)
      		{ return write((const char*)s, n);}
    	ostream& write(const signed char *s, streamsize n)
      		{ return write((const char*)s, n);}
    	ostream& write(const void *s, streamsize n)
      		{ return write((const char*)s, n);}
*/

};

#endif
