/*  Open GENIE, a program for neutron scattering data analysis & display
    ISIS facility, Rutherford Appleton Laboratory, Chilton, Didcot, OXON UK
    Copyright (C) 1996,1997

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA. */

#ifndef __GENIE_TYPES__
#define __GENIE_TYPES__

#include <limits.h>
#include <float.h>

#ifdef __cplusplus
extern "C" {
#endif

#define MAX_WORK_FIELDS 256
#define MAX_DIMS 10
#define MIN(a,b) (((a)<(b))?(a):(b))	/* Used in printGvar */

#define Strdup(a) strcpy((char *)M_alloc(strlen(a)+1),a)

/* Typedefs for GENIE types in C */
typedef enum
{
    _Undefined=0, _Integer, _Real, _String, _Workspace, _IntegerArray, _RealArray, _StringArray, _WorkspaceArray
} g_var_types;

/* Forward declarations */
typedef struct work_element_struct  work_element;	    /* a single workspace field */
typedef struct g_variable_struct    g_variable;		    /* holds any g_variable */

/* Simple types */
typedef long		g_integer;
typedef double		g_real;
typedef char		*g_string;

/* Workspace type */
typedef struct g_workspace_struct   { long len; work_element *elems; } *g_workspace;

/* Array types */
typedef struct g_arr_integer_struct   { long dims[MAX_DIMS], dope[MAX_DIMS]; g_integer   *values; } *g_arr_integer;
typedef struct g_arr_real_struct      { long dims[MAX_DIMS], dope[MAX_DIMS]; g_real	 *values; } *g_arr_real;
typedef struct g_arr_string_struct    { long dims[MAX_DIMS], dope[MAX_DIMS]; g_string    *values; } *g_arr_string;
typedef struct g_arr_workspace_struct { long dims[MAX_DIMS], dope[MAX_DIMS]; g_workspace *values; } *g_arr_workspace;

/* A genie variable type */
struct g_variable_struct {
    g_var_types	    its_type;
    union g_variable_union {
	g_integer	g_i;
	g_real		g_r;
	g_string	g_s;
	g_workspace	g_w;
	g_arr_integer	g_i_a;
	g_arr_real	g_r_a;
	g_arr_string	g_s_a;
	g_arr_workspace g_w_a;
    } it;
};

struct work_element_struct {
    char		name[SHORTCHARBUFF];	/* name of field */
    g_variable		value;			/* value of variable */
};


/*
 * Macros for easy access to the structures
 */
#define getInteger(__gvar)		((__gvar).its_type==_Undefined?LONG_MAX:(__gvar).it.g_i)
#define putInteger(__gvar,__val)	( (__val)==LONG_MAX ? ((__gvar).its_type = _Undefined,LONG_MAX): \
					    ((__gvar).its_type = _Integer, (__gvar).it.g_i = (__val)) )

/*#define putInteger(__gvar,__val)	((__gvar).its_type = _Integer, (__gvar).it.g_i = (__val))*/

#define getReal(__gvar)			((__gvar).its_type==_Undefined?DBL_MAX:(__gvar).it.g_r)
#define putReal(__gvar,__val)		( (__val)==DBL_MAX ? ((__gvar).its_type = _Undefined,DBL_MAX) : \
					    ((__gvar).its_type = _Real, (__gvar).it.g_r = (__val)) )

/*#define putReal(__gvar,__val)		((__gvar).its_type = _Real, (__gvar).it.g_r = (__val))*/

#define getString(__gvar)		((__gvar).its_type==_Undefined?"(undefined)":(__gvar).it.g_s)
#define putString(__gvar,__val, __len)	((__gvar).its_type = _String, (__gvar).it.g_s=strndup((__val), (__len)))

#define getWorkspace(__gvar)		((__gvar).its_type==_Undefined?newGworkspace(1).it.g_w:(__gvar).it.g_w)
#define putWorkspace(__gvar,__val)	((__gvar).its_type = _Workspace, (__gvar).it.g_w = (__val))


/* check for operating on a workspace type */
#define _AA(__work,__expr,__dummy) \
    ( ((__work).its_type==_Workspace)?(__expr):(PERROR("get-or-put(WorkspaceAccess)"),__dummy) )

#define getElementByIndex(__work, __index) \
    _AA(__work, (__work).it.g_w->elems[__index].value, UNDEFINED)

#define putElementByIndex(__work, __index, __val) \
    _AA(__work, (__work).it.g_w->elems[__index].value = (__val), UNDEFINED)

#define getElementByField(__work, __field) \
    _AA(__work, ((__work).it.g_w->elems[findElementIndex((__work), (__field), 0)].value), UNDEFINED)

#define putElementByField(__work, __field, __val) \
    _AA(__work, (__work).it.g_w->elems[findElementIndex((__work), (__field), 0)].value = (__val), UNDEFINED)

#define getFieldByIndex(__work, __index) \
    _AA(__work, (__index<wrklen(__work)?(__work).it.g_w->elems[__index].name:0), (char *) 0)

#define putFieldByIndex(__work, __index, __val) \
    _AA(__work, (__index<wrklen(__work)?strcpy((__work).it.g_w->elems[__index].name, (__val)):0), (char *) 0)


/* Build in checks for operating on array types */
#define arrayGet(__arr,__index,__arrtype,__accessor,__cast) \
    ((__arr.its_type==__arrtype)?\
	__arr.it.__accessor->values[__index]:\
	__cast (PERROR("arrayGet"),-1))

#define arrayPut(__arr,__index,__val,__arrtype,__accessor,__cast)\
    ((__arr.its_type==__arrtype)?\
	(__arr.its_type=__arrtype, __arr.it.__accessor->values[__index] = (__val)):\
	__cast (PERROR("arrayPut"),0))

#define arrayGetInteger(__arr,__index)		 arrayGet((__arr), (__index), _IntegerArray, g_i_a, (g_integer))
#define arrayPutInteger(__arr,__index, __val)	 arrayPut((__arr), (__index), (__val), _IntegerArray, g_i_a, (g_integer))
#define arrayGetReal(__arr,__index)		 arrayGet((__arr), (__index), _RealArray, g_r_a, (g_real))
#define arrayPutReal(__arr,__index, __val)	 arrayPut((__arr), (__index), (__val), _RealArray, g_r_a, (g_real))
#define arrayGetString(__arr,__index)		 arrayGet((__arr), (__index), _StringArray, g_s_a, (g_string))

#define arrayPutString(__arr,__index, __val, __len) \
    arrayPut((__arr), (__index), strndup((__val),(__len)), _StringArray, g_s_a, (g_string))

#define arrayGetWorkspace(__arr,__index)	 arrayGet((__arr), (__index), _WorkspaceArray, g_w_a, (g_workspace))

#define arrayPutWorkspace(__arr,__index,__val) \
    arrayPut((__arr), (__index), wrkndup((__val), wrklenp(__val)), _WorkspaceArray, g_w_a, (g_workspace))


#define arrayGetDims(__arr)		    ((__arr).it.g_i_a->dims)	    /*** any accessor will do here ***/
#define arrayGetDope(__arr)		    ((__arr).it.g_i_a->dope)

#define arrayGetAllInteger(__arr) \
    ( (g_integer *)(__arr.its_type==_IntegerArray?__arr.it.g_i_a->values:(g_integer *) (PERROR("arrayGetAllInteger"),0)) )
#define arrayGetAllReal(__arr) \
    ( (g_real *)(__arr.its_type==_RealArray?__arr.it.g_r_a->values:(g_real *) (PERROR("arrayGetAllReal"),0)) )
#define arrayGetAllString(__arr) \
    ( (g_string *)(__arr.its_type==_StringArray?__arr.it.g_s_a->values:(g_string *) (PERROR("arrayGetAllString"),0)) )
#define arrayGetAllWorkspace(__arr) \
    ( (g_workspace *)(__arr.its_type==_WorkspaceArray?__arr.it.g_w_a->values:(g_workspace *) (PERROR("arrayGetAllWorkspace"),0)) )


#define variableType(__var)		    ((__var).its_type)
#define workspaceLength(__work)		    ((__work).it.g_w->len)	    /*** maximum length of workspace ***/
#define wrklen(__work)			    wrklenp(getWorkspace(__work))   /*** number of fields used ***/

#define undefineGvar(__var)		    ((__var).its_type = _Undefined)
#define undefined(__var)		    ((__var).its_type == _Undefined)

/*
 * External function declarations
 */
g_variable newGinteger(long val);		    /* Creation of variables (allocation of store where necessary) */
g_variable newGreal(double val);
g_variable newGstring(const char* val, int len);
g_variable newGworkspace(int num_elems);
g_variable newGarrayReal(int num_elems);
g_variable newGarrayInteger(int num_elems);
g_variable newGarrayString(int num_elems);
g_variable newGarrayWorkspace(int num_elems);
	
int wrklenp(g_workspace workspace);						/* Length of a workspace */
g_workspace wrkndup( g_workspace work, int len );
char *strndup(const char *string, int length);

int addToWorkspace(g_variable workspace, const char *field, g_variable value);	/* Value is passed by reference */
int findElementIndex( g_variable workspace, const char *field, int failflag );	/* Workspace field name to index utility */
long ArrayLength( g_variable array );						/* Array length in total elements */
char* printGvar(char outbuff[], g_variable var);				/* Prints a representation of ANY g_variable */


const g_variable UNDEFINED = { _Undefined, {0} };


void *M_alloc(size_t len);
size_t M_freeup();

#ifdef __cplusplus
}
#endif

#endif /* __GENIE_TYPES__ */
