/*  Open GENIE, a program for neutron scattering data analysis & display
    ISIS facility, Rutherford Appleton Laboratory, Chilton, Didcot, OXON UK
    Copyright (C) 1996,1997

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA. */

#ifndef GENIE_VAR
#define GENIE_VAR

#ifdef __GNUG__
#pragma interface
#endif /* __GNUG__ */

class IntermediateFile;

class GenieVariable
{
public:
    OOP m_params;
    int m_location;
    GenieVariableType m_type;
    union
    {
	GenieVariableBase* gvb;
	GenieInteger* gi;
	GenieReal* gr;
	GenieString* gs;
	GenieWorkspace* gw;
	GenieIntegerArray* gia;
	GenieRealArray* gra;
	GenieStringArray* gsa;
	GenieWorkspaceArray* gwa;
	void* gu;	// for GenieUndefinedType
    };
    void freeType();
    GenieVariableType type() const;
    void newType(GenieVariableType type, OOP value_oop = nilOOP, OOP subclass_oop = undefinedObjectClass);    
    void newType(const char* subclass_type, OOP value_oop = nilOOP);
    void checkType(GenieVariableType type);
    int defined() const;
    GenieVariableType write(IntermediateFile* gif);
    GenieVariableType read(IntermediateFile* gif);
    GenieVariableType readAscii(fstream& fs);
    GenieVariableType writeAscii(fstream& fs, int indent = 0);
    static GenieVariableType skip(IntermediateFile* gif);
    operator g_integer();
    operator c_g_string();
    operator g_real();
    operator GenieInteger();
    operator GenieReal();
    operator GenieString();
    operator GenieIntegerArray();
    operator GenieIntegerArray*();
    operator GenieRealArray();
    operator GenieRealArray*();
    operator GenieStringArray();
    operator GenieStringArray*();
    operator GenieWorkspace();
    operator GenieWorkspaceArray();
    GenieVariableBase* operator&() { return pointerType(); }
//    GenieVariableBase* operator->();
    GenieVariableBase* pointerType();
    const GenieVariableBase* pointerTypeC() const;
    GenieVariable(OOP oop = nilOOP);
    GenieVariable(OOP parameters, int location);		// anonymous callout
    GenieVariable(GenieVariableType gvt);
    GenieVariable(const GenieVariable& gv);
    GenieVariable(const GenieVariableBase& gvb);
    GenieVariable& operator=(const GenieVariable& gv);
    ~GenieVariable();
    int associate(OOP parameters, const char* name);
    int associate(const GenieCalloutList& gcl, const char* name);
    GenieVariable(const GenieCalloutList& gcl, const char* name);
    friend ostream& operator<<(ostream& os, const GenieVariable& gv);
    ostream& print(ostream& os, const char* note = NULL) const;
    static OOP deepCopyOOP(OOP oop);
    GenieVariable& operator=(const GenieVariableBase& gvb);
    OOP inspect();
    OOP value() const;
    OOP become(OOP oop);
    operator OOP() const;
};

#endif /* GENIE_VAR */
