/*  Open GENIE, a program for neutron scattering data analysis & display
    ISIS facility, Rutherford Appleton Laboratory, Chilton, Didcot, OXON UK
    Copyright (C) 1996,1997

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA. */

/*
 *  function to log a message to a server process via UDP
 *  Freddie Akeroyd, 28/6/94
 */
#include "geniedefs.h"
/* Fiddle these to allow non-posix types which are used in socket.h and in.h */
//#undef _POSIX_SOURCE

#if defined(__VMS) && __VMS_VER < 70000000
#include "vms_password_funcs.h"
#include "strcasecmp.h"
#endif /* __VMS && __VMS_VER < 70000000 */

static const char* rcsid = "@(#)$Id: genielog.cxx,v 1.21 1997/06/29 15:15:27 faa Exp $";
USE_RCSID(rcsid);

#include <stdio.h>
#include <stdlib.h>

#ifdef HAVE_UNISTD_H
#	include <unistd.h>
#else
#	include <io.h>
#endif

#include <sys/types.h>
#ifdef _WIN32
#	include <winsock.h>
#else
#	include <sys/socket.h>
#	include <netdb.h>
#	include <netinet/in.h>
#	include <arpa/inet.h>
#	include <pwd.h>
#endif
#include <string.h>
#include <time.h>
#include "genielog.h"

// Sort out a mess on Digital UNIX 4.0 
#ifdef HAVE_LIBC_R
#if _POSIX_C_SOURCE>=199506L	/* P1003.1c */
        extern "C" int getpwnam_r(const char*, struct passwd *, char *, size_t, struct passwd **);
        extern "C" int getpwuid_r(uid_t, struct passwd *, char *, size_t, struct passwd **);
#	define GETPWNAM(name,pwname) getpwnam_r(name, NULL, buffer1, (int)sizeof(buffer1),&pwname)
#       define GETPWUID(uid,pwname) getpwuid_r(getuid(), NULL, buffer1, sizeof(buffer1),&pwname)
#else
#	define GETPWNAM(name,pwname) getpwnam_r(name, pwname, buffer1, (int)sizeof(buffer1))
#       define GETPWUID(uid,pwname) getpwuid_r(getuid(), pwname, buffer1, sizeof(buffer1))
#endif
#else
#	define GETPWNAM(name,pwname) pwname = getpwnam(name)
#       define GETPWUID(uid,pwname) pwname = getpwuid(uid)
#endif /* HAVE_LIBC_R */

#if defined(__ultrix) && defined(__DECCXX)
extern "C" u_short htons(u_short s);
#endif

#ifndef SIABUFSIZ
const unsigned long SIABUFSIZ = 2048;		// Room for password structure
#endif

#define	MACHINE		"isolde.nd.rl.ac.uk"	/* where to send message */
#define	MACHINE_IP	"130.246.49.21"		/* where to send message */
/* #define USE_GETHOSTBYNAME 	/* Lookup MACHINE rather than just use MACHINE_IP - can cause a non-networked machine to hang */
#define	PORT	1157			/* udp port on MACHINE */

/* #define	DEBUG				/* if want errors printed to stdout */

// On success, return socket file descriptor and populate sockaddr_in
// On error, Return -1
static int get_socket_descriptor(struct sockaddr_in* ps_in)
{
    int s;
    struct hostent* hostp;
#ifdef USE_GETHOSTBYNAME
    if ((hostp = gethostbyname(MACHINE)) == 0)
    {
#    ifdef	DEBUG
	gprintef("Cannot determine address of \"%s\"\n", MACHINE);
	herror("gethostbyname");
#    endif
	return -1;
    }
#else	/* emulate gethostbyname() */
    in_addr_t a = inet_addr(MACHINE_IP);
    if (a == (in_addr_t)-1)
    {
#    ifdef	DEBUG
	gprintef("Cannot convert address \"%s\"\n", MACHINE_IP);
#    endif
	return -1;
    }
    hostp = new hostent;
    hostp->h_addr_list = new char*[1];
    hostp->h_addr_list[0] = (char*)&a;
    hostp->h_length = sizeof(in_addr_t);
#endif /* USE_GETHOSTBYNAME */
    if ((s = socket(AF_INET, SOCK_DGRAM, 0)) == -1)
    {
#ifdef	DEBUG
	perror("genielog - socket");
#endif
	return -1;
    }
    memset(ps_in, 0, sizeof(struct sockaddr_in));
    ps_in->sin_family = AF_INET;
    ps_in->sin_port = htons(PORT);
    memcpy(&(ps_in->sin_addr.s_addr), hostp->h_addr_list[0], hostp->h_length);
    return s;
}
/*
 *  attempt to send message to udp port PORT at internet host MACHINE
 *  return 0 on success, -1 on error
 */
int genielog(const char* message)
{
    static int first_call = 1;
    static int s = -1;	// open socket file descriptor
    static struct sockaddr_in s_in;
    static time_t session_id;
    int blen;
    int retval = 0;
    char header[256];
    char *name, *buffer, buffer1[SIABUFSIZ]; 
#ifdef HAVE_LIBC_R
    struct passwd* pwname = new passwd;
#else
    struct passwd* pwname = 0;
#endif /* HAVE_LIBC_R */

// Do we allow logging?
    name = getenv("GENIE_LOGGING");
    if (name == 0 || strcasecmp(name, "yes") != 0)
	return 0;

    if (message == 0)
	return -1;

    if (first_call == 1)
    {
	first_call = 0;
	session_id = time(NULL);
	s = get_socket_descriptor(&s_in);
    }
    if (s == -1)
	return -1;

#ifndef _WIN32
    name = getlogin();
    if (name != 0)	// interactive
    {
	    GETPWNAM(name,pwname);
    }
    else		// non-interactive
    {
	    GETPWUID(getuid(),pwname);
    }
#else
	GetUserName(name, (unsigned long *)SIABUFSIZ);
#endif

#ifdef __VMS
    sprintf(header, "From user %s\nSession-id: %ld\n", (pwname == 0 ? "unknown" : pwname->pw_name), (unsigned long)session_id);
#else
#ifdef _WIN32
	sprintf(header, "From user %s\nSession-id: %ld\n", (name ? "unknown" : name), (unsigned long)session_id);
#else
    sprintf(header, "From user %s (%s)\nSession-id: %ld\n", (pwname == 0 ? "unknown" : name), 
				(pwname == 0 ? "" : pwname->pw_gecos), (unsigned long)session_id);
#endif /* _WIN32 */
#endif /* __VMS */

#ifdef HAVE_LIBC_R
    delete pwname;
    pwname = 0;
#endif /* HAVE_LIBC_R */
    buffer = (char*)malloc(1+strlen(header)+strlen(message));
    if (buffer == 0)
    {
#ifdef	DEBUG
	fprintf(stderr, "genielog: malloc failure\n");
#endif
	return -1;
    }
    buffer[0] = '\0';
    strcat(buffer, header);
    strcat(buffer, message);
    blen = strlen(buffer);

    if (sendto(s, buffer, blen, 0, (struct sockaddr*)&s_in, sizeof(struct sockaddr_in)) != blen)
    {
	retval = -1;
#ifdef	DEBUG
	perror("genielog - sendto");
#endif
    }
    free(buffer);
    return retval;
}
