/*  Open GENIE, a program for neutron scattering data analysis & display
    ISIS facility, Rutherford Appleton Laboratory, Chilton, Didcot, OXON UK
    Copyright (C) 1996,1997

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA. */

#include "geniedefs.h"
#include "hdf_utils.h"
#include "hdf_dfsd.h"

DFSD::DFSD(HDFFile& hdf, uint16 ref) : HDFObject(hdf, DFTAG_SD, ref), m_rank(-1), m_data(0), m_dims_sizes(0), m_type(INVALID_TYPE),
	m_label(0), m_unit(0), m_format(0), m_coordsys(0)
{
}
    
DFSD::DFSD(HDFFile& hdf, int32 type, int32 rank, int32* dims_sizes, void* data, uint16 ref) : HDFObject(hdf, DFTAG_SD, ref),
	m_rank(rank), m_data(data), m_dims_sizes(dims_sizes), m_type(type), m_label(0), m_unit(0), m_format(0), m_coordsys(0)
{
}

int DFSD::readFromFile()
{
    intn ndims, status;
    if (m_ref != INVALID_REF)
    {
        status = DFSDreadref((char*)(const char*)m_filename, m_ref);
	HDF_ERROR("DFSD::readFromFile", status, -1);
    }
    status = DFSDgetdims(m_filename, &ndims, 0, 0);
    HDF_ERROR("DFSD::readFromFile", status, -1);
    m_dims_sizes = new int32[ndims];
    status = DFSDgetdims(m_filename, &m_rank, m_dims_sizes, ndims);
    HDF_ERROR("DFSD::readFromFile", status, -1);
    status = DFSDgetNT(&m_type);
    HDF_ERROR("DFSD::readFromFile", status, -1);
    readAttributes();
    int n = 1;
    for(int i=0; i<m_rank; i++)
    {
	n *= m_dims_sizes[i];
    }
    m_data = new double[n];
    status = DFSDgetdata(m_filename, m_rank, m_dims_sizes, m_data);
    HDF_ERROR("DFSD::readFromFile", status, -1);
    status = DFSDclear();
    HDF_ERROR("DFSD::writeToFile", status, -1);
    return 0;
}

int DFSD::writeToFile()
{
    int32 status;
    if (m_ref != INVALID_REF)
    {
	status = DFSDwriteref(m_filename, m_ref);
	HDF_ERROR("DFSD::writeToFile", status, -1);
    }
    status = DFSDsetdims(m_rank, m_dims_sizes);
    HDF_ERROR("DFSD::writeToFile", status, -1);
    status = DFSDsetNT(m_type);
    HDF_ERROR("DFSD::writeToFile", status, -1);
    writeAttributes();
    status = DFSDadddata(m_filename, m_rank, m_dims_sizes, m_data);
    HDF_ERROR("DFSD::writeToFile", status, -1);
    setRef(DFSDlastref());
    status = DFSDclear();
    HDF_ERROR("DFSD::writeToFile", status, -1);
    return 0;
}

int DFSD::writeAttributes()
{
    int32 status = DFSDsetdatastrs(m_label, m_unit, m_format, m_coordsys);
    HDF_ERROR("DFSD::writeAttributes", status, -1);
    return 0;
}

int DFSD::readAttributes()
{
    int32 status;
    intn label_len, unit_len, format_len, coordsys_len;
    status = DFSDgetdatalen(&label_len, &unit_len, &format_len, &coordsys_len);
    HDF_ERROR("DFSD::readAttributes", status, -1);
    m_label = new char[label_len+1];
    m_unit = new char[unit_len+1];
    m_format = new char[format_len+1];
    m_coordsys = new char[coordsys_len+1];
    status = DFSDgetdatastrs(m_label, m_unit, m_format, m_coordsys);
    HDF_ERROR("DFSD::readAttributes", status, -1);
    return 0;
}

ostream& DFSD::print(ostream& os) const
{
    return os;
}

int DFSD::numberInFile(HDFFile& hdf)
{
    int n = DFSDndatasets((char*)hdf.name());
    HDF_ERROR("DFSD::numberInFile", n, -1);
    return n;
}

// File not passed as only one is allowed open
int DFSD::rewind(HDFFile& hdf)
{
    int status = DFSDrestart();
    HDF_ERROR("DFSD::rewind", status, -1);
    return 0;
}
