/*  Open GENIE, a program for neutron scattering data analysis & display
    ISIS facility, Rutherford Appleton Laboratory, Chilton, Didcot, OXON UK
    Copyright (C) 1996,1997

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA. */

#ifndef HDF_FILE
#define HDF_FILE

class HDFObject;

class HDFFile
{
    protected:
        String m_filename;              // Filename for Hopen
        int32 m_access_mode;            // Access mode used in Hopen
	int32 m_file_id;		// file_id from Hopen
        uint32 m_major_v;               // Major version number of HDF library
        uint32 m_minor_v;               // Minor version number of HDF library
        uint32 m_release;               // Complete library version number
        char m_description[80];         // Additional library version information
	void getFileVersion();		// Read HDF library version used on file
 	int numberOfAnnotations(ann_type type) const;
	int addFileAnnotation(const char* annotation, ann_type type);
	char* readFileAnnotation(int index, ann_type type) const;			// string on heap

    public:
	HDFFile() : m_file_id(INVALID_FID) {}
	int open(const char* filename, int32 file_access_mode, int32 n_dds);
	int close();
	const char* name() const { return m_filename; }
	int32 id() const { return m_file_id; }
	int32 mode() const { return m_access_mode; }
	~HDFFile() { close(); }
	int numberOfObjects(int32 tag = DFTAG_WILDCARD) const;
	int findNext(uint16 search_tag = DFTAG_WILDCARD, uint16 search_ref = DFREF_WILDCARD, intn direction = DF_FORWARD);
	ostream& print(ostream& os) const;
	int addFileDescription(const char* description) { return addFileAnnotation(description, AN_FILE_DESC); }
	int addFileLabel(const char* label) { return addFileAnnotation(label, AN_FILE_LABEL); }
	int numberOfFileLabels() const { return numberOfAnnotations(AN_FILE_LABEL); }
	int numberOfFileDescriptions() const { return numberOfAnnotations(AN_FILE_DESC); }
	int numberOfDataLabels() { return numberOfAnnotations(AN_DATA_LABEL); }
	int numberOfDataDescriptions() { return numberOfAnnotations(AN_DATA_DESC); }
	char* readFileLabel(int index) const { return readFileAnnotation(index, AN_FILE_LABEL); }		// string on heap
	char* readFileDescription(int index) const { return readFileAnnotation(index, AN_FILE_DESC); }	// string on heap
//	int addObject(HDFObject& object);
};

inline ostream& operator<<(ostream& os, const HDFFile& hdf)
{
    return hdf.print(os);
}

#endif /* HDF_FILE */
