/*  Open GENIE, a program for neutron scattering data analysis & display
    ISIS facility, Rutherford Appleton Laboratory, Chilton, Didcot, OXON UK
    Copyright (C) 1996,1997

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA. */

#ifndef HDF_INTERMEDIATE_FILE
#define HDF_INTERMEDIATE_FILE

#include "intermediate_file.h"
#include "hdf_utils.h"
#include "hdf_file.h"

class Vgroup;

class HDFIntermediateFile : public IntermediateFile
{
    private:
	HDFFile m_file;

    public:
	int open(const char* filename, ModeFlags mode = Append);
	int close();
 	int numberOfBlocks() const;
	HDFIntermediateFile();
	ostream& print(ostream& os) const { return os << m_file << endl; }
	int appendBlock(GenieVariable& gv, const char* label = "<no label>", const char* comment = "<no comment>");
	int appendBlock(GenieVariableBase& gvb, const char* label = "<no label>", const char* comment = "<no comment>");
        int skipInteger() { return 1; }
        int readInteger(g3io_int_t& i) { return 1; }
        int writeInteger(g3io_int_t i) { return 1; }
    	int skipString() { return 1; }
    	int freeString(char*& s) { return 1; }
    	int readString(char*& s) { return 1; }
    	int writeString(const char* s) { return 1; }
    	int skipReal() { return 1; }
    	int readReal(g3io_real_t& r) { return 1; }
    	int writeReal(g3io_real_t r) { return 1; }
        int readRealArray(GenieRealArray& gra) { return 1; }
        int writeRealArray(GenieRealArray& gra) { return 1; }
        int skipRealArray() { return 1; }
   	int skipIntegerArray() { return 1; }
    	int readIntegerArray(GenieIntegerArray& ia) { return 1; }
    	int writeIntegerArray(GenieIntegerArray& ia) { return 1; }
    	int readStringArray(GenieStringArray& gsa) { return 1; }
    	int writeStringArray(GenieStringArray& gsa) { return 1; }
    	int skipStringArray() { return 1; }
    	int readItemSubclassType(char*& subclass_name) { return 1; }
    	int writeItemSubclassType(const char* subclass_name) { return 1; }
    	int skipItemSubclassType() { return 1; }
    	GenieVariableType readItemType() { return GenieTypeError; }
    	int skipItemType() { return 1; }
    	int writeItemType(GenieVariableType gvt) { return 1; }
    	int skipOffset() { return 1; }
    	int writeOffset(u_int saved_offset) { return 1; }    // write the current offset to a previosly allocated offset field
    	int saveOffset(u_int& saved_offset) { return 1; }    // allocate space for an offset field and make a not of its position
    	int readOffset(u_int& offset) { return 1; }
    	u_int setFileOffset(u_int offset) { return 1; }
    	int readArrayDims(GenieArrayBase& gab) { return 1; }
    	int writeArrayDims(const GenieArrayBase& gab) { return 1; }

	int addGenieVariable(GenieVariable& gv, Vgroup& vg, const char* name, int set_class = 1);
};

#endif /* HDF_INTERMEDIATE_FILE */
