/*  Open GENIE, a program for neutron scattering data analysis & display
    ISIS facility, Rutherford Appleton Laboratory, Chilton, Didcot, OXON UK
    Copyright (C) 1996,1997

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA. */

#ifndef HDF_OBJECT
#define HDF_OBJECT

#include "hdf_file.h"

class Vgroup;

class HDFObject
{
    protected:
	uint16 m_ref;						// reference number
	uint16 m_tag;						// tag
	int32 m_file_id;					// file_id we are attached to
	String m_filename;
	HDFObject() : m_ref(INVALID_REF), m_tag(INVALID_TAG), m_file_id(INVALID_FID) {} 
	char* readAnnotation(int index, ann_type type) const;
	int numberOfAnnotations(ann_type type) const;

    public:
	HDFObject(HDFFile& hdf, uint16 tag = INVALID_TAG, uint16 ref = INVALID_REF) : m_ref(ref), m_tag(tag), m_file_id(hdf.id()),
		m_filename(hdf.name()) {}
        HDFObject(const char* filename, int32 file_id, uint16 tag = INVALID_TAG, uint16 ref = INVALID_REF) : m_ref(ref), 
		m_tag(tag), m_file_id(file_id), m_filename(filename) {}
	virtual ~HDFObject() {}
	int ref() const { return m_ref; }
	int tag() const { return m_tag; }
	const char* fileName() const { return m_filename; }
	int32 fileId() const { return m_file_id; }
	void setTag(int32 tag) { m_tag = tag; }
	void setRef(int32 ref) { m_ref = ref; }
	void setFile(const HDFFile& hdf) { m_file_id = hdf.id(); m_filename = hdf.name(); }
	void setFile(const char* filename, int32 file_id) { m_file_id = file_id; m_filename = filename; }
	virtual int insertInVgroup(Vgroup& vg);	// insert ourselves into a vgroup
	virtual int addToFile(HDFFile& hdf);
// Multifile Annotation (AN) interface 
	int addAnnotatation(const char* annotation, ann_type type);
	int addLabel(const char* label) { return addAnnotatation(label, AN_DATA_LABEL); }
	int addDescription(const char* description) { return addAnnotatation(description, AN_DATA_DESC); }
	int numberOfLabels() const { return numberOfAnnotations(AN_DATA_LABEL); }
	int numberOfDescriptions() const { return numberOfAnnotations(AN_DATA_DESC); }
	char* readLabel(int index) const { return readAnnotation(index, AN_DATA_LABEL); }
	char* readDescription(int index) const { return readAnnotation(index, AN_DATA_DESC); }
	ostream& print(ostream& os) const;
};

inline ostream& operator<<(ostream& os, const HDFObject& object)
{
    return object.print(os);
}

#endif /* HDF_OBJECT */
