/*  Open GENIE, a program for neutron scattering data analysis & display
    ISIS facility, Rutherford Appleton Laboratory, Chilton, Didcot, OXON UK
    Copyright (C) 1996,1997

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA. */

#ifndef HDS_SDS_H
#define HDS_SDS_H

// Routines returning an "int" return -1 on error and (usually) 0 on success
// Routines returning a pointer return NULL on error

class SDS
{
    private:
        int32 m_sd_id;		// file handle from SDstart
        int32 m_sds_id;		// data set handle from SDselect
	int flush();		// terminate access to current m_sds_id, and ready for moving to a new SDS
				// flush() MUST be called before "m_sds_id" is changed

    public:
	SDS(const char* filename, int32 access_mode) : 
		m_sd_id(-1), m_sds_id(-1), m_filename(filename), m_access_mode(access_mode) {}
	int open();
	int seek(int32 sds_index);
	int seek(const char* sds_name);
	int refToIndex(int32 ref);
	int idToRef(int32 sds_id);
	int ref() { return idToRef(m_sds_id); }		// reference number of current block
	int close();
	int write(const char* sds_name, const char* comment, int32 data_type, int32 ndims, int32* dims_array, void* data);
	int read(int32 ndims, int32* dims_array, void* data);
	~SDS() { close(); }
	int32* blockInfo(int32& ndims, int32& data_type);
	const char* blockName();
	void* readAttribute(const char* attr_name, int32& data_type, int32& count);
};

#endif /* HDF_SDS_H */
