/*  Open GENIE, a program for neutron scattering data analysis & display
    ISIS facility, Rutherford Appleton Laboratory, Chilton, Didcot, OXON UK
    Copyright (C) 1996,1997

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA. */

#ifndef HDF_VGROUP
#define HDF_VGROUP

#include "splash.h"
#include "hdf_object.h"

class Vgroup : public HDFObject
{
    private:
	int32 m_vgroup_id;			// currently accessed group
	int* m_vgroup_id_ref_count;			// how many references to this
	String m_access_mode;
	int setup(const char* filename, int32 file_id, int32 vgroup_ref, const char* vgroup_access_mode);
	int cleanup();

    public:
	Vgroup(const Vgroup& vg);
	Vgroup& operator=(const Vgroup& vg);
	Vgroup() : m_vgroup_id(-1), m_vgroup_id_ref_count(new int), m_access_mode("w") { *m_vgroup_id_ref_count = 1; }
	int32 vid() const { return m_vgroup_id; }
	Vgroup(const HDFFile& hdf, int32 vgroup_ref = -1, const char* vgroup_access_mode = "w");
	int addToFile(HDFFile& hdf);
	int setName(const char* vgroup_name);
	int setClass(const char* vgroup_class);
	int insertInVgroup(Vgroup& vg);		// insert ourselves into "vg"
	int insertObject(HDFObject& object);	// insert an object into this vgroup
	int numberOfMembers() const;			// number of objects stored in Vgroup
	HDFObject member(int index) const;		// object at member position "index" (zero based index)
  	isMember(HDFObject& object);		// return 1 if object is in Vgroup
	const char* getName() const;
	const char* getClass() const;
	ostream& print(ostream& os) const;
	const char* accessMode() const { return m_access_mode; }
	~Vgroup();
	static void* loneVgroups(const HDFFile& hdf);		// on heap, really VgroupList*
	static void* allVgroups(HDFFile& hdf);		// on heap, really VgroupList*
};

typedef SPList<Vgroup> VgroupList;
inline int operator<(const Vgroup& a, const Vgroup& b)
{
    return a.vid() < b.vid();
}

inline ostream& operator<<(ostream& os, const Vgroup& vg)
{
    return vg.print(os);
}

#endif /* HDF_VGROUP */
