/*  Open GENIE, a program for neutron scattering data analysis & display
    ISIS facility, Rutherford Appleton Laboratory, Chilton, Didcot, OXON UK
    Copyright (C) 1996,1997

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA. */

#include "geniedefs.h"
//+++
//
// CLASS: Histogram
//
//
//---
// ----
// define String data type
//
#include <gnustring.h>
#include "Drawable_objects.h"
#include "Plot.h"
#include "Histogram.h"
#include "Pginclude.h"
#include "Errors.h"
#include "SinglePrecision.h"
 
static const char* rcsid = "@(#)$Id: histogram.cxx,v 1.7 1997/04/15 14:41:28 cmm Exp $";
USE_RCSID(rcsid);

Histogram::Histogram(int size,
		   double* xdata,
		   double* ydata,
		   int bin,
		   int colour,
		   int thickness,
		   int type) {

	    // *** set visibility
	    invisible = 0;

	    // *** get size (check for zero)
	    if (size <= 0) {
		        Errors syserr;
			syserr.add_error ("Histogram ",
			    "too few data points ",
			    "make number_of_points greater than 0");
			return;
			}
	    number_of_points = size;

	    // *** allocate X array
	    X_array = new float[size];
	    if (X_array == 0) {
			    Errors syserr;
			    syserr.add_error ("Histogram ",
			    "insufficient virtual memory for X array ",
			    "call procedure DELETE_PICTURES");
			return;
			}
	    else
		    // copy array
		    DoubleArrayToFloatArray(X_array,xdata,size);

	    // *** allocate Y array
	    Y_array = new float[size];
	    if (Y_array == 0) {
			    Errors syserr;
			    syserr.add_error ("Histogram ",
			    "insufficient virtual memory for Y array",
			    "call procedure DELETE_PICTURES");
			return;
			}
	    else
		    // copy array
		    DoubleArrayToFloatArray(Y_array,ydata,size);

	    // *** fill in bin centre
	    if (bin == 2)
		    bin_centre = BIN_CENTRE;
	    else
		    bin_centre = bin;

	    // *** fill in symbol
	    if (colour == -1)
		    line_colour = LINE_COLOUR;
	    else
		    line_colour = colour;

	    // *** fill in thickness
	    if (thickness == 0)
		    line_thickness = LINE_THICKNESS;
	    else
		    line_thickness = thickness;

	    // *** fill in linetype
	    if (type == 0)
		    line_type = LINE_TYPE;
	    else
		    line_type = type;

	   }

Histogram::~Histogram() {

	    // *** delete the NEWed arrays
	    delete [] X_array;
	    delete [] Y_array;

	    }

void Histogram::display() {

	    // *** set the colour
	    pgsci(line_colour);

	    // *** set the line thickness
	    pgslw(line_thickness);

	    // *** set the line type
	    pgsls(line_type);

	    // *** plot the line
	    pgbin(number_of_points,X_array,Y_array,bin_centre);

	    }
