/*  Open GENIE, a program for neutron scattering data analysis & display
    ISIS facility, Rutherford Appleton Laboratory, Chilton, Didcot, OXON UK
    Copyright (C) 1996,1997

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA. */

#ifndef INTERMEDIATE_FILE
#define INTERMEDIATE_FILE

#if HAVE_RPC_RPC_H && !defined(__ultrix)
#	include <rpc/rpc.h>
#elif defined(_WIN32)
#	include <winsock.h>
#else
#	include "../xdr/xdr.h"
#endif /* HAVE_RPC_RPC_H */


#include <stdio.h>
#include "g3io_types.h"

class IntermediateFile
{
public:
    enum Look  { NotKnown = -1 };
    enum Offset { Current, Start, End };
    enum ModeFlags { Input=01, Output=02, Append=04 };	// Output creates new file

protected:
    g3io_int_t m_block;			// current position in file,   -1 = null file, or file not opened
					//				0 = after header, but before 1st block etc
					//				1 = after 1st, but before 2nd, block etc
    g3io_int_t m_bytes;			// size of file when first opened (used as check for corrupted file)
    g3io_int_t m_total_blocks;		// total number of blocks in file (0 = empty file)
					//      m_block < m_total_blocks for normal reading
    g3io_int_t m_major_version;		// version of the opened file (may be != g3io_major, which is what we would write)
    g3io_int_t m_minor_version;
    String m_file;			// name of file
    ModeFlags m_mode;

public:
    virtual ~IntermediateFile();
    IntermediateFile() : m_block(-1), m_bytes(0) {}

    virtual int open(const char* file_name, ModeFlags mode = Append) = 0;
    virtual int close() = 0;
    virtual int numberOfBlocks() const = 0;
    virtual ostream& print(ostream& os) const = 0;
    virtual int appendBlock(GenieVariable& gv, const char* label = "<no label>", const char* comment = "<no comment>") = 0;
    virtual int appendBlock(GenieVariableBase& gvb, const char* label = "<no label>", const char* comment = "<no comment>") = 0;
    virtual int skipInteger() = 0;
    virtual int readInteger(g3io_int_t& i) = 0;
    virtual int writeInteger(g3io_int_t i) = 0;
    virtual int skipString() = 0;
    virtual int freeString(char*& s) = 0;
    virtual int readString(char*& s) = 0;
    virtual int writeString(const char* s) = 0;
    virtual int skipReal() = 0;
    virtual int readReal(g3io_real_t& r) = 0;
    virtual int writeReal(g3io_real_t r) = 0;
    virtual int readRealArray(GenieRealArray& gra) = 0;
    virtual int writeRealArray(GenieRealArray& gra) = 0;
    virtual int skipRealArray() = 0;
    virtual int skipIntegerArray() = 0;
    virtual int readIntegerArray(GenieIntegerArray& ia) = 0;
    virtual int writeIntegerArray(GenieIntegerArray& ia) = 0;
    virtual int readStringArray(GenieStringArray& gsa) = 0;
    virtual int writeStringArray(GenieStringArray& gsa) = 0;
    virtual int skipStringArray() = 0;
    virtual int readItemSubclassType(char*& subclass_name) = 0;
    virtual int writeItemSubclassType(const char* subclass_name) = 0;
    virtual int skipItemSubclassType() = 0;
    virtual GenieVariableType readItemType() = 0;
    virtual int skipItemType() = 0;
    virtual int writeItemType(GenieVariableType gvt) = 0;
    virtual int skipOffset() = 0;
    virtual int writeOffset(u_int saved_offset) = 0;	// write the current offset to a previosly allocated offset field
    virtual int saveOffset(u_int& saved_offset) = 0;	// allocate space for an offset field and make a not of its position
    virtual int readOffset(u_int& offset) = 0;
    virtual u_int setFileOffset(u_int offset) = 0;

    virtual int readArrayDims(GenieArrayBase& gab) = 0;
    virtual int writeArrayDims(const GenieArrayBase& gab) = 0;
    static String creator();		// info about person writing file
    static String dateAndTime();	// date and time
#if 0
    virtual int readLabel(char*& label) = 0;
    virtual int skipLabel() = 0;
    virtual int writeLabel(const char* label) = 0;
    virtual int seek(int block = -1) = 0; 			// -1 means "end of file", i.e. get ready for an "append"
						// return 1 on success, 0 on error
    virtual int seek(const char* label) = 0;		// returns block number, -1 on error	
    virtual int appendBlock(GenieVariableBase& gvb, const char* label = "<no label>", const char* comment = "<no comment>") = 0;
    virtual int appendBlock(GenieVariable& gv, const char* label = "<no label>", const char* comment = "<no comment>") = 0;
    virtual int writeBlock(GenieVariableBase& gvb, const char* label = "<no label>", const char* comment = "<no comment>") = 0;
    virtual int writeBlock(GenieVariable& gv, const char* label = "<no label>", const char* comment = "<no comment>") = 0;
    virtual int readBlock(char*& label, GenieVariable& gv) = 0;
    virtual int readBlock(GenieVariable& gv) = 0;
    virtual u_int getFileOffset() = 0;
    virtual u_int setFileOffset(u_int offset) = 0;
    virtual int saveOffset(u_int& saved_offset) = 0;	// allocate space for an offset field and make a not of its position
    virtual int writeOffset(u_int saved_offset) = 0;	// write the current offset to a previosly allocated offset field
    virtual int readOffset(u_int& offset) = 0;
    virtual GenieVariableType skipBlock() = 0;
    virtual int readItem(GenieVariable& gv, char*& comment, char*& user, char*& date) = 0;
    virtual int readItem(GenieVariable& gv) = 0;
    virtual int writeItem(GenieVariableBase& gvb, const char* comment = "<no comment>") = 0;
    virtual int writeItem(GenieVariable& gv, const char* comment = "<no comment>") = 0;
    virtual int readInfo(char*& comment, char*& user, char*& date) = 0;
    virtual int readHeader(bool restore_position) = 0;
    virtual int writeHeader(bool restore_position) = 0;
    virtual int skipInfo() = 0;
    virtual int writeInfo(const char* comment = "<no comment>") = 0;
//    virtual bool validFile() = 0;
    virtual int block() { return m_block; }
#endif
};

#endif /* INTERMEDIATE_FILE */

