/*  Open GENIE, a program for neutron scattering data analysis & display
    ISIS facility, Rutherford Appleton Laboratory, Chilton, Didcot, OXON UK
    Copyright (C) 1996,1997

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA. */

#ifndef NAMED_UNIT_H
#define NAMED_UNIT_H

class NamedUnit
{

};

class GenieWorkspace;

class SIDimensionalExponents
{
    public:
    	double length;
    	double mass;
    	double time;
    	double current;			// electric current
    	double temperature;		// thermodynamic temperature
    	double amount;			// amount of substance
    	double intensity;		// luminous intensity

	SIDimensionalExponents() : 
	 	length(0.0), mass(0.0), time(0.0), current(0.0), temperature(0.0), amount(0.0), intensity(0.0) {} 
    	SIDimensionalExponents(double l, double m, double tm, double c, double tp, double a, double i) :
		length(l), mass(m), time(tm), current(c), temperature(tp), amount(a), intensity(i) {}
	bool operator==(const SIDimensionalExponents& d) const;
	bool operator!=(const SIDimensionalExponents& d) const { return !(*this == d); }
	void operator+=(const SIDimensionalExponents& d);
	friend SIDimensionalExponents operator*(const SIDimensionalExponents& d, double f);
	friend SIDimensionalExponents operator+(const SIDimensionalExponents& d1, const SIDimensionalExponents& d2);
	friend SIDimensionalExponents operator-(const SIDimensionalExponents& d1, const SIDimensionalExponents& d2);
	ostream& print(ostream& os) const;
	String asString() const;
	operator GenieWorkspace() const;
};

inline ostream& operator<<(ostream& os, const SIDimensionalExponents& d)
{
    return d.print(os);
}

#endif /* NAMED_UNIT_H */
