/*  Open GENIE, a program for neutron scattering data analysis & display
    ISIS facility, Rutherford Appleton Laboratory, Chilton, Didcot, OXON UK
    Copyright (C) 1996,1997

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA. */

#include <stdio.h>
#include <string.h>
#include <ctype.h>

#include "geniedefs.h"
#include "parse_funcs.h"


/*
 * Set of general string routines
 * ==============================
 *
 * These were originally written for the parser but
 * are naturally appropriate for use in the main process
 * as this is subject to Garbling/Ungarbling needs as well
 *
 * These could do with being put into C++ really as they are
 * messy !
 */

static const char* rcsid = "@(#)$Id: parse_funcs.cxx,v 1.2 1997/05/06 09:57:13 cmm Exp $";
USE_RCSID(rcsid);

/*
 * Lowercases all characters in the string
 */
void Lowercase( char Astring[] ){

    int i;

    for( i=0; Astring[i] ; i++ ){
	 Astring[i] = tolower(Astring[i]);
    }

    return;
}


/*
 * Uppercases all characters in the string
 */
void Uppercase( char Astring[] ){

    int i;

    for( i=0; Astring[i] ; i++ ){
	 Astring[i] = toupper(Astring[i]);
    }

    return;
}


/*
 * Uppercases the first character. Note that to capitalize unconditionally
 * a Lowercase() should be used first.
 */
void Capitalize( char Astring[] ){
    Astring[0]=toupper(Astring[0]);

    return;
}



/* A bit parser specific, but could be used, for example to encode/decode
 * in connection with error messages from smalltalk.
 */

void Garble( char Astring[] ){

    int i;
    char Temp[SHORTCHARBUFF+3];		// Max possible length - see g_extensions

    for( i=0; Astring[i] ; i++ ){
	if ( Astring[i] == '_' )
	    Temp[i] = 'U';
	else if ( Astring[i] == '$' )
	    Temp[i] = 'D';
	else
	    Temp[i] = Astring[i];
    }

    Temp[i] = '\0';
    sprintf(Astring, "gX%s",Temp);
}


void UnGarble( char Astring[] ){

    int i;
    char Temp[SHORTCHARBUFF+3];		// Max possible length - see g_extensions

    for( i=2; Astring[i] ; i++ ){
	if ( Astring[i] == 'U' )
	    Temp[i-2] = '_';
	else if ( Astring[i] == 'D' )
	    Temp[i-2] = '$';
	else
	    Temp[i-2] = Astring[i];
    }

    Temp[i-2] = '\0';
    sprintf(Astring, "%s",Temp);
}

/*
 * Some useful name manglers.  Note that although clashes are rare in
 * the mangling system, procedure names beginning with _ or $ could
 * conceivably clash with similar type names beginning with D or U.
 */

/* Mangle variables names eg My_var --> gXmyUvar */

 void MangleVar( char *buff ) {
	Lowercase(buff);
	Garble(buff);
	return;
}


/* Mangle procedure & type names eg My$proc --> GXmyDproc */

void MangleProc ( char *buff ) {
	MangleVar(buff);
	Capitalize(buff);
	return;
}


/* Mangle type names eg My$type --> GXMyDtype */

void MangleType ( char *buff ) {
	MangleProc(buff);
	buff[2] = toupper(buff[2]);
        return;
}

void DeMangle( char *buff ) {
    char mybuff[SHORTCHARBUFF+3];		// Max length - see g_extensions
    strcpy(mybuff, buff);
    UnGarble(buff);
    if ( mybuff[0] == 'g' )
        Lowercase(buff);
    else if ( mybuff[0] == 'G' && islower(mybuff[2]))
        Uppercase(buff);
    else
        Capitalize(buff);
    return;
}


int Strcasecmp( const char *s1, const char *s2 ) {

    char temp1[SHORTCHARBUFF], temp2[SHORTCHARBUFF];

    Lowercase(strcpy(temp1, s1));
    Lowercase(strcpy(temp2, s2));

    return strcmp(temp1, temp2);
}
