/*  Open GENIE, a program for neutron scattering data analysis & display
    ISIS facility, Rutherford Appleton Laboratory, Chilton, Didcot, OXON UK
    Copyright (C) 1996,1997

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA. */

#ifndef __PARSEDEFS__
#define __PARSEDEFS__
#ifdef __cplusplus
#define DEFINIT reallyquit = 1      /* Only declare initializer in C++ */
extern "C" {
#endif


/* MACROS */

/* Common parser maximums */
#define MAXSTRLEN SHORTCHARBUFF*8	/* Maximum length of any concatenated program string */
#define MAXVARLEN SHORTCHARBUFF	    /* Maximum length of a single variable or identifier string */
#define MAXNEST 100		            /* Generally limits the depth of textual nesting (eg loops within loops) */
#define LEXTOKENBUFFERSIZ 512	    /* A bit arbitary, but by default it is set at around 200 */

enum    compile_mode { BUFFER=3, STARTUP = 2, INTERACTIVE=1, BATCH=0 };

/* parse tree structure */

typedef struct snode_struct *snode;

typedef struct node_struct
{
    long valid;
    snode next;
} node;

struct snode_struct
{
    node    *side;            /* Link to next node in chain */
    char    *leaf;
};

static const node NULL_NODE = {0, NULL};
#define Valid(node)  ((node)?(node)->valid==1:0)

/* FUNCTION PROTOTYPES */

#define yyparse ggparse
#define yylex gglex
#define yyerror ggerror		/* Avoid clash with POSIX lex declaration of yyerror! */
#define yylval gglval
#define yychar ggchar
#define yydebug ggdebug
#define yynerrs ggnerrs

void yyerror(const char err[]);
int  yyparse(void);
int  yylex(void);
int  getc_parser(FILE *, int mode);     /* A look alike for getc but which gets from a readline buffer */
int  prompt_mode(void);                 /* check whether prompting should be done */
void Kill_smalltalk(int DEFINIT);       /* To kill the process externally */
#ifdef __cplusplus
int  load_file( const char symbol[], const char filename[], int mode, const time_t date, int warn = 1 );
#endif


/* Functions to Initialize static parser variables before the next parse cycle and to retrieve results respectively */
void  initialize_parser(int mode, FILE *channel, int ydebflag);
node  *interrogate_parser(int *errcnt, int *linecnt);
void  fill_buffers(void);
int   check_buffers(void);
void  initialize_parser_stats(void);
void  printNode(char * buffer, node *node);
void  deleteNode(node *node);
void  nprintf(node **result, const char* fmt, ...);
void  p_stats(void);
int   nlength(node *node);
node  *newLeaf(const char *value);
void  sweep_nodes(void);

int evalGCLCommand(const char* command);	/* execute a single GCL command in the parser */
int evalSmalltalkCommand(const char* comamnd);	/* execute a piece of Smalltalk */

extern FILE* g_type_file;

/* START variables shared by lex & yacc only */

#ifdef __INSIDE_PARSER__
FILE *InChan;				/* Channel from which lex gets its input */

/* Parser record keeping */
int errors_found;			/* Up to date count of errors found */
int lines_parsed;			/* Count of lines successfully parsed (only incremented at the end of a line) */
int continuation_line;                  /* Set when we are in a continuation line */
int auto_declare;                       /* When this is set, declare an undeclared variable on assignment */

int Interactive;			/* Tells whether the parser is in batch or interactive mode */
#endif

/* END variables shared by lex & yacc only */

#ifdef __cplusplus
}
#endif
#endif /* __PARSEDEFS__ */
