/*  Open GENIE, a program for neutron scattering data analysis & display
    ISIS facility, Rutherford Appleton Laboratory, Chilton, Didcot, OXON UK
    Copyright (C) 1996,1997

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA. */

#include "geniedefs.h"
#include <math.h>
#include "Pginclude.h"
#include "g_types.h"
#include "pg_coord_trans_func.h"

static const char* rcsid = "@(#)$Id: pg_coord_trans_func.cxx,v 1.8 1997/05/14 10:35:15 faa Exp $";
USE_RCSID(rcsid);

PGCoordinateTransformFunction* PGCoordinateTransformFunction::class_ptr = 0;

member_ptr_t PGCoordinateTransformFunction::member_ptr = &PGCoordinateTransformFunction::forwardFunction;

// For drawing contours
void PGCoordinateTransformFunction::contourFunction(pg_int& visible, pg_float& x, pg_float& y, pg_float& z)
{
    int flags[] = { visible };
    g_real val_in[] = { x, y, z }, val_out[2];
    pg_int visible_out;
    pg_float xout, yout;
    if (class_ptr != 0)
    {
	(class_ptr->*member_ptr)(flags, val_in, val_out);
	visible_out = flags[0];
	xout = (float)val_out[0];
	yout = (float)val_out[1];
	if (z == UNDEFINED_PG_FLOAT)
	{
	    visible_out = 0;
	}
	drawIt(visible_out, xout, yout);
    }
}

// For labelling contours
void PGCoordinateTransformFunction::contourLabelFunction(pg_int* flags, pg_float& x_in, pg_float& y_in, pg_float& x_out, 
							 pg_float& y_out)
{
// FORTRAN arrays start at 1, C arrays at 0
    g_real val_in[3] = { x_in, y_in, 0.0 }, val_out[2];
    if (class_ptr != 0)
    {
        (class_ptr->*member_ptr)(flags, val_in, val_out);
        x_out = (float)val_out[0];
        y_out = (float)val_out[1];
    }
}

void PGCoordinateTransformFunction::drawIt(pg_int& visible, pg_float& x, pg_float& y)
{
    if (visible == 0)
    {
	pgmove(x, y);
    }
    else if (visible == 1)
    {
	pgdraw(x, y);
    }
}
    
PGCoordinateTransformFunction::PGCoordinateTransformFunction(coord_tran_func_t forward_func, coord_tran_func_t reverse_func) :
	TwoDimensionalCoordinateTransformFunction(forward_func, reverse_func)
{
}

PGCoordinateTransformFunction::PGCoordinateTransformFunction(g_real* matrix) :
	TwoDimensionalCoordinateTransformFunction(cMatrix(matrix))
{
}

PGCoordinateTransformFunction::PGCoordinateTransformFunction(g_real* wx, int* wx_dims, int wx_ndims, CoordinateTransformMode x_mode,
					g_real* wy, int* wy_dims, int wy_ndims, CoordinateTransformMode y_mode)
 : TwoDimensionalCoordinateTransformFunction(wx, wx_dims, wx_ndims, x_mode, wy, wy_dims, wy_ndims, y_mode)
{
}

// Set static pointers
void PGCoordinateTransformFunction::setPointers(bool forward)
{
    if (forward)
    {
	member_ptr = &PGCoordinateTransformFunction::forwardFunction;
    }
    else
    {
	member_ptr = &PGCoordinateTransformFunction::reverseFunction;
    }
    class_ptr = this;
}

int PGCoordinateTransformFunction::forwardFunction(int* flags, const g_real* val_in, g_real* val_out)
{
// FORTRAN arrays start at 1, C arrays at 0
    g_real val_in_[] = { val_in[0] - 1.0, val_in[1] - 1.0 };
    return TwoDimensionalCoordinateTransformFunction::forwardFunction(flags, val_in_, val_out);
}

int PGCoordinateTransformFunction::reverseFunction(int* flags, const g_real* val_in, g_real* val_out)
{
// FORTRAN arrays start at 1, C arrays at 0
    TwoDimensionalCoordinateTransformFunction::reverseFunction(flags, val_in, val_out);
    val_out[0] += 1.0;
    val_out[1] += 1.0;
    return 0;
}

// Construct a C coefficient matrix from a pgplot Fortran one
g_real* PGCoordinateTransformFunction::cMatrix(g_real* matrix)
{
    g_real* temp_matrix = new g_real[6];
    for(int i=0; i<6; i++)
	temp_matrix[i] = matrix[i];
// Make i=0,j=0 give same result as i=1,j=1 i.e. Fortran -> C
    temp_matrix[0] += (matrix[1] + matrix[2]);
    temp_matrix[3] += (matrix[4] + matrix[5]);
    return temp_matrix;
}
