/*  Open GENIE, a program for neutron scattering data analysis & display
    ISIS facility, Rutherford Appleton Laboratory, Chilton, Didcot, OXON UK
    Copyright (C) 1996,1997

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA. */

#include "geniedefs.h"
//+++
//
// CLASS: Polyline
//
//
//---
// ----
// define String data type
//
#include <gnustring.h>
#include "Drawable_objects.h"
#include "Plot.h"
#include "Polyline.h"
#include "Pginclude.h"
#include "Errors.h"
#include "SinglePrecision.h"
 
static const char* rcsid = "@(#)$Id: polyline.cxx,v 1.9 1997/04/18 09:08:51 cmm Exp $";
USE_RCSID(rcsid);

Polyline::Polyline(int size,
		   double* xdata,
		   double* ydata,
		   int line,
		   int colour,
		   int thickness) : Plot()
{
//	    int c;

	    // *** set visibility
	    invisible = 0;

	    // *** get size (check for zero)
	    if (size <= 0) {
		Errors syserr;
		syserr.add_error ("Polyline plot ",
			    "Illegal number of data points ",
			    "make number_of_points greater than 0");
		return;
		}
	    number_of_points = size;

	    // *** allocate X array
	    X_array = new float[size];
	    if (X_array == 0) {
		Errors syserr;
		syserr.add_error ("Polyline ",
		    "Insufficient virtual memory for X array ",
		    "call procedure DELETE_PICTURES");
		 return;
			}
	    else
		    DoubleArrayToFloatArray(X_array, xdata, size);

	    // *** allocate Y array
	    Y_array = new float[size];
	    if (Y_array == 0) {
		Errors syserr;
		syserr.add_error ("Polyline ",
		    "Insufficient virtual memory for Y array ",
		    "call procedure DELETE_PICTURES");
		return;
			}
	    else
		    DoubleArrayToFloatArray(Y_array, ydata, size);

	    // *** fill in line colour
	    if (colour == -1)
		    line_colour = LINE_COLOUR;
	    else
		    line_colour = colour;

	    // *** fill in line thickness;
	    if (thickness == 0)
		    line_thickness = LINE_THICKNESS;
	    else
		    line_thickness = thickness;

	    // *** fill in line type
	    if (line == 0)
		    linetype = LINE_TYPE;
	    else
		    linetype = line;

	    }

Polyline::~Polyline() {

	    // *** delete the NEWed arrays
	    delete [] X_array;
	    delete [] Y_array;

	    }

void Polyline::display() {

	    // *** set the colour
	    pgsci(line_colour);

	    // *** set the line thickness
	    pgslw(line_thickness);

	    // *** set the line style
	    pgsls(linetype);

	    // *** plot the line
	    if (number_of_points > 0)
	        pgline(number_of_points, X_array, Y_array);

	    }
