/*  Open GENIE, a program for neutron scattering data analysis & display
    ISIS facility, Rutherford Appleton Laboratory, Chilton, Didcot, OXON UK
    Copyright (C) 1996,1997

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA. */

/*
 *      Module:         ranges
 *      Author:         Freddie Akeroyd, ISIS
 *      Purpose:        Functions to manipulate strings representing "ranges of numbers"
 *
 */
#include "geniedefs.h"
#include <gnustring.h>
#include "ranges.h"

static const char* rcsid = "@(#)$Id: ranges.cxx,v 1.6 1997/03/26 16:26:41 cmm Exp $";
USE_RCSID(rcsid);

/*
 * Split a string of the form "1,3,6-9,11" int a list of "Range" structures
 * A pointer to the generated list is returned, which must be freed by the caller after use
 */
RangeList* parse_range(const char* a_range)
{
    static const int MAXRANGES = 10;
    String ranges[MAXRANGES];
    String range(a_range), comma(",");
    RangeStruct item;
    RangeList* range_list = new RangeList;
    int numranges = split(range, ranges, MAXRANGES, comma);  	// individual ranges separated by ","
    for(int i=0; i<numranges; i++)
    {
	if (ranges[i].contains('-'))
	{
	    item.start = atoi(String(ranges[i].before('-')));    
	    item.end = atoi(String(ranges[i].after('-')));    
	}
	else
	{
	    item.start = item.end = atoi(ranges[i]);
  	}
	range_list->push(item);
    }
    return range_list;
}

int sum_range(RangeList* range_list)
{
    int sum(0);
    for(int i=0; i<range_list->count(); i++)
    {
	sum += ( (*range_list)[i].end - (*range_list)[i].start + 1 );
    }
    return sum;
}

int operator<(RangeStruct a, RangeStruct b)
{
    return a.start <= b.start && a.end < b.end;
}

int* RangeIterator::next()
{
    return &m_range_value;
}

#ifdef __GNUG__
template class SPList<RangeStruct>;	
template class SPListBase<RangeStruct>;
#endif /* __GNUG__ */

#ifdef __DECCXX
#pragma define_template SPList<RangeStruct>
#pragma define_template SPListBase<RangeStruct>
#endif /* __DECCXX */
