/*  Open GENIE, a program for neutron scattering data analysis & display
    ISIS facility, Rutherford Appleton Laboratory, Chilton, Didcot, OXON UK
    Copyright (C) 1996,1997

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA. */

/* 
 * A range is a string of the form "1,3,5-8,10"
 * It is converted into a RangeList by parse_range()
 */
#ifndef RANGES_H
#define RANGES_H

#include "splash.h"

struct RangeStruct
{
    int start;
    int end;
};

extern int operator<(RangeStruct a, RangeStruct b);

typedef SPList<RangeStruct> RangeList;

extern RangeList* parse_range(const char* a_range);
extern int sum_range(RangeList* range_list);

class NumberRange
{
    private:
    	RangeList* m_list;
	friend class RangeIterator;

    public:
	NumberRange(const char* a_range) { m_list = parse_range(a_range); }
	int sum() { return sum_range(m_list); }
	~NumberRange() { delete m_list; }
	int size() { return m_list->count(); }
	RangeStruct& operator[](int i) { return (*m_list)[i]; }
};

class RangeIterator
{
    private:
	NumberRange* m_range;
	int m_list_pos;			// position in RangeStruct list (-1 = haven't started yet)
	int m_range_pos;		// position between "start" and "end" in RangeStruct

    public:
	int m_range_value;			// current value for next()
        RangeIterator(NumberRange& range) : m_range(&range), m_list_pos(-1), m_range_pos(0), m_range_value(0) { }
	int* next();
};
#endif /* RANGES_H */
