/*  Open GENIE, a program for neutron scattering data analysis & display
    ISIS facility, Rutherford Appleton Laboratory, Chilton, Didcot, OXON UK
    Copyright (C) 1996,1997

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA. */

#include <stdlib.h>
#include "geniedefs.h"
#include "smalltalk.h"
#include "smalltalk_object.h"

OOP saveOOP(OOP oop);

inline OOP GETCLASS(OOP oop)
{
    return isInt(oop) ? integerClass : oopClass(oop);
}

SmalltalkObject::SmalltalkObject(OOP value)
{
    m_value = value;
    m_class = GETCLASS(value);
    setup();
}

SmalltalkObject::SmalltalkObject(const char* type)
{
    static OOP gxnew = registerOOP(internString("new"));
    m_class = saveOOP(typeNameToOOP(type));
    m_value = saveOOP(msgSend(m_class, gxnew, 0));
    setup();
}

SmalltalkObject::SmalltalkObject(OOP parameters, int location)
{
    m_value = ::arrayAt(parameters, location);
    m_class = GETCLASS(m_value);
    setup();
}

int SmalltalkObject::setup()
{
    m_instVarArray = instanceVariableArray(m_class);
    m_numInstVars = numOOPs(oopToObj(m_instVarArray));
    return 0;
}
	
// return instance variable location, 0 on error
int SmalltalkObject::lookupName(const char* name)
{
    int index;
    OOP symbol = internString(name);
        for(index = 1; index <= m_numInstVars; index++) 
        {
    	    if (::arrayAt(m_instVarArray, index) == symbol) 
	    {
                return index;
            }
        }
    return 0;
}

OOP SmalltalkObject::arrayAt(int i)
{
    return ::arrayAt(m_value, i);
}

OOP SmalltalkObject::at(const char* name)
{
    int index = lookupName(name);
    if (index != 0)
    {
	return instVarAt(m_value, index);
    }
    else
    {
	gerr << "SmalltalkObject: no such instance variable as \"" << name << "\"" << endl;
	return nilOOP;
    }
}

int SmalltalkObject::arrayAtPut(int i, OOP oop)
{
    ::arrayAtPut(m_value, i, oop);
    return 0;
}

int SmalltalkObject::atPut(const char* name, OOP oop)
{
    int index = lookupName(name);
    if (index != 0)
    {
	return instVarAtPut(m_value, index, oop);
    }
    else
    {
        gerr << "SmalltalkObject: no such instance variable as \"" << name << "\"" << endl;
        return -1;
    }
}

ostream& SmalltalkObject::print(ostream& os) const
{
    static OOP Inspect = registerOOP(internString("inspect"));
    if (isOOP(m_value))
    {
	printOOPConstructor(m_value);
    }
    else
    {
	os << "Integer" << endl;
    }
    msgSend(m_value, Inspect, 0);
    return os;
}

int SmalltalkObject::fromString(const char* name, const char* value)
{
    Regex int_item("^[0-9]+$"), float_item("\\.");
    String v(value);
    OOP inst_var_class = GETCLASS(at(name));
    if (inst_var_class == undefinedObjectClass)
    {
        if (v.matches(int_item))
	    inst_var_class = integerClass;
	else if (v.contains(float_item))
	    inst_var_class = floatClass;
	else
	    inst_var_class = stringClass;
    }
    if (isAKindOf(inst_var_class, integerClass))
	(*this)(name, fromInt(atol(value)));
    else if (isAKindOf(inst_var_class, floatClass))
	(*this)(name, floatNew(atof(value)));
    else
	(*this)(name, stringNew(value));
    return 0;
}

int SmalltalkObject::arrayLength()
{
    return numIndexableFields(m_value);
}
