/*  Open GENIE, a program for neutron scattering data analysis & display
    ISIS facility, Rutherford Appleton Laboratory, Chilton, Didcot, OXON UK
    Copyright (C) 1996,1997

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA. */

#ifndef _SPECTRUM_H_
#define _SPECTRUM_H_

#define NSPECFIELD	27	/* number of fields in a genie spectrum workspace */
#define NUSEMAX		30	/* maximum number of user parameters to pass out in transform command */

typedef struct
{
    g_real* x;
    g_real* y;			
    g_real* e;			/* errors */
    g_real* ut;			/* user tables */
    g_integer n;		/* number of time channels */
    g_integer nut;		/* number of user tables */
    String tl;			/* title */
    String tm;			/* time */
    String h;			/* history */
    g_real fi; 			/* phi */
    String un;			/* user_name */
    String in;			/* inst_name */
    String rn;			/* run_no */
    String f;			/* file */
    g_real d;			/* delta */
    g_real dn;			/* duration */
    String sn;			/* spectrum number */
    g_integer bg;		/* bin grouping */
    String xl;			/* xlabel */
    String yl;			/* ylabel */
    g_real ym;			/* ymax */
    g_real l1;			/* L1 */
    g_real l2;			/* L2 */
    g_real tt;			/* two theta */
    g_integer em;		/* emode */
    g_real ef;			/* efixed */
} g_spectrum_struct;

typedef struct
{
    fort_real* x;
    fort_real* y;			
    fort_real* e;		/* errors */
    fort_real* ut;			/* user tables */
    fort_int n;			/* number of time channels */
    fort_int nut;		/* number of user tables */
    String tl;			/* title */
    String tm;			/* time */
    String h;			/* history */
    fort_real fi; 		/* phi */
    String un;			/* user_name */
    String in;			/* inst_name */
    String rn;			/* run_no */
    String f;			/* file */
    fort_real d;		/* delta */
    fort_real dn;		/* duration */
    String sn;			/* spectrum number */
    fort_int bg;		/* bin grouping */
    String xl;			/* xlabel */
    String yl;			/* ylabel */
    fort_real ym;		/* ymax */
    fort_real l1;		/* L1 */
    fort_real l2;		/* L2 */
    fort_real tt;		/* two theta */
    fort_int em;		/* emode */
    fort_real ef;		/* efixed */
} f_spectrum_struct;

/* prototypes */

g_spectrum_struct* initGSpectrum(g_integer, g_integer);
f_spectrum_struct* initFSpectrum(fort_int, fort_int);

g_variable createGSpectrum(g_spectrum_struct*);
g_variable createFSpectrum(f_spectrum_struct*);

f_spectrum_struct* extractFSpectrum(g_variable, g_integer, g_integer);
g_spectrum_struct* extractGSpectrum(g_variable, g_integer, g_integer);

void replaceFSpectrum(g_variable*, f_spectrum_struct*);
void replaceGSpectrum(g_variable*, g_spectrum_struct*);

void deleteFSpectrum(f_spectrum_struct*);
void deleteGSpectrum(g_spectrum_struct*);

g_spectrum_struct* readGSS(int fd);
int updateSSHeader(int fd, int increment);
unsigned long writeGSS(int fd, g_spectrum_struct* gss);
int seekSS(int fd, int block);

#endif	/* _SPECTRUM_H_ */
