/*  Open GENIE, a program for neutron scattering data analysis & display
    ISIS facility, Rutherford Appleton Laboratory, Chilton, Didcot, OXON UK
    Copyright (C) 1996,1997

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA. */

#ifndef __STMACROS__
#define __STMACROS__

/* Macro definitions to simplify code generation and keep the parser itself tidyer
   (These must NOT contain $$ or $n parameters)

These Macros are all printing statements which provide a 'C' builtin version of the
longer bits of smalltalk code specifically to run GENIE. They are used where Multi-line statements
might make the parser rather confusing !

*/


/*====================================================================================================*/
#define PROCEDURE_BODY(tree, code, name, result) nprintf(tree, "\
\"BODY\n\
 ----\"\n\
\n\
!%s methodsFor: 'running'!\n\
\n\
evaluate\n\
%n\
%s\n\
!!\n\
nil! \"(fix for smalltalk bug, this line is ignored if there is a variable declaration error)\"\n\
\n\
", name, code, result);

/*====================================================================================================*/
#define PROCEDURE_CLASS(tree, name) nprintf(tree, "\
((Smalltalk at: #%s ifAbsent: [ nil ]) isKindOf: Function class) ifTrue:\
 [ ('Warning: Overriding previously defined procedure ', '%s' gtype) prinl ].\n\
Smalltalk at: #%s put: nil!\n\
Function subclass: '%s'\n\
       instanceVariableNames: ''\n\
       classVariableNames: ''\n\
       poolDictionaries: ''\n\
       category: 'Genie Procedure'\n\
!\n\
\n\
", name, name, name, name);

/*====================================================================================================*/
/* For a declared structural variable, this will work OK, for a nil or undeclared we will get a nil return */
#define ASSIGN_STRUCT_I(tree, destvar, destaccess, source, deepCopy) nprintf(tree, "\
Smalltalk at: #%s ifAbsent: [ Smalltalk at: #%s put: nil ]!\n\
(%s isKindOf: GXArray) | (%s isKindOf: GXWorkspace)\n\
    ifTrue:  [ (%n put: %n %s) ]\n\
    ifFalse: [ self error: 'Assignment to array or workspace failed, check that the destination is a structured type']\
", destvar, destvar, destvar, destvar, destaccess, source, deepCopy)

/*====================================================================================================*/
/* Outside a procedure, create a new variable, inside, just assign and assume it is declared */
#define ASSIGN_ATOMIC_I(tree, dest, source, deepCopy) nprintf(tree, "\
Smalltalk at: #%n ifAbsent: [ Smalltalk at: #%n put: nil ]!\n\
%n _ %n %s\
", copyNode(dest), copyNode(dest), dest, source, deepCopy)

/*====================================================================================================*/
/* For a declared structural variable, this will work OK, for a nil or undeclared we will get a nil return */
#define ASSIGN_STRUCT(tree, destvar,destaccess, source, deepCopy) nprintf(tree, "\
(%s isKindOf: GXArray) | (%s isKindOf: GXWorkspace)\n\
    ifTrue:  [ (%n put: %n %s) ]\n\
    ifFalse: [ self error: 'Assignment to array or workspace failed, check that the destination is a structured type']\
", destvar, destvar, destaccess, source, deepCopy)

/*====================================================================================================*/
/* Outside a procedure, create a new variable, inside, just assign and assume it is declared */
#define ASSIGN_ATOMIC(tree, dest, source, deepCopy) nprintf(tree, "\
%n _ %n %s\
", dest, source, deepCopy)

#endif /* __STMACROS__ */
