/*  Open GENIE, a program for neutron scattering data analysis & display
    ISIS facility, Rutherford Appleton Laboratory, Chilton, Didcot, OXON UK
    Copyright (C) 1996,1997

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA. */

#ifndef TRANSFORM_UTILS_H
#define TRANSFORM_UTILS_H

template <class Type>
inline Type max(Type a, Type b)
{
    return(a > b ? a : b);
}

template <class Type>
inline Type min(Type a, Type b)
{
    return(a < b ? a : b);
}

extern char* crunid(g_integer id);

template <class __g_real, class __g_integer>
extern int newAxis(__g_real xbounds[], __g_integer ibounds, __g_real xsteps[], 
              __g_integer isteps, __g_real xnew[], int assign,
	      __g_integer itype);

extern int newAxis(float[], int, float[], int, float[], int, int);
extern int newAxis(double[], long, double[], long, double[], int, g_integer);

template <class __g_real, class __g_integer>
extern int rebin(__g_real xold[], __g_real yold[], __g_real eold[], __g_integer ntcold, 
	  __g_real xnew[], __g_real ynew[], __g_real enew[], __g_integer ntcnew, __g_real& ymax);

extern int rebin(float[], float[], float[], int, float[], float[], float[], int, float&);
extern int rebin(double[], double[], double[], long, double[], double[], double[], long, double&);

template <class __g_real, class __g_integer>

extern int bracket(__g_real xval, const __g_real xarray[], __g_integer np);
extern int bracket(float, const float[], int);
extern int bracket(double, const double[], long);

template <class __g_real, class __g_integer>
extern int rebinRange(__g_real xold[], __g_real yold[], __g_real eold[], 
	       __g_real xnew[], __g_real ynew[], __g_real enew[],
		__g_integer& ntc, __g_real xmin, __g_real xmax, __g_real& ymax);

extern int rebinRange(double[], double[], double[], double[], double[], double[], long&, double, double, double&);
extern int rebinRange(float[], float[], float[], float[], float[], float[], int&, float, float, float&);

template <class __g_real_old, class __g_real_new, class __g_integer>
extern int group_bins(__g_real_old xarray[], __g_real_old yarray[], __g_real_old earray[],
	       __g_integer ntc, __g_integer nspec, __g_real_new xnew[], __g_real_new ynew[],
	       __g_real_new enew[], __g_integer nbin);

extern int group_bins(double[], double[], double[], long, long, float[], float[], float[], long);
extern int group_bins(double[], double[], double[], long, long, double[], double[], double[], long);
extern int group_bins(float[], float[], float[], int, int, float[], float[], float[], int);

template <class __g_real, class __g_integer>
extern int integrateSpectrum(const __g_real xarray[], const __g_real yarray[], const __g_real earray[],
			__g_integer ntc, __g_real xmin, __g_real xmax, 
			__g_real& sum, __g_real& error);

template <class __g_real, class __g_integer>
extern int integrateSpectra(__g_real xarray[], __g_real yarray[], __g_real earray[],
			__g_integer ntc, __g_integer nspec, __g_real xmin[], 
			__g_integer nmin, __g_real xmax[], __g_integer nmax, 
			__g_real sum[], __g_real error[]);

#endif /* TRANSFORM_UTILS_H */
