/*  Open GENIE, a program for neutron scattering data analysis & display
    ISIS facility, Rutherford Appleton Laboratory, Chilton, Didcot, OXON UK
    Copyright (C) 1996,1997

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA. */

#include <iostream.h>
#include <gnustring.h>

#ifndef FALSE
#define FALSE	(0)
#endif

#ifndef TRUE
#define TRUE	(1)
#endif

/*
 * Program to translate a VMS file name into an NFS mounted path
 *
 *  FAA, 12/3/96
 */

static const char* rcsid = "$Id: translate_vms.cxx,v 1.6 1997/03/26 16:26:43 cmm Exp $";

static String nfs_prefix("/vax/");

/*
 * The resulting name will consist of:
 *
 *  (1) the "nfs_prefix"
 *  (2) the VMS disk mapped according to mappings[] array
 *  (3) the vms directory part with [ . ] -> /
 */

static const char* mappings[] = 
{
// Major disks
    "USER$DISK", "user",
    "SCRATCH$DISK", "scratch",
    "VAXLIB$DISK", "lib",
    "MISC$DISK", "misc",
    "INST$DISK", "inst",
// FEM disks
    "CRIPS$DISK0", "crisp0", "CRIPS$DUA0", "crisp0",
    "EVS$DISK0", "evs0", "EVS$DUA0", "evs0",
    "HET$DISK0", "het0", "HET$DUA0", "het0",
    "IRIS$DISK0", "iris0", "IRIS$DUA0", "iris0",
    "LAD$DISK0", "lad0", "LAD$DUA0", "lad0",
    "LOQ$DISK0", "loq0", "LOQ$DUA0", "loq0",
    "MAPS$DISK0", "maps0", "MAPS$DKA0", "maps0",
    "MARI$DISK0", "mari0", "MARI$DUA0", "mari0",
    "MUSR$DISK0", "musr0", "MUSR$DUA0", "musr0",
    "PIG$DISK0", "pig0", "PIG$DUA0", "pig0",
    "POL$DISK0", "pol0", "POL$DUA0", "pol0",
    "PRISMA$DISK0", "prisma0", "PRISMA$DUA0", "prisma0",
    "SLS$DISK0", "sls0", "SLS$DUA0", "sls0",
    "SURF$DISK0", "surf0", "SURF$DKA300", "surf0",
    "SXD$DISK0", "sxd0", "SXD$DUA0", "sxd0",
    "TEB$DISK0", "teb0", "TEB$DUA0:", "teb0",
    "TFXA$DISK0", "tfxa0", "TFXA$DUA0:", "tfxa0",
// Extra disks
    "CRIPS$DISK1", "crisp1",
    "EVS$DISK1", "evs1",
    "HET$DISK1", "het1",
    "IRIS$DISK1", "iris1",
    "LAD$DISK1", "lad1",
    "LOQ$DISK1", "loq1",
    "MAPS$DISK1", "maps1",
    "MARI$DISK1", "mari1",
    "MUSR$DISK1", "musr1",
    "PIG$DISK1", "pig1",
    "POL$DISK1", "pol1",
    "PRISMA$DISK1", "prisma1",
    "SLS$DISK1", "sls1",
    "SURF$DISK1", "surf1",
    "SXD$DISK1", "sxd1",
    "TEB$DISK1", "teb1",
    "TFXA$DISK1", "tfxa1",
// INST disk aliases
    "CRISP$DISK", "inst",
    "EVS$DISK", "inst",
    "HET$DISK", "inst",
    "IRIS$DISK", "inst",
    "LAD$DISK", "inst",
    "LOQ$DISK", "inst",
    "MAPS$DISK", "inst",
    "MARI$DISK", "inst",
    "MUSR$DISK", "inst",
    "PIG$DISK", "inst",
    "POL$DISK", "inst",
    "PRISMA$DISK", "inst",
    "SLS$DISK", "inst",
    "SURF$DISK", "inst",
    "SXD$DISK", "inst",
    "TEB$DISK", "inst",
    "TFXA$DISK", "inst",
// Must double NULL terminate this list
    0, 0
};

int main(int argc, char* argv[])
{
    if (argc != 2)
	return 1;		// must be passed just a VMS directory name

    String path(argv[1]);
    path.upcase();	
    String disk = path.before(':');
    disk.gsub(" ", "");		// we seem to get a stray space at the start
    String directory = path.after(':');
    int found = FALSE;
    int imap=0;
// Map the disk part
    while(mappings[imap] != 0)
    {
	if (disk == mappings[imap])
	{
	    found = TRUE;
	    break;
	}
	imap += 2;
    }
// Now translate the directory part:  characters  "[ ] ."  ->  "/"
    Regex bits("[][.]");
    directory.gsub(bits, "/");

    String nfs_path;
    if (found)
    {
        nfs_path = nfs_prefix + String(mappings[imap+1]) + directory;
    }
    nfs_path.downcase();
    cout << nfs_path << endl;
    return 0;
}
