/*  Open GENIE, a program for neutron scattering data analysis & display
    ISIS facility, Rutherford Appleton Laboratory, Chilton, Didcot, OXON UK
    Copyright (C) 1996,1997

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA. */

#ifndef TWO_D_COORD_TRANS_FUNC_H
#define TWO_D_COORD_TRANS_FUNC_H

#include "coord_trans_func.h"
#include "drawable_values.h"

enum CoordinateTransformMode { PointMode, BinMode };

class TwoDimensionalCoordinateTransformFunction : public CoordinateTransformFunction
{
    protected:
	DrawableValues m_x_data;
	DrawableValues m_y_data;
	CoordinateTransformMode m_x_mode;
	CoordinateTransformMode m_y_mode;

	double reverseInterpolate1(DrawableValues& data, double val);
	double interpolate(DrawableValues& data, double i);
	double interpolate(DrawableValues& data, double i, double j);

    public:
	TwoDimensionalCoordinateTransformFunction(coord_tran_func_t forward_func, coord_tran_func_t reverse_func = 0);
	TwoDimensionalCoordinateTransformFunction(g_real* matrix);	// coefficient matrix
	TwoDimensionalCoordinateTransformFunction(g_real* wx, int* wx_dims, int wx_ndims, CoordinateTransformMode x_mode,
						g_real* wy, int* wy_dims, int wy_ndims, CoordinateTransformMode y_mode);
  	int forwardFunction(int* flags, const g_real* val_in, g_real* val_out);
	int reverseFunction(int* flags, const g_real* val_in, g_real* val_out);
        inline int matrixFunction(const pg_float* matrix, const g_real* val_in, g_real* val_out);
};

// optimised 2D matrixFunction()
inline int TwoDimensionalCoordinateTransformFunction::matrixFunction(const pg_float* matrix, const g_real* val_in, g_real* val_out)
{
    val_out[0] = matrix[0] + matrix[1] * val_in[0] + matrix[2] * val_in[1];
    val_out[1] = matrix[3] + matrix[4] * val_in[0] + matrix[5] * val_in[1];
    return 0;
}

#endif /* TWO_D_COORD_TRANS_FUNC_H */
