/*  Open GENIE, a program for neutron scattering data analysis & display
    ISIS facility, Rutherford Appleton Laboratory, Chilton, Didcot, OXON UK
    Copyright (C) 1996,1997

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA. */

#include "geniedefs.h"
//+++
//
// CLASS: H_err_bars
//
//
//---
// ----
// define String data type
//
#include <gnustring.h>
#include "Drawable_objects.h"
#include "Plot.h"
#include "V_err_bars.h"
#include "Pginclude.h"
#include "Errors.h"
#include "SinglePrecision.h"
 
static const char* rcsid = "@(#)$Id: v_err_bars.cxx,v 1.7 1997/03/26 16:26:44 cmm Exp $";
USE_RCSID(rcsid);

// *** constructor for +/- errors equal
V_err_bars::V_err_bars(int size,
		   double* xdata,
		   double* ydata,
		   double* ydatalo,
		   float end,
		   int colour,
		   int thickness) {

	    int c;
 	    double y_lower, y_upper;

	    // *** set visibility
	    invisible = 0;

	    // *** get size (check for zero)
	    if (size <= 0) {
			Errors syserr;
			syserr.add_error ("Horizontal error bars ",
			    "too few data points ",
			    "make number_of_points greater than 0");
			return;
			}
	    number_of_points = size;

	    // *** allocate X array
	    Y_lower_array = new float[size];
	    Y_upper_array = new float[size];

	    if (Y_upper_array == 0) {
			Errors syserr;
			syserr.add_error ("Horizontal error bars ",
			    "Insufficient virtual memory for X arrays ",
			    "call procedure DELETE_PICTURES");
			return;
			}
	    else
		    for (c = 0; c < size; c++) 
		    {
			y_lower = ydata[c] - ydatalo[c];
			y_upper = ydata[c] + ydatalo[c];
			Y_upper_array[c] = y_upper;
			Y_lower_array[c] = y_lower;
		    }

	    // *** allocate Y array
	    X_array = new float[size];
	    if (X_array == 0) {
			Errors syserr;
			syserr.add_error ("Horizontal error bars ",
			    "Insufficient virtual memory for Y array ",
			    "call procedure DELETE_PICTURES");
			return;
			}
	    else {
		DoubleArrayToFloatArray(X_array,xdata,size);
	    }

	    // *** fill in line colour
	    if (colour == -1)
		    line_colour = LINE_COLOUR;
	    else
		    line_colour = colour;


	    // *** fill in thickness
	    if (thickness == 0)
		    line_thickness = LINE_THICKNESS;
	    else
		    line_thickness = thickness;

	    // *** fill in end_size
	    if (end == 0.0)
		    end_size = END_SIZE;
	    else
		    end_size = end;


	   }

// *** constructor for +/- errors NOT equal
V_err_bars::V_err_bars(int size,
		   double* xdata,
		   double* ydata,
		   double* ydatalo,
		   double* ydatahi,
		   float end,
		   int colour,
		   int thickness) {

	    int c;
	    double y_lower, y_upper;

	    // *** set visibility
	    invisible = 0;

	    // *** get size (check for zero)
	    if (size <= 0) {
			Errors syserr;
			syserr.add_error ("Horizontal error bars ",
			    "too few data points ",
			    "make number_of_points greater than 0");
			return;
			}
	    number_of_points = size;

	    // *** allocate X array
	    Y_lower_array = new float[size];
	    Y_upper_array = new float[size];

	    if (Y_upper_array == 0) {
			Errors syserr;
			syserr.add_error ("Horizontal error bars ",
			    "Insufficient virtual memory for X array ",
			    "call procedure DELETE_PICTURES");
			return;
			}
	    else {
		    for (c = 0; c < size; c++) 
		    {
			y_lower = ydata[c] - ydatalo[c];
			Y_lower_array[c] = y_lower;
			y_upper = ydata[c] + ydatahi[c];
			Y_upper_array[c] = y_upper;
		    }
	    }

	    // *** allocate Y array
	    X_array = new float[size];
	    if (X_array == 0) {
			Errors syserr;
			syserr.add_error ("Horizontal error bars ",
			    "Insufficient virtual memory for Y array ",
			    "call procedure DELETE_PICTURES");
			return;
			}
	    else {
		DoubleArrayToFloatArray(X_array,xdata,size);
	    }

	    // *** fill in line colour
	    if (colour == -1)
		    line_colour = LINE_COLOUR;
	    else
		    line_colour = colour;


	    // *** fill in thickness
	    if (thickness == 0)
		    line_thickness = LINE_THICKNESS;
	    else
		    line_thickness = thickness;

	    // *** fill in end_size
	    if (end == 0.0)
		    end_size = END_SIZE;
	    else
		    end_size = end;


	   }

V_err_bars::~V_err_bars() {

	    // *** delete the NEWed arrays
	    delete [] Y_lower_array;
	    delete [] Y_upper_array;
	    delete [] X_array;

	    }

void V_err_bars::display() {

	    // *** set the colour
	    pgsci(line_colour);

	    // *** set the line thickness
	    pgslw(line_thickness);

	    // *** plot the error bars

	    pgerry(number_of_points, X_array, Y_lower_array, Y_upper_array, end_size);

	    }
