/*  Open GENIE, a program for neutron scattering data analysis & display
    ISIS facility, Rutherford Appleton Laboratory, Chilton, Didcot, OXON UK
    Copyright (C) 1996,1997

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA. */


#include "geniedefs.h"
// -----------------------------------------------------------------------------
// VIEWPORT_FUNCTIONS.C
//
// Description: C++ code to allow creation/manipulation of a Genie 
//	    object-graphics viewport
//
//
#include <math.h>
#include <limits.h>
#include <float.h>

// *** includes for the picture list
#include "Pictlist.h"

// *** local includes
#include "Genie_functions.h"
#include "Viewport.h"
#include "Plot.h"
#include "Multiplot.h"
#include "g_types.h"
#include "transform_utils.h"

static const char* rcsid = "@(#)$Id: viewport_functions.cxx,v 1.19 1997/07/01 23:46:53 faa Exp $";
USE_RCSID(rcsid);

extern Multiplot *mult;

// *** pointer must be static for cursor routine
static Drawable_objects *VIEWPTR;

// ---
// ROUND UP routine.
//	This takes a floating point number and rounds it
//	positively to 2 significant figures. It will round any
//	positive or negative number including zero.
//
float round_up(double value) {
	double x;
	int count = 0;
	int signn = 0;

	// *** check for zero (rounded anyway!!!)
	if (value == 0.0) return 0.0;

	// *** keep a note of negative numbers
	if (value > 0.0) signn = 1;

	// *** get absolute value of number
	if (signn == 0) 
		x = 0.0 - value;
	else
		x = value;

	// *** if (absolute value of number)>1 then divide by 10
	//     if (absolute value of number)<0.1 then multiply by 10
	//     count how many times it is done.
	while((x>10.0) || (x<0.1)) {
	    if (value > 1.0 || value < -1.0)
		    x = x / 10.0;
	    else
		    x = x * 10.0;

	    count++;
	}

	// *** The number must then be made more positive by 0.1
	//     and then rounded to 1 decimal place
	// *** if number is positive then increase by 0.1
	//     if number is negative then decrease its magnitude by 0.1
	//     In either case multiply the number by 10.
	if (signn == 0)
	    x = (x - 0.1) * 10;
	else
	    x = (x + 0.1) * 10;

	// *** Find the integer part and then divide by 10.
	x = ((float)((int)x))/10.0;

	// *** return the value to its original power of 10
	while(count > 0) {
	    if (value > 1.0 || value < -1.0)
		    x = x * 10.0;
	    else
		    x = x / 10.0;
	    count--;
	    }

	// *** return the original sign
	if (signn == 0) 
		    value = 0.0 - x;
	else
		    value = x;

	return value;
}
// ---
// ROUND DOWN routine.
//	This takes a floating point number and rounds it
//	negatively to 2 significant figures. It will round any
//	positive or negative number including zero.
//
float round_down(double value) {
	double x;
	int count = 0;
	int signn = 0;

	// *** check for zero (rounded anyway!!!)
	if (value == 0.0) return 0.0;

	// *** keep a note of negative numbers
	if (value > 0.0) signn = 1;

	// *** get absolute value of number
	if (signn == 0) 
		x = 0.0 - value;
	else
		x = value;

	// *** if (absolute value of number)>1 then divide by 10
	//     if (absolute value of number)<0.1 then multiply by 10
	//     count how many times it is done.
	while((x>10.0) || (x<0.1)) {
	    if (value > 1.0 || value < -1.0)
		    x = x / 10.0;
	    else
		    x = x * 10.0;

	    count++;
	}

	// *** The number must then be
	//     rounded to 1 decimal place
	x = x * 10;

	// *** Find the integer part and then divide by 10.
	x = ((float)((int)x))/10.0;

	// *** return the value to its original power of 10
	while(count > 0) {
	    if (value > 1.0 || value < -1.0)
		    x = x * 10.0;
	    else
		    x = x / 10.0;
	    count--;
	    }

	// *** return the original sign
	if (signn == 0) 
		    value = 0.0 - x;
	else
		    value = x;

	return value;
}

// ---
// *** create an unscaled window for text annotation
//
    int new_window_unscaled(float xDlower,
			       float yDlower,
			       float xDupper,
			       float yDupper,
			       int colour,
			       float& tmp1,
			       float& tmp2,
			       float& tmp3,
			       float& tmp4,
			       g_objectid objid) {

	float xmin,xmax,ymin,ymax;
        g_integer dummy_objid[GENIE_OBJECTID_SIZE];

	if(tmp1 != FLT_MAX  && tmp2 != FLT_MAX && tmp3 != FLT_MAX &&
           tmp4 != FLT_MAX)
	{
	    xmin = tmp1;
	    xmax = tmp2;
	    ymin = tmp3;
	    ymax = tmp4;
	}
	else
	{
		xmin = 0.0;
		ymin = 0.0;
		xmax = 1.0;
		ymax = 1.0;
	}
	// *** NEW an object
	VIEWPTR = new Viewport(xDlower,xDupper,yDlower,yDupper,xmin,xmax,ymin,ymax,colour);

	// *** add to the display list
	objid[ObjectPosition] = PicturePTR[Current_Picture]->disp_add(VIEWPTR);
        objid[ObjectPicture] = Current_Picture;

	// *** set the colour
//	if (colour != 0) new_box(xmin,ymin,xmax,ymax,colour,0,1,dummy_objid);

	return 0;
    }

    int create_window_unscaled(float xDlower,
			       float yDlower,
			       float xDupper,
			       float yDupper,
			       int colour) {

    float xmin(FLT_MAX), xmax(FLT_MAX), ymin(FLT_MAX), ymax(FLT_MAX);
    g_integer objid[GENIE_OBJECTID_SIZE];
    new_window_unscaled(xDlower, yDlower, xDupper, yDupper, colour, xmin, xmax,
			ymin, ymax, objid);

    display_object(objid);
    return objid[ObjectPosition];
    }


// ---
// *** create a scaled window for a plot
// *** if earray == 0, it is ignored in calculating the limits
//
    int new_window_scaled(float xDlower,
		       float yDlower,
		       float xDupper,
		       float yDupper,
		       double *xarray,
		       double *yarray,
		       double *earray,
		       int npts,
		       int colour,
		       int scaling_max_mode,
		       int scaling_min_mode,
		       float LXMIN,
		       float LXMAX,
		       float LYMIN,
		       float LYMAX,
		       g_objectid objid) {

	float xmin,xmax,ymin,ymax;
        g_integer dummy_objid[GENIE_OBJECTID_SIZE];
	int f;

	    // *** initialise the variables
	    xmin = FLT_MAX;
	    xmax = -FLT_MAX;
	    ymin = FLT_MAX;
	    ymax = -FLT_MAX;
	    int istart(0), iend(npts);
	    if (LXMIN != FLT_MAX)
	    {
		istart = bracket(LXMIN, xarray, npts);
		istart = (istart == -1 ? 0 : istart);
	    }
	    if (LXMAX != FLT_MAX)
	    {
		iend = bracket(LXMAX, xarray, npts);
		iend = (iend == -1 ? npts-1 : iend);
	    }

	    // *** loop over all xdata
	    for (f = istart; f<iend; f++) {
		if (xarray[f] > FLT_MAX) xarray[f] = FLT_MAX;
		if (yarray[f] > FLT_MAX) yarray[f] = FLT_MAX;

		if (xmin > xarray[f]) xmin = xarray[f];
		if (xmax < xarray[f]) xmax = xarray[f];

		if (earray != 0)
		{
		    if (ymin > (yarray[f]-earray[f])) 
			ymin = yarray[f]-earray[f];
		    if (ymax < (yarray[f]+earray[f])) 
			ymax = yarray[f]+earray[f];
		}
		else
		{
		    if (ymin > yarray[f]) 
			ymin = yarray[f];
		    if (ymax < yarray[f]) 		
			ymax = yarray[f];
		}
	    }

	// *** scale the YMIN value
	// *** scaling_min_mode = 0 (absolute values)

	// *** scaling_min_mode = 1 or 3 ( + 10 %)
	if (scaling_min_mode == 1 || scaling_min_mode == 3)
		ymin = ymin - (0.1 * fabs(ymin));

	// *** scaling_min_mode = 2 or 3 (round down)
	if (scaling_min_mode == 2 || scaling_min_mode == 3)
		ymin = round_down(ymin);

	// *** scaling_min_mode = 4 (force zero)
	if (scaling_min_mode == 4)
		ymin = 0.0;

	// *** scale the YMAX value
	// *** scaling_max_mode = 0 (absolute values)

	// *** scaling_min_mode = 1 or 3 ( + 10 %)
	if (scaling_max_mode == 1 || scaling_max_mode == 3)
		ymax = ymax + (0.1 * fabs(ymax));

	// *** scaling_min_mode = 2 or 3 (round down)
	if (scaling_min_mode == 2 || scaling_min_mode == 3)
		ymax = round_up(ymax);

	// *** scaling_min_mode = 4 (force zero)
	if (scaling_min_mode == 4)
		ymax = 0.0;

	// *** enforce any limits
	if (LXMIN != FLT_MAX) xmin = LXMIN;
	if (LXMAX != FLT_MAX) xmax = LXMAX;
	if (LYMIN != FLT_MAX) ymin = LYMIN;
	if (LYMAX != FLT_MAX) ymax = LYMAX;

	// *** NEW an object
	VIEWPTR = new Viewport(xDlower,xDupper,yDlower,yDupper,
			xmin,xmax,ymin,ymax,colour);

	// *** add to the display list
	objid[ObjectPosition] = PicturePTR[Current_Picture]->disp_add(VIEWPTR);
        objid[ObjectPicture] = Current_Picture;

	// *** set the colour
//	if (colour != 0) new_box(xmin,ymin,xmax,ymax,colour,0,1,dummy_objid);

	return objid[ObjectPosition];
    }

    int create_window_scaled(float xDlower,
		       float yDlower,
		       float xDupper,
		       float yDupper,
		       double *xarray,
		       double *yarray,
		       double *earray,
		       int npts,
		       int colour,
		       int scaling_max_mode,
		       int scaling_min_mode,
		       float LXMIN,
		       float LXMAX,
		       float LYMIN,
		       float LYMAX) {
    
    g_integer objid[GENIE_OBJECTID_SIZE];
    new_window_scaled(xDlower, yDlower, xDupper, yDupper, xarray, yarray, earray, npts, colour, scaling_max_mode, 
			scaling_min_mode, LXMIN, LXMAX, LYMIN, LYMAX, objid);
    display_object(objid);
    return objid[ObjectPosition];
    }

// ---
// *** scale a window for a multiplot
//
    int new_window_multiplot(float xDlower,
		       float yDlower,
		       float xDupper,
		       float yDupper,
		       int colour,
		       int scaling_max_mode,
		       int scaling_min_mode,
		       float& tmp1,
		       float& tmp2,
		       float& tmp3,
		       float& tmp4,
		       g_objectid objid) {
	float xmin,xmax,ymin,ymax;

	// *** GET THE WORLD CO-ORDINATES
	if(tmp1 != FLT_MAX  && tmp2 != FLT_MAX && tmp3 != FLT_MAX &&
           tmp4 != FLT_MAX)
	{
	    xmin = tmp1;
	    xmax = tmp2;
	    ymin = tmp3;
	    ymax = tmp4;
	}
	else if (mult != 0)
	{
	    mult->get_size(xmin, xmax, ymin, ymax);	
	    tmp1 = xmin;
	    tmp2 = xmax;
	    tmp3 = ymin;
	    tmp4 = ymax;
	} 
	else
	{
	    gerr << "MULTIPLOT limits error - this needs looking at" << endl;
	}

	// *** scale the YMIN value
	// *** scaling_min_mode = 0 (absolute values)

	// *** scaling_min_mode = 1 or 3 ( + 10 %)
	if (scaling_min_mode == 1 || scaling_min_mode == 3)
		ymin = ymin - (0.1 * fabs(ymin));

	// *** scaling_min_mode = 2 or 3 (round down)
	if (scaling_min_mode == 2 || scaling_min_mode == 3)
		ymin = round_down(ymin);

	// *** scaling_min_mode = 4 (force zero)
	if (scaling_min_mode == 4 && ymax > 0.0)
		ymin = 0.0;

	// *** scale the YMAX value
	// *** scaling_max_mode = 0 (absolute values)

	// *** scaling_min_mode = 1 or 3 ( + 10 %)
	if (scaling_max_mode == 1 || scaling_max_mode == 3)
		ymax = ymax + (0.1 * fabs(ymax));

	// *** scaling_max_mode = 2 or 3 (round down)
	if (scaling_max_mode == 2 || scaling_max_mode == 3)
		ymax = round_up(ymax);

	// *** scaling_max_mode = 4 (force zero)
	if (scaling_max_mode == 4 && ymin < 0.0)
		ymax = 0.0;

	// *** NEW an object
	VIEWPTR = new Viewport(xDlower,xDupper,yDlower,yDupper,
			xmin,xmax,ymin,ymax,colour);

	// *** add to the display list
	objid[ObjectPosition] = PicturePTR[Current_Picture]->disp_add(VIEWPTR);
        objid[ObjectPicture] = Current_Picture;


	return objid[ObjectPosition];
    }

    int create_window_multiplot(float xDlower,
   	                    	float yDlower,
       		                float xDupper,
       		                float yDupper,
       		                int colour,
       		                int scaling_max_mode,
       		                int scaling_min_mode)
       		                {

	float xmin(FLT_MAX), xmax(FLT_MAX), ymin(FLT_MAX), ymax(FLT_MAX);
	g_integer objid[GENIE_OBJECTID_SIZE];
	new_window_multiplot(xDlower, yDlower, xDupper, yDupper, colour,
			     scaling_max_mode, scaling_min_mode, xmin, xmax, 
			     ymin, ymax, objid);

	display_object(objid);
    	return objid[ObjectPosition];
    }

// ---
// *** get a cursor in a window
//
    void get_cursor(float* world,
			  float* device,
			  char *letter,
			  g_objectid id) {

	    String reply = " ";
	    Drawable_objects* vptr;

//	     ** update the display
//	    flush_buffer();

//	    id[ObjectPosition] = PicturePTR[Current_Picture]->disp_add(VIEWPTR);
//          id[ObjectPicture] = Current_Picture;

// We'll not just use the last window, we'll actually try and select the one asked for
	    vptr = VIEWPTR;
	    VIEWPTR = PicturePTR[id[ObjectPicture]]->objectPointer(id[ObjectPosition]);	    
	    if (vptr != VIEWPTR)
	    {
		gdbg << "get_cursor(): new window behaviour, check result is OK" << endl;
	    }

	    // *** get cursor in last viewport
	    VIEWPTR->cursor(world,device,reply);
	    if (reply == "^") picture_display(world);

	    // *** put reply back
	    strcpy(letter,reply);

	}

    void get_window_cursor(float* world,
			   float* device,
			   char *letter)  {
        g_integer id[GENIE_OBJECTID_SIZE];
        id[ObjectPosition] = PicturePTR[Current_Picture]->find(VIEWPTR);
        id[ObjectPicture] = Current_Picture;
        get_cursor(world,device,letter,id);
    }

    int new_window_two_d(g_real dev_xmin, 
			    g_real dev_xmax, 
			    g_real dev_ymin, 
			    g_real dev_ymax,
                            g_real* x, 
			    int nx, 
			    g_real* y, 
			    int ny, 
			    int colour,
			    float& tmp1,
			    float& tmp2,
			    float& tmp3,
			    float& tmp4, 
			    g_objectid objid)  {

    
	int i; 
	
	float xmin,xmax,ymin,ymax;
	int f;

	if(tmp1 != FLT_MAX  && tmp2 != FLT_MAX && tmp3 != FLT_MAX &&
	   tmp4 != FLT_MAX)  
	{}	
	else
	{
    	    tmp1 = x[0];
    	    tmp2 = x[0];
    	    for(i=1; i<nx; i++)
    	    {
	        if (x[i] < tmp1)
                     tmp1 = x[i];
	        if (x[i] > tmp2)
	            tmp2 = x[i];
            }
            tmp3 = y[0];
            tmp4 = y[0];
            for(i=1; i<ny; i++)
            {
	        if (y[i] < tmp3)
	            tmp3 = y[i];
	        if (y[i] > tmp4)
	            tmp4 = y[i];
            }
	}

	xmin = tmp1;
	xmax = tmp2;
	ymin = tmp3;
	ymax = tmp4;
	
	VIEWPTR = new Viewport(dev_xmin,dev_xmax,dev_ymin,dev_ymax,
			xmin,xmax,ymin,ymax,colour);


	// *** add to the display list
        objid[ObjectPosition] = PicturePTR[Current_Picture]->disp_add(VIEWPTR);
        objid[ObjectPicture] = Current_Picture;


	// *** set the colour
//	if (colour != 0) draw_box(xmin,ymin,xmax,ymax,colour,0,1);

        return objid[ObjectPosition];
}

    int create_window_two_d(g_real dev_xmin,
                            g_real dev_xmax,
                            g_real dev_ymin,
                            g_real dev_ymax,
                            g_real* x,
                            int nx,
                            g_real* y,
                            int ny,
                            int colour)
	{

    float xmin(FLT_MAX), xmax(FLT_MAX), ymin(FLT_MAX), ymax(FLT_MAX);
    g_integer objid[GENIE_OBJECTID_SIZE];
    new_window_two_d(dev_xmin, dev_xmax, dev_ymin, 
		     dev_ymax, x, nx,  y, 
		     ny, colour, xmin, xmax, ymin, ymax, objid);
    display_object(objid);
    return objid[ObjectPosition];
    }

// Old graphics display
int display_object(g_objectid objid)
{
    Drawable_objects* obj_ptr;
    obj_ptr = PicturePTR[objid[ObjectPicture]]->objectPointer(objid[ObjectPosition]);
    obj_ptr->display();
    flush_buffer();
    return  0;
}

// New graphics display
int display_graphical_object(g_objectid objid, void* oop)
{
    Drawable_objects* obj_ptr;
    char object_tag[32];
//    sprintf(object_tag, "pg%ld", (long)(objid[ObjectPicture] + 1000 * objid[ObjectPosition]));
    sprintf(object_tag, "pg%lu", (unsigned long)oop);
    set_current_object_tag(object_tag);		/* set tag for Tk canvas */
    obj_ptr = PicturePTR[objid[ObjectPicture]]->objectPointer(objid[ObjectPosition]);
    obj_ptr->display();
    flush_buffer();
    set_current_object_tag(0);			/* reset tag */
    return  0; 
}

int remove_object(g_objectid objid)
{
    PicturePTR[objid[ObjectPicture]]->disp_remove(-objid[ObjectPosition]);
    return 0;
}
