/*  Open GENIE, a program for neutron scattering data analysis & display
    ISIS facility, Rutherford Appleton Laboratory, Chilton, Didcot, OXON UK
    Copyright (C) 1996,1997

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA. */

#include "geniedefs.h"
//+++
//
// CLASS: Window
//
//
//---
#include <string.h>

#include "Drawable_objects.h"

#include "motifdriver_functions.h"
#include "Pginclude.h"
#include "g_types.h"
#include "colourmap.h"
#include "Buffering.h"
#include "Window.h"

static const char* rcsid = "@(#)$Id: window.cxx,v 1.17 1997/07/02 10:50:46 faa Exp $";
USE_RCSID(rcsid);

//---
// *** default constructor
Window::Window(String device, 
	       String file,
	       float x, 
	       float y,
	       float ra,
	       String title,
	       String icont,
	       funcPtrI exf,
	       funcPtr redrf,
	       String menutitle,
	       int n,
	       String menuitems,
	       funcPtrI menuf) : device_id(0), m_map(0) {


	X_dimension = x;
	Y_dimension = y;
	h_pictures = 1;
	v_pictures = 1;
	device_type = device;
	device_type.upcase();
	filename = file;
	ratio = ra;

	WindowTitle = title;
	IconName = icont;

	MenuTitle = menutitle;
	MenuItems = menuitems;
	Nitems = n;
	ExitFunction = exf;
	MenuFunction = menuf;
	RedrawFunction = redrf;
	}

//---
// *** destructor
Window::~Window() {
	}

void Window::installColourmap(ColourMap* map) 
{
	// Set up colours
	m_map = map;
	if (m_map != 0)
	{
	    m_map->install();
	}

}

//---
// *** put the window on the display surface
// Return 0 on success, -1 on error
int Window::create() {


	String pgdevice;
	int zero = 0;
	float xyratio = 1.0;
	char s1[100],s2[100];

	// set parameters for XWINDOWS device only
	if (device_type == "XW" || device_type == "XS") {

		    // *** set the title
		    strcpy(s1,WindowTitle);
		    strcpy(s2,IconName);
// These call routines in motifutils/motifdriver so
// comment out if we do not have motif 
#ifdef HAVE_LIBXM
		    starttitle(s1,s2);

		    // *** register the redraw function
		    setredrawfunct(RedrawFunction);

		    // *** register the exit function
		    setexitfunct(ExitFunction);

		    // *** set the menu
		    strcpy(s1,MenuTitle);
		    strcpy(s2,MenuItems);
		    startusermenu(s1, Nitems, s2, MenuFunction);
#endif	/* HAVE_LIBXM */

		    // *** set the initial size
		    X_dimension = X_dimension * ratio;
		    Y_dimension = Y_dimension * ratio;
#ifdef HAVE_LIBXM
		    startsizeins(X_dimension, Y_dimension);
#endif	/* HAVE_LIBXM */
	}		    		    

	// construct the device name 
	pgdevice = filename + "/" + device_type;

	// call pgbegin
        PGOPEN(pgdevice, device_id);
        if (device_id <= 0)
	{
	    *g_err << "WINDOW: Error creating device \"" << pgdevice << "\"" << endl;
	    return -1;
	}

	pgsubp(h_pictures, v_pictures);

	// set size for non TK devices
	if (strncmp(device_type, "TK", 2))	// TK1, TK2 etc
 	{
	    if ( (device_type == "VPS") && (Y_dimension != 0.0) )
	    {
		xyratio = X_dimension / Y_dimension;
	    }
	    else if ( (device_type != "VPS") && (X_dimension != 0.0) )
	    {
	    	xyratio = Y_dimension / X_dimension;
	    }
	    pgpaper(X_dimension,xyratio);
	}

	// switch OFF terminal prompting
	pgask(zero);

	select();

	flush_buffer();
	return 0;

	}

//---
// *** remove the window (END pgplot)
void Window::remove() {

	if (select())
	    pgclos();
	}

//---
// *** clear the window
void Window::clean() {

	// call pgpage
	if (select())
	    pgpage();
	}

// return 0 on failure, >0 on success
pg_int Window::select()
{
    if (device_id > 0)
    {
        pgslct(device_id);
	if (m_map != 0)
	    m_map->install();
	return device_id;
    }
    else
    {
        return 0;
    }
}
