%{
#include <assert.h>
#include <stdarg.h>
#include "geniedefs.h"
#ifdef HAVE_ALLOCA_H		/* defined in config.h, which is included by geniedefs.h */
#include <alloca.h>
#endif /* HAVE_ALLOCA_H */
#include "parse_funcs.h"

static const char* rcsid = "@(#)$Id: yacccommands.y,v 1.45 1997/06/30 22:15:41 faa Exp $";
USE_RCSID(rcsid);

/*

This is the GENIE-V3 parser
===========================

This file is pre-processed by yacc before compilation in C. This does mean that pure
C code and yacc syntax are hopelessly intermingled !  Any code delimited by %{ ... %}
is straight C which will pass unchanged into the yacc output.  As such it may contain
references to things produced by yacc.


===========================
Preamble and functions in C
===========================*/

#define __INSIDE_PARSER__

#include <ctype.h>
#include <stdio.h>
#include <string.h>
#include <signal.h>
#include <stdlib.h>
#include <sys/types.h>
#include "parsedefs.h"
#include "stmacros.h"		/* Macros containing bits of smalltalk code for parser (kept separate for tidyness) */

#ifndef __VMS				/* These don't work in POSIX 2.0 Yacc */
#undef YYDEBUG
#define YYDEBUG 1
extern int yydebug;			/* debug flag for YACC */
#endif


/*
    Definition of parse tree nodes and related functions.
    Note: This is not a comprehensive semantic parse tree,
    just a temporary represenatation of the translated GCL
    code as a tree of strings (to avoid copying).
*/
static node *get_node(void);

/* Keep a list of parser nodes to ease garbage collection */
#define     OLD_NODES_MAX 10000   /* Number of old nodes recorded by nprintf */
static int  old_node_count = 0;
static node old_nodes[OLD_NODES_MAX];

int malloc_count = 0;

static node *addNode(node *original, node *newnode)
{
    assert(Valid(newnode));     /* original node can be null (to start a tree) but not the new list */
    assert(Valid(original) || !original);    /* Avoid half nodes of any sort */

    if (Valid(original))
    {
        original->next->side = addNode(original->next->side, newnode);
        return original;
    }
    else
        return newnode;        
}

/*#define addNode(a,b) ( (__LINE__>73)?(gprintif("%d ",__LINE__), addNode((a),(b)) ):addNode((a),(b)) )*/

node *newLeaf(const char* value)
{
    node *tmp;

    /* Allocate a node from the nodelist */
    tmp = get_node();

    tmp->next = (snode) malloc(sizeof(struct snode_struct));
    malloc_count++;
    tmp->next->side = NULL;   /* Leave this until another leaf is added by addNode */
    tmp->next->leaf = (char *) malloc(strlen(value)+1);
    malloc_count++;
    strcpy(tmp->next->leaf, value);
    tmp->valid = 1;                 /* This node is now allocated properly */

    return tmp;
}


void deleteNode(node *pnode)
{
    if (Valid(pnode))
    {
        /* Delete any nodes on the side chain */
        deleteNode( pnode->next->side );

        /* Delete leaf */
        free(pnode->next->leaf);
	    malloc_count--;
        pnode->next->leaf = NULL;

        /* Free up storage held by this node */
        free(pnode->next);
            malloc_count--;
	pnode->next = NULL;

        pnode->valid = 0;
    }
}


int nlength(node *pnode)
{
    if (Valid(pnode))
	    return strlen(pnode->next->leaf) + nlength(pnode->next->side);
    return 0;
}


static void prnNode(char *buffer, node *anode)
{
    if (Valid(anode))
    {
        strcat(buffer, anode->next->leaf);
        prnNode(buffer, anode->next->side);
    }
}


void printNode(char *buffer, node *anode)
{
    buffer[0] = '\0';
    prnNode(buffer, anode);
}


static void debugNode(node *anode)
{
    if (Valid(anode))
    {
        gprintdf(anode->next->leaf);
        debugNode(anode->next->side);
    }
    else
        gprintdf("(NULL)");
}


static node *copyNode(node *anode)
{
    node *tmp = NULL;

    if (Valid(anode))
    {
        tmp = newLeaf(anode->next->leaf);
        if ( Valid(anode->next->side) )
            addNode(tmp, copyNode(anode->next->side));
    }

    return tmp;
}

/*
 * Returns a pointer to an allcated node slot
 */

/* just out here for inspection */
static int i_max;

static node *get_node(void)
{
    node *result = NULL;

    if (old_node_count < OLD_NODES_MAX - 1)
    {
        /* Scan to the first free slot */
        for ( i_max=0; (i_max <= old_node_count) && old_nodes[i_max].valid; i_max++ );

        /* Now fill a used slot or create a new one depending on the result */
        if ( i_max > old_node_count )
            result = &old_nodes[++old_node_count];
        else
            result = &old_nodes[i_max];

        *result = NULL_NODE;
    }
    else
    {
        gprintef("Save aside list length for nodes exceeded,\n");
        gprintif("  Suggestion: increase OLD_NODES_MAX in parser)\n");
        Kill_smalltalk(1);
    }

    return result;
}
        

/* Deletes any nodes left allocated to ensure memory security */
void sweep_nodes()
{
    int i;

    if (i_max > 1)
        gprintdf("Swept %d nodes\n", i_max);
    for ( i=0; i < old_node_count; i++)
        if (old_nodes[i].valid)
            deleteNode(&old_nodes[i]);
    old_node_count = 0;
}


/*
    nprintf
    =======
    Used like sprintf but nprintf constructs a parse tree made out of
    strings.  This avoids the massive duplication which occurs if the
    whole parse tree is created with sprintf (as it was before!)

    Calling nprintf with NULL instead of a format string is a special case
    which deletes the last n nodes (in reverse order). This is to allow
    a parser clean up of otherwise difficult to find nodes !

*/

void nprintf(node **result, const char* fmt, ...)
{
    static va_list	args, ap;
    static char    *s, fmtbuff[MAXSTRLEN];
    static const char *fp;
    static char    formtmp[MAXSTRLEN];

    fmtbuff[0] = formtmp[0] = '\0';
    va_start(args, fmt);

    *result = newLeaf("");        /* Just allocate a node */

    for (s = fmtbuff, fp = fmt, ap=args; fp && *fp; fp++)
    {
        if (*fp == '%')
        {
            fp++;
            switch (*fp)
            {
                case 'n':
                    *s = '\0';
                    vsprintf(formtmp, fmtbuff, ap);
                    *result = addNode(addNode(*result,newLeaf(formtmp)), va_arg(args, node*));
                    s = fmtbuff;    /* Reset for next part of format string */
                    ap = args;      /* Keep last position in args */
              	    break;

                default:
                    va_arg(args, void*);
                case '%':
                    *s++ = '%';
                    *s++ = *fp;
            }
        }
        else
            *s++ = *fp;
    }

    if ( s != fmtbuff )     /* if last thing in format was a %n there is nothing else to add */
    {
        *s = '\0';
        vsprintf(formtmp, fmtbuff, ap);
        *result = addNode(*result, newLeaf(formtmp));
    }
    va_end(args);

}

/*
Including the line below is a very NATTY DEBUGGING TECHNIQUE for the parser. In the event of
a parser crash, there is usually no sensible debugging information for the yacc actions.  This
prints out the position of all nprintfs up to the last one before the crash to show which
line number was the one which actually caused the explosion!
*/

/* #define nprintf (gprintdf("%d ", (long) __LINE__), &nprintf) */


/*
	Variables holding extra information about parser state
	======================================================
	(Organized under the appropriate syntactic constructs)
*/


/*
 * General variables
 * ====================
 */


/* Some temporaries for assembling generated code into */

node *parseTree = NULL, *tmp1_tree = NULL, *tmp2_tree = NULL;

char buffer1[MAXSTRLEN];
char buffer2[MAXSTRLEN];
char buffer3[MAXSTRLEN];


/*
Procedure definition
--------------------
The syntactic construct for a procedure cannot be used recursively (ie, procedures cannot be defined
within procedures). The main use of temporary storage is to keep a track on information made available
early on in the procedure but used later or in several places in the code generation.
*/

enum	{ Local, Global, Result } Current_line_type ;	/* For recording the type of declaration line being parsed
							   (some of the code is common to all types) */

char	Current_procedure[MAXVARLEN];			/* The name of the procedure being parsed
							   - heavily used */

node	*Current_initializer;			/* Hold the body of a smalltalk method to initialize all local 
							   and global variables at the invocation of a procedure. This
							   is built incrementally throughout the declaration block */

char	Current_result[MAXVARLEN];			/* Holds the result returning string for a function method */


/* The following syntactic structures can be textually nested so some simple stacking macros are provided to make recording
   of control lvalues a little safer. */

/* for integers*/
#define PPUSH(stack, sp)	if ( sp < MAXNEST-1 ) stack[++sp] = 1; else \
	gprintef("parser: Attempted Pstack overflow\n");
#define PPOP(stack, sp)		if (sp > -1) stack[sp--] = 0; else \
	gprintef("parser: Attempted Pstack underflow\n");
#define PTOP(stack, sp)		stack[sp]

/* for strings */
#define SSPUSH(stack, sp)	if ( sp < MAXNEST-1 ) stack[++sp] = NULL; else \
	gprintef("parser: Attempted Sstack overflow\n");
#define SSPOP(stack, sp)	if (sp > -1) deleteNode(stack[sp--]); else \
	gprintef("parser: Attempted Sstack underflow\n");
#define SSTOP(stack, sp)	copyNode(stack[sp])


/*
Function invocation (Keyword, function or method syntax)
--------------------------------------------------------
Hold count of anonymous parameters found on a command line ie p1, p2..., pn
use a simple stack to allow saving of default paramter name counts when several
functions are used on one line (ie, when the syntax is used recursively).
*/

int Function_depth = -1;		/* Becomes 0 on entry to the first keyword or function */
int Param_no[MAXNEST];			/* Saves n for the p`nth' parameter */


/*
LOOP construct
--------------
This construct may also be used recursively which means we have to keep a record of nesting depth and also
record variables for inner and outer loops simultaneously.
*/

int Loop_depth = -1;			/* depth becomes 0 on entry into the first textual loop */
node *Loop_lvalue[MAXNEST];	    /* Holds the name of the loop control variable (it is an lvalue) */


/*
CASE construct
--------------
This construct may also be used recursively which means we have to keep a record of nesting depth and also
record variables for inner and outer case statements simultaneously.
*/

int Case_depth = -1;			/* depth becomes 0 on entry into the first textual loop */
node *Case_lvalue[MAXNEST];	    /* Holds the name of the case testing variable (it is an lvalue) */


/*
IF construct
--------------
This construct may also be used recursively which means we have to keep a record of nesting depth.
*/

int If_depth = -1;			/* depth becomes 0 on entry into the first textual IF */


/*
Arrays & structures
-------------------
Counter to give number of array index in multidimensional arrays (not a recursive construct)
*/

int Array_index_depth = -1;			/* Becomes 0 on entry to the first array index */
int Array_index_no[MAXNEST];

int method_pass_case;                           /* Used by parser to determine whether to pass on capitalization information */
int structure_flag, keep_structure_flag;	/* flags whether the last l-value parsed was a structure or variable */
char	structure_name[MAXVARLEN],		/* keeps the name of the structure variable */
	keep_structure_name[MAXVARLEN];

/*
Passthru code blocks
---------------------
Section of escaped smalltalk code embedded into GCL (as with lots of startup files).
*/
int in_smalltalk = 0;


/*============== FUNCTIONS  ================*/



/*
 * Initializes static parser variables before initatiating a parse cycle
 */
void initialize_parser(int mode, FILE *channel, int ydebflag)
{

/*    malloc_count = 0;       /* For debugging */

/*  Small test of tree builder (nprintf) + output */
#if 0
    node n,n1,n2;

    malloc_count = 0;
    nprintf(&n1, "Test%dnew", 34L);
    nprintf(&n, "Surr%nounded", n1);
    n2 = copyNode(n);
/*    gprintdf("malloc_count(A) = %d\n", malloc_count); */
    deleteNode(&n);
    nprintf(&n1, "Test%dnew", 34L);
    nprintf(&n, "Surr%noun%nded", n2, n1);
    debugNode(n);
    deleteNode(&n);
/*    gprintdf("\nmalloc_count(B) = %d\n", malloc_count); */
    malloc_count = 0;

#endif

#if 0
    node s, s1, s2, s3, s4, s5;
    malloc_count = 0;
    nprintf(&s2, "123%d456", 9999L, "asdf", 5.69, 1L, 2L, 3L, 4L, 5L);
    nprintf(&s5, "OOOOOO%sOOOOOOhhh!!!", "o");
    nprintf(&s4, "This is just a sub-leafy bit");
    nprintf(&s3, "12%n3%d456%n", s4, 9999L, s5, "asdf", 5.69, 1L, 2L, 3L, 4L, 5L);
    nprintf(&s1, "%d=%n=%f=(%n)", 1111L, s2, 45.88, s3);
    gprintdf("\ns1=");
    debugNode(s1);
    gprintdf("\nD=%d", nlength(&s1));
    s = copyNode(s1);
    deleteNode(&s1);

    gprintdf("\nWhat's left=");
    debugNode(s1);
    gprintdf("\nD=%d", nlength(&s1));
    gprintdf("\nCopied node=");
    debugNode(s);
    gprintdf("\nD=%d", nlength(&s));
    deleteNode(&s);
    gprintdf("\nWhat's left (of s)=");
    debugNode(s);
    gprintdf("\nD=%d", nlength(&s));
    gprintdf("\n");
    gprintdf("malloc_count(A) = %d\n", malloc_count);
#endif

/*
===Output===
s1=1111=1239999456=45.880000=(12This is just a sub-leafy bit39999456OOOOOOoOOOOOOhhh!!!)(NULL)
D=85
What's left=(NULL)
D=0
Copied node=1111=1239999456=45.880000=(12This is just a sub-leafy bit39999456OOOOOOoOOOOOOhhh!!!)(NULL)
D=85
What's left (of s)=(NULL)
D=0
malloc_count(A) = 0
============
*/


/* Initialize local variables */

    Current_initializer = newLeaf("");                               /* Start with an empty leaf */
    strcpy(Current_result, ".\nself up.\n^nil");
    strcpy(Current_procedure, "  <No current procedure>");

    Function_depth	= -1;
    Loop_depth		= -1;		/* Static variables in the parser */
    Case_depth		= -1;
    If_depth		= -1;
    Array_index_depth	= -1;

    in_smalltalk        = 0;    /* Flags that we are in a smalltalk code block */

    Interactive = mode;

    InChan = channel;

#ifndef __VMS				/* Doesn't work in POSIX 2.0 yacc */
    yydebug = ydebflag;
#endif

/*    gprintdf("malloc_count(A) = %d\n", malloc_count); */

    PPUSH(Param_no, Function_depth);	/* Almost part of the parsing but easier to do here */
    return;
}



void initialize_parser_stats()
{
    lines_parsed = 0;
    errors_found = 0;
    return;
}


void p_stats()
{
    gprintdf("nodes used = %d, allocated now = %d", old_node_count, i_max);
}


/*
 * To return parser output to the calling program
 */
node *interrogate_parser(int *errcnt, int *linecnt)
{
    *linecnt = lines_parsed;
    *errcnt = errors_found;	
    if (nlength(Current_initializer) == 0)
        deleteNode(Current_initializer);

    return parseTree;
}



/*
 * Store some sentinel values in the buffers to check for overflow later
 */
void fill_buffers()
{
    int i;

    for( i=0; i < MAXSTRLEN; i++)
    {
	buffer1[i] = '~';
	buffer2[i] = '~';
	buffer3[i] = '~';
    }

    return;
}



/*
 * Sort out whether to use a prompt or not
 */
int prompt_mode()
{
    return Interactive && !continuation_line &&
	(Current_procedure[0] == ' ') &&
	(Loop_depth == -1) &&
	(Case_depth == -1) &&
	(If_depth == -1) &&
	! in_smalltalk ;
}



/*
 * Count backwards to find the smallest free space left in a buffer after parsing
 */
int check_buffers()
{
    int i;

    for( i = MAXSTRLEN - 1; i >= 0; i-- )
    {
    	if( buffer1[i] != '~') break;
    	if( buffer2[i] != '~') break;
    	if( buffer3[i] != '~') break;
    }
 
    return MAXSTRLEN - (i + 1);
}



/* Action routine for debugging parser syntax */

static void Paction(char x[]) {
    gprintdf("%s\n", x);
}


/* replaces "+" signs where smalltalk doesn't like them */

static void ReplacePlusSigns( char Astring[] ){
    char *p;
    for( p=Astring; (p=strchr(p, '+')); *p++='0' );	/* Replaces any '+' characters in the string with a '0' */
    return;
}


/* Only define an executable Paction if in Parser debugging mode */

#ifdef _DEB_
/*#define PACTION(a,b,c,d,e,f) Paction(a,b,c,d,e,f);break;*/
#define LHS(x) x
#define RHS(x) x
#else

#if 0
#define PACTION(a,b,c,d,e,f) Paction(e);
#else
#define PACTION(a,b,c,d,e,f)
#endif

#define LHS(x) x.lhs
#define RHS(x) x.rhs
#endif

/* End C preamble */
%}


/*
===================================================================================================================
Yacc code for parser from now on
===================================================================================================================
*/


/* Set up return types for parser tokens */
%union {					/* Declare returned value type */
    node        *nonterminal;
	int		    aninteger;
	double		areal;
	char		astring[MAXSTRLEN];
	char		terminal[MAXSTRLEN];
/*	char		nonterminal[MAXSTRLEN]; */

	char		debugging_string[MAXSTRLEN];

	struct
	{				/* define a structure for holding both halves of a split expression */
	    node *lhs;
	    node *rhs;
	} split_exp;

}


/* For syntax debugging of the parser, this effectively disables
   the typing by making everything into a debugging_string */
%{
#ifdef _DEB_
#define aninteger   debugging_string
#define areal	    debugging_string
#define astring	    debugging_string
#define terminal    debugging_string
#define nonterminal debugging_string
#define split_exp   debugging_string
#endif
%}

/* Token Definitions */


	/* Keywords */
	%token <terminal>	OR AND PROCEDURE ENDPROCEDURE PARAMETERS FORWARD
	%token <terminal>	QUALIFIERS RESULT GLOBAL LOCAL LOOP FROM TO STEP RETURN
	%token <terminal>	ENDLOOP EXITIF CASE IS ENDCASE OTHERWISE IF ELSEIF ELSE ENDIF

	/* Multi-character tokens reduced by lex */
	%token <terminal>	IDENTIFIER

	%token <aninteger>	UINTEGER
	%token <areal>		UREAL

	%token <astring>	STRING COMMENT
	%token <astring>	ENDCODE CODELINE		/* Pasthru of internal code from lex */

	%token <terminal>	ALIAS NOTEQUALS GEQUAL LEQUAL
	%token <terminal>	ENDOFLINE			/* Matches various end of lines ( eg ; \cr \cr\lf ) */
	%token <terminal>	UPLUS UMINUS NOT		/* Specials to disambiguate as far as possible */

	/* Single character tokens which can legally be used in the yacc syntax (RUBBISH is a catchall for illegal characters) */
	%token <terminal> '-' ']' '^' '&' '+' '*' '/' '|' '<' '>' '=' '%' '(' ')' '.' ':' ',' '[' '@' RUBBISH


/* Precedence rules */
	%nonassoc IDENTIFIER					/* Ensure identifier is translated less readily than .	 */
	%right '.'
	%right '[' ']'
	%nonassoc '(' ')'					/* an IDENTIFIER '(' ... combination of a function call  */
	%nonassoc '>' '<' '=' ALIAS GEQUAL LEQUAL NOTEQUALS
    %left ':' '@'                       /* Could conceivably compare intervals so put above comparison ops */
	%left '&'
	%left OR '+' '-'
	%left AND '*' '/' '|'
	%right '^'
	%right UPLUS UMINUS NOT

/* Non terminal types */
	%type <nonterminal> procedure block command_line assignment keyword lvalue expression
	%type <nonterminal> loop_construct exit_control loop_line exitif_line endloop_line count_control loop_range
	%type <nonterminal> case_construct alternatives otherwise_part case_line endcase_line test_clause is_line
	%type <nonterminal> otherwise_line case_range if_construct expr_line endif_line else_part else_line
	%type <nonterminal> proc_header proc_decl proc_top_block end_of_line
	%type <nonterminal> param_line qual_line result_line header_line decl_line local_line global_line code_block code
	%type <nonterminal> function_parameter qual_list1 qual_list2 arguments fp_list qual_defs fp_defs type_specifier
	%type <nonterminal> var_defs variable_declarator method_args m_arg_list method_parameter
	%type <nonterminal> arg_list function_call
	%type <nonterminal> proc_line end_proc_line
	%type <nonterminal> program_unit
	%type <nonterminal> error

/* Type where the resultant expressions grows by both sides */
	%type <split_exp> struct_spec method_ref array_index index_ref cond_block elif_part

%start program_unit

%%

/*
    Control parser (high level) multi-line syntax
    ==================================
*/

/* Parse one program chunk at a time. Once a valid program unit is found force an exit to the parsing
   with YYABORT.  NOTE: don't use a YYABORT on the null generating EOF rule, this will match and
   terminate naturally as desired with a YYACCEPT status code */

program_unit	: procedure
        		    {
            			PACTION($1, "", "", "", "<procedure>", $$);
            			nprintf(&parseTree, "\"Procedure class definition:\n ===========================\"\n%n\n\n", $1);
                        YYABORT;
        		    }
        		| command_line
        		    {
            			PACTION($1, "", "", "", "<command_line>", $$);
            			nprintf(&parseTree, "%n!\n", $1);
            			YYABORT;
        		    }
        		| end_of_line
        		    {
            			PACTION($1, "", "", "", "<end_of_line>", $$);
            			nprintf(&parseTree, "");					/* Do nothing with comment lines */
            			YYABORT;
        		    }
        		| /* END OF FILE - matches $end immediatly after this */
        		    {
            			PACTION("", "", "", "", "<eof>", $$);
            			nprintf(&parseTree, "");
        		    } ;

/*
   Have to get round the problem of comments producing indistinguishable start symbols for the
   different parts of a procedure.  To do this, all possibilities of lambda producing procedure
   parts are enumerated separately.  All internal parts can produce comments but only after the
   appropriate start symbol has been found once.
*/

procedure	: proc_line end_proc_line
    		    {
        			PACTION($1, $2, "", "", "<proc_line> <end_proc_line>", $$);
                    nprintf(&tmp2_tree, "('*** Empty procedure \"', self class name gtype, '\" called.') prnl");
                    PROCEDURE_BODY(&tmp1_tree,tmp2_tree, Current_procedure, Current_result)
        			nprintf(&$$, "%n%n%n", $1,tmp1_tree, $2);
        			strcpy(Current_procedure, "  <No current procedure>");
    		    }
    		| proc_line block end_proc_line
    		    {
        			PACTION($1, $2, $3, "", "<proc_line> <block> <end_proc_line>", $$);
        			PROCEDURE_BODY(&tmp1_tree, $2, Current_procedure, Current_result)
        			nprintf(&$$, "%n%n%n", $1,tmp1_tree, $3);
        			strcpy(Current_procedure, "  <No current procedure>");
    		    }
    		| proc_line proc_top_block end_proc_line
    		    {
        			PACTION($1, $2, $3, "", "<proc_line> <proc_header> <end_proc_line>", $$);
        			nprintf(&tmp1_tree, "\"HEADER\n ------\"\n\n%n", $2);
                    nprintf(&$$, "%n%n\n\n%n", $1, tmp1_tree, $3);
        			strcpy(Current_result, ".\nself up.\n^nil");
        			strcpy(Current_procedure, "  <No current procedure>");
    		    }
    		| proc_line proc_top_block block end_proc_line
    		    {
        			PACTION($1, $2, $3, $4, "<proc_line> <proc_header> <block> <end_proc_line>", $$);
        			nprintf(&tmp1_tree, "\"HEADER\n ------\"\n\n%n", $2);
                    PROCEDURE_BODY(&tmp2_tree, $3, Current_procedure, Current_result)
        			nprintf(&$$, "%n%n\n\n%n%n", $1, tmp1_tree, tmp2_tree, $4);
        			
        			strcpy(Current_result, ".\nself up.\n^nil");
        			strcpy(Current_procedure, "  <No current procedure>");
    		    } ;

proc_top_block	: proc_header proc_decl
        		    {
            			PACTION($1, $2, "", "", "<proc_header> <proc_decl>", $$);

            			/* when header is complete, append an initializer method for all variables */
            			printNode(buffer1, Current_initializer);
                        deleteNode(Current_initializer);
                        if ( buffer1[0] != '\0' )
            			    buffer1[strlen(buffer1)-2] = '\0';      /* If variables have been added, clip the string short */

            			nprintf(&$$, "%n\n!\n\n%n\n!\n\n!%s methodsFor: 'initializing'!\n\ninitVars\n%s\n!!",
            				$1, $2, Current_procedure, buffer1);
        		    }
        		| proc_decl
        		    {
            			PACTION($1, "", "", "", "<proc_decl>", $$);
            			/* when header is complete, append an initializer method for all variables */
            			printNode(buffer1, Current_initializer);
                        deleteNode(Current_initializer);
            			if ( strlen(buffer1) > 0 )
                            buffer1[strlen(buffer1)-2] = '\0';
                        else
                            buffer1[0]= '\0';

            			nprintf(&$$, "%n\n!\n\n!%s methodsFor: 'initializing'!\n\ninitVars\n%s\n!!",
            				$1, Current_procedure, buffer1);
        		    }
        		| proc_header
         		    {
            			PACTION($1, "", "", "", "<proc_header>", $$);
            			/* when header is complete, append an initializer method for all variables */
            			    printNode(buffer1,Current_initializer);
                            deleteNode(Current_initializer);
            			if ( strlen(buffer1) > 0 )
            			    buffer1[strlen(buffer1)-2] = '\0';
                        else
                            buffer1[0]= '\0';

                        nprintf(&$$, "%n\n!\n\n!%s methodsFor: 'initializing'!\n\ninitVars\n%s\n!!",
                            $1, Current_procedure, buffer1);
        		    } ;

proc_header	: proc_header header_line
    		    {
        			PACTION($1, $2, "", "", "<proc_header> <header_line>", $$);
        			nprintf(&$$, "%n!\n\n%n", $1, $2);		/* Concatenate header lines */
    		    }
    		| proc_header end_of_line
    		    {
        			PACTION($1, $2, "", "", "<proc_header> <end_of_line>", $$);
        			nprintf(&$$, "%n", $1);				/* Ignore the comment lines */
    		    }
    		| header_line
    		    {
        			PACTION($1, "", "", "", "<header_line>", $$);
        			nprintf(&$$, "%n", $1);
    		    } ;

header_line	: param_line
    		    {
        			PACTION($1, "", "", "", "<param_line>", $$);
        			nprintf(&$$, "\"parameters\n ----------\"\n\n%s\n\n%n", Current_procedure, $1);
    		    }
    		| qual_line
    		    {
        			PACTION($1, "", "", "", "<qual_line>", $$);
        			nprintf(&$$, "\"qualifiers\n ----------\"\n\n%s\n\n%n", Current_procedure, $1);
    		    }
    		| result_line
    		    {
        			PACTION($1, "", "", "", "<result_line>", $$);
        			nprintf(&$$, "\"result\n ------\"\n\n%s\n\n%n", Current_procedure, $1);
    		    } ;

proc_decl	: proc_decl decl_line
    		    {
        			PACTION($1, $2, "", "", "<proc_decl> <decl_line>", $$);
        			nprintf(&$$, "%n!\n\n%n", $1, $2);		/* Concatenate declaration lines */
    		    }
    		| proc_decl end_of_line
    		    {
        			PACTION($1, $2, "", "", "<proc_decl> <end_of_line>", $$);
        			nprintf(&$$, "%n", $1);				/* Ignore the comment lines */
    		    }
    		| decl_line
    		    {
        			PACTION($1, "", "", "", "<decl_line>", $$);
        			nprintf(&$$, "%n", $1);
    		    } ;

decl_line	: local_line
    		    {
        			PACTION($1, "", "", "", "<local_line>", $$);
        			nprintf(&$$, "\"Local variables\n ---------------\"\n\n%s\n\n%n", Current_procedure, $1);
              }
    		| global_line
    		    {
        			PACTION($1, "", "", "", "<global_line>", $$);
        			nprintf(&$$, "\"Global variables\n ----------------\"\n\n%s\n\n%n", Current_procedure, $1);
    		    } ;

block		: block command_line
    		    {
        			PACTION("*BLOCK*", $2, "", "", "<block> <command_line>", $$) ;
        			/* Build code into a method block */
        			nprintf(&$$, "%n.\n%n", $1, $2);
    		    }
    		| block end_of_line
    		    {
        			PACTION($1, $2, "", "", "<block> <end_of_line>", $$);
        			nprintf(&$$, "%n", $1);
    		    }
    		| command_line
    		    {
        			PACTION($1, "", "", "", "<command_line>", $$);
        			/* Add code for the command line to the method block for the class */
        			nprintf(&$$, "%n", $1);
    		    };

/*
    Syntax for the primary control constructs
    =========================================
*/

	/* LOOP */

loop_construct	: loop_line block endloop_line
        		    {
            			PACTION($1, $2, $3, "", "<loop_line> <block> <endloop_line>", $$);
                        assert(Valid(Loop_lvalue[Loop_depth]));
            			nprintf(&$$, "Function new loop: %n [ %n\n%n ] \ncondition: [false] after: [ %nnil ]",
            				$1, SSTOP(Loop_lvalue,Loop_depth), $2, SSTOP(Loop_lvalue,Loop_depth));
            			SSPOP(Loop_lvalue, Loop_depth)
                        assert(!Valid(Loop_lvalue[Loop_depth+1]));
        		    }
        		| loop_line block exit_control endloop_line
        		    {
            			PACTION($1, $2, $3, $4, "<loop_line> <block> <exit_control> <endloop_line>", $$);
                        assert(Valid(Loop_lvalue[Loop_depth]));
            			nprintf(&$$, "Function new loop: %n [ %n\n%n ] \n%n", $1, SSTOP(Loop_lvalue,Loop_depth), $2, $3);
            			SSPOP(Loop_lvalue, Loop_depth)
                        assert(!Valid(Loop_lvalue[Loop_depth+1]));
        		    }
        		| loop_line exit_control endloop_line
        		    {
            			PACTION($1, $2, $3, "", "<loop_line> <exit_control> <endloop_line>", $$);
                        assert(Valid(Loop_lvalue[Loop_depth]));
            			nprintf(&$$, "Function new loop: %n [ %nnil ] \n%n", $1, SSTOP(Loop_lvalue,Loop_depth), $2);
            			SSPOP(Loop_lvalue, Loop_depth)
                        assert(!Valid(Loop_lvalue[Loop_depth+1]));
        		    }
        		| loop_line endloop_line
        		    {
            			PACTION($1, $2, "", "", "<loop_line> <endloop_line>", $$);
            			nprintf(&$$, "('*** Empty loop executed (add a dummy statement to get an infinite loop)') prnl");
                        assert(Valid(Loop_lvalue[Loop_depth]));
            			SSPOP(Loop_lvalue, Loop_depth)
                        assert(!Valid(Loop_lvalue[Loop_depth+1]));
                        deleteNode($1);    /* Need to eat up $1 here (it may contain unsed count control) */
        		    } ;

exit_control	: exitif_line block
        		    {
            			PACTION($1, $2, "", "", "<exitif_line> <block>", $$);
            			nprintf(&$$, "condition: %n after: [ %n\n%n ]", $1, SSTOP(Loop_lvalue,Loop_depth), $2);
        		    }
        		| exitif_line
        		    {
            			PACTION($1, "", "", "", "<exitif_line>", $$);
            			nprintf(&$$, "condition: %n after: [ %nnil ]", $1, SSTOP(Loop_lvalue,Loop_depth));
        		    } ;

loop_line	: LOOP end_of_line
    		    {
        			PACTION($1, "", "", "", "LOOP", $$);
        			SSPUSH(Loop_lvalue, Loop_depth)
            	    nprintf(&Loop_lvalue[Loop_depth], "");  /* Need to make a null node here */
        			nprintf(&$$,"");                        /* This node will be eaten higher up ! */
    		    }
    		| LOOP { SSPUSH(Loop_lvalue, Loop_depth) } count_control end_of_line
    		    {
        			PACTION($1, $3, "", "", "LOOP <count_control> <end_of_line>", $$);
        			nprintf(&$$, "%n before:" , $3);
    		    }
    		| loop_line end_of_line
    		    {
        			PACTION($1, $2, "", "", "<loop_line> <end_of_line>", $$);
        			nprintf(&$$, "%n", $1);
    		    } ;

exitif_line	: EXITIF expression end_of_line
    		    {
        			PACTION($1, $2, "", "", "EXITIF <expression>", $$);
                    assert(Valid(Loop_lvalue[Loop_depth]));
        			nprintf(&$$, "[ %n%n ]", SSTOP(Loop_lvalue,Loop_depth), $2);
    		    }
    		| exitif_line end_of_line
    		    {
        			PACTION($1, $2, "", "", "<exitif_line> <end_of_line>", $$);
        			nprintf(&$$, "%n", $1);
    		    } ;

endloop_line    : ENDLOOP end_of_line
        		    {
            			PACTION($1, $2, "", "", "ENDLOOP <end_of_line>", $$);
            			$$ = NULL;
        		    } ;

count_control	: lvalue loop_range
        		    {
            			PACTION($1, $2, "", "", "<lvalue> <loop_range>", $$);
                        assert(!Valid(Loop_lvalue[Loop_depth]));
            			nprintf(&Loop_lvalue[Loop_depth], ":%n | ", $1);
            			nprintf(&$$, "%n by: 1", $2);
        		    }
        		| lvalue loop_range STEP expression
        		    {
            			PACTION($1, $2, $3, $4, "<lvalue> <loop_range> STEP <expression>", $$);
                        assert(!Valid(Loop_lvalue[Loop_depth]));
            			nprintf(&Loop_lvalue[Loop_depth], ":%n | ", $1);
            			nprintf(&$$, "%n by: %n", $2, $4);
        		    } ;


loop_range		: FROM expression TO expression
        		    {
            			PACTION($1, $2, $3, $4, "FROM <expression> TO <expression>", $$);
            			nprintf(&$$, "%n to: %n", $2, $4);
        		    } ;


	/* CASE */

case_construct	: case_line alternatives endcase_line
        		    {
            			PACTION($1, $2, $3, "", "<case_line> <alternatives> <endcase_line>", $$);
            			nprintf(&$$, "Function new %n body: [ :caseTempVar :break |\n%n ]", $1, $2);
            			SSPOP(Case_lvalue,Case_depth)
        		    }
        		| case_line alternatives otherwise_part endcase_line
        		    {
            			PACTION($1, $2, $3, $4, "<case_line> <alternatives> <otherwise_part> <endcase_line>", $$);
            			nprintf(&$$, "Function new %n body: [ :caseTempVar :break |\n%n.\n%n ]", $1, $2, $3);
            			SSPOP(Case_lvalue,Case_depth)
        		    }
        		| case_line otherwise_part endcase_line
        		    {
            			PACTION($1, $2, $3, "", "<case_line> <otherwise_part> <endcase_line>", $$);
            			nprintf(&$$, "Function new %n body: [ :caseTempVar :break |\n%n ]", $1, $2);
            			SSPOP(Case_lvalue,Case_depth)
        		    }
        		| case_line endcase_line
        		    {
            			PACTION($1, $2, "", "", "<case_line> <endcase_line>", $$);
            			nprintf(&$$, "nil \"(null case)\"");
            			SSPOP(Case_lvalue,Case_depth)
        		    } ;

alternatives	: alternatives test_clause
        		    {
            			PACTION($1, $2, "", "", "<alternatives> <test_clause>", $$);
            			nprintf(&$$, "%n.\n%n", $1, $2);
        		    }
        		| test_clause
        		    {
            			PACTION($1, "", "", "", "<test_clause>", $$);
            			nprintf(&$$, "%n", $1);
        		    } ;

otherwise_part	: otherwise_line block
        		    {
            			PACTION($1, $2, "", "", "<otherwise_line> <block>", $$);
            			nprintf(&$$, "%n", $2);
        		    }
        		| otherwise_line
        		    {
            			PACTION($1, "", "", "", "<otherwise_line>", $$);
                        $$ = NULL;
        		    } ;

test_clause	: is_line block
        		    {
            			PACTION($1, $2, "", "", "<is_line> <block>", $$);
            			nprintf(&$$, "%n [\n %n. break value ]", $1, $2);
        		    }
        		| is_line
        		    {
            			PACTION($1, "", "", "", "<is_line>", $$);
            			nprintf(&$$, "%n [\n nil. break value ]", $1);
        		    } ;

case_line	: CASE expression end_of_line
    		    {
        			PACTION($1, $2, $3, "", "CASE <expression> <end_of_line>", $$);
                    gprintdf("CASE depth = %d\n", Case_depth);
        			SSPUSH(Case_lvalue, Case_depth)
                    assert(!Valid(Case_lvalue[Case_depth]));
        			nprintf(&Case_lvalue[Case_depth], "caseTempVar");
        			nprintf(&$$, "case: %n", $2);
    		    }
    		| case_line end_of_line
    		    {
        			PACTION($1, $2, "", "", "<case_line> <end_of_line>", $$);
        			nprintf(&$$, "%n", $1);
    		    } ;


is_line		: IS case_range end_of_line
    		    {
        			PACTION($1, $2, $3, "", "IS <case_range> <end_of_line>", $$);
        			nprintf(&$$, "%n ifTrue:", $2);
    		    }
    		| is_line end_of_line
    		    {
        			PACTION($1, $2, "", "", "<is_line> <end_of_line>", $$);
        			nprintf(&$$, "%n", $1);
    		    } ;

case_range	: expression
    		    {
        			PACTION($1, "", "", "", "<expression>", $$);
        			nprintf(&$$, "(%n = %n)", SSTOP(Case_lvalue,Case_depth), $1);
    		    }
    		| expression TO expression
    		    {
        			PACTION($1, $2, $3, "", "<expression> TO <expression>", $$);
        			nprintf(&$$, "((%n <= %n) & (%n <= %n))",$1, SSTOP(Case_lvalue,Case_depth),
    				    SSTOP(Case_lvalue,Case_depth), $3 );
    		    } ;

otherwise_line	: OTHERWISE end_of_line
    		    {
        			PACTION($1, $2, "", "", "OTHERWISE <end_of_line>", $$);
        			nprintf(&$$, "");
    		    }
    		| otherwise_line end_of_line
    		    {
        			PACTION($1, $2, "", "", "<otherwise_line> <end_of_line>", $$);
        			nprintf(&$$, "");
    		    } ;

endcase_line	: ENDCASE end_of_line
        		    {
            			PACTION($1, $2, "", "", "ENDCASE <end_of_line>", $$);
            			nprintf(&$$, "");
        		    } ;


	/* IF */

if_construct    : IF elif_part endif_line
        		    {
            			PACTION($1, $2, $3, "", "IF <elif_part> <endif_line>", $$);
                        nprintf(&$$, "%n%n", LHS($2), RHS($2) );
        		    }
        		| IF elif_part else_part endif_line
        		    {
            			PACTION($1, $2, $3, $4, "IF <elif_part> <else_part> <endif_line>", $$);
            			nprintf(&$$, "%n %n %n", LHS($2), $3, RHS($2) );
        		    } ;

elif_part	: elif_part ELSEIF cond_block
		        {
			        PACTION($1, $2, $3, "", "<elif_part> ELSEIF <cond_block>", $$);
			        nprintf(&LHS($$), "%n ifFalse: [ %n ]", LHS($1), LHS($3) );
			        nprintf(&RHS($$), "%n%n", RHS($3), RHS($1) );
		        }
    		| cond_block
		        {
			        PACTION($1, "", "", "", "<cond_block>", $$);
			        nprintf(&LHS($$), "%n %n", LHS($1), RHS($1) );
			        nprintf(&RHS($$),"");
		        } ;

else_part	: else_line block
		    {
			PACTION($1, $2, "", "", "<else_line> <block>", $$);
			nprintf(&$$, "ifFalse: [ \n%n ]", $2);
			If_depth--;
		    }
		| else_line
		    {
			PACTION($1, "", "", "", "<else_line>", $$);
			nprintf(&$$, "");
			If_depth--;
		    } ;

cond_block	: expr_line block
    		    {
        			PACTION($1, $2, "", "", "<expr_line> <block>", $$);
                    nprintf(&LHS($$), "%n ifTrue: [ \n%n", $1, $2);
                    nprintf(&RHS($$), "]");
        			If_depth--;
    		    }
    		| expr_line
    		    {
        			PACTION($1, "", "", "", "<expr_line>", $$);
        			nprintf(&LHS($$), "%n", $1);
        			nprintf(&RHS($$),"");
        			If_depth--;
    		    } ;

expr_line	: expression { If_depth++; } end_of_line
    		    {
        			PACTION($1, $3, "", "", "<expression> <end_of_line>", $$);
        			nprintf(&$$, "%n", $1);
    		    }
    		| expr_line end_of_line
    		    {
        			PACTION($1, $2, "", "", "<expr_line> <end_of_line>", $$);
        			nprintf(&$$, "%n", $1);
    		    } ;

else_line	: ELSE { If_depth++; } end_of_line
    		    {
        			PACTION($1, $3, "", "", "ELSE <end_of_line>", $$);
        			nprintf(&$$, "");
    		    }
    		| else_line end_of_line
    		    {
        			PACTION($1, $2, "", "", "<else_line> <end_of_line>", $$);
        			nprintf(&$$, "");
    		    } ;

endif_line	: ENDIF end_of_line
    		    {
        			PACTION($1, $2, "", "", "ENDIF <end_of_line>", $$);
                    $$ = NULL;
    		    } ;

	/* Pass through code block for doing deep dirty internal things */

code_block	: code ENDCODE
    		    {
        			PACTION($1, $2, "", "", "<code> ENDCODE", $$);
                    in_smalltalk = 0;
        			nprintf(&$$, "%n%s", $1, $2); 
    		    }
	        | ENDCODE
    		    {
        			PACTION($1, "", "", "", "ENDCODE", $$);
                    in_smalltalk = 0;
        			nprintf(&$$, "%s",$1); 
    		    } ;

code		: CODELINE
    		    {
        			lines_parsed++;
        			PACTION($1, "", "", "", "CODELINE", $$);
                    in_smalltalk = 1;
        			nprintf(&$$, "%s\n", $1); 
    		    }
    		| code CODELINE
    		    {
        			lines_parsed++;
        			PACTION($1, $2, "", "", "code CODELINE", $$);
        			nprintf(&$$, "%n%s\n", $1, $2); 
    		    } ;

/*
    Token syntax for line parsed parts of the control parser
    ========================================================
    These all go one line at a time and for this reason, these are
    counted to get an idea of line number statistics.
*/

proc_line	: PROCEDURE IDENTIFIER end_of_line
    		    {
        			PACTION($1, $2, $3, "", "PROCEDURE IDENTIFIER <end_of_line>", $$);
        			strcpy(Current_procedure,$2);
				if (g_type_file != 0)
				    fprintf(g_type_file, "H%s\n", Current_procedure);
        			MangleProc(Current_procedure);

        			/* Create a subclass of Function under an appropriate identifier */
        			PROCEDURE_CLASS(&$$, Current_procedure)
    		    }
    		| proc_line end_of_line
    		    {
        			PACTION($1, $2, "", "", "<proc_line> <end_of_line>", $$);
        			nprintf(&$$, "%n", $1);		/* eat comments */
    		    } ;

end_proc_line	: ENDPROCEDURE end_of_line
        		    {
            			PACTION($1, $2, "", "", "ENDPROCEDURE <end_of_line>", $$);
            			nprintf(&$$, "\"End of procedure\"");
				if (g_type_file != 0)
				    fprintf(g_type_file, "E\n");
        		    } ;

param_line	: PARAMETERS fp_defs end_of_line
    		    {
        			PACTION($1, $2, $3, "", "PARAMETERS <fp_defs> <end_of_line>", $$);
        			nprintf(&$$, "%n", $2);
    		    } ;

qual_line	: QUALIFIERS qual_defs end_of_line
    		    {
        			PACTION($1, $2, $3, "", "QUALIFIERS <qual_defs> <end_of_line>", $$);
        			nprintf(&$$, "%n", $2);
    		    } ;

result_line	: RESULT { Current_line_type = Result; } variable_declarator end_of_line
    		    {
        			PACTION($1, $3, $4, "", "RESULT <variable_declarator> <end_of_line>", $$);
        			nprintf(&$$, "%n", $3);
    		    } ;

local_line	: LOCAL { Current_line_type = Local; } var_defs end_of_line
    		    {
        			PACTION($1, $3, $4, "", "LOCAL <var_defs> <end_of_line>", $$);
        			nprintf(&$$, "%n", $3);
    		    } ;

global_line	: GLOBAL { Current_line_type = Global; } var_defs end_of_line
    		    {
        			PACTION($1, $3, $4, "", "GLOBAL <var_defs> <end_of_line>", $$);
        			nprintf(&$$, "%n", $3);
    		    } ;

command_line	: error	end_of_line				/* Handle error recovery on a command line, recover at EOL */
        		    {
            			PACTION($1, $2, "", "", "<error> <end_of_line>", $$);
            			nprintf(&$$, "nil \"(line with syntax error)\"");
                        /* Note: there is a node leak here as we cannot tell what has been half created */
                        yyerrok;
        		    }
        		| assignment end_of_line
        		    {
            			PACTION($1, $2, "", "", "<assignment> <end_of_line>", $$);
             			nprintf(&$$, "%n", $1 );
        		    }
        		| keyword end_of_line
        		    {
            			PACTION($1, $2, "", "", "<keyword> <end_of_line>", $$);
             			nprintf(&$$, "%n", $1 );
        		    }
        		| '[' expression ']' end_of_line
        		    {
            			PACTION($1, $2, $3, $4, "[ <expression> ] <end_of_line>", $$);
             			nprintf(&$$, "%n", $2 );
        		    }
    	        	| loop_construct
        		    {
            			PACTION($1, "", "", "", "<loop_construct>", $$);
        		        nprintf(&$$, "%n", $1 );
        		    }
        		| case_construct
        		    {
            			PACTION($1, "", "", "", "<loop_construct>", $$);
        		        nprintf(&$$, "%n", $1 );
        		    }
        		| if_construct
        		    {
            			PACTION($1, "", "", "", "<if_construct>", $$);
            			nprintf(&$$, "%n", $1 );
        		    }
    	        	| code_block
        		    {
            			PACTION($1, "", "", "", "<code_block>", $$);
            			nprintf(&$$, "%n", $1 );
        		    }
        		| FORWARD IDENTIFIER end_of_line
        		    {
        			PACTION($1, $2, $3, "", "FORWARD IDENTIFIER <end_of_line>", $$);
        			strcpy(buffer1,$2);
        			MangleProc(buffer1);
        		        nprintf(&$$, "((Smalltalk at: #%s ifAbsent: [Smalltalk at: #%s put: \
				  'Procedure \"%s\" has only been FORWARD declared, it must also have a PROCEDURE definition' ] ) \
				  isKindOf: Function class) ifTrue: \
				[ 'Warning: FORWARD statement for a previously defined procedure \"%s\" - no action taken' prinl ]",\
				   buffer1, buffer1, $2, $2);
        		    } ;
        		| RETURN end_of_line
        		    {
        		        PACTION($1, "", "", "", "RETURN <end_of_line>", $$);
        		        nprintf(&$$, "[nil%s] value", Current_result);
        		    } ;


/*
    All parsing carried out on one line ie the line parser
    ======================================================
    Anything below the level of one line comes here.
*/

end_of_line	: ENDOFLINE
    		    {
        			lines_parsed++;
        			PACTION($1, "", "", "", "ENDOFLINE", $$);
        			$$ = NULL;		    /* Allow simple blank lines, (nothing to pass back) */
    		    }
    		| COMMENT ENDOFLINE
    		    {
        			lines_parsed++;
        			PACTION($1, $2, "", "", "COMMENT ENDOFLINE", $$);
        			$$ = NULL;     /* Just parse comments, don't make use of them */
    		    } ;


/*
 * record variables right here in parsing of the expression
 * NOTE: need info from lvalue parsing as they will be overwritten by any
 * lvalues parsed later in <expression>
 */
assignment	: lvalue { keep_structure_flag = structure_flag; strcpy(keep_structure_name, structure_name); } '=' expression
    		    {
        			PACTION($1, $3, $4, "", "<lvalue> = <expression>", $$);

	                auto_declare = prompt_mode();

        			if (keep_structure_flag)		/* Element assign for structural lvalues */

        			    if (auto_declare)
                            {ASSIGN_STRUCT_I(&$$, keep_structure_name, $1, $4, "deepCopy");}
        			    else
                            {ASSIGN_STRUCT(&$$, keep_structure_name, $1, $4, "deepCopy");}

        			else					/* destructive copy for all other types */

        			    if (auto_declare)
                            {ASSIGN_ATOMIC_I(&$$, $1, $4, "deepCopy");}
        			    else
                            {ASSIGN_ATOMIC(&$$, $1, $4, "deepCopy");}
    		    }
            | lvalue { keep_structure_flag = structure_flag; strcpy(keep_structure_name, structure_name); } ALIAS expression
    		    {
        			PACTION($1, $3, $4, "", "<lvalue> ALIAS <expression>", $$);

	                auto_declare = prompt_mode();

        			if (keep_structure_flag)		/* Element assign for structural lvalues */

        			    if (auto_declare)
                            {ASSIGN_STRUCT_I(&$$, keep_structure_name, $1, $4, "");}
        			    else
                            {ASSIGN_STRUCT(&$$, keep_structure_name, $1, $4, "");}

        			else					/* destructive copy for all other types */

        			    if (auto_declare)
                            {ASSIGN_ATOMIC_I(&$$, $1, $4, "");}
        			    else
                            {ASSIGN_ATOMIC(&$$, $1, $4, "");}
    		    } ;

expression	: expression '>' expression			/* General expression syntax, widely used for both syntaxes */
    		    {
        			PACTION($1, $2, $3, "", "<expression> > <expression>", $$);
        			nprintf(&$$, "(%n > %n)", $1, $3);
    		    }
    		| expression '<' expression
    		    {
        			PACTION($1, $2, $3, "", "<expression> < <expression>", $$);
        			nprintf(&$$, "(%n < %n)", $1, $3);
    		    }
    		| expression '=' expression
    		    {
        			PACTION($1, $2, $3, "", "<expression> = <expression>", $$);
        			nprintf(&$$, "(%n = %n)", $1, $3);
    		    }
    		| expression GEQUAL expression
    		    {
        			PACTION($1, $2, $3, "", "<expression> GEQUAL <expression>", $$);
        			nprintf(&$$, "(%n >= %n)", $1, $3);
    		    }
    		| expression LEQUAL expression
    		    {
        			PACTION($1, $2, $3, "", "<expression> LEQUAL <expression>", $$);
        			nprintf(&$$, "(%n <= %n)", $1, $3);
    		    }
    		| expression NOTEQUALS expression
    		    {
        			PACTION($1, $2, $3, "", "<expression> NOTEQUALS <expression>", $$);
        			nprintf(&$$, "(%n ~= %n)", $1, $3);
    		    }
    		| expression '&' expression
    		    {
        			PACTION($1, $2, $3, "", "<expression> & <expression>", $$);
        			nprintf(&$$, "(%n , %n)", $1, $3);
    		    }
    		| expression OR expression
    		    {
        			PACTION($1, $2, $3, "", "<expression> OR <expression>", $$);
        			nprintf(&$$, "(%n | %n)", $1, $3);
    		    }
    		| expression '+' expression
    		    {
        			PACTION($1, $2, $3, "", "<expression> + <expression>", $$);
        			nprintf(&$$, "(%n + %n)", $1, $3);
    		    }
    		| expression '-' expression
    		    {
        			PACTION($1, $2, $3, "", "<expression> - <expression>", $$);
        			nprintf(&$$, "(%n - %n)", $1, $3);
    		    }
    		| expression AND expression
    		    {
        			PACTION($1, $2, $3, "", "<expression> AND <expression>", $$);
        			nprintf(&$$, "(%n & %n)", $1, $3);
    		    }
    		| expression '*' expression
    		    {
        			PACTION($1, $2, $3, "", "<expression> * <expression>", $$);
        			nprintf(&$$, "(%n * %n)", $1, $3);
    		    }
    		| expression '/' expression
    		    {
        			PACTION($1, $2, $3, "", "<expression> / <expression>", $$);
        			nprintf(&$$, "(%n / %n)", $1, $3);
    		    }
    		| expression '|' expression
    		    {
        			PACTION($1, $2, $3, "", "<expression> | <expression>", $$);
        			nprintf(&$$, "(%n rem: %n)", $1, $3);
    		    }
    		| expression '^' expression
    		    {
        			PACTION($1, $2, $3, "", "<expression> | <expression>", $$);
        			nprintf(&$$, "(%n raisedTo: %n)", $1, $3);
    		    }
    		| expression ':' expression
    		    {
        			PACTION($1, $2, $3, "", "<expression> ':' <expression>", $$);
        			nprintf(&$$, "(%n to: %n)", $1, $3);
    		    }
    		| expression '@' expression
    		    {
        			PACTION($1, $2, $3, "", "<expression> '@' <expression>", $$);
        			nprintf(&$$, "(%n by: %n)", $1, $3);
    		    }
    		| '(' expression ')'
    		    {
        			PACTION($1, $2, $3, "", "( <expression> )", $$);
        			nprintf(&$$, "(%n)", $2);
    		    }
    		| function_call
    		    {
        			PACTION($1, "", "", "", "<function_call>", $$);
        			nprintf(&$$, "(%n)", $1);
    		    }
    		| NOT expression
    		    {
        			PACTION($1, $2, "", "", "NOT <expression>", $$);
        			nprintf(&$$, "%n not", $2);
    		    }
    		| UPLUS expression
    		    {
        			PACTION($1, $2, "", "", "UPLUS <expression>", $$);
        			nprintf(&$$, "%n", $2);
    		    }
    		| UMINUS expression
    		    {
        			PACTION($1, $2, "", "", "UMINUS <expression>", $$);
        			nprintf(&$$, "%n negated", $2);
    		    }
    		| '(' expression ')' struct_spec            /* Direct access to members of a structural temporary */
    		    {
        			PACTION($1, $2, $3, $4, "( <expression> ) <struct_spec>", $$);
        			nprintf(&$$,"(%n%n%n)", LHS($4), $2, RHS($4));
        			structure_flag = 0;                 /* FALSE */
        			printNode(structure_name, $2);      /* Need to keep a copy in ASCII for comparison */
    		    }
    		| UINTEGER
    		    {
        			PACTION($1, "", "", "", "UINTEGER", $$);
        			nprintf(&$$, "%d", (long)$1);
    		    }
    		| UREAL
    		    {
        			PACTION($1, "", "", "", "UREAL", $$);
        			sprintf(buffer1, "%.15e", $1);
        			ReplacePlusSigns(buffer1);	/* Smalltalk doesn't like '+' signs. eg. change +5e+20 to 05e020 */
        			nprintf(&$$, "%s", buffer1);
    		    }
    		| STRING
    		    {
        			PACTION($1, "", "", "", "STRING", $$);
        			nprintf(&$$, "'%s'", $1);
    		    }
    		| lvalue                                    /* Any legal l-value specifier returns its r-value here*/
    		    {
        			PACTION($1, "", "", "", "<lvalue>", $$);
        			nprintf(&$$, "(%n)", $1);
    		    } ;



function_call	: IDENTIFIER '(' ')'				/* Full function call syntax */
    		    {
        			PACTION($1, $2, $3, "", "IDENTIFIER ( )", $$);
        			strcpy(buffer1,$1);		    /* Build smalltalk command */
        			MangleProc(buffer1);
        			nprintf(&$$, "%s run line: 0 proc: self method: true; condEvaluate",buffer1 /*, lines_parsed+1*/);
    		    }
    		| IDENTIFIER qual_list1 '(' ')'
    		    {
        			PACTION($1, $2, $3, $4, "IDENTIFIER <qual_list1> ( )", $$);
        			strcpy(buffer1,$1);		    /* Build smalltalk command */
        			MangleProc(buffer1);
        			nprintf(&$$, "%s run line: 0 proc: self method: true; %n; condEvaluate",buffer1 /*, lines_parsed+1*/, $2);
    		    }
    		| IDENTIFIER arguments
    		    {
        			PACTION($1, $2, "", "", "IDENTIFIER <arguments>", $$);
        			strcpy(buffer1,$1);		    /* Build smalltalk command */
        			MangleProc(buffer1);
        			nprintf(&$$, "%s run line: 0 proc: self method: true; %n; condEvaluate",buffer1 /*, lines_parsed+1 */, $2);
    		    }
    		| IDENTIFIER qual_list1 arguments
    		    {
        			PACTION($1, $2, $3, "", "IDENTIFIER <qual_list1> <arguments>", $$);
        			strcpy(buffer1,$1);		    /* Build smalltalk command */
        			MangleProc(buffer1);
        			nprintf(&$$, "%s run line: 0 proc: self method: true; %n; %n; condEvaluate",buffer1 /*, lines_parsed+1 */, $2, $3);
    		    } ;

qual_list1	: ':' IDENTIFIER				/* For qualifiers in the a=f:q1:q2(w1,20.0) style syntax */
    		    {
        			PACTION($1, $2, "", "", ": IDENTIFIER", $$);
        			strcpy(buffer1,$2);
        			MangleVar(buffer1);
        			nprintf(&$$, "%s", buffer1 );
/*                    gprintdf("\033[34m\033[1mObsolete qualifier syntax on line %d, please change expressions like\n%s%s%s\033[0m",
                        lines_parsed+1,
                        "    \"function:qual1:qual2(x,y,z)\"\n",
                        "to use \"%\" instead of \":\" to separate the qualifiers, eg\n",
                        "    \"function%qual1%qual2(x,y,z)\"\n");
*/    		    }
/*            | '%' IDENTIFIER				// For qualifiers in the a=f:q1:q2(w1,20.0) style syntax
    		    {
        			PACTION($1, $2, "", "", "% IDENTIFIER", $$);
        			strcpy(buffer1,$2);
        			MangleVar(buffer1);
        			nprintf(&$$, "%s", buffer1 );
    		    }
*/    		| qual_list1 ':' IDENTIFIER
    		    {
        			PACTION($1, $2, $3, "", "<qual_list1> : IDENTIFIER", $$);
        			strcpy(buffer1,$3);
        			MangleVar(buffer1);
        			nprintf(&$$, "%n; %s", $1, buffer1);
    		    } ;

qual_list2	: '/' IDENTIFIER				/* For qualifiers in the F/Q1/Q2 W1 20.0 style syntax */
    		    {
        			PACTION($1, $2, "", "", "/ IDENTIFIER", $$);
        			strcpy(buffer1,$2);
        			MangleVar(buffer1);
        			nprintf(&$$, "%s", buffer1 );
    		    }
    		| qual_list2 '/' IDENTIFIER
    		    {
        			PACTION($1, $2, $3, "", "<qual_list2> / IDENTIFIER", $$);
        			strcpy(buffer1,$3);
        			MangleVar(buffer1);
        			nprintf(&$$, "%n; %s", $1, buffer1);
    		    } ;

qual_defs	: '/' IDENTIFIER				/* For qualifiers definitions in QUALIFIERS line */
    		    {						/* Syntactically the same as <qual_list2>, different semantics */
        			PACTION($1, $2, "", "", "/ IDENTIFIER", $$);
        			strcpy(buffer1,$2);
				if (g_type_file != 0)
				    fprintf(g_type_file, "Q%s\n", buffer1);
        			MangleVar(buffer1);
        			nprintf(&$$, "addInstVarName: #%s;\ncompile:\n'%s\n\t%s _ true';\ncompile:\n'%s: aBoolean\n\t%s _ aBoolean'", buffer1, buffer1, buffer1, buffer1, buffer1);

        			/* add a line to the initializer for this procedure (need to ensure qualifier is false by default) */
                    if (!Valid(Current_initializer)) Current_initializer = newLeaf("");
                    nprintf(&tmp1_tree, "%n\t%s _ false.\n",Current_initializer, buffer1);
                    nprintf(&Current_initializer, "%n",tmp1_tree);
    		    }
    		| qual_defs '/' IDENTIFIER
    		    {
        			PACTION($1, $2, $3, "", "<qual_list2> / IDENTIFIER", $$);
        			strcpy(buffer1,$3);
				if (g_type_file != 0)
				    fprintf(g_type_file, "Q%s\n", buffer1);
        			MangleVar(buffer1);
        			nprintf(&$$, "%n;\n\naddInstVarName: #%s;\ncompile:\n'%s\n\t%s _ true';\ncompile:\n'%s: aBoolean\n\t%s _ aBoolean'", $1, buffer1, buffer1, buffer1, buffer1, buffer1);

        			/* add a line to the initializer for this procedure (need to ensure qualifier is false by default) */
                    if (!Valid(Current_initializer)) Current_initializer = newLeaf("");
        			nprintf(&tmp1_tree, "%n\t%s _ false.\n", Current_initializer, buffer1);
        			nprintf(&Current_initializer, "%n",tmp1_tree);
    		    } ;

arguments	: '(' { PPUSH(Param_no,Function_depth) } arg_list { PPOP(Param_no,Function_depth) } ')'	/* Arg list for the function syntax f(x) */
    		    {
        			PACTION($1, $3, $5, "", "( <arg_list> )", $$);
        			nprintf(&$$, "%n", $3 );
    		    } ;

arg_list	: function_parameter				/* Comma separated argument list for function style syntax */
    		    {
        			PACTION($1, "", "", "", "<function_parameter>", $$);
        			nprintf(&$$, "%n", $1);
    		    }
    		| arg_list ',' function_parameter
    		    {
        			PACTION($1, $2, $3, "", "<arg_list> , <function_parameter>", $$);
        			nprintf(&$$, "%n; %n", $1, $3);
    		    } ;

keyword		: IDENTIFIER					/* Keyword syntax for procedure invocation */
    		    {
        			PACTION($1, "", "", "", "IDENTIFIER", $$);
        			strcpy(buffer1,$1);		    /* Build smalltalk command */
        			MangleProc(buffer1);
        			nprintf(&$$, "%s run line: 0 proc: self method: false; condEvaluate", buffer1 /*, lines_parsed+1 */);
    		    }
    		| IDENTIFIER qual_list2
    		    {
        			PACTION($1, $2, "", "", "IDENTIFIER <qual_list2>", $$);
        			strcpy(buffer1,$1);		    /* Build smalltalk command */
        			MangleProc(buffer1);
        			nprintf(&$$, "%s run line: 0 proc: self method: false; %n; condEvaluate", buffer1 /*, lines_parsed+1 */, $2);
    		    }
    		| IDENTIFIER { PPUSH(Param_no,Function_depth) } fp_list	/* Save default param name count before doing fp_list */
    		    {
        			PACTION($1, $3, "", "", "IDENTIFIER <fp_list>", $$);
        			strcpy(buffer1,$1);		    /* Build smalltalk command */
        			MangleProc(buffer1);
        			nprintf(&$$, "%s run line: 0 proc: self method: false; %n; condEvaluate", buffer1 /*, lines_parsed+1 */, $3);
        			PPOP(Param_no,Function_depth)				/* restore default param name count */
    		    }
    		| IDENTIFIER qual_list2 { PPUSH(Param_no,Function_depth) } fp_list
    		    {
        			PACTION($1, $2, $4, "", "IDENTIFIER <qual_list2> <fp_list>", $$);
        			strcpy(buffer1,$1);		    /* Build smalltalk command */
        			MangleProc(buffer1);
        			nprintf(&$$, "%s run line: 0 proc: self method: false; %n; %n; condEvaluate",buffer1 /*, lines_parsed+1 */, $2, $4);
        			PPOP(Param_no,Function_depth)
    		    } ;

		    
fp_list		: function_parameter				/* Space separated list of keyworded function parameters */
    		    {
        			PACTION($1, "", "", "", "<function_parameter>", $$);
        			nprintf(&$$, "%n", $1);
    		    }
    		| fp_list function_parameter
    		    {
        			PACTION($1, $2, "", "", "<fp_list> <function_parameter>", $$);
        			nprintf(&$$, "%n; %n", $1, $2);
    		    } ;


function_parameter	: IDENTIFIER '=' expression		/* Optionally keyworded function parameters for both syntaxes */
        			    {
            				PACTION($1, $2, $3, "", "IDENTIFIER = <expression>", $$);
            				strcpy(buffer1,$1);
            				MangleVar(buffer1);
            				nprintf(&$$, "%s:%n",buffer1, $3);
        			    }
        			| expression
        			    {
            				PACTION($1, "", "", "", "<expression>", $$);
        			        nprintf(&$$, "p%d:%n", (long)PTOP(Param_no,Function_depth)++, $1);
        			    } ;


fp_defs		: type_specifier				/* Space separated list of parameter names and types */
    		    {
        			PACTION($1, "", "", "", "<type_specifier>", $$);
        			nprintf(&$$, "%n", $1);
    		    }
    		| fp_defs type_specifier
    		    {
        			PACTION($1, $2, "", "", "<fp_defs> <type_specifier>", $$);
        			nprintf(&$$, "%n;\n\n%n", $1, $2);
    		    } ;

type_specifier	: IDENTIFIER '=' IDENTIFIER			/* single Keyword or Keyword/type pairs */
        		    {
            			PACTION($1, $2, $3, "", "IDENTIFIER = IDENTIFIER", $$);
            			strcpy(buffer1,$1);
               			strcpy(buffer2,$3);
				if (g_type_file != 0)
				    fprintf(g_type_file, "P%s=%s\n", buffer1, buffer2);
            			MangleVar(buffer1);
                        	MangleType(buffer2);

            			nprintf(&$$, "%s%s%s%s%s%s%s%s%s%s%s%s%s%d%s%s%s%s%s%s%s%s",
            				"addInstVarName: #", buffer1,
            				";\ncompile:\n'", buffer1, ": aVar\n",
        					"\taVar notNil ifTrue:[", buffer1, " _ self typeCheck: aVar shouldBe: ", buffer2,
    						" paramName: ''", buffer1, "'']';",
            				"\ncompile:\n'p", (long)PTOP(Param_no,Function_depth)++, ": aVar\n",
        					"\taVar notNil ifTrue:[", buffer1, " _ self typeCheck: aVar shouldBe: ", buffer2,
    						" paramName: ''", buffer1, "'']'" );
        		    }
        		| IDENTIFIER
        		    {
            			PACTION($1, "", "", "", "IDENTIFIER", $$);
            			strcpy(buffer1,$1);
				if (g_type_file != 0)
				    fprintf(g_type_file, "P%s\n", buffer1);
            			MangleVar(buffer1);

            			/* create instance variable */
        		        nprintf(&tmp1_tree, "addInstVarName: #%s;\n",buffer1);

            			/* create keyword entry (any type) */
            			nprintf(&tmp2_tree, "compile:\n'%s: aVar\n\taVar notNil ifTrue:[ %s _ aVar ]';\n", buffer1, buffer1);

            			/* create default entry (any type) */
            			nprintf(&$$, "%n%ncompile:\n'p%d: aVar\n\taVar notNil ifTrue:[ %s _ aVar ]'",
            				tmp1_tree,tmp2_tree, (long)PTOP(Param_no,Function_depth)++, buffer1);
        		    } ;

var_defs    : variable_declarator				/* Space separated list of variable names and types */
    		    {
        			PACTION($1, "", "", "", "<variable_declarator>", $$);
        			nprintf(&$$, "%n", $1);
    		    }
    		| var_defs variable_declarator
    		    {
        			PACTION($1, $2, "", "", "<var_defs> <variable_declarator>", $$);
        			nprintf(&$$, "%n;\n\n%n", $1, $2);
    		    } ;

variable_declarator	: IDENTIFIER '=' expression		/* Optionally initialized variable name declarations */
        			    {
            				PACTION($1, $2, $3, "", "IDENTIFIER = <expression>", $$);
            				strcpy(buffer1,$1);
            				if ( Current_line_type == Local || Current_line_type == Result ) {
            					MangleVar(buffer1);

            					/* create instance variable */
            					nprintf(&$$, "addInstVarName: #%s",buffer1);

            					/* add a line to the initializer for this procedure */
                                if (!Valid(Current_initializer)) Current_initializer = newLeaf("");
                                nprintf(&tmp1_tree, "%n\t%s _ %n.\n",Current_initializer, buffer1, $3);
            					nprintf(&Current_initializer, "%n",tmp1_tree); /* Don't try to do this in one step, it won't work! */
            					if ( Current_line_type == Result )
            						sprintf(Current_result,".\nself up.\n^%s", buffer1); }
            				else {
            					MangleVar(buffer1);
            					/* create global variable if not already created by another procedure */
            					/* Initialize the variable only if it is currently set to nil */

            					nprintf(&$$, "addPoolVar: #%s value: %n", buffer1, $3); };
        			    }
        			| IDENTIFIER
        			    {
            				PACTION($1, "", "", "", "IDENTIFIER", $$);
            				strcpy(buffer1, $1);
            				if ( Current_line_type == Local  || Current_line_type == Result ) {
            					MangleVar(buffer1);

            					/* create instance variable */
            					nprintf(&$$, "addInstVarName: #%s",buffer1);
            					if ( Current_line_type == Result )
            						sprintf(Current_result,".\nself up.\n^%s", buffer1); }
            				else {
            					MangleVar(buffer1);
            					/* create global variable if not already created by another procedure */
            					/* Initialize the variable only if it is currently set to nil */

            					nprintf(&$$, "addPoolVar: #%s", buffer1); } ;
        			    } ;

/*
    L-values which include variable specifications, array element references and workspace field references
    =======================================================================================================
*/

lvalue		: IDENTIFIER { method_pass_case = 0; } struct_spec
    		    {
        			PACTION($1, $3, "", "",  "IDENTIFIER <struct_spec>", $$);

        			strcpy(buffer1, $1);
                    if (isupper(buffer1[0]) && method_pass_case)
        			    MangleType(buffer1);
                    else
                        MangleVar(buffer1);

        			nprintf(&$$,"%n%s%n",LHS($3), buffer1, RHS($3));
        			structure_flag = 1; /* TRUE */
        			strcpy(structure_name, buffer1);
    		    }
    		| IDENTIFIER
    		    {
        			PACTION($1, "", "", "", "IDENTIFIER", $$);

        			strcpy(buffer1, $1);
        			MangleVar(buffer1);
        			nprintf(&$$,"%s",buffer1);
        			structure_flag = 0; /* FALSE */
    		    } ;

struct_spec	: index_ref
    		    {
        			PACTION($1, "", "", "", "<index_ref>", $$);
        			/* $$ = $1 */
    		    }
    		| struct_spec index_ref
    		    {
        			PACTION($1, $2, "", "", "<struct_spec> <index_ref>", $$);

        			nprintf(&LHS($$), "%n%n", LHS($2),LHS($1));
        			nprintf(&RHS($$), "%n%n", RHS($1),RHS($2));
    		    }
        	| method_ref
    		    {
        			PACTION($1, "", "", "", "<method_ref>", $$);
        			/* $$ = $1 */
    		    }
    		| struct_spec method_ref
    		    {
        			PACTION($1, $2, "", "", "<struct_spec> <method_ref>", $$);

        			nprintf(&LHS($$), "%n%n", LHS($2),LHS($1));
        			nprintf(&RHS($$), "%n%n", RHS($1),RHS($2));
    		    } ;

method_args : '(' m_arg_list ')'	/* Arg list for the function syntax f(x) */
    		    {
        			PACTION($1, $2, $3, "", "( <m_arg_list> )", $$);
        			nprintf(&$$, "%n", $2 );
    		    }
            | '(' ')'
       		    {			
       		        PACTION($1, $2, "", "", "( )", $$);
        			nprintf(&$$, "");
    		    } ;

m_arg_list	: method_parameter				/* Comma separated argument list for function style syntax */
    		    {
        			PACTION($1, "", "", "", "<method_parameter>", $$);
        			nprintf(&$$, "%n", $1);
    		    }
    		| m_arg_list ',' method_parameter
    		    {
        			PACTION($1, $2, $3, "", "<m_arg_list> , <method_parameter>", $$);
        			nprintf(&$$, "%n %n", $1, $3);
    		    } ;

method_parameter    : IDENTIFIER '=' expression		/* Optionally keyworded function parameters for both syntaxes */
        	            {
                			PACTION($1, $2, $3, "", "IDENTIFIER = <expression>", $$);
                			strcpy(buffer1,$1);
                			nprintf(&$$, "%s:%n",buffer1, $3);
            		    }
            		| expression
            		    {
                			PACTION($1, "", "", "", "<expression>", $$);
                			nprintf(&$$, ":%n", $1);
            		    } ;


method_ref	: '.' method_args
    		    {
        			PACTION($1, $2, "", "", ". <method_args>", $$);

        			method_pass_case = 1;
        			nprintf(&LHS($$), "(" );
        			nprintf(&RHS($$), ") %n", $2);
    		    }
        	| '.' IDENTIFIER method_args
    		    {
        			PACTION($1, $2, $3, "", ". IDENTIFIER <method_args>", $$);

        			method_pass_case = 1;
        			strcpy(buffer1, $2);
        			nprintf(&LHS($$), "(" );
        			nprintf(&RHS($$), ") %s%n", buffer1, $3);
    		    }
            | '.' IDENTIFIER
    		    {
        			PACTION($1, $2, "", "", ". IDENTIFIER", $$);

        			strcpy(buffer1, $2);
        			MangleVar(buffer1);
        			nprintf(&LHS($$), "(" );
        			nprintf(&RHS($$), ") at:#%s", buffer1);
    		    } ;

index_ref	: '[' { PPUSH(Array_index_no,Array_index_depth) } array_index ']'
    		    {
        			PACTION($1, $3, $4, "", "[ <array_index> ]", $$);
        			nprintf(&LHS($$), "%n", LHS($3));
        			nprintf(&tmp1_tree, "%n; yourself)", RHS($3));
                    printNode(buffer1,tmp1_tree);
                    deleteNode(tmp1_tree);                         /* Remove this branch now it is not being used */

                    /* See note below in "array_index" */
        			nprintf(&RHS($$), buffer1, PTOP(Array_index_no,Array_index_depth)-1);
        			PPOP(Array_index_no, Array_index_depth);
    		    } ;


array_index	: expression
    		    {
        			PACTION($1, "", "", "", "<expression>", $$);
        			nprintf(&LHS($$), "(" );

                    /*                                   | 
                     * Note: The the %%d is OK here      V      ( it is instantiated above in "index_ref" with the value PTOP )
                     */
        			nprintf(&RHS($$), ") at:((Array new:%%d) at: %d put: %n", (long)PTOP(Array_index_no,Array_index_depth)++, $1);
    		    }
    		| array_index ',' expression
    		    {
        			PACTION($1, $2, $3, "", "<array_index> , <expression>", $$);

        			/* Possible run time type check for integer expression value here */
        			nprintf(&LHS($$), "%n", LHS($1));
        			nprintf(&RHS($$), "%n; at: %d put: %n", RHS($1), (long)PTOP(Array_index_no,Array_index_depth)++, $3);
    		    } ;
