

"Fix to Smalltalk interpreter V1.1.1 addInstVarName: and removeInstVarName, (instance spec was not being updated)
 ================================================================================================================"

!ClassDescription methodsFor: 'accessing class description'!

addInstVarName: aString
"bug fix for smalltalk here, the 'copy grow' messages are not necessary but
 'asSet' fails to work with zero length array"
    (self instVarNames copy grow asSet includes: aString)
    ifFalse:[
	instanceVariables _ instanceVariables copyWith: aString.
	self setInstanceSpec: self isVariable words: self isWords pointers: self isPointers instVars: instanceVariables size ]
!

removeInstVarName: aString
    instanceVariables _ instanceVariables copyWithout: aString.
    self setInstanceSpec: self isVariable words: self isWords pointers: self isPointers instVars: instanceVariables size
!!

"Fix to precision problem where 15 digits is just too many for the maximum precision (Float printOn:)
 ====================================================================================================
 (keeping this working depends on using Smalltalk-1.1.1) - probably !!"

!Float methodsFor: 'private'!
printFracPartOn: aStream
    | num count zeroCount digit |
    num _ self fractionPart.
    num = 0.0 ifTrue: [ ^aStream nextPut: $0 ].
    zeroCount _ count _ 0.

    " produce the digits, up to a maximum of 15, suppressing trailing
      zeros "
    [ (num ~= 0.0) & (count < 13) ] whileTrue:
        [ num _ num * 10.0.
          digit _ num floor.
          digit = 0             "for trailing zero suppression"
            ifTrue: [ zeroCount _ zeroCount + 1 ]
            ifFalse: [ aStream next: zeroCount put: $0.
                       aStream nextPut: (Character digitValue: digit).
                       zeroCount _ 0 ].
          num _ num fractionPart.
          count _ count + 1 ].

    zeroCount = 13 ifTrue: [ aStream nextPut: $0 ]
!!

"Fix Symbol to copy like integer, floats and other literals (except string)	- I think this is an omission (ie bug)
 =========================================================================="
!Symbol methodsFor: 'copying'!
shallowCopy
	^self
!

deepCopy
	^self
!!

"Fix Booleans to copy correctly - Otherwise deepCopy gives an infinite recursion ?
 ================================================================================="
!Boolean methodsFor: 'copying'!
deepCopy
	^self
!!

"Fix UndefinedObjects to copy correctly - Otherwise the the 'C' variable nilOOP gets re-defined!
 ================================================================================================"
!UndefinedObject methodsFor: 'copying'!
deepCopy
	^self
!
!

    Smalltalk gcMessage: false
!

!String methodsFor: 'genie'!

collect: aBlock					"Current st 1.1.1 collect doesn't work !"
	| tmp |
	tmp _ self class new: self size.
	1 to: self size do: [ :i | tmp at: i put: ( aBlock value: ( self at: i ) ) ].
	^tmp
!

gtype
    (self = 'UndefinedObject') ifTrue:
    [
        ^'(none)'
    ]
    ifFalse:
    [
        ^self demangle.
    ]
!


+ aString
	^self , aString
!!


" Re-implement the `old-fashioned' st 1.1.1 StoreOn:
"

!Set methodsFor: 'sorting' !

sort
| aCollection|
aCollection:= SortedCollection new.

        self do:[:i|aCollection add: i].
        ^aCollection
!!


!String methodsFor: 'storing'!

storeOn: aStream
    self do:
        [ :char | char == $' ifTrue: [ aStream nextPut: char ].
                  aStream nextPut: char ].
!!


" Re-implement the `old-fashioned' st 1.1.1 StoreOn:
"
!Character methodsFor: 'storing'!

storeOn: aStream
    aStream nextPut: self
!!


" Re-implement the `old-fashioned' st 1.1.1 StoreOn:
"
!Symbol methodsFor: 'storing'!

storeOn: aStream
        nextPutAll: self
!!


"Enhance the behaviour of Number to return 'undefined' instead of a 'retry:coercing called with
 arguments of the same generality' error.

 ( This is I believe only a quirk of the way the builtins operate, they do not distinguish between
   a type match failure (requiring a coercion) and other sorts of failure eg div 0 )"

Smalltalk at: #NumberRetryCoercingMethodString put:		" Save as a string for later use by array & workspace"
'retry: aSymbol coercing: aNumber
    | selfGen aNumGen |
    aSymbol == #=
        ifTrue: [ (aNumber isKindOf: Number) ifFalse: [ ^false ] ].
    selfGen _ self generality.
    aNumGen _ aNumber generality.
    selfGen > aNumGen
        ifTrue: [ ^self perform: aSymbol with: (self coerce: aNumber) ].
    selfGen < aNumGen
        ifTrue: [ ^(aNumber coerce: self) perform: aSymbol with: aNumber ].
    "Generalities are identical and the operation has still failed and called
     this method, assume something like a /0 error and return undefind"
    ^nil
'
!

Number compile: NumberRetryCoercingMethodString
!


"Add some arithmetic attributes to the UndefinedObject class, see also array.gcl for
 the generality matrix"

!UndefinedObject methodsFor: 'conversion'!

generality
	^9
!

coerce: aSomething			"A something will be an operable object (ie something responding to generality)"
	^nil
!

> anything
	^false
!

< anything
	^false
!

>= anything
	^false
!

<= anything
	^false
!

= anything
	anything isNil
	ifTrue: [ ^true ]		"This exception helps in array comparisons but is questionable logic !"
	ifFalse: [ ^false ]
!

~= anything
	^false
!

+ anything
	^nil
!

- anything
	^nil
!

* anything
	^nil
!

/ anything
	^nil
!

\\ anything	" remainder "
	^nil
!

raisedTo: anything
	^nil
!



"UNARY OPERATIONS
 ================="

negated
	^nil
!

sin
	^nil
!

cos
	^nil
!

tan
	^nil
!

arcSin
	^nil
!

arcCos
	^nil
!

arcTan
	^nil
!

ln
	^nil
!

exp
	^nil
!

log
	^nil
!

sqrt
	^nil
!!

"allow intervals to do '( 1 to: 6 ) by: 10' as an allowable syntax"

!Interval methodsFor: 'modifying'!
by: aStep
    step _ aStep
!!



"Add some formatted printing primitives, these just give a way of printing
 out a slightly modified local format."

!String methodsFor: 'printing'!
fprintOn: aStream
        aStream nextPut: $".
        self printOn: aStream.
        aStream nextPut: $"
!

stringPrint: aDescriptor
        nil     "Just a forward declaration for the one in builtins.gcl"
!

stringPrint
        nil     "Just a forward declaration for the one in builtins.gcl"
!!



!Float methodsFor: 'printing'!
fprintOn: aStream
        self printOn: aStream
!!
        


!Integer methodsFor: 'printing'!
fprintOn: aStream
        self printOn: aStream
!!
        


!UndefinedObject methodsFor: 'printing'!
fprintOn: aStream
        aStream nextPut: $_
!

printOn: aStream
	self fprintOn: aStream
!!



!Boolean methodsFor: 'printing'!
fprintOn: aStream
        self
                ifTrue:  [ aStream nextPutAll: '$TRUE' ]
                ifFalse: [ aStream nextPutAll: '$FALSE' ]
!!

!True methodsFor: 'printing'!
printOn: aStream
	self fprintOn: aStream	"(goes up to boolean fprintOn)"
!!

!False methodsFor: 'printing'!
printOn: aStream
	self fprintOn: aStream
!!


"Overload object printing to use appropriate streams for GENIE"
Smalltalk at: #Nl ifAbsent: [ Smalltalk at: #Nl put: nil ]!             "Just a forward declaration"

"Here we define some Strings which take and Standard I/O to the smalltalk
 streams stdin, stdout, stderr.  These strings are then un-buffered whenever
 the stringPrint primitive is used."

"These are reassigned from here on"
Smalltalk at: #StreamOut put: ''.
Smalltalk at: #StreamErr put: ''
!


stdout _ WriteStream on: StreamOut.
stderr _ WriteStream on: StreamErr.
stdin  _ ReadStream  on: 'THIS SHOULD NEVER GET READ'!




!Object methodsFor: 'printing'!
prn
        self printString stringPrint
!

prnl
        self printString stringPrint.
        Nl stringPrint
!

prin
        self printString stringPrint: 'i'
!

prinl
        self printString stringPrint: 'i'.
        Nl stringPrint: 'i'
!

print
	self prdn
!

printNl
	self prdnl
!

pren
        self printString stringPrint: 'e'.
!

prenl
        self printString stringPrint: 'e'.
        Nl stringPrint: 'e'
!

prdn
        self printString stringPrint: 'd'
!

prdnl
        self printString stringPrint: 'd'.
        Nl stringPrint: 'd'
!!


!Object methodsFor: 'handling errors'!

error: aString
	aString prenl
!!

"Forward declaration"
Smalltalk at: #GXIntegerarray ifAbsent: [ Smalltalk at: #GXIntegerarray put: nil ]
!

!Interval methodsFor: 'hacking'!

"Return an array containing all numbers in the interval"
asIntegerArray
        | anArray |
        anArray _ GXIntegerarray new: (self size).
    "    j _ 1. self do: [ :i | anArray at: j put: i. j _ j + 1 ].  "
	^anArray extend: start with: step mode: 0		"Use arithmetic fill - quicker"
!!

!String methodsFor: 'Catching errors'!

run
	self prenl
!!

!UndefinedObject methodsFor: 'Catching errors'!

run
	'Attempting to run an undefined procedure!!' prenl
!!

!Integer methodsFor: 'raisedTo'!
raisedTo: aValue
	(aValue isKindOf: Integer) 
	    ifTrue: [ ^self raisedToInteger: aValue ]
	    ifFalse: [ ^((self asFloat) raisedTo: aValue) floor ]
!!

nil
!


"EXTERNAL OBJECT LINKING MECHANISM (through smalltalk 'C' function interface)
 ================================================================================
 ================================================================================"
Object subclass: 'External'
	   instanceVariableNames: 'facilityName facilityCode'
	   classVariableNames: ''
	   poolDictionaries: ''
	   category: 'genieV3'
!

External comment:
'My instances represent external objects being called from smalltalk
 via the C callout mechanism.  I ensure that the messages  sent to
 my instances go correctly to the external object.  Each of my instances
 is a separate interface to one specific external object with the sole
 responsibility of ensuring the object can receives the correct parameters'
!


"CLASS METHODS (public)
 ================================================================================"
!External class methodsFor: 'creation'!

"---Create a new facility interface---

	facility:	String giving name of facility
	code:		Unique integer code for facility"

facility: aString code: anInteger
    ^super new setFacilityName: aString withCode: anInteger
!!


"INSTANCE METHODS (public)
 ================================================================================"
!External methodsFor: 'basic'!

"---Invoke a function from the specified facility---

	function:	Code for a particular function within a facility
	parameters:	An array of IN/OUT parameters for the function"

function: anInt parameters: anArray
	^self facility: facilityCode function: anInt parameters: anArray
!

description			"Supply a descriptive name if required"
	^facilityName
!!


"INSTANCE METHODS (private)
 ================================================================================"
!External methodsFor: 'private'!

"---Set up the Smalltalk 'C' function interface for one facility---

	Initialize this instance to refer to just one facility but to be
	able to receive and pass on function codes and parameters whenever
	it is refered to. Name and facility code are stored in the instance."

setFacilityName: aString withCode: anInt
    Behavior defineCFunc: 'geniecall'
	 withSelectorArgs: 'facility: anInt function: anInt parameters: anArray'
	 forClass: External
	 returning: #smalltalk
	 args: #( long long smalltalk ).
    facilityName _ aString.
    facilityCode _ anInt	
!

!


"A FEW CONSTANTS
 ==============="

Smalltalk at: #gXU put: nil.		"Define _ as an undefined constant"
Smalltalk at: #gXDU put: nil.
Smalltalk at: #gXDtrue put: true.
Smalltalk at: #gXDfalse put: false.
Smalltalk at: #gXUUdrivers ifAbsent: [ Smalltalk at: #gXUUdrivers put: nil ].
Smalltalk at: #gXUdebug ifAbsent: [ Smalltalk at: #gXUdebug put: false ].
Smalltalk at: #gXUarrayUelementsUtoUprint ifAbsent: [ Smalltalk at: #gXUarrayUelementsUtoUprint put: 5 ].
!

Smalltalk at: #Nl  put: (10 asCharacter printString).
Smalltalk at: #Esc put: (27 asCharacter printString).
Smalltalk at: #Depth put: ''
!


"EXTERNAL FACILITIES CURRENTLY DEFINED
 =====================================

	Any new facilities can be added here without reference
	to other initialization files"

Smalltalk at: #extDebugging	put: ( External facility: 'Debugging callouts'		code: 0 ).
Smalltalk at: #lowSystem	put: ( External facility: 'Low level system calls'	code: 1 ).
Smalltalk at: #intOperations	put: ( External facility: 'Internal operations callout'	code: 2 ).
Smalltalk at: #intExtensions	put: ( External facility: 'Internal extensions callout'	code: 3 ).
Smalltalk at: #objGraphics	put: ( External facility: 'Genie object graphics'	code: 4 ).
Smalltalk at: #extFortran	put: ( External facility: 'Fortran Externals'		code: 5 ).
Smalltalk at: #extTransform	put: ( External facility: 'External workspace xForm'	code: 6 ).
Smalltalk at: #intPeak		put: ( External facility: 'Internal extended peak fns.'	code: 7 ).
Smalltalk at: #extModule	put: ( External facility: 'External user modules.'	code: 8 ).
Smalltalk at: #newGraphics	put: ( External facility: 'New Graphics Callout.'	code: 9 ).
Smalltalk at: #fileAccess	put: ( External facility: 'External IO Callout.'	code: 10 )

!


"GENIE PROCEDURE EVALUATION MECHANISM
 ================================================================================
 ================================================================================"
Object subclass: 'Function'
       instanceVariableNames: 'line help'		"Local variables put here"
       classVariableNames: 'depth'		        "Not externally visible"
       poolDictionaries: ''
       category: 'BasePrimitive'    	    	    	"Essential parts of interpreter"
!

Function comment:
'My instances are created from my subclasses, I have none of my own.
 My subclasses are defined GENIE functions as stored in Smalltalk. Each
 instance of one of my subclasses represents one function invocation. The function
 method is stored as a class variable, the Global variables in the GENIE pool
 and local variables as instance variables.'
!

"CLASS METHODS
 ================================================================================"
!Function class methodsFor: 'Initializing'!


"---Add new pool variables---

	These can be added with or without an initial value.
	If initializing, don't overwrite any values which may
	already be there unless of course the value is nil."

addPoolVar: aName value: aValue
     ( Smalltalk at: aName ifAbsent: [Smalltalk at: aName put: aValue] ) isNil
     ifTrue: [ Smalltalk at: aName put: aValue ]
!

addPoolVar: aName
     Smalltalk at: aName ifAbsent: [Smalltalk at: aName put: nil]
!


"---Create an instance of the procedure class ready to run---

	This is analogous to creating a new stack frame in a conventional
	run-time environment. "

run 
    ^self new initVars
!!


"INSTANCE METHODS (public)
 ================================================================================"
"Forward declaration"
Smalltalk at: #GXGobject ifAbsent: [ Smalltalk at: #GXGobject put: nil ]!
Smalltalk at: #GXInteger ifAbsent: [ Smalltalk at: #GXInteger put: nil ]!

!Function methodsFor: 'evaluating'!

"---Initialize local variables, parameters and run commands---"

evaluate
	nil	"Only called for functions with no commands of their own"
!

"---Conditionally evaluate the procedure based on parameters.---"
condEvaluate
	help isNil ifTrue: [ ^self evaluate ]
!

initVars
	nil	"Only called for functions with no variables of their own"
!

"---Fixed parameters all procedures will respond to in a generic fashion---"

gXhelp
	self class comment prnl.
	help _ true
!

doesNotUnderstand: aMessage
	| var |
	var _ aMessage selector gtype copyWithout: $:.
	help _ 'Please do not run me Mr Evaluator'.
	(var size = 0)
	ifTrue: [ self error2 ]
	ifFalse: [ self error3: var ]
!

"Handle coercing of of types where the intention was obvious"
coerceTo: aType ifPossible: aVar called: aName
	| conv |
	"Try a coercion to the appropriate GX type"
	conv _ aType asMe: aVar.
	conv notNil
	ifTrue: [
		"Avoid printing user messages when this is our fault!!"
		( (aType = GXGobject) | ((aType = GXInteger) & (aVar isKindOf: Symbol)) )
		ifFalse: [ self warning1: aName type: aType ].
		^conv
	]
	ifFalse: [
		"if the variable cannot be coerced, print a warning and return UndefinedObject"
		self error1: aVar type: aType name: aName.
		^nil
	]
!

"---Provide type check on parameter instantiation if needed---"

typeCheck: aVar shouldBe: aType paramName: aName
	| res |
	(( aVar class isKindOf: aType species ) | aVar isNil )
		ifTrue:	[ ^aVar ]
		ifFalse:[ res _ self coerceTo: aType ifPossible: aVar called: aName].
	res isNil
		"Note that setting help to anything will stop condEvaluate continuing"
		ifTrue: [ help _ 'Do not run this procedure please Mr Evaluator' ].
	^res 
!


"---Error messages---"

error1: aVar type: aType name: aName
    ('In procedure "', self class printString gtype asUppercase,
    '", variable type of "',aVar class printString, '" does not match with formal parameter') prenl.
    ('	(Formal parameter "',aName gtype asUppercase,'" must be of type "',aType printString gtype,'").') prenl
!

error2
    ('In procedure "', self class printString gtype asUppercase,
    '", Too many arguments passed to procedure by calling program') prenl
!

error3: aName
    ('In procedure "', self class printString gtype asUppercase,
    '", Unknown parameter "', aName, '" specified in call to procedure') prenl
!

warning1: aName type: aType
    ('In procedure "', self class printString gtype asUppercase,
    '", Converting supplied "', aName gtype asUppercase, '=" parameter into a "', aType printString gtype, '"') prinl
!


"---Procedure depth count---"
depth: anInt
    depth _ anInt
!

up
    depth _ depth - 1
!

down
    depth _ depth + 1
!


"---Make procedure calling information available available---"

line: anInt proc: aProcInst method: aBool
    | pad |
    self down.
    line _ Array new: 3.
    line at: 1 put: aProcInst.	    "the the calling procedure instance"
    line at: 2 put: anInt.	    "The line number from which this was called"
    line at: 3 put: aBool.	    "Flag, set true if this was invoked as a function"
    ( gXUdebug ) ifTrue:
    [
        pad _ ''.
        1 to: depth - 1 do: [ pad _ pad, '  ' ].
        ( Esc, '[35m', Esc, '[1m' ) prn.
        ( pad, anInt printString, ':', self class name gtype ) prn.
        ( Esc, '[0m' ) prnl
    ]
!


"---Methods to implement GENIE control constructs---

	These are most easily implemented by calling a method from the
	parser as writing them directly from the parser is very complicated!"


loop: block1 condition: cBlock after: block2
    [true] whileTrue:[
    	block1 value.
    	(cBlock value) ifTrue: [ ^self ].
    	block2 value
    	].
!

loop: start to: stop by: step before: block1 condition: cBlock after: block2
    start to: stop by: step do:[ :i |
    	block1 value: i.
    	(cBlock value: i) ifTrue: [ ^self ].
    	block2 value: i
    	].
!

case: aValue body: aBlock
    aBlock value: aValue value: [ ^self ]
!
!
"Procedure class definition:
 ==========================="
((Smalltalk at: #GXsin ifAbsent: [ nil ]) isKindOf: Function class) ifTrue: [ ('Warning: Overriding previously defined procedure ', 'GXsin' gtype) prinl ].
Smalltalk at: #GXsin put: nil!
Function subclass: 'GXsin'
       instanceVariableNames: ''
       classVariableNames: ''
       poolDictionaries: ''
       category: 'Genie Procedure'
!

"HEADER
 ------"

"parameters
 ----------"

GXsin

addInstVarName: #gXx;
compile:
'gXx: aVar
	aVar notNil ifTrue:[ gXx _ aVar ]';
compile:
'p1: aVar
	aVar notNil ifTrue:[ gXx _ aVar ]'!

"result
 ------"

GXsin

addInstVarName: #gXres
!

!GXsin methodsFor: 'initializing'!

initVars

!!

"BODY
 ----"

!GXsin methodsFor: 'running'!

evaluate
 gXres _ gXx sin .
self up.
^gXres
!!
nil! "(fix for smalltalk bug, this line is ignored if there is a variable declaration error)"

"End of procedure"

"Procedure class definition:
 ==========================="
((Smalltalk at: #GXcos ifAbsent: [ nil ]) isKindOf: Function class) ifTrue: [ ('Warning: Overriding previously defined procedure ', 'GXcos' gtype) prinl ].
Smalltalk at: #GXcos put: nil!
Function subclass: 'GXcos'
       instanceVariableNames: ''
       classVariableNames: ''
       poolDictionaries: ''
       category: 'Genie Procedure'
!

"HEADER
 ------"

"parameters
 ----------"

GXcos

addInstVarName: #gXx;
compile:
'gXx: aVar
	aVar notNil ifTrue:[ gXx _ aVar ]';
compile:
'p1: aVar
	aVar notNil ifTrue:[ gXx _ aVar ]'!

"result
 ------"

GXcos

addInstVarName: #gXres
!

!GXcos methodsFor: 'initializing'!

initVars

!!

"BODY
 ----"

!GXcos methodsFor: 'running'!

evaluate
 gXres _ gXx cos .
self up.
^gXres
!!
nil! "(fix for smalltalk bug, this line is ignored if there is a variable declaration error)"

"End of procedure"

"Procedure class definition:
 ==========================="
((Smalltalk at: #GXtan ifAbsent: [ nil ]) isKindOf: Function class) ifTrue: [ ('Warning: Overriding previously defined procedure ', 'GXtan' gtype) prinl ].
Smalltalk at: #GXtan put: nil!
Function subclass: 'GXtan'
       instanceVariableNames: ''
       classVariableNames: ''
       poolDictionaries: ''
       category: 'Genie Procedure'
!

"HEADER
 ------"

"parameters
 ----------"

GXtan

addInstVarName: #gXx;
compile:
'gXx: aVar
	aVar notNil ifTrue:[ gXx _ aVar ]';
compile:
'p1: aVar
	aVar notNil ifTrue:[ gXx _ aVar ]'!

"result
 ------"

GXtan

addInstVarName: #gXres
!

!GXtan methodsFor: 'initializing'!

initVars

!!

"BODY
 ----"

!GXtan methodsFor: 'running'!

evaluate
 gXres _ gXx tan .
self up.
^gXres
!!
nil! "(fix for smalltalk bug, this line is ignored if there is a variable declaration error)"

"End of procedure"

"Procedure class definition:
 ==========================="
((Smalltalk at: #GXarcsin ifAbsent: [ nil ]) isKindOf: Function class) ifTrue: [ ('Warning: Overriding previously defined procedure ', 'GXarcsin' gtype) prinl ].
Smalltalk at: #GXarcsin put: nil!
Function subclass: 'GXarcsin'
       instanceVariableNames: ''
       classVariableNames: ''
       poolDictionaries: ''
       category: 'Genie Procedure'
!

"HEADER
 ------"

"parameters
 ----------"

GXarcsin

addInstVarName: #gXx;
compile:
'gXx: aVar
	aVar notNil ifTrue:[ gXx _ aVar ]';
compile:
'p1: aVar
	aVar notNil ifTrue:[ gXx _ aVar ]'!

"result
 ------"

GXarcsin

addInstVarName: #gXres
!

!GXarcsin methodsFor: 'initializing'!

initVars

!!

"BODY
 ----"

!GXarcsin methodsFor: 'running'!

evaluate
 gXres _ gXx arcSin .
self up.
^gXres
!!
nil! "(fix for smalltalk bug, this line is ignored if there is a variable declaration error)"

"End of procedure"

"Procedure class definition:
 ==========================="
((Smalltalk at: #GXarccos ifAbsent: [ nil ]) isKindOf: Function class) ifTrue: [ ('Warning: Overriding previously defined procedure ', 'GXarccos' gtype) prinl ].
Smalltalk at: #GXarccos put: nil!
Function subclass: 'GXarccos'
       instanceVariableNames: ''
       classVariableNames: ''
       poolDictionaries: ''
       category: 'Genie Procedure'
!

"HEADER
 ------"

"parameters
 ----------"

GXarccos

addInstVarName: #gXx;
compile:
'gXx: aVar
	aVar notNil ifTrue:[ gXx _ aVar ]';
compile:
'p1: aVar
	aVar notNil ifTrue:[ gXx _ aVar ]'!

"result
 ------"

GXarccos

addInstVarName: #gXres
!

!GXarccos methodsFor: 'initializing'!

initVars

!!

"BODY
 ----"

!GXarccos methodsFor: 'running'!

evaluate
 gXres _ gXx arcCos .
self up.
^gXres
!!
nil! "(fix for smalltalk bug, this line is ignored if there is a variable declaration error)"

"End of procedure"

"Procedure class definition:
 ==========================="
((Smalltalk at: #GXarctan ifAbsent: [ nil ]) isKindOf: Function class) ifTrue: [ ('Warning: Overriding previously defined procedure ', 'GXarctan' gtype) prinl ].
Smalltalk at: #GXarctan put: nil!
Function subclass: 'GXarctan'
       instanceVariableNames: ''
       classVariableNames: ''
       poolDictionaries: ''
       category: 'Genie Procedure'
!

"HEADER
 ------"

"parameters
 ----------"

GXarctan

addInstVarName: #gXx;
compile:
'gXx: aVar
	aVar notNil ifTrue:[ gXx _ aVar ]';
compile:
'p1: aVar
	aVar notNil ifTrue:[ gXx _ aVar ]'!

"result
 ------"

GXarctan

addInstVarName: #gXres
!

!GXarctan methodsFor: 'initializing'!

initVars

!!

"BODY
 ----"

!GXarctan methodsFor: 'running'!

evaluate
 gXres _ gXx arcTan .
self up.
^gXres
!!
nil! "(fix for smalltalk bug, this line is ignored if there is a variable declaration error)"

"End of procedure"

"Procedure class definition:
 ==========================="
((Smalltalk at: #GXln ifAbsent: [ nil ]) isKindOf: Function class) ifTrue: [ ('Warning: Overriding previously defined procedure ', 'GXln' gtype) prinl ].
Smalltalk at: #GXln put: nil!
Function subclass: 'GXln'
       instanceVariableNames: ''
       classVariableNames: ''
       poolDictionaries: ''
       category: 'Genie Procedure'
!

"HEADER
 ------"

"parameters
 ----------"

GXln

addInstVarName: #gXx;
compile:
'gXx: aVar
	aVar notNil ifTrue:[ gXx _ aVar ]';
compile:
'p1: aVar
	aVar notNil ifTrue:[ gXx _ aVar ]'!

"result
 ------"

GXln

addInstVarName: #gXres
!

!GXln methodsFor: 'initializing'!

initVars

!!

"BODY
 ----"

!GXln methodsFor: 'running'!

evaluate
 gXres _ gXx ln .
self up.
^gXres
!!
nil! "(fix for smalltalk bug, this line is ignored if there is a variable declaration error)"

"End of procedure"

"Procedure class definition:
 ==========================="
((Smalltalk at: #GXexp ifAbsent: [ nil ]) isKindOf: Function class) ifTrue: [ ('Warning: Overriding previously defined procedure ', 'GXexp' gtype) prinl ].
Smalltalk at: #GXexp put: nil!
Function subclass: 'GXexp'
       instanceVariableNames: ''
       classVariableNames: ''
       poolDictionaries: ''
       category: 'Genie Procedure'
!

"HEADER
 ------"

"parameters
 ----------"

GXexp

addInstVarName: #gXx;
compile:
'gXx: aVar
	aVar notNil ifTrue:[ gXx _ aVar ]';
compile:
'p1: aVar
	aVar notNil ifTrue:[ gXx _ aVar ]'!

"result
 ------"

GXexp

addInstVarName: #gXres
!

!GXexp methodsFor: 'initializing'!

initVars

!!

"BODY
 ----"

!GXexp methodsFor: 'running'!

evaluate
 gXres _ gXx exp .
self up.
^gXres
!!
nil! "(fix for smalltalk bug, this line is ignored if there is a variable declaration error)"

"End of procedure"

"Procedure class definition:
 ==========================="
((Smalltalk at: #GXlog ifAbsent: [ nil ]) isKindOf: Function class) ifTrue: [ ('Warning: Overriding previously defined procedure ', 'GXlog' gtype) prinl ].
Smalltalk at: #GXlog put: nil!
Function subclass: 'GXlog'
       instanceVariableNames: ''
       classVariableNames: ''
       poolDictionaries: ''
       category: 'Genie Procedure'
!

"HEADER
 ------"

"parameters
 ----------"

GXlog

addInstVarName: #gXx;
compile:
'gXx: aVar
	aVar notNil ifTrue:[ gXx _ aVar ]';
compile:
'p1: aVar
	aVar notNil ifTrue:[ gXx _ aVar ]'!

"result
 ------"

GXlog

addInstVarName: #gXres
!

!GXlog methodsFor: 'initializing'!

initVars

!!

"BODY
 ----"

!GXlog methodsFor: 'running'!

evaluate
 gXres _ gXx log:10 .
self up.
^gXres
!!
nil! "(fix for smalltalk bug, this line is ignored if there is a variable declaration error)"

"End of procedure"

"Procedure class definition:
 ==========================="
((Smalltalk at: #GXsqrt ifAbsent: [ nil ]) isKindOf: Function class) ifTrue: [ ('Warning: Overriding previously defined procedure ', 'GXsqrt' gtype) prinl ].
Smalltalk at: #GXsqrt put: nil!
Function subclass: 'GXsqrt'
       instanceVariableNames: ''
       classVariableNames: ''
       poolDictionaries: ''
       category: 'Genie Procedure'
!

"HEADER
 ------"

"parameters
 ----------"

GXsqrt

addInstVarName: #gXx;
compile:
'gXx: aVar
	aVar notNil ifTrue:[ gXx _ aVar ]';
compile:
'p1: aVar
	aVar notNil ifTrue:[ gXx _ aVar ]'!

"result
 ------"

GXsqrt

addInstVarName: #gXres
!

!GXsqrt methodsFor: 'initializing'!

initVars

!!

"BODY
 ----"

!GXsqrt methodsFor: 'running'!

evaluate
 gXres _ gXx sqrt .
self up.
^gXres
!!
nil! "(fix for smalltalk bug, this line is ignored if there is a variable declaration error)"

"End of procedure"

"Procedure class definition:
 ==========================="
((Smalltalk at: #GXabs ifAbsent: [ nil ]) isKindOf: Function class) ifTrue: [ ('Warning: Overriding previously defined procedure ', 'GXabs' gtype) prinl ].
Smalltalk at: #GXabs put: nil!
Function subclass: 'GXabs'
       instanceVariableNames: ''
       classVariableNames: ''
       poolDictionaries: ''
       category: 'Genie Procedure'
!

"HEADER
 ------"

"parameters
 ----------"

GXabs

addInstVarName: #gXx;
compile:
'gXx: aVar
	aVar notNil ifTrue:[ gXx _ aVar ]';
compile:
'p1: aVar
	aVar notNil ifTrue:[ gXx _ aVar ]'!

"result
 ------"

GXabs

addInstVarName: #gXres
!

!GXabs methodsFor: 'initializing'!

initVars

!!

"BODY
 ----"

!GXabs methodsFor: 'running'!

evaluate
 gXres _ gXx abs .
self up.
^gXres
!!
nil! "(fix for smalltalk bug, this line is ignored if there is a variable declaration error)"

"End of procedure"

((Smalltalk at: #GXworkspaceUcoerce ifAbsent: [Smalltalk at: #GXworkspaceUcoerce put: 				  'Procedure "workspace_coerce" has only been FORWARD declared, it must also have a PROCEDURE definition' ] ) 				  isKindOf: Function class) ifTrue: 				[ 'Warning: FORWARD statement for a previously defined procedure "workspace_coerce" - no action taken' prinl ]!
((Smalltalk at: #GXworkspaceUgreaterUthan ifAbsent: [Smalltalk at: #GXworkspaceUgreaterUthan put: 				  'Procedure "workspace_greater_than" has only been FORWARD declared, it must also have a PROCEDURE definition' ] ) 				  isKindOf: Function class) ifTrue: 				[ 'Warning: FORWARD statement for a previously defined procedure "workspace_greater_than" - no action taken' prinl ]!
((Smalltalk at: #GXworkspaceUlessUthan ifAbsent: [Smalltalk at: #GXworkspaceUlessUthan put: 				  'Procedure "workspace_less_than" has only been FORWARD declared, it must also have a PROCEDURE definition' ] ) 				  isKindOf: Function class) ifTrue: 				[ 'Warning: FORWARD statement for a previously defined procedure "workspace_less_than" - no action taken' prinl ]!
((Smalltalk at: #GXworkspaceUequal ifAbsent: [Smalltalk at: #GXworkspaceUequal put: 				  'Procedure "workspace_equal" has only been FORWARD declared, it must also have a PROCEDURE definition' ] ) 				  isKindOf: Function class) ifTrue: 				[ 'Warning: FORWARD statement for a previously defined procedure "workspace_equal" - no action taken' prinl ]!
((Smalltalk at: #GXworkspaceUgreaterUthanUorUequal ifAbsent: [Smalltalk at: #GXworkspaceUgreaterUthanUorUequal put: 				  'Procedure "workspace_greater_than_or_equal" has only been FORWARD declared, it must also have a PROCEDURE definition' ] ) 				  isKindOf: Function class) ifTrue: 				[ 'Warning: FORWARD statement for a previously defined procedure "workspace_greater_than_or_equal" - no action taken' prinl ]!
((Smalltalk at: #GXworkspaceUlessUthanUorUequal ifAbsent: [Smalltalk at: #GXworkspaceUlessUthanUorUequal put: 				  'Procedure "workspace_less_than_or_equal" has only been FORWARD declared, it must also have a PROCEDURE definition' ] ) 				  isKindOf: Function class) ifTrue: 				[ 'Warning: FORWARD statement for a previously defined procedure "workspace_less_than_or_equal" - no action taken' prinl ]!
((Smalltalk at: #GXworkspaceUnotUequal ifAbsent: [Smalltalk at: #GXworkspaceUnotUequal put: 				  'Procedure "workspace_not_equal" has only been FORWARD declared, it must also have a PROCEDURE definition' ] ) 				  isKindOf: Function class) ifTrue: 				[ 'Warning: FORWARD statement for a previously defined procedure "workspace_not_equal" - no action taken' prinl ]!
((Smalltalk at: #GXworkspaceUappend ifAbsent: [Smalltalk at: #GXworkspaceUappend put: 				  'Procedure "workspace_append" has only been FORWARD declared, it must also have a PROCEDURE definition' ] ) 				  isKindOf: Function class) ifTrue: 				[ 'Warning: FORWARD statement for a previously defined procedure "workspace_append" - no action taken' prinl ]!
((Smalltalk at: #GXworkspaceUor ifAbsent: [Smalltalk at: #GXworkspaceUor put: 				  'Procedure "workspace_or" has only been FORWARD declared, it must also have a PROCEDURE definition' ] ) 				  isKindOf: Function class) ifTrue: 				[ 'Warning: FORWARD statement for a previously defined procedure "workspace_or" - no action taken' prinl ]!
((Smalltalk at: #GXworkspaceUadd ifAbsent: [Smalltalk at: #GXworkspaceUadd put: 				  'Procedure "workspace_add" has only been FORWARD declared, it must also have a PROCEDURE definition' ] ) 				  isKindOf: Function class) ifTrue: 				[ 'Warning: FORWARD statement for a previously defined procedure "workspace_add" - no action taken' prinl ]!
((Smalltalk at: #GXworkspaceUsubtract ifAbsent: [Smalltalk at: #GXworkspaceUsubtract put: 				  'Procedure "workspace_subtract" has only been FORWARD declared, it must also have a PROCEDURE definition' ] ) 				  isKindOf: Function class) ifTrue: 				[ 'Warning: FORWARD statement for a previously defined procedure "workspace_subtract" - no action taken' prinl ]!
((Smalltalk at: #GXworkspaceUand ifAbsent: [Smalltalk at: #GXworkspaceUand put: 				  'Procedure "workspace_and" has only been FORWARD declared, it must also have a PROCEDURE definition' ] ) 				  isKindOf: Function class) ifTrue: 				[ 'Warning: FORWARD statement for a previously defined procedure "workspace_and" - no action taken' prinl ]!
((Smalltalk at: #GXworkspaceUmultiply ifAbsent: [Smalltalk at: #GXworkspaceUmultiply put: 				  'Procedure "workspace_multiply" has only been FORWARD declared, it must also have a PROCEDURE definition' ] ) 				  isKindOf: Function class) ifTrue: 				[ 'Warning: FORWARD statement for a previously defined procedure "workspace_multiply" - no action taken' prinl ]!
((Smalltalk at: #GXworkspaceUdivide ifAbsent: [Smalltalk at: #GXworkspaceUdivide put: 				  'Procedure "workspace_divide" has only been FORWARD declared, it must also have a PROCEDURE definition' ] ) 				  isKindOf: Function class) ifTrue: 				[ 'Warning: FORWARD statement for a previously defined procedure "workspace_divide" - no action taken' prinl ]!
((Smalltalk at: #GXworkspaceUmodulo ifAbsent: [Smalltalk at: #GXworkspaceUmodulo put: 				  'Procedure "workspace_modulo" has only been FORWARD declared, it must also have a PROCEDURE definition' ] ) 				  isKindOf: Function class) ifTrue: 				[ 'Warning: FORWARD statement for a previously defined procedure "workspace_modulo" - no action taken' prinl ]!
((Smalltalk at: #GXworkspaceUraisedUto ifAbsent: [Smalltalk at: #GXworkspaceUraisedUto put: 				  'Procedure "workspace_raised_to" has only been FORWARD declared, it must also have a PROCEDURE definition' ] ) 				  isKindOf: Function class) ifTrue: 				[ 'Warning: FORWARD statement for a previously defined procedure "workspace_raised_to" - no action taken' prinl ]!
((Smalltalk at: #GXworkspaceUsin ifAbsent: [Smalltalk at: #GXworkspaceUsin put: 				  'Procedure "workspace_sin" has only been FORWARD declared, it must also have a PROCEDURE definition' ] ) 				  isKindOf: Function class) ifTrue: 				[ 'Warning: FORWARD statement for a previously defined procedure "workspace_sin" - no action taken' prinl ]!
((Smalltalk at: #GXworkspaceUcos ifAbsent: [Smalltalk at: #GXworkspaceUcos put: 				  'Procedure "workspace_cos" has only been FORWARD declared, it must also have a PROCEDURE definition' ] ) 				  isKindOf: Function class) ifTrue: 				[ 'Warning: FORWARD statement for a previously defined procedure "workspace_cos" - no action taken' prinl ]!
((Smalltalk at: #GXworkspaceUtan ifAbsent: [Smalltalk at: #GXworkspaceUtan put: 				  'Procedure "workspace_tan" has only been FORWARD declared, it must also have a PROCEDURE definition' ] ) 				  isKindOf: Function class) ifTrue: 				[ 'Warning: FORWARD statement for a previously defined procedure "workspace_tan" - no action taken' prinl ]!
((Smalltalk at: #GXworkspaceUarcsin ifAbsent: [Smalltalk at: #GXworkspaceUarcsin put: 				  'Procedure "workspace_arcsin" has only been FORWARD declared, it must also have a PROCEDURE definition' ] ) 				  isKindOf: Function class) ifTrue: 				[ 'Warning: FORWARD statement for a previously defined procedure "workspace_arcsin" - no action taken' prinl ]!
((Smalltalk at: #GXworkspaceUarccos ifAbsent: [Smalltalk at: #GXworkspaceUarccos put: 				  'Procedure "workspace_arccos" has only been FORWARD declared, it must also have a PROCEDURE definition' ] ) 				  isKindOf: Function class) ifTrue: 				[ 'Warning: FORWARD statement for a previously defined procedure "workspace_arccos" - no action taken' prinl ]!
((Smalltalk at: #GXworkspaceUarctan ifAbsent: [Smalltalk at: #GXworkspaceUarctan put: 				  'Procedure "workspace_arctan" has only been FORWARD declared, it must also have a PROCEDURE definition' ] ) 				  isKindOf: Function class) ifTrue: 				[ 'Warning: FORWARD statement for a previously defined procedure "workspace_arctan" - no action taken' prinl ]!
((Smalltalk at: #GXworkspaceUln ifAbsent: [Smalltalk at: #GXworkspaceUln put: 				  'Procedure "workspace_ln" has only been FORWARD declared, it must also have a PROCEDURE definition' ] ) 				  isKindOf: Function class) ifTrue: 				[ 'Warning: FORWARD statement for a previously defined procedure "workspace_ln" - no action taken' prinl ]!
((Smalltalk at: #GXworkspaceUexp ifAbsent: [Smalltalk at: #GXworkspaceUexp put: 				  'Procedure "workspace_exp" has only been FORWARD declared, it must also have a PROCEDURE definition' ] ) 				  isKindOf: Function class) ifTrue: 				[ 'Warning: FORWARD statement for a previously defined procedure "workspace_exp" - no action taken' prinl ]!
((Smalltalk at: #GXworkspaceUlog ifAbsent: [Smalltalk at: #GXworkspaceUlog put: 				  'Procedure "workspace_log" has only been FORWARD declared, it must also have a PROCEDURE definition' ] ) 				  isKindOf: Function class) ifTrue: 				[ 'Warning: FORWARD statement for a previously defined procedure "workspace_log" - no action taken' prinl ]!
((Smalltalk at: #GXworkspaceUsqrt ifAbsent: [Smalltalk at: #GXworkspaceUsqrt put: 				  'Procedure "workspace_sqrt" has only been FORWARD declared, it must also have a PROCEDURE definition' ] ) 				  isKindOf: Function class) ifTrue: 				[ 'Warning: FORWARD statement for a previously defined procedure "workspace_sqrt" - no action taken' prinl ]!
((Smalltalk at: #GXworkspaceUabs ifAbsent: [Smalltalk at: #GXworkspaceUabs put: 				  'Procedure "workspace_abs" has only been FORWARD declared, it must also have a PROCEDURE definition' ] ) 				  isKindOf: Function class) ifTrue: 				[ 'Warning: FORWARD statement for a previously defined procedure "workspace_abs" - no action taken' prinl ]!
((Smalltalk at: #GXworkspaceUnot ifAbsent: [Smalltalk at: #GXworkspaceUnot put: 				  'Procedure "workspace_not" has only been FORWARD declared, it must also have a PROCEDURE definition' ] ) 				  isKindOf: Function class) ifTrue: 				[ 'Warning: FORWARD statement for a previously defined procedure "workspace_not" - no action taken' prinl ]!
((Smalltalk at: #GXworkspaceUnegated ifAbsent: [Smalltalk at: #GXworkspaceUnegated put: 				  'Procedure "workspace_negated" has only been FORWARD declared, it must also have a PROCEDURE definition' ] ) 				  isKindOf: Function class) ifTrue: 				[ 'Warning: FORWARD statement for a previously defined procedure "workspace_negated" - no action taken' prinl ]!

IdentityDictionary variableSubclass: 'GXWorkspace'
	instanceVariableNames: '' classVariableNames: '' poolDictionaries: '' category: 'GX types'
!

"CLASS METHODS for all GENIE workspaces
 ======================================"

!GXWorkspace class methodsFor: 'creating'!

"Re-define new to initialize 'fixed' fields"

new
        | tmp |
        tmp _ super new.
        self allInstVarNames do: [ :i | ((i at: 1) == $g) ifTrue: [tmp at: i put: nil] ].
        ^tmp
!

"Make the syntax for subclass creation easy"

subclass: name fields: attributes comment: aComment
        | ts fixedFields className |
        fixedFields _ ''.
        attributes _ attributes asLowercase.
        name _ name asLowercase.
        name at: 1 put: (name at: 1) asUppercase.
        ts _ (TokenStream on: attributes) contents.
        ts do: [ :i | fixedFields _ fixedFields, ' gX', i ].
        className _ 'GX', name.
        self    variableSubclass: className
                instanceVariableNames: fixedFields
                classVariableNames: ''
                poolDictionaries: ''
                category: 'GX Types'.
        (Smalltalk at: className asSymbol) comment: aComment
!

"Allow a printout of the heirachy"

hierarchy
        self hierarchy: ''.
        self setDepth
!

hierarchy: depth
        Depth _ depth.
        self new prnl.
        self subclasses do: [ :i | i hierarchy: depth, '  ' ]
!

setDepth: depth
        Depth _ depth
!

setDepth
        Depth _ ''
!

"Coerce if possible"
asMe: aVal
        ^nil

!!


"Initialize the class variable"
GXWorkspace setDepth!

"Forward declaration (as array.gcl is not loaded yet)"
Smalltalk at: #GXWorkspacearray ifAbsent: [ Smalltalk at: #GXWorkspacearray put: nil ]!

"INSTANCE METHODS for GENIE workspaces (public)
 =============================================="
!GXWorkspace methodsFor: 'accessing'!

"---Extend IdentityDictionary at: and at:put with some type checks---"

at: aSymbol
    ( aSymbol isMemberOf: Symbol )
		ifTrue:
			[ ^super at: aSymbol ifAbsent:
			    [ self error: 'Workspace field ' , aSymbol gtype printString, ' not found'. ^nil ]
			]
		ifFalse: [ self error: (self error1: aSymbol). ^nil ]
!

at: aSymbol put: aVal
	( aSymbol isMemberOf: Symbol )
		ifTrue:  [ ^super at: aSymbol put: aVal ]
		ifFalse: [ self error: (self error1: aSymbol) ]
!


"---and better printing -- "

printOn: aStream depth: depth
        | keep |
        keep _ Depth.
        Depth _ Depth, depth.
        self printOn: aStream.
        self class setDepth: keep
!

printOn: aStream
    | v name className comment highlight|
    className _ self class name.
    comment _ self class comment.
    comment isNil ifTrue: [ comment _ '' ].
    aStream nextPutAll: Depth, '  ', className gtype, ' [', comment, ']'.
    aStream nl.
    aStream nextPutAll: Depth, '  ('.
    self indicesDo:
      [ :i |
	aStream nl.
               (Depth, '    ') printOn: aStream.
               name _ self basicAt: i.
               highlight _ ( self class instVarNames indexOf: name ) ~= 0.      "Only highlight if major name"
"               highlight ifTrue: [ '[' printOn: aStream ]."
               name gtype printOn: aStream.
"               highlight ifTrue: [ ']' printOn: aStream ]."
               aStream nextPutAll: ' = '.
               v _ values basicAt: i.
               (v isKindOf: GXWorkspace ) | (v isKindOf: GXWorkspacearray)
"               (v isKindOf: GXWorkspace)"
                   ifTrue:
                     [
                       aStream nl.
                       v printOn: aStream depth: '    '
                     ]
                   ifFalse:
                     [
                       v fprintOn: aStream
                     ]
      ].
    aStream nl.
    aStream nextPutAll: Depth, '  )'
!

fprintOn: aStream
        self printOn: aStream.
!


"---Error messages for class---"

error1: offendingType
        ( offendingType isMemberOf: Array )		"member of class Array ? - make a guess at problem"
        ifTrue:
		[ ^'Possibly an attempt to treat a workspace as an array !,',
		'        don''t understand "', offendingType printString, '"' ]
        ifFalse:
		[ ^'Invalid workspace field type,',
		'        type was "', offendingType class printString gtype, '"' ]
!!

"NUMERICAL WORKSPACE OPERATIONS
 ==============================

	Could do with multiple inheritance here to make workspaces behave like numbers but
	we can cheet and do a roughly similar job by taking the #retry:coercing from Number and adding
	to the generality hierachy"


GXWorkspace compile: NumberRetryCoercingMethodString		"Borrow the code from class Number for this"
!

!GXWorkspace methodsFor: 'conversion'!

generality
	^7
!

coerce: aSomething			"A something will be an operable object (ie something responding to generality)"
	^GXworkspaceUcoerce run line: 0 proc: self method: true; p1: aSomething; condEvaluate
!!
nil
!


!GXWorkspace methodsFor: 'arithmetic'!

"callback is used to call an appropriate user GCL routine (if it exists)
 note that 'res' is always set to 'self' for the apprpriate data object
 on invocation of the method"

callback: anOpString res: res w1: w1 w2: w2
        | className procName proc pc|
        className _ self class name copy.               "Make a copy to avoid changing the symbol"
        procName _ className.
        procName at: 3 put: (className at: 3) asLowercase.
        procName _ procName , 'U', anOpString.          "(We always have an underscore in the name)"
        proc _ Smalltalk at: procName asSymbol ifAbsent: [ nil ].
        proc isNil
                ifTrue: [ ^res ]
                ifFalse:
                  [
                        pc _ proc run line: 0 proc: self class method: true.
                        pc gXres: res.
                        pc gXw1: w1.
                        ( w2 isNil ) ifFalse: [ pc gXw2: w2 ].
                        ^pc condEvaluate
                  ]
!


"For the more complex types, make the basic arithmetic operations call unary or
 binary functions which can be declared as a normal GENIE procedure but with a
 standard name"

> aWorkspace
	^GXworkspaceUgreaterUthan run line: 0 proc: self class method: true; p1: self; p2: aWorkspace; condEvaluate
!

< aWorkspace
	^GXworkspaceUlessUthan run line: 0 proc: self class method: true; p1: self; p2: aWorkspace; condEvaluate
!

= aWorkspace
	^GXworkspaceUequal run line: 0 proc: self class method: true; p1: self; p2: aWorkspace; condEvaluate
!

>= aWorkspace
	^GXworkspaceUgreaterUthanUorUequal run line: 0 proc: self class method: true; p1: self; p2: aWorkspace; condEvaluate
!

<= aWorkspace
	^GXworkspaceUlessUthanUorUequal run line: 0 proc: self class method: true; p1: self; p2: aWorkspace; condEvaluate
!

~= aWorkspace
	^GXworkspaceUnotUequal run line: 0 proc: self class method: true; p1: self; p2: aWorkspace; condEvaluate
!

, aWorkspace
	^GXworkspaceUappend run line: 0 proc: self class method: true; p1: self; p2: aWorkspace; condEvaluate
!

| aWorkspace
	^GXworkspaceUor run line: 0 proc: self class method: true; p1: self; p2: aWorkspace; condEvaluate
!

+ aWorkspace
      (aWorkspace isMemberOf: self class)
      ifTrue: [^GXworkspaceUadd run line: 0 proc: self class method: true; p1: self; p2: aWorkspace; condEvaluate]
      ifFalse: [^self retry: #+ coercing: aWorkspace]
!

- aWorkspace
      (aWorkspace isMemberOf: self class)
      ifTrue: [^GXworkspaceUsubtract run line: 0 proc: self class method: true; p1: self; p2: aWorkspace; condEvaluate]
      ifFalse: [^self retry: #- coercing: aWorkspace]
!

"
#+ aWorkspace
#        | res |
#	(aWorkspace isMemberOf: self class)
#	        ifTrue:  [res _ self binop: aWorkspace op: #+ ]
#	        ifFalse: [res _ self retry: #+ coercing: aWorkspace].
#        ^self callback: 'add' res: res w1: self w2: aWorkspace.
#!
#
#- aWorkspace
#        | res |
#	(aWorkspace isMemberOf: self class)
#	        ifTrue:  [res _ self binop: aWorkspace op: #- ]
#	        ifFalse: [res _ self retry: #- coercing: aWorkspace].
#        ^self callback: 'subtract' res: res w1: self w2: aWorkspace.
#!
"


& aWorkspace
	^GXworkspaceUand run line: 0 proc: self class method: true; p1: self; p2: aWorkspace; condEvaluate
!

* aWorkspace
	(aWorkspace isMemberOf: self class)
	ifTrue: [^GXworkspaceUmultiply run line: 0 proc: self class method: true; p1: self; p2: aWorkspace; condEvaluate]
	ifFalse: [^self retry: #* coercing: aWorkspace]
!

/ aWorkspace
	(aWorkspace isMemberOf: self class)
	ifTrue: [^GXworkspaceUdivide run line: 0 proc: self class method: true; p1: self; p2: aWorkspace; condEvaluate]
	ifFalse: [^self retry: #/ coercing: aWorkspace]
!

rem: aWorkspace
	^self \\ aWorkspace
!

\\ aWorkspace
	(aWorkspace isMemberOf: self class)
	ifTrue: [^GXworkspaceUmodulo run line: 0 proc: self class method: true; p1: self; p2: aWorkspace; condEvaluate]
	ifFalse: [^self retry: #\\ coercing: aWorkspace]
!

raisedTo: aWorkspace
	(aWorkspace isMemberOf: self class)
	ifTrue: [^GXworkspaceUraisedUto run line: 0 proc: self class method: true; p1: self; p2: aWorkspace; condEvaluate]
	ifFalse: [^self retry: #raisedTo: coercing: aWorkspace]
!

not
	^GXworkspaceUnot run line: 0 proc: self class method: true; p1: self; condEvaluate
!

negated
	^GXworkspaceUnegated run line: 0 proc: self class method: true; p1: self; condEvaluate
!


"Unary functions extended from smalltalk generics to workspaces
 =============================================================="

sin
	^GXworkspaceUsin run line: 0 proc: self class method: true; p1: self; condEvaluate
!

cos
	^GXworkspaceUcos run line: 0 proc: self class method: true; p1: self; condEvaluate
!

tan
	^GXworkspaceUtan run line: 0 proc: self class method: true; p1: self; condEvaluate
!

arcSin
	^GXworkspaceUarcsin run line: 0 proc: self class method: true; p1: self; condEvaluate
!

arcCos
	^GXworkspaceUarccos run line: 0 proc: self class method: true; p1: self; condEvaluate
!

arcTan
	^GXworkspaceUarctan run line: 0 proc: self class method: true; p1: self; condEvaluate
!

ln
	^GXworkspaceUln run line: 0 proc: self class method: true; p1: self; condEvaluate
!

exp
	^GXworkspaceUexp run line: 0 proc: self class method: true; p1: self; condEvaluate
!

log: base
	^GXworkspaceUlog run line: 0 proc: self class method: true; p1: self; condEvaluate
!

sqrt
	^GXworkspaceUsqrt run line: 0 proc: self class method: true; p1: self; condEvaluate
!

abs
	^GXworkspaceUabs run line: 0 proc: self class method: true; p1: self; condEvaluate
!!
nil
!


"GENIE MULTI-DIMENSIONAL ARRAY CLASS
 ================================================================================
 ================================================================================"
Array variableSubclass: 'MultiDimensionalArray'
	instanceVariableNames: 'dims dopeVector'
	classVariableNames: ''
	poolDictionaries: ''
	category:'genieV3'
!

MultiDimensionalArray comment:
'My instances inherit all the expected behaviour of a normal smalltalk Array object
 but extend the behaviour with multi-dimesional access using a dope vector. Normal
 Array operations ignore my multi-dimensional data and so may make my contents
 inconsistent with my expected behaviour as a multi-dimensional array.  My instances
 are stored so that the last index is the fastest changing (like "C" or Pascal)'
!


"CLASS METHODS (public)
 ================================================================================"
!MultiDimensionalArray class methodsFor: 'creation'!

"---Create a new MDA---

	Wherever the message variables `anArrayOrInteger' appear, the argument may
	be either an integer or an instance of Array class containing integer
	indices.  In this case the indices are used to size the dimensions ie

		new: #(3 4 5)

	creates a Three-dimensional array with indices varying 1-3, 1-4 and 1-5."



new: anArrayOrInteger
	| length boundsArray |
	boundsArray _ self convertIndex: anArrayOrInteger.
	length _ 1.
	1 to: boundsArray size do: [ :i | length _ length * (boundsArray at: i) ].
	^(self basicNew: length) setDims: boundsArray
!


"---Provide a means of hiding the difference between integer and array indices---

	Convert an integer index into an array of 1 element to
	be consistent with multi-dimensional indexing (This is
	defined as a class method to allow access from both class and
	object methods)."


convertIndex: anArrayOrInteger
        | temp |
        ( anArrayOrInteger isKindOf: Integer )
                ifFalse: [ ^anArrayOrInteger ]
                ifTrue: [
                        temp _ Array new: 1.
                        temp at:1 put: anArrayOrInteger.
                        ^temp
                        ]
!!



"INSTANCE METHODS (public)
 ================================================================================"
!MultiDimensionalArray methodsFor: 'accessing'!

"---Augmented versions of at: and at:put: messages---

	These function transparently with integer indices but act
	multi-dimensionally with array indices"


at: idx	
    |tmp|
    ( idx isKindOf: Integer )
	ifTrue:  [ ^self basicAt: idx ].
    ( idx isMemberOf: Array )
	ifTrue:
        [ 
	    tmp _ idx at: 1.
            ( tmp isMemberOf: Interval )
		ifTrue: [
"I'm making 1 a special case as this can use existing code
 and we are quite near a release. The larger one was broken
 anyway, so no harm by changing it i hope"
		    ( (idx size) = 1 )
			ifTrue: [
                	    ^self copyFrom: tmp start to: tmp stop 
			] ifFalse: [
                	    ^self intervalCopy: idx
			]
		].
            ^self basicAt: (self indexInto: idx)
        ].
	^self error: (self error1: idx)
!

at: idx put: aValue
	( idx isKindOf: Integer )
		ifTrue:  [ ^self basicAt: idx put: aValue ].
    ( idx isMemberOf: Array )
		ifTrue:
        [ 
            ( (idx at: 1) isMemberOf: Interval )
                ifTrue: [ ^self copyReplaceFrom: (idx at: 1) start to: (idx at: 1) stop with: aValue ].
            ^self basicAt: (self indexInto: idx) put: aValue
        ].
	^self error: (self error1: idx)
!

collect: aBlock
	| tmp |
	tmp _ self class new: dims.
	1 to: self size do: [ :i | tmp at: i put: ( aBlock value: ( self at: i ) ) ].
	^tmp
!

with: anArray collect: aBlock
	| tmp siz |
	tmp _ self class new: dims.
	self size > anArray size
		ifTrue: [ siz _ anArray size ]
		ifFalse: [ siz _ self size ].
	1 to: siz do: [ :i | tmp at: i put: ( aBlock value: ( self at: i ) value: ( anArray at: i) ) ].
	^tmp
!

intervalCopy: anIntervalArray
    ^self error: 'Multi-dimensional slicing is not implemented for this array type'
!!


"---Make some information about the array public---"

!MultiDimensionalArray methodsFor: 'informational'!

dimensionality
	^dims size
!

dimensions
	^dims
!

relatedArrayClass: anAtomicClass	"This is specific to GX types, nothing to do with MDA"
	self shouldNotImplement
!


"---Error messages for class---"

error1: offendingType
	( offendingType isMemberOf: Symbol )
	ifTrue:
		[ ^'Possibly an attempt to treat an array as a workspace !,',
		'	don''t understand "', offendingType printString gtype, '"' ]
	ifFalse:
		[ ^'Attempt to index an array with an invalid index type,',
		'	type was "', offendingType class printString gtype, '"' ]
!!



"INSTANCE METHODS (private)
 ================================================================================"
!MultiDimensionalArray methodsFor: 'private'!

"---Converts an array index of the form at: #( 1 3 4 )---

	Calculate the flat index given an array of multi-dimensional
	indices. This uses the stored dope vector created at array
	initialization time"

indexInto: indexArray
	| offset idx bound length |

	length _ indexArray size.
	length = dims size
		ifFalse: [ self error: 'number of indices supplied do not match declared array dimensions!' ]
		ifTrue:
		[
			offset _ 0.
			1 to: length do: [ :i |
				idx _ indexArray at: i.	
				bound _ dims at: i.
				(1 <= idx ) & ( idx <= bound )		" check bounds (exception for 1-d access)"
					ifFalse:[ self error: 'Array bounds exceeded no action taken'. ^nil ]
					ifTrue: [ offset _ offset + ((idx - 1) * (dopeVector at: i )) ]
				]
		].
	^offset+1
!


"---Initialize bounds checking and the dope vector---"

setDims: anArray
	| noDims |
	dims _ anArray copy.					"keep a copy for bounds checking"
	noDims _ dims size.					"get the number of dimensions"
	dopeVector _ (Array new: noDims) atAllPut: 1.
	noDims - 1 to: 1 by: -1 do: [ :i |
		dopeVector at: i put:  (dims at: i+1 ) * (dopeVector at: i+1)
		]
!
!


" The base GENIE array class
  =========================="

MultiDimensionalArray variableSubclass: 'GXArray'
	instanceVariableNames: '' classVariableNames: '' poolDictionaries: '' category: 'GX types'
!


" Declare all the GENIE subclasses of GXArray here so they can be used by all coercions etc.
  =========================================================================================="

GXArray variableSubclass: 'GXRealarray'
	instanceVariableNames: 'data' classVariableNames: '' poolDictionaries: '' category: 'GX types'.
"data instance variable is only used in flat array representation"

GXArray variableSubclass: 'GXIntegerarray'
	instanceVariableNames: 'data' classVariableNames: '' poolDictionaries: '' category: 'GX types'.
"data instance variable is only used in flat array representation"

GXArray variableSubclass: 'GXStringarray'
	instanceVariableNames: '' classVariableNames: '' poolDictionaries: '' category: 'GX types'.

GXArray variableSubclass: 'GXWorkspacearray'
	instanceVariableNames: '' classVariableNames: '' poolDictionaries: '' category: 'GX types'
!

"CLASS METHODS for GENIE arrays
 ================================="

!GXArray class methodsFor: 'type compatibility'!
asMe: aVar
	^nil
!!


"Provide a way of initializing and using the class variable Depth"
!GXWorkspacearray class methodsFor: 'Initializing'!

setDepth: depth
        Depth _ depth
!

setDepth
        Depth _ ''
!!


"Initialize the class variable"
GXWorkspacearray setDepth!



"INSTANCE METHODS for GENIE arrays (private)
 =========================================="

!GXArray methodsFor: 'information'!

"---Information on related classes---

	The GXArray class provides a neutral untyped array which is converted
	into one of the types below on assignment of an element. It is only
	an type between two states, new-ed and with elements!"


"---Return the Array class which would hold elements of the type being assigned---"

relatedArrayClass: anAtomicClass
	(anAtomicClass isKindOf: Integer)            ifTrue: [ ^GXIntegerarray ].
	(anAtomicClass isKindOf: Float)              ifTrue: [ ^GXRealarray ].
	(anAtomicClass isKindOf: String)             ifTrue: [ ^GXStringarray ].
	(anAtomicClass isKindOf: IdentityDictionary) ifTrue: [ ^GXWorkspacearray ].
	^nil
!!



!GXArray methodsFor: 'accessing'!

"---Type check for array (set type on first assignment)---"

at: anIndex put: aValue
	| clone expectedtype |

	expectedtype _ self relatedArrayClass: aValue.		"Find our new or expected type"
	expectedtype isNil & aValue notNil
	    ifTrue: [ self error: ('Cannot have arrays containing ', aValue class printString, 's') ]
	    ifFalse: [
		    ( self isMemberOf: GXArray ) & aValue notNil		"( no elements have been given yet to set type )"
			    ifTrue:[
				    clone _ expectedtype new: dims.		"create a clone of ourself as the new type"
				    self become: clone
				    ].

		    ( self isMemberOf: expectedtype ) | aValue isNil	"now type check"
			    ifFalse: [ self error: 'Can''t assign more than one type to an array' ]
			    ifTrue:  [ ^super at: anIndex put: aValue ]
	    ]
!

copyFrom: start to: stop
        (( start isMemberOf: Integer ) & ( stop isMemberOf: Integer ) & ( start  <= stop ))
            ifTrue:
            [
                ( self isMemberOf: GXIntegerarray ) | ( self isMemberOf: GXRealarray)
                ifTrue:
                    [^self basicCopyFrom: start to: stop]
                ifFalse:
                    [^super copyFrom: start to: stop]
                ].

        "If things don't work"
        self error: 'Array slice incorrectly specified'
!

"Best to coerce second array here to ensure we are filling with reals or integers appropriately"
replaceFrom: start to: stop with: anArray startingAt: repStart
             (( start isMemberOf: Integer )
              & ( stop isMemberOf: Integer )
              & ( repStart isMemberOf: Integer )
              & ( start  <= stop ))
            ifTrue:
            [
                ( self isMemberOf: GXIntegerarray ) | ( self isMemberOf: GXRealarray)
                ifTrue:
                [
                    (anArray isMemberOf: self class)
	                    ifTrue: [ ^self basicReplaceFrom: start to: stop with: anArray startingAt: repStart]
                        ifFalse: [^self retry: #replaceFrom:to:with:startingAt: coercing: anArray]
                ]
                ifFalse:
                    [^super replaceFrom: start to: stop with: anArray startingAt: repStart]
            ] .

        "If things don't work"
        self error: 'Array slice incorrectly specified'
!!



"NUMERICAL ARRAY OPERATIONS
 ==========================

	Could do with multiple inheritance here to make arrays behave like numbers but
	we can cheat and do a roughly similar job by nicking the #retry:coercing from Number and adding
	to the generality hierachy"


GXArray compile: NumberRetryCoercingMethodString	"Borrow the code from Number for this (works for all arrays)"
!


!GXArray methodsFor: 'conversion'!

extend: aSomething						"Extend aSomething into an array containing aSomethings"
	| temp |
	temp _ self class new: dims.
	^temp atAllPut: aSomething
!

"aStep and aMode used in derived numerical array types"
extend: aSomething with: aStep mode: aMode
    ^self extend: aSomething
!

asFloat
	self subclassResponsibility
!!



"ALLOW OPTIMIZATION IN SUBCLASSES
 ================================

	All array operations work on all types (if defined for that type) because the basic collect:
	and with:collect: messages are defined by the MultiDimensionalArray class.  Almost by definition
	these are slow because they have to evaluate a block for each element of the array.

	By defining a message with an extra parameter (hint:) we can default to the more general but slower
	operations where there are no subclass definitions or alternatively implement faster operations in
	subclasses.  The low speed defaults are defined here where hint: is ignored."

!GXArray methodsFor: 'conversion'!

with: anArray collect: aBlock hint: aChar
	^self with: anArray collect: aBlock
!

collect: aBlock hint: aChar
	^self collect: aBlock
!!



"PRINTING ARRAYS
 ==============="

"Only print out the first few elements of arrays, this is just to allow inspection, not
 much more!"

!GXArray methodsFor: 'printing'!

printOn: aStream
        | len elip |
        self size > gXUarrayUelementsUtoUprint
                ifTrue: [ len _ gXUarrayUelementsUtoUprint. elip _ '...' ]
                ifFalse:[ len _ self size. elip _ '' ].
        aStream nextPutAll: '[' printString.
        1 to: len do: [ :i | aStream nextPutAll: ((self at: i) printString, ' ' ) ].
        aStream nextPutAll: elip, '] Array', dims printString
!


"Formatted printing here is already covered by printOn so this is just for completeness"
fprintOn: aStream
        self printOn: aStream.
!!



"customize printing for arrays of workspaces.  These are a special case as they need to
 be printed multiline and hence have to take account of indentation depth"

!GXWorkspacearray methodsFor: 'printing'!

printOn: aStream depth: depth
        | keep |
        keep _ Depth.
        Depth _ Depth, depth.
        self printOn: aStream.
        self class setDepth: keep
!


printOn: aStream
    | v index className |
    className _ self class name.
    aStream nextPutAll: Depth, '  ', className gtype.
    aStream nl.
    aStream nextPutAll: Depth, '  ['.
    1 to: self size do:
      [ :i |
	aStream nl.
               (Depth, '    ') printOn: aStream.
               index _ i printString.
               index printOn: aStream.
               aStream nextPutAll: ' = '.
               v _ self basicAt: i.
               (v isKindOf: GXWorkspace ) | (v isKindOf: GXWorkspacearray)
                   ifTrue:
                     [
                       aStream nl.
                       v printOn: aStream depth: '    '
                     ]
                   ifFalse:
                     [
                       v fprintOn: aStream
                     ]
      ].
    aStream nl.
    aStream nextPutAll: Depth, '  ]'
!!



"ARITHMETIC OPERATIONS ON ARRAYS
 ===============================

	This is the teeth of the code on which GENIE operations are based.  Hopefully it
	provides a more rigourous and orthogonal definition of that which was intuitive
	in GENIE-V2."


!GXArray methodsFor: 'arithmetic'!

"BINARY OPERATIONS
 ================="

, anArray
    ^(self species new: (self size + anArray size))
        replaceFrom: 1 to: self size with: self startingAt: 1;
        replaceFrom: self size + 1 to: self size + anArray size with: anArray startingAt: 1
!

> anArray
	^self size > anArray size
!

< anArray
	^self size < anArray size
!

>= anArray
	^self size >= anArray size
!

<= anArray
	^self size <= anArray size
!

"NOTE: both = and ~= are inherited without modification"

+ anArray
	(anArray isMemberOf: self class)
	ifTrue: [ ^self with: anArray collect: [ :i :j | i + j ] hint: $+ ]
	ifFalse: [^self retry: #+ coercing: anArray]
!

- anArray
	(anArray isMemberOf: self class)
	ifTrue: [ ^self with: anArray collect: [ :i :j | i - j ] hint: $- ]
	ifFalse: [^self retry: #- coercing: anArray]
!

* anArray
	(anArray isMemberOf: self class)
	ifTrue: [ ^self with: anArray collect: [ :i :j | i * j ] hint: $* ]
	ifFalse: [^self retry: #* coercing: anArray]
!

/ anArray
	(anArray isMemberOf: self class)
	ifTrue: [ ^self with: anArray collect: [ :i :j | j = 0 ifTrue: [ nil ] ifFalse: [ i / j ] ] hint: $/ ]
	ifFalse: [^self retry: #/ coercing: anArray]
!

rem: anArray	" remainder "
	(anArray isMemberOf: self class)
	ifTrue: [ ^self with: anArray collect: [ :i :j | j = 0 ifTrue: [ nil ] ifFalse: [ i rem: j ] ] hint: $\ ]
	ifFalse: [^self retry: #rem: coercing: anArray]
!

raisedTo: anArray
	(anArray isMemberOf: self class)
	ifTrue:  [ ^self with: anArray collect: [ :i :j | i raisedTo: j ] hint: $^ ]
	ifFalse: [ ^self retry: #raisedTo: coercing: anArray ]
!

"SEMI-UNARY OPERATIONS
 ====================="

"This could be made a full binary operation but as it works with strings we would have to make
 string arrays respond to generality or give up on strings! "

fix: aValue
	^self with: (self extend: aValue) collect: [ :i :j | i isNil ifFalse: [ i ] ifTrue: [ j ] ] hint: $f
!


"UNARY OPERATIONS
 ================="

negated
	^self collect: [ :i | i negated ] hint: $n
!

sin
	^self asFloat collect: [ :i | i sin ] hint: $s
!

cos
	^self asFloat collect: [ :i | i cos ] hint: $c
!

tan
	^self asFloat collect: [ :i | i tan ] hint: $t
!

arcSin
	^self asFloat collect: [ :i | ( ( -1 < i ) & ( i < 1 ) ) ifFalse: [ nil ] ifTrue: [ i arcSin ] ] hint: $S
!

arcCos
	^self asFloat collect: [ :i | ( ( -1 < i ) & ( i < 1 ) ) ifFalse: [ nil ] ifTrue: [ i arcCos ] ] hint: $C
!

arcTan
	^self asFloat collect: [ :i | i arcTan ] hint: $T
!

ln
	^self asFloat collect: [ :i | ( i > 0 ) ifFalse: [ nil ] ifTrue: [ i ln ] ] hint: $l
!

exp
	^self asFloat collect: [ :i | i exp ] hint: $e
!

log: base
	^self asFloat collect: [ :i | ( i > 0 ) ifFalse: [ nil ] ifTrue: [ i log: base ] ] hint: $L
!

sqrt
	^self asFloat collect: [ :i | ( i >= 0 ) ifFalse: [ nil ] ifTrue: [ i sqrt ] ] hint: $q
!

abs
	^self asFloat collect: [ :i | i abs ] hint: $a
!

"UNARY reductions
 ================="

maximum
        | max |
        max _ self at: 1.
        self do: [ :i | ( i > max ) ifTrue: [ max _ i ] ].
	^max
!

minimum
        | min |
        min _ self at: 1.
        self do: [ :i | ( i < min ) ifTrue: [ min _ i ] ].
	^min
!

sum
        | sum |
        sum _ (self at: 1) - (self at: 1).	"Copes with real,integer and workspaces this way"
        self do: [ :i | sum _ sum + i ].
	^sum
!!
nil
!



" Class methods
  ============="

!GXIntegerarray class methodsFor: 'creation'!

basicNew: anInt
        ^((super basicNew: 1) setByteArray: (ByteArray new: (GXIntegerarray sizeOfElement * anInt))) unfix: 0
!! 


"Object Methods
 =============="

!GXIntegerarray methodsFor: 'Operating'!

deepCopy
	^self shallowCopy
!

generality
	^5
!

size		"Number of elements in the array"
	^(data size) / (GXIntegerarray sizeOfElement)
!

coerce: aSomething			        "A something will be an operable object (ie something responding to generality)"
	(aSomething isKindOf: Array)
		ifTrue: [^self asFloat].        "(Obviously if it was an integer array we wouldn't be here anyway!)"
	(aSomething isKindOf: Integer)
		ifTrue: [^self extend: aSomething].
	(aSomething isKindOf: Float)
		ifTrue: [^self asFloat extend: aSomething]

"
# Kept here for historical reasons, this is the only piece
# of redundant code which must be replaced if the primitive
# callouts are removed. Everything else is handled in the
# superclass GXArray
# 
# asFloat
#	^(GXRealarray new: self size) with: self collect: [ :i :j | j notNil ifTrue: [j asFloat] ]
# !!
"

!!


!GXIntegerarray methodsFor: 'object primitives'!

setByteArray: anArray				"assign a byteArray to a new flat array object"
	data _ anArray
!
!



" Class Methods
  ============="

!GXRealarray class methodsFor: 'creation'!

basicNew: anInt
        ^((super basicNew: 1) setByteArray: (ByteArray new: (GXRealarray sizeOfElement * anInt))) unfix: 0.0
!! 


"Object Methods
 =============="


!GXRealarray methodsFor: 'Operating'!

deepCopy
	^self shallowCopy
!

generality
	^6
!

coerce: aSomething				"A something will be an operable object (ie something responding to generality)"
	(aSomething isKindOf: Array)
		ifTrue: [^aSomething asFloat].
	(aSomething isKindOf: Integer)
		ifTrue: [^self extend: aSomething asFloat].
	(aSomething isKindOf: Float)
		ifTrue: [^self extend: aSomething]
!

asFloat         "Only here because asFloat is subclass responsibility in GXArray operations"
        ^self
!

size		"Number of elements in the array"
	^(data size) / (GXRealarray sizeOfElement)
!!


!GXRealarray methodsFor: 'object primitives'!

setByteArray: anArray				"assign a byteArray to a new flat array object"
	data _ anArray
!
!


"GENIE TYPE CLASS DEFINITIONS
 ============================
 ============================

	All genie types are declared as subclasses even though not all are
	extended in any way.  This is just to make a clean sweep, especially
	as they may all be extended in the future.

	As yet, there is no specific extra functionality for the classes defined
	in this file."


Integer subclass: 'GXInteger'
instanceVariableNames: '' classVariableNames: '' poolDictionaries: '' category: 'GX types'
!

"Type checking subclasses of Symbol"
Symbol variableByteSubclass: 'GXColour'
	instanceVariableNames: '' classVariableNames: '' poolDictionaries: '' category: 'GX types'.

Symbol variableByteSubclass: 'GXFont'
	instanceVariableNames: '' classVariableNames: '' poolDictionaries: '' category: 'GX types'.

Symbol variableByteSubclass: 'GXMarker'
	instanceVariableNames: '' classVariableNames: '' poolDictionaries: '' category: 'GX types'.

Symbol variableByteSubclass: 'GXLinestyle'
	instanceVariableNames: '' classVariableNames: '' poolDictionaries: '' category: 'GX types'.

Float variableByteSubclass: 'GXReal'
	instanceVariableNames: '' classVariableNames: '' poolDictionaries: '' category: 'GX types'.

String variableByteSubclass: 'GXString'
	instanceVariableNames: '' classVariableNames: '' poolDictionaries: '' category: 'GX types'
!


"TYPE COMPATIBILITIES - (specify explicitly)
 ===========================================

	These must be specified explicitly as it is not obvious how to change the
	class of a literal typed in at the keyboard and these are in the superclass
	of the GENIE types."

!GXInteger class methodsFor: 'type compatibility'!
species
	^Integer class
!

"Coerce if possible"
asMe: aVal
        (aVal class = Symbol)		"Used for enumerated types such as fonts and colours"
        ifTrue: [ 'Converting Symbol to integer
     - this worked OK but you probably need a Font, LineStyle, Colour or Marker type' prdnl. ^aVal asVariable ].
        ^nil

!!

!GXColour class methodsFor: 'type compatibility'!
asMe: aVal
        ^nil
!

species
	^Symbol class
!!


!GXFont class methodsFor: 'type compatibility'!
asMe: aVal
        ^nil
!

species
	^Symbol class
!!


!GXMarker class methodsFor: 'type compatibility'!
asMe: aVal
        ^nil
!

species
	^Symbol class
!!


!GXLinestyle class methodsFor: 'type compatibility'!
asMe: aVal
        ^nil
!

species
	^Symbol class
!!


!GXReal class methodsFor: 'type compatibility'!
species
	^Float class
!

"Coerce if possible"
asMe: aVal
        (aVal species = Integer)
        ifTrue: [ 'Converting Integer parameter to Real' prinl. ^aVal asFloat ].
        ^nil

!!

!GXString class methodsFor: 'type compatibility'!
species
	^String class
!

"Coerce if possible"
asMe: aVal
        ^nil

!!
nil
!


"MODULE CODES"
"============================================================================================="

"lowSystem callout"


"intOperations callout (doesn't use a callout list)"
Smalltalk at: #gxIntegerarrayExtend put: 2.
Smalltalk at: #gxIntegerarrayAsFloat put: 3.
Smalltalk at: #gxIntegerarrayCollectHint put: 7.
Smalltalk at: #gxIntegerarrayWithCollectHint put: 8.
Smalltalk at: #gxIntegerarrayEquals put: 9.
Smalltalk at: #gxIntegerarrayShallowCopy put: 10.
Smalltalk at: #gxIntegerarrayMaximum put: 15.
Smalltalk at: #gxIntegerarrayMinimum put: 16.
Smalltalk at: #gxIntegerarraySum put: 17.
Smalltalk at: #gxIntegerarrayFix put: 19.
Smalltalk at: #gxIntegerarrayAt put: 22.
Smalltalk at: #gxIntegerarrayAtPut put: 23.
Smalltalk at: #gxIntegerarrayElementSize put: 25.
Smalltalk at: #gxIntegerarrayExtendWithMode put: 26.
Smalltalk at: #gxIntegerarrayUnfix put: 28.
Smalltalk at: #gxIntegerarrayCopyFromTo put: 30.
Smalltalk at: #gxIntegerarrayReplaceFromToWithStartingAt put: 32.
Smalltalk at: #gxIntegerarrayApplyFunction put: 34
!
Smalltalk at: #gxRealarrayExtend put: 1.
Smalltalk at: #gxRealarrayCollectHint put: 4.
Smalltalk at: #gxRealarrayWithCollectHint put: 5.
Smalltalk at: #gxRealarrayEquals put: 6.
Smalltalk at: #gxRealarrayShallowCopy put: 11.
Smalltalk at: #gxRealarrayMaximum put: 12.
Smalltalk at: #gxRealarrayMinimum put: 13.
Smalltalk at: #gxRealarraySum put: 14.
Smalltalk at: #gxRealarrayFix put: 18.
Smalltalk at: #gxRealarrayAt put: 20.
Smalltalk at: #gxRealarrayAtPut put: 21.
Smalltalk at: #gxRealarrayElementSize put: 24.
Smalltalk at: #gxRealarrayExtendWithMode put: 27.
Smalltalk at: #gxRealarrayUnfix put: 29.
Smalltalk at: #gxRealarrayCopyFromTo put: 31.
Smalltalk at: #gxRealarrayReplaceFromToWithStartingAt put: 33.
Smalltalk at: #gxRealarrayApplyFunction put: 35.
Smalltalk at: #gxRealarrayCentre put: 36.
Smalltalk at: #gxRealarrayCut put: 37.
Smalltalk at: #gxRealarrayBracket put: 38
!

"intExtensions callout (doesn't use a callout list)"
Smalltalk at: #gxStringAsVariable put: 1.
Smalltalk at: #gxStringPrint put: 2.
Smalltalk at: #gxStreamMode put: 3.
Smalltalk at: #gxMangleAs put: 4.
Smalltalk at: #gxDemangle put: 5.
Smalltalk at: #gxQuitTheLot put: 6.
Smalltalk at: #gxLoadTheCode put: 7.
Smalltalk at: #gxStringRead put: 8.
Smalltalk at: #gxAddCommndToReadline put: 9
! 

"METHOD DEFINITIONS"

"Function"
"============================================================================================="
!Function methodsFor: 'Session control'!

quitTheLot
    ^intExtensions
        function: gxQuitTheLot
        parameters: nil
!

loadTheCode: fromFile
    ^intExtensions
        function: gxLoadTheCode
        parameters: ((Array new: 1)
                        at: 1 put: fromFile;
                        yourself)
!!



"String"
"============================================================================================="

!String methodsFor: 'genie'!

asVariable
        ^intExtensions
                function: gxStringAsVariable
                parameters: ((Array new: 1)
                        at: 1 put: self;
                        yourself)
!

stringPrint: aDescriptor
        ^intExtensions
                function: gxStringPrint
                parameters: ((Array new: 2)
                        at: 1 put: self printString;
                        at: 2 put: aDescriptor;
                        yourself)
!

streamMode: aDescriptor mode: anInteger
        ^intExtensions
                function: gxStreamMode
                parameters: ((Array new: 2)
                        at: 1 put: aDescriptor;
                        at: 2 put: anInteger;
                        yourself)
!

stringPrint
        ^intExtensions
                function: gxStringPrint
                parameters: ((Array new: 2)
                        at: 1 put: self printString;
                        at: 2 put: 'o';
                        yourself)
!

stringRead
        ^intExtensions
                function: gxStringRead
                parameters: nil
!

mangleAs: aString
        ^intExtensions
                function: gxMangleAs
                parameters: ((Array new: 2)
                        at: 1 put: self;
                        at: 2 put: aString;
                        yourself)
!

demangle
        ^intExtensions
                function: gxDemangle
                parameters: ((Array new: 1)
                        at: 1 put: self printString;
                        yourself)
!

addCommandToReadline: aComment
	^intExtensions
                function: gxAddCommndToReadline
                parameters: ((Array new: 2)
                        at: 1 put: self;
                        at: 2 put: aComment;
                        yourself)
!

addCommandToReadline
	^self addCommandToReadline: ''


!!


"GXIntegerarray class"
"============================================================================================="

!GXIntegerarray class methodsFor: 'class primitives'!

sizeOfElement
        ^intOperations
                function: gxIntegerarrayElementSize
                parameters: nil
!!


"gxIntegerarray"
"============================================================================================="

!GXIntegerarray methodsFor: 'object primitives'!

extend: anInteger        "Extends an integer into a new, filled array of the same size as self"
        ^intOperations
                function: gxIntegerarrayExtend
                parameters: ((Array new: 2)
                        at: 1 put: self;
                        at: 2 put: anInteger;
                        yourself)
! 

extend: anInteger  with: aStep  mode: aMode    "Extends an integer into a new, filled array of the same size as self"
        ^intOperations
                function: gxIntegerarrayExtendWithMode
                parameters: ((Array new: 4)
                        at: 1 put: self;
                        at: 2 put: anInteger;
                        at: 3 put: aStep;
                        at: 4 put: aMode;
                        yourself)
! 

asFloat
	^intOperations
	        function: gxIntegerarrayAsFloat
	        parameters: ((Array new: 1)
                        at: 1 put: self;
                        yourself)
!

collect: aBlock hint: aSymbol
	^intOperations
	        function: gxIntegerarrayCollectHint
	        parameters: ((Array new: 3)
                        at: 1 put: self;
                        at: 2 put: aBlock;
                        at: 3 put: aSymbol;
                        yourself)
!

with: anArray collect: aBlock hint: aSymbol
	^intOperations
	        function: gxIntegerarrayWithCollectHint
	        parameters: ((Array new: 4)
                        at: 1 put: self;
                        at: 2 put: anArray;
                        at: 3 put: aBlock;
                        at: 4 put: aSymbol;
                        yourself)
!

= anArray                  "(Note: ~= is inherited automatically)"
	^intOperations
	        function: gxIntegerarrayEquals
	        parameters: ((Array new: 2)
                        at: 1 put: self;
                        at: 2 put: anArray;
                        yourself)
!

shallowCopy
	^intOperations
	        function: gxIntegerarrayShallowCopy
	        parameters: ((Array new: 1)
                        at: 1 put: self;
                        yourself)
!

maximum
	^intOperations
	        function: gxIntegerarrayMaximum
	        parameters: ((Array new: 1)
                        at: 1 put: self;
                        yourself)
!

minimum
	^intOperations
	        function: gxIntegerarrayMinimum
	        parameters: ((Array new: 1)
                        at: 1 put: self;
                        yourself)
!

sum
	^intOperations
	        function: gxIntegerarraySum
	        parameters: ((Array new: 1)
                        at: 1 put: self;
                        yourself)
!

fix: anInteger
	^intOperations
	        function: gxIntegerarrayFix
	        parameters: ((Array new: 2)
                        at: 1 put: self;
                        at: 2 put: anInteger;
                        yourself)
!

unfix: anInteger
	^intOperations
	        function: gxIntegerarrayUnfix
	        parameters: ((Array new: 2)
                        at: 1 put: self;
                        at: 2 put: anInteger;
                        yourself)
!

basicCopyFrom: start to: stop
	^intOperations
	        function: gxIntegerarrayCopyFromTo
	        parameters: ((Array new: 3)
                        at: 1 put: self;
                        at: 2 put: start;
                        at: 3 put: stop;
                        yourself)
!

basicReplaceFrom: start to: stop with: anArrayOrInteger startingAt: repStart
	^intOperations
	        function: gxIntegerarrayReplaceFromToWithStartingAt
	        parameters: ((Array new: 5)
                        at: 1 put: self;
                        at: 2 put: start;
                        at: 3 put: stop;
                        at: 4 put: anArrayOrInteger;
                        at: 5 put: repStart;
                        yourself)
!

basicAt: index
        ^intOperations
                function: gxIntegerarrayAt
                parameters: ((Array new: 2)
                        at: 1 put: self;
                        at: 2 put: index;
                        yourself)
!

basicAt: index put: anInteger
        ^intOperations
                function: gxIntegerarrayAtPut
                parameters: ((Array new: 3)
                        at: 1 put: self;
                        at: 2 put: index;
                        at: 3 put: anInteger;
                        yourself)
!

applyFunction: aString
	^intOperations
		function: gxIntegerarrayApplyFunction
		parameters: ((Array new: 2)
			at: 1 put: self;
			at: 2 put: aString;
			yourself)
!!


"GXRealarray class"
"============================================================================================="

!GXRealarray class methodsFor: 'class primitives'!

sizeOfElement
        ^intOperations
                function: gxRealarrayElementSize
                parameters: nil
!!


"gxRealarray"
"============================================================================================="


!GXRealarray methodsFor: 'object primitives'!

extend: aFloat                   "Extends an real into a new, filled array of the same size as self"
        ^intOperations
                function: gxRealarrayExtend
                parameters: ((Array new: 2)
                        at: 1 put: self;
                        at: 2 put: aFloat;
                        yourself)
! 

extend: aFloat  with: aStep  mode: aMode    "Extends an real into a new, filled array of the same size as self"
        ^intOperations
                function: gxRealarrayExtendWithMode
                parameters: ((Array new: 4)
                        at: 1 put: self;
                        at: 2 put: aFloat;
                        at: 3 put: aStep;
                        at: 4 put: aMode;
                        yourself)
! 

collect: aBlock hint: aSymbol
	^intOperations
	        function: gxRealarrayCollectHint
	        parameters: ((Array new: 3)
                        at: 1 put: self;
                        at: 2 put: aBlock;
                        at: 3 put: aSymbol;
                        yourself)
!

centre
	^intOperations
		function: gxRealarrayCentre
		parameters: ((Array new: 1)
			at: 1 put: self;
			yourself)
!

bracket: value
	^intOperations
		function: gxRealarrayBracket
		parameters: ((Array new: 2)
			at: 1 put: self;
			at: 2 put: value;
			yourself)
!

"Fix axis dimension at value, and return the reduced dimensionality array"
cut: dimension with: value
	^intOperations
		function: gxRealarrayCut
		parameters: ((Array new: 3)
			at: 1 put: self;
			at: 2 put: dimension;
			at: 3 put: value;
			yourself)
!

with: anArray collect: aBlock hint: aSymbol
	^intOperations
	        function: gxRealarrayWithCollectHint
	        parameters: ((Array new: 4)
                        at: 1 put: self;
                        at: 2 put: anArray;
                        at: 3 put: aBlock;
                        at: 4 put: aSymbol;
                        yourself)
!

= anArray                  "(Note: ~= is inherited automatically)"
	^intOperations
	        function: gxRealarrayEquals
	        parameters: ((Array new: 2)
                        at: 1 put: self;
                        at: 2 put: anArray;
                        yourself)
!

shallowCopy
	^intOperations
	        function: gxRealarrayShallowCopy
	        parameters: ((Array new: 1)
                        at: 1 put: self;
                        yourself)
!

maximum
	^intOperations
	        function: gxRealarrayMaximum
	        parameters: ((Array new: 1)
                        at: 1 put: self;
                        yourself)
!

minimum
	^intOperations
	        function: gxRealarrayMinimum
	        parameters: ((Array new: 1)
                        at: 1 put: self;
                        yourself)
!

sum
	^intOperations
	        function: gxRealarraySum
	        parameters: ((Array new: 1)
                        at: 1 put: self;
                        yourself)
!

fix: aFloat
	^intOperations
	        function: gxRealarrayFix
	        parameters: ((Array new: 2)
                        at: 1 put: self;
                        at: 2 put: aFloat;
                        yourself)
!

unfix: aFloat
	^intOperations
	        function: gxRealarrayUnfix
	        parameters: ((Array new: 2)
                        at: 1 put: self;
                        at: 2 put: aFloat;
                        yourself)
!

basicCopyFrom: start to: stop
	^intOperations
	        function: gxRealarrayCopyFromTo
	        parameters: ((Array new: 3)
                        at: 1 put: self;
                        at: 2 put: start;
                        at: 3 put: stop;
                        yourself)
!

basicReplaceFrom: start to: stop with: anArrayOrReal startingAt: repStart
	^intOperations
	        function: gxRealarrayReplaceFromToWithStartingAt
	        parameters: ((Array new: 5)
                        at: 1 put: self;
                        at: 2 put: start;
                        at: 3 put: stop;
                        at: 4 put: anArrayOrReal;
                        at: 5 put: repStart;
                        yourself)
!

basicAt: index
        ^intOperations
                function: gxRealarrayAt
                parameters: ((Array new: 2)
                        at: 1 put: self;
                        at: 2 put: index;
                        yourself)
!

basicAt: index put: aFloat
        ^intOperations
                function: gxRealarrayAtPut
                parameters: ((Array new: 3)
                        at: 1 put: self;
                        at: 2 put: index;
                        at: 3 put: aFloat;
                        yourself)
!

applyFunction: aString
	^intOperations
		function: gxRealarrayApplyFunction
		parameters: ((Array new: 2)
			at: 1 put: self;
			at: 2 put: aString;
			yourself)
!!
nil
!
"Procedure class definition:
 ==========================="
((Smalltalk at: #GXcomment ifAbsent: [ nil ]) isKindOf: Function class) ifTrue: [ ('Warning: Overriding previously defined procedure ', 'GXcomment' gtype) prinl ].
Smalltalk at: #GXcomment put: nil!
Function subclass: 'GXcomment'
       instanceVariableNames: ''
       classVariableNames: ''
       poolDictionaries: ''
       category: 'Genie Procedure'
!

"HEADER
 ------"

"parameters
 ----------"

GXcomment

addInstVarName: #gXprocUname;
compile:
'gXprocUname: aVar
	aVar notNil ifTrue:[gXprocUname _ self typeCheck: aVar shouldBe: GXString paramName: ''gXprocUname'']';
compile:
'p1: aVar
	aVar notNil ifTrue:[gXprocUname _ self typeCheck: aVar shouldBe: GXString paramName: ''gXprocUname'']';

addInstVarName: #gXcommentUstring;
compile:
'gXcommentUstring: aVar
	aVar notNil ifTrue:[gXcommentUstring _ self typeCheck: aVar shouldBe: GXString paramName: ''gXcommentUstring'']';
compile:
'p2: aVar
	aVar notNil ifTrue:[gXcommentUstring _ self typeCheck: aVar shouldBe: GXString paramName: ''gXcommentUstring'']'
!

!GXcomment methodsFor: 'initializing'!

initVars

!!

"End of procedure"

GXcomment run line: 0 proc: self method: false; p1:'comment'; p2:'Procedure which adds the comments you are reading !'; condEvaluate!
"Procedure class definition:
 ==========================="
((Smalltalk at: #GXcalledUasUfunction ifAbsent: [ nil ]) isKindOf: Function class) ifTrue: [ ('Warning: Overriding previously defined procedure ', 'GXcalledUasUfunction' gtype) prinl ].
Smalltalk at: #GXcalledUasUfunction put: nil!
Function subclass: 'GXcalledUasUfunction'
       instanceVariableNames: ''
       classVariableNames: ''
       poolDictionaries: ''
       category: 'Genie Procedure'
!

"HEADER
 ------"

"result
 ------"

GXcalledUasUfunction

addInstVarName: #gXres
!

!GXcalledUasUfunction methodsFor: 'initializing'!

initVars

!!

"BODY
 ----"

!GXcalledUasUfunction methodsFor: 'running'!

evaluate
 gXres _ ((line at:1) instVarAt: 1) at: 3.
self up.
^gXres
!!
nil! "(fix for smalltalk bug, this line is ignored if there is a variable declaration error)"

"End of procedure"

GXcomment run line: 0 proc: self method: false; p1:'called_as_function'; p2:'Returns true if the parent was called as a function'; condEvaluate!
"Procedure class definition:
 ==========================="
((Smalltalk at: #GXdebug ifAbsent: [ nil ]) isKindOf: Function class) ifTrue: [ ('Warning: Overriding previously defined procedure ', 'GXdebug' gtype) prinl ].
Smalltalk at: #GXdebug put: nil!
Function subclass: 'GXdebug'
       instanceVariableNames: ''
       classVariableNames: ''
       poolDictionaries: ''
       category: 'Genie Procedure'
!

"HEADER
 ------"

"qualifiers
 ----------"

GXdebug

addInstVarName: #gXon;
compile:
'gXon
	gXon _ true';
compile:
'gXon: aBoolean
	gXon _ aBoolean';

addInstVarName: #gXoff;
compile:
'gXoff
	gXoff _ true';
compile:
'gXoff: aBoolean
	gXoff _ aBoolean'
!

!GXdebug methodsFor: 'initializing'!

initVars
	gXon _ false.
	gXoff _ false
!!

"BODY
 ----"

!GXdebug methodsFor: 'running'!

evaluate
(gXoff) ifTrue: [ 
gXUdebug _ (gXDfalse) deepCopy.
 '' streamMode: 'd' mode: 0  ] ifFalse: [ 
gXUdebug _ (gXDtrue) deepCopy.
 '' streamMode: 'd' mode: 1  ] .
self up.
^nil
!!
nil! "(fix for smalltalk bug, this line is ignored if there is a variable declaration error)"

"End of procedure"

"Procedure class definition:
 ==========================="
((Smalltalk at: #GXcopying ifAbsent: [ nil ]) isKindOf: Function class) ifTrue: [ ('Warning: Overriding previously defined procedure ', 'GXcopying' gtype) prinl ].
Smalltalk at: #GXcopying put: nil!
Function subclass: 'GXcopying'
       instanceVariableNames: ''
       classVariableNames: ''
       poolDictionaries: ''
       category: 'Genie Procedure'
!

"BODY
 ----"

!GXcopying methodsFor: 'running'!

evaluate

'
		    GNU GENERAL PUBLIC LICENSE
		       Version 2, June 1991

 Copyright (C) 1989, 1991 Free Software Foundation, Inc.
                          675 Mass Ave, Cambridge, MA 02139, USA
 Everyone is permitted to copy and distribute verbatim copies
 of this license document, but changing it is not allowed.

			    Preamble

  The licenses for most software are designed to take away your
freedom to share and change it.  By contrast, the GNU General Public
License is intended to guarantee your freedom to share and change free
software--to make sure the software is free for all its users.  This
General Public License applies to most of the Free Software
Foundation''s software and to any other program whose authors commit to
using it.  (Some other Free Software Foundation software is covered by
the GNU Library General Public License instead.)  You can apply it to
your programs, too.

  When we speak of free software, we are referring to freedom, not
price.  Our General Public Licenses are designed to make sure that you
have the freedom to distribute copies of free software (and charge for
this service if you wish), that you receive source code or can get it
if you want it, that you can change the software or use pieces of it
in new free programs; and that you know you can do these things.

  To protect your rights, we need to make restrictions that forbid
anyone to deny you these rights or to ask you to surrender the rights.
These restrictions translate to certain responsibilities for you if you
distribute copies of the software, or if you modify it.

  For example, if you distribute copies of such a program, whether
gratis or for a fee, you must give the recipients all the rights that
you have.  You must make sure that they, too, receive or can get the
source code.  And you must show them these terms so they know their
rights.

  We protect your rights with two steps: (1) copyright the software, and
(2) offer you this license which gives you legal permission to copy,
distribute and/or modify the software.

  Also, for each author''s protection and ours, we want to make certain
that everyone understands that there is no warranty for this free
software.  If the software is modified by someone else and passed on, we
want its recipients to know that what they have is not the original, so
that any problems introduced by others will not reflect on the original
authors'' reputations.

  Finally, any free program is threatened constantly by software
patents.  We wish to avoid the danger that redistributors of a free
program will individually obtain patent licenses, in effect making the
program proprietary.  To prevent this, we have made it clear that any
patent must be licensed for everyone''s free use or not licensed at all.

  The precise terms and conditions for copying, distribution and
modification follow.

		    GNU GENERAL PUBLIC LICENSE
   TERMS AND CONDITIONS FOR COPYING, DISTRIBUTION AND MODIFICATION

  0. This License applies to any program or other work which contains
a notice placed by the copyright holder saying it may be distributed
under the terms of this General Public License.  The "Program", below,
refers to any such program or work, and a "work based on the Program"
means either the Program or any derivative work under copyright law:
that is to say, a work containing the Program or a portion of it,
either verbatim or with modifications and/or translated into another
language.  (Hereinafter, translation is included without limitation in
the term "modification".)  Each licensee is addressed as "you".

Activities other than copying, distribution and modification are not
covered by this License; they are outside its scope.  The act of
running the Program is not restricted, and the output from the Program
is covered only if its contents constitute a work based on the
Program (independent of having been made by running the Program).
Whether that is true depends on what the Program does.

  1. You may copy and distribute verbatim copies of the Program''s
source code as you receive it, in any medium, provided that you
conspicuously and appropriately publish on each copy an appropriate
copyright notice and disclaimer of warranty; keep intact all the
notices that refer to this License and to the absence of any warranty;
and give any other recipients of the Program a copy of this License
along with the Program.

You may charge a fee for the physical act of transferring a copy, and
you may at your option offer warranty protection in exchange for a fee.

  2. You may modify your copy or copies of the Program or any portion
of it, thus forming a work based on the Program, and copy and
distribute such modifications or work under the terms of Section 1
above, provided that you also meet all of these conditions:

    a) You must cause the modified files to carry prominent notices
    stating that you changed the files and the date of any change.

    b) You must cause any work that you distribute or publish, that in
    whole or in part contains or is derived from the Program or any
    part thereof, to be licensed as a whole at no charge to all third
    parties under the terms of this License.

    c) If the modified program normally reads commands interactively
    when run, you must cause it, when started running for such
    interactive use in the most ordinary way, to print or display an
    announcement including an appropriate copyright notice and a
    notice that there is no warranty (or else, saying that you provide
    a warranty) and that users may redistribute the program under
    these conditions, and telling the user how to view a copy of this
    License.  (Exception: if the Program itself is interactive but
    does not normally print such an announcement, your work based on
    the Program is not required to print an announcement.)

These requirements apply to the modified work as a whole.  If
identifiable sections of that work are not derived from the Program,
and can be reasonably considered independent and separate works in
themselves, then this License, and its terms, do not apply to those
sections when you distribute them as separate works.  But when you
distribute the same sections as part of a whole which is a work based
on the Program, the distribution of the whole must be on the terms of
this License, whose permissions for other licensees extend to the
entire whole, and thus to each and every part regardless of who wrote it.

Thus, it is not the intent of this section to claim rights or contest
your rights to work written entirely by you; rather, the intent is to
exercise the right to control the distribution of derivative or
collective works based on the Program.

In addition, mere aggregation of another work not based on the Program
with the Program (or with a work based on the Program) on a volume of
a storage or distribution medium does not bring the other work under
the scope of this License.

  3. You may copy and distribute the Program (or a work based on it,
under Section 2) in object code or executable form under the terms of
Sections 1 and 2 above provided that you also do one of the following:

    a) Accompany it with the complete corresponding machine-readable
    source code, which must be distributed under the terms of Sections
    1 and 2 above on a medium customarily used for software interchange; or,

    b) Accompany it with a written offer, valid for at least three
    years, to give any third party, for a charge no more than your
    cost of physically performing source distribution, a complete
    machine-readable copy of the corresponding source code, to be
    distributed under the terms of Sections 1 and 2 above on a medium
    customarily used for software interchange; or,

    c) Accompany it with the information you received as to the offer
    to distribute corresponding source code.  (This alternative is
    allowed only for noncommercial distribution and only if you
    received the program in object code or executable form with such
    an offer, in accord with Subsection b above.)

The source code for a work means the preferred form of the work for
making modifications to it.  For an executable work, complete source
code means all the source code for all modules it contains, plus any
associated interface definition files, plus the scripts used to
control compilation and installation of the executable.  However, as a
special exception, the source code distributed need not include
anything that is normally distributed (in either source or binary
form) with the major components (compiler, kernel, and so on) of the
operating system on which the executable runs, unless that component
itself accompanies the executable.

If distribution of executable or object code is made by offering
access to copy from a designated place, then offering equivalent
access to copy the source code from the same place counts as
distribution of the source code, even though third parties are not
compelled to copy the source along with the object code.

  4. You may not copy, modify, sublicense, or distribute the Program
except as expressly provided under this License.  Any attempt
otherwise to copy, modify, sublicense or distribute the Program is
void, and will automatically terminate your rights under this License.
However, parties who have received copies, or rights, from you under
this License will not have their licenses terminated so long as such
parties remain in full compliance.

  5. You are not required to accept this License, since you have not
signed it.  However, nothing else grants you permission to modify or
distribute the Program or its derivative works.  These actions are
prohibited by law if you do not accept this License.  Therefore, by
modifying or distributing the Program (or any work based on the
Program), you indicate your acceptance of this License to do so, and
all its terms and conditions for copying, distributing or modifying
the Program or works based on it.

  6. Each time you redistribute the Program (or any work based on the
Program), the recipient automatically receives a license from the
original licensor to copy, distribute or modify the Program subject to
these terms and conditions.  You may not impose any further
restrictions on the recipients'' exercise of the rights granted herein.
You are not responsible for enforcing compliance by third parties to
this License.

  7. If, as a consequence of a court judgment or allegation of patent
infringement or for any other reason (not limited to patent issues),
conditions are imposed on you (whether by court order, agreement or
otherwise) that contradict the conditions of this License, they do not
excuse you from the conditions of this License.  If you cannot
distribute so as to satisfy simultaneously your obligations under this
License and any other pertinent obligations, then as a consequence you
may not distribute the Program at all.  For example, if a patent
license would not permit royalty-free redistribution of the Program by
all those who receive copies directly or indirectly through you, then
the only way you could satisfy both it and this License would be to
refrain entirely from distribution of the Program.

If any portion of this section is held invalid or unenforceable under
any particular circumstance, the balance of the section is intended to
apply and the section as a whole is intended to apply in other
circumstances.

It is not the purpose of this section to induce you to infringe any
patents or other property right claims or to contest validity of any
such claims; this section has the sole purpose of protecting the
integrity of the free software distribution system, which is
implemented by public license practices.  Many people have made
generous contributions to the wide range of software distributed
through that system in reliance on consistent application of that
system; it is up to the author/donor to decide if he or she is willing
to distribute software through any other system and a licensee cannot
impose that choice.

This section is intended to make thoroughly clear what is believed to
be a consequence of the rest of this License.

  8. If the distribution and/or use of the Program is restricted in
certain countries either by patents or by copyrighted interfaces, the
original copyright holder who places the Program under this License
may add an explicit geographical distribution limitation excluding
those countries, so that distribution is permitted only in or among
countries not thus excluded.  In such case, this License incorporates
the limitation as if written in the body of this License.

  9. The Free Software Foundation may publish revised and/or new versions
of the General Public License from time to time.  Such new versions will
be similar in spirit to the present version, but may differ in detail to
address new problems or concerns.

Each version is given a distinguishing version number.  If the Program
specifies a version number of this License which applies to it and "any
later version", you have the option of following the terms and conditions
either of that version or of any later version published by the Free
Software Foundation.  If the Program does not specify a version number of
this License, you may choose any version ever published by the Free Software
Foundation.

  10. If you wish to incorporate parts of the Program into other free
programs whose distribution conditions are different, write to the author
to ask for permission.  For software which is copyrighted by the Free
Software Foundation, write to the Free Software Foundation; we sometimes
make exceptions for this.  Our decision will be guided by the two goals
of preserving the free status of all derivatives of our free software and
of promoting the sharing and reuse of software generally.

			    NO WARRANTY

  11. BECAUSE THE PROGRAM IS LICENSED FREE OF CHARGE, THERE IS NO WARRANTY
FOR THE PROGRAM, TO THE EXTENT PERMITTED BY APPLICABLE LAW.  EXCEPT WHEN
OTHERWISE STATED IN WRITING THE COPYRIGHT HOLDERS AND/OR OTHER PARTIES
PROVIDE THE PROGRAM "AS IS" WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED
OR IMPLIED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.  THE ENTIRE RISK AS
TO THE QUALITY AND PERFORMANCE OF THE PROGRAM IS WITH YOU.  SHOULD THE
PROGRAM PROVE DEFECTIVE, YOU ASSUME THE COST OF ALL NECESSARY SERVICING,
REPAIR OR CORRECTION.

  12. IN NO EVENT UNLESS REQUIRED BY APPLICABLE LAW OR AGREED TO IN WRITING
WILL ANY COPYRIGHT HOLDER, OR ANY OTHER PARTY WHO MAY MODIFY AND/OR
REDISTRIBUTE THE PROGRAM AS PERMITTED ABOVE, BE LIABLE TO YOU FOR DAMAGES,
INCLUDING ANY GENERAL, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES ARISING
OUT OF THE USE OR INABILITY TO USE THE PROGRAM (INCLUDING BUT NOT LIMITED
TO LOSS OF DATA OR DATA BEING RENDERED INACCURATE OR LOSSES SUSTAINED BY
YOU OR THIRD PARTIES OR A FAILURE OF THE PROGRAM TO OPERATE WITH ANY OTHER
PROGRAMS), EVEN IF SUCH HOLDER OR OTHER PARTY HAS BEEN ADVISED OF THE
POSSIBILITY OF SUCH DAMAGES.

		     END OF TERMS AND CONDITIONS
' prinl
.
self up.
^nil
!!
nil! "(fix for smalltalk bug, this line is ignored if there is a variable declaration error)"

"End of procedure"

"Procedure class definition:
 ==========================="
((Smalltalk at: #GXwarranty ifAbsent: [ nil ]) isKindOf: Function class) ifTrue: [ ('Warning: Overriding previously defined procedure ', 'GXwarranty' gtype) prinl ].
Smalltalk at: #GXwarranty put: nil!
Function subclass: 'GXwarranty'
       instanceVariableNames: ''
       classVariableNames: ''
       poolDictionaries: ''
       category: 'Genie Procedure'
!

"BODY
 ----"

!GXwarranty methodsFor: 'running'!

evaluate

'
   Open GENIE, a program for neutron scattering data analysis & display
   ISIS facility, Rutherford Appleton Laboratory, Chilton, Didcot, OXON UK
   Copyright (C) 1996,1997

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
' prenl
.
self up.
^nil
!!
nil! "(fix for smalltalk bug, this line is ignored if there is a variable declaration error)"

"End of procedure"

"Procedure class definition:
 ==========================="
((Smalltalk at: #GXfields ifAbsent: [ nil ]) isKindOf: Function class) ifTrue: [ ('Warning: Overriding previously defined procedure ', 'GXfields' gtype) prinl ].
Smalltalk at: #GXfields put: nil!
Function subclass: 'GXfields'
       instanceVariableNames: ''
       classVariableNames: ''
       poolDictionaries: ''
       category: 'Genie Procedure'
!

"HEADER
 ------"

"parameters
 ----------"

GXfields

addInstVarName: #gXnumfields;
compile:
'gXnumfields: aVar
	aVar notNil ifTrue:[gXnumfields _ self typeCheck: aVar shouldBe: GXInteger paramName: ''gXnumfields'']';
compile:
'p1: aVar
	aVar notNil ifTrue:[gXnumfields _ self typeCheck: aVar shouldBe: GXInteger paramName: ''gXnumfields'']'!

"result
 ------"

GXfields

addInstVarName: #gXres
!

!GXfields methodsFor: 'initializing'!

initVars

!!

"BODY
 ----"

!GXfields methodsFor: 'running'!

evaluate

	gXnumfields isNil
		ifTrue:  [ gXres _ GXWorkspace new ]
		ifFalse: [ (gXnumfields < 0) 
			ifTrue: [ 'Cannot create a workspace with a negative number of fields' prenl. ^nil ] 
			ifFalse: [ gXres _ GXWorkspace new: gXnumfields ]
		]
.
self up.
^gXres
!!
nil! "(fix for smalltalk bug, this line is ignored if there is a variable declaration error)"

"End of procedure"

GXcomment run line: 0 proc: self method: false; p1:'fields'; p2:'Initializes a GENIE workspace with a default number of fields'; condEvaluate!
"Procedure class definition:
 ==========================="
((Smalltalk at: #GXcallout ifAbsent: [ nil ]) isKindOf: Function class) ifTrue: [ ('Warning: Overriding previously defined procedure ', 'GXcallout' gtype) prinl ].
Smalltalk at: #GXcallout put: nil!
Function subclass: 'GXcallout'
       instanceVariableNames: ''
       classVariableNames: ''
       poolDictionaries: ''
       category: 'Genie Procedure'
!

"HEADER
 ------"

"parameters
 ----------"

GXcallout

addInstVarName: #gXfacility;
compile:
'gXfacility: aVar
	aVar notNil ifTrue:[gXfacility _ self typeCheck: aVar shouldBe: GXString paramName: ''gXfacility'']';
compile:
'p1: aVar
	aVar notNil ifTrue:[gXfacility _ self typeCheck: aVar shouldBe: GXString paramName: ''gXfacility'']';

addInstVarName: #gXfunction;
compile:
'gXfunction: aVar
	aVar notNil ifTrue:[gXfunction _ self typeCheck: aVar shouldBe: GXInteger paramName: ''gXfunction'']';
compile:
'p2: aVar
	aVar notNil ifTrue:[gXfunction _ self typeCheck: aVar shouldBe: GXInteger paramName: ''gXfunction'']'!

"result
 ------"

GXcallout

addInstVarName: #gXres
!

!GXcallout methodsFor: 'initializing'!

initVars

!!

"BODY
 ----"

!GXcallout methodsFor: 'running'!

evaluate


	| facility func |

	facility _ Smalltalk at: gXfacility asSymbol.
	func _ gXfunction.

	gXres _ facility
		function: func
		parameters: 
			( (Array new:2)
				at:1 put: #( line help ), ( line at: 1 ) class instVarNames;
				at:2 put: ( line at: 1 );
				yourself )
.
self up.
^gXres
!!
nil! "(fix for smalltalk bug, this line is ignored if there is a variable declaration error)"

"End of procedure"

GXcomment run line: 0 proc: self method: false; p1:'callout'; p2:'Used to implement a callout to a hard coded C or C++ routine.'; condEvaluate!
"Procedure class definition:
 ==========================="
((Smalltalk at: #GXprint ifAbsent: [ nil ]) isKindOf: Function class) ifTrue: [ ('Warning: Overriding previously defined procedure ', 'GXprint' gtype) prinl ].
Smalltalk at: #GXprint put: nil!
Function subclass: 'GXprint'
       instanceVariableNames: ''
       classVariableNames: ''
       poolDictionaries: ''
       category: 'Genie Procedure'
!

"HEADER
 ------"

"parameters
 ----------"

GXprint

addInstVarName: #gXp1;
compile:
'gXp1: aVar
	aVar notNil ifTrue:[ gXp1 _ aVar ]';
compile:
'p1: aVar
	aVar notNil ifTrue:[ gXp1 _ aVar ]';

addInstVarName: #gXp2;
compile:
'gXp2: aVar
	aVar notNil ifTrue:[ gXp2 _ aVar ]';
compile:
'p2: aVar
	aVar notNil ifTrue:[ gXp2 _ aVar ]';

addInstVarName: #gXp3;
compile:
'gXp3: aVar
	aVar notNil ifTrue:[ gXp3 _ aVar ]';
compile:
'p3: aVar
	aVar notNil ifTrue:[ gXp3 _ aVar ]';

addInstVarName: #gXp4;
compile:
'gXp4: aVar
	aVar notNil ifTrue:[ gXp4 _ aVar ]';
compile:
'p4: aVar
	aVar notNil ifTrue:[ gXp4 _ aVar ]';

addInstVarName: #gXp5;
compile:
'gXp5: aVar
	aVar notNil ifTrue:[ gXp5 _ aVar ]';
compile:
'p5: aVar
	aVar notNil ifTrue:[ gXp5 _ aVar ]';

addInstVarName: #gXp6;
compile:
'gXp6: aVar
	aVar notNil ifTrue:[ gXp6 _ aVar ]';
compile:
'p6: aVar
	aVar notNil ifTrue:[ gXp6 _ aVar ]';

addInstVarName: #gXp7;
compile:
'gXp7: aVar
	aVar notNil ifTrue:[ gXp7 _ aVar ]';
compile:
'p7: aVar
	aVar notNil ifTrue:[ gXp7 _ aVar ]';

addInstVarName: #gXp8;
compile:
'gXp8: aVar
	aVar notNil ifTrue:[ gXp8 _ aVar ]';
compile:
'p8: aVar
	aVar notNil ifTrue:[ gXp8 _ aVar ]';

addInstVarName: #gXp9;
compile:
'gXp9: aVar
	aVar notNil ifTrue:[ gXp9 _ aVar ]';
compile:
'p9: aVar
	aVar notNil ifTrue:[ gXp9 _ aVar ]';

addInstVarName: #gXp10;
compile:
'gXp10: aVar
	aVar notNil ifTrue:[ gXp10 _ aVar ]';
compile:
'p10: aVar
	aVar notNil ifTrue:[ gXp10 _ aVar ]';

addInstVarName: #gXp11;
compile:
'gXp11: aVar
	aVar notNil ifTrue:[ gXp11 _ aVar ]';
compile:
'p11: aVar
	aVar notNil ifTrue:[ gXp11 _ aVar ]';

addInstVarName: #gXp12;
compile:
'gXp12: aVar
	aVar notNil ifTrue:[ gXp12 _ aVar ]';
compile:
'p12: aVar
	aVar notNil ifTrue:[ gXp12 _ aVar ]';

addInstVarName: #gXp13;
compile:
'gXp13: aVar
	aVar notNil ifTrue:[ gXp13 _ aVar ]';
compile:
'p13: aVar
	aVar notNil ifTrue:[ gXp13 _ aVar ]';

addInstVarName: #gXp14;
compile:
'gXp14: aVar
	aVar notNil ifTrue:[ gXp14 _ aVar ]';
compile:
'p14: aVar
	aVar notNil ifTrue:[ gXp14 _ aVar ]';

addInstVarName: #gXp15;
compile:
'gXp15: aVar
	aVar notNil ifTrue:[ gXp15 _ aVar ]';
compile:
'p15: aVar
	aVar notNil ifTrue:[ gXp15 _ aVar ]';

addInstVarName: #gXp16;
compile:
'gXp16: aVar
	aVar notNil ifTrue:[ gXp16 _ aVar ]';
compile:
'p16: aVar
	aVar notNil ifTrue:[ gXp16 _ aVar ]';

addInstVarName: #gXp17;
compile:
'gXp17: aVar
	aVar notNil ifTrue:[ gXp17 _ aVar ]';
compile:
'p17: aVar
	aVar notNil ifTrue:[ gXp17 _ aVar ]';

addInstVarName: #gXp18;
compile:
'gXp18: aVar
	aVar notNil ifTrue:[ gXp18 _ aVar ]';
compile:
'p18: aVar
	aVar notNil ifTrue:[ gXp18 _ aVar ]';

addInstVarName: #gXp19;
compile:
'gXp19: aVar
	aVar notNil ifTrue:[ gXp19 _ aVar ]';
compile:
'p19: aVar
	aVar notNil ifTrue:[ gXp19 _ aVar ]';

addInstVarName: #gXp20;
compile:
'gXp20: aVar
	aVar notNil ifTrue:[ gXp20 _ aVar ]';
compile:
'p20: aVar
	aVar notNil ifTrue:[ gXp20 _ aVar ]'
!

"Local variables
 ---------------"

GXprint

addInstVarName: #gXbuff;

addInstVarName: #gXtmp
!

!GXprint methodsFor: 'initializing'!

initVars
	gXbuff _ ''
!!

"BODY
 ----"

!GXprint methodsFor: 'running'!

evaluate

2 to: 21 do:
    [ :i |
        gXtmp _ self instVarAt: i.
        gXtmp notNil ifTrue: [gXbuff _ gXbuff, gXtmp printString]
    ].
gXbuff stringPrint
.
self up.
^nil
!!
nil! "(fix for smalltalk bug, this line is ignored if there is a variable declaration error)"

"End of procedure"

GXcomment run line: 0 proc: self method: false; p1:'print'; p2:'Makes a stab at printing out any sort of genie variable'; condEvaluate!
"Procedure class definition:
 ==========================="
((Smalltalk at: #GXprintn ifAbsent: [ nil ]) isKindOf: Function class) ifTrue: [ ('Warning: Overriding previously defined procedure ', 'GXprintn' gtype) prinl ].
Smalltalk at: #GXprintn put: nil!
Function subclass: 'GXprintn'
       instanceVariableNames: ''
       classVariableNames: ''
       poolDictionaries: ''
       category: 'Genie Procedure'
!

"HEADER
 ------"

"parameters
 ----------"

GXprintn

addInstVarName: #gXp1;
compile:
'gXp1: aVar
	aVar notNil ifTrue:[ gXp1 _ aVar ]';
compile:
'p1: aVar
	aVar notNil ifTrue:[ gXp1 _ aVar ]';

addInstVarName: #gXp2;
compile:
'gXp2: aVar
	aVar notNil ifTrue:[ gXp2 _ aVar ]';
compile:
'p2: aVar
	aVar notNil ifTrue:[ gXp2 _ aVar ]';

addInstVarName: #gXp3;
compile:
'gXp3: aVar
	aVar notNil ifTrue:[ gXp3 _ aVar ]';
compile:
'p3: aVar
	aVar notNil ifTrue:[ gXp3 _ aVar ]';

addInstVarName: #gXp4;
compile:
'gXp4: aVar
	aVar notNil ifTrue:[ gXp4 _ aVar ]';
compile:
'p4: aVar
	aVar notNil ifTrue:[ gXp4 _ aVar ]';

addInstVarName: #gXp5;
compile:
'gXp5: aVar
	aVar notNil ifTrue:[ gXp5 _ aVar ]';
compile:
'p5: aVar
	aVar notNil ifTrue:[ gXp5 _ aVar ]';

addInstVarName: #gXp6;
compile:
'gXp6: aVar
	aVar notNil ifTrue:[ gXp6 _ aVar ]';
compile:
'p6: aVar
	aVar notNil ifTrue:[ gXp6 _ aVar ]';

addInstVarName: #gXp7;
compile:
'gXp7: aVar
	aVar notNil ifTrue:[ gXp7 _ aVar ]';
compile:
'p7: aVar
	aVar notNil ifTrue:[ gXp7 _ aVar ]';

addInstVarName: #gXp8;
compile:
'gXp8: aVar
	aVar notNil ifTrue:[ gXp8 _ aVar ]';
compile:
'p8: aVar
	aVar notNil ifTrue:[ gXp8 _ aVar ]';

addInstVarName: #gXp9;
compile:
'gXp9: aVar
	aVar notNil ifTrue:[ gXp9 _ aVar ]';
compile:
'p9: aVar
	aVar notNil ifTrue:[ gXp9 _ aVar ]';

addInstVarName: #gXp10;
compile:
'gXp10: aVar
	aVar notNil ifTrue:[ gXp10 _ aVar ]';
compile:
'p10: aVar
	aVar notNil ifTrue:[ gXp10 _ aVar ]';

addInstVarName: #gXp11;
compile:
'gXp11: aVar
	aVar notNil ifTrue:[ gXp11 _ aVar ]';
compile:
'p11: aVar
	aVar notNil ifTrue:[ gXp11 _ aVar ]';

addInstVarName: #gXp12;
compile:
'gXp12: aVar
	aVar notNil ifTrue:[ gXp12 _ aVar ]';
compile:
'p12: aVar
	aVar notNil ifTrue:[ gXp12 _ aVar ]';

addInstVarName: #gXp13;
compile:
'gXp13: aVar
	aVar notNil ifTrue:[ gXp13 _ aVar ]';
compile:
'p13: aVar
	aVar notNil ifTrue:[ gXp13 _ aVar ]';

addInstVarName: #gXp14;
compile:
'gXp14: aVar
	aVar notNil ifTrue:[ gXp14 _ aVar ]';
compile:
'p14: aVar
	aVar notNil ifTrue:[ gXp14 _ aVar ]';

addInstVarName: #gXp15;
compile:
'gXp15: aVar
	aVar notNil ifTrue:[ gXp15 _ aVar ]';
compile:
'p15: aVar
	aVar notNil ifTrue:[ gXp15 _ aVar ]';

addInstVarName: #gXp16;
compile:
'gXp16: aVar
	aVar notNil ifTrue:[ gXp16 _ aVar ]';
compile:
'p16: aVar
	aVar notNil ifTrue:[ gXp16 _ aVar ]';

addInstVarName: #gXp17;
compile:
'gXp17: aVar
	aVar notNil ifTrue:[ gXp17 _ aVar ]';
compile:
'p17: aVar
	aVar notNil ifTrue:[ gXp17 _ aVar ]';

addInstVarName: #gXp18;
compile:
'gXp18: aVar
	aVar notNil ifTrue:[ gXp18 _ aVar ]';
compile:
'p18: aVar
	aVar notNil ifTrue:[ gXp18 _ aVar ]';

addInstVarName: #gXp19;
compile:
'gXp19: aVar
	aVar notNil ifTrue:[ gXp19 _ aVar ]';
compile:
'p19: aVar
	aVar notNil ifTrue:[ gXp19 _ aVar ]';

addInstVarName: #gXp20;
compile:
'gXp20: aVar
	aVar notNil ifTrue:[ gXp20 _ aVar ]';
compile:
'p20: aVar
	aVar notNil ifTrue:[ gXp20 _ aVar ]'
!

"Local variables
 ---------------"

GXprintn

addInstVarName: #gXbuff;

addInstVarName: #gXtmp
!

!GXprintn methodsFor: 'initializing'!

initVars
	gXbuff _ ''
!!

"BODY
 ----"

!GXprintn methodsFor: 'running'!

evaluate

2 to: 21 do:
    [ :i |
        gXtmp _ self instVarAt: i.
        gXtmp notNil ifTrue: [gXbuff _ gXbuff, gXtmp printString]
    ].
(gXbuff, Nl) stringPrint
.
self up.
^nil
!!
nil! "(fix for smalltalk bug, this line is ignored if there is a variable declaration error)"

"End of procedure"

GXcomment run line: 0 proc: self method: false; p1:'printn'; p2:'Identical to PRINT but with a newline'; condEvaluate!
"Procedure class definition:
 ==========================="
((Smalltalk at: #GXprinti ifAbsent: [ nil ]) isKindOf: Function class) ifTrue: [ ('Warning: Overriding previously defined procedure ', 'GXprinti' gtype) prinl ].
Smalltalk at: #GXprinti put: nil!
Function subclass: 'GXprinti'
       instanceVariableNames: ''
       classVariableNames: ''
       poolDictionaries: ''
       category: 'Genie Procedure'
!

"HEADER
 ------"

"parameters
 ----------"

GXprinti

addInstVarName: #gXp1;
compile:
'gXp1: aVar
	aVar notNil ifTrue:[ gXp1 _ aVar ]';
compile:
'p1: aVar
	aVar notNil ifTrue:[ gXp1 _ aVar ]';

addInstVarName: #gXp2;
compile:
'gXp2: aVar
	aVar notNil ifTrue:[ gXp2 _ aVar ]';
compile:
'p2: aVar
	aVar notNil ifTrue:[ gXp2 _ aVar ]';

addInstVarName: #gXp3;
compile:
'gXp3: aVar
	aVar notNil ifTrue:[ gXp3 _ aVar ]';
compile:
'p3: aVar
	aVar notNil ifTrue:[ gXp3 _ aVar ]';

addInstVarName: #gXp4;
compile:
'gXp4: aVar
	aVar notNil ifTrue:[ gXp4 _ aVar ]';
compile:
'p4: aVar
	aVar notNil ifTrue:[ gXp4 _ aVar ]';

addInstVarName: #gXp5;
compile:
'gXp5: aVar
	aVar notNil ifTrue:[ gXp5 _ aVar ]';
compile:
'p5: aVar
	aVar notNil ifTrue:[ gXp5 _ aVar ]';

addInstVarName: #gXp6;
compile:
'gXp6: aVar
	aVar notNil ifTrue:[ gXp6 _ aVar ]';
compile:
'p6: aVar
	aVar notNil ifTrue:[ gXp6 _ aVar ]';

addInstVarName: #gXp7;
compile:
'gXp7: aVar
	aVar notNil ifTrue:[ gXp7 _ aVar ]';
compile:
'p7: aVar
	aVar notNil ifTrue:[ gXp7 _ aVar ]';

addInstVarName: #gXp8;
compile:
'gXp8: aVar
	aVar notNil ifTrue:[ gXp8 _ aVar ]';
compile:
'p8: aVar
	aVar notNil ifTrue:[ gXp8 _ aVar ]';

addInstVarName: #gXp9;
compile:
'gXp9: aVar
	aVar notNil ifTrue:[ gXp9 _ aVar ]';
compile:
'p9: aVar
	aVar notNil ifTrue:[ gXp9 _ aVar ]';

addInstVarName: #gXp10;
compile:
'gXp10: aVar
	aVar notNil ifTrue:[ gXp10 _ aVar ]';
compile:
'p10: aVar
	aVar notNil ifTrue:[ gXp10 _ aVar ]';

addInstVarName: #gXp11;
compile:
'gXp11: aVar
	aVar notNil ifTrue:[ gXp11 _ aVar ]';
compile:
'p11: aVar
	aVar notNil ifTrue:[ gXp11 _ aVar ]';

addInstVarName: #gXp12;
compile:
'gXp12: aVar
	aVar notNil ifTrue:[ gXp12 _ aVar ]';
compile:
'p12: aVar
	aVar notNil ifTrue:[ gXp12 _ aVar ]';

addInstVarName: #gXp13;
compile:
'gXp13: aVar
	aVar notNil ifTrue:[ gXp13 _ aVar ]';
compile:
'p13: aVar
	aVar notNil ifTrue:[ gXp13 _ aVar ]';

addInstVarName: #gXp14;
compile:
'gXp14: aVar
	aVar notNil ifTrue:[ gXp14 _ aVar ]';
compile:
'p14: aVar
	aVar notNil ifTrue:[ gXp14 _ aVar ]';

addInstVarName: #gXp15;
compile:
'gXp15: aVar
	aVar notNil ifTrue:[ gXp15 _ aVar ]';
compile:
'p15: aVar
	aVar notNil ifTrue:[ gXp15 _ aVar ]';

addInstVarName: #gXp16;
compile:
'gXp16: aVar
	aVar notNil ifTrue:[ gXp16 _ aVar ]';
compile:
'p16: aVar
	aVar notNil ifTrue:[ gXp16 _ aVar ]';

addInstVarName: #gXp17;
compile:
'gXp17: aVar
	aVar notNil ifTrue:[ gXp17 _ aVar ]';
compile:
'p17: aVar
	aVar notNil ifTrue:[ gXp17 _ aVar ]';

addInstVarName: #gXp18;
compile:
'gXp18: aVar
	aVar notNil ifTrue:[ gXp18 _ aVar ]';
compile:
'p18: aVar
	aVar notNil ifTrue:[ gXp18 _ aVar ]';

addInstVarName: #gXp19;
compile:
'gXp19: aVar
	aVar notNil ifTrue:[ gXp19 _ aVar ]';
compile:
'p19: aVar
	aVar notNil ifTrue:[ gXp19 _ aVar ]';

addInstVarName: #gXp20;
compile:
'gXp20: aVar
	aVar notNil ifTrue:[ gXp20 _ aVar ]';
compile:
'p20: aVar
	aVar notNil ifTrue:[ gXp20 _ aVar ]'
!

"Local variables
 ---------------"

GXprinti

addInstVarName: #gXbuff;

addInstVarName: #gXtmp
!

!GXprinti methodsFor: 'initializing'!

initVars
	gXbuff _ ''
!!

"BODY
 ----"

!GXprinti methodsFor: 'running'!

evaluate

2 to: 21 do:
    [ :i |
        gXtmp _ self instVarAt: i.
        gXtmp notNil ifTrue: [gXbuff _ gXbuff, gXtmp printString]
    ].
gXbuff stringPrint: 'i'
.
self up.
^nil
!!
nil! "(fix for smalltalk bug, this line is ignored if there is a variable declaration error)"

"End of procedure"

GXcomment run line: 0 proc: self method: false; p1:'printi'; p2:'Prints variables to the information stream'; condEvaluate!
"Procedure class definition:
 ==========================="
((Smalltalk at: #GXprintin ifAbsent: [ nil ]) isKindOf: Function class) ifTrue: [ ('Warning: Overriding previously defined procedure ', 'GXprintin' gtype) prinl ].
Smalltalk at: #GXprintin put: nil!
Function subclass: 'GXprintin'
       instanceVariableNames: ''
       classVariableNames: ''
       poolDictionaries: ''
       category: 'Genie Procedure'
!

"HEADER
 ------"

"parameters
 ----------"

GXprintin

addInstVarName: #gXp1;
compile:
'gXp1: aVar
	aVar notNil ifTrue:[ gXp1 _ aVar ]';
compile:
'p1: aVar
	aVar notNil ifTrue:[ gXp1 _ aVar ]';

addInstVarName: #gXp2;
compile:
'gXp2: aVar
	aVar notNil ifTrue:[ gXp2 _ aVar ]';
compile:
'p2: aVar
	aVar notNil ifTrue:[ gXp2 _ aVar ]';

addInstVarName: #gXp3;
compile:
'gXp3: aVar
	aVar notNil ifTrue:[ gXp3 _ aVar ]';
compile:
'p3: aVar
	aVar notNil ifTrue:[ gXp3 _ aVar ]';

addInstVarName: #gXp4;
compile:
'gXp4: aVar
	aVar notNil ifTrue:[ gXp4 _ aVar ]';
compile:
'p4: aVar
	aVar notNil ifTrue:[ gXp4 _ aVar ]';

addInstVarName: #gXp5;
compile:
'gXp5: aVar
	aVar notNil ifTrue:[ gXp5 _ aVar ]';
compile:
'p5: aVar
	aVar notNil ifTrue:[ gXp5 _ aVar ]';

addInstVarName: #gXp6;
compile:
'gXp6: aVar
	aVar notNil ifTrue:[ gXp6 _ aVar ]';
compile:
'p6: aVar
	aVar notNil ifTrue:[ gXp6 _ aVar ]';

addInstVarName: #gXp7;
compile:
'gXp7: aVar
	aVar notNil ifTrue:[ gXp7 _ aVar ]';
compile:
'p7: aVar
	aVar notNil ifTrue:[ gXp7 _ aVar ]';

addInstVarName: #gXp8;
compile:
'gXp8: aVar
	aVar notNil ifTrue:[ gXp8 _ aVar ]';
compile:
'p8: aVar
	aVar notNil ifTrue:[ gXp8 _ aVar ]';

addInstVarName: #gXp9;
compile:
'gXp9: aVar
	aVar notNil ifTrue:[ gXp9 _ aVar ]';
compile:
'p9: aVar
	aVar notNil ifTrue:[ gXp9 _ aVar ]';

addInstVarName: #gXp10;
compile:
'gXp10: aVar
	aVar notNil ifTrue:[ gXp10 _ aVar ]';
compile:
'p10: aVar
	aVar notNil ifTrue:[ gXp10 _ aVar ]';

addInstVarName: #gXp11;
compile:
'gXp11: aVar
	aVar notNil ifTrue:[ gXp11 _ aVar ]';
compile:
'p11: aVar
	aVar notNil ifTrue:[ gXp11 _ aVar ]';

addInstVarName: #gXp12;
compile:
'gXp12: aVar
	aVar notNil ifTrue:[ gXp12 _ aVar ]';
compile:
'p12: aVar
	aVar notNil ifTrue:[ gXp12 _ aVar ]';

addInstVarName: #gXp13;
compile:
'gXp13: aVar
	aVar notNil ifTrue:[ gXp13 _ aVar ]';
compile:
'p13: aVar
	aVar notNil ifTrue:[ gXp13 _ aVar ]';

addInstVarName: #gXp14;
compile:
'gXp14: aVar
	aVar notNil ifTrue:[ gXp14 _ aVar ]';
compile:
'p14: aVar
	aVar notNil ifTrue:[ gXp14 _ aVar ]';

addInstVarName: #gXp15;
compile:
'gXp15: aVar
	aVar notNil ifTrue:[ gXp15 _ aVar ]';
compile:
'p15: aVar
	aVar notNil ifTrue:[ gXp15 _ aVar ]';

addInstVarName: #gXp16;
compile:
'gXp16: aVar
	aVar notNil ifTrue:[ gXp16 _ aVar ]';
compile:
'p16: aVar
	aVar notNil ifTrue:[ gXp16 _ aVar ]';

addInstVarName: #gXp17;
compile:
'gXp17: aVar
	aVar notNil ifTrue:[ gXp17 _ aVar ]';
compile:
'p17: aVar
	aVar notNil ifTrue:[ gXp17 _ aVar ]';

addInstVarName: #gXp18;
compile:
'gXp18: aVar
	aVar notNil ifTrue:[ gXp18 _ aVar ]';
compile:
'p18: aVar
	aVar notNil ifTrue:[ gXp18 _ aVar ]';

addInstVarName: #gXp19;
compile:
'gXp19: aVar
	aVar notNil ifTrue:[ gXp19 _ aVar ]';
compile:
'p19: aVar
	aVar notNil ifTrue:[ gXp19 _ aVar ]';

addInstVarName: #gXp20;
compile:
'gXp20: aVar
	aVar notNil ifTrue:[ gXp20 _ aVar ]';
compile:
'p20: aVar
	aVar notNil ifTrue:[ gXp20 _ aVar ]'
!

"Local variables
 ---------------"

GXprintin

addInstVarName: #gXbuff;

addInstVarName: #gXtmp
!

!GXprintin methodsFor: 'initializing'!

initVars
	gXbuff _ ''
!!

"BODY
 ----"

!GXprintin methodsFor: 'running'!

evaluate

2 to: 21 do:
    [ :i |
        gXtmp _ self instVarAt: i.
        gXtmp notNil ifTrue: [gXbuff _ gXbuff, gXtmp printString]
    ].
(gXbuff, Nl) stringPrint: 'i'
.
self up.
^nil
!!
nil! "(fix for smalltalk bug, this line is ignored if there is a variable declaration error)"

"End of procedure"

GXcomment run line: 0 proc: self method: false; p1:'printin'; p2:'Identical to PRINTIN but with a newline'; condEvaluate!
"Procedure class definition:
 ==========================="
((Smalltalk at: #GXprinte ifAbsent: [ nil ]) isKindOf: Function class) ifTrue: [ ('Warning: Overriding previously defined procedure ', 'GXprinte' gtype) prinl ].
Smalltalk at: #GXprinte put: nil!
Function subclass: 'GXprinte'
       instanceVariableNames: ''
       classVariableNames: ''
       poolDictionaries: ''
       category: 'Genie Procedure'
!

"HEADER
 ------"

"parameters
 ----------"

GXprinte

addInstVarName: #gXp1;
compile:
'gXp1: aVar
	aVar notNil ifTrue:[ gXp1 _ aVar ]';
compile:
'p1: aVar
	aVar notNil ifTrue:[ gXp1 _ aVar ]';

addInstVarName: #gXp2;
compile:
'gXp2: aVar
	aVar notNil ifTrue:[ gXp2 _ aVar ]';
compile:
'p2: aVar
	aVar notNil ifTrue:[ gXp2 _ aVar ]';

addInstVarName: #gXp3;
compile:
'gXp3: aVar
	aVar notNil ifTrue:[ gXp3 _ aVar ]';
compile:
'p3: aVar
	aVar notNil ifTrue:[ gXp3 _ aVar ]';

addInstVarName: #gXp4;
compile:
'gXp4: aVar
	aVar notNil ifTrue:[ gXp4 _ aVar ]';
compile:
'p4: aVar
	aVar notNil ifTrue:[ gXp4 _ aVar ]';

addInstVarName: #gXp5;
compile:
'gXp5: aVar
	aVar notNil ifTrue:[ gXp5 _ aVar ]';
compile:
'p5: aVar
	aVar notNil ifTrue:[ gXp5 _ aVar ]';

addInstVarName: #gXp6;
compile:
'gXp6: aVar
	aVar notNil ifTrue:[ gXp6 _ aVar ]';
compile:
'p6: aVar
	aVar notNil ifTrue:[ gXp6 _ aVar ]';

addInstVarName: #gXp7;
compile:
'gXp7: aVar
	aVar notNil ifTrue:[ gXp7 _ aVar ]';
compile:
'p7: aVar
	aVar notNil ifTrue:[ gXp7 _ aVar ]';

addInstVarName: #gXp8;
compile:
'gXp8: aVar
	aVar notNil ifTrue:[ gXp8 _ aVar ]';
compile:
'p8: aVar
	aVar notNil ifTrue:[ gXp8 _ aVar ]';

addInstVarName: #gXp9;
compile:
'gXp9: aVar
	aVar notNil ifTrue:[ gXp9 _ aVar ]';
compile:
'p9: aVar
	aVar notNil ifTrue:[ gXp9 _ aVar ]';

addInstVarName: #gXp10;
compile:
'gXp10: aVar
	aVar notNil ifTrue:[ gXp10 _ aVar ]';
compile:
'p10: aVar
	aVar notNil ifTrue:[ gXp10 _ aVar ]';

addInstVarName: #gXp11;
compile:
'gXp11: aVar
	aVar notNil ifTrue:[ gXp11 _ aVar ]';
compile:
'p11: aVar
	aVar notNil ifTrue:[ gXp11 _ aVar ]';

addInstVarName: #gXp12;
compile:
'gXp12: aVar
	aVar notNil ifTrue:[ gXp12 _ aVar ]';
compile:
'p12: aVar
	aVar notNil ifTrue:[ gXp12 _ aVar ]';

addInstVarName: #gXp13;
compile:
'gXp13: aVar
	aVar notNil ifTrue:[ gXp13 _ aVar ]';
compile:
'p13: aVar
	aVar notNil ifTrue:[ gXp13 _ aVar ]';

addInstVarName: #gXp14;
compile:
'gXp14: aVar
	aVar notNil ifTrue:[ gXp14 _ aVar ]';
compile:
'p14: aVar
	aVar notNil ifTrue:[ gXp14 _ aVar ]';

addInstVarName: #gXp15;
compile:
'gXp15: aVar
	aVar notNil ifTrue:[ gXp15 _ aVar ]';
compile:
'p15: aVar
	aVar notNil ifTrue:[ gXp15 _ aVar ]';

addInstVarName: #gXp16;
compile:
'gXp16: aVar
	aVar notNil ifTrue:[ gXp16 _ aVar ]';
compile:
'p16: aVar
	aVar notNil ifTrue:[ gXp16 _ aVar ]';

addInstVarName: #gXp17;
compile:
'gXp17: aVar
	aVar notNil ifTrue:[ gXp17 _ aVar ]';
compile:
'p17: aVar
	aVar notNil ifTrue:[ gXp17 _ aVar ]';

addInstVarName: #gXp18;
compile:
'gXp18: aVar
	aVar notNil ifTrue:[ gXp18 _ aVar ]';
compile:
'p18: aVar
	aVar notNil ifTrue:[ gXp18 _ aVar ]';

addInstVarName: #gXp19;
compile:
'gXp19: aVar
	aVar notNil ifTrue:[ gXp19 _ aVar ]';
compile:
'p19: aVar
	aVar notNil ifTrue:[ gXp19 _ aVar ]';

addInstVarName: #gXp20;
compile:
'gXp20: aVar
	aVar notNil ifTrue:[ gXp20 _ aVar ]';
compile:
'p20: aVar
	aVar notNil ifTrue:[ gXp20 _ aVar ]'!

"qualifiers
 ----------"

GXprinte

addInstVarName: #gXnolocate;
compile:
'gXnolocate
	gXnolocate _ true';
compile:
'gXnolocate: aBoolean
	gXnolocate _ aBoolean'
!

"Local variables
 ---------------"

GXprinte

addInstVarName: #gXbuff;

addInstVarName: #gXtmp
!

!GXprinte methodsFor: 'initializing'!

initVars
	gXnolocate _ false.
	gXbuff _ ''
!!

"BODY
 ----"

!GXprinte methodsFor: 'running'!

evaluate
(gXnolocate) not ifTrue: [ 

    gXbuff _ ((self instVarAt: 1) at: 2) printString, '/'.
    gXbuff _ gXbuff, ((self instVarAt: 1) at:1) class name gtype, '/'
     ].

2 to: 21 do:
    [ :i |
        gXtmp _ self instVarAt: i.
        gXtmp notNil ifTrue: [gXbuff _ gXbuff, gXtmp printString]
    ].
gXbuff stringPrint: 'e'
.
self up.
^nil
!!
nil! "(fix for smalltalk bug, this line is ignored if there is a variable declaration error)"

"End of procedure"

GXcomment run line: 0 proc: self method: false; p1:'printe'; p2:'Prints variables to the error stream'; condEvaluate!
"Procedure class definition:
 ==========================="
((Smalltalk at: #GXprinten ifAbsent: [ nil ]) isKindOf: Function class) ifTrue: [ ('Warning: Overriding previously defined procedure ', 'GXprinten' gtype) prinl ].
Smalltalk at: #GXprinten put: nil!
Function subclass: 'GXprinten'
       instanceVariableNames: ''
       classVariableNames: ''
       poolDictionaries: ''
       category: 'Genie Procedure'
!

"HEADER
 ------"

"parameters
 ----------"

GXprinten

addInstVarName: #gXp1;
compile:
'gXp1: aVar
	aVar notNil ifTrue:[ gXp1 _ aVar ]';
compile:
'p1: aVar
	aVar notNil ifTrue:[ gXp1 _ aVar ]';

addInstVarName: #gXp2;
compile:
'gXp2: aVar
	aVar notNil ifTrue:[ gXp2 _ aVar ]';
compile:
'p2: aVar
	aVar notNil ifTrue:[ gXp2 _ aVar ]';

addInstVarName: #gXp3;
compile:
'gXp3: aVar
	aVar notNil ifTrue:[ gXp3 _ aVar ]';
compile:
'p3: aVar
	aVar notNil ifTrue:[ gXp3 _ aVar ]';

addInstVarName: #gXp4;
compile:
'gXp4: aVar
	aVar notNil ifTrue:[ gXp4 _ aVar ]';
compile:
'p4: aVar
	aVar notNil ifTrue:[ gXp4 _ aVar ]';

addInstVarName: #gXp5;
compile:
'gXp5: aVar
	aVar notNil ifTrue:[ gXp5 _ aVar ]';
compile:
'p5: aVar
	aVar notNil ifTrue:[ gXp5 _ aVar ]';

addInstVarName: #gXp6;
compile:
'gXp6: aVar
	aVar notNil ifTrue:[ gXp6 _ aVar ]';
compile:
'p6: aVar
	aVar notNil ifTrue:[ gXp6 _ aVar ]';

addInstVarName: #gXp7;
compile:
'gXp7: aVar
	aVar notNil ifTrue:[ gXp7 _ aVar ]';
compile:
'p7: aVar
	aVar notNil ifTrue:[ gXp7 _ aVar ]';

addInstVarName: #gXp8;
compile:
'gXp8: aVar
	aVar notNil ifTrue:[ gXp8 _ aVar ]';
compile:
'p8: aVar
	aVar notNil ifTrue:[ gXp8 _ aVar ]';

addInstVarName: #gXp9;
compile:
'gXp9: aVar
	aVar notNil ifTrue:[ gXp9 _ aVar ]';
compile:
'p9: aVar
	aVar notNil ifTrue:[ gXp9 _ aVar ]';

addInstVarName: #gXp10;
compile:
'gXp10: aVar
	aVar notNil ifTrue:[ gXp10 _ aVar ]';
compile:
'p10: aVar
	aVar notNil ifTrue:[ gXp10 _ aVar ]';

addInstVarName: #gXp11;
compile:
'gXp11: aVar
	aVar notNil ifTrue:[ gXp11 _ aVar ]';
compile:
'p11: aVar
	aVar notNil ifTrue:[ gXp11 _ aVar ]';

addInstVarName: #gXp12;
compile:
'gXp12: aVar
	aVar notNil ifTrue:[ gXp12 _ aVar ]';
compile:
'p12: aVar
	aVar notNil ifTrue:[ gXp12 _ aVar ]';

addInstVarName: #gXp13;
compile:
'gXp13: aVar
	aVar notNil ifTrue:[ gXp13 _ aVar ]';
compile:
'p13: aVar
	aVar notNil ifTrue:[ gXp13 _ aVar ]';

addInstVarName: #gXp14;
compile:
'gXp14: aVar
	aVar notNil ifTrue:[ gXp14 _ aVar ]';
compile:
'p14: aVar
	aVar notNil ifTrue:[ gXp14 _ aVar ]';

addInstVarName: #gXp15;
compile:
'gXp15: aVar
	aVar notNil ifTrue:[ gXp15 _ aVar ]';
compile:
'p15: aVar
	aVar notNil ifTrue:[ gXp15 _ aVar ]';

addInstVarName: #gXp16;
compile:
'gXp16: aVar
	aVar notNil ifTrue:[ gXp16 _ aVar ]';
compile:
'p16: aVar
	aVar notNil ifTrue:[ gXp16 _ aVar ]';

addInstVarName: #gXp17;
compile:
'gXp17: aVar
	aVar notNil ifTrue:[ gXp17 _ aVar ]';
compile:
'p17: aVar
	aVar notNil ifTrue:[ gXp17 _ aVar ]';

addInstVarName: #gXp18;
compile:
'gXp18: aVar
	aVar notNil ifTrue:[ gXp18 _ aVar ]';
compile:
'p18: aVar
	aVar notNil ifTrue:[ gXp18 _ aVar ]';

addInstVarName: #gXp19;
compile:
'gXp19: aVar
	aVar notNil ifTrue:[ gXp19 _ aVar ]';
compile:
'p19: aVar
	aVar notNil ifTrue:[ gXp19 _ aVar ]';

addInstVarName: #gXp20;
compile:
'gXp20: aVar
	aVar notNil ifTrue:[ gXp20 _ aVar ]';
compile:
'p20: aVar
	aVar notNil ifTrue:[ gXp20 _ aVar ]'!

"qualifiers
 ----------"

GXprinten

addInstVarName: #gXnolocate;
compile:
'gXnolocate
	gXnolocate _ true';
compile:
'gXnolocate: aBoolean
	gXnolocate _ aBoolean'
!

"Local variables
 ---------------"

GXprinten

addInstVarName: #gXbuff;

addInstVarName: #gXtmp
!

!GXprinten methodsFor: 'initializing'!

initVars
	gXnolocate _ false.
	gXbuff _ ''
!!

"BODY
 ----"

!GXprinten methodsFor: 'running'!

evaluate
(gXnolocate) not ifTrue: [ 

    gXbuff _ ((self instVarAt: 1) at: 2) printString, '/'.
    gXbuff _ gXbuff, ((self instVarAt: 1) at:1) class name gtype, '/'
     ].

2 to: 21 do:
    [ :i |
        gXtmp _ self instVarAt: i.
        gXtmp notNil ifTrue: [gXbuff _ gXbuff, gXtmp printString]
    ].
(gXbuff, Nl) stringPrint: 'e'
.
self up.
^nil
!!
nil! "(fix for smalltalk bug, this line is ignored if there is a variable declaration error)"

"End of procedure"

GXcomment run line: 0 proc: self method: false; p1:'printen'; p2:'Identical to PRINTEN but with a newline'; condEvaluate!
"Procedure class definition:
 ==========================="
((Smalltalk at: #GXprintd ifAbsent: [ nil ]) isKindOf: Function class) ifTrue: [ ('Warning: Overriding previously defined procedure ', 'GXprintd' gtype) prinl ].
Smalltalk at: #GXprintd put: nil!
Function subclass: 'GXprintd'
       instanceVariableNames: ''
       classVariableNames: ''
       poolDictionaries: ''
       category: 'Genie Procedure'
!

"HEADER
 ------"

"parameters
 ----------"

GXprintd

addInstVarName: #gXp1;
compile:
'gXp1: aVar
	aVar notNil ifTrue:[ gXp1 _ aVar ]';
compile:
'p1: aVar
	aVar notNil ifTrue:[ gXp1 _ aVar ]';

addInstVarName: #gXp2;
compile:
'gXp2: aVar
	aVar notNil ifTrue:[ gXp2 _ aVar ]';
compile:
'p2: aVar
	aVar notNil ifTrue:[ gXp2 _ aVar ]';

addInstVarName: #gXp3;
compile:
'gXp3: aVar
	aVar notNil ifTrue:[ gXp3 _ aVar ]';
compile:
'p3: aVar
	aVar notNil ifTrue:[ gXp3 _ aVar ]';

addInstVarName: #gXp4;
compile:
'gXp4: aVar
	aVar notNil ifTrue:[ gXp4 _ aVar ]';
compile:
'p4: aVar
	aVar notNil ifTrue:[ gXp4 _ aVar ]';

addInstVarName: #gXp5;
compile:
'gXp5: aVar
	aVar notNil ifTrue:[ gXp5 _ aVar ]';
compile:
'p5: aVar
	aVar notNil ifTrue:[ gXp5 _ aVar ]';

addInstVarName: #gXp6;
compile:
'gXp6: aVar
	aVar notNil ifTrue:[ gXp6 _ aVar ]';
compile:
'p6: aVar
	aVar notNil ifTrue:[ gXp6 _ aVar ]';

addInstVarName: #gXp7;
compile:
'gXp7: aVar
	aVar notNil ifTrue:[ gXp7 _ aVar ]';
compile:
'p7: aVar
	aVar notNil ifTrue:[ gXp7 _ aVar ]';

addInstVarName: #gXp8;
compile:
'gXp8: aVar
	aVar notNil ifTrue:[ gXp8 _ aVar ]';
compile:
'p8: aVar
	aVar notNil ifTrue:[ gXp8 _ aVar ]';

addInstVarName: #gXp9;
compile:
'gXp9: aVar
	aVar notNil ifTrue:[ gXp9 _ aVar ]';
compile:
'p9: aVar
	aVar notNil ifTrue:[ gXp9 _ aVar ]';

addInstVarName: #gXp10;
compile:
'gXp10: aVar
	aVar notNil ifTrue:[ gXp10 _ aVar ]';
compile:
'p10: aVar
	aVar notNil ifTrue:[ gXp10 _ aVar ]';

addInstVarName: #gXp11;
compile:
'gXp11: aVar
	aVar notNil ifTrue:[ gXp11 _ aVar ]';
compile:
'p11: aVar
	aVar notNil ifTrue:[ gXp11 _ aVar ]';

addInstVarName: #gXp12;
compile:
'gXp12: aVar
	aVar notNil ifTrue:[ gXp12 _ aVar ]';
compile:
'p12: aVar
	aVar notNil ifTrue:[ gXp12 _ aVar ]';

addInstVarName: #gXp13;
compile:
'gXp13: aVar
	aVar notNil ifTrue:[ gXp13 _ aVar ]';
compile:
'p13: aVar
	aVar notNil ifTrue:[ gXp13 _ aVar ]';

addInstVarName: #gXp14;
compile:
'gXp14: aVar
	aVar notNil ifTrue:[ gXp14 _ aVar ]';
compile:
'p14: aVar
	aVar notNil ifTrue:[ gXp14 _ aVar ]';

addInstVarName: #gXp15;
compile:
'gXp15: aVar
	aVar notNil ifTrue:[ gXp15 _ aVar ]';
compile:
'p15: aVar
	aVar notNil ifTrue:[ gXp15 _ aVar ]';

addInstVarName: #gXp16;
compile:
'gXp16: aVar
	aVar notNil ifTrue:[ gXp16 _ aVar ]';
compile:
'p16: aVar
	aVar notNil ifTrue:[ gXp16 _ aVar ]';

addInstVarName: #gXp17;
compile:
'gXp17: aVar
	aVar notNil ifTrue:[ gXp17 _ aVar ]';
compile:
'p17: aVar
	aVar notNil ifTrue:[ gXp17 _ aVar ]';

addInstVarName: #gXp18;
compile:
'gXp18: aVar
	aVar notNil ifTrue:[ gXp18 _ aVar ]';
compile:
'p18: aVar
	aVar notNil ifTrue:[ gXp18 _ aVar ]';

addInstVarName: #gXp19;
compile:
'gXp19: aVar
	aVar notNil ifTrue:[ gXp19 _ aVar ]';
compile:
'p19: aVar
	aVar notNil ifTrue:[ gXp19 _ aVar ]';

addInstVarName: #gXp20;
compile:
'gXp20: aVar
	aVar notNil ifTrue:[ gXp20 _ aVar ]';
compile:
'p20: aVar
	aVar notNil ifTrue:[ gXp20 _ aVar ]'!

"qualifiers
 ----------"

GXprintd

addInstVarName: #gXlocate;
compile:
'gXlocate
	gXlocate _ true';
compile:
'gXlocate: aBoolean
	gXlocate _ aBoolean'
!

"Local variables
 ---------------"

GXprintd

addInstVarName: #gXbuff;

addInstVarName: #gXtmp
!

!GXprintd methodsFor: 'initializing'!

initVars
	gXlocate _ false.
	gXbuff _ ''
!!

"BODY
 ----"

!GXprintd methodsFor: 'running'!

evaluate
(gXlocate) ifTrue: [ 

    gXbuff _ ((self instVarAt: 1) at: 2) printString, '/'.
    gXbuff _ gXbuff, ((self instVarAt: 1) at:1) class name gtype, '/'
     ].

2 to: 21 do:
    [ :i |
        gXtmp _ self instVarAt: i.
        gXtmp notNil ifTrue: [gXbuff _ gXbuff, gXtmp printString]
    ].
gXbuff stringPrint: 'd'
.
self up.
^nil
!!
nil! "(fix for smalltalk bug, this line is ignored if there is a variable declaration error)"

"End of procedure"

GXcomment run line: 0 proc: self method: false; p1:'printd'; p2:'Prints variables to the debug stream'; condEvaluate!
"Procedure class definition:
 ==========================="
((Smalltalk at: #GXprintdn ifAbsent: [ nil ]) isKindOf: Function class) ifTrue: [ ('Warning: Overriding previously defined procedure ', 'GXprintdn' gtype) prinl ].
Smalltalk at: #GXprintdn put: nil!
Function subclass: 'GXprintdn'
       instanceVariableNames: ''
       classVariableNames: ''
       poolDictionaries: ''
       category: 'Genie Procedure'
!

"HEADER
 ------"

"parameters
 ----------"

GXprintdn

addInstVarName: #gXp1;
compile:
'gXp1: aVar
	aVar notNil ifTrue:[ gXp1 _ aVar ]';
compile:
'p1: aVar
	aVar notNil ifTrue:[ gXp1 _ aVar ]';

addInstVarName: #gXp2;
compile:
'gXp2: aVar
	aVar notNil ifTrue:[ gXp2 _ aVar ]';
compile:
'p2: aVar
	aVar notNil ifTrue:[ gXp2 _ aVar ]';

addInstVarName: #gXp3;
compile:
'gXp3: aVar
	aVar notNil ifTrue:[ gXp3 _ aVar ]';
compile:
'p3: aVar
	aVar notNil ifTrue:[ gXp3 _ aVar ]';

addInstVarName: #gXp4;
compile:
'gXp4: aVar
	aVar notNil ifTrue:[ gXp4 _ aVar ]';
compile:
'p4: aVar
	aVar notNil ifTrue:[ gXp4 _ aVar ]';

addInstVarName: #gXp5;
compile:
'gXp5: aVar
	aVar notNil ifTrue:[ gXp5 _ aVar ]';
compile:
'p5: aVar
	aVar notNil ifTrue:[ gXp5 _ aVar ]';

addInstVarName: #gXp6;
compile:
'gXp6: aVar
	aVar notNil ifTrue:[ gXp6 _ aVar ]';
compile:
'p6: aVar
	aVar notNil ifTrue:[ gXp6 _ aVar ]';

addInstVarName: #gXp7;
compile:
'gXp7: aVar
	aVar notNil ifTrue:[ gXp7 _ aVar ]';
compile:
'p7: aVar
	aVar notNil ifTrue:[ gXp7 _ aVar ]';

addInstVarName: #gXp8;
compile:
'gXp8: aVar
	aVar notNil ifTrue:[ gXp8 _ aVar ]';
compile:
'p8: aVar
	aVar notNil ifTrue:[ gXp8 _ aVar ]';

addInstVarName: #gXp9;
compile:
'gXp9: aVar
	aVar notNil ifTrue:[ gXp9 _ aVar ]';
compile:
'p9: aVar
	aVar notNil ifTrue:[ gXp9 _ aVar ]';

addInstVarName: #gXp10;
compile:
'gXp10: aVar
	aVar notNil ifTrue:[ gXp10 _ aVar ]';
compile:
'p10: aVar
	aVar notNil ifTrue:[ gXp10 _ aVar ]';

addInstVarName: #gXp11;
compile:
'gXp11: aVar
	aVar notNil ifTrue:[ gXp11 _ aVar ]';
compile:
'p11: aVar
	aVar notNil ifTrue:[ gXp11 _ aVar ]';

addInstVarName: #gXp12;
compile:
'gXp12: aVar
	aVar notNil ifTrue:[ gXp12 _ aVar ]';
compile:
'p12: aVar
	aVar notNil ifTrue:[ gXp12 _ aVar ]';

addInstVarName: #gXp13;
compile:
'gXp13: aVar
	aVar notNil ifTrue:[ gXp13 _ aVar ]';
compile:
'p13: aVar
	aVar notNil ifTrue:[ gXp13 _ aVar ]';

addInstVarName: #gXp14;
compile:
'gXp14: aVar
	aVar notNil ifTrue:[ gXp14 _ aVar ]';
compile:
'p14: aVar
	aVar notNil ifTrue:[ gXp14 _ aVar ]';

addInstVarName: #gXp15;
compile:
'gXp15: aVar
	aVar notNil ifTrue:[ gXp15 _ aVar ]';
compile:
'p15: aVar
	aVar notNil ifTrue:[ gXp15 _ aVar ]';

addInstVarName: #gXp16;
compile:
'gXp16: aVar
	aVar notNil ifTrue:[ gXp16 _ aVar ]';
compile:
'p16: aVar
	aVar notNil ifTrue:[ gXp16 _ aVar ]';

addInstVarName: #gXp17;
compile:
'gXp17: aVar
	aVar notNil ifTrue:[ gXp17 _ aVar ]';
compile:
'p17: aVar
	aVar notNil ifTrue:[ gXp17 _ aVar ]';

addInstVarName: #gXp18;
compile:
'gXp18: aVar
	aVar notNil ifTrue:[ gXp18 _ aVar ]';
compile:
'p18: aVar
	aVar notNil ifTrue:[ gXp18 _ aVar ]';

addInstVarName: #gXp19;
compile:
'gXp19: aVar
	aVar notNil ifTrue:[ gXp19 _ aVar ]';
compile:
'p19: aVar
	aVar notNil ifTrue:[ gXp19 _ aVar ]';

addInstVarName: #gXp20;
compile:
'gXp20: aVar
	aVar notNil ifTrue:[ gXp20 _ aVar ]';
compile:
'p20: aVar
	aVar notNil ifTrue:[ gXp20 _ aVar ]'!

"qualifiers
 ----------"

GXprintdn

addInstVarName: #gXlocate;
compile:
'gXlocate
	gXlocate _ true';
compile:
'gXlocate: aBoolean
	gXlocate _ aBoolean'
!

"Local variables
 ---------------"

GXprintdn

addInstVarName: #gXbuff;

addInstVarName: #gXtmp
!

!GXprintdn methodsFor: 'initializing'!

initVars
	gXlocate _ false.
	gXbuff _ ''
!!

"BODY
 ----"

!GXprintdn methodsFor: 'running'!

evaluate
(gXlocate) ifTrue: [ 

    gXbuff _ ((self instVarAt: 1) at: 2) printString, '/'.
    gXbuff _ gXbuff, ((self instVarAt: 1) at:1) class name gtype, '/'
     ].

2 to: 21 do:
    [ :i |
        gXtmp _ self instVarAt: i.
        gXtmp notNil ifTrue: [gXbuff _ gXbuff, gXtmp printString]
    ].
(gXbuff, Nl) stringPrint: 'd'
.
self up.
^nil
!!
nil! "(fix for smalltalk bug, this line is ignored if there is a variable declaration error)"

"End of procedure"

GXcomment run line: 0 proc: self method: false; p1:'printdn'; p2:'Identical to PRINTDN but with a newline'; condEvaluate!
"Procedure class definition:
 ==========================="
((Smalltalk at: #GXdefined ifAbsent: [ nil ]) isKindOf: Function class) ifTrue: [ ('Warning: Overriding previously defined procedure ', 'GXdefined' gtype) prinl ].
Smalltalk at: #GXdefined put: nil!
Function subclass: 'GXdefined'
       instanceVariableNames: ''
       classVariableNames: ''
       poolDictionaries: ''
       category: 'Genie Procedure'
!

"HEADER
 ------"

"parameters
 ----------"

GXdefined

addInstVarName: #gXvar;
compile:
'gXvar: aVar
	aVar notNil ifTrue:[ gXvar _ aVar ]';
compile:
'p1: aVar
	aVar notNil ifTrue:[ gXvar _ aVar ]'!

"result
 ------"

GXdefined

addInstVarName: #gXres
!

!GXdefined methodsFor: 'initializing'!

initVars

!!

"BODY
 ----"

!GXdefined methodsFor: 'running'!

evaluate
 gXres _ gXvar notNil & (gXvar isMemberOf: GXArray) not .
self up.
^gXres
!!
nil! "(fix for smalltalk bug, this line is ignored if there is a variable declaration error)"

"End of procedure"

GXcomment run line: 0 proc: self method: false; p1:'defined'; p2:'Returns true if the variable has a defined value'; condEvaluate!
"Procedure class definition:
 ==========================="
((Smalltalk at: #GXdimensions ifAbsent: [ nil ]) isKindOf: Function class) ifTrue: [ ('Warning: Overriding previously defined procedure ', 'GXdimensions' gtype) prinl ].
Smalltalk at: #GXdimensions put: nil!
Function subclass: 'GXdimensions'
       instanceVariableNames: ''
       classVariableNames: ''
       poolDictionaries: ''
       category: 'Genie Procedure'
!

"HEADER
 ------"

"parameters
 ----------"

GXdimensions

addInstVarName: #gXdim1;
compile:
'gXdim1: aVar
	aVar notNil ifTrue:[gXdim1 _ self typeCheck: aVar shouldBe: GXInteger paramName: ''gXdim1'']';
compile:
'p1: aVar
	aVar notNil ifTrue:[gXdim1 _ self typeCheck: aVar shouldBe: GXInteger paramName: ''gXdim1'']';

addInstVarName: #gXdim2;
compile:
'gXdim2: aVar
	aVar notNil ifTrue:[gXdim2 _ self typeCheck: aVar shouldBe: GXInteger paramName: ''gXdim2'']';
compile:
'p2: aVar
	aVar notNil ifTrue:[gXdim2 _ self typeCheck: aVar shouldBe: GXInteger paramName: ''gXdim2'']';

addInstVarName: #gXdim3;
compile:
'gXdim3: aVar
	aVar notNil ifTrue:[gXdim3 _ self typeCheck: aVar shouldBe: GXInteger paramName: ''gXdim3'']';
compile:
'p3: aVar
	aVar notNil ifTrue:[gXdim3 _ self typeCheck: aVar shouldBe: GXInteger paramName: ''gXdim3'']';

addInstVarName: #gXdim4;
compile:
'gXdim4: aVar
	aVar notNil ifTrue:[gXdim4 _ self typeCheck: aVar shouldBe: GXInteger paramName: ''gXdim4'']';
compile:
'p4: aVar
	aVar notNil ifTrue:[gXdim4 _ self typeCheck: aVar shouldBe: GXInteger paramName: ''gXdim4'']';

addInstVarName: #gXdim5;
compile:
'gXdim5: aVar
	aVar notNil ifTrue:[gXdim5 _ self typeCheck: aVar shouldBe: GXInteger paramName: ''gXdim5'']';
compile:
'p5: aVar
	aVar notNil ifTrue:[gXdim5 _ self typeCheck: aVar shouldBe: GXInteger paramName: ''gXdim5'']';

addInstVarName: #gXdim6;
compile:
'gXdim6: aVar
	aVar notNil ifTrue:[gXdim6 _ self typeCheck: aVar shouldBe: GXInteger paramName: ''gXdim6'']';
compile:
'p6: aVar
	aVar notNil ifTrue:[gXdim6 _ self typeCheck: aVar shouldBe: GXInteger paramName: ''gXdim6'']';

addInstVarName: #gXdim7;
compile:
'gXdim7: aVar
	aVar notNil ifTrue:[gXdim7 _ self typeCheck: aVar shouldBe: GXInteger paramName: ''gXdim7'']';
compile:
'p7: aVar
	aVar notNil ifTrue:[gXdim7 _ self typeCheck: aVar shouldBe: GXInteger paramName: ''gXdim7'']';

addInstVarName: #gXdim8;
compile:
'gXdim8: aVar
	aVar notNil ifTrue:[gXdim8 _ self typeCheck: aVar shouldBe: GXInteger paramName: ''gXdim8'']';
compile:
'p8: aVar
	aVar notNil ifTrue:[gXdim8 _ self typeCheck: aVar shouldBe: GXInteger paramName: ''gXdim8'']';

addInstVarName: #gXdim9;
compile:
'gXdim9: aVar
	aVar notNil ifTrue:[gXdim9 _ self typeCheck: aVar shouldBe: GXInteger paramName: ''gXdim9'']';
compile:
'p9: aVar
	aVar notNil ifTrue:[gXdim9 _ self typeCheck: aVar shouldBe: GXInteger paramName: ''gXdim9'']';

addInstVarName: #gXdim10;
compile:
'gXdim10: aVar
	aVar notNil ifTrue:[gXdim10 _ self typeCheck: aVar shouldBe: GXInteger paramName: ''gXdim10'']';
compile:
'p10: aVar
	aVar notNil ifTrue:[gXdim10 _ self typeCheck: aVar shouldBe: GXInteger paramName: ''gXdim10'']'!

"result
 ------"

GXdimensions

addInstVarName: #gXres
!

"Local variables
 ---------------"

GXdimensions

addInstVarName: #gXtemp;

addInstVarName: #gXdims
!

!GXdimensions methodsFor: 'initializing'!

initVars

!!

"BODY
 ----"

!GXdimensions methodsFor: 'running'!

evaluate
Function new loop: 10 to: 1 by: 1 negated before: [ :gXi | 
 gXtemp _ self instVarAt: gXi + 2 .
(GXdefined run line: 0 proc: self method: true; p1:(gXtemp); condEvaluate) ifTrue: [ 
((gXtemp) <= 0) ifTrue: [ 
gXtemp _ 1 deepCopy.
GXprinten run line: 0 proc: self method: false; p1:'DIMENSIONS: A zero or -ve array dimension was specified'; condEvaluate.
GXprintin run line: 0 proc: self method: false; p1:'	(The dimension has been made 1 by default)'; condEvaluate ].
(GXdefined run line: 0 proc: self method: true; p1:(gXdims); condEvaluate) not ifTrue: [ 
 gXdims _ Array new: gXi  ].
 gXdims at: gXi put: gXtemp  ] ] 
condition: [false] after: [ :gXi | nil ].
(GXdefined run line: 0 proc: self method: true; p1:(gXdims); condEvaluate) not ifTrue: [ 
GXprinten run line: 0 proc: self method: false; p1:'DIMENSIONS: Don''t be silly chum, can''t create a dimensionless array!'; condEvaluate ] ifFalse: [ 
 gXres _ GXArray new: gXdims  ] .
self up.
^gXres
!!
nil! "(fix for smalltalk bug, this line is ignored if there is a variable declaration error)"

"End of procedure"

GXcomment run line: 0 proc: self method: false; p1:'dimensions'; p2:'Initializes a GENIE single or multi-dimensional array'; condEvaluate!
"Procedure class definition:
 ==========================="
((Smalltalk at: #GXdimensionality ifAbsent: [ nil ]) isKindOf: Function class) ifTrue: [ ('Warning: Overriding previously defined procedure ', 'GXdimensionality' gtype) prinl ].
Smalltalk at: #GXdimensionality put: nil!
Function subclass: 'GXdimensionality'
       instanceVariableNames: ''
       classVariableNames: ''
       poolDictionaries: ''
       category: 'Genie Procedure'
!

"HEADER
 ------"

"parameters
 ----------"

GXdimensionality

addInstVarName: #gXarray;
compile:
'gXarray: aVar
	aVar notNil ifTrue:[ gXarray _ aVar ]';
compile:
'p1: aVar
	aVar notNil ifTrue:[ gXarray _ aVar ]'!

"result
 ------"

GXdimensionality

addInstVarName: #gXres
!

!GXdimensionality methodsFor: 'initializing'!

initVars

!!

"BODY
 ----"

!GXdimensionality methodsFor: 'running'!

evaluate
 gXres _ gXarray dimensionality.
self up.
^gXres
!!
nil! "(fix for smalltalk bug, this line is ignored if there is a variable declaration error)"

"End of procedure"

GXcomment run line: 0 proc: self method: false; p1:'dimensionality'; p2:'Finds the dimensionality of a genie array'; condEvaluate!
"Procedure class definition:
 ==========================="
((Smalltalk at: #GXsearch ifAbsent: [ nil ]) isKindOf: Function class) ifTrue: [ ('Warning: Overriding previously defined procedure ', 'GXsearch' gtype) prinl ].
Smalltalk at: #GXsearch put: nil!
Function subclass: 'GXsearch'
       instanceVariableNames: ''
       classVariableNames: ''
       poolDictionaries: ''
       category: 'Genie Procedure'
!

"HEADER
 ------"

"parameters
 ----------"

GXsearch

addInstVarName: #gXstart;
compile:
'gXstart: aVar
	aVar notNil ifTrue:[gXstart _ self typeCheck: aVar shouldBe: GXString paramName: ''gXstart'']';
compile:
'p1: aVar
	aVar notNil ifTrue:[gXstart _ self typeCheck: aVar shouldBe: GXString paramName: ''gXstart'']'!

"qualifiers
 ----------"

GXsearch

addInstVarName: #gXproc;
compile:
'gXproc
	gXproc _ true';
compile:
'gXproc: aBoolean
	gXproc _ aBoolean';

addInstVarName: #gXvar;
compile:
'gXvar
	gXvar _ true';
compile:
'gXvar: aBoolean
	gXvar _ aBoolean';

addInstVarName: #gXconst;
compile:
'gXconst
	gXconst _ true';
compile:
'gXconst: aBoolean
	gXconst _ aBoolean';

addInstVarName: #gXsys;
compile:
'gXsys
	gXsys _ true';
compile:
'gXsys: aBoolean
	gXsys _ aBoolean';

addInstVarName: #gXconst2;
compile:
'gXconst2
	gXconst2 _ true';
compile:
'gXconst2: aBoolean
	gXconst2 _ aBoolean';

addInstVarName: #gXtype2;
compile:
'gXtype2
	gXtype2 _ true';
compile:
'gXtype2: aBoolean
	gXtype2 _ aBoolean';

addInstVarName: #gXtype;
compile:
'gXtype
	gXtype _ true';
compile:
'gXtype: aBoolean
	gXtype _ aBoolean'
!

!GXsearch methodsFor: 'initializing'!

initVars
	gXproc _ false.
	gXvar _ false.
	gXconst _ false.
	gXsys _ false.
	gXconst2 _ false.
	gXtype2 _ false.
	gXtype _ false
!!

"BODY
 ----"

!GXsearch methodsFor: 'running'!

evaluate


|allKeys1 allKeys2 |

allKeys1_Smalltalk keys select:[:k|k size>=4].
allKeys2_Smalltalk keys select:[:a|a size>=3].

gXproc
ifTrue:
[ (allKeys2_allKeys2 select:[:a|((a at:1)= $G)
			& ((a at:2)=$X) 
			& (((a at:3)~=$A) & ((a at:3)~=$B) & ((a at:3)~=$C)
			& ((a at:3)~=$D) & ((a at:3)~=$E) & ((a at:3)~=$F)
			& ((a at:3)~=$G) & ((a at:3)~=$H) & ((a at:3)~=$I)
			& ((a at:3)~=$J) & ((a at:3)~=$K) & ((a at:3)~=$L)
			& ((a at:3)~=$M) & ((a at:3)~=$N) & ((a at:3)~=$O)
			& ((a at:3)~=$P) & ((a at:3)~=$Q) & ((a at:3)~=$R)
			& ((a at:3)~=$S) & ((a at:3)~=$T) & ((a at:3)~=$U)
			& ((a at:3)~=$V) & ((a at:3)~=$W) & ((a at:3)~=$X)
			& ((a at:3)~=$Y) & ((a at:3)~=$Z))
			])sort do:[:i|i gtype prn.
				(':   ' )prn.
				(Smalltalk at: i) comment prnl.
				 (' ') prnl]].
	

				
gXvar
ifTrue:
[(allKeys2_allKeys2 select:[:a|((a at:1)=$g) 
			& ((a at:2)=$X)
			& (((a at:3)~=$D) & ((a at:3)~=$U))
			]) sort do:[:i|i gtype prn.
				(':   ' )prn.
				(Smalltalk at: i) species prnl.
				(' ') prnl]].



gXconst
ifTrue:
[(allKeys1_allKeys1 select:[:k|((k at:1)=$g)
			& ((k at:2)=$X)
			& ((k at:3)=$D) 
			& ((k at:4)~=$U) 
			]) sort do:[:i|i gtype prn.
				(':   ' )prn.
				(Smalltalk at: i) species prnl.
				(' ') prnl]].

gXsys
ifTrue:
[(allKeys1_allKeys1 select:[:k|((k at:1)=$g)
			& ((k at:2)=$X)
			& ((k at:3)=$U)
			& ((k at:4)~=$U)
			]) sort do:[:i|i gtype prn.
				(':   ' )prn.
				(Smalltalk at: i) species prnl.
				(' 'prnl)]].

gXconst2
ifTrue:					
[(allKeys1_allKeys1 select:[:k|((k at:1)=$g)
			& ((k at:2)=$X)
			& ((k at:3)=$D) 
			& ((k at:4)=$U) 
			]) sort do:[:i|i gtype prn.
				(':   ' )prn.
				(Smalltalk at: i) species prnl.
				(' ' prnl)]]. 
			
		
gXtype2
ifTrue:					
[(allKeys1_allKeys1 select:[:k|((k at:1)=$G)
			& ((k at:2)=$X)	
			& ((k at:3)=$U)
			&(((k at:4)=$A) | ((k at:4)=$B) | ((k at:4)=$C)
                        | ((k at:4)=$D) | ((k at:4)=$E) | ((k at:4)=$F)
                        | ((k at:4)=$G) | ((k at:4)=$H) | ((k at:4)=$I)
                        | ((k at:4)=$J) | ((k at:4)=$K) | ((k at:4)=$L)
                        | ((k at:4)=$M) | ((k at:4)=$N) | ((k at:4)=$O)
                        | ((k at:4)=$P) | ((k at:4)=$Q) | ((k at:4)=$R)
			| ((k at:4)=$S) | ((k at:4)=$T) | ((k at:4)=$U)
			| ((k at:4)=$V) | ((k at:4)=$W) | ((k at:4)=$X)
			| ((k at:4)=$Y) | ((k at:4)=$Z))			
			]) sort do:[:i|i gtype prn.
				(':   ' )prn.
				(Smalltalk at: i) comment prnl.
				(' ') prnl]].


gXtype
ifTrue:
[(allKeys2_allKeys2 select:[:a|((a at:1)=$G) 
			& ((a at:2)=$X)
                        & ((a at:3)~=$U) & (((a at:3)=$A) | ((a at:3)=$B) 
			| ((a at:3)=$C)
                        | ((a at:3)=$D) | ((a at:3)=$E) | ((a at:3)=$F)
                        | ((a at:3)=$G) | ((a at:3)=$H) | ((a at:3)=$I)
                        | ((a at:3)=$J) | ((a at:3)=$K) | ((a at:3)=$L)
                        | ((a at:3)=$M) | ((a at:3)=$N) | ((a at:3)=$O)
                        | ((a at:3)=$P) | ((a at:3)=$Q) | ((a at:3)=$R)
			| ((a at:3)=$S) | ((a at:3)=$T) | ((a at:3)=$V) 
			| ((a at:3)=$W) | ((a at:3)=$X) | ((a at:3)=$Y)
			| ((a at:3)=$Z))	
			]) sort do:[:i|i gtype prn.
				(':   ' )prn.
				(Smalltalk at: i) comment prnl.
				(' ' prnl)]]

.
self up.
^nil
!!
nil! "(fix for smalltalk bug, this line is ignored if there is a variable declaration error)"

"End of procedure"

GXcomment run line: 0 proc: self method: false; p1:'search'; p2:'(Used to implement the DUMP and SHOWPROCS procedures)'; condEvaluate!
"Procedure class definition:
 ==========================="
((Smalltalk at: #GXload ifAbsent: [ nil ]) isKindOf: Function class) ifTrue: [ ('Warning: Overriding previously defined procedure ', 'GXload' gtype) prinl ].
Smalltalk at: #GXload put: nil!
Function subclass: 'GXload'
       instanceVariableNames: ''
       classVariableNames: ''
       poolDictionaries: ''
       category: 'Genie Procedure'
!

"HEADER
 ------"

"parameters
 ----------"

GXload

addInstVarName: #gXfilename;
compile:
'gXfilename: aVar
	aVar notNil ifTrue:[gXfilename _ self typeCheck: aVar shouldBe: GXString paramName: ''gXfilename'']';
compile:
'p1: aVar
	aVar notNil ifTrue:[gXfilename _ self typeCheck: aVar shouldBe: GXString paramName: ''gXfilename'']'!

"result
 ------"

GXload

addInstVarName: #gXres
!

"Global variables
 ----------------"

GXload

addPoolVar: #gXUUloadingUfile value: ''!

"Local variables
 ---------------"

GXload

addInstVarName: #gXoldUfile
!

!GXload methodsFor: 'initializing'!

initVars
	gXoldUfile _ (gXUUloadingUfile)
!!

"BODY
 ----"

!GXload methodsFor: 'running'!

evaluate
gXUUloadingUfile _ (gXfilename) deepCopy.
(GXdefined run line: 0 proc: self method: true; p1:(gXfilename); condEvaluate) ifTrue: [ 
 gXres _ self loadTheCode: gXfilename  ] ifFalse: [ 
GXprintn run line: 0 proc: self method: false; p1:'Please give a filename'; condEvaluate.
gXres _ (gXDfalse) deepCopy ] .
gXUUloadingUfile _ (gXoldUfile) deepCopy.
self up.
^gXres
!!
nil! "(fix for smalltalk bug, this line is ignored if there is a variable declaration error)"

"End of procedure"

GXcomment run line: 0 proc: self method: false; p1:'load'; p2:'Loads a file containing GCL procedures into GENIE'; condEvaluate!
"Procedure class definition:
 ==========================="
((Smalltalk at: #GXsave ifAbsent: [ nil ]) isKindOf: Function class) ifTrue: [ ('Warning: Overriding previously defined procedure ', 'GXsave' gtype) prinl ].
Smalltalk at: #GXsave put: nil!
Function subclass: 'GXsave'
       instanceVariableNames: ''
       classVariableNames: ''
       poolDictionaries: ''
       category: 'Genie Procedure'
!

"HEADER
 ------"

"parameters
 ----------"

GXsave

addInstVarName: #gXfilename;
compile:
'gXfilename: aVar
	aVar notNil ifTrue:[gXfilename _ self typeCheck: aVar shouldBe: GXString paramName: ''gXfilename'']';
compile:
'p1: aVar
	aVar notNil ifTrue:[gXfilename _ self typeCheck: aVar shouldBe: GXString paramName: ''gXfilename'']'
!

!GXsave methodsFor: 'initializing'!

initVars

!!

"BODY
 ----"

!GXsave methodsFor: 'running'!

evaluate
(GXdefined run line: 0 proc: self method: true; p1:(gXfilename); condEvaluate) ifTrue: [ 
GXprintin run line: 0 proc: self method: false; p1:'Saving GENIE in file '; p2:(gXfilename); p3:' ...'; condEvaluate.
 Smalltalk snapshot: gXfilename  ] ifFalse: [ 
GXprintn run line: 0 proc: self method: false; p1:'Please give a filename'; condEvaluate ] .
self up.
^nil
!!
nil! "(fix for smalltalk bug, this line is ignored if there is a variable declaration error)"

"End of procedure"

GXcomment run line: 0 proc: self method: false; p1:'save'; p2:'Saves the current state of GENIE as a Smalltalk image'; condEvaluate!
"Procedure class definition:
 ==========================="
((Smalltalk at: #GXasUstring ifAbsent: [ nil ]) isKindOf: Function class) ifTrue: [ ('Warning: Overriding previously defined procedure ', 'GXasUstring' gtype) prinl ].
Smalltalk at: #GXasUstring put: nil!
Function subclass: 'GXasUstring'
       instanceVariableNames: ''
       classVariableNames: ''
       poolDictionaries: ''
       category: 'Genie Procedure'
!

"HEADER
 ------"

"parameters
 ----------"

GXasUstring

addInstVarName: #gXvar;
compile:
'gXvar: aVar
	aVar notNil ifTrue:[ gXvar _ aVar ]';
compile:
'p1: aVar
	aVar notNil ifTrue:[ gXvar _ aVar ]'!

"result
 ------"

GXasUstring

addInstVarName: #gXres
!

!GXasUstring methodsFor: 'initializing'!

initVars

!!

"BODY
 ----"

!GXasUstring methodsFor: 'running'!

evaluate
 gXres _ gXvar printString .
self up.
^gXres
!!
nil! "(fix for smalltalk bug, this line is ignored if there is a variable declaration error)"

"End of procedure"

GXcomment run line: 0 proc: self method: false; p1:'as_string'; p2:'Converts a genie variable into a printable string'; condEvaluate!
"Procedure class definition:
 ==========================="
((Smalltalk at: #GXasUinteger ifAbsent: [ nil ]) isKindOf: Function class) ifTrue: [ ('Warning: Overriding previously defined procedure ', 'GXasUinteger' gtype) prinl ].
Smalltalk at: #GXasUinteger put: nil!
Function subclass: 'GXasUinteger'
       instanceVariableNames: ''
       classVariableNames: ''
       poolDictionaries: ''
       category: 'Genie Procedure'
!

"HEADER
 ------"

"parameters
 ----------"

GXasUinteger

addInstVarName: #gXvar;
compile:
'gXvar: aVar
	aVar notNil ifTrue:[gXvar _ self typeCheck: aVar shouldBe: GXReal paramName: ''gXvar'']';
compile:
'p1: aVar
	aVar notNil ifTrue:[gXvar _ self typeCheck: aVar shouldBe: GXReal paramName: ''gXvar'']'!

"result
 ------"

GXasUinteger

addInstVarName: #gXres
!

!GXasUinteger methodsFor: 'initializing'!

initVars

!!

"BODY
 ----"

!GXasUinteger methodsFor: 'running'!

evaluate
 gXres _ gXvar truncated .
self up.
^gXres
!!
nil! "(fix for smalltalk bug, this line is ignored if there is a variable declaration error)"

"End of procedure"

GXcomment run line: 0 proc: self method: false; p1:'as_integer'; p2:'Converts a genie real variable to an integer'; condEvaluate!
"Procedure class definition:
 ==========================="
((Smalltalk at: #GXasUvariable ifAbsent: [ nil ]) isKindOf: Function class) ifTrue: [ ('Warning: Overriding previously defined procedure ', 'GXasUvariable' gtype) prinl ].
Smalltalk at: #GXasUvariable put: nil!
Function subclass: 'GXasUvariable'
       instanceVariableNames: ''
       classVariableNames: ''
       poolDictionaries: ''
       category: 'Genie Procedure'
!

"HEADER
 ------"

"parameters
 ----------"

GXasUvariable

addInstVarName: #gXastring;
compile:
'gXastring: aVar
	aVar notNil ifTrue:[gXastring _ self typeCheck: aVar shouldBe: GXString paramName: ''gXastring'']';
compile:
'p1: aVar
	aVar notNil ifTrue:[gXastring _ self typeCheck: aVar shouldBe: GXString paramName: ''gXastring'']'!

"result
 ------"

GXasUvariable

addInstVarName: #gXres
!

!GXasUvariable methodsFor: 'initializing'!

initVars

!!

"BODY
 ----"

!GXasUvariable methodsFor: 'running'!

evaluate
gXres _ ((gXastring) asVariable) deepCopy.
self up.
^gXres
!!
nil! "(fix for smalltalk bug, this line is ignored if there is a variable declaration error)"

"End of procedure"

GXcomment run line: 0 proc: self method: false; p1:'as_variable'; p2:'Converts a string into a genie variable of appropriate type'; condEvaluate!
"Procedure class definition:
 ==========================="
((Smalltalk at: #GXexit ifAbsent: [ nil ]) isKindOf: Function class) ifTrue: [ ('Warning: Overriding previously defined procedure ', 'GXexit' gtype) prinl ].
Smalltalk at: #GXexit put: nil!
Function subclass: 'GXexit'
       instanceVariableNames: ''
       classVariableNames: ''
       poolDictionaries: ''
       category: 'Genie Procedure'
!

"BODY
 ----"

!GXexit methodsFor: 'running'!

evaluate
GXprintin run line: 0 proc: self method: false; p1:'GENIE exiting'; condEvaluate.
 self quitTheLot .
self up.
^nil
!!
nil! "(fix for smalltalk bug, this line is ignored if there is a variable declaration error)"

"End of procedure"

GXcomment run line: 0 proc: self method: false; p1:'exit'; p2:'exits GENIE'; condEvaluate!
"Procedure class definition:
 ==========================="
((Smalltalk at: #GXquit ifAbsent: [ nil ]) isKindOf: Function class) ifTrue: [ ('Warning: Overriding previously defined procedure ', 'GXquit' gtype) prinl ].
Smalltalk at: #GXquit put: nil!
Function subclass: 'GXquit'
       instanceVariableNames: ''
       classVariableNames: ''
       poolDictionaries: ''
       category: 'Genie Procedure'
!

"BODY
 ----"

!GXquit methodsFor: 'running'!

evaluate
GXexit run line: 0 proc: self method: false; condEvaluate.
self up.
^nil
!!
nil! "(fix for smalltalk bug, this line is ignored if there is a variable declaration error)"

"End of procedure"

GXcomment run line: 0 proc: self method: false; p1:'quit'; p2:'exits GENIE'; condEvaluate!
"Procedure class definition:
 ==========================="
((Smalltalk at: #GXisUa ifAbsent: [ nil ]) isKindOf: Function class) ifTrue: [ ('Warning: Overriding previously defined procedure ', 'GXisUa' gtype) prinl ].
Smalltalk at: #GXisUa put: nil!
Function subclass: 'GXisUa'
       instanceVariableNames: ''
       classVariableNames: ''
       poolDictionaries: ''
       category: 'Genie Procedure'
!

"HEADER
 ------"

"parameters
 ----------"

GXisUa

addInstVarName: #gXvar;
compile:
'gXvar: aVar
	aVar notNil ifTrue:[ gXvar _ aVar ]';
compile:
'p1: aVar
	aVar notNil ifTrue:[ gXvar _ aVar ]';

addInstVarName: #gXtypestring;
compile:
'gXtypestring: aVar
	aVar notNil ifTrue:[gXtypestring _ self typeCheck: aVar shouldBe: GXString paramName: ''gXtypestring'']';
compile:
'p2: aVar
	aVar notNil ifTrue:[gXtypestring _ self typeCheck: aVar shouldBe: GXString paramName: ''gXtypestring'']'!

"result
 ------"

GXisUa

addInstVarName: #gXres
!

!GXisUa methodsFor: 'initializing'!

initVars

!!

"BODY
 ----"

!GXisUa methodsFor: 'running'!

evaluate
 
| strclass strtype |
strclass _ gXvar class printString asLowercase.         "Work in lowercase"

"Special case as literals can't be controlled into a GENIE type"
(strclass = 'float')   ifTrue: [ strclass _ 'gxreal' ].
(strclass = 'integer') ifTrue: [ strclass _ 'gxinteger' ].
(strclass = 'string' ) ifTrue: [ strclass _ 'gxstring' ].

strclass _ strclass copyFrom: 3 to: strclass size.      "Remove the GX bit"
strtype	_ gXtypestring asLowercase.

gXres _ ( strtype = strclass )

.
self up.
^gXres
!!
nil! "(fix for smalltalk bug, this line is ignored if there is a variable declaration error)"

"End of procedure"

GXcomment run line: 0 proc: self method: false; p1:'is_a'; p2:'Checks that the variable is of the type specified'; condEvaluate!
"Procedure class definition:
 ==========================="
((Smalltalk at: #GXlength ifAbsent: [ nil ]) isKindOf: Function class) ifTrue: [ ('Warning: Overriding previously defined procedure ', 'GXlength' gtype) prinl ].
Smalltalk at: #GXlength put: nil!
Function subclass: 'GXlength'
       instanceVariableNames: ''
       classVariableNames: ''
       poolDictionaries: ''
       category: 'Genie Procedure'
!

"HEADER
 ------"

"parameters
 ----------"

GXlength

addInstVarName: #gXitem;
compile:
'gXitem: aVar
	aVar notNil ifTrue:[ gXitem _ aVar ]';
compile:
'p1: aVar
	aVar notNil ifTrue:[ gXitem _ aVar ]'!

"result
 ------"

GXlength

addInstVarName: #gXsize
!

!GXlength methodsFor: 'initializing'!

initVars

!!

"BODY
 ----"

!GXlength methodsFor: 'running'!

evaluate
 gXsize _ gXitem size .
self up.
^gXsize
!!
nil! "(fix for smalltalk bug, this line is ignored if there is a variable declaration error)"

"End of procedure"

GXcomment run line: 0 proc: self method: false; p1:'length'; p2:'Returns the length of any type in number of elements'; condEvaluate!
"Procedure class definition:
 ==========================="
((Smalltalk at: #GXsubstring ifAbsent: [ nil ]) isKindOf: Function class) ifTrue: [ ('Warning: Overriding previously defined procedure ', 'GXsubstring' gtype) prinl ].
Smalltalk at: #GXsubstring put: nil!
Function subclass: 'GXsubstring'
       instanceVariableNames: ''
       classVariableNames: ''
       poolDictionaries: ''
       category: 'Genie Procedure'
!

"HEADER
 ------"

"parameters
 ----------"

GXsubstring

addInstVarName: #gXastring;
compile:
'gXastring: aVar
	aVar notNil ifTrue:[gXastring _ self typeCheck: aVar shouldBe: GXString paramName: ''gXastring'']';
compile:
'p1: aVar
	aVar notNil ifTrue:[gXastring _ self typeCheck: aVar shouldBe: GXString paramName: ''gXastring'']';

addInstVarName: #gXstart;
compile:
'gXstart: aVar
	aVar notNil ifTrue:[gXstart _ self typeCheck: aVar shouldBe: GXInteger paramName: ''gXstart'']';
compile:
'p2: aVar
	aVar notNil ifTrue:[gXstart _ self typeCheck: aVar shouldBe: GXInteger paramName: ''gXstart'']';

addInstVarName: #gXlength;
compile:
'gXlength: aVar
	aVar notNil ifTrue:[gXlength _ self typeCheck: aVar shouldBe: GXInteger paramName: ''gXlength'']';
compile:
'p3: aVar
	aVar notNil ifTrue:[gXlength _ self typeCheck: aVar shouldBe: GXInteger paramName: ''gXlength'']'!

"result
 ------"

GXsubstring

addInstVarName: #gXasubstring
!

"Local variables
 ---------------"

GXsubstring

addInstVarName: #gXendpos;

addInstVarName: #gXstart
!

!GXsubstring methodsFor: 'initializing'!

initVars
	gXasubstring _ ''.
	gXstart _ 1
!!

"BODY
 ----"

!GXsubstring methodsFor: 'running'!

evaluate
(GXdefined run line: 0 proc: self method: true; p1:(gXlength); condEvaluate) not ifTrue: [ 
gXlength _ (((GXlength run line: 0 proc: self method: true; p1:(gXastring); condEvaluate) - (gXstart)) + 1) deepCopy ].
gXendpos _ (((gXstart) + (gXlength)) - 1) deepCopy.
((gXlength) >= 0) ifTrue: [ 
 gXasubstring _ gXastring copyFrom: gXstart to: gXendpos  ] ifFalse: [ 
GXprinten run line: 0 proc: self method: false; p1:'substring() Can''t take a substring of -ve length'; condEvaluate ] .
self up.
^gXasubstring
!!
nil! "(fix for smalltalk bug, this line is ignored if there is a variable declaration error)"

"End of procedure"

GXcomment run line: 0 proc: self method: false; p1:'substring'; p2:'Extracts a substring from a string'; condEvaluate!
"Procedure class definition:
 ==========================="
((Smalltalk at: #GXlocate ifAbsent: [ nil ]) isKindOf: Function class) ifTrue: [ ('Warning: Overriding previously defined procedure ', 'GXlocate' gtype) prinl ].
Smalltalk at: #GXlocate put: nil!
Function subclass: 'GXlocate'
       instanceVariableNames: ''
       classVariableNames: ''
       poolDictionaries: ''
       category: 'Genie Procedure'
!

"HEADER
 ------"

"parameters
 ----------"

GXlocate

addInstVarName: #gXastring;
compile:
'gXastring: aVar
	aVar notNil ifTrue:[ gXastring _ aVar ]';
compile:
'p1: aVar
	aVar notNil ifTrue:[ gXastring _ aVar ]';

addInstVarName: #gXsubstring;
compile:
'gXsubstring: aVar
	aVar notNil ifTrue:[ gXsubstring _ aVar ]';
compile:
'p2: aVar
	aVar notNil ifTrue:[ gXsubstring _ aVar ]';

addInstVarName: #gXcount;
compile:
'gXcount: aVar
	aVar notNil ifTrue:[gXcount _ self typeCheck: aVar shouldBe: GXInteger paramName: ''gXcount'']';
compile:
'p3: aVar
	aVar notNil ifTrue:[gXcount _ self typeCheck: aVar shouldBe: GXInteger paramName: ''gXcount'']'!

"result
 ------"

GXlocate

addInstVarName: #gXtheindex
!

"Local variables
 ---------------"

GXlocate

addInstVarName: #gXreturn;

addInstVarName: #gXcount
!

!GXlocate methodsFor: 'initializing'!

initVars
	gXreturn _ 0.
	gXcount _ 1
!!

"BODY
 ----"

!GXlocate methodsFor: 'running'!

evaluate
((gXcount) = 0) ifTrue: [ 
gXcount _ 1 deepCopy.
gXreturn _ (GXlength run line: 0 proc: self method: true; p1:(gXastring); condEvaluate) deepCopy ].

	gXtheindex _ 0.
	1 to: gXcount do:
	[
		gXtheindex _ gXastring indexOfSubCollection: gXsubstring startingAt: gXtheindex + 1 ifAbsent: [ gXreturn ]
	]
.
self up.
^gXtheindex
!!
nil! "(fix for smalltalk bug, this line is ignored if there is a variable declaration error)"

"End of procedure"

GXcomment run line: 0 proc: self method: false; p1:'locate'; p2:'Finds the index of an element in a string or array'; condEvaluate!
"Procedure class definition:
 ==========================="
((Smalltalk at: #GXreadUterminal ifAbsent: [ nil ]) isKindOf: Function class) ifTrue: [ ('Warning: Overriding previously defined procedure ', 'GXreadUterminal' gtype) prinl ].
Smalltalk at: #GXreadUterminal put: nil!
Function subclass: 'GXreadUterminal'
       instanceVariableNames: ''
       classVariableNames: ''
       poolDictionaries: ''
       category: 'Genie Procedure'
!

"HEADER
 ------"

"result
 ------"

GXreadUterminal

addInstVarName: #gXres
!

!GXreadUterminal methodsFor: 'initializing'!

initVars
	gXres _ ''
!!

"BODY
 ----"

!GXreadUterminal methodsFor: 'running'!

evaluate
 gXres _ gXres stringRead .
self up.
^gXres
!!
nil! "(fix for smalltalk bug, this line is ignored if there is a variable declaration error)"

"End of procedure"

GXcomment run line: 0 proc: self method: false; p1:'read_terminal'; p2:'reads a string directly from the controlling terminal'; condEvaluate!
"Procedure class definition:
 ==========================="
((Smalltalk at: #GXUsystem ifAbsent: [ nil ]) isKindOf: Function class) ifTrue: [ ('Warning: Overriding previously defined procedure ', 'GXUsystem' gtype) prinl ].
Smalltalk at: #GXUsystem put: nil!
Function subclass: 'GXUsystem'
       instanceVariableNames: ''
       classVariableNames: ''
       poolDictionaries: ''
       category: 'Genie Procedure'
!

"HEADER
 ------"

"parameters
 ----------"

GXUsystem

addInstVarName: #gXcommand;
compile:
'gXcommand: aVar
	aVar notNil ifTrue:[gXcommand _ self typeCheck: aVar shouldBe: GXString paramName: ''gXcommand'']';
compile:
'p1: aVar
	aVar notNil ifTrue:[gXcommand _ self typeCheck: aVar shouldBe: GXString paramName: ''gXcommand'']'!

"result
 ------"

GXUsystem

addInstVarName: #gXres
!

"Local variables
 ---------------"

GXUsystem

addInstVarName: #gXcommand
!

!GXUsystem methodsFor: 'initializing'!

initVars
	gXcommand _ ''
!!

"BODY
 ----"

!GXUsystem methodsFor: 'running'!

evaluate
GXcallout run line: 0 proc: self method: false; p1:'lowSystem'; p2:1; condEvaluate.
self up.
^gXres
!!
nil! "(fix for smalltalk bug, this line is ignored if there is a variable declaration error)"

"End of procedure"

"Procedure class definition:
 ==========================="
((Smalltalk at: #GXUgetenv ifAbsent: [ nil ]) isKindOf: Function class) ifTrue: [ ('Warning: Overriding previously defined procedure ', 'GXUgetenv' gtype) prinl ].
Smalltalk at: #GXUgetenv put: nil!
Function subclass: 'GXUgetenv'
       instanceVariableNames: ''
       classVariableNames: ''
       poolDictionaries: ''
       category: 'Genie Procedure'
!

"HEADER
 ------"

"parameters
 ----------"

GXUgetenv

addInstVarName: #gXname;
compile:
'gXname: aVar
	aVar notNil ifTrue:[gXname _ self typeCheck: aVar shouldBe: GXString paramName: ''gXname'']';
compile:
'p1: aVar
	aVar notNil ifTrue:[gXname _ self typeCheck: aVar shouldBe: GXString paramName: ''gXname'']'!

"result
 ------"

GXUgetenv

addInstVarName: #gXvalue
!

!GXUgetenv methodsFor: 'initializing'!

initVars

!!

"BODY
 ----"

!GXUgetenv methodsFor: 'running'!

evaluate
GXcallout run line: 0 proc: self method: false; p1:'lowSystem'; p2:2; condEvaluate.
self up.
^gXvalue
!!
nil! "(fix for smalltalk bug, this line is ignored if there is a variable declaration error)"

"End of procedure"

"Procedure class definition:
 ==========================="
((Smalltalk at: #GXos ifAbsent: [ nil ]) isKindOf: Function class) ifTrue: [ ('Warning: Overriding previously defined procedure ', 'GXos' gtype) prinl ].
Smalltalk at: #GXos put: nil!
Function subclass: 'GXos'
       instanceVariableNames: ''
       classVariableNames: ''
       poolDictionaries: ''
       category: 'Genie Procedure'
!

"HEADER
 ------"

"result
 ------"

GXos

addInstVarName: #gXres
!

"Local variables
 ---------------"

GXos

addInstVarName: #gXdetails;

addInstVarName: #gXres
!

!GXos methodsFor: 'initializing'!

initVars
	gXres _ '<unknown>'
!!

"BODY
 ----"

!GXos methodsFor: 'running'!

evaluate
gXdetails _ (GXUgetenv run line: 0 proc: self method: true; p1:'GENIE_MACHINE'; condEvaluate) deepCopy.
((GXlocate run line: 0 proc: self method: true; p1:(gXdetails); p2:'OSF'; condEvaluate) > 0) ifTrue: [ 
gXres _ 'OSF' deepCopy ] ifFalse: [ ((GXlocate run line: 0 proc: self method: true; p1:(gXdetails); p2:'ULTRIX'; condEvaluate) > 0) ifTrue: [ 
gXres _ 'ULTRIX' deepCopy ] ifFalse: [ ((GXlocate run line: 0 proc: self method: true; p1:(gXdetails); p2:'VMS'; condEvaluate) > 0) ifTrue: [ 
gXres _ 'VMS' deepCopy ] ifFalse: [ ((GXlocate run line: 0 proc: self method: true; p1:(gXdetails); p2:'IRIX'; condEvaluate) > 0) ifTrue: [ 
gXres _ 'IRIX' deepCopy ] ifFalse: [ ((GXlocate run line: 0 proc: self method: true; p1:(gXdetails); p2:'Linux'; condEvaluate) > 0) ifTrue: [ 
gXres _ 'LINUX' deepCopy ]]]]].
(GXcalledUasUfunction run line: 0 proc: self method: true; condEvaluate) not ifTrue: [ 
GXprintn run line: 0 proc: self method: false; p1:(gXres); condEvaluate ].
self up.
^gXres
!!
nil! "(fix for smalltalk bug, this line is ignored if there is a variable declaration error)"

"End of procedure"

GXcomment run line: 0 proc: self method: false; p1:'os'; p2:'Returns the name of the operating system'; condEvaluate!
"Procedure class definition:
 ==========================="
((Smalltalk at: #GXsystem ifAbsent: [ nil ]) isKindOf: Function class) ifTrue: [ ('Warning: Overriding previously defined procedure ', 'GXsystem' gtype) prinl ].
Smalltalk at: #GXsystem put: nil!
Function subclass: 'GXsystem'
       instanceVariableNames: ''
       classVariableNames: ''
       poolDictionaries: ''
       category: 'Genie Procedure'
!

"HEADER
 ------"

"parameters
 ----------"

GXsystem

addInstVarName: #gXcommand;
compile:
'gXcommand: aVar
	aVar notNil ifTrue:[gXcommand _ self typeCheck: aVar shouldBe: GXString paramName: ''gXcommand'']';
compile:
'p1: aVar
	aVar notNil ifTrue:[gXcommand _ self typeCheck: aVar shouldBe: GXString paramName: ''gXcommand'']'!

"result
 ------"

GXsystem

addInstVarName: #gXres
!

"Local variables
 ---------------"

GXsystem

addInstVarName: #gXcommandUprefix;

addInstVarName: #gXshellUprefix;

addInstVarName: #gXcommandUsuffix
!

!GXsystem methodsFor: 'initializing'!

initVars
	gXcommandUprefix _ ''.
	gXshellUprefix _ ''.
	gXcommandUsuffix _ ''
!!

"BODY
 ----"

!GXsystem methodsFor: 'running'!

evaluate
((GXos run line: 0 proc: self method: true; condEvaluate) ~= 'VMS') ifTrue: [ 
gXcommandUprefix _ '$SHELL -c ' deepCopy.
gXshellUprefix _ '$SHELL -i' deepCopy.
(GXdefined run line: 0 proc: self method: true; p1:(gXcommand); condEvaluate) ifTrue: [ 
gXcommand _ (('''' + (gXcommand)) + '''') deepCopy ] ].
(GXdefined run line: 0 proc: self method: true; p1:(gXcommand); condEvaluate) ifTrue: [ 
gXres _ (GXUsystem run line: 0 proc: self method: true; p1:(((gXcommandUprefix) + (gXcommand)) + (gXcommandUsuffix)); condEvaluate) deepCopy ] ifFalse: [ 
gXres _ (GXUsystem run line: 0 proc: self method: true; p1:((gXshellUprefix) + (gXcommandUsuffix)); condEvaluate) deepCopy ] .
self up.
^gXres
!!
nil! "(fix for smalltalk bug, this line is ignored if there is a variable declaration error)"

"End of procedure"

GXcomment run line: 0 proc: self method: false; p1:'system'; p2:'Executes a system command'; condEvaluate!
"Procedure class definition:
 ==========================="
((Smalltalk at: #GXalias ifAbsent: [ nil ]) isKindOf: Function class) ifTrue: [ ('Warning: Overriding previously defined procedure ', 'GXalias' gtype) prinl ].
Smalltalk at: #GXalias put: nil!
Function subclass: 'GXalias'
       instanceVariableNames: ''
       classVariableNames: ''
       poolDictionaries: ''
       category: 'Genie Procedure'
!

"HEADER
 ------"

"parameters
 ----------"

GXalias

addInstVarName: #gXaliasUname;
compile:
'gXaliasUname: aVar
	aVar notNil ifTrue:[gXaliasUname _ self typeCheck: aVar shouldBe: GXString paramName: ''gXaliasUname'']';
compile:
'p1: aVar
	aVar notNil ifTrue:[gXaliasUname _ self typeCheck: aVar shouldBe: GXString paramName: ''gXaliasUname'']';

addInstVarName: #gXfullUname;
compile:
'gXfullUname: aVar
	aVar notNil ifTrue:[gXfullUname _ self typeCheck: aVar shouldBe: GXString paramName: ''gXfullUname'']';
compile:
'p2: aVar
	aVar notNil ifTrue:[gXfullUname _ self typeCheck: aVar shouldBe: GXString paramName: ''gXfullUname'']'
!

"Local variables
 ---------------"

GXalias

addInstVarName: #gXproc;

addInstVarName: #gXqual;

addInstVarName: #gXpoint;

addInstVarName: #gXnextUpoint;

addInstVarName: #gXdoneflag!

"Local variables
 ---------------"

GXalias

addInstVarName: #gXprocUalias;

addInstVarName: #gXqualUalias;

addInstVarName: #gXpointUalias;

addInstVarName: #gXnextUpointUalias!

"Local variables
 ---------------"

GXalias

addInstVarName: #gXclass;

addInstVarName: #gXindex;

addInstVarName: #gXindexUalias
!

!GXalias methodsFor: 'initializing'!

initVars
	gXdoneflag _ ((1 = 0))
!!

"BODY
 ----"

!GXalias methodsFor: 'running'!

evaluate
gXindex _ (GXlocate run line: 0 proc: self method: true; p1:(gXfullUname); p2:'/'; condEvaluate) deepCopy.
gXindexUalias _ (GXlocate run line: 0 proc: self method: true; p1:(gXaliasUname); p2:'/'; condEvaluate) deepCopy.
((gXindex) = 0) ifTrue: [ 
gXproc _ ('GX' + (GXsubstring run line: 0 proc: self method: true; p1:(gXfullUname); p2:1; condEvaluate)) deepCopy.
gXprocUalias _ ('GX' + (GXsubstring run line: 0 proc: self method: true; p1:(gXaliasUname); p2:1; condEvaluate)) deepCopy ] ifFalse: [ 
gXproc _ ('GX' + (GXsubstring run line: 0 proc: self method: true; p1:(gXfullUname); p2:1; p3:((gXindex) - 1); condEvaluate)) deepCopy.
gXprocUalias _ ('GX' + (GXsubstring run line: 0 proc: self method: true; p1:(gXaliasUname); p2:1; p3:((gXindexUalias) - 1); condEvaluate)) deepCopy ] .
	( Smalltalk at: gXproc asSymbol ) subclass: gXprocUalias
	instanceVariableNames: '' classVariableNames: '' poolDictionaries: '' category: 'GENIE aliases'.
	gXclass _ ( Smalltalk at: gXprocUalias asSymbol ) .
Function new loop: 1 to: 50 by: 1 before: [ :gXi | 
gXpoint _ (GXlocate run line: 0 proc: self method: true; p1:(gXfullUname); p2:'/'; p3:(gXi); condEvaluate) deepCopy.
gXpointUalias _ (GXlocate run line: 0 proc: self method: true; p1:(gXaliasUname); p2:'/'; p3:(gXi); condEvaluate) deepCopy.
((gXpoint) ~= 0) ifTrue: [ 
gXpoint _ ((gXpoint) + 1) deepCopy.
gXpointUalias _ ((gXpointUalias) + 1) deepCopy.
gXnextUpoint _ (GXlocate run line: 0 proc: self method: true; p1:(gXfullUname); p2:'/'; p3:((gXi) + 1); condEvaluate) deepCopy.
gXnextUpointUalias _ (GXlocate run line: 0 proc: self method: true; p1:(gXaliasUname); p2:'/'; p3:((gXi) + 1); condEvaluate) deepCopy.
((gXnextUpoint) ~= 0) ifTrue: [ 
gXqual _ ('gX' + (GXsubstring run line: 0 proc: self method: true; p1:(gXfullUname); p2:(gXpoint); p3:((gXnextUpoint) - (gXpoint)); condEvaluate)) deepCopy.
gXqualUalias _ ('gX' + (GXsubstring run line: 0 proc: self method: true; p1:(gXaliasUname); p2:(gXpointUalias); p3:((gXnextUpointUalias) - (gXpointUalias)); condEvaluate)) deepCopy ] ifFalse: [ 
gXqual _ ('gX' + (GXsubstring run line: 0 proc: self method: true; p1:(gXfullUname); p2:(gXpoint); condEvaluate)) deepCopy.
gXqualUalias _ ('gX' + (GXsubstring run line: 0 proc: self method: true; p1:(gXaliasUname); p2:(gXpointUalias); condEvaluate)) deepCopy.
gXdoneflag _ ((1 = 1)) deepCopy ]  ] ifFalse: [ 
gXdoneflag _ ((1 = 1)) deepCopy ] .
(GXdefined run line: 0 proc: self method: true; p1:(gXqual); condEvaluate) ifTrue: [ 
 gXclass compile: gXqualUalias, '
			self ', gXqual ] ] 
condition: [ :gXi | (gXdoneflag) ] after: [ :gXi | nil ].
GXprintin run line: 0 proc: self method: false; p1:(gXaliasUname); p2:' is now an abbreviation for '; p3:(gXfullUname); condEvaluate.
self up.
^nil
!!
nil! "(fix for smalltalk bug, this line is ignored if there is a variable declaration error)"

"End of procedure"

"Procedure class definition:
 ==========================="
((Smalltalk at: #GXUUcheckvar ifAbsent: [ nil ]) isKindOf: Function class) ifTrue: [ ('Warning: Overriding previously defined procedure ', 'GXUUcheckvar' gtype) prinl ].
Smalltalk at: #GXUUcheckvar put: nil!
Function subclass: 'GXUUcheckvar'
       instanceVariableNames: ''
       classVariableNames: ''
       poolDictionaries: ''
       category: 'Genie Procedure'
!

"HEADER
 ------"

"parameters
 ----------"

GXUUcheckvar

addInstVarName: #gXnmUpD1;
compile:
'gXnmUpD1: aVar
	aVar notNil ifTrue:[ gXnmUpD1 _ aVar ]';
compile:
'p1: aVar
	aVar notNil ifTrue:[ gXnmUpD1 _ aVar ]';

addInstVarName: #gXnmUpD2;
compile:
'gXnmUpD2: aVar
	aVar notNil ifTrue:[gXnmUpD2 _ self typeCheck: aVar shouldBe: GXString paramName: ''gXnmUpD2'']';
compile:
'p2: aVar
	aVar notNil ifTrue:[gXnmUpD2 _ self typeCheck: aVar shouldBe: GXString paramName: ''gXnmUpD2'']';

addInstVarName: #gXnmUpD3;
compile:
'gXnmUpD3: aVar
	aVar notNil ifTrue:[gXnmUpD3 _ self typeCheck: aVar shouldBe: GXInteger paramName: ''gXnmUpD3'']';
compile:
'p3: aVar
	aVar notNil ifTrue:[gXnmUpD3 _ self typeCheck: aVar shouldBe: GXInteger paramName: ''gXnmUpD3'']';

addInstVarName: #gXnmUpD4;
compile:
'gXnmUpD4: aVar
	aVar notNil ifTrue:[gXnmUpD4 _ self typeCheck: aVar shouldBe: GXInteger paramName: ''gXnmUpD4'']';
compile:
'p4: aVar
	aVar notNil ifTrue:[gXnmUpD4 _ self typeCheck: aVar shouldBe: GXInteger paramName: ''gXnmUpD4'']'!

"result
 ------"

GXUUcheckvar

addInstVarName: #gXretval
!

"Local variables
 ---------------"

GXUUcheckvar

addInstVarName: #gXnmUpD1!

"Local variables
 ---------------"

GXUUcheckvar

addInstVarName: #gXnmUpD2!

"Local variables
 ---------------"

GXUUcheckvar

addInstVarName: #gXnmUpD3!

"Local variables
 ---------------"

GXUUcheckvar

addInstVarName: #gXnmUpD4!

"Local variables
 ---------------"

GXUUcheckvar

addInstVarName: #gXtrue!

"Local variables
 ---------------"

GXUUcheckvar

addInstVarName: #gXfalse
!

!GXUUcheckvar methodsFor: 'initializing'!

initVars
	gXnmUpD2 _ 'UNDEFINED'.
	gXnmUpD3 _ 0.
	gXnmUpD4 _ 0.
	gXtrue _ (1 = 1).
	gXfalse _ (1 = 2)
!!

"BODY
 ----"

!GXUUcheckvar methodsFor: 'running'!

evaluate
(GXdefined run line: 0 proc: self method: true; p1:(gXnmUpD1); condEvaluate) not ifTrue: [ 
((gXnmUpD4) = 1) ifTrue: [ 
GXprinten run line: 0 proc: self method: false; p1:'  ERROR - parameter '; p2:(GXasUstring run line: 0 proc: self method: true; p1:(gXnmUpD3); condEvaluate); p3:' not specified.'; condEvaluate.
gXretval _ (gXDfalse) deepCopy.
[nil.
self up.
^gXretval] value ].
gXretval _ (gXDtrue) deepCopy.
[nil.
self up.
^gXretval] value ].
(GXisUa run line: 0 proc: self method: true; p1:(gXnmUpD1); p2:(gXnmUpD2); condEvaluate) ifTrue: [ 
gXretval _ (gXDtrue) deepCopy.
[nil.
self up.
^gXretval] value ].
GXprinte run line: 0 proc: self method: false; p1:'  ERROR - parameter '; p2:(GXasUstring run line: 0 proc: self method: true; p1:(gXnmUpD3); condEvaluate); p3:' is of type '; condEvaluate.
(GXisUa run line: 0 proc: self method: true; p1:(gXnmUpD1); p2:'INTEGER'; condEvaluate) ifTrue: [ 
GXprinten run line: 0 proc: self method: false; p1:'INTEGER'; condEvaluate ].
(GXisUa run line: 0 proc: self method: true; p1:(gXnmUpD1); p2:'REAL'; condEvaluate) ifTrue: [ 
GXprinten run line: 0 proc: self method: false; p1:'REAL'; condEvaluate ].
(GXisUa run line: 0 proc: self method: true; p1:(gXnmUpD1); p2:'STRING'; condEvaluate) ifTrue: [ 
GXprinten run line: 0 proc: self method: false; p1:'STRING'; condEvaluate ].
(GXisUa run line: 0 proc: self method: true; p1:(gXnmUpD1); p2:'WORKSPACE'; condEvaluate) ifTrue: [ 
GXprinten run line: 0 proc: self method: false; p1:'WORKSPACE'; condEvaluate ].
(GXisUa run line: 0 proc: self method: true; p1:(gXnmUpD1); p2:'INTEGERARRAY'; condEvaluate) ifTrue: [ 
GXprinten run line: 0 proc: self method: false; p1:'INTEGER ARRAY'; condEvaluate ].
(GXisUa run line: 0 proc: self method: true; p1:(gXnmUpD1); p2:'REALARRAY'; condEvaluate) ifTrue: [ 
GXprinten run line: 0 proc: self method: false; p1:'REAL ARRAY'; condEvaluate ].
(GXisUa run line: 0 proc: self method: true; p1:(gXnmUpD1); p2:'STRINGARRAY'; condEvaluate) ifTrue: [ 
GXprinten run line: 0 proc: self method: false; p1:'STRING ARRAY'; condEvaluate ].
(GXisUa run line: 0 proc: self method: true; p1:(gXnmUpD1); p2:'WORKSPACEARRAY'; condEvaluate) ifTrue: [ 
GXprinten run line: 0 proc: self method: false; p1:'WORKSPACE ARRAY'; condEvaluate ].
GXprintin run line: 0 proc: self method: false; p1:'          It should be of type '; p2:(gXnmUpD2); condEvaluate.
gXretval _ (gXfalse) deepCopy.
self up.
^gXretval
!!
nil! "(fix for smalltalk bug, this line is ignored if there is a variable declaration error)"

"End of procedure"

GXcomment run line: 0 proc: self method: false; p1:'__checkvar'; p2:'checks parameter types'; condEvaluate!
"Procedure class definition:
 ==========================="
((Smalltalk at: #GXinteractive ifAbsent: [ nil ]) isKindOf: Function class) ifTrue: [ ('Warning: Overriding previously defined procedure ', 'GXinteractive' gtype) prinl ].
Smalltalk at: #GXinteractive put: nil!
Function subclass: 'GXinteractive'
       instanceVariableNames: ''
       classVariableNames: ''
       poolDictionaries: ''
       category: 'Genie Procedure'
!

"HEADER
 ------"

"result
 ------"

GXinteractive

addInstVarName: #gXres
!

!GXinteractive methodsFor: 'initializing'!

initVars

!!

"BODY
 ----"

!GXinteractive methodsFor: 'running'!

evaluate
 gXres _ ( line at: 1 ) isNil .
self up.
^gXres
!!
nil! "(fix for smalltalk bug, this line is ignored if there is a variable declaration error)"

"End of procedure"

GXcomment run line: 0 proc: self method: false; p1:'interactive'; p2:'Returns true if  procedure is being run interactively'; condEvaluate!
"Procedure class definition:
 ==========================="
((Smalltalk at: #GXnow ifAbsent: [ nil ]) isKindOf: Function class) ifTrue: [ ('Warning: Overriding previously defined procedure ', 'GXnow' gtype) prinl ].
Smalltalk at: #GXnow put: nil!
Function subclass: 'GXnow'
       instanceVariableNames: ''
       classVariableNames: ''
       poolDictionaries: ''
       category: 'Genie Procedure'
!

"HEADER
 ------"

"result
 ------"

GXnow

addInstVarName: #gXres
!

!GXnow methodsFor: 'initializing'!

initVars

!!

"BODY
 ----"

!GXnow methodsFor: 'running'!

evaluate
 gXres _ ( Date today printString, ' ', Time now printString ) .
self up.
^gXres
!!
nil! "(fix for smalltalk bug, this line is ignored if there is a variable declaration error)"

"End of procedure"

GXcomment run line: 0 proc: self method: false; p1:'now'; p2:'Returns current date and time in a string'; condEvaluate!
"Procedure class definition:
 ==========================="
((Smalltalk at: #GXUpoops ifAbsent: [ nil ]) isKindOf: Function class) ifTrue: [ ('Warning: Overriding previously defined procedure ', 'GXUpoops' gtype) prinl ].
Smalltalk at: #GXUpoops put: nil!
Function subclass: 'GXUpoops'
       instanceVariableNames: ''
       classVariableNames: ''
       poolDictionaries: ''
       category: 'Genie Procedure'
!

"BODY
 ----"

!GXUpoops methodsFor: 'running'!

evaluate
GXcallout run line: 0 proc: self method: false; gXfacility:'extDebugging'; gXfunction:0; condEvaluate.
self up.
^nil
!!
nil! "(fix for smalltalk bug, this line is ignored if there is a variable declaration error)"

"End of procedure"

GXcomment run line: 0 proc: self method: false; p1:'_poops'; p2:'Prints number of free OOPs remaining'; condEvaluate!
"Procedure class definition:
 ==========================="
((Smalltalk at: #GXfill ifAbsent: [ nil ]) isKindOf: Function class) ifTrue: [ ('Warning: Overriding previously defined procedure ', 'GXfill' gtype) prinl ].
Smalltalk at: #GXfill put: nil!
Function subclass: 'GXfill'
       instanceVariableNames: ''
       classVariableNames: ''
       poolDictionaries: ''
       category: 'Genie Procedure'
!

"HEADER
 ------"

"qualifiers
 ----------"

GXfill

addInstVarName: #gXgeometric;
compile:
'gXgeometric
	gXgeometric _ true';
compile:
'gXgeometric: aBoolean
	gXgeometric _ aBoolean'!

"parameters
 ----------"

GXfill

addInstVarName: #gXarray;
compile:
'gXarray: aVar
	aVar notNil ifTrue:[gXarray _ self typeCheck: aVar shouldBe: GXArray paramName: ''gXarray'']';
compile:
'p1: aVar
	aVar notNil ifTrue:[gXarray _ self typeCheck: aVar shouldBe: GXArray paramName: ''gXarray'']';

addInstVarName: #gXvalue;
compile:
'gXvalue: aVar
	aVar notNil ifTrue:[ gXvalue _ aVar ]';
compile:
'p2: aVar
	aVar notNil ifTrue:[ gXvalue _ aVar ]';

addInstVarName: #gXstep;
compile:
'gXstep: aVar
	aVar notNil ifTrue:[ gXstep _ aVar ]';
compile:
'p3: aVar
	aVar notNil ifTrue:[ gXstep _ aVar ]'!

"result
 ------"

GXfill

addInstVarName: #gXres
!

"Local variables
 ---------------"

GXfill

addInstVarName: #gXtypeUok;

addInstVarName: #gXDtmp;

addInstVarName: #gXmode
!

!GXfill methodsFor: 'initializing'!

initVars
	gXgeometric _ false
!!

"BODY
 ----"

!GXfill methodsFor: 'running'!

evaluate
 gXgeometric ifTrue: [ gXmode _ 1 ] ifFalse: [ gXmode _ 0 ] .
(GXdefined run line: 0 proc: self method: true; p1:(gXvalue); condEvaluate) not ifTrue: [ 
GXprinten run line: 0 proc: self method: false; p1:'usage:  Fill <Array> <value>  or  x=fill(<Array>,<Value>)'; condEvaluate.
[nil.
self up.
^gXres] value ].
 gXDtmp _ gXarray relatedArrayClass: gXvalue.
   gXtypeUok _ gXDtmp notNil
	ifTrue: [(gXDtmp new:1) isKindOf: gXarray class]
	ifFalse: [false] .
(gXtypeUok) ifTrue: [ 
(GXcalledUasUfunction run line: 0 proc: self method: true; condEvaluate) ifTrue: [ 
gXres _ (gXarray) deepCopy.
 gXres at: 1 put: gXvalue .
 gXres _ gXres extend: gXvalue with: gXstep mode: gXmode  ] ifFalse: [ 
 gXarray at: 1 put: gXvalue .
 gXarray become: (gXarray extend: gXvalue with: gXstep mode: gXmode)  ]  ] ifFalse: [ 
GXprinten run line: 0 proc: self method: false; p1:'FILL: An array can only be filled with elements of the same type'; condEvaluate ] .
self up.
^gXres
!!
nil! "(fix for smalltalk bug, this line is ignored if there is a variable declaration error)"

"End of procedure"

GXcomment run line: 0 proc: self method: false; p1:'fill'; p2:'fills an array with a value or a series of values'; condEvaluate!
"Procedure class definition:
 ==========================="
((Smalltalk at: #GXfix ifAbsent: [ nil ]) isKindOf: Function class) ifTrue: [ ('Warning: Overriding previously defined procedure ', 'GXfix' gtype) prinl ].
Smalltalk at: #GXfix put: nil!
Function subclass: 'GXfix'
       instanceVariableNames: ''
       classVariableNames: ''
       poolDictionaries: ''
       category: 'Genie Procedure'
!

"HEADER
 ------"

"parameters
 ----------"

GXfix

addInstVarName: #gXarray;
compile:
'gXarray: aVar
	aVar notNil ifTrue:[gXarray _ self typeCheck: aVar shouldBe: GXArray paramName: ''gXarray'']';
compile:
'p1: aVar
	aVar notNil ifTrue:[gXarray _ self typeCheck: aVar shouldBe: GXArray paramName: ''gXarray'']';

addInstVarName: #gXvalue;
compile:
'gXvalue: aVar
	aVar notNil ifTrue:[ gXvalue _ aVar ]';
compile:
'p2: aVar
	aVar notNil ifTrue:[ gXvalue _ aVar ]'!

"result
 ------"

GXfix

addInstVarName: #gXres
!

"Local variables
 ---------------"

GXfix

addInstVarName: #gXtypeUok;

addInstVarName: #gXtmp
!

!GXfix methodsFor: 'initializing'!

initVars

!!

"BODY
 ----"

!GXfix methodsFor: 'running'!

evaluate
(GXdefined run line: 0 proc: self method: true; p1:(gXvalue); condEvaluate) not ifTrue: [ 
GXprinten run line: 0 proc: self method: false; p1:'usage:  Fix <Array> <value>  or  x=fix(<Array>,<Value>)'; condEvaluate.
[nil.
self up.
^gXres] value ].
 gXtmp _ gXarray relatedArrayClass: gXvalue.
   gXtypeUok _ gXtmp notNil
	ifTrue: [(gXtmp new:1) isMemberOf: gXarray class]
	ifFalse: [false] .
(gXtypeUok) ifTrue: [ 
(GXcalledUasUfunction run line: 0 proc: self method: true; condEvaluate) ifTrue: [ 
gXres _ (gXarray) deepCopy.
 gXres _ gXres fix: gXvalue  ] ifFalse: [ 
 gXarray become: (gXarray fix: gXvalue)  ]  ] ifFalse: [ 
GXprinten run line: 0 proc: self method: false; p1:'FIX: The replacement value must be of the same type as the array'; condEvaluate.
GXprintin run line: 0 proc: self method: false; p1:'	(The array must also contain some defined values)'; condEvaluate ] .
self up.
^gXres
!!
nil! "(fix for smalltalk bug, this line is ignored if there is a variable declaration error)"

"End of procedure"

GXcomment run line: 0 proc: self method: false; p1:'fix'; p2:'fixes undefined values in an array to a constant'; condEvaluate!
"Procedure class definition:
 ==========================="
((Smalltalk at: #GXunfix ifAbsent: [ nil ]) isKindOf: Function class) ifTrue: [ ('Warning: Overriding previously defined procedure ', 'GXunfix' gtype) prinl ].
Smalltalk at: #GXunfix put: nil!
Function subclass: 'GXunfix'
       instanceVariableNames: ''
       classVariableNames: ''
       poolDictionaries: ''
       category: 'Genie Procedure'
!

"HEADER
 ------"

"parameters
 ----------"

GXunfix

addInstVarName: #gXarray;
compile:
'gXarray: aVar
	aVar notNil ifTrue:[gXarray _ self typeCheck: aVar shouldBe: GXArray paramName: ''gXarray'']';
compile:
'p1: aVar
	aVar notNil ifTrue:[gXarray _ self typeCheck: aVar shouldBe: GXArray paramName: ''gXarray'']';

addInstVarName: #gXvalue;
compile:
'gXvalue: aVar
	aVar notNil ifTrue:[ gXvalue _ aVar ]';
compile:
'p2: aVar
	aVar notNil ifTrue:[ gXvalue _ aVar ]'!

"result
 ------"

GXunfix

addInstVarName: #gXres
!

"Local variables
 ---------------"

GXunfix

addInstVarName: #gXtypeUok;

addInstVarName: #gXtmp
!

!GXunfix methodsFor: 'initializing'!

initVars

!!

"BODY
 ----"

!GXunfix methodsFor: 'running'!

evaluate
(GXdefined run line: 0 proc: self method: true; p1:(gXvalue); condEvaluate) not ifTrue: [ 
GXprinten run line: 0 proc: self method: false; p1:'usage:  Unfix <Array> <value>  or  x=Unfix(<Array>,<Value>)'; condEvaluate.
[nil.
self up.
^gXres] value ].
 gXtmp _ gXarray relatedArrayClass: gXvalue.
   gXtypeUok _ gXtmp notNil
	ifTrue: [(gXtmp new:1) isMemberOf: gXarray class]
	ifFalse: [false] .
(gXtypeUok) ifTrue: [ 
(GXcalledUasUfunction run line: 0 proc: self method: true; condEvaluate) ifTrue: [ 
gXres _ (gXarray) deepCopy.
 gXres _ gXres unfix: gXvalue  ] ifFalse: [ 
 gXarray become: (gXarray unfix: gXvalue)  ]  ] ifFalse: [ 
GXprinten run line: 0 proc: self method: false; p1:'UNFIX: The replacement value must be of the same type as the array'; condEvaluate.
GXprintin run line: 0 proc: self method: false; p1:'	(The array must also contain some defined values)'; condEvaluate ] .
self up.
^gXres
!!
nil! "(fix for smalltalk bug, this line is ignored if there is a variable declaration error)"

"End of procedure"

GXcomment run line: 0 proc: self method: false; p1:'unfix'; p2:'fixes defined values in an array to undefined'; condEvaluate!
"Procedure class definition:
 ==========================="
((Smalltalk at: #GXmax ifAbsent: [ nil ]) isKindOf: Function class) ifTrue: [ ('Warning: Overriding previously defined procedure ', 'GXmax' gtype) prinl ].
Smalltalk at: #GXmax put: nil!
Function subclass: 'GXmax'
       instanceVariableNames: ''
       classVariableNames: ''
       poolDictionaries: ''
       category: 'Genie Procedure'
!

"HEADER
 ------"

"parameters
 ----------"

GXmax

addInstVarName: #gXarray;
compile:
'gXarray: aVar
	aVar notNil ifTrue:[gXarray _ self typeCheck: aVar shouldBe: GXArray paramName: ''gXarray'']';
compile:
'p1: aVar
	aVar notNil ifTrue:[gXarray _ self typeCheck: aVar shouldBe: GXArray paramName: ''gXarray'']'!

"result
 ------"

GXmax

addInstVarName: #gXres
!

!GXmax methodsFor: 'initializing'!

initVars

!!

"BODY
 ----"

!GXmax methodsFor: 'running'!

evaluate
 gXres _ gXarray maximum .
self up.
^gXres
!!
nil! "(fix for smalltalk bug, this line is ignored if there is a variable declaration error)"

"End of procedure"

GXcomment run line: 0 proc: self method: false; p1:'max'; p2:'Returns the largest element value in an array'; condEvaluate!
"Procedure class definition:
 ==========================="
((Smalltalk at: #GXmin ifAbsent: [ nil ]) isKindOf: Function class) ifTrue: [ ('Warning: Overriding previously defined procedure ', 'GXmin' gtype) prinl ].
Smalltalk at: #GXmin put: nil!
Function subclass: 'GXmin'
       instanceVariableNames: ''
       classVariableNames: ''
       poolDictionaries: ''
       category: 'Genie Procedure'
!

"HEADER
 ------"

"parameters
 ----------"

GXmin

addInstVarName: #gXarray;
compile:
'gXarray: aVar
	aVar notNil ifTrue:[gXarray _ self typeCheck: aVar shouldBe: GXArray paramName: ''gXarray'']';
compile:
'p1: aVar
	aVar notNil ifTrue:[gXarray _ self typeCheck: aVar shouldBe: GXArray paramName: ''gXarray'']'!

"result
 ------"

GXmin

addInstVarName: #gXres
!

!GXmin methodsFor: 'initializing'!

initVars

!!

"BODY
 ----"

!GXmin methodsFor: 'running'!

evaluate
 gXres _ gXarray minimum .
self up.
^gXres
!!
nil! "(fix for smalltalk bug, this line is ignored if there is a variable declaration error)"

"End of procedure"

GXcomment run line: 0 proc: self method: false; p1:'min'; p2:'Returns the smallest element value in an array'; condEvaluate!
"Procedure class definition:
 ==========================="
((Smalltalk at: #GXsum ifAbsent: [ nil ]) isKindOf: Function class) ifTrue: [ ('Warning: Overriding previously defined procedure ', 'GXsum' gtype) prinl ].
Smalltalk at: #GXsum put: nil!
Function subclass: 'GXsum'
       instanceVariableNames: ''
       classVariableNames: ''
       poolDictionaries: ''
       category: 'Genie Procedure'
!

"HEADER
 ------"

"parameters
 ----------"

GXsum

addInstVarName: #gXarray;
compile:
'gXarray: aVar
	aVar notNil ifTrue:[gXarray _ self typeCheck: aVar shouldBe: GXArray paramName: ''gXarray'']';
compile:
'p1: aVar
	aVar notNil ifTrue:[gXarray _ self typeCheck: aVar shouldBe: GXArray paramName: ''gXarray'']'!

"result
 ------"

GXsum

addInstVarName: #gXres
!

!GXsum methodsFor: 'initializing'!

initVars

!!

"BODY
 ----"

!GXsum methodsFor: 'running'!

evaluate
 gXres _ gXarray sum .
self up.
^gXres
!!
nil! "(fix for smalltalk bug, this line is ignored if there is a variable declaration error)"

"End of procedure"

GXcomment run line: 0 proc: self method: false; p1:'sum'; p2:'Returns the sum of the elements in an array'; condEvaluate!
"Procedure class definition:
 ==========================="
((Smalltalk at: #GXcentreUbins ifAbsent: [ nil ]) isKindOf: Function class) ifTrue: [ ('Warning: Overriding previously defined procedure ', 'GXcentreUbins' gtype) prinl ].
Smalltalk at: #GXcentreUbins put: nil!
Function subclass: 'GXcentreUbins'
       instanceVariableNames: ''
       classVariableNames: ''
       poolDictionaries: ''
       category: 'Genie Procedure'
!

"HEADER
 ------"

"parameters
 ----------"

GXcentreUbins

addInstVarName: #gXarray;
compile:
'gXarray: aVar
	aVar notNil ifTrue:[gXarray _ self typeCheck: aVar shouldBe: GXRealarray paramName: ''gXarray'']';
compile:
'p1: aVar
	aVar notNil ifTrue:[gXarray _ self typeCheck: aVar shouldBe: GXRealarray paramName: ''gXarray'']'!

"result
 ------"

GXcentreUbins

addInstVarName: #gXres
!

!GXcentreUbins methodsFor: 'initializing'!

initVars

!!

"BODY
 ----"

!GXcentreUbins methodsFor: 'running'!

evaluate
 gXres _ gXarray centre .
(GXcalledUasUfunction run line: 0 proc: self method: true; condEvaluate) not ifTrue: [ 
 gXarray become: gXres  ].
self up.
^gXres
!!
nil! "(fix for smalltalk bug, this line is ignored if there is a variable declaration error)"

"End of procedure"

GXcomment run line: 0 proc: self method: false; p1:'centre_bins'; p2:'Centre the bins of an real array (i.e. convert from histogram to point mode)'; condEvaluate!
"Procedure class definition:
 ==========================="
((Smalltalk at: #GXcut ifAbsent: [ nil ]) isKindOf: Function class) ifTrue: [ ('Warning: Overriding previously defined procedure ', 'GXcut' gtype) prinl ].
Smalltalk at: #GXcut put: nil!
Function subclass: 'GXcut'
       instanceVariableNames: ''
       classVariableNames: ''
       poolDictionaries: ''
       category: 'Genie Procedure'
!

"HEADER
 ------"

"parameters
 ----------"

GXcut

addInstVarName: #gXarray;
compile:
'gXarray: aVar
	aVar notNil ifTrue:[gXarray _ self typeCheck: aVar shouldBe: GXRealarray paramName: ''gXarray'']';
compile:
'p1: aVar
	aVar notNil ifTrue:[gXarray _ self typeCheck: aVar shouldBe: GXRealarray paramName: ''gXarray'']';

addInstVarName: #gXdimension;
compile:
'gXdimension: aVar
	aVar notNil ifTrue:[gXdimension _ self typeCheck: aVar shouldBe: GXInteger paramName: ''gXdimension'']';
compile:
'p2: aVar
	aVar notNil ifTrue:[gXdimension _ self typeCheck: aVar shouldBe: GXInteger paramName: ''gXdimension'']';

addInstVarName: #gXindex;
compile:
'gXindex: aVar
	aVar notNil ifTrue:[gXindex _ self typeCheck: aVar shouldBe: GXReal paramName: ''gXindex'']';
compile:
'p3: aVar
	aVar notNil ifTrue:[gXindex _ self typeCheck: aVar shouldBe: GXReal paramName: ''gXindex'']'!

"result
 ------"

GXcut

addInstVarName: #gXres
!

"Local variables
 ---------------"

GXcut

addInstVarName: #gXdimension;

addInstVarName: #gXindex
!

!GXcut methodsFor: 'initializing'!

initVars
	gXdimension _ 0.
	gXindex _ 0.000000000000000e000
!!

"BODY
 ----"

!GXcut methodsFor: 'running'!

evaluate
 gXres _ gXarray cut: gXdimension with: gXindex .
(GXcalledUasUfunction run line: 0 proc: self method: true; condEvaluate) not ifTrue: [ 
 gXarray become: gXres  ].
self up.
^gXres
!!
nil! "(fix for smalltalk bug, this line is ignored if there is a variable declaration error)"

"End of procedure"

"Procedure class definition:
 ==========================="
((Smalltalk at: #GXredim ifAbsent: [ nil ]) isKindOf: Function class) ifTrue: [ ('Warning: Overriding previously defined procedure ', 'GXredim' gtype) prinl ].
Smalltalk at: #GXredim put: nil!
Function subclass: 'GXredim'
       instanceVariableNames: ''
       classVariableNames: ''
       poolDictionaries: ''
       category: 'Genie Procedure'
!

"HEADER
 ------"

"parameters
 ----------"

GXredim

addInstVarName: #gXarray;
compile:
'gXarray: aVar
	aVar notNil ifTrue:[gXarray _ self typeCheck: aVar shouldBe: GXArray paramName: ''gXarray'']';
compile:
'p1: aVar
	aVar notNil ifTrue:[gXarray _ self typeCheck: aVar shouldBe: GXArray paramName: ''gXarray'']';

addInstVarName: #gXi;
compile:
'gXi: aVar
	aVar notNil ifTrue:[gXi _ self typeCheck: aVar shouldBe: GXInteger paramName: ''gXi'']';
compile:
'p2: aVar
	aVar notNil ifTrue:[gXi _ self typeCheck: aVar shouldBe: GXInteger paramName: ''gXi'']';

addInstVarName: #gXj;
compile:
'gXj: aVar
	aVar notNil ifTrue:[gXj _ self typeCheck: aVar shouldBe: GXInteger paramName: ''gXj'']';
compile:
'p3: aVar
	aVar notNil ifTrue:[gXj _ self typeCheck: aVar shouldBe: GXInteger paramName: ''gXj'']';

addInstVarName: #gXk;
compile:
'gXk: aVar
	aVar notNil ifTrue:[gXk _ self typeCheck: aVar shouldBe: GXInteger paramName: ''gXk'']';
compile:
'p4: aVar
	aVar notNil ifTrue:[gXk _ self typeCheck: aVar shouldBe: GXInteger paramName: ''gXk'']';

addInstVarName: #gXl;
compile:
'gXl: aVar
	aVar notNil ifTrue:[gXl _ self typeCheck: aVar shouldBe: GXInteger paramName: ''gXl'']';
compile:
'p5: aVar
	aVar notNil ifTrue:[gXl _ self typeCheck: aVar shouldBe: GXInteger paramName: ''gXl'']'!

"result
 ------"

GXredim

addInstVarName: #gXres
!

"Local variables
 ---------------"

GXredim

addInstVarName: #gXdims;

addInstVarName: #gXlength
!

!GXredim methodsFor: 'initializing'!

initVars

!!

"BODY
 ----"

!GXredim methodsFor: 'running'!

evaluate
(GXdefined run line: 0 proc: self method: true; p1:(gXl); condEvaluate) ifTrue: [ 
 gXdims _ Array new: 4.
       gXdims at: 4 put: gXl; at: 3 put: gXk; at: 2 put: gXj; at: 1 put: gXi .
gXlength _ ((((gXi) * (gXj)) * (gXk)) * (gXl)) deepCopy ].
((GXdefined run line: 0 proc: self method: true; p1:(gXk); condEvaluate) & (GXdefined run line: 0 proc: self method: true; p1:(gXl); condEvaluate) not) ifTrue: [ 
 gXdims _ Array new: 3.
       gXdims at: 3 put: gXk; at: 2 put: gXj; at: 1 put: gXi .
gXlength _ (((gXi) * (gXj)) * (gXk)) deepCopy ].
((GXdefined run line: 0 proc: self method: true; p1:(gXj); condEvaluate) & (GXdefined run line: 0 proc: self method: true; p1:(gXk); condEvaluate) not) ifTrue: [ 
 gXdims _ Array new: 2.
       gXdims at: 2 put: gXj; at: 1 put: gXi .
gXlength _ ((gXi) * (gXj)) deepCopy ].
((GXdefined run line: 0 proc: self method: true; p1:(gXi); condEvaluate) & (GXdefined run line: 0 proc: self method: true; p1:(gXj); condEvaluate) not) ifTrue: [ 
 gXdims _ Array new: 1.
       gXdims at: 1 put: gXi .
gXlength _ (gXi) deepCopy ].
(GXdefined run line: 0 proc: self method: true; p1:(gXi); condEvaluate) not ifTrue: [ 
GXprinten run line: 0 proc: self method: false; p1:'No dimensions specified!'; condEvaluate.
[nil.
self up.
^gXres] value ].
((gXlength) = (GXlength run line: 0 proc: self method: true; p1:(gXarray); condEvaluate)) ifTrue: [ 
(GXcalledUasUfunction run line: 0 proc: self method: true; condEvaluate) ifTrue: [ 
gXres _ (gXarray) deepCopy.
((gXres) setDims:(gXdims)) ] ifFalse: [ 
((gXarray) setDims:(gXdims)) ]  ] ifFalse: [ 
GXprinten run line: 0 proc: self method: false; p1:'Attempt to resize an array while re-dimensioning it'; condEvaluate.
GXprintin run line: 0 proc: self method: false; p1:'Check that the new dimensions mutiply out to the same size as the original'; condEvaluate ] .
self up.
^gXres
!!
nil! "(fix for smalltalk bug, this line is ignored if there is a variable declaration error)"

"End of procedure"

"Procedure class definition:
 ==========================="
((Smalltalk at: #GXinspect ifAbsent: [ nil ]) isKindOf: Function class) ifTrue: [ ('Warning: Overriding previously defined procedure ', 'GXinspect' gtype) prinl ].
Smalltalk at: #GXinspect put: nil!
Function subclass: 'GXinspect'
       instanceVariableNames: ''
       classVariableNames: ''
       poolDictionaries: ''
       category: 'Genie Procedure'
!

"HEADER
 ------"

"parameters
 ----------"

GXinspect

addInstVarName: #gXp1;
compile:
'gXp1: aVar
	aVar notNil ifTrue:[ gXp1 _ aVar ]';
compile:
'p1: aVar
	aVar notNil ifTrue:[ gXp1 _ aVar ]'
!

!GXinspect methodsFor: 'initializing'!

initVars

!!

"BODY
 ----"

!GXinspect methodsFor: 'running'!

evaluate
 
    gXp1 inspect
.
self up.
^nil
!!
nil! "(fix for smalltalk bug, this line is ignored if there is a variable declaration error)"

"End of procedure"

GXcomment run line: 0 proc: self method: false; p1:'inspect'; p2:'Send an inspect message to a Smalltalk object'; condEvaluate!
"Procedure class definition:
 ==========================="
((Smalltalk at: #GXfree ifAbsent: [ nil ]) isKindOf: Function class) ifTrue: [ ('Warning: Overriding previously defined procedure ', 'GXfree' gtype) prinl ].
Smalltalk at: #GXfree put: nil!
Function subclass: 'GXfree'
       instanceVariableNames: ''
       classVariableNames: ''
       poolDictionaries: ''
       category: 'Genie Procedure'
!

"HEADER
 ------"

"parameters
 ----------"

GXfree

addInstVarName: #gXvariable;
compile:
'gXvariable: aVar
	aVar notNil ifTrue:[ gXvariable _ aVar ]';
compile:
'p1: aVar
	aVar notNil ifTrue:[ gXvariable _ aVar ]';

addInstVarName: #gXsubvariable;
compile:
'gXsubvariable: aVar
	aVar notNil ifTrue:[gXsubvariable _ self typeCheck: aVar shouldBe: GXString paramName: ''gXsubvariable'']';
compile:
'p2: aVar
	aVar notNil ifTrue:[gXsubvariable _ self typeCheck: aVar shouldBe: GXString paramName: ''gXsubvariable'']'!

"qualifiers
 ----------"

GXfree

addInstVarName: #gXproc;
compile:
'gXproc
	gXproc _ true';
compile:
'gXproc: aBoolean
	gXproc _ aBoolean';

addInstVarName: #gXtype;
compile:
'gXtype
	gXtype _ true';
compile:
'gXtype: aBoolean
	gXtype _ aBoolean'
!

"Local variables
 ---------------"

GXfree

addInstVarName: #gXname;

addInstVarName: #gXfield
!

!GXfree methodsFor: 'initializing'!

initVars
	gXproc _ false.
	gXtype _ false
!!

"BODY
 ----"

!GXfree methodsFor: 'running'!

evaluate
((GXdefined run line: 0 proc: self method: true; p1:(gXsubvariable); condEvaluate) not & (GXisUa run line: 0 proc: self method: true; p1:(gXvariable); p2:'string'; condEvaluate) not) ifTrue: [ 
GXprinten run line: 0 proc: self method: false; p1:'Please give the variable or procedure name as a string'; condEvaluate.
[nil.
self up.
^nil] value ].
(GXdefined run line: 0 proc: self method: true; p1:(gXsubvariable); condEvaluate) not ifTrue: [ 
(gXproc) ifTrue: [ 
gXname _ (((gXvariable) mangleAs:'procedure') asSymbol) deepCopy ] ifFalse: [ (gXtype) ifTrue: [ 
gXname _ (((gXvariable) mangleAs:'type') asSymbol) deepCopy ] ifFalse: [ 
gXname _ (((gXvariable) mangleAs:'variable') asSymbol) deepCopy ] ].
 Smalltalk removeKey: gXname ifAbsent: [ self error: 'Item does not exist' ]  ] ifFalse: [ 
gXname _ (((gXvariable) mangleAs:'variable') asSymbol) deepCopy.
gXfield _ (((gXsubvariable) mangleAs:'variable') asSymbol) deepCopy.
 (Smalltalk at: gXname) removeKey: gXfield ifAbsent: [ self error: 'field does not exist'. ^nil ]  ] .
self up.
^nil
!!
nil! "(fix for smalltalk bug, this line is ignored if there is a variable declaration error)"

"End of procedure"

GXcomment run line: 0 proc: self method: false; p1:'free'; p2:'Procedure which deletes a genie variable or procedure'; condEvaluate!
"Procedure class definition:
 ==========================="
((Smalltalk at: #GXUUgctorturehelp ifAbsent: [ nil ]) isKindOf: Function class) ifTrue: [ ('Warning: Overriding previously defined procedure ', 'GXUUgctorturehelp' gtype) prinl ].
Smalltalk at: #GXUUgctorturehelp put: nil!
Function subclass: 'GXUUgctorturehelp'
       instanceVariableNames: ''
       classVariableNames: ''
       poolDictionaries: ''
       category: 'Genie Procedure'
!

"BODY
 ----"

!GXUUgctorturehelp methodsFor: 'running'!

evaluate
GXprintn run line: 0 proc: self method: false; p1:'[34m  GCTORTURE FACILITY=<Integer>
'; condEvaluate.
GXprintn run line: 0 proc: self method: false; p1:'/ON     Trigger garbage collection after EVERY object allocation (this will make things go VERY slow)'; condEvaluate.
GXprintn run line: 0 proc: self method: false; p1:'/OFF    Restore normal garbage collection'; condEvaluate.
GXprintn run line: 0 proc: self method: false; p1:'[0m'; condEvaluate.
self up.
^nil
!!
nil! "(fix for smalltalk bug, this line is ignored if there is a variable declaration error)"

"End of procedure"

GXcomment run line: 0 proc: self method: false; p1:'__gctorturehelp'; p2:'Help procedure for GCTORTURE command'; condEvaluate!
"Procedure class definition:
 ==========================="
((Smalltalk at: #GXgctorture ifAbsent: [ nil ]) isKindOf: Function class) ifTrue: [ ('Warning: Overriding previously defined procedure ', 'GXgctorture' gtype) prinl ].
Smalltalk at: #GXgctorture put: nil!
Function subclass: 'GXgctorture'
       instanceVariableNames: ''
       classVariableNames: ''
       poolDictionaries: ''
       category: 'Genie Procedure'
!

"HEADER
 ------"

"qualifiers
 ----------"

GXgctorture

addInstVarName: #gXon;
compile:
'gXon
	gXon _ true';
compile:
'gXon: aBoolean
	gXon _ aBoolean';

addInstVarName: #gXoff;
compile:
'gXoff
	gXoff _ true';
compile:
'gXoff: aBoolean
	gXoff _ aBoolean'!

"parameters
 ----------"

GXgctorture

addInstVarName: #gXfacility;
compile:
'gXfacility: aVar
	aVar notNil ifTrue:[gXfacility _ self typeCheck: aVar shouldBe: GXInteger paramName: ''gXfacility'']';
compile:
'p1: aVar
	aVar notNil ifTrue:[gXfacility _ self typeCheck: aVar shouldBe: GXInteger paramName: ''gXfacility'']'
!

"Local variables
 ---------------"

GXgctorture

addInstVarName: #gXflag;

addInstVarName: #gXfacility
!

!GXgctorture methodsFor: 'initializing'!

initVars
	gXon _ false.
	gXoff _ false.
	gXfacility _ 1 negated
!!

"BODY
 ----"

!GXgctorture methodsFor: 'running'!

evaluate
(gXon) ifTrue: [ 
gXflag _ 1 deepCopy ].
(gXoff) ifTrue: [ 
gXflag _ 0 deepCopy ].
GXcallout run line: 0 proc: self method: false; p1:'lowSystem'; p2:3; condEvaluate.
self up.
^nil
!!
nil! "(fix for smalltalk bug, this line is ignored if there is a variable declaration error)"

"End of procedure"

GXcomment run line: 0 proc: self method: false; p1:'gctorture'; p2:'Toggle garbage collection on every allocation (used for debugging)'; condEvaluate!
"Procedure class definition:
 ==========================="
((Smalltalk at: #GXversion ifAbsent: [ nil ]) isKindOf: Function class) ifTrue: [ ('Warning: Overriding previously defined procedure ', 'GXversion' gtype) prinl ].
Smalltalk at: #GXversion put: nil!
Function subclass: 'GXversion'
       instanceVariableNames: ''
       classVariableNames: ''
       poolDictionaries: ''
       category: 'Genie Procedure'
!

"HEADER
 ------"

"Local variables
 ---------------"

GXversion

addInstVarName: #gXversionUinfo
!

!GXversion methodsFor: 'initializing'!

initVars
	gXversionUinfo _ ''
!!

"BODY
 ----"

!GXversion methodsFor: 'running'!

evaluate
GXcallout run line: 0 proc: self method: false; p1:'lowSystem'; p2:4; condEvaluate.
GXprintn run line: 0 proc: self method: false; p1:(gXversionUinfo); condEvaluate.
self up.
^nil
!!
nil! "(fix for smalltalk bug, this line is ignored if there is a variable declaration error)"

"End of procedure"

GXcomment run line: 0 proc: self method: false; p1:'version'; p2:'Print information about the version of Genie being used'; condEvaluate!
GXalias run line: 0 proc: self method: false; p1:'licence'; p2:'copying'; condEvaluate!
GXalias run line: 0 proc: self method: false; p1:'license'; p2:'copying'; condEvaluate!

    Behavior defineCFunc: 'unlink'
	 withSelectorArgs: 'unlink: aString'
	 forClass: SystemDictionary
	 returning: #int
	 args: #(string)
!
    
    Behavior defineCFunc: 'getStrerror'
	 withSelectorArgs: 'getStrerror'
	 forClass: SystemDictionary
	 returning: #string
	 args: #()
!
"Procedure class definition:
 ==========================="
((Smalltalk at: #GXUUunlink ifAbsent: [ nil ]) isKindOf: Function class) ifTrue: [ ('Warning: Overriding previously defined procedure ', 'GXUUunlink' gtype) prinl ].
Smalltalk at: #GXUUunlink put: nil!
Function subclass: 'GXUUunlink'
       instanceVariableNames: ''
       classVariableNames: ''
       poolDictionaries: ''
       category: 'Genie Procedure'
!

"HEADER
 ------"

"parameters
 ----------"

GXUUunlink

addInstVarName: #gXfile;
compile:
'gXfile: aVar
	aVar notNil ifTrue:[gXfile _ self typeCheck: aVar shouldBe: GXString paramName: ''gXfile'']';
compile:
'p1: aVar
	aVar notNil ifTrue:[gXfile _ self typeCheck: aVar shouldBe: GXString paramName: ''gXfile'']'
!

!GXUUunlink methodsFor: 'initializing'!

initVars

!!

"BODY
 ----"

!GXUUunlink methodsFor: 'running'!

evaluate
 
    | tmp |
    gXfile notNil ifTrue: [ 
	tmp _ Smalltalk unlink: gXfile.
	( tmp = 0 ) ifFalse: [ 
	    '__UNLINK: Error removing file (' pren.
	    ( Smalltalk getStrerror ) pren.
	    ')' prenl.
        ]
     ] 
                  ifFalse: [
	'__UNLINK: No file specified' prenl
     ]
.
self up.
^nil
!!
nil! "(fix for smalltalk bug, this line is ignored if there is a variable declaration error)"

"End of procedure"

GXcomment run line: 0 proc: self method: false; p1:'__unlink'; p2:'Delete a file'; condEvaluate!
"Procedure class definition:
 ==========================="
((Smalltalk at: #GXUUgenielog ifAbsent: [ nil ]) isKindOf: Function class) ifTrue: [ ('Warning: Overriding previously defined procedure ', 'GXUUgenielog' gtype) prinl ].
Smalltalk at: #GXUUgenielog put: nil!
Function subclass: 'GXUUgenielog'
       instanceVariableNames: ''
       classVariableNames: ''
       poolDictionaries: ''
       category: 'Genie Procedure'
!

"HEADER
 ------"

"parameters
 ----------"

GXUUgenielog

addInstVarName: #gXmessage;
compile:
'gXmessage: aVar
	aVar notNil ifTrue:[gXmessage _ self typeCheck: aVar shouldBe: GXString paramName: ''gXmessage'']';
compile:
'p1: aVar
	aVar notNil ifTrue:[gXmessage _ self typeCheck: aVar shouldBe: GXString paramName: ''gXmessage'']'
!

"Local variables
 ---------------"

GXUUgenielog

addInstVarName: #gXmessage
!

!GXUUgenielog methodsFor: 'initializing'!

initVars
	gXmessage _ '<No Message>'
!!

"BODY
 ----"

!GXUUgenielog methodsFor: 'running'!

evaluate
GXcallout run line: 0 proc: self method: false; p1:'lowSystem'; p2:5; condEvaluate.
self up.
^nil
!!
nil! "(fix for smalltalk bug, this line is ignored if there is a variable declaration error)"

"End of procedure"

"Procedure class definition:
 ==========================="
((Smalltalk at: #GXUUtmpfile ifAbsent: [ nil ]) isKindOf: Function class) ifTrue: [ ('Warning: Overriding previously defined procedure ', 'GXUUtmpfile' gtype) prinl ].
Smalltalk at: #GXUUtmpfile put: nil!
Function subclass: 'GXUUtmpfile'
       instanceVariableNames: ''
       classVariableNames: ''
       poolDictionaries: ''
       category: 'Genie Procedure'
!

"HEADER
 ------"

"parameters
 ----------"

GXUUtmpfile

addInstVarName: #gXprefix;
compile:
'gXprefix: aVar
	aVar notNil ifTrue:[gXprefix _ self typeCheck: aVar shouldBe: GXString paramName: ''gXprefix'']';
compile:
'p1: aVar
	aVar notNil ifTrue:[gXprefix _ self typeCheck: aVar shouldBe: GXString paramName: ''gXprefix'']';

addInstVarName: #gXsuffix;
compile:
'gXsuffix: aVar
	aVar notNil ifTrue:[gXsuffix _ self typeCheck: aVar shouldBe: GXString paramName: ''gXsuffix'']';
compile:
'p2: aVar
	aVar notNil ifTrue:[gXsuffix _ self typeCheck: aVar shouldBe: GXString paramName: ''gXsuffix'']';

addInstVarName: #gXdirectory;
compile:
'gXdirectory: aVar
	aVar notNil ifTrue:[gXdirectory _ self typeCheck: aVar shouldBe: GXString paramName: ''gXdirectory'']';
compile:
'p3: aVar
	aVar notNil ifTrue:[gXdirectory _ self typeCheck: aVar shouldBe: GXString paramName: ''gXdirectory'']'!

"result
 ------"

GXUUtmpfile

addInstVarName: #gXtempfile
!

"Local variables
 ---------------"

GXUUtmpfile

addInstVarName: #gXprefix;

addInstVarName: #gXsuffix
!

!GXUUtmpfile methodsFor: 'initializing'!

initVars
	gXprefix _ 'genie'.
	gXsuffix _ 'tmp'
!!

"BODY
 ----"

!GXUUtmpfile methodsFor: 'running'!

evaluate
GXcallout run line: 0 proc: self method: false; p1:'lowSystem'; p2:6; condEvaluate.
self up.
^gXtempfile
!!
nil! "(fix for smalltalk bug, this line is ignored if there is a variable declaration error)"

"End of procedure"

"Procedure class definition:
 ==========================="
((Smalltalk at: #GXUUraisesignal ifAbsent: [ nil ]) isKindOf: Function class) ifTrue: [ ('Warning: Overriding previously defined procedure ', 'GXUUraisesignal' gtype) prinl ].
Smalltalk at: #GXUUraisesignal put: nil!
Function subclass: 'GXUUraisesignal'
       instanceVariableNames: ''
       classVariableNames: ''
       poolDictionaries: ''
       category: 'Genie Procedure'
!

"HEADER
 ------"

"parameters
 ----------"

GXUUraisesignal

addInstVarName: #gXsig;
compile:
'gXsig: aVar
	aVar notNil ifTrue:[ gXsig _ aVar ]';
compile:
'p1: aVar
	aVar notNil ifTrue:[ gXsig _ aVar ]'
!

!GXUUraisesignal methodsFor: 'initializing'!

initVars

!!

"BODY
 ----"

!GXUUraisesignal methodsFor: 'running'!

evaluate
GXcallout run line: 0 proc: self method: false; p1:'lowSystem'; p2:7; condEvaluate.
self up.
^nil
!!
nil! "(fix for smalltalk bug, this line is ignored if there is a variable declaration error)"

"End of procedure"

"Procedure class definition:
 ==========================="
((Smalltalk at: #GXUUtimes ifAbsent: [ nil ]) isKindOf: Function class) ifTrue: [ ('Warning: Overriding previously defined procedure ', 'GXUUtimes' gtype) prinl ].
Smalltalk at: #GXUUtimes put: nil!
Function subclass: 'GXUUtimes'
       instanceVariableNames: ''
       classVariableNames: ''
       poolDictionaries: ''
       category: 'Genie Procedure'
!

"HEADER
 ------"

"result
 ------"

GXUUtimes

addInstVarName: #gXstructtms
!

!GXUUtimes methodsFor: 'initializing'!

initVars
	gXstructtms _ (GXfields run line: 0 proc: self method: true; condEvaluate)
!!

"BODY
 ----"

!GXUUtimes methodsFor: 'running'!

evaluate
GXcallout run line: 0 proc: self method: false; p1:'lowSystem'; p2:8; condEvaluate.
self up.
^gXstructtms
!!
nil! "(fix for smalltalk bug, this line is ignored if there is a variable declaration error)"

"End of procedure"

"Procedure class definition:
 ==========================="
((Smalltalk at: #GXUUbgedit ifAbsent: [ nil ]) isKindOf: Function class) ifTrue: [ ('Warning: Overriding previously defined procedure ', 'GXUUbgedit' gtype) prinl ].
Smalltalk at: #GXUUbgedit put: nil!
Function subclass: 'GXUUbgedit'
       instanceVariableNames: ''
       classVariableNames: ''
       poolDictionaries: ''
       category: 'Genie Procedure'
!

"HEADER
 ------"

"parameters
 ----------"

GXUUbgedit

addInstVarName: #gXfile;
compile:
'gXfile: aVar
	aVar notNil ifTrue:[gXfile _ self typeCheck: aVar shouldBe: GXString paramName: ''gXfile'']';
compile:
'p1: aVar
	aVar notNil ifTrue:[gXfile _ self typeCheck: aVar shouldBe: GXString paramName: ''gXfile'']'
!

!GXUUbgedit methodsFor: 'initializing'!

initVars

!!

"BODY
 ----"

!GXUUbgedit methodsFor: 'running'!

evaluate
((GXos run line: 0 proc: self method: true; condEvaluate) = 'VMS') ifTrue: [ 
GXsystem run line: 0 proc: self method: false; p1:('SPAWN/WAIT/INPUT=NL: MC DECW$NOTEPAD ' + (gXfile)); condEvaluate ] ifFalse: [ 
GXsystem run line: 0 proc: self method: false; p1:(('xedit ' + (gXfile)) + '&'); condEvaluate ] .
self up.
^nil
!!
nil! "(fix for smalltalk bug, this line is ignored if there is a variable declaration error)"

"End of procedure"

"Procedure class definition:
 ==========================="
((Smalltalk at: #GXUUfieldUnames ifAbsent: [ nil ]) isKindOf: Function class) ifTrue: [ ('Warning: Overriding previously defined procedure ', 'GXUUfieldUnames' gtype) prinl ].
Smalltalk at: #GXUUfieldUnames put: nil!
Function subclass: 'GXUUfieldUnames'
       instanceVariableNames: ''
       classVariableNames: ''
       poolDictionaries: ''
       category: 'Genie Procedure'
!

"HEADER
 ------"

"parameters
 ----------"

GXUUfieldUnames

addInstVarName: #gXwksp;
compile:
'gXwksp: aVar
	aVar notNil ifTrue:[gXwksp _ self typeCheck: aVar shouldBe: GXWorkspace paramName: ''gXwksp'']';
compile:
'p1: aVar
	aVar notNil ifTrue:[gXwksp _ self typeCheck: aVar shouldBe: GXWorkspace paramName: ''gXwksp'']'!

"result
 ------"

GXUUfieldUnames

addInstVarName: #gXres
!

!GXUUfieldUnames methodsFor: 'initializing'!

initVars

!!

"BODY
 ----"

!GXUUfieldUnames methodsFor: 'running'!

evaluate

    |name s|
    gXwksp isNil ifTrue: [ '__FIELD_NAMES: No workspace given' prenl. ^nil ].
    s _ ''.
"loop through keys"
    gXwksp indicesDo: [ :i | 		
	name _ (gXwksp basicAt: i) demangle.
	s _ s , name , ','
    ].
    ((s size) > 1) 
"discard final ,"
	ifTrue: [
    	    gXres _ GXString new: (s size - 1).
    	    gXres replaceFrom: 1 to: (gXres size) with: s
	]
	ifFalse: [ gXres _ '' ]
.
self up.
^gXres
!!
nil! "(fix for smalltalk bug, this line is ignored if there is a variable declaration error)"

"End of procedure"

"Procedure class definition:
 ==========================="
((Smalltalk at: #GXUUfieldUtypes ifAbsent: [ nil ]) isKindOf: Function class) ifTrue: [ ('Warning: Overriding previously defined procedure ', 'GXUUfieldUtypes' gtype) prinl ].
Smalltalk at: #GXUUfieldUtypes put: nil!
Function subclass: 'GXUUfieldUtypes'
       instanceVariableNames: ''
       classVariableNames: ''
       poolDictionaries: ''
       category: 'Genie Procedure'
!

"HEADER
 ------"

"parameters
 ----------"

GXUUfieldUtypes

addInstVarName: #gXwksp;
compile:
'gXwksp: aVar
	aVar notNil ifTrue:[gXwksp _ self typeCheck: aVar shouldBe: GXWorkspace paramName: ''gXwksp'']';
compile:
'p1: aVar
	aVar notNil ifTrue:[gXwksp _ self typeCheck: aVar shouldBe: GXWorkspace paramName: ''gXwksp'']'!

"result
 ------"

GXUUfieldUtypes

addInstVarName: #gXres
!

!GXUUfieldUtypes methodsFor: 'initializing'!

initVars

!!

"BODY
 ----"

!GXUUfieldUtypes methodsFor: 'running'!

evaluate

    |name s|
    gXwksp isNil ifTrue: [ '__FIELD_TYPES: No workspace given' prenl. ^nil ].
    s _ ''.
    gXwksp do: [ :i | 		
	name _ i class name.
	s _ s , name , ','
    ].
    ((s size) > 1) 
"discard final ,"
	ifTrue: [
    	    gXres _ GXString new: (s size - 1).
    	    gXres replaceFrom: 1 to: (gXres size) with: s
	]
	ifFalse: [ gXres _ '' ]
.
self up.
^gXres
!!
nil! "(fix for smalltalk bug, this line is ignored if there is a variable declaration error)"

"End of procedure"



Object subclass: #Directory
		   instanceVariableNames: ''
		   classVariableNames: ''
		   poolDictionaries: ''
		   category: nil
!

Directory comment: 'An instance of me can execute chdir() and getcwd() calls' !

Behavior defineCFunc: 'getcwd'
	 withSelectorArgs: 'getcwd: aString len: anInt'
	 forClass: Directory
	 returning: #string
	 args: #(stringOut long)
!

Behavior defineCFunc: 'chdir'
	 withSelectorArgs: 'chdir: aString'
	 forClass: Directory
	 returning: #int
	 args: #(string)
!

!Directory class methodsFor: 'instance creation'!

new
	^super new
! !

!Directory methodsFor: 'instance creation'!

getcwd: aString
	^self getcwd: aString len: (aString size - 1)		"-1 to allow space for NULL"
!

getcwd
	| aString |
	aString _ String new: 256. 
	^self getcwd: aString len: (aString size - 1)		"-1 to allow space for NULL"
!

cd: aString
    aString isNil ifTrue: [ ^self chdir: (Smalltalk getenv: 'HOME') ]
                  ifFalse: [ ^self chdir: aString ]
! !

nil

!
"Procedure class definition:
 ==========================="
((Smalltalk at: #GXcd ifAbsent: [ nil ]) isKindOf: Function class) ifTrue: [ ('Warning: Overriding previously defined procedure ', 'GXcd' gtype) prinl ].
Smalltalk at: #GXcd put: nil!
Function subclass: 'GXcd'
       instanceVariableNames: ''
       classVariableNames: ''
       poolDictionaries: ''
       category: 'Genie Procedure'
!

"HEADER
 ------"

"parameters
 ----------"

GXcd

addInstVarName: #gXpath;
compile:
'gXpath: aVar
	aVar notNil ifTrue:[gXpath _ self typeCheck: aVar shouldBe: GXString paramName: ''gXpath'']';
compile:
'p1: aVar
	aVar notNil ifTrue:[gXpath _ self typeCheck: aVar shouldBe: GXString paramName: ''gXpath'']'
!

"Local variables
 ---------------"

GXcd

addInstVarName: #gXtmp
!

!GXcd methodsFor: 'initializing'!

initVars

!!

"BODY
 ----"

!GXcd methodsFor: 'running'!

evaluate

    gXtmp _ Directory new.
    ( (gXtmp cd: gXpath) = 0 ) ifFalse: 
	['CHDIR: Error setting working directory to "' pren. 
         gXpath pren. '"' prenl. 'CHDIR: ' pren.^(Smalltalk getStrerror) prenl ]
.
self up.
^nil
!!
nil! "(fix for smalltalk bug, this line is ignored if there is a variable declaration error)"

"End of procedure"

GXcomment run line: 0 proc: self method: false; p1:'cd'; p2:'Change the current directory'; condEvaluate!
GXalias run line: 0 proc: self method: false; p1:'chdir'; p2:'cd'; condEvaluate!
"Procedure class definition:
 ==========================="
((Smalltalk at: #GXpwd ifAbsent: [ nil ]) isKindOf: Function class) ifTrue: [ ('Warning: Overriding previously defined procedure ', 'GXpwd' gtype) prinl ].
Smalltalk at: #GXpwd put: nil!
Function subclass: 'GXpwd'
       instanceVariableNames: ''
       classVariableNames: ''
       poolDictionaries: ''
       category: 'Genie Procedure'
!

"HEADER
 ------"

"result
 ------"

GXpwd

addInstVarName: #gXres
!

!GXpwd methodsFor: 'initializing'!

initVars

!!

"BODY
 ----"

!GXpwd methodsFor: 'running'!

evaluate

    gXres _ (Directory new) getcwd
.
(GXcalledUasUfunction run line: 0 proc: self method: true; condEvaluate) not ifTrue: [ 
GXprintn run line: 0 proc: self method: false; p1:(gXres); condEvaluate ].
self up.
^gXres
!!
nil! "(fix for smalltalk bug, this line is ignored if there is a variable declaration error)"

"End of procedure"

GXcomment run line: 0 proc: self method: false; p1:'pwd'; p2:'Return the current directory'; condEvaluate!
GXalias run line: 0 proc: self method: false; p1:'getcwd'; p2:'pwd'; condEvaluate!
"Procedure class definition:
 ==========================="
((Smalltalk at: #GXdir ifAbsent: [ nil ]) isKindOf: Function class) ifTrue: [ ('Warning: Overriding previously defined procedure ', 'GXdir' gtype) prinl ].
Smalltalk at: #GXdir put: nil!
Function subclass: 'GXdir'
       instanceVariableNames: ''
       classVariableNames: ''
       poolDictionaries: ''
       category: 'Genie Procedure'
!

"HEADER
 ------"

"qualifiers
 ----------"

GXdir

addInstVarName: #gXdate;
compile:
'gXdate
	gXdate _ true';
compile:
'gXdate: aBoolean
	gXdate _ aBoolean';

addInstVarName: #gXfull;
compile:
'gXfull
	gXfull _ true';
compile:
'gXfull: aBoolean
	gXfull _ aBoolean'!

"parameters
 ----------"

GXdir

addInstVarName: #gXp1;
compile:
'gXp1: aVar
	aVar notNil ifTrue:[gXp1 _ self typeCheck: aVar shouldBe: GXString paramName: ''gXp1'']';
compile:
'p1: aVar
	aVar notNil ifTrue:[gXp1 _ self typeCheck: aVar shouldBe: GXString paramName: ''gXp1'']'
!

"Local variables
 ---------------"

GXdir

addInstVarName: #gXp1;

addInstVarName: #gXopts
!

!GXdir methodsFor: 'initializing'!

initVars
	gXdate _ false.
	gXfull _ false.
	gXp1 _ ''.
	gXopts _ ''
!!

"BODY
 ----"

!GXdir methodsFor: 'running'!

evaluate
((GXos run line: 0 proc: self method: true; condEvaluate) = 'VMS') ifTrue: [ 
(gXfull) ifTrue: [ 
gXopts _ ((gXopts) + '/FULL') deepCopy ].
(gXdate) ifTrue: [ 
gXopts _ ((gXopts) + '/DATE') deepCopy ].
GXsystem run line: 0 proc: self method: false; p1:((('DIRECTORY ' + (gXopts)) + ' ') + (gXp1)); condEvaluate ] ifFalse: [ 
(gXfull) ifTrue: [ 
gXopts _ ((gXopts) + '-l ') deepCopy ].
(gXdate) ifTrue: [ 
gXopts _ ((gXopts) + '-t ') deepCopy ].
GXsystem run line: 0 proc: self method: false; p1:((('ls ' + (gXopts)) + ' ') + (gXp1)); condEvaluate ] .
self up.
^nil
!!
nil! "(fix for smalltalk bug, this line is ignored if there is a variable declaration error)"

"End of procedure"

GXalias run line: 0 proc: self method: false; p1:'ls'; p2:'dir'; condEvaluate!
"Procedure class definition:
 ==========================="
((Smalltalk at: #GXUUerrorstatus ifAbsent: [ nil ]) isKindOf: Function class) ifTrue: [ ('Warning: Overriding previously defined procedure ', 'GXUUerrorstatus' gtype) prinl ].
Smalltalk at: #GXUUerrorstatus put: nil!
Function subclass: 'GXUUerrorstatus'
       instanceVariableNames: ''
       classVariableNames: ''
       poolDictionaries: ''
       category: 'Genie Procedure'
!

"HEADER
 ------"

"result
 ------"

GXUUerrorstatus

addInstVarName: #gXstatus
!

!GXUUerrorstatus methodsFor: 'initializing'!

initVars
	gXstatus _ 0
!!

"BODY
 ----"

!GXUUerrorstatus methodsFor: 'running'!

evaluate
GXcallout run line: 0 proc: self method: false; gXfacility:'objGraphics'; gXfunction:2; condEvaluate.
self up.
^gXstatus
!!
nil! "(fix for smalltalk bug, this line is ignored if there is a variable declaration error)"

"End of procedure"

GXcomment run line: 0 proc: self method: false; p1:'__errorstatus'; p2:'Returns TRUE if a graphics error occured'; condEvaluate!
"Procedure class definition:
 ==========================="
((Smalltalk at: #GXUUerroradd ifAbsent: [ nil ]) isKindOf: Function class) ifTrue: [ ('Warning: Overriding previously defined procedure ', 'GXUUerroradd' gtype) prinl ].
Smalltalk at: #GXUUerroradd put: nil!
Function subclass: 'GXUUerroradd'
       instanceVariableNames: ''
       classVariableNames: ''
       poolDictionaries: ''
       category: 'Genie Procedure'
!

"HEADER
 ------"

"parameters
 ----------"

GXUUerroradd

addInstVarName: #gXobject;
compile:
'gXobject: aVar
	aVar notNil ifTrue:[gXobject _ self typeCheck: aVar shouldBe: GXString paramName: ''gXobject'']';
compile:
'p1: aVar
	aVar notNil ifTrue:[gXobject _ self typeCheck: aVar shouldBe: GXString paramName: ''gXobject'']';

addInstVarName: #gXfailure;
compile:
'gXfailure: aVar
	aVar notNil ifTrue:[gXfailure _ self typeCheck: aVar shouldBe: GXString paramName: ''gXfailure'']';
compile:
'p2: aVar
	aVar notNil ifTrue:[gXfailure _ self typeCheck: aVar shouldBe: GXString paramName: ''gXfailure'']';

addInstVarName: #gXsolution;
compile:
'gXsolution: aVar
	aVar notNil ifTrue:[gXsolution _ self typeCheck: aVar shouldBe: GXString paramName: ''gXsolution'']';
compile:
'p3: aVar
	aVar notNil ifTrue:[gXsolution _ self typeCheck: aVar shouldBe: GXString paramName: ''gXsolution'']'
!

"Local variables
 ---------------"

GXUUerroradd

addInstVarName: #gXobject!

"Local variables
 ---------------"

GXUUerroradd

addInstVarName: #gXfailure!

"Local variables
 ---------------"

GXUUerroradd

addInstVarName: #gXsoloution
!

!GXUUerroradd methodsFor: 'initializing'!

initVars
	gXobject _ ''.
	gXfailure _ ''.
	gXsoloution _ ''
!!

"BODY
 ----"

!GXUUerroradd methodsFor: 'running'!

evaluate
GXcallout run line: 0 proc: self method: false; gXfacility:'objGraphics'; gXfunction:41; condEvaluate.
self up.
^nil
!!
nil! "(fix for smalltalk bug, this line is ignored if there is a variable declaration error)"

"End of procedure"

"Procedure class definition:
 ==========================="
((Smalltalk at: #GXUUclearerrors ifAbsent: [ nil ]) isKindOf: Function class) ifTrue: [ ('Warning: Overriding previously defined procedure ', 'GXUUclearerrors' gtype) prinl ].
Smalltalk at: #GXUUclearerrors put: nil!
Function subclass: 'GXUUclearerrors'
       instanceVariableNames: ''
       classVariableNames: ''
       poolDictionaries: ''
       category: 'Genie Procedure'
!

"BODY
 ----"

!GXUUclearerrors methodsFor: 'running'!

evaluate
GXcallout run line: 0 proc: self method: false; gXfacility:'objGraphics'; gXfunction:3; condEvaluate.
self up.
^nil
!!
nil! "(fix for smalltalk bug, this line is ignored if there is a variable declaration error)"

"End of procedure"

GXcomment run line: 0 proc: self method: false; p1:'__clearerrors'; p2:'Clears all graphics errors'; condEvaluate!
"Procedure class definition:
 ==========================="
((Smalltalk at: #GXUUerrorobject ifAbsent: [ nil ]) isKindOf: Function class) ifTrue: [ ('Warning: Overriding previously defined procedure ', 'GXUUerrorobject' gtype) prinl ].
Smalltalk at: #GXUUerrorobject put: nil!
Function subclass: 'GXUUerrorobject'
       instanceVariableNames: ''
       classVariableNames: ''
       poolDictionaries: ''
       category: 'Genie Procedure'
!

"HEADER
 ------"

"parameters
 ----------"

GXUUerrorobject

addInstVarName: #gXobjnum;
compile:
'gXobjnum: aVar
	aVar notNil ifTrue:[gXobjnum _ self typeCheck: aVar shouldBe: GXInteger paramName: ''gXobjnum'']';
compile:
'p1: aVar
	aVar notNil ifTrue:[gXobjnum _ self typeCheck: aVar shouldBe: GXInteger paramName: ''gXobjnum'']'!

"result
 ------"

GXUUerrorobject

addInstVarName: #gXobject
!

"Local variables
 ---------------"

GXUUerrorobject

addInstVarName: #gXobjnum
!

!GXUUerrorobject methodsFor: 'initializing'!

initVars
	gXobject _ ' '.
	gXobjnum _ 1
!!

"BODY
 ----"

!GXUUerrorobject methodsFor: 'running'!

evaluate
GXcallout run line: 0 proc: self method: false; gXfacility:'objGraphics'; gXfunction:4; condEvaluate.
self up.
^gXobject
!!
nil! "(fix for smalltalk bug, this line is ignored if there is a variable declaration error)"

"End of procedure"

GXcomment run line: 0 proc: self method: false; p1:'__errorobject'; p2:'Gives the object causing the error'; condEvaluate!
"Procedure class definition:
 ==========================="
((Smalltalk at: #GXUUerrorfailure ifAbsent: [ nil ]) isKindOf: Function class) ifTrue: [ ('Warning: Overriding previously defined procedure ', 'GXUUerrorfailure' gtype) prinl ].
Smalltalk at: #GXUUerrorfailure put: nil!
Function subclass: 'GXUUerrorfailure'
       instanceVariableNames: ''
       classVariableNames: ''
       poolDictionaries: ''
       category: 'Genie Procedure'
!

"HEADER
 ------"

"parameters
 ----------"

GXUUerrorfailure

addInstVarName: #gXobjnum;
compile:
'gXobjnum: aVar
	aVar notNil ifTrue:[gXobjnum _ self typeCheck: aVar shouldBe: GXInteger paramName: ''gXobjnum'']';
compile:
'p1: aVar
	aVar notNil ifTrue:[gXobjnum _ self typeCheck: aVar shouldBe: GXInteger paramName: ''gXobjnum'']'!

"result
 ------"

GXUUerrorfailure

addInstVarName: #gXfailure
!

"Local variables
 ---------------"

GXUUerrorfailure

addInstVarName: #gXobjnum
!

!GXUUerrorfailure methodsFor: 'initializing'!

initVars
	gXfailure _ ' '.
	gXobjnum _ 1
!!

"BODY
 ----"

!GXUUerrorfailure methodsFor: 'running'!

evaluate
GXcallout run line: 0 proc: self method: false; gXfacility:'objGraphics'; gXfunction:5; condEvaluate.
self up.
^gXfailure
!!
nil! "(fix for smalltalk bug, this line is ignored if there is a variable declaration error)"

"End of procedure"

GXcomment run line: 0 proc: self method: false; p1:'__errorfailure'; p2:'Returns the reason for the error'; condEvaluate!
"Procedure class definition:
 ==========================="
((Smalltalk at: #GXUUerrorsolution ifAbsent: [ nil ]) isKindOf: Function class) ifTrue: [ ('Warning: Overriding previously defined procedure ', 'GXUUerrorsolution' gtype) prinl ].
Smalltalk at: #GXUUerrorsolution put: nil!
Function subclass: 'GXUUerrorsolution'
       instanceVariableNames: ''
       classVariableNames: ''
       poolDictionaries: ''
       category: 'Genie Procedure'
!

"HEADER
 ------"

"parameters
 ----------"

GXUUerrorsolution

addInstVarName: #gXobjnum;
compile:
'gXobjnum: aVar
	aVar notNil ifTrue:[gXobjnum _ self typeCheck: aVar shouldBe: GXInteger paramName: ''gXobjnum'']';
compile:
'p1: aVar
	aVar notNil ifTrue:[gXobjnum _ self typeCheck: aVar shouldBe: GXInteger paramName: ''gXobjnum'']'!

"result
 ------"

GXUUerrorsolution

addInstVarName: #gXsolution
!

"Local variables
 ---------------"

GXUUerrorsolution

addInstVarName: #gXobjnum
!

!GXUUerrorsolution methodsFor: 'initializing'!

initVars
	gXsolution _ ' '.
	gXobjnum _ 1
!!

"BODY
 ----"

!GXUUerrorsolution methodsFor: 'running'!

evaluate
GXcallout run line: 0 proc: self method: false; gXfacility:'objGraphics'; gXfunction:6; condEvaluate.
self up.
^gXsolution
!!
nil! "(fix for smalltalk bug, this line is ignored if there is a variable declaration error)"

"End of procedure"

GXcomment run line: 0 proc: self method: false; p1:'__errorsolution'; p2:'Returns a possible solution'; condEvaluate!
"Procedure class definition:
 ==========================="
((Smalltalk at: #GXUUerrormessage ifAbsent: [ nil ]) isKindOf: Function class) ifTrue: [ ('Warning: Overriding previously defined procedure ', 'GXUUerrormessage' gtype) prinl ].
Smalltalk at: #GXUUerrormessage put: nil!
Function subclass: 'GXUUerrormessage'
       instanceVariableNames: ''
       classVariableNames: ''
       poolDictionaries: ''
       category: 'Genie Procedure'
!

"HEADER
 ------"

"parameters
 ----------"

GXUUerrormessage

addInstVarName: #gXobjnum;
compile:
'gXobjnum: aVar
	aVar notNil ifTrue:[gXobjnum _ self typeCheck: aVar shouldBe: GXInteger paramName: ''gXobjnum'']';
compile:
'p1: aVar
	aVar notNil ifTrue:[gXobjnum _ self typeCheck: aVar shouldBe: GXInteger paramName: ''gXobjnum'']'!

"result
 ------"

GXUUerrormessage

addInstVarName: #gXmessage
!

"Local variables
 ---------------"

GXUUerrormessage

addInstVarName: #gXobjnum
!

!GXUUerrormessage methodsFor: 'initializing'!

initVars
	gXmessage _ ' '.
	gXobjnum _ 1
!!

"BODY
 ----"

!GXUUerrormessage methodsFor: 'running'!

evaluate
GXcallout run line: 0 proc: self method: false; gXfacility:'objGraphics'; gXfunction:7; condEvaluate.
self up.
^gXmessage
!!
nil! "(fix for smalltalk bug, this line is ignored if there is a variable declaration error)"

"End of procedure"

GXcomment run line: 0 proc: self method: false; p1:'__errormessage'; p2:'Returns all error information'; condEvaluate!
"Procedure class definition:
 ==========================="
((Smalltalk at: #GXgUerrorUhandler ifAbsent: [ nil ]) isKindOf: Function class) ifTrue: [ ('Warning: Overriding previously defined procedure ', 'GXgUerrorUhandler' gtype) prinl ].
Smalltalk at: #GXgUerrorUhandler put: nil!
Function subclass: 'GXgUerrorUhandler'
       instanceVariableNames: ''
       classVariableNames: ''
       poolDictionaries: ''
       category: 'Genie Procedure'
!

"HEADER
 ------"

"result
 ------"

GXgUerrorUhandler

addInstVarName: #gXretval
!

"Local variables
 ---------------"

GXgUerrorUhandler

addInstVarName: #gXa!

"Local variables
 ---------------"

GXgUerrorUhandler

addInstVarName: #gXb!

"Local variables
 ---------------"

GXgUerrorUhandler

addInstVarName: #gXstr;

addInstVarName: #gXstr2
!

!GXgUerrorUhandler methodsFor: 'initializing'!

initVars

!!

"BODY
 ----"

!GXgUerrorUhandler methodsFor: 'running'!

evaluate
gXb _ (GXUUerrorstatus run line: 0 proc: self method: true; condEvaluate) deepCopy.
gXretval _ (1 = 2) deepCopy.
((gXb) > 0) ifTrue: [ 
Function new loop: (gXb) to: 1 by: 1 negated before: [ :gXa | 
gXstr _ (GXUUerrorobject run line: 0 proc: self method: true; p1:(gXa); condEvaluate) deepCopy.
((gXstr) = 'INFORMATION') ifTrue: [ 
GXprintin run line: 0 proc: self method: false; p1:'  '; p2:(GXUUerrorfailure run line: 0 proc: self method: true; p1:(gXa); condEvaluate); condEvaluate.
GXprintin run line: 0 proc: self method: false; p1:'  '; p2:(GXUUerrorsolution run line: 0 proc: self method: true; p1:(gXa); condEvaluate); condEvaluate ] ifFalse: [ 
gXretval _ (1 = 1) deepCopy.
GXprinten run line: 0 proc: self method: false; p1:'  ERROR returned from '; p2:(gXstr); p3:' -'; condEvaluate.
GXprinten run line: 0 proc: self method: false; p1:'  '; p2:(GXUUerrorfailure run line: 0 proc: self method: true; p1:(gXa); condEvaluate); p3:'.'; condEvaluate.
gXstr2 _ (GXUUerrorsolution run line: 0 proc: self method: true; p1:(gXa); condEvaluate) deepCopy.
(((gXstr2) ~= '')) ifTrue: [ 
GXprintin run line: 0 proc: self method: false; p1:'  SOLUTION: '; p2:(gXstr2); condEvaluate ] ]  ] 
condition: [false] after: [ :gXa | nil ].
GXUUclearerrors run line: 0 proc: self method: false; condEvaluate ].
self up.
^gXretval
!!
nil! "(fix for smalltalk bug, this line is ignored if there is a variable declaration error)"

"End of procedure"

GXcomment run line: 0 proc: self method: false; p1:'g_error_handler'; p2:'checks and reports graphics errors'; condEvaluate!
((GXWorkspace) subclass:'Structure' fields:'' comment:'Underlying Smalltalk type - use Structure instead')!
((GXStructure) subclass:'DataSet' fields:'' comment:'Any form of experimental dataset')!
((GXDataset) subclass:'Spectrum' fields:'X V' comment:'A Single spectrum')!
((GXDataset) subclass:'MultiSpectrum' fields:'S' comment:'A group of single spectra')!
((GXDataset) subclass:'SpectrumSet' fields:'V X' comment:'A set of spectra with common X')!
((GXDataset) subclass:'MultiSpectrumSet' fields:'SS' comment:'A group of Spectrum sets')!
((GXDataset) subclass:'Grid' fields:'V X' comment:'A gridded data set')!
((GXDataset) subclass:'MultiGrid' fields:'G' comment:'A group of gridded data sets')!
((GXDataset) subclass:'List' fields:'V' comment:'A list of Values')!
((GXDataset) subclass:'MultiList' fields:'L' comment:'A group of lists of values')!
((GXStructure) subclass:'Values' fields:'Err Val' comment:'A single array of data values')!
((GXStructure) subclass:'Space' fields:'Err Val' comment:'An array of locating information - eg X array')!
((GXSpectrum) subclass:'Scan' fields:'Measurements' comment:'A spectrum with varying experimental measurements')!
((GXMultispectrum) subclass:'MultiScan' fields:'Measurements' comment:'A multiple spectrum with varying experimental measurements')!
((GXSpectrumset) subclass:'ScanSet' fields:'Measurements' comment:'A spectrum set with varying experimental measurements')!
((GXMultispectrumset) subclass:'MultiScanSet' fields:'Measurements' comment:'A multiple spectrum set with varying experimental measurements')!
((GXValues) subclass:'Intensity' fields:'Units' comment:'An array of of intensity values (counts)')!
((GXIntensity) subclass:'DetectorIntensity' fields:'Ldet Lsamp TwoTheta' comment:'Per detector parameters')!
((GXDetectorintensity) subclass:'Indirect' fields:'Measurements' comment:'Per detector parameters (indirect geometry)')!
((GXIntensity) subclass:'MonitorIntensity' fields:'Lmon' comment:'Parameters for a single monitor')!
((GXSpace) subclass:'MajorSpace' fields:'Units' comment:'Primary abcissa, eg. Wavelength')!
((GXStructure) subclass:'Measurement' fields:'Units' comment:'Any measured value')!
((GXMeasurement) subclass:'Efield' fields:'' comment:'Electric field strength')!
((GXMeasurement) subclass:'Bfield' fields:'' comment:'Magnetic field strength')!
((GXMeasurement) subclass:'Temperature' fields:'' comment:'Temperature')!
((GXMeasurement) subclass:'Pressure' fields:'' comment:'Pressure')!
((GXMeasurement) subclass:'Polarization' fields:'Px Py Pz' comment:'A fixed polarization value')!
((GXMeasurement) subclass:'Energy' fields:'E' comment:'A fixed energy value')!
((GXMeasurement) subclass:'Strain' fields:'' comment:'Measured strain')!
((GXMeasurement) subclass:'Time' fields:'' comment:'A time, eg scan start')!
((GXMeasurement) subclass:'Position' fields:'' comment:'Sample positioning information')!
((GXPosition) subclass:'Orientation' fields:'Chi Phi Omega' comment:'Crystal orientation')!
((GXPosition) subclass:'ReciprocalPosition' fields:'Qx Qy Qz' comment:'Reciprocal lattice position')!
((GXStructure) subclass:'Unit' fields:'Name Label Components Scale Offset' comment:'A complete description of the physical quantity being measured')!
((GXStructure) subclass:'Experiment' fields:'Instrument User Data Sample History DataSource Name Duration Start' comment:'One complete experiment')!
((GXStructure) subclass:'Sample' fields:'Name Measurements' comment:'Description of sample parameters')!
((GXSample) subclass:'Crystaline' fields:'LatticeParameters Plane' comment:'Crystal parameters')!
((GXSample) subclass:'Amorphous' fields:'' comment:'An amorphous sample')!
((GXSample) subclass:'Liquid' fields:'' comment:'A liquid sample')!
((GXStructure) subclass:'User' fields:'Name' comment:'Information about the experimentor')!
((GXStructure) subclass:'Instrument' fields:'Name Detector Analyser Monitor' comment:'Complete instrument parameter description')!
((GXStructure) subclass:'Detector' fields:'Angle delta' comment:'Individual detector description')!
((GXStructure) subclass:'Analyser' fields:'TwoTheta' comment:'Individual analyser description')!
((GXStructure) subclass:'Monitor' fields:'Efficiency' comment:'Individual monitor description')!
((GXStructure) subclass:'History' fields:'Display' comment:'History and/or settings data')!
"Procedure class definition:
 ==========================="
((Smalltalk at: #GXUunitUhelp ifAbsent: [ nil ]) isKindOf: Function class) ifTrue: [ ('Warning: Overriding previously defined procedure ', 'GXUunitUhelp' gtype) prinl ].
Smalltalk at: #GXUunitUhelp put: nil!
Function subclass: 'GXUunitUhelp'
       instanceVariableNames: ''
       classVariableNames: ''
       poolDictionaries: ''
       category: 'Genie Procedure'
!

"BODY
 ----"

!GXUunitUhelp methodsFor: 'running'!

evaluate
GXprintn run line: 0 proc: self method: false; p1:'/NEW  NAME=<String> LABEL=<String> COMPONENTS=<String> SCALE=<Real>'; condEvaluate.
GXprintn run line: 0 proc: self method: false; p1:'/CONV UNIT1=<Unit> UNIT2=<Unit> 	return factor to convert UNIT2 into UNIT1, 0.0 if impossible'; condEvaluate.
GXprintn run line: 0 proc: self method: false; p1:'/LIST    				List known units'; condEvaluate.
GXprintn run line: 0 proc: self method: false; p1:'/DIMS UNIT=<Unit> 			Return dimensions of a unit'; condEvaluate.
GXprintn run line: 0 proc: self method: false; p1:'/MULT UNIT1=<Unit> UNIT2=<Unit> 	multiply two units'; condEvaluate.
GXprintn run line: 0 proc: self method: false; p1:'/SHOW UNIT=<Unit>			Display information on a unit'; condEvaluate.
GXprintn run line: 0 proc: self method: false; p1:'Example'; condEvaluate.
GXprintn run line: 0 proc: self method: false; p1:'   GLOBAL $ANGSTROM=UNIT:NEW("Angstrom", "A", "m,1", 1e-10)'; condEvaluate.
self up.
^nil
!!
nil! "(fix for smalltalk bug, this line is ignored if there is a variable declaration error)"

"End of procedure"

"Procedure class definition:
 ==========================="
((Smalltalk at: #GXUunitUdims ifAbsent: [ nil ]) isKindOf: Function class) ifTrue: [ ('Warning: Overriding previously defined procedure ', 'GXUunitUdims' gtype) prinl ].
Smalltalk at: #GXUunitUdims put: nil!
Function subclass: 'GXUunitUdims'
       instanceVariableNames: ''
       classVariableNames: ''
       poolDictionaries: ''
       category: 'Genie Procedure'
!

"HEADER
 ------"

"parameters
 ----------"

GXUunitUdims

addInstVarName: #gXencodedUunit;
compile:
'gXencodedUunit: aVar
	aVar notNil ifTrue:[gXencodedUunit _ self typeCheck: aVar shouldBe: GXUnit paramName: ''gXencodedUunit'']';
compile:
'p1: aVar
	aVar notNil ifTrue:[gXencodedUunit _ self typeCheck: aVar shouldBe: GXUnit paramName: ''gXencodedUunit'']'!

"result
 ------"

GXUunitUdims

addInstVarName: #gXdims
!

!GXUunitUdims methodsFor: 'initializing'!

initVars

!!

"BODY
 ----"

!GXUunitUdims methodsFor: 'running'!

evaluate
GXcallout run line: 0 proc: self method: false; p1:'extTransform'; p2:35; condEvaluate.
GXgUerrorUhandler run line: 0 proc: self method: false; condEvaluate.
self up.
^gXdims
!!
nil! "(fix for smalltalk bug, this line is ignored if there is a variable declaration error)"

"End of procedure"

GXcomment run line: 0 proc: self method: false; p1:'_unit_dims'; p2:'Get unit dimensions'; condEvaluate!
"Procedure class definition:
 ==========================="
((Smalltalk at: #GXUunitUproperties ifAbsent: [ nil ]) isKindOf: Function class) ifTrue: [ ('Warning: Overriding previously defined procedure ', 'GXUunitUproperties' gtype) prinl ].
Smalltalk at: #GXUunitUproperties put: nil!
Function subclass: 'GXUunitUproperties'
       instanceVariableNames: ''
       classVariableNames: ''
       poolDictionaries: ''
       category: 'Genie Procedure'
!

"HEADER
 ------"

"parameters
 ----------"

GXUunitUproperties

addInstVarName: #gXencodedUunit;
compile:
'gXencodedUunit: aVar
	aVar notNil ifTrue:[gXencodedUunit _ self typeCheck: aVar shouldBe: GXUnit paramName: ''gXencodedUunit'']';
compile:
'p1: aVar
	aVar notNil ifTrue:[gXencodedUunit _ self typeCheck: aVar shouldBe: GXUnit paramName: ''gXencodedUunit'']'!

"result
 ------"

GXUunitUproperties

addInstVarName: #gXproperties
!

!GXUunitUproperties methodsFor: 'initializing'!

initVars

!!

"BODY
 ----"

!GXUunitUproperties methodsFor: 'running'!

evaluate
GXcallout run line: 0 proc: self method: false; p1:'extTransform'; p2:32; condEvaluate.
GXgUerrorUhandler run line: 0 proc: self method: false; condEvaluate.
self up.
^gXproperties
!!
nil! "(fix for smalltalk bug, this line is ignored if there is a variable declaration error)"

"End of procedure"

GXcomment run line: 0 proc: self method: false; p1:'_unit_properties'; p2:'Show information about a unit'; condEvaluate!
"Procedure class definition:
 ==========================="
((Smalltalk at: #GXUunitUmult ifAbsent: [ nil ]) isKindOf: Function class) ifTrue: [ ('Warning: Overriding previously defined procedure ', 'GXUunitUmult' gtype) prinl ].
Smalltalk at: #GXUunitUmult put: nil!
Function subclass: 'GXUunitUmult'
       instanceVariableNames: ''
       classVariableNames: ''
       poolDictionaries: ''
       category: 'Genie Procedure'
!

"HEADER
 ------"

"parameters
 ----------"

GXUunitUmult

addInstVarName: #gXunit1;
compile:
'gXunit1: aVar
	aVar notNil ifTrue:[gXunit1 _ self typeCheck: aVar shouldBe: GXUnit paramName: ''gXunit1'']';
compile:
'p1: aVar
	aVar notNil ifTrue:[gXunit1 _ self typeCheck: aVar shouldBe: GXUnit paramName: ''gXunit1'']';

addInstVarName: #gXunit2;
compile:
'gXunit2: aVar
	aVar notNil ifTrue:[gXunit2 _ self typeCheck: aVar shouldBe: GXUnit paramName: ''gXunit2'']';
compile:
'p2: aVar
	aVar notNil ifTrue:[gXunit2 _ self typeCheck: aVar shouldBe: GXUnit paramName: ''gXunit2'']'!

"result
 ------"

GXUunitUmult

addInstVarName: #gXres
!

!GXUunitUmult methodsFor: 'initializing'!

initVars

!!

"BODY
 ----"

!GXUunitUmult methodsFor: 'running'!

evaluate
GXcallout run line: 0 proc: self method: false; p1:'extTransform'; p2:33; condEvaluate.
GXgUerrorUhandler run line: 0 proc: self method: false; condEvaluate.
self up.
^gXres
!!
nil! "(fix for smalltalk bug, this line is ignored if there is a variable declaration error)"

"End of procedure"

GXcomment run line: 0 proc: self method: false; p1:'_unit_mult'; p2:'Multiply two units'; condEvaluate!
"Procedure class definition:
 ==========================="
((Smalltalk at: #GXUunitUconv ifAbsent: [ nil ]) isKindOf: Function class) ifTrue: [ ('Warning: Overriding previously defined procedure ', 'GXUunitUconv' gtype) prinl ].
Smalltalk at: #GXUunitUconv put: nil!
Function subclass: 'GXUunitUconv'
       instanceVariableNames: ''
       classVariableNames: ''
       poolDictionaries: ''
       category: 'Genie Procedure'
!

"HEADER
 ------"

"parameters
 ----------"

GXUunitUconv

addInstVarName: #gXunit1;
compile:
'gXunit1: aVar
	aVar notNil ifTrue:[gXunit1 _ self typeCheck: aVar shouldBe: GXUnit paramName: ''gXunit1'']';
compile:
'p1: aVar
	aVar notNil ifTrue:[gXunit1 _ self typeCheck: aVar shouldBe: GXUnit paramName: ''gXunit1'']';

addInstVarName: #gXunit2;
compile:
'gXunit2: aVar
	aVar notNil ifTrue:[gXunit2 _ self typeCheck: aVar shouldBe: GXUnit paramName: ''gXunit2'']';
compile:
'p2: aVar
	aVar notNil ifTrue:[gXunit2 _ self typeCheck: aVar shouldBe: GXUnit paramName: ''gXunit2'']'!

"result
 ------"

GXUunitUconv

addInstVarName: #gXconv
!

!GXUunitUconv methodsFor: 'initializing'!

initVars

!!

"BODY
 ----"

!GXUunitUconv methodsFor: 'running'!

evaluate
GXcallout run line: 0 proc: self method: false; p1:'extTransform'; p2:34; condEvaluate.
GXgUerrorUhandler run line: 0 proc: self method: false; condEvaluate.
self up.
^gXconv
!!
nil! "(fix for smalltalk bug, this line is ignored if there is a variable declaration error)"

"End of procedure"

GXcomment run line: 0 proc: self method: false; p1:'_unit_conv'; p2:'Return the scaling factor to convert one unit into another'; condEvaluate!
"Procedure class definition:
 ==========================="
((Smalltalk at: #GXUunitUlist ifAbsent: [ nil ]) isKindOf: Function class) ifTrue: [ ('Warning: Overriding previously defined procedure ', 'GXUunitUlist' gtype) prinl ].
Smalltalk at: #GXUunitUlist put: nil!
Function subclass: 'GXUunitUlist'
       instanceVariableNames: ''
       classVariableNames: ''
       poolDictionaries: ''
       category: 'Genie Procedure'
!

"HEADER
 ------"

"result
 ------"

GXUunitUlist

addInstVarName: #gXlist
!

!GXUunitUlist methodsFor: 'initializing'!

initVars

!!

"BODY
 ----"

!GXUunitUlist methodsFor: 'running'!

evaluate
GXcallout run line: 0 proc: self method: false; p1:'extTransform'; p2:36; condEvaluate.
GXgUerrorUhandler run line: 0 proc: self method: false; condEvaluate.
self up.
^gXlist
!!
nil! "(fix for smalltalk bug, this line is ignored if there is a variable declaration error)"

"End of procedure"

GXcomment run line: 0 proc: self method: false; p1:'_unit_list'; p2:'List know units'; condEvaluate!
"Procedure class definition:
 ==========================="
((Smalltalk at: #GXUunitUnew ifAbsent: [ nil ]) isKindOf: Function class) ifTrue: [ ('Warning: Overriding previously defined procedure ', 'GXUunitUnew' gtype) prinl ].
Smalltalk at: #GXUunitUnew put: nil!
Function subclass: 'GXUunitUnew'
       instanceVariableNames: ''
       classVariableNames: ''
       poolDictionaries: ''
       category: 'Genie Procedure'
!

"HEADER
 ------"

"parameters
 ----------"

GXUunitUnew

addInstVarName: #gXname;
compile:
'gXname: aVar
	aVar notNil ifTrue:[gXname _ self typeCheck: aVar shouldBe: GXString paramName: ''gXname'']';
compile:
'p1: aVar
	aVar notNil ifTrue:[gXname _ self typeCheck: aVar shouldBe: GXString paramName: ''gXname'']';

addInstVarName: #gXlabel;
compile:
'gXlabel: aVar
	aVar notNil ifTrue:[gXlabel _ self typeCheck: aVar shouldBe: GXString paramName: ''gXlabel'']';
compile:
'p2: aVar
	aVar notNil ifTrue:[gXlabel _ self typeCheck: aVar shouldBe: GXString paramName: ''gXlabel'']';

addInstVarName: #gXcomponents;
compile:
'gXcomponents: aVar
	aVar notNil ifTrue:[gXcomponents _ self typeCheck: aVar shouldBe: GXString paramName: ''gXcomponents'']';
compile:
'p3: aVar
	aVar notNil ifTrue:[gXcomponents _ self typeCheck: aVar shouldBe: GXString paramName: ''gXcomponents'']';

addInstVarName: #gXscale;
compile:
'gXscale: aVar
	aVar notNil ifTrue:[gXscale _ self typeCheck: aVar shouldBe: GXReal paramName: ''gXscale'']';
compile:
'p4: aVar
	aVar notNil ifTrue:[gXscale _ self typeCheck: aVar shouldBe: GXReal paramName: ''gXscale'']'!

"result
 ------"

GXUunitUnew

addInstVarName: #gXencodedUunit
!

"Local variables
 ---------------"

GXUunitUnew

addInstVarName: #gXscale
!

!GXUunitUnew methodsFor: 'initializing'!

initVars
	gXscale _ 1.000000000000000e000
!!

"BODY
 ----"

!GXUunitUnew methodsFor: 'running'!

evaluate
GXgUerrorUhandler run line: 0 proc: self method: false; condEvaluate.
self up.
^gXencodedUunit
!!
nil! "(fix for smalltalk bug, this line is ignored if there is a variable declaration error)"

"End of procedure"

GXcomment run line: 0 proc: self method: false; p1:'_unit_new'; p2:'Create a new unit'; condEvaluate!
"Procedure class definition:
 ==========================="
((Smalltalk at: #GXunit ifAbsent: [ nil ]) isKindOf: Function class) ifTrue: [ ('Warning: Overriding previously defined procedure ', 'GXunit' gtype) prinl ].
Smalltalk at: #GXunit put: nil!
Function subclass: 'GXunit'
       instanceVariableNames: ''
       classVariableNames: ''
       poolDictionaries: ''
       category: 'Genie Procedure'
!

"HEADER
 ------"

"qualifiers
 ----------"

GXunit

addInstVarName: #gXnew;
compile:
'gXnew
	gXnew _ true';
compile:
'gXnew: aBoolean
	gXnew _ aBoolean';

addInstVarName: #gXshow;
compile:
'gXshow
	gXshow _ true';
compile:
'gXshow: aBoolean
	gXshow _ aBoolean';

addInstVarName: #gXmult;
compile:
'gXmult
	gXmult _ true';
compile:
'gXmult: aBoolean
	gXmult _ aBoolean';

addInstVarName: #gXdiv;
compile:
'gXdiv
	gXdiv _ true';
compile:
'gXdiv: aBoolean
	gXdiv _ aBoolean';

addInstVarName: #gXconv;
compile:
'gXconv
	gXconv _ true';
compile:
'gXconv: aBoolean
	gXconv _ aBoolean';

addInstVarName: #gXdims;
compile:
'gXdims
	gXdims _ true';
compile:
'gXdims: aBoolean
	gXdims _ aBoolean';

addInstVarName: #gXlist;
compile:
'gXlist
	gXlist _ true';
compile:
'gXlist: aBoolean
	gXlist _ aBoolean'!

"parameters
 ----------"

GXunit

addInstVarName: #gXp1;
compile:
'gXp1: aVar
	aVar notNil ifTrue:[ gXp1 _ aVar ]';
compile:
'p1: aVar
	aVar notNil ifTrue:[ gXp1 _ aVar ]';

addInstVarName: #gXp2;
compile:
'gXp2: aVar
	aVar notNil ifTrue:[ gXp2 _ aVar ]';
compile:
'p2: aVar
	aVar notNil ifTrue:[ gXp2 _ aVar ]';

addInstVarName: #gXp3;
compile:
'gXp3: aVar
	aVar notNil ifTrue:[ gXp3 _ aVar ]';
compile:
'p3: aVar
	aVar notNil ifTrue:[ gXp3 _ aVar ]';

addInstVarName: #gXp4;
compile:
'gXp4: aVar
	aVar notNil ifTrue:[ gXp4 _ aVar ]';
compile:
'p4: aVar
	aVar notNil ifTrue:[ gXp4 _ aVar ]'!

"result
 ------"

GXunit

addInstVarName: #gXres
!

!GXunit methodsFor: 'initializing'!

initVars
	gXnew _ false.
	gXshow _ false.
	gXmult _ false.
	gXdiv _ false.
	gXconv _ false.
	gXdims _ false.
	gXlist _ false
!!

"BODY
 ----"

!GXunit methodsFor: 'running'!

evaluate
(gXnew) ifTrue: [ 
gXres _ (GXUunitUnew run line: 0 proc: self method: true; p1:(gXp1); p2:(gXp2); p3:(gXp3); p4:(gXp4); condEvaluate) deepCopy ] ifFalse: [ (gXshow) ifTrue: [ 
GXprintn run line: 0 proc: self method: false; p1:(GXUunitUproperties run line: 0 proc: self method: true; p1:(gXp1); condEvaluate); condEvaluate ] ifFalse: [ (gXmult) ifTrue: [ 
gXres _ (GXUunitUmult run line: 0 proc: self method: true; p1:(gXp1); p2:(gXp2); condEvaluate) deepCopy ] ifFalse: [ (gXconv) ifTrue: [ 
gXres _ (GXUunitUconv run line: 0 proc: self method: true; p1:(gXp1); p2:(gXp2); condEvaluate) deepCopy ] ifFalse: [ (gXdims) ifTrue: [ 
gXres _ (GXUunitUdims run line: 0 proc: self method: true; p1:(gXp1); condEvaluate) deepCopy ] ifFalse: [ (gXlist) ifTrue: [ 
GXprintn run line: 0 proc: self method: false; p1:'*** User defined units ***'; condEvaluate.
GXprintn run line: 0 proc: self method: false; p1:(GXUunitUlist run line: 0 proc: self method: true; condEvaluate); condEvaluate ]]]]]].
self up.
^gXres
!!
nil! "(fix for smalltalk bug, this line is ignored if there is a variable declaration error)"

"End of procedure"

GXcomment run line: 0 proc: self method: false; p1:'unit'; p2:'Manipulate unit types'; condEvaluate!
"Procedure class definition:
 ==========================="
((Smalltalk at: #GXUUinitunits ifAbsent: [ nil ]) isKindOf: Function class) ifTrue: [ ('Warning: Overriding previously defined procedure ', 'GXUUinitunits' gtype) prinl ].
Smalltalk at: #GXUUinitunits put: nil!
Function subclass: 'GXUUinitunits'
       instanceVariableNames: ''
       classVariableNames: ''
       poolDictionaries: ''
       category: 'Genie Procedure'
!

"HEADER
 ------"

"Global variables
 ----------------"

GXUUinitunits

addPoolVar: #gXDpi value: 3.141592653589790e000!

"Global variables
 ----------------"

GXUUinitunits

addPoolVar: #gXDmetre value: (GXunit run line: 0 proc: self method: true; gXnew; p1:'metres'; p2:'m'; p3:'m,1'; p4:1.000000000000000e000; condEvaluate)!

"Global variables
 ----------------"

GXUUinitunits

addPoolVar: #gXDsecond value: (GXunit run line: 0 proc: self method: true; gXnew; p1:'seconds'; p2:'s'; p3:'s,1'; p4:1.000000000000000e000; condEvaluate)!

"Global variables
 ----------------"

GXUUinitunits

addPoolVar: #gXDkelvin value: (GXunit run line: 0 proc: self method: true; gXnew; p1:'Kelvin'; p2:'K'; p3:'K,1'; p4:1.000000000000000e000; condEvaluate)!

"Global variables
 ----------------"

GXUUinitunits

addPoolVar: #gXDamp value: (GXunit run line: 0 proc: self method: true; gXnew; p1:'Amps'; p2:'A'; p3:'A,1'; p4:1.000000000000000e000; condEvaluate)!

"Global variables
 ----------------"

GXUUinitunits

addPoolVar: #gXDcandela value: (GXunit run line: 0 proc: self method: true; gXnew; p1:'Candelas'; p2:'Cd'; p3:'Cd,1'; p4:1.000000000000000e000; condEvaluate)!

"Global variables
 ----------------"

GXUUinitunits

addPoolVar: #gXDmole value: (GXunit run line: 0 proc: self method: true; gXnew; p1:'moles'; p2:'M'; p3:'Mole,1'; p4:1.000000000000000e000; condEvaluate)!

"Global variables
 ----------------"

GXUUinitunits

addPoolVar: #gXDkilogram value: (GXunit run line: 0 proc: self method: true; gXnew; p1:'Kilograms'; p2:'Kg'; p3:'kilo-gram,1'; p4:1.000000000000000e000; condEvaluate)!

"Global variables
 ----------------"

GXUUinitunits

addPoolVar: #gXDradian value: (GXunit run line: 0 proc: self method: true; gXnew; p1:'radians'; p2:'rad'; p3:'rad,1'; p4:1.000000000000000e000; condEvaluate)!

"Global variables
 ----------------"

GXUUinitunits

addPoolVar: #gXDangstrom value: (GXunit run line: 0 proc: self method: true; gXnew; p1:'Angstroms'; p2:'A'; p3:'m,1'; p4:1.000000000000000e-10; condEvaluate)!

"Global variables
 ----------------"

GXUUinitunits

addPoolVar: #gXDev value: (GXunit run line: 0 proc: self method: true; gXnew; p1:'Electron Volts'; p2:'eV'; p3:'J,1'; p4:1.602000000000000e-19; condEvaluate)!

"Global variables
 ----------------"

GXUUinitunits

addPoolVar: #gXDdegree value: (GXunit run line: 0 proc: self method: true; gXnew; p1:'degrees'; p2:'deg'; p3:'rad,1'; p4:((gXDpi) / 1.800000000000000e002); condEvaluate)
!

!GXUUinitunits methodsFor: 'initializing'!

initVars

!!

"End of procedure"

GXcomment run line: 0 proc: self method: false; p1:'__initunits'; p2:'Set up pre-defined common units'; condEvaluate!

!GXExperiment methodsFor: 'binary operations'!

binop: anExperiment op: anOperator
        'Experiment binop called' prnl.
        ^self class new
                at: #gXdata put: ( ( self at: #gXdata ) binop: ( anExperiment at: #gXdata ) op: anOperator );
                yourself
!
!

!GXDataset methodsFor: 'binary operations'!

binop: aDataset op: anOperator
        'DataSet binop called' prnl.
        ^self class new
                at: #gXv put: ( ( self at: #gXv ) binop: ( aDataset at: #gXv ) op: anOperator );
                yourself
!
!

!GXValues methodsFor: 'binary operations'!

binop: aValue op: anOperator
        | sv se ov oe |
        'Value binop called' prnl.
        sv _ self at: #gXval.
        se _ self at: #gXerr.
        ov _ aValue at: #gXval.
        oe _ aValue at: #gXerr.

        "There is no lower class we can put off differing operations to, do the work here!"
        anOperator = #+ ifTrue: [ ^self class new
                at: #gXval put: sv + ov;                "Operate on the constituent values"
                at: #gXerr put: (se*se + oe*oe) sqrt;   "Calculate gaussian errors"
                yourself ].

        anOperator = #- ifTrue: [ ^self class new
                at: #gXval put: sv - ov;                "Operate on the constituent values"
                at: #gXerr put: (se*se + oe*oe) sqrt;   "Calculate gaussian errors"
                yourself ].
nil
!
!
((Smalltalk at: #GXuser ifAbsent: [Smalltalk at: #GXuser put: 				  'Procedure "user" has only been FORWARD declared, it must also have a PROCEDURE definition' ] ) 				  isKindOf: Function class) ifTrue: 				[ 'Warning: FORWARD statement for a previously defined procedure "user" - no action taken' prinl ]!
((Smalltalk at: #GXinstrument ifAbsent: [Smalltalk at: #GXinstrument put: 				  'Procedure "instrument" has only been FORWARD declared, it must also have a PROCEDURE definition' ] ) 				  isKindOf: Function class) ifTrue: 				[ 'Warning: FORWARD statement for a previously defined procedure "instrument" - no action taken' prinl ]!
((Smalltalk at: #GXsample ifAbsent: [Smalltalk at: #GXsample put: 				  'Procedure "sample" has only been FORWARD declared, it must also have a PROCEDURE definition' ] ) 				  isKindOf: Function class) ifTrue: 				[ 'Warning: FORWARD statement for a previously defined procedure "sample" - no action taken' prinl ]!
((Smalltalk at: #GXhistory ifAbsent: [Smalltalk at: #GXhistory put: 				  'Procedure "history" has only been FORWARD declared, it must also have a PROCEDURE definition' ] ) 				  isKindOf: Function class) ifTrue: 				[ 'Warning: FORWARD statement for a previously defined procedure "history" - no action taken' prinl ]!
((Smalltalk at: #GXdata ifAbsent: [Smalltalk at: #GXdata put: 				  'Procedure "data" has only been FORWARD declared, it must also have a PROCEDURE definition' ] ) 				  isKindOf: Function class) ifTrue: 				[ 'Warning: FORWARD statement for a previously defined procedure "data" - no action taken' prinl ]!
((Smalltalk at: #GXget ifAbsent: [Smalltalk at: #GXget put: 				  'Procedure "get" has only been FORWARD declared, it must also have a PROCEDURE definition' ] ) 				  isKindOf: Function class) ifTrue: 				[ 'Warning: FORWARD statement for a previously defined procedure "get" - no action taken' prinl ]!
((Smalltalk at: #GXtoggle ifAbsent: [Smalltalk at: #GXtoggle put: 				  'Procedure "toggle" has only been FORWARD declared, it must also have a PROCEDURE definition' ] ) 				  isKindOf: Function class) ifTrue: 				[ 'Warning: FORWARD statement for a previously defined procedure "toggle" - no action taken' prinl ]!
"Procedure class definition:
 ==========================="
((Smalltalk at: #GXexperiment ifAbsent: [ nil ]) isKindOf: Function class) ifTrue: [ ('Warning: Overriding previously defined procedure ', 'GXexperiment' gtype) prinl ].
Smalltalk at: #GXexperiment put: nil!
Function subclass: 'GXexperiment'
       instanceVariableNames: ''
       classVariableNames: ''
       poolDictionaries: ''
       category: 'Genie Procedure'
!

"HEADER
 ------"

"parameters
 ----------"

GXexperiment

addInstVarName: #gXsource;
compile:
'gXsource: aVar
	aVar notNil ifTrue:[ gXsource _ aVar ]';
compile:
'p1: aVar
	aVar notNil ifTrue:[ gXsource _ aVar ]';

addInstVarName: #gXspecno;
compile:
'gXspecno: aVar
	aVar notNil ifTrue:[ gXspecno _ aVar ]';
compile:
'p2: aVar
	aVar notNil ifTrue:[ gXspecno _ aVar ]'!

"result
 ------"

GXexperiment

addInstVarName: #gXres
!

"Global variables
 ----------------"

GXexperiment

addPoolVar: #gXUUtmp;

addPoolVar: #gXUcurrentUinputUsource
!

!GXexperiment methodsFor: 'initializing'!

initVars

!!

"BODY
 ----"

!GXexperiment methodsFor: 'running'!

evaluate
gXUUtmp _ (GXget run line: 0 proc: self method: true; p1:(gXspecno); p2:(gXsource); condEvaluate) deepCopy.
gXres _ ((GXExperiment) new) deepCopy.
(gXres isKindOf: GXArray) | (gXres isKindOf: GXWorkspace)
    ifTrue:  [ ((gXres) at:#gXdatasource put: (gXUcurrentUinputUsource) deepCopy) ]
    ifFalse: [ self error: 'Assignment to array or workspace failed, check that the destination is a structured type'].
(gXres isKindOf: GXArray) | (gXres isKindOf: GXWorkspace)
    ifTrue:  [ ((gXres) at:#gXname put: ((gXUUtmp) at:#gXtitle) deepCopy) ]
    ifFalse: [ self error: 'Assignment to array or workspace failed, check that the destination is a structured type'].
(gXres isKindOf: GXArray) | (gXres isKindOf: GXWorkspace)
    ifTrue:  [ ((gXres) at:#gXduration put: ((gXUUtmp) at:#gXrunUduration) deepCopy) ]
    ifFalse: [ self error: 'Assignment to array or workspace failed, check that the destination is a structured type'].
(gXres isKindOf: GXArray) | (gXres isKindOf: GXWorkspace)
    ifTrue:  [ ((gXres) at:#gXstart put: ((gXUUtmp) at:#gXtime) deepCopy) ]
    ifFalse: [ self error: 'Assignment to array or workspace failed, check that the destination is a structured type'].
GXtoggle run line: 0 proc: self method: false; gXinfo; condEvaluate.
(gXres isKindOf: GXArray) | (gXres isKindOf: GXWorkspace)
    ifTrue:  [ ((gXres) at:#gXsample put: (GXsample run line: 0 proc: self method: true; condEvaluate) deepCopy) ]
    ifFalse: [ self error: 'Assignment to array or workspace failed, check that the destination is a structured type'].
(gXres isKindOf: GXArray) | (gXres isKindOf: GXWorkspace)
    ifTrue:  [ ((gXres) at:#gXinstrument put: (GXinstrument run line: 0 proc: self method: true; condEvaluate) deepCopy) ]
    ifFalse: [ self error: 'Assignment to array or workspace failed, check that the destination is a structured type'].
(gXres isKindOf: GXArray) | (gXres isKindOf: GXWorkspace)
    ifTrue:  [ ((gXres) at:#gXuser put: (GXuser run line: 0 proc: self method: true; condEvaluate) deepCopy) ]
    ifFalse: [ self error: 'Assignment to array or workspace failed, check that the destination is a structured type'].
(gXres isKindOf: GXArray) | (gXres isKindOf: GXWorkspace)
    ifTrue:  [ ((gXres) at:#gXhistory put: (GXhistory run line: 0 proc: self method: true; condEvaluate) deepCopy) ]
    ifFalse: [ self error: 'Assignment to array or workspace failed, check that the destination is a structured type'].
(gXres isKindOf: GXArray) | (gXres isKindOf: GXWorkspace)
    ifTrue:  [ ((gXres) at:#gXdata put: (GXdata run line: 0 proc: self method: true; condEvaluate) deepCopy) ]
    ifFalse: [ self error: 'Assignment to array or workspace failed, check that the destination is a structured type'].
GXtoggle run line: 0 proc: self method: false; gXinfo; condEvaluate.
self up.
^gXres
!!
nil! "(fix for smalltalk bug, this line is ignored if there is a variable declaration error)"

"End of procedure"

"Procedure class definition:
 ==========================="
((Smalltalk at: #GXdata ifAbsent: [ nil ]) isKindOf: Function class) ifTrue: [ ('Warning: Overriding previously defined procedure ', 'GXdata' gtype) prinl ].
Smalltalk at: #GXdata put: nil!
Function subclass: 'GXdata'
       instanceVariableNames: ''
       classVariableNames: ''
       poolDictionaries: ''
       category: 'Genie Procedure'
!

"HEADER
 ------"

"result
 ------"

GXdata

addInstVarName: #gXdata
!

"Global variables
 ----------------"

GXdata

addPoolVar: #gXUUtmp
!

!GXdata methodsFor: 'initializing'!

initVars

!!

"BODY
 ----"

!GXdata methodsFor: 'running'!

evaluate
gXdata _ ((GXScan) new) deepCopy.
(gXdata isKindOf: GXArray) | (gXdata isKindOf: GXWorkspace)
    ifTrue:  [ ((gXdata) at:#gXv put: ((GXDetectorintensity) new) deepCopy) ]
    ifFalse: [ self error: 'Assignment to array or workspace failed, check that the destination is a structured type'].
(gXdata isKindOf: GXArray) | (gXdata isKindOf: GXWorkspace)
    ifTrue:  [ ((gXdata) at:#gXx put: ((GXMajorspace) new) deepCopy) ]
    ifFalse: [ self error: 'Assignment to array or workspace failed, check that the destination is a structured type'].
(gXdata isKindOf: GXArray) | (gXdata isKindOf: GXWorkspace)
    ifTrue:  [ (((gXdata) at:#gXv) at:#gXval put: ((gXUUtmp) at:#gXy) deepCopy) ]
    ifFalse: [ self error: 'Assignment to array or workspace failed, check that the destination is a structured type'].
(gXdata isKindOf: GXArray) | (gXdata isKindOf: GXWorkspace)
    ifTrue:  [ (((gXdata) at:#gXv) at:#gXerr put: ((gXUUtmp) at:#gXe) deepCopy) ]
    ifFalse: [ self error: 'Assignment to array or workspace failed, check that the destination is a structured type'].
(gXdata isKindOf: GXArray) | (gXdata isKindOf: GXWorkspace)
    ifTrue:  [ (((gXdata) at:#gXv) at:#gXlsamp put: ((gXUUtmp) at:#gXl1) deepCopy) ]
    ifFalse: [ self error: 'Assignment to array or workspace failed, check that the destination is a structured type'].
(gXdata isKindOf: GXArray) | (gXdata isKindOf: GXWorkspace)
    ifTrue:  [ (((gXdata) at:#gXv) at:#gXldet put: ((gXUUtmp) at:#gXl2) deepCopy) ]
    ifFalse: [ self error: 'Assignment to array or workspace failed, check that the destination is a structured type'].
(gXdata isKindOf: GXArray) | (gXdata isKindOf: GXWorkspace)
    ifTrue:  [ (((gXdata) at:#gXv) at:#gXtwotheta put: ((gXUUtmp) at:#gXtwotheta) deepCopy) ]
    ifFalse: [ self error: 'Assignment to array or workspace failed, check that the destination is a structured type'].
(gXdata isKindOf: GXArray) | (gXdata isKindOf: GXWorkspace)
    ifTrue:  [ (((gXdata) at:#gXv) at:#gXunit put: ((GXUnit) new) deepCopy) ]
    ifFalse: [ self error: 'Assignment to array or workspace failed, check that the destination is a structured type'].
(gXdata isKindOf: GXArray) | (gXdata isKindOf: GXWorkspace)
    ifTrue:  [ ((((gXdata) at:#gXv) at:#gXunit) at:#gXlabel put: ((gXUUtmp) at:#gXylabel) deepCopy) ]
    ifFalse: [ self error: 'Assignment to array or workspace failed, check that the destination is a structured type'].
(gXdata isKindOf: GXArray) | (gXdata isKindOf: GXWorkspace)
    ifTrue:  [ (((gXdata) at:#gXx) at:#gXval put: (((gXUUtmp) at:#gXx) - ((gXUUtmp) at:#gXdelta)) deepCopy) ]
    ifFalse: [ self error: 'Assignment to array or workspace failed, check that the destination is a structured type'].
(gXdata isKindOf: GXArray) | (gXdata isKindOf: GXWorkspace)
    ifTrue:  [ (((gXdata) at:#gXx) at:#gXunit put: ((GXUnit) new) deepCopy) ]
    ifFalse: [ self error: 'Assignment to array or workspace failed, check that the destination is a structured type'].
(gXdata isKindOf: GXArray) | (gXdata isKindOf: GXWorkspace)
    ifTrue:  [ ((((gXdata) at:#gXx) at:#gXunit) at:#gXlabel put: ((gXUUtmp) at:#gXxlabel) deepCopy) ]
    ifFalse: [ self error: 'Assignment to array or workspace failed, check that the destination is a structured type'].
self up.
^gXdata
!!
nil! "(fix for smalltalk bug, this line is ignored if there is a variable declaration error)"

"End of procedure"

"Procedure class definition:
 ==========================="
((Smalltalk at: #GXsample ifAbsent: [ nil ]) isKindOf: Function class) ifTrue: [ ('Warning: Overriding previously defined procedure ', 'GXsample' gtype) prinl ].
Smalltalk at: #GXsample put: nil!
Function subclass: 'GXsample'
       instanceVariableNames: ''
       classVariableNames: ''
       poolDictionaries: ''
       category: 'Genie Procedure'
!

"HEADER
 ------"

"result
 ------"

GXsample

addInstVarName: #gXsample
!

"Global variables
 ----------------"

GXsample

addPoolVar: #gXUUtmp
!

!GXsample methodsFor: 'initializing'!

initVars

!!

"BODY
 ----"

!GXsample methodsFor: 'running'!

evaluate
gXsample _ ((GXSample) new) deepCopy.
(gXsample isKindOf: GXArray) | (gXsample isKindOf: GXWorkspace)
    ifTrue:  [ ((gXsample) at:#gXname put: 'A sample' deepCopy) ]
    ifFalse: [ self error: 'Assignment to array or workspace failed, check that the destination is a structured type'].
(gXsample isKindOf: GXArray) | (gXsample isKindOf: GXWorkspace)
    ifTrue:  [ ((gXsample) at:#gXmeasurements put: ((GXWorkspacearray) new:1) deepCopy) ]
    ifFalse: [ self error: 'Assignment to array or workspace failed, check that the destination is a structured type'].
(gXsample isKindOf: GXArray) | (gXsample isKindOf: GXWorkspace)
    ifTrue:  [ (((gXsample) at:#gXmeasurements) at:((Array new:1) at: 1 put: 1; yourself) put: ((GXEnergy) new) deepCopy) ]
    ifFalse: [ self error: 'Assignment to array or workspace failed, check that the destination is a structured type'].
(gXsample isKindOf: GXArray) | (gXsample isKindOf: GXWorkspace)
    ifTrue:  [ ((((gXsample) at:#gXmeasurements) at:((Array new:1) at: 1 put: 1; yourself)) at:#gXe put: ((gXUUtmp) at:#gXefixed) deepCopy) ]
    ifFalse: [ self error: 'Assignment to array or workspace failed, check that the destination is a structured type'].
(gXsample isKindOf: GXArray) | (gXsample isKindOf: GXWorkspace)
    ifTrue:  [ ((((gXsample) at:#gXmeasurements) at:((Array new:1) at: 1 put: 1; yourself)) at:#gXunits put: ((gXUUtmp) at:#gXemode) deepCopy) ]
    ifFalse: [ self error: 'Assignment to array or workspace failed, check that the destination is a structured type'].
self up.
^gXsample
!!
nil! "(fix for smalltalk bug, this line is ignored if there is a variable declaration error)"

"End of procedure"

"Procedure class definition:
 ==========================="
((Smalltalk at: #GXinstrument ifAbsent: [ nil ]) isKindOf: Function class) ifTrue: [ ('Warning: Overriding previously defined procedure ', 'GXinstrument' gtype) prinl ].
Smalltalk at: #GXinstrument put: nil!
Function subclass: 'GXinstrument'
       instanceVariableNames: ''
       classVariableNames: ''
       poolDictionaries: ''
       category: 'Genie Procedure'
!

"HEADER
 ------"

"result
 ------"

GXinstrument

addInstVarName: #gXinstrument
!

"Global variables
 ----------------"

GXinstrument

addPoolVar: #gXUUtmp
!

!GXinstrument methodsFor: 'initializing'!

initVars

!!

"BODY
 ----"

!GXinstrument methodsFor: 'running'!

evaluate
gXinstrument _ ((GXInstrument) new) deepCopy.
(gXinstrument isKindOf: GXArray) | (gXinstrument isKindOf: GXWorkspace)
    ifTrue:  [ ((gXinstrument) at:#gXname put: ((gXUUtmp) at:#gXinstUname) deepCopy) ]
    ifFalse: [ self error: 'Assignment to array or workspace failed, check that the destination is a structured type'].
(gXinstrument isKindOf: GXArray) | (gXinstrument isKindOf: GXWorkspace)
    ifTrue:  [ ((gXinstrument) at:#gXmonitor put: ((GXMonitor) new) deepCopy) ]
    ifFalse: [ self error: 'Assignment to array or workspace failed, check that the destination is a structured type'].
(gXinstrument isKindOf: GXArray) | (gXinstrument isKindOf: GXWorkspace)
    ifTrue:  [ (((gXinstrument) at:#gXmonitor) at:#gXefficiency put: (GXget run line: 0 proc: self method: true; p1:'MONP'; condEvaluate) deepCopy) ]
    ifFalse: [ self error: 'Assignment to array or workspace failed, check that the destination is a structured type'].
(gXinstrument isKindOf: GXArray) | (gXinstrument isKindOf: GXWorkspace)
    ifTrue:  [ (((gXinstrument) at:#gXmonitor) at:#gXmdmapping put: (GXget run line: 0 proc: self method: true; p1:'MDET'; condEvaluate) deepCopy) ]
    ifFalse: [ self error: 'Assignment to array or workspace failed, check that the destination is a structured type'].
(gXinstrument isKindOf: GXArray) | (gXinstrument isKindOf: GXWorkspace)
    ifTrue:  [ ((gXinstrument) at:#gXdetector put: ((GXDetector) new) deepCopy) ]
    ifFalse: [ self error: 'Assignment to array or workspace failed, check that the destination is a structured type'].
(gXinstrument isKindOf: GXArray) | (gXinstrument isKindOf: GXWorkspace)
    ifTrue:  [ (((gXinstrument) at:#gXdetector) at:#gXdelta put: (GXget run line: 0 proc: self method: true; p1:'DELT'; condEvaluate) deepCopy) ]
    ifFalse: [ self error: 'Assignment to array or workspace failed, check that the destination is a structured type'].
(gXinstrument isKindOf: GXArray) | (gXinstrument isKindOf: GXWorkspace)
    ifTrue:  [ (((gXinstrument) at:#gXdetector) at:#gXdsmapping put: (GXget run line: 0 proc: self method: true; p1:'SPEC'; condEvaluate) deepCopy) ]
    ifFalse: [ self error: 'Assignment to array or workspace failed, check that the destination is a structured type'].
self up.
^gXinstrument
!!
nil! "(fix for smalltalk bug, this line is ignored if there is a variable declaration error)"

"End of procedure"

"Procedure class definition:
 ==========================="
((Smalltalk at: #GXuser ifAbsent: [ nil ]) isKindOf: Function class) ifTrue: [ ('Warning: Overriding previously defined procedure ', 'GXuser' gtype) prinl ].
Smalltalk at: #GXuser put: nil!
Function subclass: 'GXuser'
       instanceVariableNames: ''
       classVariableNames: ''
       poolDictionaries: ''
       category: 'Genie Procedure'
!

"HEADER
 ------"

"result
 ------"

GXuser

addInstVarName: #gXuser
!

"Global variables
 ----------------"

GXuser

addPoolVar: #gXUUtmp
!

!GXuser methodsFor: 'initializing'!

initVars

!!

"BODY
 ----"

!GXuser methodsFor: 'running'!

evaluate
gXuser _ ((GXUser) new) deepCopy.
(gXuser isKindOf: GXArray) | (gXuser isKindOf: GXWorkspace)
    ifTrue:  [ ((gXuser) at:#gXname put: ((gXUUtmp) at:#gXuserUname) deepCopy) ]
    ifFalse: [ self error: 'Assignment to array or workspace failed, check that the destination is a structured type'].
self up.
^gXuser
!!
nil! "(fix for smalltalk bug, this line is ignored if there is a variable declaration error)"

"End of procedure"

"Procedure class definition:
 ==========================="
((Smalltalk at: #GXhistory ifAbsent: [ nil ]) isKindOf: Function class) ifTrue: [ ('Warning: Overriding previously defined procedure ', 'GXhistory' gtype) prinl ].
Smalltalk at: #GXhistory put: nil!
Function subclass: 'GXhistory'
       instanceVariableNames: ''
       classVariableNames: ''
       poolDictionaries: ''
       category: 'Genie Procedure'
!

"HEADER
 ------"

"result
 ------"

GXhistory

addInstVarName: #gXhistory
!

"Global variables
 ----------------"

GXhistory

addPoolVar: #gXUUtmp
!

!GXhistory methodsFor: 'initializing'!

initVars

!!

"BODY
 ----"

!GXhistory methodsFor: 'running'!

evaluate
gXhistory _ ((GXHistory) new) deepCopy.
(gXhistory isKindOf: GXArray) | (gXhistory isKindOf: GXWorkspace)
    ifTrue:  [ ((gXhistory) at:#gXdisplay put: ((GXStructure) new) deepCopy) ]
    ifFalse: [ self error: 'Assignment to array or workspace failed, check that the destination is a structured type'].
(gXhistory isKindOf: GXArray) | (gXhistory isKindOf: GXWorkspace)
    ifTrue:  [ (((gXhistory) at:#gXdisplay) at:#gXbingrouping put: ((gXUUtmp) at:#gXbinUgrouping) deepCopy) ]
    ifFalse: [ self error: 'Assignment to array or workspace failed, check that the destination is a structured type'].
(gXhistory isKindOf: GXArray) | (gXhistory isKindOf: GXWorkspace)
    ifTrue:  [ ((gXhistory) at:#gXtemp put: ((GXStructure) new) deepCopy) ]
    ifFalse: [ self error: 'Assignment to array or workspace failed, check that the destination is a structured type'].
(gXhistory isKindOf: GXArray) | (gXhistory isKindOf: GXWorkspace)
    ifTrue:  [ (((gXhistory) at:#gXtemp) at:#gXspectrum put: ((gXUUtmp) at:#gXspecUno) deepCopy) ]
    ifFalse: [ self error: 'Assignment to array or workspace failed, check that the destination is a structured type'].
self up.
^gXhistory
!!
nil! "(fix for smalltalk bug, this line is ignored if there is a variable declaration error)"

"End of procedure"

"Procedure class definition:
 ==========================="
((Smalltalk at: #GXasUold ifAbsent: [ nil ]) isKindOf: Function class) ifTrue: [ ('Warning: Overriding previously defined procedure ', 'GXasUold' gtype) prinl ].
Smalltalk at: #GXasUold put: nil!
Function subclass: 'GXasUold'
       instanceVariableNames: ''
       classVariableNames: ''
       poolDictionaries: ''
       category: 'Genie Procedure'
!

"HEADER
 ------"

"parameters
 ----------"

GXasUold

addInstVarName: #gXexp;
compile:
'gXexp: aVar
	aVar notNil ifTrue:[gXexp _ self typeCheck: aVar shouldBe: GXExperiment paramName: ''gXexp'']';
compile:
'p1: aVar
	aVar notNil ifTrue:[gXexp _ self typeCheck: aVar shouldBe: GXExperiment paramName: ''gXexp'']'!

"result
 ------"

GXasUold

addInstVarName: #gXres
!

!GXasUold methodsFor: 'initializing'!

initVars

!!

"BODY
 ----"

!GXasUold methodsFor: 'running'!

evaluate
gXres _ ((GXWorkspace) new) deepCopy.
((gXres) at:(('gXl1') asSymbol) put:((((gXexp) at:#gXdata) at:#gXv) at:#gXlsamp)).
((gXres) at:(('gXtitle') asSymbol) put:((gXexp) at:#gXname)).
((gXres) at:(('gXl2') asSymbol) put:((((gXexp) at:#gXdata) at:#gXv) at:#gXldet)).
((gXres) at:(('gXinstUname') asSymbol) put:(((gXexp) at:#gXinstrument) at:#gXname)).
((gXres) at:(('gXrunUno') asSymbol) put:(gXU)).
((gXres) at:(('gXdelta') asSymbol) put:0.000000000000000e000).
((gXres) at:(('gXuserUname') asSymbol) put:(((gXexp) at:#gXuser) at:#gXname)).
((gXres) at:(('gXspecUno') asSymbol) put:((((gXexp) at:#gXhistory) at:#gXtemp) at:#gXspectrum)).
((gXres) at:(('gXtime') asSymbol) put:((gXexp) at:#gXstart)).
((gXres) at:(('gXefixed') asSymbol) put:(((((gXexp) at:#gXsample) at:#gXmeasurements) at:((Array new:1) at: 1 put: 1; yourself)) at:#gXe)).
((gXres) at:(('gXbinUgrouping') asSymbol) put:((((gXexp) at:#gXhistory) at:#gXdisplay) at:#gXbingrouping)).
((gXres) at:(('gXrunUduration') asSymbol) put:((gXexp) at:#gXduration)).
((gXres) at:(('gXx') asSymbol) put:((((gXexp) at:#gXdata) at:#gXx) at:#gXval)).
((gXres) at:(('gXy') asSymbol) put:((((gXexp) at:#gXdata) at:#gXv) at:#gXval)).
((gXres) at:(('gXe') asSymbol) put:((((gXexp) at:#gXdata) at:#gXv) at:#gXerr)).
((gXres) at:(('gXntc') asSymbol) put:(((gXres) at:#gXy) size)).
((gXres) at:(('gXemode') asSymbol) put:(((((gXexp) at:#gXsample) at:#gXmeasurements) at:((Array new:1) at: 1 put: 1; yourself)) at:#gXunits)).
((gXres) at:(('gXylabel') asSymbol) put:(((((gXexp) at:#gXdata) at:#gXv) at:#gXunit) at:#gXlabel)).
((gXres) at:(('gXxlabel') asSymbol) put:(((((gXexp) at:#gXdata) at:#gXx) at:#gXunit) at:#gXlabel)).
((gXres) at:(('gXfile') asSymbol) put:((gXexp) at:#gXdatasource)).
((gXres) at:(('gXymax') asSymbol) put:(GXmax run line: 0 proc: self method: true; p1:((gXres) at:#gXy); condEvaluate)).
((gXres) at:(('gXfi') asSymbol) put:0.000000000000000e000).
((gXres) at:(('gXtwotheta') asSymbol) put:((((gXexp) at:#gXdata) at:#gXv) at:#gXtwotheta)).
self up.
^gXres
!!
nil! "(fix for smalltalk bug, this line is ignored if there is a variable declaration error)"

"End of procedure"

"Procedure class definition:
 ==========================="
((Smalltalk at: #GXasUnew ifAbsent: [ nil ]) isKindOf: Function class) ifTrue: [ ('Warning: Overriding previously defined procedure ', 'GXasUnew' gtype) prinl ].
Smalltalk at: #GXasUnew put: nil!
Function subclass: 'GXasUnew'
       instanceVariableNames: ''
       classVariableNames: ''
       poolDictionaries: ''
       category: 'Genie Procedure'
!

"HEADER
 ------"

"parameters
 ----------"

GXasUnew

addInstVarName: #gXw;
compile:
'gXw: aVar
	aVar notNil ifTrue:[gXw _ self typeCheck: aVar shouldBe: GXWorkspace paramName: ''gXw'']';
compile:
'p1: aVar
	aVar notNil ifTrue:[gXw _ self typeCheck: aVar shouldBe: GXWorkspace paramName: ''gXw'']';

addInstVarName: #gXexp;
compile:
'gXexp: aVar
	aVar notNil ifTrue:[gXexp _ self typeCheck: aVar shouldBe: GXExperiment paramName: ''gXexp'']';
compile:
'p2: aVar
	aVar notNil ifTrue:[gXexp _ self typeCheck: aVar shouldBe: GXExperiment paramName: ''gXexp'']'!

"result
 ------"

GXasUnew

addInstVarName: #gXexp
!

!GXasUnew methodsFor: 'initializing'!

initVars

!!

"BODY
 ----"

!GXasUnew methodsFor: 'running'!

evaluate
(GXdefined run line: 0 proc: self method: true; p1:(gXexp); condEvaluate) not ifTrue: [ 
GXprinten run line: 0 proc: self method: false; p1:'Usage:  As_new(workspace, experiment) - both parameters required'; condEvaluate ].
((((gXexp) at:#gXdata) at:#gXv) at:(('gXlsamp') asSymbol) put:((gXw) at:#gXl1)).
((gXexp) at:(('gXname') asSymbol) put:((gXw) at:#gXtitle)).
((((gXexp) at:#gXdata) at:#gXv) at:(('gXldet') asSymbol) put:((gXw) at:#gXl2)).
(((gXexp) at:#gXinstrument) at:(('gXname') asSymbol) put:((gXw) at:#gXinstUname)).
(((gXexp) at:#gXuser) at:(('gXname') asSymbol) put:((gXw) at:#gXuserUname)).
((((gXexp) at:#gXhistory) at:#gXtemp) at:(('gXspectrum') asSymbol) put:((gXw) at:#gXspecUno)).
((gXexp) at:(('gXstart') asSymbol) put:((gXw) at:#gXtime)).
(((((gXexp) at:#gXsample) at:#gXmeasurements) at:((Array new:1) at: 1 put: 1; yourself)) at:(('gXe') asSymbol) put:((gXw) at:#gXefixed)).
((((gXexp) at:#gXhistory) at:#gXdisplay) at:(('gXbingrouping') asSymbol) put:((gXw) at:#gXbinUgrouping)).
((gXexp) at:(('gXduration') asSymbol) put:((gXw) at:#gXrunUduration)).
((((gXexp) at:#gXdata) at:#gXx) at:(('gXval') asSymbol) put:((gXw) at:#gXx)).
((((gXexp) at:#gXdata) at:#gXv) at:(('gXval') asSymbol) put:((gXw) at:#gXy)).
((((gXexp) at:#gXdata) at:#gXv) at:(('gXerr') asSymbol) put:((gXw) at:#gXe)).
(((((gXexp) at:#gXsample) at:#gXmeasurements) at:((Array new:1) at: 1 put: 1; yourself)) at:(('gXunits') asSymbol) put:((gXw) at:#gXemode)).
(((((gXexp) at:#gXdata) at:#gXv) at:#gXunit) at:(('gXlabel') asSymbol) put:((gXw) at:#gXylabel)).
(((((gXexp) at:#gXdata) at:#gXx) at:#gXunit) at:(('gXlabel') asSymbol) put:((gXw) at:#gXxlabel)).
((gXexp) at:(('gXdatasource') asSymbol) put:((gXw) at:#gXfile)).
((((gXexp) at:#gXdata) at:#gXv) at:(('gXtwotheta') asSymbol) put:((gXw) at:#gXtwotheta)).
self up.
^gXexp
!!
nil! "(fix for smalltalk bug, this line is ignored if there is a variable declaration error)"

"End of procedure"


!GXWorkspace methodsFor: 'converting'!

asOld
        ^GXasUold run line: 0 proc: self method: true; p1:(self); condEvaluate.
!        

asNew: aTemplate
        ^GXasUnew run line: 0 proc: self method: true; p1:(self); p2:(aTemplate); condEvaluate
!
!
"Procedure class definition:
 ==========================="
((Smalltalk at: #GXworkspaceUcoerce ifAbsent: [ nil ]) isKindOf: Function class) ifTrue: [ ('Warning: Overriding previously defined procedure ', 'GXworkspaceUcoerce' gtype) prinl ].
Smalltalk at: #GXworkspaceUcoerce put: nil!
Function subclass: 'GXworkspaceUcoerce'
       instanceVariableNames: ''
       classVariableNames: ''
       poolDictionaries: ''
       category: 'Genie Procedure'
!

"HEADER
 ------"

"parameters
 ----------"

GXworkspaceUcoerce

addInstVarName: #gXsomething;
compile:
'gXsomething: aVar
	aVar notNil ifTrue:[ gXsomething _ aVar ]';
compile:
'p1: aVar
	aVar notNil ifTrue:[ gXsomething _ aVar ]'!

"result
 ------"

GXworkspaceUcoerce

addInstVarName: #gXwres
!

"Local variables
 ---------------"

GXworkspaceUcoerce

addInstVarName: #gXself
!

!GXworkspaceUcoerce methodsFor: 'initializing'!

initVars

!!

"BODY
 ----"

!GXworkspaceUcoerce methodsFor: 'running'!

evaluate
 gXself _ line at: 1 .
gXwres _ (GXfields run line: 0 proc: self method: true; condEvaluate) deepCopy.
((GXisUa run line: 0 proc: self method: true; p1:(gXsomething); p2:'workspace'; condEvaluate) | (GXisUa run line: 0 proc: self method: true; p1:(gXsomething); p2:'workspacearray'; condEvaluate)) ifTrue: [ 
GXprinten run line: 0 proc: self method: false; p1:'Coercion failure - workspace(array) -> workspace'; condEvaluate ] ifFalse: [ 
gXwres _ (gXself) deepCopy.
(gXwres isKindOf: GXArray) | (gXwres isKindOf: GXWorkspace)
    ifTrue:  [ ((gXwres) at:#gXylabel put: '(dimensionless)' deepCopy) ]
    ifFalse: [ self error: 'Assignment to array or workspace failed, check that the destination is a structured type'].
GXfill run line: 0 proc: self method: false; p1:((gXwres) at:#gXe); p2:0.000000000000000e000; condEvaluate.
(gXwres isKindOf: GXArray) | (gXwres isKindOf: GXWorkspace)
    ifTrue:  [ ((gXwres) at:#gXy put: ((GXfill run line: 0 proc: self method: true; p1:((gXwres) at:#gXy); p2:0.000000000000000e000; condEvaluate) + (gXsomething)) deepCopy) ]
    ifFalse: [ self error: 'Assignment to array or workspace failed, check that the destination is a structured type'] ] .
self up.
^gXwres
!!
nil! "(fix for smalltalk bug, this line is ignored if there is a variable declaration error)"

"End of procedure"

"Procedure class definition:
 ==========================="
((Smalltalk at: #GXworkspaceUgreaterUthan ifAbsent: [ nil ]) isKindOf: Function class) ifTrue: [ ('Warning: Overriding previously defined procedure ', 'GXworkspaceUgreaterUthan' gtype) prinl ].
Smalltalk at: #GXworkspaceUgreaterUthan put: nil!
Function subclass: 'GXworkspaceUgreaterUthan'
       instanceVariableNames: ''
       classVariableNames: ''
       poolDictionaries: ''
       category: 'Genie Procedure'
!

"HEADER
 ------"

"parameters
 ----------"

GXworkspaceUgreaterUthan

addInstVarName: #gXw1;
compile:
'gXw1: aVar
	aVar notNil ifTrue:[gXw1 _ self typeCheck: aVar shouldBe: GXWorkspace paramName: ''gXw1'']';
compile:
'p1: aVar
	aVar notNil ifTrue:[gXw1 _ self typeCheck: aVar shouldBe: GXWorkspace paramName: ''gXw1'']';

addInstVarName: #gXw2;
compile:
'gXw2: aVar
	aVar notNil ifTrue:[gXw2 _ self typeCheck: aVar shouldBe: GXWorkspace paramName: ''gXw2'']';
compile:
'p2: aVar
	aVar notNil ifTrue:[gXw2 _ self typeCheck: aVar shouldBe: GXWorkspace paramName: ''gXw2'']'!

"result
 ------"

GXworkspaceUgreaterUthan

addInstVarName: #gXwres
!

!GXworkspaceUgreaterUthan methodsFor: 'initializing'!

initVars

!!

"BODY
 ----"

!GXworkspaceUgreaterUthan methodsFor: 'running'!

evaluate
GXprintin run line: 0 proc: self method: false; p1:'w1 > w2 is not currently defined for workspaces'; condEvaluate.
self up.
^gXwres
!!
nil! "(fix for smalltalk bug, this line is ignored if there is a variable declaration error)"

"End of procedure"

"Procedure class definition:
 ==========================="
((Smalltalk at: #GXworkspaceUlessUthan ifAbsent: [ nil ]) isKindOf: Function class) ifTrue: [ ('Warning: Overriding previously defined procedure ', 'GXworkspaceUlessUthan' gtype) prinl ].
Smalltalk at: #GXworkspaceUlessUthan put: nil!
Function subclass: 'GXworkspaceUlessUthan'
       instanceVariableNames: ''
       classVariableNames: ''
       poolDictionaries: ''
       category: 'Genie Procedure'
!

"HEADER
 ------"

"parameters
 ----------"

GXworkspaceUlessUthan

addInstVarName: #gXw1;
compile:
'gXw1: aVar
	aVar notNil ifTrue:[gXw1 _ self typeCheck: aVar shouldBe: GXWorkspace paramName: ''gXw1'']';
compile:
'p1: aVar
	aVar notNil ifTrue:[gXw1 _ self typeCheck: aVar shouldBe: GXWorkspace paramName: ''gXw1'']';

addInstVarName: #gXw2;
compile:
'gXw2: aVar
	aVar notNil ifTrue:[gXw2 _ self typeCheck: aVar shouldBe: GXWorkspace paramName: ''gXw2'']';
compile:
'p2: aVar
	aVar notNil ifTrue:[gXw2 _ self typeCheck: aVar shouldBe: GXWorkspace paramName: ''gXw2'']'!

"result
 ------"

GXworkspaceUlessUthan

addInstVarName: #gXwres
!

!GXworkspaceUlessUthan methodsFor: 'initializing'!

initVars

!!

"BODY
 ----"

!GXworkspaceUlessUthan methodsFor: 'running'!

evaluate
GXprintin run line: 0 proc: self method: false; p1:'w1 < w2 is not currently defined for workspaces'; condEvaluate.
self up.
^gXwres
!!
nil! "(fix for smalltalk bug, this line is ignored if there is a variable declaration error)"

"End of procedure"

"Procedure class definition:
 ==========================="
((Smalltalk at: #GXworkspaceUequal ifAbsent: [ nil ]) isKindOf: Function class) ifTrue: [ ('Warning: Overriding previously defined procedure ', 'GXworkspaceUequal' gtype) prinl ].
Smalltalk at: #GXworkspaceUequal put: nil!
Function subclass: 'GXworkspaceUequal'
       instanceVariableNames: ''
       classVariableNames: ''
       poolDictionaries: ''
       category: 'Genie Procedure'
!

"HEADER
 ------"

"parameters
 ----------"

GXworkspaceUequal

addInstVarName: #gXw1;
compile:
'gXw1: aVar
	aVar notNil ifTrue:[gXw1 _ self typeCheck: aVar shouldBe: GXWorkspace paramName: ''gXw1'']';
compile:
'p1: aVar
	aVar notNil ifTrue:[gXw1 _ self typeCheck: aVar shouldBe: GXWorkspace paramName: ''gXw1'']';

addInstVarName: #gXw2;
compile:
'gXw2: aVar
	aVar notNil ifTrue:[gXw2 _ self typeCheck: aVar shouldBe: GXWorkspace paramName: ''gXw2'']';
compile:
'p2: aVar
	aVar notNil ifTrue:[gXw2 _ self typeCheck: aVar shouldBe: GXWorkspace paramName: ''gXw2'']'!

"result
 ------"

GXworkspaceUequal

addInstVarName: #gXwres
!

!GXworkspaceUequal methodsFor: 'initializing'!

initVars

!!

"BODY
 ----"

!GXworkspaceUequal methodsFor: 'running'!

evaluate
gXwres _ ((((((gXw1) at:#gXx) = ((gXw2) at:#gXx))) & ((((gXw1) at:#gXxlabel) = ((gXw2) at:#gXxlabel)))) & ((((gXw1) at:#gXy) = ((gXw2) at:#gXy)))) deepCopy.
self up.
^gXwres
!!
nil! "(fix for smalltalk bug, this line is ignored if there is a variable declaration error)"

"End of procedure"

"Procedure class definition:
 ==========================="
((Smalltalk at: #GXworkspaceUgreaterUthanUorUequal ifAbsent: [ nil ]) isKindOf: Function class) ifTrue: [ ('Warning: Overriding previously defined procedure ', 'GXworkspaceUgreaterUthanUorUequal' gtype) prinl ].
Smalltalk at: #GXworkspaceUgreaterUthanUorUequal put: nil!
Function subclass: 'GXworkspaceUgreaterUthanUorUequal'
       instanceVariableNames: ''
       classVariableNames: ''
       poolDictionaries: ''
       category: 'Genie Procedure'
!

"HEADER
 ------"

"parameters
 ----------"

GXworkspaceUgreaterUthanUorUequal

addInstVarName: #gXw1;
compile:
'gXw1: aVar
	aVar notNil ifTrue:[gXw1 _ self typeCheck: aVar shouldBe: GXWorkspace paramName: ''gXw1'']';
compile:
'p1: aVar
	aVar notNil ifTrue:[gXw1 _ self typeCheck: aVar shouldBe: GXWorkspace paramName: ''gXw1'']';

addInstVarName: #gXw2;
compile:
'gXw2: aVar
	aVar notNil ifTrue:[gXw2 _ self typeCheck: aVar shouldBe: GXWorkspace paramName: ''gXw2'']';
compile:
'p2: aVar
	aVar notNil ifTrue:[gXw2 _ self typeCheck: aVar shouldBe: GXWorkspace paramName: ''gXw2'']'!

"result
 ------"

GXworkspaceUgreaterUthanUorUequal

addInstVarName: #gXwres
!

!GXworkspaceUgreaterUthanUorUequal methodsFor: 'initializing'!

initVars

!!

"BODY
 ----"

!GXworkspaceUgreaterUthanUorUequal methodsFor: 'running'!

evaluate
GXprintin run line: 0 proc: self method: false; p1:'w1 >= w2 is not currently defined for workspaces'; condEvaluate.
self up.
^gXwres
!!
nil! "(fix for smalltalk bug, this line is ignored if there is a variable declaration error)"

"End of procedure"

"Procedure class definition:
 ==========================="
((Smalltalk at: #GXworkspaceUlessUthanUorUequal ifAbsent: [ nil ]) isKindOf: Function class) ifTrue: [ ('Warning: Overriding previously defined procedure ', 'GXworkspaceUlessUthanUorUequal' gtype) prinl ].
Smalltalk at: #GXworkspaceUlessUthanUorUequal put: nil!
Function subclass: 'GXworkspaceUlessUthanUorUequal'
       instanceVariableNames: ''
       classVariableNames: ''
       poolDictionaries: ''
       category: 'Genie Procedure'
!

"HEADER
 ------"

"parameters
 ----------"

GXworkspaceUlessUthanUorUequal

addInstVarName: #gXw1;
compile:
'gXw1: aVar
	aVar notNil ifTrue:[gXw1 _ self typeCheck: aVar shouldBe: GXWorkspace paramName: ''gXw1'']';
compile:
'p1: aVar
	aVar notNil ifTrue:[gXw1 _ self typeCheck: aVar shouldBe: GXWorkspace paramName: ''gXw1'']';

addInstVarName: #gXw2;
compile:
'gXw2: aVar
	aVar notNil ifTrue:[gXw2 _ self typeCheck: aVar shouldBe: GXWorkspace paramName: ''gXw2'']';
compile:
'p2: aVar
	aVar notNil ifTrue:[gXw2 _ self typeCheck: aVar shouldBe: GXWorkspace paramName: ''gXw2'']'!

"result
 ------"

GXworkspaceUlessUthanUorUequal

addInstVarName: #gXwres
!

!GXworkspaceUlessUthanUorUequal methodsFor: 'initializing'!

initVars

!!

"BODY
 ----"

!GXworkspaceUlessUthanUorUequal methodsFor: 'running'!

evaluate
GXprintin run line: 0 proc: self method: false; p1:'w1 <= w2 is not currently defined for workspaces'; condEvaluate.
self up.
^gXwres
!!
nil! "(fix for smalltalk bug, this line is ignored if there is a variable declaration error)"

"End of procedure"

"Procedure class definition:
 ==========================="
((Smalltalk at: #GXworkspaceUnotUequal ifAbsent: [ nil ]) isKindOf: Function class) ifTrue: [ ('Warning: Overriding previously defined procedure ', 'GXworkspaceUnotUequal' gtype) prinl ].
Smalltalk at: #GXworkspaceUnotUequal put: nil!
Function subclass: 'GXworkspaceUnotUequal'
       instanceVariableNames: ''
       classVariableNames: ''
       poolDictionaries: ''
       category: 'Genie Procedure'
!

"HEADER
 ------"

"parameters
 ----------"

GXworkspaceUnotUequal

addInstVarName: #gXw1;
compile:
'gXw1: aVar
	aVar notNil ifTrue:[gXw1 _ self typeCheck: aVar shouldBe: GXWorkspace paramName: ''gXw1'']';
compile:
'p1: aVar
	aVar notNil ifTrue:[gXw1 _ self typeCheck: aVar shouldBe: GXWorkspace paramName: ''gXw1'']';

addInstVarName: #gXw2;
compile:
'gXw2: aVar
	aVar notNil ifTrue:[gXw2 _ self typeCheck: aVar shouldBe: GXWorkspace paramName: ''gXw2'']';
compile:
'p2: aVar
	aVar notNil ifTrue:[gXw2 _ self typeCheck: aVar shouldBe: GXWorkspace paramName: ''gXw2'']'!

"result
 ------"

GXworkspaceUnotUequal

addInstVarName: #gXwres
!

!GXworkspaceUnotUequal methodsFor: 'initializing'!

initVars

!!

"BODY
 ----"

!GXworkspaceUnotUequal methodsFor: 'running'!

evaluate
gXwres _ (((((gXw1) at:#gXx) ~= ((gXw2) at:#gXx))) | ((((gXw1) at:#gXy) ~= ((gXw2) at:#gXy)))) deepCopy.
self up.
^gXwres
!!
nil! "(fix for smalltalk bug, this line is ignored if there is a variable declaration error)"

"End of procedure"

"Procedure class definition:
 ==========================="
((Smalltalk at: #GXworkspaceUappend ifAbsent: [ nil ]) isKindOf: Function class) ifTrue: [ ('Warning: Overriding previously defined procedure ', 'GXworkspaceUappend' gtype) prinl ].
Smalltalk at: #GXworkspaceUappend put: nil!
Function subclass: 'GXworkspaceUappend'
       instanceVariableNames: ''
       classVariableNames: ''
       poolDictionaries: ''
       category: 'Genie Procedure'
!

"HEADER
 ------"

"parameters
 ----------"

GXworkspaceUappend

addInstVarName: #gXw1;
compile:
'gXw1: aVar
	aVar notNil ifTrue:[gXw1 _ self typeCheck: aVar shouldBe: GXWorkspace paramName: ''gXw1'']';
compile:
'p1: aVar
	aVar notNil ifTrue:[gXw1 _ self typeCheck: aVar shouldBe: GXWorkspace paramName: ''gXw1'']';

addInstVarName: #gXw2;
compile:
'gXw2: aVar
	aVar notNil ifTrue:[gXw2 _ self typeCheck: aVar shouldBe: GXWorkspace paramName: ''gXw2'']';
compile:
'p2: aVar
	aVar notNil ifTrue:[gXw2 _ self typeCheck: aVar shouldBe: GXWorkspace paramName: ''gXw2'']'!

"result
 ------"

GXworkspaceUappend

addInstVarName: #gXwres
!

"Local variables
 ---------------"

GXworkspaceUappend

addInstVarName: #gXlenx1;

addInstVarName: #gXlenx2;

addInstVarName: #gXleny1;

addInstVarName: #gXleny2
!

!GXworkspaceUappend methodsFor: 'initializing'!

initVars

!!

"BODY
 ----"

!GXworkspaceUappend methodsFor: 'running'!

evaluate
gXlenx1 _ (GXlength run line: 0 proc: self method: true; p1:((gXw1) at:#gXx); condEvaluate) deepCopy.
gXlenx2 _ (GXlength run line: 0 proc: self method: true; p1:((gXw2) at:#gXx); condEvaluate) deepCopy.
gXleny1 _ (GXlength run line: 0 proc: self method: true; p1:((gXw1) at:#gXy); condEvaluate) deepCopy.
gXleny2 _ (GXlength run line: 0 proc: self method: true; p1:((gXw2) at:#gXy); condEvaluate) deepCopy.
(((GXmax run line: 0 proc: self method: true; p1:((gXw1) at:#gXx); condEvaluate) = (GXmin run line: 0 proc: self method: true; p1:((gXw2) at:#gXx); condEvaluate))) ifTrue: [ 
gXwres _ (gXw1) deepCopy.
(gXwres isKindOf: GXArray) | (gXwres isKindOf: GXWorkspace)
    ifTrue:  [ ((gXwres) at:#gXx put: (((gXw1) at:#gXx) , (((gXw2) at:#gXx) at:((Array new:1) at: 1 put: (2 to: (gXlenx2)); yourself))) deepCopy) ]
    ifFalse: [ self error: 'Assignment to array or workspace failed, check that the destination is a structured type'].
(gXwres isKindOf: GXArray) | (gXwres isKindOf: GXWorkspace)
    ifTrue:  [ ((gXwres) at:#gXy put: (((gXw1) at:#gXy) , ((gXw2) at:#gXy)) deepCopy) ]
    ifFalse: [ self error: 'Assignment to array or workspace failed, check that the destination is a structured type'].
(gXwres isKindOf: GXArray) | (gXwres isKindOf: GXWorkspace)
    ifTrue:  [ ((gXwres) at:#gXe put: (((gXw1) at:#gXe) , ((gXw2) at:#gXe)) deepCopy) ]
    ifFalse: [ self error: 'Assignment to array or workspace failed, check that the destination is a structured type'] ] ifFalse: [ ((((gXleny1) = (gXlenx1))) | (((gXleny2) = (gXlenx2)))) ifTrue: [ 
GXprinten run line: 0 proc: self method: false; p1:'Error - both workspaces must be histograms'; condEvaluate ] ifFalse: [ 
GXprinten run line: 0 proc: self method: false; p1:'Error - Start and end X values must match'; condEvaluate ] ].
self up.
^gXwres
!!
nil! "(fix for smalltalk bug, this line is ignored if there is a variable declaration error)"

"End of procedure"

"Procedure class definition:
 ==========================="
((Smalltalk at: #GXworkspaceUor ifAbsent: [ nil ]) isKindOf: Function class) ifTrue: [ ('Warning: Overriding previously defined procedure ', 'GXworkspaceUor' gtype) prinl ].
Smalltalk at: #GXworkspaceUor put: nil!
Function subclass: 'GXworkspaceUor'
       instanceVariableNames: ''
       classVariableNames: ''
       poolDictionaries: ''
       category: 'Genie Procedure'
!

"HEADER
 ------"

"parameters
 ----------"

GXworkspaceUor

addInstVarName: #gXw1;
compile:
'gXw1: aVar
	aVar notNil ifTrue:[gXw1 _ self typeCheck: aVar shouldBe: GXWorkspace paramName: ''gXw1'']';
compile:
'p1: aVar
	aVar notNil ifTrue:[gXw1 _ self typeCheck: aVar shouldBe: GXWorkspace paramName: ''gXw1'']';

addInstVarName: #gXw2;
compile:
'gXw2: aVar
	aVar notNil ifTrue:[gXw2 _ self typeCheck: aVar shouldBe: GXWorkspace paramName: ''gXw2'']';
compile:
'p2: aVar
	aVar notNil ifTrue:[gXw2 _ self typeCheck: aVar shouldBe: GXWorkspace paramName: ''gXw2'']'!

"result
 ------"

GXworkspaceUor

addInstVarName: #gXwres
!

!GXworkspaceUor methodsFor: 'initializing'!

initVars

!!

"BODY
 ----"

!GXworkspaceUor methodsFor: 'running'!

evaluate
GXprintin run line: 0 proc: self method: false; p1:'Operation OR is not defined for workspaces'; condEvaluate.
self up.
^gXwres
!!
nil! "(fix for smalltalk bug, this line is ignored if there is a variable declaration error)"

"End of procedure"

"Procedure class definition:
 ==========================="
((Smalltalk at: #GXworkspaceUadd ifAbsent: [ nil ]) isKindOf: Function class) ifTrue: [ ('Warning: Overriding previously defined procedure ', 'GXworkspaceUadd' gtype) prinl ].
Smalltalk at: #GXworkspaceUadd put: nil!
Function subclass: 'GXworkspaceUadd'
       instanceVariableNames: ''
       classVariableNames: ''
       poolDictionaries: ''
       category: 'Genie Procedure'
!

"HEADER
 ------"

"parameters
 ----------"

GXworkspaceUadd

addInstVarName: #gXw1;
compile:
'gXw1: aVar
	aVar notNil ifTrue:[gXw1 _ self typeCheck: aVar shouldBe: GXWorkspace paramName: ''gXw1'']';
compile:
'p1: aVar
	aVar notNil ifTrue:[gXw1 _ self typeCheck: aVar shouldBe: GXWorkspace paramName: ''gXw1'']';

addInstVarName: #gXw2;
compile:
'gXw2: aVar
	aVar notNil ifTrue:[gXw2 _ self typeCheck: aVar shouldBe: GXWorkspace paramName: ''gXw2'']';
compile:
'p2: aVar
	aVar notNil ifTrue:[gXw2 _ self typeCheck: aVar shouldBe: GXWorkspace paramName: ''gXw2'']'!

"result
 ------"

GXworkspaceUadd

addInstVarName: #gXwres
!

!GXworkspaceUadd methodsFor: 'initializing'!

initVars

!!

"BODY
 ----"

!GXworkspaceUadd methodsFor: 'running'!

evaluate
(((((gXw1) at:#gXx) = ((gXw2) at:#gXx))) & ((((gXw1) at:#gXxlabel) = ((gXw2) at:#gXxlabel)))) ifTrue: [ 
gXwres _ (gXw1) deepCopy.
(gXwres isKindOf: GXArray) | (gXwres isKindOf: GXWorkspace)
    ifTrue:  [ ((gXwres) at:#gXy put: (((gXw1) at:#gXy) + ((gXw2) at:#gXy)) deepCopy) ]
    ifFalse: [ self error: 'Assignment to array or workspace failed, check that the destination is a structured type'].
(gXwres isKindOf: GXArray) | (gXwres isKindOf: GXWorkspace)
    ifTrue:  [ ((gXwres) at:#gXe put: (GXsqrt run line: 0 proc: self method: true; p1:((((gXw1) at:#gXe) * ((gXw1) at:#gXe)) + (((gXw2) at:#gXe) * ((gXw2) at:#gXe))); condEvaluate) deepCopy) ]
    ifFalse: [ self error: 'Assignment to array or workspace failed, check that the destination is a structured type'] ] ifFalse: [ 
GXprinten run line: 0 proc: self method: false; p1:'Workspaces are not compatible - please re-bin'; condEvaluate ] .
self up.
^gXwres
!!
nil! "(fix for smalltalk bug, this line is ignored if there is a variable declaration error)"

"End of procedure"

"Procedure class definition:
 ==========================="
((Smalltalk at: #GXworkspaceUsubtract ifAbsent: [ nil ]) isKindOf: Function class) ifTrue: [ ('Warning: Overriding previously defined procedure ', 'GXworkspaceUsubtract' gtype) prinl ].
Smalltalk at: #GXworkspaceUsubtract put: nil!
Function subclass: 'GXworkspaceUsubtract'
       instanceVariableNames: ''
       classVariableNames: ''
       poolDictionaries: ''
       category: 'Genie Procedure'
!

"HEADER
 ------"

"parameters
 ----------"

GXworkspaceUsubtract

addInstVarName: #gXw1;
compile:
'gXw1: aVar
	aVar notNil ifTrue:[gXw1 _ self typeCheck: aVar shouldBe: GXWorkspace paramName: ''gXw1'']';
compile:
'p1: aVar
	aVar notNil ifTrue:[gXw1 _ self typeCheck: aVar shouldBe: GXWorkspace paramName: ''gXw1'']';

addInstVarName: #gXw2;
compile:
'gXw2: aVar
	aVar notNil ifTrue:[gXw2 _ self typeCheck: aVar shouldBe: GXWorkspace paramName: ''gXw2'']';
compile:
'p2: aVar
	aVar notNil ifTrue:[gXw2 _ self typeCheck: aVar shouldBe: GXWorkspace paramName: ''gXw2'']'!

"result
 ------"

GXworkspaceUsubtract

addInstVarName: #gXwres
!

!GXworkspaceUsubtract methodsFor: 'initializing'!

initVars

!!

"BODY
 ----"

!GXworkspaceUsubtract methodsFor: 'running'!

evaluate
(((((gXw1) at:#gXx) = ((gXw2) at:#gXx))) & ((((gXw1) at:#gXxlabel) = ((gXw2) at:#gXxlabel)))) ifTrue: [ 
gXwres _ (gXw1) deepCopy.
(gXwres isKindOf: GXArray) | (gXwres isKindOf: GXWorkspace)
    ifTrue:  [ ((gXwres) at:#gXy put: (((gXw1) at:#gXy) - ((gXw2) at:#gXy)) deepCopy) ]
    ifFalse: [ self error: 'Assignment to array or workspace failed, check that the destination is a structured type'].
(gXwres isKindOf: GXArray) | (gXwres isKindOf: GXWorkspace)
    ifTrue:  [ ((gXwres) at:#gXe put: (GXsqrt run line: 0 proc: self method: true; p1:((((gXw1) at:#gXe) * ((gXw1) at:#gXe)) + (((gXw2) at:#gXe) * ((gXw2) at:#gXe))); condEvaluate) deepCopy) ]
    ifFalse: [ self error: 'Assignment to array or workspace failed, check that the destination is a structured type'] ] ifFalse: [ 
GXprintin run line: 0 proc: self method: false; p1:'Workspaces are not compatible - please re-bin'; condEvaluate ] .
self up.
^gXwres
!!
nil! "(fix for smalltalk bug, this line is ignored if there is a variable declaration error)"

"End of procedure"

"Procedure class definition:
 ==========================="
((Smalltalk at: #GXworkspaceUand ifAbsent: [ nil ]) isKindOf: Function class) ifTrue: [ ('Warning: Overriding previously defined procedure ', 'GXworkspaceUand' gtype) prinl ].
Smalltalk at: #GXworkspaceUand put: nil!
Function subclass: 'GXworkspaceUand'
       instanceVariableNames: ''
       classVariableNames: ''
       poolDictionaries: ''
       category: 'Genie Procedure'
!

"HEADER
 ------"

"parameters
 ----------"

GXworkspaceUand

addInstVarName: #gXw1;
compile:
'gXw1: aVar
	aVar notNil ifTrue:[gXw1 _ self typeCheck: aVar shouldBe: GXWorkspace paramName: ''gXw1'']';
compile:
'p1: aVar
	aVar notNil ifTrue:[gXw1 _ self typeCheck: aVar shouldBe: GXWorkspace paramName: ''gXw1'']';

addInstVarName: #gXw2;
compile:
'gXw2: aVar
	aVar notNil ifTrue:[gXw2 _ self typeCheck: aVar shouldBe: GXWorkspace paramName: ''gXw2'']';
compile:
'p2: aVar
	aVar notNil ifTrue:[gXw2 _ self typeCheck: aVar shouldBe: GXWorkspace paramName: ''gXw2'']'!

"result
 ------"

GXworkspaceUand

addInstVarName: #gXwres
!

!GXworkspaceUand methodsFor: 'initializing'!

initVars

!!

"BODY
 ----"

!GXworkspaceUand methodsFor: 'running'!

evaluate
GXprintin run line: 0 proc: self method: false; p1:'Operation AND is not defined for workspaces'; condEvaluate.
self up.
^gXwres
!!
nil! "(fix for smalltalk bug, this line is ignored if there is a variable declaration error)"

"End of procedure"

"Procedure class definition:
 ==========================="
((Smalltalk at: #GXworkspaceUmultiply ifAbsent: [ nil ]) isKindOf: Function class) ifTrue: [ ('Warning: Overriding previously defined procedure ', 'GXworkspaceUmultiply' gtype) prinl ].
Smalltalk at: #GXworkspaceUmultiply put: nil!
Function subclass: 'GXworkspaceUmultiply'
       instanceVariableNames: ''
       classVariableNames: ''
       poolDictionaries: ''
       category: 'Genie Procedure'
!

"HEADER
 ------"

"parameters
 ----------"

GXworkspaceUmultiply

addInstVarName: #gXw1;
compile:
'gXw1: aVar
	aVar notNil ifTrue:[gXw1 _ self typeCheck: aVar shouldBe: GXWorkspace paramName: ''gXw1'']';
compile:
'p1: aVar
	aVar notNil ifTrue:[gXw1 _ self typeCheck: aVar shouldBe: GXWorkspace paramName: ''gXw1'']';

addInstVarName: #gXw2;
compile:
'gXw2: aVar
	aVar notNil ifTrue:[gXw2 _ self typeCheck: aVar shouldBe: GXWorkspace paramName: ''gXw2'']';
compile:
'p2: aVar
	aVar notNil ifTrue:[gXw2 _ self typeCheck: aVar shouldBe: GXWorkspace paramName: ''gXw2'']'!

"result
 ------"

GXworkspaceUmultiply

addInstVarName: #gXwres
!

!GXworkspaceUmultiply methodsFor: 'initializing'!

initVars

!!

"BODY
 ----"

!GXworkspaceUmultiply methodsFor: 'running'!

evaluate
(((((gXw1) at:#gXx) = ((gXw2) at:#gXx))) & ((((gXw1) at:#gXxlabel) = ((gXw2) at:#gXxlabel)))) ifTrue: [ 
gXwres _ (gXw1) deepCopy.
(gXwres isKindOf: GXArray) | (gXwres isKindOf: GXWorkspace)
    ifTrue:  [ ((gXwres) at:#gXy put: (((gXw1) at:#gXy) * ((gXw2) at:#gXy)) deepCopy) ]
    ifFalse: [ self error: 'Assignment to array or workspace failed, check that the destination is a structured type'].
(gXwres isKindOf: GXArray) | (gXwres isKindOf: GXWorkspace)
    ifTrue:  [ ((gXwres) at:#gXe put: (GXsqrt run line: 0 proc: self method: true; p1:((((((gXw1) at:#gXy) * ((gXw2) at:#gXe))) raisedTo: 2) + (((((gXw2) at:#gXy) * ((gXw1) at:#gXe))) raisedTo: 2)); condEvaluate) deepCopy) ]
    ifFalse: [ self error: 'Assignment to array or workspace failed, check that the destination is a structured type'] ] ifFalse: [ 
GXprintin run line: 0 proc: self method: false; p1:'Workspaces are not compatible - please re-bin'; condEvaluate ] .
self up.
^gXwres
!!
nil! "(fix for smalltalk bug, this line is ignored if there is a variable declaration error)"

"End of procedure"

"Procedure class definition:
 ==========================="
((Smalltalk at: #GXworkspaceUdivide ifAbsent: [ nil ]) isKindOf: Function class) ifTrue: [ ('Warning: Overriding previously defined procedure ', 'GXworkspaceUdivide' gtype) prinl ].
Smalltalk at: #GXworkspaceUdivide put: nil!
Function subclass: 'GXworkspaceUdivide'
       instanceVariableNames: ''
       classVariableNames: ''
       poolDictionaries: ''
       category: 'Genie Procedure'
!

"HEADER
 ------"

"parameters
 ----------"

GXworkspaceUdivide

addInstVarName: #gXw1;
compile:
'gXw1: aVar
	aVar notNil ifTrue:[gXw1 _ self typeCheck: aVar shouldBe: GXWorkspace paramName: ''gXw1'']';
compile:
'p1: aVar
	aVar notNil ifTrue:[gXw1 _ self typeCheck: aVar shouldBe: GXWorkspace paramName: ''gXw1'']';

addInstVarName: #gXw2;
compile:
'gXw2: aVar
	aVar notNil ifTrue:[gXw2 _ self typeCheck: aVar shouldBe: GXWorkspace paramName: ''gXw2'']';
compile:
'p2: aVar
	aVar notNil ifTrue:[gXw2 _ self typeCheck: aVar shouldBe: GXWorkspace paramName: ''gXw2'']'!

"result
 ------"

GXworkspaceUdivide

addInstVarName: #gXwres
!

!GXworkspaceUdivide methodsFor: 'initializing'!

initVars

!!

"BODY
 ----"

!GXworkspaceUdivide methodsFor: 'running'!

evaluate
(((((gXw1) at:#gXx) = ((gXw2) at:#gXx))) & ((((gXw1) at:#gXxlabel) = ((gXw2) at:#gXxlabel)))) ifTrue: [ 
gXwres _ (gXw1) deepCopy.
(gXwres isKindOf: GXArray) | (gXwres isKindOf: GXWorkspace)
    ifTrue:  [ ((gXwres) at:#gXy put: (((gXw1) at:#gXy) / ((gXw2) at:#gXy)) deepCopy) ]
    ifFalse: [ self error: 'Assignment to array or workspace failed, check that the destination is a structured type'].
(gXwres isKindOf: GXArray) | (gXwres isKindOf: GXWorkspace)
    ifTrue:  [ ((gXwres) at:#gXe put: (GXsqrt run line: 0 proc: self method: true; p1:(((gXwres) at:#gXy) * (((((((gXw1) at:#gXe) / ((gXw1) at:#gXy))) raisedTo: 2) + (((((gXw2) at:#gXe) / ((gXw2) at:#gXy))) raisedTo: 2)))); condEvaluate) deepCopy) ]
    ifFalse: [ self error: 'Assignment to array or workspace failed, check that the destination is a structured type'].
(((gXw2) at:#gXylabel) ~= '(dimensionless)') ifTrue: [ 
(gXwres isKindOf: GXArray) | (gXwres isKindOf: GXWorkspace)
    ifTrue:  [ ((gXwres) at:#gXylabel put: '(dimensionless)' deepCopy) ]
    ifFalse: [ self error: 'Assignment to array or workspace failed, check that the destination is a structured type'] ] ] ifFalse: [ 
GXprintin run line: 0 proc: self method: false; p1:'Workspaces are not compatible - please re-bin'; condEvaluate ] .
self up.
^gXwres
!!
nil! "(fix for smalltalk bug, this line is ignored if there is a variable declaration error)"

"End of procedure"

"Procedure class definition:
 ==========================="
((Smalltalk at: #GXworkspaceUmodulo ifAbsent: [ nil ]) isKindOf: Function class) ifTrue: [ ('Warning: Overriding previously defined procedure ', 'GXworkspaceUmodulo' gtype) prinl ].
Smalltalk at: #GXworkspaceUmodulo put: nil!
Function subclass: 'GXworkspaceUmodulo'
       instanceVariableNames: ''
       classVariableNames: ''
       poolDictionaries: ''
       category: 'Genie Procedure'
!

"HEADER
 ------"

"parameters
 ----------"

GXworkspaceUmodulo

addInstVarName: #gXw1;
compile:
'gXw1: aVar
	aVar notNil ifTrue:[gXw1 _ self typeCheck: aVar shouldBe: GXWorkspace paramName: ''gXw1'']';
compile:
'p1: aVar
	aVar notNil ifTrue:[gXw1 _ self typeCheck: aVar shouldBe: GXWorkspace paramName: ''gXw1'']';

addInstVarName: #gXw2;
compile:
'gXw2: aVar
	aVar notNil ifTrue:[gXw2 _ self typeCheck: aVar shouldBe: GXWorkspace paramName: ''gXw2'']';
compile:
'p2: aVar
	aVar notNil ifTrue:[gXw2 _ self typeCheck: aVar shouldBe: GXWorkspace paramName: ''gXw2'']'!

"result
 ------"

GXworkspaceUmodulo

addInstVarName: #gXwres
!

!GXworkspaceUmodulo methodsFor: 'initializing'!

initVars

!!

"BODY
 ----"

!GXworkspaceUmodulo methodsFor: 'running'!

evaluate
(((((gXw1) at:#gXx) = ((gXw2) at:#gXx))) & ((((gXw1) at:#gXxlabel) = ((gXw2) at:#gXxlabel)))) ifTrue: [ 
gXwres _ (gXw1) deepCopy.
(gXwres isKindOf: GXArray) | (gXwres isKindOf: GXWorkspace)
    ifTrue:  [ ((gXwres) at:#gXy put: (((gXw1) at:#gXy) rem: ((gXw2) at:#gXy)) deepCopy) ]
    ifFalse: [ self error: 'Assignment to array or workspace failed, check that the destination is a structured type'].
(gXwres isKindOf: GXArray) | (gXwres isKindOf: GXWorkspace)
    ifTrue:  [ ((gXwres) at:#gXe put: (GXsqrt run line: 0 proc: self method: true; p1:(((gXwres) at:#gXy) * (((((((gXw1) at:#gXe) / ((gXw1) at:#gXy))) raisedTo: 2) + (((((gXw2) at:#gXe) / ((gXw2) at:#gXy))) raisedTo: 2)))); condEvaluate) deepCopy) ]
    ifFalse: [ self error: 'Assignment to array or workspace failed, check that the destination is a structured type'].
(gXwres isKindOf: GXArray) | (gXwres isKindOf: GXWorkspace)
    ifTrue:  [ ((gXwres) at:#gXylabel put: '(dimensionless)' deepCopy) ]
    ifFalse: [ self error: 'Assignment to array or workspace failed, check that the destination is a structured type'] ] ifFalse: [ 
GXprintin run line: 0 proc: self method: false; p1:'Workspaces are not compatible - please re-bin'; condEvaluate ] .
self up.
^gXwres
!!
nil! "(fix for smalltalk bug, this line is ignored if there is a variable declaration error)"

"End of procedure"

"Procedure class definition:
 ==========================="
((Smalltalk at: #GXworkspaceUraisedUto ifAbsent: [ nil ]) isKindOf: Function class) ifTrue: [ ('Warning: Overriding previously defined procedure ', 'GXworkspaceUraisedUto' gtype) prinl ].
Smalltalk at: #GXworkspaceUraisedUto put: nil!
Function subclass: 'GXworkspaceUraisedUto'
       instanceVariableNames: ''
       classVariableNames: ''
       poolDictionaries: ''
       category: 'Genie Procedure'
!

"HEADER
 ------"

"parameters
 ----------"

GXworkspaceUraisedUto

addInstVarName: #gXw1;
compile:
'gXw1: aVar
	aVar notNil ifTrue:[gXw1 _ self typeCheck: aVar shouldBe: GXWorkspace paramName: ''gXw1'']';
compile:
'p1: aVar
	aVar notNil ifTrue:[gXw1 _ self typeCheck: aVar shouldBe: GXWorkspace paramName: ''gXw1'']';

addInstVarName: #gXw2;
compile:
'gXw2: aVar
	aVar notNil ifTrue:[gXw2 _ self typeCheck: aVar shouldBe: GXWorkspace paramName: ''gXw2'']';
compile:
'p2: aVar
	aVar notNil ifTrue:[gXw2 _ self typeCheck: aVar shouldBe: GXWorkspace paramName: ''gXw2'']'!

"result
 ------"

GXworkspaceUraisedUto

addInstVarName: #gXwres
!

!GXworkspaceUraisedUto methodsFor: 'initializing'!

initVars

!!

"BODY
 ----"

!GXworkspaceUraisedUto methodsFor: 'running'!

evaluate
(((((gXw1) at:#gXx) = ((gXw2) at:#gXx))) & ((((gXw1) at:#gXxlabel) = ((gXw2) at:#gXxlabel)))) ifTrue: [ 
gXwres _ (gXw1) deepCopy.
(gXwres isKindOf: GXArray) | (gXwres isKindOf: GXWorkspace)
    ifTrue:  [ ((gXwres) at:#gXy put: (((gXw1) at:#gXy) raisedTo: ((gXw2) at:#gXy)) deepCopy) ]
    ifFalse: [ self error: 'Assignment to array or workspace failed, check that the destination is a structured type'].
(gXwres isKindOf: GXArray) | (gXwres isKindOf: GXWorkspace)
    ifTrue:  [ ((gXwres) at:#gXe put: (GXsqrt run line: 0 proc: self method: true; p1:(((((GXln run line: 0 proc: self method: true; p1:((gXw1) at:#gXy); condEvaluate) * (((gXw2) at:#gXe)))) raisedTo: 2) + (((((((gXw2) at:#gXy) / ((gXw1) at:#gXy))) * ((gXw1) at:#gXe))) raisedTo: 2)); condEvaluate) deepCopy) ]
    ifFalse: [ self error: 'Assignment to array or workspace failed, check that the destination is a structured type'] ] ifFalse: [ 
GXprintin run line: 0 proc: self method: false; p1:'Workspaces are not compatible - please re-bin'; condEvaluate ] .
self up.
^gXwres
!!
nil! "(fix for smalltalk bug, this line is ignored if there is a variable declaration error)"

"End of procedure"

"Procedure class definition:
 ==========================="
((Smalltalk at: #GXworkspaceUsin ifAbsent: [ nil ]) isKindOf: Function class) ifTrue: [ ('Warning: Overriding previously defined procedure ', 'GXworkspaceUsin' gtype) prinl ].
Smalltalk at: #GXworkspaceUsin put: nil!
Function subclass: 'GXworkspaceUsin'
       instanceVariableNames: ''
       classVariableNames: ''
       poolDictionaries: ''
       category: 'Genie Procedure'
!

"HEADER
 ------"

"parameters
 ----------"

GXworkspaceUsin

addInstVarName: #gXw1;
compile:
'gXw1: aVar
	aVar notNil ifTrue:[gXw1 _ self typeCheck: aVar shouldBe: GXWorkspace paramName: ''gXw1'']';
compile:
'p1: aVar
	aVar notNil ifTrue:[gXw1 _ self typeCheck: aVar shouldBe: GXWorkspace paramName: ''gXw1'']'!

"result
 ------"

GXworkspaceUsin

addInstVarName: #gXwres
!

!GXworkspaceUsin methodsFor: 'initializing'!

initVars

!!

"BODY
 ----"

!GXworkspaceUsin methodsFor: 'running'!

evaluate
gXwres _ (gXw1) deepCopy.
(gXwres isKindOf: GXArray) | (gXwres isKindOf: GXWorkspace)
    ifTrue:  [ ((gXwres) at:#gXy put: (GXsin run line: 0 proc: self method: true; p1:((gXw1) at:#gXy); condEvaluate) deepCopy) ]
    ifFalse: [ self error: 'Assignment to array or workspace failed, check that the destination is a structured type'].
(gXwres isKindOf: GXArray) | (gXwres isKindOf: GXWorkspace)
    ifTrue:  [ ((gXwres) at:#gXe put: (GXabs run line: 0 proc: self method: true; p1:((GXcos run line: 0 proc: self method: true; p1:((gXw1) at:#gXy); condEvaluate) * ((gXw1) at:#gXe)); condEvaluate) deepCopy) ]
    ifFalse: [ self error: 'Assignment to array or workspace failed, check that the destination is a structured type'].
self up.
^gXwres
!!
nil! "(fix for smalltalk bug, this line is ignored if there is a variable declaration error)"

"End of procedure"

"Procedure class definition:
 ==========================="
((Smalltalk at: #GXworkspaceUcos ifAbsent: [ nil ]) isKindOf: Function class) ifTrue: [ ('Warning: Overriding previously defined procedure ', 'GXworkspaceUcos' gtype) prinl ].
Smalltalk at: #GXworkspaceUcos put: nil!
Function subclass: 'GXworkspaceUcos'
       instanceVariableNames: ''
       classVariableNames: ''
       poolDictionaries: ''
       category: 'Genie Procedure'
!

"HEADER
 ------"

"parameters
 ----------"

GXworkspaceUcos

addInstVarName: #gXw1;
compile:
'gXw1: aVar
	aVar notNil ifTrue:[gXw1 _ self typeCheck: aVar shouldBe: GXWorkspace paramName: ''gXw1'']';
compile:
'p1: aVar
	aVar notNil ifTrue:[gXw1 _ self typeCheck: aVar shouldBe: GXWorkspace paramName: ''gXw1'']'!

"result
 ------"

GXworkspaceUcos

addInstVarName: #gXwres
!

!GXworkspaceUcos methodsFor: 'initializing'!

initVars

!!

"BODY
 ----"

!GXworkspaceUcos methodsFor: 'running'!

evaluate
gXwres _ (gXw1) deepCopy.
(gXwres isKindOf: GXArray) | (gXwres isKindOf: GXWorkspace)
    ifTrue:  [ ((gXwres) at:#gXy put: (GXcos run line: 0 proc: self method: true; p1:((gXw1) at:#gXy); condEvaluate) deepCopy) ]
    ifFalse: [ self error: 'Assignment to array or workspace failed, check that the destination is a structured type'].
(gXwres isKindOf: GXArray) | (gXwres isKindOf: GXWorkspace)
    ifTrue:  [ ((gXwres) at:#gXe put: (GXabs run line: 0 proc: self method: true; p1:((GXsin run line: 0 proc: self method: true; p1:((gXw1) at:#gXy); condEvaluate) * ((gXw1) at:#gXe)); condEvaluate) deepCopy) ]
    ifFalse: [ self error: 'Assignment to array or workspace failed, check that the destination is a structured type'].
self up.
^gXwres
!!
nil! "(fix for smalltalk bug, this line is ignored if there is a variable declaration error)"

"End of procedure"

"Procedure class definition:
 ==========================="
((Smalltalk at: #GXworkspaceUtan ifAbsent: [ nil ]) isKindOf: Function class) ifTrue: [ ('Warning: Overriding previously defined procedure ', 'GXworkspaceUtan' gtype) prinl ].
Smalltalk at: #GXworkspaceUtan put: nil!
Function subclass: 'GXworkspaceUtan'
       instanceVariableNames: ''
       classVariableNames: ''
       poolDictionaries: ''
       category: 'Genie Procedure'
!

"HEADER
 ------"

"parameters
 ----------"

GXworkspaceUtan

addInstVarName: #gXw1;
compile:
'gXw1: aVar
	aVar notNil ifTrue:[gXw1 _ self typeCheck: aVar shouldBe: GXWorkspace paramName: ''gXw1'']';
compile:
'p1: aVar
	aVar notNil ifTrue:[gXw1 _ self typeCheck: aVar shouldBe: GXWorkspace paramName: ''gXw1'']'!

"result
 ------"

GXworkspaceUtan

addInstVarName: #gXwres
!

!GXworkspaceUtan methodsFor: 'initializing'!

initVars

!!

"BODY
 ----"

!GXworkspaceUtan methodsFor: 'running'!

evaluate
gXwres _ (gXw1) deepCopy.
(gXwres isKindOf: GXArray) | (gXwres isKindOf: GXWorkspace)
    ifTrue:  [ ((gXwres) at:#gXy put: (GXtan run line: 0 proc: self method: true; p1:((gXw1) at:#gXy); condEvaluate) deepCopy) ]
    ifFalse: [ self error: 'Assignment to array or workspace failed, check that the destination is a structured type'].
(gXwres isKindOf: GXArray) | (gXwres isKindOf: GXWorkspace)
    ifTrue:  [ ((gXwres) at:#gXe put: (((gXw1) at:#gXe) / (((GXcos run line: 0 proc: self method: true; p1:((gXw1) at:#gXy); condEvaluate) raisedTo: 2))) deepCopy) ]
    ifFalse: [ self error: 'Assignment to array or workspace failed, check that the destination is a structured type'].
self up.
^gXwres
!!
nil! "(fix for smalltalk bug, this line is ignored if there is a variable declaration error)"

"End of procedure"

"Procedure class definition:
 ==========================="
((Smalltalk at: #GXworkspaceUarcsin ifAbsent: [ nil ]) isKindOf: Function class) ifTrue: [ ('Warning: Overriding previously defined procedure ', 'GXworkspaceUarcsin' gtype) prinl ].
Smalltalk at: #GXworkspaceUarcsin put: nil!
Function subclass: 'GXworkspaceUarcsin'
       instanceVariableNames: ''
       classVariableNames: ''
       poolDictionaries: ''
       category: 'Genie Procedure'
!

"HEADER
 ------"

"parameters
 ----------"

GXworkspaceUarcsin

addInstVarName: #gXw1;
compile:
'gXw1: aVar
	aVar notNil ifTrue:[gXw1 _ self typeCheck: aVar shouldBe: GXWorkspace paramName: ''gXw1'']';
compile:
'p1: aVar
	aVar notNil ifTrue:[gXw1 _ self typeCheck: aVar shouldBe: GXWorkspace paramName: ''gXw1'']'!

"result
 ------"

GXworkspaceUarcsin

addInstVarName: #gXwres
!

!GXworkspaceUarcsin methodsFor: 'initializing'!

initVars

!!

"BODY
 ----"

!GXworkspaceUarcsin methodsFor: 'running'!

evaluate
gXwres _ (gXw1) deepCopy.
(gXwres isKindOf: GXArray) | (gXwres isKindOf: GXWorkspace)
    ifTrue:  [ ((gXwres) at:#gXy put: (GXarcsin run line: 0 proc: self method: true; p1:((gXw1) at:#gXy); condEvaluate) deepCopy) ]
    ifFalse: [ self error: 'Assignment to array or workspace failed, check that the destination is a structured type'].
(gXwres isKindOf: GXArray) | (gXwres isKindOf: GXWorkspace)
    ifTrue:  [ ((gXwres) at:#gXe put: (((gXw1) at:#gXe) / (GXsqrt run line: 0 proc: self method: true; p1:(1 - (((gXw1) at:#gXy) raisedTo: 2)); condEvaluate)) deepCopy) ]
    ifFalse: [ self error: 'Assignment to array or workspace failed, check that the destination is a structured type'].
self up.
^gXwres
!!
nil! "(fix for smalltalk bug, this line is ignored if there is a variable declaration error)"

"End of procedure"

"Procedure class definition:
 ==========================="
((Smalltalk at: #GXworkspaceUarccos ifAbsent: [ nil ]) isKindOf: Function class) ifTrue: [ ('Warning: Overriding previously defined procedure ', 'GXworkspaceUarccos' gtype) prinl ].
Smalltalk at: #GXworkspaceUarccos put: nil!
Function subclass: 'GXworkspaceUarccos'
       instanceVariableNames: ''
       classVariableNames: ''
       poolDictionaries: ''
       category: 'Genie Procedure'
!

"HEADER
 ------"

"parameters
 ----------"

GXworkspaceUarccos

addInstVarName: #gXw1;
compile:
'gXw1: aVar
	aVar notNil ifTrue:[gXw1 _ self typeCheck: aVar shouldBe: GXWorkspace paramName: ''gXw1'']';
compile:
'p1: aVar
	aVar notNil ifTrue:[gXw1 _ self typeCheck: aVar shouldBe: GXWorkspace paramName: ''gXw1'']'!

"result
 ------"

GXworkspaceUarccos

addInstVarName: #gXwres
!

!GXworkspaceUarccos methodsFor: 'initializing'!

initVars

!!

"BODY
 ----"

!GXworkspaceUarccos methodsFor: 'running'!

evaluate
gXwres _ (gXw1) deepCopy.
(gXwres isKindOf: GXArray) | (gXwres isKindOf: GXWorkspace)
    ifTrue:  [ ((gXwres) at:#gXy put: (GXarccos run line: 0 proc: self method: true; p1:((gXw1) at:#gXy); condEvaluate) deepCopy) ]
    ifFalse: [ self error: 'Assignment to array or workspace failed, check that the destination is a structured type'].
(gXwres isKindOf: GXArray) | (gXwres isKindOf: GXWorkspace)
    ifTrue:  [ ((gXwres) at:#gXe put: (((gXw1) at:#gXe) / (GXsqrt run line: 0 proc: self method: true; p1:(1 - (((gXw1) at:#gXy) raisedTo: 2)); condEvaluate)) deepCopy) ]
    ifFalse: [ self error: 'Assignment to array or workspace failed, check that the destination is a structured type'].
self up.
^gXwres
!!
nil! "(fix for smalltalk bug, this line is ignored if there is a variable declaration error)"

"End of procedure"

"Procedure class definition:
 ==========================="
((Smalltalk at: #GXworkspaceUarctan ifAbsent: [ nil ]) isKindOf: Function class) ifTrue: [ ('Warning: Overriding previously defined procedure ', 'GXworkspaceUarctan' gtype) prinl ].
Smalltalk at: #GXworkspaceUarctan put: nil!
Function subclass: 'GXworkspaceUarctan'
       instanceVariableNames: ''
       classVariableNames: ''
       poolDictionaries: ''
       category: 'Genie Procedure'
!

"HEADER
 ------"

"parameters
 ----------"

GXworkspaceUarctan

addInstVarName: #gXw1;
compile:
'gXw1: aVar
	aVar notNil ifTrue:[gXw1 _ self typeCheck: aVar shouldBe: GXWorkspace paramName: ''gXw1'']';
compile:
'p1: aVar
	aVar notNil ifTrue:[gXw1 _ self typeCheck: aVar shouldBe: GXWorkspace paramName: ''gXw1'']'!

"result
 ------"

GXworkspaceUarctan

addInstVarName: #gXwres
!

!GXworkspaceUarctan methodsFor: 'initializing'!

initVars

!!

"BODY
 ----"

!GXworkspaceUarctan methodsFor: 'running'!

evaluate
gXwres _ (gXw1) deepCopy.
(gXwres isKindOf: GXArray) | (gXwres isKindOf: GXWorkspace)
    ifTrue:  [ ((gXwres) at:#gXy put: (GXarctan run line: 0 proc: self method: true; p1:((gXw1) at:#gXy); condEvaluate) deepCopy) ]
    ifFalse: [ self error: 'Assignment to array or workspace failed, check that the destination is a structured type'].
(gXwres isKindOf: GXArray) | (gXwres isKindOf: GXWorkspace)
    ifTrue:  [ ((gXwres) at:#gXe put: (((gXw1) at:#gXe) / (GXsqrt run line: 0 proc: self method: true; p1:(1 - (((gXw1) at:#gXy) raisedTo: 2)); condEvaluate)) deepCopy) ]
    ifFalse: [ self error: 'Assignment to array or workspace failed, check that the destination is a structured type'].
self up.
^gXwres
!!
nil! "(fix for smalltalk bug, this line is ignored if there is a variable declaration error)"

"End of procedure"

"Procedure class definition:
 ==========================="
((Smalltalk at: #GXworkspaceUln ifAbsent: [ nil ]) isKindOf: Function class) ifTrue: [ ('Warning: Overriding previously defined procedure ', 'GXworkspaceUln' gtype) prinl ].
Smalltalk at: #GXworkspaceUln put: nil!
Function subclass: 'GXworkspaceUln'
       instanceVariableNames: ''
       classVariableNames: ''
       poolDictionaries: ''
       category: 'Genie Procedure'
!

"HEADER
 ------"

"parameters
 ----------"

GXworkspaceUln

addInstVarName: #gXw1;
compile:
'gXw1: aVar
	aVar notNil ifTrue:[gXw1 _ self typeCheck: aVar shouldBe: GXWorkspace paramName: ''gXw1'']';
compile:
'p1: aVar
	aVar notNil ifTrue:[gXw1 _ self typeCheck: aVar shouldBe: GXWorkspace paramName: ''gXw1'']'!

"result
 ------"

GXworkspaceUln

addInstVarName: #gXwres
!

!GXworkspaceUln methodsFor: 'initializing'!

initVars

!!

"BODY
 ----"

!GXworkspaceUln methodsFor: 'running'!

evaluate
gXwres _ (gXw1) deepCopy.
(gXwres isKindOf: GXArray) | (gXwres isKindOf: GXWorkspace)
    ifTrue:  [ ((gXwres) at:#gXy put: (GXln run line: 0 proc: self method: true; p1:((gXw1) at:#gXy); condEvaluate) deepCopy) ]
    ifFalse: [ self error: 'Assignment to array or workspace failed, check that the destination is a structured type'].
(gXwres isKindOf: GXArray) | (gXwres isKindOf: GXWorkspace)
    ifTrue:  [ ((gXwres) at:#gXe put: (((gXw1) at:#gXe) / ((gXw1) at:#gXy)) deepCopy) ]
    ifFalse: [ self error: 'Assignment to array or workspace failed, check that the destination is a structured type'].
self up.
^gXwres
!!
nil! "(fix for smalltalk bug, this line is ignored if there is a variable declaration error)"

"End of procedure"

"Procedure class definition:
 ==========================="
((Smalltalk at: #GXworkspaceUexp ifAbsent: [ nil ]) isKindOf: Function class) ifTrue: [ ('Warning: Overriding previously defined procedure ', 'GXworkspaceUexp' gtype) prinl ].
Smalltalk at: #GXworkspaceUexp put: nil!
Function subclass: 'GXworkspaceUexp'
       instanceVariableNames: ''
       classVariableNames: ''
       poolDictionaries: ''
       category: 'Genie Procedure'
!

"HEADER
 ------"

"parameters
 ----------"

GXworkspaceUexp

addInstVarName: #gXw1;
compile:
'gXw1: aVar
	aVar notNil ifTrue:[gXw1 _ self typeCheck: aVar shouldBe: GXWorkspace paramName: ''gXw1'']';
compile:
'p1: aVar
	aVar notNil ifTrue:[gXw1 _ self typeCheck: aVar shouldBe: GXWorkspace paramName: ''gXw1'']'!

"result
 ------"

GXworkspaceUexp

addInstVarName: #gXwres
!

!GXworkspaceUexp methodsFor: 'initializing'!

initVars

!!

"BODY
 ----"

!GXworkspaceUexp methodsFor: 'running'!

evaluate
gXwres _ (gXw1) deepCopy.
(gXwres isKindOf: GXArray) | (gXwres isKindOf: GXWorkspace)
    ifTrue:  [ ((gXwres) at:#gXy put: (GXexp run line: 0 proc: self method: true; p1:((gXw1) at:#gXy); condEvaluate) deepCopy) ]
    ifFalse: [ self error: 'Assignment to array or workspace failed, check that the destination is a structured type'].
(gXwres isKindOf: GXArray) | (gXwres isKindOf: GXWorkspace)
    ifTrue:  [ ((gXwres) at:#gXe put: (((gXw1) at:#gXe) * ((gXwres) at:#gXy)) deepCopy) ]
    ifFalse: [ self error: 'Assignment to array or workspace failed, check that the destination is a structured type'].
self up.
^gXwres
!!
nil! "(fix for smalltalk bug, this line is ignored if there is a variable declaration error)"

"End of procedure"

"Procedure class definition:
 ==========================="
((Smalltalk at: #GXworkspaceUlog ifAbsent: [ nil ]) isKindOf: Function class) ifTrue: [ ('Warning: Overriding previously defined procedure ', 'GXworkspaceUlog' gtype) prinl ].
Smalltalk at: #GXworkspaceUlog put: nil!
Function subclass: 'GXworkspaceUlog'
       instanceVariableNames: ''
       classVariableNames: ''
       poolDictionaries: ''
       category: 'Genie Procedure'
!

"HEADER
 ------"

"parameters
 ----------"

GXworkspaceUlog

addInstVarName: #gXw1;
compile:
'gXw1: aVar
	aVar notNil ifTrue:[gXw1 _ self typeCheck: aVar shouldBe: GXWorkspace paramName: ''gXw1'']';
compile:
'p1: aVar
	aVar notNil ifTrue:[gXw1 _ self typeCheck: aVar shouldBe: GXWorkspace paramName: ''gXw1'']'!

"result
 ------"

GXworkspaceUlog

addInstVarName: #gXwres
!

!GXworkspaceUlog methodsFor: 'initializing'!

initVars

!!

"BODY
 ----"

!GXworkspaceUlog methodsFor: 'running'!

evaluate
gXwres _ (gXw1) deepCopy.
(gXwres isKindOf: GXArray) | (gXwres isKindOf: GXWorkspace)
    ifTrue:  [ ((gXwres) at:#gXy put: (GXlog run line: 0 proc: self method: true; p1:((gXw1) at:#gXy); condEvaluate) deepCopy) ]
    ifFalse: [ self error: 'Assignment to array or workspace failed, check that the destination is a structured type'].
(gXwres isKindOf: GXArray) | (gXwres isKindOf: GXWorkspace)
    ifTrue:  [ ((gXwres) at:#gXe put: (((gXw1) at:#gXe) / ((((gXw1) at:#gXy) * (GXln run line: 0 proc: self method: true; p1:1.000000000000000e001; condEvaluate)))) deepCopy) ]
    ifFalse: [ self error: 'Assignment to array or workspace failed, check that the destination is a structured type'].
self up.
^gXwres
!!
nil! "(fix for smalltalk bug, this line is ignored if there is a variable declaration error)"

"End of procedure"

"Procedure class definition:
 ==========================="
((Smalltalk at: #GXworkspaceUsqrt ifAbsent: [ nil ]) isKindOf: Function class) ifTrue: [ ('Warning: Overriding previously defined procedure ', 'GXworkspaceUsqrt' gtype) prinl ].
Smalltalk at: #GXworkspaceUsqrt put: nil!
Function subclass: 'GXworkspaceUsqrt'
       instanceVariableNames: ''
       classVariableNames: ''
       poolDictionaries: ''
       category: 'Genie Procedure'
!

"HEADER
 ------"

"parameters
 ----------"

GXworkspaceUsqrt

addInstVarName: #gXw1;
compile:
'gXw1: aVar
	aVar notNil ifTrue:[gXw1 _ self typeCheck: aVar shouldBe: GXWorkspace paramName: ''gXw1'']';
compile:
'p1: aVar
	aVar notNil ifTrue:[gXw1 _ self typeCheck: aVar shouldBe: GXWorkspace paramName: ''gXw1'']'!

"result
 ------"

GXworkspaceUsqrt

addInstVarName: #gXwres
!

!GXworkspaceUsqrt methodsFor: 'initializing'!

initVars

!!

"BODY
 ----"

!GXworkspaceUsqrt methodsFor: 'running'!

evaluate
gXwres _ (gXw1) deepCopy.
(gXwres isKindOf: GXArray) | (gXwres isKindOf: GXWorkspace)
    ifTrue:  [ ((gXwres) at:#gXy put: (GXsqrt run line: 0 proc: self method: true; p1:((gXw1) at:#gXy); condEvaluate) deepCopy) ]
    ifFalse: [ self error: 'Assignment to array or workspace failed, check that the destination is a structured type'].
(gXwres isKindOf: GXArray) | (gXwres isKindOf: GXWorkspace)
    ifTrue:  [ ((gXwres) at:#gXe put: (((gXw1) at:#gXe) / ((2.000000000000000e000 * ((gXwres) at:#gXy)))) deepCopy) ]
    ifFalse: [ self error: 'Assignment to array or workspace failed, check that the destination is a structured type'].
self up.
^gXwres
!!
nil! "(fix for smalltalk bug, this line is ignored if there is a variable declaration error)"

"End of procedure"

"Procedure class definition:
 ==========================="
((Smalltalk at: #GXworkspaceUabs ifAbsent: [ nil ]) isKindOf: Function class) ifTrue: [ ('Warning: Overriding previously defined procedure ', 'GXworkspaceUabs' gtype) prinl ].
Smalltalk at: #GXworkspaceUabs put: nil!
Function subclass: 'GXworkspaceUabs'
       instanceVariableNames: ''
       classVariableNames: ''
       poolDictionaries: ''
       category: 'Genie Procedure'
!

"HEADER
 ------"

"parameters
 ----------"

GXworkspaceUabs

addInstVarName: #gXw1;
compile:
'gXw1: aVar
	aVar notNil ifTrue:[gXw1 _ self typeCheck: aVar shouldBe: GXWorkspace paramName: ''gXw1'']';
compile:
'p1: aVar
	aVar notNil ifTrue:[gXw1 _ self typeCheck: aVar shouldBe: GXWorkspace paramName: ''gXw1'']'!

"result
 ------"

GXworkspaceUabs

addInstVarName: #gXwres
!

!GXworkspaceUabs methodsFor: 'initializing'!

initVars

!!

"BODY
 ----"

!GXworkspaceUabs methodsFor: 'running'!

evaluate
gXwres _ (gXw1) deepCopy.
(gXwres isKindOf: GXArray) | (gXwres isKindOf: GXWorkspace)
    ifTrue:  [ ((gXwres) at:#gXy put: (GXabs run line: 0 proc: self method: true; p1:((gXw1) at:#gXy); condEvaluate) deepCopy) ]
    ifFalse: [ self error: 'Assignment to array or workspace failed, check that the destination is a structured type'].
(gXwres isKindOf: GXArray) | (gXwres isKindOf: GXWorkspace)
    ifTrue:  [ ((gXwres) at:#gXe put: ((gXw1) at:#gXe) deepCopy) ]
    ifFalse: [ self error: 'Assignment to array or workspace failed, check that the destination is a structured type'].
self up.
^gXwres
!!
nil! "(fix for smalltalk bug, this line is ignored if there is a variable declaration error)"

"End of procedure"

"Procedure class definition:
 ==========================="
((Smalltalk at: #GXworkspaceUnot ifAbsent: [ nil ]) isKindOf: Function class) ifTrue: [ ('Warning: Overriding previously defined procedure ', 'GXworkspaceUnot' gtype) prinl ].
Smalltalk at: #GXworkspaceUnot put: nil!
Function subclass: 'GXworkspaceUnot'
       instanceVariableNames: ''
       classVariableNames: ''
       poolDictionaries: ''
       category: 'Genie Procedure'
!

"HEADER
 ------"

"parameters
 ----------"

GXworkspaceUnot

addInstVarName: #gXw1;
compile:
'gXw1: aVar
	aVar notNil ifTrue:[gXw1 _ self typeCheck: aVar shouldBe: GXWorkspace paramName: ''gXw1'']';
compile:
'p1: aVar
	aVar notNil ifTrue:[gXw1 _ self typeCheck: aVar shouldBe: GXWorkspace paramName: ''gXw1'']'!

"result
 ------"

GXworkspaceUnot

addInstVarName: #gXwres
!

!GXworkspaceUnot methodsFor: 'initializing'!

initVars

!!

"BODY
 ----"

!GXworkspaceUnot methodsFor: 'running'!

evaluate
GXprinten run line: 0 proc: self method: false; p1:'Operation NOT is not defined for workspaces'; condEvaluate.
self up.
^gXwres
!!
nil! "(fix for smalltalk bug, this line is ignored if there is a variable declaration error)"

"End of procedure"

"Procedure class definition:
 ==========================="
((Smalltalk at: #GXworkspaceUnegated ifAbsent: [ nil ]) isKindOf: Function class) ifTrue: [ ('Warning: Overriding previously defined procedure ', 'GXworkspaceUnegated' gtype) prinl ].
Smalltalk at: #GXworkspaceUnegated put: nil!
Function subclass: 'GXworkspaceUnegated'
       instanceVariableNames: ''
       classVariableNames: ''
       poolDictionaries: ''
       category: 'Genie Procedure'
!

"HEADER
 ------"

"parameters
 ----------"

GXworkspaceUnegated

addInstVarName: #gXw1;
compile:
'gXw1: aVar
	aVar notNil ifTrue:[gXw1 _ self typeCheck: aVar shouldBe: GXWorkspace paramName: ''gXw1'']';
compile:
'p1: aVar
	aVar notNil ifTrue:[gXw1 _ self typeCheck: aVar shouldBe: GXWorkspace paramName: ''gXw1'']'!

"result
 ------"

GXworkspaceUnegated

addInstVarName: #gXwres
!

!GXworkspaceUnegated methodsFor: 'initializing'!

initVars

!!

"BODY
 ----"

!GXworkspaceUnegated methodsFor: 'running'!

evaluate
gXwres _ (gXw1) deepCopy.
(gXwres isKindOf: GXArray) | (gXwres isKindOf: GXWorkspace)
    ifTrue:  [ ((gXwres) at:#gXy put: ((gXw1) at:#gXy) negated deepCopy) ]
    ifFalse: [ self error: 'Assignment to array or workspace failed, check that the destination is a structured type'].
self up.
^gXwres
!!
nil! "(fix for smalltalk bug, this line is ignored if there is a variable declaration error)"

"End of procedure"



Smalltalk at: #gxAsciiFileOpen put: 1 !
Smalltalk at: #gxAsciiFileClose put: 2 !
Smalltalk at: #gxAsciiFileSkip put: 3 !
Smalltalk at: #gxAsciiFileLines put: 4 !
Smalltalk at: #gxAsciiFileReadFreeFormat put: 5 !
Smalltalk at: #gxAsciiFileReadFixedFormat put: 6 !
Smalltalk at: #gxAsciiFileWriteFreeFormat put: 7 !
Smalltalk at: #gxAsciiFileWriteFixedFormat put: 8 !
Smalltalk at: #gxAsciiFileRewind put: 9 !

Object subclass: #AsciiFile 
        instanceVariableNames: 'file handle comment delimiter wksp'
        classVariableNames: 'defComment defDelimiter defFieldNames defWriteSeparator defReadSeparator'
        poolDictionaries: ''
        category: nil
!
	
AsciiFile comment: 'Class for reading ASCII data from file' !

!AsciiFile class methodsFor: 'instance creation'!

new
	^super new initAsciiFile
!

setDefFieldNames: aValue
	defFieldNames _ aValue
!

setDefReadSeparator: aValue
	defReadSeparator _ aValue
!

setDefWriteSeparator: aValue
	defWriteSeparator _ aValue
!

setDefDelimiter: aValue
	defDelimiter _ aValue
!

setDefComment: aValue
	defComment _ aValue
! !

!AsciiFile methodsFor: 'initialisation'!

initAsciiFile
	file _ nil.
	handle _ nil.
	comment _ defComment.
	delimiter _ defDelimiter.
	wksp _ GXWorkspace new
! !

!AsciiFile methodsFor: 'file access'!
open: fileName comment: aComment delimiter: aDelimiter
	file _ fileName.
	( aComment isKindOf: String ) ifTrue: [ comment _ aComment ].
	( aDelimiter isKindOf: String ) ifTrue: [ delimiter _ aDelimiter ].
	( file isKindOf: String ) ifFalse: [ ^'AsciiFile/Open: file not specified' prenl ].
	^fileAccess
	    function: gxAsciiFileOpen
	    parameters: ((Array new: 1) at: 1 put: self; yourself)
!

open: fileName
	^self open: fileName comment: defComment delimiter: defDelimiter
!

close
	^fileAccess
	    function: gxAsciiFileClose
	    parameters: ((Array new: 1) at: 1 put: self; yourself)
!

skip: nlines
	( nlines isKindOf: Integer ) ifFalse: [ nlines _ 1 ].
	^fileAccess
	    function: gxAsciiFileSkip
	    parameters: ((Array new: 2) at: 1 put: self; at: 2 put: nlines; yourself)
!

rewind
	^fileAccess
	    function: gxAsciiFileRewind
	    parameters: ((Array new: 1) at: 1 put: self; yourself)
!

"number of lines in file"
lines
	^fileAccess
	    function: gxAsciiFileLines
	    parameters: ((Array new: 1) at: 1 put: self; yourself)
!

writeFreeFormat: aVariable separator: aChar userComment: aString
	( aChar isKindOf: String ) ifFalse: [ aChar _ defWriteSeparator ].
	^fileAccess
	    function: gxAsciiFileWriteFreeFormat
	    parameters: ((Array new: 4) at: 1 put: self; 
				at: 2 put: aVariable;
				at: 3 put: aChar;
				at: 4 put: aString;
			yourself)
!

readFreeFormat: fieldNames separator: aChar count: anInt
	| tmp |
	( fieldNames isKindOf: String ) ifFalse: [ fieldNames _ defFieldNames ].
	( aChar isKindOf: String ) ifFalse: [ aChar _ defReadSeparator ].
	( anInt isKindOf: Integer ) ifFalse: [ anInt _ 1 ].
	tmp _ fileAccess
	    function: gxAsciiFileReadFreeFormat
	    parameters: ((Array new: 4) at: 1 put: self; 
				at: 2 put: fieldNames;
				at: 3 put: aChar;
				at: 4 put: anInt;
			yourself).
	( tmp isKindOf: IdentityDictionary ) ifTrue: 
		[ tmp associationsDo: [ :i | wksp add: i ] ].
	^tmp
!

readFixedFormat: fieldNames format: aChar count: anInt
	| tmp |
	( fieldNames isKindOf: String ) ifFalse: [ fieldNames _ defFieldNames ].
	( aChar isKindOf: String ) ifFalse: [ ^'AsciiFile/readFixedFormat: format not specified' prenl ].
	( anInt isKindOf: Integer ) ifFalse: [ anInt _ 1 ].
	tmp _ fileAccess
	    function: gxAsciiFileReadFixedFormat
	    parameters: ((Array new: 4) at: 1 put: self; 
				at: 2 put: fieldNames;
				at: 3 put: aChar;
				at: 4 put: anInt;
			yourself).
	( tmp isKindOf: IdentityDictionary ) ifTrue: 
		[ tmp associationsDo: [ :i | wksp add: i ] ].
	^tmp
!

"Return the current fields and reset them if necessary"
wksp: doAReset
	| tmp |
	tmp _ wksp.
	doAReset ifTrue: [ wksp _ GXWorkspace new ].
	^tmp
! !

AsciiFile setDefComment: '' !	"No default comment character"

AsciiFile setDefDelimiter: '
' !

AsciiFile setDefReadSeparator: '[ 	
]+' !

AsciiFile setDefWriteSeparator: ' ' !

AsciiFile setDefFieldNames: 'field01 field02 field03 field04 filed05 field06' !

nil
!
"Procedure class definition:
 ==========================="
((Smalltalk at: #GXUUasciifilehelp ifAbsent: [ nil ]) isKindOf: Function class) ifTrue: [ ('Warning: Overriding previously defined procedure ', 'GXUUasciifilehelp' gtype) prinl ].
Smalltalk at: #GXUUasciifilehelp put: nil!
Function subclass: 'GXUUasciifilehelp'
       instanceVariableNames: ''
       classVariableNames: ''
       poolDictionaries: ''
       category: 'Genie Procedure'
!

"BODY
 ----"

!GXUUasciifilehelp methodsFor: 'running'!

evaluate
GXprintin run line: 0 proc: self method: false; p1:'HANDLE=ASCIIFILE:OPEN(FILE=String, [COMMENT=String], [DELIMITER=String])'; condEvaluate.
GXprintin run line: 0 proc: self method: false; p1:'ASCIIFILE/CLOSE HANDLE'; condEvaluate.
GXprintin run line: 0 proc: self method: false; p1:'ASCIIFILE/SKIP HANDLE [NLINES=Integer]'; condEvaluate.
GXprintin run line: 0 proc: self method: false; p1:'NLINES=ASCIIFILE:LINES(HANDLE)'; condEvaluate.
GXprintin run line: 0 proc: self method: false; p1:'ASCIIFILE/READFREE HANDLE FIELDS SPEARATOR COUNT'; condEvaluate.
GXprintin run line: 0 proc: self method: false; p1:'ASCIIFILE/READFIXED HANDLE FIELDS FORMAT COUNT'; condEvaluate.
GXprintin run line: 0 proc: self method: false; p1:'ASCIIFILE/DATA[/RESET] HANDLE'; condEvaluate.
GXprintin run line: 0 proc: self method: false; p1:'ASCIIFILE/WRITEFREE HANDLE COMMENT SEPARATOR GV1 GV2 GV3 GV4'; condEvaluate.
self up.
^nil
!!
nil! "(fix for smalltalk bug, this line is ignored if there is a variable declaration error)"

"End of procedure"

"Procedure class definition:
 ==========================="
((Smalltalk at: #GXasciifile ifAbsent: [ nil ]) isKindOf: Function class) ifTrue: [ ('Warning: Overriding previously defined procedure ', 'GXasciifile' gtype) prinl ].
Smalltalk at: #GXasciifile put: nil!
Function subclass: 'GXasciifile'
       instanceVariableNames: ''
       classVariableNames: ''
       poolDictionaries: ''
       category: 'Genie Procedure'
!

"HEADER
 ------"

"qualifiers
 ----------"

GXasciifile

addInstVarName: #gXopen;
compile:
'gXopen
	gXopen _ true';
compile:
'gXopen: aBoolean
	gXopen _ aBoolean';

addInstVarName: #gXclose;
compile:
'gXclose
	gXclose _ true';
compile:
'gXclose: aBoolean
	gXclose _ aBoolean';

addInstVarName: #gXskip;
compile:
'gXskip
	gXskip _ true';
compile:
'gXskip: aBoolean
	gXskip _ aBoolean';

addInstVarName: #gXlines;
compile:
'gXlines
	gXlines _ true';
compile:
'gXlines: aBoolean
	gXlines _ aBoolean';

addInstVarName: #gXreadfree;
compile:
'gXreadfree
	gXreadfree _ true';
compile:
'gXreadfree: aBoolean
	gXreadfree _ aBoolean';

addInstVarName: #gXreadfixed;
compile:
'gXreadfixed
	gXreadfixed _ true';
compile:
'gXreadfixed: aBoolean
	gXreadfixed _ aBoolean';

addInstVarName: #gXdata;
compile:
'gXdata
	gXdata _ true';
compile:
'gXdata: aBoolean
	gXdata _ aBoolean';

addInstVarName: #gXreset;
compile:
'gXreset
	gXreset _ true';
compile:
'gXreset: aBoolean
	gXreset _ aBoolean';

addInstVarName: #gXwritefree;
compile:
'gXwritefree
	gXwritefree _ true';
compile:
'gXwritefree: aBoolean
	gXwritefree _ aBoolean';

addInstVarName: #gXrewind;
compile:
'gXrewind
	gXrewind _ true';
compile:
'gXrewind: aBoolean
	gXrewind _ aBoolean'!

"parameters
 ----------"

GXasciifile

addInstVarName: #gXp1;
compile:
'gXp1: aVar
	aVar notNil ifTrue:[ gXp1 _ aVar ]';
compile:
'p1: aVar
	aVar notNil ifTrue:[ gXp1 _ aVar ]';

addInstVarName: #gXp2;
compile:
'gXp2: aVar
	aVar notNil ifTrue:[ gXp2 _ aVar ]';
compile:
'p2: aVar
	aVar notNil ifTrue:[ gXp2 _ aVar ]';

addInstVarName: #gXp3;
compile:
'gXp3: aVar
	aVar notNil ifTrue:[ gXp3 _ aVar ]';
compile:
'p3: aVar
	aVar notNil ifTrue:[ gXp3 _ aVar ]';

addInstVarName: #gXp4;
compile:
'gXp4: aVar
	aVar notNil ifTrue:[ gXp4 _ aVar ]';
compile:
'p4: aVar
	aVar notNil ifTrue:[ gXp4 _ aVar ]';

addInstVarName: #gXp5;
compile:
'gXp5: aVar
	aVar notNil ifTrue:[ gXp5 _ aVar ]';
compile:
'p5: aVar
	aVar notNil ifTrue:[ gXp5 _ aVar ]';

addInstVarName: #gXp6;
compile:
'gXp6: aVar
	aVar notNil ifTrue:[ gXp6 _ aVar ]';
compile:
'p6: aVar
	aVar notNil ifTrue:[ gXp6 _ aVar ]';

addInstVarName: #gXp7;
compile:
'gXp7: aVar
	aVar notNil ifTrue:[ gXp7 _ aVar ]';
compile:
'p7: aVar
	aVar notNil ifTrue:[ gXp7 _ aVar ]';

addInstVarName: #gXfile;
compile:
'gXfile: aVar
	aVar notNil ifTrue:[gXfile _ self typeCheck: aVar shouldBe: GXString paramName: ''gXfile'']';
compile:
'p8: aVar
	aVar notNil ifTrue:[gXfile _ self typeCheck: aVar shouldBe: GXString paramName: ''gXfile'']';

addInstVarName: #gXhandle;
compile:
'gXhandle: aVar
	aVar notNil ifTrue:[ gXhandle _ aVar ]';
compile:
'p9: aVar
	aVar notNil ifTrue:[ gXhandle _ aVar ]';

addInstVarName: #gXcomment;
compile:
'gXcomment: aVar
	aVar notNil ifTrue:[gXcomment _ self typeCheck: aVar shouldBe: GXString paramName: ''gXcomment'']';
compile:
'p10: aVar
	aVar notNil ifTrue:[gXcomment _ self typeCheck: aVar shouldBe: GXString paramName: ''gXcomment'']';

addInstVarName: #gXdelimiter;
compile:
'gXdelimiter: aVar
	aVar notNil ifTrue:[gXdelimiter _ self typeCheck: aVar shouldBe: GXString paramName: ''gXdelimiter'']';
compile:
'p11: aVar
	aVar notNil ifTrue:[gXdelimiter _ self typeCheck: aVar shouldBe: GXString paramName: ''gXdelimiter'']';

addInstVarName: #gXnlines;
compile:
'gXnlines: aVar
	aVar notNil ifTrue:[gXnlines _ self typeCheck: aVar shouldBe: GXInteger paramName: ''gXnlines'']';
compile:
'p12: aVar
	aVar notNil ifTrue:[gXnlines _ self typeCheck: aVar shouldBe: GXInteger paramName: ''gXnlines'']';

addInstVarName: #gXformat;
compile:
'gXformat: aVar
	aVar notNil ifTrue:[gXformat _ self typeCheck: aVar shouldBe: GXString paramName: ''gXformat'']';
compile:
'p13: aVar
	aVar notNil ifTrue:[gXformat _ self typeCheck: aVar shouldBe: GXString paramName: ''gXformat'']';

addInstVarName: #gXseparator;
compile:
'gXseparator: aVar
	aVar notNil ifTrue:[gXseparator _ self typeCheck: aVar shouldBe: GXString paramName: ''gXseparator'']';
compile:
'p14: aVar
	aVar notNil ifTrue:[gXseparator _ self typeCheck: aVar shouldBe: GXString paramName: ''gXseparator'']';

addInstVarName: #gXcount;
compile:
'gXcount: aVar
	aVar notNil ifTrue:[gXcount _ self typeCheck: aVar shouldBe: GXInteger paramName: ''gXcount'']';
compile:
'p15: aVar
	aVar notNil ifTrue:[gXcount _ self typeCheck: aVar shouldBe: GXInteger paramName: ''gXcount'']';

addInstVarName: #gXfields;
compile:
'gXfields: aVar
	aVar notNil ifTrue:[gXfields _ self typeCheck: aVar shouldBe: GXString paramName: ''gXfields'']';
compile:
'p16: aVar
	aVar notNil ifTrue:[gXfields _ self typeCheck: aVar shouldBe: GXString paramName: ''gXfields'']';

addInstVarName: #gXgv1;
compile:
'gXgv1: aVar
	aVar notNil ifTrue:[ gXgv1 _ aVar ]';
compile:
'p17: aVar
	aVar notNil ifTrue:[ gXgv1 _ aVar ]';

addInstVarName: #gXgv2;
compile:
'gXgv2: aVar
	aVar notNil ifTrue:[ gXgv2 _ aVar ]';
compile:
'p18: aVar
	aVar notNil ifTrue:[ gXgv2 _ aVar ]';

addInstVarName: #gXgv3;
compile:
'gXgv3: aVar
	aVar notNil ifTrue:[ gXgv3 _ aVar ]';
compile:
'p19: aVar
	aVar notNil ifTrue:[ gXgv3 _ aVar ]';

addInstVarName: #gXgv4;
compile:
'gXgv4: aVar
	aVar notNil ifTrue:[ gXgv4 _ aVar ]';
compile:
'p20: aVar
	aVar notNil ifTrue:[ gXgv4 _ aVar ]'
!

"Local variables
 ---------------"

GXasciifile

addInstVarName: #gXnlines;

addInstVarName: #gXdelimiter;

addInstVarName: #gXcount!

"Local variables
 ---------------"

GXasciifile

addInstVarName: #gXvariable;

addInstVarName: #gXfields
!

!GXasciifile methodsFor: 'initializing'!

initVars
	gXopen _ false.
	gXclose _ false.
	gXskip _ false.
	gXlines _ false.
	gXreadfree _ false.
	gXreadfixed _ false.
	gXdata _ false.
	gXreset _ false.
	gXwritefree _ false.
	gXrewind _ false.
	gXnlines _ 1.
	gXdelimiter _ '
'.
	gXcount _ 1.
	gXfields _ 'FIELD01,FIELD02,FIELD03,FIELD04,FIELD05,FIELD06'
!!

"BODY
 ----"

!GXasciifile methodsFor: 'running'!

evaluate

"set up values"
    gXp1 notNil ifTrue: [ gXfile _ gXp1. gXhandle _ gXp1 ].
    gXp2 notNil ifTrue: [ gXcomment _ gXp2. gXnlines _ gXp2. gXfields _ gXp2 ].
    gXp3 notNil ifTrue: [ gXdelimiter _ gXp3. gXformat _ gXp3. gXseparator _ gXp3 ].
    gXp4 notNil ifTrue: [ gXcount _ gXp4. gXgv1 _ gXp4 ].
    gXp5 notNil ifTrue: [ gXgv2 _ gXp5 ].
    gXp6 notNil ifTrue: [ gXgv3 _ gXp6 ].
    gXp7 notNil ifTrue: [ gXgv4 _ gXp7 ].
"set up array for free format output"
    gXvariable _ Array new: 4.
    gXvariable at: 1 put: gXgv1; at: 2 put: gXgv2; at: 3 put: gXgv3; at:4 put: gXgv4; yourself.
"open is only method that doesn't need a handle"
    gXopen ifTrue: [ ^(AsciiFile new) open: gXfile comment: gXcomment delimiter: gXdelimiter ].

"Check for missing parameters"
    ( gXhandle isKindOf: AsciiFile ) ifFalse: [ ^'ASCIIFILE: Invalid Handle' prenl ].

    gXlines ifTrue: [ ^gXhandle lines ].
    gXclose ifTrue: [ ^gXhandle close ].
    gXskip ifTrue: [ ^gXhandle skip: gXnlines ].
    gXreadfixed ifTrue: [ ^gXhandle readFixedFormat: gXfields format: gXformat count: gXcount ].
    gXreadfree ifTrue: [ ^gXhandle readFreeFormat: gXfields separator: gXseparator count: gXcount ].
    gXwritefree ifTrue: [ ^gXhandle writeFreeFormat: gXvariable separator: gXseparator userComment: gXcomment ].
    gXdata ifTrue: [ ^gXhandle wksp: gXreset ].
    gXrewind ifTrue: [ ^gXhandle rewind ]
.
self up.
^nil
!!
nil! "(fix for smalltalk bug, this line is ignored if there is a variable declaration error)"

"End of procedure"

GXcomment run line: 0 proc: self method: false; p1:'asciifile'; p2:'Read data from ASCII input file'; condEvaluate!
Smalltalk at: #gXUUdrivers ifAbsent: [ Smalltalk at: #gXUUdrivers put: nil ]!
gXUUdrivers _ (GXfields run line: 0 proc: self method: true; condEvaluate) deepCopy!
Smalltalk at: #gXUUdrivers ifAbsent: [ Smalltalk at: #gXUUdrivers put: nil ]!
(gXUUdrivers isKindOf: GXArray) | (gXUUdrivers isKindOf: GXWorkspace)
    ifTrue:  [ ((gXUUdrivers) at:#gXccp put: 'DEC LJ250 Color Companion printer ' deepCopy) ]
    ifFalse: [ self error: 'Assignment to array or workspace failed, check that the destination is a structured type']!
Smalltalk at: #gXUUdrivers ifAbsent: [ Smalltalk at: #gXUUdrivers put: nil ]!
(gXUUdrivers isKindOf: GXArray) | (gXUUdrivers isKindOf: GXWorkspace)
    ifTrue:  [ ((gXUUdrivers) at:#gXcps put: 'PostScript printers, color, landscape Std F77' deepCopy) ]
    ifFalse: [ self error: 'Assignment to array or workspace failed, check that the destination is a structured type']!
Smalltalk at: #gXUUdrivers ifAbsent: [ Smalltalk at: #gXUUdrivers put: nil ]!
(gXUUdrivers isKindOf: GXArray) | (gXUUdrivers isKindOf: GXWorkspace)
    ifTrue:  [ ((gXUUdrivers) at:#gXfile put: 'PGPLOT graphics metafile   ' deepCopy) ]
    ifFalse: [ self error: 'Assignment to array or workspace failed, check that the destination is a structured type']!
Smalltalk at: #gXUUdrivers ifAbsent: [ Smalltalk at: #gXUUdrivers put: nil ]!
(gXUUdrivers isKindOf: GXArray) | (gXUUdrivers isKindOf: GXWorkspace)
    ifTrue:  [ ((gXUUdrivers) at:#gXgf put: 'GraphOn terminal Std F77  ' deepCopy) ]
    ifFalse: [ self error: 'Assignment to array or workspace failed, check that the destination is a structured type']!
Smalltalk at: #gXUUdrivers ifAbsent: [ Smalltalk at: #gXUUdrivers put: nil ]!
(gXUUdrivers isKindOf: GXArray) | (gXUUdrivers isKindOf: GXWorkspace)
    ifTrue:  [ ((gXUUdrivers) at:#gXgif put: 'GIF-format file, landscape   ' deepCopy) ]
    ifFalse: [ self error: 'Assignment to array or workspace failed, check that the destination is a structured type']!
Smalltalk at: #gXUUdrivers ifAbsent: [ Smalltalk at: #gXUUdrivers put: nil ]!
(gXUUdrivers isKindOf: GXArray) | (gXUUdrivers isKindOf: GXWorkspace)
    ifTrue:  [ ((gXUUdrivers) at:#gXhj put: 'Hewlett-Packard Desk/Laserjet printer   ' deepCopy) ]
    ifFalse: [ self error: 'Assignment to array or workspace failed, check that the destination is a structured type']!
Smalltalk at: #gXUUdrivers ifAbsent: [ Smalltalk at: #gXUUdrivers put: nil ]!
(gXUUdrivers isKindOf: GXArray) | (gXUUdrivers isKindOf: GXWorkspace)
    ifTrue:  [ ((gXUUdrivers) at:#gXhpgl put: 'Hewlett-Packard HP-GL plotters, landscape Std F77' deepCopy) ]
    ifFalse: [ self error: 'Assignment to array or workspace failed, check that the destination is a structured type']!
Smalltalk at: #gXUUdrivers ifAbsent: [ Smalltalk at: #gXUUdrivers put: nil ]!
(gXUUdrivers isKindOf: GXArray) | (gXUUdrivers isKindOf: GXWorkspace)
    ifTrue:  [ ((gXUUdrivers) at:#gXlj put: 'Hewlett-Packard LaserJet printers   ' deepCopy) ]
    ifFalse: [ self error: 'Assignment to array or workspace failed, check that the destination is a structured type']!
Smalltalk at: #gXUUdrivers ifAbsent: [ Smalltalk at: #gXUUdrivers put: nil ]!
(gXUUdrivers isKindOf: GXArray) | (gXUUdrivers isKindOf: GXWorkspace)
    ifTrue:  [ ((gXUUdrivers) at:#gXnull put: 'Null device (no output) Std F77' deepCopy) ]
    ifFalse: [ self error: 'Assignment to array or workspace failed, check that the destination is a structured type']!
Smalltalk at: #gXUUdrivers ifAbsent: [ Smalltalk at: #gXUUdrivers put: nil ]!
(gXUUdrivers isKindOf: GXArray) | (gXUUdrivers isKindOf: GXWorkspace)
    ifTrue:  [ ((gXUUdrivers) at:#gXppm put: 'Portable Pixel Map file, landscape ' deepCopy) ]
    ifFalse: [ self error: 'Assignment to array or workspace failed, check that the destination is a structured type']!
Smalltalk at: #gXUUdrivers ifAbsent: [ Smalltalk at: #gXUUdrivers put: nil ]!
(gXUUdrivers isKindOf: GXArray) | (gXUUdrivers isKindOf: GXWorkspace)
    ifTrue:  [ ((gXUUdrivers) at:#gXps put: 'PostScript printers, monochrome, landscape Std F77' deepCopy) ]
    ifFalse: [ self error: 'Assignment to array or workspace failed, check that the destination is a structured type']!
Smalltalk at: #gXUUdrivers ifAbsent: [ Smalltalk at: #gXUUdrivers put: nil ]!
(gXUUdrivers isKindOf: GXArray) | (gXUUdrivers isKindOf: GXWorkspace)
    ifTrue:  [ ((gXUUdrivers) at:#gXtek4010 put: 'Tektronix 4006/4010 storage-tube terminal Std F77' deepCopy) ]
    ifFalse: [ self error: 'Assignment to array or workspace failed, check that the destination is a structured type']!
Smalltalk at: #gXUUdrivers ifAbsent: [ Smalltalk at: #gXUUdrivers put: nil ]!
(gXUUdrivers isKindOf: GXArray) | (gXUUdrivers isKindOf: GXWorkspace)
    ifTrue:  [ ((gXUUdrivers) at:#gXtfile put: 'Tektronix-format disk file VMS  ' deepCopy) ]
    ifFalse: [ self error: 'Assignment to array or workspace failed, check that the destination is a structured type']!
Smalltalk at: #gXUUdrivers ifAbsent: [ Smalltalk at: #gXUUdrivers put: nil ]!
(gXUUdrivers isKindOf: GXArray) | (gXUUdrivers isKindOf: GXWorkspace)
    ifTrue:  [ ((gXUUdrivers) at:#gXtk put: 'Tk driver for widget pgwin ' deepCopy) ]
    ifFalse: [ self error: 'Assignment to array or workspace failed, check that the destination is a structured type']!
Smalltalk at: #gXUUdrivers ifAbsent: [ Smalltalk at: #gXUUdrivers put: nil ]!
(gXUUdrivers isKindOf: GXArray) | (gXUUdrivers isKindOf: GXWorkspace)
    ifTrue:  [ ((gXUUdrivers) at:#gXtk1 put: 'Tk driver for widget pgwin1 ' deepCopy) ]
    ifFalse: [ self error: 'Assignment to array or workspace failed, check that the destination is a structured type']!
Smalltalk at: #gXUUdrivers ifAbsent: [ Smalltalk at: #gXUUdrivers put: nil ]!
(gXUUdrivers isKindOf: GXArray) | (gXUUdrivers isKindOf: GXWorkspace)
    ifTrue:  [ ((gXUUdrivers) at:#gXtk2 put: 'Tk driver for widget pgwin2 ' deepCopy) ]
    ifFalse: [ self error: 'Assignment to array or workspace failed, check that the destination is a structured type']!
Smalltalk at: #gXUUdrivers ifAbsent: [ Smalltalk at: #gXUUdrivers put: nil ]!
(gXUUdrivers isKindOf: GXArray) | (gXUUdrivers isKindOf: GXWorkspace)
    ifTrue:  [ ((gXUUdrivers) at:#gXtk3 put: 'Tk driver for widget pgwin3 ' deepCopy) ]
    ifFalse: [ self error: 'Assignment to array or workspace failed, check that the destination is a structured type']!
Smalltalk at: #gXUUdrivers ifAbsent: [ Smalltalk at: #gXUUdrivers put: nil ]!
(gXUUdrivers isKindOf: GXArray) | (gXUUdrivers isKindOf: GXWorkspace)
    ifTrue:  [ ((gXUUdrivers) at:#gXtk4 put: 'Tk driver for widget pgwin4 ' deepCopy) ]
    ifFalse: [ self error: 'Assignment to array or workspace failed, check that the destination is a structured type']!
Smalltalk at: #gXUUdrivers ifAbsent: [ Smalltalk at: #gXUUdrivers put: nil ]!
(gXUUdrivers isKindOf: GXArray) | (gXUUdrivers isKindOf: GXWorkspace)
    ifTrue:  [ ((gXUUdrivers) at:#gXtk4100 put: 'Tektronix 4100-series terminals Std F77 ' deepCopy) ]
    ifFalse: [ self error: 'Assignment to array or workspace failed, check that the destination is a structured type']!
Smalltalk at: #gXUUdrivers ifAbsent: [ Smalltalk at: #gXUUdrivers put: nil ]!
(gXUUdrivers isKindOf: GXArray) | (gXUUdrivers isKindOf: GXWorkspace)
    ifTrue:  [ ((gXUUdrivers) at:#gXtk5 put: 'Tk driver for widget pgwin5 ' deepCopy) ]
    ifFalse: [ self error: 'Assignment to array or workspace failed, check that the destination is a structured type']!
Smalltalk at: #gXUUdrivers ifAbsent: [ Smalltalk at: #gXUUdrivers put: nil ]!
(gXUUdrivers isKindOf: GXArray) | (gXUUdrivers isKindOf: GXWorkspace)
    ifTrue:  [ ((gXUUdrivers) at:#gXvcps put: 'PostScript printers, color, portrait Std F77' deepCopy) ]
    ifFalse: [ self error: 'Assignment to array or workspace failed, check that the destination is a structured type']!
Smalltalk at: #gXUUdrivers ifAbsent: [ Smalltalk at: #gXUUdrivers put: nil ]!
(gXUUdrivers isKindOf: GXArray) | (gXUUdrivers isKindOf: GXWorkspace)
    ifTrue:  [ ((gXUUdrivers) at:#gXvgif put: 'GIF-format file, portrait   ' deepCopy) ]
    ifFalse: [ self error: 'Assignment to array or workspace failed, check that the destination is a structured type']!
Smalltalk at: #gXUUdrivers ifAbsent: [ Smalltalk at: #gXUUdrivers put: nil ]!
(gXUUdrivers isKindOf: GXArray) | (gXUUdrivers isKindOf: GXWorkspace)
    ifTrue:  [ ((gXUUdrivers) at:#gXvhpgl put: 'Hewlett-Packard HP-GL plotters, portrait Std F77' deepCopy) ]
    ifFalse: [ self error: 'Assignment to array or workspace failed, check that the destination is a structured type']!
Smalltalk at: #gXUUdrivers ifAbsent: [ Smalltalk at: #gXUUdrivers put: nil ]!
(gXUUdrivers isKindOf: GXArray) | (gXUUdrivers isKindOf: GXWorkspace)
    ifTrue:  [ ((gXUUdrivers) at:#gXvppm put: 'Portable PIxel Map file, portrait ' deepCopy) ]
    ifFalse: [ self error: 'Assignment to array or workspace failed, check that the destination is a structured type']!
Smalltalk at: #gXUUdrivers ifAbsent: [ Smalltalk at: #gXUUdrivers put: nil ]!
(gXUUdrivers isKindOf: GXArray) | (gXUUdrivers isKindOf: GXWorkspace)
    ifTrue:  [ ((gXUUdrivers) at:#gXvps put: 'Postscript printers, monochrome, portrait Std F77' deepCopy) ]
    ifFalse: [ self error: 'Assignment to array or workspace failed, check that the destination is a structured type']!
Smalltalk at: #gXUUdrivers ifAbsent: [ Smalltalk at: #gXUUdrivers put: nil ]!
(gXUUdrivers isKindOf: GXArray) | (gXUUdrivers isKindOf: GXWorkspace)
    ifTrue:  [ ((gXUUdrivers) at:#gXvt125 put: 'Dec Regis terminals (VT125 etc.) Std' deepCopy) ]
    ifFalse: [ self error: 'Assignment to array or workspace failed, check that the destination is a structured type']!
Smalltalk at: #gXUUdrivers ifAbsent: [ Smalltalk at: #gXUUdrivers put: nil ]!
(gXUUdrivers isKindOf: GXArray) | (gXUUdrivers isKindOf: GXWorkspace)
    ifTrue:  [ ((gXUUdrivers) at:#gXvwd put: 'X Window dump file, portrait ' deepCopy) ]
    ifFalse: [ self error: 'Assignment to array or workspace failed, check that the destination is a structured type']!
Smalltalk at: #gXUUdrivers ifAbsent: [ Smalltalk at: #gXUUdrivers put: nil ]!
(gXUUdrivers isKindOf: GXArray) | (gXUUdrivers isKindOf: GXWorkspace)
    ifTrue:  [ ((gXUUdrivers) at:#gXwd put: 'X Window dump file, landscape ' deepCopy) ]
    ifFalse: [ self error: 'Assignment to array or workspace failed, check that the destination is a structured type']!
Smalltalk at: #gXUUdrivers ifAbsent: [ Smalltalk at: #gXUUdrivers put: nil ]!
(gXUUdrivers isKindOf: GXArray) | (gXUUdrivers isKindOf: GXWorkspace)
    ifTrue:  [ ((gXUUdrivers) at:#gXxdisp put: 'PGDISP or FIGDISP server for X' deepCopy) ]
    ifFalse: [ self error: 'Assignment to array or workspace failed, check that the destination is a structured type']!
Smalltalk at: #gXUUdrivers ifAbsent: [ Smalltalk at: #gXUUdrivers put: nil ]!
(gXUUdrivers isKindOf: GXArray) | (gXUUdrivers isKindOf: GXWorkspace)
    ifTrue:  [ ((gXUUdrivers) at:#gXxserve put: 'Persistent window on X Window System' deepCopy) ]
    ifFalse: [ self error: 'Assignment to array or workspace failed, check that the destination is a structured type']!
Smalltalk at: #gXUUdrivers ifAbsent: [ Smalltalk at: #gXUUdrivers put: nil ]!
(gXUUdrivers isKindOf: GXArray) | (gXUUdrivers isKindOf: GXWorkspace)
    ifTrue:  [ ((gXUUdrivers) at:#gXxterm put: 'XTERM Tektronix terminal emulator Std F77' deepCopy) ]
    ifFalse: [ self error: 'Assignment to array or workspace failed, check that the destination is a structured type']!
Smalltalk at: #gXUUdrivers ifAbsent: [ Smalltalk at: #gXUUdrivers put: nil ]!
(gXUUdrivers isKindOf: GXArray) | (gXUUdrivers isKindOf: GXWorkspace)
    ifTrue:  [ ((gXUUdrivers) at:#gXxwindow put: 'Workstations running X Window System C' deepCopy) ]
    ifFalse: [ self error: 'Assignment to array or workspace failed, check that the destination is a structured type']!
"Procedure class definition:
 ==========================="
((Smalltalk at: #GXUUinitio ifAbsent: [ nil ]) isKindOf: Function class) ifTrue: [ ('Warning: Overriding previously defined procedure ', 'GXUUinitio' gtype) prinl ].
Smalltalk at: #GXUUinitio put: nil!
Function subclass: 'GXUUinitio'
       instanceVariableNames: ''
       classVariableNames: ''
       poolDictionaries: ''
       category: 'Genie Procedure'
!

"HEADER
 ------"

"Global variables
 ----------------"

GXUUinitio

addPoolVar: #gXUcurrentUinputUsource value: (gXU)!

"Global variables
 ----------------"

GXUUinitio

addPoolVar: #gXUcurrentUoutputUsource value: (gXU)!

"Global variables
 ----------------"

GXUUinitio

addPoolVar: #gXUcurrentUdisk value: ''!

"Global variables
 ----------------"

GXUUinitio

addPoolVar: #gXUcurrentUdirectory value: ''!

"Global variables
 ----------------"

GXUUinitio

addPoolVar: #gXUcurrentUinstrument value: ''!

"Global variables
 ----------------"

GXUUinitio

addPoolVar: #gXUcurrentUextension value: ''!

"Global variables
 ----------------"

GXUUinitio

addPoolVar: #gXv value: (GXdimensions run line: 0 proc: self method: true; p1:20; condEvaluate)!

"Global variables
 ----------------"

GXUUinitio

addPoolVar: #gXDwhitespace value: '[ 	]+'!

"Global variables
 ----------------"

GXUUinitio

addPoolVar: #gXUcurrentUbinning value: 1!

"Global variables
 ----------------"

GXUUinitio

addPoolVar: #gXUstevesUgooglyUinitialised value: 0!

"Global variables
 ----------------"

GXUUinitio

addPoolVar: #gXhe3detector value: (GXfields run line: 0 proc: self method: true; condEvaluate)
!

!GXUUinitio methodsFor: 'initializing'!

initVars

!!

"BODY
 ----"

!GXUUinitio methodsFor: 'running'!

evaluate
(gXhe3detector isKindOf: GXArray) | (gXhe3detector isKindOf: GXWorkspace)
    ifTrue:  [ ((gXhe3detector) at:#gXtype put: 'He3 gas detector' deepCopy) ]
    ifFalse: [ self error: 'Assignment to array or workspace failed, check that the destination is a structured type'].
(gXhe3detector isKindOf: GXArray) | (gXhe3detector isKindOf: GXWorkspace)
    ifTrue:  [ ((gXhe3detector) at:#gXpressure put: 1.000000000000000e001 deepCopy) ]
    ifFalse: [ self error: 'Assignment to array or workspace failed, check that the destination is a structured type'].
(gXhe3detector isKindOf: GXArray) | (gXhe3detector isKindOf: GXWorkspace)
    ifTrue:  [ ((gXhe3detector) at:#gXwavelength put: 4.000000000000000e000 deepCopy) ]
    ifFalse: [ self error: 'Assignment to array or workspace failed, check that the destination is a structured type'].
(gXhe3detector isKindOf: GXArray) | (gXhe3detector isKindOf: GXWorkspace)
    ifTrue:  [ ((gXhe3detector) at:#gXgasUsigma0 put: 1.185153000000000e-20 deepCopy) ]
    ifFalse: [ self error: 'Assignment to array or workspace failed, check that the destination is a structured type'].
(gXhe3detector isKindOf: GXArray) | (gXhe3detector isKindOf: GXWorkspace)
    ifTrue:  [ ((gXhe3detector) at:#gXwallUsigma0 put: 5.620000000000000e-24 deepCopy) ]
    ifFalse: [ self error: 'Assignment to array or workspace failed, check that the destination is a structured type'].
(gXhe3detector isKindOf: GXArray) | (gXhe3detector isKindOf: GXWorkspace)
    ifTrue:  [ ((gXhe3detector) at:#gXpathlength put: 1.000000000000000e000 deepCopy) ]
    ifFalse: [ self error: 'Assignment to array or workspace failed, check that the destination is a structured type'].
(gXhe3detector isKindOf: GXArray) | (gXhe3detector isKindOf: GXWorkspace)
    ifTrue:  [ ((gXhe3detector) at:#gXweight put: 5.584700000000000e001 deepCopy) ]
    ifFalse: [ self error: 'Assignment to array or workspace failed, check that the destination is a structured type'].
(gXhe3detector isKindOf: GXArray) | (gXhe3detector isKindOf: GXWorkspace)
    ifTrue:  [ ((gXhe3detector) at:#gXdensity put: 7.860000000000000e000 deepCopy) ]
    ifFalse: [ self error: 'Assignment to array or workspace failed, check that the destination is a structured type'].
(gXhe3detector isKindOf: GXArray) | (gXhe3detector isKindOf: GXWorkspace)
    ifTrue:  [ ((gXhe3detector) at:#gXthickness put: 1.000000000000000e-01 deepCopy) ]
    ifFalse: [ self error: 'Assignment to array or workspace failed, check that the destination is a structured type'].
self up.
^nil
!!
nil! "(fix for smalltalk bug, this line is ignored if there is a variable declaration error)"

"End of procedure"

GXcomment run line: 0 proc: self method: false; p1:'__initio'; p2:'Initial I/O definitions'; condEvaluate!
"Procedure class definition:
 ==========================="
((Smalltalk at: #GXUUinitUerrorUhandler ifAbsent: [ nil ]) isKindOf: Function class) ifTrue: [ ('Warning: Overriding previously defined procedure ', 'GXUUinitUerrorUhandler' gtype) prinl ].
Smalltalk at: #GXUUinitUerrorUhandler put: nil!
Function subclass: 'GXUUinitUerrorUhandler'
       instanceVariableNames: ''
       classVariableNames: ''
       poolDictionaries: ''
       category: 'Genie Procedure'
!

"BODY
 ----"

!GXUUinitUerrorUhandler methodsFor: 'running'!

evaluate
((gXUstevesUgooglyUinitialised) = 0) ifTrue: [ 
GXcallout run line: 0 proc: self method: false; gXfacility:'objGraphics'; gXfunction:1; condEvaluate ].
self up.
^nil
!!
nil! "(fix for smalltalk bug, this line is ignored if there is a variable declaration error)"

"End of procedure"

GXcomment run line: 0 proc: self method: false; p1:'__init_error_handler'; p2:'Callout to set up C++ error handler'; condEvaluate!
"Procedure class definition:
 ==========================="
((Smalltalk at: #GXfiletype ifAbsent: [ nil ]) isKindOf: Function class) ifTrue: [ ('Warning: Overriding previously defined procedure ', 'GXfiletype' gtype) prinl ].
Smalltalk at: #GXfiletype put: nil!
Function subclass: 'GXfiletype'
       instanceVariableNames: ''
       classVariableNames: ''
       poolDictionaries: ''
       category: 'Genie Procedure'
!

"HEADER
 ------"

"parameters
 ----------"

GXfiletype

addInstVarName: #gXfile;
compile:
'gXfile: aVar
	aVar notNil ifTrue:[gXfile _ self typeCheck: aVar shouldBe: GXString paramName: ''gXfile'']';
compile:
'p1: aVar
	aVar notNil ifTrue:[gXfile _ self typeCheck: aVar shouldBe: GXString paramName: ''gXfile'']'!

"result
 ------"

GXfiletype

addInstVarName: #gXtype
!

"Local variables
 ---------------"

GXfiletype

addInstVarName: #gXfile
!

!GXfiletype methodsFor: 'initializing'!

initVars
	gXtype _ ''.
	gXfile _ (gXUcurrentUinputUsource)
!!

"BODY
 ----"

!GXfiletype methodsFor: 'running'!

evaluate
 gXfile isNil ifTrue: [^'FILETYPE: parameter FILE must be given' prenl] .
GXcallout run line: 0 proc: self method: false; gXfacility:'extFortran'; gXfunction:14; condEvaluate.
self up.
^gXtype
!!
nil! "(fix for smalltalk bug, this line is ignored if there is a variable declaration error)"

"End of procedure"

GXcomment run line: 0 proc: self method: false; p1:'filetype'; p2:'Return the type of a file'; condEvaluate!
"Procedure class definition:
 ==========================="
((Smalltalk at: #GXUgroupbinsUarrays ifAbsent: [ nil ]) isKindOf: Function class) ifTrue: [ ('Warning: Overriding previously defined procedure ', 'GXUgroupbinsUarrays' gtype) prinl ].
Smalltalk at: #GXUgroupbinsUarrays put: nil!
Function subclass: 'GXUgroupbinsUarrays'
       instanceVariableNames: ''
       classVariableNames: ''
       poolDictionaries: ''
       category: 'Genie Procedure'
!

"HEADER
 ------"

"parameters
 ----------"

GXUgroupbinsUarrays

addInstVarName: #gXxarray;
compile:
'gXxarray: aVar
	aVar notNil ifTrue:[gXxarray _ self typeCheck: aVar shouldBe: GXRealarray paramName: ''gXxarray'']';
compile:
'p1: aVar
	aVar notNil ifTrue:[gXxarray _ self typeCheck: aVar shouldBe: GXRealarray paramName: ''gXxarray'']';

addInstVarName: #gXyarray;
compile:
'gXyarray: aVar
	aVar notNil ifTrue:[gXyarray _ self typeCheck: aVar shouldBe: GXRealarray paramName: ''gXyarray'']';
compile:
'p2: aVar
	aVar notNil ifTrue:[gXyarray _ self typeCheck: aVar shouldBe: GXRealarray paramName: ''gXyarray'']';

addInstVarName: #gXearray;
compile:
'gXearray: aVar
	aVar notNil ifTrue:[gXearray _ self typeCheck: aVar shouldBe: GXRealarray paramName: ''gXearray'']';
compile:
'p3: aVar
	aVar notNil ifTrue:[gXearray _ self typeCheck: aVar shouldBe: GXRealarray paramName: ''gXearray'']';

addInstVarName: #gXnbin;
compile:
'gXnbin: aVar
	aVar notNil ifTrue:[gXnbin _ self typeCheck: aVar shouldBe: GXInteger paramName: ''gXnbin'']';
compile:
'p4: aVar
	aVar notNil ifTrue:[gXnbin _ self typeCheck: aVar shouldBe: GXInteger paramName: ''gXnbin'']';

addInstVarName: #gXundef;
compile:
'gXundef: aVar
	aVar notNil ifTrue:[gXundef _ self typeCheck: aVar shouldBe: GXReal paramName: ''gXundef'']';
compile:
'p5: aVar
	aVar notNil ifTrue:[gXundef _ self typeCheck: aVar shouldBe: GXReal paramName: ''gXundef'']'!

"result
 ------"

GXUgroupbinsUarrays

addInstVarName: #gXres
!

"Local variables
 ---------------"

GXUgroupbinsUarrays

addInstVarName: #gXres;

addInstVarName: #gXundef
!

!GXUgroupbinsUarrays methodsFor: 'initializing'!

initVars
	gXres _ (GXfields run line: 0 proc: self method: true; p1:3; condEvaluate).
	gXundef _ 0.000000000000000e000
!!

"BODY
 ----"

!GXUgroupbinsUarrays methodsFor: 'running'!

evaluate

    gXxarray isNil ifTrue: [^'GROUPBINS: paramater XARRAY not specified' prenl].
    gXyarray isNil ifTrue: [^'GROUPBINS: paramater YARRAY not specified' prenl].
    gXearray isNil ifTrue: [^'GROUPBINS: paramater EARRAY not specified' prenl].
    gXnbin isNil ifTrue: [^'GROUPBINS: paramater NBIN not specified' prenl].
    (gXxarray size = (gXyarray size + 1)) 
	ifFalse: ['GROUPBINS: data is not in bins' prenl. ^nil ]
.
GXcallout run line: 0 proc: self method: false; gXfacility:'extTransform'; gXfunction:8; condEvaluate.
(GXgUerrorUhandler run line: 0 proc: self method: true; condEvaluate) ifTrue: [ 
(gXres isKindOf: GXArray) | (gXres isKindOf: GXWorkspace)
    ifTrue:  [ ((gXres) at:#gXx put: (gXU) deepCopy) ]
    ifFalse: [ self error: 'Assignment to array or workspace failed, check that the destination is a structured type'].
(gXres isKindOf: GXArray) | (gXres isKindOf: GXWorkspace)
    ifTrue:  [ ((gXres) at:#gXy put: (gXU) deepCopy) ]
    ifFalse: [ self error: 'Assignment to array or workspace failed, check that the destination is a structured type'].
(gXres isKindOf: GXArray) | (gXres isKindOf: GXWorkspace)
    ifTrue:  [ ((gXres) at:#gXe put: (gXU) deepCopy) ]
    ifFalse: [ self error: 'Assignment to array or workspace failed, check that the destination is a structured type'] ] ifFalse: [ 

	((gXres) at:#gXx put: (gXxarray)).
	((gXres) at:#gXy put: (gXyarray)).
	((gXres) at:#gXe put: (gXearray))
    ] .
self up.
^gXres
!!
nil! "(fix for smalltalk bug, this line is ignored if there is a variable declaration error)"

"End of procedure"

GXcomment run line: 0 proc: self method: false; p1:'_groupbins_arrays'; p2:'Group arrays into a given bin size'; condEvaluate!
"Procedure class definition:
 ==========================="
((Smalltalk at: #GXUgroupbinsUwksp ifAbsent: [ nil ]) isKindOf: Function class) ifTrue: [ ('Warning: Overriding previously defined procedure ', 'GXUgroupbinsUwksp' gtype) prinl ].
Smalltalk at: #GXUgroupbinsUwksp put: nil!
Function subclass: 'GXUgroupbinsUwksp'
       instanceVariableNames: ''
       classVariableNames: ''
       poolDictionaries: ''
       category: 'Genie Procedure'
!

"HEADER
 ------"

"parameters
 ----------"

GXUgroupbinsUwksp

addInstVarName: #gXwksp;
compile:
'gXwksp: aVar
	aVar notNil ifTrue:[gXwksp _ self typeCheck: aVar shouldBe: GXWorkspace paramName: ''gXwksp'']';
compile:
'p1: aVar
	aVar notNil ifTrue:[gXwksp _ self typeCheck: aVar shouldBe: GXWorkspace paramName: ''gXwksp'']';

addInstVarName: #gXnbin;
compile:
'gXnbin: aVar
	aVar notNil ifTrue:[gXnbin _ self typeCheck: aVar shouldBe: GXInteger paramName: ''gXnbin'']';
compile:
'p2: aVar
	aVar notNil ifTrue:[gXnbin _ self typeCheck: aVar shouldBe: GXInteger paramName: ''gXnbin'']';

addInstVarName: #gXundef;
compile:
'gXundef: aVar
	aVar notNil ifTrue:[gXundef _ self typeCheck: aVar shouldBe: GXReal paramName: ''gXundef'']';
compile:
'p3: aVar
	aVar notNil ifTrue:[gXundef _ self typeCheck: aVar shouldBe: GXReal paramName: ''gXundef'']'!

"result
 ------"

GXUgroupbinsUwksp

addInstVarName: #gXres
!

"Local variables
 ---------------"

GXUgroupbinsUwksp

addInstVarName: #gXlres;

addInstVarName: #gXnbin;

addInstVarName: #gXundef!

"Local variables
 ---------------"

GXUgroupbinsUwksp

addInstVarName: #gXyfix;

addInstVarName: #gXefix
!

!GXUgroupbinsUwksp methodsFor: 'initializing'!

initVars
	gXlres _ (gXU).
	gXnbin _ (gXUcurrentUbinning).
	gXundef _ 0.000000000000000e000
!!

"BODY
 ----"

!GXUgroupbinsUwksp methodsFor: 'running'!

evaluate
gXres _ (gXwksp) deepCopy.

    gXwksp isNil ifTrue: [^'GROUP: parameter WKSP must be given' prenl].
    gXnbin isNil ifTrue: [^'GROUP: parameter NBIN must be given' prenl].
    gXyfix _ (GXfix run line: 0 proc: self method: true; p1:((gXwksp) at:#gXy); p2:(gXundef); condEvaluate).
    gXefix _ (GXfix run line: 0 proc: self method: true; p1:((gXwksp) at:#gXe); p2:(gXundef); condEvaluate).
    gXlres _ (GXUgroupbinsUarrays run line: 0 proc: self method: true; 
	p1:((gXwksp) at:#gXx); p2:(gXyfix); p3:(gXefix); p4:(gXnbin); condEvaluate).
    ((gXres) at:#gXx put: ((gXlres) at:#gXx)).    
    ((gXres) at:#gXy put: ((gXlres) at:#gXy)).    
    ((gXres) at:#gXe put: ((gXlres) at:#gXe))
.
self up.
^gXres
!!
nil! "(fix for smalltalk bug, this line is ignored if there is a variable declaration error)"

"End of procedure"

GXcomment run line: 0 proc: self method: false; p1:'_groupbins_wksp'; p2:'group bins of a spectrum workspace'; condEvaluate!
"Procedure class definition:
 ==========================="
((Smalltalk at: #GXgroupbins ifAbsent: [ nil ]) isKindOf: Function class) ifTrue: [ ('Warning: Overriding previously defined procedure ', 'GXgroupbins' gtype) prinl ].
Smalltalk at: #GXgroupbins put: nil!
Function subclass: 'GXgroupbins'
       instanceVariableNames: ''
       classVariableNames: ''
       poolDictionaries: ''
       category: 'Genie Procedure'
!

"HEADER
 ------"

"parameters
 ----------"

GXgroupbins

addInstVarName: #gXp1;
compile:
'gXp1: aVar
	aVar notNil ifTrue:[ gXp1 _ aVar ]';
compile:
'p1: aVar
	aVar notNil ifTrue:[ gXp1 _ aVar ]';

addInstVarName: #gXp2;
compile:
'gXp2: aVar
	aVar notNil ifTrue:[ gXp2 _ aVar ]';
compile:
'p2: aVar
	aVar notNil ifTrue:[ gXp2 _ aVar ]';

addInstVarName: #gXp3;
compile:
'gXp3: aVar
	aVar notNil ifTrue:[ gXp3 _ aVar ]';
compile:
'p3: aVar
	aVar notNil ifTrue:[ gXp3 _ aVar ]';

addInstVarName: #gXp4;
compile:
'gXp4: aVar
	aVar notNil ifTrue:[ gXp4 _ aVar ]';
compile:
'p4: aVar
	aVar notNil ifTrue:[ gXp4 _ aVar ]';

addInstVarName: #gXp5;
compile:
'gXp5: aVar
	aVar notNil ifTrue:[ gXp5 _ aVar ]';
compile:
'p5: aVar
	aVar notNil ifTrue:[ gXp5 _ aVar ]';

addInstVarName: #gXwksp;
compile:
'gXwksp: aVar
	aVar notNil ifTrue:[gXwksp _ self typeCheck: aVar shouldBe: GXWorkspace paramName: ''gXwksp'']';
compile:
'p6: aVar
	aVar notNil ifTrue:[gXwksp _ self typeCheck: aVar shouldBe: GXWorkspace paramName: ''gXwksp'']';

addInstVarName: #gXnbin;
compile:
'gXnbin: aVar
	aVar notNil ifTrue:[gXnbin _ self typeCheck: aVar shouldBe: GXInteger paramName: ''gXnbin'']';
compile:
'p7: aVar
	aVar notNil ifTrue:[gXnbin _ self typeCheck: aVar shouldBe: GXInteger paramName: ''gXnbin'']';

addInstVarName: #gXundef;
compile:
'gXundef: aVar
	aVar notNil ifTrue:[gXundef _ self typeCheck: aVar shouldBe: GXReal paramName: ''gXundef'']';
compile:
'p8: aVar
	aVar notNil ifTrue:[gXundef _ self typeCheck: aVar shouldBe: GXReal paramName: ''gXundef'']';

addInstVarName: #gXxarray;
compile:
'gXxarray: aVar
	aVar notNil ifTrue:[gXxarray _ self typeCheck: aVar shouldBe: GXRealarray paramName: ''gXxarray'']';
compile:
'p9: aVar
	aVar notNil ifTrue:[gXxarray _ self typeCheck: aVar shouldBe: GXRealarray paramName: ''gXxarray'']';

addInstVarName: #gXyarray;
compile:
'gXyarray: aVar
	aVar notNil ifTrue:[gXyarray _ self typeCheck: aVar shouldBe: GXRealarray paramName: ''gXyarray'']';
compile:
'p10: aVar
	aVar notNil ifTrue:[gXyarray _ self typeCheck: aVar shouldBe: GXRealarray paramName: ''gXyarray'']';

addInstVarName: #gXearray;
compile:
'gXearray: aVar
	aVar notNil ifTrue:[gXearray _ self typeCheck: aVar shouldBe: GXRealarray paramName: ''gXearray'']';
compile:
'p11: aVar
	aVar notNil ifTrue:[gXearray _ self typeCheck: aVar shouldBe: GXRealarray paramName: ''gXearray'']'!

"result
 ------"

GXgroupbins

addInstVarName: #gXres
!

"Local variables
 ---------------"

GXgroupbins

addInstVarName: #gXcaf;

addInstVarName: #gXargsok
!

!GXgroupbins methodsFor: 'initializing'!

initVars

!!

"BODY
 ----"

!GXgroupbins methodsFor: 'running'!

evaluate
gXcaf _ (GXcalledUasUfunction run line: 0 proc: self method: true; condEvaluate) deepCopy.

    gXargsok _ false.
    ((gXp1 isKindOf: GXWorkspace) | (gXwksp notNil)) "WKSP=Workspace NBIN=Integer UNDEF=Real"
    ifTrue: 
    [ 
	gXargsok _ true.
	gXp1 notNil ifTrue: [ gXwksp _ gXp1 ].
	gXp2 notNil ifTrue: [ gXnbin _ gXp2 ].
	gXp3 notNil ifTrue: [ gXundef _ gXp3 ].
        gXres _ (GXUgroupbinsUwksp run line: 0 proc: self method: true; p1:(gXwksp); p2:(gXnbin); p3:(gXundef); condEvaluate).
	gXcaf ifFalse: [ gXwksp at: #gXx put: (gXres at: #gXx). 
			 gXwksp at: #gXy put: (gXres at: #gXy).
			 gXwksp at: #gXe put: (gXres at: #gXe) ]
    ].
    ((gXp1 isKindOf: GXRealarray) | (gXxarray notNil)) "XARRAY=Realarray YARRAY=Realarray EARRAY=Realarray NBIN=Integer UNDEF=Real"
    ifTrue:
    [
	gXargsok _ true.
	gXp1 notNil ifTrue: [ gXxarray _ gXp1 ].
	gXp2 notNil ifTrue: [ gXyarray _ gXp2 ].
	gXp3 notNil ifTrue: [ gXearray _ gXp3 ].
	gXp4 notNil ifTrue: [ gXnbin _ gXp4 ].
	gXp5 notNil ifTrue: [ gXundef _ gXp5 ].
	gXres _ (GXUgroupbinsUarrays run line: 0 proc: self method: true; 
	    p1:(gXxarray); p2:(gXyarray); p3:(gXearray); p4:(gXnbin); p5:(gXundef); condEvaluate).
	gXcaf ifFalse: [ gXxarray _ ((gXres) at:#gXx). gXyarray _ ((gXres) at:#gXy). gXearray _ ((gXres) at:#gXe) ]
    ].
    gXargsok ifFalse: [ ^'GROUPBINS: Invalid arguments specified' prenl ]
.
self up.
^gXres
!!
nil! "(fix for smalltalk bug, this line is ignored if there is a variable declaration error)"

"End of procedure"

GXcomment run line: 0 proc: self method: false; p1:'groupbins'; p2:'group bins of a set of arrays or of a spectrum workspace'; condEvaluate!
"Procedure class definition:
 ==========================="
((Smalltalk at: #GXUconstructUfileUname ifAbsent: [ nil ]) isKindOf: Function class) ifTrue: [ ('Warning: Overriding previously defined procedure ', 'GXUconstructUfileUname' gtype) prinl ].
Smalltalk at: #GXUconstructUfileUname put: nil!
Function subclass: 'GXUconstructUfileUname'
       instanceVariableNames: ''
       classVariableNames: ''
       poolDictionaries: ''
       category: 'Genie Procedure'
!

"HEADER
 ------"

"parameters
 ----------"

GXUconstructUfileUname

addInstVarName: #gXfile;
compile:
'gXfile: aVar
	aVar notNil ifTrue:[gXfile _ self typeCheck: aVar shouldBe: GXString paramName: ''gXfile'']';
compile:
'p1: aVar
	aVar notNil ifTrue:[gXfile _ self typeCheck: aVar shouldBe: GXString paramName: ''gXfile'']';

addInstVarName: #gXdisk;
compile:
'gXdisk: aVar
	aVar notNil ifTrue:[gXdisk _ self typeCheck: aVar shouldBe: GXString paramName: ''gXdisk'']';
compile:
'p2: aVar
	aVar notNil ifTrue:[gXdisk _ self typeCheck: aVar shouldBe: GXString paramName: ''gXdisk'']';

addInstVarName: #gXdirectory;
compile:
'gXdirectory: aVar
	aVar notNil ifTrue:[gXdirectory _ self typeCheck: aVar shouldBe: GXString paramName: ''gXdirectory'']';
compile:
'p3: aVar
	aVar notNil ifTrue:[gXdirectory _ self typeCheck: aVar shouldBe: GXString paramName: ''gXdirectory'']';

addInstVarName: #gXinstrument;
compile:
'gXinstrument: aVar
	aVar notNil ifTrue:[gXinstrument _ self typeCheck: aVar shouldBe: GXString paramName: ''gXinstrument'']';
compile:
'p4: aVar
	aVar notNil ifTrue:[gXinstrument _ self typeCheck: aVar shouldBe: GXString paramName: ''gXinstrument'']';

addInstVarName: #gXrunUnumber;
compile:
'gXrunUnumber: aVar
	aVar notNil ifTrue:[gXrunUnumber _ self typeCheck: aVar shouldBe: GXInteger paramName: ''gXrunUnumber'']';
compile:
'p5: aVar
	aVar notNil ifTrue:[gXrunUnumber _ self typeCheck: aVar shouldBe: GXInteger paramName: ''gXrunUnumber'']';

addInstVarName: #gXextension;
compile:
'gXextension: aVar
	aVar notNil ifTrue:[gXextension _ self typeCheck: aVar shouldBe: GXString paramName: ''gXextension'']';
compile:
'p6: aVar
	aVar notNil ifTrue:[gXextension _ self typeCheck: aVar shouldBe: GXString paramName: ''gXextension'']'!

"result
 ------"

GXUconstructUfileUname

addInstVarName: #gXfilename
!

"Local variables
 ---------------"

GXUconstructUfileUname

addInstVarName: #gXrunUnumber!

"Local variables
 ---------------"

GXUconstructUfileUname

addInstVarName: #gXrunUnumberUstring!

"Local variables
 ---------------"

GXUconstructUfileUname

addInstVarName: #gXdisk!

"Local variables
 ---------------"

GXUconstructUfileUname

addInstVarName: #gXdirectory!

"Local variables
 ---------------"

GXUconstructUfileUname

addInstVarName: #gXinstrument!

"Local variables
 ---------------"

GXUconstructUfileUname

addInstVarName: #gXextension!

"Local variables
 ---------------"

GXUconstructUfileUname

addInstVarName: #gXfile!

"Local variables
 ---------------"

GXUconstructUfileUname

addInstVarName: #gXzero
!

!GXUconstructUfileUname methodsFor: 'initializing'!

initVars
	gXrunUnumber _ (gXU).
	gXrunUnumberUstring _ (gXU).
	gXdisk _ (gXUcurrentUdisk).
	gXdirectory _ (gXUcurrentUdirectory).
	gXinstrument _ (gXUcurrentUinstrument).
	gXextension _ (gXUcurrentUextension).
	gXfile _ (gXU).
	gXzero _ '00000'
!!

"BODY
 ----"

!GXUconstructUfileUname methodsFor: 'running'!

evaluate
(GXdefined run line: 0 proc: self method: true; p1:(gXfile); condEvaluate) ifTrue: [ 
gXfilename _ (((gXdisk) + (gXdirectory)) + (gXfile)) deepCopy ] ifFalse: [ (GXdefined run line: 0 proc: self method: true; p1:(gXrunUnumber); condEvaluate) ifTrue: [ 
gXrunUnumberUstring _ (GXasUstring run line: 0 proc: self method: true; p1:(gXrunUnumber); condEvaluate) deepCopy.
gXfilename _ (((((((gXdisk) + (gXdirectory)) + (gXinstrument)) + (GXsubstring run line: 0 proc: self method: true; p1:(gXzero); p2:1; p3:(5 - (GXlength run line: 0 proc: self method: true; p1:(gXrunUnumberUstring); condEvaluate)); condEvaluate)) + (gXrunUnumberUstring)) + '.') + (gXextension)) deepCopy ] ifFalse: [ (GXdefined run line: 0 proc: self method: true; p1:(gXUcurrentUinputUsource); condEvaluate) ifTrue: [ 
gXfilename _ (gXUcurrentUinputUsource) deepCopy ] ifFalse: [ 
GXprinten run line: 0 proc: self method: false; p1:'_CONSTRUCT_FILE_NAME: Error in constructing filename'; condEvaluate.
gXfilename _ (gXU) deepCopy ] ]].
self up.
^gXfilename
!!
nil! "(fix for smalltalk bug, this line is ignored if there is a variable declaration error)"

"End of procedure"

GXcomment run line: 0 proc: self method: false; p1:'_construct_file_name'; p2:'Build a filename from supplied data and current defaults'; condEvaluate!
"Procedure class definition:
 ==========================="
((Smalltalk at: #GXcfn ifAbsent: [ nil ]) isKindOf: Function class) ifTrue: [ ('Warning: Overriding previously defined procedure ', 'GXcfn' gtype) prinl ].
Smalltalk at: #GXcfn put: nil!
Function subclass: 'GXcfn'
       instanceVariableNames: ''
       classVariableNames: ''
       poolDictionaries: ''
       category: 'Genie Procedure'
!

"HEADER
 ------"

"parameters
 ----------"

GXcfn

addInstVarName: #gXp1;
compile:
'gXp1: aVar
	aVar notNil ifTrue:[ gXp1 _ aVar ]';
compile:
'p1: aVar
	aVar notNil ifTrue:[ gXp1 _ aVar ]';

addInstVarName: #gXnumber;
compile:
'gXnumber: aVar
	aVar notNil ifTrue:[gXnumber _ self typeCheck: aVar shouldBe: GXInteger paramName: ''gXnumber'']';
compile:
'p2: aVar
	aVar notNil ifTrue:[gXnumber _ self typeCheck: aVar shouldBe: GXInteger paramName: ''gXnumber'']';

addInstVarName: #gXname;
compile:
'gXname: aVar
	aVar notNil ifTrue:[gXname _ self typeCheck: aVar shouldBe: GXString paramName: ''gXname'']';
compile:
'p3: aVar
	aVar notNil ifTrue:[gXname _ self typeCheck: aVar shouldBe: GXString paramName: ''gXname'']'!

"result
 ------"

GXcfn

addInstVarName: #gXfile
!

!GXcfn methodsFor: 'initializing'!

initVars

!!

"BODY
 ----"

!GXcfn methodsFor: 'running'!

evaluate

    (gXp1 isKindOf: Integer) ifTrue: [ gXnumber _ gXp1 ].
    (gXp1 isKindOf: String) ifTrue: [ gXname _ gXp1 ]
.
(GXdefined run line: 0 proc: self method: true; p1:(gXname); condEvaluate) ifTrue: [ 
gXfile _ (GXUconstructUfileUname run line: 0 proc: self method: true; gXfile:(gXname); condEvaluate) deepCopy ] ifFalse: [ (GXdefined run line: 0 proc: self method: true; p1:(gXnumber); condEvaluate) ifTrue: [ 
gXfile _ (GXUconstructUfileUname run line: 0 proc: self method: true; gXrunUnumber:(gXnumber); condEvaluate) deepCopy ] ifFalse: [ 
gXfile _ (GXUconstructUfileUname run line: 0 proc: self method: true; condEvaluate) deepCopy ] ].
self up.
^gXfile
!!
nil! "(fix for smalltalk bug, this line is ignored if there is a variable declaration error)"

"End of procedure"

GXcomment run line: 0 proc: self method: false; p1:'cfn'; p2:'Construct a file name from a run number'; condEvaluate!
"Procedure class definition:
 ==========================="
((Smalltalk at: #GXUgetUsnsUspectrum ifAbsent: [ nil ]) isKindOf: Function class) ifTrue: [ ('Warning: Overriding previously defined procedure ', 'GXUgetUsnsUspectrum' gtype) prinl ].
Smalltalk at: #GXUgetUsnsUspectrum put: nil!
Function subclass: 'GXUgetUsnsUspectrum'
       instanceVariableNames: ''
       classVariableNames: ''
       poolDictionaries: ''
       category: 'Genie Procedure'
!

"HEADER
 ------"

"parameters
 ----------"

GXUgetUsnsUspectrum

addInstVarName: #gXimode;
compile:
'gXimode: aVar
	aVar notNil ifTrue:[gXimode _ self typeCheck: aVar shouldBe: GXInteger paramName: ''gXimode'']';
compile:
'p1: aVar
	aVar notNil ifTrue:[gXimode _ self typeCheck: aVar shouldBe: GXInteger paramName: ''gXimode'']';

addInstVarName: #gXspectrumUno;
compile:
'gXspectrumUno: aVar
	aVar notNil ifTrue:[ gXspectrumUno _ aVar ]';
compile:
'p2: aVar
	aVar notNil ifTrue:[ gXspectrumUno _ aVar ]';

addInstVarName: #gXfile;
compile:
'gXfile: aVar
	aVar notNil ifTrue:[gXfile _ self typeCheck: aVar shouldBe: GXString paramName: ''gXfile'']';
compile:
'p3: aVar
	aVar notNil ifTrue:[gXfile _ self typeCheck: aVar shouldBe: GXString paramName: ''gXfile'']'!

"result
 ------"

GXUgetUsnsUspectrum

addInstVarName: #gXspectrum
!

"Local variables
 ---------------"

GXUgetUsnsUspectrum

addInstVarName: #gXstatus!

"Local variables
 ---------------"

GXUgetUsnsUspectrum

addInstVarName: #gXspectrumUno!

"Local variables
 ---------------"

GXUgetUsnsUspectrum

addInstVarName: #gXfile!

"Local variables
 ---------------"

GXUgetUsnsUspectrum

addInstVarName: #gXUioUerrorUcode!

"Local variables
 ---------------"

GXUgetUsnsUspectrum

addInstVarName: #gXUioUerrorUmessage!

"Local variables
 ---------------"

GXUgetUsnsUspectrum

addInstVarName: #gXiarray!

"Local variables
 ---------------"

GXUgetUsnsUspectrum

addInstVarName: #gXtmp
!

!GXUgetUsnsUspectrum methodsFor: 'initializing'!

initVars
	gXstatus _ (gXU).
	gXspectrumUno _ (gXU).
	gXfile _ (gXUcurrentUinputUsource).
	gXUioUerrorUcode _ 0.
	gXUioUerrorUmessage _ (gXU)
!!

"BODY
 ----"

!GXUgetUsnsUspectrum methodsFor: 'running'!

evaluate

    gXfile isNil ifTrue: [ ^'_GET_SNS_SPECTRUM: Parameter FILE not specified' prenl ].
    (gXspectrumUno isKindOf: GXIntegerarray) ifTrue: [ gXiarray _ gXspectrumUno ].
    (gXspectrumUno isKindOf: Interval) ifTrue: [ gXiarray _ gXspectrumUno asIntegerArray ].
    (gXspectrumUno isKindOf: Integer) ifTrue: [ gXiarray _ (GXIntegerarray new: 1) at: 1 put: gXspectrumUno; yourself ]
.

    gXiarray notNil ifTrue: [
    (gXspectrumUno isKindOf: Integer) 
        ifTrue: [ gXspectrum _ GXWorkspace new: 35. 
                  GXcallout run line: 0 proc: self method: false; p1: 'extFortran'; p2: 1; condEvaluate ]
        ifFalse: [ (gXimode = 0) ifTrue: [ GXcallout run line: 0 proc: self method: false; p1: 'extFortran'; p2: 30; condEvaluate ].
                   (gXimode = 1) ifTrue: [ GXcallout run line: 0 proc: self method: false; p1: 'extFortran'; p2: 28; condEvaluate ].
                 ]
    ] ifFalse: [^'_GET_SNS_SPECTRUM: Invalid item type requested' prenl]
.

    (gXspectrum isKindOf: GXWorkspacearray) 
		ifTrue: [ gXtmp _ gXspectrum at: 1 ]
		ifFalse: [ gXtmp _ gXspectrum ].
    gXtmp at: #gXx ifAbsent: [ gXspectrum _ nil ]
.
self up.
^gXspectrum
!!
nil! "(fix for smalltalk bug, this line is ignored if there is a variable declaration error)"

"End of procedure"

GXcomment run line: 0 proc: self method: false; p1:'_get_sns_spectrum'; p2:'Retrieves a spectrum from an instrument data file'; condEvaluate!
"Procedure class definition:
 ==========================="
((Smalltalk at: #GXUgetUsnsUitem ifAbsent: [ nil ]) isKindOf: Function class) ifTrue: [ ('Warning: Overriding previously defined procedure ', 'GXUgetUsnsUitem' gtype) prinl ].
Smalltalk at: #GXUgetUsnsUitem put: nil!
Function subclass: 'GXUgetUsnsUitem'
       instanceVariableNames: ''
       classVariableNames: ''
       poolDictionaries: ''
       category: 'Genie Procedure'
!

"HEADER
 ------"

"parameters
 ----------"

GXUgetUsnsUitem

addInstVarName: #gXimode;
compile:
'gXimode: aVar
	aVar notNil ifTrue:[gXimode _ self typeCheck: aVar shouldBe: GXInteger paramName: ''gXimode'']';
compile:
'p1: aVar
	aVar notNil ifTrue:[gXimode _ self typeCheck: aVar shouldBe: GXInteger paramName: ''gXimode'']';

addInstVarName: #gXitem;
compile:
'gXitem: aVar
	aVar notNil ifTrue:[gXitem _ self typeCheck: aVar shouldBe: GXString paramName: ''gXitem'']';
compile:
'p2: aVar
	aVar notNil ifTrue:[gXitem _ self typeCheck: aVar shouldBe: GXString paramName: ''gXitem'']';

addInstVarName: #gXfile;
compile:
'gXfile: aVar
	aVar notNil ifTrue:[gXfile _ self typeCheck: aVar shouldBe: GXString paramName: ''gXfile'']';
compile:
'p3: aVar
	aVar notNil ifTrue:[gXfile _ self typeCheck: aVar shouldBe: GXString paramName: ''gXfile'']'!

"result
 ------"

GXUgetUsnsUitem

addInstVarName: #gXitemUvalue
!

"Local variables
 ---------------"

GXUgetUsnsUitem

addInstVarName: #gXitemUvalue!

"Local variables
 ---------------"

GXUgetUsnsUitem

addInstVarName: #gXUioUerrorUcode!

"Local variables
 ---------------"

GXUgetUsnsUitem

addInstVarName: #gXUioUerrorUmessage!

"Local variables
 ---------------"

GXUgetUsnsUitem

addInstVarName: #gXitem!

"Local variables
 ---------------"

GXUgetUsnsUitem

addInstVarName: #gXfile
!

!GXUgetUsnsUitem methodsFor: 'initializing'!

initVars
	gXitemUvalue _ (gXU).
	gXUioUerrorUcode _ 0.
	gXUioUerrorUmessage _ (gXU).
	gXitem _ (gXU).
	gXfile _ (gXUcurrentUinputUsource)
!!

"BODY
 ----"

!GXUgetUsnsUitem methodsFor: 'running'!

evaluate

    gXitem isNil ifTrue: [^'_GET_SNS_ITEM: Parameter ITEM not specified' prenl ].
    gXfile isNil ifTrue: [^'_GET_SNS_ITEM: Parameter FILE not specified' prenl ]
.
GXcallout run line: 0 proc: self method: false; gXfacility:'extFortran'; gXfunction:2; condEvaluate.
self up.
^gXitemUvalue
!!
nil! "(fix for smalltalk bug, this line is ignored if there is a variable declaration error)"

"End of procedure"

GXcomment run line: 0 proc: self method: false; p1:'_get_sns_item'; p2:'Retrieves information from an instrument data file'; condEvaluate!
"Procedure class definition:
 ==========================="
((Smalltalk at: #GXUgetUg3Uascii ifAbsent: [ nil ]) isKindOf: Function class) ifTrue: [ ('Warning: Overriding previously defined procedure ', 'GXUgetUg3Uascii' gtype) prinl ].
Smalltalk at: #GXUgetUg3Uascii put: nil!
Function subclass: 'GXUgetUg3Uascii'
       instanceVariableNames: ''
       classVariableNames: ''
       poolDictionaries: ''
       category: 'Genie Procedure'
!

"HEADER
 ------"

"parameters
 ----------"

GXUgetUg3Uascii

addInstVarName: #gXitem;
compile:
'gXitem: aVar
	aVar notNil ifTrue:[ gXitem _ aVar ]';
compile:
'p1: aVar
	aVar notNil ifTrue:[ gXitem _ aVar ]';

addInstVarName: #gXfile;
compile:
'gXfile: aVar
	aVar notNil ifTrue:[gXfile _ self typeCheck: aVar shouldBe: GXString paramName: ''gXfile'']';
compile:
'p2: aVar
	aVar notNil ifTrue:[gXfile _ self typeCheck: aVar shouldBe: GXString paramName: ''gXfile'']'!

"result
 ------"

GXUgetUg3Uascii

addInstVarName: #gXres
!

"Local variables
 ---------------"

GXUgetUg3Uascii

addInstVarName: #gXnblocks;

addInstVarName: #gXfile
!

!GXUgetUg3Uascii methodsFor: 'initializing'!

initVars
	gXfile _ (gXUcurrentUinputUsource)
!!

"BODY
 ----"

!GXUgetUg3Uascii methodsFor: 'running'!

evaluate
GXprintin run line: 0 proc: self method: false; p1:'Reading block '; p2:(gXitem); p3:' from Open Genie ASCII file '; p4:(gXfile); condEvaluate.

    ((gXitem isKindOf: String) | (gXitem isKindOf: Integer)) ifFalse: [ ^'_GET_G3_ASCII: Invalid item type' prenl ].
    gXfile isNil ifTrue: [^'_GET_G3_ASCII: Parameter FILE not specified' prenl ]
.
GXcallout run line: 0 proc: self method: false; gXfacility:'extFortran'; gXfunction:32; condEvaluate.
(GXgUerrorUhandler run line: 0 proc: self method: true; condEvaluate) ifTrue: [ 
gXres _ (gXU) deepCopy ] ifFalse: [ 
GXprintin run line: 0 proc: self method: false; p1:(gXnblocks); p2:' blocks in Open Genie ASCII file '; p3:(gXfile); condEvaluate ] .
self up.
^gXres
!!
nil! "(fix for smalltalk bug, this line is ignored if there is a variable declaration error)"

"End of procedure"

"Procedure class definition:
 ==========================="
((Smalltalk at: #GXUgetUg3Uintermediate ifAbsent: [ nil ]) isKindOf: Function class) ifTrue: [ ('Warning: Overriding previously defined procedure ', 'GXUgetUg3Uintermediate' gtype) prinl ].
Smalltalk at: #GXUgetUg3Uintermediate put: nil!
Function subclass: 'GXUgetUg3Uintermediate'
       instanceVariableNames: ''
       classVariableNames: ''
       poolDictionaries: ''
       category: 'Genie Procedure'
!

"HEADER
 ------"

"parameters
 ----------"

GXUgetUg3Uintermediate

addInstVarName: #gXitem;
compile:
'gXitem: aVar
	aVar notNil ifTrue:[ gXitem _ aVar ]';
compile:
'p1: aVar
	aVar notNil ifTrue:[ gXitem _ aVar ]';

addInstVarName: #gXfile;
compile:
'gXfile: aVar
	aVar notNil ifTrue:[gXfile _ self typeCheck: aVar shouldBe: GXString paramName: ''gXfile'']';
compile:
'p2: aVar
	aVar notNil ifTrue:[gXfile _ self typeCheck: aVar shouldBe: GXString paramName: ''gXfile'']'!

"result
 ------"

GXUgetUg3Uintermediate

addInstVarName: #gXres
!

"Local variables
 ---------------"

GXUgetUg3Uintermediate

addInstVarName: #gXnblocks;

addInstVarName: #gXfile
!

!GXUgetUg3Uintermediate methodsFor: 'initializing'!

initVars
	gXfile _ (gXUcurrentUinputUsource)
!!

"BODY
 ----"

!GXUgetUg3Uintermediate methodsFor: 'running'!

evaluate
GXprintin run line: 0 proc: self method: false; p1:'Reading block '; p2:(gXitem); p3:' from Open Genie intermediate file '; p4:(gXfile); condEvaluate.

    ((gXitem isKindOf: String) | (gXitem isKindOf: Integer)) ifFalse: [ ^'_GET_G3_INTERMEDIATE: Invalid item type' prenl ].
    gXfile isNil ifTrue: [^'_GET_G3_INTERMEDIATE: Parameter FILE not specified' prenl ]
.
GXcallout run line: 0 proc: self method: false; gXfacility:'extFortran'; gXfunction:23; condEvaluate.
(GXgUerrorUhandler run line: 0 proc: self method: true; condEvaluate) ifTrue: [ 
gXres _ (gXU) deepCopy ] ifFalse: [ 
GXprintin run line: 0 proc: self method: false; p1:(gXnblocks); p2:' blocks in Open Genie intermediate file '; p3:(gXfile); condEvaluate ] .
self up.
^gXres
!!
nil! "(fix for smalltalk bug, this line is ignored if there is a variable declaration error)"

"End of procedure"

GXcomment run line: 0 proc: self method: false; p1:'_get_g3_intermediate'; p2:'Read a block from an Open Genie intermediate file'; condEvaluate!
"Procedure class definition:
 ==========================="
((Smalltalk at: #GXUgetUg2Uintermediate ifAbsent: [ nil ]) isKindOf: Function class) ifTrue: [ ('Warning: Overriding previously defined procedure ', 'GXUgetUg2Uintermediate' gtype) prinl ].
Smalltalk at: #GXUgetUg2Uintermediate put: nil!
Function subclass: 'GXUgetUg2Uintermediate'
       instanceVariableNames: ''
       classVariableNames: ''
       poolDictionaries: ''
       category: 'Genie Procedure'
!

"HEADER
 ------"

"parameters
 ----------"

GXUgetUg2Uintermediate

addInstVarName: #gXblock;
compile:
'gXblock: aVar
	aVar notNil ifTrue:[gXblock _ self typeCheck: aVar shouldBe: GXInteger paramName: ''gXblock'']';
compile:
'p1: aVar
	aVar notNil ifTrue:[gXblock _ self typeCheck: aVar shouldBe: GXInteger paramName: ''gXblock'']';

addInstVarName: #gXfile;
compile:
'gXfile: aVar
	aVar notNil ifTrue:[gXfile _ self typeCheck: aVar shouldBe: GXString paramName: ''gXfile'']';
compile:
'p2: aVar
	aVar notNil ifTrue:[gXfile _ self typeCheck: aVar shouldBe: GXString paramName: ''gXfile'']'!

"result
 ------"

GXUgetUg2Uintermediate

addInstVarName: #gXres
!

"Local variables
 ---------------"

GXUgetUg2Uintermediate

addInstVarName: #gXfile;

addInstVarName: #gXres!

"Local variables
 ---------------"

GXUgetUg2Uintermediate

addInstVarName: #gXnblocks
!

!GXUgetUg2Uintermediate methodsFor: 'initializing'!

initVars
	gXfile _ (gXUcurrentUinputUsource).
	gXres _ (GXfields run line: 0 proc: self method: true; p1:40; condEvaluate)
!!

"BODY
 ----"

!GXUgetUg2Uintermediate methodsFor: 'running'!

evaluate

    gXblock isNil ifTrue: [^'_GET_G2_INTERMEDIATE: Parameter BLOCK not specified' prenl ].
    gXfile isNil ifTrue: [^'_GET_G2_INTERMEDIATE: Parameter FILE not specified' prenl ]
.
GXprintin run line: 0 proc: self method: false; p1:'Reading block '; p2:(gXblock); p3:' from Genie II intermediate file '; p4:(gXfile); condEvaluate.
GXcallout run line: 0 proc: self method: false; gXfacility:'extFortran'; gXfunction:21; condEvaluate.
((gXnblocks) = 1 negated) ifTrue: [ 
gXres _ (gXU) deepCopy ] ifFalse: [ 
GXprintin run line: 0 proc: self method: false; p1:(gXnblocks); p2:' blocks in Genie II intermediate file '; p3:(gXfile); condEvaluate ] .
self up.
^gXres
!!
nil! "(fix for smalltalk bug, this line is ignored if there is a variable declaration error)"

"End of procedure"

GXcomment run line: 0 proc: self method: false; p1:'_get_g2_intermediate'; p2:'Read a block from a Genie II intermediate file'; condEvaluate!
"Procedure class definition:
 ==========================="
((Smalltalk at: #GXUgetUsnsUgenericUraw ifAbsent: [ nil ]) isKindOf: Function class) ifTrue: [ ('Warning: Overriding previously defined procedure ', 'GXUgetUsnsUgenericUraw' gtype) prinl ].
Smalltalk at: #GXUgetUsnsUgenericUraw put: nil!
Function subclass: 'GXUgetUsnsUgenericUraw'
       instanceVariableNames: ''
       classVariableNames: ''
       poolDictionaries: ''
       category: 'Genie Procedure'
!

"HEADER
 ------"

"parameters
 ----------"

GXUgetUsnsUgenericUraw

addInstVarName: #gXimode;
compile:
'gXimode: aVar
	aVar notNil ifTrue:[gXimode _ self typeCheck: aVar shouldBe: GXInteger paramName: ''gXimode'']';
compile:
'p1: aVar
	aVar notNil ifTrue:[gXimode _ self typeCheck: aVar shouldBe: GXInteger paramName: ''gXimode'']';

addInstVarName: #gXitem;
compile:
'gXitem: aVar
	aVar notNil ifTrue:[ gXitem _ aVar ]';
compile:
'p2: aVar
	aVar notNil ifTrue:[ gXitem _ aVar ]';

addInstVarName: #gXfile;
compile:
'gXfile: aVar
	aVar notNil ifTrue:[gXfile _ self typeCheck: aVar shouldBe: GXString paramName: ''gXfile'']';
compile:
'p3: aVar
	aVar notNil ifTrue:[gXfile _ self typeCheck: aVar shouldBe: GXString paramName: ''gXfile'']'!

"result
 ------"

GXUgetUsnsUgenericUraw

addInstVarName: #gXvalue
!

"Local variables
 ---------------"

GXUgetUsnsUgenericUraw

addInstVarName: #gXfile;

addInstVarName: #gXvalue
!

!GXUgetUsnsUgenericUraw methodsFor: 'initializing'!

initVars
	gXfile _ (gXUcurrentUinputUsource).
	gXvalue _ (gXU)
!!

"BODY
 ----"

!GXUgetUsnsUgenericUraw methodsFor: 'running'!

evaluate
(GXisUa run line: 0 proc: self method: true; p1:(gXitem); p2:'String'; condEvaluate) ifTrue: [ 
GXprintin run line: 0 proc: self method: false; p1:'GET: Reading item '; p2:(gXitem); p3:' from '; p4:(gXfile); condEvaluate.
gXvalue _ (GXUgetUsnsUitem run line: 0 proc: self method: true; p1:(gXimode); gXitem:(gXitem); gXfile:(gXfile); condEvaluate)  ] ifFalse: [ 
GXprintin run line: 0 proc: self method: false; p1:'GET: Reading spectra from '; p2:(gXfile); condEvaluate.
gXvalue _ (GXUgetUsnsUspectrum run line: 0 proc: self method: true; p1:(gXimode); gXspectrumUno:(gXitem); gXfile:(gXfile); condEvaluate)  ] .
self up.
^gXvalue
!!
nil! "(fix for smalltalk bug, this line is ignored if there is a variable declaration error)"

"End of procedure"

"Procedure class definition:
 ==========================="
((Smalltalk at: #GXUgetUsnsUgeneric ifAbsent: [ nil ]) isKindOf: Function class) ifTrue: [ ('Warning: Overriding previously defined procedure ', 'GXUgetUsnsUgeneric' gtype) prinl ].
Smalltalk at: #GXUgetUsnsUgeneric put: nil!
Function subclass: 'GXUgetUsnsUgeneric'
       instanceVariableNames: ''
       classVariableNames: ''
       poolDictionaries: ''
       category: 'Genie Procedure'
!

"HEADER
 ------"

"parameters
 ----------"

GXUgetUsnsUgeneric

addInstVarName: #gXimode;
compile:
'gXimode: aVar
	aVar notNil ifTrue:[gXimode _ self typeCheck: aVar shouldBe: GXInteger paramName: ''gXimode'']';
compile:
'p1: aVar
	aVar notNil ifTrue:[gXimode _ self typeCheck: aVar shouldBe: GXInteger paramName: ''gXimode'']';

addInstVarName: #gXitem;
compile:
'gXitem: aVar
	aVar notNil ifTrue:[ gXitem _ aVar ]';
compile:
'p2: aVar
	aVar notNil ifTrue:[ gXitem _ aVar ]';

addInstVarName: #gXfile;
compile:
'gXfile: aVar
	aVar notNil ifTrue:[gXfile _ self typeCheck: aVar shouldBe: GXString paramName: ''gXfile'']';
compile:
'p3: aVar
	aVar notNil ifTrue:[gXfile _ self typeCheck: aVar shouldBe: GXString paramName: ''gXfile'']'!

"result
 ------"

GXUgetUsnsUgeneric

addInstVarName: #gXvalue
!

"Local variables
 ---------------"

GXUgetUsnsUgeneric

addInstVarName: #gXvalue;

addInstVarName: #gXfile!

"Local variables
 ---------------"

GXUgetUsnsUgeneric

addInstVarName: #gXtype
!

!GXUgetUsnsUgeneric methodsFor: 'initializing'!

initVars
	gXvalue _ (gXU).
	gXfile _ (gXUcurrentUinputUsource).
	gXtype _ ''
!!

"BODY
 ----"

!GXUgetUsnsUgeneric methodsFor: 'running'!

evaluate
gXtype _ (GXfiletype run line: 0 proc: self method: true; p1:(gXfile); condEvaluate) deepCopy.
((GXdefined run line: 0 proc: self method: true; p1:(gXtype); condEvaluate) not) ifTrue: [ 
GXprinten run line: 0 proc: self method: false; p1:'GET: file does not exist'; condEvaluate.
[nil.
self up.
^gXvalue] value ].
(((gXtype) = '<unknown>')) ifTrue: [ 
GXprinten run line: 0 proc: self method: false; p1:'GET: Unable to determine file type'; condEvaluate.
[nil.
self up.
^gXvalue] value ].
GXprintin run line: 0 proc: self method: false; p1:'GET: '; p2:(gXfile); p3:' is of type '; p4:(gXtype); condEvaluate.
((((gXtype) = 'raw')) | (((gXtype) = 'CRPT'))) ifTrue: [ 
gXvalue _ (GXUgetUsnsUgenericUraw run line: 0 proc: self method: true; p1:(gXimode); p2:(gXitem); p3:(gXfile); condEvaluate)  ].
(((gXtype) = 'G2 intermediate')) ifTrue: [ 
gXvalue _ (GXUgetUg2Uintermediate run line: 0 proc: self method: true; p1:(gXitem); p2:(gXfile); condEvaluate)  ].
(((gXtype) = 'G3 intermediate')) ifTrue: [ 
gXvalue _ (GXUgetUg3Uintermediate run line: 0 proc: self method: true; p1:(gXitem); p2:(gXfile); condEvaluate)  ].
(((gXtype) = 'G3 ASCII')) ifTrue: [ 
gXvalue _ (GXUgetUg3Uascii run line: 0 proc: self method: true; p1:(gXitem); p2:(gXfile); condEvaluate)  ].
self up.
^gXvalue
!!
nil! "(fix for smalltalk bug, this line is ignored if there is a variable declaration error)"

"End of procedure"

"Procedure class definition:
 ==========================="
((Smalltalk at: #GXget ifAbsent: [ nil ]) isKindOf: Function class) ifTrue: [ ('Warning: Overriding previously defined procedure ', 'GXget' gtype) prinl ].
Smalltalk at: #GXget put: nil!
Function subclass: 'GXget'
       instanceVariableNames: ''
       classVariableNames: ''
       poolDictionaries: ''
       category: 'Genie Procedure'
!

"HEADER
 ------"

"qualifiers
 ----------"

GXget

addInstVarName: #gXarray;
compile:
'gXarray
	gXarray _ true';
compile:
'gXarray: aBoolean
	gXarray _ aBoolean'!

"parameters
 ----------"

GXget

addInstVarName: #gXitem;
compile:
'gXitem: aVar
	aVar notNil ifTrue:[ gXitem _ aVar ]';
compile:
'p1: aVar
	aVar notNil ifTrue:[ gXitem _ aVar ]';

addInstVarName: #gXfile;
compile:
'gXfile: aVar
	aVar notNil ifTrue:[gXfile _ self typeCheck: aVar shouldBe: GXString paramName: ''gXfile'']';
compile:
'p2: aVar
	aVar notNil ifTrue:[gXfile _ self typeCheck: aVar shouldBe: GXString paramName: ''gXfile'']'!

"result
 ------"

GXget

addInstVarName: #gXvalue
!

"Local variables
 ---------------"

GXget

addInstVarName: #gXvalue;

addInstVarName: #gXimode!

"Local variables
 ---------------"

GXget

addInstVarName: #gXfile
!

!GXget methodsFor: 'initializing'!

initVars
	gXarray _ false.
	gXimode _ 0.
	gXfile _ (gXUcurrentUinputUsource)
!!

"BODY
 ----"

!GXget methodsFor: 'running'!

evaluate
 
    gXarray ifTrue: [ gXimode _ 1 ]
.
((GXdefined run line: 0 proc: self method: true; p1:(gXfile); condEvaluate) & (GXdefined run line: 0 proc: self method: true; p1:(gXitem); condEvaluate)) ifTrue: [ 
gXvalue _ (GXUgetUsnsUgeneric run line: 0 proc: self method: true; p1:(gXimode); gXitem:(gXitem); gXfile:(gXfile); condEvaluate)  ] ifFalse: [ 
GXprinten run line: 0 proc: self method: false; p1:'GET: File or item details incorrectly specified'; condEvaluate.
GXprintin run line: 0 proc: self method: false; p1:'ITEM = '; p2:(gXitem); p3:', FILE = '; p4:(gXfile); condEvaluate ] .
self up.
^gXvalue
!!
nil! "(fix for smalltalk bug, this line is ignored if there is a variable declaration error)"

"End of procedure"

GXcomment run line: 0 proc: self method: false; p1:'get'; p2:'Retrieve information from an instrument data file or Genie intermediate file'; condEvaluate!
"Procedure class definition:
 ==========================="
((Smalltalk at: #GXset ifAbsent: [ nil ]) isKindOf: Function class) ifTrue: [ ('Warning: Overriding previously defined procedure ', 'GXset' gtype) prinl ].
Smalltalk at: #GXset put: nil!
Function subclass: 'GXset'
       instanceVariableNames: ''
       classVariableNames: ''
       poolDictionaries: ''
       category: 'Genie Procedure'
!

"HEADER
 ------"

"qualifiers
 ----------"

GXset

addInstVarName: #gXdisk;
compile:
'gXdisk
	gXdisk _ true';
compile:
'gXdisk: aBoolean
	gXdisk _ aBoolean';

addInstVarName: #gXdirectory;
compile:
'gXdirectory
	gXdirectory _ true';
compile:
'gXdirectory: aBoolean
	gXdirectory _ aBoolean';

addInstVarName: #gXinstrument;
compile:
'gXinstrument
	gXinstrument _ true';
compile:
'gXinstrument: aBoolean
	gXinstrument _ aBoolean';

addInstVarName: #gXextension;
compile:
'gXextension
	gXextension _ true';
compile:
'gXextension: aBoolean
	gXextension _ aBoolean';

addInstVarName: #gXi;
compile:
'gXi
	gXi _ true';
compile:
'gXi: aBoolean
	gXi _ aBoolean';

addInstVarName: #gXdir;
compile:
'gXdir
	gXdir _ true';
compile:
'gXdir: aBoolean
	gXdir _ aBoolean';

addInstVarName: #gXe;
compile:
'gXe
	gXe _ true';
compile:
'gXe: aBoolean
	gXe _ aBoolean';

addInstVarName: #gXsource;
compile:
'gXsource
	gXsource _ true';
compile:
'gXsource: aBoolean
	gXsource _ aBoolean';

addInstVarName: #gXs;
compile:
'gXs
	gXs _ true';
compile:
'gXs: aBoolean
	gXs _ aBoolean';

addInstVarName: #gXinput;
compile:
'gXinput
	gXinput _ true';
compile:
'gXinput: aBoolean
	gXinput _ aBoolean';

addInstVarName: #gXoutput;
compile:
'gXoutput
	gXoutput _ true';
compile:
'gXoutput: aBoolean
	gXoutput _ aBoolean';

addInstVarName: #gXfile;
compile:
'gXfile
	gXfile _ true';
compile:
'gXfile: aBoolean
	gXfile _ aBoolean';

addInstVarName: #gXinst;
compile:
'gXinst
	gXinst _ true';
compile:
'gXinst: aBoolean
	gXinst _ aBoolean';

addInstVarName: #gXext;
compile:
'gXext
	gXext _ true';
compile:
'gXext: aBoolean
	gXext _ aBoolean'!

"parameters
 ----------"

GXset

addInstVarName: #gXvalue;
compile:
'gXvalue: aVar
	aVar notNil ifTrue:[ gXvalue _ aVar ]';
compile:
'p1: aVar
	aVar notNil ifTrue:[ gXvalue _ aVar ]'!

"result
 ------"

GXset

addInstVarName: #gXoldUvalue
!

"Local variables
 ---------------"

GXset

addInstVarName: #gXvalueUname;

addInstVarName: #gXsvalue
!

!GXset methodsFor: 'initializing'!

initVars
	gXdisk _ false.
	gXdirectory _ false.
	gXinstrument _ false.
	gXextension _ false.
	gXi _ false.
	gXdir _ false.
	gXe _ false.
	gXsource _ false.
	gXs _ false.
	gXinput _ false.
	gXoutput _ false.
	gXfile _ false.
	gXinst _ false.
	gXext _ false
!!

"BODY
 ----"

!GXset methodsFor: 'running'!

evaluate
gXsvalue _ (GXasUstring run line: 0 proc: self method: true; p1:(gXvalue); condEvaluate) deepCopy.
(gXdisk) ifTrue: [ 
gXoldUvalue _ (gXUcurrentUdisk) deepCopy.
gXUcurrentUdisk _ (gXsvalue) deepCopy.
gXvalueUname _ 'Default disk: ' deepCopy ] ifFalse: [ (((gXdir) | (gXdirectory))) ifTrue: [ 
gXoldUvalue _ (gXUcurrentUdirectory) deepCopy.
gXUcurrentUdirectory _ (gXsvalue) deepCopy.
gXvalueUname _ 'Default directory: ' deepCopy ] ifFalse: [ ((((gXi) | (gXinstrument)) | (gXinst))) ifTrue: [ 
gXoldUvalue _ (gXUcurrentUinstrument) deepCopy.
gXUcurrentUinstrument _ (gXsvalue) deepCopy.
gXvalueUname _ 'Default instrument: ' deepCopy ] ifFalse: [ ((((gXe) | (gXextension)) | (gXext))) ifTrue: [ 
gXoldUvalue _ (gXUcurrentUextension) deepCopy.
(((GXsubstring run line: 0 proc: self method: true; p1:(gXsvalue); p2:1; p3:1; condEvaluate) = '.')) ifTrue: [ 
gXUcurrentUextension _ (GXsubstring run line: 0 proc: self method: true; p1:(gXsvalue); p2:2; condEvaluate) deepCopy ] ifFalse: [ 
gXUcurrentUextension _ (gXsvalue) deepCopy ] .
gXvalueUname _ 'Default filename extension: .' deepCopy ] ifFalse: [ ((gXinput)) ifTrue: [ 
gXoldUvalue _ (gXUcurrentUinputUsource) deepCopy.
gXUcurrentUinputUsource _ (gXsvalue) deepCopy.
gXvalueUname _ 'Default input: ' deepCopy ] ifFalse: [ ((gXoutput)) ifTrue: [ 
gXoldUvalue _ (gXUcurrentUoutputUsource) deepCopy.
gXUcurrentUoutputUsource _ (gXsvalue) deepCopy.
gXvalueUname _ 'Default output: ' deepCopy ] ifFalse: [ (((gXsource) | (gXfile))) ifTrue: [ 
gXoldUvalue _ (gXUcurrentUinputUsource) deepCopy.
gXUcurrentUinputUsource _ (gXsvalue) deepCopy.
gXvalueUname _ 'Default input: ' deepCopy ] ifFalse: [ 
GXprinten run line: 0 proc: self method: false; p1:'SET: Invalid or missing qualifier - please check syntax with SET/HELP'; condEvaluate ] ]]]]]].
(GXcalledUasUfunction run line: 0 proc: self method: true; condEvaluate) not ifTrue: [ 
GXprintin run line: 0 proc: self method: false; p1:(gXvalueUname); p2:' '; p3:(gXsvalue); condEvaluate ].
self up.
^gXoldUvalue
!!
nil! "(fix for smalltalk bug, this line is ignored if there is a variable declaration error)"

"End of procedure"

GXcomment run line: 0 proc: self method: false; p1:'set'; p2:'Set various defaults'; condEvaluate!
"Procedure class definition:
 ==========================="
((Smalltalk at: #GXbracket ifAbsent: [ nil ]) isKindOf: Function class) ifTrue: [ ('Warning: Overriding previously defined procedure ', 'GXbracket' gtype) prinl ].
Smalltalk at: #GXbracket put: nil!
Function subclass: 'GXbracket'
       instanceVariableNames: ''
       classVariableNames: ''
       poolDictionaries: ''
       category: 'Genie Procedure'
!

"HEADER
 ------"

"parameters
 ----------"

GXbracket

addInstVarName: #gXxarray;
compile:
'gXxarray: aVar
	aVar notNil ifTrue:[gXxarray _ self typeCheck: aVar shouldBe: GXRealarray paramName: ''gXxarray'']';
compile:
'p1: aVar
	aVar notNil ifTrue:[gXxarray _ self typeCheck: aVar shouldBe: GXRealarray paramName: ''gXxarray'']';

addInstVarName: #gXxval;
compile:
'gXxval: aVar
	aVar notNil ifTrue:[gXxval _ self typeCheck: aVar shouldBe: GXReal paramName: ''gXxval'']';
compile:
'p2: aVar
	aVar notNil ifTrue:[gXxval _ self typeCheck: aVar shouldBe: GXReal paramName: ''gXxval'']'!

"result
 ------"

GXbracket

addInstVarName: #gXimin
!

!GXbracket methodsFor: 'initializing'!

initVars

!!

"BODY
 ----"

!GXbracket methodsFor: 'running'!

evaluate

    gXxarray isNil ifTrue: [^'BRACKET: must specify parameter XARRAY' prenl].
    gXxval isNil ifTrue: [^'BRACKET: must specify parameter XVAL' prenl]
.
GXcallout run line: 0 proc: self method: false; gXfacility:'extTransform'; gXfunction:14; condEvaluate.
self up.
^gXimin
!!
nil! "(fix for smalltalk bug, this line is ignored if there is a variable declaration error)"

"End of procedure"

GXcomment run line: 0 proc: self method: false; p1:'bracket'; p2:'Bracket a number in a real array'; condEvaluate!
"Procedure class definition:
 ==========================="
((Smalltalk at: #GXUbracket ifAbsent: [ nil ]) isKindOf: Function class) ifTrue: [ ('Warning: Overriding previously defined procedure ', 'GXUbracket' gtype) prinl ].
Smalltalk at: #GXUbracket put: nil!
Function subclass: 'GXUbracket'
       instanceVariableNames: ''
       classVariableNames: ''
       poolDictionaries: ''
       category: 'Genie Procedure'
!

"HEADER
 ------"

"parameters
 ----------"

GXUbracket

addInstVarName: #gXxval;
compile:
'gXxval: aVar
	aVar notNil ifTrue:[ gXxval _ aVar ]';
compile:
'p1: aVar
	aVar notNil ifTrue:[ gXxval _ aVar ]';

addInstVarName: #gXxarray;
compile:
'gXxarray: aVar
	aVar notNil ifTrue:[gXxarray _ self typeCheck: aVar shouldBe: GXRealarray paramName: ''gXxarray'']';
compile:
'p2: aVar
	aVar notNil ifTrue:[gXxarray _ self typeCheck: aVar shouldBe: GXRealarray paramName: ''gXxarray'']';

addInstVarName: #gXdefault;
compile:
'gXdefault: aVar
	aVar notNil ifTrue:[gXdefault _ self typeCheck: aVar shouldBe: GXInteger paramName: ''gXdefault'']';
compile:
'p3: aVar
	aVar notNil ifTrue:[gXdefault _ self typeCheck: aVar shouldBe: GXInteger paramName: ''gXdefault'']';

addInstVarName: #gXoffset;
compile:
'gXoffset: aVar
	aVar notNil ifTrue:[gXoffset _ self typeCheck: aVar shouldBe: GXInteger paramName: ''gXoffset'']';
compile:
'p4: aVar
	aVar notNil ifTrue:[gXoffset _ self typeCheck: aVar shouldBe: GXInteger paramName: ''gXoffset'']'!

"result
 ------"

GXUbracket

addInstVarName: #gXival
!

!GXUbracket methodsFor: 'initializing'!

initVars

!!

"BODY
 ----"

!GXUbracket methodsFor: 'running'!

evaluate
(GXdefined run line: 0 proc: self method: true; p1:(gXxval); condEvaluate) ifTrue: [ 
(GXisUa run line: 0 proc: self method: true; p1:(gXxval); p2:'INTEGER'; condEvaluate) ifTrue: [ 
gXival _ (gXxval) deepCopy ] ifFalse: [ 
gXival _ (GXbracket run line: 0 proc: self method: true; p1:(gXxarray); p2:(gXxval); condEvaluate) deepCopy.
(((gXival) = 1 negated)) not ifTrue: [ 
gXival _ ((gXival) + (gXoffset)) deepCopy ] ifFalse: [ 
gXival _ (gXdefault) deepCopy ]  ]  ] ifFalse: [ 
gXival _ (gXdefault) deepCopy ] .
self up.
^gXival
!!
nil! "(fix for smalltalk bug, this line is ignored if there is a variable declaration error)"

"End of procedure"

"Procedure class definition:
 ==========================="
((Smalltalk at: #GXscatmode ifAbsent: [ nil ]) isKindOf: Function class) ifTrue: [ ('Warning: Overriding previously defined procedure ', 'GXscatmode' gtype) prinl ].
Smalltalk at: #GXscatmode put: nil!
Function subclass: 'GXscatmode'
       instanceVariableNames: ''
       classVariableNames: ''
       poolDictionaries: ''
       category: 'Genie Procedure'
!

"HEADER
 ------"

"parameters
 ----------"

GXscatmode

addInstVarName: #gXemode;
compile:
'gXemode: aVar
	aVar notNil ifTrue:[gXemode _ self typeCheck: aVar shouldBe: GXInteger paramName: ''gXemode'']';
compile:
'p1: aVar
	aVar notNil ifTrue:[gXemode _ self typeCheck: aVar shouldBe: GXInteger paramName: ''gXemode'']'!

"result
 ------"

GXscatmode

addInstVarName: #gXsmode
!

"Local variables
 ---------------"

GXscatmode

addInstVarName: #gXemode
!

!GXscatmode methodsFor: 'initializing'!

initVars
	gXemode _ 1 negated
!!

"BODY
 ----"

!GXscatmode methodsFor: 'running'!

evaluate
((gXemode) = 0) ifTrue: [ 
gXsmode _ 'elastic' deepCopy ] ifFalse: [ ((gXemode) = 1) ifTrue: [ 
gXsmode _ 'inelastic (direct geometry)' deepCopy ] ifFalse: [ ((gXemode) = 2) ifTrue: [ 
gXsmode _ 'inelastic (indirect geometry)' deepCopy ] ifFalse: [ 
gXsmode _ (gXU) deepCopy.
GXprinten run line: 0 proc: self method: false; p1:'SCATMODE: absent or invalid mode'; condEvaluate ] ]].
self up.
^gXsmode
!!
nil! "(fix for smalltalk bug, this line is ignored if there is a variable declaration error)"

"End of procedure"

GXcomment run line: 0 proc: self method: false; p1:'scatmode'; p2:'Return the neutron scattering mode from an integer (0, 1 or 2)'; condEvaluate!
"Procedure class definition:
 ==========================="
((Smalltalk at: #GXUsetpar ifAbsent: [ nil ]) isKindOf: Function class) ifTrue: [ ('Warning: Overriding previously defined procedure ', 'GXUsetpar' gtype) prinl ].
Smalltalk at: #GXUsetpar put: nil!
Function subclass: 'GXUsetpar'
       instanceVariableNames: ''
       classVariableNames: ''
       poolDictionaries: ''
       category: 'Genie Procedure'
!

"HEADER
 ------"

"parameters
 ----------"

GXUsetpar

addInstVarName: #gXwksp;
compile:
'gXwksp: aVar
	aVar notNil ifTrue:[gXwksp _ self typeCheck: aVar shouldBe: GXWorkspace paramName: ''gXwksp'']';
compile:
'p1: aVar
	aVar notNil ifTrue:[gXwksp _ self typeCheck: aVar shouldBe: GXWorkspace paramName: ''gXwksp'']';

addInstVarName: #gXl1;
compile:
'gXl1: aVar
	aVar notNil ifTrue:[gXl1 _ self typeCheck: aVar shouldBe: GXReal paramName: ''gXl1'']';
compile:
'p2: aVar
	aVar notNil ifTrue:[gXl1 _ self typeCheck: aVar shouldBe: GXReal paramName: ''gXl1'']';

addInstVarName: #gXl2;
compile:
'gXl2: aVar
	aVar notNil ifTrue:[gXl2 _ self typeCheck: aVar shouldBe: GXReal paramName: ''gXl2'']';
compile:
'p3: aVar
	aVar notNil ifTrue:[gXl2 _ self typeCheck: aVar shouldBe: GXReal paramName: ''gXl2'']';

addInstVarName: #gXtt;
compile:
'gXtt: aVar
	aVar notNil ifTrue:[gXtt _ self typeCheck: aVar shouldBe: GXReal paramName: ''gXtt'']';
compile:
'p4: aVar
	aVar notNil ifTrue:[gXtt _ self typeCheck: aVar shouldBe: GXReal paramName: ''gXtt'']';

addInstVarName: #gXrad;
compile:
'gXrad: aVar
	aVar notNil ifTrue:[gXrad _ self typeCheck: aVar shouldBe: GXInteger paramName: ''gXrad'']';
compile:
'p5: aVar
	aVar notNil ifTrue:[gXrad _ self typeCheck: aVar shouldBe: GXInteger paramName: ''gXrad'']';

addInstVarName: #gXem;
compile:
'gXem: aVar
	aVar notNil ifTrue:[gXem _ self typeCheck: aVar shouldBe: GXInteger paramName: ''gXem'']';
compile:
'p6: aVar
	aVar notNil ifTrue:[gXem _ self typeCheck: aVar shouldBe: GXInteger paramName: ''gXem'']';

addInstVarName: #gXef;
compile:
'gXef: aVar
	aVar notNil ifTrue:[gXef _ self typeCheck: aVar shouldBe: GXReal paramName: ''gXef'']';
compile:
'p7: aVar
	aVar notNil ifTrue:[gXef _ self typeCheck: aVar shouldBe: GXReal paramName: ''gXef'']';

addInstVarName: #gXd;
compile:
'gXd: aVar
	aVar notNil ifTrue:[gXd _ self typeCheck: aVar shouldBe: GXReal paramName: ''gXd'']';
compile:
'p8: aVar
	aVar notNil ifTrue:[gXd _ self typeCheck: aVar shouldBe: GXReal paramName: ''gXd'']';

addInstVarName: #gXt;
compile:
'gXt: aVar
	aVar notNil ifTrue:[gXt _ self typeCheck: aVar shouldBe: GXString paramName: ''gXt'']';
compile:
'p9: aVar
	aVar notNil ifTrue:[gXt _ self typeCheck: aVar shouldBe: GXString paramName: ''gXt'']'!

"result
 ------"

GXUsetpar

addInstVarName: #gXwksp
!

"Local variables
 ---------------"

GXUsetpar

addInstVarName: #gXttd!

"Local variables
 ---------------"

GXUsetpar

addInstVarName: #gXttr!

"Local variables
 ---------------"

GXUsetpar

addInstVarName: #gXpi
!

!GXUsetpar methodsFor: 'initializing'!

initVars
	gXttd _ 0.000000000000000e000.
	gXttr _ 0.000000000000000e000.
	gXpi _ 3.141592653589793e000
!!

"BODY
 ----"

!GXUsetpar methodsFor: 'running'!

evaluate
(GXdefined run line: 0 proc: self method: true; p1:(gXl1); condEvaluate) ifTrue: [ 
(gXwksp isKindOf: GXArray) | (gXwksp isKindOf: GXWorkspace)
    ifTrue:  [ ((gXwksp) at:#gXl1 put: (gXl1) deepCopy) ]
    ifFalse: [ self error: 'Assignment to array or workspace failed, check that the destination is a structured type'].
GXprintin run line: 0 proc: self method: false; p1:'Principle flight path L1 set to '; p2:(gXl1); p3:' metres'; condEvaluate ].
(GXdefined run line: 0 proc: self method: true; p1:(gXl2); condEvaluate) ifTrue: [ 
(gXwksp isKindOf: GXArray) | (gXwksp isKindOf: GXWorkspace)
    ifTrue:  [ ((gXwksp) at:#gXl2 put: (gXl2) deepCopy) ]
    ifFalse: [ self error: 'Assignment to array or workspace failed, check that the destination is a structured type'].
GXprintin run line: 0 proc: self method: false; p1:'Secondary flight path L2 set to '; p2:(gXl2); p3:' metres'; condEvaluate ].
(GXdefined run line: 0 proc: self method: true; p1:(gXtt); condEvaluate) ifTrue: [ 
((gXrad) = 0) ifTrue: [ 
gXttd _ (gXtt) deepCopy.
gXttr _ (((gXtt) * (gXpi)) / 1.800000000000000e002) deepCopy ] ifFalse: [ 
gXttr _ (gXtt) deepCopy.
gXttd _ (((gXtt) * 1.800000000000000e002) / (gXpi)) deepCopy ] .
(gXwksp isKindOf: GXArray) | (gXwksp isKindOf: GXWorkspace)
    ifTrue:  [ ((gXwksp) at:#gXtwotheta put: (gXttd) deepCopy) ]
    ifFalse: [ self error: 'Assignment to array or workspace failed, check that the destination is a structured type'].
GXprintin run line: 0 proc: self method: false; p1:'Scattering angle (2*THETA) set to '; p2:(gXttd); p3:' degrees ('; p4:(gXttr); p5:' radians)'; condEvaluate ].
(GXdefined run line: 0 proc: self method: true; p1:(gXem); condEvaluate) ifTrue: [ 
(gXwksp isKindOf: GXArray) | (gXwksp isKindOf: GXWorkspace)
    ifTrue:  [ ((gXwksp) at:#gXemode put: (gXem) deepCopy) ]
    ifFalse: [ self error: 'Assignment to array or workspace failed, check that the destination is a structured type'].
GXprintin run line: 0 proc: self method: false; p1:'Energy mode EMODE set to '; p2:(gXem); p3:' ('; p4:(GXscatmode run line: 0 proc: self method: true; p1:(gXem); condEvaluate); p5:')'; condEvaluate ].
(GXdefined run line: 0 proc: self method: true; p1:(gXef); condEvaluate) ifTrue: [ 
(gXwksp isKindOf: GXArray) | (gXwksp isKindOf: GXWorkspace)
    ifTrue:  [ ((gXwksp) at:#gXefixed put: (gXef) deepCopy) ]
    ifFalse: [ self error: 'Assignment to array or workspace failed, check that the destination is a structured type'].
GXprintin run line: 0 proc: self method: false; p1:'Value of fixed energy EFIXED set to '; p2:(gXef); p3:' meV'; condEvaluate ].
(GXdefined run line: 0 proc: self method: true; p1:(gXd); condEvaluate) ifTrue: [ 
(gXwksp isKindOf: GXArray) | (gXwksp isKindOf: GXWorkspace)
    ifTrue:  [ ((gXwksp) at:#gXdelta put: (gXd) deepCopy) ]
    ifFalse: [ self error: 'Assignment to array or workspace failed, check that the destination is a structured type'].
GXprintin run line: 0 proc: self method: false; p1:'Hold off DELTA set to '; p2:(gXd); p3:' microseconds'; condEvaluate ].
(GXdefined run line: 0 proc: self method: true; p1:(gXt); condEvaluate) ifTrue: [ 
(gXwksp isKindOf: GXArray) | (gXwksp isKindOf: GXWorkspace)
    ifTrue:  [ ((gXwksp) at:#gXtitle put: (gXt) deepCopy) ]
    ifFalse: [ self error: 'Assignment to array or workspace failed, check that the destination is a structured type'].
GXprintin run line: 0 proc: self method: false; p1:'workspace title now: '; p2:(gXt); condEvaluate ].
self up.
^gXwksp
!!
nil! "(fix for smalltalk bug, this line is ignored if there is a variable declaration error)"

"End of procedure"

GXcomment run line: 0 proc: self method: false; p1:'_setpar'; p2:'Set workspace parameters'; condEvaluate!
"Procedure class definition:
 ==========================="
((Smalltalk at: #GXsetpar ifAbsent: [ nil ]) isKindOf: Function class) ifTrue: [ ('Warning: Overriding previously defined procedure ', 'GXsetpar' gtype) prinl ].
Smalltalk at: #GXsetpar put: nil!
Function subclass: 'GXsetpar'
       instanceVariableNames: ''
       classVariableNames: ''
       poolDictionaries: ''
       category: 'Genie Procedure'
!

"HEADER
 ------"

"qualifiers
 ----------"

GXsetpar

addInstVarName: #gXr;
compile:
'gXr
	gXr _ true';
compile:
'gXr: aBoolean
	gXr _ aBoolean'!

"parameters
 ----------"

GXsetpar

addInstVarName: #gXwksp;
compile:
'gXwksp: aVar
	aVar notNil ifTrue:[gXwksp _ self typeCheck: aVar shouldBe: GXWorkspace paramName: ''gXwksp'']';
compile:
'p1: aVar
	aVar notNil ifTrue:[gXwksp _ self typeCheck: aVar shouldBe: GXWorkspace paramName: ''gXwksp'']';

addInstVarName: #gXl1;
compile:
'gXl1: aVar
	aVar notNil ifTrue:[gXl1 _ self typeCheck: aVar shouldBe: GXReal paramName: ''gXl1'']';
compile:
'p2: aVar
	aVar notNil ifTrue:[gXl1 _ self typeCheck: aVar shouldBe: GXReal paramName: ''gXl1'']';

addInstVarName: #gXl2;
compile:
'gXl2: aVar
	aVar notNil ifTrue:[gXl2 _ self typeCheck: aVar shouldBe: GXReal paramName: ''gXl2'']';
compile:
'p3: aVar
	aVar notNil ifTrue:[gXl2 _ self typeCheck: aVar shouldBe: GXReal paramName: ''gXl2'']';

addInstVarName: #gXtt;
compile:
'gXtt: aVar
	aVar notNil ifTrue:[gXtt _ self typeCheck: aVar shouldBe: GXReal paramName: ''gXtt'']';
compile:
'p4: aVar
	aVar notNil ifTrue:[gXtt _ self typeCheck: aVar shouldBe: GXReal paramName: ''gXtt'']';

addInstVarName: #gXem;
compile:
'gXem: aVar
	aVar notNil ifTrue:[gXem _ self typeCheck: aVar shouldBe: GXInteger paramName: ''gXem'']';
compile:
'p5: aVar
	aVar notNil ifTrue:[gXem _ self typeCheck: aVar shouldBe: GXInteger paramName: ''gXem'']';

addInstVarName: #gXef;
compile:
'gXef: aVar
	aVar notNil ifTrue:[gXef _ self typeCheck: aVar shouldBe: GXReal paramName: ''gXef'']';
compile:
'p6: aVar
	aVar notNil ifTrue:[gXef _ self typeCheck: aVar shouldBe: GXReal paramName: ''gXef'']';

addInstVarName: #gXd;
compile:
'gXd: aVar
	aVar notNil ifTrue:[gXd _ self typeCheck: aVar shouldBe: GXReal paramName: ''gXd'']';
compile:
'p7: aVar
	aVar notNil ifTrue:[gXd _ self typeCheck: aVar shouldBe: GXReal paramName: ''gXd'']';

addInstVarName: #gXt;
compile:
'gXt: aVar
	aVar notNil ifTrue:[gXt _ self typeCheck: aVar shouldBe: GXString paramName: ''gXt'']';
compile:
'p8: aVar
	aVar notNil ifTrue:[gXt _ self typeCheck: aVar shouldBe: GXString paramName: ''gXt'']'!

"result
 ------"

GXsetpar

addInstVarName: #gXwksp
!

"Local variables
 ---------------"

GXsetpar

addInstVarName: #gXrad
!

!GXsetpar methodsFor: 'initializing'!

initVars
	gXr _ false.
	gXrad _ 0
!!

"BODY
 ----"

!GXsetpar methodsFor: 'running'!

evaluate
(gXr) ifTrue: [ 
gXrad _ 1 deepCopy ].
(GXdefined run line: 0 proc: self method: true; p1:(gXwksp); condEvaluate) ifTrue: [ 
gXwksp _ (GXUsetpar run line: 0 proc: self method: true; p1:(gXwksp); p2:(gXl1); p3:(gXl2); p4:(gXtt); p5:(gXrad); p6:(gXem); p7:(gXef); p8:(gXd); p9:(gXt); condEvaluate) deepCopy ] ifFalse: [ 
GXprinten run line: 0 proc: self method: false; p1:'SETPAR: Please specify a workspace as the first argument'; condEvaluate ] .
self up.
^gXwksp
!!
nil! "(fix for smalltalk bug, this line is ignored if there is a variable declaration error)"

"End of procedure"

GXcomment run line: 0 proc: self method: false; p1:'setpar'; p2:'Set Genie II workspace parameters'; condEvaluate!
"Procedure class definition:
 ==========================="
((Smalltalk at: #GXUshowUdefaults ifAbsent: [ nil ]) isKindOf: Function class) ifTrue: [ ('Warning: Overriding previously defined procedure ', 'GXUshowUdefaults' gtype) prinl ].
Smalltalk at: #GXUshowUdefaults put: nil!
Function subclass: 'GXUshowUdefaults'
       instanceVariableNames: ''
       classVariableNames: ''
       poolDictionaries: ''
       category: 'Genie Procedure'
!

"BODY
 ----"

!GXUshowUdefaults methodsFor: 'running'!

evaluate
GXprintn run line: 0 proc: self method: false; p1:'Current default disk = '; p2:(gXUcurrentUdisk); condEvaluate.
GXprintn run line: 0 proc: self method: false; p1:'Current default directory = '; p2:(gXUcurrentUdirectory); condEvaluate.
GXprintn run line: 0 proc: self method: false; p1:'Current default instrument = '; p2:(gXUcurrentUinstrument); condEvaluate.
GXprintn run line: 0 proc: self method: false; p1:'Current default extension = '; p2:(gXUcurrentUextension); condEvaluate.
GXprintn run line: 0 proc: self method: false; p1:'Current default input = '; p2:(gXUcurrentUinputUsource); condEvaluate.
GXprintn run line: 0 proc: self method: false; p1:'Current default output = '; p2:(gXUcurrentUoutputUsource); condEvaluate.
self up.
^nil
!!
nil! "(fix for smalltalk bug, this line is ignored if there is a variable declaration error)"

"End of procedure"

GXcomment run line: 0 proc: self method: false; p1:'_show_defaults'; p2:'show defaults used in building a filename'; condEvaluate!
"Procedure class definition:
 ==========================="
((Smalltalk at: #GXUshowUpar ifAbsent: [ nil ]) isKindOf: Function class) ifTrue: [ ('Warning: Overriding previously defined procedure ', 'GXUshowUpar' gtype) prinl ].
Smalltalk at: #GXUshowUpar put: nil!
Function subclass: 'GXUshowUpar'
       instanceVariableNames: ''
       classVariableNames: ''
       poolDictionaries: ''
       category: 'Genie Procedure'
!

"HEADER
 ------"

"parameters
 ----------"

GXUshowUpar

addInstVarName: #gXwksp;
compile:
'gXwksp: aVar
	aVar notNil ifTrue:[gXwksp _ self typeCheck: aVar shouldBe: GXWorkspace paramName: ''gXwksp'']';
compile:
'p1: aVar
	aVar notNil ifTrue:[gXwksp _ self typeCheck: aVar shouldBe: GXWorkspace paramName: ''gXwksp'']'
!

"Local variables
 ---------------"

GXUshowUpar

addInstVarName: #gXwksp!

"Local variables
 ---------------"

GXUshowUpar

addInstVarName: #gXistr!

"Local variables
 ---------------"

GXUshowUpar

addInstVarName: #gXi!

"Local variables
 ---------------"

GXUshowUpar

addInstVarName: #gXpi
!

!GXUshowUpar methodsFor: 'initializing'!

initVars
	gXwksp _ (gXU).
	gXistr _ ' '.
	gXi _ 0.
	gXpi _ 3.141592653589793e000
!!

"BODY
 ----"

!GXUshowUpar methodsFor: 'running'!

evaluate
GXprintn run line: 0 proc: self method: false; p1:'Principle flight path L1 = '; p2:((gXwksp) at:#gXl1); p3:' metres'; condEvaluate.
GXprintn run line: 0 proc: self method: false; p1:'Secondary flight path L2 = '; p2:((gXwksp) at:#gXl2); p3:' metres'; condEvaluate.
GXprintn run line: 0 proc: self method: false; p1:'Scattering angle (2*THETA) = '; p2:((gXwksp) at:#gXtwotheta); p3:' degrees ('; p4:((((gXwksp) at:#gXtwotheta) * (gXpi)) / 1.800000000000000e002); p5:' radians)'; condEvaluate.
GXprintn run line: 0 proc: self method: false; p1:'Hold off DELTA = '; p2:((gXwksp) at:#gXdelta); p3:' microseconds'; condEvaluate.
gXi _ ((gXwksp) at:#gXemode) deepCopy.
gXistr _ (GXscatmode run line: 0 proc: self method: true; p1:(gXi); condEvaluate) deepCopy.
GXprintn run line: 0 proc: self method: false; p1:'Energy mode EMODE = '; p2:(gXi); p3:' ('; p4:(gXistr); p5:')'; condEvaluate.
((((gXwksp) at:#gXefixed) = 0.000000000000000e000)) not ifTrue: [ 
GXprintn run line: 0 proc: self method: false; p1:'Value of fixed energy EFIXED = '; p2:((gXwksp) at:#gXefixed); p3:' meV'; condEvaluate ] ifFalse: [ 
GXprintn run line: 0 proc: self method: false; p1:'Value of fixed energy EFIXED is undefined'; condEvaluate ] .
self up.
^nil
!!
nil! "(fix for smalltalk bug, this line is ignored if there is a variable declaration error)"

"End of procedure"

GXcomment run line: 0 proc: self method: false; p1:'_show_par'; p2:'show workspace parameters'; condEvaluate!
"Procedure class definition:
 ==========================="
((Smalltalk at: #GXUshowUrealUarray ifAbsent: [ nil ]) isKindOf: Function class) ifTrue: [ ('Warning: Overriding previously defined procedure ', 'GXUshowUrealUarray' gtype) prinl ].
Smalltalk at: #GXUshowUrealUarray put: nil!
Function subclass: 'GXUshowUrealUarray'
       instanceVariableNames: ''
       classVariableNames: ''
       poolDictionaries: ''
       category: 'Genie Procedure'
!

"HEADER
 ------"

"parameters
 ----------"

GXUshowUrealUarray

addInstVarName: #gXarray;
compile:
'gXarray: aVar
	aVar notNil ifTrue:[gXarray _ self typeCheck: aVar shouldBe: GXRealarray paramName: ''gXarray'']';
compile:
'p1: aVar
	aVar notNil ifTrue:[gXarray _ self typeCheck: aVar shouldBe: GXRealarray paramName: ''gXarray'']';

addInstVarName: #gXmin;
compile:
'gXmin: aVar
	aVar notNil ifTrue:[ gXmin _ aVar ]';
compile:
'p2: aVar
	aVar notNil ifTrue:[ gXmin _ aVar ]';

addInstVarName: #gXmax;
compile:
'gXmax: aVar
	aVar notNil ifTrue:[ gXmax _ aVar ]';
compile:
'p3: aVar
	aVar notNil ifTrue:[ gXmax _ aVar ]'
!

"Local variables
 ---------------"

GXUshowUrealUarray

addInstVarName: #gXimin;

addInstVarName: #gXimax
!

!GXUshowUrealUarray methodsFor: 'initializing'!

initVars

!!

"BODY
 ----"

!GXUshowUrealUarray methodsFor: 'running'!

evaluate
(GXisUa run line: 0 proc: self method: true; p1:(gXmin); p2:'Real'; condEvaluate) ifTrue: [ 
gXimin _ (GXUbracket run line: 0 proc: self method: true; p1:(gXmin); p2:(gXarray); p3:1; p4:1; condEvaluate) deepCopy ] ifFalse: [ 
gXimin _ (gXmin) deepCopy ] .
(GXisUa run line: 0 proc: self method: true; p1:(gXmax); p2:'Real'; condEvaluate) ifTrue: [ 
gXimax _ (GXUbracket run line: 0 proc: self method: true; p1:(gXmax); p2:(gXarray); p3:(GXlength run line: 0 proc: self method: true; p1:(gXarray); condEvaluate); p4:0; condEvaluate) deepCopy ] ifFalse: [ 
gXimax _ (gXmax) deepCopy ] .
Function new loop: (gXimin) to: (gXimax) by: 1 before: [ :gXi | 
GXprintn run line: 0 proc: self method: false; p1:(gXi); p2:'	'; p3:((gXarray) at:((Array new:1) at: 1 put: (gXi); yourself)); condEvaluate ] 
condition: [false] after: [ :gXi | nil ].
self up.
^nil
!!
nil! "(fix for smalltalk bug, this line is ignored if there is a variable declaration error)"

"End of procedure"

"Procedure class definition:
 ==========================="
((Smalltalk at: #GXUshowUdata ifAbsent: [ nil ]) isKindOf: Function class) ifTrue: [ ('Warning: Overriding previously defined procedure ', 'GXUshowUdata' gtype) prinl ].
Smalltalk at: #GXUshowUdata put: nil!
Function subclass: 'GXUshowUdata'
       instanceVariableNames: ''
       classVariableNames: ''
       poolDictionaries: ''
       category: 'Genie Procedure'
!

"HEADER
 ------"

"parameters
 ----------"

GXUshowUdata

addInstVarName: #gXwksp;
compile:
'gXwksp: aVar
	aVar notNil ifTrue:[gXwksp _ self typeCheck: aVar shouldBe: GXWorkspace paramName: ''gXwksp'']';
compile:
'p1: aVar
	aVar notNil ifTrue:[gXwksp _ self typeCheck: aVar shouldBe: GXWorkspace paramName: ''gXwksp'']';

addInstVarName: #gXxmin;
compile:
'gXxmin: aVar
	aVar notNil ifTrue:[ gXxmin _ aVar ]';
compile:
'p2: aVar
	aVar notNil ifTrue:[ gXxmin _ aVar ]';

addInstVarName: #gXxmax;
compile:
'gXxmax: aVar
	aVar notNil ifTrue:[ gXxmax _ aVar ]';
compile:
'p3: aVar
	aVar notNil ifTrue:[ gXxmax _ aVar ]'
!

"Local variables
 ---------------"

GXUshowUdata

addInstVarName: #gXxmin!

"Local variables
 ---------------"

GXUshowUdata

addInstVarName: #gXxmax!

"Local variables
 ---------------"

GXUshowUdata

addInstVarName: #gXimin!

"Local variables
 ---------------"

GXUshowUdata

addInstVarName: #gXimax!

"Local variables
 ---------------"

GXUshowUdata

addInstVarName: #gXi
!

!GXUshowUdata methodsFor: 'initializing'!

initVars
	gXxmin _ (gXU).
	gXxmax _ (gXU).
	gXimin _ (gXU).
	gXimax _ (gXU)
!!

"BODY
 ----"

!GXUshowUdata methodsFor: 'running'!

evaluate
gXimin _ (GXUbracket run line: 0 proc: self method: true; p1:(gXxmin); p2:((gXwksp) at:#gXx); p3:1; p4:1; condEvaluate) deepCopy.
gXi _ (((gXwksp) at:#gXntc) + 1) deepCopy.
gXimax _ (GXUbracket run line: 0 proc: self method: true; p1:(gXxmax); p2:((gXwksp) at:#gXx); p3:(gXi); p4:0; condEvaluate) deepCopy.
GXprintn run line: 0 proc: self method: false; p1:'Title: '; p2:((gXwksp) at:#gXtitle); condEvaluate.
GXprintn run line: 0 proc: self method: false; p1:'X range: '; p2:(((gXwksp) at:#gXx) at:((Array new:1) at: 1 put: 1; yourself)); p3:' to '; p4:(((gXwksp) at:#gXx) at:((Array new:1) at: 1 put: (gXi); yourself)); condEvaluate.
GXUshowUpar run line: 0 proc: self method: false; p1:(gXwksp); condEvaluate.
GXprintn run line: 0 proc: self method: false; condEvaluate.
GXprintn run line: 0 proc: self method: false; p1:'I	   X[I]		   Y[I]		   E[I]'; condEvaluate.
Function new loop: (gXimin) to: (gXimax) by: 1 before: [ :gXi | 
GXprintn run line: 0 proc: self method: false; p1:(gXi); p2:'	'; p3:(((gXwksp) at:#gXx) at:((Array new:1) at: 1 put: (gXi); yourself)); p4:'	'; p5:(((gXwksp) at:#gXy) at:((Array new:1) at: 1 put: (gXi); yourself)); p6:'	'; p7:(((gXwksp) at:#gXe) at:((Array new:1) at: 1 put: (gXi); yourself)); condEvaluate ] 
condition: [false] after: [ :gXi | nil ].
self up.
^nil
!!
nil! "(fix for smalltalk bug, this line is ignored if there is a variable declaration error)"

"End of procedure"

GXcomment run line: 0 proc: self method: false; p1:'_show_data'; p2:'display data in a spectrum workspace'; condEvaluate!
"Procedure class definition:
 ==========================="
((Smalltalk at: #GXshow ifAbsent: [ nil ]) isKindOf: Function class) ifTrue: [ ('Warning: Overriding previously defined procedure ', 'GXshow' gtype) prinl ].
Smalltalk at: #GXshow put: nil!
Function subclass: 'GXshow'
       instanceVariableNames: ''
       classVariableNames: ''
       poolDictionaries: ''
       category: 'Genie Procedure'
!

"HEADER
 ------"

"qualifiers
 ----------"

GXshow

addInstVarName: #gXdisk;
compile:
'gXdisk
	gXdisk _ true';
compile:
'gXdisk: aBoolean
	gXdisk _ aBoolean';

addInstVarName: #gXdirectory;
compile:
'gXdirectory
	gXdirectory _ true';
compile:
'gXdirectory: aBoolean
	gXdirectory _ aBoolean';

addInstVarName: #gXinstrument;
compile:
'gXinstrument
	gXinstrument _ true';
compile:
'gXinstrument: aBoolean
	gXinstrument _ aBoolean';

addInstVarName: #gXextension;
compile:
'gXextension
	gXextension _ true';
compile:
'gXextension: aBoolean
	gXextension _ aBoolean';

addInstVarName: #gXi;
compile:
'gXi
	gXi _ true';
compile:
'gXi: aBoolean
	gXi _ aBoolean';

addInstVarName: #gXdir;
compile:
'gXdir
	gXdir _ true';
compile:
'gXdir: aBoolean
	gXdir _ aBoolean';

addInstVarName: #gXe;
compile:
'gXe
	gXe _ true';
compile:
'gXe: aBoolean
	gXe _ aBoolean';

addInstVarName: #gXfile;
compile:
'gXfile
	gXfile _ true';
compile:
'gXfile: aBoolean
	gXfile _ aBoolean';

addInstVarName: #gXf;
compile:
'gXf
	gXf _ true';
compile:
'gXf: aBoolean
	gXf _ aBoolean';

addInstVarName: #gXio;
compile:
'gXio
	gXio _ true';
compile:
'gXio: aBoolean
	gXio _ aBoolean';

addInstVarName: #gXext;
compile:
'gXext
	gXext _ true';
compile:
'gXext: aBoolean
	gXext _ aBoolean';

addInstVarName: #gXinst;
compile:
'gXinst
	gXinst _ true';
compile:
'gXinst: aBoolean
	gXinst _ aBoolean';

addInstVarName: #gXdefaults;
compile:
'gXdefaults
	gXdefaults _ true';
compile:
'gXdefaults: aBoolean
	gXdefaults _ aBoolean';

addInstVarName: #gXdef;
compile:
'gXdef
	gXdef _ true';
compile:
'gXdef: aBoolean
	gXdef _ aBoolean';

addInstVarName: #gXproc;
compile:
'gXproc
	gXproc _ true';
compile:
'gXproc: aBoolean
	gXproc _ aBoolean';

addInstVarName: #gXpar;
compile:
'gXpar
	gXpar _ true';
compile:
'gXpar: aBoolean
	gXpar _ aBoolean';

addInstVarName: #gXtitle;
compile:
'gXtitle
	gXtitle _ true';
compile:
'gXtitle: aBoolean
	gXtitle _ aBoolean';

addInstVarName: #gXt;
compile:
'gXt
	gXt _ true';
compile:
'gXt: aBoolean
	gXt _ aBoolean';

addInstVarName: #gXdata;
compile:
'gXdata
	gXdata _ true';
compile:
'gXdata: aBoolean
	gXdata _ aBoolean';

addInstVarName: #gXvar;
compile:
'gXvar
	gXvar _ true';
compile:
'gXvar: aBoolean
	gXvar _ aBoolean';

addInstVarName: #gXsys;
compile:
'gXsys
	gXsys _ true';
compile:
'gXsys: aBoolean
	gXsys _ aBoolean';

addInstVarName: #gXtype;
compile:
'gXtype
	gXtype _ true';
compile:
'gXtype: aBoolean
	gXtype _ aBoolean';

addInstVarName: #gXconst;
compile:
'gXconst
	gXconst _ true';
compile:
'gXconst: aBoolean
	gXconst _ aBoolean'!

"parameters
 ----------"

GXshow

addInstVarName: #gXwksp;
compile:
'gXwksp: aVar
	aVar notNil ifTrue:[gXwksp _ self typeCheck: aVar shouldBe: GXWorkspace paramName: ''gXwksp'']';
compile:
'p1: aVar
	aVar notNil ifTrue:[gXwksp _ self typeCheck: aVar shouldBe: GXWorkspace paramName: ''gXwksp'']';

addInstVarName: #gXxmin;
compile:
'gXxmin: aVar
	aVar notNil ifTrue:[ gXxmin _ aVar ]';
compile:
'p2: aVar
	aVar notNil ifTrue:[ gXxmin _ aVar ]';

addInstVarName: #gXxmax;
compile:
'gXxmax: aVar
	aVar notNil ifTrue:[ gXxmax _ aVar ]';
compile:
'p3: aVar
	aVar notNil ifTrue:[ gXxmax _ aVar ]'
!

!GXshow methodsFor: 'initializing'!

initVars
	gXdisk _ false.
	gXdirectory _ false.
	gXinstrument _ false.
	gXextension _ false.
	gXi _ false.
	gXdir _ false.
	gXe _ false.
	gXfile _ false.
	gXf _ false.
	gXio _ false.
	gXext _ false.
	gXinst _ false.
	gXdefaults _ false.
	gXdef _ false.
	gXproc _ false.
	gXpar _ false.
	gXtitle _ false.
	gXt _ false.
	gXdata _ false.
	gXvar _ false.
	gXsys _ false.
	gXtype _ false.
	gXconst _ false
!!

"BODY
 ----"

!GXshow methodsFor: 'running'!

evaluate
(((gXdefaults) | (gXdef))) ifTrue: [ 
GXUshowUdefaults run line: 0 proc: self method: false; condEvaluate ] ifFalse: [ ((gXdisk)) ifTrue: [ 
GXprintn run line: 0 proc: self method: false; p1:'Current default disk = '; p2:(gXUcurrentUdisk); condEvaluate ] ifFalse: [ (((gXdir) | (gXdirectory))) ifTrue: [ 
GXprintn run line: 0 proc: self method: false; p1:'Current default directory = '; p2:(gXUcurrentUdirectory); condEvaluate ] ifFalse: [ (((gXinst) | (gXinstrument))) ifTrue: [ 
GXprintn run line: 0 proc: self method: false; p1:'Current default instrument = '; p2:(gXUcurrentUinstrument); condEvaluate ] ifFalse: [ (((gXext) | (gXextension))) ifTrue: [ 
GXprintn run line: 0 proc: self method: false; p1:'Current default extension = '; p2:(gXUcurrentUextension); condEvaluate ] ifFalse: [ ((gXio)) ifTrue: [ 
GXprintn run line: 0 proc: self method: false; p1:'Current input default = '; p2:(gXUcurrentUinputUsource); condEvaluate.
GXprintn run line: 0 proc: self method: false; p1:'Current output default = '; p2:(gXUcurrentUoutputUsource); condEvaluate ] ifFalse: [ ((gXproc)) ifTrue: [ 
(gXsys) ifTrue: [ 
GXsearch run line: 0 proc: self method: false; gXtype2; condEvaluate ] ifFalse: [ 
GXsearch run line: 0 proc: self method: false; gXproc; condEvaluate ]  ] ifFalse: [ ((gXvar)) ifTrue: [ 
(gXsys) ifTrue: [ 
GXsearch run line: 0 proc: self method: false; gXsys; condEvaluate ] ifFalse: [ 
GXsearch run line: 0 proc: self method: false; gXvar; condEvaluate ]  ] ifFalse: [ ((gXtype)) ifTrue: [ 
GXsearch run line: 0 proc: self method: false; gXtype; condEvaluate ] ifFalse: [ ((gXconst)) ifTrue: [ 
(gXsys) ifTrue: [ 
GXsearch run line: 0 proc: self method: false; gXconst2; condEvaluate ] ifFalse: [ 
GXsearch run line: 0 proc: self method: false; gXconst; condEvaluate ]  ] ifFalse: [ ((gXpar)) ifTrue: [ 
GXUshowUpar run line: 0 proc: self method: false; p1:(gXwksp); condEvaluate ] ifFalse: [ (((gXtitle) | (gXt))) ifTrue: [ 
GXprintn run line: 0 proc: self method: false; p1:'Title = '; p2:((gXwksp) at:#gXtitle); condEvaluate ] ifFalse: [ (gXdata) ifTrue: [ 
GXUshowUdata run line: 0 proc: self method: false; p1:(gXwksp); p2:(gXxmin); p3:(gXxmax); condEvaluate ]  ]]]]]]]]]]]].
self up.
^nil
!!
nil! "(fix for smalltalk bug, this line is ignored if there is a variable declaration error)"

"End of procedure"

GXcomment run line: 0 proc: self method: false; p1:'show'; p2:'Display parameters about environment'; condEvaluate!
"Procedure class definition:
 ==========================="
((Smalltalk at: #GXassign ifAbsent: [ nil ]) isKindOf: Function class) ifTrue: [ ('Warning: Overriding previously defined procedure ', 'GXassign' gtype) prinl ].
Smalltalk at: #GXassign put: nil!
Function subclass: 'GXassign'
       instanceVariableNames: ''
       classVariableNames: ''
       poolDictionaries: ''
       category: 'Genie Procedure'
!

"HEADER
 ------"

"parameters
 ----------"

GXassign

addInstVarName: #gXp1;
compile:
'gXp1: aVar
	aVar notNil ifTrue:[ gXp1 _ aVar ]';
compile:
'p1: aVar
	aVar notNil ifTrue:[ gXp1 _ aVar ]';

addInstVarName: #gXdummy;
compile:
'gXdummy: aVar
	aVar notNil ifTrue:[ gXdummy _ aVar ]';
compile:
'p2: aVar
	aVar notNil ifTrue:[ gXdummy _ aVar ]'
!

!GXassign methodsFor: 'initializing'!

initVars

!!

"BODY
 ----"

!GXassign methodsFor: 'running'!

evaluate
(GXisUa run line: 0 proc: self method: true; p1:(gXdummy); p2:'Workspace'; condEvaluate) ifTrue: [ 
GXprinten run line: 0 proc: self method: false; p1:'ASSIGN: Assignment to workspace not allowed'; condEvaluate ] ifFalse: [ 
GXset run line: 0 proc: self method: false; gXinput; p1:(GXcfn run line: 0 proc: self method: true; p1:(gXp1); condEvaluate); condEvaluate ] .
self up.
^nil
!!
nil! "(fix for smalltalk bug, this line is ignored if there is a variable declaration error)"

"End of procedure"

GXcomment run line: 0 proc: self method: false; p1:'assign'; p2:'Assign a default input file from an ISIS run number'; condEvaluate!
"Procedure class definition:
 ==========================="
((Smalltalk at: #GXs ifAbsent: [ nil ]) isKindOf: Function class) ifTrue: [ ('Warning: Overriding previously defined procedure ', 'GXs' gtype) prinl ].
Smalltalk at: #GXs put: nil!
Function subclass: 'GXs'
       instanceVariableNames: ''
       classVariableNames: ''
       poolDictionaries: ''
       category: 'Genie Procedure'
!

"HEADER
 ------"

"parameters
 ----------"

GXs

addInstVarName: #gXspectra;
compile:
'gXspectra: aVar
	aVar notNil ifTrue:[ gXspectra _ aVar ]';
compile:
'p1: aVar
	aVar notNil ifTrue:[ gXspectra _ aVar ]';

addInstVarName: #gXfile;
compile:
'gXfile: aVar
	aVar notNil ifTrue:[gXfile _ self typeCheck: aVar shouldBe: GXString paramName: ''gXfile'']';
compile:
'p2: aVar
	aVar notNil ifTrue:[gXfile _ self typeCheck: aVar shouldBe: GXString paramName: ''gXfile'']'!

"result
 ------"

GXs

addInstVarName: #gXvalue
!

!GXs methodsFor: 'initializing'!

initVars

!!

"BODY
 ----"

!GXs methodsFor: 'running'!

evaluate
gXvalue _ (GXget run line: 0 proc: self method: true; p1:(gXspectra); p2:(gXfile); condEvaluate) .
self up.
^gXvalue
!!
nil! "(fix for smalltalk bug, this line is ignored if there is a variable declaration error)"

"End of procedure"

GXcomment run line: 0 proc: self method: false; p1:'s'; p2:'Short form of GET() (For Genie-II compatability)'; condEvaluate!
"Procedure class definition:
 ==========================="
((Smalltalk at: #GXUrebinUboundaries ifAbsent: [ nil ]) isKindOf: Function class) ifTrue: [ ('Warning: Overriding previously defined procedure ', 'GXUrebinUboundaries' gtype) prinl ].
Smalltalk at: #GXUrebinUboundaries put: nil!
Function subclass: 'GXUrebinUboundaries'
       instanceVariableNames: ''
       classVariableNames: ''
       poolDictionaries: ''
       category: 'Genie Procedure'
!

"HEADER
 ------"

"parameters
 ----------"

GXUrebinUboundaries

addInstVarName: #gXitype;
compile:
'gXitype: aVar
	aVar notNil ifTrue:[gXitype _ self typeCheck: aVar shouldBe: GXInteger paramName: ''gXitype'']';
compile:
'p1: aVar
	aVar notNil ifTrue:[gXitype _ self typeCheck: aVar shouldBe: GXInteger paramName: ''gXitype'']';

addInstVarName: #gXxarray;
compile:
'gXxarray: aVar
	aVar notNil ifTrue:[gXxarray _ self typeCheck: aVar shouldBe: GXRealarray paramName: ''gXxarray'']';
compile:
'p2: aVar
	aVar notNil ifTrue:[gXxarray _ self typeCheck: aVar shouldBe: GXRealarray paramName: ''gXxarray'']';

addInstVarName: #gXyarray;
compile:
'gXyarray: aVar
	aVar notNil ifTrue:[gXyarray _ self typeCheck: aVar shouldBe: GXRealarray paramName: ''gXyarray'']';
compile:
'p3: aVar
	aVar notNil ifTrue:[gXyarray _ self typeCheck: aVar shouldBe: GXRealarray paramName: ''gXyarray'']';

addInstVarName: #gXearray;
compile:
'gXearray: aVar
	aVar notNil ifTrue:[gXearray _ self typeCheck: aVar shouldBe: GXRealarray paramName: ''gXearray'']';
compile:
'p4: aVar
	aVar notNil ifTrue:[gXearray _ self typeCheck: aVar shouldBe: GXRealarray paramName: ''gXearray'']';

addInstVarName: #gXp1;
compile:
'gXp1: aVar
	aVar notNil ifTrue:[gXp1 _ self typeCheck: aVar shouldBe: GXReal paramName: ''gXp1'']';
compile:
'p5: aVar
	aVar notNil ifTrue:[gXp1 _ self typeCheck: aVar shouldBe: GXReal paramName: ''gXp1'']';

addInstVarName: #gXp2;
compile:
'gXp2: aVar
	aVar notNil ifTrue:[gXp2 _ self typeCheck: aVar shouldBe: GXReal paramName: ''gXp2'']';
compile:
'p6: aVar
	aVar notNil ifTrue:[gXp2 _ self typeCheck: aVar shouldBe: GXReal paramName: ''gXp2'']';

addInstVarName: #gXp3;
compile:
'gXp3: aVar
	aVar notNil ifTrue:[gXp3 _ self typeCheck: aVar shouldBe: GXReal paramName: ''gXp3'']';
compile:
'p7: aVar
	aVar notNil ifTrue:[gXp3 _ self typeCheck: aVar shouldBe: GXReal paramName: ''gXp3'']';

addInstVarName: #gXp4;
compile:
'gXp4: aVar
	aVar notNil ifTrue:[gXp4 _ self typeCheck: aVar shouldBe: GXReal paramName: ''gXp4'']';
compile:
'p8: aVar
	aVar notNil ifTrue:[gXp4 _ self typeCheck: aVar shouldBe: GXReal paramName: ''gXp4'']';

addInstVarName: #gXp5;
compile:
'gXp5: aVar
	aVar notNil ifTrue:[gXp5 _ self typeCheck: aVar shouldBe: GXReal paramName: ''gXp5'']';
compile:
'p9: aVar
	aVar notNil ifTrue:[gXp5 _ self typeCheck: aVar shouldBe: GXReal paramName: ''gXp5'']';

addInstVarName: #gXp6;
compile:
'gXp6: aVar
	aVar notNil ifTrue:[gXp6 _ self typeCheck: aVar shouldBe: GXReal paramName: ''gXp6'']';
compile:
'p10: aVar
	aVar notNil ifTrue:[gXp6 _ self typeCheck: aVar shouldBe: GXReal paramName: ''gXp6'']';

addInstVarName: #gXp7;
compile:
'gXp7: aVar
	aVar notNil ifTrue:[gXp7 _ self typeCheck: aVar shouldBe: GXReal paramName: ''gXp7'']';
compile:
'p11: aVar
	aVar notNil ifTrue:[gXp7 _ self typeCheck: aVar shouldBe: GXReal paramName: ''gXp7'']';

addInstVarName: #gXp8;
compile:
'gXp8: aVar
	aVar notNil ifTrue:[gXp8 _ self typeCheck: aVar shouldBe: GXReal paramName: ''gXp8'']';
compile:
'p12: aVar
	aVar notNil ifTrue:[gXp8 _ self typeCheck: aVar shouldBe: GXReal paramName: ''gXp8'']';

addInstVarName: #gXp9;
compile:
'gXp9: aVar
	aVar notNil ifTrue:[gXp9 _ self typeCheck: aVar shouldBe: GXReal paramName: ''gXp9'']';
compile:
'p13: aVar
	aVar notNil ifTrue:[gXp9 _ self typeCheck: aVar shouldBe: GXReal paramName: ''gXp9'']';

addInstVarName: #gXp10;
compile:
'gXp10: aVar
	aVar notNil ifTrue:[gXp10 _ self typeCheck: aVar shouldBe: GXReal paramName: ''gXp10'']';
compile:
'p14: aVar
	aVar notNil ifTrue:[gXp10 _ self typeCheck: aVar shouldBe: GXReal paramName: ''gXp10'']';

addInstVarName: #gXp11;
compile:
'gXp11: aVar
	aVar notNil ifTrue:[gXp11 _ self typeCheck: aVar shouldBe: GXReal paramName: ''gXp11'']';
compile:
'p15: aVar
	aVar notNil ifTrue:[gXp11 _ self typeCheck: aVar shouldBe: GXReal paramName: ''gXp11'']';

addInstVarName: #gXp12;
compile:
'gXp12: aVar
	aVar notNil ifTrue:[gXp12 _ self typeCheck: aVar shouldBe: GXReal paramName: ''gXp12'']';
compile:
'p16: aVar
	aVar notNil ifTrue:[gXp12 _ self typeCheck: aVar shouldBe: GXReal paramName: ''gXp12'']'!

"result
 ------"

GXUrebinUboundaries

addInstVarName: #gXres
!

"Local variables
 ---------------"

GXUrebinUboundaries

addInstVarName: #gXxbounds;

addInstVarName: #gXxsteps;

addInstVarName: #gXibounds;

addInstVarName: #gXisteps;

addInstVarName: #gXtemp
!

!GXUrebinUboundaries methodsFor: 'initializing'!

initVars
	gXres _ (GXfields run line: 0 proc: self method: true; p1:3; condEvaluate).
	gXibounds _ 0.
	gXisteps _ 0
!!

"BODY
 ----"

!GXUrebinUboundaries methodsFor: 'running'!

evaluate

    gXxarray isNil ifTrue: [^'REBIN: parameter XARRAY must be specified' prenl].
    gXyarray isNil ifTrue: [^'REBIN: parameter YARRAY must be specified' prenl].
    gXearray isNil ifTrue: [^'REBIN: parameter EARRAY must be specified' prenl].
    gXp1 isNil ifTrue: [^'REBIN: parameter BOUND1 must be specified' prenl].
    gXp2 isNil ifTrue: [^'REBIN: parameter STEP1 must be specified' prenl].
    gXp3 isNil ifTrue: [^'REBIN: parameter BOUND2 must be specified' prenl].
    gXxbounds _ GXRealarray new: 12.
    gXxsteps _ GXRealarray new: 12.
    gXxbounds at: 1 put: gXp1.
    gXxbounds at: 2 put: gXp3.
    gXxbounds at: 3 put: gXp5.
    gXxbounds at: 4 put: gXp7.
    gXxbounds at: 5 put: gXp9.
    gXxbounds at: 6 put: gXp11.
    gXxsteps at: 1 put: gXp2.
    gXxsteps at: 2 put: gXp4.
    gXxsteps at: 3 put: gXp6.
    gXxsteps at: 4 put: gXp8.
    gXxsteps at: 5 put: gXp10.
    gXxsteps at: 6 put: gXp12
.
Function new loop: 1 to: 12 by: 1 before: [ :gXi | 

    gXtemp _ gXxsteps at: gXi.
    ((gXtemp notNil) & (gXtemp ~= 0.0)) ifTrue: [ gXisteps _ gXi ]
 ] 
condition: [false] after: [ :gXi | nil ].
gXibounds _ ((gXisteps) + 1) deepCopy.
GXcallout run line: 0 proc: self method: false; gXfacility:'extTransform'; gXfunction:10; condEvaluate.
GXgUerrorUhandler run line: 0 proc: self method: false; condEvaluate.
(gXres isKindOf: GXArray) | (gXres isKindOf: GXWorkspace)
    ifTrue:  [ ((gXres) at:#gXx put: (gXxarray) deepCopy) ]
    ifFalse: [ self error: 'Assignment to array or workspace failed, check that the destination is a structured type'].
(gXres isKindOf: GXArray) | (gXres isKindOf: GXWorkspace)
    ifTrue:  [ ((gXres) at:#gXy put: (gXyarray) deepCopy) ]
    ifFalse: [ self error: 'Assignment to array or workspace failed, check that the destination is a structured type'].
(gXres isKindOf: GXArray) | (gXres isKindOf: GXWorkspace)
    ifTrue:  [ ((gXres) at:#gXe put: (gXearray) deepCopy) ]
    ifFalse: [ self error: 'Assignment to array or workspace failed, check that the destination is a structured type'].
self up.
^gXres
!!
nil! "(fix for smalltalk bug, this line is ignored if there is a variable declaration error)"

"End of procedure"

GXcomment run line: 0 proc: self method: false; p1:'_rebin_boundaries'; p2:'Rebin workspace to linear or logarithmic sized bins'; condEvaluate!
"Procedure class definition:
 ==========================="
((Smalltalk at: #GXUrebinUusing ifAbsent: [ nil ]) isKindOf: Function class) ifTrue: [ ('Warning: Overriding previously defined procedure ', 'GXUrebinUusing' gtype) prinl ].
Smalltalk at: #GXUrebinUusing put: nil!
Function subclass: 'GXUrebinUusing'
       instanceVariableNames: ''
       classVariableNames: ''
       poolDictionaries: ''
       category: 'Genie Procedure'
!

"HEADER
 ------"

"parameters
 ----------"

GXUrebinUusing

addInstVarName: #gXxold;
compile:
'gXxold: aVar
	aVar notNil ifTrue:[gXxold _ self typeCheck: aVar shouldBe: GXRealarray paramName: ''gXxold'']';
compile:
'p1: aVar
	aVar notNil ifTrue:[gXxold _ self typeCheck: aVar shouldBe: GXRealarray paramName: ''gXxold'']';

addInstVarName: #gXyold;
compile:
'gXyold: aVar
	aVar notNil ifTrue:[gXyold _ self typeCheck: aVar shouldBe: GXRealarray paramName: ''gXyold'']';
compile:
'p2: aVar
	aVar notNil ifTrue:[gXyold _ self typeCheck: aVar shouldBe: GXRealarray paramName: ''gXyold'']';

addInstVarName: #gXeold;
compile:
'gXeold: aVar
	aVar notNil ifTrue:[gXeold _ self typeCheck: aVar shouldBe: GXRealarray paramName: ''gXeold'']';
compile:
'p3: aVar
	aVar notNil ifTrue:[gXeold _ self typeCheck: aVar shouldBe: GXRealarray paramName: ''gXeold'']';

addInstVarName: #gXxnew;
compile:
'gXxnew: aVar
	aVar notNil ifTrue:[gXxnew _ self typeCheck: aVar shouldBe: GXRealarray paramName: ''gXxnew'']';
compile:
'p4: aVar
	aVar notNil ifTrue:[gXxnew _ self typeCheck: aVar shouldBe: GXRealarray paramName: ''gXxnew'']'!

"result
 ------"

GXUrebinUusing

addInstVarName: #gXres
!

!GXUrebinUusing methodsFor: 'initializing'!

initVars
	gXres _ (GXfields run line: 0 proc: self method: true; p1:3; condEvaluate)
!!

"BODY
 ----"

!GXUrebinUusing methodsFor: 'running'!

evaluate

    gXxold isNil ifTrue: [^'REBIN: Parameter XOLD not specified' prenl ].
    gXyold isNil ifTrue: [^'REBIN: Parameter YOLD not specified' prenl ].
    gXeold isNil ifTrue: [^'REBIN: Parameter EOLD not specified' prenl ].
    gXxnew isNil ifTrue: [^'REBIN: Parameter XNEW not specified' prenl ]
.
GXcallout run line: 0 proc: self method: false; gXfacility:'extTransform'; gXfunction:9; condEvaluate.
GXgUerrorUhandler run line: 0 proc: self method: false; condEvaluate.
(gXres isKindOf: GXArray) | (gXres isKindOf: GXWorkspace)
    ifTrue:  [ ((gXres) at:#gXx put: (gXxnew) deepCopy) ]
    ifFalse: [ self error: 'Assignment to array or workspace failed, check that the destination is a structured type'].

    gXres at: #gXy put: gXyold.
    gXres at: #gXe put: gXeold
.
self up.
^gXres
!!
nil! "(fix for smalltalk bug, this line is ignored if there is a variable declaration error)"

"End of procedure"

GXcomment run line: 0 proc: self method: false; p1:'_rebin_using'; p2:'Rebin a workspace to a new set of supplied X values'; condEvaluate!
"Procedure class definition:
 ==========================="
((Smalltalk at: #GXUrebinUrange ifAbsent: [ nil ]) isKindOf: Function class) ifTrue: [ ('Warning: Overriding previously defined procedure ', 'GXUrebinUrange' gtype) prinl ].
Smalltalk at: #GXUrebinUrange put: nil!
Function subclass: 'GXUrebinUrange'
       instanceVariableNames: ''
       classVariableNames: ''
       poolDictionaries: ''
       category: 'Genie Procedure'
!

"HEADER
 ------"

"parameters
 ----------"

GXUrebinUrange

addInstVarName: #gXxarray;
compile:
'gXxarray: aVar
	aVar notNil ifTrue:[gXxarray _ self typeCheck: aVar shouldBe: GXRealarray paramName: ''gXxarray'']';
compile:
'p1: aVar
	aVar notNil ifTrue:[gXxarray _ self typeCheck: aVar shouldBe: GXRealarray paramName: ''gXxarray'']';

addInstVarName: #gXyarray;
compile:
'gXyarray: aVar
	aVar notNil ifTrue:[gXyarray _ self typeCheck: aVar shouldBe: GXRealarray paramName: ''gXyarray'']';
compile:
'p2: aVar
	aVar notNil ifTrue:[gXyarray _ self typeCheck: aVar shouldBe: GXRealarray paramName: ''gXyarray'']';

addInstVarName: #gXearray;
compile:
'gXearray: aVar
	aVar notNil ifTrue:[gXearray _ self typeCheck: aVar shouldBe: GXRealarray paramName: ''gXearray'']';
compile:
'p3: aVar
	aVar notNil ifTrue:[gXearray _ self typeCheck: aVar shouldBe: GXRealarray paramName: ''gXearray'']';

addInstVarName: #gXxmin;
compile:
'gXxmin: aVar
	aVar notNil ifTrue:[gXxmin _ self typeCheck: aVar shouldBe: GXReal paramName: ''gXxmin'']';
compile:
'p4: aVar
	aVar notNil ifTrue:[gXxmin _ self typeCheck: aVar shouldBe: GXReal paramName: ''gXxmin'']';

addInstVarName: #gXxmax;
compile:
'gXxmax: aVar
	aVar notNil ifTrue:[gXxmax _ self typeCheck: aVar shouldBe: GXReal paramName: ''gXxmax'']';
compile:
'p5: aVar
	aVar notNil ifTrue:[gXxmax _ self typeCheck: aVar shouldBe: GXReal paramName: ''gXxmax'']'!

"result
 ------"

GXUrebinUrange

addInstVarName: #gXres
!

!GXUrebinUrange methodsFor: 'initializing'!

initVars
	gXres _ (GXfields run line: 0 proc: self method: true; p1:3; condEvaluate)
!!

"BODY
 ----"

!GXUrebinUrange methodsFor: 'running'!

evaluate

    gXxarray isNil ifTrue: [^'REBIN: parameter XARRAY not specified' prenl ].
    gXyarray isNil ifTrue: [^'REBIN: parameter YARRAY not specified' prenl ].
    gXearray isNil ifTrue: [^'REBIN: parameter EARRAY not specified' prenl ].
    gXxmin isNil ifTrue: [^'REBIN: parameter XMIN not specified' prenl ].
    gXxmax isNil ifTrue: [^'REBIN: parameter XMAX not specified' prenl ]
.
GXcallout run line: 0 proc: self method: false; gXfacility:'extTransform'; gXfunction:11; condEvaluate.
GXgUerrorUhandler run line: 0 proc: self method: false; condEvaluate.
(gXres isKindOf: GXArray) | (gXres isKindOf: GXWorkspace)
    ifTrue:  [ ((gXres) at:#gXx put: (gXxarray) deepCopy) ]
    ifFalse: [ self error: 'Assignment to array or workspace failed, check that the destination is a structured type'].
(gXres isKindOf: GXArray) | (gXres isKindOf: GXWorkspace)
    ifTrue:  [ ((gXres) at:#gXy put: (gXyarray) deepCopy) ]
    ifFalse: [ self error: 'Assignment to array or workspace failed, check that the destination is a structured type'].
(gXres isKindOf: GXArray) | (gXres isKindOf: GXWorkspace)
    ifTrue:  [ ((gXres) at:#gXe put: (gXearray) deepCopy) ]
    ifFalse: [ self error: 'Assignment to array or workspace failed, check that the destination is a structured type'].
self up.
^gXres
!!
nil! "(fix for smalltalk bug, this line is ignored if there is a variable declaration error)"

"End of procedure"

GXcomment run line: 0 proc: self method: false; p1:'_rebin_range'; p2:'Rebin a histogram to a given range'; condEvaluate!
"Procedure class definition:
 ==========================="
((Smalltalk at: #GXrebin ifAbsent: [ nil ]) isKindOf: Function class) ifTrue: [ ('Warning: Overriding previously defined procedure ', 'GXrebin' gtype) prinl ].
Smalltalk at: #GXrebin put: nil!
Function subclass: 'GXrebin'
       instanceVariableNames: ''
       classVariableNames: ''
       poolDictionaries: ''
       category: 'Genie Procedure'
!

"HEADER
 ------"

"qualifiers
 ----------"

GXrebin

addInstVarName: #gXlin;
compile:
'gXlin
	gXlin _ true';
compile:
'gXlin: aBoolean
	gXlin _ aBoolean';

addInstVarName: #gXlinear;
compile:
'gXlinear
	gXlinear _ true';
compile:
'gXlinear: aBoolean
	gXlinear _ aBoolean';

addInstVarName: #gXlog;
compile:
'gXlog
	gXlog _ true';
compile:
'gXlog: aBoolean
	gXlog _ aBoolean'!

"parameters
 ----------"

GXrebin

addInstVarName: #gXp1;
compile:
'gXp1: aVar
	aVar notNil ifTrue:[ gXp1 _ aVar ]';
compile:
'p1: aVar
	aVar notNil ifTrue:[ gXp1 _ aVar ]';

addInstVarName: #gXp2;
compile:
'gXp2: aVar
	aVar notNil ifTrue:[ gXp2 _ aVar ]';
compile:
'p2: aVar
	aVar notNil ifTrue:[ gXp2 _ aVar ]';

addInstVarName: #gXp3;
compile:
'gXp3: aVar
	aVar notNil ifTrue:[ gXp3 _ aVar ]';
compile:
'p3: aVar
	aVar notNil ifTrue:[ gXp3 _ aVar ]';

addInstVarName: #gXp4;
compile:
'gXp4: aVar
	aVar notNil ifTrue:[ gXp4 _ aVar ]';
compile:
'p4: aVar
	aVar notNil ifTrue:[ gXp4 _ aVar ]';

addInstVarName: #gXp5;
compile:
'gXp5: aVar
	aVar notNil ifTrue:[ gXp5 _ aVar ]';
compile:
'p5: aVar
	aVar notNil ifTrue:[ gXp5 _ aVar ]';

addInstVarName: #gXp6;
compile:
'gXp6: aVar
	aVar notNil ifTrue:[ gXp6 _ aVar ]';
compile:
'p6: aVar
	aVar notNil ifTrue:[ gXp6 _ aVar ]';

addInstVarName: #gXp7;
compile:
'gXp7: aVar
	aVar notNil ifTrue:[ gXp7 _ aVar ]';
compile:
'p7: aVar
	aVar notNil ifTrue:[ gXp7 _ aVar ]';

addInstVarName: #gXp8;
compile:
'gXp8: aVar
	aVar notNil ifTrue:[ gXp8 _ aVar ]';
compile:
'p8: aVar
	aVar notNil ifTrue:[ gXp8 _ aVar ]';

addInstVarName: #gXwksp;
compile:
'gXwksp: aVar
	aVar notNil ifTrue:[gXwksp _ self typeCheck: aVar shouldBe: GXWorkspace paramName: ''gXwksp'']';
compile:
'p9: aVar
	aVar notNil ifTrue:[gXwksp _ self typeCheck: aVar shouldBe: GXWorkspace paramName: ''gXwksp'']';

addInstVarName: #gXbound1;
compile:
'gXbound1: aVar
	aVar notNil ifTrue:[gXbound1 _ self typeCheck: aVar shouldBe: GXReal paramName: ''gXbound1'']';
compile:
'p10: aVar
	aVar notNil ifTrue:[gXbound1 _ self typeCheck: aVar shouldBe: GXReal paramName: ''gXbound1'']';

addInstVarName: #gXstep1;
compile:
'gXstep1: aVar
	aVar notNil ifTrue:[gXstep1 _ self typeCheck: aVar shouldBe: GXReal paramName: ''gXstep1'']';
compile:
'p11: aVar
	aVar notNil ifTrue:[gXstep1 _ self typeCheck: aVar shouldBe: GXReal paramName: ''gXstep1'']';

addInstVarName: #gXbound2;
compile:
'gXbound2: aVar
	aVar notNil ifTrue:[gXbound2 _ self typeCheck: aVar shouldBe: GXReal paramName: ''gXbound2'']';
compile:
'p12: aVar
	aVar notNil ifTrue:[gXbound2 _ self typeCheck: aVar shouldBe: GXReal paramName: ''gXbound2'']';

addInstVarName: #gXstep2;
compile:
'gXstep2: aVar
	aVar notNil ifTrue:[gXstep2 _ self typeCheck: aVar shouldBe: GXReal paramName: ''gXstep2'']';
compile:
'p13: aVar
	aVar notNil ifTrue:[gXstep2 _ self typeCheck: aVar shouldBe: GXReal paramName: ''gXstep2'']';

addInstVarName: #gXbound3;
compile:
'gXbound3: aVar
	aVar notNil ifTrue:[gXbound3 _ self typeCheck: aVar shouldBe: GXReal paramName: ''gXbound3'']';
compile:
'p14: aVar
	aVar notNil ifTrue:[gXbound3 _ self typeCheck: aVar shouldBe: GXReal paramName: ''gXbound3'']';

addInstVarName: #gXstep3;
compile:
'gXstep3: aVar
	aVar notNil ifTrue:[gXstep3 _ self typeCheck: aVar shouldBe: GXReal paramName: ''gXstep3'']';
compile:
'p15: aVar
	aVar notNil ifTrue:[gXstep3 _ self typeCheck: aVar shouldBe: GXReal paramName: ''gXstep3'']';

addInstVarName: #gXbound4;
compile:
'gXbound4: aVar
	aVar notNil ifTrue:[gXbound4 _ self typeCheck: aVar shouldBe: GXReal paramName: ''gXbound4'']';
compile:
'p16: aVar
	aVar notNil ifTrue:[gXbound4 _ self typeCheck: aVar shouldBe: GXReal paramName: ''gXbound4'']';

addInstVarName: #gXxarray;
compile:
'gXxarray: aVar
	aVar notNil ifTrue:[gXxarray _ self typeCheck: aVar shouldBe: GXRealarray paramName: ''gXxarray'']';
compile:
'p17: aVar
	aVar notNil ifTrue:[gXxarray _ self typeCheck: aVar shouldBe: GXRealarray paramName: ''gXxarray'']';

addInstVarName: #gXxmin;
compile:
'gXxmin: aVar
	aVar notNil ifTrue:[gXxmin _ self typeCheck: aVar shouldBe: GXReal paramName: ''gXxmin'']';
compile:
'p18: aVar
	aVar notNil ifTrue:[gXxmin _ self typeCheck: aVar shouldBe: GXReal paramName: ''gXxmin'']';

addInstVarName: #gXxmax;
compile:
'gXxmax: aVar
	aVar notNil ifTrue:[gXxmax _ self typeCheck: aVar shouldBe: GXReal paramName: ''gXxmax'']';
compile:
'p19: aVar
	aVar notNil ifTrue:[gXxmax _ self typeCheck: aVar shouldBe: GXReal paramName: ''gXxmax'']'!

"result
 ------"

GXrebin

addInstVarName: #gXres
!

"Local variables
 ---------------"

GXrebin

addInstVarName: #gXtemp
!

!GXrebin methodsFor: 'initializing'!

initVars
	gXlin _ false.
	gXlinear _ false.
	gXlog _ false
!!

"BODY
 ----"

!GXrebin methodsFor: 'running'!

evaluate

    (gXp1 isKindOf: GXWorkspace) ifTrue: [ gXwksp _ gXp1 ].
    gXwksp isNil ifTrue: [^'REBIN: Parameter WKSP must be specified' prenl].
    (gXp2 isKindOf: Float) ifTrue: [ gXbound1 _ gXp2. gXxmin _ gXp2 ].
    (gXp2 isKindOf: GXRealarray) ifTrue: [ gXxarray _ gXp2 ].
    (gXp3 isKindOf: Float) ifTrue: [ gXstep1 _ gXp3. gXxmax _ gXp3 ].
    (gXp4 isKindOf: Float) ifTrue: [ gXbound2 _ gXp4 ].
    (gXp5 isKindOf: Float) ifTrue: [ gXstep2 _ gXp5 ].
    (gXp6 isKindOf: Float) ifTrue: [ gXbound3 _ gXp6 ].
    (gXp7 isKindOf: Float) ifTrue: [ gXstep3 _ gXp7 ].
    (gXp8 isKindOf: Float) ifTrue: [ gXbound4 _ gXp8 ]
.
gXres _ (gXwksp) deepCopy.
(gXlog) ifTrue: [ 
gXtemp _ (GXUrebinUboundaries run line: 0 proc: self method: true; p1:2; p2:((gXwksp) at:#gXx); p3:((gXwksp) at:#gXy); p4:((gXwksp) at:#gXe); p5:(gXbound1); p6:(gXstep1); p7:(gXbound2); p8:(gXstep2); p9:(gXbound3); p10:(gXstep3); p11:(gXbound4); condEvaluate)  ] ifFalse: [ (((gXlinear) | (gXlin))) ifTrue: [ 
gXtemp _ (GXUrebinUboundaries run line: 0 proc: self method: true; p1:1; p2:((gXwksp) at:#gXx); p3:((gXwksp) at:#gXy); p4:((gXwksp) at:#gXe); p5:(gXbound1); p6:(gXstep1); p7:(gXbound2); p8:(gXstep2); p9:(gXbound3); p10:(gXstep3); p11:(gXbound4); condEvaluate)  ] ifFalse: [ (GXdefined run line: 0 proc: self method: true; p1:(gXxarray); condEvaluate) ifTrue: [ 
gXtemp _ (GXUrebinUusing run line: 0 proc: self method: true; p1:((gXwksp) at:#gXx); p2:((gXwksp) at:#gXy); p3:((gXwksp) at:#gXe); p4:(gXxarray); condEvaluate)  ] ifFalse: [ ((GXdefined run line: 0 proc: self method: true; p1:(gXxmin); condEvaluate) & (GXdefined run line: 0 proc: self method: true; p1:(gXxmax); condEvaluate)) ifTrue: [ 
gXtemp _ (GXUrebinUrange run line: 0 proc: self method: true; p1:((gXwksp) at:#gXx); p2:((gXwksp) at:#gXy); p3:((gXwksp) at:#gXe); p4:(gXxmin); p5:(gXxmax); condEvaluate)  ] ifFalse: [ 
GXprinten run line: 0 proc: self method: false; p1:'REBIN: Invalid parameters or qualifiers'; condEvaluate.
[nil.
self up.
^gXres] value ] ]]].

    gXres at: #gXx put: ( gXtemp at: #gXx ).
    gXres at: #gXy put: ( gXtemp at: #gXy ).
    gXres at: #gXe put: ( gXtemp at: #gXe )
.
GXprintin run line: 0 proc: self method: false; p1:'REBIN: X range now '; p2:(((gXres) at:#gXx) at:((Array new:1) at: 1 put: 1; yourself)); p3:' to '; p4:(((gXres) at:#gXx) at:((Array new:1) at: 1 put: (GXlength run line: 0 proc: self method: true; p1:((gXres) at:#gXx); condEvaluate); yourself)); condEvaluate.
(gXres isKindOf: GXArray) | (gXres isKindOf: GXWorkspace)
    ifTrue:  [ ((gXres) at:#gXntc put: (GXlength run line: 0 proc: self method: true; p1:((gXres) at:#gXy); condEvaluate) deepCopy) ]
    ifFalse: [ self error: 'Assignment to array or workspace failed, check that the destination is a structured type'].
(GXcalledUasUfunction run line: 0 proc: self method: true; condEvaluate) not ifTrue: [ 
 gXwksp become: gXres  ].
self up.
^gXres
!!
nil! "(fix for smalltalk bug, this line is ignored if there is a variable declaration error)"

"End of procedure"

GXcomment run line: 0 proc: self method: false; p1:'rebin'; p2:'rebin a workspace'; condEvaluate!
"Procedure class definition:
 ==========================="
((Smalltalk at: #GXUputUg3Uascii ifAbsent: [ nil ]) isKindOf: Function class) ifTrue: [ ('Warning: Overriding previously defined procedure ', 'GXUputUg3Uascii' gtype) prinl ].
Smalltalk at: #GXUputUg3Uascii put: nil!
Function subclass: 'GXUputUg3Uascii'
       instanceVariableNames: ''
       classVariableNames: ''
       poolDictionaries: ''
       category: 'Genie Procedure'
!

"HEADER
 ------"

"parameters
 ----------"

GXUputUg3Uascii

addInstVarName: #gXtype;
compile:
'gXtype: aVar
	aVar notNil ifTrue:[gXtype _ self typeCheck: aVar shouldBe: GXInteger paramName: ''gXtype'']';
compile:
'p1: aVar
	aVar notNil ifTrue:[gXtype _ self typeCheck: aVar shouldBe: GXInteger paramName: ''gXtype'']';

addInstVarName: #gXvariable;
compile:
'gXvariable: aVar
	aVar notNil ifTrue:[ gXvariable _ aVar ]';
compile:
'p2: aVar
	aVar notNil ifTrue:[ gXvariable _ aVar ]';

addInstVarName: #gXlabel;
compile:
'gXlabel: aVar
	aVar notNil ifTrue:[gXlabel _ self typeCheck: aVar shouldBe: GXString paramName: ''gXlabel'']';
compile:
'p3: aVar
	aVar notNil ifTrue:[gXlabel _ self typeCheck: aVar shouldBe: GXString paramName: ''gXlabel'']';

addInstVarName: #gXcomment;
compile:
'gXcomment: aVar
	aVar notNil ifTrue:[gXcomment _ self typeCheck: aVar shouldBe: GXString paramName: ''gXcomment'']';
compile:
'p4: aVar
	aVar notNil ifTrue:[gXcomment _ self typeCheck: aVar shouldBe: GXString paramName: ''gXcomment'']';

addInstVarName: #gXfile;
compile:
'gXfile: aVar
	aVar notNil ifTrue:[gXfile _ self typeCheck: aVar shouldBe: GXString paramName: ''gXfile'']';
compile:
'p5: aVar
	aVar notNil ifTrue:[gXfile _ self typeCheck: aVar shouldBe: GXString paramName: ''gXfile'']';

addInstVarName: #gXmode;
compile:
'gXmode: aVar
	aVar notNil ifTrue:[gXmode _ self typeCheck: aVar shouldBe: GXInteger paramName: ''gXmode'']';
compile:
'p6: aVar
	aVar notNil ifTrue:[gXmode _ self typeCheck: aVar shouldBe: GXInteger paramName: ''gXmode'']'!

"result
 ------"

GXUputUg3Uascii

addInstVarName: #gXnblocks
!

"Local variables
 ---------------"

GXUputUg3Uascii

addInstVarName: #gXnblocks
!

!GXUputUg3Uascii methodsFor: 'initializing'!

initVars
	gXnblocks _ 0
!!

"BODY
 ----"

!GXUputUg3Uascii methodsFor: 'running'!

evaluate
GXprintin run line: 0 proc: self method: false; p1:'PUT: Writing variable to "'; p2:(gXfile); p3:'"'; condEvaluate.
GXprintin run line: 0 proc: self method: false; p1:'PUT: Label = "'; p2:(gXlabel); p3:'", Comment = "'; p4:(gXcomment); p5:'"'; condEvaluate.
GXcallout run line: 0 proc: self method: false; gXfacility:'extFortran'; gXfunction:31; condEvaluate.
(GXgUerrorUhandler run line: 0 proc: self method: true; condEvaluate) ifTrue: [ 
gXnblocks _ (gXU) deepCopy ].
self up.
^gXnblocks
!!
nil! "(fix for smalltalk bug, this line is ignored if there is a variable declaration error)"

"End of procedure"

GXcomment run line: 0 proc: self method: false; p1:'_put_g3'; p2:'Callout procedure for writing an Open Genie Intermediate file'; condEvaluate!
"Procedure class definition:
 ==========================="
((Smalltalk at: #GXUputUg3 ifAbsent: [ nil ]) isKindOf: Function class) ifTrue: [ ('Warning: Overriding previously defined procedure ', 'GXUputUg3' gtype) prinl ].
Smalltalk at: #GXUputUg3 put: nil!
Function subclass: 'GXUputUg3'
       instanceVariableNames: ''
       classVariableNames: ''
       poolDictionaries: ''
       category: 'Genie Procedure'
!

"HEADER
 ------"

"parameters
 ----------"

GXUputUg3

addInstVarName: #gXtype;
compile:
'gXtype: aVar
	aVar notNil ifTrue:[gXtype _ self typeCheck: aVar shouldBe: GXInteger paramName: ''gXtype'']';
compile:
'p1: aVar
	aVar notNil ifTrue:[gXtype _ self typeCheck: aVar shouldBe: GXInteger paramName: ''gXtype'']';

addInstVarName: #gXvariable;
compile:
'gXvariable: aVar
	aVar notNil ifTrue:[ gXvariable _ aVar ]';
compile:
'p2: aVar
	aVar notNil ifTrue:[ gXvariable _ aVar ]';

addInstVarName: #gXlabel;
compile:
'gXlabel: aVar
	aVar notNil ifTrue:[gXlabel _ self typeCheck: aVar shouldBe: GXString paramName: ''gXlabel'']';
compile:
'p3: aVar
	aVar notNil ifTrue:[gXlabel _ self typeCheck: aVar shouldBe: GXString paramName: ''gXlabel'']';

addInstVarName: #gXcomment;
compile:
'gXcomment: aVar
	aVar notNil ifTrue:[gXcomment _ self typeCheck: aVar shouldBe: GXString paramName: ''gXcomment'']';
compile:
'p4: aVar
	aVar notNil ifTrue:[gXcomment _ self typeCheck: aVar shouldBe: GXString paramName: ''gXcomment'']';

addInstVarName: #gXfile;
compile:
'gXfile: aVar
	aVar notNil ifTrue:[gXfile _ self typeCheck: aVar shouldBe: GXString paramName: ''gXfile'']';
compile:
'p5: aVar
	aVar notNil ifTrue:[gXfile _ self typeCheck: aVar shouldBe: GXString paramName: ''gXfile'']';

addInstVarName: #gXmode;
compile:
'gXmode: aVar
	aVar notNil ifTrue:[gXmode _ self typeCheck: aVar shouldBe: GXInteger paramName: ''gXmode'']';
compile:
'p6: aVar
	aVar notNil ifTrue:[gXmode _ self typeCheck: aVar shouldBe: GXInteger paramName: ''gXmode'']'!

"result
 ------"

GXUputUg3

addInstVarName: #gXnblocks
!

"Local variables
 ---------------"

GXUputUg3

addInstVarName: #gXnblocks
!

!GXUputUg3 methodsFor: 'initializing'!

initVars
	gXnblocks _ 0
!!

"BODY
 ----"

!GXUputUg3 methodsFor: 'running'!

evaluate
GXprintin run line: 0 proc: self method: false; p1:'PUT: Writing variable to "'; p2:(gXfile); p3:'"'; condEvaluate.
GXprintin run line: 0 proc: self method: false; p1:'PUT: Label = "'; p2:(gXlabel); p3:'", Comment = "'; p4:(gXcomment); p5:'"'; condEvaluate.
GXcallout run line: 0 proc: self method: false; gXfacility:'extFortran'; gXfunction:19; condEvaluate.
(GXgUerrorUhandler run line: 0 proc: self method: true; condEvaluate) ifTrue: [ 
gXnblocks _ (gXU) deepCopy ].
self up.
^gXnblocks
!!
nil! "(fix for smalltalk bug, this line is ignored if there is a variable declaration error)"

"End of procedure"

GXcomment run line: 0 proc: self method: false; p1:'_put_g3'; p2:'Callout procedure for writing an Open Genie Intermediate file'; condEvaluate!
"Procedure class definition:
 ==========================="
((Smalltalk at: #GXput ifAbsent: [ nil ]) isKindOf: Function class) ifTrue: [ ('Warning: Overriding previously defined procedure ', 'GXput' gtype) prinl ].
Smalltalk at: #GXput put: nil!
Function subclass: 'GXput'
       instanceVariableNames: ''
       classVariableNames: ''
       poolDictionaries: ''
       category: 'Genie Procedure'
!

"HEADER
 ------"

"qualifiers
 ----------"

GXput

addInstVarName: #gXappend;
compile:
'gXappend
	gXappend _ true';
compile:
'gXappend: aBoolean
	gXappend _ aBoolean';

addInstVarName: #gXnew;
compile:
'gXnew
	gXnew _ true';
compile:
'gXnew: aBoolean
	gXnew _ aBoolean';

addInstVarName: #gXg3;
compile:
'gXg3
	gXg3 _ true';
compile:
'gXg3: aBoolean
	gXg3 _ aBoolean';

addInstVarName: #gXhdf;
compile:
'gXhdf
	gXhdf _ true';
compile:
'gXhdf: aBoolean
	gXhdf _ aBoolean';

addInstVarName: #gXascii;
compile:
'gXascii
	gXascii _ true';
compile:
'gXascii: aBoolean
	gXascii _ aBoolean'!

"parameters
 ----------"

GXput

addInstVarName: #gXgv;
compile:
'gXgv: aVar
	aVar notNil ifTrue:[ gXgv _ aVar ]';
compile:
'p1: aVar
	aVar notNil ifTrue:[ gXgv _ aVar ]';

addInstVarName: #gXlabel;
compile:
'gXlabel: aVar
	aVar notNil ifTrue:[gXlabel _ self typeCheck: aVar shouldBe: GXString paramName: ''gXlabel'']';
compile:
'p2: aVar
	aVar notNil ifTrue:[gXlabel _ self typeCheck: aVar shouldBe: GXString paramName: ''gXlabel'']';

addInstVarName: #gXcomment;
compile:
'gXcomment: aVar
	aVar notNil ifTrue:[gXcomment _ self typeCheck: aVar shouldBe: GXString paramName: ''gXcomment'']';
compile:
'p3: aVar
	aVar notNil ifTrue:[gXcomment _ self typeCheck: aVar shouldBe: GXString paramName: ''gXcomment'']';

addInstVarName: #gXfile;
compile:
'gXfile: aVar
	aVar notNil ifTrue:[gXfile _ self typeCheck: aVar shouldBe: GXString paramName: ''gXfile'']';
compile:
'p4: aVar
	aVar notNil ifTrue:[gXfile _ self typeCheck: aVar shouldBe: GXString paramName: ''gXfile'']'!

"result
 ------"

GXput

addInstVarName: #gXnblocks
!

"Local variables
 ---------------"

GXput

addInstVarName: #gXfile!

"Local variables
 ---------------"

GXput

addInstVarName: #gXmode!

"Local variables
 ---------------"

GXput

addInstVarName: #gXcomment;

addInstVarName: #gXlabel!

"Local variables
 ---------------"

GXput

addInstVarName: #gXtype
!

!GXput methodsFor: 'initializing'!

initVars
	gXappend _ false.
	gXnew _ false.
	gXg3 _ false.
	gXhdf _ false.
	gXascii _ false.
	gXfile _ (gXUcurrentUoutputUsource).
	gXmode _ 4.
	gXcomment _ '<no comment>'.
	gXlabel _ '<no label>'.
	gXtype _ 3
!!

"BODY
 ----"

!GXput methodsFor: 'running'!

evaluate

    gXfile isNil ifTrue: [^'PUT: Parameter FILE not specified and no default available' prenl].
    gXgv isNil ifTrue: ['PUT: Warning - writing <undefined> GENIE variable' prinl]
.
(gXhdf) ifTrue: [ 
gXtype _ 5 deepCopy ].
(gXascii) ifTrue: [ 
gXtype _ 8 deepCopy ].
(gXnew) not ifTrue: [ 
((((gXtype) = 3)) & (((GXfiletype run line: 0 proc: self method: true; p1:(gXfile); condEvaluate) ~= 'G3 intermediate'))) ifTrue: [ 
GXprinten run line: 0 proc: self method: false; p1:'PUT: "'; p2:(gXfile); p3:'" is not of type "G3 intermediate"'; condEvaluate.
[nil.
self up.
^gXnblocks] value ].
((((gXtype) = 5)) & (((GXfiletype run line: 0 proc: self method: true; p1:(gXfile); condEvaluate) ~= 'HDF'))) ifTrue: [ 
GXprinten run line: 0 proc: self method: false; p1:'PUT: "'; p2:(gXfile); p3:'" is not of type "HDF"'; condEvaluate.
[nil.
self up.
^gXnblocks] value ].
((((gXtype) = 8)) & (((GXfiletype run line: 0 proc: self method: true; p1:(gXfile); condEvaluate) ~= 'G3 ASCII'))) ifTrue: [ 
GXprinten run line: 0 proc: self method: false; p1:'PUT: "'; p2:(gXfile); p3:'" is not of type "G3 ASCII"'; condEvaluate.
[nil.
self up.
^gXnblocks] value ] ].
(gXappend) ifTrue: [ 
gXmode _ 4 deepCopy ].
(gXnew) ifTrue: [ 
gXmode _ 2 deepCopy ].
((gXtype) = 8) ifTrue: [ 
gXnblocks _ (GXUputUg3Uascii run line: 0 proc: self method: true; p1:(gXtype); p2:(gXgv); p3:(gXlabel); p4:(gXcomment); p5:(gXfile); p6:(gXmode); condEvaluate) deepCopy ] ifFalse: [ 
gXnblocks _ (GXUputUg3 run line: 0 proc: self method: true; p1:(gXtype); p2:(gXgv); p3:(gXlabel); p4:(gXcomment); p5:(gXfile); p6:(gXmode); condEvaluate) deepCopy ] .
GXprintin run line: 0 proc: self method: false; p1:(gXnblocks); p2:' blocks now in '; p3:(gXfile); condEvaluate.
self up.
^gXnblocks
!!
nil! "(fix for smalltalk bug, this line is ignored if there is a variable declaration error)"

"End of procedure"

GXcomment run line: 0 proc: self method: false; p1:'put'; p2:'Place a Genie variable into an Open Genie intermediate file'; condEvaluate!
"Procedure class definition:
 ==========================="
((Smalltalk at: #GXUintegrateUwksp ifAbsent: [ nil ]) isKindOf: Function class) ifTrue: [ ('Warning: Overriding previously defined procedure ', 'GXUintegrateUwksp' gtype) prinl ].
Smalltalk at: #GXUintegrateUwksp put: nil!
Function subclass: 'GXUintegrateUwksp'
       instanceVariableNames: ''
       classVariableNames: ''
       poolDictionaries: ''
       category: 'Genie Procedure'
!

"HEADER
 ------"

"parameters
 ----------"

GXUintegrateUwksp

addInstVarName: #gXwksp;
compile:
'gXwksp: aVar
	aVar notNil ifTrue:[gXwksp _ self typeCheck: aVar shouldBe: GXWorkspace paramName: ''gXwksp'']';
compile:
'p1: aVar
	aVar notNil ifTrue:[gXwksp _ self typeCheck: aVar shouldBe: GXWorkspace paramName: ''gXwksp'']';

addInstVarName: #gXxmin;
compile:
'gXxmin: aVar
	aVar notNil ifTrue:[gXxmin _ self typeCheck: aVar shouldBe: GXRealarray paramName: ''gXxmin'']';
compile:
'p2: aVar
	aVar notNil ifTrue:[gXxmin _ self typeCheck: aVar shouldBe: GXRealarray paramName: ''gXxmin'']';

addInstVarName: #gXxmax;
compile:
'gXxmax: aVar
	aVar notNil ifTrue:[gXxmax _ self typeCheck: aVar shouldBe: GXRealarray paramName: ''gXxmax'']';
compile:
'p3: aVar
	aVar notNil ifTrue:[gXxmax _ self typeCheck: aVar shouldBe: GXRealarray paramName: ''gXxmax'']'!

"result
 ------"

GXUintegrateUwksp

addInstVarName: #gXres
!

!GXUintegrateUwksp methodsFor: 'initializing'!

initVars
	gXres _ (GXfields run line: 0 proc: self method: true; condEvaluate)
!!

"BODY
 ----"

!GXUintegrateUwksp methodsFor: 'running'!

evaluate
GXcallout run line: 0 proc: self method: false; gXfacility:'extTransform'; gXfunction:12; condEvaluate.
GXgUerrorUhandler run line: 0 proc: self method: false; condEvaluate.
self up.
^gXres
!!
nil! "(fix for smalltalk bug, this line is ignored if there is a variable declaration error)"

"End of procedure"

GXcomment run line: 0 proc: self method: false; p1:'_integrate_wksp'; p2:'Helper application for workspace integration'; condEvaluate!
"Procedure class definition:
 ==========================="
((Smalltalk at: #GXUintegrateUiarray ifAbsent: [ nil ]) isKindOf: Function class) ifTrue: [ ('Warning: Overriding previously defined procedure ', 'GXUintegrateUiarray' gtype) prinl ].
Smalltalk at: #GXUintegrateUiarray put: nil!
Function subclass: 'GXUintegrateUiarray'
       instanceVariableNames: ''
       classVariableNames: ''
       poolDictionaries: ''
       category: 'Genie Procedure'
!

"HEADER
 ------"

"parameters
 ----------"

GXUintegrateUiarray

addInstVarName: #gXiarray;
compile:
'gXiarray: aVar
	aVar notNil ifTrue:[gXiarray _ self typeCheck: aVar shouldBe: GXIntegerarray paramName: ''gXiarray'']';
compile:
'p1: aVar
	aVar notNil ifTrue:[gXiarray _ self typeCheck: aVar shouldBe: GXIntegerarray paramName: ''gXiarray'']';

addInstVarName: #gXxmin;
compile:
'gXxmin: aVar
	aVar notNil ifTrue:[gXxmin _ self typeCheck: aVar shouldBe: GXRealarray paramName: ''gXxmin'']';
compile:
'p2: aVar
	aVar notNil ifTrue:[gXxmin _ self typeCheck: aVar shouldBe: GXRealarray paramName: ''gXxmin'']';

addInstVarName: #gXxmax;
compile:
'gXxmax: aVar
	aVar notNil ifTrue:[gXxmax _ self typeCheck: aVar shouldBe: GXRealarray paramName: ''gXxmax'']';
compile:
'p3: aVar
	aVar notNil ifTrue:[gXxmax _ self typeCheck: aVar shouldBe: GXRealarray paramName: ''gXxmax'']';

addInstVarName: #gXfile;
compile:
'gXfile: aVar
	aVar notNil ifTrue:[gXfile _ self typeCheck: aVar shouldBe: GXString paramName: ''gXfile'']';
compile:
'p4: aVar
	aVar notNil ifTrue:[gXfile _ self typeCheck: aVar shouldBe: GXString paramName: ''gXfile'']'!

"result
 ------"

GXUintegrateUiarray

addInstVarName: #gXres
!

"Local variables
 ---------------"

GXUintegrateUiarray

addInstVarName: #gXres;

addInstVarName: #gXfile
!

!GXUintegrateUiarray methodsFor: 'initializing'!

initVars
	gXres _ (GXfields run line: 0 proc: self method: true; condEvaluate).
	gXfile _ (gXUcurrentUinputUsource)
!!

"BODY
 ----"

!GXUintegrateUiarray methodsFor: 'running'!

evaluate
GXcallout run line: 0 proc: self method: false; gXfacility:'extTransform'; gXfunction:13; condEvaluate.
self up.
^gXres
!!
nil! "(fix for smalltalk bug, this line is ignored if there is a variable declaration error)"

"End of procedure"

GXcomment run line: 0 proc: self method: false; p1:'_integrate_iarray'; p2:'Helper application for spectra range integration'; condEvaluate!
"Procedure class definition:
 ==========================="
((Smalltalk at: #GXintegrate ifAbsent: [ nil ]) isKindOf: Function class) ifTrue: [ ('Warning: Overriding previously defined procedure ', 'GXintegrate' gtype) prinl ].
Smalltalk at: #GXintegrate put: nil!
Function subclass: 'GXintegrate'
       instanceVariableNames: ''
       classVariableNames: ''
       poolDictionaries: ''
       category: 'Genie Procedure'
!

"HEADER
 ------"

"parameters
 ----------"

GXintegrate

addInstVarName: #gXp1;
compile:
'gXp1: aVar
	aVar notNil ifTrue:[ gXp1 _ aVar ]';
compile:
'p1: aVar
	aVar notNil ifTrue:[ gXp1 _ aVar ]';

addInstVarName: #gXxmin;
compile:
'gXxmin: aVar
	aVar notNil ifTrue:[ gXxmin _ aVar ]';
compile:
'p2: aVar
	aVar notNil ifTrue:[ gXxmin _ aVar ]';

addInstVarName: #gXxmax;
compile:
'gXxmax: aVar
	aVar notNil ifTrue:[ gXxmax _ aVar ]';
compile:
'p3: aVar
	aVar notNil ifTrue:[ gXxmax _ aVar ]';

addInstVarName: #gXfile;
compile:
'gXfile: aVar
	aVar notNil ifTrue:[gXfile _ self typeCheck: aVar shouldBe: GXString paramName: ''gXfile'']';
compile:
'p4: aVar
	aVar notNil ifTrue:[gXfile _ self typeCheck: aVar shouldBe: GXString paramName: ''gXfile'']';

addInstVarName: #gXwksp;
compile:
'gXwksp: aVar
	aVar notNil ifTrue:[gXwksp _ self typeCheck: aVar shouldBe: GXWorkspace paramName: ''gXwksp'']';
compile:
'p5: aVar
	aVar notNil ifTrue:[gXwksp _ self typeCheck: aVar shouldBe: GXWorkspace paramName: ''gXwksp'']';

addInstVarName: #gXspectra;
compile:
'gXspectra: aVar
	aVar notNil ifTrue:[ gXspectra _ aVar ]';
compile:
'p6: aVar
	aVar notNil ifTrue:[ gXspectra _ aVar ]'!

"result
 ------"

GXintegrate

addInstVarName: #gXres
!

"Local variables
 ---------------"

GXintegrate

addInstVarName: #gXiarray;

addInstVarName: #gXfile
!

!GXintegrate methodsFor: 'initializing'!

initVars
	gXfile _ (gXUcurrentUinputUsource)
!!

"BODY
 ----"

!GXintegrate methodsFor: 'running'!

evaluate

    (gXxmin isKindOf: Float) ifTrue: [ gXxmin _ (GXRealarray new: 1) at: 1 put: gXxmin; yourself ].
    (gXxmax isKindOf: Float) ifTrue: [ gXxmax _ (GXRealarray new: 1) at: 1 put: gXxmax; yourself ].
    gXspectra isNil ifFalse: [ gXp1 _ gXspectra ].
    (gXp1 isKindOf: GXWorkspace) ifTrue: [ gXwksp _ gXp1 ].
    (gXp1 isKindOf: GXIntegerarray) ifTrue: [ gXiarray _ gXp1 ].
    (gXp1 isKindOf: Interval) ifTrue: [ gXiarray _ gXp1 asIntegerArray ].
    (gXp1 isKindOf: Integer) ifTrue: [ gXiarray _ (GXIntegerarray new: 1) at: 1 put: gXp1; yourself ].
    ((gXxmin isNil) | (gXxmin isKindOf: GXRealarray)) 
	ifFalse: [ ^'INTEGRATE: Invalid type for XMIN' prenl ].
    ((gXxmax isNil) | (gXxmax isKindOf: GXRealarray)) 
	ifFalse: [ ^'INTEGRATE: Invalid type for XMAX' prenl ]
.
(GXdefined run line: 0 proc: self method: true; p1:(gXwksp); condEvaluate) ifTrue: [ 
gXres _ (GXUintegrateUwksp run line: 0 proc: self method: true; p1:(gXwksp); p2:(gXxmin); p3:(gXxmax); condEvaluate)  ].
(GXdefined run line: 0 proc: self method: true; p1:(gXiarray); condEvaluate) ifTrue: [ 
gXres _ (GXUintegrateUiarray run line: 0 proc: self method: true; p1:(gXiarray); p2:(gXxmin); p3:(gXxmax); p4:(gXfile); condEvaluate)  ].
((GXdefined run line: 0 proc: self method: true; p1:(gXres); condEvaluate) & (GXcalledUasUfunction run line: 0 proc: self method: true; condEvaluate) not) ifTrue: [ 
GXprintn run line: 0 proc: self method: false; p1:'Sum of bins = '; p2:((gXres) at:#gXsum); p3:' counts'; condEvaluate.
GXprintn run line: 0 proc: self method: false; p1:'Error = '; p2:((gXres) at:#gXerror); p3:' counts'; condEvaluate ].
self up.
^gXres
!!
nil! "(fix for smalltalk bug, this line is ignored if there is a variable declaration error)"

"End of procedure"

GXcomment run line: 0 proc: self method: false; p1:'integrate'; p2:'sum bins in a spectrum workspace'; condEvaluate!
"Procedure class definition:
 ==========================="
((Smalltalk at: #GXUunits ifAbsent: [ nil ]) isKindOf: Function class) ifTrue: [ ('Warning: Overriding previously defined procedure ', 'GXUunits' gtype) prinl ].
Smalltalk at: #GXUunits put: nil!
Function subclass: 'GXUunits'
       instanceVariableNames: ''
       classVariableNames: ''
       poolDictionaries: ''
       category: 'Genie Procedure'
!

"HEADER
 ------"

"parameters
 ----------"

GXUunits

addInstVarName: #gXiunit;
compile:
'gXiunit: aVar
	aVar notNil ifTrue:[gXiunit _ self typeCheck: aVar shouldBe: GXInteger paramName: ''gXiunit'']';
compile:
'p1: aVar
	aVar notNil ifTrue:[gXiunit _ self typeCheck: aVar shouldBe: GXInteger paramName: ''gXiunit'']';

addInstVarName: #gXworksp;
compile:
'gXworksp: aVar
	aVar notNil ifTrue:[gXworksp _ self typeCheck: aVar shouldBe: GXWorkspace paramName: ''gXworksp'']';
compile:
'p2: aVar
	aVar notNil ifTrue:[gXworksp _ self typeCheck: aVar shouldBe: GXWorkspace paramName: ''gXworksp'']';

addInstVarName: #gXxmin;
compile:
'gXxmin: aVar
	aVar notNil ifTrue:[gXxmin _ self typeCheck: aVar shouldBe: GXReal paramName: ''gXxmin'']';
compile:
'p3: aVar
	aVar notNil ifTrue:[gXxmin _ self typeCheck: aVar shouldBe: GXReal paramName: ''gXxmin'']';

addInstVarName: #gXxmax;
compile:
'gXxmax: aVar
	aVar notNil ifTrue:[gXxmax _ self typeCheck: aVar shouldBe: GXReal paramName: ''gXxmax'']';
compile:
'p4: aVar
	aVar notNil ifTrue:[gXxmax _ self typeCheck: aVar shouldBe: GXReal paramName: ''gXxmax'']'!

"result
 ------"

GXUunits

addInstVarName: #gXtUworksp
!

"Local variables
 ---------------"

GXUunits

addInstVarName: #gXiunit
!

!GXUunits methodsFor: 'initializing'!

initVars
	gXiunit _ (gXU)
!!

"BODY
 ----"

!GXUunits methodsFor: 'running'!

evaluate
gXtUworksp _ (gXworksp) deepCopy.
GXcallout run line: 0 proc: self method: false; gXfacility:'extTransform'; gXfunction:20; condEvaluate.
(GXgUerrorUhandler run line: 0 proc: self method: true; condEvaluate) ifTrue: [ 
gXtUworksp _ (gXU) deepCopy ].
self up.
^gXtUworksp
!!
nil! "(fix for smalltalk bug, this line is ignored if there is a variable declaration error)"

"End of procedure"

GXcomment run line: 0 proc: self method: false; p1:'_units'; p2:'Callout procedure for UNITS'; condEvaluate!
"Procedure class definition:
 ==========================="
((Smalltalk at: #GXunits ifAbsent: [ nil ]) isKindOf: Function class) ifTrue: [ ('Warning: Overriding previously defined procedure ', 'GXunits' gtype) prinl ].
Smalltalk at: #GXunits put: nil!
Function subclass: 'GXunits'
       instanceVariableNames: ''
       classVariableNames: ''
       poolDictionaries: ''
       category: 'Genie Procedure'
!

"HEADER
 ------"

"qualifiers
 ----------"

GXunits

addInstVarName: #gXc;
compile:
'gXc
	gXc _ true';
compile:
'gXc: aBoolean
	gXc _ aBoolean';

addInstVarName: #gXchannel;
compile:
'gXchannel
	gXchannel _ true';
compile:
'gXchannel: aBoolean
	gXchannel _ aBoolean';

addInstVarName: #gXd;
compile:
'gXd
	gXd _ true';
compile:
'gXd: aBoolean
	gXd _ aBoolean';

addInstVarName: #gXdspacing;
compile:
'gXdspacing
	gXdspacing _ true';
compile:
'gXdspacing: aBoolean
	gXdspacing _ aBoolean';

addInstVarName: #gXe;
compile:
'gXe
	gXe _ true';
compile:
'gXe: aBoolean
	gXe _ aBoolean';

addInstVarName: #gXenergy;
compile:
'gXenergy
	gXenergy _ true';
compile:
'gXenergy: aBoolean
	gXenergy _ aBoolean';

addInstVarName: #gXlam;
compile:
'gXlam
	gXlam _ true';
compile:
'gXlam: aBoolean
	gXlam _ aBoolean';

addInstVarName: #gXq;
compile:
'gXq
	gXq _ true';
compile:
'gXq: aBoolean
	gXq _ aBoolean';

addInstVarName: #gXsq;
compile:
'gXsq
	gXsq _ true';
compile:
'gXsq: aBoolean
	gXsq _ aBoolean';

addInstVarName: #gXt;
compile:
'gXt
	gXt _ true';
compile:
'gXt: aBoolean
	gXt _ aBoolean';

addInstVarName: #gXtau;
compile:
'gXtau
	gXtau _ true';
compile:
'gXtau: aBoolean
	gXtau _ aBoolean';

addInstVarName: #gXla1;
compile:
'gXla1
	gXla1 _ true';
compile:
'gXla1: aBoolean
	gXla1 _ aBoolean';

addInstVarName: #gXtau;
compile:
'gXtau
	gXtau _ true';
compile:
'gXtau: aBoolean
	gXtau _ aBoolean';

addInstVarName: #gXw;
compile:
'gXw
	gXw _ true';
compile:
'gXw: aBoolean
	gXw _ aBoolean';

addInstVarName: #gXwn;
compile:
'gXwn
	gXwn _ true';
compile:
'gXwn: aBoolean
	gXwn _ aBoolean'!

"parameters
 ----------"

GXunits

addInstVarName: #gXwksp;
compile:
'gXwksp: aVar
	aVar notNil ifTrue:[gXwksp _ self typeCheck: aVar shouldBe: GXWorkspace paramName: ''gXwksp'']';
compile:
'p1: aVar
	aVar notNil ifTrue:[gXwksp _ self typeCheck: aVar shouldBe: GXWorkspace paramName: ''gXwksp'']';

addInstVarName: #gXxmin;
compile:
'gXxmin: aVar
	aVar notNil ifTrue:[gXxmin _ self typeCheck: aVar shouldBe: GXReal paramName: ''gXxmin'']';
compile:
'p2: aVar
	aVar notNil ifTrue:[gXxmin _ self typeCheck: aVar shouldBe: GXReal paramName: ''gXxmin'']';

addInstVarName: #gXxmax;
compile:
'gXxmax: aVar
	aVar notNil ifTrue:[gXxmax _ self typeCheck: aVar shouldBe: GXReal paramName: ''gXxmax'']';
compile:
'p3: aVar
	aVar notNil ifTrue:[gXxmax _ self typeCheck: aVar shouldBe: GXReal paramName: ''gXxmax'']'!

"result
 ------"

GXunits

addInstVarName: #gXtUworksp
!

"Local variables
 ---------------"

GXunits

addInstVarName: #gXiunit!

"Local variables
 ---------------"

GXunits

addInstVarName: #gXunit!

"Local variables
 ---------------"

GXunits

addInstVarName: #gXntc
!

!GXunits methodsFor: 'initializing'!

initVars
	gXc _ false.
	gXchannel _ false.
	gXd _ false.
	gXdspacing _ false.
	gXe _ false.
	gXenergy _ false.
	gXlam _ false.
	gXq _ false.
	gXsq _ false.
	gXt _ false.
	gXtau _ false.
	gXla1 _ false.
	gXtau _ false.
	gXw _ false.
	gXwn _ false.
	gXiunit _ 1 negated.
	gXunit _ (gXU)
!!

"BODY
 ----"

!GXunits methodsFor: 'running'!

evaluate

    gXwksp isNil ifTrue: [^'UNITS: Parameter WKSP must be specified' prenl]
.
((gXw) & (GXdefined run line: 0 proc: self method: true; p1:(gXxmin); condEvaluate) not) ifTrue: [ 
gXxmin _ 1.000000000000000e003 negated deepCopy ].
((gXw) & (GXdefined run line: 0 proc: self method: true; p1:(gXxmax); condEvaluate) not) ifTrue: [ 
gXxmax _ 5.000000000000000e003 deepCopy ].
(((gXc) | (gXchannel))) ifTrue: [ 
gXiunit _ 1 deepCopy.
gXunit _ 'channel numbers (a one way trip)' deepCopy ] ifFalse: [ (((gXt) | (gXtau))) ifTrue: [ 
gXiunit _ 2 deepCopy.
gXunit _ 'time of flight (us)' deepCopy ] ifFalse: [ (gXlam) ifTrue: [ 
gXiunit _ 3 deepCopy.
gXunit _ 'wavelength (A)' deepCopy ] ifFalse: [ (gXd) ifTrue: [ 
gXiunit _ 4 deepCopy.
gXunit _ 'd-spacing (A)' deepCopy ] ifFalse: [ (gXe) ifTrue: [ 
gXiunit _ 5 deepCopy.
gXunit _ 'energy (meV)' deepCopy ] ifFalse: [ (gXq) ifTrue: [ 
gXiunit _ 6 deepCopy.
gXunit _ 'momentum transfer (A**-1)' deepCopy ] ifFalse: [ (gXsq) ifTrue: [ 
gXiunit _ 7 deepCopy.
gXunit _ 'momentum transfer squared (A**-2)' deepCopy ] ifFalse: [ (gXtau) ifTrue: [ 
gXiunit _ 8 deepCopy.
gXunit _ 'reciprocal time-of-flight (us/m)' deepCopy ] ifFalse: [ (gXw) ifTrue: [ 
gXiunit _ 9 deepCopy.
gXunit _ 'energy transfer E1-E2 (meV)' deepCopy ] ifFalse: [ (gXla1) ifTrue: [ 
gXiunit _ 10 deepCopy.
gXunit _ 'primary flight-path wavelength (A)' deepCopy ] ifFalse: [ (gXwn) ifTrue: [ 
gXiunit _ 11 deepCopy.
gXunit _ 'energy transfer E1-E2 (cm-1)' deepCopy ] ifFalse: [ 
GXprinten run line: 0 proc: self method: false; p1:'UNITS: Invalid or non-existent unit qualifier'; condEvaluate ] ]]]]]]]]]].
(GXdefined run line: 0 proc: self method: true; p1:(gXunit); condEvaluate) ifTrue: [ 
GXprintin run line: 0 proc: self method: false; p1:'UNITS: Changing units to '; p2:(gXunit); p3:' ...'; condEvaluate.
(gXt) ifTrue: [ 
GXprintin run line: 0 proc: self method: false; p1:'UNITS: Using hold off (DELTA) of '; p2:((gXwksp) at:#gXdelta); p3:' microseconds ...'; condEvaluate ].

	gXtUworksp _ (GXUunits run line: 0 proc: self method: true; 
		p1:(gXiunit); p2:(gXwksp); p3:(gXxmin); p4:(gXxmax); condEvaluate)
    .
gXntc _ ((gXtUworksp) at:#gXntc) deepCopy.
GXprintin run line: 0 proc: self method: false; p1:'UNITS: X range now '; p2:(((gXtUworksp) at:#gXx) at:((Array new:1) at: 1 put: 1; yourself)); p3:' to '; p4:(((gXtUworksp) at:#gXx) at:((Array new:1) at: 1 put: ((gXntc) + 1); yourself)); condEvaluate ] ifFalse: [ 
GXprinten run line: 0 proc: self method: false; p1:'UNITS: No unit type specified'; condEvaluate ] .
(GXcalledUasUfunction run line: 0 proc: self method: true; condEvaluate) not ifTrue: [ 
 gXwksp become: gXtUworksp  ].
self up.
^gXtUworksp
!!
nil! "(fix for smalltalk bug, this line is ignored if there is a variable declaration error)"

"End of procedure"

GXcomment run line: 0 proc: self method: false; p1:'units'; p2:'Alter units of a spectrum workspace'; condEvaluate!
"Procedure class definition:
 ==========================="
((Smalltalk at: #GXUdirUintUfile ifAbsent: [ nil ]) isKindOf: Function class) ifTrue: [ ('Warning: Overriding previously defined procedure ', 'GXUdirUintUfile' gtype) prinl ].
Smalltalk at: #GXUdirUintUfile put: nil!
Function subclass: 'GXUdirUintUfile'
       instanceVariableNames: ''
       classVariableNames: ''
       poolDictionaries: ''
       category: 'Genie Procedure'
!

"HEADER
 ------"

"parameters
 ----------"

GXUdirUintUfile

addInstVarName: #gXfullUlisting;
compile:
'gXfullUlisting: aVar
	aVar notNil ifTrue:[gXfullUlisting _ self typeCheck: aVar shouldBe: GXInteger paramName: ''gXfullUlisting'']';
compile:
'p1: aVar
	aVar notNil ifTrue:[gXfullUlisting _ self typeCheck: aVar shouldBe: GXInteger paramName: ''gXfullUlisting'']';

addInstVarName: #gXfile;
compile:
'gXfile: aVar
	aVar notNil ifTrue:[gXfile _ self typeCheck: aVar shouldBe: GXString paramName: ''gXfile'']';
compile:
'p2: aVar
	aVar notNil ifTrue:[gXfile _ self typeCheck: aVar shouldBe: GXString paramName: ''gXfile'']'!

"result
 ------"

GXUdirUintUfile

addInstVarName: #gXinfo
!

"Local variables
 ---------------"

GXUdirUintUfile

addInstVarName: #gXinfo
!

!GXUdirUintUfile methodsFor: 'initializing'!

initVars
	gXinfo _ (GXfields run line: 0 proc: self method: true; p1:6; condEvaluate)
!!

"BODY
 ----"

!GXUdirUintUfile methodsFor: 'running'!

evaluate
GXcallout run line: 0 proc: self method: false; gXfacility:'extFortran'; gXfunction:20; condEvaluate.
(GXgUerrorUhandler run line: 0 proc: self method: true; condEvaluate) ifTrue: [ 
gXinfo _ (gXU) deepCopy ].
self up.
^gXinfo
!!
nil! "(fix for smalltalk bug, this line is ignored if there is a variable declaration error)"

"End of procedure"

GXcomment run line: 0 proc: self method: false; p1:'_dir_int_file'; p2:'Callout procedure for DIR command'; condEvaluate!
"Procedure class definition:
 ==========================="
((Smalltalk at: #GXlist ifAbsent: [ nil ]) isKindOf: Function class) ifTrue: [ ('Warning: Overriding previously defined procedure ', 'GXlist' gtype) prinl ].
Smalltalk at: #GXlist put: nil!
Function subclass: 'GXlist'
       instanceVariableNames: ''
       classVariableNames: ''
       poolDictionaries: ''
       category: 'Genie Procedure'
!

"HEADER
 ------"

"qualifiers
 ----------"

GXlist

addInstVarName: #gXfull;
compile:
'gXfull
	gXfull _ true';
compile:
'gXfull: aBoolean
	gXfull _ aBoolean';

addInstVarName: #gXin;
compile:
'gXin
	gXin _ true';
compile:
'gXin: aBoolean
	gXin _ aBoolean';

addInstVarName: #gXout;
compile:
'gXout
	gXout _ true';
compile:
'gXout: aBoolean
	gXout _ aBoolean'!

"parameters
 ----------"

GXlist

addInstVarName: #gXfile;
compile:
'gXfile: aVar
	aVar notNil ifTrue:[gXfile _ self typeCheck: aVar shouldBe: GXString paramName: ''gXfile'']';
compile:
'p1: aVar
	aVar notNil ifTrue:[gXfile _ self typeCheck: aVar shouldBe: GXString paramName: ''gXfile'']'!

"result
 ------"

GXlist

addInstVarName: #gXinfo
!

"Local variables
 ---------------"

GXlist

addInstVarName: #gXfullUlisting
!

!GXlist methodsFor: 'initializing'!

initVars
	gXfull _ false.
	gXin _ false.
	gXout _ false.
	gXfullUlisting _ 0
!!

"BODY
 ----"

!GXlist methodsFor: 'running'!

evaluate
(gXin) ifTrue: [ 
gXfile _ (gXUcurrentUinputUsource) deepCopy ].
(gXout) ifTrue: [ 
gXfile _ (gXUcurrentUoutputUsource) deepCopy ].
(gXfull) ifTrue: [ 
gXfullUlisting _ 1 deepCopy ].
(GXdefined run line: 0 proc: self method: true; p1:(gXfile); condEvaluate) not ifTrue: [ 
GXprinten run line: 0 proc: self method: false; p1:'LIST: No input file specified'; condEvaluate.
[nil.
self up.
^gXinfo] value ].
gXinfo _ (GXUdirUintUfile run line: 0 proc: self method: true; p1:(gXfullUlisting); p2:(gXfile); condEvaluate) deepCopy.
((GXdefined run line: 0 proc: self method: true; p1:(gXinfo); condEvaluate) not | (GXcalledUasUfunction run line: 0 proc: self method: true; condEvaluate)) ifTrue: [ 
[nil.
self up.
^gXinfo] value ].
GXprintn run line: 0 proc: self method: false; p1:((gXinfo) at:#gXheader); condEvaluate.
(gXfull) ifTrue: [ 
Function new loop: 1 to: (GXlength run line: 0 proc: self method: true; p1:((gXinfo) at:#gXlabel); condEvaluate) by: 1 before: [ :gXi | 
GXprintn run line: 0 proc: self method: false; p1:'Block = '; p2:(gXi); p3:', Label = '; p4:(((gXinfo) at:#gXlabel) at:((Array new:1) at: 1 put: (gXi); yourself)); p5:', Type = '; p6:(((gXinfo) at:#gXtype) at:((Array new:1) at: 1 put: (gXi); yourself)); condEvaluate.
GXprintn run line: 0 proc: self method: false; p1:'Created: '; p2:(((gXinfo) at:#gXdate) at:((Array new:1) at: 1 put: (gXi); yourself)); p3:' by '; p4:(((gXinfo) at:#gXuser) at:((Array new:1) at: 1 put: (gXi); yourself)); condEvaluate.
GXprintn run line: 0 proc: self method: false; p1:'Comment: '; p2:(((gXinfo) at:#gXcomment) at:((Array new:1) at: 1 put: (gXi); yourself)); condEvaluate.
GXprintn run line: 0 proc: self method: false; condEvaluate ] 
condition: [false] after: [ :gXi | nil ] ] ifFalse: [ 
GXprintn run line: 0 proc: self method: false; p1:'Block 	 Label 		 Type'; condEvaluate.
Function new loop: 1 to: (GXlength run line: 0 proc: self method: true; p1:((gXinfo) at:#gXlabel); condEvaluate) by: 1 before: [ :gXi | 
GXprintn run line: 0 proc: self method: false; p1:(gXi); p2:'	'; p3:(((gXinfo) at:#gXlabel) at:((Array new:1) at: 1 put: (gXi); yourself)); p4:'	'; p5:(((gXinfo) at:#gXtype) at:((Array new:1) at: 1 put: (gXi); yourself)); condEvaluate ] 
condition: [false] after: [ :gXi | nil ] ] .
self up.
^gXinfo
!!
nil! "(fix for smalltalk bug, this line is ignored if there is a variable declaration error)"

"End of procedure"

GXcomment run line: 0 proc: self method: false; p1:'list'; p2:'Produce a listing (directory) of an Open Genie intermediate file'; condEvaluate!
"Procedure class definition:
 ==========================="
((Smalltalk at: #GXUcloseUstream ifAbsent: [ nil ]) isKindOf: Function class) ifTrue: [ ('Warning: Overriding previously defined procedure ', 'GXUcloseUstream' gtype) prinl ].
Smalltalk at: #GXUcloseUstream put: nil!
Function subclass: 'GXUcloseUstream'
       instanceVariableNames: ''
       classVariableNames: ''
       poolDictionaries: ''
       category: 'Genie Procedure'
!

"HEADER
 ------"

"parameters
 ----------"

GXUcloseUstream

addInstVarName: #gXstreamUid;
compile:
'gXstreamUid: aVar
	aVar notNil ifTrue:[gXstreamUid _ self typeCheck: aVar shouldBe: GXInteger paramName: ''gXstreamUid'']';
compile:
'p1: aVar
	aVar notNil ifTrue:[gXstreamUid _ self typeCheck: aVar shouldBe: GXInteger paramName: ''gXstreamUid'']'
!

"Local variables
 ---------------"

GXUcloseUstream

addInstVarName: #gXstreamUid
!

!GXUcloseUstream methodsFor: 'initializing'!

initVars
	gXstreamUid _ 1 negated
!!

"BODY
 ----"

!GXUcloseUstream methodsFor: 'running'!

evaluate
GXcallout run line: 0 proc: self method: false; gXfacility:'extFortran'; gXfunction:18; condEvaluate.
GXgUerrorUhandler run line: 0 proc: self method: false; condEvaluate.
self up.
^nil
!!
nil! "(fix for smalltalk bug, this line is ignored if there is a variable declaration error)"

"End of procedure"

GXcomment run line: 0 proc: self method: false; p1:'_close_stream'; p2:'Close a stream'; condEvaluate!
"Procedure class definition:
 ==========================="
((Smalltalk at: #GXUopenUstream ifAbsent: [ nil ]) isKindOf: Function class) ifTrue: [ ('Warning: Overriding previously defined procedure ', 'GXUopenUstream' gtype) prinl ].
Smalltalk at: #GXUopenUstream put: nil!
Function subclass: 'GXUopenUstream'
       instanceVariableNames: ''
       classVariableNames: ''
       poolDictionaries: ''
       category: 'Genie Procedure'
!

"HEADER
 ------"

"parameters
 ----------"

GXUopenUstream

addInstVarName: #gXfile;
compile:
'gXfile: aVar
	aVar notNil ifTrue:[gXfile _ self typeCheck: aVar shouldBe: GXString paramName: ''gXfile'']';
compile:
'p1: aVar
	aVar notNil ifTrue:[gXfile _ self typeCheck: aVar shouldBe: GXString paramName: ''gXfile'']';

addInstVarName: #gXmode;
compile:
'gXmode: aVar
	aVar notNil ifTrue:[gXmode _ self typeCheck: aVar shouldBe: GXInteger paramName: ''gXmode'']';
compile:
'p2: aVar
	aVar notNil ifTrue:[gXmode _ self typeCheck: aVar shouldBe: GXInteger paramName: ''gXmode'']'!

"result
 ------"

GXUopenUstream

addInstVarName: #gXstreamUid
!

!GXUopenUstream methodsFor: 'initializing'!

initVars

!!

"BODY
 ----"

!GXUopenUstream methodsFor: 'running'!

evaluate
GXcallout run line: 0 proc: self method: false; gXfacility:'extFortran'; gXfunction:17; condEvaluate.
(GXgUerrorUhandler run line: 0 proc: self method: true; condEvaluate) ifTrue: [ 
gXstreamUid _ 1 negated deepCopy ].
self up.
^gXstreamUid
!!
nil! "(fix for smalltalk bug, this line is ignored if there is a variable declaration error)"

"End of procedure"

GXcomment run line: 0 proc: self method: false; p1:'_open_stream'; p2:'Open a stream'; condEvaluate!
"Procedure class definition:
 ==========================="
((Smalltalk at: #GXUopen ifAbsent: [ nil ]) isKindOf: Function class) ifTrue: [ ('Warning: Overriding previously defined procedure ', 'GXUopen' gtype) prinl ].
Smalltalk at: #GXUopen put: nil!
Function subclass: 'GXUopen'
       instanceVariableNames: ''
       classVariableNames: ''
       poolDictionaries: ''
       category: 'Genie Procedure'
!

"HEADER
 ------"

"qualifiers
 ----------"

GXUopen

addInstVarName: #gXstream;
compile:
'gXstream
	gXstream _ true';
compile:
'gXstream: aBoolean
	gXstream _ aBoolean';

addInstVarName: #gXnew;
compile:
'gXnew
	gXnew _ true';
compile:
'gXnew: aBoolean
	gXnew _ aBoolean';

addInstVarName: #gXread;
compile:
'gXread
	gXread _ true';
compile:
'gXread: aBoolean
	gXread _ aBoolean';

addInstVarName: #gXappend;
compile:
'gXappend
	gXappend _ true';
compile:
'gXappend: aBoolean
	gXappend _ aBoolean'!

"parameters
 ----------"

GXUopen

addInstVarName: #gXfile;
compile:
'gXfile: aVar
	aVar notNil ifTrue:[gXfile _ self typeCheck: aVar shouldBe: GXString paramName: ''gXfile'']';
compile:
'p1: aVar
	aVar notNil ifTrue:[gXfile _ self typeCheck: aVar shouldBe: GXString paramName: ''gXfile'']'!

"result
 ------"

GXUopen

addInstVarName: #gXstreamUid
!

"Local variables
 ---------------"

GXUopen

addInstVarName: #gXmode
!

!GXUopen methodsFor: 'initializing'!

initVars
	gXstream _ false.
	gXnew _ false.
	gXread _ false.
	gXappend _ false.
	gXstreamUid _ (gXU).
	gXmode _ 1
!!

"BODY
 ----"

!GXUopen methodsFor: 'running'!

evaluate
(gXread) ifTrue: [ 
gXmode _ 1 deepCopy ].
(gXnew) ifTrue: [ 
gXmode _ 2 deepCopy ].
(gXappend) ifTrue: [ 
gXmode _ 4 deepCopy ].
(GXdefined run line: 0 proc: self method: true; p1:(gXfile); condEvaluate) ifTrue: [ 
gXstreamUid _ (GXUopenUstream run line: 0 proc: self method: true; p1:(gXfile); p2:(gXmode); condEvaluate) deepCopy ] ifFalse: [ 
GXprinten run line: 0 proc: self method: false; p1:'OPEN/STREAM - Please specify a file to open'; condEvaluate ] .
self up.
^gXstreamUid
!!
nil! "(fix for smalltalk bug, this line is ignored if there is a variable declaration error)"

"End of procedure"

GXcomment run line: 0 proc: self method: false; p1:'_open'; p2:'Connect a stream to a file'; condEvaluate!
"Procedure class definition:
 ==========================="
((Smalltalk at: #GXUclose ifAbsent: [ nil ]) isKindOf: Function class) ifTrue: [ ('Warning: Overriding previously defined procedure ', 'GXUclose' gtype) prinl ].
Smalltalk at: #GXUclose put: nil!
Function subclass: 'GXUclose'
       instanceVariableNames: ''
       classVariableNames: ''
       poolDictionaries: ''
       category: 'Genie Procedure'
!

"HEADER
 ------"

"qualifiers
 ----------"

GXUclose

addInstVarName: #gXstream;
compile:
'gXstream
	gXstream _ true';
compile:
'gXstream: aBoolean
	gXstream _ aBoolean'!

"parameters
 ----------"

GXUclose

addInstVarName: #gXstreamUid;
compile:
'gXstreamUid: aVar
	aVar notNil ifTrue:[gXstreamUid _ self typeCheck: aVar shouldBe: GXInteger paramName: ''gXstreamUid'']';
compile:
'p1: aVar
	aVar notNil ifTrue:[gXstreamUid _ self typeCheck: aVar shouldBe: GXInteger paramName: ''gXstreamUid'']'
!

!GXUclose methodsFor: 'initializing'!

initVars
	gXstream _ false
!!

"BODY
 ----"

!GXUclose methodsFor: 'running'!

evaluate
(GXdefined run line: 0 proc: self method: true; p1:(gXstreamUid); condEvaluate) ifTrue: [ 
GXUcloseUstream run line: 0 proc: self method: false; p1:(gXstreamUid); condEvaluate ] ifFalse: [ 
GXprinten run line: 0 proc: self method: false; p1:'CLOSE/STREAM - Please specify a stream to close'; condEvaluate ] .
self up.
^nil
!!
nil! "(fix for smalltalk bug, this line is ignored if there is a variable declaration error)"

"End of procedure"

GXcomment run line: 0 proc: self method: false; p1:'close'; p2:'Close a previously opened stream'; condEvaluate!
"Procedure class definition:
 ==========================="
((Smalltalk at: #GXUwriteUasciiUstream ifAbsent: [ nil ]) isKindOf: Function class) ifTrue: [ ('Warning: Overriding previously defined procedure ', 'GXUwriteUasciiUstream' gtype) prinl ].
Smalltalk at: #GXUwriteUasciiUstream put: nil!
Function subclass: 'GXUwriteUasciiUstream'
       instanceVariableNames: ''
       classVariableNames: ''
       poolDictionaries: ''
       category: 'Genie Procedure'
!

"HEADER
 ------"

"parameters
 ----------"

GXUwriteUasciiUstream

addInstVarName: #gXmode;
compile:
'gXmode: aVar
	aVar notNil ifTrue:[gXmode _ self typeCheck: aVar shouldBe: GXInteger paramName: ''gXmode'']';
compile:
'p1: aVar
	aVar notNil ifTrue:[gXmode _ self typeCheck: aVar shouldBe: GXInteger paramName: ''gXmode'']';

addInstVarName: #gXstream;
compile:
'gXstream: aVar
	aVar notNil ifTrue:[gXstream _ self typeCheck: aVar shouldBe: GXInteger paramName: ''gXstream'']';
compile:
'p2: aVar
	aVar notNil ifTrue:[gXstream _ self typeCheck: aVar shouldBe: GXInteger paramName: ''gXstream'']';

addInstVarName: #gXcomment;
compile:
'gXcomment: aVar
	aVar notNil ifTrue:[gXcomment _ self typeCheck: aVar shouldBe: GXString paramName: ''gXcomment'']';
compile:
'p3: aVar
	aVar notNil ifTrue:[gXcomment _ self typeCheck: aVar shouldBe: GXString paramName: ''gXcomment'']';

addInstVarName: #gXgv;
compile:
'gXgv: aVar
	aVar notNil ifTrue:[ gXgv _ aVar ]';
compile:
'p4: aVar
	aVar notNil ifTrue:[ gXgv _ aVar ]'
!

"Local variables
 ---------------"

GXUwriteUasciiUstream

addInstVarName: #gXstream
!

!GXUwriteUasciiUstream methodsFor: 'initializing'!

initVars
	gXstream _ 1 negated
!!

"BODY
 ----"

!GXUwriteUasciiUstream methodsFor: 'running'!

evaluate
GXcallout run line: 0 proc: self method: false; gXfacility:'extFortran'; gXfunction:27; condEvaluate.
GXgUerrorUhandler run line: 0 proc: self method: false; condEvaluate.
self up.
^nil
!!
nil! "(fix for smalltalk bug, this line is ignored if there is a variable declaration error)"

"End of procedure"

GXcomment run line: 0 proc: self method: false; p1:'_write_ascii_stream'; p2:'Write an ASCII representation of genie data into a file'; condEvaluate!
"Procedure class definition:
 ==========================="
((Smalltalk at: #GXUreadUasciiUstream ifAbsent: [ nil ]) isKindOf: Function class) ifTrue: [ ('Warning: Overriding previously defined procedure ', 'GXUreadUasciiUstream' gtype) prinl ].
Smalltalk at: #GXUreadUasciiUstream put: nil!
Function subclass: 'GXUreadUasciiUstream'
       instanceVariableNames: ''
       classVariableNames: ''
       poolDictionaries: ''
       category: 'Genie Procedure'
!

"HEADER
 ------"

"parameters
 ----------"

GXUreadUasciiUstream

addInstVarName: #gXstream;
compile:
'gXstream: aVar
	aVar notNil ifTrue:[gXstream _ self typeCheck: aVar shouldBe: GXInteger paramName: ''gXstream'']';
compile:
'p1: aVar
	aVar notNil ifTrue:[gXstream _ self typeCheck: aVar shouldBe: GXInteger paramName: ''gXstream'']'!

"result
 ------"

GXUreadUasciiUstream

addInstVarName: #gXres
!

"Local variables
 ---------------"

GXUreadUasciiUstream

addInstVarName: #gXcomment;

addInstVarName: #gXstream
!

!GXUreadUasciiUstream methodsFor: 'initializing'!

initVars
	gXcomment _ '#'.
	gXstream _ 1 negated
!!

"BODY
 ----"

!GXUreadUasciiUstream methodsFor: 'running'!

evaluate
GXcallout run line: 0 proc: self method: false; gXfacility:'extFortran'; gXfunction:25; condEvaluate.
(GXgUerrorUhandler run line: 0 proc: self method: true; condEvaluate) ifTrue: [ 
gXres _ (gXU) deepCopy ].
self up.
^gXres
!!
nil! "(fix for smalltalk bug, this line is ignored if there is a variable declaration error)"

"End of procedure"

GXcomment run line: 0 proc: self method: false; p1:'_read_ascii_stream'; p2:'Read an ASCII representation of genie data into a variable'; condEvaluate!
"Procedure class definition:
 ==========================="
((Smalltalk at: #GXUwriteUasciiUcolumns ifAbsent: [ nil ]) isKindOf: Function class) ifTrue: [ ('Warning: Overriding previously defined procedure ', 'GXUwriteUasciiUcolumns' gtype) prinl ].
Smalltalk at: #GXUwriteUasciiUcolumns put: nil!
Function subclass: 'GXUwriteUasciiUcolumns'
       instanceVariableNames: ''
       classVariableNames: ''
       poolDictionaries: ''
       category: 'Genie Procedure'
!

"HEADER
 ------"

"parameters
 ----------"

GXUwriteUasciiUcolumns

addInstVarName: #gXmode;
compile:
'gXmode: aVar
	aVar notNil ifTrue:[gXmode _ self typeCheck: aVar shouldBe: GXInteger paramName: ''gXmode'']';
compile:
'p1: aVar
	aVar notNil ifTrue:[gXmode _ self typeCheck: aVar shouldBe: GXInteger paramName: ''gXmode'']';

addInstVarName: #gXfile;
compile:
'gXfile: aVar
	aVar notNil ifTrue:[gXfile _ self typeCheck: aVar shouldBe: GXString paramName: ''gXfile'']';
compile:
'p2: aVar
	aVar notNil ifTrue:[gXfile _ self typeCheck: aVar shouldBe: GXString paramName: ''gXfile'']';

addInstVarName: #gXformat;
compile:
'gXformat: aVar
	aVar notNil ifTrue:[gXformat _ self typeCheck: aVar shouldBe: GXString paramName: ''gXformat'']';
compile:
'p3: aVar
	aVar notNil ifTrue:[gXformat _ self typeCheck: aVar shouldBe: GXString paramName: ''gXformat'']';

addInstVarName: #gXcomment;
compile:
'gXcomment: aVar
	aVar notNil ifTrue:[gXcomment _ self typeCheck: aVar shouldBe: GXString paramName: ''gXcomment'']';
compile:
'p4: aVar
	aVar notNil ifTrue:[gXcomment _ self typeCheck: aVar shouldBe: GXString paramName: ''gXcomment'']';

addInstVarName: #gXgv1;
compile:
'gXgv1: aVar
	aVar notNil ifTrue:[gXgv1 _ self typeCheck: aVar shouldBe: GXArray paramName: ''gXgv1'']';
compile:
'p5: aVar
	aVar notNil ifTrue:[gXgv1 _ self typeCheck: aVar shouldBe: GXArray paramName: ''gXgv1'']';

addInstVarName: #gXgv2;
compile:
'gXgv2: aVar
	aVar notNil ifTrue:[gXgv2 _ self typeCheck: aVar shouldBe: GXArray paramName: ''gXgv2'']';
compile:
'p6: aVar
	aVar notNil ifTrue:[gXgv2 _ self typeCheck: aVar shouldBe: GXArray paramName: ''gXgv2'']';

addInstVarName: #gXgv3;
compile:
'gXgv3: aVar
	aVar notNil ifTrue:[gXgv3 _ self typeCheck: aVar shouldBe: GXArray paramName: ''gXgv3'']';
compile:
'p7: aVar
	aVar notNil ifTrue:[gXgv3 _ self typeCheck: aVar shouldBe: GXArray paramName: ''gXgv3'']';

addInstVarName: #gXgv4;
compile:
'gXgv4: aVar
	aVar notNil ifTrue:[gXgv4 _ self typeCheck: aVar shouldBe: GXArray paramName: ''gXgv4'']';
compile:
'p8: aVar
	aVar notNil ifTrue:[gXgv4 _ self typeCheck: aVar shouldBe: GXArray paramName: ''gXgv4'']';

addInstVarName: #gXgv5;
compile:
'gXgv5: aVar
	aVar notNil ifTrue:[gXgv5 _ self typeCheck: aVar shouldBe: GXArray paramName: ''gXgv5'']';
compile:
'p9: aVar
	aVar notNil ifTrue:[gXgv5 _ self typeCheck: aVar shouldBe: GXArray paramName: ''gXgv5'']';

addInstVarName: #gXgv6;
compile:
'gXgv6: aVar
	aVar notNil ifTrue:[gXgv6 _ self typeCheck: aVar shouldBe: GXArray paramName: ''gXgv6'']';
compile:
'p10: aVar
	aVar notNil ifTrue:[gXgv6 _ self typeCheck: aVar shouldBe: GXArray paramName: ''gXgv6'']';

addInstVarName: #gXgv7;
compile:
'gXgv7: aVar
	aVar notNil ifTrue:[gXgv7 _ self typeCheck: aVar shouldBe: GXArray paramName: ''gXgv7'']';
compile:
'p11: aVar
	aVar notNil ifTrue:[gXgv7 _ self typeCheck: aVar shouldBe: GXArray paramName: ''gXgv7'']';

addInstVarName: #gXgv8;
compile:
'gXgv8: aVar
	aVar notNil ifTrue:[gXgv8 _ self typeCheck: aVar shouldBe: GXArray paramName: ''gXgv8'']';
compile:
'p12: aVar
	aVar notNil ifTrue:[gXgv8 _ self typeCheck: aVar shouldBe: GXArray paramName: ''gXgv8'']';

addInstVarName: #gXgv9;
compile:
'gXgv9: aVar
	aVar notNil ifTrue:[gXgv9 _ self typeCheck: aVar shouldBe: GXArray paramName: ''gXgv9'']';
compile:
'p13: aVar
	aVar notNil ifTrue:[gXgv9 _ self typeCheck: aVar shouldBe: GXArray paramName: ''gXgv9'']'
!

"Local variables
 ---------------"

GXUwriteUasciiUcolumns

addInstVarName: #gXgv;

addInstVarName: #gXfile
!

!GXUwriteUasciiUcolumns methodsFor: 'initializing'!

initVars
	gXfile _ (gXUcurrentUoutputUsource)
!!

"BODY
 ----"

!GXUwriteUasciiUcolumns methodsFor: 'running'!

evaluate

    gXgv _ Array new: 9.
    gXfile isNil ifTrue: [^'_WRITE_ASCII_COLUMNS: Parameter FILE not specified and no default available' prenl ].
    gXgv1 isNil ifFalse: [ gXgv at: 1 put: gXgv1 ].
    gXgv2 isNil ifFalse: [ gXgv at: 2 put: gXgv2 ].
    gXgv3 isNil ifFalse: [ gXgv at: 3 put: gXgv3 ].
    gXgv4 isNil ifFalse: [ gXgv at: 4 put: gXgv4 ].
    gXgv5 isNil ifFalse: [ gXgv at: 5 put: gXgv5 ].
    gXgv6 isNil ifFalse: [ gXgv at: 6 put: gXgv6 ].
    gXgv7 isNil ifFalse: [ gXgv at: 7 put: gXgv7 ].
    gXgv8 isNil ifFalse: [ gXgv at: 8 put: gXgv8 ].
    gXgv9 isNil ifFalse: [ gXgv at: 9 put: gXgv9 ]
.
GXcallout run line: 0 proc: self method: false; gXfacility:'extFortran'; gXfunction:26; condEvaluate.
GXgUerrorUhandler run line: 0 proc: self method: false; condEvaluate.
self up.
^nil
!!
nil! "(fix for smalltalk bug, this line is ignored if there is a variable declaration error)"

"End of procedure"

GXcomment run line: 0 proc: self method: false; p1:'_write_ascii_columns'; p2:'Write columns of ASCII data from arrays into a file'; condEvaluate!
"Procedure class definition:
 ==========================="
((Smalltalk at: #GXUreadUasciiUcolumns ifAbsent: [ nil ]) isKindOf: Function class) ifTrue: [ ('Warning: Overriding previously defined procedure ', 'GXUreadUasciiUcolumns' gtype) prinl ].
Smalltalk at: #GXUreadUasciiUcolumns put: nil!
Function subclass: 'GXUreadUasciiUcolumns'
       instanceVariableNames: ''
       classVariableNames: ''
       poolDictionaries: ''
       category: 'Genie Procedure'
!

"HEADER
 ------"

"parameters
 ----------"

GXUreadUasciiUcolumns

addInstVarName: #gXfieldUnames;
compile:
'gXfieldUnames: aVar
	aVar notNil ifTrue:[gXfieldUnames _ self typeCheck: aVar shouldBe: GXString paramName: ''gXfieldUnames'']';
compile:
'p1: aVar
	aVar notNil ifTrue:[gXfieldUnames _ self typeCheck: aVar shouldBe: GXString paramName: ''gXfieldUnames'']';

addInstVarName: #gXfile;
compile:
'gXfile: aVar
	aVar notNil ifTrue:[gXfile _ self typeCheck: aVar shouldBe: GXString paramName: ''gXfile'']';
compile:
'p2: aVar
	aVar notNil ifTrue:[gXfile _ self typeCheck: aVar shouldBe: GXString paramName: ''gXfile'']'!

"result
 ------"

GXUreadUasciiUcolumns

addInstVarName: #gXres
!

"Local variables
 ---------------"

GXUreadUasciiUcolumns

addInstVarName: #gXfile!

"Local variables
 ---------------"

GXUreadUasciiUcolumns

addInstVarName: #gXres;

addInstVarName: #gXdelim;

addInstVarName: #gXcomment;

addInstVarName: #gXfile!

"Local variables
 ---------------"

GXUreadUasciiUcolumns

addInstVarName: #gXfieldUnames
!

!GXUreadUasciiUcolumns methodsFor: 'initializing'!

initVars
	gXfile _ (gXUcurrentUinputUsource).
	gXres _ (GXfields run line: 0 proc: self method: true; p1:10; condEvaluate).
	gXdelim _ '
'.
	gXcomment _ '#'.
	gXfile _ (gXUcurrentUinputUsource).
	gXfieldUnames _ 'FIELD01,FIELD02,FIELD03,FIELD04,FIELD05,FIELD06'
!!

"BODY
 ----"

!GXUreadUasciiUcolumns methodsFor: 'running'!

evaluate
 gXfile isNil ifTrue: [^'_READ_ASCII_COLUMNS: Parameter FILE not specified' prenl ] .
GXcallout run line: 0 proc: self method: false; gXfacility:'extFortran'; gXfunction:24; condEvaluate.
(GXgUerrorUhandler run line: 0 proc: self method: true; condEvaluate) ifTrue: [ 
gXres _ (gXU) deepCopy ].
self up.
^gXres
!!
nil! "(fix for smalltalk bug, this line is ignored if there is a variable declaration error)"

"End of procedure"

GXcomment run line: 0 proc: self method: false; p1:'_read_ascii_columns'; p2:'Read columns of ASCII data into arrays'; condEvaluate!
"Procedure class definition:
 ==========================="
((Smalltalk at: #GXwrite ifAbsent: [ nil ]) isKindOf: Function class) ifTrue: [ ('Warning: Overriding previously defined procedure ', 'GXwrite' gtype) prinl ].
Smalltalk at: #GXwrite put: nil!
Function subclass: 'GXwrite'
       instanceVariableNames: ''
       classVariableNames: ''
       poolDictionaries: ''
       category: 'Genie Procedure'
!

"HEADER
 ------"

"qualifiers
 ----------"

GXwrite

addInstVarName: #gXascii;
compile:
'gXascii
	gXascii _ true';
compile:
'gXascii: aBoolean
	gXascii _ aBoolean';

addInstVarName: #gXcolumns;
compile:
'gXcolumns
	gXcolumns _ true';
compile:
'gXcolumns: aBoolean
	gXcolumns _ aBoolean';

addInstVarName: #gXstream;
compile:
'gXstream
	gXstream _ true';
compile:
'gXstream: aBoolean
	gXstream _ aBoolean';

addInstVarName: #gXappend;
compile:
'gXappend
	gXappend _ true';
compile:
'gXappend: aBoolean
	gXappend _ aBoolean'!

"parameters
 ----------"

GXwrite

addInstVarName: #gXp1;
compile:
'gXp1: aVar
	aVar notNil ifTrue:[ gXp1 _ aVar ]';
compile:
'p1: aVar
	aVar notNil ifTrue:[ gXp1 _ aVar ]';

addInstVarName: #gXp2;
compile:
'gXp2: aVar
	aVar notNil ifTrue:[ gXp2 _ aVar ]';
compile:
'p2: aVar
	aVar notNil ifTrue:[ gXp2 _ aVar ]';

addInstVarName: #gXp3;
compile:
'gXp3: aVar
	aVar notNil ifTrue:[ gXp3 _ aVar ]';
compile:
'p3: aVar
	aVar notNil ifTrue:[ gXp3 _ aVar ]';

addInstVarName: #gXp4;
compile:
'gXp4: aVar
	aVar notNil ifTrue:[ gXp4 _ aVar ]';
compile:
'p4: aVar
	aVar notNil ifTrue:[ gXp4 _ aVar ]';

addInstVarName: #gXp5;
compile:
'gXp5: aVar
	aVar notNil ifTrue:[ gXp5 _ aVar ]';
compile:
'p5: aVar
	aVar notNil ifTrue:[ gXp5 _ aVar ]';

addInstVarName: #gXp6;
compile:
'gXp6: aVar
	aVar notNil ifTrue:[ gXp6 _ aVar ]';
compile:
'p6: aVar
	aVar notNil ifTrue:[ gXp6 _ aVar ]';

addInstVarName: #gXp7;
compile:
'gXp7: aVar
	aVar notNil ifTrue:[ gXp7 _ aVar ]';
compile:
'p7: aVar
	aVar notNil ifTrue:[ gXp7 _ aVar ]';

addInstVarName: #gXp8;
compile:
'gXp8: aVar
	aVar notNil ifTrue:[ gXp8 _ aVar ]';
compile:
'p8: aVar
	aVar notNil ifTrue:[ gXp8 _ aVar ]';

addInstVarName: #gXp9;
compile:
'gXp9: aVar
	aVar notNil ifTrue:[ gXp9 _ aVar ]';
compile:
'p9: aVar
	aVar notNil ifTrue:[ gXp9 _ aVar ]';

addInstVarName: #gXp10;
compile:
'gXp10: aVar
	aVar notNil ifTrue:[ gXp10 _ aVar ]';
compile:
'p10: aVar
	aVar notNil ifTrue:[ gXp10 _ aVar ]';

addInstVarName: #gXp11;
compile:
'gXp11: aVar
	aVar notNil ifTrue:[ gXp11 _ aVar ]';
compile:
'p11: aVar
	aVar notNil ifTrue:[ gXp11 _ aVar ]';

addInstVarName: #gXp12;
compile:
'gXp12: aVar
	aVar notNil ifTrue:[ gXp12 _ aVar ]';
compile:
'p12: aVar
	aVar notNil ifTrue:[ gXp12 _ aVar ]'!

"parameters
 ----------"

GXwrite

addInstVarName: #gXfile;
compile:
'gXfile: aVar
	aVar notNil ifTrue:[gXfile _ self typeCheck: aVar shouldBe: GXString paramName: ''gXfile'']';
compile:
'p13: aVar
	aVar notNil ifTrue:[gXfile _ self typeCheck: aVar shouldBe: GXString paramName: ''gXfile'']';

addInstVarName: #gXformat;
compile:
'gXformat: aVar
	aVar notNil ifTrue:[gXformat _ self typeCheck: aVar shouldBe: GXString paramName: ''gXformat'']';
compile:
'p14: aVar
	aVar notNil ifTrue:[gXformat _ self typeCheck: aVar shouldBe: GXString paramName: ''gXformat'']';

addInstVarName: #gXcomment;
compile:
'gXcomment: aVar
	aVar notNil ifTrue:[gXcomment _ self typeCheck: aVar shouldBe: GXString paramName: ''gXcomment'']';
compile:
'p15: aVar
	aVar notNil ifTrue:[gXcomment _ self typeCheck: aVar shouldBe: GXString paramName: ''gXcomment'']';

addInstVarName: #gXstreamUid;
compile:
'gXstreamUid: aVar
	aVar notNil ifTrue:[gXstreamUid _ self typeCheck: aVar shouldBe: GXInteger paramName: ''gXstreamUid'']';
compile:
'p16: aVar
	aVar notNil ifTrue:[gXstreamUid _ self typeCheck: aVar shouldBe: GXInteger paramName: ''gXstreamUid'']'!

"parameters
 ----------"

GXwrite

addInstVarName: #gXgv1;
compile:
'gXgv1: aVar
	aVar notNil ifTrue:[ gXgv1 _ aVar ]';
compile:
'p17: aVar
	aVar notNil ifTrue:[ gXgv1 _ aVar ]';

addInstVarName: #gXgv2;
compile:
'gXgv2: aVar
	aVar notNil ifTrue:[ gXgv2 _ aVar ]';
compile:
'p18: aVar
	aVar notNil ifTrue:[ gXgv2 _ aVar ]';

addInstVarName: #gXgv3;
compile:
'gXgv3: aVar
	aVar notNil ifTrue:[ gXgv3 _ aVar ]';
compile:
'p19: aVar
	aVar notNil ifTrue:[ gXgv3 _ aVar ]';

addInstVarName: #gXgv4;
compile:
'gXgv4: aVar
	aVar notNil ifTrue:[ gXgv4 _ aVar ]';
compile:
'p20: aVar
	aVar notNil ifTrue:[ gXgv4 _ aVar ]';

addInstVarName: #gXgv5;
compile:
'gXgv5: aVar
	aVar notNil ifTrue:[ gXgv5 _ aVar ]';
compile:
'p21: aVar
	aVar notNil ifTrue:[ gXgv5 _ aVar ]';

addInstVarName: #gXgv6;
compile:
'gXgv6: aVar
	aVar notNil ifTrue:[ gXgv6 _ aVar ]';
compile:
'p22: aVar
	aVar notNil ifTrue:[ gXgv6 _ aVar ]';

addInstVarName: #gXgv7;
compile:
'gXgv7: aVar
	aVar notNil ifTrue:[ gXgv7 _ aVar ]';
compile:
'p23: aVar
	aVar notNil ifTrue:[ gXgv7 _ aVar ]';

addInstVarName: #gXgv8;
compile:
'gXgv8: aVar
	aVar notNil ifTrue:[ gXgv8 _ aVar ]';
compile:
'p24: aVar
	aVar notNil ifTrue:[ gXgv8 _ aVar ]';

addInstVarName: #gXgv9;
compile:
'gXgv9: aVar
	aVar notNil ifTrue:[ gXgv9 _ aVar ]';
compile:
'p25: aVar
	aVar notNil ifTrue:[ gXgv9 _ aVar ]'
!

"Local variables
 ---------------"

GXwrite

addInstVarName: #gXtemp;

addInstVarName: #gXmode!

"Local variables
 ---------------"

GXwrite

addInstVarName: #gXfile
!

!GXwrite methodsFor: 'initializing'!

initVars
	gXascii _ false.
	gXcolumns _ false.
	gXstream _ false.
	gXappend _ false.
	gXmode _ 0.
	gXfile _ (gXUcurrentUoutputUsource)
!!

"BODY
 ----"

!GXwrite methodsFor: 'running'!

evaluate
(gXcolumns) ifTrue: [ 

    ( gXp1 isKindOf: String ) ifTrue: [ gXfile _ gXp1 ].
    ( gXp2 isKindOf: String ) ifTrue: [ gXformat _ gXp2 ].
    ( gXp3 isKindOf: String ) ifTrue: [ gXcomment _ gXp3 ].
    gXp4 notNil ifTrue: [ gXgv1 _ gXp4 ].
    gXp5 notNil ifTrue: [ gXgv2 _ gXp5 ].
    gXp6 notNil ifTrue: [ gXgv3 _ gXp6 ].
    gXp7 notNil ifTrue: [ gXgv4 _ gXp7 ].
    gXp8 notNil ifTrue: [ gXgv5 _ gXp8 ].
    gXp9 notNil ifTrue: [ gXgv6 _ gXp9 ].
    gXp10 notNil ifTrue: [ gXgv7 _ gXp10 ].
    gXp11 notNil ifTrue: [ gXgv8 _ gXp11 ].
    gXp12 notNil ifTrue: [ gXgv9 _ gXp12 ]
.
GXUwriteUasciiUcolumns run line: 0 proc: self method: false; p1:(gXmode); p2:(gXfile); p3:(gXformat); p4:(gXcomment); p5:(gXgv1); p6:(gXgv2); p7:(gXgv3); p8:(gXgv4); p9:(gXgv5); p10:(gXgv6); p11:(gXgv7); p12:(gXgv8); p13:(gXgv9); condEvaluate ] ifFalse: [ (gXstream) ifTrue: [ 

    ( gXp1 isKindOf: Integer ) ifTrue: [ gXstreamUid _ gXp1 ].
    ( gXp2 isKindOf: String ) ifTrue: [ gXcomment _ gXp2 ].
    gXp3 notNil ifTrue: [ gXgv1 _ gXp3 ]
.
(GXdefined run line: 0 proc: self method: true; p1:(gXstreamUid); condEvaluate) ifTrue: [ 
GXUwriteUasciiUstream run line: 0 proc: self method: false; p1:(gXmode); p2:(gXstreamUid); p3:(gXcomment); p4:(gXgv1); condEvaluate ] ifFalse: [ 
(gXappend) ifTrue: [ 
gXtemp _ (GXUopen run line: 0 proc: self method: true; gXstream; gXappend; p1:(gXfile); condEvaluate) deepCopy ] ifFalse: [ 
gXtemp _ (GXUopen run line: 0 proc: self method: true; gXstream; gXnew; p1:(gXfile); condEvaluate) deepCopy ] .
GXUwriteUasciiUstream run line: 0 proc: self method: false; p1:(gXmode); p2:(gXtemp); p3:(gXcomment); p4:(gXgv1); condEvaluate.
GXUclose run line: 0 proc: self method: false; gXstream; p1:(gXtemp); condEvaluate ]  ] ifFalse: [ 
GXprinten run line: 0 proc: self method: false; p1:'WRITE: Invalid or non-existent qualifier'; condEvaluate ] ].
self up.
^nil
!!
nil! "(fix for smalltalk bug, this line is ignored if there is a variable declaration error)"

"End of procedure"

GXcomment run line: 0 proc: self method: false; p1:'write'; p2:'Write an ASCII genie variable into a file'; condEvaluate!
"Procedure class definition:
 ==========================="
((Smalltalk at: #GXread ifAbsent: [ nil ]) isKindOf: Function class) ifTrue: [ ('Warning: Overriding previously defined procedure ', 'GXread' gtype) prinl ].
Smalltalk at: #GXread put: nil!
Function subclass: 'GXread'
       instanceVariableNames: ''
       classVariableNames: ''
       poolDictionaries: ''
       category: 'Genie Procedure'
!

"HEADER
 ------"

"qualifiers
 ----------"

GXread

addInstVarName: #gXascii;
compile:
'gXascii
	gXascii _ true';
compile:
'gXascii: aBoolean
	gXascii _ aBoolean';

addInstVarName: #gXcolumns;
compile:
'gXcolumns
	gXcolumns _ true';
compile:
'gXcolumns: aBoolean
	gXcolumns _ aBoolean';

addInstVarName: #gXstream;
compile:
'gXstream
	gXstream _ true';
compile:
'gXstream: aBoolean
	gXstream _ aBoolean'!

"parameters
 ----------"

GXread

addInstVarName: #gXp1;
compile:
'gXp1: aVar
	aVar notNil ifTrue:[ gXp1 _ aVar ]';
compile:
'p1: aVar
	aVar notNil ifTrue:[ gXp1 _ aVar ]';

addInstVarName: #gXp2;
compile:
'gXp2: aVar
	aVar notNil ifTrue:[ gXp2 _ aVar ]';
compile:
'p2: aVar
	aVar notNil ifTrue:[ gXp2 _ aVar ]';

addInstVarName: #gXfile;
compile:
'gXfile: aVar
	aVar notNil ifTrue:[gXfile _ self typeCheck: aVar shouldBe: GXString paramName: ''gXfile'']';
compile:
'p3: aVar
	aVar notNil ifTrue:[gXfile _ self typeCheck: aVar shouldBe: GXString paramName: ''gXfile'']';

addInstVarName: #gXstreamUid;
compile:
'gXstreamUid: aVar
	aVar notNil ifTrue:[gXstreamUid _ self typeCheck: aVar shouldBe: GXInteger paramName: ''gXstreamUid'']';
compile:
'p4: aVar
	aVar notNil ifTrue:[gXstreamUid _ self typeCheck: aVar shouldBe: GXInteger paramName: ''gXstreamUid'']';

addInstVarName: #gXfields;
compile:
'gXfields: aVar
	aVar notNil ifTrue:[gXfields _ self typeCheck: aVar shouldBe: GXString paramName: ''gXfields'']';
compile:
'p5: aVar
	aVar notNil ifTrue:[gXfields _ self typeCheck: aVar shouldBe: GXString paramName: ''gXfields'']'!

"result
 ------"

GXread

addInstVarName: #gXres
!

"Local variables
 ---------------"

GXread

addInstVarName: #gXtemp;

addInstVarName: #gXfile
!

!GXread methodsFor: 'initializing'!

initVars
	gXascii _ false.
	gXcolumns _ false.
	gXstream _ false.
	gXfile _ (gXUcurrentUinputUsource)
!!

"BODY
 ----"

!GXread methodsFor: 'running'!

evaluate

    ( gXp1 isKindOf: String ) ifTrue: [ gXfile _ gXp1 ].
    ( gXp1 isKindOf: Integer ) ifTrue: [ gXstreamUid _ gXp1 ].
    ( gXp2 isKindOf: String ) ifTrue: [ gXfields _ gXp2 ].
    ( gXfile notNil | gXstreamUid notNil ) ifFalse: [^'READ: Neither FILE nor STREAM_ID specified' prenl].
    gXcolumns ifTrue: [ gXres _ GXUreadUasciiUcolumns run line: 0 proc: self method: true; p1: gXfields; p2: gXfile; condEvaluate ].
    gXstream ifTrue: 
    [
	gXstreamUid notNil ifTrue: [ gXres _ GXUreadUasciiUstream run line: 0 proc: self method: true; p1: gXstreamUid; condEvaluate ].
	gXfile notNil ifTrue:
	[
	    gXtemp _ GXUopen run line: 0 proc: self method: true; gXstream; gXread; p1: gXfile; condEvaluate.
	    gXres _ GXUreadUasciiUstream run line: 0 proc: self method: true; p1: gXtemp; condEvaluate.
	    GXUclose run line: 0 proc: self method: false; gXstream; p1: gXtemp; condEvaluate
	]
    ]
.
self up.
^gXres
!!
nil! "(fix for smalltalk bug, this line is ignored if there is a variable declaration error)"

"End of procedure"

GXcomment run line: 0 proc: self method: false; p1:'read'; p2:'Read user data from a file into genie variables'; condEvaluate!
"Procedure class definition:
 ==========================="
((Smalltalk at: #GXUfocusUrange ifAbsent: [ nil ]) isKindOf: Function class) ifTrue: [ ('Warning: Overriding previously defined procedure ', 'GXUfocusUrange' gtype) prinl ].
Smalltalk at: #GXUfocusUrange put: nil!
Function subclass: 'GXUfocusUrange'
       instanceVariableNames: ''
       classVariableNames: ''
       poolDictionaries: ''
       category: 'Genie Procedure'
!

"HEADER
 ------"

"parameters
 ----------"

GXUfocusUrange

addInstVarName: #gXiverbose;
compile:
'gXiverbose: aVar
	aVar notNil ifTrue:[gXiverbose _ self typeCheck: aVar shouldBe: GXInteger paramName: ''gXiverbose'']';
compile:
'p1: aVar
	aVar notNil ifTrue:[gXiverbose _ self typeCheck: aVar shouldBe: GXInteger paramName: ''gXiverbose'']';

addInstVarName: #gXspecpars;
compile:
'gXspecpars: aVar
	aVar notNil ifTrue:[gXspecpars _ self typeCheck: aVar shouldBe: GXWorkspace paramName: ''gXspecpars'']';
compile:
'p2: aVar
	aVar notNil ifTrue:[gXspecpars _ self typeCheck: aVar shouldBe: GXWorkspace paramName: ''gXspecpars'']';

addInstVarName: #gXdetpars;
compile:
'gXdetpars: aVar
	aVar notNil ifTrue:[gXdetpars _ self typeCheck: aVar shouldBe: GXWorkspace paramName: ''gXdetpars'']';
compile:
'p3: aVar
	aVar notNil ifTrue:[gXdetpars _ self typeCheck: aVar shouldBe: GXWorkspace paramName: ''gXdetpars'']';

addInstVarName: #gXfile;
compile:
'gXfile: aVar
	aVar notNil ifTrue:[gXfile _ self typeCheck: aVar shouldBe: GXString paramName: ''gXfile'']';
compile:
'p4: aVar
	aVar notNil ifTrue:[gXfile _ self typeCheck: aVar shouldBe: GXString paramName: ''gXfile'']';

addInstVarName: #gXiunit;
compile:
'gXiunit: aVar
	aVar notNil ifTrue:[gXiunit _ self typeCheck: aVar shouldBe: GXInteger paramName: ''gXiunit'']';
compile:
'p5: aVar
	aVar notNil ifTrue:[gXiunit _ self typeCheck: aVar shouldBe: GXInteger paramName: ''gXiunit'']';

addInstVarName: #gXiarray;
compile:
'gXiarray: aVar
	aVar notNil ifTrue:[gXiarray _ self typeCheck: aVar shouldBe: GXIntegerarray paramName: ''gXiarray'']';
compile:
'p6: aVar
	aVar notNil ifTrue:[gXiarray _ self typeCheck: aVar shouldBe: GXIntegerarray paramName: ''gXiarray'']'!

"result
 ------"

GXUfocusUrange

addInstVarName: #gXwksp
!

"Local variables
 ---------------"

GXUfocusUrange

addInstVarName: #gXtmp
!

!GXUfocusUrange methodsFor: 'initializing'!

initVars
	gXwksp _ (GXfields run line: 0 proc: self method: true; condEvaluate)
!!

"BODY
 ----"

!GXUfocusUrange methodsFor: 'running'!

evaluate

    gXiarray isNil ifTrue: [^'_FOCUS_RANGE: Parameter IARRAY not specified' printNl ]
.

    gXspecpars notNil ifTrue: [
	gXtmp _ (gXspecpars at: #gXtwotheta) size.
	((gXspecpars at: #gXl1) isKindOf: GXRealarray) ifFalse: [
	    gXspecpars at: #gXl1 put: ((GXRealarray new: gXtmp) extend: (gXspecpars at: #gXl1))
	].
	((gXspecpars at: #gXefixed) isKindOf: GXRealarray) ifFalse: [
	    gXspecpars at: #gXefixed put: ((GXRealarray new: gXtmp) extend: (gXspecpars at: #gXefixed))
	].
	((gXspecpars at: #gXemode) isKindOf: GXIntegerarray) ifFalse: [
	    gXspecpars at: #gXemode put: ((GXIntegerarray new: gXtmp) extend: (gXspecpars at: #gXemode))
	]
    ]
.
GXcallout run line: 0 proc: self method: false; gXfacility:'extFortran'; gXfunction:22; condEvaluate.
(GXgUerrorUhandler run line: 0 proc: self method: true; condEvaluate) ifTrue: [ 
gXwksp _ (gXU) deepCopy ].
self up.
^gXwksp
!!
nil! "(fix for smalltalk bug, this line is ignored if there is a variable declaration error)"

"End of procedure"

GXcomment run line: 0 proc: self method: false; p1:'_focus_range'; p2:'focus a numerical range of spectra'; condEvaluate!
"Procedure class definition:
 ==========================="
((Smalltalk at: #GXfocus ifAbsent: [ nil ]) isKindOf: Function class) ifTrue: [ ('Warning: Overriding previously defined procedure ', 'GXfocus' gtype) prinl ].
Smalltalk at: #GXfocus put: nil!
Function subclass: 'GXfocus'
       instanceVariableNames: ''
       classVariableNames: ''
       poolDictionaries: ''
       category: 'Genie Procedure'
!

"HEADER
 ------"

"qualifiers
 ----------"

GXfocus

addInstVarName: #gXd;
compile:
'gXd
	gXd _ true';
compile:
'gXd: aBoolean
	gXd _ aBoolean';

addInstVarName: #gXt;
compile:
'gXt
	gXt _ true';
compile:
'gXt: aBoolean
	gXt _ aBoolean';

addInstVarName: #gXq;
compile:
'gXq
	gXq _ true';
compile:
'gXq: aBoolean
	gXq _ aBoolean';

addInstVarName: #gXverbose;
compile:
'gXverbose
	gXverbose _ true';
compile:
'gXverbose: aBoolean
	gXverbose _ aBoolean'!

"parameters
 ----------"

GXfocus

addInstVarName: #gXp1;
compile:
'gXp1: aVar
	aVar notNil ifTrue:[ gXp1 _ aVar ]';
compile:
'p1: aVar
	aVar notNil ifTrue:[ gXp1 _ aVar ]';

addInstVarName: #gXp2;
compile:
'gXp2: aVar
	aVar notNil ifTrue:[ gXp2 _ aVar ]';
compile:
'p2: aVar
	aVar notNil ifTrue:[ gXp2 _ aVar ]';

addInstVarName: #gXp3;
compile:
'gXp3: aVar
	aVar notNil ifTrue:[ gXp3 _ aVar ]';
compile:
'p3: aVar
	aVar notNil ifTrue:[ gXp3 _ aVar ]';

addInstVarName: #gXp4;
compile:
'gXp4: aVar
	aVar notNil ifTrue:[ gXp4 _ aVar ]';
compile:
'p4: aVar
	aVar notNil ifTrue:[ gXp4 _ aVar ]';

addInstVarName: #gXspectra;
compile:
'gXspectra: aVar
	aVar notNil ifTrue:[ gXspectra _ aVar ]';
compile:
'p5: aVar
	aVar notNil ifTrue:[ gXspectra _ aVar ]';

addInstVarName: #gXfile;
compile:
'gXfile: aVar
	aVar notNil ifTrue:[gXfile _ self typeCheck: aVar shouldBe: GXString paramName: ''gXfile'']';
compile:
'p6: aVar
	aVar notNil ifTrue:[gXfile _ self typeCheck: aVar shouldBe: GXString paramName: ''gXfile'']';

addInstVarName: #gXdetpars;
compile:
'gXdetpars: aVar
	aVar notNil ifTrue:[gXdetpars _ self typeCheck: aVar shouldBe: GXWorkspace paramName: ''gXdetpars'']';
compile:
'p7: aVar
	aVar notNil ifTrue:[gXdetpars _ self typeCheck: aVar shouldBe: GXWorkspace paramName: ''gXdetpars'']';

addInstVarName: #gXspecpars;
compile:
'gXspecpars: aVar
	aVar notNil ifTrue:[gXspecpars _ self typeCheck: aVar shouldBe: GXWorkspace paramName: ''gXspecpars'']';
compile:
'p8: aVar
	aVar notNil ifTrue:[gXspecpars _ self typeCheck: aVar shouldBe: GXWorkspace paramName: ''gXspecpars'']'!

"result
 ------"

GXfocus

addInstVarName: #gXwksp
!

"Local variables
 ---------------"

GXfocus

addInstVarName: #gXfile;

addInstVarName: #gXiunit;

addInstVarName: #gXunit;

addInstVarName: #gXiverbose;

addInstVarName: #gXiarray
!

!GXfocus methodsFor: 'initializing'!

initVars
	gXd _ false.
	gXt _ false.
	gXq _ false.
	gXverbose _ false.
	gXfile _ (gXUcurrentUinputUsource).
	gXiunit _ 4.
	gXunit _ 'd-spacing (A)'.
	gXiverbose _ 0
!!

"BODY
 ----"

!GXfocus methodsFor: 'running'!

evaluate

"Parameters"
    gXp1 notNil ifTrue: [ gXspectra _ gXp1 ].
    ( gXp2 isKindOf: String ) ifTrue: [ gXfile _ gXp2 ].
    ( gXp3 isKindOf: GXWorkspace ) ifTrue: [ gXdetpars _ gXp3 ].
    ( gXp4 isKindOf: GXWorkspace ) ifTrue: [ gXspecpars _ gXp4 ].
    gXfile isNil ifTrue: [^'FOCUS: Parameter FILE not specified and no default available' prenl].
    ( gXspectra isKindOf: GXIntegerarray ) ifTrue: [ gXiarray _ gXspectra ].
    ( gXspectra isKindOf: Interval ) ifTrue: [ gXiarray _ gXspectra asIntegerArray ].
    gXiarray isNil ifTrue: [^'FOCUS: Parameter SPECTRA not specified or is of incorrect type' prenl].
"qualifiers"
    gXverbose ifTrue: [ gXiverbose _ 1].
    gXd ifTrue: [ gXiunit _ 4. gXunit _ 'd-spacing (A)' ].
    gXt ifTrue: [ gXiunit _ 2. gXunit _ 'time of flight (us)' ].
    gXq ifTrue: [ gXiunit _ 6. gXunit _ 'momentum transfer (A**-1)' ]
.
GXprintin run line: 0 proc: self method: false; p1:'Focusing spectra from '; p2:(gXfile); p3:' using '; p4:(gXunit); condEvaluate.
gXwksp _ (GXUfocusUrange run line: 0 proc: self method: true; p1:(gXiverbose); p2:(gXspecpars); p3:(gXdetpars); p4:(gXfile); p5:(gXiunit); p6:(gXiarray); condEvaluate) .
self up.
^gXwksp
!!
nil! "(fix for smalltalk bug, this line is ignored if there is a variable declaration error)"

"End of procedure"

GXcomment run line: 0 proc: self method: false; p1:'focus'; p2:'Focus a range of spectra'; condEvaluate!
"Procedure class definition:
 ==========================="
((Smalltalk at: #GXsumspec ifAbsent: [ nil ]) isKindOf: Function class) ifTrue: [ ('Warning: Overriding previously defined procedure ', 'GXsumspec' gtype) prinl ].
Smalltalk at: #GXsumspec put: nil!
Function subclass: 'GXsumspec'
       instanceVariableNames: ''
       classVariableNames: ''
       poolDictionaries: ''
       category: 'Genie Procedure'
!

"HEADER
 ------"

"parameters
 ----------"

GXsumspec

addInstVarName: #gXp1;
compile:
'gXp1: aVar
	aVar notNil ifTrue:[ gXp1 _ aVar ]';
compile:
'p1: aVar
	aVar notNil ifTrue:[ gXp1 _ aVar ]';

addInstVarName: #gXp2;
compile:
'gXp2: aVar
	aVar notNil ifTrue:[ gXp2 _ aVar ]';
compile:
'p2: aVar
	aVar notNil ifTrue:[ gXp2 _ aVar ]';

addInstVarName: #gXspectra;
compile:
'gXspectra: aVar
	aVar notNil ifTrue:[ gXspectra _ aVar ]';
compile:
'p3: aVar
	aVar notNil ifTrue:[ gXspectra _ aVar ]';

addInstVarName: #gXfile;
compile:
'gXfile: aVar
	aVar notNil ifTrue:[gXfile _ self typeCheck: aVar shouldBe: GXString paramName: ''gXfile'']';
compile:
'p4: aVar
	aVar notNil ifTrue:[gXfile _ self typeCheck: aVar shouldBe: GXString paramName: ''gXfile'']'!

"result
 ------"

GXsumspec

addInstVarName: #gXres
!

"Local variables
 ---------------"

GXsumspec

addInstVarName: #gXstart;

addInstVarName: #gXtmpUwksp;

addInstVarName: #gXiarray;

addInstVarName: #gXfile
!

!GXsumspec methodsFor: 'initializing'!

initVars
	gXfile _ (gXUcurrentUinputUsource)
!!

"BODY
 ----"

!GXsumspec methodsFor: 'running'!

evaluate

    gXp1 isNil ifFalse: [ gXspectra _ gXp1 ].
    ( gXp2 isKindOf: String ) ifTrue: [ gXfile _ gXp2 ].
    gXfile isNil ifTrue: [^'SUMSPEC: Parameter FILE not specified, and no default available' prenl].
    ( gXspectra isKindOf: Interval ) ifTrue: [ gXiarray _ gXspectra asIntegerArray ].
    ( gXspectra isKindOf: GXIntegerarray ) ifTrue: [ gXiarray _ gXspectra ]. 
    gXiarray isNil ifTrue: [^'SUMSPEC: Parameter SPECTRA not specified or is of incorrect type' prenl].
    gXstart _ gXiarray at: 1
.
gXres _ (GXUgetUsnsUspectrum run line: 0 proc: self method: true; p1:0; p2:(gXstart); p3:(gXfile); condEvaluate) .
gXtmpUwksp _ (GXUfocusUrange run line: 0 proc: self method: true; p1:0; p2:(gXU); p3:(gXU); p4:(gXfile); p5:2; p6:(gXiarray); condEvaluate) .
(gXres isKindOf: GXArray) | (gXres isKindOf: GXWorkspace)
    ifTrue:  [ ((gXres) at:#gXy put: ((gXtmpUwksp) at:#gXy) ) ]
    ifFalse: [ self error: 'Assignment to array or workspace failed, check that the destination is a structured type'].
(gXres isKindOf: GXArray) | (gXres isKindOf: GXWorkspace)
    ifTrue:  [ ((gXres) at:#gXe put: ((gXtmpUwksp) at:#gXe) ) ]
    ifFalse: [ self error: 'Assignment to array or workspace failed, check that the destination is a structured type'].
self up.
^gXres
!!
nil! "(fix for smalltalk bug, this line is ignored if there is a variable declaration error)"

"End of procedure"

GXcomment run line: 0 proc: self method: false; p1:'sumspec'; p2:'Sum spectra in a given range from the specified file'; condEvaluate!
"Procedure class definition:
 ==========================="
((Smalltalk at: #GXnblocks ifAbsent: [ nil ]) isKindOf: Function class) ifTrue: [ ('Warning: Overriding previously defined procedure ', 'GXnblocks' gtype) prinl ].
Smalltalk at: #GXnblocks put: nil!
Function subclass: 'GXnblocks'
       instanceVariableNames: ''
       classVariableNames: ''
       poolDictionaries: ''
       category: 'Genie Procedure'
!

"HEADER
 ------"

"parameters
 ----------"

GXnblocks

addInstVarName: #gXfile;
compile:
'gXfile: aVar
	aVar notNil ifTrue:[gXfile _ self typeCheck: aVar shouldBe: GXString paramName: ''gXfile'']';
compile:
'p1: aVar
	aVar notNil ifTrue:[gXfile _ self typeCheck: aVar shouldBe: GXString paramName: ''gXfile'']'!

"result
 ------"

GXnblocks

addInstVarName: #gXnblocks
!

"Local variables
 ---------------"

GXnblocks

addInstVarName: #gXfile;

addInstVarName: #gXnblocks;

addInstVarName: #gXtype
!

!GXnblocks methodsFor: 'initializing'!

initVars
	gXfile _ (gXUcurrentUinputUsource).
	gXnblocks _ 0
!!

"BODY
 ----"

!GXnblocks methodsFor: 'running'!

evaluate
 gXfile isNil ifTrue: [^'NBLOCKS: Parameter FILE not specified and no default available' prenl ] .
gXtype _ (GXfiletype run line: 0 proc: self method: true; p1:(gXfile); condEvaluate) deepCopy.
((((gXtype) = 'raw')) | (((gXtype) = 'CRPT'))) ifTrue: [ 
gXnblocks _ (GXUgetUsnsUitem run line: 0 proc: self method: true; p1:0; p2:'NSP1'; p3:(gXfile); condEvaluate) deepCopy.
[nil.
self up.
^gXnblocks] value ].
GXcallout run line: 0 proc: self method: false; p1:'extFortran'; p2:29; condEvaluate.
(GXgUerrorUhandler run line: 0 proc: self method: true; condEvaluate) ifTrue: [ 
gXnblocks _ (gXU) deepCopy ].
self up.
^gXnblocks
!!
nil! "(fix for smalltalk bug, this line is ignored if there is a variable declaration error)"

"End of procedure"

GXcomment run line: 0 proc: self method: false; p1:'nblocks'; p2:'Return the number of blocks in an intermediate file'; condEvaluate!

Smalltalk at:#gxPolygonCreate put: 1.
Smalltalk at:#gxPolylineCreate put: 2.
Smalltalk at:#gxTextCreate put: 3.
Smalltalk at:#gxAxesCreate put: 4.
Smalltalk at:#gxGraticuleCreate put: 5.
Smalltalk at:#gxLabelsCreate put: 6.
Smalltalk at:#gxTitleCreate put: 7.
Smalltalk at:#gxPlotCreate put: 8.
Smalltalk at:#gxHistogramCreate put: 9.
Smalltalk at:#gxMarkersCreate put: 10.
Smalltalk at:#gxVErrorsCreate put: 11.
Smalltalk at:#gxHErrorsCreate put: 12.
Smalltalk at:#gxContourCreate put: 13.
Smalltalk at:#gxContourCreateArray put: 14.
Smalltalk at:#gxContourCreateFunction put: 15.
Smalltalk at:#gxContourCreateLabel put: 16.
Smalltalk at:#gxCellArrayCreate put: 17.
Smalltalk at:#gxCellWedgeCreate put: 18.
Smalltalk at:#gxColourTableAllocate put: 19.
Smalltalk at:#gxColourTableDeallocate put: 20.
Smalltalk at:#gxNewUnScaledWindow put: 21.
Smalltalk at:#gxNewScaledWindow put: 22.
Smalltalk at:#gxCreateMultiplotWindow put: 23.
Smalltalk at:#gxCreateWindowTwoD put: 24.
Smalltalk at:#gxCreateMultiplot put: 25.
Smalltalk at:#gxMultiplotMake put: 26.
Smalltalk at:#gxAddSpectra put: 27.
Smalltalk at:#gxDeviceOpen put: 28.
Smalltalk at:#gxDeviceClose put: 29.
Smalltalk at:#gxDeviceClear put: 30.
Smalltalk at:#gxPictureCreate put: 31.
Smalltalk at:#gxPictureDelete put: 32.
Smalltalk at:#gxPictureRedraw put: 33.
Smalltalk at:#gxPictureUndraw put: 34.
Smalltalk at:#gxPictureUndrawAll put: 35.
Smalltalk at:#gxPictureWindowDelete put: 36.
Smalltalk at:#gxPictureUndoUndraw put: 37.
Smalltalk at:#gxGetCursor put: 38.
Smalltalk at:#gxDisplay put: 39.
Smalltalk at:#gxCellFunctionCreate put: 40.
Smalltalk at:#gxNewPicture put: 41.
Smalltalk at:#gxAddSpectraArray put: 42.
Smalltalk at:#gxDeviceSelect put: 43.
Smalltalk at:#gxGetObjectFromTag put: 44.
Smalltalk at:#gxErrorsCreate put: 45.
Smalltalk at:#gxCellCreate put: 46.
Smalltalk at:#gxPictureSelect put: 47
!


Link subclass:#GraphicalObjects
        instanceVariableNames:'objid visible picture pictNum on id 
			       deviceList pictureList undrawPicture
			       device'
        classVariableNames: ' '
        poolDictionaries: ''
        category: nil
!

GraphicalObjects subclass: 'GXGobject'
	instanceVariableNames:''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GX types'
!

"Returns Open GENIE compatible base type for this class"
!GXGobject class methodsFor: 'type compatibility'!
species
	^GraphicalObjects class
!

"Coerce a type to Graphical object if possible"
asMe: aVal
	(aVal species = Integer)				"Note this is a special in that we don't really coerce"
	ifTrue: [ '**** Passing integer instead of Gobject here failing...' prdnl. ^nil "^aVal" ].
	^nil
!!
nil
!


! GraphicalObjects class methodsFor: 'instance creation' !

new
	^super new initGraphicalObjects
!

"Object tag is the pointer address of the OOP in C++"
getObjectFromTag: tagString
	^newGraphics function: gxGetObjectFromTag parameters: ((Array new: 1)
                                        at: 1 put: tagString;
                                        yourself)
!!
nil
!


"Return instance variables in form suitable for a TCL list
 we split on ^B"
! GraphicalObjects methodsFor: 'TCL' !

getinstvars
    | class instVars tmp |
    class _  self class.
    instVars _ class allInstVarNames.
    1 to: instVars size do:
	[ :i | tmp _ self instVarAt: i. 
	    ((tmp isKindOf: Integer) | (tmp isKindOf: Float) | (tmp isKindOf: String) | (tmp isKindOf: GXIntegerarray)) 
	    ifTrue: [
	        ( instVars at: i ) prn. '' prn. tmp prn. '' prn
	    ]
	]
!!
nil
!


! GraphicalObjects methodsFor: 'tester' !
getClassName
	|class instVars tmp|
  	class _ self class.
	class prn. 
!!
nil
!


! GraphicalObjects methodsFor: 'objid' !

objid
	^objid
!!
nil
!


! GraphicalObjects methodsFor: 'hide' !

hide                  
	visible
	ifTrue:[ visible _ false]                  

!!
nil
!


! GraphicalObjects methodsFor: 'unhide' !

unhide
        visible _ true

!!
nil
!


! GraphicalObjects methodsFor: 'switchOn' !
switchOn
	on _ true
!!
nil
!


!GraphicalObjects methodsFor: 'switchOff' !
switchOff
	on _ false
!!
nil
!


! GraphicalObjects methodsFor: 'switchOn' !

switchOn
	picture _ true

!!
nil
!


! GraphicalObjects methodsFor: 'switchOff' !

switchOff
        picture _ false

!!
nil
!


! GraphicalObjects methodsFor: 'check' !

check

^pictNum

!!
nil
!


! GraphicalObjects methodsFor: 'display' !

                                                " Only displays the objects  "
display                                         " which are not hidden, and  "
	                                    	" have been created properly "
(visible)
ifTrue:[ (objid class = GXIntegerarray)
        ifTrue:[
                ^newGraphics function: gxDisplay parameters: ((Array new: 2)
                                        at: 1 put: objid;
                                        at: 2 put: self;
                                        yourself)
               	]
 
        ifFalse:['ERROR, Drawable Object or ' pren.
                'Window has not been Created !!' prenl]]

!!
nil
!


! GraphicalObjects methodsFor: 'printList' !

printList
        deviceList class prn.
        ' (' prn.
        deviceList do:[: i | i class prn.
                            ' ' prn].
        ')' prnl

!!
nil
!


! GraphicalObjects methodsFor: 'select' !

select: aNumber
|tmp|
(aNumber class = Integer)
	ifTrue:[((aNumber > deviceList size ) | (aNumber negated >= deviceList size ))
       		ifFalse:[tmp _ ((deviceList at: aNumber) getID).
		newGraphics function: gxDeviceSelect parameters: ((Array new: 1)
       	                         at: 1 put: tmp;
             	                   yourself).
			^(deviceList at: aNumber)]
       		ifTrue:['SELECT ERROR' pren.
       		        'There are not that many devices! ' prinl]]

!!					
nil
!


! GraphicalObjects methodsFor: 'TCLSelect' !

tclSelect: aNumber

|tmp count temp anum|
count _ 0.
anum _ 0.
tmp _ (aNumber getID).
(tmp class = Integer)
ifTrue:[deviceList do:[: i | temp _ i getID.
			     count _ count + 1.
			     (tmp = temp)
			     ifTrue:[anum _ count]].
     	temp _ (deviceList at: anum).
	tmp _ (temp getID).
	(tmp isNil)
       ifFalse:[newGraphics function: gxDeviceSelect parameters: ((Array new: 1)
                         	at: 1 put: tmp;
               	                yourself).
   	^temp]]
!!
nil
!


! GraphicalObjects methodsFor: 'pointToDevice' !

pointToDevice: aNumber

|tmp|
(aNumber class = Integer)
ifTrue:[((aNumber > deviceList size) | (aNumber negated >= deviceList size))
       ifFalse:[tmp _ (deviceList at: aNumber).
                ^tmp]
       ifTrue:['SELECT ERROR' pren.
               'There are not that many devices! ' prinl]]
ifFalse:['ERROR' prinl]

!!
nil
!


! GraphicalObjects methodsFor: 'alterTCL' !

alterTCL: anObject and: aPicture with: anID

|tmp| 

tmp _ ((pictureList at: aPicture) testTCLAlter: anID).
			    
^tmp 

!!
nil
!


! GraphicalObjects methodsFor: 'private' !

initGraphicalObjects
	deviceList _ LinkedList new.
	pictureList _ LinkedList new
!!
nil
!


! GraphicalObjects methodsFor: 'printing' !

fprintOn: aStream
	self printOn: aStream	"Ensures we can print anything in a workspace"
!

printOn: aStream
        |className|

        className _ self class name.
        aStream nextPutAll: 'An instance of '.
        aStream nextPutAll: (className copyFrom: 1 to: className size); nl.
        aStream nextPutAll: '  deviceList: '.
        aStream nextPutAll: ' LinkedList( '.
        deviceList  do:[: i| aStream print: ( i class printString).
                              aStream nextPutAll:'  ' printString ].
        aStream nextPutAll: ' )'; nl.
        aStream nextPutAll: '  pictureList: '.
        aStream nextPutAll: ' LinkedList( '.
        pictureList  do:[: i| aStream print: ( i class printString).
                              aStream nextPutAll:'  ' printString ].
        aStream nextPutAll: ' )'; nl.

!!
nil
!

! GraphicalObjects methodsFor: 'update' !

update: aDrawableObject
    |class|
    class _ aDrawableObject class.
    (self isKindOf: class) ifFalse: [
	'update: invalid object type' prenl. 
	'updating ' pren. self class pren. ' with ' pren. class prenl.
	^nil
    ].
    " copy the instance variables (if any) "
    1 to: class instSize do:
    	[ :i | self instVarAt: i
	    	     put: (aDrawableObject instVarAt: i) ].

    " copy the indexed variables (if any) "
    class isVariable
    	ifTrue: [ 1 to: self basicSize do:
	    	    [ :i | self basicAt: i
		    	    	 put: (aDrawableObject basicAt: i) ] ].
    ^self
!!
nil
!
"Procedure class definition:
 ==========================="
((Smalltalk at: #GXUUinitUgraphicsUglobals ifAbsent: [ nil ]) isKindOf: Function class) ifTrue: [ ('Warning: Overriding previously defined procedure ', 'GXUUinitUgraphicsUglobals' gtype) prinl ].
Smalltalk at: #GXUUinitUgraphicsUglobals put: nil!
Function subclass: 'GXUUinitUgraphicsUglobals'
       instanceVariableNames: ''
       classVariableNames: ''
       poolDictionaries: ''
       category: 'Genie Procedure'
!

"HEADER
 ------"

"Global variables
 ----------------"

GXUUinitUgraphicsUglobals

addPoolVar: #gXUUgraphics value: (GXfields run line: 0 proc: self method: true; condEvaluate)!

"Global variables
 ----------------"

GXUUinitUgraphicsUglobals

addPoolVar: #gXUUwinnum value: 0!

"Global variables
 ----------------"

GXUUinitUgraphicsUglobals

addPoolVar: #gXUUpicnum value: 0!

"Global variables
 ----------------"

GXUUinitUgraphicsUglobals

addPoolVar: #gXUUstatus value: 0!

"Global variables
 ----------------"

GXUUinitUgraphicsUglobals

addPoolVar: #gXUUcount value: 0!

"Global variables
 ----------------"

GXUUinitUgraphicsUglobals

addPoolVar: #gXUUcol value: (gXU)!

"Global variables
 ----------------"

GXUUinitUgraphicsUglobals

addPoolVar: #gXDblack value: (('0') asSymbol)!

"Global variables
 ----------------"

GXUUinitUgraphicsUglobals

addPoolVar: #gXDwhite value: (('1') asSymbol)!

"Global variables
 ----------------"

GXUUinitUgraphicsUglobals

addPoolVar: #gXDred value: (('2') asSymbol)!

"Global variables
 ----------------"

GXUUinitUgraphicsUglobals

addPoolVar: #gXDgreen value: (('3') asSymbol)!

"Global variables
 ----------------"

GXUUinitUgraphicsUglobals

addPoolVar: #gXDblue value: (('4') asSymbol)!

"Global variables
 ----------------"

GXUUinitUgraphicsUglobals

addPoolVar: #gXDcyan value: (('5') asSymbol)!

"Global variables
 ----------------"

GXUUinitUgraphicsUglobals

addPoolVar: #gXDmagenta value: (('6') asSymbol)!

"Global variables
 ----------------"

GXUUinitUgraphicsUglobals

addPoolVar: #gXDyellow value: (('7') asSymbol)!

"Global variables
 ----------------"

GXUUinitUgraphicsUglobals

addPoolVar: #gXDorange value: (('8') asSymbol)!

"Global variables
 ----------------"

GXUUinitUgraphicsUglobals

addPoolVar: #gXDlightUgreen value: (('9') asSymbol)!

"Global variables
 ----------------"

GXUUinitUgraphicsUglobals

addPoolVar: #gXDseaUgreen value: (('10') asSymbol)!

"Global variables
 ----------------"

GXUUinitUgraphicsUglobals

addPoolVar: #gXDlightUblue value: (('11') asSymbol)!

"Global variables
 ----------------"

GXUUinitUgraphicsUglobals

addPoolVar: #gXDpurple value: (('12') asSymbol)!

"Global variables
 ----------------"

GXUUinitUgraphicsUglobals

addPoolVar: #gXDcrimson value: (('13') asSymbol)!

"Global variables
 ----------------"

GXUUinitUgraphicsUglobals

addPoolVar: #gXDdarkUgray value: (('14') asSymbol)!

"Global variables
 ----------------"

GXUUinitUgraphicsUglobals

addPoolVar: #gXDlightUgray value: (('15') asSymbol)!

"Global variables
 ----------------"

GXUUinitUgraphicsUglobals

addPoolVar: #gXUforeground value: (gXDwhite)!

"Global variables
 ----------------"

GXUUinitUgraphicsUglobals

addPoolVar: #gXUbackground value: (gXDblack)!

"Global variables
 ----------------"

GXUUinitUgraphicsUglobals

addPoolVar: #gXDfull value: (('1') asSymbol)!

"Global variables
 ----------------"

GXUUinitUgraphicsUglobals

addPoolVar: #gXDdash value: (('2') asSymbol)!

"Global variables
 ----------------"

GXUUinitUgraphicsUglobals

addPoolVar: #gXDdotUdash value: (('3') asSymbol)!

"Global variables
 ----------------"

GXUUinitUgraphicsUglobals

addPoolVar: #gXDdot value: (('4') asSymbol)!

"Global variables
 ----------------"

GXUUinitUgraphicsUglobals

addPoolVar: #gXDnormal value: (('1') asSymbol)!

"Global variables
 ----------------"

GXUUinitUgraphicsUglobals

addPoolVar: #gXDroman value: (('2') asSymbol)!

"Global variables
 ----------------"

GXUUinitUgraphicsUglobals

addPoolVar: #gXDitalic value: (('3') asSymbol)!

"Global variables
 ----------------"

GXUUinitUgraphicsUglobals

addPoolVar: #gXDscript value: (('4') asSymbol)!

"Global variables
 ----------------"

GXUUinitUgraphicsUglobals

addPoolVar: #gXDpoint value: (('1') asSymbol)!

"Global variables
 ----------------"

GXUUinitUgraphicsUglobals

addPoolVar: #gXDplus value: (('2') asSymbol)!

"Global variables
 ----------------"

GXUUinitUgraphicsUglobals

addPoolVar: #gXDstar value: (('3') asSymbol)!

"Global variables
 ----------------"

GXUUinitUgraphicsUglobals

addPoolVar: #gXDcircle value: (('4') asSymbol)!

"Global variables
 ----------------"

GXUUinitUgraphicsUglobals

addPoolVar: #gXDcross value: (('5') asSymbol)!

"Global variables
 ----------------"

GXUUinitUgraphicsUglobals

addPoolVar: #gXDbox value: (('6') asSymbol)!

"Global variables
 ----------------"

GXUUinitUgraphicsUglobals

addPoolVar: #gXUsdjUgw value: (GXfields run line: 0 proc: self method: true; condEvaluate)!

"Global variables
 ----------------"

GXUUinitUgraphicsUglobals

addPoolVar: #gXUgrUwksp value: (gXU)
!

!GXUUinitUgraphicsUglobals methodsFor: 'initializing'!

initVars

!!

"BODY
 ----"

!GXUUinitUgraphicsUglobals methodsFor: 'running'!

evaluate
(gXUUgraphics isKindOf: GXArray) | (gXUUgraphics isKindOf: GXWorkspace)
    ifTrue:  [ ((gXUUgraphics) at:#gXcurrentpicture put: (gXU) deepCopy) ]
    ifFalse: [ self error: 'Assignment to array or workspace failed, check that the destination is a structured type'].
(gXUUgraphics isKindOf: GXArray) | (gXUUgraphics isKindOf: GXWorkspace)
    ifTrue:  [ ((gXUUgraphics) at:#gXcurrentwindow put: (gXU) deepCopy) ]
    ifFalse: [ self error: 'Assignment to array or workspace failed, check that the destination is a structured type'].
(gXUUgraphics isKindOf: GXArray) | (gXUUgraphics isKindOf: GXWorkspace)
    ifTrue:  [ ((gXUUgraphics) at:#gXcurrentdevice put: (gXU) deepCopy) ]
    ifFalse: [ self error: 'Assignment to array or workspace failed, check that the destination is a structured type'].
(gXUUgraphics isKindOf: GXArray) | (gXUUgraphics isKindOf: GXWorkspace)
    ifTrue:  [ ((gXUUgraphics) at:#gXcurrentgraphics put: (gXU) deepCopy) ]
    ifFalse: [ self error: 'Assignment to array or workspace failed, check that the destination is a structured type'].
 gXUUgraphics at: #gXcurrentgraphics put: (GraphicalObjects new) .
self up.
^nil
!!
nil! "(fix for smalltalk bug, this line is ignored if there is a variable declaration error)"

"End of procedure"

GXUUinitUgraphicsUglobals run line: 0 proc: self method: false; condEvaluate!
((GXStructure) subclass:'Colourtable' fields:'R G B CI CONTRAST BRIGHTNESS DEVICE' comment:'Genie colour table structure')!
"Procedure class definition:
 ==========================="
((Smalltalk at: #GXUUrgbUcolour ifAbsent: [ nil ]) isKindOf: Function class) ifTrue: [ ('Warning: Overriding previously defined procedure ', 'GXUUrgbUcolour' gtype) prinl ].
Smalltalk at: #GXUUrgbUcolour put: nil!
Function subclass: 'GXUUrgbUcolour'
       instanceVariableNames: ''
       classVariableNames: ''
       poolDictionaries: ''
       category: 'Genie Procedure'
!

"HEADER
 ------"

"parameters
 ----------"

GXUUrgbUcolour

addInstVarName: #gXr;
compile:
'gXr: aVar
	aVar notNil ifTrue:[ gXr _ aVar ]';
compile:
'p1: aVar
	aVar notNil ifTrue:[ gXr _ aVar ]';

addInstVarName: #gXg;
compile:
'gXg: aVar
	aVar notNil ifTrue:[ gXg _ aVar ]';
compile:
'p2: aVar
	aVar notNil ifTrue:[ gXg _ aVar ]';

addInstVarName: #gXb;
compile:
'gXb: aVar
	aVar notNil ifTrue:[ gXb _ aVar ]';
compile:
'p3: aVar
	aVar notNil ifTrue:[ gXb _ aVar ]'!

"result
 ------"

GXUUrgbUcolour

addInstVarName: #gXcol
!

"Local variables
 ---------------"

GXUUrgbUcolour

addInstVarName: #gXcolourUindex
!

!GXUUrgbUcolour methodsFor: 'initializing'!

initVars
	gXcolourUindex _ 1 negated
!!

"BODY
 ----"

!GXUUrgbUcolour methodsFor: 'running'!

evaluate
 
    ( gXr isKindOf: Float ) ifFalse: [^'COLOUR: Parameter R is of the wrong type' prenl].
    ( gXg isKindOf: Float ) ifFalse: [^'COLOUR: Parameter G is of the wrong type' prenl].
    ( gXb isKindOf: Float ) ifFalse: [^'COLOUR: Parameter B is of the wrong type' prenl]
.
GXcallout run line: 0 proc: self method: false; p1:'objGraphics'; p2:50; condEvaluate.
gXcol _ (((GXasUstring run line: 0 proc: self method: true; p1:(gXcolourUindex); condEvaluate)) asSymbol) deepCopy.
self up.
^gXcol
!!
nil! "(fix for smalltalk bug, this line is ignored if there is a variable declaration error)"

"End of procedure"

"Procedure class definition:
 ==========================="
((Smalltalk at: #GXUUnamedUcolour ifAbsent: [ nil ]) isKindOf: Function class) ifTrue: [ ('Warning: Overriding previously defined procedure ', 'GXUUnamedUcolour' gtype) prinl ].
Smalltalk at: #GXUUnamedUcolour put: nil!
Function subclass: 'GXUUnamedUcolour'
       instanceVariableNames: ''
       classVariableNames: ''
       poolDictionaries: ''
       category: 'Genie Procedure'
!

"HEADER
 ------"

"parameters
 ----------"

GXUUnamedUcolour

addInstVarName: #gXname;
compile:
'gXname: aVar
	aVar notNil ifTrue:[gXname _ self typeCheck: aVar shouldBe: GXString paramName: ''gXname'']';
compile:
'p1: aVar
	aVar notNil ifTrue:[gXname _ self typeCheck: aVar shouldBe: GXString paramName: ''gXname'']'!

"result
 ------"

GXUUnamedUcolour

addInstVarName: #gXcol
!

"Local variables
 ---------------"

GXUUnamedUcolour

addInstVarName: #gXcolourUindex
!

!GXUUnamedUcolour methodsFor: 'initializing'!

initVars
	gXcolourUindex _ 1 negated
!!

"BODY
 ----"

!GXUUnamedUcolour methodsFor: 'running'!

evaluate
GXcallout run line: 0 proc: self method: false; p1:'objGraphics'; p2:51; condEvaluate.
gXcol _ (((GXasUstring run line: 0 proc: self method: true; p1:(gXcolourUindex); condEvaluate)) asSymbol) deepCopy.
self up.
^gXcol
!!
nil! "(fix for smalltalk bug, this line is ignored if there is a variable declaration error)"

"End of procedure"

"Procedure class definition:
 ==========================="
((Smalltalk at: #GXUUhlsUcolour ifAbsent: [ nil ]) isKindOf: Function class) ifTrue: [ ('Warning: Overriding previously defined procedure ', 'GXUUhlsUcolour' gtype) prinl ].
Smalltalk at: #GXUUhlsUcolour put: nil!
Function subclass: 'GXUUhlsUcolour'
       instanceVariableNames: ''
       classVariableNames: ''
       poolDictionaries: ''
       category: 'Genie Procedure'
!

"HEADER
 ------"

"parameters
 ----------"

GXUUhlsUcolour

addInstVarName: #gXh;
compile:
'gXh: aVar
	aVar notNil ifTrue:[ gXh _ aVar ]';
compile:
'p1: aVar
	aVar notNil ifTrue:[ gXh _ aVar ]';

addInstVarName: #gXl;
compile:
'gXl: aVar
	aVar notNil ifTrue:[ gXl _ aVar ]';
compile:
'p2: aVar
	aVar notNil ifTrue:[ gXl _ aVar ]';

addInstVarName: #gXs;
compile:
'gXs: aVar
	aVar notNil ifTrue:[ gXs _ aVar ]';
compile:
'p3: aVar
	aVar notNil ifTrue:[ gXs _ aVar ]'!

"result
 ------"

GXUUhlsUcolour

addInstVarName: #gXcol
!

"Local variables
 ---------------"

GXUUhlsUcolour

addInstVarName: #gXcolourUindex
!

!GXUUhlsUcolour methodsFor: 'initializing'!

initVars
	gXcolourUindex _ 1 negated
!!

"BODY
 ----"

!GXUUhlsUcolour methodsFor: 'running'!

evaluate

    ( gXh isKindOf: Float ) ifFalse: [^'COLOUR: Parameter H is of the wrong type' prenl].
    ( gXl isKindOf: Float ) ifFalse: [^'COLOUR: Parameter L is of the wrong type' prenl].
    ( gXs isKindOf: Float ) ifFalse: [^'COLOUR: Parameter S is of the wrong type' prenl]
.
GXcallout run line: 0 proc: self method: false; p1:'objGraphics'; p2:52; condEvaluate.
gXcol _ (((GXasUstring run line: 0 proc: self method: true; p1:(gXcolourUindex); condEvaluate)) asSymbol) deepCopy.
self up.
^gXcol
!!
nil! "(fix for smalltalk bug, this line is ignored if there is a variable declaration error)"

"End of procedure"

"Procedure class definition:
 ==========================="
((Smalltalk at: #GXcolour ifAbsent: [ nil ]) isKindOf: Function class) ifTrue: [ ('Warning: Overriding previously defined procedure ', 'GXcolour' gtype) prinl ].
Smalltalk at: #GXcolour put: nil!
Function subclass: 'GXcolour'
       instanceVariableNames: ''
       classVariableNames: ''
       poolDictionaries: ''
       category: 'Genie Procedure'
!

"HEADER
 ------"

"qualifiers
 ----------"

GXcolour

addInstVarName: #gXhls;
compile:
'gXhls
	gXhls _ true';
compile:
'gXhls: aBoolean
	gXhls _ aBoolean';

addInstVarName: #gXrgb;
compile:
'gXrgb
	gXrgb _ true';
compile:
'gXrgb: aBoolean
	gXrgb _ aBoolean';

addInstVarName: #gXnamed;
compile:
'gXnamed
	gXnamed _ true';
compile:
'gXnamed: aBoolean
	gXnamed _ aBoolean'!

"parameters
 ----------"

GXcolour

addInstVarName: #gXp1;
compile:
'gXp1: aVar
	aVar notNil ifTrue:[ gXp1 _ aVar ]';
compile:
'p1: aVar
	aVar notNil ifTrue:[ gXp1 _ aVar ]';

addInstVarName: #gXp2;
compile:
'gXp2: aVar
	aVar notNil ifTrue:[ gXp2 _ aVar ]';
compile:
'p2: aVar
	aVar notNil ifTrue:[ gXp2 _ aVar ]';

addInstVarName: #gXp3;
compile:
'gXp3: aVar
	aVar notNil ifTrue:[ gXp3 _ aVar ]';
compile:
'p3: aVar
	aVar notNil ifTrue:[ gXp3 _ aVar ]';

addInstVarName: #gXh;
compile:
'gXh: aVar
	aVar notNil ifTrue:[ gXh _ aVar ]';
compile:
'p4: aVar
	aVar notNil ifTrue:[ gXh _ aVar ]';

addInstVarName: #gXl;
compile:
'gXl: aVar
	aVar notNil ifTrue:[ gXl _ aVar ]';
compile:
'p5: aVar
	aVar notNil ifTrue:[ gXl _ aVar ]';

addInstVarName: #gXs;
compile:
'gXs: aVar
	aVar notNil ifTrue:[ gXs _ aVar ]';
compile:
'p6: aVar
	aVar notNil ifTrue:[ gXs _ aVar ]';

addInstVarName: #gXr;
compile:
'gXr: aVar
	aVar notNil ifTrue:[ gXr _ aVar ]';
compile:
'p7: aVar
	aVar notNil ifTrue:[ gXr _ aVar ]';

addInstVarName: #gXg;
compile:
'gXg: aVar
	aVar notNil ifTrue:[ gXg _ aVar ]';
compile:
'p8: aVar
	aVar notNil ifTrue:[ gXg _ aVar ]';

addInstVarName: #gXb;
compile:
'gXb: aVar
	aVar notNil ifTrue:[ gXb _ aVar ]';
compile:
'p9: aVar
	aVar notNil ifTrue:[ gXb _ aVar ]'!

"result
 ------"

GXcolour

addInstVarName: #gXcol
!

!GXcolour methodsFor: 'initializing'!

initVars
	gXhls _ false.
	gXrgb _ false.
	gXnamed _ false
!!

"BODY
 ----"

!GXcolour methodsFor: 'running'!

evaluate
(gXhls) ifTrue: [ 

    gXp1 notNil ifTrue: [ gXh _ gXp1 ].
    gXp2 notNil ifTrue: [ gXl _ gXp2 ].
    gXp3 notNil ifTrue: [ gXs _ gXp3 ].
    gXh isNil ifTrue: [^'COLOUR: Parameter H not specified' prenl].
    gXl isNil ifTrue: [^'COLOUR: Parameter L not specified' prenl].
    gXs isNil ifTrue: [^'COLOUR: Parameter S not specified' prenl]
.
gXcol _ (GXUUhlsUcolour run line: 0 proc: self method: true; p1:(gXh); p2:(gXl); p3:(gXs); condEvaluate) deepCopy ] ifFalse: [ (gXrgb) ifTrue: [ 

    gXp1 notNil ifTrue: [ gXr _ gXp1 ].
    gXp2 notNil ifTrue: [ gXg _ gXp2 ].
    gXp3 notNil ifTrue: [ gXb _ gXp3 ].
    gXr isNil ifTrue: [^'COLOUR: Parameter R not specified' prenl].
    gXg isNil ifTrue: [^'COLOUR: Parameter G not specified' prenl].
    gXb isNil ifTrue: [^'COLOUR: Parameter B not specified' prenl]
.
gXcol _ (GXUUrgbUcolour run line: 0 proc: self method: true; p1:(gXr); p2:(gXg); p3:(gXb); condEvaluate) deepCopy ] ifFalse: [ (gXnamed) ifTrue: [ 
gXcol _ (GXUUnamedUcolour run line: 0 proc: self method: true; p1:(gXp1); condEvaluate) deepCopy ] ifFalse: [ 
GXprinten run line: 0 proc: self method: false; p1:'COLOUR: Invalid or non existant qualifier'; condEvaluate.
[nil.
self up.
^gXcol] value ] ]].
self up.
^gXcol
!!
nil! "(fix for smalltalk bug, this line is ignored if there is a variable declaration error)"

"End of procedure"

GXcomment run line: 0 proc: self method: false; p1:'colour'; p2:'Define a new graphics colour based on one of several models'; condEvaluate!
"Procedure class definition:
 ==========================="
((Smalltalk at: #GXUUfreeUcolourtable ifAbsent: [ nil ]) isKindOf: Function class) ifTrue: [ ('Warning: Overriding previously defined procedure ', 'GXUUfreeUcolourtable' gtype) prinl ].
Smalltalk at: #GXUUfreeUcolourtable put: nil!
Function subclass: 'GXUUfreeUcolourtable'
       instanceVariableNames: ''
       classVariableNames: ''
       poolDictionaries: ''
       category: 'Genie Procedure'
!

"HEADER
 ------"

"parameters
 ----------"

GXUUfreeUcolourtable

addInstVarName: #gXtable;
compile:
'gXtable: aVar
	aVar notNil ifTrue:[gXtable _ self typeCheck: aVar shouldBe: GXColourtable paramName: ''gXtable'']';
compile:
'p1: aVar
	aVar notNil ifTrue:[gXtable _ self typeCheck: aVar shouldBe: GXColourtable paramName: ''gXtable'']'
!

!GXUUfreeUcolourtable methodsFor: 'initializing'!

initVars

!!

"BODY
 ----"

!GXUUfreeUcolourtable methodsFor: 'running'!

evaluate
 gXtable isNil ifTrue: [ 'FREE_COLOURTABLE: Parameter TABLE not given' printNl. ^nil ] .
GXcallout run line: 0 proc: self method: false; p1:'objGraphics'; p2:57; condEvaluate.
GXgUerrorUhandler run line: 0 proc: self method: false; condEvaluate.
self up.
^nil
!!
nil! "(fix for smalltalk bug, this line is ignored if there is a variable declaration error)"

"End of procedure"

"Procedure class definition:
 ==========================="
((Smalltalk at: #GXUUrgbUcolourtable ifAbsent: [ nil ]) isKindOf: Function class) ifTrue: [ ('Warning: Overriding previously defined procedure ', 'GXUUrgbUcolourtable' gtype) prinl ].
Smalltalk at: #GXUUrgbUcolourtable put: nil!
Function subclass: 'GXUUrgbUcolourtable'
       instanceVariableNames: ''
       classVariableNames: ''
       poolDictionaries: ''
       category: 'Genie Procedure'
!

"HEADER
 ------"

"parameters
 ----------"

GXUUrgbUcolourtable

addInstVarName: #gXr;
compile:
'gXr: aVar
	aVar notNil ifTrue:[gXr _ self typeCheck: aVar shouldBe: GXRealarray paramName: ''gXr'']';
compile:
'p1: aVar
	aVar notNil ifTrue:[gXr _ self typeCheck: aVar shouldBe: GXRealarray paramName: ''gXr'']';

addInstVarName: #gXg;
compile:
'gXg: aVar
	aVar notNil ifTrue:[gXg _ self typeCheck: aVar shouldBe: GXRealarray paramName: ''gXg'']';
compile:
'p2: aVar
	aVar notNil ifTrue:[gXg _ self typeCheck: aVar shouldBe: GXRealarray paramName: ''gXg'']';

addInstVarName: #gXb;
compile:
'gXb: aVar
	aVar notNil ifTrue:[gXb _ self typeCheck: aVar shouldBe: GXRealarray paramName: ''gXb'']';
compile:
'p3: aVar
	aVar notNil ifTrue:[gXb _ self typeCheck: aVar shouldBe: GXRealarray paramName: ''gXb'']';

addInstVarName: #gXncol;
compile:
'gXncol: aVar
	aVar notNil ifTrue:[gXncol _ self typeCheck: aVar shouldBe: GXInteger paramName: ''gXncol'']';
compile:
'p4: aVar
	aVar notNil ifTrue:[gXncol _ self typeCheck: aVar shouldBe: GXInteger paramName: ''gXncol'']';

addInstVarName: #gXcontrast;
compile:
'gXcontrast: aVar
	aVar notNil ifTrue:[gXcontrast _ self typeCheck: aVar shouldBe: GXReal paramName: ''gXcontrast'']';
compile:
'p5: aVar
	aVar notNil ifTrue:[gXcontrast _ self typeCheck: aVar shouldBe: GXReal paramName: ''gXcontrast'']';

addInstVarName: #gXbrightness;
compile:
'gXbrightness: aVar
	aVar notNil ifTrue:[gXbrightness _ self typeCheck: aVar shouldBe: GXReal paramName: ''gXbrightness'']';
compile:
'p6: aVar
	aVar notNil ifTrue:[gXbrightness _ self typeCheck: aVar shouldBe: GXReal paramName: ''gXbrightness'']'!

"result
 ------"

GXUUrgbUcolourtable

addInstVarName: #gXtable
!

"Local variables
 ---------------"

GXUUrgbUcolourtable

addInstVarName: #gXcontrast;

addInstVarName: #gXbrightness
!

!GXUUrgbUcolourtable methodsFor: 'initializing'!

initVars
	gXcontrast _ 1.000000000000000e000.
	gXbrightness _ 5.000000000000000e-01
!!

"BODY
 ----"

!GXUUrgbUcolourtable methodsFor: 'running'!

evaluate
 
    gXr isNil ifTrue: [ '_RGB_COLOURTABLE: Parameter R not given' prenl. ^nil ].
    gXg isNil ifTrue: [ '_RGB_COLOURTABLE: Parameter G not given' prenl. ^nil ].
    gXb isNil ifTrue: [ '_RGB_COLOURTABLE: Parameter B not given' prenl. ^nil ].
    gXncol isNil ifTrue: [ gXncol _ gXr size ].
    gXtable _ GXColourtable new
.
GXcallout run line: 0 proc: self method: false; p1:'objGraphics'; p2:56; condEvaluate.
(GXgUerrorUhandler run line: 0 proc: self method: true; condEvaluate) ifTrue: [ 
gXtable _ (gXU) deepCopy ].
(gXtable isKindOf: GXArray) | (gXtable isKindOf: GXWorkspace)
    ifTrue:  [ ((gXtable) at:#gXcontrast put: (gXcontrast) deepCopy) ]
    ifFalse: [ self error: 'Assignment to array or workspace failed, check that the destination is a structured type'].
(gXtable isKindOf: GXArray) | (gXtable isKindOf: GXWorkspace)
    ifTrue:  [ ((gXtable) at:#gXbrightness put: (gXbrightness) deepCopy) ]
    ifFalse: [ self error: 'Assignment to array or workspace failed, check that the destination is a structured type'].
 
    "(gXUUgraphics at: #gXcurrentdevice) colourtable: gXtable."
    gXtable at: #gXdevice put: ((gXUUgraphics at: #gXcurrentdevice) devType)
.
self up.
^gXtable
!!
nil! "(fix for smalltalk bug, this line is ignored if there is a variable declaration error)"

"End of procedure"

"Procedure class definition:
 ==========================="
((Smalltalk at: #GXcolourtable ifAbsent: [ nil ]) isKindOf: Function class) ifTrue: [ ('Warning: Overriding previously defined procedure ', 'GXcolourtable' gtype) prinl ].
Smalltalk at: #GXcolourtable put: nil!
Function subclass: 'GXcolourtable'
       instanceVariableNames: ''
       classVariableNames: ''
       poolDictionaries: ''
       category: 'Genie Procedure'
!

"HEADER
 ------"

"qualifiers
 ----------"

GXcolourtable

addInstVarName: #gXrgb;
compile:
'gXrgb
	gXrgb _ true';
compile:
'gXrgb: aBoolean
	gXrgb _ aBoolean';

addInstVarName: #gXcreate;
compile:
'gXcreate
	gXcreate _ true';
compile:
'gXcreate: aBoolean
	gXcreate _ aBoolean';

addInstVarName: #gXdelete;
compile:
'gXdelete
	gXdelete _ true';
compile:
'gXdelete: aBoolean
	gXdelete _ aBoolean';

addInstVarName: #gXrainbow;
compile:
'gXrainbow
	gXrainbow _ true';
compile:
'gXrainbow: aBoolean
	gXrainbow _ aBoolean';

addInstVarName: #gXheat;
compile:
'gXheat
	gXheat _ true';
compile:
'gXheat: aBoolean
	gXheat _ aBoolean'!

"parameters
 ----------"

GXcolourtable

addInstVarName: #gXp1;
compile:
'gXp1: aVar
	aVar notNil ifTrue:[ gXp1 _ aVar ]';
compile:
'p1: aVar
	aVar notNil ifTrue:[ gXp1 _ aVar ]';

addInstVarName: #gXp2;
compile:
'gXp2: aVar
	aVar notNil ifTrue:[ gXp2 _ aVar ]';
compile:
'p2: aVar
	aVar notNil ifTrue:[ gXp2 _ aVar ]';

addInstVarName: #gXp3;
compile:
'gXp3: aVar
	aVar notNil ifTrue:[ gXp3 _ aVar ]';
compile:
'p3: aVar
	aVar notNil ifTrue:[ gXp3 _ aVar ]';

addInstVarName: #gXp4;
compile:
'gXp4: aVar
	aVar notNil ifTrue:[ gXp4 _ aVar ]';
compile:
'p4: aVar
	aVar notNil ifTrue:[ gXp4 _ aVar ]';

addInstVarName: #gXp5;
compile:
'gXp5: aVar
	aVar notNil ifTrue:[ gXp5 _ aVar ]';
compile:
'p5: aVar
	aVar notNil ifTrue:[ gXp5 _ aVar ]';

addInstVarName: #gXp6;
compile:
'gXp6: aVar
	aVar notNil ifTrue:[ gXp6 _ aVar ]';
compile:
'p6: aVar
	aVar notNil ifTrue:[ gXp6 _ aVar ]';

addInstVarName: #gXncolours;
compile:
'gXncolours: aVar
	aVar notNil ifTrue:[gXncolours _ self typeCheck: aVar shouldBe: GXInteger paramName: ''gXncolours'']';
compile:
'p7: aVar
	aVar notNil ifTrue:[gXncolours _ self typeCheck: aVar shouldBe: GXInteger paramName: ''gXncolours'']';

addInstVarName: #gXred;
compile:
'gXred: aVar
	aVar notNil ifTrue:[gXred _ self typeCheck: aVar shouldBe: GXRealarray paramName: ''gXred'']';
compile:
'p8: aVar
	aVar notNil ifTrue:[gXred _ self typeCheck: aVar shouldBe: GXRealarray paramName: ''gXred'']';

addInstVarName: #gXgreen;
compile:
'gXgreen: aVar
	aVar notNil ifTrue:[gXgreen _ self typeCheck: aVar shouldBe: GXRealarray paramName: ''gXgreen'']';
compile:
'p9: aVar
	aVar notNil ifTrue:[gXgreen _ self typeCheck: aVar shouldBe: GXRealarray paramName: ''gXgreen'']';

addInstVarName: #gXblue;
compile:
'gXblue: aVar
	aVar notNil ifTrue:[gXblue _ self typeCheck: aVar shouldBe: GXRealarray paramName: ''gXblue'']';
compile:
'p10: aVar
	aVar notNil ifTrue:[gXblue _ self typeCheck: aVar shouldBe: GXRealarray paramName: ''gXblue'']';

addInstVarName: #gXcontrast;
compile:
'gXcontrast: aVar
	aVar notNil ifTrue:[gXcontrast _ self typeCheck: aVar shouldBe: GXReal paramName: ''gXcontrast'']';
compile:
'p11: aVar
	aVar notNil ifTrue:[gXcontrast _ self typeCheck: aVar shouldBe: GXReal paramName: ''gXcontrast'']';

addInstVarName: #gXbrightness;
compile:
'gXbrightness: aVar
	aVar notNil ifTrue:[gXbrightness _ self typeCheck: aVar shouldBe: GXReal paramName: ''gXbrightness'']';
compile:
'p12: aVar
	aVar notNil ifTrue:[gXbrightness _ self typeCheck: aVar shouldBe: GXReal paramName: ''gXbrightness'']';

addInstVarName: #gXtable;
compile:
'gXtable: aVar
	aVar notNil ifTrue:[gXtable _ self typeCheck: aVar shouldBe: GXColourtable paramName: ''gXtable'']';
compile:
'p13: aVar
	aVar notNil ifTrue:[gXtable _ self typeCheck: aVar shouldBe: GXColourtable paramName: ''gXtable'']'!

"result
 ------"

GXcolourtable

addInstVarName: #gXres
!

"Global variables
 ----------------"

GXcolourtable

addPoolVar: #gXUUstatus
!

!GXcolourtable methodsFor: 'initializing'!

initVars
	gXrgb _ false.
	gXcreate _ false.
	gXdelete _ false.
	gXrainbow _ false.
	gXheat _ false
!!

"BODY
 ----"

!GXcolourtable methodsFor: 'running'!

evaluate
((gXUUstatus) = 0) ifTrue: [ 
GXprinten run line: 0 proc: self method: false; p1:'COLOURATBLE: No device open'; condEvaluate.
[nil.
self up.
^gXres] value ].
 gXncolours isNil ifTrue: [ gXncolours _ gXp1 ] .
(gXrgb) ifTrue: [ 

	gXred isNil ifTrue: [ gXred _ gXp2 ].
	gXgreen isNil ifTrue: [ gXgreen _ gXp3 ].
	gXblue isNil ifTrue: [ gXblue _ gXp4 ].
	gXcontrast isNil ifTrue: [ gXcontrast _ gXp5 ].
	gXbrightness isNil ifTrue: [ gXbrightness _ gXp6 ]
    .
gXres _ (GXUUrgbUcolourtable run line: 0 proc: self method: true; p1:(gXred); p2:(gXgreen); p3:(gXblue); p4:(gXncolours); p5:(gXcontrast); p6:(gXbrightness); condEvaluate) deepCopy ] ifFalse: [ (gXdelete) ifTrue: [ 
 gXtable isNil ifTrue: [ gXtable _ gXp1 ] .
GXUUfreeUcolourtable run line: 0 proc: self method: false; p1:(gXtable); condEvaluate ] ifFalse: [ (gXrainbow) ifTrue: [ 

	gXcontrast isNil ifTrue: [ gXcontrast _ gXp2 ].
	gXbrightness isNil ifTrue: [ gXbrightness _ gXp3 ].
	gXred _ GXRealarray new: 10. gXgreen _ GXRealarray new: 10. gXblue _ GXRealarray new: 10.
	gXred at: 1 put: 0.0; at: 2 put: 0.0; at: 3 put: 0.0; at: 4 put: 0.0; at: 5 put: 0.6; 
	    at: 6 put: 1.0; at: 7 put: 1.0; at: 8 put: 1.0; at: 9 put: 1.0; at: 10 put: 1.0 .
	gXgreen at: 1 put: 0.0; at: 2 put: 0.0; at: 3 put: 0.0; at: 4 put: 1.0; at: 5 put: 1.0; 
	    at: 6 put: 1.0; at: 7 put: 0.6; at: 8 put: 0.0; at: 9 put: 0.5; at: 10 put: 1.0 .
	gXblue at: 1 put: 0.01; at: 2 put: 0.4; at: 3 put: 0.8; at: 4 put: 1.0; at: 5 put: 0.3; 
	    at: 6 put: 0.0; at: 7 put: 0.0; at: 8 put: 0.0; at: 9 put: 0.5; at: 10 put: 1.0
.
gXres _ (GXUUrgbUcolourtable run line: 0 proc: self method: true; p1:(gXred); p2:(gXgreen); p3:(gXblue); p4:(gXncolours); p5:(gXcontrast); p6:(gXbrightness); condEvaluate) deepCopy ] ifFalse: [ (gXheat) ifTrue: [ 

	gXcontrast isNil ifTrue: [ gXcontrast _ gXp2 ].
	gXbrightness isNil ifTrue: [ gXbrightness _ gXp3 ].
	gXred _ GXRealarray new: 6. gXgreen _ GXRealarray new: 6. gXblue _ GXRealarray new: 6.
	gXred at: 1 put: 0.0; at: 2 put: 0.0; at: 3 put: 0.0; at: 4 put: 0.0; at: 5 put: 0.5; at: 6 put: 1.0 .
	gXgreen at: 1 put: 0.0; at: 2 put: 0.0; at: 3 put: 0.5; at: 4 put: 1.0; at: 5 put: 1.0; at: 6 put: 1.0 .
	gXblue at: 1 put: 0.5; at: 2 put: 1.0; at: 3 put: 1.0; at: 4 put: 1.0; at: 5 put: 1.0; at: 6 put: 1.0 
.
gXres _ (GXUUrgbUcolourtable run line: 0 proc: self method: true; p1:(gXred); p2:(gXgreen); p3:(gXblue); p4:(gXncolours); p5:(gXcontrast); p6:(gXbrightness); condEvaluate) deepCopy ] ifFalse: [ 
GXprinten run line: 0 proc: self method: false; p1:'COLOURTABLE: Invalid or non existant qualifier'; condEvaluate ] ]]].
self up.
^gXres
!!
nil! "(fix for smalltalk bug, this line is ignored if there is a variable declaration error)"

"End of procedure"

GXcomment run line: 0 proc: self method: false; p1:'colourtable'; p2:'Define a new graphics colour table based on one of several models'; condEvaluate!


GraphicalObjects subclass:#Device
        instanceVariableNames: 'colourtable devType fileName width height ra 
				pictureList undrawPicture status picture device'
        classVariableNames: ' DefaultWidth DefaultHeight DefaultRa check
			      DefaultDevType DefaultFileName 
			      DefaultUndrawPicture DefaultStatus'
        poolDictionaries: ' '
        category: nil
!
nil
!


! Device class methodsFor: 'instance creation' !

new: devType
	devType _ devType asLowercase.
	(devType = 'xw') ifTrue: [devType _ 'xwindow'].
	( gXUUdrivers keys includes: 'gX', devType )
	ifFalse:['DEVICE ERROR' prenl.
		 'For Device, you must use a supported device type: ' prinl.
		 gXUUdrivers keys sort do:[ :k |
			k gtype prin. '	- ' prin.
			(gXUUdrivers at: k) prinl].
		 DefaultDevType _ nil.
		 ^nil
		]
	ifTrue:[
	 	  DefaultDevType _ devType.	
		  ^super new initDevice
		]
!

defaultWidth
		^DefaultWidth
!

defaultWidth: aWi
		DefaultWidth _ aWi
!

defaultHeight
		^DefaultHeight
!

defaultHeight: aHe
		DefaultHeight _ aHe
!

defaultRa
		^DefaultRa
!

defaultRa: aRa
		DefaultRa _ aRa
!

defaultFileName
		^DefaultFileName
!


defaultFileName: afN
		DefaultFileName _ afN
!

defaultUndrawPicture
		^DefaultUndrawPicture
!

defaultUndrawPicture: aUP
		DefaultUndrawPicture _ aUP
!
defaultStatus 
		^DefaultStatus
!
defaultStatus: ads
		DefaultStatus _ ads
!!
nil
!
Device defaultWidth: 8.0!
Device defaultHeight: 8.0!
Device defaultRa: 0.7!
Device defaultFileName: ' '!
Device defaultUndrawPicture: nil!
Device defaultStatus: 0!


! Device methodsFor: 'status' !
status
	^status
!!
nil
!


! Device methodsFor: 'id' !
id
	^id
!!
nil
!


! Device methodsFor: 'fileName' !

fileName
	^fileName
!

fileName: afN
	fileName _ afN.

(fileName isKindOf: String) 
ifFalse:[(fileName = nil)
	 ifTrue:[fileName _ DefaultFileName]
	 ifFalse:['FILENAME ERROR' prenl.
		  'For fileName enter a String' prinl.
		  fileName _ DefaultFileName]]
!!
nil
!


! Device methodsFor: 'width' !

width
	^width
!

width: aWi
	width _ aWi.

(width class = Float)
ifFalse:[(width = nil)
	ifTrue:[width _ DefaultWidth]
	ifFalse:['WIDTH ERROR' prenl.
		 'For width enter a float' prinl.
		 width _ DefaultWidth]]
!!
nil
!


! Device methodsFor: 'height' !

height
	^height
!

height: aHe
	height _ aHe.

(height class = Float)
ifFalse:[(height = nil)
	ifTrue:[height _ DefaultHeight]
	ifFalse:['HEIGHT ERROR' prenl.
        	 'For height enter a float' prinl.
        	 height _ DefaultHeight]]
!!
nil
!


! Device methodsFor: 'ra' !

ra
	^ra
!

ra: aRa
	ra _ aRa.

(ra class = Float)
ifFalse:[(ra = nil)
	ifTrue:[ra _ DefaultRa]
	ifFalse:['RA ERROR' prenl.
		 'For ra enter a float' prinl.
		 ra _ DefaultRa]]

!!
nil
!


! Device methodsFor: 'colourtable' !

colourtable: allocate

colourtable _ allocate.
^colourtable

!!
nil
!


! Device methodsFor: 'private' !


initDevice

	pictureList _ LinkedList new.

	height _ DefaultHeight.

	width _ DefaultWidth.
	
	ra _ DefaultRa.

	fileName _ DefaultFileName.
!!
nil
!


! Device methodsFor: 'Open' !
					" Opens the device "
Open

(DefaultDevType = nil)
ifTrue:['DEVICE ERROR' prenl.
	'devType has not been specified' prinl]
ifFalse:[status _ 1.
id _ newGraphics function: gxDeviceOpen parameters: ((Array new: 5)
			at: 1 put: DefaultDevType;
			at: 2 put: fileName;
			at: 3 put: width;
			at: 4 put: height;
			at: 5 put: ra;
			yourself)
	]
!!
nil
!


! Device methodsFor: 'Close' !
					" Closes the device "
Close

|tmp|
status _ 0.
tmp _ newGraphics function: gxDeviceClose parameters: ((Array new: 0)
			yourself)
!!
nil
!


! Device methodsFor: 'Clear' !
					" Clears the device "
Clear		
|tmp|
tmp _ newGraphics function: gxDeviceClear parameters: ((Array new: 0)
			yourself)

!!
nil
!


! Device class methodsFor: 'Clearing' !
					" Clears the device when the name "
Clearing				" is not known !		  "
|tmp|
tmp _ newGraphics function: gxDeviceClear parameters: ((Array new: 0)
                        yourself)

!!
nil
!


! Device class methodsFor: 'Closing' !

Closing
|tmp|
tmp _ newGraphics function: gxDeviceClose parameters: ((Array new: 0)
			yourself)

!!
nil
!


! Device methodsFor: 'getID' !

getID
	^id
!!
nil
!


! Device methodsFor: 'getPictureNumber' !

getPictureNumber
	^picture
!!
nil
!


! Device methodsFor: 'printing' !
                                        " Lets the command 'prnl' & 'printNl' "
printOn: aStream                        " print out the parameters, of the    "
 	|className|                     " device (AND NOTHING ELSE)           "
        className _ self class name.
        aStream nextPutAll: 'An instance of '.
        aStream nextPutAll: (className copyFrom: 1 to: className size); nl.
	aStream nextPutAll: '  Device Type: '.
	aStream nextPutAll: DefaultDevType printString; nl.
	aStream nextPutAll: '  width: '.
	aStream nextPutAll: width printString; nl.
	aStream nextPutAll: '  height: '.
	aStream nextPutAll: height printString; nl.
	aStream nextPutAll: '  colourtable:  '.
	aStream nextPutAll: colourtable printString; nl.
	aStream nextPutAll: '  picture:  '.
	aStream nextPutAll: picture printString; nl.
	aStream nextPutAll: '  status:  '.
	aStream nextPutAll: status printString; nl.
	aStream nextPutAll: '  id:  '.
	aStream nextPutAll: id printString; nl
!

devType
		^devType
!!
nil
!



GraphicalObjects subclass:#DrawableObjects
        instanceVariableNames:'colour lineThickness winColour'   
        classVariableNames: 'DefaultColour DefaultLineThickness'
        poolDictionaries: ''
        category: nil
!
nil
!

!DrawableObjects class methodsFor: 'instance creation' !

new
        ^super new initDefaults
!

defaultLineThickness
                        ^DefaultLineThickness

!

defaultLineThickness: aLT
                        DefaultLineThickness _ aLT

!
defaultColour
                ^DefaultColour
!

defaultColour: aLC
                DefaultColour _ aLC
!!
nil
!
DrawableObjects defaultColour: gXDwhite !
DrawableObjects defaultLineThickness: 1.0!


! DrawableObjects methodsFor: 'colour' !

colour
        ^colour

!

colour: aLC
        colour _ aLC.

(colour isKindOf: Symbol)
ifFalse:[(colour = nil)
	ifTrue:[colour _ DefaultColour]
	ifFalse:['COLOUR ERROR, ' pren.
	 	 colour _ DefaultColour]]

!!
nil
!


!DrawableObjects methodsFor: 'lineThickness' !

lineThickness
        ^lineThickness
!

lineThickness: aLT
        lineThickness _ aLT.
	(lineThickness isKindOf: Integer) ifTrue: [ lineThickness _ lineThickness * 1.0 ].
	(lineThickness isKindOf: Float)
	    ifFalse:[(lineThickness = nil)
		ifTrue:[lineThickness _ DefaultLineThickness]
		ifFalse:['LINETHICKNESS Error, ' pren.
	 	 'enter a Float' prinl.
	  	 lineThickness _ DefaultLineThickness] 
	    ]
!!
nil
!


! DrawableObjects methodsFor: 'private' !

initDefaults

        colour _ DefaultColour.

        lineThickness _ DefaultLineThickness

!!
nil
!


DrawableObjects subclass:#TwoD
        instanceVariableNames: 'itype'
        classVariableNames: 'DefaultIType '
        poolDictionaries: ' '
        category: nil
!
nil
!


! TwoD class methodsFor: 'instance creation' !

new
	^super new initTwoD

!

defaultIType
                ^DefaultIType

!

defaultIType: aIT
                DefaultIType _ aIT

!!
nil
!
TwoD defaultIType: 0 !


! TwoD methodsFor: 'itype' !

itype
        ^itype

!

itype: aIT
        itype _ aIT.

(itype = nil)
ifTrue:[itype _ DefaultIType].
(itype = 0)
ifFalse:[ (itype =1)
        ifFalse:[ (itype = 2)
		ifFalse:['ITYPE ERROR, Using Default Value' prenl.
        		'For itype, enter: ' prinl.
			'		0 - Linear Levels' prinl.
			'		1 - Log Levels' prinl.
			' 		2 - SqRt Levels' prinl. 
			' ' prnl.
        		 itype _ DefaultIType]]]
ifTrue:[DefaultIType _ itype]
!!
nil
!


! TwoD methodsFor: 'private' !

initTwoD

	itype _ DefaultIType
!!
nil
!


TwoD subclass:#TwoDContour
	instanceVariableNames: ' values contours ncont lineStyle'
	classVariableNames: ' DefaultNCont DefaultLineStyle 
			      DefaultContours DefaultValues' 
	poolDictionaries: ' '
	category: nil

!
nil
!


! TwoDContour class methodsFor: 'instance creation' !

new
	^super new initTwoDContour
!

defaultValues
		^DefaultValues
!

defaultValues: aVa
		DefaultValues _ aVa
!

defaultNCont
		^DefaultNCont
!

defaultNCont: aNC
		DefaultNCont _ aNC
!

defaultContours
		^DefaultContours
!

defaultContours: aCo
		DefaultContours _ aCo
!

defaultLineStyle
		^DefaultLineStyle
!

defaultLineStyle: aLS
		DefaultLineStyle _ aLS

!!
nil
!
TwoDContour defaultValues: nil!
TwoDContour defaultNCont: 5!
TwoDContour defaultContours: nil!
TwoDContour defaultLineStyle: 1!


! TwoDContour methodsFor: 'ncont' !

ncont
	^ncont

!

ncont: aNC
	ncont _ aNC.

(ncont class = Integer)
ifFalse:[(ncont = nil)
	ifTrue:[ ncont _ DefaultNCont]
	ifFalse:['NCONT ERROR, using default value' prenl.
	 	 'For ncont, enter an Integer' prinl.
	 	 ncont _ DefaultNCont]]
ifTrue:[DefaultNCont _ ncont]
!!
nil
!


! TwoDContour methodsFor: ' values' !

values
	^values
!

values: aVs
	values _ aVs.

(values class ~= GXRealarray)
ifTrue:[ (values = nil)
	ifTrue:[values _ DefaultValues]
	ifFalse:['VALUES ERROR,' prenl.
	 	 'For values enter an array of real numbers' prinl.
	 	 values _ DefaultValues]]

!!
nil
!

! TwoDContour methodsFor: 'contours' !

contours
	^contours
!

contours: aCo
	contours _ aCo.

(contours class = GXRealarray)
ifFalse:[(contours = nil)
	ifTrue:[contours _ DefaultContours]
	ifFalse:['CONTOURS ERROR, using default value' prenl.
	 	 'For contours enter a Real array' prinl.
	 	 contours _ DefaultContours]]
ifTrue:[DefaultContours _ contours]

!!
nil
!


! TwoDContour methodsFor: 'lineStyle' !

lineStyle
	^lineStyle
!

lineStyle: aLS
	lineStyle _ aLS.

(lineStyle = nil)
ifTrue:[lineStyle _ DefaultLineStyle].

((lineStyle = 1) |
 (lineStyle = 2) |
 (lineStyle = 3) |
 (lineStyle = 4) )
ifFalse:['LINESTYLE ERROR, Using Default Value' prenl.
	 'For lineStyle enter 1 or 2' prinl.
	 lineStyle _ DefaultLineStyle]
!!
nil
!


! TwoDContour methodsFor: 'private' !

initTwoDContour
	
	values _ DefaultValues.

	ncont _ DefaultNCont.

	contours _ DefaultContours.

	lineStyle _ DefaultLineStyle

!!
nil
!


TwoD subclass:#TwoDCell
	instanceVariableNames: 'values valMax valMin  
				table type '
	classVariableNames: 'DefaultValMin DefaultValMax
			     DefaultType DefaultValues
			     DefaultTable'
	poolDictionaries: ' '
	category: nil

!
nil
!


! TwoDCell class methodsFor: 'instance creation' !

new
	^super new initTwoDCell
!

defaultValMin
		^DefaultValMin
!

defaultValMin: aVi
		DefaultValMin _ aVi
!

defaultValMax
		^DefaultValMax
!

defaultValMax: aVa
		DefaultValMax _ aVa
!

defaultType
		^DefaultType
!

defaultType: aTs
		DefaultType _ aTs
!

defaultValues
		^DefaultValues
!

defaultValues: aVa
		DefaultValues _ aVa
!

defaultTable
		^DefaultTable
!

defaultTable: aTa
		DefaultTable _ aTa
!!
nil
!
TwoDCell defaultValMin: nil!
TwoDCell defaultValMax: nil!
TwoDCell defaultType: 1!
TwoDCell defaultValues: nil!
TwoDCell defaultTable: nil!


! TwoDCell methodsFor: 'values' !

values
	^values
!

values: ava
	values _ ava.

(values class ~= GXRealarray)
ifTrue:['VALUES ERROR.' prenl.
	'For, values enter a Real Array' prinl.
	values _ DefaultValues]

!!
nil
!


! TwoDCell methodsFor: 'valMin' !

valMin
	^valMin

!

valMin: aVi
	valMin _ aVi.

(valMin class ~= Float)
ifTrue:[(valMin = nil)
	ifTrue:[valMin _ DefaultValMin]
	ifFalse:['VALMIN ERROR, using Default Value' prenl.
		 'For valMin enter a Float' prinl.
		 valMin _ DefaultValMin]]

!!
nil
!


! TwoDCell methodsFor: 'valMax' !

valMax
	^valMax

!

valMax: aVa
	valMax _ aVa.

(valMax class ~= Float)
ifTrue:[(valMax = nil)
	ifTrue:[valMax _ DefaultValMax]
	ifFalse:['VALMAX ERROR, using Default Value' prenl.
        	 'For valMax enter a Float' prinl.
        	 valMax _ DefaultValMax]]

!!
nil
!


! TwoDCell methodsFor: 'type' !

type
	^type
!

type: aTs
	type _ aTs.

((type class ~= Integer) |
 ((type = 0) |
 (type = 1)) )
ifFalse:[(type = nil)
	ifTrue:[type _ DefaultType]
	ifFalse:['TYPE ERROR, Using Default Value' prenl.
        	 'For type, enter: ' prinl.
		 '		0 - Same colour for all of the Cell' prinl.
		 '		1 - Interpolate colour across the Cell' prinl. 
        	 type _ DefaultType]]
ifTrue:[DefaultType _ type]

!!
nil
!


! TwoDCell methodsFor: 'table' !

table
	^table
!

table: aTa
	table _ aTa.

(table class = GXColourtable)
ifFalse:['TABLE ERROR, ' prenl.
	'For table enter a colourtable (workspace)' prinl.
	table _ DefaultTable]
ifTrue:[DefaultTable _ table]
!!
nil
!


! TwoDCell methodsFor: 'private' !

initTwoDCell

	valMin _ DefaultValMin.

	valMax _ DefaultValMax.

	type _ DefaultType.

	values _ DefaultValues.

	table _ DefaultTable

!!
nil
!


TwoDContour subclass:#Contour
	instanceVariableNames: ' tr' 
	classVariableNames: ' DefaultTr ' 
	poolDictionaries: ' '
	category: nil

!
nil
!


! Contour class methodsFor: 'instance creation' !

new

	^super new initContour
!
defaultTr
		^DefaultTr
!

defaultTr: aTr
		DefaultTr _ aTr
!!
nil
!
Contour defaultTr: nil!


! Contour methodsFor: 'tr' !

tr
	^tr
!

tr: aTr
	tr _ aTr.

(tr class = GXRealarray)
ifFalse:[(tr = nil)
	ifTrue:[tr _ DefaultTr]
	ifFalse:['TR ERROR, using default value' prenl.
	 	 'For tr enter a Real array of size 6' prinl.
	 	 tr _ DefaultTr]]
ifTrue:[DefaultTr _ tr]

!!
nil
!


! Contour methodsFor: ' private' !

initContour
	
	tr _ DefaultTr.

!!
nil
!



!Contour methodsFor: 'create' !

create

objid _ newGraphics function: gxContourCreate parameters: ((Array new: 8)
			at: 1 put: values;
			at: 2 put: tr;
			at: 3 put: contours;
			at: 4 put: lineStyle;
			at: 5 put: colour;
			at: 6 put: lineThickness;
			at: 7 put: itype;
			at: 8 put: ncont;
			yourself)

!!
nil
!


! Contour methodsFor: 'printing' !

printOn: aStream

        | className |

        className _ self class name.
        aStream nextPutAll: 'An instance of '.
        aStream nextPutAll: (className copyFrom: 1 to: className size); nl.
        aStream nextPutAll: '  values: '.
        (values class = GXRealarray)
	ifFalse:[ aStream nextPutAll: '  Parameter has not been specified' 
		 printString; nl]
	ifTrue:[values printOn: aStream.
		 aStream nextPutAll: ' ' printString; nl]. 
	aStream nextPutAll: '  tr: '.
	(tr class = GXRealarray)
	ifFalse:[ aStream nextPutAll: '  Parameter has not been specified' 
		 printString; nl.]
	ifTrue: [tr printOn: aStream.
		  aStream nextPutAll: ' ' printString; nl].
	aStream nextPutAll: '  contours: '.
        (contours class = GXRealarray)
	ifFalse:[ aStream nextPutAll: '  Parameter has not been specified'
		 printString; nl.]
	ifTrue:[contours printOn: aStream.
		 aStream nextPutAll: ' ' printString; nl].
        aStream nextPutAll: '  lineStyle: '.
        aStream nextPutAll: lineStyle printString; nl.
        aStream nextPutAll: '  lineColour: '.
        aStream nextPutAll: colour printString; nl.
        aStream nextPutAll: '  lineThickness: '.
        aStream nextPutAll: lineThickness printString; nl.
        aStream nextPutAll: '  itype: '.
        aStream nextPutAll: itype printString; nl.
        aStream nextPutAll: '  ncont: '.
        aStream nextPutAll: ncont printString; nl
!!
nil
!


TwoDContour subclass:#ContourArray
        instanceVariableNames: 'wX wY' 
        classVariableNames: ' DefaultWX DefaultWY' 
        poolDictionaries: ' '
        category: nil

!
nil
!


! ContourArray class methodsFor: 'instance creation' !

new
        ^super new initContourArray
!

defaultWX
                ^DefaultWX
!

defaultWX: aWX
                DefaultWX _ aWX
!

defaultWY
                ^DefaultWY
!

defaultWY: aWY
                DefaultWY _ aWY
!!
nil
!
ContourArray defaultWX: nil!
ContourArray defaultWY: nil!


! ContourArray methodsFor: ' wX ' !

wX
        ^wX
!

wX: awX
        wX _ awX.

(wX class =GXRealarray)
ifFalse:['WX ERROR,' prenl.
        'For wX enter a real array' prinl.
        wX _ DefaultWX]

!!
nil
!


! ContourArray methodsFor: 'wY' !

wY
        ^wY
!

wY: awY
        wY _ awY.

(wY class =GXRealarray)
ifFalse:['WY ERROR,' prenl.
        'For wY enter a real array' prinl.
        wY _ DefaultWY]


!!
nil
!


! ContourArray methodsFor: 'objid' !

objid
	^objid
!!
nil
!


! ContourArray  methodsFor: 'private' !

initContourArray
        wX _ DefaultWX.

        wY _ DefaultWY

!!
nil
!


! ContourArray methodsFor: 'create' !

create

objid _  newGraphics function: gxContourCreateArray parameters: ((Array new: 9)
                        at: 1 put: values;
                        at: 2 put: wX;
                        at: 3 put: wY;
                        at: 4 put: contours;
                        at: 5 put: lineStyle;
                        at: 6 put: colour;
                        at: 7 put: lineThickness;
                        at: 8 put: itype;
                        at: 9 put: ncont;
                        yourself)
!!
nil
!


! ContourArray methodsFor: 'printing' !

printOn: aStream

        | className |

        className _ self class name.
        aStream nextPutAll: 'An instance of '.
        aStream nextPutAll: (className copyFrom: 1 to: className size); nl.
	aStream nextPutAll: '  values: '.
	(values class = GXRealarray)
        ifFalse:[ aStream nextPutAll: '  Parameter has not been specified'
                 printString; nl.]
        ifTrue:[values printOn: aStream.
		 aStream nextPutAll: ' ' printString; nl].
	aStream nextPutAll: '  wX: '.
	(wX class = GXRealarray)  
        ifFalse:[ aStream nextPutAll: '  Parameter has not been specified'  
                 printString; nl.]  	
	ifTrue:[wX printOn: aStream.
                 aStream nextPutAll: ' ' printString; nl].
	aStream nextPutAll: '  wY: '.
	(wY class = GXRealarray)          
        ifFalse:[ aStream nextPutAll: '  Parameter has not been specified'
                 printString; nl.]  	
	ifTrue:[wY printOn: aStream.
                 aStream nextPutAll: ' ' printString; nl].
	aStream nextPutAll: '  contours: '.
	(contours class = GXRealarray)
        ifFalse:[ aStream nextPutAll: '  Parameter has not been specified'
                 printString; nl]
        ifTrue:[contours printOn: aStream.
		 aStream nextPutAll: ' ' printString; nl].
	aStream nextPutAll: '  lineStyle: '.
	aStream nextPutAll: lineStyle printString; nl.
	aStream nextPutAll: '  lineColour: '.
	aStream nextPutAll: colour printString; nl.
	aStream nextPutAll: '  lineThickness: '.
	aStream nextPutAll: lineThickness printString; nl.
	aStream nextPutAll: '  itype: '.
	aStream nextPutAll: itype printString; nl.
	aStream nextPutAll: '  ncont: '.
	aStream nextPutAll: ncont printString; nl
!!
nil
!


TwoDContour subclass:#ContourFunction
        instanceVariableNames: 'funcPtr' 
        classVariableNames: ' DefaultFuncPtr'
        poolDictionaries: ' '
        category: nil

!
nil
!


! ContourFunction class methodsFor: 'instance creation' !

new
        ^super new initContourFunction
!

defaultFuncPtr
                ^DefaultFuncPtr
!

defaultFuncPtr: aFP
                DefaultFuncPtr _ aFP
!!
nil
!
ContourFunction defaultFuncPtr: nil!


! ContourFunction methodsFor: 'funcPtr' !

funcPtr
        ^funcPtr
!

funcPtr: aFP
	(aFP isKindOf: String) ifTrue: [ funcPtr _ aFP. ]
				ifFalse:['FUNCPTR ERROR, using default value' prenl.
        			    'For funcPtr, enter a string (after creating a function in fortran)' prinl.
        			    funcPtr _ DefaultFuncPtr]
!!
nil
!


! ContourFunction methodsFor: 'private' !

initContourFunction

        funcPtr _ DefaultFuncPtr
!!
nil
!


! ContourFunction methodsFor: 'create' !

create

objid _ newGraphics function: gxContourCreateFunction parameters: (
                        (Array new: 8)
                        at: 1 put: values;
                        at: 2 put: funcPtr;
                        at: 3 put: contours;
                        at: 4 put: lineStyle;
                        at: 5 put: colour;
                        at: 6 put: lineThickness;
                        at: 7 put: itype;
                        at: 8 put: ncont;
                        yourself)

!!
nil
!


! ContourFunction methodsFor: 'printing' !

printOn: aStream

        | className |

        className _ self class name.
        aStream nextPutAll: 'An instance of '.
        aStream nextPutAll: (className copyFrom: 1 to: className size); nl.
        aStream nextPutAll: '  values: '.
       	(values = nil)
        ifTrue:[ aStream nextPutAll: '  Parameter has not been specified'
                 printString; nl] 
	ifFalse:[values printOn: aStream.
		 aStream nextPutAll: ' ' printString; nl].
	aStream nextPutAll: '  funcPtr: '.
	aStream nextPutAll: funcPtr printString; nl.
        aStream nextPutAll: '  contours: '.
       	(contours = nil)
        ifTrue:[ aStream nextPutAll: '  Parameter has not been specified'
                 printString; nl] 
	ifFalse:[contours printOn: aStream.
		 aStream nextPutAll: ' ' printString; nl].
        aStream nextPutAll: '  lineStyle: '.
        aStream nextPutAll: lineStyle printString; nl.
        aStream nextPutAll: '  lineColour: '.
        aStream nextPutAll: colour printString; nl.
        aStream nextPutAll: '  lineThickness: '.
        aStream nextPutAll: lineThickness printString; nl.
        aStream nextPutAll: '  itype: '.
        aStream nextPutAll: itype printString; nl.
        aStream nextPutAll: '  ncont: '.
        aStream nextPutAll: ncont printString; nl
!!
nil
!


TwoDContour subclass:#ContourLabel
	instanceVariableNames: ' words start end step intval minint 
				 font height contourName contourid'
	classVariableNames: ' DefaultFont DefaultHeight DefaultStart 
			      DefaultEnd DefaultStep DefaultIntVal 
			      DefaultMinInt DefaultWords DefaultValues'
	poolDictionaries: ' '
	category: nil

!
nil
!


! ContourLabel class methodsFor: 'instance creation' !

new

	^super new initContourLabel
!

defaultFont
		^DefaultFont
!

defaultFont: aFo
		DefaultFont _ aFo
!

defaultHeight
		^DefaultHeight
!

defaultHeight: aHe
		DefaultHeight _ aHe
!

defaultStart
		^DefaultStart
!

defaultStart: aSt
		DefaultStart _ aSt
!

defaultEnd
		^DefaultEnd
!

defaultEnd: aEn
		DefaultEnd _ aEn
!

defaultStep
		^DefaultStep
!

defaultStep: aSt
		^DefaultStep _ aSt
!

defaultIntVal
		^DefaultIntVal
!

defaultIntVal: aIV
		DefaultIntVal _ aIV
!

defaultMinInt
		^DefaultMinInt
!

defaultMinInt: aMI
		DefaultMinInt _ aMI
!

defaultWords
		^DefaultWords
!

defaultWords: aWo
		DefaultWords _ aWo
!!
nil
!
ContourLabel defaultFont: 1!
ContourLabel defaultHeight: 0.5!
ContourLabel defaultStart: 1!
ContourLabel defaultEnd: -1!
ContourLabel defaultStep: 1!
ContourLabel defaultIntVal: 5!
ContourLabel defaultMinInt: 2!
ContourLabel defaultWords: '%d' !


! ContourLabel methodsFor: 'words' !

words
	^words

!

words: aWo
	words _ aWo.

(words class = String)
ifFalse:[ 'WORDS ERROR, using default value' prenl.
	 'For words, enter a string' prinl.
	 words _ DefaultWords]

!!
nil
!


! ContourLabel methodsFor: 'start' !

start
	^start
!

start: aSt
	start _ aSt.

(start = nil)
ifTrue:[start _ DefaultStart].
(start class = Integer)
ifFalse:['START ERROR, Using default value' prenl.
	'For start, enter an integer' prinl.
	start _ DefaultStart]

!!
nil
!


! ContourLabel methodsFor: 'end' !

end
	^end
!

end: aEn
	end _ aEn.

(end = nil)
ifTrue:[end _ DefaultEnd].
(end class = Integer)
ifFalse:['END ERROR, Using default value' prenl. 
	'For end, enter an integer' prinl.
	end _ DefaultEnd]

!!
nil
!


! ContourLabel methodsFor: 'step' !

step
	^step
!

step: aSt
	step _ aSt.
(step = nil)
ifTrue:[step _ DefaultStep].
(step class = Integer)
ifFalse:['STEP ERROR, Using default value' prenl.
        'For step, enter an integer' prinl.
        step _ DefaultStep]


!!
nil
!


! ContourLabel methodsFor: 'intval' !

intval
	^intval
!

intval: aIV
	intval _ aIV.
(intval = nil)
ifTrue:[intval _ DefaultIntVal].
(intval class = Integer)
ifFalse:['INTVAL ERROR, Using default value' prenl.
        'For intval, enter an integer' prinl.
        intval _ DefaultIntVal]
!!
nil
!


! ContourLabel methodsFor: 'minint' !

minint
	^minint
!

minint: aMI
	minint _ aMI.
(minint = nil)
ifTrue:[minint _ DefaultMinInt].
(minint class = Integer)
ifFalse:['MININT ERROR, Using default value' prenl.
        'For minint, enter an integer' prinl.
        minint _ DefaultMinInt]
!!
nil
!


! ContourLabel methodsFor: 'font' !

font
	^font
!

font: aFo
	font _ aFo.
(font = nil)
ifTrue:[font _ DefaultFont].
(font class = Integer)
ifFalse:['FONT ERROR, Using default value' prenl.
	' For, font enter a number between 1 and 4' prinl.
	font _ DefaultFont]
!!
nil
!


! ContourLabel methodsFor: 'height' !

height
	^height
!

height: aHe
	height _ aHe.
(height = nil)
ifTrue:[height _ DefaultHeight].
(height class = Float)
ifFalse:['HEIGHT ERROR, Using default value' prenl.
	'For height enter a float' prinl.
	height _ DefaultHeight]
!!
nil
!


! ContourLabel methodsFor: 'contourName' !

contourName	
	^contourName
!

contourName: aName
	contourName _ aName.

(contourName isKindOf: TwoDContour) 
ifFalse:['CONTOURNAME ERROR' prenl.
	'For contourName enter the name of the contour array/function
	 to enable creation of labels' prinl.
	contourName _ nil]

ifTrue:[(contourName isNil)
	ifFalse:[contourid _ contourName objid]].

!!
nil
!

! ContourLabel methodsFor: 'contourid' !

contourid
	^contourid
!
contourid: anID
	contourid _ anID
!!
nil
!


! ContourLabel methodsFor: 'private' !

initContourLabel
	
	font _ DefaultFont.

	height _ DefaultHeight.

	words _ DefaultWords.

	start _ DefaultStart.

	end _ DefaultEnd.

	step _ DefaultStep.

	intval _ DefaultIntVal.

	minint _ DefaultMinInt.

!!
nil
!


! ContourLabel methodsFor: 'create' !

create

objid _ newGraphics function: gxContourCreateLabel parameters: ((Array new: 11)
			at: 1 put: words ;
			at: 2 put: start;
			at: 3 put: end;
			at: 4 put: step;
			at: 5 put: intval;
			at: 6 put: minint;
			at: 7 put: colour;
			at: 8 put: font;
			at: 9 put: height;
			at: 10 put: lineThickness;
			at: 11 put: contourid;
			yourself).

!!
nil
!


! ContourLabel methodsFor: ' printing' !

printOn: aStream

        | className |

        className _ self class name.
        aStream nextPutAll: 'An instance of '.
        aStream nextPutAll: (className copyFrom: 1 to: className size); nl.
	aStream nextPutAll: '  words: '.
	aStream nextPutAll: words printString; nl.
        aStream nextPutAll: '  start: '.
	aStream nextPutAll: start printString; nl.
	aStream nextPutAll: '  end: '.
	aStream nextPutAll: end printString; nl.
	aStream nextPutAll: '  step: '.
        aStream nextPutAll: step printString; nl.
	aStream nextPutAll: '  intval: '.
	aStream nextPutAll: intval printString; nl.
	aStream nextPutAll: '  minint: '.
	aStream nextPutAll: minint printString; nl.
	aStream nextPutAll: '  lineColour: '.
        aStream nextPutAll: colour printString; nl.
	aStream nextPutAll: '  font: '.
	aStream nextPutAll: font printString; nl.
	aStream nextPutAll: '  height: '.
	aStream nextPutAll: height printString; nl.
	aStream nextPutAll: '  lineThickness: '.
        aStream nextPutAll: lineThickness printString; nl
!!
nil
!


TwoDCell subclass:#Cell
	instanceVariableNames: ' tr' 
	classVariableNames: ' DefaultTr ' 
	poolDictionaries: ' '
	category: nil

!
nil
!


! Cell class methodsFor: 'instance creation' !

new

	^super new initCell
!
defaultTr
		^DefaultTr
!

defaultTr: aTr
		DefaultTr _ aTr
!!
nil
!
Cell defaultTr: nil!


! Cell methodsFor: 'tr' !

tr
	^tr
!

tr: aTr
	tr _ aTr.

(tr class = GXRealarray)
ifFalse:[(tr = nil)
	ifTrue:[tr _ DefaultTr]
	ifFalse:['TR ERROR, using default value' prenl.
	 	 'For tr enter a Real array of size 6' prinl.
	 	 tr _ DefaultTr]]
ifTrue:[DefaultTr _ tr]

!!
nil
!


! Cell methodsFor: ' private' !

initCell
	
	tr _ DefaultTr.

!!
nil
!



!Cell methodsFor: 'create' !

create

objid _ newGraphics function: gxCellCreate parameters: ((Array new: 7)
			at: 1 put: values;
			at: 2 put: tr;
			at: 3 put: valMax;
			at: 4 put: valMin;
			at: 5 put: itype;
			at: 6 put: type;
			at: 7 put: table;
			yourself)

!!
nil
!


TwoDCell subclass: #CellArray
	instanceVariableNames: 'wX wY '
	classVariableNames: 'DefaultWX DefaultWY' 
	poolDictionaries: ' '
	category: nil
!
nil
!


! CellArray class methodsFor: 'instance creation' !

new
        ^super new initCellArray
!

defaultWX
                ^DefaultWX
!

defaultWX: aWX
                DefaultWX _ aWX
!

defaultWY
                ^DefaultWY
!

defaultWY: aWY
                DefaultWY _ aWY

!!
nil
!
CellArray defaultWX: nil!
CellArray defaultWY: nil!


! CellArray methodsFor: 'wX' !

wX
        ^wX
!

wX: aWX
        wX _ aWX.

(wX class = GXRealarray)
ifFalse:['WX ERROR.' prenl.
        'For, wX enter a Real Array' prinl.
        wX _ DefaultWX]


!!
nil
!


! CellArray methodsFor: 'wY' !

wY
        ^wY
!

wY: aWY
        wY _ aWY.

(wY class = GXRealarray)
ifFalse:['WY ERROR.' prenl.
        'For, wY enter a Real Array' prinl.
        wY _ DefaultWY]
!!
nil
!


! CellArray methodsFor: 'objid' !

objid
	^objid
!!
nil
!


! CellArray methodsFor: 'private' !

initCellArray

        wX _ DefaultWX.

        wY _ DefaultWY.
!!
nil
!


! CellArray methodsFor: 'create' !

create
objid _ newGraphics function: gxCellArrayCreate parameters: ((Array new: 8)
                        at: 1 put: values;
                        at: 2 put: wX;
                        at: 3 put: wY;
                        at: 4 put: valMax;
                        at: 5 put: valMin;
                        at: 6 put: itype;
                        at: 7 put: type;
                        at: 8 put: table;
                        yourself)

!!
nil
!


! CellArray methodsFor: 'printing' !

printOn: aStream

        | className |

        className _ self class name.
        aStream nextPutAll: 'An instance of '.
        aStream nextPutAll: (className copyFrom: 1 to: className size); nl.
        aStream nextPutAll: '  values: '.
        (values class = GXRealarray)
        ifFalse:[ aStream nextPutAll: '  Parameter has not been specified'
                 printString; nl]
        ifTrue:[values printOn: aStream.
		 aStream nextPutAll: ' ' printString; nl].
        aStream nextPutAll: '  wX: '.
        (wX class = GXRealarray)
        ifFalse:[ aStream nextPutAll: '  Parameter has not been specified'
                 printString; nl.]
        ifTrue:[wX printOn: aStream.
                 aStream nextPutAll: ' ' printString; nl].
        aStream nextPutAll: '  wY: '.
        (wX class ~= GXRealarray)
        ifTrue:[ aStream nextPutAll: '  Parameter has not been specified'
                 printString; nl]
        ifFalse:[wY printOn: aStream.
                 aStream nextPutAll: ' ' printString; nl].
	aStream nextPutAll: '  valMax: '.
	aStream nextPutAll: valMax printString; nl.
	aStream nextPutAll: '  valMin: '.
	aStream nextPutAll: valMin printString; nl.
	aStream nextPutAll: '  iType: '.
	aStream nextPutAll: itype printString; nl.
	aStream nextPutAll: '  type: '.
	aStream nextPutAll: type printString; nl.
	aStream nextPutAll: '  table: '.
	aStream nextPutAll: table printString; nl

!!
nil
!


TwoDCell subclass:#CellFunction
	instanceVariableNames: 'funcPtr'  
	classVariableNames: 'DefaultfuncPtr'
	poolDictionaries: ' '
	category: nil

!
nil
!


! CellFunction class methodsFor: 'instance creation' !

new
	^super new initCellFunction
!

defaultfuncPtr
		^DefaultfuncPtr
!

defaultfuncPtr: aFP
		DefaultfuncPtr _ aFP

!!
nil
!
CellFunction defaultfuncPtr: nil!


! CellFunction methodsFor: 'funcPtr' !

funcPtr
	^funcPtr

!

funcPtr: aFP
	(aFP isKindOf: String) ifTrue: [ funcPtr _ aFP. ]
				ifFalse:['FUNCPTR ERROR.' prenl.  'For, funcPtr enter a String ' prinl.  funcPtr _ DefaultfuncPtr]
!!
nil
!


! CellFunction methodsFor: 'private' !

initCellFunction

	funcPtr _ DefaultfuncPtr

!!
nil
!


! CellFunction methodsFor: 'create' !

create

objid _ newGraphics function: gxCellFunctionCreate parameters: ((Array new: 7)
			at: 1 put: values;
			at: 2 put: valMax;
			at: 3 put: valMin;
			at: 4 put: funcPtr;	
			at: 5 put: itype;
			at: 6 put: type;
			at: 7 put: table;
			yourself)

!!
nil
!


! CellFunction methodsFor: 'printing' !

printOn: aStream

        | className |

        className _ self class name.
        aStream nextPutAll: 'An instance of '.
        aStream nextPutAll: (className copyFrom: 1 to: className size); nl.
        aStream nextPutAll: '  values: '.
        (values class = GXRealarray)
        ifFalse:[ aStream nextPutAll: '  Parameter has not been specified'
                 printString; nl.]
	ifTrue:[(aStream print: (values do:[: i | i printString]); nl)].	
	aStream nextPutAll: '  funcPtr: '.
	aStream nextPutAll: funcPtr printString; nl.
	aStream nextPutAll: '  valMax: '.
        aStream nextPutAll: valMax printString; nl.
        aStream nextPutAll: '  valMin: '.
        aStream nextPutAll: valMin printString; nl.
        aStream nextPutAll: '  iType: '.
        aStream nextPutAll: itype printString; nl.
        aStream nextPutAll: '  type: '.
        aStream nextPutAll: type printString; nl.
        aStream nextPutAll: '  table: '.
        aStream nextPutAll: table printString; nl

!!
nil
!


TwoD subclass:#CellWedge
        instanceVariableNames: ' objnum iMode cellName cellid pictName cellPic
				 cName'
        classVariableNames: ' DefaultIMode DefaultPictName DefaultCellName'
        poolDictionaries: ' '
        category: nil

!
nil
!


! CellWedge class methodsFor: 'instance creation' !

new
	^super new initCellWedge
!

defaultIMode
                ^DefaultIMode
!

defaultIMode: aIM
                DefaultIMode _ aIM
!

defaultCellName
		^DefaultCellName
!

defaultCellName: aName
		DefaultCellName _ aName

!!
nil
!
CellWedge defaultIMode: 0!


! CellWedge methodsFor: 'cellid' !

cellid
	^cellid
!
cellid: anID
	cellid _ anID.
!!
nil
!


! CellWedge methodsFor: 'iMode' !

iMode
        ^iMode
!

iMode: aIM
        iMode _ aIM.

(iMode = nil)
ifTrue:[iMode _ DefaultIMode].
(iMode class ~= Integer)
ifTrue:['IMODE ERROR, Using Default Value' prenl.
        'For iMode, enter an integer' prinl.
        iMode _ DefaultIMode]


!!
nil
!


! CellWedge methodsFor: 'cellName' !

cellName
	^cellName
!
cellName: aName
	cellName _ aName.
(cellName isKindOf: TwoDCell) 
ifFalse:['CELLNAME ERROR, ' prenl.
	 'The name given does not belong to a cell' prinl.
	 cellName _ nil]
ifTrue:[(cellName isNil)
	ifFalse:[cellid _ cellName objid]]

!!
nil
!


! CellWedge methodsFor: 'private' !

initCellWedge

        iMode _ DefaultIMode.

	cellName _ DefaultCellName
!!
nil
!


! CellWedge methodsFor: 'create' !

create

(cellid isNil)
ifTrue:['ERROR cannot draw wedge' prenl.
	'A cell has not been specified' prinl]
ifFalse:[	
objid _ newGraphics function: gxCellWedgeCreate parameters: ((Array new: 3)
                        at: 1 put: cellid;
                        at: 2 put: iMode;
                        yourself) ]




!!
nil
!


! CellWedge methodsFor: 'printing' !

printOn: aStream

        | className |

        className _ self class name.
        aStream nextPutAll: 'An instance of '.
        aStream nextPutAll: (className copyFrom: 1 to: className size); nl.
	aStream nextPutAll: '  iMode: '.
	aStream nextPutAll: iMode printString; nl
!!
nil
!


DrawableObjects subclass:#Window
        instanceVariableNames: 'xMin xMax winColour yMin yMax displayList
			        undraw lxMin lxMax lyMin lyMax alterClass
				getNumber getObject'
	classVariableNames: 'DefaultXMin DefaultXMax DefaultWinColour
        		     DefaultYMin DefaultYMax DefaultUndraw
			     DefaultLXMin DefaultLXMax DefaultLYMin
			     DefaultLYMax'
	poolDictionaries: ' '
	category: nil
!
nil
!


! Window class methodsFor: 'instanceCreation' !

new
	^super new initWindow 
!

defaultXMin
		^DefaultXMin
!

defaultXMin: XMi
		DefaultXMin _ XMi
!

defaultXMax
		^DefaultXMax
!

defaultXMax: XMa
		DefaultXMax _ XMa
!

defaultWinColour
 		^DefaultWinColour
!

defaultWinColour: aWC
		DefaultWinColour _ aWC
!

defaultYMin
		^DefaultYMin
!

defaultYMin: YMi
		^DefaultYMin _ YMi
!

defaultYMax
                ^DefaultYMax
!

defaultYMax: YMa
                DefaultYMax _ YMa
!

defaultUndraw
		^DefaultUndraw
!

defaultUndraw: aDU
		DefaultUndraw _ aDU
!

defaultLXMin
                ^DefaultLXMin
!

defaultLXMin: aXM
                DefaultLXMin _ aXM
!

defaultLXMax
                ^DefaultLXMax
!

defaultLXMax: aXMa
                DefaultLXMax _ aXMa
!

defaultLYMin
                ^DefaultLYMin
!

defaultLYMin: aYM
                DefaultLYMin _ aYM
!

defaultLYMax
                ^DefaultLYMax
!

defaultLYMax: aYMa
                DefaultLYMax _ aYMa
!!
nil
!
Window defaultXMin: 0.0!
Window defaultXMax: 1.0!
Window defaultWinColour: gXDblack!
Window defaultYMin: 0.0!
Window defaultYMax: 1.0!
Window defaultUndraw: nil!
Window defaultLXMin: nil!
Window defaultLXMax: nil!
Window defaultLYMin: nil!
Window defaultLYMax: nil!


! Window methodsFor: 'xMin' !

xMin
	^xMin

!

xMin: XMi
	xMin _ XMi.

(xMin class ~= Float)
ifTrue:['XMIN ERROR, Using Default Value' prenl.
        'For xMin, enter a Float' prinl.
        xMin _ DefaultXMin]

!!
nil
!


! Window methodsFor: 'xMax' !

xMax
	^xMax

!

xMax: XMa
	xMax _ XMa.

(xMax class ~= Float)
ifTrue:['XMAX ERROR, Using Default Value' prenl.
	'For xMax, enter a Float' prinl.
	xMax _ DefaultXMax]
!!
nil
!


! Window methodsFor: 'winColour' !

winColour 
	^winColour

!

winColour: aWC
	winColour _ aWC.

(winColour isKindOf: Symbol)
ifFalse:[(winColour = nil)
	ifTrue:[winColour _ DefaultWinColour]
	ifFalse:['WINCOLOUR ERROR, Using Default Value' prenl.
		winColour _ DefaultWinColour]]

!!
nil
!


! Window methodsFor: 'yMin' !

yMin
	^yMin

!

yMin: YMi
	yMin _ YMi.

(yMin class ~= Float)
ifTrue:['YMIN ERROR, Using Default Value' prenl.
        'For yMin, enter a Float' prinl.
        yMin _ DefaultYMin]

!!
nil
!


! Window methodsFor: 'yMax' !

yMax
	^yMax

!

yMax: YMa
	yMax _ YMa.

(yMax class ~= Float)
ifTrue:['YMAX ERROR, Using Default Value' prenl.
        'For yMax, enter a Float' prinl.
        yMax _ DefaultYMax]

!!
nil
!


! Window methodsFor: 'lxMin' !

lxMin
        ^lxMin
!

lxMin: XMi
        lxMin _ XMi.

(lxMin class ~= Float)
ifTrue:[ (lxMin = nil)
	ifTrue:[lxMin _ DefaultLXMin]
	ifFalse:['LXMIN ERROR, ' prenl.
         	 'For lxMin enter a float' prinl.
         	 lxMin _ DefaultLXMin]]
!!
nil
!


! Window methodsFor: 'lxMax' !

lxMax
        ^lxMax
!

lxMax: XMa
        lxMax _ XMa.

(lxMax class ~= Float)
ifTrue:[(lxMax = nil)
	ifTrue:[lxMax _ DefaultLXMax]
	ifFalse:['LXMAX ERROR, ' prenl.
	 	 'For lxMax enter a float' prinl.
         	 lxMax _ DefaultLXMax]]  
!!
nil
!


! Window methodsFor: 'lyMin' !

lyMin
        ^lyMin

!

lyMin: YMi
        lyMin _ YMi.

(lyMin class ~= Float)
ifTrue:[(lyMin = nil)
	ifTrue:[lyMin _ DefaultLYMin]
	ifFalse:['LYMIN ERROR, ' prenl.
         	 'For lyMin enter a float' prinl.
         	 lyMin _ DefaultLYMin]]
!!
nil
!


! Window methodsFor: 'lyMax' !

lyMax
        ^lyMax

!

lyMax: YMa
        lyMax _ YMa.

(lyMax class ~= Float)
ifTrue:[(lyMax = nil)
	ifTrue:[lyMax _ DefaultLYMax]
	ifFalse:['LYMAX ERROR, ' prenl.
         	 'For lyMax enter a float' prinl.
         	 lyMax _ DefaultLYMax]]
!!
nil
!


! Window methodsFor: 'printList' !
						" Displays the type of each "
printList
	displayList class prn.
	' (' prn.				" Link, i.e, 		    "
	displayList do:[:i |i class prn.
				' ' prn].
	' )' prnl				" LinkedList( Text  Text) "
			    

!!
nil
!


! Window methodsFor: 'displayList' !
displayList
	^displayList
!!
nil
!


! Window methodsFor: 'checkClass' !

checkClass: anObject
"
# comment this out as C++ actually does a conversion 
# between bin/point mode for us
#
#((anObject class == ContourFunction) |
# (anObject class == ContourArray)    |
# (anObject class == Contour)         |
# (anObject class == CellArray)       |
# (anObject class == CellFunction)    )
#
#ifTrue:[ ((anObject values size) == (((self x size) - 1)  * 
#				     ((self y size) - 1)))
#        ifFalse:[anObject class pren.
#                 ' ERROR' prenl.
#                 'Values MUST be true to: ((X - 1) * (Y - 1))  ' prinl.
#		 '	(where X, Y are the x & y arrays of TwoDWindow)' prinl.
#                 ^false]
#        ifTrue:[^true]]
#ifFalse:[^true]
"
	^true

!!
nil
!


! Window methodsFor: 'count' !

count: anObject

|h|
h_0.

displayList do:[: i| (i class = anObject)
			ifTrue:[h _  1 + h]
			ifFalse:[h _ h]].
^h

!!
nil
!


! Window methodsFor: 'draw' !			" Draws a specified object,   "
						" i.e, draw: 1, will draw     "
draw: integer					" the first object in the     "
						" list.			      "
|anObject tmp|					" The conditions which are    "

(integer class = Integer)			" set are:		      "
ifTrue:[					" 1)Make sure entered no. is  "
	(displayList isEmpty)		" an Integer                  "
						" 2)Make sure given number is " 
	ifTrue: [ 'DRAW ERROR' prenl.
		  'The List has no ' prin.	" within the List size        "
	   	  'drawable objects' prinl] 	" 3)Make sure given number  "
		   				"   is not hidden  	      "
	ifFalse:[ (displayList size >= integer)
		  ifTrue:[Device Clearing.
			  (self checkClass: (displayList at: integer))
			  ifTrue:[ (displayList at: integer) unhide.
                          	   (displayList at: integer) display]]
		  ifFalse:[ (displayList size = 1)
			    ifTrue:['DRAW ERROR' prenl.
				   'The list only has 1 drawable object' prinl]	
			    ifFalse:['DRAW ERROR' prenl.
				     'The List only has ' prin.
				     displayList size prin.
				     ' drawable objects' prinl]]]]

ifFalse:[ 'DRAW ERROR, ' pren.
	  'Enter an integer' prinl]


!!
nil
!


! Window methodsFor: 'drawAll' !

drawAll

|tmp|

"Device Clearing.			 Draws all of the objects in "
self display.	
(displayList isEmpty)				" the list		      "
ifTrue:['DRAWALL ERROR' prenl.
	'The list has no drawable objects' prinl]
ifFalse:[ displayList do: [: i | (self checkClass: i)
				 ifTrue:[ i unhide. 
				 	  i display]]]

!!
nil
!

					
! Window methodsFor: 'drawAllOnTop' !	
				
drawAllOnTop					 
		
displayList do: [: i | (self checkClass: i)
		       ifTrue:[ i unhide. 
		       		i display]] 
!!
nil
!


! Window methodsFor: 'drawUnhiddenOnTop' !

drawUnhiddenOnTop

displayList do: [: i | (self checkClass: i)
                       ifTrue:[i display]]
!!
nil
!


! Window methodsFor: 'drawAllUnflaggedObjects' !

drawAllUnflaggedObjects

displayList do: [: i | (self checkClass: i)
                       ifTrue:[ i display]]
!!
nil
!


! Window methodsFor: 'undraw' !

						" Undraws a specified object "	
undraw: integer					" i.e, undraw: 1, will undraw"
						" the first object in the    "
						" list.                      "

(integer class ~= Integer)			" The conditions which I have"	
ifFalse:[(displayList isEmpty)			" set are:		     "
						" 1)Make sure entered no. is  "	
  ifTrue:['UNDRAW ERROR,' prenl.
	  'The list has no drawable objects' prinl] 
  ifFalse:[(displayList size >= integer)		" an Integer                  "
						" 2)Make sure given number is "
      ifTrue:[(displayList at: integer) hide.	" within the List size        "
	      undraw _ integer. Device Clearing] 			
	    "  self display.
	      displayList do: [: i | i display]]
"
      ifFalse:[ (displayList size = 1)	
        					
          ifTrue: ['UNDRAW ERROR,' prenl. 
		   'The List only has ' prin.
                  displayList size prin.   
                  ' drawable object' prinl]
						
          ifFalse:['UNDRAW ERROR, ' prenl. 
		   'The List only has ' prin.
                  displayList size prin.	
                  ' drawable objects' prinl]]]]

ifTrue:['UNDRAW ERROR, ' pren.
	'Enter an integer' prinl]	
!!
nil
!


! Window methodsFor: 'drawObject' !

drawObject: anObject
(anObject isKindOf: DrawableObjects)
ifTrue:[displayList do:[: i| (i objid = anObject objid)
			     ifTrue:[(self checkClass: i)
				     ifTrue:[i unhide.
				     	     i display]]]]
!!
nil
!


! Window methodsFor: 'undrawObject' !

undrawObject: anObject
|h|
h=0.
(anObject isKindOf: DrawableObjects)
ifTrue:[displayList do:[: i | h _ h + 1. (i objid = anObject objid)
			      ifTrue:[i hide. undraw _ h.
				      ^nil]]]
!!
nil
!


! Window methodsFor: 'undrawAll' !
						" Undraws all of the objects "
undrawAll					" in the list (or which ever "
						" have been specified to be  "
displayList do:[: i | i hide]			" drawn			     "
!!
nil
!


! Window methodsFor: 'deleteObject' !

deleteObject: anObject
(anObject isKindOf: DrawableObjects)
ifTrue:[displayList do:[: i | (i objid = anObject objid)
                              ifTrue:[displayList remove: i ifAbsent:['oops' prenl].
                                      ^nil]]]

!!
nil
!


! Window methodsFor: 'deleteAll' !
						" Deletes all of the List "
deleteAll

Device Clearing.
displayList _ LinkedList new.

!!
nil
!


! Window methodsFor: 'drawOnTop' !
						" Annotates an Object on the "
drawOnTop: anInteger				" device 		     "

(anInteger class ~= Integer)                      
ifFalse:[  
	(displayList isEmpty)
	ifTrue:['DRAWONTOP ERROR' prenl.
		'The list has no drawable objects' prinl]
        
	ifFalse:[(displayList size >= anInteger)
		ifTrue:[(self checkClass: (displayList at: anInteger))
			ifTrue:[(displayList at: anInteger) unhide. 
				(displayList at: anInteger) display]]

		ifFalse:[ (displayList size = 1)

			ifTrue:[ 'DRAWONTOP ERROR' prenl.
				 'The List only has ' prin.
				displayList size prin.
				' drawable object' prinl]

			ifFalse:[ 'DRAWONTOP ERROR' prenl.
				 'The List only has ' prin.
				displayList size prin.
				' drawable objects' prinl]]]]
 
        
ifTrue:['DRAWONTOP ERROR, ' prenl.
	'Enter an integer' prinl]

!!
nil
!


! Window methodsFor: 'at' !
		
at: anInteger
	
(anInteger class ~= Integer)
ifFalse:[		
        (displayList isEmpty)
        ifTrue:['AT ERROR' prenl.
		'The list has no drawable objects' prinl]

        ifFalse:[(displayList size >= anInteger)
		 ifTrue:[ ^displayList at: anInteger]
		 ifFalse:['AT: ERROR, ' prenl.
			  'Enter an integer' prinl]]]

ifTrue:['AT: ERROR, ' prenl.
	'Enter an Integer' prinl]

!!
nil
!


! Window methodsFor: 'undoUndraw' !
							" Undoes the last     "
undoUndraw						" undraw command      "
(undraw class = Integer)				" Conditions are:     "
ifTrue:[Device Clearing.				" 1)Make sure undraw  "
	(displayList at: undraw) unhide.		"  has been used      "
	displayList do:[: i | i display]. 		"  previously without "
	undraw _ DefaultUndraw]				"  a mistake being    "
							"  made		      "
ifFalse:['UNDOUNDRAW ERROR' prenl.
	 'Nothing valid has been undrawn' prinl]

!!
nil
!


! Window methodsFor: 'displayListAt' !
							" Allows the user to  "
displayListAt: anInteger				" see the parameters  "
(anInteger class = Integer)
ifTrue:[
(displayList isEmpty)					" of a given drawable "
ifTrue:['DISPLAYLISTAT ERROR' prenl.	
	'The list has no drawable objects' prinl]	" object, i.e.	      "
							" To see the          "
ifFalse:[(displayList size >= anInteger)		" parameters of Text  "
	ifTrue:[(displayList at: anInteger) prnl]	" Window displayList" 
							"   At: 2 [where    "
	ifFalse:[(displayList size = 1)  		" Text is the 2nd     " 
                                        		" element of the      " 
              ifTrue: [ 'DISPLAYLISTAT ERROR' prenl. 
			'The List only has ' prin.	" Linked List]        "
                        displayList size prin.   
                        ' drawable object' prinl]       
                                              
              ifFalse:[ 'DISPLAYLISTAT ERROR ' prenl. 
			'The List only has ' prin.  
                        displayList size prin.   
                       ' drawable objects' prinl]]]]
ifFalse:['DISPLAYLISTAT ERROR' prenl.
	 'For displayListAt enter an integer' prinl]     
!!
nil
!


! Window methodsFor: 'getID' !

getID

|id|

^objid.
	
!!
nil
!


! Window methodsFor: 'hideAll' !

hideAll

self hide.
self displayList do:[: i|i hide]

!!
nil
!


! Window methodsFor: 'unhideAll' !

unhideAll

self unhide.
self displayList do:[: i|i unhide]

!!
nil
!
 
! Window methodsFor: 'alterClass' !
alterClass: anObject with: anything
|temp count number a|
count _ 0.
number _ 0.
temp _ 0.
a _ 0.

(anObject class = Integer)
ifTrue:[displayList do:[: i | count _ count + 1.
			      (i class = anything)
			      ifTrue:[a _ (a + 1).
				      (a = anObject)
				     ifTrue:[ temp _ 0.
					      number _ count.
					     ^(self at: number)]
				     ifFalse:[temp _ temp + 1]]].
(a < anObject)
ifTrue:['ALTER ERROR, there is not  ' pren.
                 anObject pren.
		' ' pren.
		anything pren.
                 ' in the window' prenl.
                 ^nil]]

ifFalse:[ displayList do:[: j | count _ count + 1.
         	             	(j class = anObject class)
               	       	        ifTrue:[ temp _ 0.
                       		        number _ count]
                      		ifFalse:[temp _ temp + 1]].
	 (temp = count)
	 ifTrue:['ALTER ERROR, there is not a ' pren.
	 	 anObject class pren.
        	 ' in the window' prenl.
		 ^nil]
	 ifFalse:[^(self at: number)]]

!!
nil
!


! Window methodsFor: 'alterTCLObject' !

alterTCLObject: anID
|count|
count _ 0.
displayList do: [:i | (anID = i objid)
		      ifTrue:[^i]
		      ifFalse:[count _ count + 1]].

(count = displayList size)
ifTrue:[^nil]
!!
nil
!

! Window methodsFor: 'findCell' !
findCell: anObject
|temp|
((displayList size < anObject) | (displayList size <= anObject negated))
ifTrue:['ERROR, ' prenl.
	'There are not that many objects in the list' prinl]
ifFalse:[(((displayList at: anObject) class = CellArray) |
	  ((displayList at: anObject) class = CellFunction) |
	  ((displayList at: anObject) class = Contour) |
	  ((displayList at: anObject) class = ContourArray) |
	  ((displayList at: anObject) class = ContourFunction) )
	 ifFalse:['ERROR, ' pren.
		  'The object specified is not a cell/contour' prinl.
		  ^nil]
	 ifTrue:[temp _ ((displayList at: anObject) objid).
		 ^temp]]
!!
nil
!


! Window methodsFor: 'getNumber' !

getNumber: aNumber

|tmp count|
count _ 0.

((aNumber = nil) |
 (aNumber = 0)   )
ifTrue:[(displayList isEmpty) 
	ifTrue:['ERROR' prenl.
		'There are no objects in this Window' prinl.
		^nil]
	ifFalse:[^(displayList size)]].

(aNumber class =  Integer)
ifTrue:[(aNumber < 0)
	ifTrue:[(displayList size = 1)
		ifTrue:['The list only has one element,' prinl.
			' enter 0 or nothing to point to the current object' 
			prinl. ^nil]
		ifFalse:[tmp _ (displayList size + aNumber).
			((tmp > displayList size) | ( tmp <= 0))
                	ifFalse:[^tmp]
			ifTrue:['ERROR, ' pren.
                        	'The object list does not have that many objects' prinl. ^nil]]].
	(aNumber > 0)
	ifTrue:[(aNumber > displayList size)
		ifFalse:[^aNumber]
		ifTrue:['ERROR, ' pren.
			'The object list does not have that many objects' prinl. ^nil]]]
ifFalse:[(aNumber objid class  = GXIntegerarray)
	 ifTrue:[displayList do:[: i | count _ count + 1.
				      (aNumber objid = i objid)
				      ifTrue:[^count]].
		(count = displayList size)
		ifTrue:['ERROR, no object with that name' prenl]]]
!!
nil
!


! Window methodsFor: 'getObject' !

getObject: objNum
|temp|

objNum isNil ifTrue: [ ^nil ].

((objNum > displayList size) | (objNum negated >= displayList size))
ifTrue:['ERROR, ' pren.
        'There are not that many objects in the list' prinl.
        ^nil]
ifFalse:[temp _ (displayList at: objNum).
         ^temp]

!!
nil
!


! Window methodsFor: 'GetObjectWithAnObjId' !

getObjectWithObjId: anObject
(anObject isKindOf: DrawableObjects)
ifTrue:[displayList do:[: i | (anObject objid = i objid)
			      ifTrue:[^i]].
	^nil]
!!
nil
!

! Window methodsFor: 'private' !

initWindow

	winColour _ DefaultWinColour.	

	xMin _ DefaultXMin.

	xMax _ DefaultXMax.

	colour _ DefaultColour.

	yMin _ DefaultYMin.

	yMax _ DefaultYMax.

	displayList _ LinkedList new.		"Allows the user to make a    "
						"LinkedList in separate parts "
	undraw _ DefaultUndraw.

	visible _ true.
		
	lxMin _ DefaultLXMin.

        lxMax _ DefaultLXMax.

        lyMin _ DefaultLYMin.

        lyMax _ DefaultLYMax	
!!
nil
!


! Window methodsFor: 'createUnscaled' !

create

|tmp|

tmp _ newGraphics function: gxNewUnScaledWindow parameters: ((Array new: 9)
                        at: 1 put: xMin;
                        at: 2 put: xMax;
                        at: 3 put: yMin;
                        at: 4 put: yMax;
			at: 5 put: winColour;
			at: 6 put: lxMin;
			at: 7 put: lxMax;
			at: 8 put: lyMin;
			at: 9 put: lyMax;
			yourself).

objid _ tmp at:#gXobjid.
lxMax _ 1.0.
lxMin _ 0.0.
lyMax _ 1.0.
lyMin _ 0.0

!!
nil
!


! Window methodsFor: 'printing' !
					" Lets the command 'prnl' & 'printNl' "
printOn: aStream			" print out the parameters, of the    "
					" object (AND NOTHING ELSE)	      "
	|className|

        className _ self class name.
        aStream nextPutAll: 'An instance of '.
        aStream nextPutAll: (className copyFrom: 1 to: className size); nl.
	aStream nextPutAll: '  xMin: '.
	aStream nextPutAll: xMin printString; nl.
	aStream nextPutAll: '  xMax: '.
	aStream nextPutAll: xMax printString; nl.
	aStream nextPutAll: '  yMin: '.
	aStream nextPutAll: yMin printString; nl.
	aStream nextPutAll: '  yMax: '.
	aStream nextPutAll: yMax printString; nl.
	aStream nextPutAll: '  colour: '.
	aStream nextPutAll: winColour printString; nl.
	aStream nextPutAll: '  lxMin: '.
	aStream nextPutAll: lxMin printString; nl.
	aStream nextPutAll: '  lxMax: '.
	aStream nextPutAll: lxMax printString; nl.
	aStream nextPutAll: '  lyMin: '.
	aStream nextPutAll: lyMin printString; nl.
	aStream nextPutAll: '  lyMax: '.
	aStream nextPutAll: lyMax printString; nl.
	aStream nextPutAll: '  objectList: '.	
     	aStream nextPutAll: ' LinkedList( '.	
	displayList  do:[: i| aStream print: ( i class printString).
			      aStream nextPutAll:'  ' printString ].
	aStream nextPutAll: ' )'; nl.

!!
nil
!


Window subclass:#WindowScaled
        instanceVariableNames: 'minMode maxMode earray xarray yarray' 
	classVariableNames: ' DefaultMinMode DefaultMaxMode DefaultXArray
			      DefaultYArray DefaultEArray'
        poolDictionaries: ' '
        category: nil
!
nil
!


! WindowScaled class methodsFor: 'instance creation' !

new
	^super new initWindowScaled
!

defaultMinMode
		^DefaultMinMode
!

defaultMinMode: MiM
		DefaultMinMode _ MiM
!

defaultMaxMode
		^DefaultMaxMode
!

defaultMaxMode: MaM
		DefaultMaxMode _ MaM
!
defaultXArray
		^DefaultXArray
!
defaultXArray: aXa
		DefaultXArray _ aXa
!
defaultYArray
		^DefaultYArray
!
defaultYArray: aYa
		DefaultYArray _ aYa
!
defaultEArray
		^DefaultEArray
!
defaultEArray: aEa
		DefaultEArray _ aEa
!!
nil
!
WindowScaled defaultMinMode: 0!
WindowScaled defaultMaxMode: 0!
WindowScaled defaultXArray: ((GXRealarray new: 2) extend: 0.0; yourself)!
WindowScaled defaultYArray: ((GXRealarray new: 2) extend: 0.0; yourself)!
WindowScaled defaultEArray: nil !


! WindowScaled methodsFor: ' minMode' !

minMode
	^minMode

!

minMode: MiM
	minMode _ MiM.

(minMode class ~= Integer)
ifTrue:[(minMode = nil)
	ifTrue:[minMode _ DefaultMinMode]
	ifFalse:['MINMODE ERROR, Using default value' prenl.
		 ' For minMode enter an integer between 0 and 4' prinl.
		 ' 	(0 = absolute values ' prinl.
 		 '	1 or 3 = +10%	' prinl.
		 '	2 or 3 = roundown ' prinl. 
		 '	4 = force zero) ' prinl.
		 minMode _ DefaultMinMode]]
 

!!
nil
!


! WindowScaled methodsFor: 'maxMode' !

maxMode
	^maxMode

!

maxMode: MaM
	maxMode _ MaM.

(maxMode class ~= Integer)
ifTrue:[(maxMode = nil)
	ifTrue:[maxMode _ DefaultMaxMode]
	ifFalse:['MAXMODE ERROR, Using default value' prenl.
        	 ' For maxMode enter an integer between 0 and 4' prinl.
        	 '      (0 = absolute values ' prinl.
        	 '       1 or 3 = +10%   ' prinl.
        	 '       2 or 3 = roundown ' prinl.
        	 '       4 = force zero) ' prinl.
        	 maxMode _ DefaultMaxMode]]


!!
nil
!


! WindowScaled methodsFor: 'xarray' !

xarray
	^xarray
!
xarray: aXa
	xarray _ aXa.
(xarray class = GXRealarray)
ifFalse:[(xarray = nil)
	 ifTrue:[xarray _ DefaultXArray]
	 ifFalse:['XARRAY ERROR' prenl.
	 	  'For EArray enter a real array' prinl.
	 	  xarray _ DefaultXArray]]
!!
nil
!


! WindowScaled methodsFor: 'yarray' !

yarray
	^yarray
!
yarray: aYa
	yarray _ aYa.
(yarray class = GXRealarray)
ifFalse:[(yarray = nil)
	 ifTrue:[yarray _ DefaultYArray]
	 ifFalse:['YARRAY ERROR' prenl.
	 	  'For YArray enter a real array' prinl.
	 	  yarray _ DefaultYArray]]
!!
nil
!


! WindowScaled methodsFor: 'earray' !

earray
	^earray
!
earray: aEa
	earray _ aEa.
(earray class = GXRealarray)
ifFalse:[(earray = nil)
	 ifTrue:[earray _ DefaultEArray]
	 ifFalse:['EARRAY ERROR' prenl.
	 	  'For EArray enter a real array' prinl.
	 	  earray _ DefaultEArray]]
!!
nil
!


! WindowScaled methodsFor: 'private' !

initWindowScaled

		minMode _ DefaultMinMode.

		maxMode _ DefaultMaxMode.

		lxMin _ DefaultLXMin.

		lxMax _ DefaultLXMax.

		lyMin _ DefaultLYMin.

		lyMax _ DefaultLYMax.

		xarray _ DefaultXArray.

		yarray _ DefaultYArray.

		earray _ DefaultEArray.
!!
nil
!


! WindowScaled methodsFor: 'create' !

create
	
objid _ newGraphics function: gxNewScaledWindow parameters: ((Array new: 14)
			at: 1 put: xMin;
			at: 2 put: yMin;
			at: 3 put: xMax;
			at: 4 put: yMax;
			at: 5 put: xarray;
			at: 6 put: yarray;
			at: 7 put: earray;
			at: 8 put: winColour;
			at: 9 put: minMode;
			at: 10 put: maxMode;
			at: 11 put: lxMin;
			at: 12 put: lyMin;
			at: 13 put: lxMax;
			at: 14 put: lyMax;
			yourself)


!!
nil
!


! WindowScaled methodsFor: 'printing' !

printOn: aStream

        |className|

        className _ self class name.
        aStream nextPutAll: 'An instance of '.
        aStream nextPutAll: (className copyFrom: 1 to: className size); nl.
        aStream nextPutAll: '  xMin: '. 
	aStream nextPutAll: xMin printString; nl.
        aStream nextPutAll: '  xMax: '. 
	aStream nextPutAll: xMax printString; nl.
        aStream nextPutAll: '  yMin: '. 
	aStream nextPutAll: yMin printString; nl.
        aStream nextPutAll: '  yMax: '.
        aStream nextPutAll: yMax printString; nl.
        aStream nextPutAll: '  colour: '.
        aStream nextPutAll: colour printString; nl.
	aStream nextPutAll: '  minMode: '.
	aStream nextPutAll: minMode printString; nl.
	aStream nextPutAll: '  maxMode: '. 
	aStream nextPutAll: maxMode printString; nl.
	aStream nextPutAll: '  lxMin: '.
	aStream nextPutAll: lxMin printString; nl.
	aStream nextPutAll: '  lxMax: '.
	aStream nextPutAll: lxMax printString; nl.
	aStream nextPutAll: '  lyMin: '.
	aStream nextPutAll: lyMin printString; nl.
	aStream nextPutAll: '  lyMax: '.
	aStream nextPutAll: lyMax printString; nl.
	aStream nextPutAll: '  ObjectList: '.
       aStream nextPutAll: ' LinkedList( '.
        displayList  do:[: i| aStream print: ( i class printString).
                              aStream nextPutAll:'  ' printString ].
        aStream nextPutAll: ' )'; nl

!!
nil
!


Window subclass: #TwoDWindow
        instanceVariableNames: 'x y '
        classVariableNames: 'DefaultX DefaultY '
        poolDictionaries: ' '
        category: nil
!
nil
!


! TwoDWindow class methodsFor: 'instance creation' !

new
	^super new initTwoDWindow
!

defaultX
		^DefaultX
!

defaultX: aX
		DefaultX _ aX
!

defaultY
		^DefaultY
!

defaultY: aY
		DefaultY _ aY
!!
nil
!
TwoDWindow defaultX: nil!
TwoDWindow defaultY: nil!


! TwoDWindow methodsFor: 'x' !

x
	^x
!

x: aX
	x _ aX.
"
(x ~= GXRealarray)
ifTrue:[ 'x ERROR,' prenl.
	 'For x enter a real array' prinl.
	 x _ DefaultX] "
!!
nil
!


! TwoDWindow methodsFor: 'y' !

y
	^y
!

y: aY
	y_ aY.
"
(y ~= GXRealarray)
ifTrue:[ 'Y ERROR,' prenl.
         'For y enter a real array' prinl.
         y _ DefaultY]
"
!!
nil
!


! TwoDWindow methodsFor: 'private' !

initTwoDWindow

	y _ DefaultY.

	x _ DefaultX.

!!
nil
!


! TwoDWindow methodsFor: 'create' !

create

|tmp|

tmp _ newGraphics function: gxCreateWindowTwoD parameters: ((Array new: 11)
			at: 1 put: xMin;
			at: 2 put: xMax;
			at: 3 put: yMin;
			at: 4 put: yMax;
			at: 5 put: x;
			at: 6 put: y;
			at: 7 put: winColour;
			at: 8 put: lxMin;
			at: 9 put: lxMax;
			at: 10 put: lyMin;
			at: 11 put: lyMax;	
			yourself). 

lxMin _ tmp at:#gXlxmin.
lxMax _ tmp at:#gXlxmax.
lyMin _ tmp at:#gXlymin.
lyMax _ tmp at:#gXlymax.
objid _ tmp at:#gXobjid.

!!
nil
!


! TwoDWindow methodsFor: 'printing' !
                                       " Lets the command 'prnl' & 'printNl' "
printOn: aStream                       " print out the parameters, of the    "
                                       " object (AND NOTHING ELSE)           "
        |className|

        className _ self class name.
        aStream nextPutAll: 'An instance of '.
        aStream nextPutAll: (className copyFrom: 1 to: className size); nl.
        aStream nextPutAll: '  xMin: '.
        aStream nextPutAll: xMin printString; nl.
        aStream nextPutAll: '  xMax: '.
        aStream nextPutAll: xMax printString; nl.
        aStream nextPutAll: '  yMin: '.
        aStream nextPutAll: yMin printString; nl.
        aStream nextPutAll: '  yMax: '.
        aStream nextPutAll: yMax printString; nl.
        aStream nextPutAll: '  colour: '.
        aStream nextPutAll: winColour printString; nl.
        aStream nextPutAll: '  objectList: '.
        aStream nextPutAll: ' LinkedList( '.
        displayList  do:[: i| aStream print: ( i class printString).
                              aStream nextPutAll:'  ' printString ].
        aStream nextPutAll: ' )'; nl

!!
nil
!


Window subclass:#MultiplotWindow
        instanceVariableNames: 'minMode maxMode'
        classVariableNames: 'DefaultMinMode DefaultMaxMode'
        poolDictionaries: ' '
        category: nil
!
nil
!


! MultiplotWindow class methodsFor: 'instance creation' !

new
	^super new initMultiplotWindow

!

defaultMinMode
		^DefaultMinMode

!

defaultMinMode: MiM
		DefaultMinMode _ MiM
!

defaultMaxMode
		^DefaultMaxMode
!

defaultMaxMode: MaM
		DefaultMaxMode _ MaM

!!
nil
!
MultiplotWindow defaultMinMode: 2!
MultiplotWindow defaultMaxMode: 2!


! MultiplotWindow methodsFor: 'minMode' !


minMode
        ^minMode

!

minMode: MiM
        minMode _ MiM

!!
nil
!


! MultiplotWindow methodsFor: 'maxMode' !


maxMode
        ^maxMode

!

maxMode: MaM
        maxMode _ MaM

!!
nil
!


! MultiplotWindow methodsFor: 'private' !

initMultiplotWindow

	minMode _ DefaultMinMode.

	maxMode _ DefaultMaxMode

!!
nil
!



! MultiplotWindow methodsFor: 'create' !

create
|tmp|
tmp _ newGraphics function: gxCreateMultiplotWindow parameters: 
							((Array new: 11)
			at: 1 put: xMin;
			at: 2 put: yMin;
			at: 3 put: xMax;
			at: 4 put: yMax;
			at: 5 put: winColour;
			at: 6 put: minMode;
			at: 7 put: maxMode;
			at: 8 put: lxMin;
			at: 9 put: lxMax;
			at: 10 put: lyMin;
			at: 11 put: lyMax;
			yourself).

lxMin _ tmp at:#gXlxmin.
lxMax _ tmp at:#gXlxmax.
lyMin _ tmp at:#gXlymin.
lyMax _ tmp at:#gXlymax.
objid _ tmp at:#gXobjid.

!!
nil
!


! MultiplotWindow methodsFor: 'printing' !

printOn: aStream

        |className|

        className _ self class name.
        aStream nextPutAll: 'An instance of '.
        aStream nextPutAll: (className copyFrom: 1 to: className size); nl.
        aStream nextPutAll: '  xMin: '.
        aStream nextPutAll: xMin printString; nl.
        aStream nextPutAll: '  xMax: '.
        aStream nextPutAll: xMax printString; nl.
        aStream nextPutAll: '  yMin: '.
        aStream nextPutAll: yMin printString; nl.
        aStream nextPutAll: '  yMax: '.
        aStream nextPutAll: yMax printString; nl.
        aStream nextPutAll: '  colour: '.
        aStream nextPutAll: winColour printString; nl.
	aStream nextPutAll: '  minMode: '.
	aStream nextPutAll: minMode printString; nl.
	aStream nextPutAll: '  maxMode: '.
	aStream nextPutAll: maxMode printString; nl.
        aStream nextPutAll: '  objectList: '.
        aStream nextPutAll: ' LinkedList( '.
        displayList  do:[: i| aStream print: ( i class printString).
                              aStream nextPutAll:'  ' printString ].
        aStream nextPutAll: ' )'; nl.

!!
nil
!


GraphicalObjects subclass:#Picture
        instanceVariableNames: 'windowList undrawWindow switchOn switchOff
				pictNum '
	classVariableNames: 'DefaultUndrawWindow'
	poolDictionaries: ' '
	category: nil
!
nil
!


! Picture class methodsFor: 'instanceCreation' !

new
		
	^super new initPicture
!

defaultUndrawWindow
		^DefaultUndrawWindow

!

defaultUndrawWindow: aUW
		DefaultUndrawWindow _ aUW
!!
nil
!
Picture defaultUndrawWindow: nil!

! Picture methodsFor: 'pictNum' !
pictNum
	^pictNum
!!
nil
!


! Picture methodsFor: 'printList' !
						" Displays the type of each "
printList
	windowList class prn.
        ' (' prn.                               " Link, i.e,                "
	windowList size prnl.
        windowList do:[:i |i prnl
			 i class prn.
                                ' ' prn].
        ' )' prnl                               " LinkedList( Text  Text) "
!!
nil
!


! Picture methodsFor: 'testAlter' !

testAlter: anObject and: aNumber with: any
|temp tmp p|
(windowList size >= aNumber)
ifTrue:[ (anObject class = Integer)
	ifFalse:[temp _ ((windowList at: aNumber) alterClass: anObject with: any)]
	ifTrue:[tmp _ ((windowList at: aNumber) getNumber: anObject).
		temp _ ((windowList at: aNumber) alterClass: tmp with: any)].
	^temp]
ifFalse:['ERROR' prenl.
	 'There are only ' prin.
	 windowList size prin.
	 ' windows in the list' prinl.
	 ^nil]
!!
nil
!


! Picture methodsFor: 'testTCLAlter' !

testTCLAlter: anID

|tmp count|
"tmp _ (GXIntegerarray new: 2).		FOR SOME REASON OBJID
tmp at: 1 put: anObject.		IS A STRING: THEREFORE
tmp at: 2 put: aWindow.			THIS IS NOT NEEDED !!!!
"
count _ 0.
windowList do:[: i| tmp _ (i alterTCLObject: anID).
		   (tmp isNil)
		   ifFalse:[ ^tmp]
		   ifTrue:[count _ count + 1]].
(count = windowList size)
ifTrue:[^nil]

!!
nil
!


! Picture methodsFor: 'getWindow' !

getWindow: winNum
|temp|

winNum isNil ifTrue: [ ^nil ].

((winNum > windowList size) | (winNum negated >= windowList size))
ifTrue:['ERROR, ' pren.
        'There are not that many windows in the list' prinl.
        ^nil]
ifFalse:[temp _ (windowList at: winNum).
         ^temp]

!!
nil
!

! Picture methodsFor: 'findWindow' !

findCellWindow: anObject and: aNumber
|temp|
temp _ ((windowList at: aNumber) findCell: anObject).
^temp
!!
nil
!


! Picture methodsFor: 'windowList' !
							" Allows the user to "
windowList: anInteger					" see the parameters "
							" of a given Window  "
(anInteger class ~= Integer)				" i.e, To see the    "
ifFalse:[(windowList isEmpty)				" parameters of win1 " 
        ifTrue:['windowList ERROR' prenl.
		'The list has no windows ' prinl]" %(Picture windowList "
							"   At: 1) [where    "
        ifFalse:[(windowList size >= anInteger)		" win1 is the 1st    "
		ifTrue:[(windowList at: anInteger) prnl ]	" element in the     "
							" Linked List]	     "
		ifFalse:[(windowList size = 1)       
                                                
              	ifTrue: ['windowList ERROR' prenl. 
			 'The List only has ' prin.
                         windowList size prin.   
                         ' window ' prinl]
                                                
              	ifFalse:['windowList ERROR' prenl. 
			 'The List only has ' prin.
                         windowList size prin.   
                        ' windows' prinl]]]]
                                                
ifTrue:['windowList ERROR, ' pren.
	'Enter an integer' prinl]

!!
nil
!


! Picture methodsFor: 'addToWindowList' !
						" Adds a Window to the list  "
addToWindowList: anObject 			" (the last Window is now the "
						" one entered)                "
((anObject class = Window) |			
 (anObject class = WindowScaled) |		" Makes sure that 'anObject'  "
 (anObject class = TwoDWindow) |		" has been created properly   "
 (anObject class = MultiplotWindow)) 
 		     ifFalse:['ADDTOLIST ERROR, ' prenl.
			     'a Window MUST be added' prinl ]
		     ifTrue:[ windowList add: anObject ].
^self

!!
nil
!


! Picture methodsFor: 'drawWindow' !

drawWindow: aWindow
(aWindow isKindOf: Window)
ifTrue:[windowList do:[: i | (i objid = aWindow objid)
			     ifTrue:[i unhide.
				     i display.
				     i drawAll]]]
!!
nil
!


! Picture methodsFor: 'drawWindowOnly' !

drawWindowOnly: aWindow
(aWindow isKindOf: Window)
ifTrue:[windowList do:[: i | (i objid = aWindow objid)
			     ifTrue:[i unhide.
				     i display]]]
!!
nil
!


! Picture methodsFor: 'undrawWindow' !

undrawWindow: aWindow

(aWindow isKindOf: Window)
ifTrue:[windowList do:[: i | (i objid = aWindow objid)
			     ifTrue:[i displayList do:[: j| j hide].
				     i hide.
				     ^nil]]]
!!
nil
!


! Picture methodsFor: 'drawAllWindows' !
						" Draws all of the Windows   "
drawAllWindows
| tmp |

(picture)					" in the List		     "
ifTrue:[Device Clearing.
		
windowList do: [: i | i unhide.
		    i display. 
		    i drawAllOnTop]]

ifFalse:['DRAWALLWINDOWS ERROR' prenl.
	 'The picture has been switched off' prenl]


!!
nil
!


! Picture methodsFor: 'drawUnhiddenWindows' !

drawUnhiddenWindows
|tmp|

(picture)                                       " in the List                "
ifTrue:[Device Clearing.

windowList do: [: i | i display.
                    i drawUnhiddenOnTop]]

ifFalse:['DRAWALLWINDOWS ERROR' prenl.
         'The picture has been switched off' prenl]
!!
nil
!


! Picture methodsFor: 'drawUnflaggedWindows' !

drawUnflaggedWindows

(picture)                      
ifTrue:[Device Clearing.
	windowList do: [: i | i display.
                          i drawAllUnflaggedObjects]]

ifFalse:['DRAWALLWINDOWS ERROR' prenl.
         'The picture has been switched off' prenl]


!!
nil
!


! Picture methodsFor: 'drawWindowOnTop' !

drawWindowOnTop: aNumber

(aNumber class = Integer)
ifFalse:['DRAWWINDOWONTOP ERROR' prenl.
         'Enter an Integer']

ifTrue:[(windowList isEmpty)
	ifTrue:['DRAWWINDOWONTOP ERROR' prenl.
	 	'The list has no drawable objects' prinl]

	ifFalse:[(windowList size >= aNumber)
		 ifFalse:[(windowList size = 1)
			  ifTrue:['DRAWWINDOWONTOP ERROR' prenl.
				  'The List only has 1 window' prin]

			  ifFalse:['DRAWWINDOWONTOP ERROR' prenl.
				   'The List only has ' prin.
				   windowList size prin.
				   ' windows' prinl]]

		 ifTrue:[(windowList at: aNumber) display.
			 (windowList at: aNumber) drawAllOnTop]]]

!!
nil
!


! Picture methodsFor: 'undrawAllWindows' !

undrawAllWindows

windowList do:[: i | i hide.
		   i undrawAll]

!!
nil
!


! Picture methodsFor: 'undoUndrawWindow' !
                                                        " Undoes the last     "
undoUndrawWindow                                        " undraw command      "
(undrawWindow class = Integer)                          " Conditions are:     "
ifTrue:[Device Clearing.                                "1)Make sure undraw   "
        (windowList at: undrawWindow) unhide.             "  has been used      "
        windowList do:[: i | i display.			"  previosly without  " 
			   i drawAllOnTop].
        undrawWindow _ DefaultUndrawWindow]             "  a mistake being    "
                                                        "  made               "
ifFalse:['UNDOUNDRAWWINDOW ERROR' prenl.
	 'Nothing valid has been undrawn' prinl]

!!
nil
!


! Picture methodsFor: 'deleteWindow' !

deleteWindow: aWindow
aWindow class prdnl.
(aWindow isKindOf: Window)
ifTrue:[windowList do:[: i | (i objid = aWindow objid)
			     ifTrue:[i deleteAll.
				     windowList remove: i 
							ifAbsent:['picture delete window error' prenl].
				     ^nil]]]

!!
nil
!


! Picture methodsFor: 'deleteAllWindows' !
						" Deletes all of the List "
deleteAllWindows

Device Clearing.
windowList _ LinkedList new.

!!
nil
!


! Picture methodsFor: 'test' !

test: anObject picture: aPicture

|k|
k_0.

windowList do:[: i| k _ k + (i count: anObject)].
^k

!!
nil
!


! Picture methodsFor: 'getCursor' !

getCursor: X and: Y

|id world | 

world _ GXRealarray new: 2.
id _ GXRealarray new: 2.

(windowList isEmpty)
ifTrue:['GETCURSOR ERROR' prenl.
	'For getCursor windowList needs to contain a window' prinl]
ifFalse:[((X class ~= Float) |
	 (Y class ~= Float) )
	ifTrue:['GETCURSOR ERROR' prenl.
		'For getCursor: and: , the parameters need to be floating numbers' prinl]
	ifFalse:[
		"id _ (windowList at: 1) getID."
		id _ (gXUUgraphics at: #gXcurrentwindow) getID.
		world at: 1 put: X.
		world at: 2 put: Y.
		
		^newGraphics function: gxGetCursor parameters: ((Array new: 3)
		                        at: 1 put: world;
					at: 2 put: id;
					at: 3 put: gXUforeground;
              		          yourself).
		]]

!

getCursor

|id world tmp atmp|
tmp _ GXWorkspace.
id _ GXRealarray new: 2.
atmp _ 0.
(windowList isEmpty)
ifTrue:['GETCURSOR ERROR' prenl.
        'For getCursor windowList needs to contain a window' prinl]
ifFalse:[ id _ (windowList at: 1) getID.
	  world _ GXRealarray new: 2.
	  world at: 1 put: 0.0.
	  world at: 2 put: 0.0.
	  tmp _ newGraphics function: gxGetCursor parameters: ((Array new: 2)
				at: 1 put: world;
				at: 2 put: id;
				yourself).
	   atmp _ (self whichWindow: tmp).
	   (atmp isNil )

	   ifFalse:[ tmp at: #gXwUx put: (atmp at: #gXa).
	   	    tmp at: #gXwUy put: (atmp at: #gXb).
		    tmp at: #gXwin put: (atmp at: #gXc)]
	   ifTrue:[tmp at: #gXwin put: 'no particular window' ].
	   ^tmp]
		    
!!
nil
!


! Picture methodsFor: 'getNumber' !

getNumber: aNumber
|tmp count counter|
count _ 0.
counter _ 0.

((aNumber = nil) |			" If a nil or 0 is given then 	"
 (aNumber = 0)   )			" the current window is the 	"
ifTrue:[(windowList isEmpty)		" last one on the list		"
	ifTrue:['ERROR ' prenl.
		'There are no windows in this picture' prinl.
		^nil]
	ifFalse:[^(windowList size)]].

(aNumber class = Integer)		"If a -ve number is given I add	"
ifTrue:[				"it on to the list size and get "
(aNumber < 0)				"the correct no. in the list    "	
ifTrue:[(windowList size = 1)
	ifTrue:['The list only has one element' prenl. ^nil]
	ifFalse:[tmp _ (windowList size + aNumber).
		 ((tmp <= windowList size) & (tmp > 0))
		 ifTrue:[^tmp]
	         ifFalse:['Error there are not that many windows' prenl. ^nil]]].

(aNumber > 0)				"If a +ve number is given, I go	"
ifTrue:[(aNumber <= windowList size)	"to that number in the list	"
	ifTrue:[^aNumber]
	ifFalse:['Error there are not that many windows' prenl.
		 ^nil]]].

" If a name is given I search for the objid to get the	"
"		correct number in the list		"

(aNumber objid class = GXIntegerarray)	
ifTrue:[windowList do:[: i | count _ count + 1.
			   (aNumber objid = i objid)
			   ifTrue:[^count]].
	(count = windowList size)
	ifTrue:['ERROR, ' pren.	
		'There is not a ' prin.
		 aNumber prin.
		' in this window list' prinl]]
!!
nil
!


! Picture methodsFor: 'GetWindowWithObjId' !

getWindowWithAnObjId: aWindow

(aWindow isKindOf: Window)
ifTrue:[windowList do: [: i | (aWindow objid = i objid)
			      ifTrue:[^i]].
	^nil]
!!
nil
!


! Picture methodsFor: 'FindWindowWithObjId' !

findWindowWithObjId: anObject
|tmp|
(anObject isKindOf: DrawableObjects)
ifTrue:[windowList do:[: i | tmp _ (i getObjectWithObjId: anObject).
			     (tmp isNil)
			     ifFalse:[^tmp]]]

!!
nil
!


! Picture methodsFor: 'FindRightWindow' !

findRightWindow: anObject
|tmp|
(anObject isKindOf: DrawableObjects)
ifTrue:[windowList do:[: i | tmp _ (i getObjectWithObjId: anObject).
			     (tmp isNil)
			     ifFalse:[^i]]]
ifFalse:[ 'findRightWindow: Object is not a drawable object' prenl ].
'findRightWindow: unable to locate window' prenl.
^nil
!!
nil
!

! Picture methodsFor: 'private' !

initPicture

		windowList _ LinkedList new.	"Allows the user to make a    "
						"LinkedList in separate parts "
		visible _ true.	

		undrawWindow _ DefaultUndrawWindow.

		picture _ true.

pictNum _ newGraphics function: gxNewPicture parameters: (( Array new: 0)
                        yourself).
!!
nil
!


! Picture methodsFor: 'printing' !

printOn: aStream
	|className|

        className _ self class name.
        aStream nextPutAll: 'An instance of '.
        aStream nextPutAll: (className copyFrom: 1 to: className size); nl.
	aStream nextPutAll: '  WindowList: '.
        aStream nextPutAll: ' LinkedList( '.
        windowList  do:[: i| aStream print: ( i class printString).
                              aStream nextPutAll:'  ' printString ].
        aStream nextPutAll: ' )'; nl

!!
nil
!

"Make picture the current one in C++, return self on success, nil on error"
! Picture methodsFor: 'selecting' !
select
	|tmp|
	tmp _ newGraphics function: gxPictureSelect parameters: ((Array new: 1)
		                        at: 1 put: pictNum;
              		          yourself).
	(tmp = 0) ifTrue: [ ^self ] ifFalse: [ ^nil ]
!!
nil
!


! GraphicalObjects methodsFor: 'addToDeviceList' !

addToDeviceList: aDevice

(aDevice class = Device)
ifTrue:[deviceList add: aDevice]
ifFalse:['ADDTOLIST ERROR, ' pren.
         'A Device must be added' prinl]
!!
nil
!


! GraphicalObjects methodsFor: 'addToPictureList' !

addToPictureList: aPicture

(aPicture class = Picture)
ifTrue:[pictureList add: aPicture]
ifFalse:['ADDTOLIST ERROR, ' pren.
         'A Picture must be added' prinl]
!!
nil
!


! GraphicalObjects methodsFor: 'getDeviceNumber' !

getDeviceNumber: aNumber
|tmp temp count|
count _ 0.
((aNumber = nil) |
 (aNumber = 0)   )
ifTrue:[ (deviceList isEmpty )
        ifTrue:['ERROR' prenl.
                'a device has not been created yet' prinl.
                ^nil]
        ifFalse:[aNumber _ deviceList size.
                ^aNumber]].

(aNumber class = Integer)
ifTrue:[
(aNumber < 0)
ifTrue:[ (deviceList size = 1)
        ifTrue:['not that many devices' prenl.
	        'This is already the current device' prinl. ^nil]
        ifFalse:[tmp _ (deviceList size + aNumber).
		 ((tmp <= deviceList size) & (tmp > 0))
                 ifTrue:[^tmp]
		 ifFalse:['not that many devices' prenl. ^nil]]].

(aNumber > 0)
ifTrue:[ ^aNumber]].

(aNumber class = Device)
ifTrue:[deviceList do:[: i | count _ count + 1.
			     tmp _ (aNumber getID).
			     temp _ (i getID).
                              (tmp = temp)
                              ifTrue:[ ^count]].
        (count = deviceList size)
        ifTrue:['ERROR, no device with that name' prenl. ^nil]]

!!
nil
!


! GraphicalObjects methodsFor: 'returnDevice' !

returnDevice: devNum
|temp|

devNum isNil ifTrue: [ ^nil ].

(devNum > deviceList size)
ifTrue:['ERROR, ' pren.
        'There are not that many devices ' prinl.
        ^nil]
ifFalse:[temp _ (deviceList at: devNum).
         ^temp]
!!
nil
!


! GraphicalObjects methodsFor: 'getDevice' !

getDevice: aDevice
(aDevice class = Device)
ifTrue:[deviceList do:[: i | ((aDevice getID) = (i getID))
			     ifTrue:[newGraphics function: gxDeviceSelect 
					 	     parameters: ((Array new: 1)
						at: 1 put: (i getID);
						yourself).
				     ^i]]]
!!
nil
!


! GraphicalObjects methodsFor: 'findDevice' !
"return device a picture object is in and select it"
findDevice: aPicture
|tmp|

tmp _ (aPicture pictNum + 1).
deviceList do:[: i | ((i getPictureNumber) = tmp)
		     ifTrue:[newGraphics function: gxDeviceSelect parameters: ((Array new: 1)
				at: 1 put: (i getID);
				yourself).
				^i]].
'findDevice: unable to find device for picture' prenl.
^nil

!!
nil
!


! GraphicalObjects methodsFor: 'getSize' !

getSize

	^(deviceList size)
!!
nil
!


! GraphicalObjects methodsFor: 'listUpdate' !

listUpdate: aNumber

|tmp|

(aNumber > deviceList size)
ifFalse:[deviceList remove: (deviceList at: aNumber) ifAbsent:['hello' prenl]]

!!
nil
!


! GraphicalObjects methodsFor: 'closeDown' !

closeDown: aNumber

|tmp temp|

(deviceList size = 0)
ifFalse:[tmp _ (self getDeviceNumber: aNumber).
	 (deviceList at: tmp) Close.
	 self listUpdate: tmp.
	 (deviceList size = 0)
	ifFalse:[tmp _ (self getDeviceNumber: nil).
		temp _ (deviceList at: tmp) getID.
	 	newGraphics function: gxDeviceSelect parameters: ((Array new: 1)
        		             at: 1 put: temp;
               		             yourself).
	 	^(deviceList at: tmp)]
	ifTrue:[^nil]]

ifTrue:['Error, ' pren.
	'There are no more devices to close' prinl]

!!
nil
!


! GraphicalObjects methodsFor: 'selectDeviceWithDeviceGiven' !

selectDevice: aDevice
"  
deviceList do:[:i| ((i getID) = anID)
  	ifTrue:[newGraphics function: gxDeviceSelect parameters: ((Array new: 1)
                                at: 1 put: anID;
                                yourself).
	 	^i]]
"|tmp|
tmp _ aDevice getID.
newGraphics function: gxDeviceSelect parameters: ((Array new: 1)
				at: 1 put: tmp;
				yourself)
!!
nil
!


! GraphicalObjects methodsFor: 'printList' !

printList
        pictureList class prn.
        ' (' prn.
        pictureList do:[: i | i class prn.
                            ' ' prn].
        ')' prnl

!!
nil
!


! GraphicalObjects methodsFor: 'drawPicture' !

drawPicture: aPicture

(aPicture class = Picture)
ifTrue:[pictureList do:[: i | (i pictNum = aPicture pictNum)
			      ifTrue:[Device Clearing.
				      i switchOn.
				      i drawUnflaggedWindows]]]
!!
nil
!


! GraphicalObjects methodsFor: 'undrawPicture' !

undrawPicture: aPicture

(aPicture class = Picture)
ifTrue:[pictureList do:[: i | (i pictNum = aPicture pictNum)
			       ifTrue:[i switchOff.
				       ^nil]]]
!!
nil
!


! GraphicalObjects methodsFor: 'deletePicture' !

deletePicture: aPicture
(aPicture class = Picture)
ifTrue:[pictureList do:[: i | (i pictNum = aPicture pictNum)
			      ifTrue:[i deleteAllWindows.
				      pictureList remove: i 
						      ifAbsent:['oops' prdnl].
				      ^nil]]]
!!
nil
!


! GraphicalObjects methodsFor: 'deleteAllPictures' !
                                                " Deletes all of the List "
deleteAllPictures

pictureList _ LinkedList new.

!!
nil
!


! GraphicalObjects methodsFor: 'getNumber' !

getNumber: aNumber
|tmp count|
count _ 0.
((aNumber = nil) |			" If a nil or 0 is given then	"
 (aNumber = 0)   )			" the current picture is the	"
ifTrue:[ (pictureList isEmpty )		" last one on the list		"
        ifTrue:['ERROR' prenl.
                'There are no pictures in the device' prinl.
                ^nil]
        ifFalse:[aNumber _ pictureList size.
                ^aNumber]].

(aNumber class = Integer)
ifTrue:[				" If a -ve number is given I add "
(aNumber < 0)				" it on to the list size and get "
ifTrue:[ (pictureList size = 1)		" the correct no. in the list	 "

        ifTrue:['There is only one picture' prenl. ^nil]
        ifFalse:[tmp _ (pictureList size + aNumber).
		 ((tmp <= pictureList size) & (tmp > 0))
		 ifTrue:[ ^tmp]
		 ifFalse:['There are not that many pictures' prenl. ^nil]]].

(aNumber > 0)				" If a +ve number is given then	"
ifTrue:[(aNumber <= pictureList size)	" I go to that no. in the list	"
	ifTrue:[^aNumber]
	ifFalse:['Error there are not that many pictures' prenl.
		  ^nil]]].

" If a name is given I search for the pictNum to get 	"
"	   the correct number in the list		"

(aNumber class = Picture)
ifTrue:[pictureList do:[: i | count _ count + 1.
                              (aNumber pictNum = i pictNum)
                              ifTrue:[ ^count]].
        (count = pictureList size)
        ifTrue:['ERROR, no picture with that name' prenl]]

!!
nil
!


! GraphicalObjects methodsFor: 'testAlter' !

testAlter: aPicture with: aWindow and: anObject with: any
|temp|
(pictureList isEmpty)
ifTrue:[^nil]
ifFalse:[
(pictureList size >= aPicture)
ifTrue:[temp _ ((pictureList at: aPicture)
                                   testAlter: anObject and: aWindow with: any).
        ^temp]
ifFalse:['ERROR' prenl.
         'There are only ' prin.
         pictureList size prin.
         ' pictures in the list' prinl.
         ^nil]]
!!
nil
!


! GraphicalObjects methodsFor: 'getPicture' !

getPicture: picNum
|temp|

picNum isNil ifTrue: [ ^nil ].

(picNum > pictureList size)
ifTrue:['ERROR, ' pren.
        'There are not that many pictures in the list' prinl.
        ^nil]
ifFalse:[temp _ (pictureList at: picNum).
         ^temp]
!!
nil
!


! GraphicalObjects methodsFor: 'getPictureWithAPicture' !
getPictureWithAPicture: aPicture

(aPicture class = Picture)
ifTrue:[pictureList do:[: i | (aPicture pictNum = i pictNum)
				ifTrue:[^i]].
	'Error, ' pren.
	'Picture is not here !!!' prinl]
ifFalse:['Error, A picture was not passed' prenl]

!!
nil
!


! GraphicalObjects methodsFor: 'findPicture' !

"return picture for an object and select it"
findPicture: anItem
|tmp|
(anItem isKindOf: DrawableObjects)
ifTrue:[tmp _ (anItem objid).
	tmp _ (tmp at: 2).
    	pictureList do:[: i | (tmp = i pictNum)
			      ifTrue:[^i select]]]
ifFalse:['findPicture: item is not a drawable object' prenl. ^nil ].
'findPicture: unable to find picture for object' prenl.
^nil
!!
nil
!


! GraphicalObjects methodsFor: 'FindDrawableObjectsPicture' !

findDrawableObjectsPicture: aDrawableObject
|tmp|

(aDrawableObject isKindOf: DrawableObjects) 
ifTrue:[tmp _ aDrawableObject objid.
 	tmp _ (tmp at: 2).
	pictureList do:[: i | (tmp = i pictNum)
			      ifTrue:[(aDrawableObject isKindOf: Window)
  				      ifTrue:[
					tmp _ (i getWindowWithAnObjId: aDrawableObject).
					^tmp]
				      ifFalse:[
					tmp _ (i findWindowWithObjId: aDrawableObject).
					^tmp]]].
	'Error, ' pren.
	'Picture is not here - ie, cannot find drawable object !!! ' prinl]

ifFalse:['Error, a drawable object was not passed' prenl]
!!
nil
!

                                                
! Device methodsFor: 'associate' !              
                                                
associate: aPicture
|tmp|
(aPicture class = Picture)
ifTrue:[ picture _ ((aPicture pictNum) + 1).
        aPicture drawAllWindows]
ifFalse:['ASSOCIATE ERROR' prenl.
	 'Only picture can be associated' prinl]
!!
nil
!


! Device methodsFor: 'getPictureNumber' !

getPictureNumber
 	^picture
!!
nil
!


! Window methodsFor: 'addToList' !
                                                " Adds an Object to the list  "
addToList: anObject                             " (the last object is now the "
(anObject isKindOf: DrawableObjects)		" one entered).	      	      "
ifTrue:[(anObject isKindOf: Window)		" has been created properly   "
        ifTrue:['ADDTOLIST ERROR' prenl.
                'A window cannot add another window to the list' prinl]
        ifFalse:[displayList add: anObject]]

ifFalse:[ 'ADDTOLIST ERROR' prenl.
          'The Drawable object has not been created' prinl]

!!
nil
!


DrawableObjects subclass: #OneD
		instanceVariableNames: 'type'
		classVariableNames: 'DefaultType'  
		poolDictionaries: ' '
		category: nil

!
nil
!


! OneD class methodsFor: 'instance creation' !

new
	^super new initOneD

!

defaultType
		^DefaultType

!

defaultType: aTy
		DefaultType _ aTy

!!
nil
!
OneD defaultType: gXDfull!


! OneD methodsFor: 'type' !

type
        ^type
!

type: aTy
        type _ aTy.

(type isKindOf: Symbol)
ifFalse:[(type = nil)
	ifTrue:[type _ DefaultType]
	ifFalse:['TYPE ERROR, Using Default Value' prenl.
        			 type _ DefaultType]]
!!
nil
!


! OneD methodsFor: 'private' !

initOneD

	type _ DefaultType

!!
nil
!


OneD subclass:#OneDGraph
	instanceVariableNames: 'xarray yarray ' 
	classVariableNames: 'DefaultXarray DefaultYarray ' 
	poolDictionaries: ' '
	category: nil
!
nil
!

!OneDGraph class methodsFor: 'instance creation' !

new
	^super new initOneDGraph
!

defaultXarray
		^DefaultXarray 
!

defaultXarray: aXA
		DefaultXarray _ aXA
!


defaultYarray
                ^DefaultYarray 
!

defaultYarray: aYA
                DefaultYarray _ aYA

!!
nil
!
OneDGraph defaultXarray: nil!
OneDGraph defaultYarray: nil!


!OneDGraph methodsFor: 'xarray' !

xarray
	^xarray
!
xarray: aXa
		xarray _ aXa.
							" Checking that the "
(xarray class = GXRealarray)				" user has entered  "
ifFalse:[(xarray = nil)
	ifTrue:[xarray _ DefaultXarray]
	ifFalse:['Xarray Error, ' pren.
		 'enter a Real array' prinl.		" the right data-   "
	 	 xarray _ DefaultXarray]]		" type 		    "
ifTrue:[DefaultXarray _ xarray]
!!
nil
!


!OneDGraph methodsFor: 'yarray' !

yarray
        ^yarray
!
yarray: aYa
                yarray _ aYa.
							" Checking that the "
(yarray class = GXRealarray)				" user has entered  "
ifFalse:[(yarray = nil)
	ifTrue:[yarray _ DefaultYarray] 
	ifFalse:['Yarray Error, ' pren.
		 'enter a Real array' prinl.		" the right data-   "
	 	 yarray _ DefaultYarray]]		" type		    "
ifTrue:[DefaultYarray _ yarray]
!!
nil
!


! OneDGraph methodsFor: 'private' !

initOneDGraph

	xarray _ DefaultXarray.

	yarray _ DefaultYarray.

!!
nil
!


! OneDGraph methodsFor: 'create' !

create

(xarray size = yarray size)
ifTrue:[
objid _ newGraphics function: gxPlotCreate parameters: ((Array new: 5)
                                at: 1 put: xarray;
                                at: 2 put: yarray;
                                at: 3 put: colour;
                                at: 4 put: type;
                                at: 5 put: lineThickness;
                                yourself)]
ifFalse:['ONEDGRAPH ERROR' prenl.
       	 'xarray and yarray MUST have the same number of elements' prinl.
	 'To plot histogram data, first use the CENTRE_BINS function' prinl ]
!!
nil
!


! OneDGraph methodsFor: 'printing' !

printOn: aStream

        | className |

        className _ self class name.
        aStream nextPutAll: 'An instance of '.
        aStream nextPutAll: (className copyFrom: 1 to: className size); nl.
	aStream nextPutAll: '  xarray: '.
	(xarray class = GXRealarray)
	ifFalse:[ aStream nextPutAll: '  Parameter has not been specified'
                 printString; nl]
	ifTrue:[xarray printOn: aStream.
		 aStream nextPutAll: ' ' printString; nl].
        aStream nextPutAll: '  yarray: '.
       	(yarray class = GXRealarray)
	ifFalse:[ aStream nextPutAll: '  Parameter has not been specified'
                 printString; nl]
	ifTrue:[yarray printOn: aStream.
		 aStream nextPutAll: ' ' printString; nl].
	aStream nextPutAll: '  colour: '.
	aStream nextPutAll: colour printString; nl.
	aStream nextPutAll: '  type: '.
	aStream nextPutAll: type printString; nl.
	aStream nextPutAll: '  lineThickness: '.
        aStream nextPutAll: lineThickness printString; nl

!!
nil
!

OneDGraph subclass:#OneDMarkers
        instanceVariableNames: 'symbol charsize'
        classVariableNames: 'DefaultSymbol DefaultCharSize'
        poolDictionaries: ' '
        category: nil
!
nil
!

!OneDMarkers  class methodsFor: 'instance creation' !

new
	^super new initOneDMarkers
!

defaultSymbol
		^DefaultSymbol
!

defaultSymbol: aSy
		DefaultSymbol _ aSy
!

defaultCharSize
		^DefaultCharSize
!

defaultCharSize: aCS
		DefaultCharSize _ aCS

!!
nil
!
OneDMarkers defaultSymbol: gXDplus !
OneDMarkers defaultCharSize: 0.7!


! OneDMarkers methodsFor: 'symbol ' !

symbol
 	^symbol
!

symbol: aSy
	symbol _ aSy.
					" Make sure that the user 	"
(symbol isKindOf: Symbol)		" enters an integer		"
ifFalse:[(symbol = nil)
	ifTrue:[symbol _ DefaultSymbol]
	ifFalse:[
	'SYMBOL Error, ' pren.
	'Using default value' prenl.
	symbol _ DefaultSymbol]]
!!
nil
!


! OneDMarkers methodsFor: 'charsize' !

charsize
	^charsize
!
					" Make sure that the user	"
charsize: aCS				" enters a Float		"
	charsize _ aCS.

(charsize class = Float)
ifFalse:[(charsize = nil)
	ifTrue:[charsize _ DefaultCharSize]
	ifFalse:['CHARSIZE Error, ' pren.
	 	 'Using default value' prenl.
	 	 'For Charsize enter a Float' prinl.	 
	 	 charsize _ DefaultCharSize]]
!!
nil
!


! OneDMarkers methodsFor: 'private' !

initOneDMarkers
	
	symbol  _ DefaultSymbol.

	charsize _ DefaultCharSize 
!!
nil
!

!OneDMarkers methodsFor: 'create' !
					" Creates Markers and returns 	"
create					" a value for objid		"	

objid _ newGraphics function: gxMarkersCreate  parameters: ((Array new: 6)
                        at: 1 put: xarray;
			at: 2 put: yarray;
			at: 3 put: colour;
			at: 4 put: symbol;
			at: 5 put: charsize;
			at: 6 put: lineThickness;
                        yourself)
!!
nil
!


! OneDMarkers methodsFor: 'printing' !

printOn: aStream

        | className |

        className _ self class name.
        aStream nextPutAll: 'An instance of '.
        aStream nextPutAll: (className copyFrom: 1 to: className size); nl.
        aStream nextPutAll: '  xarray: '.
       	(xarray class = GXRealarray)
        ifFalse:[ aStream nextPutAll: '  Parameter has not been specified'
                 printString; nl]
	ifTrue:[xarray printOn: aStream.
		 aStream nextPutAll: ' ' printString; nl].
        aStream nextPutAll: '  yarray: '.
       	(yarray class = GXRealarray)
        ifFalse:[ aStream nextPutAll: '  Parameter has not been specified'
                 printString; nl]
        ifTrue:[yarray printOn: aStream.
		 aStream nextPutAll: ' ' printString; nl].
	aStream nextPutAll: '  colour: '.
        aStream nextPutAll: colour printString; nl.
	aStream nextPutAll: '  symbol: '.
	aStream nextPutAll: symbol printString; nl.
	aStream nextPutAll: '  charsize: '.
	aStream nextPutAll: charsize printString; nl.
	aStream nextPutAll: '  lineThickness: '.
        aStream nextPutAll: lineThickness printString; nl
!!
nil
!
					
					
! Window methodsFor: 'isInAWindowII' !	
				
isInAWindowII: windowList and: win with: aList
				
|number win wind tmpList count |
number _ GXInteger.	
number _ 0.
tmpList _ LinkedList new.
count _ 0.
"aList do:[i |  win  _ i shallowCopy .
		win nextLink: nil; create.
		((( i xMax > win xMin) &                 ( i yMax > win yMin) &                 ( win xMin > i xMin) &                 ( win yMin > i yMin) )|
                (( i xMax > win xMin) &                 ( i yMin < win yMax) &                 ( win xMin > i xMin) &                 ( win yMax < i yMin) )|
                (( i xMin < win xMin) &                 ( i yMax > win yMin) &                 ( win xMin < i xMax) &                 ( win yMin > i yMin) )|
                (( i xMin < win xMax) &                 ( i yMin < win yMax) &                 ( win xMax < i xMax) &                 ( win yMax < i yMax) ))
		 ifFalse:[ ^win]]
				]
		 ifTrue:[
			 windowList do:[: j | wind _ j shallowCopy.
					    wind nextLink: nil; create. 
				   (((win xMax) = (wind xMax))  &				    ((win xMin) = (wind xMin))  &				    ((win yMax) = (wind yMax))  &				    ((win yMin) = (wind yMin))  )
				    ifTrue:[^wind 
						]]]].
"
!!
nil
!


! Window methodsFor: 'isOverlapped' !

isOverlapped: aList

|number win tmpList|

tmpList _ LinkedList new.
number _ 0.

aList do:[: i | ((i xMin <= self xMax) &	 	 (i yMax <= self yMin) &		 (i yMax >= self yMax) )
		ifTrue:[^true ]
		ifFalse:[((i xMax >= self xMin) &			  (i yMax >= self yMin) &			  (i yMax <= self yMax) )
			 ifTrue:[^true ]
			 ifFalse:[((i xMin >= self xMax) &				   (i yMin <= self yMax) &				   (i yMin >= self yMin) )
				 ifTrue:[^true ]
				 ifFalse:[ ((i xMax <= self xMin) &					   (i yMin >= self yMax) &					   (i yMin <= self yMin) )
					  ifTrue:[^true ]
					  ifFalse:[^false]]]]].
!!
nil
!


! Window methodsFor: 'isOverlappedII' !

isOverlappedII: points and: bList 

|win number count tmpList wind|

tmpList _ LinkedList new.
count _ 0.
number _ 0.

bList do:[: i|  count _ count + 1.
		win _ i shallowCopy.
		win nextLink: nil; create.

		((i xMin <= (points at: 2)) &                 (i yMax <= (points at: 3)) )
		ifTrue:[tmpList add: win]
                ifFalse:[((i xMax >= (points at: 1)) &                        (i yMax >= (points at: 3)) )
			ifTrue:[tmpList add: win]
                	ifFalse:[((i xMin >=(points at: 2) ) &                       		   (i yMin >= (points at: 4)) )
				 ifTrue:[tmpList add: win]
                        	 ifFalse:[((i xMax >= (points at: 1)) &                                  	   (i yMin <= (points at: 4)) )
                                 ifTrue:[tmpList add: win]]]]].
				   "wind nextLink: nil; create.
				   wind prenl.
				   win prinl.
                                   (((win xMax) = (wind xMax))  &                                    ((win xMin) = (wind xMin))  &                                    ((win yMax) = (wind yMax))  &                                    ((win yMin) = (wind yMin))  )
                                    ifTrue:[ tmpList add: wind]
]]]]]].
"
^tmpList

!!
nil
!
					" By making a temporary list in    "
					" FindWindows, I am able to decifer"
! Window methodsFor: 'isInAWindow' !	" which windows are actually part  "
					" of the magnification process     "
isInAWindow: aList
					" Tests to see if a window is in a "
|number|				" window, if so it will return a   "
number _ GXInteger.			" value of '1'. Otherwise '0'	   "
number _ 0.

aList do:[: i | ((( i xMax >= self xMin) &                 ( i yMax >= self yMin) &                 ( self xMin >= i xMin) &                 ( self yMin >= i yMin) )|
                (( i xMax >= self xMin) &                 ( i yMin <= self yMax) &                 ( self xMin >= i xMin) &                 ( self yMax <= i yMin) )|
                (( i xMin <= self xMin) &                 ( i yMax >= self yMin) &                 ( self xMin <= i xMax) &                 ( self yMin >= i yMin) )|
                (( i xMin <= self xMax) &                 ( i yMin <= self yMax) &                 ( self xMax <= i xMax) &                 ( self yMax <= i yMax) )) 
		 ifTrue:[ number _ (number + 0)] 
		 ifFalse:[ number _ 1]]. 	

^number
!!
nil
!


! Window methodsFor: 'findOverlappingPoint' !

findOverlappingPoint: points

|coordinates count |

coordinates _ Array new: 4.
					" Tests whether there are any valid "
 					" overlaps of windows. 		    " 
((xMax < (points at: 2)) & (yMax < (points at: 4))  )
ifTrue:[ ((xMax > (points at: 1)) &	  (yMax > (points at: 3))  )
	 ifTrue:[coordinates at: 1 put: (points at: 1).
		coordinates at: 2 put: xMax. 
		coordinates at: 3 put: (points at: 3). 
		coordinates at: 4 put: yMax]
	ifFalse:[coordinates _ nil]]

ifFalse:[
((xMin > (points at: 1)) & (yMin > (points at: 3))   )		" coordinates (for that particular "
ifTrue:[ ((xMin < (points at: 2)) &	  (yMin < (points at: 4)) )	" window			   "
	ifTrue:[coordinates at: 1 put: xMin.
		coordinates at: 2 put: (points at: 2).
		coordinates at: 3 put: yMin.
		coordinates at: 4 put: (points at: 4)]
	ifFalse:[ coordinates _ nil]]

ifFalse:[
((xMin > (points at: 1)) & (yMax < (points at: 4)) )
ifTrue:[ ((xMin < (points at: 2)) &	  (yMax > (points at: 3)) )
	ifTrue:[coordinates at: 1 put: xMin.
		coordinates at: 2 put: (points at: 2).
		coordinates at: 3 put: (points at: 3).
		coordinates at: 4 put: yMax]
	ifFalse:[coordinates _ nil]]
					" Otherwise it will resturn a value "
ifFalse:[ 				" of 'nil' (so then it can still    "
((xMax < (points at: 2)) & (yMin > (points at: 3)) )		" draw the window in the overall    " 
ifTrue:[
	((xMax > (points at: 1)) &	 (yMin < (points at: 4)) )
	ifTrue:[coordinates at: 1 put: (points at: 1).
		coordinates at: 2 put: xMax.
		coordinates at: 3 put: yMin.
		coordinates at: 4 put: (points at: 4)]
	ifFalse:[
		coordinates _ nil]]
ifFalse:[ coordinates _ nil]]]].
					" returns the variable 		    "

^coordinates				" 'COORDINATES' 		    "

!!
nil
!


! Window methodsFor: 'makeNewWindow' !

makeNewWindow: newPoints 

|win|

((self  class = WindowScaled) |
 (self  class = MultiplotWindow) |
 (self  class = TwoDWindow) |
 (self class = Window)) 
ifTrue:[win _ self shallowCopy.

	win  lxMax: (newPoints at: 2); lxMin: (newPoints at: 1); 
	lyMin: (newPoints at: 3); lyMax: (newPoints at: 4);
	nextLink: nil.
	win create.
	^win]

ifFalse:[ 'ZOOM ERROR' prenl.
	  'Co-ordinates are not within a suitable window' prinl]


!!
nil
!


! Window methodsFor: 'GetNewCoordinates' !

GetNewCoordinates: points 

|newPoints world a b c d e f g h WXmin WXmax WYmin WYmax tmp |

world _ GXRealarray.
world _ Array new: 4.

a _ xMax.
b _ xMin.
c _ yMin.
d _ yMax.
e _ lxMin.
f _ lxMax.
g _ lyMin.
h _ lyMax. 

WXmin _ ((((points at: 1) - b)/ (a - b)) * (f - e)) + e.
WXmax _ ((((points at: 2) - b)/ (a - b)) * (f - e)) + e.
WYmin _ ((((points at: 3) - c)/ (d - c)) * (h - g)) + g.
WYmax _ ((((points at: 4) - c)/ (d - c)) * (h - g)) + g.

world at: 1 put: WXmin.
world at: 2 put: WXmax.
world at: 3 put: WYmin.
world at: 4 put: WYmax.

^world

!!
nil
!


! Picture methodsFor: 'GetCoordinates' !

GetCoordinates: gotXmin and: gotYmin with: x and: y

|cursor tmp gotXmax gotYmax coordinates|

gotXmax _ GXReal.
gotYmax _ GXReal.
cursor _ GXWorkspace.
coordinates _ GXRealarray.

(windowList isEmpty)
ifTrue:[ 'ZOOM ERROR' prenl.
	 'In order to zoom, window MUST be added to picture' prinl]

ifFalse:[
cursor _ self getCursor: x and: y. 

gotXmax _ cursor at: #gXdUx.            " Assign the device coordinates to "
gotYmax _ cursor at: #gXdUy.            " temporary variables              "

(gotXmin < gotXmax)                     " Make sure that gotXmin is the    "
ifFalse:[ tmp _ gotXmin.                " MINIMUM X value, and gotYmin is "
          gotXmin _ gotXmax.            " the MINIMUM Y value             "
          gotXmax _ tmp].

(gotYmin < gotYmax)
ifFalse:[ tmp _ gotYmin.
          gotYmin _ gotYmax.
          gotYmax _ tmp].

coordinates _ Array new: 4.
coordinates at: 1 put: gotXmin.
coordinates at: 2 put: gotXmax.
coordinates at: 3 put: gotYmin.
coordinates at: 4 put: gotYmax.

^coordinates]

!!
nil
!


! Picture methodsFor: 'FindWindows' !

FindWindows: aList coordinates: points

|tmpList k a zoomList newPoints win count newPts coords object b bList window
 cList c wind|

zoomList _ LinkedList new.
tmpList _ LinkedList new.
bList _ LinkedList new.
newPoints _ Array new: 4.
newPts _ Array new: 4.
coords _ Array new: 4.
count _ 0.			" Goes through the list (either windowList or "
k _ 0.				" the previous zoomList) to find which      "
a _ 0.
b _ LinkedList new.
c _ LinkedList new.
				" window is to be magnified, then add it to "
aList do:[: i | 		"         a NEW ZoomList		    "
	  ((i xMin < (points at: 1)) &           (i xMax > (points at: 2)) &           (i yMin < (points at: 3)) &           (i yMax > (points at: 4))  )	"checks if window is in specified "
	   ifTrue:[			" points "
		(i class = Window)
		ifTrue:[newPoints _ (i GetNewCoordinates: points).
			win _ (i makeNewWindow: points)
			"'UnScaledWindow not yet implemented' prinl"]
		ifFalse:[newPoints _ (i GetNewCoordinates: points).
			 win _ (i makeNewWindow: newPoints)].
		i hideAll.
		tmpList add: win]
ifFalse:[ count _ count + 1]].


tmpList prinl.
(tmpList size > 1)
ifTrue:[
tmpList do:[: i | win _ i shallowCopy.
		  win nextLink: nil; create. 
 	tmpList do:[: j | wind _ j shallowCopy.
                          wind nextLink: nil; create.
			  (((j xMax - j xMin) > (i xMax - i xMin )) &		  	   ((j yMax - j yMin) > (i yMax - i yMin )))prenl 
		  	  ifTrue:[ zoomList addFirst: win]]].

tmpList _ LinkedList new.
zoomList do:[: i | tmpList add: i].
tmpList prdnl]
ifFalse:[tmpList do:[: i | zoomList add: i]].

aList do:[: i | win _ i shallowCopy.
		win nextLink: nil; create.
		
		tmpList do:[: j | wind _ j shallowCopy.
				  wind nextLink: nil; create.
				  (((win xMax) == (wind xMax))  &                                    ((win xMin) == (wind xMin))  &                                    ((win yMax) == (wind yMax))  &                                    ((win yMin) == (wind yMin))  )
   ifFalse:[
	   coords _ (i findOverlappingPoint: points).
	   (coords = nil)			
	   ifTrue:[ 
		  (((j xMax - j xMin) > (i xMax - i xMin )) &		  ((j yMax - j yMin) > (i yMax - i yMin ))) 
		  ifFalse:[ zoomList addFirst: win]].
	   (i isOverlapped: tmpList)	
	   ifFalse:[(((j xMax - j xMin) > (i xMax - i xMin )) &                  ((j yMax - j yMin) > (i yMax - i yMin )))
		   ifFalse:[  newPts _ (i GetNewCoordinates: coords).
		      	      win _ (i makeNewWindow: newPts).	
		      	      zoomList add: win]]]]].
		      
		

(count = aList size)

ifTrue:[ 'ZOOM ERROR' prenl.
	 'The co-ordinates specified are not within a window' prinl.
	 zoomList _ LinkedList new]. 
^zoomList	

!!
nil	
!


! Picture methodsFor: 'zoom' !

zoom 

|zoomList gotXmin gotYmin char iteration 
 tmp  b a points cursor count x y tmpList|
a _ 0.
b _ 0.
zoomList _ LinkedList new.
tmpList _ LinkedList new.
points _ GXRealarray. 
char _ GXString.
gotXmin _ GXRealarray.
gotYmin _ GXRealarray.
cursor _ GXWorkspace.
count _ 0.

cursor _  self getCursor: 0.0 and: 0.0.
char _ cursor at: #gXchar.
					" Loops around until any key (apart "
[char = 'A'] whileTrue:			" from 'A') is pressed.  When pressed "
[					" it will first go back to the 	      "
  gotXmin _ cursor at: #gXdUx.		" original picture.  Pressed a second "
  gotYmin _ cursor at: #gXdUy.		" time it will exit the ZOOM command  "
  x _ cursor at: #gXwUx.		" <- Positions the cursor to the last "
  y _ cursor at: #gXwUy.		"    point the mouse clicked on.      " 

  points _ self GetCoordinates: gotXmin and: gotYmin with: x and: y.
 (count = 0)
 ifTrue:[zoomList _ (self FindWindows: windowList coordinates: points)]
 ifFalse:[zoomList _ (self FindWindows: zoomList coordinates: points)].


	 Device Clearing.
						
	 (zoomList isEmpty)		
	 ifTrue:[windowList do:[: i|i unhideAll]. 
		 self drawAllWindows.	
		  count _ 0]		
	 ifFalse:[ zoomList do:[: i| i unhideAll. 
				     i display.
	  			     i displayList do:[: i| i display]].
	  	 count _ count + 1].
					
	cursor _ self getCursor: 0.0 and: 0.0.
	char _ cursor at: #gXchar.

	(char ~= 'A')
	ifTrue:[windowList do:[: i|i unhideAll].
		self drawAllWindows.
		cursor _ self getCursor: 0.0 and: 0.0.
		char _ cursor at: #gXchar.
		zoomList _ LinkedList new.
		count _ 0.
		]]


!!
nil
!


DrawableObjects subclass:#Polyline
	instanceVariableNames: 'xstart ystart xend yend type'
	classVariableNames: 'DefaultXStart DefaultYStart DefaultXEnd
			     DefaultYEnd DefaultType '
	poolDictionaries: ' '
	category: nil
!
nil
!


! Polyline class methodsFor: 'instance creation' !

new
		^super new initPolyline
!

defaultXStart
		^DefaultXStart
!

defaultXStart: aXS
		DefaultXStart _ aXS
!

defaultYStart
                ^DefaultYStart
!

defaultYStart: aYS
                DefaultYStart _ aYS
!

defaultXEnd
		^DefaultXEnd
!

defaultXEnd: aXE
		DefaultXEnd _ aXE
!

defaultYEnd
                ^DefaultYEnd
!

defaultYEnd: aYE
                DefaultYEnd _ aYE
!

defaultType
		^DefaultType
!

defaultType: aTy
		DefaultType _ aTy
!!
nil
!
Polyline defaultType: gXDfull !
Polyline defaultXStart: nil!
Polyline defaultYStart: nil!
Polyline defaultXEnd: nil!
Polyline defaultYEnd: nil!


! Polyline methodsFor: 'xstart' !

xstart
	^xstart
!

xstart: aXS
	xstart _ aXS.
							" Checking that the "
(xstart class = Float)					" user has entered  "
ifFalse:[ 'XStart ERROR, ' pren.			" the right data-   "
	 'For XStart Enter a Float' prinl.		" type. "
	  xstart _ DefaultXStart]
!!
nil
!


! Polyline methodsFor: 'ystart' !

ystart
        ^ystart
!

ystart: aYS
        ystart _ aYS.

(ystart class = Float)					" Checking that the "	
ifFalse:[ 'YStart ERROR, ' pren.			" user has entered  "
	 'For YStart Enter a Float' prinl.		" the right data-   "
	 ystart _ DefaultYStart]			" type 		    "

!!
nil
!


! Polyline methodsFor: 'xend' !

xend
	^xend
!
xend: aXE
	xend _ aXE.

(xend class = Float)					" Checking that the "
ifFalse:[ 'XEnd ERROR, ' pren.				" user has entered  "
	 'For XEnd Enter a Float' prinl.		" the right data-   "
	 xend _ DefaultXEnd]				" type              "
!!
nil
!


! Polyline methodsFor: 'yend' !

yend
        ^yend

!

yend: aYE
        yend _ aYE.

(yend class = Float)					"Checking that the "
ifFalse:[ 'YEnd ERROR, ' pren.				" user has entered "
         'For YEnd Enter a Float' prinl.		" the right data-  "
         yend _ DefaultYEnd]				" type	   	   "

!!
nil
!


! Polyline methodsFor: 'type' !

type
	^type
!
						" Checking that the "
type: aTy					" user has entered  "
	type _ aTy.				" the correct       "
						" parameters        "
( type isKindOf: Symbol )
ifFalse:[(type = nil)
	ifTrue:[type _ DefaultType]
	ifFalse:['TYPE ERROR, Using Default Value' prenl.
                 type _ DefaultType]]
!!
nil
!


! Polyline methodsFor: 'private' !

initPolyline

	xstart _ DefaultXStart.

	ystart _ DefaultYStart.
	
	xend _ DefaultXEnd.
	
	yend _ DefaultYEnd.

	type _ DefaultType

!!
nil
!


! Polyline methodsFor: 'create' !

create

objid _ newGraphics function: gxPolylineCreate parameters: ((Array new: 7)
			at: 1 put: xstart;
			at: 2 put: ystart;
			at: 3 put: xend;
			at: 4 put: yend;
			at: 5 put: colour;
			at: 6 put: type;
			at: 7 put: lineThickness;
			yourself)

!!
nil
!


! Polyline methodsFor: 'printing' !

printOn: aStream

	|className|

	className _ self class name.
        aStream nextPutAll: 'An instance of '.
        aStream nextPutAll: (className copyFrom: 1 to: className size); nl.
	aStream nextPutAll: '  xstart: '.
	aStream nextPutAll: xstart printString; nl.
	aStream nextPutAll: '  ystart: '.
	aStream nextPutAll: ystart printString; nl.
	aStream nextPutAll: '  xend: '.
	aStream nextPutAll: xend printString; nl.
	aStream nextPutAll: '  yend: '.
	aStream nextPutAll: yend printString; nl.
	aStream nextPutAll: '  colour: '.
	aStream nextPutAll: colour printString; nl.
	aStream nextPutAll: '  type: '.
	aStream nextPutAll: type printString; nl.
	aStream nextPutAll: '  lineThickness: '.
	aStream nextPutAll: lineThickness printString; nl

!!
nil
!


DrawableObjects subclass:#Text
	instanceVariableNames: 'xcoord ycoord words size angle quality justify'
	classVariableNames: ' 	DefaultQuality DefaultSize DefaultAngle 
				DefaultXcoord DefaultYcoord DefaultWords
				DefaultJustify'
	poolDictionaries: ' '
	category: nil

!
nil
!

!Text class methodsFor: 'instance creation'! 

new
	^super new initText

!

defaultQuality					" Setting defaults for:	"
		^DefaultQuality			 
!						"	Quality		"

defaultQuality: aQu				"	Size		"
		DefaultQuality _ aQu
!						"	Angle		"

defaultSize
		^DefaultSize
!

defaultSize: aSi
		DefaultSize _ aSi

!

defaultAngle
		^DefaultAngle

!
defaultAngle: aAn
		DefaultAngle _ aAn

!

defaultXcoord
		^DefaultXcoord
!

defaultXcoord: aXc
		DefaultXcoord _ aXc

!

defaultYcoord
                ^DefaultYcoord
!

defaultYcoord: aYc
                DefaultYcoord _ aYc

!

defaultJustify
                ^DefaultJustify
!

defaultJustify: aReal
                DefaultJustify _ aReal
!

defaultWords
		^DefaultWords
!

defaultWords: aWo
		DefaultWords _ aWo
!!
nil
!
Text defaultQuality: gXDnormal !
Text defaultSize: 1.0!
Text defaultAngle: 0.0!
Text defaultXcoord: nil!
Text defaultYcoord: nil!
Text defaultWords: 'no words specified'!
Text defaultJustify: 0.0!

! Text methodsFor: 'xcoord' !

xcoord
	^xcoord

!

xcoord: aXc
	xcoord _ aXc.

(xcoord class = Float)
ifFalse:['XCOORD Error, ' pren.
	'Enter a Float' prinl.
         xcoord _ DefaultXcoord]

!!
nil
!

!Text methodsFor: 'ycoord' !

ycoord
	^ycoord

!

ycoord: aYc
	ycoord _ aYc.

(ycoord class = Float)
ifFalse:['YCOORD Error, ' pren.
	'Enter a Float' prinl. 
	 ycoord _ DefaultYcoord]

!!
nil
!

!Text methodsFor: 'words' !

words
	^words.

!

words: aWo
	words _ aWo.

(words class  = String) 
ifFalse:['WORDS Error, ' pren.
	'Enter a string' prinl.
	words _ DefaultWords]

!!
nil
!

!Text methodsFor: 'size' !

size
	^size

!

size: aSi
	size _ aSi.

(size class = Float)
ifFalse:[(size = nil)
	ifTrue:[size _ DefaultSize]
	ifFalse:['SIZE Error, ' pren.
		 'Enter an Float ' prinl.
		 size _ DefaultSize]]
 
!!
nil
!

!Text methodsFor: 'justify' !

justify
	^justify

!

justify: aReal
	(aReal isKindOf: Float) ifTrue: [ justify _ aReal ]
		ifFalse: [ justify _ DefaultJustify ]
!!
nil
!

!Text methodsFor: 'angle' !

angle
	^angle

!

angle: aAn
	angle _ aAn.

(angle class =Float)
ifFalse:[(angle = nil)
	ifTrue:[angle _ DefaultAngle]
	ifFalse:['ANGLE Error, ' pren.
		 'Enter a Float ' prinl.
        	 angle _ DefaultAngle]]

!!
nil
!

!Text methodsFor: 'quality'!
 quality
	^quality

!

quality: aQu
	quality _ aQu.

(quality isKindOf: Symbol)
ifFalse: [(quality = nil)
	ifTrue:[quality _ DefaultQuality]
	ifFalse:['QUALITY Error, ' pren.
	  	 quality _ DefaultQuality]]

!!
nil
!

! Text methodsFor: 'private' !

initText
	
	size _ DefaultSize.

	quality _ DefaultQuality.

	angle _ DefaultAngle.

	xcoord _ DefaultXcoord.

	ycoord _ DefaultYcoord.

	words _ DefaultWords.

	justify _ DefaultJustify

!!
nil
!

! Text methodsFor: 'create' !

create


objid _ newGraphics function: gxTextCreate parameters: ((Array new: 9)
                        at: 1 put: xcoord;
			at: 2 put: ycoord;
			at: 3 put: words;
			at: 4 put: size;
			at: 5 put: angle;
			at: 6 put: colour;
			at: 7 put: quality;
			at: 8 put: lineThickness; 
			at: 9 put: justify; 
                        yourself).




!!
nil
!


! Text methodsFor: 'printing' !

printOn: aStream
        
	| className |
        
	className _ self class name.
        aStream nextPutAll: 'An instance of '.
        aStream nextPutAll: (className copyFrom: 1 to: className size); nl.
        aStream nextPutAll: '  xcoord: '.
        aStream nextPutAll: xcoord printString; nl.
        aStream nextPutAll: '  ycoord: '.      
        aStream nextPutAll: ycoord printString; nl.
        aStream nextPutAll: '  quality: '.
        aStream nextPutAll: quality printString; nl.
        aStream nextPutAll: '  size: '.
        aStream nextPutAll: size printString; nl.
        aStream nextPutAll: '  angle: '.
        aStream nextPutAll: angle printString; nl.
       	aStream nextPutAll: '  colour: '.
	aStream nextPutAll: colour printString; nl. 
	aStream nextPutAll: '  words: '.
        aStream nextPutAll: words printString; nl.
	aStream nextPutAll: '  justify: '.
        aStream nextPutAll: justify printString; nl.
!!
nil
!

DrawableObjects subclass:#Polygon
        instanceVariableNames: 'xcoord ycoord solid'
        classVariableNames: 'DefaultSolid DefaultXCoord DefaultYCoord' 
        poolDictionaries: ''
        category: nil

!
nil
!


!Polygon class methodsFor: 'instance initialization' !


new
        ^super new initPolygon

!

defaultSolid				 " Creates a default method	"
		^DefaultSolid
!

defaultSolid: aS
		DefaultSolid _ aS

!

defaultXCoord
		^DefaultXCoord

!

defaultXCoord: xc
		DefaultYCoord _ xc

!

defaultYCoord
		^DefaultYCoord

!

defaultYCoord: yc
		DefaultYCoord _ yc

!!
nil
!
Polygon defaultSolid: 0!
Polygon defaultXCoord: nil!
Polygon defaultYCoord: nil!


!Polygon methodsFor: 'ycoord' !

ycoord					" Takes in the y-coordinates	"
	^ycoord

!

ycoord: yc 
        ycoord _ yc.

(ycoord class ~= GXRealarray)	
ifTrue:[ 'ycoord Error, ' pren.
	 'Enter a Real Array' prinl.
	  ycoord _ DefaultYCoord ] 
!!
nil
!


!Polygon methodsFor: 'solid' !

solid					" Takes in the filled/unfilled 	"
        ^solid 				" message 			"
		
!				
solid: aSolid		
	solid _ aSolid.

(solid = nil)				" If the message is not '0' or  "
ifTrue:[solid _ DefaultSolid].
((solid = 1) |				" '1' then it goes to default   "
 (solid = 0) )				" mode				"	
ifFalse:['Solid Error using default value' prenl.
	 'For Solid, enter "0" (outline) or "1" (filled)' prinl.
	 solid _ DefaultSolid ]

!!
nil
!


!Polygon methodsFor: 'xcoord' !

xcoord					" Takes in the x-coordinates	"
        ^xcoord
!


xcoord: xc
        xcoord _ xc.
							" Checking that the "
(xcoord class ~= GXRealarray)				" user has entered  "
ifTrue:[ 'XCoord Error, ' pren.
	 'Enter a Real Array' prinl.
	 xcoord _ DefaultXCoord]
!!
nil
!


! Polygon methodsFor: 'private' !

initPolygon

	solid _ DefaultSolid.		" Puts in default value		"

	xcoord _ DefaultXCoord.

	ycoord _ DefaultYCoord

!!
nil
!


!Polygon methodsFor: 'create' !	" Sends the appropriate 	"
					" messages to the C++ code, to 	"
create					" enable the drawing.		"


objid _ newGraphics function: gxPolygonCreate parameters: ((Array new: 5)
                        at: 1 put: xcoord;
                        at: 2 put: ycoord;
                        at: 3 put: colour;
                        at: 4 put: solid;
                        at: 5 put: lineThickness;
                        yourself)
!!
nil
!


! Polygon methodsFor: 'printing' !

printOn: aStream
        
        | className |

        className _ self class name.
        aStream nextPutAll: 'An instance of '.
        aStream nextPutAll: (className copyFrom: 1 to: className size); nl.
	aStream nextPutAll: '  xcoord: '.
	xcoord printOn: aStream.
	aStream nextPutAll: ' ' printString; nl.
	aStream nextPutAll: '  ycoord: '.	
	ycoord printOn: aStream.
	aStream nextPutAll: ' ' printString; nl.
	aStream nextPutAll: '  colour: '.
	aStream nextPutAll: colour printString; nl.	
	aStream nextPutAll: '  solid: '.
	aStream nextPutAll: solid printString; nl.
	aStream nextPutAll: '  lineThickness: '.
	aStream nextPutAll: lineThickness printString; nl

!!
nil
!

OneD subclass:#OneDAnnotate
	instanceVariableNames: 'quality charsize'
	classVariableNames: 'DefaultQuality DefaultCharsize' 
	poolDictionaries: ' '
	category: nil
!
nil
!

!OneDAnnotate class methodsFor: 'instance creation' !

new
	^super new initOneDAnnotate
!

defaultQuality
		^DefaultQuality
!

defaultQuality: aQu
		DefaultQuality _ aQu
!

defaultCharsize
		^DefaultCharsize
!

defaultCharsize: aCS
		DefaultCharsize _ aCS
!!
nil
!
OneDAnnotate defaultQuality: gXDnormal !
OneDAnnotate defaultCharsize: 1.0!


! OneDAnnotate methodsFor: 'quality' !

quality
        ^quality
					" Making sure that the user    "
!					" enters an integer, and to    "
					" make sure the number in less "
quality: aQu				" than 4.		       "
        quality _ aQu.

(quality isKindOf: Symbol)
ifFalse:[(quality = nil)
	ifTrue:[quality _ DefaultQuality]
	ifFalse:['QUALITY ERROR, Using Default Value' prenl.
        	 quality _ DefaultQuality]]
!!
nil
!


! OneDAnnotate methodsFor: 'charsize' !

charsize
        ^charsize

!
					" Make sure the user enters "
charsize: aCS				" a Float.		    "
        charsize _ aCS.

(charsize class ~= Float)
ifTrue:[(charsize = nil)
	ifTrue:[charsize _ DefaultCharsize]
	ifFalse:['CHARSIZE ERROR, Using Default Value' prenl.
        	 'For charsize, enter a Float' prinl.
        	 charsize _ DefaultCharsize]]
!!
nil
!


! OneDAnnotate methodsFor: 'private' !


initOneDAnnotate

	quality _ DefaultQuality.

	charsize _ DefaultCharsize
	
!!
nil 
!


OneDAnnotate subclass: #OneDAxes
	instanceVariableNames: 'xlog ylog xnum ynum yhoriz type'
	classVariableNames: 'DefaultXLog DefaultYLog DefaultXNum
			     DefaultYNum DefaultYHoriz DefaultType'
	poolDictionaries: ' '
	category: nil
!
nil
!
Smalltalk at: #gXAxesCreate put: 4!


! OneDAxes class methodsFor: 'instanceCreation' !

new
	^super new initOneDAxes
!

defaultXLog
                ^DefaultXLog
!

defaultXLog: aXL
                DefaultXLog _ aXL
!

defaultYLog
                ^DefaultYLog
!

defaultYLog: aYL
                DefaultYLog _ aYL
!

defaultXNum
                ^DefaultXNum

!

defaultXNum: aXN
                DefaultXNum _ aXN
!

defaultYNum
                ^DefaultYNum

!

defaultYNum: aYN
                DefaultYNum _ aYN
!
                                     
defaultYHoriz
                ^DefaultYHoriz
!

defaultYHoriz: aYH
                DefaultYHoriz _ aYH
!
defaultType
		^DefaultType
!
defaultType: aTy
		DefaultType _ aTy
!!
nil
!
OneDAxes defaultXLog: 0!
OneDAxes defaultYLog: 0!
OneDAxes defaultXNum: 1!
OneDAxes defaultYNum: 1!
OneDAxes defaultYHoriz: 1!
OneDAxes defaultType: gXDfull!


! OneDAxes methodsFor: 'xlog' !

xlog
                ^xlog
!
                                        "============================== "
xlog: aXL                               "                               "
                xlog _ aXL.             " Putting in default conditions "

                                        " which explains to the user    "
(xlog = nil)
ifTrue:[xlog _ DefaultXLog].

((xlog = 1) |        
 (xlog =0)  ) 	                  	"what he has done wrong        "
ifFalse:['XLOG ERROR, using default value' prenl.
        'For xlog, enter "0" (log) or "1" (Linear) ' prinl.
        '     ' prinl.
         xlog _ DefaultXLog]
!!
nil
!

! OneDAxes methodsFor: 'ylog' !

ylog
                ^ylog
!

ylog: aYL                 
                ylog _ aYL.
                          
                         
((ylog = 1)  |        
 (ylog =0)   )        
ifFalse:[ (ylog = nil)
	 ifTrue:[ylog _ DefaultYLog]
	 ifFalse:['YLOG ERROR, using default value' prenl.
         	  'For ylog, enter "0" (log) or "1" (Linear) ' prinl.
         	  '     ' prnl.
         	  ylog _ DefaultYLog]]
!!
nil
!

! OneDAxes methodsFor: 'xnum' !

xnum
                ^xnum
!
xnum: aXN
                xnum _ aXN.

((xnum = 1) |
 (xnum = 0) )
ifFalse:[ (xnum = nil) 
	ifTrue:[xnum _ DefaultXNum]
	ifFalse:['XNUM ERROR, using default value' prenl.
         	 'For xnum enter "0" (unlabelled) or "1" (labelled)' prinl.
         	 '      ' prnl.
         	 xnum _ DefaultXNum]]
!!
nil
!


! OneDAxes methodsFor: 'ynum' !

ynum
                ^ynum
!
ynum: aYN
                ynum _ aYN.

((ynum = 1) |
 (ynum = 0) )
ifFalse:[ (ynum = nil)
	ifTrue:[ynum _ DefaultYNum]
	ifFalse:['YNUM ERROR, using default value' prenl.
                 'For ynum enter "0" (unlabelled) or "1" (labelled)' prinl.
                 '      ' prnl.
                 ynum _ DefaultYNum]]
!!
nil
!


! OneDAxes methodsFor: 'yhoriz' !

yhoriz
                ^yhoriz
!

yhoriz: aYH
                yhoriz _ aYH.

((yhoriz =1) |
 (yhoriz =0) )
ifFalse:[(yhoriz = nil)
	ifTrue:[yhoriz _ DefaultYHoriz ]
	ifFalse:['YHORIZ ERROR, using default value' prenl.
                 'For yhoriz, enter "0" (label vertically) or ' prinl.
                 ' "1" (label horizontally)' prinl.
                 '      ' prnl.
                 yhoriz _ DefaultYHoriz ]]
!!
nil
!


! OneDAxes methodsFor: 'type' !
type 
	^type
!
type: aTy
	type _ aTy.
( type isKindOf: Symbol )
ifFalse:[(type = nil)
        ifTrue:[type _ DefaultType]
        ifFalse:['TYPE ERROR, Using Default Value' prenl.
                 type _ DefaultType]]
!!
nil
!


! OneDAxes methodsFor: 'private' !

initOneDAxes
	
	xlog _ DefaultXLog.

        ylog _ DefaultYLog.

	xnum _ DefaultXNum.

        ynum _ DefaultYNum.

	yhoriz _ DefaultYHoriz.

	type _ DefaultType.

	self charsize: 0.8		"change default character size"
!!
nil
!


! OneDAxes methodsFor: 'create' !

create

objid _ newGraphics function: gxAxesCreate parameters: ((Array new: 10)
                                at: 1 put: colour;
                                at: 2 put: lineThickness;
                                at: 3 put: charsize;
                                at: 4 put: quality;
                                at: 5 put: xlog;
                                at: 6 put: ylog;
				at: 7 put: xnum;
                                at: 8 put: ynum;
				at: 9 put: yhoriz;
				at: 10 put: type;
                                yourself)

!!
nil
!


! OneDAxes methodsFor: 'printing' !

printOn: aStream

        | className |

        className _ self class name.
        aStream nextPutAll: 'An instance of '.
        aStream nextPutAll: (className copyFrom: 1 to: className size); nl.
        aStream nextPutAll: '  colour: '.
        aStream nextPutAll: colour printString; nl.
        aStream nextPutAll: '  lineThickness: '.
        aStream nextPutAll: lineThickness printString; nl.
        aStream nextPutAll: '  xlog: '.
        aStream nextPutAll: xlog printString; nl.
        aStream nextPutAll: '  ylog: '.
        aStream nextPutAll: ylog printString; nl.
	aStream nextPutAll: '  xnum: '.
	aStream nextPutAll: xnum printString; nl.
	aStream nextPutAll: '  ynum: '.
        aStream nextPutAll: ynum printString; nl.
	aStream nextPutAll: '  yhorizontal: '.
	aStream nextPutAll: yhoriz printString; nl.
	aStream nextPutAll: '  line_type: '.
	aStream nextPutAll: type printString; nl.

!!
nil
!


OneDAnnotate subclass:#OneDGraticule
	instanceVariableNames: ' xlog ylog xnum ynum yhoriz type'
	classVariableNames: 'DefaultXLog DefaultYLog DefaultXNum DefaultYNum
			     DefaultYHoriz DefaultType'
	poolDictionaries: ' '
	category: nil
!
nil
!


! OneDGraticule class methodsFor: 'instance creation' !

new
	^super new initOneDGraticule
!

defaultXLog
		^DefaultXLog
!

defaultXLog: aXL
		DefaultXLog _ aXL

!

defaultYLog
                ^DefaultYLog
!

defaultYLog: aYL
                DefaultYLog _ aYL
!
defaultXNum
                ^DefaultXNum

!

defaultXNum: aXN
                DefaultXNum _ aXN
!

defaultYNum
                ^DefaultYNum

!

defaultYNum: aYN
                DefaultYNum _ aYN
!

defaultYHoriz
                ^DefaultYHoriz
!

defaultYHoriz: aYH
                DefaultYHoriz _ aYH
!
defaultType
		^DefaultType
!
defaultType: aTy
		DefaultType _ aTy
!!
nil
!
OneDGraticule defaultXLog: 0!
OneDGraticule defaultYLog: 0!
OneDGraticule defaultYNum: 0!
OneDGraticule defaultXNum: 0!
OneDGraticule defaultYHoriz: 0!
OneDGraticule defaultType: gXDfull !


! OneDGraticule methodsFor: 'xlog' !

xlog
	^xlog
!

xlog: aXL
	xlog _ aXL.
	
((xlog = 1) |				
 (xlog = 0) )
ifFalse:[(xlog = nil)
	ifTrue:[xlog _ DefaultXLog]
	ifFalse:['XLog Error, Using Default Value' prenl.
		 'Enter "0" (linear) or "1" (log)' prinl.
		 xlog _ DefaultXLog]]
!!
nil
!


! OneDGraticule methodsFor: 'ylog' !

ylog
	^ylog
!
					" Make sure that the user	"
ylog: aYL				" enters a number 0 or 1 only   "
	ylog _ aYL.

((ylog = 1) |
 (ylog = 0) )
ifFalse:[(ylog = nil)
	ifTrue:[ylog _ DefaultYLog]
	ifFalse:['YLog Error, Using Default Value' prenl.
                 'Enter "0" (linear) or "1" (log)' prinl.
                 ylog _ DefaultYLog]]

!!
nil
!

! OneDGraticule methodsFor: 'xnum' !

xnum
                ^xnum
!
xnum: aXN
                xnum _ aXN.

((xnum = 1) |
 (xnum = 0) )
ifFalse:[ (xnum = nil)
        ifTrue:[xnum _ DefaultXNum]
        ifFalse:['XNUM ERROR, using default value' prenl.
                 'For xnum enter "0" (unlabelled) or "1" (labelled)' prinl.
                 '      ' prnl.
                 xnum _ DefaultXNum]]
!!
nil
!


! OneDGraticule methodsFor: 'ynum' !

ynum
                ^ynum
!
ynum: aYN
                ynum _ aYN.

((ynum = 1) |
 (ynum = 0) )
ifFalse:[ (ynum = nil)
        ifTrue:[ynum _ DefaultYNum]
        ifFalse:['YNUM ERROR, using default value' prenl.
                 'For ynum enter "0" (unlabelled) or "1" (labelled)' prinl.
                 '      ' prnl.
                 ynum _ DefaultYNum]]
!!
nil
!


! OneDGraticule methodsFor: 'yhoriz' !

yhoriz
                ^yhoriz
!

yhoriz: aYH
                yhoriz _ aYH.

((yhoriz =1) |
 (yhoriz =0) )
ifFalse:[(yhoriz = nil)
        ifTrue:[yhoriz _ DefaultYHoriz ]
        ifFalse:['YHORIZ ERROR, using default value' prenl.
                 'For yhoriz, enter "0" (label vertically) or ' prinl.
                 ' "1" (label horizontally)' prinl.
                 '      ' prnl.
                 yhoriz _ DefaultYHoriz ]]
!!
nil
!


! OneDGraticule methodsFor: 'type' !

type
	^type
!
type: aTy
	type _ aTy.
( type isKindOf: Symbol )
ifFalse:[(type = nil)
        ifTrue:[type _ DefaultType]
        ifFalse:['TYPE ERROR, Using Default Value' prenl.
                 type _ DefaultType]]
!!
nil
!

! OneDGraticule methodsFor: 'private' !

initOneDGraticule

		xlog _ DefaultXLog.

		ylog _ DefaultYLog.

		xnum _ DefaultXNum.

		ynum _ DefaultYNum.

		yhoriz _ DefaultYHoriz.

		type _ DefaultType

!!
nil
!


! OneDGraticule methodsFor: 'create' !
					" Creates Graticule and returns "
create					" a value for objid		"

objid _ newGraphics function: gxGraticuleCreate parameters: ((Array new: 10)
			at: 1 put: colour;
			at: 2 put: lineThickness;
			at: 3 put: xlog;
			at: 4 put: ylog;
			at: 5 put: charsize;
			at: 6 put: xnum;
			at: 7 put: ynum;
			at: 8 put: yhoriz;
			at: 9 put: quality;
			at: 10 put: type;
			yourself)

!!
nil
!


! OneDGraticule methodsFor: 'printing' !

printOn: aStream

        | className |

        className _ self class name.
        aStream nextPutAll: 'An instance of '.
        aStream nextPutAll: (className copyFrom: 1 to: className size); nl.
        aStream nextPutAll: '  colour: '.
        aStream nextPutAll: colour printString; nl.
        aStream nextPutAll: '  lineThickness: '.
        aStream nextPutAll: lineThickness printString; nl.
        aStream nextPutAll: '  xlog: '.
        aStream nextPutAll: xlog printString; nl.
        aStream nextPutAll: '  ylog: '.
        aStream nextPutAll: ylog printString; nl.
        aStream nextPutAll: '  xnum: '.
        aStream nextPutAll: xnum printString; nl.
        aStream nextPutAll: '  ynum: '.
        aStream nextPutAll: ynum printString; nl.
        aStream nextPutAll: '  yhoriz: '.
        aStream nextPutAll: yhoriz printString; nl.
        aStream nextPutAll: '  line_type: '.
        aStream nextPutAll: type printString; nl.
!!
nil
!


OneDAnnotate subclass:#Labels
	instanceVariableNames: ' xlabel ylabel xvert yvert'
	classVariableNames: ' DefaultXLabel DefaultYLabel 
			      DefaultXVert DefaultYVert' 
	poolDictionaries: ' '
	category: nil
!
nil
!


! Labels class methodsFor: 'instance creation' !

new
		^super new initLabels
!

defaultXLabel
                ^DefaultXLabel
!

defaultXLabel: aXL
                DefaultXLabel _ aXL
!

defaultYLabel
                ^DefaultYLabel
!

defaultYLabel: aYL
                DefaultYLabel _ aYL
!

defaultXVert
		^DefaultXVert
!

defaultXVert: aXV
		DefaultXVert _ aXV
!

defaultYVert
                ^DefaultYVert
!

defaultYVert: aYV
                DefaultYVert _ aYV

!!
nil
!
Labels defaultXLabel: '' !
Labels defaultYLabel: '' !
Labels defaultXVert: 0!
Labels defaultYVert: 1!


! Labels methodsFor: 'xlabel' !

xlabel
	^xlabel
!

xlabel: aXl
	xlabel _ aXl.
(xlabel = nil)
ifTrue:[xlabel _ DefaultXLabel].	" Make sure that the user	"
(xlabel class = String)			" enters a String		"
ifFalse:['XLabel Error ' prenl.
	'Enter a String' prinl.
	xlabel _ DefaultXLabel]
!!
nil
!


! Labels methodsFor: 'ylabel' !

ylabel
        ^ylabel 
!
					" Make sure that the user	"
ylabel: aYl				" enters a String		"
        ylabel _ aYl.
(ylabel = nil)
ifTrue:[ylabel _ DefaultYLabel].
(ylabel class =String)
ifFalse:['YLabel Error.' prenl. 
	'Enter a String' prinl.
	ylabel _ DefaultYLabel]

!!
nil
!


! Labels methodsFor: 'xvert' !

xvert
	^xvert
!
					" Make sure that the user	"
xvert: aXV				" enters the number 0 or 1 only	"
	xvert _ aXV.
(xvert = nil)
ifTrue:[xvert _ DefaultXVert].
((xvert = 0) |
 (xvert = 1)  )
ifFalse:[(xvert = nil)
	ifTrue:[xvert _ DefaultXVert]
	ifFalse:['XVert Error, Using Default Value' prenl.
		 'For xvert, enter "0" (horizontal) or  ' prin.
                 ' "1" (vertical)' prinl.
		 xvert _ DefaultXVert]]

!!
nil
!


! Labels methodsFor: 'yvert' !

yvert
        ^yvert
!
					" Make sure that the user	"
yvert: aYV				" enters the number 0 or 1 only "
        yvert _ aYV.
(yvert = nil)
ifTrue:[yvert _ DefaultYVert].
((yvert = 0) |
 (yvert = 1)  )
ifFalse:[(yvert = nil) 
	ifTrue:[yvert _ DefaultYVert]
	ifFalse:['YVert Error, Using Default Value' prenl.
        	 'For yvert, enter "0" (horizontal) or  ' prin.
		 ' "1" (vertical)' prinl.
		 yvert _ DefaultYVert]]

!!
nil
!


! Labels methodsFor: 'private ' !

initLabels

	xlabel _ DefaultXLabel.

        ylabel _ DefaultYLabel.

        xvert _ DefaultXVert.

        yvert _ DefaultYVert


!!
nil
!


! Labels methodsFor: 'create' !
				" Creates Labels and returns a value "
create				" for objid			     "

objid _ newGraphics function: gxLabelsCreate parameters: ((Array new: 8)
				at: 1 put: xlabel;
				at: 2 put: ylabel;
				at: 3 put: colour;
				at: 4 put: quality;
				at: 5 put: charsize;
				at: 6 put: lineThickness;
				at: 7 put: xvert;
				at: 8 put: yvert;
				yourself)

!!
nil
!


! Labels methodsFor: 'printing' !

printOn: aStream

        | className |

        className _ self class name.
        aStream nextPutAll: 'An instance of '.
        aStream nextPutAll: (className copyFrom: 1 to: className size); nl.
	aStream nextPutAll: '  xlabel: '.
	aStream nextPutAll: xlabel printString; nl.
	aStream nextPutAll: '  ylabel: '.
	aStream nextPutAll: ylabel printString; nl.
	aStream nextPutAll: '  colour: '.
        aStream nextPutAll: colour printString; nl.
        aStream nextPutAll: '  quality: '.
        aStream nextPutAll: quality printString; nl.
        aStream nextPutAll: '  charsize: '.
        aStream nextPutAll: charsize printString; nl.
        aStream nextPutAll: '  lineThickness: '.
        aStream nextPutAll: lineThickness printString; nl.
	aStream nextPutAll: '  xvert: '.
	aStream nextPutAll: xvert printString; nl.
	aStream nextPutAll: '  yvert: '.
	aStream nextPutAll: yvert printString; nl.
!!
nil
!

OneDAnnotate subclass:#Title
        instanceVariableNames: 'title '
        classVariableNames: 'DefaultTitle '
        poolDictionaries: ' '
        category: nil
!
nil
!


! Title class methodsFor: 'instance creation' !

new
        ^super new initTitle
!

defaultTitle
                ^DefaultTitle
!

defaultTitle: aTi
                DefaultTitle _ aTi

!!
nil
!
Title defaultTitle: ''!


! Title methodsFor: 'title' !

title
	^title
!
					" Make sure that the user	"
title: aTi				" enters a string		"
	title _ aTi.

(title class =String)
ifFalse:['Title ERROR, ' prenl.
        'Enter a String' prinl.
        title _ DefaultTitle]

!!
nil
!


! Title methodsFor: 'private' !

initTitle

	title _ DefaultTitle
!!
nil
!


! Title methodsFor: 'create' !
				" Creates Title and returns a value "
create				" for objid			    "

objid _ newGraphics function: gxTitleCreate parameters: ((Array new: 4)
				at: 1 put: title;
				at: 2 put: colour;
				at: 3 put: quality;
				at: 4 put: charsize;
				yourself)

!!
nil
!


! Title methodsFor: 'printing' !

printOn: aStream

        | className |

        className _ self class name.
        aStream nextPutAll: 'An instance of '.
        aStream nextPutAll: (className copyFrom: 1 to: className size); nl.
	aStream nextPutAll: '  title: '.
	aStream nextPutAll: title printString; nl.
	aStream nextPutAll: '  colour: '.
	aStream nextPutAll: colour printString; nl.
	aStream nextPutAll: '  quality: '.
        aStream nextPutAll: quality printString; nl.
        aStream nextPutAll: '  charsize: '.
        aStream nextPutAll: charsize printString; nl

!!
nil
!


OneDGraph subclass:#OneDVErrors
	instanceVariableNames: 'earray'
	classVariableNames: ' DefaultEArray'
	poolDictionaries: ' '
	category: nil

!
nil
!


! OneDVErrors class methodsFor: 'instance creation' !

new
	^super new initOneDVErrors

!

defaultEArray
		^DefaultEArray
!

defaultEArray: aEA
		DefaultEArray _ aEA
!!
nil
!
OneDVErrors defaultEArray: nil!


! OneDVErrors methodsFor: 'earray' !

earray
	^earray
!
					" Make sure that a Real Array	"
earray: aEA				" has been entered		"
	earray _ aEA.
(earray class = GXRealarray) 
ifFalse:[ 'EARRAY Error, ' pren.
	 'Enter an array of Real values' prinl.
	 earray _ DefaultEArray]
!!
nil
!


! OneDVErrors methodsFor: 'private' !

initOneDVErrors
	
	earray _ DefaultEArray

!!
nil
!


! OneDVErrors methodsFor: 'create' !
					" Creates Verrors and returns a	"
create					"value for objid               "	

(earray size = yarray size) 
ifTrue:[
objid _ newGraphics function: gxVErrorsCreate  parameters: ((Array new: 6)
			at: 1 put: xarray;
                        at: 2 put: yarray;
			at: 3 put: earray;
			at: 4 put: colour;
			at: 5 put: 1;
			at: 6 put: lineThickness;
			yourself)]
ifFalse:['EARRAY ERROR' prenl.
	 'earray MUST have the same number of elements as x/yarray' prinl]

!!
nil
!


! OneDVErrors methodsFor: 'printing' !

printOn: aStream

        | className |

        className _ self class name.
        aStream nextPutAll: 'An instance of '.
        aStream nextPutAll: (className copyFrom: 1 to: className size); nl.
       	aStream nextPutAll: '  xarray: '.
        (xarray class = GXRealarray)
        ifFalse:[ aStream nextPutAll: '  Parameter has not been specified'
                 printString; nl]
        ifTrue:[xarray printOn: aStream.
		 aStream nextPutAll: ' ' printString; nl].
        aStream nextPutAll: '  yarray: '.
        (yarray class = GXRealarray)
        ifFalse:[ aStream nextPutAll: '  Parameter has not been specified'
                 printString; nl]
        ifTrue:[yarray printOn: aStream.
		 aStream nextPutAll: ' ' printString; nl].
        aStream nextPutAll: '  earray: '.
        (earray class = GXRealarray)
        ifFalse:[ aStream nextPutAll: '  Parameter has not been specified'
                 printString; nl]
        ifTrue:[earray printOn: aStream.
		 aStream nextPutAll: ' ' printString; nl].
        aStream nextPutAll: '  colour: '.
        aStream nextPutAll: colour printString; nl.
        aStream nextPutAll: '  lineThickness: '.
        aStream nextPutAll: lineThickness printString; nl
!!
nil
!


OneDGraph subclass:#OneDHErrors
	instanceVariableNames: 'earray'
	classVariableNames: ' DefaultEArray'
	poolDictionaries: ' '
	category: nil

!
nil
!


! OneDHErrors class methodsFor: 'instance creation' !

new
	^super new initOneDHErrors
!

defaultEArray
		^DefaultEArray

!

defaultEArray: aEA
		DefaultEArray _ aEA

!!
nil
!
OneDHErrors defaultEArray: nil!


! OneDHErrors methodsFor: 'earray' !

earray
	^earray
!
					" Make sure that the user "
earray: aEA				" enters a Real Array	  "
	earray _ aEA.

(earray class = GXRealarray)
ifFalse:[ 'EARRAY Error, ' pren.
	 'Enter an array of Real values' prinl.
	 earray _ DefaultEArray]
!!
nil
!


! OneDHErrors methodsFor: 'private' !

initOneDHErrors
	
	earray _ DefaultEArray
!!
nil
!


! OneDHErrors methodsFor: 'create' !
					" Creates Herrors and returns a	"
create					" value for objid		"

((earray size = xarray size) & (earray size = yarray size) )
ifTrue:[
objid _ newGraphics function: gxHErrorsCreate  parameters: ((Array new: 6)
			at: 1 put: xarray;
                        at: 2 put: yarray;
			at: 3 put: earray;
			at: 4 put: colour;
			at: 5 put: 0;
			at: 6 put: lineThickness;
			yourself)]
ifFalse:['EARRAY ERROR' prenl.
	 'earray MUST have the same number of elements as x/yarray' prinl.]


!!
nil
!


! OneDHErrors methodsFor: 'printing' !

printOn: aStream

	| className |

        className _ self class name.
        aStream nextPutAll: 'An instance of '.
        aStream nextPutAll: (className copyFrom: 1 to: className size); nl.
        aStream nextPutAll: '  xarray: '.
       	(xarray class =GXRealarray)
        ifFalse:[ aStream nextPutAll: '  Parameter has not been specified'
                 printString; nl]
	ifTrue:[xarray printOn: aStream.
		 aStream nextPutAll: ' ' printString; nl].
        aStream nextPutAll: '  yarray: '.
       	(yarray class =GXRealarray)
        ifFalse:[ aStream nextPutAll: '  Parameter has not been specified'
                 printString; nl]
        ifTrue:[yarray printOn: aStream.
		 aStream nextPutAll: ' ' printString; nl].
	aStream nextPutAll: '  earray: '.
       	(earray class = GXRealarray)
        ifFalse:[ aStream nextPutAll: '  Parameter has not been specified'
                 printString; nl]
	ifTrue:[earray printOn: aStream.
		 aStream nextPutAll: ' ' printString; nl].
	aStream nextPutAll: '  colour: '.
        aStream nextPutAll: colour printString; nl.
        aStream nextPutAll: '  lineThickness: '.
        aStream nextPutAll: lineThickness printString; nl

!!
nil
!


OneDGraph subclass:#OneDErrors
	instanceVariableNames: 'earray vertical'
	classVariableNames: ' DefaultEArray DefaultVertical'
	poolDictionaries: ' '
	category: nil

!
nil
!


! OneDErrors class methodsFor: 'instance creation' !

new
	^super new initOneDErrors

!

defaultEArray
		^DefaultEArray
!

defaultEArray: aEA
		DefaultEArray _ aEA
!
defaultVertical
		^DefaultVertical
!
defaultVertical: aVe
		DefaultVertical _ aVe
!!
nil
!
OneDErrors defaultEArray: nil!
OneDErrors defaultVertical: 0!


! OneDErrors methodsFor: 'earray' !

earray
	^earray
!
					" Make sure that a Real Array	"
earray: aEA				" has been entered		"
	earray _ aEA.
(earray class = GXRealarray) 
ifFalse:[ 'EARRAY Error, ' pren.
	 'Enter an array of Real values' prinl.
	 earray _ DefaultEArray]
!!
nil
!


! OneDErrors methodsFor: 'vertical' !

vertical
	^vertical
!
vertical: aVe
	aVe notNil ifTrue: [ vertical _ aVe ]
!!
nil
!


! OneDErrors methodsFor: 'private' !

initOneDErrors
	
	earray _ DefaultEArray.
	vertical _ DefaultVertical

!!
nil
!


! OneDErrors methodsFor: 'create' !
					" Creates Verrors and returns a	"
create					"value for objid               "	

(earray size = yarray size) 
ifTrue:[
objid _ newGraphics function: gxErrorsCreate  parameters: ((Array new: 6)
			at: 1 put: xarray;
                        at: 2 put: yarray;
			at: 3 put: earray;
			at: 4 put: colour;
			at: 5 put: vertical;
			at: 6 put: lineThickness;
			yourself)]
ifFalse:['EARRAY ERROR' prenl.
	 'earray MUST have the same number of elements as x/yarray' prinl]

!!
nil
!


! OneDErrors methodsFor: 'printing' !

printOn: aStream

        | className |

        className _ self class name.
        aStream nextPutAll: 'An instance of '.
        aStream nextPutAll: (className copyFrom: 1 to: className size); nl.
       	aStream nextPutAll: '  xarray: '.
        (xarray class = GXRealarray)
        ifFalse:[ aStream nextPutAll: '  Parameter has not been specified'
                 printString; nl]
        ifTrue:[xarray printOn: aStream.
		 aStream nextPutAll: ' ' printString; nl].
        aStream nextPutAll: '  yarray: '.
        (yarray class = GXRealarray)
        ifFalse:[ aStream nextPutAll: '  Parameter has not been specified'
                 printString; nl]
        ifTrue:[yarray printOn: aStream.
		 aStream nextPutAll: ' ' printString; nl].
        aStream nextPutAll: '  earray: '.
        (earray class = GXRealarray)
        ifFalse:[ aStream nextPutAll: '  Parameter has not been specified'
                 printString; nl]
        ifTrue:[earray printOn: aStream.
		 aStream nextPutAll: ' ' printString; nl].
        aStream nextPutAll: '  colour: '.
        aStream nextPutAll: colour printString; nl.
        aStream nextPutAll: '  vertical: '.
        aStream nextPutAll: vertical printString; nl.
        aStream nextPutAll: '  lineThickness: '.
        aStream nextPutAll: lineThickness printString; nl
!!
nil
!


DrawableObjects subclass:#Multiplot
	instanceVariableNames: 'yarray xarray nHist chmin chmax ygap binCentre
				type' 
	classVariableNames: ' DefaultNHist DefaultChMin DefaultChMax 
			      DefaultYGap DefaultType DefaultBin  
			      DefaultXArray DefaultYArray ' 
	poolDictionaries: ' '
	category: nil
!
nil
!


! Multiplot class methodsFor: 'instance creation' !

new
	^super new initMultiplot

!

defaultNHist
		^DefaultNHist
!

defaultNHist: aNH
		DefaultNHist _ aNH
!

defaultChMin
		^DefaultChMin
!

defaultChMin: aMi
		DefaultChMin _ aMi
!

defaultChMax
		^DefaultChMax
!

defaultChMax: aMa
		DefaultChMax _ aMa
!

defaultYGap
		^DefaultYGap
!

defaultYGap: aYG
		DefaultYGap _ aYG
!

defaultType
		^DefaultType
!

defaultType: aTy
		DefaultType _ aTy
!

defaultBin
		^DefaultBin
!

defaultBin: aBi
		DefaultBin _ aBi
!

defaultXArray
		^DefaultXArray
!

defaultXArray: aXA
		DefaultXArray _ aXA
!

defaultYArray
		^DefaultYArray
!

defaultYArray: aYA
		DefaultYArray _ aYA
!!
nil
!
Multiplot defaultNHist: 1!
Multiplot defaultChMin: -1!
Multiplot defaultChMax: -1!
Multiplot defaultYGap: 1.0!
Multiplot defaultBin: 1!
Multiplot defaultType: gXDfull!
Multiplot defaultYArray: nil!
Multiplot defaultXArray: nil!


! Multiplot methodsFor: 'yarray' !

yarray
	^yarray
!
yarray: aYa
	yarray _ aYa.
(yarray class = GXRealarray)
ifFalse:['YARRAY ERROR' prenl.
	 'For yarray enter a realarray' prinl.
	 yarray _ DefaultYArray]
!!
nil
!


! Multiplot methodsFor: 'xarray' !

xarray
	^xarray
!

xarray: aXa
	xarray _ aXa.
(xarray class = GXRealarray)
ifFalse:['XARRAY ERROR' prenl.
	 'For xarray enter a realarray' prinl.
	 xarray _ DefaultXArray]
!!
nil
!


! Multiplot methodsFor: 'nHist' !

nHist
	^nHist
!

nHist: aNH
	nHist _ aNH.
(nHist = nil)
ifTrue:[nHist _ DefaultNHist].
(nHist class = Integer)
ifFalse:['NHIST ERROR' prenl.
	 'For nHist enter an integer' prinl.
	 nHist _ DefaultNHist]
!!
nil
!


! Multiplot methodsFor: 'chmin' !

chmin
	^chmin
!
chmin: aci
	chmin _ aci.
(chmin = nil)
ifTrue:[chmin _ DefaultChMin].
(chmin class = Integer)
ifFalse:['CHMIN ERROR' prenl.
	 'For chmin enter an integer' prinl.
	 chmin _ DefaultChMin]
!!
nil
!


! Multiplot methodsFor: 'chmax' !

chmax
	^chmax
!
chmax: aca
	chmax _ aca.
(chmax = nil)
ifTrue:[chmax _ DefaultChMax].
(chmax class = Integer)
ifFalse:['CHMAX ERROR' prenl.
	 'For chmax enter an Integer' prinl.
	 chmax _ DefaultChMax]
!!
nil
!


! Multiplot methodsFor: 'ygap' !

ygap
	^ygap
!
ygap: aYG
	ygap _ aYG.
(ygap = nil)
ifTrue:[ygap _ DefaultYGap].
(ygap class = Float)
ifFalse:['YARRAY ERROR' prenl.
	 'For ygap enter a float' prinl.
	 ygap _ DefaultYGap]
!!
nil
!


! Multiplot methodsFor: 'binCentre' !

binCentre
	^binCentre
!
binCentre: aBC
	binCentre _ aBC
!!
nil
!


! Multiplot methodsFor: 'type' !

type
	^type
!
type: aLT
	type _ aLT.
(type = nil)
ifTrue:[type _ DefaultType].
(type isKindOf: Symbol)
ifFalse:['TYPE ERROR' prenl.
	 type _ DefaultType]
!!
nil
!


! Multiplot methodsFor: 'private' !

initMultiplot

	yarray _ DefaultYArray.

	xarray _ DefaultXArray.

	nHist _ DefaultNHist.

	chmin _ DefaultChMin.

	chmax _ DefaultChMax.

	ygap _ DefaultYGap.

	binCentre _ DefaultBin.

	type _ DefaultType

!!
nil
!


! Multiplot methodsFor: 'create' !
create
|tmp|
tmp _ newGraphics function: gxCreateMultiplot parameters: ((Array new: 9)
                        at: 1 put: nHist;
                        at: 2 put: xarray;
                        at: 3 put: chmin;
                        at: 4 put: chmax;
                        at: 5 put: ygap;
                        at: 6 put: colour;
                        at: 7 put: binCentre;
                        at: 8 put: type;
                        at: 9 put: lineThickness;
                        yourself).

!!
nil
!


! Multiplot methodsFor: 'add' !

add
|tmp|
tmp _ newGraphics function: gxAddSpectra parameters: ((Array new: 2)
			at: 1 put: yarray;
		        at: 2 put: nHist;
			yourself)
!!
nil
!


! Multiplot methodsFor: 'addSpectra' !

addSpectra: spectraArray from: file with: grouping
|tmp|
tmp _ newGraphics function: gxAddSpectraArray parameters: ((Array new: 4)
			at: 1 put: (xarray size - 1);
		        at: 2 put: grouping;
		        at: 3 put: file;
		        at: 4 put: spectraArray;
			yourself)
!!
nil
!


! Multiplot methodsFor: 'make' !

make

objid _ newGraphics function: gxMultiplotMake parameters: ((Array new: 0)
			yourself).
!!
nil
!


! Multiplot methodsFor: 'printing' !

printOn: aStream

	|className|

        className _ self class name.
        aStream nextPutAll: 'An instance of '.
        aStream nextPutAll: (className copyFrom: 1 to: className size); nl.
        aStream nextPutAll: '  xarray: '.
        aStream nextPutAll: xarray printString; nl.
        aStream nextPutAll: '  yarray: '.
        aStream nextPutAll: yarray printString; nl.
        aStream nextPutAll: '  nHist: '.
        aStream nextPutAll: nHist printString; nl.
        aStream nextPutAll: '  chmin: '.
        aStream nextPutAll: chmin printString; nl.
        aStream nextPutAll: '  chmax: '.
        aStream nextPutAll: chmax printString; nl.
        aStream nextPutAll: '  ygap: '.
        aStream nextPutAll: ygap printString; nl.
        aStream nextPutAll: '  binCentre: '.
        aStream nextPutAll: binCentre printString; nl.
        aStream nextPutAll: '  line_type: '.
        aStream nextPutAll: type printString; nl.
	
!!
nil
!


OneDGraph subclass:#OneDHistogram
	instanceVariableNames: ' binCentre '
	classVariableNames: 'DefaultBinCentre '
	poolDictionaries: ' '
        category: nil
!
nil
!

! OneDHistogram class methodsFor: 'instance creation' !

new
	^super new initOneDHistogram
!
defaultBinCentre
                ^DefaultBinCentre
!

defaultBinCentre: aBi
                DefaultBinCentre _ aBi
!!
nil
!
OneDHistogram defaultBinCentre: 0!


! OneDHistogram methodsFor: 'binCentre' !

binCentre
        ^binCentre
!

binCentre: aBi
        binCentre _ aBi.
                                        " Makes sure that the user      "
((binCentre = 1) |
 (binCentre = 0)  )                    " enters 1 or 0 only            "
ifFalse:[(binCentre = nil)
	ifTrue:[binCentre _ DefaultBinCentre]
	ifFalse:['BinCentre Error, using Default value' prenl.
        	 'Enter "0" (not centred) or "1" (centred) ' prinl.
         	 binCentre _ DefaultBinCentre]]
!!
nil
!


! OneDHistogram methodsFor: 'private' !

initOneDHistogram

	binCentre _ DefaultBinCentre

!!
nil
!

! OneDHistogram methodsFor: 'create' !
					" Creates histogram and returns	"
create					" a value for objid	  	"
|tmp|
(binCentre = 0) ifTrue: [ tmp _ (yarray size + 1) ] ifFalse: [ tmp _ yarray size ].
(tmp = xarray size)
ifTrue:[
objid _ newGraphics function: gxHistogramCreate  parameters: ((Array new: 6)
                        at: 1 put: xarray;
			at: 2 put: yarray;
			at: 3 put: colour;
			at: 4 put: binCentre;
			at: 5 put: type;
			at: 6 put: lineThickness;
                        yourself)]

ifFalse:['ONEDHISTOGRAM ERROR' prenl.
	 'xarray and yarray inconsistent lengths' prinl]

!!
nil
!


! OneDHistogram methodsFor: 'printing' !

printOn: aStream

        | className |

        className _ self class name.
        aStream nextPutAll: 'An instance of '.
        aStream nextPutAll: (className copyFrom: 1 to: className size); nl.
        aStream nextPutAll: '  xarray: '.
       	(xarray class = GXRealarray)
        ifFalse:[ aStream nextPutAll: '  Parameter has not been specified'
                 printString; nl]
        ifTrue:[xarray printOn: aStream.
		 aStream nextPutAll: ' ' printString; nl].
        aStream nextPutAll: '  yarray: '.
        (yarray class = GXRealarray)
        ifFalse:[ aStream nextPutAll: '  Parameter has not been specified'
                 printString; nl]
        ifTrue:[yarray printOn: aStream.
		 aStream nextPutAll: ' ' printString; nl].
       	aStream nextPutAll: '  colour: ' printString.  
	aStream nextPutAll: colour printString; nl.
	aStream nextPutAll: '  binCentre: '.
	aStream nextPutAll: binCentre printString; nl.
	aStream nextPutAll: '  type: '.
	aStream nextPutAll: type printString; nl.
	aStream nextPutAll: '  lineThickness: '.
        aStream nextPutAll: lineThickness printString; nl
!!
nil
!


! Picture methodsFor: 'whichWindow' !

whichWindow: tmp

|points count number world a b c e d f g h WXmin WYmin|

points _ Array new: 2.
count _ 0.
number _ 0.
world _ GXWorkspace new. 
points at: 1 put: (tmp at: #gXdUx).
points at: 2 put: (tmp at: #gXdUy).

windowList do:[: i | count _ count + 1. 
          ((i xMin <= (points at: 1)) &           (i xMax >= (points at: 1)) &           (i yMin <= (points at: 2)) &           (i yMax >= (points at: 2))  )
	   ifTrue:[number _ 1. 
		   a _ i xMax.
		   b _ i xMin.
		   c _ i yMin.
		   d _ i yMax.
		   e _ i lxMin.
		   f _ i lxMax.
		   g _ i lyMin.
		   h _ i lyMax.
		   (i class ~= Window) 
		   ifTrue:[
			WXmin _ ((((points at: 1) - b)/ (a - b)) * (f - e)) + e.
			WYmin _ ((((points at: 2) - c)/ (d - c)) * (h - g)) + g.

			world at: #gXa put: WXmin.
			world at: #gXb put: WYmin.
			world at: #gXc put: count]
		ifFalse:[
			 WXmin _ ((f /(a - b )) *((points at: 1) - b)).
			 WYmin _ ((h /(d - c )) * ((points at: 2) - c)).

			  world at: #gXa put: WXmin.
			  world at: #gXb put: WYmin.
			  world at: #gXc put: count]]


	  ifFalse:[number _ number + 0.
		   ]].

(number = 1)
ifTrue:[^world]
ifFalse:[^nil]

!!
nil
!
"Procedure class definition:
 ==========================="
((Smalltalk at: #GXaxes ifAbsent: [ nil ]) isKindOf: Function class) ifTrue: [ ('Warning: Overriding previously defined procedure ', 'GXaxes' gtype) prinl ].
Smalltalk at: #GXaxes put: nil!
Function subclass: 'GXaxes'
       instanceVariableNames: ''
       classVariableNames: ''
       poolDictionaries: ''
       category: 'Genie Procedure'
!

"HEADER
 ------"

"qualifiers
 ----------"

GXaxes

addInstVarName: #gXalter;
compile:
'gXalter
	gXalter _ true';
compile:
'gXalter: aBoolean
	gXalter _ aBoolean';

addInstVarName: #gXdraw;
compile:
'gXdraw
	gXdraw _ true';
compile:
'gXdraw: aBoolean
	gXdraw _ aBoolean';

addInstVarName: #gXxlog;
compile:
'gXxlog
	gXxlog _ true';
compile:
'gXxlog: aBoolean
	gXxlog _ aBoolean';

addInstVarName: #gXylog;
compile:
'gXylog
	gXylog _ true';
compile:
'gXylog: aBoolean
	gXylog _ aBoolean';

addInstVarName: #gXyhoriz;
compile:
'gXyhoriz
	gXyhoriz _ true';
compile:
'gXyhoriz: aBoolean
	gXyhoriz _ aBoolean';

addInstVarName: #gXnoxnum;
compile:
'gXnoxnum
	gXnoxnum _ true';
compile:
'gXnoxnum: aBoolean
	gXnoxnum _ aBoolean';

addInstVarName: #gXnoynum;
compile:
'gXnoynum
	gXnoynum _ true';
compile:
'gXnoynum: aBoolean
	gXnoynum _ aBoolean';

addInstVarName: #gXxlinear;
compile:
'gXxlinear
	gXxlinear _ true';
compile:
'gXxlinear: aBoolean
	gXxlinear _ aBoolean';

addInstVarName: #gXylinear;
compile:
'gXylinear
	gXylinear _ true';
compile:
'gXylinear: aBoolean
	gXylinear _ aBoolean'!

"qualifiers
 ----------"

GXaxes

addInstVarName: #gXyvertical;
compile:
'gXyvertical
	gXyvertical _ true';
compile:
'gXyvertical: aBoolean
	gXyvertical _ aBoolean';

addInstVarName: #gXxnum;
compile:
'gXxnum
	gXxnum _ true';
compile:
'gXxnum: aBoolean
	gXxnum _ aBoolean';

addInstVarName: #gXynum;
compile:
'gXynum
	gXynum _ true';
compile:
'gXynum: aBoolean
	gXynum _ aBoolean'!

"parameters
 ----------"

GXaxes

addInstVarName: #gXcolour;
compile:
'gXcolour: aVar
	aVar notNil ifTrue:[gXcolour _ self typeCheck: aVar shouldBe: GXColour paramName: ''gXcolour'']';
compile:
'p1: aVar
	aVar notNil ifTrue:[gXcolour _ self typeCheck: aVar shouldBe: GXColour paramName: ''gXcolour'']';

addInstVarName: #gXsize;
compile:
'gXsize: aVar
	aVar notNil ifTrue:[gXsize _ self typeCheck: aVar shouldBe: GXReal paramName: ''gXsize'']';
compile:
'p2: aVar
	aVar notNil ifTrue:[gXsize _ self typeCheck: aVar shouldBe: GXReal paramName: ''gXsize'']';

addInstVarName: #gXfont;
compile:
'gXfont: aVar
	aVar notNil ifTrue:[gXfont _ self typeCheck: aVar shouldBe: GXFont paramName: ''gXfont'']';
compile:
'p3: aVar
	aVar notNil ifTrue:[gXfont _ self typeCheck: aVar shouldBe: GXFont paramName: ''gXfont'']';

addInstVarName: #gXlineUthickness;
compile:
'gXlineUthickness: aVar
	aVar notNil ifTrue:[gXlineUthickness _ self typeCheck: aVar shouldBe: GXReal paramName: ''gXlineUthickness'']';
compile:
'p4: aVar
	aVar notNil ifTrue:[gXlineUthickness _ self typeCheck: aVar shouldBe: GXReal paramName: ''gXlineUthickness'']'!

"parameters
 ----------"

GXaxes

addInstVarName: #gXlineUtype;
compile:
'gXlineUtype: aVar
	aVar notNil ifTrue:[gXlineUtype _ self typeCheck: aVar shouldBe: GXLinestyle paramName: ''gXlineUtype'']';
compile:
'p5: aVar
	aVar notNil ifTrue:[gXlineUtype _ self typeCheck: aVar shouldBe: GXLinestyle paramName: ''gXlineUtype'']';

addInstVarName: #gXobject;
compile:
'gXobject: aVar
	aVar notNil ifTrue:[gXobject _ self typeCheck: aVar shouldBe: GXGobject paramName: ''gXobject'']';
compile:
'p6: aVar
	aVar notNil ifTrue:[gXobject _ self typeCheck: aVar shouldBe: GXGobject paramName: ''gXobject'']'!

"result
 ------"

GXaxes

addInstVarName: #gXres
!

!GXaxes methodsFor: 'initializing'!

initVars
	gXalter _ false.
	gXdraw _ false.
	gXxlog _ false.
	gXylog _ false.
	gXyhoriz _ false.
	gXnoxnum _ false.
	gXnoynum _ false.
	gXxlinear _ false.
	gXylinear _ false.
	gXyvertical _ false.
	gXxnum _ false.
	gXynum _ false
!!

"BODY
 ----"

!GXaxes methodsFor: 'running'!

evaluate


   |tmp a b c d e any object dev pic wind|
((gXalter) & (gXdraw)  ) 
ifTrue:['ERROR, ' pren.
	'You Cannot draw and Alter at the same time' prinl]
ifFalse:[
(gXUUstatus = 0)
   ifTrue:['ERROR, graphics device is not open' prenl.
           'To open enter DEVICE/OPEN' prinl]
   ifFalse:[gXdraw ifTrue:[ gXxlog ifTrue:[(OneDAxes defaultXLog: 1)]
                          ifFalse:[(OneDAxes defaultXLog: 0)].
                   gXylog ifTrue:[(OneDAxes defaultYLog: 1)]
                          ifFalse:[(OneDAxes defaultYLog: 0)].
                   gXyhoriz ifTrue:[(OneDAxes defaultYHoriz: 1)]
                            ifFalse:[(OneDAxes defaultYHoriz: 0)].
                   gXnoxnum ifTrue:[(OneDAxes defaultXNum: 0)]
                          ifFalse:[(OneDAxes defaultXNum: 1)].
                   gXnoynum ifTrue:[(OneDAxes defaultYNum: 0)]
                          ifFalse:[(OneDAxes defaultYNum: 1)].
		   gXxlinear ifTrue:[(OneDAxes defaultXLog: 0)].
		   gXylinear ifTrue:[(OneDAxes defaultYLog: 0)].
		   gXxnum ifTrue:[(OneDAxes defaultXNum: 1)].
		   gXynum ifTrue:[(OneDAxes defaultYNum: 1)].
		   gXyvertical ifTrue:[(OneDAxes defaultYHoriz: 0)].
                   tmp _ (OneDAxes new colour: gXcolour; charsize: gXsize;
                          quality: gXfont; lineThickness: gXlineUthickness;
		          type: gXlineUtype; create).
   ((gXUUgraphics at: #gXcurrentwindow) = nil)
            ifTrue:['DRAW ERROR, ' pren.
		    'Window has not been created' prinl]
            ifFalse:[ ((gXUUgraphics at: #gXcurrentwindow) addToList: tmp).
		     a _ ((gXUUgraphics at: #gXcurrentwindow) getNumber: nil).
                      (gXUUgraphics at: #gXcurrentwindow) drawOnTop: a.
		     gXres _ tmp]]].


   gXalter ifTrue:[
     (gXobject = nil) ifTrue:['object not specified' prinl].
     (gXobject class = OneDAxes)
     ifTrue:[ object _ ((gXUUgraphics at: #gXcurrentgraphics)
         			    	  findDrawableObjectsPicture: gXobject).
              (object class = OneDAxes)
	      ifTrue:[pic _ ((gXUUgraphics at: #gXcurrentgraphics) 
							findPicture: object).
 	   	      dev _ ((gXUUgraphics at: #gXcurrentgraphics) 
							       findDevice: pic).
	   	      wind _ (pic findRightWindow: object).

        	gXxlog ifTrue:[(a _ 1)]
                        ifFalse:[gXxlinear ifTrue:[(a _ 0)]
                                           ifFalse:[a _ (object xlog)]].
                 gXylog ifTrue:[(b _ 1)]
                        ifFalse:[gXylinear ifTrue:[(b _ 0)]
                                           ifFalse:[b _ (object ylog)]].
                 gXyhoriz ifTrue:[(c _ 1)]
                          ifFalse:[gXyvertical ifTrue:[(c _ 0)]
                                               ifFalse:[c _ (object yhoriz)]].
                 gXxnum ifTrue:[(d _ 1)]
                        ifFalse:[gXnoxnum ifTrue:[(d _ 0)]
                                          ifFalse:[d _ (object xnum)]].
                 gXynum ifTrue:[(e _ 1)]
                        ifFalse:[gXnoynum ifTrue:[(e _ 0)]
                                          ifFalse:[e _ (object ynum)]].
                 (gXcolour isNil)
                 ifFalse:[object _ (object colour: gXcolour; xlog: a; ylog: b;
                          xnum: d; ynum: e; yhoriz: c; create)].
                 (gXsize isNil)
                 ifFalse:[object _ (object charsize: gXsize; xlog: a; ylog: b;
                                  xnum: d; ynum: e; yhoriz: c; create)].
                 (gXfont isNil)
                 ifFalse:[object _ (object quality: gXfont; xlog: a; ylog: b;
                                   xnum: d; ynum: e; yhoriz: c; create)].
		 (gXlineUthickness isNil)
		 ifFalse:[ object _ (object lineThickness: gXlineUthickness;
				     xlog: a; ylog: b; xnum: d; ynum: e;
				     yhoriz: c; create)].
		 (gXlineUtype isNil)
		 ifFalse: [object _ (object type: gXlineUtype; xlog: a; ylog: b;
				    xnum: d; ynum: e; yhoriz: c; create)].
                 ((gXcolour isNil) &                  (gXsize isNil) &		  (gXlineUthickness isNil) &		  (gXlineUtype isNil) &                  (gXfont isNil) )
                 ifTrue:[ object _ (object xlog: a; ylog: b; yhoriz: c; 
				      xnum: d; ynum: e; create)].
                 (object class = OneDAxes)
	         ifTrue:[gXobject update: object. 
	 	     	  pic drawAllWindows]	
		 ifFalse:['Object has not been altered' prinl]]]]].

(gXalter | gXdraw)
ifFalse:['ERROR, ' pren.
	 '/DRAW or /ALTER has not been specified' prinl]

.
self up.
^gXres
!!
nil! "(fix for smalltalk bug, this line is ignored if there is a variable declaration error)"

"End of procedure"

"Procedure class definition:
 ==========================="
((Smalltalk at: #GXgraticule ifAbsent: [ nil ]) isKindOf: Function class) ifTrue: [ ('Warning: Overriding previously defined procedure ', 'GXgraticule' gtype) prinl ].
Smalltalk at: #GXgraticule put: nil!
Function subclass: 'GXgraticule'
       instanceVariableNames: ''
       classVariableNames: ''
       poolDictionaries: ''
       category: 'Genie Procedure'
!

"HEADER
 ------"

"qualifiers
 ----------"

GXgraticule

addInstVarName: #gXalter;
compile:
'gXalter
	gXalter _ true';
compile:
'gXalter: aBoolean
	gXalter _ aBoolean';

addInstVarName: #gXdraw;
compile:
'gXdraw
	gXdraw _ true';
compile:
'gXdraw: aBoolean
	gXdraw _ aBoolean';

addInstVarName: #gXxlog;
compile:
'gXxlog
	gXxlog _ true';
compile:
'gXxlog: aBoolean
	gXxlog _ aBoolean';

addInstVarName: #gXylog;
compile:
'gXylog
	gXylog _ true';
compile:
'gXylog: aBoolean
	gXylog _ aBoolean';

addInstVarName: #gXxnum;
compile:
'gXxnum
	gXxnum _ true';
compile:
'gXxnum: aBoolean
	gXxnum _ aBoolean';

addInstVarName: #gXynum;
compile:
'gXynum
	gXynum _ true';
compile:
'gXynum: aBoolean
	gXynum _ aBoolean';

addInstVarName: #gXyhoriz;
compile:
'gXyhoriz
	gXyhoriz _ true';
compile:
'gXyhoriz: aBoolean
	gXyhoriz _ aBoolean';

addInstVarName: #gXnoxnum;
compile:
'gXnoxnum
	gXnoxnum _ true';
compile:
'gXnoxnum: aBoolean
	gXnoxnum _ aBoolean';

addInstVarName: #gXnoynum;
compile:
'gXnoynum
	gXnoynum _ true';
compile:
'gXnoynum: aBoolean
	gXnoynum _ aBoolean'!

"qualifiers
 ----------"

GXgraticule

addInstVarName: #gXyvertical;
compile:
'gXyvertical
	gXyvertical _ true';
compile:
'gXyvertical: aBoolean
	gXyvertical _ aBoolean';

addInstVarName: #gXxlinear;
compile:
'gXxlinear
	gXxlinear _ true';
compile:
'gXxlinear: aBoolean
	gXxlinear _ aBoolean';

addInstVarName: #gXylinear;
compile:
'gXylinear
	gXylinear _ true';
compile:
'gXylinear: aBoolean
	gXylinear _ aBoolean'!

"parameters
 ----------"

GXgraticule

addInstVarName: #gXcolour;
compile:
'gXcolour: aVar
	aVar notNil ifTrue:[gXcolour _ self typeCheck: aVar shouldBe: GXColour paramName: ''gXcolour'']';
compile:
'p1: aVar
	aVar notNil ifTrue:[gXcolour _ self typeCheck: aVar shouldBe: GXColour paramName: ''gXcolour'']';

addInstVarName: #gXsize;
compile:
'gXsize: aVar
	aVar notNil ifTrue:[gXsize _ self typeCheck: aVar shouldBe: GXReal paramName: ''gXsize'']';
compile:
'p2: aVar
	aVar notNil ifTrue:[gXsize _ self typeCheck: aVar shouldBe: GXReal paramName: ''gXsize'']';

addInstVarName: #gXfont;
compile:
'gXfont: aVar
	aVar notNil ifTrue:[gXfont _ self typeCheck: aVar shouldBe: GXFont paramName: ''gXfont'']';
compile:
'p3: aVar
	aVar notNil ifTrue:[gXfont _ self typeCheck: aVar shouldBe: GXFont paramName: ''gXfont'']';

addInstVarName: #gXlineUthickness;
compile:
'gXlineUthickness: aVar
	aVar notNil ifTrue:[gXlineUthickness _ self typeCheck: aVar shouldBe: GXReal paramName: ''gXlineUthickness'']';
compile:
'p4: aVar
	aVar notNil ifTrue:[gXlineUthickness _ self typeCheck: aVar shouldBe: GXReal paramName: ''gXlineUthickness'']'!

"parameters
 ----------"

GXgraticule

addInstVarName: #gXlineUtype;
compile:
'gXlineUtype: aVar
	aVar notNil ifTrue:[gXlineUtype _ self typeCheck: aVar shouldBe: GXLinestyle paramName: ''gXlineUtype'']';
compile:
'p5: aVar
	aVar notNil ifTrue:[gXlineUtype _ self typeCheck: aVar shouldBe: GXLinestyle paramName: ''gXlineUtype'']';

addInstVarName: #gXobject;
compile:
'gXobject: aVar
	aVar notNil ifTrue:[gXobject _ self typeCheck: aVar shouldBe: GXGobject paramName: ''gXobject'']';
compile:
'p6: aVar
	aVar notNil ifTrue:[gXobject _ self typeCheck: aVar shouldBe: GXGobject paramName: ''gXobject'']'!

"result
 ------"

GXgraticule

addInstVarName: #gXres
!

!GXgraticule methodsFor: 'initializing'!

initVars
	gXalter _ false.
	gXdraw _ false.
	gXxlog _ false.
	gXylog _ false.
	gXxnum _ false.
	gXynum _ false.
	gXyhoriz _ false.
	gXnoxnum _ false.
	gXnoynum _ false.
	gXyvertical _ false.
	gXxlinear _ false.
	gXylinear _ false
!!

"BODY
 ----"

!GXgraticule methodsFor: 'running'!

evaluate

   |tmp a b c d e dev object pic wind|
((gXalter) & (gXdraw)  ) 
ifTrue:['ERROR, ' pren.
	'You Cannot draw and Alter at the same time' prinl]
ifFalse:[
   (gXUUstatus = 0)
   ifTrue:['ERROR, graphics device is not open' prenl.
           'To open enter DEVICE/OPEN' prinl]
   ifFalse:[gXdraw 
	    ifTrue:[ gXxlog ifTrue:[(OneDGraticule defaultXLog: 1)]
                          ifFalse:[(OneDGraticule defaultXLog: 0)].
                   gXylog ifTrue:[(OneDGraticule defaultYLog: 1)]
                          ifFalse:[(OneDGraticule defaultYLog: 0)].
		   gXyhoriz ifTrue:[(OneDGraticule defaultYHoriz: 1)]
                            ifFalse:[(OneDGraticule defaultYHoriz: 0)].
                   gXxnum ifTrue:[(OneDGraticule defaultXNum: 1)]
                          ifFalse:[(OneDGraticule defaultXNum: 0)].
                   gXynum ifTrue:[(OneDGraticule defaultYNum: 1)]
                          ifFalse:[(OneDGraticule defaultYNum: 0)].
		   gXxlinear ifTrue:[(OneDGraticule defaultXLog: 0)].
		   gXylinear ifTrue:[(OneDGraticule defaultYLog: 0)].
		   gXnoxnum ifTrue:[(OneDGraticule defaultXNum: 0)].
                   gXnoynum ifTrue:[(OneDGraticule defaultYNum: 0)].
                   gXyvertical ifTrue:[(OneDGraticule defaultYHoriz: 0)].
                   tmp _ (OneDGraticule new colour: gXcolour; charsize: gXsize;
                          quality: gXfont; lineThickness: gXlineUthickness;
			  type: gXlineUtype; create).
                  ((gXUUgraphics at: #gXcurrentwindow) = nil)
                          ifTrue:['DRAW ERROR, ' pren.
				  'Window has not been created' prinl]
                ifFalse:[ ((gXUUgraphics at: #gXcurrentwindow) addToList: tmp).
                      a _ ((gXUUgraphics at: #gXcurrentwindow) getNumber: nil).
                      (gXUUgraphics at: #gXcurrentwindow) drawOnTop: a.
		     gXres _ tmp]]].

   gXalter ifTrue:[gXobject isNil ifTrue:['object not specified' prinl].
	   	   (gXobject class = OneDGraticule)
	  	   ifTrue:[object _ ((gXUUgraphics at: #gXcurrentgraphics)
					  findDrawableObjectsPicture: gXobject).
		   (object class = OneDGraticule)
		   ifTrue:[pic _ ((gXUUgraphics at: #gXcurrentgraphics)
							 findPicture: object).
			    dev _ ((gXUUgraphics at: #gXcurrentgraphics)
							       findDevice: pic).
			    wind _ (pic findRightWindow: object).

                  	   gXxlog ifTrue:[(a _ 1)]
                           	 ifFalse:[gXxlinear ifTrue:[(a _ 0)]
			 		      	   ifFalse:[a _ (object xlog)]].
                   	   gXylog ifTrue:[(b _ 1)]
                          	 ifFalse:[gXylinear ifTrue:[(b _ 0)]
                                         	   ifFalse:[b _ (object ylog)]].
		   	   gXyhoriz ifTrue:[(c _ 1)]
                            	   ifFalse:[gXyvertical ifTrue:[(c _ 0)]
                                        	 ifFalse:[c _ (object yhoriz)]].
                   	   gXxnum ifTrue:[(d _ 1)]
                          	  ifFalse:[gXnoxnum ifTrue:[(d _ 0)]
                                          	   ifFalse:[d _ (object xnum)]].
                   	   gXynum ifTrue:[(e _ 1)]
                          	  ifFalse:[gXnoynum ifTrue:[(e _ 0)]
                                          	   ifFalse:[e _ (object ynum)]].
                   	   (gXcolour isNil)
                   	   ifFalse:[object _ (object colour: gXcolour; xlog: a; 
				    	      ylog: b; xnum: d; ynum: e; 
					      yhoriz: c; create)].
                   	   (gXsize isNil)
                   	   ifFalse:[object _ (object charsize: gXsize; xlog: a; 
					      ylog: b; xnum: d; ynum: e; 
					      yhoriz: c; create)].
                   	   (gXfont isNil)
                   	   ifFalse:[object _ (object quality: gXfont; 
					      xlog: a; ylog: b; xnum: d; 
					      ynum: e; yhoriz: c; create)].
		   	   (gXlineUthickness isNil)
		    	   ifFalse:[object _ (object xlog: a; ylog: b;
					      lineThickness: gXlineUthickness; 
					      xnum: d; ynum: e; create)].
		   	   (gXlineUtype isNil)
		   	   ifFalse: [object _ (object type: gXlineUtype; 
					       xlog: a; ylog: b; xnum: d; 
					       ynum: e; yhoriz: c; create)].
                   	   ((gXcolour isNil) &                    	   (gXsize isNil) &		    	   (gXlineUthickness isNil) &		    	   (gXlineUtype isNil) &                    	   (gXfont isNil) )
                   	   ifTrue:[ object _ (object xlog: a; ylog: b; xnum: d;
					      ynum: e; yhoriz: c; create)].
                   	   (object class = OneDGraticule)
                	   ifTrue:[gXobject update: object.
				    pic drawAllWindows]
			   ifFalse:['Object has not been altered' prinl]]]]].

(gXalter | gXdraw)
ifFalse:['ERROR, ' pren.
	 '/DRAW or /ALTER has not been specified' prinl]

.
self up.
^gXres
!!
nil! "(fix for smalltalk bug, this line is ignored if there is a variable declaration error)"

"End of procedure"

"Procedure class definition:
 ==========================="
((Smalltalk at: #GXtitle ifAbsent: [ nil ]) isKindOf: Function class) ifTrue: [ ('Warning: Overriding previously defined procedure ', 'GXtitle' gtype) prinl ].
Smalltalk at: #GXtitle put: nil!
Function subclass: 'GXtitle'
       instanceVariableNames: ''
       classVariableNames: ''
       poolDictionaries: ''
       category: 'Genie Procedure'
!

"HEADER
 ------"

"qualifiers
 ----------"

GXtitle

addInstVarName: #gXalter;
compile:
'gXalter
	gXalter _ true';
compile:
'gXalter: aBoolean
	gXalter _ aBoolean';

addInstVarName: #gXdraw;
compile:
'gXdraw
	gXdraw _ true';
compile:
'gXdraw: aBoolean
	gXdraw _ aBoolean'!

"parameters
 ----------"

GXtitle

addInstVarName: #gXtext;
compile:
'gXtext: aVar
	aVar notNil ifTrue:[gXtext _ self typeCheck: aVar shouldBe: GXString paramName: ''gXtext'']';
compile:
'p1: aVar
	aVar notNil ifTrue:[gXtext _ self typeCheck: aVar shouldBe: GXString paramName: ''gXtext'']';

addInstVarName: #gXcolour;
compile:
'gXcolour: aVar
	aVar notNil ifTrue:[gXcolour _ self typeCheck: aVar shouldBe: GXColour paramName: ''gXcolour'']';
compile:
'p2: aVar
	aVar notNil ifTrue:[gXcolour _ self typeCheck: aVar shouldBe: GXColour paramName: ''gXcolour'']';

addInstVarName: #gXsize;
compile:
'gXsize: aVar
	aVar notNil ifTrue:[gXsize _ self typeCheck: aVar shouldBe: GXReal paramName: ''gXsize'']';
compile:
'p3: aVar
	aVar notNil ifTrue:[gXsize _ self typeCheck: aVar shouldBe: GXReal paramName: ''gXsize'']';

addInstVarName: #gXfont;
compile:
'gXfont: aVar
	aVar notNil ifTrue:[gXfont _ self typeCheck: aVar shouldBe: GXFont paramName: ''gXfont'']';
compile:
'p4: aVar
	aVar notNil ifTrue:[gXfont _ self typeCheck: aVar shouldBe: GXFont paramName: ''gXfont'']'!

"parameters
 ----------"

GXtitle

addInstVarName: #gXobject;
compile:
'gXobject: aVar
	aVar notNil ifTrue:[gXobject _ self typeCheck: aVar shouldBe: GXGobject paramName: ''gXobject'']';
compile:
'p5: aVar
	aVar notNil ifTrue:[gXobject _ self typeCheck: aVar shouldBe: GXGobject paramName: ''gXobject'']'!

"result
 ------"

GXtitle

addInstVarName: #gXres
!

!GXtitle methodsFor: 'initializing'!

initVars
	gXalter _ false.
	gXdraw _ false
!!

"BODY
 ----"

!GXtitle methodsFor: 'running'!

evaluate

   |tmp a pic dev wind object|
((gXalter) & (gXdraw)  ) 
ifTrue:['ERROR, ' pren.
	'You Cannot draw and Alter at the same time' prinl]
ifFalse:[
   (gXUUstatus = 0)
   ifTrue:['ERROR, graphics device is not open' prenl.
           'To open enter DEVICE/OPEN' prinl]
   ifFalse:[gXdraw 
	    ifTrue:[ tmp _ (Title new title: gXtext; colour: gXcolour;
                          charsize: gXsize; quality: gXfont; create).
                  ((gXUUgraphics at: #gXcurrentwindow) = nil)
                          ifTrue:['DRAW ERROR, ' pren.
				  'Window has not been created' prinl]
                ifFalse:[ ((gXUUgraphics at: #gXcurrentwindow) addToList: tmp).
                    a _ ((gXUUgraphics at: #gXcurrentwindow) getNumber: nil).
                      (gXUUgraphics at: #gXcurrentwindow) drawOnTop: a.
		     gXres _ tmp]]].

   gXalter ifTrue:[(gXobject = nil) ifTrue:['object not specified' prinl. ^nil].
		   (gXobject class = Title)
	  	   	   ifTrue:[object _ ((gXUUgraphics at: #gXcurrentgraphics)
					  findDrawableObjectsPicture: gXobject).
			   pic _ ((gXUUgraphics at: #gXcurrentgraphics)
							findPicture: gXobject).
				    dev _ ((gXUUgraphics at: #gXcurrentgraphics)
							findDevice: pic).
				    wind _ (pic findRightWindow: gXobject).
		   (gXtext = nil)
                   ifFalse:[ object title: gXtext ].
                   (gXcolour = nil)
                   ifFalse:[ object colour: gXcolour ].
                   (gXsize = nil)
                   ifFalse:[ object charsize: gXsize ].
                   (gXfont = nil)
                   ifFalse:[ object quality: gXfont ].
	  	   object create. gXobject update: object.
                   pic drawAllWindows]
                   ifFalse:['Object has not been altered' prinl]]].

(gXalter | gXdraw)
ifFalse:['ERROR, ' pren.
	 '/DRAW or /ALTER has not been specified' prinl]

.
self up.
^gXres
!!
nil! "(fix for smalltalk bug, this line is ignored if there is a variable declaration error)"

"End of procedure"

"Procedure class definition:
 ==========================="
((Smalltalk at: #GXlabels ifAbsent: [ nil ]) isKindOf: Function class) ifTrue: [ ('Warning: Overriding previously defined procedure ', 'GXlabels' gtype) prinl ].
Smalltalk at: #GXlabels put: nil!
Function subclass: 'GXlabels'
       instanceVariableNames: ''
       classVariableNames: ''
       poolDictionaries: ''
       category: 'Genie Procedure'
!

"HEADER
 ------"

"qualifiers
 ----------"

GXlabels

addInstVarName: #gXalter;
compile:
'gXalter
	gXalter _ true';
compile:
'gXalter: aBoolean
	gXalter _ aBoolean';

addInstVarName: #gXdraw;
compile:
'gXdraw
	gXdraw _ true';
compile:
'gXdraw: aBoolean
	gXdraw _ aBoolean';

addInstVarName: #gXxhorizontal;
compile:
'gXxhorizontal
	gXxhorizontal _ true';
compile:
'gXxhorizontal: aBoolean
	gXxhorizontal _ aBoolean';

addInstVarName: #gXxvertical;
compile:
'gXxvertical
	gXxvertical _ true';
compile:
'gXxvertical: aBoolean
	gXxvertical _ aBoolean';

addInstVarName: #gXyvertical;
compile:
'gXyvertical
	gXyvertical _ true';
compile:
'gXyvertical: aBoolean
	gXyvertical _ aBoolean';

addInstVarName: #gXyhorizontal;
compile:
'gXyhorizontal
	gXyhorizontal _ true';
compile:
'gXyhorizontal: aBoolean
	gXyhorizontal _ aBoolean'!

"parameters
 ----------"

GXlabels

addInstVarName: #gXxlabel;
compile:
'gXxlabel: aVar
	aVar notNil ifTrue:[gXxlabel _ self typeCheck: aVar shouldBe: GXString paramName: ''gXxlabel'']';
compile:
'p1: aVar
	aVar notNil ifTrue:[gXxlabel _ self typeCheck: aVar shouldBe: GXString paramName: ''gXxlabel'']';

addInstVarName: #gXylabel;
compile:
'gXylabel: aVar
	aVar notNil ifTrue:[gXylabel _ self typeCheck: aVar shouldBe: GXString paramName: ''gXylabel'']';
compile:
'p2: aVar
	aVar notNil ifTrue:[gXylabel _ self typeCheck: aVar shouldBe: GXString paramName: ''gXylabel'']';

addInstVarName: #gXsize;
compile:
'gXsize: aVar
	aVar notNil ifTrue:[gXsize _ self typeCheck: aVar shouldBe: GXReal paramName: ''gXsize'']';
compile:
'p3: aVar
	aVar notNil ifTrue:[gXsize _ self typeCheck: aVar shouldBe: GXReal paramName: ''gXsize'']';

addInstVarName: #gXcolour;
compile:
'gXcolour: aVar
	aVar notNil ifTrue:[gXcolour _ self typeCheck: aVar shouldBe: GXColour paramName: ''gXcolour'']';
compile:
'p4: aVar
	aVar notNil ifTrue:[gXcolour _ self typeCheck: aVar shouldBe: GXColour paramName: ''gXcolour'']';

addInstVarName: #gXfont;
compile:
'gXfont: aVar
	aVar notNil ifTrue:[gXfont _ self typeCheck: aVar shouldBe: GXFont paramName: ''gXfont'']';
compile:
'p5: aVar
	aVar notNil ifTrue:[gXfont _ self typeCheck: aVar shouldBe: GXFont paramName: ''gXfont'']'!

"parameters
 ----------"

GXlabels

addInstVarName: #gXobject;
compile:
'gXobject: aVar
	aVar notNil ifTrue:[gXobject _ self typeCheck: aVar shouldBe: GXGobject paramName: ''gXobject'']';
compile:
'p6: aVar
	aVar notNil ifTrue:[gXobject _ self typeCheck: aVar shouldBe: GXGobject paramName: ''gXobject'']'!

"result
 ------"

GXlabels

addInstVarName: #gXres
!

!GXlabels methodsFor: 'initializing'!

initVars
	gXalter _ false.
	gXdraw _ false.
	gXxhorizontal _ false.
	gXxvertical _ false.
	gXyvertical _ false.
	gXyhorizontal _ false
!!

"BODY
 ----"

!GXlabels methodsFor: 'running'!

evaluate

   |tmp a b dev pic wind object|

((gXalter) & (gXdraw)  ) 
ifTrue:['ERROR, ' pren.
	'You Cannot draw and Alter at the same time' prinl]
ifFalse:[
   (gXUUstatus = 0)
   ifTrue:['ERROR, graphics device is not open' prenl.
           'To open enter DEVICE/OPEN' prinl]
   ifFalse:[gXdraw 
	    ifTrue:[gXyvertical ifTrue:[(Labels defaultYVert: 1)]
                                ifFalse:[(Labels defaultYVert: 1)].
                    gXxhorizontal ifTrue:[(Labels defaultXVert: 0)]
                                  ifFalse:[(Labels defaultXVert: 0)].
		    gXxvertical ifTrue:[(Labels defaultXVert: 1)].
		    gXyhorizontal ifTrue:[(Labels defaultYVert: 0)].
                    tmp _ (Labels new xlabel: gXxlabel; ylabel: gXylabel;
                          charsize: gXsize; colour: gXcolour;
                          quality: gXfont; create).
                  ((gXUUgraphics at: #gXcurrentwindow) = nil)
                          ifTrue:['DRAW ERROR, ' pren.
				  'Window has not been created' prinl]
                	  ifFalse:[ ((gXUUgraphics at: #gXcurrentwindow) 
								addToList: tmp).
                     	  a _ ((gXUUgraphics at: #gXcurrentwindow) 
								getNumber: nil).
                      	  (gXUUgraphics at: #gXcurrentwindow) drawOnTop: a.
		     	  gXres _ tmp]]].

   gXalter ifTrue:[(gXobject = nil) ifTrue:['object not specified' prinl].
		   (gXobject class = Labels)
		   ifTrue:[object _ ((gXUUgraphics at: #gXcurrentgraphics)
					  findDrawableObjectsPicture: gXobject).
			   (object class = Labels)
			   ifTrue:[pic _ ((gXUUgraphics at: #gXcurrentgraphics)
							 findPicture: gXobject).
				    dev _ ((gXUUgraphics at: #gXcurrentgraphics)
							       findDevice: pic).
				    wind _ (pic findRightWindow: gXobject).
                   	    gXyvertical 
			    ifTrue:[(a _ 1)]
                            ifFalse:[gXyhorizontal ifTrue:[a _ 0]
				           	 ifFalse:[a _ (object xvert)]].
                   	    gXxhorizontal 
			    ifTrue:[(b _ 0)]
                            ifFalse:[gXxvertical ifTrue:[b _ 1]
					         ifFalse:[b _ (object yvert)]].
                   (gXxlabel isNil)
                   ifFalse:[object _ (object xlabel: gXxlabel; xvert: b; 
				      yvert: a; create)].
                   (gXylabel isNil)
                   ifFalse:[object _ (object ylabel: gXylabel; xvert: b; 
				      yvert: a; create)].
                   (gXsize isNil)
                   ifFalse:[object _ (object charsize: gXsize; xvert: b; 
			 	      yvert: a; create)].
                   (gXcolour isNil)
                   ifFalse:[object _ (object colour: gXcolour; xvert: b; 
				      yvert: a; create)].
                   (gXfont isNil)
                   ifFalse:[object _ (object quality: gXfont; xvert: b; 
				      yvert: a; create)].
                   ((gXxlabel isNil) &                    (gXylabel isNil) &                    (gXcolour isNil) &                    (gXsize isNil)   &                    (gXfont isNil) )
                   ifTrue:[object _ (object xvert: b; yvert: a; create)].

                   (object class = Labels)
                   ifTrue:[gXobject update: object.
			    pic drawAllWindows]
                   ifFalse:['Object has not been altered' prinl]]]]].

(gXalter | gXdraw)
ifFalse:['ERROR, ' pren.
	 '/DRAW or /ALTER has not been specified' prinl]


.
self up.
^gXres
!!
nil! "(fix for smalltalk bug, this line is ignored if there is a variable declaration error)"

"End of procedure"

"Procedure class definition:
 ==========================="
((Smalltalk at: #GXline ifAbsent: [ nil ]) isKindOf: Function class) ifTrue: [ ('Warning: Overriding previously defined procedure ', 'GXline' gtype) prinl ].
Smalltalk at: #GXline put: nil!
Function subclass: 'GXline'
       instanceVariableNames: ''
       classVariableNames: ''
       poolDictionaries: ''
       category: 'Genie Procedure'
!

"HEADER
 ------"

"qualifiers
 ----------"

GXline

addInstVarName: #gXalter;
compile:
'gXalter
	gXalter _ true';
compile:
'gXalter: aBoolean
	gXalter _ aBoolean';

addInstVarName: #gXdraw;
compile:
'gXdraw
	gXdraw _ true';
compile:
'gXdraw: aBoolean
	gXdraw _ aBoolean'!

"parameters
 ----------"

GXline

addInstVarName: #gXxstart;
compile:
'gXxstart: aVar
	aVar notNil ifTrue:[gXxstart _ self typeCheck: aVar shouldBe: GXReal paramName: ''gXxstart'']';
compile:
'p1: aVar
	aVar notNil ifTrue:[gXxstart _ self typeCheck: aVar shouldBe: GXReal paramName: ''gXxstart'']';

addInstVarName: #gXystart;
compile:
'gXystart: aVar
	aVar notNil ifTrue:[gXystart _ self typeCheck: aVar shouldBe: GXReal paramName: ''gXystart'']';
compile:
'p2: aVar
	aVar notNil ifTrue:[gXystart _ self typeCheck: aVar shouldBe: GXReal paramName: ''gXystart'']';

addInstVarName: #gXxend;
compile:
'gXxend: aVar
	aVar notNil ifTrue:[gXxend _ self typeCheck: aVar shouldBe: GXReal paramName: ''gXxend'']';
compile:
'p3: aVar
	aVar notNil ifTrue:[gXxend _ self typeCheck: aVar shouldBe: GXReal paramName: ''gXxend'']';

addInstVarName: #gXyend;
compile:
'gXyend: aVar
	aVar notNil ifTrue:[gXyend _ self typeCheck: aVar shouldBe: GXReal paramName: ''gXyend'']';
compile:
'p4: aVar
	aVar notNil ifTrue:[gXyend _ self typeCheck: aVar shouldBe: GXReal paramName: ''gXyend'']';

addInstVarName: #gXcolour;
compile:
'gXcolour: aVar
	aVar notNil ifTrue:[gXcolour _ self typeCheck: aVar shouldBe: GXColour paramName: ''gXcolour'']';
compile:
'p5: aVar
	aVar notNil ifTrue:[gXcolour _ self typeCheck: aVar shouldBe: GXColour paramName: ''gXcolour'']'!

"parameters
 ----------"

GXline

addInstVarName: #gXlineUtype;
compile:
'gXlineUtype: aVar
	aVar notNil ifTrue:[gXlineUtype _ self typeCheck: aVar shouldBe: GXLinestyle paramName: ''gXlineUtype'']';
compile:
'p6: aVar
	aVar notNil ifTrue:[gXlineUtype _ self typeCheck: aVar shouldBe: GXLinestyle paramName: ''gXlineUtype'']';

addInstVarName: #gXlineUthickness;
compile:
'gXlineUthickness: aVar
	aVar notNil ifTrue:[gXlineUthickness _ self typeCheck: aVar shouldBe: GXReal paramName: ''gXlineUthickness'']';
compile:
'p7: aVar
	aVar notNil ifTrue:[gXlineUthickness _ self typeCheck: aVar shouldBe: GXReal paramName: ''gXlineUthickness'']';

addInstVarName: #gXobject;
compile:
'gXobject: aVar
	aVar notNil ifTrue:[gXobject _ self typeCheck: aVar shouldBe: GXGobject paramName: ''gXobject'']';
compile:
'p8: aVar
	aVar notNil ifTrue:[gXobject _ self typeCheck: aVar shouldBe: GXGobject paramName: ''gXobject'']'!

"result
 ------"

GXline

addInstVarName: #gXres
!

!GXline methodsFor: 'initializing'!

initVars
	gXalter _ false.
	gXdraw _ false
!!

"BODY
 ----"

!GXline methodsFor: 'running'!

evaluate

   |tmp a pic wind dev object|
((gXalter) & (gXdraw)  ) 
ifTrue:['ERROR, ' pren.
	'You Cannot draw and Alter at the same time' prinl]
ifFalse:[
   (gXUUstatus = 0)
   ifTrue:['ERROR, graphics device is not open' prenl.
           'To open enter DEVICE/OPEN' prinl]
   ifFalse:[gXdraw 
	    ifTrue:[tmp _ (Polyline new xstart: gXxstart; ystart: gXystart;
                           xend: gXxend; yend: gXyend; colour: gXcolour;
                           type: gXlineUtype; lineThickness: gXlineUthickness;
                           create).
                    ((gXUUgraphics at: #gXcurrentwindow) = nil)
                    ifTrue:['DRAW ERROR, ' pren.
		  	    'Window has not been created' prinl]
                    ifFalse:[((gXUUgraphics at: #gXcurrentwindow) 
							addToList: tmp).
                    		   a _ ((gXUUgraphics at: #gXcurrentwindow) 
								getNumber: nil).
                      		   (gXUUgraphics at: #gXcurrentwindow) 
								   drawOnTop: a.
		     		   gXres _ tmp]]].

   gXalter ifTrue:[(gXobject isNil) ifTrue:['object not specified' prinl].
		   (gXobject class = Polyline)
		   ifTrue:[object _ ((gXUUgraphics at: #gXcurrentgraphics)
					 findDrawableObjectsPicture: gXobject).
			   (object class = Polyline)
			   ifTrue:[pic _ ((gXUUgraphics at: #gXcurrentgraphics)
							findPicture: object).
				    dev _ ((gXUUgraphics at: #gXcurrentgraphics)
							findDevice: pic).
				    wind _ (pic findRightWindow: object).
                   (gXxstart = nil)
                   ifFalse:[object _ (object xstart: gXxstart; create)].
                   (gXxend = nil)
                   ifFalse:[object _ (object xend: gXxend; create)].
                   (gXystart = nil)
                   ifFalse:[object _ (object  ystart: gXystart; create)].
                   (gXyend = nil)
                   ifFalse:[object _ (object yend: gXyend;  create)].
                   (gXcolour = nil)
                   ifFalse:[object _ (object colour: gXcolour; create)].
                   (gXlineUtype = nil)
                   ifFalse:[object _ (object type: gXlineUtype; create)].
                   (gXlineUthickness = nil)
                   ifFalse:[object _ (object lineThickness: gXlineUthickness; 
				   create)].
                (object class = Polyline)
                ifTrue:[gXobject update: object.
			     pic drawAllWindows]
                ifFalse:['Object has not been altered' prinl]]]]].

(gXalter | gXdraw)
ifFalse:['ERROR, ' pren.
	 '/DRAW or /ALTER has not been specified' prinl]


.
self up.
^gXres
!!
nil! "(fix for smalltalk bug, this line is ignored if there is a variable declaration error)"

"End of procedure"

"Procedure class definition:
 ==========================="
((Smalltalk at: #GXpolygon ifAbsent: [ nil ]) isKindOf: Function class) ifTrue: [ ('Warning: Overriding previously defined procedure ', 'GXpolygon' gtype) prinl ].
Smalltalk at: #GXpolygon put: nil!
Function subclass: 'GXpolygon'
       instanceVariableNames: ''
       classVariableNames: ''
       poolDictionaries: ''
       category: 'Genie Procedure'
!

"HEADER
 ------"

"qualifiers
 ----------"

GXpolygon

addInstVarName: #gXalter;
compile:
'gXalter
	gXalter _ true';
compile:
'gXalter: aBoolean
	gXalter _ aBoolean';

addInstVarName: #gXdraw;
compile:
'gXdraw
	gXdraw _ true';
compile:
'gXdraw: aBoolean
	gXdraw _ aBoolean';

addInstVarName: #gXfilled;
compile:
'gXfilled
	gXfilled _ true';
compile:
'gXfilled: aBoolean
	gXfilled _ aBoolean';

addInstVarName: #gXoutline;
compile:
'gXoutline
	gXoutline _ true';
compile:
'gXoutline: aBoolean
	gXoutline _ aBoolean'!

"parameters
 ----------"

GXpolygon

addInstVarName: #gXxarray;
compile:
'gXxarray: aVar
	aVar notNil ifTrue:[gXxarray _ self typeCheck: aVar shouldBe: GXRealarray paramName: ''gXxarray'']';
compile:
'p1: aVar
	aVar notNil ifTrue:[gXxarray _ self typeCheck: aVar shouldBe: GXRealarray paramName: ''gXxarray'']';

addInstVarName: #gXyarray;
compile:
'gXyarray: aVar
	aVar notNil ifTrue:[gXyarray _ self typeCheck: aVar shouldBe: GXRealarray paramName: ''gXyarray'']';
compile:
'p2: aVar
	aVar notNil ifTrue:[gXyarray _ self typeCheck: aVar shouldBe: GXRealarray paramName: ''gXyarray'']';

addInstVarName: #gXcolour;
compile:
'gXcolour: aVar
	aVar notNil ifTrue:[gXcolour _ self typeCheck: aVar shouldBe: GXColour paramName: ''gXcolour'']';
compile:
'p3: aVar
	aVar notNil ifTrue:[gXcolour _ self typeCheck: aVar shouldBe: GXColour paramName: ''gXcolour'']'!

"parameters
 ----------"

GXpolygon

addInstVarName: #gXlineUthickness;
compile:
'gXlineUthickness: aVar
	aVar notNil ifTrue:[gXlineUthickness _ self typeCheck: aVar shouldBe: GXReal paramName: ''gXlineUthickness'']';
compile:
'p4: aVar
	aVar notNil ifTrue:[gXlineUthickness _ self typeCheck: aVar shouldBe: GXReal paramName: ''gXlineUthickness'']';

addInstVarName: #gXobject;
compile:
'gXobject: aVar
	aVar notNil ifTrue:[gXobject _ self typeCheck: aVar shouldBe: GXGobject paramName: ''gXobject'']';
compile:
'p5: aVar
	aVar notNil ifTrue:[gXobject _ self typeCheck: aVar shouldBe: GXGobject paramName: ''gXobject'']'!

"result
 ------"

GXpolygon

addInstVarName: #gXres
!

!GXpolygon methodsFor: 'initializing'!

initVars
	gXalter _ false.
	gXdraw _ false.
	gXfilled _ false.
	gXoutline _ false
!!

"BODY
 ----"

!GXpolygon methodsFor: 'running'!

evaluate

   |tmp a dev pic wind object|

((gXalter) & (gXdraw)  ) 
ifTrue:['ERROR, ' pren.
	'You Cannot draw and Alter at the same time' prinl]
ifFalse:[
   (gXUUstatus = 0)
   ifTrue:['ERROR, graphics device is not open' prenl.
           'To open enter DEVICE/OPEN' prinl]
   ifFalse:[gXdraw 
	    ifTrue:[gXfilled ifTrue:[(Polygon defaultSolid: 1)]
                           ifFalse:[(Polygon defaultSolid: 0)].
		  gXoutline ifTrue:[(Polygon defaultSolid: 0)].
                     tmp _ (Polygon new xcoord: gXxarray; ycoord: gXyarray;
                            colour: gXcolour; lineThickness: gXlineUthickness;
                            create).
                  ((gXUUgraphics at: #gXcurrentwindow) = nil)
                          ifTrue:['DRAW ERROR, ' pren.
				  'Window has not been created' prinl]
                ifFalse:[ ((gXUUgraphics at: #gXcurrentwindow) addToList: tmp).
                     a _ ((gXUUgraphics at: #gXcurrentwindow) getNumber: nil).
                      (gXUUgraphics at: #gXcurrentwindow) drawOnTop: a.
		     gXres _ tmp]]].

   gXalter ifTrue:[
	(gXobject isNil) ifTrue:['object not specified' prinl].
        (gXobject class = Polygon)
	ifTrue:[object _ ((gXUUgraphics at: #gXcurrentgraphics)
					findDrawableObjectsPicture: gXobject).
                (object class = Polygon)
		ifTrue:[pic _ ((gXUUgraphics at: #gXcurrentgraphics)
							findPicture: object).
			dev _ ((gXUUgraphics at: #gXcurrentgraphics)
							findDevice: pic).
			wind _ (pic findRightWindow: object).

                        gXfilled
                        ifTrue:[(a _ 1)]
                        ifFalse:[gXoutline ifTrue:[(a _ 0)]
                                          ifFalse:[a _ (object solid)]].
                   (gXxarray isNil)
                   ifFalse:['XArray not Yet Implemented' prnl].
                   (gXyarray isNil)
                   ifFalse:['YArray not Yet Implemented' prnl].
                   (gXcolour isNil)
                   ifFalse:[object _ (object colour: gXcolour; solid: a; 
				      create)].
                   (gXlineUthickness isNil)
                   ifFalse:[object _ (object lineThickness: gXlineUthickness;
                                   solid: a; create)].
                   ((gXcolour isNil) &                   (gXlineUthickness isNil) )
                   ifTrue:[object _ (object solid: a; create)].
                   (object class = Polygon)
                   ifTrue:[gXobject update: object.
			  pic drawAllWindows]
                   ifFalse:['Object has not been altered' prinl]]]]].

(gXalter | gXdraw)
ifFalse:['ERROR, ' pren.
	 '/DRAW or /ALTER has not been specified' prinl]

                                                                  
.
self up.
^gXres
!!
nil! "(fix for smalltalk bug, this line is ignored if there is a variable declaration error)"

"End of procedure"

"Procedure class definition:
 ==========================="
((Smalltalk at: #GXtext ifAbsent: [ nil ]) isKindOf: Function class) ifTrue: [ ('Warning: Overriding previously defined procedure ', 'GXtext' gtype) prinl ].
Smalltalk at: #GXtext put: nil!
Function subclass: 'GXtext'
       instanceVariableNames: ''
       classVariableNames: ''
       poolDictionaries: ''
       category: 'Genie Procedure'
!

"HEADER
 ------"

"qualifiers
 ----------"

GXtext

addInstVarName: #gXalter;
compile:
'gXalter
	gXalter _ true';
compile:
'gXalter: aBoolean
	gXalter _ aBoolean';

addInstVarName: #gXdraw;
compile:
'gXdraw
	gXdraw _ true';
compile:
'gXdraw: aBoolean
	gXdraw _ aBoolean';

addInstVarName: #gXcentre;
compile:
'gXcentre
	gXcentre _ true';
compile:
'gXcentre: aBoolean
	gXcentre _ aBoolean'!

"parameters
 ----------"

GXtext

addInstVarName: #gXxcoord;
compile:
'gXxcoord: aVar
	aVar notNil ifTrue:[gXxcoord _ self typeCheck: aVar shouldBe: GXReal paramName: ''gXxcoord'']';
compile:
'p1: aVar
	aVar notNil ifTrue:[gXxcoord _ self typeCheck: aVar shouldBe: GXReal paramName: ''gXxcoord'']';

addInstVarName: #gXycoord;
compile:
'gXycoord: aVar
	aVar notNil ifTrue:[gXycoord _ self typeCheck: aVar shouldBe: GXReal paramName: ''gXycoord'']';
compile:
'p2: aVar
	aVar notNil ifTrue:[gXycoord _ self typeCheck: aVar shouldBe: GXReal paramName: ''gXycoord'']';

addInstVarName: #gXtext;
compile:
'gXtext: aVar
	aVar notNil ifTrue:[gXtext _ self typeCheck: aVar shouldBe: GXString paramName: ''gXtext'']';
compile:
'p3: aVar
	aVar notNil ifTrue:[gXtext _ self typeCheck: aVar shouldBe: GXString paramName: ''gXtext'']';

addInstVarName: #gXcolour;
compile:
'gXcolour: aVar
	aVar notNil ifTrue:[gXcolour _ self typeCheck: aVar shouldBe: GXColour paramName: ''gXcolour'']';
compile:
'p4: aVar
	aVar notNil ifTrue:[gXcolour _ self typeCheck: aVar shouldBe: GXColour paramName: ''gXcolour'']';

addInstVarName: #gXsize;
compile:
'gXsize: aVar
	aVar notNil ifTrue:[gXsize _ self typeCheck: aVar shouldBe: GXReal paramName: ''gXsize'']';
compile:
'p5: aVar
	aVar notNil ifTrue:[gXsize _ self typeCheck: aVar shouldBe: GXReal paramName: ''gXsize'']'!

"parameters
 ----------"

GXtext

addInstVarName: #gXangle;
compile:
'gXangle: aVar
	aVar notNil ifTrue:[gXangle _ self typeCheck: aVar shouldBe: GXReal paramName: ''gXangle'']';
compile:
'p6: aVar
	aVar notNil ifTrue:[gXangle _ self typeCheck: aVar shouldBe: GXReal paramName: ''gXangle'']';

addInstVarName: #gXfont;
compile:
'gXfont: aVar
	aVar notNil ifTrue:[gXfont _ self typeCheck: aVar shouldBe: GXFont paramName: ''gXfont'']';
compile:
'p7: aVar
	aVar notNil ifTrue:[gXfont _ self typeCheck: aVar shouldBe: GXFont paramName: ''gXfont'']';

addInstVarName: #gXlinethickness;
compile:
'gXlinethickness: aVar
	aVar notNil ifTrue:[gXlinethickness _ self typeCheck: aVar shouldBe: GXReal paramName: ''gXlinethickness'']';
compile:
'p8: aVar
	aVar notNil ifTrue:[gXlinethickness _ self typeCheck: aVar shouldBe: GXReal paramName: ''gXlinethickness'']';

addInstVarName: #gXobject;
compile:
'gXobject: aVar
	aVar notNil ifTrue:[gXobject _ self typeCheck: aVar shouldBe: GXGobject paramName: ''gXobject'']';
compile:
'p9: aVar
	aVar notNil ifTrue:[gXobject _ self typeCheck: aVar shouldBe: GXGobject paramName: ''gXobject'']'!

"result
 ------"

GXtext

addInstVarName: #gXres
!

!GXtext methodsFor: 'initializing'!

initVars
	gXalter _ false.
	gXdraw _ false.
	gXcentre _ false
!!

"BODY
 ----"

!GXtext methodsFor: 'running'!

evaluate

   |tmp a dev pic wind object just|

   gXcentre ifTrue: [ just _ 0.5 ] ifFalse: [ just _ 0.0 ].
((gXalter) & (gXdraw)  ) 
ifTrue:['ERROR, ' pren.
	'You Cannot draw and Alter at the same time' prinl]
ifFalse:[
   (gXUUstatus = 0)
   ifTrue:['ERROR, graphics device is not open' prenl.
           'To open enter DEVICE/OPEN' prinl]
   ifFalse:[gXdraw 
	    ifTrue:[tmp _ (Text new xcoord: gXxcoord; ycoord: gXycoord;
                         words: gXtext; colour: gXcolour; size: gXsize;
                         angle: gXangle; quality: gXfont; justify: just;
			 lineThickness: gXlinethickness; create).
                  ((gXUUgraphics at: #gXcurrentwindow) = nil)
                          ifTrue:['DRAW ERROR, ' pren.
				  'Window has not been created' prinl]
                ifFalse:[ ((gXUUgraphics at: #gXcurrentwindow) addToList: tmp).
                      a _ ((gXUUgraphics at: #gXcurrentwindow) getNumber: nil).
                      (gXUUgraphics at: #gXcurrentwindow) drawOnTop: a.
		     gXres _ tmp]]].

   gXalter ifTrue:[
	(gXobject = nil) ifTrue:['object not specified' prinl].
        (gXobject class =  Text)
	ifTrue:[object _ ((gXUUgraphics at: #gXcurrentgraphics)
                                          findDrawableObjectsPicture: gXobject).
               (object class = Text)
               ifTrue:[pic _ ((gXUUgraphics at: #gXcurrentgraphics)
                                                        findPicture: object).
                       dev _ ((gXUUgraphics at: #gXcurrentgraphics)
                                                              findDevice: pic).
                       wind _ (pic findRightWindow: object).

                   object _ object justify: just.
                   (gXxcoord isNil)
                   ifFalse:[object _ (object xcoord: gXxcoord; create)].
                   (gXycoord isNil)
                   ifFalse:[object _ (object ycoord: gXycoord; create)].
                   (gXtext isNil)
                   ifFalse:[object _ (object words: gXtext; create)].
                   (gXcolour isNil)
                   ifFalse:[object _ (object colour: gXcolour; create)].
                   (gXlinethickness isNil)
                   ifFalse:[object _ (object lineThickness: gXlinethickness; 
				      create)].
                   (gXsize isNil)
                   ifFalse:[object _ (object size: gXsize; create)].
                   (gXangle isNil)
                   ifFalse:[object _ (object angle: gXangle; create)].
                   (gXfont isNil)
                   ifFalse:[object _ (object quality: gXfont; create)].


                   (object class = Text) 
                   ifTrue:[gXobject update: object.
			   pic drawAllWindows]
                   ifFalse:['Object has not been altered' prinl]]]]].

(gXalter | gXdraw)
ifFalse:['ERROR, ' pren.
	 '/DRAW or /ALTER has not been specified' prinl]


.
self up.
^gXres
!!
nil! "(fix for smalltalk bug, this line is ignored if there is a variable declaration error)"

"End of procedure"

"Procedure class definition:
 ==========================="
((Smalltalk at: #GXgraph ifAbsent: [ nil ]) isKindOf: Function class) ifTrue: [ ('Warning: Overriding previously defined procedure ', 'GXgraph' gtype) prinl ].
Smalltalk at: #GXgraph put: nil!
Function subclass: 'GXgraph'
       instanceVariableNames: ''
       classVariableNames: ''
       poolDictionaries: ''
       category: 'Genie Procedure'
!

"HEADER
 ------"

"qualifiers
 ----------"

GXgraph

addInstVarName: #gXalter;
compile:
'gXalter
	gXalter _ true';
compile:
'gXalter: aBoolean
	gXalter _ aBoolean';

addInstVarName: #gXdraw;
compile:
'gXdraw
	gXdraw _ true';
compile:
'gXdraw: aBoolean
	gXdraw _ aBoolean'!

"parameters
 ----------"

GXgraph

addInstVarName: #gXxarray;
compile:
'gXxarray: aVar
	aVar notNil ifTrue:[gXxarray _ self typeCheck: aVar shouldBe: GXRealarray paramName: ''gXxarray'']';
compile:
'p1: aVar
	aVar notNil ifTrue:[gXxarray _ self typeCheck: aVar shouldBe: GXRealarray paramName: ''gXxarray'']';

addInstVarName: #gXyarray;
compile:
'gXyarray: aVar
	aVar notNil ifTrue:[gXyarray _ self typeCheck: aVar shouldBe: GXRealarray paramName: ''gXyarray'']';
compile:
'p2: aVar
	aVar notNil ifTrue:[gXyarray _ self typeCheck: aVar shouldBe: GXRealarray paramName: ''gXyarray'']';

addInstVarName: #gXcolour;
compile:
'gXcolour: aVar
	aVar notNil ifTrue:[gXcolour _ self typeCheck: aVar shouldBe: GXColour paramName: ''gXcolour'']';
compile:
'p3: aVar
	aVar notNil ifTrue:[gXcolour _ self typeCheck: aVar shouldBe: GXColour paramName: ''gXcolour'']';

addInstVarName: #gXlineUtype;
compile:
'gXlineUtype: aVar
	aVar notNil ifTrue:[gXlineUtype _ self typeCheck: aVar shouldBe: GXLinestyle paramName: ''gXlineUtype'']';
compile:
'p4: aVar
	aVar notNil ifTrue:[gXlineUtype _ self typeCheck: aVar shouldBe: GXLinestyle paramName: ''gXlineUtype'']'!

"parameters
 ----------"

GXgraph

addInstVarName: #gXlineUthickness;
compile:
'gXlineUthickness: aVar
	aVar notNil ifTrue:[gXlineUthickness _ self typeCheck: aVar shouldBe: GXReal paramName: ''gXlineUthickness'']';
compile:
'p5: aVar
	aVar notNil ifTrue:[gXlineUthickness _ self typeCheck: aVar shouldBe: GXReal paramName: ''gXlineUthickness'']';

addInstVarName: #gXobject;
compile:
'gXobject: aVar
	aVar notNil ifTrue:[gXobject _ self typeCheck: aVar shouldBe: GXGobject paramName: ''gXobject'']';
compile:
'p6: aVar
	aVar notNil ifTrue:[gXobject _ self typeCheck: aVar shouldBe: GXGobject paramName: ''gXobject'']'!

"result
 ------"

GXgraph

addInstVarName: #gXres
!

!GXgraph methodsFor: 'initializing'!

initVars
	gXalter _ false.
	gXdraw _ false
!!

"BODY
 ----"

!GXgraph methodsFor: 'running'!

evaluate

   |tmp a dev wind pic object|
((gXalter) & (gXdraw)  ) 
ifTrue:['ERROR, ' pren.
	'You Cannot draw and Alter at the same time' prinl]
ifFalse:[
   (gXUUstatus = 0)
   ifTrue:['ERROR, graphics device is not open' prenl.
           'To open enter DEVICE/OPEN' prinl]
   ifFalse:[gXdraw 
	    ifTrue:[tmp _ (OneDGraph new xarray: gXxarray; yarray: gXyarray;
                         colour: gXcolour; type: gXlineUtype;
                         lineThickness: gXlineUthickness; create).
                  ((gXUUgraphics at: #gXcurrentwindow) = nil)
                          ifTrue:['DRAW ERROR, ' pren.
				  'Window has not been created' prinl]
                ifFalse:[ ((gXUUgraphics at: #gXcurrentwindow) addToList: tmp).
                     a _ ((gXUUgraphics at: #gXcurrentwindow) getNumber: nil).
                      (gXUUgraphics at: #gXcurrentwindow) drawOnTop: a.
		     gXres _ tmp]]].

   gXalter ifTrue:[
	(gXobject = nil) ifTrue:['object not specified' prinl].
        (gXobject class = OneDGraph)
        ifTrue:[object _ ((gXUUgraphics at: #gXcurrentgraphics)
                                          findDrawableObjectsPicture: gXobject).
                (object class = OneDGraph)
                ifTrue:[pic _ ((gXUUgraphics at: #gXcurrentgraphics)
                                                         findPicture: object).
                        dev _ ((gXUUgraphics at: #gXcurrentgraphics)
                                                               findDevice: pic).
                        wind _ (pic findRightWindow: object).

                   (gXxarray isNil)
                   ifFalse:['XArray Not Yet Implemented' prnl].
                   (gXyarray isNil)
                   ifFalse:['YArray Not Yet Implemented' prnl].
		   (gXcolour isNil)
		   ifFalse:[object _ (object colour: gXcolour; create)].
                  (gXlineUtype isNil)
                   ifFalse:[object _ (object type: gXlineUtype; create)].
                   (gXlineUthickness isNil)
                   ifFalse:[object _ (object lineThickness: gXlineUthickness;
                                   create)].

                   (object class = OneDGraph)
                   ifTrue:[gXobject update: object.
			   pic drawAllWindows]
                   ifFalse:['Object has not been altered' prinl]]]]].

(gXalter | gXdraw)
ifFalse:['ERROR, ' pren.
	 '/DRAW or /ALTER has not been specified' prinl]


.
self up.
^gXres
!!
nil! "(fix for smalltalk bug, this line is ignored if there is a variable declaration error)"

"End of procedure"

"Procedure class definition:
 ==========================="
((Smalltalk at: #GXhistogram ifAbsent: [ nil ]) isKindOf: Function class) ifTrue: [ ('Warning: Overriding previously defined procedure ', 'GXhistogram' gtype) prinl ].
Smalltalk at: #GXhistogram put: nil!
Function subclass: 'GXhistogram'
       instanceVariableNames: ''
       classVariableNames: ''
       poolDictionaries: ''
       category: 'Genie Procedure'
!

"HEADER
 ------"

"qualifiers
 ----------"

GXhistogram

addInstVarName: #gXalter;
compile:
'gXalter
	gXalter _ true';
compile:
'gXalter: aBoolean
	gXalter _ aBoolean';

addInstVarName: #gXdraw;
compile:
'gXdraw
	gXdraw _ true';
compile:
'gXdraw: aBoolean
	gXdraw _ aBoolean';

addInstVarName: #gXcentre;
compile:
'gXcentre
	gXcentre _ true';
compile:
'gXcentre: aBoolean
	gXcentre _ aBoolean';

addInstVarName: #gXnotcentred;
compile:
'gXnotcentred
	gXnotcentred _ true';
compile:
'gXnotcentred: aBoolean
	gXnotcentred _ aBoolean'!

"parameters
 ----------"

GXhistogram

addInstVarName: #gXxarray;
compile:
'gXxarray: aVar
	aVar notNil ifTrue:[gXxarray _ self typeCheck: aVar shouldBe: GXRealarray paramName: ''gXxarray'']';
compile:
'p1: aVar
	aVar notNil ifTrue:[gXxarray _ self typeCheck: aVar shouldBe: GXRealarray paramName: ''gXxarray'']';

addInstVarName: #gXyarray;
compile:
'gXyarray: aVar
	aVar notNil ifTrue:[gXyarray _ self typeCheck: aVar shouldBe: GXRealarray paramName: ''gXyarray'']';
compile:
'p2: aVar
	aVar notNil ifTrue:[gXyarray _ self typeCheck: aVar shouldBe: GXRealarray paramName: ''gXyarray'']';

addInstVarName: #gXcolour;
compile:
'gXcolour: aVar
	aVar notNil ifTrue:[gXcolour _ self typeCheck: aVar shouldBe: GXColour paramName: ''gXcolour'']';
compile:
'p3: aVar
	aVar notNil ifTrue:[gXcolour _ self typeCheck: aVar shouldBe: GXColour paramName: ''gXcolour'']';

addInstVarName: #gXlineUtype;
compile:
'gXlineUtype: aVar
	aVar notNil ifTrue:[gXlineUtype _ self typeCheck: aVar shouldBe: GXLinestyle paramName: ''gXlineUtype'']';
compile:
'p4: aVar
	aVar notNil ifTrue:[gXlineUtype _ self typeCheck: aVar shouldBe: GXLinestyle paramName: ''gXlineUtype'']'!

"parameters
 ----------"

GXhistogram

addInstVarName: #gXlineUthickness;
compile:
'gXlineUthickness: aVar
	aVar notNil ifTrue:[gXlineUthickness _ self typeCheck: aVar shouldBe: GXReal paramName: ''gXlineUthickness'']';
compile:
'p5: aVar
	aVar notNil ifTrue:[gXlineUthickness _ self typeCheck: aVar shouldBe: GXReal paramName: ''gXlineUthickness'']';

addInstVarName: #gXobject;
compile:
'gXobject: aVar
	aVar notNil ifTrue:[gXobject _ self typeCheck: aVar shouldBe: GXGobject paramName: ''gXobject'']';
compile:
'p6: aVar
	aVar notNil ifTrue:[gXobject _ self typeCheck: aVar shouldBe: GXGobject paramName: ''gXobject'']'!

"result
 ------"

GXhistogram

addInstVarName: #gXres
!

!GXhistogram methodsFor: 'initializing'!

initVars
	gXalter _ false.
	gXdraw _ false.
	gXcentre _ false.
	gXnotcentred _ false
!!

"BODY
 ----"

!GXhistogram methodsFor: 'running'!

evaluate

   |tmp a dev pic wind object|

((gXalter) & (gXdraw)  ) 
ifTrue:['ERROR, ' pren.
	'You Cannot draw and Alter at the same time' prinl]
ifFalse:[
   (gXUUstatus = 0)
   ifTrue:['ERROR, graphics device is not open' prenl.
          'To open enter DEVICE/OPEN' prinl]
   ifFalse:[gXdraw 
	    ifTrue:[gXcentre ifTrue:[(OneDHistogram defaultBinCentre: 1)]
                           ifFalse:[(OneDHistogram defaultBinCentre: 0)].
		  gXnotcentred ifTrue:[(OneDHistogram defaultBinCentre: 0)].
                  tmp _ (OneDHistogram new xarray: gXxarray; yarray: gXyarray;
                              colour: gXcolour; type: gXlineUtype;
                              lineThickness: gXlineUthickness; create).
                  ((gXUUgraphics at: #gXcurrentwindow) = nil)
                          ifTrue:['DRAW ERROR, ' pren.
				  'Window has not been created' prinl]
                ifFalse:[ ((gXUUgraphics at: #gXcurrentwindow) addToList: tmp).
                         gXres _ tmp.
                      a _ ((gXUUgraphics at: #gXcurrentwindow) getNumber: nil).
                      (gXUUgraphics at: #gXcurrentwindow) drawOnTop: a]]].

   gXalter ifTrue:[ 
	(gXobject isNil) ifTrue:['object not specified' prinl].
	(gXobject class = OneDHistogram)
        ifTrue:[object _ ((gXUUgraphics at: #gXcurrentgraphics)
                                          findDrawableObjectsPicture: gXobject).
                (object class = OneDHistogram)
                ifTrue:[pic _ ((gXUUgraphics at: #gXcurrentgraphics)
                                                         findPicture: object).
                        dev _ ((gXUUgraphics at: #gXcurrentgraphics)
                                                               findDevice: pic).
                        wind _ (pic findRightWindow: object).		   

                        gXcentre 
			    ifTrue:[a _ 1]
                            ifFalse:[gXnotcentred ifTrue:[a _ 0]
                                              ifFalse:[a _ (object binCentre)]].
                   (gXxarray isNil)
                   ifFalse:['XArray Not Yet Implemented' prnl].
                   (gXyarray isNil)
                   ifFalse:['YArray Not Yet Implemented' prnl].
                   (gXcolour isNil)
                   ifFalse:[object _ (object colour: gXcolour; binCentre: a;
                                   create)].
                   (gXlineUtype isNil)
                   ifFalse:[object _ (object type: gXlineUtype; binCentre: a;
                                   create)].
                   (gXlineUthickness isNil)
                   ifFalse:[object _ (object lineThickness: gXlineUthickness;
                                   binCentre: a; create)].

                   (object class = OneDHistogram)
                   ifTrue:[gXobject update: object.
			   pic drawAllWindows]
                   ifFalse:['Object has not been altered' prinl]]]]].

(gXalter | gXdraw)
ifFalse:['ERROR, ' pren.
	 '/DRAW or /ALTER has not been specified' prinl]

.
self up.
^gXres
!!
nil! "(fix for smalltalk bug, this line is ignored if there is a variable declaration error)"

"End of procedure"

"Procedure class definition:
 ==========================="
((Smalltalk at: #GXmarkers ifAbsent: [ nil ]) isKindOf: Function class) ifTrue: [ ('Warning: Overriding previously defined procedure ', 'GXmarkers' gtype) prinl ].
Smalltalk at: #GXmarkers put: nil!
Function subclass: 'GXmarkers'
       instanceVariableNames: ''
       classVariableNames: ''
       poolDictionaries: ''
       category: 'Genie Procedure'
!

"HEADER
 ------"

"qualifiers
 ----------"

GXmarkers

addInstVarName: #gXalter;
compile:
'gXalter
	gXalter _ true';
compile:
'gXalter: aBoolean
	gXalter _ aBoolean';

addInstVarName: #gXdraw;
compile:
'gXdraw
	gXdraw _ true';
compile:
'gXdraw: aBoolean
	gXdraw _ aBoolean'!

"parameters
 ----------"

GXmarkers

addInstVarName: #gXxarray;
compile:
'gXxarray: aVar
	aVar notNil ifTrue:[gXxarray _ self typeCheck: aVar shouldBe: GXRealarray paramName: ''gXxarray'']';
compile:
'p1: aVar
	aVar notNil ifTrue:[gXxarray _ self typeCheck: aVar shouldBe: GXRealarray paramName: ''gXxarray'']';

addInstVarName: #gXyarray;
compile:
'gXyarray: aVar
	aVar notNil ifTrue:[gXyarray _ self typeCheck: aVar shouldBe: GXRealarray paramName: ''gXyarray'']';
compile:
'p2: aVar
	aVar notNil ifTrue:[gXyarray _ self typeCheck: aVar shouldBe: GXRealarray paramName: ''gXyarray'']';

addInstVarName: #gXcolour;
compile:
'gXcolour: aVar
	aVar notNil ifTrue:[gXcolour _ self typeCheck: aVar shouldBe: GXColour paramName: ''gXcolour'']';
compile:
'p3: aVar
	aVar notNil ifTrue:[gXcolour _ self typeCheck: aVar shouldBe: GXColour paramName: ''gXcolour'']';

addInstVarName: #gXsymbol;
compile:
'gXsymbol: aVar
	aVar notNil ifTrue:[gXsymbol _ self typeCheck: aVar shouldBe: GXMarker paramName: ''gXsymbol'']';
compile:
'p4: aVar
	aVar notNil ifTrue:[gXsymbol _ self typeCheck: aVar shouldBe: GXMarker paramName: ''gXsymbol'']'!

"parameters
 ----------"

GXmarkers

addInstVarName: #gXsize;
compile:
'gXsize: aVar
	aVar notNil ifTrue:[gXsize _ self typeCheck: aVar shouldBe: GXReal paramName: ''gXsize'']';
compile:
'p5: aVar
	aVar notNil ifTrue:[gXsize _ self typeCheck: aVar shouldBe: GXReal paramName: ''gXsize'']';

addInstVarName: #gXlineUthickness;
compile:
'gXlineUthickness: aVar
	aVar notNil ifTrue:[gXlineUthickness _ self typeCheck: aVar shouldBe: GXReal paramName: ''gXlineUthickness'']';
compile:
'p6: aVar
	aVar notNil ifTrue:[gXlineUthickness _ self typeCheck: aVar shouldBe: GXReal paramName: ''gXlineUthickness'']';

addInstVarName: #gXobject;
compile:
'gXobject: aVar
	aVar notNil ifTrue:[gXobject _ self typeCheck: aVar shouldBe: GXGobject paramName: ''gXobject'']';
compile:
'p7: aVar
	aVar notNil ifTrue:[gXobject _ self typeCheck: aVar shouldBe: GXGobject paramName: ''gXobject'']'!

"result
 ------"

GXmarkers

addInstVarName: #gXres
!

!GXmarkers methodsFor: 'initializing'!

initVars
	gXalter _ false.
	gXdraw _ false
!!

"BODY
 ----"

!GXmarkers methodsFor: 'running'!

evaluate

   |tmp a dev pic wind object|

((gXalter) & (gXdraw)  ) 
ifTrue:['ERROR, ' pren.
	'You Cannot draw and Alter at the same time' prinl]
ifFalse:[
   (gXUUstatus = 0)
   ifTrue:['ERROR, graphics device is not open' prenl.
           'To open enter DEVICE/OPEN' prinl]
   ifFalse:[gXdraw 
	    ifTrue:[tmp _ (OneDMarkers new xarray: gXxarray; yarray: gXyarray;
                         colour: gXcolour; symbol: gXsymbol; charsize: gXsize;
                         lineThickness: gXlineUthickness; create).
                  ((gXUUgraphics at: #gXcurrentwindow) = nil)
                  ifTrue:['DRAW ERROR, ' pren.
				  'Window has not been created' prinl]
                ifFalse:[ ((gXUUgraphics at: #gXcurrentwindow) addToList: tmp).
                         gXres _ tmp.
                      a _ ((gXUUgraphics at: #gXcurrentwindow) getNumber: nil).
                      (gXUUgraphics at: #gXcurrentwindow) drawOnTop: a]]].

   gXalter ifTrue:[
	(gXobject = nil) ifTrue:['object not specified' prinl].
	(gXobject class = OneDMarkers)
                   ifTrue:[object _ ((gXUUgraphics at: #gXcurrentgraphics)
                                          findDrawableObjectsPicture: gXobject).
                   (object class = OneDMarkers)
                   ifTrue:[pic _ ((gXUUgraphics at: #gXcurrentgraphics)
                                                         findPicture: object).
                            dev _ ((gXUUgraphics at: #gXcurrentgraphics)
                                                               findDevice: pic).
                            wind _ (pic findRightWindow: object).

                      	   (gXxarray isNil)
                      	   ifFalse:['XArray Not Yet Implemented' prnl].
                      	   (gXyarray isNil)
                      	   ifFalse:['YArray Not Yet Implemented' prnl].
                      	   (gXcolour isNil)
                      	   ifFalse:[object _ (object colour: gXcolour; create)].
                      	   (gXsymbol isNil)
                      	   ifFalse:[object _ (object symbol: gXsymbol; create)].
                      	   (gXlineUthickness isNil)
                      	   ifFalse:[object _ (object 
					      lineThickness: gXlineUthickness; 
					      create)].
                      	   (gXsize isNil)
                      	   ifFalse:[object _ (object charsize: gXsize; create)].
			   (object class = OneDMarkers)
                   	   ifTrue:[gXobject update: object.
			  	   pic drawAllWindows]
                	   ifFalse:['Object has not been altered' prinl]]]]].

(gXalter | gXdraw)
ifFalse:['ERROR, ' pren.
	 '/DRAW or /ALTER has not been specified' prinl]

.
self up.
^gXres
!!
nil! "(fix for smalltalk bug, this line is ignored if there is a variable declaration error)"

"End of procedure"

"Procedure class definition:
 ==========================="
((Smalltalk at: #GXerrors ifAbsent: [ nil ]) isKindOf: Function class) ifTrue: [ ('Warning: Overriding previously defined procedure ', 'GXerrors' gtype) prinl ].
Smalltalk at: #GXerrors put: nil!
Function subclass: 'GXerrors'
       instanceVariableNames: ''
       classVariableNames: ''
       poolDictionaries: ''
       category: 'Genie Procedure'
!

"HEADER
 ------"

"qualifiers
 ----------"

GXerrors

addInstVarName: #gXalter;
compile:
'gXalter
	gXalter _ true';
compile:
'gXalter: aBoolean
	gXalter _ aBoolean';

addInstVarName: #gXdraw;
compile:
'gXdraw
	gXdraw _ true';
compile:
'gXdraw: aBoolean
	gXdraw _ aBoolean';

addInstVarName: #gXvertical;
compile:
'gXvertical
	gXvertical _ true';
compile:
'gXvertical: aBoolean
	gXvertical _ aBoolean';

addInstVarName: #gXhorizontal;
compile:
'gXhorizontal
	gXhorizontal _ true';
compile:
'gXhorizontal: aBoolean
	gXhorizontal _ aBoolean'!

"parameters
 ----------"

GXerrors

addInstVarName: #gXxarray;
compile:
'gXxarray: aVar
	aVar notNil ifTrue:[gXxarray _ self typeCheck: aVar shouldBe: GXRealarray paramName: ''gXxarray'']';
compile:
'p1: aVar
	aVar notNil ifTrue:[gXxarray _ self typeCheck: aVar shouldBe: GXRealarray paramName: ''gXxarray'']';

addInstVarName: #gXyarray;
compile:
'gXyarray: aVar
	aVar notNil ifTrue:[gXyarray _ self typeCheck: aVar shouldBe: GXRealarray paramName: ''gXyarray'']';
compile:
'p2: aVar
	aVar notNil ifTrue:[gXyarray _ self typeCheck: aVar shouldBe: GXRealarray paramName: ''gXyarray'']';

addInstVarName: #gXearray;
compile:
'gXearray: aVar
	aVar notNil ifTrue:[gXearray _ self typeCheck: aVar shouldBe: GXRealarray paramName: ''gXearray'']';
compile:
'p3: aVar
	aVar notNil ifTrue:[gXearray _ self typeCheck: aVar shouldBe: GXRealarray paramName: ''gXearray'']';

addInstVarName: #gXcolour;
compile:
'gXcolour: aVar
	aVar notNil ifTrue:[gXcolour _ self typeCheck: aVar shouldBe: GXColour paramName: ''gXcolour'']';
compile:
'p4: aVar
	aVar notNil ifTrue:[gXcolour _ self typeCheck: aVar shouldBe: GXColour paramName: ''gXcolour'']'!

"parameters
 ----------"

GXerrors

addInstVarName: #gXlineUthickness;
compile:
'gXlineUthickness: aVar
	aVar notNil ifTrue:[gXlineUthickness _ self typeCheck: aVar shouldBe: GXReal paramName: ''gXlineUthickness'']';
compile:
'p5: aVar
	aVar notNil ifTrue:[gXlineUthickness _ self typeCheck: aVar shouldBe: GXReal paramName: ''gXlineUthickness'']';

addInstVarName: #gXobject;
compile:
'gXobject: aVar
	aVar notNil ifTrue:[gXobject _ self typeCheck: aVar shouldBe: GXGobject paramName: ''gXobject'']';
compile:
'p6: aVar
	aVar notNil ifTrue:[gXobject _ self typeCheck: aVar shouldBe: GXGobject paramName: ''gXobject'']'!

"result
 ------"

GXerrors

addInstVarName: #gXres
!

!GXerrors methodsFor: 'initializing'!

initVars
	gXalter _ false.
	gXdraw _ false.
	gXvertical _ false.
	gXhorizontal _ false
!!

"BODY
 ----"

!GXerrors methodsFor: 'running'!

evaluate

   |tmp errortype a pic object wind dev|

((gXalter) & (gXdraw)  ) 
ifTrue:['ERROR, ' pren.
	'You Cannot draw and Alter at the same time' prinl]
ifFalse:[
   (gXUUstatus = 0)
   ifTrue:['ERROR, graphics device is not open' prenl.
           'To open enter DEVICE/OPEN' prinl]
   ifFalse:[gXdraw ifTrue:[ gXvertical ifTrue:[errortype _ 1]
			      ifFalse:[errortype _ 1].
                    gXhorizontal ifTrue:[errortype _ 0]
				 ifFalse:[errortype _ 1].
	   tmp _ (OneDErrors new xarray: gXxarray; yarray: gXyarray; 
	  	  earray: gXearray; colour: gXcolour; vertical: errortype;
		  lineThickness: gXlineUthickness; create). 

                  ((gXUUgraphics at: #gXcurrentwindow) = nil)
                          ifTrue:['DRAW ERROR, ' pren.
				  'Window has not been created' prinl]
                ifFalse:[ ((gXUUgraphics at: #gXcurrentwindow) addToList: tmp).
                         gXres _ tmp.
                     a _ ((gXUUgraphics at: #gXcurrentwindow) getNumber: nil).
                      (gXUUgraphics at: #gXcurrentwindow) drawOnTop: a]]].

   gXalter ifTrue:[
	errortype _ nil.
	gXvertical ifTrue:[errortype _ 1].
        gXhorizontal ifTrue:[errortype _ 0].
        (gXobject = nil) ifTrue:['object not specified' prinl].
  	(gXobject class = OneDErrors)
        ifTrue:[object _ ((gXUUgraphics at: #gXcurrentgraphics)
                                          findDrawableObjectsPicture: gXobject).
               (object class = OneDErrors)
               ifTrue:[pic _ ((gXUUgraphics at: #gXcurrentgraphics)
                                                         findPicture: object).
                       dev _ ((gXUUgraphics at: #gXcurrentgraphics)
                                                               findDevice: pic).
                       wind _ (pic findRightWindow: object).

                       (gXxarray isNil)
                       ifFalse:['XArray Not Yet Implemented' prnl].
                       (gXyarray isNil)
                       ifFalse:['YArray Not Yet Implemented' prnl].
                       (gXearray isNil)
                       ifFalse:['EArray Not Yet Implemented' prnl].
                       (gXcolour isNil)
                       ifFalse:[object _ (object colour: gXcolour; 
					  vertical: errortype; create)].
                       (gXlineUthickness = nil)
                      ifFalse:[object _ (object lineThickness: gXlineUthickness;
				          vertical: errortype; create)].
                       (object class = OneDErrors)
                       ifTrue:[gXobject update: object.
			       pic drawAllWindows]
                       ifFalse:['Object has not been altered' prinl]]]]].

(gXalter | gXdraw)
ifFalse:['ERROR, ' pren.
	 '/DRAW or /ALTER has not been specified' prinl]

.
self up.
^gXres
!!
nil! "(fix for smalltalk bug, this line is ignored if there is a variable declaration error)"

"End of procedure"

"Procedure class definition:
 ==========================="
((Smalltalk at: #GXcontour ifAbsent: [ nil ]) isKindOf: Function class) ifTrue: [ ('Warning: Overriding previously defined procedure ', 'GXcontour' gtype) prinl ].
Smalltalk at: #GXcontour put: nil!
Function subclass: 'GXcontour'
       instanceVariableNames: ''
       classVariableNames: ''
       poolDictionaries: ''
       category: 'Genie Procedure'
!

"HEADER
 ------"

"qualifiers
 ----------"

GXcontour

addInstVarName: #gXalter;
compile:
'gXalter
	gXalter _ true';
compile:
'gXalter: aBoolean
	gXalter _ aBoolean';

addInstVarName: #gXdraw;
compile:
'gXdraw
	gXdraw _ true';
compile:
'gXdraw: aBoolean
	gXdraw _ aBoolean';

addInstVarName: #gXlog;
compile:
'gXlog
	gXlog _ true';
compile:
'gXlog: aBoolean
	gXlog _ aBoolean';

addInstVarName: #gXsqrt;
compile:
'gXsqrt
	gXsqrt _ true';
compile:
'gXsqrt: aBoolean
	gXsqrt _ aBoolean';

addInstVarName: #gXlinear;
compile:
'gXlinear
	gXlinear _ true';
compile:
'gXlinear: aBoolean
	gXlinear _ aBoolean'!

"parameters
 ----------"

GXcontour

addInstVarName: #gXvalues;
compile:
'gXvalues: aVar
	aVar notNil ifTrue:[gXvalues _ self typeCheck: aVar shouldBe: GXRealarray paramName: ''gXvalues'']';
compile:
'p1: aVar
	aVar notNil ifTrue:[gXvalues _ self typeCheck: aVar shouldBe: GXRealarray paramName: ''gXvalues'']';

addInstVarName: #gXcontours;
compile:
'gXcontours: aVar
	aVar notNil ifTrue:[gXcontours _ self typeCheck: aVar shouldBe: GXRealarray paramName: ''gXcontours'']';
compile:
'p2: aVar
	aVar notNil ifTrue:[gXcontours _ self typeCheck: aVar shouldBe: GXRealarray paramName: ''gXcontours'']';

addInstVarName: #gXncont;
compile:
'gXncont: aVar
	aVar notNil ifTrue:[gXncont _ self typeCheck: aVar shouldBe: GXInteger paramName: ''gXncont'']';
compile:
'p3: aVar
	aVar notNil ifTrue:[gXncont _ self typeCheck: aVar shouldBe: GXInteger paramName: ''gXncont'']';

addInstVarName: #gXtr;
compile:
'gXtr: aVar
	aVar notNil ifTrue:[gXtr _ self typeCheck: aVar shouldBe: GXRealarray paramName: ''gXtr'']';
compile:
'p4: aVar
	aVar notNil ifTrue:[gXtr _ self typeCheck: aVar shouldBe: GXRealarray paramName: ''gXtr'']'!

"parameters
 ----------"

GXcontour

addInstVarName: #gXcolour;
compile:
'gXcolour: aVar
	aVar notNil ifTrue:[gXcolour _ self typeCheck: aVar shouldBe: GXColour paramName: ''gXcolour'']';
compile:
'p5: aVar
	aVar notNil ifTrue:[gXcolour _ self typeCheck: aVar shouldBe: GXColour paramName: ''gXcolour'']';

addInstVarName: #gXlineUtype;
compile:
'gXlineUtype: aVar
	aVar notNil ifTrue:[gXlineUtype _ self typeCheck: aVar shouldBe: GXLinestyle paramName: ''gXlineUtype'']';
compile:
'p6: aVar
	aVar notNil ifTrue:[gXlineUtype _ self typeCheck: aVar shouldBe: GXLinestyle paramName: ''gXlineUtype'']';

addInstVarName: #gXlineUthickness;
compile:
'gXlineUthickness: aVar
	aVar notNil ifTrue:[gXlineUthickness _ self typeCheck: aVar shouldBe: GXReal paramName: ''gXlineUthickness'']';
compile:
'p7: aVar
	aVar notNil ifTrue:[gXlineUthickness _ self typeCheck: aVar shouldBe: GXReal paramName: ''gXlineUthickness'']';

addInstVarName: #gXobject;
compile:
'gXobject: aVar
	aVar notNil ifTrue:[gXobject _ self typeCheck: aVar shouldBe: GXGobject paramName: ''gXobject'']';
compile:
'p8: aVar
	aVar notNil ifTrue:[gXobject _ self typeCheck: aVar shouldBe: GXGobject paramName: ''gXobject'']'!

"result
 ------"

GXcontour

addInstVarName: #gXres
!

!GXcontour methodsFor: 'initializing'!

initVars
	gXalter _ false.
	gXdraw _ false.
	gXlog _ false.
	gXsqrt _ false.
	gXlinear _ false
!!

"BODY
 ----"

!GXcontour methodsFor: 'running'!

evaluate

   |tmp a dev pic wind object|

((gXalter) & (gXdraw)  ) 
ifTrue:['ERROR, ' pren.
	'You Cannot draw and Alter at the same time' prinl]
ifFalse:[
   (gXUUstatus = 0)
   ifTrue:['ERROR, graphics device is not open' prenl.
           'To open enter DEVICE/OPEN' prinl]
   ifFalse:[gXdraw 
	    ifTrue:[gXlog ifTrue:[a _ 1].
                  gXsqrt ifTrue:[a _ 2].
                  gXlinear ifTrue:[a _ 0].
                  tmp _ (Contour new values: gXvalues; contours: gXcontours;
                         ncont: gXncont; tr: gXtr; colour: gXcolour; itype: a;
                        lineStyle: gXlineUtype; lineThickness: gXlineUthickness;
                         create).

                  ((gXUUgraphics at: #gXcurrentwindow) = nil)
                          ifTrue:['DRAW ERROR, ' pren.
				  'Window has not been created' prinl]
                ifFalse:[ ((gXUUgraphics at: #gXcurrentwindow) addToList: tmp).
                         gXres _ tmp.
                      a _ ((gXUUgraphics at: #gXcurrentwindow) getNumber: nil).
                      (gXUUgraphics at: #gXcurrentwindow) drawOnTop: a]]].

   gXalter ifTrue:[
	(gXobject = nil) ifTrue:['object not specified' prinl].
	(gXobject class = Contour)
        ifTrue:[object _ ((gXUUgraphics at: #gXcurrentgraphics)
                                          findDrawableObjectsPicture: gXobject).
                (object class = Contour)
                ifTrue:[pic _ ((gXUUgraphics at: #gXcurrentgraphics)
                                                         findPicture: object).
                        dev _ ((gXUUgraphics at: #gXcurrentgraphics)
                                                               findDevice: pic).
                        wind _ (pic findRightWindow: object).

                  gXlog ifTrue:[(a _ 1)]
                        ifFalse:[gXsqrt ifTrue:[(a _ 2)]
                                        ifFalse:[gXlinear ifTrue:[a _ 0]
                                                 ifFalse:[a _ (object itype)]]].
                  (gXvalues isNil)
                  ifFalse:['Values Not Yet Implemented' prnl].
                  (gXcontours isNil)
                  ifFalse:['Contours Not Yet Implemented' prnl].
                  (gXncont isNil)
                  ifFalse:[object _ (object ncont: gXncont; itype: a; create)].
                  (gXtr isNil)
                  ifFalse:[object _ (object tr: gXtr; itype: a; create)].
                  (gXcolour isNil)
                 ifFalse:[object _ (object colour: gXcolour; itype: a; create)].
                  (gXlineUtype isNil)
                ifFalse:[object _ (object lineStyle: gXlineUtype; itype: a; 
				   create)].
                  (gXlineUthickness isNil)
                  ifFalse:[object _ (object lineThickness: gXlineUthickness;
                                  itype: a; create)].
                  ((gXvalues isNil) &                   (gXncont isNil) &                   (gXtr isNil) &		   (gXcolour isNil) &                   (gXlineUtype isNil) &                   (gXlineUthickness isNil) )
                   ifTrue:[object _ (object itype: a; create)].

                  (object class = Contour)
                  ifTrue:[gXobject update: object.
			  pic drawAllWindows]
                  ifFalse:['Object has not been altered' prinl]]]]].

(gXalter | gXdraw)
ifFalse:['ERROR, ' pren.
	 '/DRAW or /ALTER has not been specified' prinl]



.
self up.
^gXres
!!
nil! "(fix for smalltalk bug, this line is ignored if there is a variable declaration error)"

"End of procedure"

"Procedure class definition:
 ==========================="
((Smalltalk at: #GXcontourUarray ifAbsent: [ nil ]) isKindOf: Function class) ifTrue: [ ('Warning: Overriding previously defined procedure ', 'GXcontourUarray' gtype) prinl ].
Smalltalk at: #GXcontourUarray put: nil!
Function subclass: 'GXcontourUarray'
       instanceVariableNames: ''
       classVariableNames: ''
       poolDictionaries: ''
       category: 'Genie Procedure'
!

"HEADER
 ------"

"qualifiers
 ----------"

GXcontourUarray

addInstVarName: #gXalter;
compile:
'gXalter
	gXalter _ true';
compile:
'gXalter: aBoolean
	gXalter _ aBoolean';

addInstVarName: #gXdraw;
compile:
'gXdraw
	gXdraw _ true';
compile:
'gXdraw: aBoolean
	gXdraw _ aBoolean';

addInstVarName: #gXlog;
compile:
'gXlog
	gXlog _ true';
compile:
'gXlog: aBoolean
	gXlog _ aBoolean';

addInstVarName: #gXsqrt;
compile:
'gXsqrt
	gXsqrt _ true';
compile:
'gXsqrt: aBoolean
	gXsqrt _ aBoolean';

addInstVarName: #gXlinear;
compile:
'gXlinear
	gXlinear _ true';
compile:
'gXlinear: aBoolean
	gXlinear _ aBoolean'!

"parameters
 ----------"

GXcontourUarray

addInstVarName: #gXvalues;
compile:
'gXvalues: aVar
	aVar notNil ifTrue:[gXvalues _ self typeCheck: aVar shouldBe: GXRealarray paramName: ''gXvalues'']';
compile:
'p1: aVar
	aVar notNil ifTrue:[gXvalues _ self typeCheck: aVar shouldBe: GXRealarray paramName: ''gXvalues'']';

addInstVarName: #gXcontours;
compile:
'gXcontours: aVar
	aVar notNil ifTrue:[gXcontours _ self typeCheck: aVar shouldBe: GXRealarray paramName: ''gXcontours'']';
compile:
'p2: aVar
	aVar notNil ifTrue:[gXcontours _ self typeCheck: aVar shouldBe: GXRealarray paramName: ''gXcontours'']';

addInstVarName: #gXncont;
compile:
'gXncont: aVar
	aVar notNil ifTrue:[gXncont _ self typeCheck: aVar shouldBe: GXInteger paramName: ''gXncont'']';
compile:
'p3: aVar
	aVar notNil ifTrue:[gXncont _ self typeCheck: aVar shouldBe: GXInteger paramName: ''gXncont'']';

addInstVarName: #gXxarray;
compile:
'gXxarray: aVar
	aVar notNil ifTrue:[gXxarray _ self typeCheck: aVar shouldBe: GXRealarray paramName: ''gXxarray'']';
compile:
'p4: aVar
	aVar notNil ifTrue:[gXxarray _ self typeCheck: aVar shouldBe: GXRealarray paramName: ''gXxarray'']'!

"parameters
 ----------"

GXcontourUarray

addInstVarName: #gXyarray;
compile:
'gXyarray: aVar
	aVar notNil ifTrue:[gXyarray _ self typeCheck: aVar shouldBe: GXRealarray paramName: ''gXyarray'']';
compile:
'p5: aVar
	aVar notNil ifTrue:[gXyarray _ self typeCheck: aVar shouldBe: GXRealarray paramName: ''gXyarray'']';

addInstVarName: #gXcolour;
compile:
'gXcolour: aVar
	aVar notNil ifTrue:[gXcolour _ self typeCheck: aVar shouldBe: GXColour paramName: ''gXcolour'']';
compile:
'p6: aVar
	aVar notNil ifTrue:[gXcolour _ self typeCheck: aVar shouldBe: GXColour paramName: ''gXcolour'']';

addInstVarName: #gXlineUtype;
compile:
'gXlineUtype: aVar
	aVar notNil ifTrue:[gXlineUtype _ self typeCheck: aVar shouldBe: GXLinestyle paramName: ''gXlineUtype'']';
compile:
'p7: aVar
	aVar notNil ifTrue:[gXlineUtype _ self typeCheck: aVar shouldBe: GXLinestyle paramName: ''gXlineUtype'']';

addInstVarName: #gXlineUthickness;
compile:
'gXlineUthickness: aVar
	aVar notNil ifTrue:[gXlineUthickness _ self typeCheck: aVar shouldBe: GXReal paramName: ''gXlineUthickness'']';
compile:
'p8: aVar
	aVar notNil ifTrue:[gXlineUthickness _ self typeCheck: aVar shouldBe: GXReal paramName: ''gXlineUthickness'']'!

"parameters
 ----------"

GXcontourUarray

addInstVarName: #gXobject;
compile:
'gXobject: aVar
	aVar notNil ifTrue:[gXobject _ self typeCheck: aVar shouldBe: GXGobject paramName: ''gXobject'']';
compile:
'p9: aVar
	aVar notNil ifTrue:[gXobject _ self typeCheck: aVar shouldBe: GXGobject paramName: ''gXobject'']'!

"result
 ------"

GXcontourUarray

addInstVarName: #gXres
!

!GXcontourUarray methodsFor: 'initializing'!

initVars
	gXalter _ false.
	gXdraw _ false.
	gXlog _ false.
	gXsqrt _ false.
	gXlinear _ false
!!

"BODY
 ----"

!GXcontourUarray methodsFor: 'running'!

evaluate

   |tmp a dev pic wind object|

((gXalter) & (gXdraw)  ) 
ifTrue:['ERROR, ' pren.
	'You Cannot draw and Alter at the same time' prinl]
ifFalse:[
   (gXUUstatus = 0)
   ifTrue:['ERROR, graphics device is not open' prenl.
           'To open enter DEVICE/OPEN' prinl]
   ifFalse:[gXdraw 
	    ifTrue:[gXlog ifTrue:[a _ 1].
                  gXsqrt ifTrue:[a _ 2].
                  gXlinear ifTrue:[a _ 0].
                  tmp _ (ContourArray new values: gXvalues; ncont: gXncont;
                         contours: gXcontours; wX: gXxarray; wY: gXyarray;
                         itype: a; colour: gXcolour; lineStyle: gXlineUtype;
                         lineThickness: gXlineUthickness; create).
                  ((gXUUgraphics at: #gXcurrentwindow) = nil)
                          ifTrue:['DRAW ERROR, ' pren.
				  'Window has not been created' prinl]
                ifFalse:[ ((gXUUgraphics at: #gXcurrentwindow) addToList: tmp).
                         gXres _ tmp.
                      a _ ((gXUUgraphics at: #gXcurrentwindow) getNumber: nil).
                      (gXUUgraphics at: #gXcurrentwindow) drawOnTop: a]]].

   gXalter ifTrue:[
	(gXobject = nil) ifTrue:['object not specified' prinl].
	(gXobject class = ContourArray)
        ifTrue:[object _ ((gXUUgraphics at: #gXcurrentgraphics)
                                          findDrawableObjectsPicture: gXobject).
                (object class = ContourArray)
                ifTrue:[pic _ ((gXUUgraphics at: #gXcurrentgraphics)
                                                         findPicture: object).
                        dev _ ((gXUUgraphics at: #gXcurrentgraphics)
                                                               findDevice: pic).
                        wind _ (pic findRightWindow: object).

                  gXlog ifTrue:[(a _ 1)]
                        ifFalse:[gXsqrt ifTrue:[(a _ 2)]
                                        ifFalse:[gXlinear ifTrue:[a _ 0]
                                                 ifFalse:[a _ (object itype)]]].
                  (gXvalues isNil)
                  ifFalse:['Values Not Yet Implemented' prnl].
                  (gXcontours isNil)
                  ifFalse:['Contours Not Yet Implemented' prnl].
                  (gXncont isNil)
                  ifFalse:[object _ (object ncont: gXncont; itype: a; create)].
                  (gXxarray isNil)
                  ifFalse:['Xarray Not Yet Implemented' prnl].
                  (gXyarray isNil)
                  ifFalse:['Yarray Not Yet Implemented' prnl].
                  (gXcolour isNil)
                 ifFalse:[object _ (object colour: gXcolour; itype: a; create)].
                  (gXlineUtype isNil)
                  ifFalse:[object _ (object lineStyle: gXlineUtype; itype: a; 
				     create)].
                  (gXlineUthickness isNil)
                  ifFalse:[object _ (object lineThickness: gXlineUthickness;
                                  itype: a; create)].
                  ((gXncont isNil) &                   (gXcolour isNil) &                   (gXlineUtype isNil) &                   (gXlineUthickness isNil) )
                   ifTrue:[object _ (object itype: a; create)].

                   (object class = ContourArray)
                   ifTrue:[gXobject update: object.
			   pic drawAllWindows]
                   ifFalse:['Object has not been altered' prinl]]]]].

(gXalter | gXdraw)
ifFalse:['ERROR, ' pren.  '/DRAW or /ALTER has not been specified' prinl]



.
self up.
^gXres
!!
nil! "(fix for smalltalk bug, this line is ignored if there is a variable declaration error)"

"End of procedure"

"Procedure class definition:
 ==========================="
((Smalltalk at: #GXcontourUfunction ifAbsent: [ nil ]) isKindOf: Function class) ifTrue: [ ('Warning: Overriding previously defined procedure ', 'GXcontourUfunction' gtype) prinl ].
Smalltalk at: #GXcontourUfunction put: nil!
Function subclass: 'GXcontourUfunction'
       instanceVariableNames: ''
       classVariableNames: ''
       poolDictionaries: ''
       category: 'Genie Procedure'
!

"HEADER
 ------"

"qualifiers
 ----------"

GXcontourUfunction

addInstVarName: #gXalter;
compile:
'gXalter
	gXalter _ true';
compile:
'gXalter: aBoolean
	gXalter _ aBoolean';

addInstVarName: #gXdraw;
compile:
'gXdraw
	gXdraw _ true';
compile:
'gXdraw: aBoolean
	gXdraw _ aBoolean';

addInstVarName: #gXlog;
compile:
'gXlog
	gXlog _ true';
compile:
'gXlog: aBoolean
	gXlog _ aBoolean';

addInstVarName: #gXsqrt;
compile:
'gXsqrt
	gXsqrt _ true';
compile:
'gXsqrt: aBoolean
	gXsqrt _ aBoolean';

addInstVarName: #gXlinear;
compile:
'gXlinear
	gXlinear _ true';
compile:
'gXlinear: aBoolean
	gXlinear _ aBoolean'!

"parameters
 ----------"

GXcontourUfunction

addInstVarName: #gXvalues;
compile:
'gXvalues: aVar
	aVar notNil ifTrue:[gXvalues _ self typeCheck: aVar shouldBe: GXRealarray paramName: ''gXvalues'']';
compile:
'p1: aVar
	aVar notNil ifTrue:[gXvalues _ self typeCheck: aVar shouldBe: GXRealarray paramName: ''gXvalues'']';

addInstVarName: #gXcontours;
compile:
'gXcontours: aVar
	aVar notNil ifTrue:[gXcontours _ self typeCheck: aVar shouldBe: GXRealarray paramName: ''gXcontours'']';
compile:
'p2: aVar
	aVar notNil ifTrue:[gXcontours _ self typeCheck: aVar shouldBe: GXRealarray paramName: ''gXcontours'']';

addInstVarName: #gXncont;
compile:
'gXncont: aVar
	aVar notNil ifTrue:[gXncont _ self typeCheck: aVar shouldBe: GXInteger paramName: ''gXncont'']';
compile:
'p3: aVar
	aVar notNil ifTrue:[gXncont _ self typeCheck: aVar shouldBe: GXInteger paramName: ''gXncont'']'!

"parameters
 ----------"

GXcontourUfunction

addInstVarName: #gXfunctionUname;
compile:
'gXfunctionUname: aVar
	aVar notNil ifTrue:[gXfunctionUname _ self typeCheck: aVar shouldBe: GXString paramName: ''gXfunctionUname'']';
compile:
'p4: aVar
	aVar notNil ifTrue:[gXfunctionUname _ self typeCheck: aVar shouldBe: GXString paramName: ''gXfunctionUname'']';

addInstVarName: #gXcolour;
compile:
'gXcolour: aVar
	aVar notNil ifTrue:[gXcolour _ self typeCheck: aVar shouldBe: GXColour paramName: ''gXcolour'']';
compile:
'p5: aVar
	aVar notNil ifTrue:[gXcolour _ self typeCheck: aVar shouldBe: GXColour paramName: ''gXcolour'']';

addInstVarName: #gXlineUtype;
compile:
'gXlineUtype: aVar
	aVar notNil ifTrue:[gXlineUtype _ self typeCheck: aVar shouldBe: GXLinestyle paramName: ''gXlineUtype'']';
compile:
'p6: aVar
	aVar notNil ifTrue:[gXlineUtype _ self typeCheck: aVar shouldBe: GXLinestyle paramName: ''gXlineUtype'']'!

"parameters
 ----------"

GXcontourUfunction

addInstVarName: #gXlineUthickness;
compile:
'gXlineUthickness: aVar
	aVar notNil ifTrue:[gXlineUthickness _ self typeCheck: aVar shouldBe: GXReal paramName: ''gXlineUthickness'']';
compile:
'p7: aVar
	aVar notNil ifTrue:[gXlineUthickness _ self typeCheck: aVar shouldBe: GXReal paramName: ''gXlineUthickness'']';

addInstVarName: #gXobject;
compile:
'gXobject: aVar
	aVar notNil ifTrue:[gXobject _ self typeCheck: aVar shouldBe: GXGobject paramName: ''gXobject'']';
compile:
'p8: aVar
	aVar notNil ifTrue:[gXobject _ self typeCheck: aVar shouldBe: GXGobject paramName: ''gXobject'']'!

"result
 ------"

GXcontourUfunction

addInstVarName: #gXres
!

!GXcontourUfunction methodsFor: 'initializing'!

initVars
	gXalter _ false.
	gXdraw _ false.
	gXlog _ false.
	gXsqrt _ false.
	gXlinear _ false
!!

"BODY
 ----"

!GXcontourUfunction methodsFor: 'running'!

evaluate

   |tmp a dev pic wind object|

((gXalter) & (gXdraw)  ) 
ifTrue:['ERROR, ' pren.
	'You Cannot draw and Alter at the same time' prinl]
ifFalse:[
   (gXUUstatus = 0)
   ifTrue:['ERROR, graphics device is not open' prenl.
           'To open enter DEVICE/OPEN' prinl]
   ifFalse:[gXdraw 
	    ifTrue:[gXlog ifTrue:[a _ 1].
                  gXsqrt ifTrue:[a _ 2].
                  gXlinear ifTrue:[a _ 0].
                  tmp _ (ContourFunction new values: gXvalues; ncont: gXncont;
                         contours: gXcontours; funcPtr: gXfunctionUname;
                         itype: a; colour: gXcolour; lineStyle: gXlineUtype;
                         lineThickness: gXlineUthickness; create).
                  ((gXUUgraphics at: #gXcurrentwindow) = nil)
                          ifTrue:['DRAW ERROR, ' pren.
				  'Window has not been created' prinl]
                ifFalse:[ ((gXUUgraphics at: #gXcurrentwindow) addToList: tmp).
			 gXres _ tmp.
			 a _ ((gXUUgraphics at: #gXcurrentwindow) 
								getNumber: nil).
                    (gXUUgraphics at: #gXcurrentwindow) drawOnTop: a]]].
   gXalter ifTrue:[
	(gXobject = nil) ifTrue:['object not specified' prinl].
	(gXobject class = ContourFunction)
        ifTrue:[object _ ((gXUUgraphics at: #gXcurrentgraphics)
                                          findDrawableObjectsPicture: gXobject).
                (object class = ContourFunction)
                ifTrue:[pic _ ((gXUUgraphics at: #gXcurrentgraphics)
                                                         findPicture: object).
                        dev _ ((gXUUgraphics at: #gXcurrentgraphics)
                                                               findDevice: pic).
                        wind _ (pic findRightWindow: object).

                  gXlog ifTrue:[(a _ 1)]
                        ifFalse:[gXsqrt ifTrue:[(a _ 2)]
                                        ifFalse:[gXlinear ifTrue:[a _ 0]
                                                 ifFalse:[a _ (object itype)]]].
                  (gXvalues isNil)
                  ifFalse:['Values Not Yet Implemented' prnl].
                  (gXcontours isNil)
                  ifFalse:['Contours Not Yet Implemented' prnl].
                  (gXncont isNil)
                  ifFalse:[object _ (object ncont: gXncont; itype: a; create)].
                  (gXfunctionUname isNil)
                  ifFalse:[object _ (object funcPtr: gXfunctionUname; itype: a;
                                  create)].
                  (gXcolour isNil)
                  ifFalse:[object _ (object colour: gXcolour; itype: a; 
				     create)].
                  (gXlineUtype isNil)
                  ifFalse:[object _ (object lineStyle: gXlineUtype; itype: a; 
				     create)].
                  (gXlineUthickness isNil)
                  ifFalse:[object _ (object lineThickness: gXlineUthickness;
                                     itype: a; create)].
                  ((gXncont isNil) &                   (gXfunctionUname isNil) &                   (gXcolour isNil) &                   (gXlineUtype isNil) &                   (gXlineUthickness isNil) )
                  ifTrue:[object _ (object itype: a; create)].

                  (object class = ContourFunction)
                  ifTrue:[gXobject update: object.
			  pic drawAllWindows]
                  ifFalse:['Object has not been altered' prinl]]]]].

(gXalter | gXdraw)
ifFalse:['ERROR, ' pren.
	 '/DRAW or /ALTER has not been specified' prinl]


.
self up.
^gXres
!!
nil! "(fix for smalltalk bug, this line is ignored if there is a variable declaration error)"

"End of procedure"

"Procedure class definition:
 ==========================="
((Smalltalk at: #GXcontourUlabel ifAbsent: [ nil ]) isKindOf: Function class) ifTrue: [ ('Warning: Overriding previously defined procedure ', 'GXcontourUlabel' gtype) prinl ].
Smalltalk at: #GXcontourUlabel put: nil!
Function subclass: 'GXcontourUlabel'
       instanceVariableNames: ''
       classVariableNames: ''
       poolDictionaries: ''
       category: 'Genie Procedure'
!

"HEADER
 ------"

"qualifiers
 ----------"

GXcontourUlabel

addInstVarName: #gXalter;
compile:
'gXalter
	gXalter _ true';
compile:
'gXalter: aBoolean
	gXalter _ aBoolean';

addInstVarName: #gXdraw;
compile:
'gXdraw
	gXdraw _ true';
compile:
'gXdraw: aBoolean
	gXdraw _ aBoolean'!

"parameters
 ----------"

GXcontourUlabel

addInstVarName: #gXtext;
compile:
'gXtext: aVar
	aVar notNil ifTrue:[gXtext _ self typeCheck: aVar shouldBe: GXString paramName: ''gXtext'']';
compile:
'p1: aVar
	aVar notNil ifTrue:[gXtext _ self typeCheck: aVar shouldBe: GXString paramName: ''gXtext'']';

addInstVarName: #gXstart;
compile:
'gXstart: aVar
	aVar notNil ifTrue:[gXstart _ self typeCheck: aVar shouldBe: GXInteger paramName: ''gXstart'']';
compile:
'p2: aVar
	aVar notNil ifTrue:[gXstart _ self typeCheck: aVar shouldBe: GXInteger paramName: ''gXstart'']';

addInstVarName: #gXend;
compile:
'gXend: aVar
	aVar notNil ifTrue:[gXend _ self typeCheck: aVar shouldBe: GXInteger paramName: ''gXend'']';
compile:
'p3: aVar
	aVar notNil ifTrue:[gXend _ self typeCheck: aVar shouldBe: GXInteger paramName: ''gXend'']';

addInstVarName: #gXst;
compile:
'gXst: aVar
	aVar notNil ifTrue:[gXst _ self typeCheck: aVar shouldBe: GXInteger paramName: ''gXst'']';
compile:
'p4: aVar
	aVar notNil ifTrue:[gXst _ self typeCheck: aVar shouldBe: GXInteger paramName: ''gXst'']';

addInstVarName: #gXintval;
compile:
'gXintval: aVar
	aVar notNil ifTrue:[gXintval _ self typeCheck: aVar shouldBe: GXInteger paramName: ''gXintval'']';
compile:
'p5: aVar
	aVar notNil ifTrue:[gXintval _ self typeCheck: aVar shouldBe: GXInteger paramName: ''gXintval'']'!

"parameters
 ----------"

GXcontourUlabel

addInstVarName: #gXminint;
compile:
'gXminint: aVar
	aVar notNil ifTrue:[gXminint _ self typeCheck: aVar shouldBe: GXInteger paramName: ''gXminint'']';
compile:
'p6: aVar
	aVar notNil ifTrue:[gXminint _ self typeCheck: aVar shouldBe: GXInteger paramName: ''gXminint'']';

addInstVarName: #gXcolour;
compile:
'gXcolour: aVar
	aVar notNil ifTrue:[gXcolour _ self typeCheck: aVar shouldBe: GXColour paramName: ''gXcolour'']';
compile:
'p7: aVar
	aVar notNil ifTrue:[gXcolour _ self typeCheck: aVar shouldBe: GXColour paramName: ''gXcolour'']';

addInstVarName: #gXsize;
compile:
'gXsize: aVar
	aVar notNil ifTrue:[gXsize _ self typeCheck: aVar shouldBe: GXReal paramName: ''gXsize'']';
compile:
'p8: aVar
	aVar notNil ifTrue:[gXsize _ self typeCheck: aVar shouldBe: GXReal paramName: ''gXsize'']';

addInstVarName: #gXfont;
compile:
'gXfont: aVar
	aVar notNil ifTrue:[gXfont _ self typeCheck: aVar shouldBe: GXFont paramName: ''gXfont'']';
compile:
'p9: aVar
	aVar notNil ifTrue:[gXfont _ self typeCheck: aVar shouldBe: GXFont paramName: ''gXfont'']';

addInstVarName: #gXlineUthickness;
compile:
'gXlineUthickness: aVar
	aVar notNil ifTrue:[gXlineUthickness _ self typeCheck: aVar shouldBe: GXReal paramName: ''gXlineUthickness'']';
compile:
'p10: aVar
	aVar notNil ifTrue:[gXlineUthickness _ self typeCheck: aVar shouldBe: GXReal paramName: ''gXlineUthickness'']'!

"parameters
 ----------"

GXcontourUlabel

addInstVarName: #gXobject;
compile:
'gXobject: aVar
	aVar notNil ifTrue:[gXobject _ self typeCheck: aVar shouldBe: GXGobject paramName: ''gXobject'']';
compile:
'p11: aVar
	aVar notNil ifTrue:[gXobject _ self typeCheck: aVar shouldBe: GXGobject paramName: ''gXobject'']'!

"result
 ------"

GXcontourUlabel

addInstVarName: #gXres
!

!GXcontourUlabel methodsFor: 'initializing'!

initVars
	gXalter _ false.
	gXdraw _ false
!!

"BODY
 ----"

!GXcontourUlabel methodsFor: 'running'!

evaluate

   |tmp a aNumber dev pic wind object|

((gXalter) & (gXdraw)  ) 
ifTrue:['ERROR, ' pren.
	'You Cannot draw and Alter at the same time' prinl]
ifFalse:[
   (gXUUstatus = 0)
   ifTrue:['ERROR, graphics device is not open' prenl.
           'To open enter DEVICE/OPEN' prinl]
   ifFalse:[gXdraw 
	    ifTrue:[(gXobject isNil)
                  ifTrue:['ERROR, a contour has not been specified' prenl.
                   'Enter window number then object number' prinl]
                  ifFalse:[(gXobject isKindOf: TwoDContour)
                           ifTrue:[tmp _ (ContourLabel new words: gXtext;
 			  		 intval: gXintval; step: gXst; 
				  	 start: gXstart;
                                         minint: gXminint; font: gXfont;
                                         height: gXsize; colour: gXcolour;
					 lineThickness: gXlineUthickness;
                                         contourName: gXobject; create).

                  ((gXUUgraphics at: #gXcurrentwindow) isNil)
                          ifTrue:['DRAW ERROR, ' pren.
				  'Window has not been created' prinl]
                ifFalse:[ ((gXUUgraphics at: #gXcurrentwindow) addToList: tmp).
                         gXres _ tmp.
                     a _ ((gXUUgraphics at: #gXcurrentwindow) getNumber: nil).
                      (gXUUgraphics at: #gXcurrentwindow) drawOnTop: a]]]].

   gXalter ifTrue:[
	(gXobject isNil) ifTrue:['Object not specified' prinl].
	(gXobject class = ContourLabel)
        ifTrue:[object _ ((gXUUgraphics at: #gXcurrentgraphics)
                                          findDrawableObjectsPicture: gXobject).
                (object class = ContourLabel)
                ifTrue:[pic _ ((gXUUgraphics at: #gXcurrentgraphics)
                                                         findPicture: object).
                        dev _ ((gXUUgraphics at: #gXcurrentgraphics)
                                                               findDevice: pic).
                        wind _ (pic findRightWindow: object).

                        (gXtext isNil)
                        ifFalse:[object _ (object words: gXtext; create)].
                        (gXintval isNil)
                        ifFalse:[object _ (object intval: gXintval; create)].
                        (gXminint isNil)
                        ifFalse:[object _ (object minint: gXminint; create)].
                        (gXfont isNil)
                        ifFalse:[object _ (object font: gXfont; create)].
                        (gXsize isNil)
                        ifFalse:[object _ (object height: gXsize; create)].
                        (gXcolour isNil)
                        ifFalse:[object _ (object colour: gXcolour; create)].

                     	(object class = ContourLabel)
                	ifTrue:[gXobject update: object.
				pic drawAllWindows]
                	ifFalse:['Object has not been altered' prinl]]]]]].

(gXalter | gXdraw)
ifFalse:['ERROR, ' pren.
	 '/DRAW or /ALTER has not been specified' prinl]


.
self up.
^gXres
!!
nil! "(fix for smalltalk bug, this line is ignored if there is a variable declaration error)"

"End of procedure"

"Procedure class definition:
 ==========================="
((Smalltalk at: #GXcell ifAbsent: [ nil ]) isKindOf: Function class) ifTrue: [ ('Warning: Overriding previously defined procedure ', 'GXcell' gtype) prinl ].
Smalltalk at: #GXcell put: nil!
Function subclass: 'GXcell'
       instanceVariableNames: ''
       classVariableNames: ''
       poolDictionaries: ''
       category: 'Genie Procedure'
!

"HEADER
 ------"

"qualifiers
 ----------"

GXcell

addInstVarName: #gXalter;
compile:
'gXalter
	gXalter _ true';
compile:
'gXalter: aBoolean
	gXalter _ aBoolean';

addInstVarName: #gXdraw;
compile:
'gXdraw
	gXdraw _ true';
compile:
'gXdraw: aBoolean
	gXdraw _ aBoolean';

addInstVarName: #gXsmooth;
compile:
'gXsmooth
	gXsmooth _ true';
compile:
'gXsmooth: aBoolean
	gXsmooth _ aBoolean';

addInstVarName: #gXraw;
compile:
'gXraw
	gXraw _ true';
compile:
'gXraw: aBoolean
	gXraw _ aBoolean';

addInstVarName: #gXlog;
compile:
'gXlog
	gXlog _ true';
compile:
'gXlog: aBoolean
	gXlog _ aBoolean';

addInstVarName: #gXsqrt;
compile:
'gXsqrt
	gXsqrt _ true';
compile:
'gXsqrt: aBoolean
	gXsqrt _ aBoolean';

addInstVarName: #gXlinear;
compile:
'gXlinear
	gXlinear _ true';
compile:
'gXlinear: aBoolean
	gXlinear _ aBoolean'!

"parameters
 ----------"

GXcell

addInstVarName: #gXvalues;
compile:
'gXvalues: aVar
	aVar notNil ifTrue:[gXvalues _ self typeCheck: aVar shouldBe: GXRealarray paramName: ''gXvalues'']';
compile:
'p1: aVar
	aVar notNil ifTrue:[gXvalues _ self typeCheck: aVar shouldBe: GXRealarray paramName: ''gXvalues'']';

addInstVarName: #gXtable;
compile:
'gXtable: aVar
	aVar notNil ifTrue:[gXtable _ self typeCheck: aVar shouldBe: GXColourtable paramName: ''gXtable'']';
compile:
'p2: aVar
	aVar notNil ifTrue:[gXtable _ self typeCheck: aVar shouldBe: GXColourtable paramName: ''gXtable'']';

addInstVarName: #gXtr;
compile:
'gXtr: aVar
	aVar notNil ifTrue:[gXtr _ self typeCheck: aVar shouldBe: GXRealarray paramName: ''gXtr'']';
compile:
'p3: aVar
	aVar notNil ifTrue:[gXtr _ self typeCheck: aVar shouldBe: GXRealarray paramName: ''gXtr'']';

addInstVarName: #gXvalmax;
compile:
'gXvalmax: aVar
	aVar notNil ifTrue:[gXvalmax _ self typeCheck: aVar shouldBe: GXReal paramName: ''gXvalmax'']';
compile:
'p4: aVar
	aVar notNil ifTrue:[gXvalmax _ self typeCheck: aVar shouldBe: GXReal paramName: ''gXvalmax'']'!

"parameters
 ----------"

GXcell

addInstVarName: #gXvalmin;
compile:
'gXvalmin: aVar
	aVar notNil ifTrue:[gXvalmin _ self typeCheck: aVar shouldBe: GXReal paramName: ''gXvalmin'']';
compile:
'p5: aVar
	aVar notNil ifTrue:[gXvalmin _ self typeCheck: aVar shouldBe: GXReal paramName: ''gXvalmin'']';

addInstVarName: #gXobject;
compile:
'gXobject: aVar
	aVar notNil ifTrue:[gXobject _ self typeCheck: aVar shouldBe: GXGobject paramName: ''gXobject'']';
compile:
'p6: aVar
	aVar notNil ifTrue:[gXobject _ self typeCheck: aVar shouldBe: GXGobject paramName: ''gXobject'']'!

"result
 ------"

GXcell

addInstVarName: #gXres
!

!GXcell methodsFor: 'initializing'!

initVars
	gXalter _ false.
	gXdraw _ false.
	gXsmooth _ false.
	gXraw _ false.
	gXlog _ false.
	gXsqrt _ false.
	gXlinear _ false
!!

"BODY
 ----"

!GXcell methodsFor: 'running'!

evaluate

   |tmp a b dev pic wind object|

((gXalter) & (gXdraw)  ) 
ifTrue:['ERROR, ' pren.
	'You Cannot draw and Alter at the same time' prinl]
ifFalse:[
   (gXUUstatus = 0)
   ifTrue:['ERROR, graphics device is not open' prenl.
           'To open enter DEVICE/OPEN' prinl]
   ifFalse:[gXdraw 
	    ifTrue:[gXsmooth ifTrue:[Cell defaultType: 1].
		    gXraw ifTrue:[Cell defaultType: 0].
		    gXlog ifTrue:[Cell defaultIType: 1].
                    gXsqrt ifTrue:[Cell defaultIType: 2].
                    gXlinear ifTrue:[Cell defaultIType: 0].

                   tmp _ (Cell new values: gXvalues; valMin: gXvalmin;
                         table: gXtable; tr: gXtr; valMax: gXvalmax; create).
                  ((gXUUgraphics at: #gXcurrentwindow) = nil)
                  ifTrue:['DRAW ERROR, ' pren.
			  'Window has not been created' prinl]
                ifFalse:[ ((gXUUgraphics at: #gXcurrentwindow) addToList: tmp).
                         gXres _ tmp.
                      a _ ((gXUUgraphics at: #gXcurrentwindow) getNumber: nil).
                      (gXUUgraphics at: #gXcurrentwindow) drawOnTop: a]]].

   gXalter ifTrue:[
	(gXobject = nil) ifTrue:['object not specified' prinl].
	(gXobject class = Cell)
                   ifTrue:[object _ ((gXUUgraphics at: #gXcurrentgraphics)
                                          findDrawableObjectsPicture: gXobject).
                   (object class = Cell)
                   ifTrue:[pic _ ((gXUUgraphics at: #gXcurrentgraphics)
                                                         findPicture: object).
                            dev _ ((gXUUgraphics at: #gXcurrentgraphics)
                                                               findDevice: pic).
                            wind _ (pic findRightWindow: object).

                           gXsmooth ifTrue:[a _ 1]
                                    ifFalse:[gXraw ifTrue:[a _ 0]
                                                   ifFalse:[a _ (object type)]].
                       gXlog ifTrue:[ b _ 1]
                          ifFalse:[gXsqrt ifTrue:[b _ 2]
                                     ifFalse:[gXlinear ifTrue:[b _ 0]
                                                ifFalse:[b _ (object itype)]]].

                            (gXvalues isNil)
                            ifFalse:['Values Not Yet Implemented' prnl].
                            (gXtable isNil)
                            ifFalse:[object _ (object table: gXtable; type: a; 
					       itype: b; create)].
                            (gXtr isNil)
                            ifFalse:['TR Not Yet Imlemented' prnl].
                            (gXvalmin isNil)
                            ifFalse:[object _ (object valMin: gXvalmin; type: a;
                                                     itype: b; create)].
                            (gXvalmax isNil)
                            ifFalse:[object _ (object valMax: gXvalmax; type: a;
                                                itype: b; create)].
                            ((gXtable isNil) &                             (gXvalmin isNil) &                             (gXvalmax isNil) )
                            ifTrue:[object _ (object itype: b; type: a; 
					      create)].

                   	    (object class = Cell)
                	    ifTrue:[gXobject update: object.
				    pic drawAllWindows]
                	    ifFalse:['Object has not been altered' prinl]]]]].

(gXalter | gXdraw)
ifFalse:['ERROR, ' pren.
	 '/DRAW or /ALTER has not been specified' prinl]


.
self up.
^gXres
!!
nil! "(fix for smalltalk bug, this line is ignored if there is a variable declaration error)"

"End of procedure"

"Procedure class definition:
 ==========================="
((Smalltalk at: #GXcellUarray ifAbsent: [ nil ]) isKindOf: Function class) ifTrue: [ ('Warning: Overriding previously defined procedure ', 'GXcellUarray' gtype) prinl ].
Smalltalk at: #GXcellUarray put: nil!
Function subclass: 'GXcellUarray'
       instanceVariableNames: ''
       classVariableNames: ''
       poolDictionaries: ''
       category: 'Genie Procedure'
!

"HEADER
 ------"

"qualifiers
 ----------"

GXcellUarray

addInstVarName: #gXalter;
compile:
'gXalter
	gXalter _ true';
compile:
'gXalter: aBoolean
	gXalter _ aBoolean';

addInstVarName: #gXdraw;
compile:
'gXdraw
	gXdraw _ true';
compile:
'gXdraw: aBoolean
	gXdraw _ aBoolean';

addInstVarName: #gXsmooth;
compile:
'gXsmooth
	gXsmooth _ true';
compile:
'gXsmooth: aBoolean
	gXsmooth _ aBoolean';

addInstVarName: #gXraw;
compile:
'gXraw
	gXraw _ true';
compile:
'gXraw: aBoolean
	gXraw _ aBoolean';

addInstVarName: #gXlog;
compile:
'gXlog
	gXlog _ true';
compile:
'gXlog: aBoolean
	gXlog _ aBoolean';

addInstVarName: #gXsqrt;
compile:
'gXsqrt
	gXsqrt _ true';
compile:
'gXsqrt: aBoolean
	gXsqrt _ aBoolean';

addInstVarName: #gXlinear;
compile:
'gXlinear
	gXlinear _ true';
compile:
'gXlinear: aBoolean
	gXlinear _ aBoolean'!

"parameters
 ----------"

GXcellUarray

addInstVarName: #gXvalues;
compile:
'gXvalues: aVar
	aVar notNil ifTrue:[gXvalues _ self typeCheck: aVar shouldBe: GXRealarray paramName: ''gXvalues'']';
compile:
'p1: aVar
	aVar notNil ifTrue:[gXvalues _ self typeCheck: aVar shouldBe: GXRealarray paramName: ''gXvalues'']';

addInstVarName: #gXtable;
compile:
'gXtable: aVar
	aVar notNil ifTrue:[gXtable _ self typeCheck: aVar shouldBe: GXColourtable paramName: ''gXtable'']';
compile:
'p2: aVar
	aVar notNil ifTrue:[gXtable _ self typeCheck: aVar shouldBe: GXColourtable paramName: ''gXtable'']';

addInstVarName: #gXxarray;
compile:
'gXxarray: aVar
	aVar notNil ifTrue:[gXxarray _ self typeCheck: aVar shouldBe: GXRealarray paramName: ''gXxarray'']';
compile:
'p3: aVar
	aVar notNil ifTrue:[gXxarray _ self typeCheck: aVar shouldBe: GXRealarray paramName: ''gXxarray'']';

addInstVarName: #gXyarray;
compile:
'gXyarray: aVar
	aVar notNil ifTrue:[gXyarray _ self typeCheck: aVar shouldBe: GXRealarray paramName: ''gXyarray'']';
compile:
'p4: aVar
	aVar notNil ifTrue:[gXyarray _ self typeCheck: aVar shouldBe: GXRealarray paramName: ''gXyarray'']'!

"parameters
 ----------"

GXcellUarray

addInstVarName: #gXvalmax;
compile:
'gXvalmax: aVar
	aVar notNil ifTrue:[gXvalmax _ self typeCheck: aVar shouldBe: GXReal paramName: ''gXvalmax'']';
compile:
'p5: aVar
	aVar notNil ifTrue:[gXvalmax _ self typeCheck: aVar shouldBe: GXReal paramName: ''gXvalmax'']';

addInstVarName: #gXvalmin;
compile:
'gXvalmin: aVar
	aVar notNil ifTrue:[gXvalmin _ self typeCheck: aVar shouldBe: GXReal paramName: ''gXvalmin'']';
compile:
'p6: aVar
	aVar notNil ifTrue:[gXvalmin _ self typeCheck: aVar shouldBe: GXReal paramName: ''gXvalmin'']';

addInstVarName: #gXobject;
compile:
'gXobject: aVar
	aVar notNil ifTrue:[gXobject _ self typeCheck: aVar shouldBe: GXGobject paramName: ''gXobject'']';
compile:
'p7: aVar
	aVar notNil ifTrue:[gXobject _ self typeCheck: aVar shouldBe: GXGobject paramName: ''gXobject'']'!

"result
 ------"

GXcellUarray

addInstVarName: #gXres
!

!GXcellUarray methodsFor: 'initializing'!

initVars
	gXalter _ false.
	gXdraw _ false.
	gXsmooth _ false.
	gXraw _ false.
	gXlog _ false.
	gXsqrt _ false.
	gXlinear _ false
!!

"BODY
 ----"

!GXcellUarray methodsFor: 'running'!

evaluate

   |tmp a b dev pic wind object|

((gXalter) & (gXdraw)  ) 
ifTrue:['ERROR, ' pren.
	'You Cannot draw and Alter at the same time' prinl]
ifFalse:[
   (gXUUstatus = 0)
   ifTrue:['ERROR, graphics device is not open' prenl.
           'To open enter DEVICE/OPEN' prinl]
   ifFalse:[gXdraw 
	    ifTrue:[gXsmooth ifTrue:[CellArray defaultType: 1].
		    gXraw ifTrue:[CellArray defaultType: 0].
		    gXlog ifTrue:[CellArray defaultIType: 1].
                    gXsqrt ifTrue:[CellArray defaultIType: 2].
                    gXlinear ifTrue:[CellArray defaultIType: 0].

                  ((gXUUgraphics at: #gXcurrentwindow) = nil)
                  ifTrue:['DRAW ERROR, ' pren.
			  'Window has not been created' prinl]
                ifFalse:[ tmp _ (CellArray new values: gXvalues; valMin: gXvalmin;
                         table: gXtable; wX: gXxarray; wY: gXyarray;
                         valMax: gXvalmax; create).
			((gXUUgraphics at: #gXcurrentwindow) addToList: tmp).
                         gXres _ tmp.
                      a _ ((gXUUgraphics at: #gXcurrentwindow) getNumber: nil).
                      (gXUUgraphics at: #gXcurrentwindow) drawOnTop: a]]].
"
                   tmp _ (CellArray new values: gXvalues; valMin: gXvalmin;
                         table: gXtable; wX: gXxarray; wY: gXyarray;
                         valMax: gXvalmax; create).
                  ((gXUUgraphics at: #gXcurrentwindow) = nil)
                  ifTrue:['DRAW ERROR, ' pren.
			  'Window has not been created' prinl]
                ifFalse:[ ((gXUUgraphics at: #gXcurrentwindow) addToList: tmp).
                         gXres _ tmp.
                      a _ ((gXUUgraphics at: #gXcurrentwindow) getNumber: nil).
                      (gXUUgraphics at: #gXcurrentwindow) drawOnTop: a]]].
"
   gXalter ifTrue:[
	(gXobject = nil) ifTrue:['object not specified' prinl].
	(gXobject class = CellArray)
                   ifTrue:[object _ ((gXUUgraphics at: #gXcurrentgraphics)
                                          findDrawableObjectsPicture: gXobject).
                   (object class = CellArray)
                   ifTrue:[pic _ ((gXUUgraphics at: #gXcurrentgraphics)
                                                         findPicture: object).
                            dev _ ((gXUUgraphics at: #gXcurrentgraphics)
                                                               findDevice: pic).
                            wind _ (pic findRightWindow: object).

                            gXsmooth ifTrue:[a _ 1]
                               ifFalse:[gXraw ifTrue:[a _ 0]
                                          ifFalse:[a _ (object type)]].

                       gXlog ifTrue:[ b _ 1]
                         ifFalse:[gXsqrt ifTrue:[b _ 2]
                                     ifFalse:[gXlinear ifTrue:[b _ 0]
                                                ifFalse:[b _ (object itype)]]].
                            (gXvalues isNil)
                            ifFalse:['Values Not Yet Implemented' prnl].
                            (gXtable isNil)
                            ifFalse:[object _ (object table: gXtable; type: a; 
					       itype: b; create)].
                            (gXxarray isNil)
                            ifFalse:['XArray Not Yet Imlemented' prnl].
                            (gXyarray isNil)
                            ifFalse:['XArray Not Yet Imlemented' prnl].
                            (gXvalmin = nil)
                            ifFalse:[object _ (object valMin: gXvalmin; 
					        type: a; itype: b; create)].
                            (gXvalmax isNil)
                            ifFalse:[object _ (object valMax: gXvalmax; 
						type: a; itype: b; create)].
                            ((gXtable isNil) &                             (gXvalmin isNil) &                             (gXvalmax isNil) )
                            ifTrue:[object _ (object itype: b; type: a; 
					      create)].

                   	    (object class = CellArray)
                	    ifTrue:[gXobject update: object.
				    pic drawAllWindows]
                	    ifFalse:['Object has not been altered' prinl]]]]].

(gXalter | gXdraw)
ifFalse:['ERROR, ' pren.
	 '/DRAW or /ALTER has not been specified' prinl]


.
self up.
^gXres
!!
nil! "(fix for smalltalk bug, this line is ignored if there is a variable declaration error)"

"End of procedure"

"Procedure class definition:
 ==========================="
((Smalltalk at: #GXcellUfunction ifAbsent: [ nil ]) isKindOf: Function class) ifTrue: [ ('Warning: Overriding previously defined procedure ', 'GXcellUfunction' gtype) prinl ].
Smalltalk at: #GXcellUfunction put: nil!
Function subclass: 'GXcellUfunction'
       instanceVariableNames: ''
       classVariableNames: ''
       poolDictionaries: ''
       category: 'Genie Procedure'
!

"HEADER
 ------"

"qualifiers
 ----------"

GXcellUfunction

addInstVarName: #gXalter;
compile:
'gXalter
	gXalter _ true';
compile:
'gXalter: aBoolean
	gXalter _ aBoolean';

addInstVarName: #gXdraw;
compile:
'gXdraw
	gXdraw _ true';
compile:
'gXdraw: aBoolean
	gXdraw _ aBoolean';

addInstVarName: #gXsmooth;
compile:
'gXsmooth
	gXsmooth _ true';
compile:
'gXsmooth: aBoolean
	gXsmooth _ aBoolean';

addInstVarName: #gXraw;
compile:
'gXraw
	gXraw _ true';
compile:
'gXraw: aBoolean
	gXraw _ aBoolean';

addInstVarName: #gXlog;
compile:
'gXlog
	gXlog _ true';
compile:
'gXlog: aBoolean
	gXlog _ aBoolean';

addInstVarName: #gXsqrt;
compile:
'gXsqrt
	gXsqrt _ true';
compile:
'gXsqrt: aBoolean
	gXsqrt _ aBoolean';

addInstVarName: #gXlinear;
compile:
'gXlinear
	gXlinear _ true';
compile:
'gXlinear: aBoolean
	gXlinear _ aBoolean'!

"parameters
 ----------"

GXcellUfunction

addInstVarName: #gXvalues;
compile:
'gXvalues: aVar
	aVar notNil ifTrue:[gXvalues _ self typeCheck: aVar shouldBe: GXRealarray paramName: ''gXvalues'']';
compile:
'p1: aVar
	aVar notNil ifTrue:[gXvalues _ self typeCheck: aVar shouldBe: GXRealarray paramName: ''gXvalues'']';

addInstVarName: #gXtable;
compile:
'gXtable: aVar
	aVar notNil ifTrue:[gXtable _ self typeCheck: aVar shouldBe: GXColourtable paramName: ''gXtable'']';
compile:
'p2: aVar
	aVar notNil ifTrue:[gXtable _ self typeCheck: aVar shouldBe: GXColourtable paramName: ''gXtable'']';

addInstVarName: #gXfunctionUname;
compile:
'gXfunctionUname: aVar
	aVar notNil ifTrue:[gXfunctionUname _ self typeCheck: aVar shouldBe: GXString paramName: ''gXfunctionUname'']';
compile:
'p3: aVar
	aVar notNil ifTrue:[gXfunctionUname _ self typeCheck: aVar shouldBe: GXString paramName: ''gXfunctionUname'']'!

"parameters
 ----------"

GXcellUfunction

addInstVarName: #gXvalmax;
compile:
'gXvalmax: aVar
	aVar notNil ifTrue:[gXvalmax _ self typeCheck: aVar shouldBe: GXReal paramName: ''gXvalmax'']';
compile:
'p4: aVar
	aVar notNil ifTrue:[gXvalmax _ self typeCheck: aVar shouldBe: GXReal paramName: ''gXvalmax'']';

addInstVarName: #gXvalmin;
compile:
'gXvalmin: aVar
	aVar notNil ifTrue:[gXvalmin _ self typeCheck: aVar shouldBe: GXReal paramName: ''gXvalmin'']';
compile:
'p5: aVar
	aVar notNil ifTrue:[gXvalmin _ self typeCheck: aVar shouldBe: GXReal paramName: ''gXvalmin'']';

addInstVarName: #gXobject;
compile:
'gXobject: aVar
	aVar notNil ifTrue:[gXobject _ self typeCheck: aVar shouldBe: GXGobject paramName: ''gXobject'']';
compile:
'p6: aVar
	aVar notNil ifTrue:[gXobject _ self typeCheck: aVar shouldBe: GXGobject paramName: ''gXobject'']'!

"result
 ------"

GXcellUfunction

addInstVarName: #gXres
!

!GXcellUfunction methodsFor: 'initializing'!

initVars
	gXalter _ false.
	gXdraw _ false.
	gXsmooth _ false.
	gXraw _ false.
	gXlog _ false.
	gXsqrt _ false.
	gXlinear _ false
!!

"BODY
 ----"

!GXcellUfunction methodsFor: 'running'!

evaluate

   |tmp a b dev pic wind object|

((gXalter) & (gXdraw)  ) 
ifTrue:['ERROR, ' pren.
	'You Cannot draw and Alter at the same time' prinl]
ifFalse:[
   (gXUUstatus = 0)
   ifTrue:['ERROR, graphics device is not open' prenl.
           'To open enter DEVICE/OPEN' prinl]
   ifFalse:[gXdraw 
	    ifTrue:[gXsmooth
                       ifTrue:[gXlog
                               ifTrue:[CellFunction defaultIType: 1]
                               ifFalse:[gXsqrt
                                        ifTrue:[CellFunction defaultIType: 2]
                                        ifFalse:[CellFunction defaultIType: 0]]]
                        ifFalse:[CellFunction defaultType: 0].
                   tmp _ (CellFunction new values: gXvalues; valMin: gXvalmin;
                         table: gXtable; funcPtr: gXfunctionUname;
                         valMax: gXvalmax; create).
                  ((gXUUgraphics at: #gXcurrentwindow) = nil)
                          ifTrue:['DRAW ERROR, ' pren.
				  'Window has not been created' prinl]
                ifFalse:[ ((gXUUgraphics at: #gXcurrentwindow) addToList: tmp).
                         gXres _ tmp.
                     a _ ((gXUUgraphics at: #gXcurrentwindow) getNumber: nil).
                      (gXUUgraphics at: #gXcurrentwindow) drawOnTop: a]]].

   gXalter ifTrue:[
	(gXobject = nil) ifTrue:['object not specified' prinl].
	(gXobject class = CellFunction)
        ifTrue:[object _ ((gXUUgraphics at: #gXcurrentgraphics)
                                          findDrawableObjectsPicture: gXobject).
                (object class = CellFunction)
                ifTrue:[pic _ ((gXUUgraphics at: #gXcurrentgraphics)
                                                         findPicture: object).
                        dev _ ((gXUUgraphics at: #gXcurrentgraphics)
                                                               findDevice: pic).
                        wind _ (pic findRightWindow: object).

                    	gXsmooth ifTrue:[a _ 1].
                       	gXraw ifTrue:[a _ 0].
                       	gXlog ifTrue:[b _ 1].
                       	gXsqrt ifTrue:[b _ 2].
                       	gXlinear ifTrue:[b _ 0].

                       	(gXvalues isNil)
                       	ifFalse:['Values Not Yet Implemented' prnl].
                       	(gXtable isNil)
                        ifFalse:[object _ (object table: gXtable; type: a; 
			 	      	   itype: b; create)].
                        (gXfunctionUname isNil)
                        ifFalse:[object _ (object wX: gXfunctionUname; 
					   type: a; itype: b; create)].
                        (gXvalmin isNil)
                        ifFalse:[object _ (object valMin: gXvalmin; type: a;
                                           itype: b; create)].
                        (gXvalmax isNil)
                        ifFalse:[object _ (object valMax: gXvalmax; type: a;
                                           itype: b; create)].
                        ((gXtable isNil) &                         (gXfunctionUname isNil) &                         (gXvalmin isNil) &                         (gXvalmax isNil) )
                        ifTrue:[object _ (object itype: b; type: a; 
					      create)].

                	(object class = CellFunction)
                	ifTrue:[gXres - object.
			    	pic drawAllWindows]
                	ifFalse:['Object has not been altered' prinl]]]]].

(gXalter | gXdraw)
ifFalse:['ERROR, ' pren.
	 '/DRAW or /ALTER has not been specified' prinl]

.
self up.
^gXres
!!
nil! "(fix for smalltalk bug, this line is ignored if there is a variable declaration error)"

"End of procedure"

"Procedure class definition:
 ==========================="
((Smalltalk at: #GXcellUwedge ifAbsent: [ nil ]) isKindOf: Function class) ifTrue: [ ('Warning: Overriding previously defined procedure ', 'GXcellUwedge' gtype) prinl ].
Smalltalk at: #GXcellUwedge put: nil!
Function subclass: 'GXcellUwedge'
       instanceVariableNames: ''
       classVariableNames: ''
       poolDictionaries: ''
       category: 'Genie Procedure'
!

"HEADER
 ------"

"qualifiers
 ----------"

GXcellUwedge

addInstVarName: #gXalter;
compile:
'gXalter
	gXalter _ true';
compile:
'gXalter: aBoolean
	gXalter _ aBoolean';

addInstVarName: #gXdraw;
compile:
'gXdraw
	gXdraw _ true';
compile:
'gXdraw: aBoolean
	gXdraw _ aBoolean';

addInstVarName: #gXhorizontal;
compile:
'gXhorizontal
	gXhorizontal _ true';
compile:
'gXhorizontal: aBoolean
	gXhorizontal _ aBoolean';

addInstVarName: #gXvertical;
compile:
'gXvertical
	gXvertical _ true';
compile:
'gXvertical: aBoolean
	gXvertical _ aBoolean'!

"parameters
 ----------"

GXcellUwedge

addInstVarName: #gXobject;
compile:
'gXobject: aVar
	aVar notNil ifTrue:[gXobject _ self typeCheck: aVar shouldBe: GXGobject paramName: ''gXobject'']';
compile:
'p1: aVar
	aVar notNil ifTrue:[gXobject _ self typeCheck: aVar shouldBe: GXGobject paramName: ''gXobject'']'!

"result
 ------"

GXcellUwedge

addInstVarName: #gXres
!

!GXcellUwedge methodsFor: 'initializing'!

initVars
	gXalter _ false.
	gXdraw _ false.
	gXhorizontal _ false.
	gXvertical _ false
!!

"BODY
 ----"

!GXcellUwedge methodsFor: 'running'!

evaluate

   |tmp aNumber a dev pic wind object|
((gXalter) & (gXdraw)  ) 
ifTrue:['ERROR, ' pren.
	'You Cannot draw and Alter at the same time' prinl]
ifFalse:[
   (gXUUstatus = 0)
   ifTrue:['ERROR, graphics device is not open' prenl.
           'To open enter DEVICE/OPEN' prinl]
   ifFalse:[gXdraw 
	    ifTrue:[gXhorizontal ifTrue:[a _ 0 ]
			       ifFalse:[a _ 1].
                  gXvertical ifTrue:[a _ 1]
			     ifFalse:[a _ 0].
                  (gXobject = nil)
           ifTrue:['ERROR, a cell has not been specified' prenl.
                   'Enter window number then object number' prinl]

           ifFalse:[(gXobject isKindOf: TwoDCell)
                    ifTrue:[tmp _ (CellWedge new cellName: gXobject;
                                   iMode: a; create).

                  ((gXUUgraphics at: #gXcurrentwindow) = nil)
                          ifTrue:['DRAW ERROR, ' pren.
				  'Window has not been created' prinl]
                ifFalse:[ ((gXUUgraphics at: #gXcurrentwindow) addToList: tmp).
                         gXres _ tmp.
                      a _ ((gXUUgraphics at: #gXcurrentwindow) getNumber: nil).
                      (gXUUgraphics at: #gXcurrentwindow) drawOnTop: a]]]].

   gXalter ifTrue:[
	(gXobject = nil) ifTrue:['object not specified' prinl].
	(gXobject class = CellWedge)
        ifTrue:[object _ ((gXUUgraphics at: #gXcurrentgraphics)
                                          findDrawableObjectsPicture: gXobject).
                (object class = CellWedge)
                ifTrue:[pic _ ((gXUUgraphics at: #gXcurrentgraphics)
                                                         findPicture: object).
                        dev _ ((gXUUgraphics at: #gXcurrentgraphics)
                                                               findDevice: pic).
                        wind _ (pic findRightWindow: object).

                   	gXhorizontal ifTrue:[a _ 1].
                   	gXvertical ifTrue:[a _ 0].
                   	object _ (object iMode: a; create).
                   	(object class = CellWedge)
                	ifTrue:[gXres - tmp.
			        pic drawAllWindows]
                	ifFalse:['Object has not been altered' prinl]]]]]].

(gXalter | gXdraw)
ifFalse:['ERROR, ' pren.
	 '/DRAW or /ALTER has not been specified' prinl]


.
self up.
^gXres
!!
nil! "(fix for smalltalk bug, this line is ignored if there is a variable declaration error)"

"End of procedure"

"Procedure class definition:
 ==========================="
((Smalltalk at: #GXmultiUplot ifAbsent: [ nil ]) isKindOf: Function class) ifTrue: [ ('Warning: Overriding previously defined procedure ', 'GXmultiUplot' gtype) prinl ].
Smalltalk at: #GXmultiUplot put: nil!
Function subclass: 'GXmultiUplot'
       instanceVariableNames: ''
       classVariableNames: ''
       poolDictionaries: ''
       category: 'Genie Procedure'
!

"HEADER
 ------"

"qualifiers
 ----------"

GXmultiUplot

addInstVarName: #gXcreate;
compile:
'gXcreate
	gXcreate _ true';
compile:
'gXcreate: aBoolean
	gXcreate _ aBoolean';

addInstVarName: #gXspectra;
compile:
'gXspectra
	gXspectra _ true';
compile:
'gXspectra: aBoolean
	gXspectra _ aBoolean';

addInstVarName: #gXdraw;
compile:
'gXdraw
	gXdraw _ true';
compile:
'gXdraw: aBoolean
	gXdraw _ aBoolean';

addInstVarName: #gXalter;
compile:
'gXalter
	gXalter _ true';
compile:
'gXalter: aBoolean
	gXalter _ aBoolean';

addInstVarName: #gXbincentre;
compile:
'gXbincentre
	gXbincentre _ true';
compile:
'gXbincentre: aBoolean
	gXbincentre _ aBoolean';

addInstVarName: #gXnotcentred;
compile:
'gXnotcentred
	gXnotcentred _ true';
compile:
'gXnotcentred: aBoolean
	gXnotcentred _ aBoolean'!

"parameters
 ----------"

GXmultiUplot

addInstVarName: #gXdata;
compile:
'gXdata: aVar
	aVar notNil ifTrue:[ gXdata _ aVar ]';
compile:
'p1: aVar
	aVar notNil ifTrue:[ gXdata _ aVar ]';

addInstVarName: #gXnumhist;
compile:
'gXnumhist: aVar
	aVar notNil ifTrue:[gXnumhist _ self typeCheck: aVar shouldBe: GXInteger paramName: ''gXnumhist'']';
compile:
'p2: aVar
	aVar notNil ifTrue:[gXnumhist _ self typeCheck: aVar shouldBe: GXInteger paramName: ''gXnumhist'']';

addInstVarName: #gXchmin;
compile:
'gXchmin: aVar
	aVar notNil ifTrue:[gXchmin _ self typeCheck: aVar shouldBe: GXInteger paramName: ''gXchmin'']';
compile:
'p3: aVar
	aVar notNil ifTrue:[gXchmin _ self typeCheck: aVar shouldBe: GXInteger paramName: ''gXchmin'']';

addInstVarName: #gXchmax;
compile:
'gXchmax: aVar
	aVar notNil ifTrue:[gXchmax _ self typeCheck: aVar shouldBe: GXInteger paramName: ''gXchmax'']';
compile:
'p4: aVar
	aVar notNil ifTrue:[gXchmax _ self typeCheck: aVar shouldBe: GXInteger paramName: ''gXchmax'']';

addInstVarName: #gXygap;
compile:
'gXygap: aVar
	aVar notNil ifTrue:[gXygap _ self typeCheck: aVar shouldBe: GXReal paramName: ''gXygap'']';
compile:
'p5: aVar
	aVar notNil ifTrue:[gXygap _ self typeCheck: aVar shouldBe: GXReal paramName: ''gXygap'']'!

"parameters
 ----------"

GXmultiUplot

addInstVarName: #gXcolour;
compile:
'gXcolour: aVar
	aVar notNil ifTrue:[gXcolour _ self typeCheck: aVar shouldBe: GXColour paramName: ''gXcolour'']';
compile:
'p6: aVar
	aVar notNil ifTrue:[gXcolour _ self typeCheck: aVar shouldBe: GXColour paramName: ''gXcolour'']';

addInstVarName: #gXlineUtype;
compile:
'gXlineUtype: aVar
	aVar notNil ifTrue:[gXlineUtype _ self typeCheck: aVar shouldBe: GXLinestyle paramName: ''gXlineUtype'']';
compile:
'p7: aVar
	aVar notNil ifTrue:[gXlineUtype _ self typeCheck: aVar shouldBe: GXLinestyle paramName: ''gXlineUtype'']';

addInstVarName: #gXlineUthickness;
compile:
'gXlineUthickness: aVar
	aVar notNil ifTrue:[gXlineUthickness _ self typeCheck: aVar shouldBe: GXReal paramName: ''gXlineUthickness'']';
compile:
'p8: aVar
	aVar notNil ifTrue:[gXlineUthickness _ self typeCheck: aVar shouldBe: GXReal paramName: ''gXlineUthickness'']';

addInstVarName: #gXobject;
compile:
'gXobject: aVar
	aVar notNil ifTrue:[gXobject _ self typeCheck: aVar shouldBe: GXGobject paramName: ''gXobject'']';
compile:
'p9: aVar
	aVar notNil ifTrue:[gXobject _ self typeCheck: aVar shouldBe: GXGobject paramName: ''gXobject'']'!

"result
 ------"

GXmultiUplot

addInstVarName: #gXres
!

!GXmultiUplot methodsFor: 'initializing'!

initVars
	gXcreate _ false.
	gXspectra _ false.
	gXdraw _ false.
	gXalter _ false.
	gXbincentre _ false.
	gXnotcentred _ false
!!

"BODY
 ----"

!GXmultiUplot methodsFor: 'running'!

evaluate

   |tmp a count dev pic wind object|

((gXspectra) & (gXcreate) )
ifTrue:['ERROR, ' pren.
	'You cannot create and add at the same time' prinl].
((gXspectra) & (gXdraw) )
ifTrue:['ERROR, ' pren.
	'You cannot add and draw at the same time' prinl].
((gXalter) & (gXcreate) & (gXspectra) & (gXdraw)  ) 
ifTrue:['ERROR, ' pren.
	'You Cannot draw, alter, create and add at the same time' prinl]
ifFalse:[
   (gXUUstatus = 0)
   ifTrue:['ERROR, graphics device is not open' prenl.
           'To open enter DEVICE/OPEN' prinl]
   ifFalse:[gXcreate 
	    ifTrue:[gXbincentre ifTrue:[Multiplot defaultBin: 1]
			        ifFalse:[Multiplot defaultBin: 0].
		    gXnotcentred ifTrue:[Multiplot defaultBin: 0].
		    tmp _ (Multiplot new xarray: gXdata; nHist: gXnumhist;
			   ygap: gXygap; chmin: gXchmin; chmax: gXchmax;
			   colour: gXcolour; type: gXlineUtype; 
			   lineThickness: gXlineUthickness; create). 
			   count _ 1.
			   Smalltalk at: #tmp put: tmp.
			   Smalltalk at: #count put: count].

   gXspectra ifTrue:[((Smalltalk at: #count) = 1)
		     ifTrue:[
			      (gXdata isKindOf: GXRealarray) 
					ifTrue: [ tmp _ ((Smalltalk at: #tmp) yarray: gXdata; add) ]
		              		ifFalse: [ tmp _ ((Smalltalk at: #tmp) addSpectra: gXdata from: gXUcurrentUinputUsource with: gXUcurrentUbinning) ].
			      Smalltalk at: #tmp put: tmp.
			      Smalltalk at: #count put: 0.
			      Smalltalk at: #ydata put: gXdata]
		     ifFalse:['ERROR, ' prenl.
			      'multiplot needs to be created first' prinl]].

   gXdraw ifTrue:[tmp _ ((Smalltalk at: #tmp ) make).
                  ((gXUUgraphics at: #gXcurrentwindow) = nil)
                          ifTrue:['DRAW ERROR, ' pren.
				  'Window has not been created' prinl]
			  ifFalse:[((gXUUgraphics at: #gXcurrentwindow) 
								addToList: tmp).
                         	   gXres _ tmp.
                      	   	   a _ ((gXUUgraphics at: #gXcurrentwindow) 
								getNumber: nil).
                      		  (gXUUgraphics at: #gXcurrentwindow) 
								drawOnTop: a]]].

   gXalter ifTrue:[
	(gXobject = nil) ifTrue:['object not specified' prinl].
	(gXobject class = Multiplot)
        ifTrue:[object _ ((gXUUgraphics at: #gXcurrentgraphics)
                                          findDrawableObjectsPicture: gXobject).
                (object class = Multiplot)
                ifTrue:[pic _ ((gXUUgraphics at: #gXcurrentgraphics)
                                                         findPicture: object).
                        dev _ ((gXUUgraphics at: #gXcurrentgraphics)
                                                              findDevice: pic).
                        wind _ (pic findRightWindow: object).

		   	(gXdata isNil)
		   	ifFalse:['Data Not Yet Implemented' prnl].
                   	(gXnumhist isNil)
                   	ifFalse:[object _ (object nHist: gXnumhist; create; 
					   yarray: (object yarray); add)].
                   	(gXchmin isNil)
                   	ifFalse:[object _ (object chmin: gXchmin; create;
					   yarray: (object yarray); add)].
                   	(gXcolour isNil)
                   	ifFalse:[object _ (object colour: gXcolour; create; 
					   yarray: (object yarray); add)].
                   	(gXchmax isNil)
                   	ifFalse:[object _ (object chmax: gXchmax; create; 
					   yarray: (object yarray); add)].
                   	(gXygap isNil)
                   	ifFalse:[object _ (object ygap: gXygap; create; 
					   yarray: (object yarray); add)].
		   	(gXlineUtype isNil)
		   	ifFalse:[object _ (object type: gXlineUtype; create; 
					   yarray: (object yarray); add)].
  
                   	(object class = Multiplot)
			ifTrue:[object make.
	 			gXres - object.
			  	pic drawAllWindows]
			ifFalse:['Object has not been altered' prinl]]]]].


(gXalter | gXdraw | gXcreate | gXspectra)
ifFalse:['ERROR, ' pren.
	 '/DRAW or /ALTER has not been specified' prinl]



.
self up.
^gXres
!!
nil! "(fix for smalltalk bug, this line is ignored if there is a variable declaration error)"

"End of procedure"

"Procedure class definition:
 ==========================="
((Smalltalk at: #GXpicUadd ifAbsent: [ nil ]) isKindOf: Function class) ifTrue: [ ('Warning: Overriding previously defined procedure ', 'GXpicUadd' gtype) prinl ].
Smalltalk at: #GXpicUadd put: nil!
Function subclass: 'GXpicUadd'
       instanceVariableNames: ''
       classVariableNames: ''
       poolDictionaries: ''
       category: 'Genie Procedure'
!

"HEADER
 ------"

"parameters
 ----------"

GXpicUadd

addInstVarName: #gXitem;
compile:
'gXitem: aVar
	aVar notNil ifTrue:[gXitem _ self typeCheck: aVar shouldBe: GXGobject paramName: ''gXitem'']';
compile:
'p1: aVar
	aVar notNil ifTrue:[gXitem _ self typeCheck: aVar shouldBe: GXGobject paramName: ''gXitem'']';

addInstVarName: #gXdest;
compile:
'gXdest: aVar
	aVar notNil ifTrue:[gXdest _ self typeCheck: aVar shouldBe: GXGobject paramName: ''gXdest'']';
compile:
'p2: aVar
	aVar notNil ifTrue:[gXdest _ self typeCheck: aVar shouldBe: GXGobject paramName: ''gXdest'']'!

"result
 ------"

GXpicUadd

addInstVarName: #gXres
!

!GXpicUadd methodsFor: 'initializing'!

initVars

!!

"BODY
 ----"

!GXpicUadd methodsFor: 'running'!

evaluate

   |dev wind pic object item tmp|

   (gXdest isKindOf: Picture)
   ifTrue:[pic _ ((gXUUgraphics at: #gXcurrentgraphics) 
					    getPictureWithAPicture: gXdest)].

   (gXdest isKindOf: Window)
   ifTrue:[wind _ ((gXUUgraphics at: #gXcurrentgraphics)
					findDrawableObjectsPicture: gXdest)].

   (gXitem isKindOf: Picture)
   ifTrue:[item _ ((gXUUgraphics at: #gXcurrentgraphics)
						getPictureWithAPicture: gXitem).
	   (item isNil)
	   ifFalse:[(dev isNil)
	   	    ifTrue:[dev _ (gXUUgraphics at: #gXcurrentdevice)].
           	    item _ (item shallowCopy).
		    item nextLink: nil.
	   	    ((gXUUgraphics at: #gXcurrentgraphics) 
						     	addToPictureList: item).
	   	    dev associate: item]].

   (gXitem isKindOf: Window)
   ifTrue:[item _ ((gXUUgraphics at: #gXcurrentgraphics)
					findDrawableObjectsPicture: gXitem).
	   (item isKindOf: Window)
     	   ifTrue:[(pic isNil)
		   ifTrue:[pic _ (gXUUgraphics at: #gXcurrentpicture)].
	   	   (dev isNil)
		   ifTrue:[dev _ (gXUUgraphics at: #gXcurrentdevice)].
	           item _ (item shallowCopy).
		   item nextLink: nil.
		   pic addToWindowList: item.
		   dev associate: pic]].

   (gXitem isKindOf: DrawableObjects)		  
   ifTrue:[(gXitem isKindOf: Window)
	   ifFalse:[item _ ((gXUUgraphics at: #gXcurrentgraphics)
					    findDrawableObjectsPicture: gXitem).
		    (item isKindOf: DrawableObjects)
		    ifTrue:[(pic isNil)
			    ifTrue:[pic _ (gXUUgraphics at: #gXcurrentpicture)].
			    (wind isNil)
			    ifTrue:[wind _ (gXUUgraphics at: #gXcurrentwindow)].
			    (dev isNil)
			    ifTrue:[dev _ (gXUUgraphics at: #gXcurrentdevice)].
			    item _ (item shallowCopy).
			    item nextLink: nil.
			    wind displayList add: item.
			    dev associate: pic]]].

   (item isNil) ifFalse:[gXres _ item]
.
self up.
^gXres
!!
nil! "(fix for smalltalk bug, this line is ignored if there is a variable declaration error)"

"End of procedure"

GXcomment run line: 0 proc: self method: false; p1:'pic_add'; p2:'Display a picture in a new device'; condEvaluate!
"Procedure class definition:
 ==========================="
((Smalltalk at: #GXdraw ifAbsent: [ nil ]) isKindOf: Function class) ifTrue: [ ('Warning: Overriding previously defined procedure ', 'GXdraw' gtype) prinl ].
Smalltalk at: #GXdraw put: nil!
Function subclass: 'GXdraw'
       instanceVariableNames: ''
       classVariableNames: ''
       poolDictionaries: ''
       category: 'Genie Procedure'
!

"HEADER
 ------"

"qualifiers
 ----------"

GXdraw

addInstVarName: #gXaxes;
compile:
'gXaxes
	gXaxes _ true';
compile:
'gXaxes: aBoolean
	gXaxes _ aBoolean';

addInstVarName: #gXtext;
compile:
'gXtext
	gXtext _ true';
compile:
'gXtext: aBoolean
	gXtext _ aBoolean';

addInstVarName: #gXhistogram;
compile:
'gXhistogram
	gXhistogram _ true';
compile:
'gXhistogram: aBoolean
	gXhistogram _ aBoolean';

addInstVarName: #gXplot;
compile:
'gXplot
	gXplot _ true';
compile:
'gXplot: aBoolean
	gXplot _ aBoolean';

addInstVarName: #gXmarkers;
compile:
'gXmarkers
	gXmarkers _ true';
compile:
'gXmarkers: aBoolean
	gXmarkers _ aBoolean'!

"qualifiers
 ----------"

GXdraw

addInstVarName: #gXgraticule;
compile:
'gXgraticule
	gXgraticule _ true';
compile:
'gXgraticule: aBoolean
	gXgraticule _ aBoolean';

addInstVarName: #gXlabels;
compile:
'gXlabels
	gXlabels _ true';
compile:
'gXlabels: aBoolean
	gXlabels _ aBoolean';

addInstVarName: #gXtitle;
compile:
'gXtitle
	gXtitle _ true';
compile:
'gXtitle: aBoolean
	gXtitle _ aBoolean';

addInstVarName: #gXbox;
compile:
'gXbox
	gXbox _ true';
compile:
'gXbox: aBoolean
	gXbox _ aBoolean';

addInstVarName: #gXline;
compile:
'gXline
	gXline _ true';
compile:
'gXline: aBoolean
	gXline _ aBoolean';

addInstVarName: #gXerrors;
compile:
'gXerrors
	gXerrors _ true';
compile:
'gXerrors: aBoolean
	gXerrors _ aBoolean';

addInstVarName: #gXquality;
compile:
'gXquality
	gXquality _ true';
compile:
'gXquality: aBoolean
	gXquality _ aBoolean'!

"qualifiers
 ----------"

GXdraw

addInstVarName: #gXxlog;
compile:
'gXxlog
	gXxlog _ true';
compile:
'gXxlog: aBoolean
	gXxlog _ aBoolean';

addInstVarName: #gXylog;
compile:
'gXylog
	gXylog _ true';
compile:
'gXylog: aBoolean
	gXylog _ aBoolean';

addInstVarName: #gXfilled;
compile:
'gXfilled
	gXfilled _ true';
compile:
'gXfilled: aBoolean
	gXfilled _ aBoolean';

addInstVarName: #gXhorizontal;
compile:
'gXhorizontal
	gXhorizontal _ true';
compile:
'gXhorizontal: aBoolean
	gXhorizontal _ aBoolean';

addInstVarName: #gXvertical;
compile:
'gXvertical
	gXvertical _ true';
compile:
'gXvertical: aBoolean
	gXvertical _ aBoolean'!

"qualifiers
 ----------"

GXdraw

addInstVarName: #gXnoylog;
compile:
'gXnoylog
	gXnoylog _ true';
compile:
'gXnoylog: aBoolean
	gXnoylog _ aBoolean';

addInstVarName: #gXnoxlog;
compile:
'gXnoxlog
	gXnoxlog _ true';
compile:
'gXnoxlog: aBoolean
	gXnoxlog _ aBoolean';

addInstVarName: #gXpolygon;
compile:
'gXpolygon
	gXpolygon _ true';
compile:
'gXpolygon: aBoolean
	gXpolygon _ aBoolean'!

"qualifiers
 ----------"

GXdraw

addInstVarName: #gXnoxnum;
compile:
'gXnoxnum
	gXnoxnum _ true';
compile:
'gXnoxnum: aBoolean
	gXnoxnum _ aBoolean';

addInstVarName: #gXnoynum;
compile:
'gXnoynum
	gXnoynum _ true';
compile:
'gXnoynum: aBoolean
	gXnoynum _ aBoolean';

addInstVarName: #gXyhorizontal;
compile:
'gXyhorizontal
	gXyhorizontal _ true';
compile:
'gXyhorizontal: aBoolean
	gXyhorizontal _ aBoolean';

addInstVarName: #gXxvertical;
compile:
'gXxvertical
	gXxvertical _ true';
compile:
'gXxvertical: aBoolean
	gXxvertical _ aBoolean'!

"qualifiers
 ----------"

GXdraw

addInstVarName: #gXcontour;
compile:
'gXcontour
	gXcontour _ true';
compile:
'gXcontour: aBoolean
	gXcontour _ aBoolean';

addInstVarName: #gXcellarray;
compile:
'gXcellarray
	gXcellarray _ true';
compile:
'gXcellarray: aBoolean
	gXcellarray _ aBoolean'!

"qualifiers
 ----------"

GXdraw

addInstVarName: #gXarray;
compile:
'gXarray
	gXarray _ true';
compile:
'gXarray: aBoolean
	gXarray _ aBoolean';

addInstVarName: #gXfunction;
compile:
'gXfunction
	gXfunction _ true';
compile:
'gXfunction: aBoolean
	gXfunction _ aBoolean'!

"qualifiers
 ----------"

GXdraw

addInstVarName: #gXlog;
compile:
'gXlog
	gXlog _ true';
compile:
'gXlog: aBoolean
	gXlog _ aBoolean';

addInstVarName: #gXsqrt;
compile:
'gXsqrt
	gXsqrt _ true';
compile:
'gXsqrt: aBoolean
	gXsqrt _ aBoolean';

addInstVarName: #gXsmooth;
compile:
'gXsmooth
	gXsmooth _ true';
compile:
'gXsmooth: aBoolean
	gXsmooth _ aBoolean'!

"parameters
 ----------"

GXdraw

addInstVarName: #gXp1;
compile:
'gXp1: aVar
	aVar notNil ifTrue:[ gXp1 _ aVar ]';
compile:
'p1: aVar
	aVar notNil ifTrue:[ gXp1 _ aVar ]';

addInstVarName: #gXp2;
compile:
'gXp2: aVar
	aVar notNil ifTrue:[ gXp2 _ aVar ]';
compile:
'p2: aVar
	aVar notNil ifTrue:[ gXp2 _ aVar ]';

addInstVarName: #gXp3;
compile:
'gXp3: aVar
	aVar notNil ifTrue:[ gXp3 _ aVar ]';
compile:
'p3: aVar
	aVar notNil ifTrue:[ gXp3 _ aVar ]';

addInstVarName: #gXp4;
compile:
'gXp4: aVar
	aVar notNil ifTrue:[ gXp4 _ aVar ]';
compile:
'p4: aVar
	aVar notNil ifTrue:[ gXp4 _ aVar ]';

addInstVarName: #gXp5;
compile:
'gXp5: aVar
	aVar notNil ifTrue:[ gXp5 _ aVar ]';
compile:
'p5: aVar
	aVar notNil ifTrue:[ gXp5 _ aVar ]';

addInstVarName: #gXp6;
compile:
'gXp6: aVar
	aVar notNil ifTrue:[ gXp6 _ aVar ]';
compile:
'p6: aVar
	aVar notNil ifTrue:[ gXp6 _ aVar ]';

addInstVarName: #gXp7;
compile:
'gXp7: aVar
	aVar notNil ifTrue:[ gXp7 _ aVar ]';
compile:
'p7: aVar
	aVar notNil ifTrue:[ gXp7 _ aVar ]';

addInstVarName: #gXp8;
compile:
'gXp8: aVar
	aVar notNil ifTrue:[ gXp8 _ aVar ]';
compile:
'p8: aVar
	aVar notNil ifTrue:[ gXp8 _ aVar ]';

addInstVarName: #gXp9;
compile:
'gXp9: aVar
	aVar notNil ifTrue:[ gXp9 _ aVar ]';
compile:
'p9: aVar
	aVar notNil ifTrue:[ gXp9 _ aVar ]';

addInstVarName: #gXp10;
compile:
'gXp10: aVar
	aVar notNil ifTrue:[ gXp10 _ aVar ]';
compile:
'p10: aVar
	aVar notNil ifTrue:[ gXp10 _ aVar ]';

addInstVarName: #gXvalues;
compile:
'gXvalues: aVar
	aVar notNil ifTrue:[gXvalues _ self typeCheck: aVar shouldBe: GXRealarray paramName: ''gXvalues'']';
compile:
'p11: aVar
	aVar notNil ifTrue:[gXvalues _ self typeCheck: aVar shouldBe: GXRealarray paramName: ''gXvalues'']';

addInstVarName: #gXncont;
compile:
'gXncont: aVar
	aVar notNil ifTrue:[gXncont _ self typeCheck: aVar shouldBe: GXInteger paramName: ''gXncont'']';
compile:
'p12: aVar
	aVar notNil ifTrue:[gXncont _ self typeCheck: aVar shouldBe: GXInteger paramName: ''gXncont'']';

addInstVarName: #gXtr;
compile:
'gXtr: aVar
	aVar notNil ifTrue:[gXtr _ self typeCheck: aVar shouldBe: GXRealarray paramName: ''gXtr'']';
compile:
'p13: aVar
	aVar notNil ifTrue:[gXtr _ self typeCheck: aVar shouldBe: GXRealarray paramName: ''gXtr'']';

addInstVarName: #gXcolour;
compile:
'gXcolour: aVar
	aVar notNil ifTrue:[gXcolour _ self typeCheck: aVar shouldBe: GXColour paramName: ''gXcolour'']';
compile:
'p14: aVar
	aVar notNil ifTrue:[gXcolour _ self typeCheck: aVar shouldBe: GXColour paramName: ''gXcolour'']';

addInstVarName: #gXlinetype;
compile:
'gXlinetype: aVar
	aVar notNil ifTrue:[gXlinetype _ self typeCheck: aVar shouldBe: GXLinestyle paramName: ''gXlinetype'']';
compile:
'p15: aVar
	aVar notNil ifTrue:[gXlinetype _ self typeCheck: aVar shouldBe: GXLinestyle paramName: ''gXlinetype'']';

addInstVarName: #gXlinewidth;
compile:
'gXlinewidth: aVar
	aVar notNil ifTrue:[gXlinewidth _ self typeCheck: aVar shouldBe: GXReal paramName: ''gXlinewidth'']';
compile:
'p16: aVar
	aVar notNil ifTrue:[gXlinewidth _ self typeCheck: aVar shouldBe: GXReal paramName: ''gXlinewidth'']';

addInstVarName: #gXwx;
compile:
'gXwx: aVar
	aVar notNil ifTrue:[gXwx _ self typeCheck: aVar shouldBe: GXRealarray paramName: ''gXwx'']';
compile:
'p17: aVar
	aVar notNil ifTrue:[gXwx _ self typeCheck: aVar shouldBe: GXRealarray paramName: ''gXwx'']';

addInstVarName: #gXwy;
compile:
'gXwy: aVar
	aVar notNil ifTrue:[gXwy _ self typeCheck: aVar shouldBe: GXRealarray paramName: ''gXwy'']';
compile:
'p18: aVar
	aVar notNil ifTrue:[gXwy _ self typeCheck: aVar shouldBe: GXRealarray paramName: ''gXwy'']';

addInstVarName: #gXfunc;
compile:
'gXfunc: aVar
	aVar notNil ifTrue:[gXfunc _ self typeCheck: aVar shouldBe: GXString paramName: ''gXfunc'']';
compile:
'p19: aVar
	aVar notNil ifTrue:[gXfunc _ self typeCheck: aVar shouldBe: GXString paramName: ''gXfunc'']';

addInstVarName: #gXcontours;
compile:
'gXcontours: aVar
	aVar notNil ifTrue:[gXcontours _ self typeCheck: aVar shouldBe: GXRealarray paramName: ''gXcontours'']';
compile:
'p20: aVar
	aVar notNil ifTrue:[gXcontours _ self typeCheck: aVar shouldBe: GXRealarray paramName: ''gXcontours'']';

addInstVarName: #gXtextheight;
compile:
'gXtextheight: aVar
	aVar notNil ifTrue:[gXtextheight _ self typeCheck: aVar shouldBe: GXReal paramName: ''gXtextheight'']';
compile:
'p21: aVar
	aVar notNil ifTrue:[gXtextheight _ self typeCheck: aVar shouldBe: GXReal paramName: ''gXtextheight'']';

addInstVarName: #gXangle;
compile:
'gXangle: aVar
	aVar notNil ifTrue:[gXangle _ self typeCheck: aVar shouldBe: GXReal paramName: ''gXangle'']';
compile:
'p22: aVar
	aVar notNil ifTrue:[gXangle _ self typeCheck: aVar shouldBe: GXReal paramName: ''gXangle'']';

addInstVarName: #gXfont;
compile:
'gXfont: aVar
	aVar notNil ifTrue:[gXfont _ self typeCheck: aVar shouldBe: GXFont paramName: ''gXfont'']';
compile:
'p23: aVar
	aVar notNil ifTrue:[gXfont _ self typeCheck: aVar shouldBe: GXFont paramName: ''gXfont'']';

addInstVarName: #gXcolourtable;
compile:
'gXcolourtable: aVar
	aVar notNil ifTrue:[gXcolourtable _ self typeCheck: aVar shouldBe: GXColourtable paramName: ''gXcolourtable'']';
compile:
'p24: aVar
	aVar notNil ifTrue:[gXcolourtable _ self typeCheck: aVar shouldBe: GXColourtable paramName: ''gXcolourtable'']';

addInstVarName: #gXvalmax;
compile:
'gXvalmax: aVar
	aVar notNil ifTrue:[gXvalmax _ self typeCheck: aVar shouldBe: GXReal paramName: ''gXvalmax'']';
compile:
'p25: aVar
	aVar notNil ifTrue:[gXvalmax _ self typeCheck: aVar shouldBe: GXReal paramName: ''gXvalmax'']';

addInstVarName: #gXvalmin;
compile:
'gXvalmin: aVar
	aVar notNil ifTrue:[gXvalmin _ self typeCheck: aVar shouldBe: GXReal paramName: ''gXvalmin'']';
compile:
'p26: aVar
	aVar notNil ifTrue:[gXvalmin _ self typeCheck: aVar shouldBe: GXReal paramName: ''gXvalmin'']';

addInstVarName: #gXx;
compile:
'gXx: aVar
	aVar notNil ifTrue:[gXx _ self typeCheck: aVar shouldBe: GXReal paramName: ''gXx'']';
compile:
'p27: aVar
	aVar notNil ifTrue:[gXx _ self typeCheck: aVar shouldBe: GXReal paramName: ''gXx'']';

addInstVarName: #gXy;
compile:
'gXy: aVar
	aVar notNil ifTrue:[gXy _ self typeCheck: aVar shouldBe: GXReal paramName: ''gXy'']';
compile:
'p28: aVar
	aVar notNil ifTrue:[gXy _ self typeCheck: aVar shouldBe: GXReal paramName: ''gXy'']';

addInstVarName: #gXstring;
compile:
'gXstring: aVar
	aVar notNil ifTrue:[gXstring _ self typeCheck: aVar shouldBe: GXString paramName: ''gXstring'']';
compile:
'p29: aVar
	aVar notNil ifTrue:[gXstring _ self typeCheck: aVar shouldBe: GXString paramName: ''gXstring'']';

addInstVarName: #gXitem;
compile:
'gXitem: aVar
	aVar notNil ifTrue:[gXitem _ self typeCheck: aVar shouldBe: GXGobject paramName: ''gXitem'']';
compile:
'p30: aVar
	aVar notNil ifTrue:[gXitem _ self typeCheck: aVar shouldBe: GXGobject paramName: ''gXitem'']';

addInstVarName: #gXwindow;
compile:
'gXwindow: aVar
	aVar notNil ifTrue:[gXwindow _ self typeCheck: aVar shouldBe: GXGobject paramName: ''gXwindow'']';
compile:
'p31: aVar
	aVar notNil ifTrue:[gXwindow _ self typeCheck: aVar shouldBe: GXGobject paramName: ''gXwindow'']';

addInstVarName: #gXpicture;
compile:
'gXpicture: aVar
	aVar notNil ifTrue:[gXpicture _ self typeCheck: aVar shouldBe: GXGobject paramName: ''gXpicture'']';
compile:
'p32: aVar
	aVar notNil ifTrue:[gXpicture _ self typeCheck: aVar shouldBe: GXGobject paramName: ''gXpicture'']';

addInstVarName: #gXdevice;
compile:
'gXdevice: aVar
	aVar notNil ifTrue:[gXdevice _ self typeCheck: aVar shouldBe: GXGobject paramName: ''gXdevice'']';
compile:
'p33: aVar
	aVar notNil ifTrue:[gXdevice _ self typeCheck: aVar shouldBe: GXGobject paramName: ''gXdevice'']'
!

"Global variables
 ----------------"

GXdraw

addPoolVar: #gXUUstatus!

"Local variables
 ---------------"

GXdraw

addInstVarName: #gXtemp;

addInstVarName: #gXmode;

addInstVarName: #gXpp
!

!GXdraw methodsFor: 'initializing'!

initVars
	gXaxes _ false.
	gXtext _ false.
	gXhistogram _ false.
	gXplot _ false.
	gXmarkers _ false.
	gXgraticule _ false.
	gXlabels _ false.
	gXtitle _ false.
	gXbox _ false.
	gXline _ false.
	gXerrors _ false.
	gXquality _ false.
	gXxlog _ false.
	gXylog _ false.
	gXfilled _ false.
	gXhorizontal _ false.
	gXvertical _ false.
	gXnoylog _ false.
	gXnoxlog _ false.
	gXpolygon _ false.
	gXnoxnum _ false.
	gXnoynum _ false.
	gXyhorizontal _ false.
	gXxvertical _ false.
	gXcontour _ false.
	gXcellarray _ false.
	gXarray _ false.
	gXfunction _ false.
	gXlog _ false.
	gXsqrt _ false.
	gXsmooth _ false
!!

"BODY
 ----"

!GXdraw methodsFor: 'running'!

evaluate
((gXUUstatus) = 0) ifTrue: [ 
[nil.
self up.
^nil] value ].
(gXcellarray) ifTrue: [ 
 
		( gXp1 isKindOf: GXRealarray ) ifTrue: [ gXvalues _ gXp1 ].
	    	( gXp2 isKindOf: GXColourtable ) ifTrue: [ gXcolourtable _ gXp2 ].
		gXvalues isNil ifTrue: [^'DRAW/CELLARRAY: Parameter VALUES not specified' prenl ].
		gXcolourtable isNil ifTrue: [^'DRAW/CELLARRAY: Parameter COLOURTABLE not specified' prenl ].
	 	( gXp3 isKindOf: GXRealarray ) ifTrue: [ gXwx _ gXp3. gXtr _ gXp3 ].
		( gXp3 isKindOf: String ) ifTrue: [ gXfunc _ gXp3 ].
		( gXp4 isKindOf: GXRealarray ) ifTrue: [ gXwy _ gXp4 ].
		gXfunc notNil ifTrue: [
" DRAW/CELLARRAY/FUNCTION[/SMOOTH][/LOG][/SQRT] values colourtable func [valmax] [valmin] "
		    ( gXp4 isKindOf: Float ) ifTrue: [ gXvalmax _ gXp4 ].
		    ( gXp5 isKindOf: Float ) ifTrue: [ gXvalmin _ gXp5 ].
		    GXcellUfunction run line: 0 proc: self method: false; gXdraw;
		    "qualifiers"
		       	gXsmooth: gXsmooth;
		        gXlog: gXlog;
		        gXsqrt: gXsqrt;
		    "parameters"
		        p1: gXvalues; 
		        p2: gXcolourtable; 
		        p3: gXfunction; 
		        p4: gXvalmax; 
		        p5: gXvalmin; 
                    condEvaluate
		] ifFalse: [
		    ( (gXwx notNil) & (gXwy notNil) ) ifTrue: [
" DRAW/CELLARRAY/ARRAY values colourtable wx wy [valmax] [valmin] "
		        ( gXp5 isKindOf: Float ) ifTrue: [ gXvalmax _ gXp5 ].
		        ( gXp6 isKindOf: Float ) ifTrue: [ gXvalmin _ gXp6 ].
		        GXcellUarray run line: 0 proc: self method: false; gXdraw;
		        "qualifiers"
			        gXsmooth: gXsmooth;
			        gXlog: gXlog;
			        gXsqrt: gXsqrt;
		        "parameters"
			        p1: gXvalues; 
			        p2: gXcolourtable; 
			        p3: gXwx; 
			        p4: gXwy; 
			        p5: gXvalmax; 
			        p6: gXvalmin; 
                        condEvaluate
		    ] ifFalse: [
" DRAW/CELLARRAY[/SMOOTH][/LOG][/SQRT] values colourtable [tr] [valmax] [valmin] "
			( gXp3 isKindOf: GXRealarray ) ifTrue: [ gXtr _ gXp3 ].
		        ( gXp5 isKindOf: Float ) ifTrue: [ gXvalmax _ gXp4 ].
		        ( gXp6 isKindOf: Float ) ifTrue: [ gXvalmin _ gXp5 ]
	"		GXUdrawUcellarray run line: 0 proc: self method: false; p1:(gXtemp); p2:(gXmode); p3:(gXvalues); 
				p4:(gXcolourtable); p5:(gXtr); p6:(gXvalmax); p7:(gXvalmin); condEvaluate"
		    ]
		]
	    .
[nil.
self up.
^nil] value ].
(gXcontour) ifTrue: [ 

		( gXp1 isKindOf: GXRealarray ) ifTrue: [ gXvalues _ gXp1 ].
		( gXp2 isKindOf: Integer ) ifTrue: [ gXncont _ gXp2 ].
		( gXp2 isKindOf: GXRealarray ) ifTrue: [ gXcontours _ gXp2 ].
" parameter 2 is special - it can be NCONT or CONTOURS "
" Set TEMP to 0 for linear contours, 1 for LOG contours, 2 for SQRT contours "
		gXvalues isNil ifTrue: [^'DRAW/CONTOUR: Parameter VALUES not specified' prenl ].
		gXcontours notNil ifTrue: [ gXpp _ gXcontours ] ifFalse: [ gXpp _ gXncont ].
		gXtemp _ 0.
		gXlog ifTrue: [ gXtemp _ 1 ].
		gXsqrt ifTrue: [ gXtemp _ 2 ].
		( gXp3 isKindOf: GXRealarray ) ifTrue: [ gXtr _ gXp3. gXwx _ gXp3 ].
		( gXp3 isKindOf: String ) ifTrue: [ gXtr _ gXp3 ].
		( gXp4 isKindOf: GXRealarray ) ifTrue: [ gXwy _ gXp4 ].
		( (gXwx notNil) & (gXwy notNil) ) ifTrue: [
" DRAW/CONTOUR/ARRAY VALUES NCONT WX WY COLOUR LINETYPE LINEWIDTH "
" DRAW/CONTOUR/ARRAY VALUES CONTOURS WX WY COLOUR LINETYPE LINEWIDTH "
		    gXcolour isNil ifTrue: [ gXcolour _ gXp5 ].
		    gXlinetype isNil ifTrue: [ gXlinetype _ gXp6 ].
		    gXlinewidth isNil ifTrue: [ gXlinewidth _ gXp7 ].
		    GXcontourUarray run line: 0 proc: self method: false; gXdraw;
		    "qualifiers"
		        gXlog: gXlog;
		        gXsqrt: gXsqrt;
		    "parameters"
		        p1: gXvalues; 
		        p2: gXcontours; 
		        p3: gXncont; 
		        p4: gXwx; 
		        p5: gXwy; 
		        p6: gXcolour; 
		        p7: gXlinetype; 
		        p8: gXlinewidth; 
                    condEvaluate
		] ifFalse: [
		    gXfunc notNil ifTrue: [
" DRAW/CONTOUR/FUNCTION VALUES NCONT FUNC COLOUR LINETYPE LINEWIDTH "
" DRAW/CONTOUR/FUNCTION VALUES CONTOURS FUNC COLOUR LINETYPE LINEWIDTH "
		        gXcolour isNil ifTrue: [ gXcolour _ gXp4 ].
		        gXlinetype isNil ifTrue: [ gXlinetype _ gXp5 ].
		        gXlinewidth isNil ifTrue: [ gXlinewidth _ gXp6 ].
		        GXcontourUfunction run line: 0 proc: self method: false; gXdraw;
		        "qualifiers"
		            gXlog: gXlog;
		            gXsqrt: gXsqrt;
		        "parameters"
		            p1: gXvalues; 
		            p2: gXcontours; 
		            p3: gXncont; 
		            p4: gXfunc; 
		            p5: gXcolour; 
		            p6: gXlinetype; 
		            p7: gXlinewidth; 
                        condEvaluate
		    ] ifFalse: [
" DRAW/CONTOUR VALUES NCONT TR COLOUR LINETYPE LINEWIDTH "
" DRAW/CONTOUR VALUES CONTOURS TR COLOUR LINETYPE LINEWIDTH "
		       gXcolour isNil ifTrue: [ gXcolour _ gXp4 ].
		       gXlinetype isNil ifTrue: [ gXlinetype _ gXp5 ].
		       gXlinewidth isNil ifTrue: [ gXlinewidth _ gXp6 ].
		       GXcontour run line: 0 proc: self method: false; gXdraw;
		       "qualifiers"
		           gXlog: gXlog;
		           gXsqrt: gXsqrt;
		       "parameters"
		           p1: gXvalues; 
		           p2: gXcontours; 
		           p3: gXncont; 
		           p4: gXtr; 
		           p5: gXcolour; 
		           p6: gXlinetype; 
		           p7: gXlinewidth; 
                        condEvaluate
		    ]
		]
	    .
[nil.
self up.
^nil] value ].
(gXaxes) ifTrue: [ 

		( gXUsdjUgw at: #gXxlog ) ifTrue: [ gXp5 _ true ] ifFalse: [ gXp5 _ false ].
		( gXUsdjUgw at: #gXylog ) ifTrue: [ gXp6 _ true ] ifFalse: [ gXp6 _ false ].
		gXquality ifTrue: [ gXfont _ gXDroman ].
		( gXp1 isKindOf: Symbol ) ifTrue: [ gXcolour _ gXp1 ].
		( gXp2 isKindOf: Float ) ifTrue: [ gXlinewidth _ gXp2 ].
		( gXp3 isKindOf: Float ) ifTrue: [ gXtextheight _ gXp3 ].
		( gXp4 isKindOf: Symbol ) ifTrue: [ gXfont _ gXp4 ].
	    	gXxlog ifTrue: [ gXp5 _ true].
	    	gXylog ifTrue: [ gXp6 _ true].
	    	gXnoxlog ifTrue: [ gXp5 _ false].
	    	gXnoylog ifTrue: [ gXp6 _ false].
	    	gXnoxnum ifTrue: [ gXp7 _ true] ifFalse: [ gXp7 _ false ].
	    	gXnoynum ifTrue: [ gXp8 _ true] ifFalse: [ gXp8 _ false ].
		gXyhorizontal ifTrue: [ gXp9 _ true ] ifFalse: [ gXp9 _ false ].
		GXaxes run line: 0 proc: self method: false; gXdraw;
		"qualifiers"
			gXxlog: gXp5; 
			gXylog: gXp6; 
			gXyhoriz: gXp9; 
	 		gXnoxnum: gXp7;
	 		gXnoynum: gXp8;
		"parameters"
			p1: gXcolour; 
			p2: gXtextheight; 
			p3: gXfont; 
			p4: gXlinewidth; 
                condEvaluate
	    .
[nil.
self up.
^nil] value ].
(gXtext) ifTrue: [ 
 
		gXquality ifTrue: [ gXfont _ gXDroman ].
		( gXp1 isKindOf: Integer ) ifTrue: [ gXp1 _ gXp1 * 1.0 ].
		( gXp2 isKindOf: Integer ) ifTrue: [ gXp2 _ gXp2 * 1.0 ].
		( gXp1 isKindOf: Float ) ifTrue: [ gXx _ gXp1 ].
		( gXp2 isKindOf: Float ) ifTrue: [ gXy _ gXp2 ].
		( gXp3 isKindOf: String ) ifTrue: [ gXstring _ gXp3 ].
		gXx isNil ifTrue: [^'DRAW/TEXT: Parameter X not supplied' prenl ].
		gXy isNil ifTrue: [^'DRAW/TEXT: Parameter Y not supplied' prenl ].
		gXstring isNil ifTrue: [^'DRAW/TEXT: Parameter STRING not supplied' prenl ].
		( gXp4 isKindOf: Symbol ) ifTrue: [ gXcolour _ gXp4 ].
		( gXp5 isKindOf: Float ) ifTrue: [ gXtextheight _ gXp5 ].
		( gXp6 isKindOf: Float ) ifTrue: [ gXangle _ gXp6 ].
		( gXp7 isKindOf: Symbol ) ifTrue: [ gXfont _ gXp7 ].
		( gXp8 isKindOf: Float ) ifTrue: [ gXlinewidth _ gXp8 ].
		GXtext run line: 0 proc: self method: false; gXdraw;
		"qualifiers"
		"parameters"
			p1: gXx; 
			p2: gXy; 
			p3: gXstring; 
			p4: gXcolour; 
			p5: gXtextheight; 
			p6: gXangle;
			p7: gXfont;
			p8: gXlinewidth;
                condEvaluate
	    .
[nil.
self up.
^nil] value ].
(gXhistogram) ifTrue: [ 

		gXp4 isNil ifTrue: [ 
			((gXp1 size) = (gXp2 size)) ifTrue: [ gXp4 _ true ] 
				ifFalse: [ gXp4 _ false ]
			   ] ifFalse: [
		(gXp4 = 1) ifTrue: [ gXp4 _ true. ] ifFalse: [ gXp4 _ false ]
                ].
		GXhistogram run line: 0 proc: self method: false; gXdraw;
		"qualifiers"
			gXcentre: gXp4;
		"parameters"
			p1: gXp1; 
			p2: gXp2; 
			p3: gXp3; 
			p4: gXp5; 
			p5: gXp6; 
                condEvaluate
	    .
[nil.
self up.
^nil] value ].
(gXplot) ifTrue: [ 

		GXgraph run line: 0 proc: self method: false; gXdraw;
		"qualifiers"
		"parameters"
			p1: gXp1; 
			p2: gXp2; 
			p3: gXp3; 
			p4: gXp4; 
			p5: gXp5; 
                condEvaluate
	    .
[nil.
self up.
^nil] value ].
(gXmarkers) ifTrue: [ 

		GXmarkers run line: 0 proc: self method: false; gXdraw;
		"qualifiers"
		"parameters"
			p1: gXp1; 
			p2: gXp2; 
			p3: gXp3; 
			p4: gXp4; 
			p5: gXp5; 
			p6: gXp6;
                condEvaluate
	    .
[nil.
self up.
^nil] value ].
(gXgraticule) ifTrue: [ 

		( gXUsdjUgw at: #gXxlog ) ifTrue: [ gXp3 _ true ].
		( gXUsdjUgw at: #gXylog ) ifTrue: [ gXp4 _ true ].
	    	gXxlog ifTrue: [ gXp3 _ true ].
	    	gXylog ifTrue: [ gXp4 _ true ].
	    	gXnoxlog ifTrue: [ gXp3 _ false ].
	    	gXnoylog ifTrue: [ gXp4 _ false ].
		GXgraticule run line: 0 proc: self method: false; gXdraw;
		"qualifiers"
			gXxlog: gXp3;
			gXylog: gXp4;
		"parameters"
			p1: gXp1; 
			p2: gXp2; 
                condEvaluate
	    .
[nil.
self up.
^nil] value ].
(gXlabels) ifTrue: [ 
 
		gXquality ifTrue: [ gXp5 _ gXDroman ].
		gXxvertical ifTrue: [ gXp7 _ true ] ifFalse: [ gXp7 _ false ].
		gXyhorizontal ifTrue: [ gXp8 _ false ] ifFalse: [ gXp8 _ true ].
		GXlabels run line: 0 proc: self method: false; gXdraw;
		"qualifiers"
			gXxvertical: gXp7;
			gXyvertical: gXp8;
		"parameters"
			p1: gXp1; 
			p2: gXp2; 
			p3: gXp4; 
			p4: gXp3; 
			p5: gXp5; 
                condEvaluate
	    .
[nil.
self up.
^nil] value ].
(gXtitle) ifTrue: [ 
 
                gXquality ifTrue: [ gXp4 _ gXDroman ].
		GXtitle run line: 0 proc: self method: false; gXdraw;
		"qualifiers"
		"parameters"
			p1: gXp1; 
			p2: gXp2; 
			p3: gXp3; 
			p4: gXp4; 
                condEvaluate
	    .
[nil.
self up.
^nil] value ].
(gXbox) ifTrue: [ 

		gXp7 _ GXRealarray new: 4.
		gXp8 _ GXRealarray new: 4.
		gXp7 at: 1 put: gXp1.
		gXp8 at: 1 put: gXp2.
		gXp7 at: 2 put: gXp1.
		gXp8 at: 2 put: gXp4.
		gXp7 at: 3 put: gXp3.
		gXp8 at: 3 put: gXp4.
		gXp7 at: 4 put: gXp3.
		gXp8 at: 4 put: gXp2.
		GXpolygon run line: 0 proc: self method: false; gXdraw;
		"qualifiers"
			gXfilled: gXfilled;
		"parameters"
			p1: gXp7; 
			p2: gXp8; 
			p3: gXp5; 
			p4: gXp6; 
                condEvaluate
	    .
[nil.
self up.
^nil] value ].
(gXpolygon) ifTrue: [ 
 
		GXpolygon run line: 0 proc: self method: false; gXdraw;
		"qualifiers"
			gXfilled: gXfilled;
		"parameters"
			p1: gXp1; 
			p2: gXp2; 	
			p3: gXp4; 	"ignore p3 - the number of points"
			p4: gXp5; 
                condEvaluate
	    .
[nil.
self up.
^nil] value ].
(gXerrors) ifTrue: [ 
 
		GXerrors run line: 0 proc: self method: false; gXdraw;
		"qualifiers"
			gXvertical: gXvertical;
			gXhorizontal: gXhorizontal;
		"parameters"
			p1: gXp1; 
			p2: gXp2; 
			p3: gXp3; 
			p4: gXp4; 
			p5: gXp5; 
                condEvaluate
	    .
[nil.
self up.
^nil] value ].
(gXline) ifTrue: [ 

		GXline run line: 0 proc: self method: false; gXdraw;
		"qualifiers"
		"parameters"
			p1: gXp1; 
			p2: gXp2; 
			p3: gXp3; 
			p4: gXp4; 
			p5: gXp5; 
			p6: gXp6; 
			p7: gXp7; 
                condEvaluate
	    .
[nil.
self up.
^nil] value ].

	    gXitem isNil ifTrue: [ gXitem _ gXp1 ].
	    gXwindow isNil ifTrue: [ gXwindow _ gXp2 ].
	    gXpicture isNil ifTrue: [ gXpicture _ gXp3 ].
	    gXdevice isNil ifTrue: [ gXdevice _ gXp4 ]
	.
GXpicUadd run line: 0 proc: self method: false; p1:(gXitem); p2:(gXwindow); p3:(gXpicture); p4:(gXdevice); condEvaluate.
self up.
^nil
!!
nil! "(fix for smalltalk bug, this line is ignored if there is a variable declaration error)"

"End of procedure"

GXcomment run line: 0 proc: self method: false; p1:'draw'; p2:'The main graphics drawing routine'; condEvaluate!
"Procedure class definition:
 ==========================="
((Smalltalk at: #GXredraw ifAbsent: [ nil ]) isKindOf: Function class) ifTrue: [ ('Warning: Overriding previously defined procedure ', 'GXredraw' gtype) prinl ].
Smalltalk at: #GXredraw put: nil!
Function subclass: 'GXredraw'
       instanceVariableNames: ''
       classVariableNames: ''
       poolDictionaries: ''
       category: 'Genie Procedure'
!

"HEADER
 ------"

"parameters
 ----------"

GXredraw

addInstVarName: #gXitem;
compile:
'gXitem: aVar
	aVar notNil ifTrue:[gXitem _ self typeCheck: aVar shouldBe: GXGobject paramName: ''gXitem'']';
compile:
'p1: aVar
	aVar notNil ifTrue:[gXitem _ self typeCheck: aVar shouldBe: GXGobject paramName: ''gXitem'']';

addInstVarName: #gXdevice;
compile:
'gXdevice: aVar
	aVar notNil ifTrue:[gXdevice _ self typeCheck: aVar shouldBe: GXGobject paramName: ''gXdevice'']';
compile:
'p2: aVar
	aVar notNil ifTrue:[gXdevice _ self typeCheck: aVar shouldBe: GXGobject paramName: ''gXdevice'']'
!

"Local variables
 ---------------"

GXredraw

addInstVarName: #gXitem;

addInstVarName: #gXdevice
!

!GXredraw methodsFor: 'initializing'!

initVars
	gXitem _ ((gXUUgraphics) at:#gXcurrentpicture).
	gXdevice _ ((gXUUgraphics) at:#gXcurrentdevice)
!!

"BODY
 ----"

!GXredraw methodsFor: 'running'!

evaluate

   |pic dev wind object|

   (gXdevice isKindOf: Device)
   ifTrue:[dev _ ((gXUUgraphics at: #gXcurrentgraphics) getDevice: gXdevice)].
   (gXitem isKindOf: Picture)
   ifTrue:[pic _ ((gXUUgraphics at: #gXcurrentgraphics)
						getPictureWithAPicture: gXitem).
		   (pic isKindOf: Picture)
		   ifTrue:[pic switchOn.
			(dev isNil)
			ifTrue:[dev _ ((gXUUgraphics at: #gXcurrentgraphics)
							findDevice: pic)].
		    (dev isKindOf: Device)
		    ifTrue:[ Device Clearing. dev associate: pic]]].

   (gXitem isKindOf: Window)
   ifTrue:[wind _ ((gXUUgraphics at: #gXcurrentgraphics)
					findDrawableObjectsPicture: gXitem).
	   (wind isKindOf: Window)
	   ifTrue:[pic _ ((gXUUgraphics at: #gXcurrentgraphics)
						findPicture: gXitem).
		    (dev isKindOf: Device) 
		    ifFalse:[(pic isKindOf: Picture)
			    ifTrue:[dev _ ((gXUUgraphics at: #gXcurrentgraphics)
							      findDevice: pic).
			    	    wind unhide.
			    	    (dev isKindOf: Device) 
				    ifTrue:[pic drawWindow: wind]]]
		    ifTrue:[pic drawWindow: wind]]].



   (gXitem isKindOf: DrawableObjects)
   ifTrue:[(gXitem isKindOf: Window)
  	   ifFalse:[object _ ((gXUUgraphics at: #gXcurrentgraphics)
					findDrawableObjectsPicture: gXitem).
		    (object isKindOf: DrawableObjects)
		    ifTrue:[pic _ ((gXUUgraphics at: #gXcurrentgraphics)
							findPicture: gXitem).
			    (pic isKindOf: Picture) ifTrue:[
 		    	     	wind _ (pic findRightWindow: gXitem).
			     	(dev isKindOf: Device)
			    	ifFalse:[dev _ ((gXUUgraphics at: #gXcurrentgraphics)
						       findDevice: pic).
			  	    object unhide.
				    pic drawUnflaggedWindows]
				ifTrue:[pic drawWindowOnly: wind.
			     	    wind drawObject: object]]]]]

.
self up.
^nil
!!
nil! "(fix for smalltalk bug, this line is ignored if there is a variable declaration error)"

"End of procedure"

GXcomment run line: 0 proc: self method: false; p1:'redraw'; p2:'Redisplay previous pictures'; condEvaluate!
"Procedure class definition:
 ==========================="
((Smalltalk at: #GXpicture ifAbsent: [ nil ]) isKindOf: Function class) ifTrue: [ ('Warning: Overriding previously defined procedure ', 'GXpicture' gtype) prinl ].
Smalltalk at: #GXpicture put: nil!
Function subclass: 'GXpicture'
       instanceVariableNames: ''
       classVariableNames: ''
       poolDictionaries: ''
       category: 'Genie Procedure'
!

"HEADER
 ------"

"result
 ------"

GXpicture

addInstVarName: #gXres
!

!GXpicture methodsFor: 'initializing'!

initVars

!!

"BODY
 ----"

!GXpicture methodsFor: 'running'!

evaluate

   |tmp|
   (gXUUstatus = 1)
   ifFalse:['ERROR, graphics device is not open' prenl.
	   'To open enter DEVICE/OPEN' prinl]
   ifTrue:[ (gXUUgraphics at: #gXcurrentdevice) Clear.
   	   tmp _ Picture new.
   	   gXUUgraphics at: #gXcurrentpicture put: (tmp select).
   	   (gXUUgraphics at: #gXcurrentgraphics) addToPictureList: tmp.
	   (gXUUgraphics at: #gXcurrentdevice) associate: tmp.
	   gXUUgraphics at: #gXcurrentwindow put: nil.	"so we don't draw in the old window"
   	   gXres _ tmp]

.
self up.
^gXres
!!
nil! "(fix for smalltalk bug, this line is ignored if there is a variable declaration error)"

"End of procedure"

"Procedure class definition:
 ==========================="
((Smalltalk at: #GXdevice ifAbsent: [ nil ]) isKindOf: Function class) ifTrue: [ ('Warning: Overriding previously defined procedure ', 'GXdevice' gtype) prinl ].
Smalltalk at: #GXdevice put: nil!
Function subclass: 'GXdevice'
       instanceVariableNames: ''
       classVariableNames: ''
       poolDictionaries: ''
       category: 'Genie Procedure'
!

"HEADER
 ------"

"qualifiers
 ----------"

GXdevice

addInstVarName: #gXopen;
compile:
'gXopen
	gXopen _ true';
compile:
'gXopen: aBoolean
	gXopen _ aBoolean';

addInstVarName: #gXclose;
compile:
'gXclose
	gXclose _ true';
compile:
'gXclose: aBoolean
	gXclose _ aBoolean';

addInstVarName: #gXclear;
compile:
'gXclear
	gXclear _ true';
compile:
'gXclear: aBoolean
	gXclear _ aBoolean';

addInstVarName: #gXstatus;
compile:
'gXstatus
	gXstatus _ true';
compile:
'gXstatus: aBoolean
	gXstatus _ aBoolean'!

"parameters
 ----------"

GXdevice

addInstVarName: #gXdevtype;
compile:
'gXdevtype: aVar
	aVar notNil ifTrue:[gXdevtype _ self typeCheck: aVar shouldBe: GXString paramName: ''gXdevtype'']';
compile:
'p1: aVar
	aVar notNil ifTrue:[gXdevtype _ self typeCheck: aVar shouldBe: GXString paramName: ''gXdevtype'']';

addInstVarName: #gXwidth;
compile:
'gXwidth: aVar
	aVar notNil ifTrue:[gXwidth _ self typeCheck: aVar shouldBe: GXReal paramName: ''gXwidth'']';
compile:
'p2: aVar
	aVar notNil ifTrue:[gXwidth _ self typeCheck: aVar shouldBe: GXReal paramName: ''gXwidth'']';

addInstVarName: #gXheight;
compile:
'gXheight: aVar
	aVar notNil ifTrue:[gXheight _ self typeCheck: aVar shouldBe: GXReal paramName: ''gXheight'']';
compile:
'p3: aVar
	aVar notNil ifTrue:[gXheight _ self typeCheck: aVar shouldBe: GXReal paramName: ''gXheight'']';

addInstVarName: #gXra;
compile:
'gXra: aVar
	aVar notNil ifTrue:[gXra _ self typeCheck: aVar shouldBe: GXReal paramName: ''gXra'']';
compile:
'p4: aVar
	aVar notNil ifTrue:[gXra _ self typeCheck: aVar shouldBe: GXReal paramName: ''gXra'']';

addInstVarName: #gXfilename;
compile:
'gXfilename: aVar
	aVar notNil ifTrue:[gXfilename _ self typeCheck: aVar shouldBe: GXString paramName: ''gXfilename'']';
compile:
'p5: aVar
	aVar notNil ifTrue:[gXfilename _ self typeCheck: aVar shouldBe: GXString paramName: ''gXfilename'']'!

"result
 ------"

GXdevice

addInstVarName: #gXres
!

!GXdevice methodsFor: 'initializing'!

initVars
	gXopen _ false.
	gXclose _ false.
	gXclear _ false.
	gXstatus _ false
!!

"BODY
 ----"

!GXdevice methodsFor: 'running'!

evaluate

|gXdev tmp temp pic dev pics|

    "Return 0 if no graphics device open, 1 of one is open"
    gXstatus ifTrue: [ ^(gXUUstatus = 1) ].

   "Check if devtype is specified"
   (gXdevtype class = String) ifFalse:[gXdev _ Device new: 'xw']
                   ifTrue:[gXdev _ Device new: gXdevtype.
			    gXdev isNil ifTrue:[ ^nil ]
			  ].

   gXopen ifTrue:[tmp _ gXdev width: gXwidth; height: gXheight; ra: gXra; 
		  	fileName: gXfilename; Open.
		  (tmp id = -1) ifTrue: [ 'DEVICE: Error creating device' prenl. ^nil ].
		  gXUUgraphics at: #gXcurrentdevice put: tmp.
		  ((gXUUgraphics at: #gXcurrentgraphics) addToDeviceList: tmp).
		  tmp selectDevice: tmp.
		  gXres _ tmp.
		  gXUUstatus _ 1.
		  GXpicture run line: 0 proc: self method: false; condEvaluate
                 ].

  gXclose ifTrue:[((gXUUstatus = 0) | ((gXUUgraphics at: #gXcurrentdevice) isNil))
                   ifTrue:['DEVICE is already closed' prinl]
		   ifFalse:[temp _ (gXUUgraphics at: #gXcurrentdevice).
			    dev _ temp getID + 1.
			    tmp _ ((gXUUgraphics at: #gXcurrentgraphics)
						    	       closeDown: temp).
			  (tmp isNil) ifTrue:[gXUUstatus _ 0].
			 gXUUgraphics at: #gXcurrentdevice put: tmp]].
				     

  gXclear ifTrue:[((gXUUstatus = 0) | ((gXUUgraphics at: #gXcurrentdevice) isNil))
		   "ifFalse:[(gXUUgraphics at: #gXcurrentdevice) Clear]"
		   ifFalse:[ GXpicture run line: 0 proc: self method: false; condEvaluate ]
		   ifTrue:['ERROR, ' pren.
			   'Device is Closed' prinl]]

.
self up.
^gXres
!!
nil! "(fix for smalltalk bug, this line is ignored if there is a variable declaration error)"

"End of procedure"

GXcomment run line: 0 proc: self method: false; p1:'device'; p2:'Select and control graphics devices'; condEvaluate!
"Procedure class definition:
 ==========================="
((Smalltalk at: #GXwinUunscaled ifAbsent: [ nil ]) isKindOf: Function class) ifTrue: [ ('Warning: Overriding previously defined procedure ', 'GXwinUunscaled' gtype) prinl ].
Smalltalk at: #GXwinUunscaled put: nil!
Function subclass: 'GXwinUunscaled'
       instanceVariableNames: ''
       classVariableNames: ''
       poolDictionaries: ''
       category: 'Genie Procedure'
!

"HEADER
 ------"

"parameters
 ----------"

GXwinUunscaled

addInstVarName: #gXxmin;
compile:
'gXxmin: aVar
	aVar notNil ifTrue:[gXxmin _ self typeCheck: aVar shouldBe: GXReal paramName: ''gXxmin'']';
compile:
'p1: aVar
	aVar notNil ifTrue:[gXxmin _ self typeCheck: aVar shouldBe: GXReal paramName: ''gXxmin'']';

addInstVarName: #gXxmax;
compile:
'gXxmax: aVar
	aVar notNil ifTrue:[gXxmax _ self typeCheck: aVar shouldBe: GXReal paramName: ''gXxmax'']';
compile:
'p2: aVar
	aVar notNil ifTrue:[gXxmax _ self typeCheck: aVar shouldBe: GXReal paramName: ''gXxmax'']';

addInstVarName: #gXymin;
compile:
'gXymin: aVar
	aVar notNil ifTrue:[gXymin _ self typeCheck: aVar shouldBe: GXReal paramName: ''gXymin'']';
compile:
'p3: aVar
	aVar notNil ifTrue:[gXymin _ self typeCheck: aVar shouldBe: GXReal paramName: ''gXymin'']';

addInstVarName: #gXymax;
compile:
'gXymax: aVar
	aVar notNil ifTrue:[gXymax _ self typeCheck: aVar shouldBe: GXReal paramName: ''gXymax'']';
compile:
'p4: aVar
	aVar notNil ifTrue:[gXymax _ self typeCheck: aVar shouldBe: GXReal paramName: ''gXymax'']';

addInstVarName: #gXcolour;
compile:
'gXcolour: aVar
	aVar notNil ifTrue:[gXcolour _ self typeCheck: aVar shouldBe: GXColour paramName: ''gXcolour'']';
compile:
'p5: aVar
	aVar notNil ifTrue:[gXcolour _ self typeCheck: aVar shouldBe: GXColour paramName: ''gXcolour'']'!

"result
 ------"

GXwinUunscaled

addInstVarName: #gXres
!

!GXwinUunscaled methodsFor: 'initializing'!

initVars

!!

"BODY
 ----"

!GXwinUunscaled methodsFor: 'running'!

evaluate

   |tmp a|
   (gXUUstatus = 0)
   ifTrue:['ERROR, graphics device is not open' prenl.
           'To open enter DEVICE/OPEN' prinl]
   ifFalse:[(gXUUgraphics at: #gXcurrentpicture) isNil
   	    ifTrue:['Creating Picture' prinl.
	   	    tmp _ Picture new.
   	   	    ((gXUUgraphics at: #gXcurrentgraphics) 
						         addToPictureList: tmp).
	   	    gXUUgraphics at: #gXcurrentpicture put: (tmp select).
		    (gXUUgraphics at: #gXcurrentdevice) associate: tmp].
   	     tmp _ (Window new xMin: gXxmin; xMax: gXxmax; yMin: gXymin;
                             yMax: gXymax; winColour: gXcolour; create).

   (tmp = nil)
   ifTrue:['Window has not been created' prenl]
   ifFalse:[gXUUgraphics at: #gXcurrentpicture put:
               ((gXUUgraphics at: #gXcurrentpicture) addToWindowList: tmp ).
   gXUUgraphics at: #gXcurrentwindow put: tmp.
   gXres _ tmp.
   (gXUUgraphics at: #gXcurrentwindow) display]]

.
self up.
^gXres
!!
nil! "(fix for smalltalk bug, this line is ignored if there is a variable declaration error)"

"End of procedure"

"Procedure class definition:
 ==========================="
((Smalltalk at: #GXwinUscaled ifAbsent: [ nil ]) isKindOf: Function class) ifTrue: [ ('Warning: Overriding previously defined procedure ', 'GXwinUscaled' gtype) prinl ].
Smalltalk at: #GXwinUscaled put: nil!
Function subclass: 'GXwinUscaled'
       instanceVariableNames: ''
       classVariableNames: ''
       poolDictionaries: ''
       category: 'Genie Procedure'
!

"HEADER
 ------"

"parameters
 ----------"

GXwinUscaled

addInstVarName: #gXdxmin;
compile:
'gXdxmin: aVar
	aVar notNil ifTrue:[gXdxmin _ self typeCheck: aVar shouldBe: GXReal paramName: ''gXdxmin'']';
compile:
'p1: aVar
	aVar notNil ifTrue:[gXdxmin _ self typeCheck: aVar shouldBe: GXReal paramName: ''gXdxmin'']';

addInstVarName: #gXdxmax;
compile:
'gXdxmax: aVar
	aVar notNil ifTrue:[gXdxmax _ self typeCheck: aVar shouldBe: GXReal paramName: ''gXdxmax'']';
compile:
'p2: aVar
	aVar notNil ifTrue:[gXdxmax _ self typeCheck: aVar shouldBe: GXReal paramName: ''gXdxmax'']';

addInstVarName: #gXdymin;
compile:
'gXdymin: aVar
	aVar notNil ifTrue:[gXdymin _ self typeCheck: aVar shouldBe: GXReal paramName: ''gXdymin'']';
compile:
'p3: aVar
	aVar notNil ifTrue:[gXdymin _ self typeCheck: aVar shouldBe: GXReal paramName: ''gXdymin'']';

addInstVarName: #gXdymax;
compile:
'gXdymax: aVar
	aVar notNil ifTrue:[gXdymax _ self typeCheck: aVar shouldBe: GXReal paramName: ''gXdymax'']';
compile:
'p4: aVar
	aVar notNil ifTrue:[gXdymax _ self typeCheck: aVar shouldBe: GXReal paramName: ''gXdymax'']';

addInstVarName: #gXwxmin;
compile:
'gXwxmin: aVar
	aVar notNil ifTrue:[gXwxmin _ self typeCheck: aVar shouldBe: GXReal paramName: ''gXwxmin'']';
compile:
'p5: aVar
	aVar notNil ifTrue:[gXwxmin _ self typeCheck: aVar shouldBe: GXReal paramName: ''gXwxmin'']';

addInstVarName: #gXwxmax;
compile:
'gXwxmax: aVar
	aVar notNil ifTrue:[gXwxmax _ self typeCheck: aVar shouldBe: GXReal paramName: ''gXwxmax'']';
compile:
'p6: aVar
	aVar notNil ifTrue:[gXwxmax _ self typeCheck: aVar shouldBe: GXReal paramName: ''gXwxmax'']'!

"parameters
 ----------"

GXwinUscaled

addInstVarName: #gXwymin;
compile:
'gXwymin: aVar
	aVar notNil ifTrue:[gXwymin _ self typeCheck: aVar shouldBe: GXReal paramName: ''gXwymin'']';
compile:
'p7: aVar
	aVar notNil ifTrue:[gXwymin _ self typeCheck: aVar shouldBe: GXReal paramName: ''gXwymin'']';

addInstVarName: #gXwymax;
compile:
'gXwymax: aVar
	aVar notNil ifTrue:[gXwymax _ self typeCheck: aVar shouldBe: GXReal paramName: ''gXwymax'']';
compile:
'p8: aVar
	aVar notNil ifTrue:[gXwymax _ self typeCheck: aVar shouldBe: GXReal paramName: ''gXwymax'']';

addInstVarName: #gXcolour;
compile:
'gXcolour: aVar
	aVar notNil ifTrue:[gXcolour _ self typeCheck: aVar shouldBe: GXColour paramName: ''gXcolour'']';
compile:
'p9: aVar
	aVar notNil ifTrue:[gXcolour _ self typeCheck: aVar shouldBe: GXColour paramName: ''gXcolour'']';

addInstVarName: #gXminmode;
compile:
'gXminmode: aVar
	aVar notNil ifTrue:[gXminmode _ self typeCheck: aVar shouldBe: GXInteger paramName: ''gXminmode'']';
compile:
'p10: aVar
	aVar notNil ifTrue:[gXminmode _ self typeCheck: aVar shouldBe: GXInteger paramName: ''gXminmode'']';

addInstVarName: #gXmaxmode;
compile:
'gXmaxmode: aVar
	aVar notNil ifTrue:[gXmaxmode _ self typeCheck: aVar shouldBe: GXInteger paramName: ''gXmaxmode'']';
compile:
'p11: aVar
	aVar notNil ifTrue:[gXmaxmode _ self typeCheck: aVar shouldBe: GXInteger paramName: ''gXmaxmode'']'!

"result
 ------"

GXwinUscaled

addInstVarName: #gXres
!

!GXwinUscaled methodsFor: 'initializing'!

initVars

!!

"BODY
 ----"

!GXwinUscaled methodsFor: 'running'!

evaluate

   |tmp|
   (gXUUstatus = 0)
   ifTrue:['ERROR, graphics device is not open' prenl.
           'To open enter DEVICE/OPEN' prinl]

   ifFalse:[((gXwxmin = nil) | (gXwxmax = nil) | (gXwymin = nil) | 
	     (gXwymax = nil) )
            ifTrue:['ERROR' prenl.
                    'World coordinates are not defined' prinl]

   	   ifFalse:[(gXUUgraphics at: #gXcurrentpicture) isNil
   	   	    ifTrue:['Creating Picture ' prinl.
	   	    	    tmp _ Picture new.
   	   	   	   ((gXUUgraphics at: #gXcurrentgraphics) addToPictureList: tmp).
		    	    (gXUUgraphics at: #gXcurrentdevice) associate: tmp.
	   	   	    gXUUgraphics at: #gXcurrentpicture put: (tmp select)].
            	   tmp _ (WindowScaled new xMin: gXdxmin; xMax: gXdxmax;
            	            yMin: gXdymin; yMax: gXdymax; lxMin: gXwxmin; 
			    lxMax: gXwxmax; lyMin: gXwymin; lyMax: gXwymax; 
			    winColour: gXcolour; minMode: gXminmode; 
			    maxMode: gXmaxmode; create). 
   	    	     (tmp = nil)
            	     ifTrue:['Window has not been created' prenl]
		     ifFalse:[gXUUgraphics at: #gXcurrentpicture put:
                         ((gXUUgraphics at: #gXcurrentpicture) addToWindowList: tmp ).
            	     	      gXUUgraphics at: #gXcurrentwindow put: tmp.
            	     	      gXres _ tmp.
            	     	      (gXUUgraphics at: #gXcurrentwindow) display]]]


.
self up.
^gXres
!!
nil! "(fix for smalltalk bug, this line is ignored if there is a variable declaration error)"

"End of procedure"

"Procedure class definition:
 ==========================="
((Smalltalk at: #GXwinUtwod ifAbsent: [ nil ]) isKindOf: Function class) ifTrue: [ ('Warning: Overriding previously defined procedure ', 'GXwinUtwod' gtype) prinl ].
Smalltalk at: #GXwinUtwod put: nil!
Function subclass: 'GXwinUtwod'
       instanceVariableNames: ''
       classVariableNames: ''
       poolDictionaries: ''
       category: 'Genie Procedure'
!

"HEADER
 ------"

"parameters
 ----------"

GXwinUtwod

addInstVarName: #gXdxmin;
compile:
'gXdxmin: aVar
	aVar notNil ifTrue:[gXdxmin _ self typeCheck: aVar shouldBe: GXReal paramName: ''gXdxmin'']';
compile:
'p1: aVar
	aVar notNil ifTrue:[gXdxmin _ self typeCheck: aVar shouldBe: GXReal paramName: ''gXdxmin'']';

addInstVarName: #gXdxmax;
compile:
'gXdxmax: aVar
	aVar notNil ifTrue:[gXdxmax _ self typeCheck: aVar shouldBe: GXReal paramName: ''gXdxmax'']';
compile:
'p2: aVar
	aVar notNil ifTrue:[gXdxmax _ self typeCheck: aVar shouldBe: GXReal paramName: ''gXdxmax'']';

addInstVarName: #gXdymin;
compile:
'gXdymin: aVar
	aVar notNil ifTrue:[gXdymin _ self typeCheck: aVar shouldBe: GXReal paramName: ''gXdymin'']';
compile:
'p3: aVar
	aVar notNil ifTrue:[gXdymin _ self typeCheck: aVar shouldBe: GXReal paramName: ''gXdymin'']';

addInstVarName: #gXdymax;
compile:
'gXdymax: aVar
	aVar notNil ifTrue:[gXdymax _ self typeCheck: aVar shouldBe: GXReal paramName: ''gXdymax'']';
compile:
'p4: aVar
	aVar notNil ifTrue:[gXdymax _ self typeCheck: aVar shouldBe: GXReal paramName: ''gXdymax'']';

addInstVarName: #gXx;
compile:
'gXx: aVar
	aVar notNil ifTrue:[gXx _ self typeCheck: aVar shouldBe: GXRealarray paramName: ''gXx'']';
compile:
'p5: aVar
	aVar notNil ifTrue:[gXx _ self typeCheck: aVar shouldBe: GXRealarray paramName: ''gXx'']';

addInstVarName: #gXy;
compile:
'gXy: aVar
	aVar notNil ifTrue:[gXy _ self typeCheck: aVar shouldBe: GXRealarray paramName: ''gXy'']';
compile:
'p6: aVar
	aVar notNil ifTrue:[gXy _ self typeCheck: aVar shouldBe: GXRealarray paramName: ''gXy'']'!

"parameters
 ----------"

GXwinUtwod

addInstVarName: #gXcolour;
compile:
'gXcolour: aVar
	aVar notNil ifTrue:[gXcolour _ self typeCheck: aVar shouldBe: GXColour paramName: ''gXcolour'']';
compile:
'p7: aVar
	aVar notNil ifTrue:[gXcolour _ self typeCheck: aVar shouldBe: GXColour paramName: ''gXcolour'']'!

"result
 ------"

GXwinUtwod

addInstVarName: #gXres
!

!GXwinUtwod methodsFor: 'initializing'!

initVars

!!

"BODY
 ----"

!GXwinUtwod methodsFor: 'running'!

evaluate

   |tmp| 
   (gXUUstatus = 0)
   ifTrue:['ERROR, graphics device is not open' prenl.
           'To open enter DEVICE/OPEN' prinl]
   ifFalse:[(gXUUgraphics at: #gXcurrentpicture) isNil
   	    ifTrue:['Creating Picture' prinl.
	   	    tmp _ Picture new.
   	   	    ((gXUUgraphics at: #gXcurrentgraphics) addToPictureList: tmp).
		    (gXUUgraphics at: #gXcurrentdevice) associate: tmp.
	   	    gXUUgraphics at: #gXcurrentpicture put: (tmp select)].
   	    tmp _ (TwoDWindow new xMin: gXdxmin; xMax: gXdxmax; yMin: gXdymin;
                         yMax: gXdymax; x: gXx; y: gXy; winColour: gXcolour;
                         create).
   	    (tmp = nil)
            ifTrue:['Window has not been created' prenl]
            ifFalse:[gXUUgraphics at: #gXcurrentpicture put:
                    ((gXUUgraphics at: #gXcurrentpicture) addToWindowList: tmp ).
              	     gXUUgraphics at: #gXcurrentwindow put: tmp.
               	     gXres _ tmp.
               	     (gXUUgraphics at: #gXcurrentwindow) display]]

.
self up.
^gXres
!!
nil! "(fix for smalltalk bug, this line is ignored if there is a variable declaration error)"

"End of procedure"

"Procedure class definition:
 ==========================="
((Smalltalk at: #GXwinUautoscaled ifAbsent: [ nil ]) isKindOf: Function class) ifTrue: [ ('Warning: Overriding previously defined procedure ', 'GXwinUautoscaled' gtype) prinl ].
Smalltalk at: #GXwinUautoscaled put: nil!
Function subclass: 'GXwinUautoscaled'
       instanceVariableNames: ''
       classVariableNames: ''
       poolDictionaries: ''
       category: 'Genie Procedure'
!

"HEADER
 ------"

"parameters
 ----------"

GXwinUautoscaled

addInstVarName: #gXdxmin;
compile:
'gXdxmin: aVar
	aVar notNil ifTrue:[gXdxmin _ self typeCheck: aVar shouldBe: GXReal paramName: ''gXdxmin'']';
compile:
'p1: aVar
	aVar notNil ifTrue:[gXdxmin _ self typeCheck: aVar shouldBe: GXReal paramName: ''gXdxmin'']';

addInstVarName: #gXdxmax;
compile:
'gXdxmax: aVar
	aVar notNil ifTrue:[gXdxmax _ self typeCheck: aVar shouldBe: GXReal paramName: ''gXdxmax'']';
compile:
'p2: aVar
	aVar notNil ifTrue:[gXdxmax _ self typeCheck: aVar shouldBe: GXReal paramName: ''gXdxmax'']';

addInstVarName: #gXdymin;
compile:
'gXdymin: aVar
	aVar notNil ifTrue:[gXdymin _ self typeCheck: aVar shouldBe: GXReal paramName: ''gXdymin'']';
compile:
'p3: aVar
	aVar notNil ifTrue:[gXdymin _ self typeCheck: aVar shouldBe: GXReal paramName: ''gXdymin'']';

addInstVarName: #gXdymax;
compile:
'gXdymax: aVar
	aVar notNil ifTrue:[gXdymax _ self typeCheck: aVar shouldBe: GXReal paramName: ''gXdymax'']';
compile:
'p4: aVar
	aVar notNil ifTrue:[gXdymax _ self typeCheck: aVar shouldBe: GXReal paramName: ''gXdymax'']';

addInstVarName: #gXx;
compile:
'gXx: aVar
	aVar notNil ifTrue:[gXx _ self typeCheck: aVar shouldBe: GXRealarray paramName: ''gXx'']';
compile:
'p5: aVar
	aVar notNil ifTrue:[gXx _ self typeCheck: aVar shouldBe: GXRealarray paramName: ''gXx'']';

addInstVarName: #gXy;
compile:
'gXy: aVar
	aVar notNil ifTrue:[gXy _ self typeCheck: aVar shouldBe: GXRealarray paramName: ''gXy'']';
compile:
'p6: aVar
	aVar notNil ifTrue:[gXy _ self typeCheck: aVar shouldBe: GXRealarray paramName: ''gXy'']';

addInstVarName: #gXe;
compile:
'gXe: aVar
	aVar notNil ifTrue:[gXe _ self typeCheck: aVar shouldBe: GXRealarray paramName: ''gXe'']';
compile:
'p7: aVar
	aVar notNil ifTrue:[gXe _ self typeCheck: aVar shouldBe: GXRealarray paramName: ''gXe'']';

addInstVarName: #gXcolour;
compile:
'gXcolour: aVar
	aVar notNil ifTrue:[gXcolour _ self typeCheck: aVar shouldBe: GXColour paramName: ''gXcolour'']';
compile:
'p8: aVar
	aVar notNil ifTrue:[gXcolour _ self typeCheck: aVar shouldBe: GXColour paramName: ''gXcolour'']';

addInstVarName: #gXminmode;
compile:
'gXminmode: aVar
	aVar notNil ifTrue:[gXminmode _ self typeCheck: aVar shouldBe: GXInteger paramName: ''gXminmode'']';
compile:
'p9: aVar
	aVar notNil ifTrue:[gXminmode _ self typeCheck: aVar shouldBe: GXInteger paramName: ''gXminmode'']';

addInstVarName: #gXmaxmode;
compile:
'gXmaxmode: aVar
	aVar notNil ifTrue:[gXmaxmode _ self typeCheck: aVar shouldBe: GXInteger paramName: ''gXmaxmode'']';
compile:
'p10: aVar
	aVar notNil ifTrue:[gXmaxmode _ self typeCheck: aVar shouldBe: GXInteger paramName: ''gXmaxmode'']';

addInstVarName: #gXwxmin;
compile:
'gXwxmin: aVar
	aVar notNil ifTrue:[gXwxmin _ self typeCheck: aVar shouldBe: GXReal paramName: ''gXwxmin'']';
compile:
'p11: aVar
	aVar notNil ifTrue:[gXwxmin _ self typeCheck: aVar shouldBe: GXReal paramName: ''gXwxmin'']';

addInstVarName: #gXwxmax;
compile:
'gXwxmax: aVar
	aVar notNil ifTrue:[gXwxmax _ self typeCheck: aVar shouldBe: GXReal paramName: ''gXwxmax'']';
compile:
'p12: aVar
	aVar notNil ifTrue:[gXwxmax _ self typeCheck: aVar shouldBe: GXReal paramName: ''gXwxmax'']';

addInstVarName: #gXwymin;
compile:
'gXwymin: aVar
	aVar notNil ifTrue:[gXwymin _ self typeCheck: aVar shouldBe: GXReal paramName: ''gXwymin'']';
compile:
'p13: aVar
	aVar notNil ifTrue:[gXwymin _ self typeCheck: aVar shouldBe: GXReal paramName: ''gXwymin'']';

addInstVarName: #gXwymax;
compile:
'gXwymax: aVar
	aVar notNil ifTrue:[gXwymax _ self typeCheck: aVar shouldBe: GXReal paramName: ''gXwymax'']';
compile:
'p14: aVar
	aVar notNil ifTrue:[gXwymax _ self typeCheck: aVar shouldBe: GXReal paramName: ''gXwymax'']'!

"result
 ------"

GXwinUautoscaled

addInstVarName: #gXres
!

!GXwinUautoscaled methodsFor: 'initializing'!

initVars

!!

"BODY
 ----"

!GXwinUautoscaled methodsFor: 'running'!

evaluate
					"NOTE: EARRAY NEED NOT BE DEFINED" 
   |tmp|
   (gXUUstatus = 0)
   ifTrue:['ERROR, graphics device is not open' prenl.
           'To open enter DEVICE/OPEN' prinl]
   ifFalse:[((gXx isNil) | (gXy isNil))
   	    ifTrue:['ERROR' prenl.
           	    'xarray or yarray is not defined' prinl.
           	    tmp _ nil]

   	    ifFalse:[(gXUUgraphics at: #gXcurrentpicture) isNil
   	    	     ifTrue:['Creating Picture' prinl.
	   	    	     tmp _ Picture new.
   	   	    	   ((gXUUgraphics at: #gXcurrentgraphics) addToPictureList: tmp).
		    	    (gXUUgraphics at: #gXcurrentdevice) associate: tmp.
	   	    	     gXUUgraphics at: #gXcurrentpicture put: (tmp select)].
   	   	     tmp _ (WindowScaled new xMin: gXdxmin; xMax: gXdxmax;
                   	    yMin: gXdymin; yMax: gXdymax; xarray: gXx;
                   	    yarray: gXy; earray: gXe; winColour: gXcolour;
                   	    minMode: gXminmode; maxMode: gXmaxmode; 
			    lxMin: gXwxmin; lxMax: gXwxmax; lyMin: gXwymin;
			    lyMax: gXwymax; create)].

   	    (tmp = nil)
            ifTrue:['Window has not been created' prenl]
            ifFalse:[gXUUgraphics at: #gXcurrentpicture put:
                     ((gXUUgraphics at: #gXcurrentpicture) addToWindowList: tmp).
            	     gXUUgraphics at: #gXcurrentwindow put: tmp.
            	     gXres _ tmp.
            	     (gXUUgraphics at: #gXcurrentwindow) display]]

.
self up.
^gXres
!!
nil! "(fix for smalltalk bug, this line is ignored if there is a variable declaration error)"

"End of procedure"

"Procedure class definition:
 ==========================="
((Smalltalk at: #GXwinUmultiplot ifAbsent: [ nil ]) isKindOf: Function class) ifTrue: [ ('Warning: Overriding previously defined procedure ', 'GXwinUmultiplot' gtype) prinl ].
Smalltalk at: #GXwinUmultiplot put: nil!
Function subclass: 'GXwinUmultiplot'
       instanceVariableNames: ''
       classVariableNames: ''
       poolDictionaries: ''
       category: 'Genie Procedure'
!

"HEADER
 ------"

"parameters
 ----------"

GXwinUmultiplot

addInstVarName: #gXdxmin;
compile:
'gXdxmin: aVar
	aVar notNil ifTrue:[gXdxmin _ self typeCheck: aVar shouldBe: GXReal paramName: ''gXdxmin'']';
compile:
'p1: aVar
	aVar notNil ifTrue:[gXdxmin _ self typeCheck: aVar shouldBe: GXReal paramName: ''gXdxmin'']';

addInstVarName: #gXdxmax;
compile:
'gXdxmax: aVar
	aVar notNil ifTrue:[gXdxmax _ self typeCheck: aVar shouldBe: GXReal paramName: ''gXdxmax'']';
compile:
'p2: aVar
	aVar notNil ifTrue:[gXdxmax _ self typeCheck: aVar shouldBe: GXReal paramName: ''gXdxmax'']';

addInstVarName: #gXdymin;
compile:
'gXdymin: aVar
	aVar notNil ifTrue:[gXdymin _ self typeCheck: aVar shouldBe: GXReal paramName: ''gXdymin'']';
compile:
'p3: aVar
	aVar notNil ifTrue:[gXdymin _ self typeCheck: aVar shouldBe: GXReal paramName: ''gXdymin'']';

addInstVarName: #gXdymax;
compile:
'gXdymax: aVar
	aVar notNil ifTrue:[gXdymax _ self typeCheck: aVar shouldBe: GXReal paramName: ''gXdymax'']';
compile:
'p4: aVar
	aVar notNil ifTrue:[gXdymax _ self typeCheck: aVar shouldBe: GXReal paramName: ''gXdymax'']';

addInstVarName: #gXwxmin;
compile:
'gXwxmin: aVar
	aVar notNil ifTrue:[gXwxmin _ self typeCheck: aVar shouldBe: GXReal paramName: ''gXwxmin'']';
compile:
'p5: aVar
	aVar notNil ifTrue:[gXwxmin _ self typeCheck: aVar shouldBe: GXReal paramName: ''gXwxmin'']';

addInstVarName: #gXwxmax;
compile:
'gXwxmax: aVar
	aVar notNil ifTrue:[gXwxmax _ self typeCheck: aVar shouldBe: GXReal paramName: ''gXwxmax'']';
compile:
'p6: aVar
	aVar notNil ifTrue:[gXwxmax _ self typeCheck: aVar shouldBe: GXReal paramName: ''gXwxmax'']'!

"parameters
 ----------"

GXwinUmultiplot

addInstVarName: #gXwymin;
compile:
'gXwymin: aVar
	aVar notNil ifTrue:[gXwymin _ self typeCheck: aVar shouldBe: GXReal paramName: ''gXwymin'']';
compile:
'p7: aVar
	aVar notNil ifTrue:[gXwymin _ self typeCheck: aVar shouldBe: GXReal paramName: ''gXwymin'']';

addInstVarName: #gXwymax;
compile:
'gXwymax: aVar
	aVar notNil ifTrue:[gXwymax _ self typeCheck: aVar shouldBe: GXReal paramName: ''gXwymax'']';
compile:
'p8: aVar
	aVar notNil ifTrue:[gXwymax _ self typeCheck: aVar shouldBe: GXReal paramName: ''gXwymax'']';

addInstVarName: #gXminmode;
compile:
'gXminmode: aVar
	aVar notNil ifTrue:[gXminmode _ self typeCheck: aVar shouldBe: GXInteger paramName: ''gXminmode'']';
compile:
'p9: aVar
	aVar notNil ifTrue:[gXminmode _ self typeCheck: aVar shouldBe: GXInteger paramName: ''gXminmode'']';

addInstVarName: #gXmaxmode;
compile:
'gXmaxmode: aVar
	aVar notNil ifTrue:[gXmaxmode _ self typeCheck: aVar shouldBe: GXInteger paramName: ''gXmaxmode'']';
compile:
'p10: aVar
	aVar notNil ifTrue:[gXmaxmode _ self typeCheck: aVar shouldBe: GXInteger paramName: ''gXmaxmode'']';

addInstVarName: #gXcolour;
compile:
'gXcolour: aVar
	aVar notNil ifTrue:[gXcolour _ self typeCheck: aVar shouldBe: GXColour paramName: ''gXcolour'']';
compile:
'p11: aVar
	aVar notNil ifTrue:[gXcolour _ self typeCheck: aVar shouldBe: GXColour paramName: ''gXcolour'']'!

"result
 ------"

GXwinUmultiplot

addInstVarName: #gXres
!

!GXwinUmultiplot methodsFor: 'initializing'!

initVars

!!

"BODY
 ----"

!GXwinUmultiplot methodsFor: 'running'!

evaluate

   |tmp|
   
   (gXUUstatus = 0)
   ifTrue:['ERROR, graphics device is not open' prenl.
           'To open enter DEVICE/OPEN' prinl]
   ifFalse:[(gXUUgraphics at: #gXcurrentpicture) isNil
   	    ifTrue:['Creating Picture' prinl.
	   	    tmp _ Picture new.
   	   	    ((gXUUgraphics at: #gXcurrentgraphics) addToPictureList: tmp).
		    (gXUUgraphics at: #gXcurrentdevice) associate: tmp.
	   	    gXUUgraphics at: #gXcurrentpicture put: (tmp select)].
   	    tmp _ (MultiplotWindow new xMin: gXdxmin; xMax: gXdxmax;
                   yMin: gXdymin; yMax: gXdymax; lxMin: gXwxmin; 
		   lxMax: gXwxmax; lyMin: gXwymin; lyMax: gXwymax; 
		   minMode: gXminmode; maxMode: gXmaxmode; 
		   winColour: gXcolour; create).

   	    (tmp = nil)
            ifTrue:['Window has not been created' prenl]
            ifFalse:[gXUUgraphics at: #gXcurrentpicture put:
               	    ((gXUUgraphics at: #gXcurrentpicture) addToWindowList: tmp).
                    gXUUgraphics at: #gXcurrentwindow put: tmp.
                    gXres _ tmp.
                    (gXUUgraphics at: #gXcurrentwindow) display]]
.
self up.
^gXres
!!
nil! "(fix for smalltalk bug, this line is ignored if there is a variable declaration error)"

"End of procedure"

"Procedure class definition:
 ==========================="
((Smalltalk at: #GXgetcursor ifAbsent: [ nil ]) isKindOf: Function class) ifTrue: [ ('Warning: Overriding previously defined procedure ', 'GXgetcursor' gtype) prinl ].
Smalltalk at: #GXgetcursor put: nil!
Function subclass: 'GXgetcursor'
       instanceVariableNames: ''
       classVariableNames: ''
       poolDictionaries: ''
       category: 'Genie Procedure'
!

"HEADER
 ------"

"parameters
 ----------"

GXgetcursor

addInstVarName: #gXx;
compile:
'gXx: aVar
	aVar notNil ifTrue:[gXx _ self typeCheck: aVar shouldBe: GXReal paramName: ''gXx'']';
compile:
'p1: aVar
	aVar notNil ifTrue:[gXx _ self typeCheck: aVar shouldBe: GXReal paramName: ''gXx'']';

addInstVarName: #gXy;
compile:
'gXy: aVar
	aVar notNil ifTrue:[gXy _ self typeCheck: aVar shouldBe: GXReal paramName: ''gXy'']';
compile:
'p2: aVar
	aVar notNil ifTrue:[gXy _ self typeCheck: aVar shouldBe: GXReal paramName: ''gXy'']'!

"result
 ------"

GXgetcursor

addInstVarName: #gXres
!

"Local variables
 ---------------"

GXgetcursor

addInstVarName: #gXx;

addInstVarName: #gXy
!

!GXgetcursor methodsFor: 'initializing'!

initVars
	gXx _ 0.000000000000000e000.
	gXy _ 0.000000000000000e000
!!

"BODY
 ----"

!GXgetcursor methodsFor: 'running'!

evaluate

   gXres _ ((gXUUgraphics at: #gXcurrentpicture) getCursor: gXx and: gXy)
.
self up.
^gXres
!!
nil! "(fix for smalltalk bug, this line is ignored if there is a variable declaration error)"

"End of procedure"

"Procedure class definition:
 ==========================="
((Smalltalk at: #GXnewUzoom ifAbsent: [ nil ]) isKindOf: Function class) ifTrue: [ ('Warning: Overriding previously defined procedure ', 'GXnewUzoom' gtype) prinl ].
Smalltalk at: #GXnewUzoom put: nil!
Function subclass: 'GXnewUzoom'
       instanceVariableNames: ''
       classVariableNames: ''
       poolDictionaries: ''
       category: 'Genie Procedure'
!

"BODY
 ----"

!GXnewUzoom methodsFor: 'running'!

evaluate

   (gXUUgraphics at: #gXcurrentpicture) isNil
   ifTrue:['ERROR, ' pren.
	   'A Picture is not created' prinl]
   ifFalse:[
   'THIS IS ONLY PARTIALLY IMPLEMENTED' prenl.
   (gXUUgraphics at: #gXcurrentpicture) zoom]
.
self up.
^nil
!!
nil! "(fix for smalltalk bug, this line is ignored if there is a variable declaration error)"

"End of procedure"

((Smalltalk at: #GXobj ifAbsent: [Smalltalk at: #GXobj put: 				  'Procedure "OBJ" has only been FORWARD declared, it must also have a PROCEDURE definition' ] ) 				  isKindOf: Function class) ifTrue: 				[ 'Warning: FORWARD statement for a previously defined procedure "OBJ" - no action taken' prinl ]!
"Procedure class definition:
 ==========================="
((Smalltalk at: #GXundraw ifAbsent: [ nil ]) isKindOf: Function class) ifTrue: [ ('Warning: Overriding previously defined procedure ', 'GXundraw' gtype) prinl ].
Smalltalk at: #GXundraw put: nil!
Function subclass: 'GXundraw'
       instanceVariableNames: ''
       classVariableNames: ''
       poolDictionaries: ''
       category: 'Genie Procedure'
!

"HEADER
 ------"

"qualifiers
 ----------"

GXundraw

addInstVarName: #gXall;
compile:
'gXall
	gXall _ true';
compile:
'gXall: aBoolean
	gXall _ aBoolean'!

"parameters
 ----------"

GXundraw

addInstVarName: #gXitem;
compile:
'gXitem: aVar
	aVar notNil ifTrue:[ gXitem _ aVar ]';
compile:
'p1: aVar
	aVar notNil ifTrue:[ gXitem _ aVar ]'
!

"Local variables
 ---------------"

GXundraw

addInstVarName: #gXitem
!

!GXundraw methodsFor: 'initializing'!

initVars
	gXall _ false.
	gXitem _ (GXobj run line: 0 proc: self method: true; p1:0; p2:0; p3:0; condEvaluate)
!!

"BODY
 ----"

!GXundraw methodsFor: 'running'!

evaluate

   |pic wind dev object|
   (gXall) ifTrue:[^nil].
   (gXitem isKindOf: Picture)
   ifTrue:[pic _ ((gXUUgraphics at: #gXcurrentgraphics) 
						getPictureWithAPicture: gXitem).
	   (pic isNil)
	   ifFalse:[dev _ ((gXUUgraphics at: #gXcurrentgraphics) 
							       findDevice: pic).
		    (gXUUgraphics at: #gXcurrentgraphics) undrawPicture: pic]].

   (gXitem isKindOf: Window)
   ifTrue:[wind _ ((gXUUgraphics at: #gXcurrentgraphics)
					   findDrawableObjectsPicture: gXitem).
	   (wind isNil)
	   ifFalse:[pic _ ((gXUUgraphics at: #gXcurrentgraphics)
							   findPicture: gXitem).
		    dev _ ((gXUUgraphics at: #gXcurrentgraphics)
							       findDevice: pic).
		    pic undrawWindow: wind]].

   (gXitem isKindOf: DrawableObjects)
   ifTrue:[(gXitem isKindOf: Window)
	   ifFalse:[object _ ((gXUUgraphics at: #gXcurrentgraphics)
					findDrawableObjectsPicture: gXitem).
		    (object isNil)
		    ifFalse:[pic _ ((gXUUgraphics at: #gXcurrentgraphics)
						findPicture: gXitem).
			     dev _ ((gXUUgraphics at: #gXcurrentgraphics)
						findDevice: pic).
			     wind _ (pic findRightWindow: gXitem).
			     wind undrawObject: object]]].
   
   Device Clearing.
   ( (gXitem isKindOf: Picture) | (pic isNil) ) ifFalse: [pic drawUnflaggedWindows]

.
self up.
^nil
!!
nil! "(fix for smalltalk bug, this line is ignored if there is a variable declaration error)"

"End of procedure"

GXcomment run line: 0 proc: self method: false; p1:'undraw'; p2:'Undraw a primitive graphics operation'; condEvaluate!
((Smalltalk at: #GXselect ifAbsent: [Smalltalk at: #GXselect put: 				  'Procedure "Select" has only been FORWARD declared, it must also have a PROCEDURE definition' ] ) 				  isKindOf: Function class) ifTrue: 				[ 'Warning: FORWARD statement for a previously defined procedure "Select" - no action taken' prinl ]!
((Smalltalk at: #GXpic ifAbsent: [Smalltalk at: #GXpic put: 				  'Procedure "Pic" has only been FORWARD declared, it must also have a PROCEDURE definition' ] ) 				  isKindOf: Function class) ifTrue: 				[ 'Warning: FORWARD statement for a previously defined procedure "Pic" - no action taken' prinl ]!
((Smalltalk at: #GXwin ifAbsent: [Smalltalk at: #GXwin put: 				  'Procedure "Win" has only been FORWARD declared, it must also have a PROCEDURE definition' ] ) 				  isKindOf: Function class) ifTrue: 				[ 'Warning: FORWARD statement for a previously defined procedure "Win" - no action taken' prinl ]!
"Procedure class definition:
 ==========================="
((Smalltalk at: #GXdelete ifAbsent: [ nil ]) isKindOf: Function class) ifTrue: [ ('Warning: Overriding previously defined procedure ', 'GXdelete' gtype) prinl ].
Smalltalk at: #GXdelete put: nil!
Function subclass: 'GXdelete'
       instanceVariableNames: ''
       classVariableNames: ''
       poolDictionaries: ''
       category: 'Genie Procedure'
!

"HEADER
 ------"

"parameters
 ----------"

GXdelete

addInstVarName: #gXitem;
compile:
'gXitem: aVar
	aVar notNil ifTrue:[gXitem _ self typeCheck: aVar shouldBe: GXGobject paramName: ''gXitem'']';
compile:
'p1: aVar
	aVar notNil ifTrue:[gXitem _ self typeCheck: aVar shouldBe: GXGobject paramName: ''gXitem'']'
!

!GXdelete methodsFor: 'initializing'!

initVars

!!

"BODY
 ----"

!GXdelete methodsFor: 'running'!

evaluate

   |pic wind dev object|
    (gXitem isKindOf: Picture)
     ifTrue:[pic _ ((gXUUgraphics at: #gXcurrentgraphics) 
					    getPictureWithAPicture: gXitem).
	     (pic isNil)
	    ifFalse:[
			dev _ gXUUgraphics at: #gXcurrentdevice.
			(gXUUgraphics at: #gXcurrentgraphics) deletePicture: pic]].

    (gXitem isKindOf: Window)
    ifTrue:[wind _ ((gXUUgraphics at: #gXcurrentgraphics)
					  findDrawableObjectsPicture: gXitem).
	    (wind isNil)
	    ifFalse:[pic _ ((gXUUgraphics at: #gXcurrentgraphics) 
							   findPicture: gXitem).
				dev _ gXUUgraphics at: #gXcurrentdevice.
				pic deleteWindow: wind]].

    (gXitem isKindOf: DrawableObjects)
    ifTrue:[(gXitem isKindOf: Window)
 	    ifFalse:[object _ ((gXUUgraphics at: #gXcurrentgraphics)
					  findDrawableObjectsPicture: gXitem).
	    	     (object isNil)
	     	     ifFalse:[pic _ ((gXUUgraphics at: #gXcurrentgraphics)
                                                           findPicture: gXitem).
"					   dev _ ((gXUUgraphics at: #gXcurrentgraphics) findDevice: pic)."
					   dev _ gXUUgraphics at: #gXcurrentdevice.
		     	      wind _ (pic findRightWindow: gXitem).
		     	      wind deleteObject: object]]].


   dev associate: pic
   
.
GXselect run line: 0 proc: self method: false; p1:(GXwin run line: 0 proc: self method: true; p1:0; p2:0; condEvaluate); condEvaluate.
GXselect run line: 0 proc: self method: false; p1:(GXpic run line: 0 proc: self method: true; p1:0; condEvaluate); condEvaluate.
self up.
^nil
!!
nil! "(fix for smalltalk bug, this line is ignored if there is a variable declaration error)"

"End of procedure"

"Procedure class definition:
 ==========================="
((Smalltalk at: #GXpic ifAbsent: [ nil ]) isKindOf: Function class) ifTrue: [ ('Warning: Overriding previously defined procedure ', 'GXpic' gtype) prinl ].
Smalltalk at: #GXpic put: nil!
Function subclass: 'GXpic'
       instanceVariableNames: ''
       classVariableNames: ''
       poolDictionaries: ''
       category: 'Genie Procedure'
!

"HEADER
 ------"

"parameters
 ----------"

GXpic

addInstVarName: #gXpicnum;
compile:
'gXpicnum: aVar
	aVar notNil ifTrue:[gXpicnum _ self typeCheck: aVar shouldBe: GXInteger paramName: ''gXpicnum'']';
compile:
'p1: aVar
	aVar notNil ifTrue:[gXpicnum _ self typeCheck: aVar shouldBe: GXInteger paramName: ''gXpicnum'']'!

"result
 ------"

GXpic

addInstVarName: #gXpic
!

!GXpic methodsFor: 'initializing'!

initVars

!!

"BODY
 ----"

!GXpic methodsFor: 'running'!

evaluate

  |picnum cg |
  (gXpicnum isNil)
  ifFalse:
  [
	cg _ gXUUgraphics at: #gXcurrentgraphics.
	picnum _ cg getNumber: gXpicnum.
	gXpic _  cg getPicture: picnum
  ]
  ifTrue:
	[ 'Pic() - Error, must specify all parameters' prenl ]
.
self up.
^gXpic
!!
nil! "(fix for smalltalk bug, this line is ignored if there is a variable declaration error)"

"End of procedure"

"Procedure class definition:
 ==========================="
((Smalltalk at: #GXwin ifAbsent: [ nil ]) isKindOf: Function class) ifTrue: [ ('Warning: Overriding previously defined procedure ', 'GXwin' gtype) prinl ].
Smalltalk at: #GXwin put: nil!
Function subclass: 'GXwin'
       instanceVariableNames: ''
       classVariableNames: ''
       poolDictionaries: ''
       category: 'Genie Procedure'
!

"HEADER
 ------"

"parameters
 ----------"

GXwin

addInstVarName: #gXpicnum;
compile:
'gXpicnum: aVar
	aVar notNil ifTrue:[gXpicnum _ self typeCheck: aVar shouldBe: GXInteger paramName: ''gXpicnum'']';
compile:
'p1: aVar
	aVar notNil ifTrue:[gXpicnum _ self typeCheck: aVar shouldBe: GXInteger paramName: ''gXpicnum'']';

addInstVarName: #gXwinnum;
compile:
'gXwinnum: aVar
	aVar notNil ifTrue:[gXwinnum _ self typeCheck: aVar shouldBe: GXInteger paramName: ''gXwinnum'']';
compile:
'p2: aVar
	aVar notNil ifTrue:[gXwinnum _ self typeCheck: aVar shouldBe: GXInteger paramName: ''gXwinnum'']'!

"result
 ------"

GXwin

addInstVarName: #gXwin
!

!GXwin methodsFor: 'initializing'!

initVars

!!

"BODY
 ----"

!GXwin methodsFor: 'running'!

evaluate

  |winnum pic picnum cg|
  ((gXwinnum isNil) |
   (gXpicnum isNil) )
  ifFalse:
  [
	cg _ gXUUgraphics at: #gXcurrentgraphics.

	picnum _ cg getNumber: gXpicnum.
	pic _ cg getPicture: picnum.
	pic notNil ifTrue:
	[
		winnum _ pic getNumber: gXwinnum.
		gXwin _ pic getWindow: winnum
	]
  ]
  ifTrue:
    [ 'Win() - Error, must specify all parameters' prenl ]
.
self up.
^gXwin
!!
nil! "(fix for smalltalk bug, this line is ignored if there is a variable declaration error)"

"End of procedure"

"Procedure class definition:
 ==========================="
((Smalltalk at: #GXobj ifAbsent: [ nil ]) isKindOf: Function class) ifTrue: [ ('Warning: Overriding previously defined procedure ', 'GXobj' gtype) prinl ].
Smalltalk at: #GXobj put: nil!
Function subclass: 'GXobj'
       instanceVariableNames: ''
       classVariableNames: ''
       poolDictionaries: ''
       category: 'Genie Procedure'
!

"HEADER
 ------"

"parameters
 ----------"

GXobj

addInstVarName: #gXpicnum;
compile:
'gXpicnum: aVar
	aVar notNil ifTrue:[gXpicnum _ self typeCheck: aVar shouldBe: GXInteger paramName: ''gXpicnum'']';
compile:
'p1: aVar
	aVar notNil ifTrue:[gXpicnum _ self typeCheck: aVar shouldBe: GXInteger paramName: ''gXpicnum'']';

addInstVarName: #gXwinnum;
compile:
'gXwinnum: aVar
	aVar notNil ifTrue:[gXwinnum _ self typeCheck: aVar shouldBe: GXInteger paramName: ''gXwinnum'']';
compile:
'p2: aVar
	aVar notNil ifTrue:[gXwinnum _ self typeCheck: aVar shouldBe: GXInteger paramName: ''gXwinnum'']';

addInstVarName: #gXobjnum;
compile:
'gXobjnum: aVar
	aVar notNil ifTrue:[gXobjnum _ self typeCheck: aVar shouldBe: GXInteger paramName: ''gXobjnum'']';
compile:
'p3: aVar
	aVar notNil ifTrue:[gXobjnum _ self typeCheck: aVar shouldBe: GXInteger paramName: ''gXobjnum'']'!

"result
 ------"

GXobj

addInstVarName: #gXobj
!

!GXobj methodsFor: 'initializing'!

initVars

!!

"BODY
 ----"

!GXobj methodsFor: 'running'!

evaluate

  |objnum winnum win picnum pic cg cw|

  ((gXobjnum isNil) |
   (gXwinnum isNil) |
   (gXpicnum isNil) )
  ifFalse:
  [
	cg _ gXUUgraphics at: #gXcurrentgraphics.

  	picnum _ cg getNumber: gXpicnum.
	pic _ cg getPicture: picnum.
	pic notNil ifTrue:
	[
		winnum _ pic getNumber: gXwinnum.
		win _ pic getWindow: winnum
	].
	win notNil ifTrue:
	[
		objnum _ win getNumber: gXobjnum.
		gXobj _ win getObject: objnum
	]
  ]
  ifTrue:
  [  'Obj() - Error, must specify all parameters' prenl ]
.
self up.
^gXobj
!!
nil! "(fix for smalltalk bug, this line is ignored if there is a variable declaration error)"

"End of procedure"

"Procedure class definition:
 ==========================="
((Smalltalk at: #GXdev ifAbsent: [ nil ]) isKindOf: Function class) ifTrue: [ ('Warning: Overriding previously defined procedure ', 'GXdev' gtype) prinl ].
Smalltalk at: #GXdev put: nil!
Function subclass: 'GXdev'
       instanceVariableNames: ''
       classVariableNames: ''
       poolDictionaries: ''
       category: 'Genie Procedure'
!

"HEADER
 ------"

"parameters
 ----------"

GXdev

addInstVarName: #gXdevnum;
compile:
'gXdevnum: aVar
	aVar notNil ifTrue:[gXdevnum _ self typeCheck: aVar shouldBe: GXInteger paramName: ''gXdevnum'']';
compile:
'p1: aVar
	aVar notNil ifTrue:[gXdevnum _ self typeCheck: aVar shouldBe: GXInteger paramName: ''gXdevnum'']'!

"result
 ------"

GXdev

addInstVarName: #gXdev
!

!GXdev methodsFor: 'initializing'!

initVars

!!

"BODY
 ----"

!GXdev methodsFor: 'running'!

evaluate

  |devnum cg|

  (gXdevnum isNil)
  ifFalse:
	[
		cg _ gXUUgraphics at: #gXcurrentgraphics.

		devnum _ cg getDeviceNumber: gXdevnum.
		gXdev _ cg returnDevice: devnum
	]

  ifTrue:
	[ 'Dev() - Error, must specify all parameters' prenl ]
.
self up.
^gXdev
!!
nil! "(fix for smalltalk bug, this line is ignored if there is a variable declaration error)"

"End of procedure"

"Procedure class definition:
 ==========================="
((Smalltalk at: #GXselect ifAbsent: [ nil ]) isKindOf: Function class) ifTrue: [ ('Warning: Overriding previously defined procedure ', 'GXselect' gtype) prinl ].
Smalltalk at: #GXselect put: nil!
Function subclass: 'GXselect'
       instanceVariableNames: ''
       classVariableNames: ''
       poolDictionaries: ''
       category: 'Genie Procedure'
!

"HEADER
 ------"

"parameters
 ----------"

GXselect

addInstVarName: #gXitem;
compile:
'gXitem: aVar
	aVar notNil ifTrue:[gXitem _ self typeCheck: aVar shouldBe: GXGobject paramName: ''gXitem'']';
compile:
'p1: aVar
	aVar notNil ifTrue:[gXitem _ self typeCheck: aVar shouldBe: GXGobject paramName: ''gXitem'']'!

"result
 ------"

GXselect

addInstVarName: #gXres
!

!GXselect methodsFor: 'initializing'!

initVars

!!

"BODY
 ----"

!GXselect methodsFor: 'running'!

evaluate

    |tmp temp|

    (( gXitem isKindOf: Picture ) |
     ( gXitem isKindOf: Device ) |
     ( gXitem isKindOf: Window ))
    ifTrue:
    [
		"Make the current picture the picture specified and, unset
		 the current window - no assumptions or user would have used
		 select win(m,n) instead"
		( gXitem isKindOf: Picture )
		    ifTrue: [
		  	gXUUgraphics at:#gXcurrentpicture put: (gXitem select).
		    ].

		"Select the current device but don't do anything else, note
		 that redraw with no paramters will put current picture here"
		( gXitem isKindOf: Device )
			ifTrue:
			[ gXUUgraphics at:#gXcurrentdevice put: gXitem.
			  gXitem selectDevice: gXitem
			].


		"Make the current window wherever specified but also change
		 the current picture to the one containing the window"
		( gXitem isKindOf: Window )
			ifTrue:
			[
				gXitem display.
				gXUUgraphics at:#gXcurrentwindow put: gXitem.
			].

		gXres _ gXitem
	]
	ifFalse:
	[
		'SELECT ERROR' prenl.
		'Must specify a device, window or picture' prinl.
		gXres _ nil
	]
.
self up.
^gXres
!!
nil! "(fix for smalltalk bug, this line is ignored if there is a variable declaration error)"

"End of procedure"

"Procedure class definition:
 ==========================="
((Smalltalk at: #GXUUdummyarray ifAbsent: [ nil ]) isKindOf: Function class) ifTrue: [ ('Warning: Overriding previously defined procedure ', 'GXUUdummyarray' gtype) prinl ].
Smalltalk at: #GXUUdummyarray put: nil!
Function subclass: 'GXUUdummyarray'
       instanceVariableNames: ''
       classVariableNames: ''
       poolDictionaries: ''
       category: 'Genie Procedure'
!

"HEADER
 ------"

"parameters
 ----------"

GXUUdummyarray

addInstVarName: #gXlength;
compile:
'gXlength: aVar
	aVar notNil ifTrue:[gXlength _ self typeCheck: aVar shouldBe: GXInteger paramName: ''gXlength'']';
compile:
'p1: aVar
	aVar notNil ifTrue:[gXlength _ self typeCheck: aVar shouldBe: GXInteger paramName: ''gXlength'']';

addInstVarName: #gXvalue;
compile:
'gXvalue: aVar
	aVar notNil ifTrue:[ gXvalue _ aVar ]';
compile:
'p2: aVar
	aVar notNil ifTrue:[ gXvalue _ aVar ]'!

"result
 ------"

GXUUdummyarray

addInstVarName: #gXretval
!

"Local variables
 ---------------"

GXUUdummyarray

addInstVarName: #gXlength;

addInstVarName: #gXvalue;

addInstVarName: #gXa
!

!GXUUdummyarray methodsFor: 'initializing'!

initVars
	gXlength _ 2.
	gXvalue _ 1
!!

"BODY
 ----"

!GXUUdummyarray methodsFor: 'running'!

evaluate

    gXretval _ ( GXArray new: gXlength ).
    1 to: gXlength do: [ :gXa |  gXretval at: gXa put: (gXa * gXvalue) ]
.
self up.
^gXretval
!!
nil! "(fix for smalltalk bug, this line is ignored if there is a variable declaration error)"

"End of procedure"

GXcomment run line: 0 proc: self method: false; p1:'__dummyarray'; p2:'Return an array of a given size and content for use as a default procedure parameter'; condEvaluate!
"Procedure class definition:
 ==========================="
((Smalltalk at: #GXUUinitgooglyworkspace1 ifAbsent: [ nil ]) isKindOf: Function class) ifTrue: [ ('Warning: Overriding previously defined procedure ', 'GXUUinitgooglyworkspace1' gtype) prinl ].
Smalltalk at: #GXUUinitgooglyworkspace1 put: nil!
Function subclass: 'GXUUinitgooglyworkspace1'
       instanceVariableNames: ''
       classVariableNames: ''
       poolDictionaries: ''
       category: 'Genie Procedure'
!

"BODY
 ----"

!GXUUinitgooglyworkspace1 methodsFor: 'running'!

evaluate
(gXUsdjUgw isKindOf: GXArray) | (gXUsdjUgw isKindOf: GXWorkspace)
    ifTrue:  [ ((gXUsdjUgw) at:#gXxlog put: (gXDfalse) deepCopy) ]
    ifFalse: [ self error: 'Assignment to array or workspace failed, check that the destination is a structured type'].
(gXUsdjUgw isKindOf: GXArray) | (gXUsdjUgw isKindOf: GXWorkspace)
    ifTrue:  [ ((gXUsdjUgw) at:#gXylog put: (gXDfalse) deepCopy) ]
    ifFalse: [ self error: 'Assignment to array or workspace failed, check that the destination is a structured type'].
(gXUsdjUgw isKindOf: GXArray) | (gXUsdjUgw isKindOf: GXWorkspace)
    ifTrue:  [ ((gXUsdjUgw) at:#gXinteractive put: 'XW' deepCopy) ]
    ifFalse: [ self error: 'Assignment to array or workspace failed, check that the destination is a structured type'].
(gXUsdjUgw isKindOf: GXArray) | (gXUsdjUgw isKindOf: GXWorkspace)
    ifTrue:  [ ((gXUsdjUgw) at:#gXsize put: 7.000000000000000e-01 deepCopy) ]
    ifFalse: [ self error: 'Assignment to array or workspace failed, check that the destination is a structured type'].
(gXUsdjUgw isKindOf: GXArray) | (gXUsdjUgw isKindOf: GXWorkspace)
    ifTrue:  [ ((gXUsdjUgw) at:#gXhardcopy put: 'PS' deepCopy) ]
    ifFalse: [ self error: 'Assignment to array or workspace failed, check that the destination is a structured type'].
(gXUsdjUgw isKindOf: GXArray) | (gXUsdjUgw isKindOf: GXWorkspace)
    ifTrue:  [ ((gXUsdjUgw) at:#gXstatus put: 'CLOSED' deepCopy) ]
    ifFalse: [ self error: 'Assignment to array or workspace failed, check that the destination is a structured type'].
(gXUsdjUgw isKindOf: GXArray) | (gXUsdjUgw isKindOf: GXWorkspace)
    ifTrue:  [ ((gXUsdjUgw) at:#gXmaxpict put: 600 deepCopy) ]
    ifFalse: [ self error: 'Assignment to array or workspace failed, check that the destination is a structured type'].
(gXUsdjUgw isKindOf: GXArray) | (gXUsdjUgw isKindOf: GXWorkspace)
    ifTrue:  [ ((gXUsdjUgw) at:#gXtitles put: (GXdimensions run line: 0 proc: self method: true; p1:((gXUsdjUgw) at:#gXmaxpict); condEvaluate) deepCopy) ]
    ifFalse: [ self error: 'Assignment to array or workspace failed, check that the destination is a structured type'].
(gXUsdjUgw isKindOf: GXArray) | (gXUsdjUgw isKindOf: GXWorkspace)
    ifTrue:  [ ((gXUsdjUgw) at:#gXnewpict put: 0 deepCopy) ]
    ifFalse: [ self error: 'Assignment to array or workspace failed, check that the destination is a structured type'].
(gXUsdjUgw isKindOf: GXArray) | (gXUsdjUgw isKindOf: GXWorkspace)
    ifTrue:  [ ((gXUsdjUgw) at:#gXdevwidth put: 5.000000000000000e-01 deepCopy) ]
    ifFalse: [ self error: 'Assignment to array or workspace failed, check that the destination is a structured type'].
(gXUsdjUgw isKindOf: GXArray) | (gXUsdjUgw isKindOf: GXWorkspace)
    ifTrue:  [ ((gXUsdjUgw) at:#gXdevheight put: 5.000000000000000e-01 deepCopy) ]
    ifFalse: [ self error: 'Assignment to array or workspace failed, check that the destination is a structured type'].
(gXUsdjUgw isKindOf: GXArray) | (gXUsdjUgw isKindOf: GXWorkspace)
    ifTrue:  [ ((gXUsdjUgw) at:#gXygap put: 1.000000000000000e001 deepCopy) ]
    ifFalse: [ self error: 'Assignment to array or workspace failed, check that the destination is a structured type'].
(gXUsdjUgw isKindOf: GXArray) | (gXUsdjUgw isKindOf: GXWorkspace)
    ifTrue:  [ ((gXUsdjUgw) at:#gXgraticule put: (gXDfalse) deepCopy) ]
    ifFalse: [ self error: 'Assignment to array or workspace failed, check that the destination is a structured type'].
(gXUsdjUgw isKindOf: GXArray) | (gXUsdjUgw isKindOf: GXWorkspace)
    ifTrue:  [ ((gXUsdjUgw) at:#gXcls put: (gXDtrue) deepCopy) ]
    ifFalse: [ self error: 'Assignment to array or workspace failed, check that the destination is a structured type'].
(gXUsdjUgw isKindOf: GXArray) | (gXUsdjUgw isKindOf: GXWorkspace)
    ifTrue:  [ ((gXUsdjUgw) at:#gXleft put: 5.000000000000000e-02 deepCopy) ]
    ifFalse: [ self error: 'Assignment to array or workspace failed, check that the destination is a structured type'].
(gXUsdjUgw isKindOf: GXArray) | (gXUsdjUgw isKindOf: GXWorkspace)
    ifTrue:  [ ((gXUsdjUgw) at:#gXright put: 9.500000000000000e-01 deepCopy) ]
    ifFalse: [ self error: 'Assignment to array or workspace failed, check that the destination is a structured type'].
(gXUsdjUgw isKindOf: GXArray) | (gXUsdjUgw isKindOf: GXWorkspace)
    ifTrue:  [ ((gXUsdjUgw) at:#gXmarkersize put: 7.000000000000000e-01 deepCopy) ]
    ifFalse: [ self error: 'Assignment to array or workspace failed, check that the destination is a structured type'].
self up.
^nil
!!
nil! "(fix for smalltalk bug, this line is ignored if there is a variable declaration error)"

"End of procedure"

GXcomment run line: 0 proc: self method: false; p1:'__initgooglyworkspace1'; p2:'Sets up the Open Genie workspace that contain graphics defaults and status information'; condEvaluate!
"Procedure class definition:
 ==========================="
((Smalltalk at: #GXUUinitgooglyworkspace2 ifAbsent: [ nil ]) isKindOf: Function class) ifTrue: [ ('Warning: Overriding previously defined procedure ', 'GXUUinitgooglyworkspace2' gtype) prinl ].
Smalltalk at: #GXUUinitgooglyworkspace2 put: nil!
Function subclass: 'GXUUinitgooglyworkspace2'
       instanceVariableNames: ''
       classVariableNames: ''
       poolDictionaries: ''
       category: 'Genie Procedure'
!

"BODY
 ----"

!GXUUinitgooglyworkspace2 methodsFor: 'running'!

evaluate
(gXUsdjUgw isKindOf: GXArray) | (gXUsdjUgw isKindOf: GXWorkspace)
    ifTrue:  [ ((gXUsdjUgw) at:#gXtop put: 9.500000000000000e-01 deepCopy) ]
    ifFalse: [ self error: 'Assignment to array or workspace failed, check that the destination is a structured type'].
(gXUsdjUgw isKindOf: GXArray) | (gXUsdjUgw isKindOf: GXWorkspace)
    ifTrue:  [ ((gXUsdjUgw) at:#gXbottom put: 5.000000000000000e-02 deepCopy) ]
    ifFalse: [ self error: 'Assignment to array or workspace failed, check that the destination is a structured type'].
(gXUsdjUgw isKindOf: GXArray) | (gXUsdjUgw isKindOf: GXWorkspace)
    ifTrue:  [ ((gXUsdjUgw) at:#gXlinewidth put: 1.000000000000000e000 deepCopy) ]
    ifFalse: [ self error: 'Assignment to array or workspace failed, check that the destination is a structured type'].
(gXUsdjUgw isKindOf: GXArray) | (gXUsdjUgw isKindOf: GXWorkspace)
    ifTrue:  [ ((gXUsdjUgw) at:#gXtextht put: 1.500000000000000e000 deepCopy) ]
    ifFalse: [ self error: 'Assignment to array or workspace failed, check that the destination is a structured type'].
(gXUsdjUgw isKindOf: GXArray) | (gXUsdjUgw isKindOf: GXWorkspace)
    ifTrue:  [ ((gXUsdjUgw) at:#gXfont put: (gXDnormal) deepCopy) ]
    ifFalse: [ self error: 'Assignment to array or workspace failed, check that the destination is a structured type'].
(gXUsdjUgw isKindOf: GXArray) | (gXUsdjUgw isKindOf: GXWorkspace)
    ifTrue:  [ ((gXUsdjUgw) at:#gXsymbol put: (gXDplus) deepCopy) ]
    ifFalse: [ self error: 'Assignment to array or workspace failed, check that the destination is a structured type'].
(gXUsdjUgw isKindOf: GXArray) | (gXUsdjUgw isKindOf: GXWorkspace)
    ifTrue:  [ ((gXUsdjUgw) at:#gXlinetype put: (gXDfull) deepCopy) ]
    ifFalse: [ self error: 'Assignment to array or workspace failed, check that the destination is a structured type'].
(gXUsdjUgw isKindOf: GXArray) | (gXUsdjUgw isKindOf: GXWorkspace)
    ifTrue:  [ ((gXUsdjUgw) at:#gXheader put: (gXDtrue) deepCopy) ]
    ifFalse: [ self error: 'Assignment to array or workspace failed, check that the destination is a structured type'].
(gXUsdjUgw isKindOf: GXArray) | (gXUsdjUgw isKindOf: GXWorkspace)
    ifTrue:  [ ((gXUsdjUgw) at:#gXbin put: (gXDfalse) deepCopy) ]
    ifFalse: [ self error: 'Assignment to array or workspace failed, check that the destination is a structured type'].
(gXUsdjUgw isKindOf: GXArray) | (gXUsdjUgw isKindOf: GXWorkspace)
    ifTrue:  [ ((gXUsdjUgw) at:#gXtextcolour put: (gXDwhite) deepCopy) ]
    ifFalse: [ self error: 'Assignment to array or workspace failed, check that the destination is a structured type'].
(gXUsdjUgw isKindOf: GXArray) | (gXUsdjUgw isKindOf: GXWorkspace)
    ifTrue:  [ ((gXUsdjUgw) at:#gXlinecolour put: (gXDwhite) deepCopy) ]
    ifFalse: [ self error: 'Assignment to array or workspace failed, check that the destination is a structured type'].
(gXUsdjUgw isKindOf: GXArray) | (gXUsdjUgw isKindOf: GXWorkspace)
    ifTrue:  [ ((gXUsdjUgw) at:#gXplotcolour put: (gXDwhite) deepCopy) ]
    ifFalse: [ self error: 'Assignment to array or workspace failed, check that the destination is a structured type'].
(gXUsdjUgw isKindOf: GXArray) | (gXUsdjUgw isKindOf: GXWorkspace)
    ifTrue:  [ ((gXUsdjUgw) at:#gXxminUlimit put: (gXU) deepCopy) ]
    ifFalse: [ self error: 'Assignment to array or workspace failed, check that the destination is a structured type'].
(gXUsdjUgw isKindOf: GXArray) | (gXUsdjUgw isKindOf: GXWorkspace)
    ifTrue:  [ ((gXUsdjUgw) at:#gXxmaxUlimit put: (gXU) deepCopy) ]
    ifFalse: [ self error: 'Assignment to array or workspace failed, check that the destination is a structured type'].
(gXUsdjUgw isKindOf: GXArray) | (gXUsdjUgw isKindOf: GXWorkspace)
    ifTrue:  [ ((gXUsdjUgw) at:#gXyminUlimit put: (gXU) deepCopy) ]
    ifFalse: [ self error: 'Assignment to array or workspace failed, check that the destination is a structured type'].
(gXUsdjUgw isKindOf: GXArray) | (gXUsdjUgw isKindOf: GXWorkspace)
    ifTrue:  [ ((gXUsdjUgw) at:#gXymaxUlimit put: (gXU) deepCopy) ]
    ifFalse: [ self error: 'Assignment to array or workspace failed, check that the destination is a structured type'].
(gXUsdjUgw isKindOf: GXArray) | (gXUsdjUgw isKindOf: GXWorkspace)
    ifTrue:  [ ((gXUsdjUgw) at:#gXxrounding put: (gXDtrue) deepCopy) ]
    ifFalse: [ self error: 'Assignment to array or workspace failed, check that the destination is a structured type'].
(gXUsdjUgw isKindOf: GXArray) | (gXUsdjUgw isKindOf: GXWorkspace)
    ifTrue:  [ ((gXUsdjUgw) at:#gXyrounding put: (gXDtrue) deepCopy) ]
    ifFalse: [ self error: 'Assignment to array or workspace failed, check that the destination is a structured type'].
self up.
^nil
!!
nil! "(fix for smalltalk bug, this line is ignored if there is a variable declaration error)"

"End of procedure"

GXcomment run line: 0 proc: self method: false; p1:'__initgooglyworkspace2'; p2:'Sets up the Open Genie workspace that contain graphics defaults and status information'; condEvaluate!
"Procedure class definition:
 ==========================="
((Smalltalk at: #GXUUinitgraphics ifAbsent: [ nil ]) isKindOf: Function class) ifTrue: [ ('Warning: Overriding previously defined procedure ', 'GXUUinitgraphics' gtype) prinl ].
Smalltalk at: #GXUUinitgraphics put: nil!
Function subclass: 'GXUUinitgraphics'
       instanceVariableNames: ''
       classVariableNames: ''
       poolDictionaries: ''
       category: 'Genie Procedure'
!

"BODY
 ----"

!GXUUinitgraphics methodsFor: 'running'!

evaluate
GXUUinitgooglyworkspace1 run line: 0 proc: self method: false; condEvaluate.
GXUUinitgooglyworkspace2 run line: 0 proc: self method: false; condEvaluate.
GXcallout run line: 0 proc: self method: false; gXfacility:'objGraphics'; gXfunction:1; condEvaluate.
self up.
^nil
!!
nil! "(fix for smalltalk bug, this line is ignored if there is a variable declaration error)"

"End of procedure"

GXcomment run line: 0 proc: self method: false; p1:'__initgraphics'; p2:'Initialise the Open Genie graphics package'; condEvaluate!
"Procedure class definition:
 ==========================="
((Smalltalk at: #GXwindow ifAbsent: [ nil ]) isKindOf: Function class) ifTrue: [ ('Warning: Overriding previously defined procedure ', 'GXwindow' gtype) prinl ].
Smalltalk at: #GXwindow put: nil!
Function subclass: 'GXwindow'
       instanceVariableNames: ''
       classVariableNames: ''
       poolDictionaries: ''
       category: 'Genie Procedure'
!

"HEADER
 ------"

"qualifiers
 ----------"

GXwindow

addInstVarName: #gXdelete;
compile:
'gXdelete
	gXdelete _ true';
compile:
'gXdelete: aBoolean
	gXdelete _ aBoolean';

addInstVarName: #gXautoscaled;
compile:
'gXautoscaled
	gXautoscaled _ true';
compile:
'gXautoscaled: aBoolean
	gXautoscaled _ aBoolean';

addInstVarName: #gXscaled;
compile:
'gXscaled
	gXscaled _ true';
compile:
'gXscaled: aBoolean
	gXscaled _ aBoolean';

addInstVarName: #gXunscaled;
compile:
'gXunscaled
	gXunscaled _ true';
compile:
'gXunscaled: aBoolean
	gXunscaled _ aBoolean';

addInstVarName: #gXmultiplot;
compile:
'gXmultiplot
	gXmultiplot _ true';
compile:
'gXmultiplot: aBoolean
	gXmultiplot _ aBoolean';

addInstVarName: #gXtwod;
compile:
'gXtwod
	gXtwod _ true';
compile:
'gXtwod: aBoolean
	gXtwod _ aBoolean'!

"parameters
 ----------"

GXwindow

addInstVarName: #gXp1;
compile:
'gXp1: aVar
	aVar notNil ifTrue:[ gXp1 _ aVar ]';
compile:
'p1: aVar
	aVar notNil ifTrue:[ gXp1 _ aVar ]';

addInstVarName: #gXp2;
compile:
'gXp2: aVar
	aVar notNil ifTrue:[ gXp2 _ aVar ]';
compile:
'p2: aVar
	aVar notNil ifTrue:[ gXp2 _ aVar ]';

addInstVarName: #gXp3;
compile:
'gXp3: aVar
	aVar notNil ifTrue:[ gXp3 _ aVar ]';
compile:
'p3: aVar
	aVar notNil ifTrue:[ gXp3 _ aVar ]';

addInstVarName: #gXp4;
compile:
'gXp4: aVar
	aVar notNil ifTrue:[ gXp4 _ aVar ]';
compile:
'p4: aVar
	aVar notNil ifTrue:[ gXp4 _ aVar ]';

addInstVarName: #gXp5;
compile:
'gXp5: aVar
	aVar notNil ifTrue:[ gXp5 _ aVar ]';
compile:
'p5: aVar
	aVar notNil ifTrue:[ gXp5 _ aVar ]';

addInstVarName: #gXp6;
compile:
'gXp6: aVar
	aVar notNil ifTrue:[ gXp6 _ aVar ]';
compile:
'p6: aVar
	aVar notNil ifTrue:[ gXp6 _ aVar ]';

addInstVarName: #gXp7;
compile:
'gXp7: aVar
	aVar notNil ifTrue:[ gXp7 _ aVar ]';
compile:
'p7: aVar
	aVar notNil ifTrue:[ gXp7 _ aVar ]';

addInstVarName: #gXp8;
compile:
'gXp8: aVar
	aVar notNil ifTrue:[ gXp8 _ aVar ]';
compile:
'p8: aVar
	aVar notNil ifTrue:[ gXp8 _ aVar ]';

addInstVarName: #gXp9;
compile:
'gXp9: aVar
	aVar notNil ifTrue:[ gXp9 _ aVar ]';
compile:
'p9: aVar
	aVar notNil ifTrue:[ gXp9 _ aVar ]';

addInstVarName: #gXp10;
compile:
'gXp10: aVar
	aVar notNil ifTrue:[ gXp10 _ aVar ]';
compile:
'p10: aVar
	aVar notNil ifTrue:[ gXp10 _ aVar ]';

addInstVarName: #gXp11;
compile:
'gXp11: aVar
	aVar notNil ifTrue:[ gXp11 _ aVar ]';
compile:
'p11: aVar
	aVar notNil ifTrue:[ gXp11 _ aVar ]';

addInstVarName: #gXp12;
compile:
'gXp12: aVar
	aVar notNil ifTrue:[ gXp12 _ aVar ]';
compile:
'p12: aVar
	aVar notNil ifTrue:[ gXp12 _ aVar ]';

addInstVarName: #gXp13;
compile:
'gXp13: aVar
	aVar notNil ifTrue:[ gXp13 _ aVar ]';
compile:
'p13: aVar
	aVar notNil ifTrue:[ gXp13 _ aVar ]';

addInstVarName: #gXp14;
compile:
'gXp14: aVar
	aVar notNil ifTrue:[ gXp14 _ aVar ]';
compile:
'p14: aVar
	aVar notNil ifTrue:[ gXp14 _ aVar ]';

addInstVarName: #gXdevUxmin;
compile:
'gXdevUxmin: aVar
	aVar notNil ifTrue:[ gXdevUxmin _ aVar ]';
compile:
'p15: aVar
	aVar notNil ifTrue:[ gXdevUxmin _ aVar ]';

addInstVarName: #gXdevUxmax;
compile:
'gXdevUxmax: aVar
	aVar notNil ifTrue:[ gXdevUxmax _ aVar ]';
compile:
'p16: aVar
	aVar notNil ifTrue:[ gXdevUxmax _ aVar ]';

addInstVarName: #gXdevUymin;
compile:
'gXdevUymin: aVar
	aVar notNil ifTrue:[ gXdevUymin _ aVar ]';
compile:
'p17: aVar
	aVar notNil ifTrue:[ gXdevUymin _ aVar ]';

addInstVarName: #gXdevUymax;
compile:
'gXdevUymax: aVar
	aVar notNil ifTrue:[ gXdevUymax _ aVar ]';
compile:
'p18: aVar
	aVar notNil ifTrue:[ gXdevUymax _ aVar ]';

addInstVarName: #gXxdata;
compile:
'gXxdata: aVar
	aVar notNil ifTrue:[gXxdata _ self typeCheck: aVar shouldBe: GXRealarray paramName: ''gXxdata'']';
compile:
'p19: aVar
	aVar notNil ifTrue:[gXxdata _ self typeCheck: aVar shouldBe: GXRealarray paramName: ''gXxdata'']';

addInstVarName: #gXydata;
compile:
'gXydata: aVar
	aVar notNil ifTrue:[gXydata _ self typeCheck: aVar shouldBe: GXRealarray paramName: ''gXydata'']';
compile:
'p20: aVar
	aVar notNil ifTrue:[gXydata _ self typeCheck: aVar shouldBe: GXRealarray paramName: ''gXydata'']';

addInstVarName: #gXedata;
compile:
'gXedata: aVar
	aVar notNil ifTrue:[gXedata _ self typeCheck: aVar shouldBe: GXRealarray paramName: ''gXedata'']';
compile:
'p21: aVar
	aVar notNil ifTrue:[gXedata _ self typeCheck: aVar shouldBe: GXRealarray paramName: ''gXedata'']';

addInstVarName: #gXcolour;
compile:
'gXcolour: aVar
	aVar notNil ifTrue:[gXcolour _ self typeCheck: aVar shouldBe: GXColour paramName: ''gXcolour'']';
compile:
'p22: aVar
	aVar notNil ifTrue:[gXcolour _ self typeCheck: aVar shouldBe: GXColour paramName: ''gXcolour'']';

addInstVarName: #gXworldUxmin;
compile:
'gXworldUxmin: aVar
	aVar notNil ifTrue:[ gXworldUxmin _ aVar ]';
compile:
'p23: aVar
	aVar notNil ifTrue:[ gXworldUxmin _ aVar ]';

addInstVarName: #gXworldUxmax;
compile:
'gXworldUxmax: aVar
	aVar notNil ifTrue:[ gXworldUxmax _ aVar ]';
compile:
'p24: aVar
	aVar notNil ifTrue:[ gXworldUxmax _ aVar ]';

addInstVarName: #gXworldUymin;
compile:
'gXworldUymin: aVar
	aVar notNil ifTrue:[ gXworldUymin _ aVar ]';
compile:
'p25: aVar
	aVar notNil ifTrue:[ gXworldUymin _ aVar ]';

addInstVarName: #gXworldUymax;
compile:
'gXworldUymax: aVar
	aVar notNil ifTrue:[ gXworldUymax _ aVar ]';
compile:
'p26: aVar
	aVar notNil ifTrue:[ gXworldUymax _ aVar ]';

addInstVarName: #gXminmode;
compile:
'gXminmode: aVar
	aVar notNil ifTrue:[gXminmode _ self typeCheck: aVar shouldBe: GXInteger paramName: ''gXminmode'']';
compile:
'p27: aVar
	aVar notNil ifTrue:[gXminmode _ self typeCheck: aVar shouldBe: GXInteger paramName: ''gXminmode'']';

addInstVarName: #gXmaxmode;
compile:
'gXmaxmode: aVar
	aVar notNil ifTrue:[gXmaxmode _ self typeCheck: aVar shouldBe: GXInteger paramName: ''gXmaxmode'']';
compile:
'p28: aVar
	aVar notNil ifTrue:[gXmaxmode _ self typeCheck: aVar shouldBe: GXInteger paramName: ''gXmaxmode'']';

addInstVarName: #gXnumber;
compile:
'gXnumber: aVar
	aVar notNil ifTrue:[gXnumber _ self typeCheck: aVar shouldBe: GXInteger paramName: ''gXnumber'']';
compile:
'p29: aVar
	aVar notNil ifTrue:[gXnumber _ self typeCheck: aVar shouldBe: GXInteger paramName: ''gXnumber'']'
!

"Global variables
 ----------------"

GXwindow

addPoolVar: #gXUUstatus!

"Local variables
 ---------------"

GXwindow

addInstVarName: #gXdevUxmin;

addInstVarName: #gXdevUxmax;

addInstVarName: #gXdevUymin;

addInstVarName: #gXdevUymax
!

!GXwindow methodsFor: 'initializing'!

initVars
	gXdelete _ false.
	gXautoscaled _ false.
	gXscaled _ false.
	gXunscaled _ false.
	gXmultiplot _ false.
	gXtwod _ false.
	gXdevUxmin _ 0.000000000000000e000.
	gXdevUxmax _ 1.000000000000000e000.
	gXdevUymin _ 0.000000000000000e000.
	gXdevUymax _ 1.000000000000000e000
!!

"BODY
 ----"

!GXwindow methodsFor: 'running'!

evaluate
((gXUUstatus) = 0) ifTrue: [ 
[nil.
self up.
^nil] value ].
(gXdelete) ifTrue: [ 
 (gXp1 isKindOf: Integer) ifTrue: [ gXnumber _ gXp1 ] .
GXdelete run line: 0 proc: self method: false; p1:(GXwin run line: 0 proc: self method: true; p1:0; p2:((gXnumber) + 1); condEvaluate); condEvaluate.
(GXinteractive run line: 0 proc: self method: true; condEvaluate) ifTrue: [ 
GXprint run line: 0 proc: self method: false; p1:''; condEvaluate ].
GXgUerrorUhandler run line: 0 proc: self method: false; condEvaluate.
[nil.
self up.
^nil] value ].

	( gXp1 isKindOf: Integer ) ifTrue: [ gXp1 _ gXp1 * 1.0 ].
	( gXp2 isKindOf: Integer ) ifTrue: [ gXp2 _ gXp2 * 1.0 ].
	( gXp3 isKindOf: Integer ) ifTrue: [ gXp3 _ gXp3 * 1.0 ].
	( gXp4 isKindOf: Integer ) ifTrue: [ gXp4 _ gXp4 * 1.0 ].
	( gXp1 isKindOf: Float ) ifTrue: [ gXdevUxmin _ gXp1 ].
	( gXp2 isKindOf: Float ) ifTrue: [ gXdevUxmax _ gXp2 ].
	( gXp3 isKindOf: Float ) ifTrue: [ gXdevUymin _ gXp3 ].
	( gXp4 isKindOf: Float ) ifTrue: [ gXdevUymax _ gXp4 ]
.
(gXunscaled) ifTrue: [ 
 ( gXp5 isKindOf: Symbol ) ifTrue: [ gXcolour _ gXp5 ] .
GXwinUunscaled run line: 0 proc: self method: false; p1:(gXdevUxmin); p2:(gXdevUxmax); p3:(gXdevUymin); p4:(gXdevUymax); p5:(gXcolour); condEvaluate.
(GXinteractive run line: 0 proc: self method: true; condEvaluate) ifTrue: [ 
GXprint run line: 0 proc: self method: false; p1:''; condEvaluate ].
GXgUerrorUhandler run line: 0 proc: self method: false; condEvaluate.
[nil.
self up.
^nil] value ].

	gXdevUxmin isNil ifTrue: [^'WINDOW: Parameter DEV_XMIN not specified' prenl ]. 
	gXdevUxmax isNil ifTrue: [^'WINDOW: Parameter DEV_XMAX not specified' prenl ]. 
	gXdevUymin isNil ifTrue: [^'WINDOW: Parameter DEV_YMIN not specified' prenl ]. 
	gXdevUymax isNil ifTrue: [^'WINDOW: Parameter DEV_YMAX not specified' prenl ]
.
(gXscaled) ifTrue: [ 

	    ( gXp5 isKindOf: Integer ) ifTrue: [ gXp5 _ gXp5 * 1.0 ].
	    ( gXp6 isKindOf: Integer ) ifTrue: [ gXp6 _ gXp6 * 1.0 ].
	    ( gXp7 isKindOf: Integer ) ifTrue: [ gXp7 _ gXp7 * 1.0 ].
	    ( gXp8 isKindOf: Integer ) ifTrue: [ gXp8 _ gXp8 * 1.0 ].
	    ( gXp5 isKindOf: Float ) ifTrue: [ gXworldUxmin _ gXp5 ].
	    ( gXp6 isKindOf: Float ) ifTrue: [ gXworldUxmax _ gXp6 ].
	    ( gXp7 isKindOf: Float ) ifTrue: [ gXworldUymin _ gXp7 ].
	    ( gXp8 isKindOf: Float ) ifTrue: [ gXworldUymax _ gXp8 ].
	    ( gXp9 isKindOf: Symbol ) ifTrue: [ gXcolour _ gXp9 ].
	    gXworldUxmin isNil ifTrue: [^'WINDOW: Parameter WORLD_XMIN not specified' prenl ]. 
	    gXworldUxmax isNil ifTrue: [^'WINDOW: Parameter WORLD_XMAX not specified' prenl ]. 
	    gXworldUymin isNil ifTrue: [^'WINDOW: Parameter WORLD_YMIN not specified' prenl ]. 
	    gXworldUymax isNil ifTrue: [^'WINDOW: Parameter WORLD_YMAX not specified' prenl ]
	.
GXwinUscaled run line: 0 proc: self method: false; p1:(gXdevUxmin); p2:(gXdevUxmax); p3:(gXdevUymin); p4:(gXdevUymax); p5:(gXworldUxmin); p6:(gXworldUxmax); p7:(gXworldUymin); p8:(gXworldUymax); p9:(gXcolour); condEvaluate ] ifFalse: [ (gXautoscaled) ifTrue: [ 

		( gXp5 isKindOf: GXRealarray ) ifTrue: [ gXxdata _ gXp5 ].
		( gXp6 isKindOf: GXRealarray ) ifTrue: [ gXydata _ gXp6 ].
		( gXp7 isKindOf: GXRealarray ) ifTrue: [ gXedata _ gXp7 ].
	        ( gXp8 isKindOf: Symbol ) ifTrue: [ gXcolour _ gXp8 ].
	        ( gXp9 isKindOf: Integer ) ifTrue: [ gXminmode _ gXp9 ].
	        ( gXp10 isKindOf: Integer ) ifTrue: [ gXmaxmode _ gXp10 ].
	        ( gXp11 isKindOf: Float ) ifTrue: [ gXworldUxmin _ gXp11 ].
	        ( gXp12 isKindOf: Float ) ifTrue: [ gXworldUxmax _ gXp12 ].
	        ( gXp13 isKindOf: Float ) ifTrue: [ gXworldUymin _ gXp13 ].
	        ( gXp14 isKindOf: Float ) ifTrue: [ gXworldUymax _ gXp14 ].
		gXxdata isNil ifTrue: [^'WINDOW: Parameter XDATA not specified' prenl ].
		gXydata isNil ifTrue: [^'WINDOW: Parameter YDATA not specified' prenl ]
	.
GXwinUautoscaled run line: 0 proc: self method: false; p1:(gXdevUxmin); p2:(gXdevUxmax); p3:(gXdevUymin); p4:(gXdevUymax); p5:(gXxdata); p6:(gXydata); p7:(gXedata); p8:(gXcolour); p9:(gXminmode); p10:(gXmaxmode); p11:(gXworldUxmin); p12:(gXworldUxmax); p13:(gXworldUymin); p14:(gXworldUymax); condEvaluate ] ifFalse: [ (gXmultiplot) ifTrue: [ 

	        ( gXp5 isKindOf: Symbol ) ifTrue: [ gXcolour _ gXp5 ].
	        ( gXp6 isKindOf: Integer ) ifTrue: [ gXminmode _ gXp6 ].
	        ( gXp7 isKindOf: Integer ) ifTrue: [ gXmaxmode _ gXp7 ]
	.
GXwinUmultiplot run line: 0 proc: self method: false; p1:(gXdevUxmin); p2:(gXdevUxmax); p3:(gXdevUymin); p4:(gXdevUymax); p5:(gXU); p6:(gXU); p7:(gXU); p8:(gXU); p9:(gXminmode); p10:(gXmaxmode); p11:(gXcolour); condEvaluate ] ifFalse: [ (gXtwod) ifTrue: [ 

		( gXp5 isKindOf: GXRealarray ) ifTrue: [ gXxdata _ gXp5 ].
		( gXp6 isKindOf: GXRealarray ) ifTrue: [ gXydata _ gXp6 ].
	        ( gXp7 isKindOf: Symbol ) ifTrue: [ gXcolour _ gXp7 ].
		gXxdata isNil ifTrue: [^'WINDOW: Parameter XDATA not specified' prenl ].
		gXydata isNil ifTrue: [^'WINDOW: Parameter YDATA not specified' prenl ]
	.
GXwinUtwod run line: 0 proc: self method: false; p1:(gXdevUxmin); p2:(gXdevUxmax); p3:(gXdevUymin); p4:(gXdevUymax); p5:(gXxdata); p6:(gXydata); p7:(gXcolour); condEvaluate ] ifFalse: [ 
GXprinten run line: 0 proc: self method: false; p1:'  ERROR - No window type specified'; condEvaluate ] ]]].
(GXinteractive run line: 0 proc: self method: true; condEvaluate) ifTrue: [ 
GXprint run line: 0 proc: self method: false; p1:''; condEvaluate ].
GXgUerrorUhandler run line: 0 proc: self method: false; condEvaluate.
self up.
^nil
!!
nil! "(fix for smalltalk bug, this line is ignored if there is a variable declaration error)"

"End of procedure"

GXcomment run line: 0 proc: self method: false; p1:'window'; p2:'Control and create windows for plotting'; condEvaluate!
"Procedure class definition:
 ==========================="
((Smalltalk at: #GXgetUcursor ifAbsent: [ nil ]) isKindOf: Function class) ifTrue: [ ('Warning: Overriding previously defined procedure ', 'GXgetUcursor' gtype) prinl ].
Smalltalk at: #GXgetUcursor put: nil!
Function subclass: 'GXgetUcursor'
       instanceVariableNames: ''
       classVariableNames: ''
       poolDictionaries: ''
       category: 'Genie Procedure'
!

"HEADER
 ------"

"qualifiers
 ----------"

GXgetUcursor

addInstVarName: #gXpicture;
compile:
'gXpicture
	gXpicture _ true';
compile:
'gXpicture: aBoolean
	gXpicture _ aBoolean'!

"parameters
 ----------"

GXgetUcursor

addInstVarName: #gXp1;
compile:
'gXp1: aVar
	aVar notNil ifTrue:[ gXp1 _ aVar ]';
compile:
'p1: aVar
	aVar notNil ifTrue:[ gXp1 _ aVar ]';

addInstVarName: #gXp2;
compile:
'gXp2: aVar
	aVar notNil ifTrue:[ gXp2 _ aVar ]';
compile:
'p2: aVar
	aVar notNil ifTrue:[ gXp2 _ aVar ]';

addInstVarName: #gXp3;
compile:
'gXp3: aVar
	aVar notNil ifTrue:[ gXp3 _ aVar ]';
compile:
'p3: aVar
	aVar notNil ifTrue:[ gXp3 _ aVar ]';

addInstVarName: #gXwindow;
compile:
'gXwindow: aVar
	aVar notNil ifTrue:[gXwindow _ self typeCheck: aVar shouldBe: GXGobject paramName: ''gXwindow'']';
compile:
'p4: aVar
	aVar notNil ifTrue:[gXwindow _ self typeCheck: aVar shouldBe: GXGobject paramName: ''gXwindow'']';

addInstVarName: #gXx;
compile:
'gXx: aVar
	aVar notNil ifTrue:[gXx _ self typeCheck: aVar shouldBe: GXReal paramName: ''gXx'']';
compile:
'p5: aVar
	aVar notNil ifTrue:[gXx _ self typeCheck: aVar shouldBe: GXReal paramName: ''gXx'']';

addInstVarName: #gXy;
compile:
'gXy: aVar
	aVar notNil ifTrue:[gXy _ self typeCheck: aVar shouldBe: GXReal paramName: ''gXy'']';
compile:
'p6: aVar
	aVar notNil ifTrue:[gXy _ self typeCheck: aVar shouldBe: GXReal paramName: ''gXy'']'!

"result
 ------"

GXgetUcursor

addInstVarName: #gXcursor
!

"Global variables
 ----------------"

GXgetUcursor

addPoolVar: #gXUUstatus
!

!GXgetUcursor methodsFor: 'initializing'!

initVars
	gXpicture _ false
!!

"BODY
 ----"

!GXgetUcursor methodsFor: 'running'!

evaluate
((gXUUstatus) = 0)  ifFalse: [ (gXpicture) ifTrue: [ 

	( gXp2 isKindOf: Integer ) ifTrue: [ gXp2 _ gXp2 * 1.0 ].
	( gXp3 isKindOf: Integer ) ifTrue: [ gXp3 _ gXp3 * 1.0 ].
	( gXp1 isKindOf: Integer ) ifTrue: [ gXwindow _ gXp1 ].
	( gXp2 isKindOf: Float ) ifTrue: [ gXx _ gXp2 ].
	( gXp3 isKindOf: Float ) ifTrue: [ gXy _ gXp3 ].
	gXp1 isNil ifTrue: [ ^'GET_CURSOR: parameter 1 not given' prenl ]
.
gXcursor _ (GXgetcursor run line: 0 proc: self method: true; p1:(gXx); p2:(gXy); condEvaluate) deepCopy ] ifFalse: [ 

	( gXp1 isKindOf: Integer ) ifTrue: [ gXp1 _ gXp1 * 1.0 ].
	( gXp2 isKindOf: Integer ) ifTrue: [ gXp2 _ gXp2 * 1.0 ].
	( gXp1 isKindOf: Float ) ifTrue: [ gXx _ gXp1 ].
	( gXp2 isKindOf: Float ) ifTrue: [ gXy _ gXp2 ]
.
gXcursor _ (GXgetcursor run line: 0 proc: self method: true; p1:(gXx); p2:(gXy); condEvaluate) deepCopy ] ].
GXgUerrorUhandler run line: 0 proc: self method: false; condEvaluate.
(GXinteractive run line: 0 proc: self method: true; condEvaluate) ifTrue: [ 
GXprint run line: 0 proc: self method: false; p1:''; condEvaluate ].
self up.
^gXcursor
!!
nil! "(fix for smalltalk bug, this line is ignored if there is a variable declaration error)"

"End of procedure"

GXcomment run line: 0 proc: self method: false; p1:'get_cursor'; p2:'cursor routine for a window or picture'; condEvaluate!
"Procedure class definition:
 ==========================="
((Smalltalk at: #GXUUhardcopyUdevice ifAbsent: [ nil ]) isKindOf: Function class) ifTrue: [ ('Warning: Overriding previously defined procedure ', 'GXUUhardcopyUdevice' gtype) prinl ].
Smalltalk at: #GXUUhardcopyUdevice put: nil!
Function subclass: 'GXUUhardcopyUdevice'
       instanceVariableNames: ''
       classVariableNames: ''
       poolDictionaries: ''
       category: 'Genie Procedure'
!

"HEADER
 ------"

"parameters
 ----------"

GXUUhardcopyUdevice

addInstVarName: #gXdevtype;
compile:
'gXdevtype: aVar
	aVar notNil ifTrue:[gXdevtype _ self typeCheck: aVar shouldBe: GXString paramName: ''gXdevtype'']';
compile:
'p1: aVar
	aVar notNil ifTrue:[gXdevtype _ self typeCheck: aVar shouldBe: GXString paramName: ''gXdevtype'']'!

"result
 ------"

GXUUhardcopyUdevice

addInstVarName: #gXres
!

"Local variables
 ---------------"

GXUUhardcopyUdevice

addInstVarName: #gXdevtype
!

!GXUUhardcopyUdevice methodsFor: 'initializing'!

initVars
	gXdevtype _ ''
!!

"BODY
 ----"

!GXUUhardcopyUdevice methodsFor: 'running'!

evaluate
 gXdevtype _ gXdevtype asUppercase .
gXres _ ((GXlocate run line: 0 proc: self method: true; p1:((((('.BCANON.CANON.CCP.CW6320.EPSON.EXCL.EXCL' + '.GENICOM.GIF.VGIF.HPGL.VHPGL.GVENICOM.HIDMP') + '.HJ.HP7221.IMPRESS.LA50.LJ.LIPS2.VLIPS2.LN03') + '.LVN03.FILE.PPM.VPPM.PS.VPS.CPS.VCPS.PRINTRONI') + '.TFILE.TOSHIBA.TX.VCANON.VBCANON.VERSATEC') + '.VVERSATEC.VIPRESS.WD.VWD.ZETA.'); p2:(('.' + (gXdevtype)) + '.'); condEvaluate) > 0) deepCopy.
self up.
^gXres
!!
nil! "(fix for smalltalk bug, this line is ignored if there is a variable declaration error)"

"End of procedure"

"Procedure class definition:
 ==========================="
((Smalltalk at: #GXhardcopy ifAbsent: [ nil ]) isKindOf: Function class) ifTrue: [ ('Warning: Overriding previously defined procedure ', 'GXhardcopy' gtype) prinl ].
Smalltalk at: #GXhardcopy put: nil!
Function subclass: 'GXhardcopy'
       instanceVariableNames: ''
       classVariableNames: ''
       poolDictionaries: ''
       category: 'Genie Procedure'
!

"HEADER
 ------"

"parameters
 ----------"

GXhardcopy

addInstVarName: #gXfilename;
compile:
'gXfilename: aVar
	aVar notNil ifTrue:[gXfilename _ self typeCheck: aVar shouldBe: GXString paramName: ''gXfilename'']';
compile:
'p1: aVar
	aVar notNil ifTrue:[gXfilename _ self typeCheck: aVar shouldBe: GXString paramName: ''gXfilename'']';

addInstVarName: #gXdevtype;
compile:
'gXdevtype: aVar
	aVar notNil ifTrue:[gXdevtype _ self typeCheck: aVar shouldBe: GXString paramName: ''gXdevtype'']';
compile:
'p2: aVar
	aVar notNil ifTrue:[gXdevtype _ self typeCheck: aVar shouldBe: GXString paramName: ''gXdevtype'']';

addInstVarName: #gXpicture;
compile:
'gXpicture: aVar
	aVar notNil ifTrue:[gXpicture _ self typeCheck: aVar shouldBe: GXGobject paramName: ''gXpicture'']';
compile:
'p3: aVar
	aVar notNil ifTrue:[gXpicture _ self typeCheck: aVar shouldBe: GXGobject paramName: ''gXpicture'']'
!

"Global variables
 ----------------"

GXhardcopy

addPoolVar: #gXUsdjUgw!

"Local variables
 ---------------"

GXhardcopy

addInstVarName: #gXdev;

addInstVarName: #gXpicture;

addInstVarName: #gXfilename
!

!GXhardcopy methodsFor: 'initializing'!

initVars
	gXpicture _ 1.
	gXfilename _ 'genie.ps'
!!

"BODY
 ----"

!GXhardcopy methodsFor: 'running'!

evaluate
(GXUUhardcopyUdevice run line: 0 proc: self method: true; p1:(gXdevtype); condEvaluate) ifTrue: [ 
(gXUsdjUgw isKindOf: GXArray) | (gXUsdjUgw isKindOf: GXWorkspace)
    ifTrue:  [ ((gXUsdjUgw) at:#gXhardcopy put: (gXdevtype) deepCopy) ]
    ifFalse: [ self error: 'Assignment to array or workspace failed, check that the destination is a structured type'] ] ifFalse: [ 
(GXdefined run line: 0 proc: self method: true; p1:(gXdevtype); condEvaluate) ifTrue: [ 
GXprinten run line: 0 proc: self method: false; p1:'HARDCOPY: '; p2:(gXdevtype); p3:' is not a hardcopy device'; condEvaluate.
[nil.
self up.
^nil] value ] ] .
gXdev _ (GXdevice run line: 0 proc: self method: true; gXopen; p1:((gXUsdjUgw) at:#gXhardcopy); p2:1.070000000000000e001; p3:7.600000000000000e000; p4:(gXU); p5:(gXfilename); condEvaluate) deepCopy.
(GXdefined run line: 0 proc: self method: true; p1:(gXdev); condEvaluate) ifTrue: [ 
GXprintin run line: 0 proc: self method: false; p1:'Writing '; p2:((gXUsdjUgw) at:#gXhardcopy); p3:' format output to '; p4:(gXfilename); condEvaluate.
GXdelete run line: 0 proc: self method: false; p1:(GXpic run line: 0 proc: self method: true; p1:0; condEvaluate); condEvaluate.
GXredraw run line: 0 proc: self method: false; p1:(GXpic run line: 0 proc: self method: true; p1:0; condEvaluate); p2:(gXdev); condEvaluate.
GXdevice run line: 0 proc: self method: false; gXclose; condEvaluate ] ifFalse: [ 
GXprinten run line: 0 proc: self method: false; p1:'DEVICE: Error creating output file "'; p2:(gXfilename); p3:'"'; condEvaluate.
GXprinten run line: 0 proc: self method: false; p1:'DEVICE: Check you have write permission to the directory'; condEvaluate ] .
self up.
^nil
!!
nil! "(fix for smalltalk bug, this line is ignored if there is a variable declaration error)"

"End of procedure"

GXcomment run line: 0 proc: self method: false; p1:'hardcopy'; p2:'Produce a file with the desired picture in it'; condEvaluate!
"Procedure class definition:
 ==========================="
((Smalltalk at: #GXbuffer ifAbsent: [ nil ]) isKindOf: Function class) ifTrue: [ ('Warning: Overriding previously defined procedure ', 'GXbuffer' gtype) prinl ].
Smalltalk at: #GXbuffer put: nil!
Function subclass: 'GXbuffer'
       instanceVariableNames: ''
       classVariableNames: ''
       poolDictionaries: ''
       category: 'Genie Procedure'
!

"HEADER
 ------"

"qualifiers
 ----------"

GXbuffer

addInstVarName: #gXstart;
compile:
'gXstart
	gXstart _ true';
compile:
'gXstart: aBoolean
	gXstart _ aBoolean';

addInstVarName: #gXend;
compile:
'gXend
	gXend _ true';
compile:
'gXend: aBoolean
	gXend _ aBoolean'
!

!GXbuffer methodsFor: 'initializing'!

initVars
	gXstart _ false.
	gXend _ false
!!

"End of procedure"

"Procedure class definition:
 ==========================="
((Smalltalk at: #GXUUg2Uopendev ifAbsent: [ nil ]) isKindOf: Function class) ifTrue: [ ('Warning: Overriding previously defined procedure ', 'GXUUg2Uopendev' gtype) prinl ].
Smalltalk at: #GXUUg2Uopendev put: nil!
Function subclass: 'GXUUg2Uopendev'
       instanceVariableNames: ''
       classVariableNames: ''
       poolDictionaries: ''
       category: 'Genie Procedure'
!

"HEADER
 ------"

"result
 ------"

GXUUg2Uopendev

addInstVarName: #gXres
!

"Global variables
 ----------------"

GXUUg2Uopendev

addPoolVar: #gXUUstatus
!

!GXUUg2Uopendev methodsFor: 'initializing'!

initVars
	gXres _ 0
!!

"BODY
 ----"

!GXUUg2Uopendev methodsFor: 'running'!

evaluate
((gXUUstatus) = 0) ifTrue: [ 
GXprintin run line: 0 proc: self method: false; p1:'  Opening graphics device '; p2:((gXUsdjUgw) at:#gXinteractive); condEvaluate.
GXdevice run line: 0 proc: self method: false; gXopen; p1:((gXUsdjUgw) at:#gXinteractive); p2:(gXU); p3:(gXU); p4:((gXUsdjUgw) at:#gXsize); condEvaluate.
GXdevice run line: 0 proc: self method: false; gXclear; condEvaluate.
gXres _ 1 deepCopy ].
self up.
^gXres
!!
nil! "(fix for smalltalk bug, this line is ignored if there is a variable declaration error)"

"End of procedure"

GXcomment run line: 0 proc: self method: false; p1:'__g2_opendev'; p2:'opens graphics device'; condEvaluate!
"Procedure class definition:
 ==========================="
((Smalltalk at: #GXUUbtext ifAbsent: [ nil ]) isKindOf: Function class) ifTrue: [ ('Warning: Overriding previously defined procedure ', 'GXUUbtext' gtype) prinl ].
Smalltalk at: #GXUUbtext put: nil!
Function subclass: 'GXUUbtext'
       instanceVariableNames: ''
       classVariableNames: ''
       poolDictionaries: ''
       category: 'Genie Procedure'
!

"HEADER
 ------"

"parameters
 ----------"

GXUUbtext

addInstVarName: #gXx;
compile:
'gXx: aVar
	aVar notNil ifTrue:[gXx _ self typeCheck: aVar shouldBe: GXReal paramName: ''gXx'']';
compile:
'p1: aVar
	aVar notNil ifTrue:[gXx _ self typeCheck: aVar shouldBe: GXReal paramName: ''gXx'']';

addInstVarName: #gXy;
compile:
'gXy: aVar
	aVar notNil ifTrue:[gXy _ self typeCheck: aVar shouldBe: GXReal paramName: ''gXy'']';
compile:
'p2: aVar
	aVar notNil ifTrue:[gXy _ self typeCheck: aVar shouldBe: GXReal paramName: ''gXy'']';

addInstVarName: #gXtext;
compile:
'gXtext: aVar
	aVar notNil ifTrue:[gXtext _ self typeCheck: aVar shouldBe: GXString paramName: ''gXtext'']';
compile:
'p3: aVar
	aVar notNil ifTrue:[gXtext _ self typeCheck: aVar shouldBe: GXString paramName: ''gXtext'']';

addInstVarName: #gXsize;
compile:
'gXsize: aVar
	aVar notNil ifTrue:[gXsize _ self typeCheck: aVar shouldBe: GXReal paramName: ''gXsize'']';
compile:
'p4: aVar
	aVar notNil ifTrue:[gXsize _ self typeCheck: aVar shouldBe: GXReal paramName: ''gXsize'']'
!

"Local variables
 ---------------"

GXUUbtext

addInstVarName: #gXx!

"Local variables
 ---------------"

GXUUbtext

addInstVarName: #gXy!

"Local variables
 ---------------"

GXUUbtext

addInstVarName: #gXtext!

"Local variables
 ---------------"

GXUUbtext

addInstVarName: #gXtUc!

"Local variables
 ---------------"

GXUUbtext

addInstVarName: #gXtUh!

"Local variables
 ---------------"

GXUUbtext

addInstVarName: #gXlUw!

"Local variables
 ---------------"

GXUUbtext

addInstVarName: #gXsize
!

!GXUUbtext methodsFor: 'initializing'!

initVars
	gXtUc _ ((gXUsdjUgw) at:#gXtextcolour).
	gXlUw _ 1.000000000000000e000.
	gXsize _ 1.000000000000000e000
!!

"BODY
 ----"

!GXUUbtext methodsFor: 'running'!

evaluate
gXtUh _ (((((((gXUsdjUgw) at:#gXright) - ((gXUsdjUgw) at:#gXleft))) + ((((gXUsdjUgw) at:#gXtop) - ((gXUsdjUgw) at:#gXbottom))))) / 2) deepCopy.
gXtUh _ ((gXtUh) * (gXsize)) deepCopy.
((gXtUh) > 1.000000000000000e000) ifTrue: [ 
gXlUw _ (gXtUh) deepCopy ].
GXtext run line: 0 proc: self method: false; gXdraw; p1:(gXx); p2:(gXy); p3:(gXtext); p4:(gXtUc); p5:(gXtUh); p6:0.000000000000000e000; p7:((gXUsdjUgw) at:#gXfont); p8:(gXlUw); condEvaluate.
self up.
^nil
!!
nil! "(fix for smalltalk bug, this line is ignored if there is a variable declaration error)"

"End of procedure"

"Procedure class definition:
 ==========================="
((Smalltalk at: #GXUUg2Ubanner ifAbsent: [ nil ]) isKindOf: Function class) ifTrue: [ ('Warning: Overriding previously defined procedure ', 'GXUUg2Ubanner' gtype) prinl ].
Smalltalk at: #GXUUg2Ubanner put: nil!
Function subclass: 'GXUUg2Ubanner'
       instanceVariableNames: ''
       classVariableNames: ''
       poolDictionaries: ''
       category: 'Genie Procedure'
!

"HEADER
 ------"

"parameters
 ----------"

GXUUg2Ubanner

addInstVarName: #gXinst;
compile:
'gXinst: aVar
	aVar notNil ifTrue:[gXinst _ self typeCheck: aVar shouldBe: GXString paramName: ''gXinst'']';
compile:
'p1: aVar
	aVar notNil ifTrue:[gXinst _ self typeCheck: aVar shouldBe: GXString paramName: ''gXinst'']';

addInstVarName: #gXrun;
compile:
'gXrun: aVar
	aVar notNil ifTrue:[gXrun _ self typeCheck: aVar shouldBe: GXString paramName: ''gXrun'']';
compile:
'p2: aVar
	aVar notNil ifTrue:[gXrun _ self typeCheck: aVar shouldBe: GXString paramName: ''gXrun'']';

addInstVarName: #gXspect;
compile:
'gXspect: aVar
	aVar notNil ifTrue:[gXspect _ self typeCheck: aVar shouldBe: GXString paramName: ''gXspect'']';
compile:
'p3: aVar
	aVar notNil ifTrue:[gXspect _ self typeCheck: aVar shouldBe: GXString paramName: ''gXspect'']';

addInstVarName: #gXfile;
compile:
'gXfile: aVar
	aVar notNil ifTrue:[gXfile _ self typeCheck: aVar shouldBe: GXString paramName: ''gXfile'']';
compile:
'p4: aVar
	aVar notNil ifTrue:[gXfile _ self typeCheck: aVar shouldBe: GXString paramName: ''gXfile'']';

addInstVarName: #gXuser;
compile:
'gXuser: aVar
	aVar notNil ifTrue:[gXuser _ self typeCheck: aVar shouldBe: GXString paramName: ''gXuser'']';
compile:
'p5: aVar
	aVar notNil ifTrue:[gXuser _ self typeCheck: aVar shouldBe: GXString paramName: ''gXuser'']';

addInstVarName: #gXruntime;
compile:
'gXruntime: aVar
	aVar notNil ifTrue:[gXruntime _ self typeCheck: aVar shouldBe: GXString paramName: ''gXruntime'']';
compile:
'p6: aVar
	aVar notNil ifTrue:[gXruntime _ self typeCheck: aVar shouldBe: GXString paramName: ''gXruntime'']';

addInstVarName: #gXgrouping;
compile:
'gXgrouping: aVar
	aVar notNil ifTrue:[gXgrouping _ self typeCheck: aVar shouldBe: GXString paramName: ''gXgrouping'']';
compile:
'p7: aVar
	aVar notNil ifTrue:[gXgrouping _ self typeCheck: aVar shouldBe: GXString paramName: ''gXgrouping'']'
!

"Local variables
 ---------------"

GXUUg2Ubanner

addInstVarName: #gXinst!

"Local variables
 ---------------"

GXUUg2Ubanner

addInstVarName: #gXrun!

"Local variables
 ---------------"

GXUUg2Ubanner

addInstVarName: #gXspect!

"Local variables
 ---------------"

GXUUg2Ubanner

addInstVarName: #gXfile!

"Local variables
 ---------------"

GXUUg2Ubanner

addInstVarName: #gXuser!

"Local variables
 ---------------"

GXUUg2Ubanner

addInstVarName: #gXruntime!

"Local variables
 ---------------"

GXUUg2Ubanner

addInstVarName: #gXgrouping!

"Local variables
 ---------------"

GXUUg2Ubanner

addInstVarName: #gXxmin!

"Local variables
 ---------------"

GXUUg2Ubanner

addInstVarName: #gXxmax!

"Local variables
 ---------------"

GXUUg2Ubanner

addInstVarName: #gXymax!

"Local variables
 ---------------"

GXUUg2Ubanner

addInstVarName: #gXymin!

"Local variables
 ---------------"

GXUUg2Ubanner

addInstVarName: #gXxoff!

"Local variables
 ---------------"

GXUUg2Ubanner

addInstVarName: #gXyoff!

"Local variables
 ---------------"

GXUUg2Ubanner

addInstVarName: #gXtext!

"Local variables
 ---------------"

GXUUg2Ubanner

addInstVarName: #gXlUc!

"Local variables
 ---------------"

GXUUg2Ubanner

addInstVarName: #gXlUw!

"Local variables
 ---------------"

GXUUg2Ubanner

addInstVarName: #gXtUc
!

!GXUUg2Ubanner methodsFor: 'initializing'!

initVars
	gXinst _ ' '.
	gXrun _ ' '.
	gXspect _ ' '.
	gXfile _ ' '.
	gXuser _ ' '.
	gXruntime _ ' '.
	gXgrouping _ ' '.
	gXxmin _ ((gXUsdjUgw) at:#gXleft).
	gXxmax _ ((gXUsdjUgw) at:#gXright).
	gXymax _ ((gXUsdjUgw) at:#gXtop).
	gXymin _ ((gXUsdjUgw) at:#gXbottom).
	gXlUc _ ((gXUsdjUgw) at:#gXlinecolour).
	gXlUw _ ((gXUsdjUgw) at:#gXlinewidth).
	gXtUc _ ((gXUsdjUgw) at:#gXtextcolour)
!!

"BODY
 ----"

!GXUUg2Ubanner methodsFor: 'running'!

evaluate
gXyoff _ (8.000000000000000e-01 * (((gXymax) - (gXymin)))) deepCopy.
gXymin _ ((gXymin) + (gXyoff)) deepCopy.
GXwinUunscaled run line: 0 proc: self method: false; p1:(gXxmin); p2:(gXxmax); p3:(gXymin); p4:(gXymax); condEvaluate.
(GXgUerrorUhandler run line: 0 proc: self method: true; condEvaluate) ifTrue: [ 
[nil.
self up.
^nil] value ].
GXdraw run line: 0 proc: self method: false; gXbox; p1:1.000000000000000e-02; p2:1.000000000000000e-02; p3:9.900000000000000e-01; p4:9.900000000000000e-01; p5:(gXlUc); p6:(gXlUw); condEvaluate.
GXUUbtext run line: 0 proc: self method: false; p1:2.100000000000000e-01; p2:8.000000000000000e-01; p3:('Instrument: ' + (gXinst)); condEvaluate.
GXUUbtext run line: 0 proc: self method: false; p1:2.100000000000000e-01; p2:6.000000000000000e-01; p3:('Run number: ' + (gXrun)); condEvaluate.
GXUUbtext run line: 0 proc: self method: false; p1:2.100000000000000e-01; p2:4.000000000000000e-01; p3:('Spectrum: ' + (gXspect)); condEvaluate.
GXUUbtext run line: 0 proc: self method: false; p1:2.100000000000000e-01; p2:7.000000000000001e-02; p3:('Location: ' + (gXfile)); condEvaluate.
GXUUbtext run line: 0 proc: self method: false; p1:4.900000000000000e-01; p2:8.000000000000000e-01; p3:('User: ' + (gXuser)); condEvaluate.
GXUUbtext run line: 0 proc: self method: false; p1:4.900000000000000e-01; p2:6.100000000000000e-01; p3:('Run start time: ' + (gXruntime)); condEvaluate.
GXUUbtext run line: 0 proc: self method: false; p1:4.900000000000000e-01; p2:4.200000000000000e-01; p3:('Plot date: ' + (GXnow run line: 0 proc: self method: true; condEvaluate)); condEvaluate.
GXUUbtext run line: 0 proc: self method: false; p1:4.900000000000000e-01; p2:2.300000000000000e-01; p3:('Grouping: ' + (gXgrouping)); condEvaluate.
GXdraw run line: 0 proc: self method: false; gXbox; p1:2.000000000000000e-02; p2:1.000000000000000e-01; p3:2.000000000000000e-01; p4:9.000000000000000e-01; p5:(gXlUc); p6:(gXlUw); condEvaluate.
GXUUbtext run line: 0 proc: self method: false; p1:3.000000000000000e-02; p2:3.000000000000000e-01; p3:'ISIS'; p4:3.800000000000000e000; condEvaluate.
self up.
^nil
!!
nil! "(fix for smalltalk bug, this line is ignored if there is a variable declaration error)"

"End of procedure"

GXcomment run line: 0 proc: self method: false; p1:'__g2_banner'; p2:'draws the ISIS banner'; condEvaluate!
"Procedure class definition:
 ==========================="
((Smalltalk at: #GXUUg2Uaxes ifAbsent: [ nil ]) isKindOf: Function class) ifTrue: [ ('Warning: Overriding previously defined procedure ', 'GXUUg2Uaxes' gtype) prinl ].
Smalltalk at: #GXUUg2Uaxes put: nil!
Function subclass: 'GXUUg2Uaxes'
       instanceVariableNames: ''
       classVariableNames: ''
       poolDictionaries: ''
       category: 'Genie Procedure'
!

"HEADER
 ------"

"qualifiers
 ----------"

GXUUg2Uaxes

addInstVarName: #gXredraw;
compile:
'gXredraw
	gXredraw _ true';
compile:
'gXredraw: aBoolean
	gXredraw _ aBoolean'!

"parameters
 ----------"

GXUUg2Uaxes

addInstVarName: #gXtype;
compile:
'gXtype: aVar
	aVar notNil ifTrue:[gXtype _ self typeCheck: aVar shouldBe: GXInteger paramName: ''gXtype'']';
compile:
'p1: aVar
	aVar notNil ifTrue:[gXtype _ self typeCheck: aVar shouldBe: GXInteger paramName: ''gXtype'']';

addInstVarName: #gXpUxarray;
compile:
'gXpUxarray: aVar
	aVar notNil ifTrue:[gXpUxarray _ self typeCheck: aVar shouldBe: GXRealarray paramName: ''gXpUxarray'']';
compile:
'p2: aVar
	aVar notNil ifTrue:[gXpUxarray _ self typeCheck: aVar shouldBe: GXRealarray paramName: ''gXpUxarray'']';

addInstVarName: #gXpUyarray;
compile:
'gXpUyarray: aVar
	aVar notNil ifTrue:[gXpUyarray _ self typeCheck: aVar shouldBe: GXRealarray paramName: ''gXpUyarray'']';
compile:
'p3: aVar
	aVar notNil ifTrue:[gXpUyarray _ self typeCheck: aVar shouldBe: GXRealarray paramName: ''gXpUyarray'']';

addInstVarName: #gXpUoarray;
compile:
'gXpUoarray: aVar
	aVar notNil ifTrue:[gXpUoarray _ self typeCheck: aVar shouldBe: GXRealarray paramName: ''gXpUoarray'']';
compile:
'p4: aVar
	aVar notNil ifTrue:[gXpUoarray _ self typeCheck: aVar shouldBe: GXRealarray paramName: ''gXpUoarray'']'!

"parameters
 ----------"

GXUUg2Uaxes

addInstVarName: #gXwxmin;
compile:
'gXwxmin: aVar
	aVar notNil ifTrue:[gXwxmin _ self typeCheck: aVar shouldBe: GXReal paramName: ''gXwxmin'']';
compile:
'p5: aVar
	aVar notNil ifTrue:[gXwxmin _ self typeCheck: aVar shouldBe: GXReal paramName: ''gXwxmin'']';

addInstVarName: #gXwxmax;
compile:
'gXwxmax: aVar
	aVar notNil ifTrue:[gXwxmax _ self typeCheck: aVar shouldBe: GXReal paramName: ''gXwxmax'']';
compile:
'p6: aVar
	aVar notNil ifTrue:[gXwxmax _ self typeCheck: aVar shouldBe: GXReal paramName: ''gXwxmax'']';

addInstVarName: #gXwymin;
compile:
'gXwymin: aVar
	aVar notNil ifTrue:[gXwymin _ self typeCheck: aVar shouldBe: GXReal paramName: ''gXwymin'']';
compile:
'p7: aVar
	aVar notNil ifTrue:[gXwymin _ self typeCheck: aVar shouldBe: GXReal paramName: ''gXwymin'']';

addInstVarName: #gXwymax;
compile:
'gXwymax: aVar
	aVar notNil ifTrue:[gXwymax _ self typeCheck: aVar shouldBe: GXReal paramName: ''gXwymax'']';
compile:
'p8: aVar
	aVar notNil ifTrue:[gXwymax _ self typeCheck: aVar shouldBe: GXReal paramName: ''gXwymax'']'
!

"Local variables
 ---------------"

GXUUg2Uaxes

addInstVarName: #gXwymin!

"Local variables
 ---------------"

GXUUg2Uaxes

addInstVarName: #gXwymax!

"Local variables
 ---------------"

GXUUg2Uaxes

addInstVarName: #gXwxmin!

"Local variables
 ---------------"

GXUUg2Uaxes

addInstVarName: #gXwxmax!

"Local variables
 ---------------"

GXUUg2Uaxes

addInstVarName: #gXltop!

"Local variables
 ---------------"

GXUUg2Uaxes

addInstVarName: #gXxmin!

"Local variables
 ---------------"

GXUUg2Uaxes

addInstVarName: #gXxmax!

"Local variables
 ---------------"

GXUUg2Uaxes

addInstVarName: #gXymin!

"Local variables
 ---------------"

GXUUg2Uaxes

addInstVarName: #gXymax!

"Local variables
 ---------------"

GXUUg2Uaxes

addInstVarName: #gXxoff!

"Local variables
 ---------------"

GXUUg2Uaxes

addInstVarName: #gXyoff!

"Local variables
 ---------------"

GXUUg2Uaxes

addInstVarName: #gXtype!

"Local variables
 ---------------"

GXUUg2Uaxes

addInstVarName: #gXxarray;

addInstVarName: #gXyarray;

addInstVarName: #gXoarray!

"Local variables
 ---------------"

GXUUg2Uaxes

addInstVarName: #gXlUc!

"Local variables
 ---------------"

GXUUg2Uaxes

addInstVarName: #gXpUc!

"Local variables
 ---------------"

GXUUg2Uaxes

addInstVarName: #gXtUc!

"Local variables
 ---------------"

GXUUg2Uaxes

addInstVarName: #gXtUh!

"Local variables
 ---------------"

GXUUg2Uaxes

addInstVarName: #gXlUw!

"Local variables
 ---------------"

GXUUg2Uaxes

addInstVarName: #gXrymin!

"Local variables
 ---------------"

GXUUg2Uaxes

addInstVarName: #gXrymax
!

!GXUUg2Uaxes methodsFor: 'initializing'!

initVars
	gXredraw _ false.
	gXwymin _ (gXU).
	gXwymax _ (gXU).
	gXwxmin _ (gXU).
	gXwxmax _ (gXU).
	gXtype _ 1.
	gXlUc _ ((gXUsdjUgw) at:#gXlinecolour).
	gXpUc _ ((gXUsdjUgw) at:#gXplotcolour).
	gXtUc _ ((gXUsdjUgw) at:#gXtextcolour).
	gXtUh _ ((gXUsdjUgw) at:#gXtextht).
	gXlUw _ ((gXUsdjUgw) at:#gXlinewidth).
	gXrymin _ 0.
	gXrymax _ 0
!!

"BODY
 ----"

!GXUUg2Uaxes methodsFor: 'running'!

evaluate
((gXUsdjUgw) at:#gXxlog) ifTrue: [ 
 gXxarray _ gXpUxarray log: 10 .
GXfix run line: 0 proc: self method: false; p1:(gXxarray); p2:0.000000000000000e000; condEvaluate ] ifFalse: [ 
 gXxarray _ gXpUxarray  ] .
((gXUsdjUgw) at:#gXylog) ifTrue: [ 
 gXyarray _ gXpUyarray log: 10 .
GXfix run line: 0 proc: self method: false; p1:(gXyarray); p2:0.000000000000000e000; condEvaluate.
gXoarray _ (((gXpUoarray) / (gXpUyarray)) / (GXln run line: 0 proc: self method: true; p1:1.000000000000000e001; condEvaluate)) deepCopy.
GXfix run line: 0 proc: self method: false; p1:(gXoarray); p2:0.000000000000000e000; condEvaluate ] ifFalse: [ 
 
	gXyarray _ gXpUyarray.
   	gXoarray _ gXpUoarray
     ] .
((gXUsdjUgw) at:#gXheader) ifTrue: [ 
gXltop _ 8.000000000000000e-01 deepCopy ] ifFalse: [ 
gXltop _ 1.000000000000000e000 deepCopy ] .
gXxmin _ ((gXUsdjUgw) at:#gXleft) deepCopy.
gXxmax _ ((gXUsdjUgw) at:#gXright) deepCopy.
gXymin _ ((gXUsdjUgw) at:#gXbottom) deepCopy.
gXymax _ (((gXUsdjUgw) at:#gXtop) * (gXltop)) deepCopy.
gXxoff _ (1.200000000000000e-01 * (((gXxmax) - (gXxmin)))) deepCopy.
gXyoff _ (1.100000000000000e-01 * (((gXymax) - (gXymin)))) deepCopy.
gXxmin _ ((gXxmin) + (gXxoff)) deepCopy.
gXxmax _ ((gXxmax) - (gXxoff)) deepCopy.
gXymin _ ((gXymin) + (gXyoff)) deepCopy.
gXymax _ ((gXymax) - (gXyoff)) deepCopy.
((gXUsdjUgw) at:#gXyrounding) ifTrue: [ 
gXrymin _ 4 deepCopy.
gXrymax _ 2 deepCopy ].

		gXwxmin isNil ifTrue: [ gXwxmin _ gXUsdjUgw at: #gXxminUlimit ].
		gXwxmax isNil ifTrue: [ gXwxmax _ gXUsdjUgw at: #gXxmaxUlimit ].
		gXwymin isNil ifTrue: [ gXwymin _ gXUsdjUgw at: #gXyminUlimit ].
		gXwymax isNil ifTrue: [ gXwymax _ gXUsdjUgw at: #gXymaxUlimit ]
	    .
((gXtype) = 5) ifTrue: [ 
GXwinUmultiplot run line: 0 proc: self method: false; p1:(gXxmin); p2:(gXxmax); p3:(gXymin); p4:(gXymax); p5:(gXwxmin); p6:(gXwxmax); p7:(gXwymin); p8:(gXwymax); p9:(gXrymin); p10:(gXrymax); p11:(gXUbackground); condEvaluate ] ifFalse: [ 
GXwinUautoscaled run line: 0 proc: self method: false; p1:(gXxmin); p2:(gXxmax); p3:(gXymin); p4:(gXymax); p5:(gXxarray); p6:(gXyarray); p7:(gXoarray); p8:(gXUbackground); p9:(gXrymin); p10:(gXrymax); p11:(gXwxmin); p12:(gXwxmax); p13:(gXwymin); p14:(gXwymax); condEvaluate ] .
(gXredraw) not ifTrue: [ 
 
		GXaxes run line: 0 proc: self method: false; gXdraw;
			gXxlog: (gXUsdjUgw at: #gXxlog);
			gXylog: (gXUsdjUgw at: #gXylog);
			p1: gXlUc; p2: gXtUh; p3: (gXUsdjUgw at: #gXfont);
			p4: gXlUw; condEvaluate
	     ].
self up.
^nil
!!
nil! "(fix for smalltalk bug, this line is ignored if there is a variable declaration error)"

"End of procedure"

GXcomment run line: 0 proc: self method: false; p1:'__g2_axes'; p2:'draws the ISIS axes template'; condEvaluate!
"Procedure class definition:
 ==========================="
((Smalltalk at: #GXUUg2Udrawplot ifAbsent: [ nil ]) isKindOf: Function class) ifTrue: [ ('Warning: Overriding previously defined procedure ', 'GXUUg2Udrawplot' gtype) prinl ].
Smalltalk at: #GXUUg2Udrawplot put: nil!
Function subclass: 'GXUUg2Udrawplot'
       instanceVariableNames: ''
       classVariableNames: ''
       poolDictionaries: ''
       category: 'Genie Procedure'
!

"HEADER
 ------"

"parameters
 ----------"

GXUUg2Udrawplot

addInstVarName: #gXtype;
compile:
'gXtype: aVar
	aVar notNil ifTrue:[gXtype _ self typeCheck: aVar shouldBe: GXInteger paramName: ''gXtype'']';
compile:
'p1: aVar
	aVar notNil ifTrue:[gXtype _ self typeCheck: aVar shouldBe: GXInteger paramName: ''gXtype'']';

addInstVarName: #gXpUxarray;
compile:
'gXpUxarray: aVar
	aVar notNil ifTrue:[gXpUxarray _ self typeCheck: aVar shouldBe: GXRealarray paramName: ''gXpUxarray'']';
compile:
'p2: aVar
	aVar notNil ifTrue:[gXpUxarray _ self typeCheck: aVar shouldBe: GXRealarray paramName: ''gXpUxarray'']';

addInstVarName: #gXpUyarray;
compile:
'gXpUyarray: aVar
	aVar notNil ifTrue:[gXpUyarray _ self typeCheck: aVar shouldBe: GXRealarray paramName: ''gXpUyarray'']';
compile:
'p3: aVar
	aVar notNil ifTrue:[gXpUyarray _ self typeCheck: aVar shouldBe: GXRealarray paramName: ''gXpUyarray'']';

addInstVarName: #gXpUoarray;
compile:
'gXpUoarray: aVar
	aVar notNil ifTrue:[gXpUoarray _ self typeCheck: aVar shouldBe: GXRealarray paramName: ''gXpUoarray'']';
compile:
'p4: aVar
	aVar notNil ifTrue:[gXpUoarray _ self typeCheck: aVar shouldBe: GXRealarray paramName: ''gXpUoarray'']';

addInstVarName: #gXpUc;
compile:
'gXpUc: aVar
	aVar notNil ifTrue:[gXpUc _ self typeCheck: aVar shouldBe: GXColour paramName: ''gXpUc'']';
compile:
'p5: aVar
	aVar notNil ifTrue:[gXpUc _ self typeCheck: aVar shouldBe: GXColour paramName: ''gXpUc'']';

addInstVarName: #gXlUw;
compile:
'gXlUw: aVar
	aVar notNil ifTrue:[gXlUw _ self typeCheck: aVar shouldBe: GXReal paramName: ''gXlUw'']';
compile:
'p6: aVar
	aVar notNil ifTrue:[gXlUw _ self typeCheck: aVar shouldBe: GXReal paramName: ''gXlUw'']';

addInstVarName: #gXtUc;
compile:
'gXtUc: aVar
	aVar notNil ifTrue:[gXtUc _ self typeCheck: aVar shouldBe: GXColour paramName: ''gXtUc'']';
compile:
'p7: aVar
	aVar notNil ifTrue:[gXtUc _ self typeCheck: aVar shouldBe: GXColour paramName: ''gXtUc'']';

addInstVarName: #gXtUh;
compile:
'gXtUh: aVar
	aVar notNil ifTrue:[gXtUh _ self typeCheck: aVar shouldBe: GXReal paramName: ''gXtUh'']';
compile:
'p8: aVar
	aVar notNil ifTrue:[gXtUh _ self typeCheck: aVar shouldBe: GXReal paramName: ''gXtUh'']'
!

"Local variables
 ---------------"

GXUUg2Udrawplot

addInstVarName: #gXtype!

"Local variables
 ---------------"

GXUUg2Udrawplot

addInstVarName: #gXpUc!

"Local variables
 ---------------"

GXUUg2Udrawplot

addInstVarName: #gXlUw!

"Local variables
 ---------------"

GXUUg2Udrawplot

addInstVarName: #gXtUc!

"Local variables
 ---------------"

GXUUg2Udrawplot

addInstVarName: #gXtUh!

"Local variables
 ---------------"

GXUUg2Udrawplot

addInstVarName: #gXxarray;

addInstVarName: #gXyarray;

addInstVarName: #gXoarray;

addInstVarName: #gXcentredUx
!

!GXUUg2Udrawplot methodsFor: 'initializing'!

initVars
	gXtype _ 1.
	gXpUc _ ((gXUsdjUgw) at:#gXplotcolour).
	gXlUw _ ((gXUsdjUgw) at:#gXlinewidth).
	gXtUc _ (gXUforeground).
	gXtUh _ ((gXUsdjUgw) at:#gXtextht)
!!

"BODY
 ----"

!GXUUg2Udrawplot methodsFor: 'running'!

evaluate

    gXpUxarray isNil ifTrue: [ gXpUxarray _ GXRealarray new: 2 ].
    gXpUyarray isNil ifTrue: [ gXpUyarray _ GXRealarray new: 2 ].
    gXpUoarray isNil ifTrue: [ gXpUoarray _ GXRealarray new: 2 ]
.
((gXUsdjUgw) at:#gXxlog) ifTrue: [ 
 gXxarray _ gXpUxarray log: 10 .
GXfix run line: 0 proc: self method: false; p1:(gXxarray); p2:0.000000000000000e000; condEvaluate ] ifFalse: [ 
 gXxarray _ gXpUxarray  ] .
((gXUsdjUgw) at:#gXylog) ifTrue: [ 
 gXyarray _ gXpUyarray log: 10 .
GXfix run line: 0 proc: self method: false; p1:(gXyarray); p2:0.000000000000000e000; condEvaluate.
gXoarray _ (((gXpUoarray) / (gXpUyarray)) / (GXln run line: 0 proc: self method: true; p1:1.000000000000000e001; condEvaluate)) deepCopy.
GXfix run line: 0 proc: self method: false; p1:(gXoarray); p2:0.000000000000000e000; condEvaluate ] ifFalse: [ 
 	
	gXyarray _ gXpUyarray.
    	gXoarray _ gXpUoarray 
     ] .
((gXtype) = 2) ifTrue: [ 

    (gXxarray size = gXyarray size) ifTrue: [ gXcentredUx _ gXxarray ] 
			ifFalse: [ gXcentredUx _ gXxarray centre. 'DISPLAY: Displaying histogram workspace in point mode' prinl ]
.
GXmarkers run line: 0 proc: self method: false; gXdraw; p1:(gXcentredUx); p2:(gXyarray); p3:(gXpUc); p4:((gXUsdjUgw) at:#gXsymbol); p5:((gXUsdjUgw) at:#gXmarkersize); p6:(gXlUw); condEvaluate ] ifFalse: [ ((gXtype) = 3) ifTrue: [ 

    (gXxarray size = gXyarray size) ifTrue: [ gXcentredUx _ gXxarray ]
			ifFalse: [ gXcentredUx _ gXxarray centre. 'DISPLAY: Displaying histogram workspace in point mode' prinl ]
.
GXgraph run line: 0 proc: self method: false; gXdraw; p1:(gXcentredUx); p2:(gXyarray); p3:(gXpUc); p4:((gXUsdjUgw) at:#gXlinetype); p5:(gXlUw); condEvaluate ] ifFalse: [ ((gXtype) = 4) ifTrue: [ 
GXerrors run line: 0 proc: self method: false; gXdraw; gXvertical; p1:(gXxarray); p2:(gXyarray); p3:(gXoarray); p4:(gXpUc); p5:(gXlUw); condEvaluate ] ifFalse: [ ((gXtype) = 5) ifTrue: [ 
GXmultiUplot run line: 0 proc: self method: false; gXdraw; condEvaluate ] ifFalse: [ 
GXdraw run line: 0 proc: self method: false; gXhistogram; p1:(gXxarray); p2:(gXyarray); p3:(gXpUc); p4:(gXU); p5:((gXUsdjUgw) at:#gXlinetype); p6:(gXlUw); condEvaluate ] ]]].
self up.
^nil
!!
nil! "(fix for smalltalk bug, this line is ignored if there is a variable declaration error)"

"End of procedure"

GXcomment run line: 0 proc: self method: false; p1:'__g2_drawplot'; p2:'draws any plot in template'; condEvaluate!
"Procedure class definition:
 ==========================="
((Smalltalk at: #GXUUg2Udisplay ifAbsent: [ nil ]) isKindOf: Function class) ifTrue: [ ('Warning: Overriding previously defined procedure ', 'GXUUg2Udisplay' gtype) prinl ].
Smalltalk at: #GXUUg2Udisplay put: nil!
Function subclass: 'GXUUg2Udisplay'
       instanceVariableNames: ''
       classVariableNames: ''
       poolDictionaries: ''
       category: 'Genie Procedure'
!

"HEADER
 ------"

"parameters
 ----------"

GXUUg2Udisplay

addInstVarName: #gXtype;
compile:
'gXtype: aVar
	aVar notNil ifTrue:[gXtype _ self typeCheck: aVar shouldBe: GXInteger paramName: ''gXtype'']';
compile:
'p1: aVar
	aVar notNil ifTrue:[gXtype _ self typeCheck: aVar shouldBe: GXInteger paramName: ''gXtype'']';

addInstVarName: #gXwork;
compile:
'gXwork: aVar
	aVar notNil ifTrue:[gXwork _ self typeCheck: aVar shouldBe: GXWorkspace paramName: ''gXwork'']';
compile:
'p2: aVar
	aVar notNil ifTrue:[gXwork _ self typeCheck: aVar shouldBe: GXWorkspace paramName: ''gXwork'']'!

"parameters
 ----------"

GXUUg2Udisplay

addInstVarName: #gXxmin;
compile:
'gXxmin: aVar
	aVar notNil ifTrue:[gXxmin _ self typeCheck: aVar shouldBe: GXReal paramName: ''gXxmin'']';
compile:
'p3: aVar
	aVar notNil ifTrue:[gXxmin _ self typeCheck: aVar shouldBe: GXReal paramName: ''gXxmin'']';

addInstVarName: #gXxmax;
compile:
'gXxmax: aVar
	aVar notNil ifTrue:[gXxmax _ self typeCheck: aVar shouldBe: GXReal paramName: ''gXxmax'']';
compile:
'p4: aVar
	aVar notNil ifTrue:[gXxmax _ self typeCheck: aVar shouldBe: GXReal paramName: ''gXxmax'']';

addInstVarName: #gXymin;
compile:
'gXymin: aVar
	aVar notNil ifTrue:[gXymin _ self typeCheck: aVar shouldBe: GXReal paramName: ''gXymin'']';
compile:
'p5: aVar
	aVar notNil ifTrue:[gXymin _ self typeCheck: aVar shouldBe: GXReal paramName: ''gXymin'']';

addInstVarName: #gXymax;
compile:
'gXymax: aVar
	aVar notNil ifTrue:[gXymax _ self typeCheck: aVar shouldBe: GXReal paramName: ''gXymax'']';
compile:
'p6: aVar
	aVar notNil ifTrue:[gXymax _ self typeCheck: aVar shouldBe: GXReal paramName: ''gXymax'']';

addInstVarName: #gXlUc;
compile:
'gXlUc: aVar
	aVar notNil ifTrue:[gXlUc _ self typeCheck: aVar shouldBe: GXColour paramName: ''gXlUc'']';
compile:
'p7: aVar
	aVar notNil ifTrue:[gXlUc _ self typeCheck: aVar shouldBe: GXColour paramName: ''gXlUc'']';

addInstVarName: #gXlUw;
compile:
'gXlUw: aVar
	aVar notNil ifTrue:[gXlUw _ self typeCheck: aVar shouldBe: GXReal paramName: ''gXlUw'']';
compile:
'p8: aVar
	aVar notNil ifTrue:[gXlUw _ self typeCheck: aVar shouldBe: GXReal paramName: ''gXlUw'']';

addInstVarName: #gXtUc;
compile:
'gXtUc: aVar
	aVar notNil ifTrue:[gXtUc _ self typeCheck: aVar shouldBe: GXColour paramName: ''gXtUc'']';
compile:
'p9: aVar
	aVar notNil ifTrue:[gXtUc _ self typeCheck: aVar shouldBe: GXColour paramName: ''gXtUc'']';

addInstVarName: #gXtUh;
compile:
'gXtUh: aVar
	aVar notNil ifTrue:[gXtUh _ self typeCheck: aVar shouldBe: GXReal paramName: ''gXtUh'']';
compile:
'p10: aVar
	aVar notNil ifTrue:[gXtUh _ self typeCheck: aVar shouldBe: GXReal paramName: ''gXtUh'']'
!

"Local variables
 ---------------"

GXUUg2Udisplay

addInstVarName: #gXtype!

"Local variables
 ---------------"

GXUUg2Udisplay

addInstVarName: #gXwork!

"Local variables
 ---------------"

GXUUg2Udisplay

addInstVarName: #gXxmin!

"Local variables
 ---------------"

GXUUg2Udisplay

addInstVarName: #gXxmax!

"Local variables
 ---------------"

GXUUg2Udisplay

addInstVarName: #gXymin!

"Local variables
 ---------------"

GXUUg2Udisplay

addInstVarName: #gXymax!

"Local variables
 ---------------"

GXUUg2Udisplay

addInstVarName: #gXlUc!

"Local variables
 ---------------"

GXUUg2Udisplay

addInstVarName: #gXtUc!

"Local variables
 ---------------"

GXUUg2Udisplay

addInstVarName: #gXtUh!

"Local variables
 ---------------"

GXUUg2Udisplay

addInstVarName: #gXlUw
!

!GXUUg2Udisplay methodsFor: 'initializing'!

initVars
	gXtype _ 1.
	gXlUc _ ((gXUsdjUgw) at:#gXlinecolour).
	gXtUc _ ((gXUsdjUgw) at:#gXtextcolour).
	gXtUh _ ((gXUsdjUgw) at:#gXtextht).
	gXlUw _ ((gXUsdjUgw) at:#gXlinewidth)
!!

"BODY
 ----"

!GXUUg2Udisplay methodsFor: 'running'!

evaluate
((gXUsdjUgw) at:#gXcls) ifTrue: [ 
GXdevice run line: 0 proc: self method: false; gXclear; condEvaluate ].
GXUUg2Uaxes run line: 0 proc: self method: false; p1:(gXtype); p2:((gXwork) at:#gXx); p3:((gXwork) at:#gXy); p4:((gXwork) at:#gXe); p5:(gXxmin); p6:(gXxmax); p7:(gXymin); p8:(gXymax); condEvaluate.
(GXgUerrorUhandler run line: 0 proc: self method: true; condEvaluate) ifTrue: [ 
[nil.
self up.
^nil] value ].
GXUUg2Udrawplot run line: 0 proc: self method: false; p1:(gXtype); p2:((gXwork) at:#gXx); p3:((gXwork) at:#gXy); p4:((gXwork) at:#gXe); p5:(gXlUc); p6:(gXlUw); p7:(gXtUc); p8:(gXtUh); condEvaluate.
(GXgUerrorUhandler run line: 0 proc: self method: true; condEvaluate) ifTrue: [ 
[nil.
self up.
^nil] value ].
((gXUsdjUgw) at:#gXgraticule) ifTrue: [ 

		GXgraticule run line: 0 proc: self method: false;
		    gXdraw; gXxlog: (gXUsdjUgw at: #gXxlog); gXylog: (gXUsdjUgw at: #gXylog);
		    p1: gXlUc; p4: gXlUw; condEvaluate
	     ].
GXtitle run line: 0 proc: self method: false; gXdraw; p1:((gXwork) at:#gXtitle); p2:(gXtUc); p3:(gXtUh); p4:((gXUsdjUgw) at:#gXfont); condEvaluate.
GXlabels run line: 0 proc: self method: false; gXdraw; p1:((gXwork) at:#gXxlabel); p2:((gXwork) at:#gXylabel); p3:(gXtUh); p4:(gXtUc); p5:((gXUsdjUgw) at:#gXfont); condEvaluate.
GXgUerrorUhandler run line: 0 proc: self method: false; condEvaluate.
((gXUsdjUgw) at:#gXheader) ifTrue: [ 
GXUUg2Ubanner run line: 0 proc: self method: false; p1:((gXwork) at:#gXinstUname); p2:((gXwork) at:#gXrunUno); p3:((gXwork) at:#gXspecUno); p4:((gXwork) at:#gXfile); p5:((gXwork) at:#gXuserUname); p6:((gXwork) at:#gXtime); p7:(GXasUstring run line: 0 proc: self method: true; p1:((gXwork) at:#gXbinUgrouping); condEvaluate); condEvaluate ].
GXUUg2Uaxes run line: 0 proc: self method: false; gXredraw; p1:(gXtype); p2:((gXwork) at:#gXx); p3:((gXwork) at:#gXy); p4:((gXwork) at:#gXe); p5:(gXxmin); p6:(gXxmax); p7:(gXymin); p8:(gXymax); condEvaluate.
GXgUerrorUhandler run line: 0 proc: self method: false; condEvaluate.
self up.
^nil
!!
nil! "(fix for smalltalk bug, this line is ignored if there is a variable declaration error)"

"End of procedure"

GXcomment run line: 0 proc: self method: false; p1:'__g2_display'; p2:'draws a GenieII plot'; condEvaluate!
"Procedure class definition:
 ==========================="
((Smalltalk at: #GXUUresizeplot ifAbsent: [ nil ]) isKindOf: Function class) ifTrue: [ ('Warning: Overriding previously defined procedure ', 'GXUUresizeplot' gtype) prinl ].
Smalltalk at: #GXUUresizeplot put: nil!
Function subclass: 'GXUUresizeplot'
       instanceVariableNames: ''
       classVariableNames: ''
       poolDictionaries: ''
       category: 'Genie Procedure'
!

"HEADER
 ------"

"parameters
 ----------"

GXUUresizeplot

addInstVarName: #gXnmUpD1;
compile:
'gXnmUpD1: aVar
	aVar notNil ifTrue:[ gXnmUpD1 _ aVar ]';
compile:
'p1: aVar
	aVar notNil ifTrue:[ gXnmUpD1 _ aVar ]';

addInstVarName: #gXnmUpD2;
compile:
'gXnmUpD2: aVar
	aVar notNil ifTrue:[ gXnmUpD2 _ aVar ]';
compile:
'p2: aVar
	aVar notNil ifTrue:[ gXnmUpD2 _ aVar ]';

addInstVarName: #gXnmUpD3;
compile:
'gXnmUpD3: aVar
	aVar notNil ifTrue:[ gXnmUpD3 _ aVar ]';
compile:
'p3: aVar
	aVar notNil ifTrue:[ gXnmUpD3 _ aVar ]';

addInstVarName: #gXnmUpD4;
compile:
'gXnmUpD4: aVar
	aVar notNil ifTrue:[ gXnmUpD4 _ aVar ]';
compile:
'p4: aVar
	aVar notNil ifTrue:[ gXnmUpD4 _ aVar ]'!

"result
 ------"

GXUUresizeplot

addInstVarName: #gXres
!

"Local variables
 ---------------"

GXUUresizeplot

addInstVarName: #gXnmUpD1!

"Local variables
 ---------------"

GXUUresizeplot

addInstVarName: #gXnmUpD2!

"Local variables
 ---------------"

GXUUresizeplot

addInstVarName: #gXnmUpD3!

"Local variables
 ---------------"

GXUUresizeplot

addInstVarName: #gXnmUpD4!

"Local variables
 ---------------"

GXUUresizeplot

addInstVarName: #gXtextUscaler
!

!GXUUresizeplot methodsFor: 'initializing'!

initVars
	gXres _ (GXfields run line: 0 proc: self method: true; p1:5; condEvaluate)
!!

"BODY
 ----"

!GXUUresizeplot methodsFor: 'running'!

evaluate
(gXres isKindOf: GXArray) | (gXres isKindOf: GXWorkspace)
    ifTrue:  [ ((gXres) at:#gXtop put: ((gXUsdjUgw) at:#gXtop) deepCopy) ]
    ifFalse: [ self error: 'Assignment to array or workspace failed, check that the destination is a structured type'].
(gXres isKindOf: GXArray) | (gXres isKindOf: GXWorkspace)
    ifTrue:  [ ((gXres) at:#gXbottom put: ((gXUsdjUgw) at:#gXbottom) deepCopy) ]
    ifFalse: [ self error: 'Assignment to array or workspace failed, check that the destination is a structured type'].
(gXres isKindOf: GXArray) | (gXres isKindOf: GXWorkspace)
    ifTrue:  [ ((gXres) at:#gXleft put: ((gXUsdjUgw) at:#gXleft) deepCopy) ]
    ifFalse: [ self error: 'Assignment to array or workspace failed, check that the destination is a structured type'].
(gXres isKindOf: GXArray) | (gXres isKindOf: GXWorkspace)
    ifTrue:  [ ((gXres) at:#gXright put: ((gXUsdjUgw) at:#gXright) deepCopy) ]
    ifFalse: [ self error: 'Assignment to array or workspace failed, check that the destination is a structured type'].

        ( gXnmUpD1 isKindOf: Integer ) ifTrue: [ gXnmUpD1 _ gXnmUpD1 * 1.0 ].
        ( gXnmUpD2 isKindOf: Integer ) ifTrue: [ gXnmUpD2 _ gXnmUpD2 * 1.0 ].
        ( gXnmUpD3 isKindOf: Integer ) ifTrue: [ gXnmUpD3 _ gXnmUpD3 * 1.0 ].
        ( gXnmUpD4 isKindOf: Integer ) ifTrue: [ gXnmUpD4 _ gXnmUpD4 * 1.0 ].
	( gXnmUpD1 isKindOf: Float ) ifFalse: [^'__RESIZEPLOT: Parameter XMIN not specified' prenl ].
	( gXnmUpD2 isKindOf: Float ) ifFalse: [^'__RESIZEPLOT: Parameter XMAX not specified' prenl ].
	( gXnmUpD3 isKindOf: Float ) ifFalse: [^'__RESIZEPLOT: Parameter YMIN not specified' prenl ].
	( gXnmUpD4 isKindOf: Float ) ifFalse: [^'__RESIZEPLOT: Parameter YMAX not specified' prenl ]
.

    gXnmUpD1 isNil ifFalse: [ gXUsdjUgw at: #gXleft put: ( gXnmUpD1 shallowCopy ) ].
    gXnmUpD2 isNil ifFalse: [ gXUsdjUgw at: #gXright put: ( gXnmUpD2 shallowCopy ) ].
    gXnmUpD3 isNil ifFalse: [ gXUsdjUgw at: #gXbottom put: ( gXnmUpD3 shallowCopy ) ].
    gXnmUpD4 isNil ifFalse: [ gXUsdjUgw at: #gXtop put: ( gXnmUpD4 shallowCopy ) ]
.
gXtextUscaler _ (((((gXUsdjUgw) at:#gXright) - ((gXUsdjUgw) at:#gXleft))) + ((((gXUsdjUgw) at:#gXtop) - ((gXUsdjUgw) at:#gXbottom)))) deepCopy.
(gXUsdjUgw isKindOf: GXArray) | (gXUsdjUgw isKindOf: GXWorkspace)
    ifTrue:  [ ((gXUsdjUgw) at:#gXtextht put: (1.000000000000000e000 * (((gXtextUscaler) / 2.000000000000000e000))) deepCopy) ]
    ifFalse: [ self error: 'Assignment to array or workspace failed, check that the destination is a structured type'].
self up.
^gXres
!!
nil! "(fix for smalltalk bug, this line is ignored if there is a variable declaration error)"

"End of procedure"

GXcomment run line: 0 proc: self method: false; p1:'__resizeplot'; p2:'Changes the screen area used for a plot'; condEvaluate!
"Procedure class definition:
 ==========================="
((Smalltalk at: #GXUUsaycolour ifAbsent: [ nil ]) isKindOf: Function class) ifTrue: [ ('Warning: Overriding previously defined procedure ', 'GXUUsaycolour' gtype) prinl ].
Smalltalk at: #GXUUsaycolour put: nil!
Function subclass: 'GXUUsaycolour'
       instanceVariableNames: ''
       classVariableNames: ''
       poolDictionaries: ''
       category: 'Genie Procedure'
!

"HEADER
 ------"

"parameters
 ----------"

GXUUsaycolour

addInstVarName: #gXcolour;
compile:
'gXcolour: aVar
	aVar notNil ifTrue:[gXcolour _ self typeCheck: aVar shouldBe: GXInteger paramName: ''gXcolour'']';
compile:
'p1: aVar
	aVar notNil ifTrue:[gXcolour _ self typeCheck: aVar shouldBe: GXInteger paramName: ''gXcolour'']'
!

"Local variables
 ---------------"

GXUUsaycolour

addInstVarName: #gXcolour!

"Local variables
 ---------------"

GXUUsaycolour

addInstVarName: #gXtable
!

!GXUUsaycolour methodsFor: 'initializing'!

initVars
	gXcolour _ (gXUbackground).
	gXtable _ (GXdimensions run line: 0 proc: self method: true; p1:16; condEvaluate)
!!

"BODY
 ----"

!GXUUsaycolour methodsFor: 'running'!

evaluate
(gXtable isKindOf: GXArray) | (gXtable isKindOf: GXWorkspace)
    ifTrue:  [ ((gXtable) at:((Array new:1) at: 1 put: 1; yourself) put: 'BACKGROUND' deepCopy) ]
    ifFalse: [ self error: 'Assignment to array or workspace failed, check that the destination is a structured type'].
(gXtable isKindOf: GXArray) | (gXtable isKindOf: GXWorkspace)
    ifTrue:  [ ((gXtable) at:((Array new:1) at: 1 put: 2; yourself) put: 'FOREGROUND' deepCopy) ]
    ifFalse: [ self error: 'Assignment to array or workspace failed, check that the destination is a structured type'].
(gXtable isKindOf: GXArray) | (gXtable isKindOf: GXWorkspace)
    ifTrue:  [ ((gXtable) at:((Array new:1) at: 1 put: 3; yourself) put: 'RED' deepCopy) ]
    ifFalse: [ self error: 'Assignment to array or workspace failed, check that the destination is a structured type'].
(gXtable isKindOf: GXArray) | (gXtable isKindOf: GXWorkspace)
    ifTrue:  [ ((gXtable) at:((Array new:1) at: 1 put: 4; yourself) put: 'GREEN' deepCopy) ]
    ifFalse: [ self error: 'Assignment to array or workspace failed, check that the destination is a structured type'].
(gXtable isKindOf: GXArray) | (gXtable isKindOf: GXWorkspace)
    ifTrue:  [ ((gXtable) at:((Array new:1) at: 1 put: 5; yourself) put: 'BLUE' deepCopy) ]
    ifFalse: [ self error: 'Assignment to array or workspace failed, check that the destination is a structured type'].
(gXtable isKindOf: GXArray) | (gXtable isKindOf: GXWorkspace)
    ifTrue:  [ ((gXtable) at:((Array new:1) at: 1 put: 6; yourself) put: 'CYAN' deepCopy) ]
    ifFalse: [ self error: 'Assignment to array or workspace failed, check that the destination is a structured type'].
(gXtable isKindOf: GXArray) | (gXtable isKindOf: GXWorkspace)
    ifTrue:  [ ((gXtable) at:((Array new:1) at: 1 put: 7; yourself) put: 'MAGENTA' deepCopy) ]
    ifFalse: [ self error: 'Assignment to array or workspace failed, check that the destination is a structured type'].
(gXtable isKindOf: GXArray) | (gXtable isKindOf: GXWorkspace)
    ifTrue:  [ ((gXtable) at:((Array new:1) at: 1 put: 8; yourself) put: 'YELLOW' deepCopy) ]
    ifFalse: [ self error: 'Assignment to array or workspace failed, check that the destination is a structured type'].
(gXtable isKindOf: GXArray) | (gXtable isKindOf: GXWorkspace)
    ifTrue:  [ ((gXtable) at:((Array new:1) at: 1 put: 9; yourself) put: 'ORANGE' deepCopy) ]
    ifFalse: [ self error: 'Assignment to array or workspace failed, check that the destination is a structured type'].
(gXtable isKindOf: GXArray) | (gXtable isKindOf: GXWorkspace)
    ifTrue:  [ ((gXtable) at:((Array new:1) at: 1 put: 10; yourself) put: 'LIGHT GREEN' deepCopy) ]
    ifFalse: [ self error: 'Assignment to array or workspace failed, check that the destination is a structured type'].
(gXtable isKindOf: GXArray) | (gXtable isKindOf: GXWorkspace)
    ifTrue:  [ ((gXtable) at:((Array new:1) at: 1 put: 11; yourself) put: 'SEA GREEN' deepCopy) ]
    ifFalse: [ self error: 'Assignment to array or workspace failed, check that the destination is a structured type'].
(gXtable isKindOf: GXArray) | (gXtable isKindOf: GXWorkspace)
    ifTrue:  [ ((gXtable) at:((Array new:1) at: 1 put: 12; yourself) put: 'LIGHT BLUE' deepCopy) ]
    ifFalse: [ self error: 'Assignment to array or workspace failed, check that the destination is a structured type'].
(gXtable isKindOf: GXArray) | (gXtable isKindOf: GXWorkspace)
    ifTrue:  [ ((gXtable) at:((Array new:1) at: 1 put: 13; yourself) put: 'PURPLE' deepCopy) ]
    ifFalse: [ self error: 'Assignment to array or workspace failed, check that the destination is a structured type'].
(gXtable isKindOf: GXArray) | (gXtable isKindOf: GXWorkspace)
    ifTrue:  [ ((gXtable) at:((Array new:1) at: 1 put: 14; yourself) put: 'CRIMSON' deepCopy) ]
    ifFalse: [ self error: 'Assignment to array or workspace failed, check that the destination is a structured type'].
(gXtable isKindOf: GXArray) | (gXtable isKindOf: GXWorkspace)
    ifTrue:  [ ((gXtable) at:((Array new:1) at: 1 put: 15; yourself) put: 'DARK_GRAY' deepCopy) ]
    ifFalse: [ self error: 'Assignment to array or workspace failed, check that the destination is a structured type'].
(gXtable isKindOf: GXArray) | (gXtable isKindOf: GXWorkspace)
    ifTrue:  [ ((gXtable) at:((Array new:1) at: 1 put: 16; yourself) put: 'LIGHT GRAY' deepCopy) ]
    ifFalse: [ self error: 'Assignment to array or workspace failed, check that the destination is a structured type'].
((gXcolour) < 17) ifTrue: [ 
GXprintin run line: 0 proc: self method: false; p1:((gXtable) at:((Array new:1) at: 1 put: ((gXcolour) + 1); yourself)); condEvaluate ] ifFalse: [ 
GXprintin run line: 0 proc: self method: false; p1:'UNDEFINED'; condEvaluate ] .
self up.
^nil
!!
nil! "(fix for smalltalk bug, this line is ignored if there is a variable declaration error)"

"End of procedure"

GXcomment run line: 0 proc: self method: false; p1:'__saycolour'; p2:'prints the name of the given colour number'; condEvaluate!
"Procedure class definition:
 ==========================="
((Smalltalk at: #GXUUprintinfoa ifAbsent: [ nil ]) isKindOf: Function class) ifTrue: [ ('Warning: Overriding previously defined procedure ', 'GXUUprintinfoa' gtype) prinl ].
Smalltalk at: #GXUUprintinfoa put: nil!
Function subclass: 'GXUUprintinfoa'
       instanceVariableNames: ''
       classVariableNames: ''
       poolDictionaries: ''
       category: 'Genie Procedure'
!

"HEADER
 ------"

"Local variables
 ---------------"

GXUUprintinfoa

addInstVarName: #gXlogic!

"Local variables
 ---------------"

GXUUprintinfoa

addInstVarName: #gXnmUpD4
!

!GXUUprintinfoa methodsFor: 'initializing'!

initVars
	gXlogic _ (GXdimensions run line: 0 proc: self method: true; p1:2; condEvaluate)
!!

"BODY
 ----"

!GXUUprintinfoa methodsFor: 'running'!

evaluate
(gXlogic isKindOf: GXArray) | (gXlogic isKindOf: GXWorkspace)
    ifTrue:  [ ((gXlogic) at:((Array new:1) at: 1 put: 1; yourself) put: 'FALSE' deepCopy) ]
    ifFalse: [ self error: 'Assignment to array or workspace failed, check that the destination is a structured type'].
(gXlogic isKindOf: GXArray) | (gXlogic isKindOf: GXWorkspace)
    ifTrue:  [ ((gXlogic) at:((Array new:1) at: 1 put: 2; yourself) put: 'TRUE' deepCopy) ]
    ifFalse: [ self error: 'Assignment to array or workspace failed, check that the destination is a structured type'].
GXprintn run line: 0 proc: self method: false; p1:'  GRAPHICS WORKSPACE PARAMETERS:
'; condEvaluate.
GXprintn run line: 0 proc: self method: false; p1:'  Interactive device: '; p2:((gXUsdjUgw) at:#gXinteractive); condEvaluate.
GXprintn run line: 0 proc: self method: false; p1:'  X display size: '; p2:((gXUsdjUgw) at:#gXsize); condEvaluate.
GXprintn run line: 0 proc: self method: false; p1:'  Hardcopy device: '; p2:((gXUsdjUgw) at:#gXhardcopy); condEvaluate.
GXprintn run line: 0 proc: self method: false; p1:'  Current binning factor: '; p2:(gXUcurrentUbinning); condEvaluate.
GXprintn run line: 0 proc: self method: false; p1:'  Graticule plotted: '; p2:((gXUsdjUgw) at:#gXgraticule); condEvaluate.
GXprint run line: 0 proc: self method: false; p1:'  Symbol to be plotted: '; condEvaluate.
gXnmUpD4 _ ((gXUsdjUgw) at:#gXsymbol) deepCopy.
((gXnmUpD4) = 1) ifTrue: [ 
GXprintn run line: 0 proc: self method: false; p1:'point'; condEvaluate ] ifFalse: [ ((gXnmUpD4) = 2) ifTrue: [ 
GXprintn run line: 0 proc: self method: false; p1:'plus'; condEvaluate ] ifFalse: [ ((gXnmUpD4) = 3) ifTrue: [ 
GXprintn run line: 0 proc: self method: false; p1:'star'; condEvaluate ] ifFalse: [ ((gXnmUpD4) = 4) ifTrue: [ 
GXprintn run line: 0 proc: self method: false; p1:'circle'; condEvaluate ] ifFalse: [ ((gXnmUpD4) = 5) ifTrue: [ 
GXprintn run line: 0 proc: self method: false; p1:'cross'; condEvaluate ] ifFalse: [ ((gXnmUpD4) = 6) ifTrue: [ 
GXprintn run line: 0 proc: self method: false; p1:'box'; condEvaluate ] ifFalse: [ 
GXprintn run line: 0 proc: self method: false; p1:'number '; p2:(gXnmUpD4); condEvaluate ] ]]]]].
self up.
^nil
!!
nil! "(fix for smalltalk bug, this line is ignored if there is a variable declaration error)"

"End of procedure"

"Procedure class definition:
 ==========================="
((Smalltalk at: #GXUUprintinfob ifAbsent: [ nil ]) isKindOf: Function class) ifTrue: [ ('Warning: Overriding previously defined procedure ', 'GXUUprintinfob' gtype) prinl ].
Smalltalk at: #GXUUprintinfob put: nil!
Function subclass: 'GXUUprintinfob'
       instanceVariableNames: ''
       classVariableNames: ''
       poolDictionaries: ''
       category: 'Genie Procedure'
!

"HEADER
 ------"

"Local variables
 ---------------"

GXUUprintinfob

addInstVarName: #gXnmUpD4
!

!GXUUprintinfob methodsFor: 'initializing'!

initVars

!!

"BODY
 ----"

!GXUUprintinfob methodsFor: 'running'!

evaluate
GXprintn run line: 0 proc: self method: false; p1:'  Plot location   X: '; p2:((gXUsdjUgw) at:#gXleft); p3:','; p4:((gXUsdjUgw) at:#gXright); condEvaluate.
GXprintn run line: 0 proc: self method: false; p1:'                  Y: '; p2:((gXUsdjUgw) at:#gXbottom); p3:','; p4:((gXUsdjUgw) at:#gXtop); condEvaluate.
GXprintn run line: 0 proc: self method: false; p1:'  Graphics linewidth: '; p2:((gXUsdjUgw) at:#gXlinewidth); condEvaluate.
GXprintn run line: 0 proc: self method: false; p1:'  Graphics text height: '; p2:((gXUsdjUgw) at:#gXtextht); condEvaluate.
GXprintn run line: 0 proc: self method: false; p1:'  Graphics text font: '; p2:((gXUsdjUgw) at:#gXfont); condEvaluate.
GXprint run line: 0 proc: self method: false; p1:'  Graphics plotting linetype: '; condEvaluate.
gXnmUpD4 _ ((gXUsdjUgw) at:#gXlinetype) deepCopy.
((gXnmUpD4) = 1) ifTrue: [ 
GXprintn run line: 0 proc: self method: false; p1:'full line'; condEvaluate ] ifFalse: [ ((gXnmUpD4) = 2) ifTrue: [ 
GXprintn run line: 0 proc: self method: false; p1:'dashed line'; condEvaluate ] ifFalse: [ ((gXnmUpD4) = 3) ifTrue: [ 
GXprintn run line: 0 proc: self method: false; p1:'dot-dash line'; condEvaluate ] ifFalse: [ ((gXnmUpD4) = 4) ifTrue: [ 
GXprintn run line: 0 proc: self method: false; p1:'dotted line'; condEvaluate ] ifFalse: [ 
GXprintn run line: 0 proc: self method: false; p1:'number '; p2:(gXnmUpD4); condEvaluate ] ]]].
self up.
^nil
!!
nil! "(fix for smalltalk bug, this line is ignored if there is a variable declaration error)"

"End of procedure"

"Procedure class definition:
 ==========================="
((Smalltalk at: #GXUUg2altersize ifAbsent: [ nil ]) isKindOf: Function class) ifTrue: [ ('Warning: Overriding previously defined procedure ', 'GXUUg2altersize' gtype) prinl ].
Smalltalk at: #GXUUg2altersize put: nil!
Function subclass: 'GXUUg2altersize'
       instanceVariableNames: ''
       classVariableNames: ''
       poolDictionaries: ''
       category: 'Genie Procedure'
!

"HEADER
 ------"

"parameters
 ----------"

GXUUg2altersize

addInstVarName: #gXnmUpD1;
compile:
'gXnmUpD1: aVar
	aVar notNil ifTrue:[ gXnmUpD1 _ aVar ]';
compile:
'p1: aVar
	aVar notNil ifTrue:[ gXnmUpD1 _ aVar ]'!

"result
 ------"

GXUUg2altersize

addInstVarName: #gXres
!

!GXUUg2altersize methodsFor: 'initializing'!

initVars

!!

"BODY
 ----"

!GXUUg2altersize methodsFor: 'running'!

evaluate
gXres _ ((gXUsdjUgw) at:#gXsize) deepCopy.
(GXisUa run line: 0 proc: self method: true; p1:(gXnmUpD1); p2:'INTEGER'; condEvaluate) ifTrue: [ 
gXnmUpD1 _ ((gXnmUpD1) * 1.000000000000000e000) deepCopy ].
(GXUUcheckvar run line: 0 proc: self method: true; p1:(gXnmUpD1); p2:'REAL'; p3:1; p4:1; condEvaluate) not ifTrue: [ 
[nil.
self up.
^gXres] value ].
(gXUsdjUgw isKindOf: GXArray) | (gXUsdjUgw isKindOf: GXWorkspace)
    ifTrue:  [ ((gXUsdjUgw) at:#gXsize put: (gXnmUpD1) deepCopy) ]
    ifFalse: [ self error: 'Assignment to array or workspace failed, check that the destination is a structured type'].
((gXUUstatus) = 1) ifTrue: [ 
GXdevice run line: 0 proc: self method: false; gXclose; condEvaluate.
GXdevice run line: 0 proc: self method: false; gXopen; p1:((gXUsdjUgw) at:#gXinteractive); p2:(gXU); p3:(gXU); p4:((gXUsdjUgw) at:#gXsize); condEvaluate.
GXdevice run line: 0 proc: self method: false; gXclear; condEvaluate.
GXredraw run line: 0 proc: self method: false; condEvaluate ].
self up.
^gXres
!!
nil! "(fix for smalltalk bug, this line is ignored if there is a variable declaration error)"

"End of procedure"

"Procedure class definition:
 ==========================="
((Smalltalk at: #GXUUprintinfoc ifAbsent: [ nil ]) isKindOf: Function class) ifTrue: [ ('Warning: Overriding previously defined procedure ', 'GXUUprintinfoc' gtype) prinl ].
Smalltalk at: #GXUUprintinfoc put: nil!
Function subclass: 'GXUUprintinfoc'
       instanceVariableNames: ''
       classVariableNames: ''
       poolDictionaries: ''
       category: 'Genie Procedure'
!

"HEADER
 ------"

"Local variables
 ---------------"

GXUUprintinfoc

addInstVarName: #gXnmUpD4
!

!GXUUprintinfoc methodsFor: 'initializing'!

initVars

!!

"BODY
 ----"

!GXUUprintinfoc methodsFor: 'running'!

evaluate
GXprint run line: 0 proc: self method: false; p1:'  Graphics axes colour: '; condEvaluate.
GXUUsaycolour run line: 0 proc: self method: false; p1:((gXUsdjUgw) at:#gXlinecolour); condEvaluate.
GXprint run line: 0 proc: self method: false; p1:'  Graphics text colour: '; condEvaluate.
GXUUsaycolour run line: 0 proc: self method: false; p1:((gXUsdjUgw) at:#gXtextcolour); condEvaluate.
GXprint run line: 0 proc: self method: false; p1:'  Graphics plot colour: '; condEvaluate.
GXUUsaycolour run line: 0 proc: self method: false; p1:((gXUsdjUgw) at:#gXplotcolour); condEvaluate.
self up.
^nil
!!
nil! "(fix for smalltalk bug, this line is ignored if there is a variable declaration error)"

"End of procedure"

"Procedure class definition:
 ==========================="
((Smalltalk at: #GXUUprintenv ifAbsent: [ nil ]) isKindOf: Function class) ifTrue: [ ('Warning: Overriding previously defined procedure ', 'GXUUprintenv' gtype) prinl ].
Smalltalk at: #GXUUprintenv put: nil!
Function subclass: 'GXUUprintenv'
       instanceVariableNames: ''
       classVariableNames: ''
       poolDictionaries: ''
       category: 'Genie Procedure'
!

"BODY
 ----"

!GXUUprintenv methodsFor: 'running'!

evaluate
GXUUprintinfoa run line: 0 proc: self method: false; condEvaluate.
GXUUprintinfob run line: 0 proc: self method: false; condEvaluate.
GXUUprintinfoc run line: 0 proc: self method: false; condEvaluate.
self up.
^nil
!!
nil! "(fix for smalltalk bug, this line is ignored if there is a variable declaration error)"

"End of procedure"

GXcomment run line: 0 proc: self method: false; p1:'__printenv'; p2:'prints the graphics workspace parameters'; condEvaluate!
"Procedure class definition:
 ==========================="
((Smalltalk at: #GXdisplay ifAbsent: [ nil ]) isKindOf: Function class) ifTrue: [ ('Warning: Overriding previously defined procedure ', 'GXdisplay' gtype) prinl ].
Smalltalk at: #GXdisplay put: nil!
Function subclass: 'GXdisplay'
       instanceVariableNames: ''
       classVariableNames: ''
       poolDictionaries: ''
       category: 'Genie Procedure'
!

"HEADER
 ------"

"qualifiers
 ----------"

GXdisplay

addInstVarName: #gXmarkers;
compile:
'gXmarkers
	gXmarkers _ true';
compile:
'gXmarkers: aBoolean
	gXmarkers _ aBoolean';

addInstVarName: #gXline;
compile:
'gXline
	gXline _ true';
compile:
'gXline: aBoolean
	gXline _ aBoolean';

addInstVarName: #gXerrors;
compile:
'gXerrors
	gXerrors _ true';
compile:
'gXerrors: aBoolean
	gXerrors _ aBoolean';

addInstVarName: #gXhistogram;
compile:
'gXhistogram
	gXhistogram _ true';
compile:
'gXhistogram: aBoolean
	gXhistogram _ aBoolean'!

"parameters
 ----------"

GXdisplay

addInstVarName: #gXp1;
compile:
'gXp1: aVar
	aVar notNil ifTrue:[ gXp1 _ aVar ]';
compile:
'p1: aVar
	aVar notNil ifTrue:[ gXp1 _ aVar ]';

addInstVarName: #gXp2;
compile:
'gXp2: aVar
	aVar notNil ifTrue:[ gXp2 _ aVar ]';
compile:
'p2: aVar
	aVar notNil ifTrue:[ gXp2 _ aVar ]';

addInstVarName: #gXp3;
compile:
'gXp3: aVar
	aVar notNil ifTrue:[ gXp3 _ aVar ]';
compile:
'p3: aVar
	aVar notNil ifTrue:[ gXp3 _ aVar ]';

addInstVarName: #gXp4;
compile:
'gXp4: aVar
	aVar notNil ifTrue:[ gXp4 _ aVar ]';
compile:
'p4: aVar
	aVar notNil ifTrue:[ gXp4 _ aVar ]';

addInstVarName: #gXp5;
compile:
'gXp5: aVar
	aVar notNil ifTrue:[ gXp5 _ aVar ]';
compile:
'p5: aVar
	aVar notNil ifTrue:[ gXp5 _ aVar ]';

addInstVarName: #gXp6;
compile:
'gXp6: aVar
	aVar notNil ifTrue:[ gXp6 _ aVar ]';
compile:
'p6: aVar
	aVar notNil ifTrue:[ gXp6 _ aVar ]';

addInstVarName: #gXp7;
compile:
'gXp7: aVar
	aVar notNil ifTrue:[ gXp7 _ aVar ]';
compile:
'p7: aVar
	aVar notNil ifTrue:[ gXp7 _ aVar ]';

addInstVarName: #gXp8;
compile:
'gXp8: aVar
	aVar notNil ifTrue:[ gXp8 _ aVar ]';
compile:
'p8: aVar
	aVar notNil ifTrue:[ gXp8 _ aVar ]';

addInstVarName: #gXp9;
compile:
'gXp9: aVar
	aVar notNil ifTrue:[ gXp9 _ aVar ]';
compile:
'p9: aVar
	aVar notNil ifTrue:[ gXp9 _ aVar ]';

addInstVarName: #gXwksp;
compile:
'gXwksp: aVar
	aVar notNil ifTrue:[gXwksp _ self typeCheck: aVar shouldBe: GXWorkspace paramName: ''gXwksp'']';
compile:
'p10: aVar
	aVar notNil ifTrue:[gXwksp _ self typeCheck: aVar shouldBe: GXWorkspace paramName: ''gXwksp'']';

addInstVarName: #gXxmin;
compile:
'gXxmin: aVar
	aVar notNil ifTrue:[gXxmin _ self typeCheck: aVar shouldBe: GXReal paramName: ''gXxmin'']';
compile:
'p11: aVar
	aVar notNil ifTrue:[gXxmin _ self typeCheck: aVar shouldBe: GXReal paramName: ''gXxmin'']';

addInstVarName: #gXxmax;
compile:
'gXxmax: aVar
	aVar notNil ifTrue:[gXxmax _ self typeCheck: aVar shouldBe: GXReal paramName: ''gXxmax'']';
compile:
'p12: aVar
	aVar notNil ifTrue:[gXxmax _ self typeCheck: aVar shouldBe: GXReal paramName: ''gXxmax'']';

addInstVarName: #gXymin;
compile:
'gXymin: aVar
	aVar notNil ifTrue:[gXymin _ self typeCheck: aVar shouldBe: GXReal paramName: ''gXymin'']';
compile:
'p13: aVar
	aVar notNil ifTrue:[gXymin _ self typeCheck: aVar shouldBe: GXReal paramName: ''gXymin'']';

addInstVarName: #gXymax;
compile:
'gXymax: aVar
	aVar notNil ifTrue:[gXymax _ self typeCheck: aVar shouldBe: GXReal paramName: ''gXymax'']';
compile:
'p14: aVar
	aVar notNil ifTrue:[gXymax _ self typeCheck: aVar shouldBe: GXReal paramName: ''gXymax'']';

addInstVarName: #gXlinecolour;
compile:
'gXlinecolour: aVar
	aVar notNil ifTrue:[gXlinecolour _ self typeCheck: aVar shouldBe: GXColour paramName: ''gXlinecolour'']';
compile:
'p15: aVar
	aVar notNil ifTrue:[gXlinecolour _ self typeCheck: aVar shouldBe: GXColour paramName: ''gXlinecolour'']';

addInstVarName: #gXlinewidth;
compile:
'gXlinewidth: aVar
	aVar notNil ifTrue:[gXlinewidth _ self typeCheck: aVar shouldBe: GXReal paramName: ''gXlinewidth'']';
compile:
'p16: aVar
	aVar notNil ifTrue:[gXlinewidth _ self typeCheck: aVar shouldBe: GXReal paramName: ''gXlinewidth'']';

addInstVarName: #gXtextcolour;
compile:
'gXtextcolour: aVar
	aVar notNil ifTrue:[gXtextcolour _ self typeCheck: aVar shouldBe: GXColour paramName: ''gXtextcolour'']';
compile:
'p17: aVar
	aVar notNil ifTrue:[gXtextcolour _ self typeCheck: aVar shouldBe: GXColour paramName: ''gXtextcolour'']';

addInstVarName: #gXtextheight;
compile:
'gXtextheight: aVar
	aVar notNil ifTrue:[gXtextheight _ self typeCheck: aVar shouldBe: GXReal paramName: ''gXtextheight'']';
compile:
'p18: aVar
	aVar notNil ifTrue:[gXtextheight _ self typeCheck: aVar shouldBe: GXReal paramName: ''gXtextheight'']'
!

"Local variables
 ---------------"

GXdisplay

addInstVarName: #gXbin!

"Local variables
 ---------------"

GXdisplay

addInstVarName: #gXtype
!

!GXdisplay methodsFor: 'initializing'!

initVars
	gXmarkers _ false.
	gXline _ false.
	gXerrors _ false.
	gXhistogram _ false.
	gXbin _ (gXUcurrentUbinning)
!!

"BODY
 ----"

!GXdisplay methodsFor: 'running'!

evaluate
 ( gXp1 class == GXExperiment ) ifTrue: [ gXp1 _ gXp1 asOld ] .
gXtype _ (GXUUg2Uopendev run line: 0 proc: self method: true; condEvaluate) deepCopy.
((gXUsdjUgw) at:#gXcls) ifTrue: [ 
GXdevice run line: 0 proc: self method: false; gXclear; condEvaluate ].

    ( gXp1 isKindOf: Integer ) ifTrue: [ gXp1 _ gXp1 * 1.0 ].
    ( gXp2 isKindOf: Integer ) ifTrue: [ gXp2 _ gXp2 * 1.0 ].
    ( gXp3 isKindOf: Integer ) ifTrue: [ gXp3 _ gXp3 * 1.0 ].
    ( gXp4 isKindOf: Integer ) ifTrue: [ gXp4 _ gXp4 * 1.0 ].
    ( gXp5 isKindOf: Integer ) ifTrue: [ gXp5 _ gXp5 * 1.0 ].
    ( gXp1 isKindOf: GXWorkspace ) ifTrue: [ gXwksp _ gXp1 ].
    gXwksp notNil ifTrue: [
	( gXp2 isKindOf: Float ) ifTrue: [ gXxmin _ gXp2 ]. 
	( gXp3 isKindOf: Float ) ifTrue: [ gXxmax _ gXp3 ]. 
	( gXp4 isKindOf: Float ) ifTrue: [ gXymin _ gXp4 ]. 
	( gXp5 isKindOf: Float ) ifTrue: [ gXymax _ gXp5 ].
	( gXp6 isKindOf: Symbol ) ifTrue: [ gXlinecolour _ gXp6 ].
	( gXp7 isKindOf: Float ) ifTrue: [ gXlinewidth _ gXp7 ].
	( gXp8 isKindOf: Symbol ) ifTrue: [ gXtextcolour _ gXp8 ].
	( gXp9 isKindOf: Float ) ifTrue: [ gXtextheight _ gXp9 ]
    ] ifFalse: [
	( gXp1 isKindOf: Float ) ifTrue: [ gXxmin _ gXp1 ]. 
	( gXp2 isKindOf: Float ) ifTrue: [ gXxmax _ gXp2 ]. 
	( gXp3 isKindOf: Float ) ifTrue: [ gXymin _ gXp3 ]. 
	( gXp4 isKindOf: Float ) ifTrue: [ gXymax _ gXp4 ].
	( gXp5 isKindOf: Symbol ) ifTrue: [ gXlinecolour _ gXp5 ].
	( gXp6 isKindOf: Float ) ifTrue: [ gXlinewidth _ gXp6 ].
	( gXp7 isKindOf: Symbol ) ifTrue: [ gXtextcolour _ gXp7 ].
	( gXp8 isKindOf: Float ) ifTrue: [ gXtextheight _ gXp8 ]
    ].
    gXwksp notNil ifTrue: [
	gXUsdjUgw at: #gXxminUlimit put: nil.
	gXUsdjUgw at: #gXxmaxUlimit put: nil.
	gXUsdjUgw at: #gXyminUlimit put: nil.
	gXUsdjUgw at: #gXymaxUlimit put: nil.
	gXwksp at: #gXbinUgrouping ifAbsent: [ gXp1 at: #gXbinUgrouping put: 0 ].
	gXwksp at: #gXxlabel ifAbsent: [ gXp1 at: #gXxlabel put: '' ].
	gXwksp at: #gXylabel ifAbsent: [ gXp1 at: #gXylabel put: '' ].
	gXwksp at: #gXtitle ifAbsent: [ gXp1 at: #gXtitle put: '' ].
	gXwksp at: #gXuserUname ifAbsent: [ gXp1 at: #gXuserUname put: '<not specified>' ].
	gXwksp at: #gXinstUname ifAbsent: [ gXp1 at: #gXinstUname put: '<not specified>' ].
	gXwksp at: #gXrunUno ifAbsent: [ gXp1 at: #gXrunUno put: '<not specified>' ].
	gXwksp at: #gXspecUno ifAbsent: [ gXp1 at: #gXspecUno put: '<not specified>' ].
	gXwksp at: #gXtime ifAbsent: [ gXp1 at: #gXtime put: '<not specified>' ].
	gXwksp at: #gXfile ifAbsent: [ gXp1 at: #gXfile put: '<not specified>' ].
	( (gXwksp at: #gXbinUgrouping) = 0 ) ifTrue: [ gXbin _ gXUcurrentUbinning ] ifFalse: [ gXbin _ gXwksp at: #gXbinUgrouping ].
	"rebin the data for plotting and make undefined values = 0.0"
        gXUgrUwksp _ GXgroupbins run line: 0 proc: self method: true; p1:(gXwksp); p2:(gXbin); p3:(0.0e0); condEvaluate.
	gXUgrUwksp at: #gXbinUgrouping put: gXbin.
	'Displayed using bin-grouping of ' prin. gXbin prinl
    ].
    ( gXUgrUwksp isKindOf: GXWorkspace ) ifFalse: [^'DISPLAY: Parameter WKSP not specified' prenl ]
.
gXtype _ 1 deepCopy.
(gXmarkers) ifTrue: [ 
gXtype _ 2 deepCopy ] ifFalse: [ (gXline) ifTrue: [ 
gXtype _ 3 deepCopy ] ifFalse: [ (gXerrors) ifTrue: [ 
gXtype _ 4 deepCopy ]]].
GXUUg2Udisplay run line: 0 proc: self method: false; p1:(gXtype); p2:(gXUgrUwksp); p3:(gXxmin); p4:(gXxmax); p5:(gXymin); p6:(gXymax); p7:(gXlinecolour); p8:(gXlinewidth); p9:(gXtextcolour); p10:(gXtextheight); condEvaluate.
self up.
^nil
!!
nil! "(fix for smalltalk bug, this line is ignored if there is a variable declaration error)"

"End of procedure"

GXcomment run line: 0 proc: self method: false; p1:'display'; p2:'Display a new workspace, or redisplay previous one'; condEvaluate!
"Procedure class definition:
 ==========================="
((Smalltalk at: #GXUUalterstatus ifAbsent: [ nil ]) isKindOf: Function class) ifTrue: [ ('Warning: Overriding previously defined procedure ', 'GXUUalterstatus' gtype) prinl ].
Smalltalk at: #GXUUalterstatus put: nil!
Function subclass: 'GXUUalterstatus'
       instanceVariableNames: ''
       classVariableNames: ''
       poolDictionaries: ''
       category: 'Genie Procedure'
!

"BODY
 ----"

!GXUUalterstatus methodsFor: 'running'!

evaluate
GXprintn run line: 0 proc: self method: false; p1:'Current values of parameters controlled by the ALTER command
'; condEvaluate.
GXprintn run line: 0 proc: self method: false; p1:'Interactive device: '; p2:((gXUsdjUgw) at:#gXinteractive); condEvaluate.
GXprintn run line: 0 proc: self method: false; p1:'Hardcopy device: '; p2:((gXUsdjUgw) at:#gXhardcopy); condEvaluate.
GXprintn run line: 0 proc: self method: false; p1:'Plot area (xmin, xmax, ymin, ymax): '; p2:((gXUsdjUgw) at:#gXleft); p3:' '; p4:((gXUsdjUgw) at:#gXright); p5:' '; p6:((gXUsdjUgw) at:#gXbottom); p7:' '; p8:((gXUsdjUgw) at:#gXtop); condEvaluate.
GXprintn run line: 0 proc: self method: false; p1:'Plot size scaler: '; p2:((gXUsdjUgw) at:#gXsize); condEvaluate.
GXprintn run line: 0 proc: self method: false; p1:'Plot colour: '; p2:((gXUsdjUgw) at:#gXplotcolour); condEvaluate.
GXprintn run line: 0 proc: self method: false; p1:'Marker type: '; p2:((gXUsdjUgw) at:#gXsymbol); condEvaluate.
GXprintn run line: 0 proc: self method: false; p1:'Marker size: '; p2:((gXUsdjUgw) at:#gXmarkersize); condEvaluate.
GXprintn run line: 0 proc: self method: false; p1:'Linewidth: '; p2:((gXUsdjUgw) at:#gXlinewidth); condEvaluate.
GXprintn run line: 0 proc: self method: false; p1:'Line type: '; p2:((gXUsdjUgw) at:#gXlinetype); condEvaluate.
GXprintn run line: 0 proc: self method: false; p1:'Line colour: '; p2:((gXUsdjUgw) at:#gXlinecolour); condEvaluate.
GXprintn run line: 0 proc: self method: false; p1:'Text height: '; p2:((gXUsdjUgw) at:#gXtextht); condEvaluate.
GXprintn run line: 0 proc: self method: false; p1:'Text colour: '; p2:((gXUsdjUgw) at:#gXtextcolour); condEvaluate.
GXprintn run line: 0 proc: self method: false; p1:'Text font: '; p2:((gXUsdjUgw) at:#gXfont); condEvaluate.
GXprintn run line: 0 proc: self method: false; p1:'Binning: '; p2:(gXUcurrentUbinning); condEvaluate.
self up.
^nil
!!
nil! "(fix for smalltalk bug, this line is ignored if there is a variable declaration error)"

"End of procedure"

"Procedure class definition:
 ==========================="
((Smalltalk at: #GXalter ifAbsent: [ nil ]) isKindOf: Function class) ifTrue: [ ('Warning: Overriding previously defined procedure ', 'GXalter' gtype) prinl ].
Smalltalk at: #GXalter put: nil!
Function subclass: 'GXalter'
       instanceVariableNames: ''
       classVariableNames: ''
       poolDictionaries: ''
       category: 'Genie Procedure'
!

"HEADER
 ------"

"qualifiers
 ----------"

GXalter

addInstVarName: #gXbinning;
compile:
'gXbinning
	gXbinning _ true';
compile:
'gXbinning: aBoolean
	gXbinning _ aBoolean';

addInstVarName: #gXmarkers;
compile:
'gXmarkers
	gXmarkers _ true';
compile:
'gXmarkers: aBoolean
	gXmarkers _ aBoolean';

addInstVarName: #gXplot;
compile:
'gXplot
	gXplot _ true';
compile:
'gXplot: aBoolean
	gXplot _ aBoolean';

addInstVarName: #gXdevice;
compile:
'gXdevice
	gXdevice _ true';
compile:
'gXdevice: aBoolean
	gXdevice _ aBoolean';

addInstVarName: #gXhardcopy;
compile:
'gXhardcopy
	gXhardcopy _ true';
compile:
'gXhardcopy: aBoolean
	gXhardcopy _ aBoolean'!

"qualifiers
 ----------"

GXalter

addInstVarName: #gXtextheight;
compile:
'gXtextheight
	gXtextheight _ true';
compile:
'gXtextheight: aBoolean
	gXtextheight _ aBoolean';

addInstVarName: #gXfont;
compile:
'gXfont
	gXfont _ true';
compile:
'gXfont: aBoolean
	gXfont _ aBoolean';

addInstVarName: #gXlinetype;
compile:
'gXlinetype
	gXlinetype _ true';
compile:
'gXlinetype: aBoolean
	gXlinetype _ aBoolean';

addInstVarName: #gXtextcolour;
compile:
'gXtextcolour
	gXtextcolour _ true';
compile:
'gXtextcolour: aBoolean
	gXtextcolour _ aBoolean';

addInstVarName: #gXlinecolour;
compile:
'gXlinecolour
	gXlinecolour _ true';
compile:
'gXlinecolour: aBoolean
	gXlinecolour _ aBoolean';

addInstVarName: #gXplotcolour;
compile:
'gXplotcolour
	gXplotcolour _ true';
compile:
'gXplotcolour: aBoolean
	gXplotcolour _ aBoolean'!

"qualifiers
 ----------"

GXalter

addInstVarName: #gXsize;
compile:
'gXsize
	gXsize _ true';
compile:
'gXsize: aBoolean
	gXsize _ aBoolean';

addInstVarName: #gXlinewidth;
compile:
'gXlinewidth
	gXlinewidth _ true';
compile:
'gXlinewidth: aBoolean
	gXlinewidth _ aBoolean';

addInstVarName: #gXmarkersize;
compile:
'gXmarkersize
	gXmarkersize _ true';
compile:
'gXmarkersize: aBoolean
	gXmarkersize _ aBoolean';

addInstVarName: #gXstatus;
compile:
'gXstatus
	gXstatus _ true';
compile:
'gXstatus: aBoolean
	gXstatus _ aBoolean';

addInstVarName: #gXbackground;
compile:
'gXbackground
	gXbackground _ true';
compile:
'gXbackground: aBoolean
	gXbackground _ aBoolean'!

"qualifiers
 ----------"

GXalter

addInstVarName: #gXforeground;
compile:
'gXforeground
	gXforeground _ true';
compile:
'gXforeground: aBoolean
	gXforeground _ aBoolean'!

"parameters
 ----------"

GXalter

addInstVarName: #gXp1;
compile:
'gXp1: aVar
	aVar notNil ifTrue:[ gXp1 _ aVar ]';
compile:
'p1: aVar
	aVar notNil ifTrue:[ gXp1 _ aVar ]';

addInstVarName: #gXp2;
compile:
'gXp2: aVar
	aVar notNil ifTrue:[ gXp2 _ aVar ]';
compile:
'p2: aVar
	aVar notNil ifTrue:[ gXp2 _ aVar ]';

addInstVarName: #gXp3;
compile:
'gXp3: aVar
	aVar notNil ifTrue:[ gXp3 _ aVar ]';
compile:
'p3: aVar
	aVar notNil ifTrue:[ gXp3 _ aVar ]';

addInstVarName: #gXp4;
compile:
'gXp4: aVar
	aVar notNil ifTrue:[ gXp4 _ aVar ]';
compile:
'p4: aVar
	aVar notNil ifTrue:[ gXp4 _ aVar ]';

addInstVarName: #gXxmin;
compile:
'gXxmin: aVar
	aVar notNil ifTrue:[gXxmin _ self typeCheck: aVar shouldBe: GXReal paramName: ''gXxmin'']';
compile:
'p5: aVar
	aVar notNil ifTrue:[gXxmin _ self typeCheck: aVar shouldBe: GXReal paramName: ''gXxmin'']';

addInstVarName: #gXxmax;
compile:
'gXxmax: aVar
	aVar notNil ifTrue:[gXxmax _ self typeCheck: aVar shouldBe: GXReal paramName: ''gXxmax'']';
compile:
'p6: aVar
	aVar notNil ifTrue:[gXxmax _ self typeCheck: aVar shouldBe: GXReal paramName: ''gXxmax'']';

addInstVarName: #gXymin;
compile:
'gXymin: aVar
	aVar notNil ifTrue:[gXymin _ self typeCheck: aVar shouldBe: GXReal paramName: ''gXymin'']';
compile:
'p7: aVar
	aVar notNil ifTrue:[gXymin _ self typeCheck: aVar shouldBe: GXReal paramName: ''gXymin'']';

addInstVarName: #gXymax;
compile:
'gXymax: aVar
	aVar notNil ifTrue:[gXymax _ self typeCheck: aVar shouldBe: GXReal paramName: ''gXymax'']';
compile:
'p8: aVar
	aVar notNil ifTrue:[gXymax _ self typeCheck: aVar shouldBe: GXReal paramName: ''gXymax'']'!

"result
 ------"

GXalter

addInstVarName: #gXres
!

!GXalter methodsFor: 'initializing'!

initVars
	gXbinning _ false.
	gXmarkers _ false.
	gXplot _ false.
	gXdevice _ false.
	gXhardcopy _ false.
	gXtextheight _ false.
	gXfont _ false.
	gXlinetype _ false.
	gXtextcolour _ false.
	gXlinecolour _ false.
	gXplotcolour _ false.
	gXsize _ false.
	gXlinewidth _ false.
	gXmarkersize _ false.
	gXstatus _ false.
	gXbackground _ false.
	gXforeground _ false
!!

"BODY
 ----"

!GXalter methodsFor: 'running'!

evaluate

    ( gXp1 isKindOf: Float ) ifTrue: [ gXxmin _ gXp1 ].
    ( gXp2 isKindOf: Float ) ifTrue: [ gXxmax _ gXp2 ].
    ( gXp3 isKindOf: Float ) ifTrue: [ gXymin _ gXp3 ].
    ( gXp4 isKindOf: Float ) ifTrue: [ gXymax _ gXp4 ].
    ( gXp1 isKindOf: GXWorkspace ) ifTrue: [ gXxmin _ gXp1 at: #gXleft. gXxmax _ gXp1 at: #gXright. 
						gXymin _ gXp1 at: #gXbottom. gXymax _ gXp1 at: #gXtop ]
.
(gXstatus) ifTrue: [ 
GXUUalterstatus run line: 0 proc: self method: false; condEvaluate.
[nil.
self up.
^gXres] value ].
(gXbinning) ifTrue: [ 
gXres _ (gXUcurrentUbinning) deepCopy.
 (gXp1 isKindOf: Integer) ifTrue: [ gXUcurrentUbinning _ gXp1 ] 
					ifFalse: [ 'ALTER: BINNING must be an INTEGER' prenl ] .
[nil.
self up.
^gXres] value ].
(gXmarkers) ifTrue: [ 
gXres _ ((gXUsdjUgw) at:#gXsymbol) deepCopy.
 (gXp1 isKindOf: Symbol) ifTrue: [ gXUsdjUgw at: #gXsymbol put: gXp1 ] 
					ifFalse: [ 'ALTER: must supply a MARKER' prenl ] .
[nil.
self up.
^gXres] value ].
(gXmarkersize) ifTrue: [ 
gXres _ ((gXUsdjUgw) at:#gXmarkersize) deepCopy.
 (gXp1 isKindOf: Float) ifTrue: [ gXUsdjUgw at: #gXmarkersize put: gXp1 ] 
					ifFalse: [ 'ALTER: must supply a REAL' prenl ] .
[nil.
self up.
^gXres] value ].
(gXplot) ifTrue: [ 
gXres _ (GXUUresizeplot run line: 0 proc: self method: true; p1:(gXxmin); p2:(gXxmax); p3:(gXymin); p4:(gXymax); condEvaluate) deepCopy.
[nil.
self up.
^gXres] value ].
(gXdevice) ifTrue: [ 
gXres _ ((gXUsdjUgw) at:#gXinteractive) deepCopy.
 (gXp1 isKindOf: String) ifTrue: [ gXUsdjUgw at: #gXinteractive put: gXp1 ] 
					ifFalse: [ 'ALTER: must supply a STRING' prenl ] .
[nil.
self up.
^gXres] value ].
(gXhardcopy) ifTrue: [ 
gXres _ ((gXUsdjUgw) at:#gXhardcopy) deepCopy.
 (gXp1 isKindOf: String) ifTrue: [ gXUsdjUgw at: #gXhardcopy put: gXp1 ] 
					ifFalse: [ 'ALTER: must supply a STRING' prenl ] .
[nil.
self up.
^gXres] value ].
(gXlinewidth) ifTrue: [ 
gXres _ ((gXUsdjUgw) at:#gXlinewidth) deepCopy.
 (gXp1 isKindOf: Integer) ifTrue: [ gXp1 _ gXp1 * 1.0 ] .
 (gXp1 isKindOf: Float) ifTrue: [ gXUsdjUgw at: #gXlinewidth put: gXp1 ] 
					ifFalse: [ 'ALTER: must supply a REAL' prenl ] .
[nil.
self up.
^gXres] value ].
(gXtextheight) ifTrue: [ 
gXres _ ((gXUsdjUgw) at:#gXtextht) deepCopy.
 (gXp1 isKindOf: Integer) ifTrue: [ gXp1 _ gXp1 * 1.0 ] .
 (gXp1 isKindOf: Float) ifTrue: [ gXUsdjUgw at: #gXtextht put: gXp1 ] 
					ifFalse: [ 'ALTER: must supply a REAL' prenl ] .
[nil.
self up.
^gXres] value ].
(gXfont) ifTrue: [ 
gXres _ ((gXUsdjUgw) at:#gXfont) deepCopy.
 (gXp1 isKindOf: Symbol) ifTrue: [ gXUsdjUgw at: #gXfont put: gXp1 ] 
					ifFalse: [ 'ALTER: must supply a FONT' prenl ] .
[nil.
self up.
^gXres] value ].
(gXlinetype) ifTrue: [ 
gXres _ ((gXUsdjUgw) at:#gXlinetype) deepCopy.
 (gXp1 isKindOf: Symbol) ifTrue: [ gXUsdjUgw at: #gXlinetype put: gXp1 ] 
					ifFalse: [ 'ALTER: must supply a LINESTYLE' prenl ] .
[nil.
self up.
^gXres] value ].
(gXlinecolour) ifTrue: [ 
gXres _ ((gXUsdjUgw) at:#gXlinecolour) deepCopy.
 (gXp1 isKindOf: Symbol) ifTrue: [ gXUsdjUgw at: #gXlinecolour put: gXp1 ] 
					ifFalse: [ 'ALTER: must supply a COLOUR' prenl ] .
[nil.
self up.
^gXres] value ].
(gXtextcolour) ifTrue: [ 
gXres _ ((gXUsdjUgw) at:#gXtextcolour) deepCopy.
 (gXp1 isKindOf: Symbol) ifTrue: [ gXUsdjUgw at: #gXtextcolour put: gXp1 ] 
					ifFalse: [ 'ALTER: must supply a COLOUR' prenl ] .
[nil.
self up.
^gXres] value ].
(gXplotcolour) ifTrue: [ 
gXres _ ((gXUsdjUgw) at:#gXplotcolour) deepCopy.
 (gXp1 isKindOf: Symbol) ifTrue: [ gXUsdjUgw at: #gXplotcolour put: gXp1 ] 
					ifFalse: [ 'ALTER: must supply a COLOUR' prenl ] .
[nil.
self up.
^gXres] value ].
(gXsize) ifTrue: [ 
gXres _ (GXUUg2altersize run line: 0 proc: self method: true; p1:(gXp1); condEvaluate) deepCopy ].
(gXbackground) ifTrue: [ 

	    (gXp1 isKindOf: Symbol) 
		ifTrue: [ gXUbackground _ gXp1 ]
		ifFalse: [ 'ALTER: Item is not a valid colour' prenl ]
	 ].
(gXforeground) ifTrue: [ 

	    (gXp1 isKindOf: Symbol) 
		ifTrue: [ gXUforeground _ gXp1 ]
		ifFalse: [ 'ALTER: Item is not a valid colour' prenl ]
	 ].
self up.
^gXres
!!
nil! "(fix for smalltalk bug, this line is ignored if there is a variable declaration error)"

"End of procedure"

GXcomment run line: 0 proc: self method: false; p1:'alter'; p2:'Alters graphics control parameters'; condEvaluate!
"Procedure class definition:
 ==========================="
((Smalltalk at: #GXtoggle ifAbsent: [ nil ]) isKindOf: Function class) ifTrue: [ ('Warning: Overriding previously defined procedure ', 'GXtoggle' gtype) prinl ].
Smalltalk at: #GXtoggle put: nil!
Function subclass: 'GXtoggle'
       instanceVariableNames: ''
       classVariableNames: ''
       poolDictionaries: ''
       category: 'Genie Procedure'
!

"HEADER
 ------"

"qualifiers
 ----------"

GXtoggle

addInstVarName: #gXstatus;
compile:
'gXstatus
	gXstatus _ true';
compile:
'gXstatus: aBoolean
	gXstatus _ aBoolean';

addInstVarName: #gXlogx;
compile:
'gXlogx
	gXlogx _ true';
compile:
'gXlogx: aBoolean
	gXlogx _ aBoolean';

addInstVarName: #gXlogy;
compile:
'gXlogy
	gXlogy _ true';
compile:
'gXlogy: aBoolean
	gXlogy _ aBoolean';

addInstVarName: #gXclear;
compile:
'gXclear
	gXclear _ true';
compile:
'gXclear: aBoolean
	gXclear _ aBoolean';

addInstVarName: #gXmode;
compile:
'gXmode
	gXmode _ true';
compile:
'gXmode: aBoolean
	gXmode _ aBoolean';

addInstVarName: #gXchar;
compile:
'gXchar
	gXchar _ true';
compile:
'gXchar: aBoolean
	gXchar _ aBoolean';

addInstVarName: #gXheader;
compile:
'gXheader
	gXheader _ true';
compile:
'gXheader: aBoolean
	gXheader _ aBoolean';

addInstVarName: #gXrx;
compile:
'gXrx
	gXrx _ true';
compile:
'gXrx: aBoolean
	gXrx _ aBoolean';

addInstVarName: #gXry;
compile:
'gXry
	gXry _ true';
compile:
'gXry: aBoolean
	gXry _ aBoolean';

addInstVarName: #gXinfo;
compile:
'gXinfo
	gXinfo _ true';
compile:
'gXinfo: aBoolean
	gXinfo _ aBoolean'!

"qualifiers
 ----------"

GXtoggle

addInstVarName: #gXgraticule;
compile:
'gXgraticule
	gXgraticule _ true';
compile:
'gXgraticule: aBoolean
	gXgraticule _ aBoolean';

addInstVarName: #gXon;
compile:
'gXon
	gXon _ true';
compile:
'gXon: aBoolean
	gXon _ aBoolean';

addInstVarName: #gXoff;
compile:
'gXoff
	gXoff _ true';
compile:
'gXoff: aBoolean
	gXoff _ aBoolean'
!

"Local variables
 ---------------"

GXtoggle

addInstVarName: #gXtmp;

addInstVarName: #gXval
!

!GXtoggle methodsFor: 'initializing'!

initVars
	gXstatus _ false.
	gXlogx _ false.
	gXlogy _ false.
	gXclear _ false.
	gXmode _ false.
	gXchar _ false.
	gXheader _ false.
	gXrx _ false.
	gXry _ false.
	gXinfo _ false.
	gXgraticule _ false.
	gXon _ false.
	gXoff _ false
!!

"BODY
 ----"

!GXtoggle methodsFor: 'running'!

evaluate

    gXon ifTrue: [ gXval _ true ].
    gXoff ifTrue: [ gXval _ false ]
.
(gXlogx) ifTrue: [ 
 gXval isNil ifTrue: [ gXval _ (gXUsdjUgw at: #gXxlog) not ] .
 gXUsdjUgw at: #gXxlog put: gXval  ].
((gXlogx) | (gXstatus)) ifTrue: [ 
((gXUsdjUgw) at:#gXxlog) ifTrue: [ 
GXprintn run line: 0 proc: self method: false; p1:'  X axis is a log axis'; condEvaluate ] ifFalse: [ 
GXprintn run line: 0 proc: self method: false; p1:'  X axis is a linear axis'; condEvaluate ]  ].
(gXlogy) ifTrue: [ 
 gXval isNil ifTrue: [ gXval _ (gXUsdjUgw at: #gXylog) not ] .
 gXUsdjUgw at: #gXylog put: gXval  ].
((gXlogy) | (gXstatus)) ifTrue: [ 
((gXUsdjUgw) at:#gXylog) ifTrue: [ 
GXprintn run line: 0 proc: self method: false; p1:'  Y axis is a log axis'; condEvaluate ] ifFalse: [ 
GXprintn run line: 0 proc: self method: false; p1:'  Y axis is a linear axis'; condEvaluate ]  ].
(gXheader) ifTrue: [ 
 gXval isNil ifTrue: [ gXval _ (gXUsdjUgw at: #gXheader) not ] .
 gXUsdjUgw at: #gXheader put: gXval  ].
((gXheader) | (gXstatus)) ifTrue: [ 
((gXUsdjUgw) at:#gXheader) ifTrue: [ 
GXprintn run line: 0 proc: self method: false; p1:'  Graph header will be plotted'; condEvaluate ] ifFalse: [ 
GXprintn run line: 0 proc: self method: false; p1:'  Graph header will not be plotted'; condEvaluate ]  ].
(gXgraticule) ifTrue: [ 
 gXval isNil ifTrue: [ gXval _ (gXUsdjUgw at: #gXgraticule) not ] .
 gXUsdjUgw at: #gXgraticule put: gXval  ].
((gXgraticule) | (gXstatus)) ifTrue: [ 
((gXUsdjUgw) at:#gXgraticule) ifTrue: [ 
GXprintn run line: 0 proc: self method: false; p1:'  Graticule will be plotted'; condEvaluate ] ifFalse: [ 
GXprintn run line: 0 proc: self method: false; p1:'  Graticule will not be plotted'; condEvaluate ]  ].
(gXmode) ifTrue: [ 
 gXval isNil ifTrue: [ gXval _ (gXUsdjUgw at: #gXbin) not ] .
 gXUsdjUgw at: #gXbin put: gXval  ].
((gXmode) | (gXstatus)) ifTrue: [ 
((gXUsdjUgw) at:#gXbin) ifTrue: [ 
GXprintn run line: 0 proc: self method: false; p1:'  Histogram bin centre is X value (i.e. point mode)'; condEvaluate ] ifFalse: [ 
GXprintn run line: 0 proc: self method: false; p1:'  Histogram bin leading edge is X value (i.e. bin mode)'; condEvaluate ]  ].
(gXclear) ifTrue: [ 
 gXval isNil ifTrue: [ gXval _ (gXUsdjUgw at: #gXcls) not ] .
 gXUsdjUgw at: #gXcls put: gXval  ].
((gXclear) | (gXstatus)) ifTrue: [ 
((gXUsdjUgw) at:#gXcls) ifTrue: [ 
GXprintn run line: 0 proc: self method: false; p1:'  Screen cleared between plots'; condEvaluate ] ifFalse: [ 
GXprintn run line: 0 proc: self method: false; p1:'  Screen not cleared between plots'; condEvaluate ]  ].
(gXchar) ifTrue: [ 
GXprinten run line: 0 proc: self method: false; p1:'  Use ALTER/FONT to change character font'; condEvaluate ].
(gXrx) ifTrue: [ 
 gXval isNil ifTrue: [ gXval _ (gXUsdjUgw at: #gXxrounding) not ] .
 gXUsdjUgw at: #gXxrounding put: gXval  ].
((gXrx) | (gXstatus)) ifTrue: [ 
((gXUsdjUgw) at:#gXxrounding) ifTrue: [ 
GXprintn run line: 0 proc: self method: false; p1:'  X axis rounding is on'; condEvaluate ] ifFalse: [ 
GXprintn run line: 0 proc: self method: false; p1:'  X axis rounding is off'; condEvaluate ]  ].
(gXry) ifTrue: [ 
 gXval isNil ifTrue: [ gXval _ (gXUsdjUgw at: #gXyrounding) not ] .
 gXUsdjUgw at: #gXyrounding put: gXval  ].
((gXry) | (gXstatus)) ifTrue: [ 
((gXUsdjUgw) at:#gXyrounding) ifTrue: [ 
GXprintn run line: 0 proc: self method: false; p1:'  Y axis rounding is on'; condEvaluate ] ifFalse: [ 
GXprintn run line: 0 proc: self method: false; p1:'  Y axis rounding is off'; condEvaluate ]  ].
(gXinfo) ifTrue: [ 

		gXtmp _ '' streamMode: 'i' mode: nil.		"get current mode"
		gXtmp _ ( 1 - gXtmp ).				"toggle"
	        gXval isNil ifTrue: [ gXval _ gXtmp ] ifFalse: [
		    gXval ifTrue: [ gXval _ 1 ] ifFalse: [ gXval _ 0 ]
 		].
		'' streamMode: 'i' mode: gXval			"reset"
	     ].
((gXinfo) | (gXstatus)) ifTrue: [ 

		gXtmp _ '' streamMode: 'i' mode: nil
	    .
(((gXtmp) = 1)) ifTrue: [ 
GXprintin run line: 0 proc: self method: false; p1:'  Informational printing is ON'; condEvaluate ] ].
self up.
^nil
!!
nil! "(fix for smalltalk bug, this line is ignored if there is a variable declaration error)"

"End of procedure"

GXcomment run line: 0 proc: self method: false; p1:'toggle'; p2:'toggles graphics control parameters'; condEvaluate!
"Procedure class definition:
 ==========================="
((Smalltalk at: #GXUUcursortext ifAbsent: [ nil ]) isKindOf: Function class) ifTrue: [ ('Warning: Overriding previously defined procedure ', 'GXUUcursortext' gtype) prinl ].
Smalltalk at: #GXUUcursortext put: nil!
Function subclass: 'GXUUcursortext'
       instanceVariableNames: ''
       classVariableNames: ''
       poolDictionaries: ''
       category: 'Genie Procedure'
!

"BODY
 ----"

!GXUUcursortext methodsFor: 'running'!

evaluate
GXprintin run line: 0 proc: self method: false; p1:'  CURSOR CONTROLS
'; condEvaluate.
GXprintin run line: 0 proc: self method: false; p1:'  X   display X coordinate'; condEvaluate.
GXprintin run line: 0 proc: self method: false; p1:'  Y   display Y coordinate'; condEvaluate.
GXprintin run line: 0 proc: self method: false; p1:'  P   display both coordinates'; condEvaluate.
GXprintin run line: 0 proc: self method: false; p1:'  T   add text'; condEvaluate.
GXprintin run line: 0 proc: self method: false; p1:'  L   lower left corner of box'; condEvaluate.
GXprintin run line: 0 proc: self method: false; p1:'  U   draw box to upper right point'; condEvaluate.
GXprintin run line: 0 proc: self method: false; p1:'  E   exit
'; condEvaluate.
self up.
^nil
!!
nil! "(fix for smalltalk bug, this line is ignored if there is a variable declaration error)"

"End of procedure"

GXcomment run line: 0 proc: self method: false; p1:'__cursortext'; p2:'some info on Genie II cursor'; condEvaluate!
"Procedure class definition:
 ==========================="
((Smalltalk at: #GXcursor ifAbsent: [ nil ]) isKindOf: Function class) ifTrue: [ ('Warning: Overriding previously defined procedure ', 'GXcursor' gtype) prinl ].
Smalltalk at: #GXcursor put: nil!
Function subclass: 'GXcursor'
       instanceVariableNames: ''
       classVariableNames: ''
       poolDictionaries: ''
       category: 'Genie Procedure'
!

"HEADER
 ------"

"qualifiers
 ----------"

GXcursor

addInstVarName: #gXvertical;
compile:
'gXvertical
	gXvertical _ true';
compile:
'gXvertical: aBoolean
	gXvertical _ aBoolean';

addInstVarName: #gXhorizontal;
compile:
'gXhorizontal
	gXhorizontal _ true';
compile:
'gXhorizontal: aBoolean
	gXhorizontal _ aBoolean'!

"parameters
 ----------"

GXcursor

addInstVarName: #gXxval;
compile:
'gXxval: aVar
	aVar notNil ifTrue:[gXxval _ self typeCheck: aVar shouldBe: GXReal paramName: ''gXxval'']';
compile:
'p1: aVar
	aVar notNil ifTrue:[gXxval _ self typeCheck: aVar shouldBe: GXReal paramName: ''gXxval'']';

addInstVarName: #gXyval;
compile:
'gXyval: aVar
	aVar notNil ifTrue:[gXyval _ self typeCheck: aVar shouldBe: GXReal paramName: ''gXyval'']';
compile:
'p2: aVar
	aVar notNil ifTrue:[gXyval _ self typeCheck: aVar shouldBe: GXReal paramName: ''gXyval'']'
!

"Local variables
 ---------------"

GXcursor

addInstVarName: #gXxval!

"Local variables
 ---------------"

GXcursor

addInstVarName: #gXyval!

"Local variables
 ---------------"

GXcursor

addInstVarName: #gXletter!

"Local variables
 ---------------"

GXcursor

addInstVarName: #gXangle!

"Local variables
 ---------------"

GXcursor

addInstVarName: #gXtUc!

"Local variables
 ---------------"

GXcursor

addInstVarName: #gXtUh!

"Local variables
 ---------------"

GXcursor

addInstVarName: #gXlUw!

"Local variables
 ---------------"

GXcursor

addInstVarName: #gXfont!

"Local variables
 ---------------"

GXcursor

addInstVarName: #gXtext!

"Local variables
 ---------------"

GXcursor

addInstVarName: #gXboxmin!

"Local variables
 ---------------"

GXcursor

addInstVarName: #gXboxmax!

"Local variables
 ---------------"

GXcursor

addInstVarName: #gXcursor!

"Local variables
 ---------------"

GXcursor

addInstVarName: #gXtype!

"Local variables
 ---------------"

GXcursor

addInstVarName: #gXx;

addInstVarName: #gXy
!

!GXcursor methodsFor: 'initializing'!

initVars
	gXvertical _ false.
	gXhorizontal _ false.
	gXxval _ 0.000000000000000e000.
	gXyval _ 0.000000000000000e000.
	gXangle _ 9.000000000000000e001.
	gXtUc _ ((gXUsdjUgw) at:#gXtextcolour).
	gXtUh _ ((gXUsdjUgw) at:#gXtextht).
	gXlUw _ ((gXUsdjUgw) at:#gXlinewidth).
	gXfont _ ((gXUsdjUgw) at:#gXfont).
	gXboxmin _ (GXdimensions run line: 0 proc: self method: true; p1:2; condEvaluate).
	gXboxmax _ (GXdimensions run line: 0 proc: self method: true; p1:2; condEvaluate)
!!

"BODY
 ----"

!GXcursor methodsFor: 'running'!

evaluate
gXtype _ (GXUUg2Uopendev run line: 0 proc: self method: true; condEvaluate) deepCopy.
(gXhorizontal) ifTrue: [ 
gXangle _ 0.000000000000000e000 deepCopy ].
(gXboxmin isKindOf: GXArray) | (gXboxmin isKindOf: GXWorkspace)
    ifTrue:  [ ((gXboxmin) at:((Array new:1) at: 1 put: 1; yourself) put: 9.990000000000000e002 negated deepCopy) ]
    ifFalse: [ self error: 'Assignment to array or workspace failed, check that the destination is a structured type'].
GXUUcursortext run line: 0 proc: self method: false; condEvaluate.
Function new loop:  [ 
gXcursor _ (GXgetcursor run line: 0 proc: self method: true; p1:(gXxval); p2:(gXyval); condEvaluate) deepCopy.
gXletter _ ((gXcursor) at:#gXchar) deepCopy.
gXxval _ ((gXcursor) at:#gXwUx) deepCopy.
gXyval _ ((gXcursor) at:#gXwUy) deepCopy ] 
condition: [ ((gXletter) = 'E') ] after: [ 

    (gXUsdjUgw at: #gXxlog) ifTrue: [ gXx _ 10.0 raisedTo: gXxval ] ifFalse: [ gXx _ gXxval ]. 
    (gXUsdjUgw at: #gXylog) ifTrue: [ gXy _ 10.0 raisedTo: gXyval ] ifFalse: [ gXy _ gXyval ]
.
((gXletter) = 'X') ifTrue: [ 
gXtext _ (GXasUstring run line: 0 proc: self method: true; p1:(gXx); condEvaluate) deepCopy.
GXtext run line: 0 proc: self method: false; gXdraw; p1:(gXxval); p2:(gXyval); p3:(gXtext); p4:(gXtUc); p5:(gXtUh); p6:(gXangle); p7:(gXfont); p8:(gXlUw); condEvaluate ] ifFalse: [ ((gXletter) = 'Y') ifTrue: [ 
gXtext _ (GXasUstring run line: 0 proc: self method: true; p1:(gXy); condEvaluate) deepCopy.
GXtext run line: 0 proc: self method: false; gXdraw; p1:(gXxval); p2:(gXyval); p3:(gXtext); p4:(gXtUc); p5:(gXtUh); p6:(gXangle); p7:(gXfont); p8:(gXlUw); condEvaluate ] ifFalse: [ ((gXletter) = 'P') ifTrue: [ 
gXtext _ (((GXasUstring run line: 0 proc: self method: true; p1:(gXx); condEvaluate) , ' ') , (GXasUstring run line: 0 proc: self method: true; p1:(gXy); condEvaluate)) deepCopy.
GXtext run line: 0 proc: self method: false; gXdraw; p1:(gXxval); p2:(gXyval); p3:(gXtext); p4:(gXtUc); p5:(gXtUh); p6:(gXangle); p7:(gXfont); p8:(gXlUw); condEvaluate ] ifFalse: [ ((gXletter) = 'T') ifTrue: [ 
GXprint run line: 0 proc: self method: false; p1:'  Enter text: '; condEvaluate.
gXtext _ (GXreadUterminal run line: 0 proc: self method: true; condEvaluate) deepCopy.
GXtext run line: 0 proc: self method: false; gXdraw; p1:(gXxval); p2:(gXyval); p3:(gXtext); p4:(gXtUc); p5:(gXtUh); p6:(gXangle); p7:(gXfont); p8:(gXlUw); condEvaluate ] ifFalse: [ ((gXletter) = 'L') ifTrue: [ 
(gXboxmin isKindOf: GXArray) | (gXboxmin isKindOf: GXWorkspace)
    ifTrue:  [ ((gXboxmin) at:((Array new:1) at: 1 put: 1; yourself) put: (gXxval) deepCopy) ]
    ifFalse: [ self error: 'Assignment to array or workspace failed, check that the destination is a structured type'].
(gXboxmin isKindOf: GXArray) | (gXboxmin isKindOf: GXWorkspace)
    ifTrue:  [ ((gXboxmin) at:((Array new:1) at: 1 put: 2; yourself) put: (gXyval) deepCopy) ]
    ifFalse: [ self error: 'Assignment to array or workspace failed, check that the destination is a structured type'] ] ifFalse: [ ((gXletter) = 'U') ifTrue: [ 
(gXboxmax isKindOf: GXArray) | (gXboxmax isKindOf: GXWorkspace)
    ifTrue:  [ ((gXboxmax) at:((Array new:1) at: 1 put: 1; yourself) put: (gXxval) deepCopy) ]
    ifFalse: [ self error: 'Assignment to array or workspace failed, check that the destination is a structured type'].
(gXboxmax isKindOf: GXArray) | (gXboxmax isKindOf: GXWorkspace)
    ifTrue:  [ ((gXboxmax) at:((Array new:1) at: 1 put: 2; yourself) put: (gXyval) deepCopy) ]
    ifFalse: [ self error: 'Assignment to array or workspace failed, check that the destination is a structured type'].
(((gXboxmin) at:((Array new:1) at: 1 put: 1; yourself)) ~= 9.990000000000000e002 negated) ifTrue: [ 
GXdraw run line: 0 proc: self method: false; gXbox; p1:((gXboxmin) at:((Array new:1) at: 1 put: 1; yourself)); p2:((gXboxmin) at:((Array new:1) at: 1 put: 2; yourself)); p3:((gXboxmax) at:((Array new:1) at: 1 put: 1; yourself)); p4:((gXboxmax) at:((Array new:1) at: 1 put: 2; yourself)); p5:((gXUsdjUgw) at:#gXlinecolour); p6:(gXlUw); condEvaluate ] ]]]]]] ].
self up.
^nil
!!
nil! "(fix for smalltalk bug, this line is ignored if there is a variable declaration error)"

"End of procedure"

GXcomment run line: 0 proc: self method: false; p1:'cursor'; p2:'Emulates Genie II cursor routine'; condEvaluate!
"Procedure class definition:
 ==========================="
((Smalltalk at: #GXkeep ifAbsent: [ nil ]) isKindOf: Function class) ifTrue: [ ('Warning: Overriding previously defined procedure ', 'GXkeep' gtype) prinl ].
Smalltalk at: #GXkeep put: nil!
Function subclass: 'GXkeep'
       instanceVariableNames: ''
       classVariableNames: ''
       poolDictionaries: ''
       category: 'Genie Procedure'
!

"HEADER
 ------"

"qualifiers
 ----------"

GXkeep

addInstVarName: #gXh;
compile:
'gXh
	gXh _ true';
compile:
'gXh: aBoolean
	gXh _ aBoolean';

addInstVarName: #gXhard;
compile:
'gXhard
	gXhard _ true';
compile:
'gXhard: aBoolean
	gXhard _ aBoolean';

addInstVarName: #gXhardcopy;
compile:
'gXhardcopy
	gXhardcopy _ true';
compile:
'gXhardcopy: aBoolean
	gXhardcopy _ aBoolean';

addInstVarName: #gXposition;
compile:
'gXposition
	gXposition _ true';
compile:
'gXposition: aBoolean
	gXposition _ aBoolean';

addInstVarName: #gXx;
compile:
'gXx
	gXx _ true';
compile:
'gXx: aBoolean
	gXx _ aBoolean';

addInstVarName: #gXy;
compile:
'gXy
	gXy _ true';
compile:
'gXy: aBoolean
	gXy _ aBoolean';

addInstVarName: #gXb;
compile:
'gXb
	gXb _ true';
compile:
'gXb: aBoolean
	gXb _ aBoolean';

addInstVarName: #gXcell;
compile:
'gXcell
	gXcell _ true';
compile:
'gXcell: aBoolean
	gXcell _ aBoolean'
!

!GXkeep methodsFor: 'initializing'!

initVars
	gXh _ false.
	gXhard _ false.
	gXhardcopy _ false.
	gXposition _ false.
	gXx _ false.
	gXy _ false.
	gXb _ false.
	gXcell _ false
!!

"BODY
 ----"

!GXkeep methodsFor: 'running'!

evaluate
GXprintin run line: 0 proc: self method: false; p1:'  KEEP is not a supported function of Open GENIE.'; condEvaluate.
GXprintin run line: 0 proc: self method: false; p1:'  Use HARDCOPY [filename [devicetype]] to print.'; condEvaluate.
self up.
^nil
!!
nil! "(fix for smalltalk bug, this line is ignored if there is a variable declaration error)"

"End of procedure"

GXcomment run line: 0 proc: self method: false; p1:'keep'; p2:'Use HARDCOPY command instead'; condEvaluate!
"Procedure class definition:
 ==========================="
((Smalltalk at: #GXlimits ifAbsent: [ nil ]) isKindOf: Function class) ifTrue: [ ('Warning: Overriding previously defined procedure ', 'GXlimits' gtype) prinl ].
Smalltalk at: #GXlimits put: nil!
Function subclass: 'GXlimits'
       instanceVariableNames: ''
       classVariableNames: ''
       poolDictionaries: ''
       category: 'Genie Procedure'
!

"HEADER
 ------"

"qualifiers
 ----------"

GXlimits

addInstVarName: #gXx;
compile:
'gXx
	gXx _ true';
compile:
'gXx: aBoolean
	gXx _ aBoolean';

addInstVarName: #gXy;
compile:
'gXy
	gXy _ true';
compile:
'gXy: aBoolean
	gXy _ aBoolean';

addInstVarName: #gXnoUauto;
compile:
'gXnoUauto
	gXnoUauto _ true';
compile:
'gXnoUauto: aBoolean
	gXnoUauto _ aBoolean';

addInstVarName: #gXdefault;
compile:
'gXdefault
	gXdefault _ true';
compile:
'gXdefault: aBoolean
	gXdefault _ aBoolean'!

"parameters
 ----------"

GXlimits

addInstVarName: #gXp1;
compile:
'gXp1: aVar
	aVar notNil ifTrue:[ gXp1 _ aVar ]';
compile:
'p1: aVar
	aVar notNil ifTrue:[ gXp1 _ aVar ]';

addInstVarName: #gXp2;
compile:
'gXp2: aVar
	aVar notNil ifTrue:[ gXp2 _ aVar ]';
compile:
'p2: aVar
	aVar notNil ifTrue:[ gXp2 _ aVar ]';

addInstVarName: #gXp3;
compile:
'gXp3: aVar
	aVar notNil ifTrue:[ gXp3 _ aVar ]';
compile:
'p3: aVar
	aVar notNil ifTrue:[ gXp3 _ aVar ]';

addInstVarName: #gXp4;
compile:
'gXp4: aVar
	aVar notNil ifTrue:[ gXp4 _ aVar ]';
compile:
'p4: aVar
	aVar notNil ifTrue:[ gXp4 _ aVar ]';

addInstVarName: #gXxmin;
compile:
'gXxmin: aVar
	aVar notNil ifTrue:[gXxmin _ self typeCheck: aVar shouldBe: GXReal paramName: ''gXxmin'']';
compile:
'p5: aVar
	aVar notNil ifTrue:[gXxmin _ self typeCheck: aVar shouldBe: GXReal paramName: ''gXxmin'']';

addInstVarName: #gXxmax;
compile:
'gXxmax: aVar
	aVar notNil ifTrue:[gXxmax _ self typeCheck: aVar shouldBe: GXReal paramName: ''gXxmax'']';
compile:
'p6: aVar
	aVar notNil ifTrue:[gXxmax _ self typeCheck: aVar shouldBe: GXReal paramName: ''gXxmax'']';

addInstVarName: #gXymin;
compile:
'gXymin: aVar
	aVar notNil ifTrue:[gXymin _ self typeCheck: aVar shouldBe: GXReal paramName: ''gXymin'']';
compile:
'p7: aVar
	aVar notNil ifTrue:[gXymin _ self typeCheck: aVar shouldBe: GXReal paramName: ''gXymin'']';

addInstVarName: #gXymax;
compile:
'gXymax: aVar
	aVar notNil ifTrue:[gXymax _ self typeCheck: aVar shouldBe: GXReal paramName: ''gXymax'']';
compile:
'p8: aVar
	aVar notNil ifTrue:[gXymax _ self typeCheck: aVar shouldBe: GXReal paramName: ''gXymax'']'
!

!GXlimits methodsFor: 'initializing'!

initVars
	gXx _ false.
	gXy _ false.
	gXnoUauto _ false.
	gXdefault _ false
!!

"BODY
 ----"

!GXlimits methodsFor: 'running'!

evaluate

        ( gXp1 isKindOf: Integer ) ifTrue: [ gXp1 _ gXp1 * 1.0 ].
        ( gXp2 isKindOf: Integer ) ifTrue: [ gXp2 _ gXp2 * 1.0 ].
        ( gXp3 isKindOf: Integer ) ifTrue: [ gXp3 _ gXp3 * 1.0 ].
        ( gXp4 isKindOf: Integer ) ifTrue: [ gXp4 _ gXp4 * 1.0 ]
.
(gXx) ifTrue: [ 

	    ( gXp1 isKindOf: Float ) ifTrue: [ gXxmin _ gXp1 ].
	    ( gXp2 isKindOf: Float ) ifTrue: [ gXxmax _ gXp2 ].
	    gXxmin isNil ifTrue: [^'LIMITS: Parameter XMIN not specified' prenl ].
	    gXxmax isNil ifTrue: [^'LIMITS: Parameter XMAX not specified' prenl ]
	.
((gXUsdjUgw) at:#gXxlog) ifTrue: [ 
(gXUsdjUgw isKindOf: GXArray) | (gXUsdjUgw isKindOf: GXWorkspace)
    ifTrue:  [ ((gXUsdjUgw) at:#gXxminUlimit put: (GXlog run line: 0 proc: self method: true; p1:(gXxmin); condEvaluate) deepCopy) ]
    ifFalse: [ self error: 'Assignment to array or workspace failed, check that the destination is a structured type'].
(gXUsdjUgw isKindOf: GXArray) | (gXUsdjUgw isKindOf: GXWorkspace)
    ifTrue:  [ ((gXUsdjUgw) at:#gXxmaxUlimit put: (GXlog run line: 0 proc: self method: true; p1:(gXxmax); condEvaluate) deepCopy) ]
    ifFalse: [ self error: 'Assignment to array or workspace failed, check that the destination is a structured type'] ] ifFalse: [ 
(gXUsdjUgw isKindOf: GXArray) | (gXUsdjUgw isKindOf: GXWorkspace)
    ifTrue:  [ ((gXUsdjUgw) at:#gXxminUlimit put: (gXxmin) deepCopy) ]
    ifFalse: [ self error: 'Assignment to array or workspace failed, check that the destination is a structured type'].
(gXUsdjUgw isKindOf: GXArray) | (gXUsdjUgw isKindOf: GXWorkspace)
    ifTrue:  [ ((gXUsdjUgw) at:#gXxmaxUlimit put: (gXxmax) deepCopy) ]
    ifFalse: [ self error: 'Assignment to array or workspace failed, check that the destination is a structured type'] ]  ] ifFalse: [ (gXy) ifTrue: [ 

	    ( gXp1 isKindOf: Float ) ifTrue: [ gXymin _ gXp1 ].
	    ( gXp2 isKindOf: Float ) ifTrue: [ gXymax _ gXp2 ].
	    gXymin isNil ifTrue: [^'LIMITS: Parameter YMIN not specified' prenl ].
	    gXymax isNil ifTrue: [^'LIMITS: Parameter YMAX not specified' prenl ]
	.
((gXUsdjUgw) at:#gXylog) ifTrue: [ 
(gXUsdjUgw isKindOf: GXArray) | (gXUsdjUgw isKindOf: GXWorkspace)
    ifTrue:  [ ((gXUsdjUgw) at:#gXyminUlimit put: (GXlog run line: 0 proc: self method: true; p1:(gXymin); condEvaluate) deepCopy) ]
    ifFalse: [ self error: 'Assignment to array or workspace failed, check that the destination is a structured type'].
(gXUsdjUgw isKindOf: GXArray) | (gXUsdjUgw isKindOf: GXWorkspace)
    ifTrue:  [ ((gXUsdjUgw) at:#gXymaxUlimit put: (GXlog run line: 0 proc: self method: true; p1:(gXymax); condEvaluate) deepCopy) ]
    ifFalse: [ self error: 'Assignment to array or workspace failed, check that the destination is a structured type'] ] ifFalse: [ 
(gXUsdjUgw isKindOf: GXArray) | (gXUsdjUgw isKindOf: GXWorkspace)
    ifTrue:  [ ((gXUsdjUgw) at:#gXyminUlimit put: (gXymin) deepCopy) ]
    ifFalse: [ self error: 'Assignment to array or workspace failed, check that the destination is a structured type'].
(gXUsdjUgw isKindOf: GXArray) | (gXUsdjUgw isKindOf: GXWorkspace)
    ifTrue:  [ ((gXUsdjUgw) at:#gXymaxUlimit put: (gXymax) deepCopy) ]
    ifFalse: [ self error: 'Assignment to array or workspace failed, check that the destination is a structured type'] ]  ] ifFalse: [ (gXnoUauto) ifTrue: [ 

	    ( gXp1 isKindOf: Float ) ifTrue: [ gXxmin _ gXp1 ].
	    ( gXp2 isKindOf: Float ) ifTrue: [ gXxmax _ gXp2 ].
	    ( gXp3 isKindOf: Float ) ifTrue: [ gXymin _ gXp3 ].
	    ( gXp4 isKindOf: Float ) ifTrue: [ gXymax _ gXp4 ].
	    gXxmin isNil ifTrue: [^'LIMITS: Parameter XMIN not specified' prenl ].
	    gXxmax isNil ifTrue: [^'LIMITS: Parameter XMAX not specified' prenl ].
	    gXymin isNil ifTrue: [^'LIMITS: Parameter YMIN not specified' prenl ].
	    gXymax isNil ifTrue: [^'LIMITS: Parameter YMAX not specified' prenl ]
	.
((gXUsdjUgw) at:#gXxlog) ifTrue: [ 
(gXUsdjUgw isKindOf: GXArray) | (gXUsdjUgw isKindOf: GXWorkspace)
    ifTrue:  [ ((gXUsdjUgw) at:#gXxminUlimit put: (GXlog run line: 0 proc: self method: true; p1:(gXxmin); condEvaluate) deepCopy) ]
    ifFalse: [ self error: 'Assignment to array or workspace failed, check that the destination is a structured type'].
(gXUsdjUgw isKindOf: GXArray) | (gXUsdjUgw isKindOf: GXWorkspace)
    ifTrue:  [ ((gXUsdjUgw) at:#gXxmaxUlimit put: (GXlog run line: 0 proc: self method: true; p1:(gXxmax); condEvaluate) deepCopy) ]
    ifFalse: [ self error: 'Assignment to array or workspace failed, check that the destination is a structured type'] ] ifFalse: [ 
(gXUsdjUgw isKindOf: GXArray) | (gXUsdjUgw isKindOf: GXWorkspace)
    ifTrue:  [ ((gXUsdjUgw) at:#gXxminUlimit put: (gXxmin) deepCopy) ]
    ifFalse: [ self error: 'Assignment to array or workspace failed, check that the destination is a structured type'].
(gXUsdjUgw isKindOf: GXArray) | (gXUsdjUgw isKindOf: GXWorkspace)
    ifTrue:  [ ((gXUsdjUgw) at:#gXxmaxUlimit put: (gXxmax) deepCopy) ]
    ifFalse: [ self error: 'Assignment to array or workspace failed, check that the destination is a structured type'] ] .
((gXUsdjUgw) at:#gXylog) ifTrue: [ 
(gXUsdjUgw isKindOf: GXArray) | (gXUsdjUgw isKindOf: GXWorkspace)
    ifTrue:  [ ((gXUsdjUgw) at:#gXyminUlimit put: (GXlog run line: 0 proc: self method: true; p1:(gXymin); condEvaluate) deepCopy) ]
    ifFalse: [ self error: 'Assignment to array or workspace failed, check that the destination is a structured type'].
(gXUsdjUgw isKindOf: GXArray) | (gXUsdjUgw isKindOf: GXWorkspace)
    ifTrue:  [ ((gXUsdjUgw) at:#gXymaxUlimit put: (GXlog run line: 0 proc: self method: true; p1:(gXymax); condEvaluate) deepCopy) ]
    ifFalse: [ self error: 'Assignment to array or workspace failed, check that the destination is a structured type'] ] ifFalse: [ 
(gXUsdjUgw isKindOf: GXArray) | (gXUsdjUgw isKindOf: GXWorkspace)
    ifTrue:  [ ((gXUsdjUgw) at:#gXyminUlimit put: (gXymin) deepCopy) ]
    ifFalse: [ self error: 'Assignment to array or workspace failed, check that the destination is a structured type'].
(gXUsdjUgw isKindOf: GXArray) | (gXUsdjUgw isKindOf: GXWorkspace)
    ifTrue:  [ ((gXUsdjUgw) at:#gXymaxUlimit put: (gXymax) deepCopy) ]
    ifFalse: [ self error: 'Assignment to array or workspace failed, check that the destination is a structured type'] ]  ] ifFalse: [ 
(gXUsdjUgw isKindOf: GXArray) | (gXUsdjUgw isKindOf: GXWorkspace)
    ifTrue:  [ ((gXUsdjUgw) at:#gXxminUlimit put: (gXU) deepCopy) ]
    ifFalse: [ self error: 'Assignment to array or workspace failed, check that the destination is a structured type'].
(gXUsdjUgw isKindOf: GXArray) | (gXUsdjUgw isKindOf: GXWorkspace)
    ifTrue:  [ ((gXUsdjUgw) at:#gXxmaxUlimit put: (gXU) deepCopy) ]
    ifFalse: [ self error: 'Assignment to array or workspace failed, check that the destination is a structured type'].
(gXUsdjUgw isKindOf: GXArray) | (gXUsdjUgw isKindOf: GXWorkspace)
    ifTrue:  [ ((gXUsdjUgw) at:#gXyminUlimit put: (gXU) deepCopy) ]
    ifFalse: [ self error: 'Assignment to array or workspace failed, check that the destination is a structured type'].
(gXUsdjUgw isKindOf: GXArray) | (gXUsdjUgw isKindOf: GXWorkspace)
    ifTrue:  [ ((gXUsdjUgw) at:#gXymaxUlimit put: (gXU) deepCopy) ]
    ifFalse: [ self error: 'Assignment to array or workspace failed, check that the destination is a structured type'] ] ]].
self up.
^nil
!!
nil! "(fix for smalltalk bug, this line is ignored if there is a variable declaration error)"

"End of procedure"

GXcomment run line: 0 proc: self method: false; p1:'limits'; p2:'Sets graph limits for DISPLAY command'; condEvaluate!
"Procedure class definition:
 ==========================="
((Smalltalk at: #GXplot ifAbsent: [ nil ]) isKindOf: Function class) ifTrue: [ ('Warning: Overriding previously defined procedure ', 'GXplot' gtype) prinl ].
Smalltalk at: #GXplot put: nil!
Function subclass: 'GXplot'
       instanceVariableNames: ''
       classVariableNames: ''
       poolDictionaries: ''
       category: 'Genie Procedure'
!

"HEADER
 ------"

"qualifiers
 ----------"

GXplot

addInstVarName: #gXhistogram;
compile:
'gXhistogram
	gXhistogram _ true';
compile:
'gXhistogram: aBoolean
	gXhistogram _ aBoolean';

addInstVarName: #gXline;
compile:
'gXline
	gXline _ true';
compile:
'gXline: aBoolean
	gXline _ aBoolean';

addInstVarName: #gXmarkers;
compile:
'gXmarkers
	gXmarkers _ true';
compile:
'gXmarkers: aBoolean
	gXmarkers _ aBoolean';

addInstVarName: #gXerrors;
compile:
'gXerrors
	gXerrors _ true';
compile:
'gXerrors: aBoolean
	gXerrors _ aBoolean'!

"parameters
 ----------"

GXplot

addInstVarName: #gXwksp;
compile:
'gXwksp: aVar
	aVar notNil ifTrue:[gXwksp _ self typeCheck: aVar shouldBe: GXWorkspace paramName: ''gXwksp'']';
compile:
'p1: aVar
	aVar notNil ifTrue:[gXwksp _ self typeCheck: aVar shouldBe: GXWorkspace paramName: ''gXwksp'']';

addInstVarName: #gXbinning;
compile:
'gXbinning: aVar
	aVar notNil ifTrue:[gXbinning _ self typeCheck: aVar shouldBe: GXInteger paramName: ''gXbinning'']';
compile:
'p2: aVar
	aVar notNil ifTrue:[gXbinning _ self typeCheck: aVar shouldBe: GXInteger paramName: ''gXbinning'']'
!

"Local variables
 ---------------"

GXplot

addInstVarName: #gXbinning!

"Local variables
 ---------------"

GXplot

addInstVarName: #gXtype!

"Local variables
 ---------------"

GXplot

addInstVarName: #gXx!

"Local variables
 ---------------"

GXplot

addInstVarName: #gXplotUspect
!

!GXplot methodsFor: 'initializing'!

initVars
	gXhistogram _ false.
	gXline _ false.
	gXmarkers _ false.
	gXerrors _ false.
	gXbinning _ (gXUcurrentUbinning).
	gXtype _ 1.
	gXplotUspect _ (gXU)
!!

"BODY
 ----"

!GXplot methodsFor: 'running'!

evaluate
gXx _ (GXUUg2Uopendev run line: 0 proc: self method: true; condEvaluate) deepCopy.
(gXmarkers) ifTrue: [ 
gXtype _ 2 deepCopy ] ifFalse: [ (gXline) ifTrue: [ 
gXtype _ 3 deepCopy ] ifFalse: [ (gXerrors) ifTrue: [ 
gXtype _ 4 deepCopy ]]].
(GXdefined run line: 0 proc: self method: true; p1:(gXwksp); condEvaluate) ifTrue: [ 
gXUgrUwksp _ (GXgroupbins run line: 0 proc: self method: true; p1:(gXwksp); p2:(gXbinning); p3:0.000000000000000e000; condEvaluate) .
GXprintin run line: 0 proc: self method: false; p1:'  Plotted using bin-grouping of '; p2:(gXbinning); condEvaluate ].
 (gXUgrUwksp isKindOf: GXWorkspace) ifFalse: [ ^'PLOT: Parameter WKSP must be specified' prenl ] .
GXUUg2Udrawplot run line: 0 proc: self method: false; p1:(gXtype); p2:((gXUgrUwksp) at:#gXx); p3:((gXUgrUwksp) at:#gXy); p4:((gXUgrUwksp) at:#gXe); condEvaluate.
GXgUerrorUhandler run line: 0 proc: self method: false; condEvaluate.
self up.
^nil
!!
nil! "(fix for smalltalk bug, this line is ignored if there is a variable declaration error)"

"End of procedure"

GXcomment run line: 0 proc: self method: false; p1:'plot'; p2:'Overplots an existing graph (Genie II style)'; condEvaluate!
"Procedure class definition:
 ==========================="
((Smalltalk at: #GXUUg2Uzoomaxes ifAbsent: [ nil ]) isKindOf: Function class) ifTrue: [ ('Warning: Overriding previously defined procedure ', 'GXUUg2Uzoomaxes' gtype) prinl ].
Smalltalk at: #GXUUg2Uzoomaxes put: nil!
Function subclass: 'GXUUg2Uzoomaxes'
       instanceVariableNames: ''
       classVariableNames: ''
       poolDictionaries: ''
       category: 'Genie Procedure'
!

"HEADER
 ------"

"parameters
 ----------"

GXUUg2Uzoomaxes

addInstVarName: #gXwxmin;
compile:
'gXwxmin: aVar
	aVar notNil ifTrue:[gXwxmin _ self typeCheck: aVar shouldBe: GXReal paramName: ''gXwxmin'']';
compile:
'p1: aVar
	aVar notNil ifTrue:[gXwxmin _ self typeCheck: aVar shouldBe: GXReal paramName: ''gXwxmin'']';

addInstVarName: #gXwxmax;
compile:
'gXwxmax: aVar
	aVar notNil ifTrue:[gXwxmax _ self typeCheck: aVar shouldBe: GXReal paramName: ''gXwxmax'']';
compile:
'p2: aVar
	aVar notNil ifTrue:[gXwxmax _ self typeCheck: aVar shouldBe: GXReal paramName: ''gXwxmax'']';

addInstVarName: #gXwymin;
compile:
'gXwymin: aVar
	aVar notNil ifTrue:[gXwymin _ self typeCheck: aVar shouldBe: GXReal paramName: ''gXwymin'']';
compile:
'p3: aVar
	aVar notNil ifTrue:[gXwymin _ self typeCheck: aVar shouldBe: GXReal paramName: ''gXwymin'']';

addInstVarName: #gXwymax;
compile:
'gXwymax: aVar
	aVar notNil ifTrue:[gXwymax _ self typeCheck: aVar shouldBe: GXReal paramName: ''gXwymax'']';
compile:
'p4: aVar
	aVar notNil ifTrue:[gXwymax _ self typeCheck: aVar shouldBe: GXReal paramName: ''gXwymax'']'
!

"Local variables
 ---------------"

GXUUg2Uzoomaxes

addInstVarName: #gXxmin!

"Local variables
 ---------------"

GXUUg2Uzoomaxes

addInstVarName: #gXxmax!

"Local variables
 ---------------"

GXUUg2Uzoomaxes

addInstVarName: #gXymin!

"Local variables
 ---------------"

GXUUg2Uzoomaxes

addInstVarName: #gXymax!

"Local variables
 ---------------"

GXUUg2Uzoomaxes

addInstVarName: #gXxoff!

"Local variables
 ---------------"

GXUUg2Uzoomaxes

addInstVarName: #gXyoff!

"Local variables
 ---------------"

GXUUg2Uzoomaxes

addInstVarName: #gXltop!

"Local variables
 ---------------"

GXUUg2Uzoomaxes

addInstVarName: #gXpUc!

"Local variables
 ---------------"

GXUUg2Uzoomaxes

addInstVarName: #gXlUc!

"Local variables
 ---------------"

GXUUg2Uzoomaxes

addInstVarName: #gXtUc!

"Local variables
 ---------------"

GXUUg2Uzoomaxes

addInstVarName: #gXtUh!

"Local variables
 ---------------"

GXUUg2Uzoomaxes

addInstVarName: #gXlUw
!

!GXUUg2Uzoomaxes methodsFor: 'initializing'!

initVars
	gXpUc _ ((gXUsdjUgw) at:#gXplotcolour).
	gXlUc _ ((gXUsdjUgw) at:#gXlinecolour).
	gXtUc _ ((gXUsdjUgw) at:#gXtextcolour).
	gXtUh _ ((gXUsdjUgw) at:#gXtextht).
	gXlUw _ ((gXUsdjUgw) at:#gXlinewidth)
!!

"BODY
 ----"

!GXUUg2Uzoomaxes methodsFor: 'running'!

evaluate
((gXUsdjUgw) at:#gXheader) ifTrue: [ 
gXltop _ 8.000000000000000e-01 deepCopy ] ifFalse: [ 
gXltop _ 1.000000000000000e000 deepCopy ] .
gXxmin _ ((gXUsdjUgw) at:#gXleft) deepCopy.
gXxmax _ ((gXUsdjUgw) at:#gXright) deepCopy.
gXymin _ ((gXUsdjUgw) at:#gXbottom) deepCopy.
gXymax _ (((gXUsdjUgw) at:#gXtop) * (gXltop)) deepCopy.
gXxoff _ (1.200000000000000e-01 * (((gXxmax) - (gXxmin)))) deepCopy.
gXyoff _ (1.100000000000000e-01 * (((gXymax) - (gXymin)))) deepCopy.
gXxmin _ ((gXxmin) + (gXxoff)) deepCopy.
gXxmax _ ((gXxmax) - (gXxoff)) deepCopy.
gXymin _ ((gXymin) + (gXyoff)) deepCopy.
gXymax _ ((gXymax) - (gXyoff)) deepCopy.
GXwinUscaled run line: 0 proc: self method: false; p1:(gXxmin); p2:(gXxmax); p3:(gXymin); p4:(gXymax); p5:(gXwxmin); p6:(gXwxmax); p7:(gXwymin); p8:(gXwymax); condEvaluate.
(GXgUerrorUhandler run line: 0 proc: self method: true; condEvaluate) ifTrue: [ 
[nil.
self up.
^nil] value ].

		GXaxes run line: 0 proc: self method: false; gXdraw;
			gXxlog: (gXUsdjUgw at: #gXxlog);
			gXylog: (gXUsdjUgw at: #gXylog);
			p1: gXlUc; p2: gXtUh; p3: (gXUsdjUgw at: #gXfont);
			p4: gXlUw; condEvaluate
	.
((gXUsdjUgw) at:#gXgraticule) ifTrue: [ 

		GXgraticule run line: 0 proc: self method: false;
		    gXdraw; gXxlog: (gXUsdjUgw at: #gXxlog); gXylog: (gXUsdjUgw at: #gXylog);
		    p1: gXlUc; p4: gXlUw; condEvaluate
	 ].
GXgUerrorUhandler run line: 0 proc: self method: false; condEvaluate.
self up.
^nil
!!
nil! "(fix for smalltalk bug, this line is ignored if there is a variable declaration error)"

"End of procedure"

"Procedure class definition:
 ==========================="
((Smalltalk at: #GXzoom ifAbsent: [ nil ]) isKindOf: Function class) ifTrue: [ ('Warning: Overriding previously defined procedure ', 'GXzoom' gtype) prinl ].
Smalltalk at: #GXzoom put: nil!
Function subclass: 'GXzoom'
       instanceVariableNames: ''
       classVariableNames: ''
       poolDictionaries: ''
       category: 'Genie Procedure'
!

"HEADER
 ------"

"qualifiers
 ----------"

GXzoom

addInstVarName: #gXerrors;
compile:
'gXerrors
	gXerrors _ true';
compile:
'gXerrors: aBoolean
	gXerrors _ aBoolean';

addInstVarName: #gXmarkers;
compile:
'gXmarkers
	gXmarkers _ true';
compile:
'gXmarkers: aBoolean
	gXmarkers _ aBoolean';

addInstVarName: #gXline;
compile:
'gXline
	gXline _ true';
compile:
'gXline: aBoolean
	gXline _ aBoolean';

addInstVarName: #gXhistogram;
compile:
'gXhistogram
	gXhistogram _ true';
compile:
'gXhistogram: aBoolean
	gXhistogram _ aBoolean';

addInstVarName: #gXmultiplot;
compile:
'gXmultiplot
	gXmultiplot _ true';
compile:
'gXmultiplot: aBoolean
	gXmultiplot _ aBoolean'
!

"Local variables
 ---------------"

GXzoom

addInstVarName: #gXxmin!

"Local variables
 ---------------"

GXzoom

addInstVarName: #gXxmax!

"Local variables
 ---------------"

GXzoom

addInstVarName: #gXymin!

"Local variables
 ---------------"

GXzoom

addInstVarName: #gXymax!

"Local variables
 ---------------"

GXzoom

addInstVarName: #gXcursor!

"Local variables
 ---------------"

GXzoom

addInstVarName: #gXt
!

!GXzoom methodsFor: 'initializing'!

initVars
	gXerrors _ false.
	gXmarkers _ false.
	gXline _ false.
	gXhistogram _ false.
	gXmultiplot _ false
!!

"BODY
 ----"

!GXzoom methodsFor: 'running'!

evaluate
gXt _ (GXUUg2Uopendev run line: 0 proc: self method: true; condEvaluate) deepCopy.
gXcursor _ (GXgetcursor run line: 0 proc: self method: true; p1:0.000000000000000e000; p2:0.000000000000000e000; condEvaluate) deepCopy.
gXxmin _ ((gXcursor) at:#gXwUx) deepCopy.
gXymin _ ((gXcursor) at:#gXwUy) deepCopy.
gXcursor _ (GXgetcursor run line: 0 proc: self method: true; p1:(gXxmin); p2:(gXymin); condEvaluate) deepCopy.
gXxmax _ ((gXcursor) at:#gXwUx) deepCopy.
gXymax _ ((gXcursor) at:#gXwUy) deepCopy.

    (gXUsdjUgw at: #gXxlog) ifTrue: [ gXxmin _ 10.0 raisedTo: gXxmin. 
					gXxmax _ 10.0 raisedTo: gXxmax ].
    (gXUsdjUgw at: #gXylog) ifTrue: [ gXymin _ 10.0 raisedTo: gXymin. 
					gXymax _ 10.0 raisedTo: gXymax ]
.
((gXxmax) < (gXxmin)) ifTrue: [ 
gXt _ (gXxmax) deepCopy.
gXxmax _ (gXxmin) deepCopy.
gXxmin _ (gXt) deepCopy ].
((gXymax) < (gXymin)) ifTrue: [ 
gXt _ (gXymax) deepCopy.
gXymax _ (gXymin) deepCopy.
gXymin _ (gXt) deepCopy ].
(GXgUerrorUhandler run line: 0 proc: self method: true; condEvaluate) ifTrue: [ 
[nil.
self up.
^nil] value ].
GXlimits run line: 0 proc: self method: false; gXnoUauto; p1:(gXxmin); p2:(gXxmax); p3:(gXymin); p4:(gXymax); condEvaluate.
(gXmarkers) ifTrue: [ 
GXdisplay run line: 0 proc: self method: false; gXmarkers; condEvaluate ] ifFalse: [ (gXerrors) ifTrue: [ 
GXdisplay run line: 0 proc: self method: false; gXerrors; condEvaluate ] ifFalse: [ (gXline) ifTrue: [ 
GXdisplay run line: 0 proc: self method: false; gXline; condEvaluate ] ifFalse: [ (gXmultiplot) ifTrue: [ 
GXdevice run line: 0 proc: self method: false; gXclear; condEvaluate.
GXredraw run line: 0 proc: self method: false; condEvaluate.
GXUUg2Uzoomaxes run line: 0 proc: self method: false; p1:(gXxmin); p2:(gXxmax); p3:(gXymin); p4:(gXymax); condEvaluate.
(GXgUerrorUhandler run line: 0 proc: self method: true; condEvaluate) ifTrue: [ 
[nil.
self up.
^nil] value ].
GXmultiUplot run line: 0 proc: self method: false; gXdraw; condEvaluate ] ifFalse: [ 
GXdisplay run line: 0 proc: self method: false; gXhistogram; condEvaluate ] ]]].
self up.
^nil
!!
nil! "(fix for smalltalk bug, this line is ignored if there is a variable declaration error)"

"End of procedure"

GXcomment run line: 0 proc: self method: false; p1:'zoom'; p2:'Uses cursor to zoom in on a plot'; condEvaluate!
"Procedure class definition:
 ==========================="
((Smalltalk at: #GXUmultiplotUfinish ifAbsent: [ nil ]) isKindOf: Function class) ifTrue: [ ('Warning: Overriding previously defined procedure ', 'GXUmultiplotUfinish' gtype) prinl ].
Smalltalk at: #GXUmultiplotUfinish put: nil!
Function subclass: 'GXUmultiplotUfinish'
       instanceVariableNames: ''
       classVariableNames: ''
       poolDictionaries: ''
       category: 'Genie Procedure'
!

"HEADER
 ------"

"parameters
 ----------"

GXUmultiplotUfinish

addInstVarName: #gXwksp;
compile:
'gXwksp: aVar
	aVar notNil ifTrue:[gXwksp _ self typeCheck: aVar shouldBe: GXWorkspace paramName: ''gXwksp'']';
compile:
'p1: aVar
	aVar notNil ifTrue:[gXwksp _ self typeCheck: aVar shouldBe: GXWorkspace paramName: ''gXwksp'']';

addInstVarName: #gXtUc;
compile:
'gXtUc: aVar
	aVar notNil ifTrue:[gXtUc _ self typeCheck: aVar shouldBe: GXColour paramName: ''gXtUc'']';
compile:
'p2: aVar
	aVar notNil ifTrue:[gXtUc _ self typeCheck: aVar shouldBe: GXColour paramName: ''gXtUc'']';

addInstVarName: #gXtUh;
compile:
'gXtUh: aVar
	aVar notNil ifTrue:[gXtUh _ self typeCheck: aVar shouldBe: GXReal paramName: ''gXtUh'']';
compile:
'p3: aVar
	aVar notNil ifTrue:[gXtUh _ self typeCheck: aVar shouldBe: GXReal paramName: ''gXtUh'']';

addInstVarName: #gXlUc;
compile:
'gXlUc: aVar
	aVar notNil ifTrue:[gXlUc _ self typeCheck: aVar shouldBe: GXColour paramName: ''gXlUc'']';
compile:
'p4: aVar
	aVar notNil ifTrue:[gXlUc _ self typeCheck: aVar shouldBe: GXColour paramName: ''gXlUc'']';

addInstVarName: #gXlUw;
compile:
'gXlUw: aVar
	aVar notNil ifTrue:[gXlUw _ self typeCheck: aVar shouldBe: GXReal paramName: ''gXlUw'']';
compile:
'p5: aVar
	aVar notNil ifTrue:[gXlUw _ self typeCheck: aVar shouldBe: GXReal paramName: ''gXlUw'']';

addInstVarName: #gXgrouping;
compile:
'gXgrouping: aVar
	aVar notNil ifTrue:[gXgrouping _ self typeCheck: aVar shouldBe: GXString paramName: ''gXgrouping'']';
compile:
'p6: aVar
	aVar notNil ifTrue:[gXgrouping _ self typeCheck: aVar shouldBe: GXString paramName: ''gXgrouping'']'
!

"Local variables
 ---------------"

GXUmultiplotUfinish

addInstVarName: #gXgrouping
!

!GXUmultiplotUfinish methodsFor: 'initializing'!

initVars

!!

"BODY
 ----"

!GXUmultiplotUfinish methodsFor: 'running'!

evaluate
((gXUsdjUgw) at:#gXcls) ifTrue: [ 
GXdevice run line: 0 proc: self method: false; gXclear; condEvaluate ].
GXUUg2Uaxes run line: 0 proc: self method: false; p1:5; condEvaluate.
(GXgUerrorUhandler run line: 0 proc: self method: true; condEvaluate) ifTrue: [ 
[nil.
self up.
^nil] value ].
GXmultiUplot run line: 0 proc: self method: false; gXdraw; condEvaluate.
((gXUsdjUgw) at:#gXgraticule) ifTrue: [ 

		GXgraticule run line: 0 proc: self method: false;
		    gXdraw; gXxlog: (gXUsdjUgw at: #gXxlog); gXylog: (gXUsdjUgw at: #gXylog);
		    p1: gXlUc; p4: gXlUw; condEvaluate
	 ].
GXtitle run line: 0 proc: self method: false; gXdraw; p1:((gXwksp) at:#gXtitle); p2:(gXtUc); p3:(gXtUh); p4:((gXUsdjUgw) at:#gXfont); condEvaluate.
GXlabels run line: 0 proc: self method: false; gXdraw; p1:((gXwksp) at:#gXxlabel); p2:((gXwksp) at:#gXylabel); p3:(gXtUh); p4:(gXtUc); p5:((gXUsdjUgw) at:#gXfont); condEvaluate.
GXgUerrorUhandler run line: 0 proc: self method: false; condEvaluate.
((gXUsdjUgw) at:#gXheader) ifTrue: [ 
gXgrouping _ (GXasUstring run line: 0 proc: self method: true; p1:(gXUcurrentUbinning); condEvaluate) deepCopy.
GXUUg2Ubanner run line: 0 proc: self method: false; p1:((gXwksp) at:#gXinstUname); p2:((gXwksp) at:#gXrunUno); p3:((gXwksp) at:#gXspecUno); p4:((gXwksp) at:#gXfile); p5:((gXwksp) at:#gXuserUname); p6:((gXwksp) at:#gXtime); p7:(gXgrouping); condEvaluate ].
GXUUg2Uaxes run line: 0 proc: self method: false; gXredraw; p1:5; condEvaluate.
GXgUerrorUhandler run line: 0 proc: self method: false; condEvaluate.
self up.
^nil
!!
nil! "(fix for smalltalk bug, this line is ignored if there is a variable declaration error)"

"End of procedure"

GXcomment run line: 0 proc: self method: false; p1:'_multiplot_finish'; p2:'Display the completed multiplot on the screen'; condEvaluate!
"Procedure class definition:
 ==========================="
((Smalltalk at: #GXUmultiplotUinit ifAbsent: [ nil ]) isKindOf: Function class) ifTrue: [ ('Warning: Overriding previously defined procedure ', 'GXUmultiplotUinit' gtype) prinl ].
Smalltalk at: #GXUmultiplotUinit put: nil!
Function subclass: 'GXUmultiplotUinit'
       instanceVariableNames: ''
       classVariableNames: ''
       poolDictionaries: ''
       category: 'Genie Procedure'
!

"HEADER
 ------"

"parameters
 ----------"

GXUmultiplotUinit

addInstVarName: #gXfile;
compile:
'gXfile: aVar
	aVar notNil ifTrue:[gXfile _ self typeCheck: aVar shouldBe: GXString paramName: ''gXfile'']';
compile:
'p1: aVar
	aVar notNil ifTrue:[gXfile _ self typeCheck: aVar shouldBe: GXString paramName: ''gXfile'']';

addInstVarName: #gXbin;
compile:
'gXbin: aVar
	aVar notNil ifTrue:[ gXbin _ aVar ]';
compile:
'p2: aVar
	aVar notNil ifTrue:[ gXbin _ aVar ]';

addInstVarName: #gXwa;
compile:
'gXwa: aVar
	aVar notNil ifTrue:[gXwa _ self typeCheck: aVar shouldBe: GXWorkspace paramName: ''gXwa'']';
compile:
'p3: aVar
	aVar notNil ifTrue:[gXwa _ self typeCheck: aVar shouldBe: GXWorkspace paramName: ''gXwa'']';

addInstVarName: #gXspectraUarray;
compile:
'gXspectraUarray: aVar
	aVar notNil ifTrue:[ gXspectraUarray _ aVar ]';
compile:
'p4: aVar
	aVar notNil ifTrue:[ gXspectraUarray _ aVar ]'!

"result
 ------"

GXUmultiplotUinit

addInstVarName: #gXoldUntc
!

"Local variables
 ---------------"

GXUmultiplotUinit

addInstVarName: #gXx
!

!GXUmultiplotUinit methodsFor: 'initializing'!

initVars

!!

"BODY
 ----"

!GXUmultiplotUinit methodsFor: 'running'!

evaluate
(GXdefined run line: 0 proc: self method: true; p1:(gXwa); condEvaluate) ifTrue: [ 
gXUgrUwksp _ (GXgroupbins run line: 0 proc: self method: true; p1:(gXwa); p2:(gXUcurrentUbinning); p3:0.000000000000000e000; condEvaluate) deepCopy.
 gXoldUntc _ (((gXwa at: #gXy) dimensions) at: 2) .
gXx _ (GXUUg2Uopendev run line: 0 proc: self method: true; condEvaluate) deepCopy ] ifFalse: [ (GXdefined run line: 0 proc: self method: true; p1:(gXspectraUarray); condEvaluate) ifTrue: [ 
gXUgrUwksp _ (GXUgetUsnsUspectrum run line: 0 proc: self method: true; p1:0; p2:((gXspectraUarray) at:((Array new:1) at: 1 put: 1; yourself)); p3:(gXfile); condEvaluate) deepCopy.
(GXdefined run line: 0 proc: self method: true; p1:(gXUgrUwksp); condEvaluate) ifTrue: [ 
gXoldUntc _ (GXlength run line: 0 proc: self method: true; p1:((gXUgrUwksp) at:#gXy); condEvaluate) deepCopy.
gXUgrUwksp _ (GXgroupbins run line: 0 proc: self method: true; p1:(gXUgrUwksp); p2:(gXUcurrentUbinning); p3:0.000000000000000e000; condEvaluate) deepCopy.
gXx _ (GXUUg2Uopendev run line: 0 proc: self method: true; condEvaluate) deepCopy ] ] ifFalse: [ 
GXprinten run line: 0 proc: self method: false; p1:'MULTIPLOT: No spectra specified'; condEvaluate ] ].
self up.
^gXoldUntc
!!
nil! "(fix for smalltalk bug, this line is ignored if there is a variable declaration error)"

"End of procedure"

"Procedure class definition:
 ==========================="
((Smalltalk at: #GXmultiplot ifAbsent: [ nil ]) isKindOf: Function class) ifTrue: [ ('Warning: Overriding previously defined procedure ', 'GXmultiplot' gtype) prinl ].
Smalltalk at: #GXmultiplot put: nil!
Function subclass: 'GXmultiplot'
       instanceVariableNames: ''
       classVariableNames: ''
       poolDictionaries: ''
       category: 'Genie Procedure'
!

"HEADER
 ------"

"parameters
 ----------"

GXmultiplot

addInstVarName: #gXp1;
compile:
'gXp1: aVar
	aVar notNil ifTrue:[ gXp1 _ aVar ]';
compile:
'p1: aVar
	aVar notNil ifTrue:[ gXp1 _ aVar ]';

addInstVarName: #gXp2;
compile:
'gXp2: aVar
	aVar notNil ifTrue:[ gXp2 _ aVar ]';
compile:
'p2: aVar
	aVar notNil ifTrue:[ gXp2 _ aVar ]';

addInstVarName: #gXp3;
compile:
'gXp3: aVar
	aVar notNil ifTrue:[ gXp3 _ aVar ]';
compile:
'p3: aVar
	aVar notNil ifTrue:[ gXp3 _ aVar ]';

addInstVarName: #gXp4;
compile:
'gXp4: aVar
	aVar notNil ifTrue:[ gXp4 _ aVar ]';
compile:
'p4: aVar
	aVar notNil ifTrue:[ gXp4 _ aVar ]';

addInstVarName: #gXp5;
compile:
'gXp5: aVar
	aVar notNil ifTrue:[ gXp5 _ aVar ]';
compile:
'p5: aVar
	aVar notNil ifTrue:[ gXp5 _ aVar ]';

addInstVarName: #gXp6;
compile:
'gXp6: aVar
	aVar notNil ifTrue:[ gXp6 _ aVar ]';
compile:
'p6: aVar
	aVar notNil ifTrue:[ gXp6 _ aVar ]';

addInstVarName: #gXp7;
compile:
'gXp7: aVar
	aVar notNil ifTrue:[ gXp7 _ aVar ]';
compile:
'p7: aVar
	aVar notNil ifTrue:[ gXp7 _ aVar ]';

addInstVarName: #gXp8;
compile:
'gXp8: aVar
	aVar notNil ifTrue:[ gXp8 _ aVar ]';
compile:
'p8: aVar
	aVar notNil ifTrue:[ gXp8 _ aVar ]';

addInstVarName: #gXp9;
compile:
'gXp9: aVar
	aVar notNil ifTrue:[ gXp9 _ aVar ]';
compile:
'p9: aVar
	aVar notNil ifTrue:[ gXp9 _ aVar ]';

addInstVarName: #gXspectra;
compile:
'gXspectra: aVar
	aVar notNil ifTrue:[ gXspectra _ aVar ]';
compile:
'p10: aVar
	aVar notNil ifTrue:[ gXspectra _ aVar ]';

addInstVarName: #gXwa;
compile:
'gXwa: aVar
	aVar notNil ifTrue:[gXwa _ self typeCheck: aVar shouldBe: GXWorkspacearray paramName: ''gXwa'']';
compile:
'p11: aVar
	aVar notNil ifTrue:[gXwa _ self typeCheck: aVar shouldBe: GXWorkspacearray paramName: ''gXwa'']';

addInstVarName: #gXygap;
compile:
'gXygap: aVar
	aVar notNil ifTrue:[gXygap _ self typeCheck: aVar shouldBe: GXReal paramName: ''gXygap'']';
compile:
'p12: aVar
	aVar notNil ifTrue:[gXygap _ self typeCheck: aVar shouldBe: GXReal paramName: ''gXygap'']';

addInstVarName: #gXfile;
compile:
'gXfile: aVar
	aVar notNil ifTrue:[gXfile _ self typeCheck: aVar shouldBe: GXString paramName: ''gXfile'']';
compile:
'p13: aVar
	aVar notNil ifTrue:[gXfile _ self typeCheck: aVar shouldBe: GXString paramName: ''gXfile'']';

addInstVarName: #gXlinecolour;
compile:
'gXlinecolour: aVar
	aVar notNil ifTrue:[gXlinecolour _ self typeCheck: aVar shouldBe: GXColour paramName: ''gXlinecolour'']';
compile:
'p14: aVar
	aVar notNil ifTrue:[gXlinecolour _ self typeCheck: aVar shouldBe: GXColour paramName: ''gXlinecolour'']';

addInstVarName: #gXlinetype;
compile:
'gXlinetype: aVar
	aVar notNil ifTrue:[gXlinetype _ self typeCheck: aVar shouldBe: GXLinestyle paramName: ''gXlinetype'']';
compile:
'p15: aVar
	aVar notNil ifTrue:[gXlinetype _ self typeCheck: aVar shouldBe: GXLinestyle paramName: ''gXlinetype'']';

addInstVarName: #gXlinewidth;
compile:
'gXlinewidth: aVar
	aVar notNil ifTrue:[gXlinewidth _ self typeCheck: aVar shouldBe: GXInteger paramName: ''gXlinewidth'']';
compile:
'p16: aVar
	aVar notNil ifTrue:[gXlinewidth _ self typeCheck: aVar shouldBe: GXInteger paramName: ''gXlinewidth'']';

addInstVarName: #gXtextcolour;
compile:
'gXtextcolour: aVar
	aVar notNil ifTrue:[gXtextcolour _ self typeCheck: aVar shouldBe: GXColour paramName: ''gXtextcolour'']';
compile:
'p17: aVar
	aVar notNil ifTrue:[gXtextcolour _ self typeCheck: aVar shouldBe: GXColour paramName: ''gXtextcolour'']';

addInstVarName: #gXtextheight;
compile:
'gXtextheight: aVar
	aVar notNil ifTrue:[gXtextheight _ self typeCheck: aVar shouldBe: GXReal paramName: ''gXtextheight'']';
compile:
'p18: aVar
	aVar notNil ifTrue:[gXtextheight _ self typeCheck: aVar shouldBe: GXReal paramName: ''gXtextheight'']'
!

"Local variables
 ---------------"

GXmultiplot

addInstVarName: #gXnumhist!

"Local variables
 ---------------"

GXmultiplot

addInstVarName: #gXoldUntc!

"Local variables
 ---------------"

GXmultiplot

addInstVarName: #gXbin!

"Local variables
 ---------------"

GXmultiplot

addInstVarName: #gXspectraUarray
!

!GXmultiplot methodsFor: 'initializing'!

initVars
	gXnumhist _ 0
!!

"BODY
 ----"

!GXmultiplot methodsFor: 'running'!

evaluate

    gXygap isNil ifTrue: [ gXygap _ gXp2 ].
    gXfile isNil ifTrue: [ gXfile _ gXp3 ].
    gXlinecolour isNil ifTrue: [ gXlinecolour _ gXp4 ].
    gXlinetype isNil ifTrue: [ gXlinetype _ gXp5 ].
    gXlinewidth isNil ifTrue: [ gXlinewidth _ gXp6 ].
    gXtextcolour isNil ifTrue: [ gXtextcolour _ gXp7 ].
    gXtextheight isNil ifTrue: [ gXtextheight _ gXp8 ]
.

    (gXygap isKindOf: Integer) ifTrue: [ gXygap _ gXygap * 1.0 ]
.

    gXygap isNil ifTrue: [ gXygap _ ( gXUsdjUgw at: #gXygap ) shallowCopy ].
    gXfile isNil ifTrue: [ gXfile _ gXUcurrentUinputUsource ].
    gXlinecolour isNil ifTrue: [ gXlinecolour _ ( gXUsdjUgw at: #gXlinecolour ) ].
    gXlinetype isNil ifTrue: [ gXlinetype _ ( gXUsdjUgw at: #gXlinetype ) ].
    gXlinewidth isNil ifTrue: [ gXlinewidth _ ( gXUsdjUgw at: #gXlinewidth ) shallowCopy ].
    gXtextcolour isNil ifTrue: [ gXtextcolour _ ( gXUsdjUgw at: #gXtextcolour ) ].
    gXtextheight isNil ifTrue: [ gXtextheight _ ( gXUsdjUgw at: #gXtextht ) shallowCopy ]
.
 
    gXbin isNil ifTrue: [ gXbin _ ( gXUsdjUgw at: #gXbin ) shallowCopy ].
    gXspectra isNil ifTrue: [ gXspectra _ gXp1 ]
.

    (gXspectra isKindOf: Interval) ifTrue: [ gXspectraUarray _ gXspectra asIntegerArray ].
    (gXspectra isKindOf: GXIntegerarray) ifTrue: [ gXspectraUarray _ gXspectra ].
    (gXspectra isKindOf: GXWorkspace) ifTrue: [ gXwa _ gXspectra. gXspectraUarray _ (gXspectra at: #gXy) ]
.
gXoldUntc _ (GXUmultiplotUinit run line: 0 proc: self method: true; p1:(gXfile); p2:(gXbin); p3:(gXwa); p4:(gXspectraUarray); condEvaluate) deepCopy.
(GXdefined run line: 0 proc: self method: true; p1:(gXoldUntc); condEvaluate) not ifTrue: [ 
[nil.
self up.
^nil] value ].

    gXnumhist _ ((gXspectraUarray dimensions) at: 1)
.
GXmultiUplot run line: 0 proc: self method: false; gXcreate; p1:((gXUgrUwksp) at:#gXx); p2:(gXnumhist); p3:(gXU); p4:(gXU); p5:(gXygap); p6:(gXlinecolour); p7:(gXlinetype); p8:(gXlinewidth); condEvaluate.
GXmultiUplot run line: 0 proc: self method: false; gXspectra; p1:(gXspectraUarray); condEvaluate.
GXUmultiplotUfinish run line: 0 proc: self method: false; p1:(gXUgrUwksp); p2:(gXtextcolour); p3:(gXtextheight); p4:(gXlinecolour); p5:(gXlinewidth); condEvaluate.
self up.
^nil
!!
nil! "(fix for smalltalk bug, this line is ignored if there is a variable declaration error)"

"End of procedure"

GXcomment run line: 0 proc: self method: false; p1:'multiplot'; p2:'Do a multiplot (Genie II style)'; condEvaluate!
"Procedure class definition:
 ==========================="
((Smalltalk at: #GXUprintenv ifAbsent: [ nil ]) isKindOf: Function class) ifTrue: [ ('Warning: Overriding previously defined procedure ', 'GXUprintenv' gtype) prinl ].
Smalltalk at: #GXUprintenv put: nil!
Function subclass: 'GXUprintenv'
       instanceVariableNames: ''
       classVariableNames: ''
       poolDictionaries: ''
       category: 'Genie Procedure'
!

"BODY
 ----"

!GXUprintenv methodsFor: 'running'!

evaluate
GXUUprintenv run line: 0 proc: self method: false; condEvaluate.
self up.
^nil
!!
nil! "(fix for smalltalk bug, this line is ignored if there is a variable declaration error)"

"End of procedure"

GXcomment run line: 0 proc: self method: false; p1:'printenv'; p2:'Print Genie graphics information'; condEvaluate!
"Procedure class definition:
 ==========================="
((Smalltalk at: #GXinquire ifAbsent: [ nil ]) isKindOf: Function class) ifTrue: [ ('Warning: Overriding previously defined procedure ', 'GXinquire' gtype) prinl ].
Smalltalk at: #GXinquire put: nil!
Function subclass: 'GXinquire'
       instanceVariableNames: ''
       classVariableNames: ''
       poolDictionaries: ''
       category: 'Genie Procedure'
!

"HEADER
 ------"

"parameters
 ----------"

GXinquire

addInstVarName: #gXaprompt;
compile:
'gXaprompt: aVar
	aVar notNil ifTrue:[gXaprompt _ self typeCheck: aVar shouldBe: GXString paramName: ''gXaprompt'']';
compile:
'p1: aVar
	aVar notNil ifTrue:[gXaprompt _ self typeCheck: aVar shouldBe: GXString paramName: ''gXaprompt'']'!

"result
 ------"

GXinquire

addInstVarName: #gXres
!

!GXinquire methodsFor: 'initializing'!

initVars

!!

"BODY
 ----"

!GXinquire methodsFor: 'running'!

evaluate
GXprint run line: 0 proc: self method: false; p1:(gXaprompt); p2:': '; condEvaluate.
gXres _ (GXreadUterminal run line: 0 proc: self method: true; condEvaluate) deepCopy.
((GXlength run line: 0 proc: self method: true; p1:(gXres); condEvaluate) ~= 0) ifTrue: [ 
gXres _ (GXasUvariable run line: 0 proc: self method: true; p1:(gXres); condEvaluate) deepCopy ].
self up.
^gXres
!!
nil! "(fix for smalltalk bug, this line is ignored if there is a variable declaration error)"

"End of procedure"

GXcomment run line: 0 proc: self method: false; p1:'inquire'; p2:'Read user input from command line'; condEvaluate!
"Procedure class definition:
 ==========================="
((Smalltalk at: #GXjump ifAbsent: [ nil ]) isKindOf: Function class) ifTrue: [ ('Warning: Overriding previously defined procedure ', 'GXjump' gtype) prinl ].
Smalltalk at: #GXjump put: nil!
Function subclass: 'GXjump'
       instanceVariableNames: ''
       classVariableNames: ''
       poolDictionaries: ''
       category: 'Genie Procedure'
!

"HEADER
 ------"

"parameters
 ----------"

GXjump

addInstVarName: #gXacommand;
compile:
'gXacommand: aVar
	aVar notNil ifTrue:[gXacommand _ self typeCheck: aVar shouldBe: GXString paramName: ''gXacommand'']';
compile:
'p1: aVar
	aVar notNil ifTrue:[gXacommand _ self typeCheck: aVar shouldBe: GXString paramName: ''gXacommand'']'!

"qualifiers
 ----------"

GXjump

addInstVarName: #gXs;
compile:
'gXs
	gXs _ true';
compile:
'gXs: aBoolean
	gXs _ aBoolean';

addInstVarName: #gXp;
compile:
'gXp
	gXp _ true';
compile:
'gXp: aBoolean
	gXp _ aBoolean'
!

!GXjump methodsFor: 'initializing'!

initVars
	gXs _ false.
	gXp _ false
!!

"BODY
 ----"

!GXjump methodsFor: 'running'!

evaluate
GXsystem run line: 0 proc: self method: false; p1:(gXacommand); condEvaluate.
self up.
^nil
!!
nil! "(fix for smalltalk bug, this line is ignored if there is a variable declaration error)"

"End of procedure"

GXcomment run line: 0 proc: self method: false; p1:'jump'; p2:'GENIE-V2 similar jump'; condEvaluate!
"Procedure class definition:
 ==========================="
((Smalltalk at: #GXUUvmsgripe ifAbsent: [ nil ]) isKindOf: Function class) ifTrue: [ ('Warning: Overriding previously defined procedure ', 'GXUUvmsgripe' gtype) prinl ].
Smalltalk at: #GXUUvmsgripe put: nil!
Function subclass: 'GXUUvmsgripe'
       instanceVariableNames: ''
       classVariableNames: ''
       poolDictionaries: ''
       category: 'Genie Procedure'
!

"HEADER
 ------"

"parameters
 ----------"

GXUUvmsgripe

addInstVarName: #gXsubject;
compile:
'gXsubject: aVar
	aVar notNil ifTrue:[gXsubject _ self typeCheck: aVar shouldBe: GXString paramName: ''gXsubject'']';
compile:
'p1: aVar
	aVar notNil ifTrue:[gXsubject _ self typeCheck: aVar shouldBe: GXString paramName: ''gXsubject'']';

addInstVarName: #gXmessage;
compile:
'gXmessage: aVar
	aVar notNil ifTrue:[gXmessage _ self typeCheck: aVar shouldBe: GXString paramName: ''gXmessage'']';
compile:
'p2: aVar
	aVar notNil ifTrue:[gXmessage _ self typeCheck: aVar shouldBe: GXString paramName: ''gXmessage'']';

addInstVarName: #gXemail;
compile:
'gXemail: aVar
	aVar notNil ifTrue:[gXemail _ self typeCheck: aVar shouldBe: GXString paramName: ''gXemail'']';
compile:
'p3: aVar
	aVar notNil ifTrue:[gXemail _ self typeCheck: aVar shouldBe: GXString paramName: ''gXemail'']'!

"result
 ------"

GXUUvmsgripe

addInstVarName: #gXret
!

"Local variables
 ---------------"

GXUUvmsgripe

addInstVarName: #gXfile;

addInstVarName: #gXhandle
!

!GXUUvmsgripe methodsFor: 'initializing'!

initVars

!!

"BODY
 ----"

!GXUUvmsgripe methodsFor: 'running'!

evaluate
(GXdefined run line: 0 proc: self method: true; p1:(gXmessage); condEvaluate) ifTrue: [ 
GXprintin run line: 0 proc: self method: false; p1:'Sending message:
'; p2:(gXmessage); condEvaluate.
gXfile _ (GXUUtmpfile run line: 0 proc: self method: true; condEvaluate) deepCopy.
gXhandle _ (GXasciifile run line: 0 proc: self method: true; gXopen; p1:(gXfile); condEvaluate) deepCopy.
GXasciifile run line: 0 proc: self method: false; gXwritefree; p1:(gXhandle); p2:(gXU); p3:(gXU); p4:(gXmessage); condEvaluate.
GXasciifile run line: 0 proc: self method: false; gXclose; p1:(gXhandle); condEvaluate.
gXret _ (GXsystem run line: 0 proc: self method: true; p1:((((('MAIL ' + (gXfile)) + '/SUBJECT="') + (gXsubject)) + '" ') + (gXemail)); condEvaluate) deepCopy.
GXUUunlink run line: 0 proc: self method: false; p1:(gXfile); condEvaluate ] ifFalse: [ 
GXprintn run line: 0 proc: self method: false; p1:'Now type you message on the VMS mailer'; condEvaluate.
gXret _ (GXsystem run line: 0 proc: self method: true; p1:((('MAIL/SUBJECT="' + (gXsubject)) + '" ') + (gXemail)); condEvaluate) deepCopy ] .
self up.
^gXret
!!
nil! "(fix for smalltalk bug, this line is ignored if there is a variable declaration error)"

"End of procedure"

"Procedure class definition:
 ==========================="
((Smalltalk at: #GXUUunixgripe ifAbsent: [ nil ]) isKindOf: Function class) ifTrue: [ ('Warning: Overriding previously defined procedure ', 'GXUUunixgripe' gtype) prinl ].
Smalltalk at: #GXUUunixgripe put: nil!
Function subclass: 'GXUUunixgripe'
       instanceVariableNames: ''
       classVariableNames: ''
       poolDictionaries: ''
       category: 'Genie Procedure'
!

"HEADER
 ------"

"parameters
 ----------"

GXUUunixgripe

addInstVarName: #gXsubject;
compile:
'gXsubject: aVar
	aVar notNil ifTrue:[gXsubject _ self typeCheck: aVar shouldBe: GXString paramName: ''gXsubject'']';
compile:
'p1: aVar
	aVar notNil ifTrue:[gXsubject _ self typeCheck: aVar shouldBe: GXString paramName: ''gXsubject'']';

addInstVarName: #gXmessage;
compile:
'gXmessage: aVar
	aVar notNil ifTrue:[gXmessage _ self typeCheck: aVar shouldBe: GXString paramName: ''gXmessage'']';
compile:
'p2: aVar
	aVar notNil ifTrue:[gXmessage _ self typeCheck: aVar shouldBe: GXString paramName: ''gXmessage'']';

addInstVarName: #gXemail;
compile:
'gXemail: aVar
	aVar notNil ifTrue:[gXemail _ self typeCheck: aVar shouldBe: GXString paramName: ''gXemail'']';
compile:
'p3: aVar
	aVar notNil ifTrue:[gXemail _ self typeCheck: aVar shouldBe: GXString paramName: ''gXemail'']'!

"result
 ------"

GXUUunixgripe

addInstVarName: #gXret
!

!GXUUunixgripe methodsFor: 'initializing'!

initVars

!!

"BODY
 ----"

!GXUUunixgripe methodsFor: 'running'!

evaluate
(GXdefined run line: 0 proc: self method: true; p1:(gXmessage); condEvaluate) ifTrue: [ 
GXprintin run line: 0 proc: self method: false; p1:'Sending message:
'; p2:(gXmessage); condEvaluate.
gXret _ (GXsystem run line: 0 proc: self method: true; p1:((((('echo "' + (gXmessage)) + '" | mailx -s "GRIPE: ') + (gXsubject)) + '" ') + (gXemail)); condEvaluate) deepCopy ] ifFalse: [ 
GXprintn run line: 0 proc: self method: false; p1:'Now type your message - finish it with a "." on a line by itself'; condEvaluate.
GXprintn run line: 0 proc: self method: false; p1:'To abort the message, type "~q" on a line by itself'; condEvaluate.
GXprintn run line: 0 proc: self method: false; p1:'To edit the message using "xedit", type "~v" on a line by itself (but'; condEvaluate.
GXprintn run line: 0 proc: self method: false; p1:'   note that you will still need to type "." after you have left "xedit")'; condEvaluate.
gXret _ (GXsystem run line: 0 proc: self method: true; p1:((('mailx -s "GRIPE: ' + (gXsubject)) + '" ') + (gXemail)); condEvaluate) deepCopy ] .
self up.
^gXret
!!
nil! "(fix for smalltalk bug, this line is ignored if there is a variable declaration error)"

"End of procedure"

"Procedure class definition:
 ==========================="
((Smalltalk at: #GXgripe ifAbsent: [ nil ]) isKindOf: Function class) ifTrue: [ ('Warning: Overriding previously defined procedure ', 'GXgripe' gtype) prinl ].
Smalltalk at: #GXgripe put: nil!
Function subclass: 'GXgripe'
       instanceVariableNames: ''
       classVariableNames: ''
       poolDictionaries: ''
       category: 'Genie Procedure'
!

"HEADER
 ------"

"parameters
 ----------"

GXgripe

addInstVarName: #gXsubject;
compile:
'gXsubject: aVar
	aVar notNil ifTrue:[gXsubject _ self typeCheck: aVar shouldBe: GXString paramName: ''gXsubject'']';
compile:
'p1: aVar
	aVar notNil ifTrue:[gXsubject _ self typeCheck: aVar shouldBe: GXString paramName: ''gXsubject'']';

addInstVarName: #gXmessage;
compile:
'gXmessage: aVar
	aVar notNil ifTrue:[gXmessage _ self typeCheck: aVar shouldBe: GXString paramName: ''gXmessage'']';
compile:
'p2: aVar
	aVar notNil ifTrue:[gXmessage _ self typeCheck: aVar shouldBe: GXString paramName: ''gXmessage'']';

addInstVarName: #gXemail;
compile:
'gXemail: aVar
	aVar notNil ifTrue:[gXemail _ self typeCheck: aVar shouldBe: GXString paramName: ''gXemail'']';
compile:
'p3: aVar
	aVar notNil ifTrue:[gXemail _ self typeCheck: aVar shouldBe: GXString paramName: ''gXemail'']'
!

"Global variables
 ----------------"

GXgripe

addPoolVar: #gXUUdefaultUemail value: 'genie@isise.rl.ac.uk'!

"Local variables
 ---------------"

GXgripe

addInstVarName: #gXret;

addInstVarName: #gXemail
!

!GXgripe methodsFor: 'initializing'!

initVars
	gXemail _ (gXUUdefaultUemail)
!!

"BODY
 ----"

!GXgripe methodsFor: 'running'!

evaluate
GXprintn run line: 0 proc: self method: false; p1:'Thank you for griping !'; condEvaluate.
GXprintn run line: 0 proc: self method: false; p1:''; condEvaluate.
GXprintn run line: 0 proc: self method: false; p1:'In your message please:'; condEvaluate.
GXprintn run line: 0 proc: self method: false; p1:'(1) Enclose the information reported by the VERSION command'; condEvaluate.
GXprintn run line: 0 proc: self method: false; p1:'(2) If you are reporing a crash, enclose any error message or'; condEvaluate.
GXprintn run line: 0 proc: self method: false; p1:'    traceback along with the name of the command typed and'; condEvaluate.
GXprintn run line: 0 proc: self method: false; p1:'    parameters passed.'; condEvaluate.
GXprintn run line: 0 proc: self method: false; p1:''; condEvaluate.
GXprintn run line: 0 proc: self method: false; p1:'The message will be forwarded to "'; p2:(gXemail); p3:'"'; condEvaluate.
(GXdefined run line: 0 proc: self method: true; p1:(gXsubject); condEvaluate) ifTrue: [ 
GXprintin run line: 0 proc: self method: false; p1:'SUBJECT: '; p2:(gXsubject); condEvaluate ] ifFalse: [ 
gXsubject _ (GXinquire run line: 0 proc: self method: true; p1:'Please give a general title for your message
'; condEvaluate) deepCopy ] .
((GXos run line: 0 proc: self method: true; condEvaluate) = 'VMS') ifTrue: [ 
gXret _ (GXUUvmsgripe run line: 0 proc: self method: true; p1:(gXsubject); p2:(gXmessage); p3:(gXemail); condEvaluate) deepCopy ] ifFalse: [ 
gXret _ (GXUUunixgripe run line: 0 proc: self method: true; p1:(gXsubject); p2:(gXmessage); p3:(gXemail); condEvaluate) deepCopy ] .
(((gXret) ~= 0)) ifTrue: [ 
GXprinten run line: 0 proc: self method: false; p1:'GRIPE: Error sending message - please forward it manually to "'; p2:(gXemail); p3:'"'; condEvaluate ].
self up.
^nil
!!
nil! "(fix for smalltalk bug, this line is ignored if there is a variable declaration error)"

"End of procedure"

"Procedure class definition:
 ==========================="
((Smalltalk at: #GXpeakfit ifAbsent: [ nil ]) isKindOf: Function class) ifTrue: [ ('Warning: Overriding previously defined procedure ', 'GXpeakfit' gtype) prinl ].
Smalltalk at: #GXpeakfit put: nil!
Function subclass: 'GXpeakfit'
       instanceVariableNames: ''
       classVariableNames: ''
       poolDictionaries: ''
       category: 'Genie Procedure'
!

"HEADER
 ------"

"qualifiers
 ----------"

GXpeakfit

addInstVarName: #gXgauss;
compile:
'gXgauss
	gXgauss _ true';
compile:
'gXgauss: aBoolean
	gXgauss _ aBoolean';

addInstVarName: #gXgexp;
compile:
'gXgexp
	gXgexp _ true';
compile:
'gXgexp: aBoolean
	gXgexp _ aBoolean';

addInstVarName: #gXloren;
compile:
'gXloren
	gXloren _ true';
compile:
'gXloren: aBoolean
	gXloren _ aBoolean';

addInstVarName: #gXlexp;
compile:
'gXlexp
	gXlexp _ true';
compile:
'gXlexp: aBoolean
	gXlexp _ aBoolean';

addInstVarName: #gXvoigt;
compile:
'gXvoigt
	gXvoigt _ true';
compile:
'gXvoigt: aBoolean
	gXvoigt _ aBoolean';

addInstVarName: #gXvexp;
compile:
'gXvexp
	gXvexp _ true';
compile:
'gXvexp: aBoolean
	gXvexp _ aBoolean';

addInstVarName: #gXinit;
compile:
'gXinit
	gXinit _ true';
compile:
'gXinit: aBoolean
	gXinit _ aBoolean';

addInstVarName: #gXpoly;
compile:
'gXpoly
	gXpoly _ true';
compile:
'gXpoly: aBoolean
	gXpoly _ aBoolean'!

"parameters
 ----------"

GXpeakfit

addInstVarName: #gXx;
compile:
'gXx: aVar
	aVar notNil ifTrue:[gXx _ self typeCheck: aVar shouldBe: GXRealarray paramName: ''gXx'']';
compile:
'p1: aVar
	aVar notNil ifTrue:[gXx _ self typeCheck: aVar shouldBe: GXRealarray paramName: ''gXx'']';

addInstVarName: #gXy;
compile:
'gXy: aVar
	aVar notNil ifTrue:[gXy _ self typeCheck: aVar shouldBe: GXRealarray paramName: ''gXy'']';
compile:
'p2: aVar
	aVar notNil ifTrue:[gXy _ self typeCheck: aVar shouldBe: GXRealarray paramName: ''gXy'']';

addInstVarName: #gXe;
compile:
'gXe: aVar
	aVar notNil ifTrue:[gXe _ self typeCheck: aVar shouldBe: GXRealarray paramName: ''gXe'']';
compile:
'p3: aVar
	aVar notNil ifTrue:[gXe _ self typeCheck: aVar shouldBe: GXRealarray paramName: ''gXe'']';

addInstVarName: #gXpars;
compile:
'gXpars: aVar
	aVar notNil ifTrue:[gXpars _ self typeCheck: aVar shouldBe: GXRealarray paramName: ''gXpars'']';
compile:
'p4: aVar
	aVar notNil ifTrue:[gXpars _ self typeCheck: aVar shouldBe: GXRealarray paramName: ''gXpars'']';

addInstVarName: #gXipropt;
compile:
'gXipropt: aVar
	aVar notNil ifTrue:[gXipropt _ self typeCheck: aVar shouldBe: GXIntegerarray paramName: ''gXipropt'']';
compile:
'p5: aVar
	aVar notNil ifTrue:[gXipropt _ self typeCheck: aVar shouldBe: GXIntegerarray paramName: ''gXipropt'']'!

"result
 ------"

GXpeakfit

addInstVarName: #gXres
!

"Local variables
 ---------------"

GXpeakfit

addInstVarName: #gXres;

addInstVarName: #gXigood;

addInstVarName: #gXsigpar
!

!GXpeakfit methodsFor: 'initializing'!

initVars
	gXgauss _ false.
	gXgexp _ false.
	gXloren _ false.
	gXlexp _ false.
	gXvoigt _ false.
	gXvexp _ false.
	gXinit _ false.
	gXpoly _ false.
	gXres _ (GXfields run line: 0 proc: self method: true; p1:3; condEvaluate).
	gXigood _ 0
!!

"BODY
 ----"

!GXpeakfit methodsFor: 'running'!

evaluate
(((GXdefined run line: 0 proc: self method: true; p1:(gXx); condEvaluate) not | (GXdefined run line: 0 proc: self method: true; p1:(gXy); condEvaluate) not) | (GXdefined run line: 0 proc: self method: true; p1:(gXe); condEvaluate) not) ifTrue: [ 
GXprinten run line: 0 proc: self method: false; p1:'PEAKFIT: Not all required parameters given'; condEvaluate.
[nil.
self up.
^gXres] value ].
((GXlength run line: 0 proc: self method: true; p1:(gXx); condEvaluate) ~= (GXlength run line: 0 proc: self method: true; p1:(gXy); condEvaluate)) ifTrue: [ 
GXprinten run line: 0 proc: self method: false; p1:'PEAKFIT: X and Y arrays must be the same length'; condEvaluate.
[nil.
self up.
^gXres] value ].
(GXdefined run line: 0 proc: self method: true; p1:(gXpars); condEvaluate) not ifTrue: [ 
(gXpoly) ifTrue: [ 
gXpars _ (GXdimensions run line: 0 proc: self method: true; p1:1; condEvaluate) deepCopy ] ifFalse: [ ((gXgauss) | (gXloren)) ifTrue: [ 
gXpars _ (GXdimensions run line: 0 proc: self method: true; p1:5; condEvaluate) deepCopy ] ifFalse: [ (gXvexp) ifTrue: [ 
gXpars _ (GXdimensions run line: 0 proc: self method: true; p1:7; condEvaluate) deepCopy ] ifFalse: [ 
gXpars _ (GXdimensions run line: 0 proc: self method: true; p1:6; condEvaluate) deepCopy ] ]].
Function new loop: 1 to: (GXlength run line: 0 proc: self method: true; p1:(gXpars); condEvaluate) by: 1 before: [ :gXi | 
(gXpars isKindOf: GXArray) | (gXpars isKindOf: GXWorkspace)
    ifTrue:  [ ((gXpars) at:((Array new:1) at: 1 put: (gXi); yourself) put: 0.000000000000000e000 deepCopy) ]
    ifFalse: [ self error: 'Assignment to array or workspace failed, check that the destination is a structured type'] ] 
condition: [false] after: [ :gXi | nil ] ].
(gXpoly) ifTrue: [ 
gXsigpar _ (GXdimensions run line: 0 proc: self method: true; p1:(GXlength run line: 0 proc: self method: true; p1:(gXpars); condEvaluate); condEvaluate) deepCopy ] ifFalse: [ ((gXgauss) | (gXloren)) ifTrue: [ 
gXsigpar _ (GXdimensions run line: 0 proc: self method: true; p1:5; condEvaluate) deepCopy ] ifFalse: [ (gXvexp) ifTrue: [ 
gXsigpar _ (GXdimensions run line: 0 proc: self method: true; p1:7; condEvaluate) deepCopy ] ifFalse: [ 
gXsigpar _ (GXdimensions run line: 0 proc: self method: true; p1:6; condEvaluate) deepCopy ] ]].
(gXsigpar isKindOf: GXArray) | (gXsigpar isKindOf: GXWorkspace)
    ifTrue:  [ ((gXsigpar) at:((Array new:1) at: 1 put: 1; yourself) put: 0.000000000000000e000 deepCopy) ]
    ifFalse: [ self error: 'Assignment to array or workspace failed, check that the destination is a structured type'].
(GXdefined run line: 0 proc: self method: true; p1:(gXipropt); condEvaluate) not ifTrue: [ 
(gXpoly) ifTrue: [ 
gXipropt _ (GXdimensions run line: 0 proc: self method: true; p1:(GXlength run line: 0 proc: self method: true; p1:(gXpars); condEvaluate); condEvaluate) deepCopy ] ifFalse: [ ((gXgauss) | (gXloren)) ifTrue: [ 
gXipropt _ (GXdimensions run line: 0 proc: self method: true; p1:5; condEvaluate) deepCopy ] ifFalse: [ (gXvexp) ifTrue: [ 
gXipropt _ (GXdimensions run line: 0 proc: self method: true; p1:7; condEvaluate) deepCopy ] ifFalse: [ 
gXipropt _ (GXdimensions run line: 0 proc: self method: true; p1:6; condEvaluate) deepCopy ] ]].
Function new loop: 1 to: (GXlength run line: 0 proc: self method: true; p1:(gXipropt); condEvaluate) by: 1 before: [ :gXi | 
(gXipropt isKindOf: GXArray) | (gXipropt isKindOf: GXWorkspace)
    ifTrue:  [ ((gXipropt) at:((Array new:1) at: 1 put: (gXi); yourself) put: 0 deepCopy) ]
    ifFalse: [ self error: 'Assignment to array or workspace failed, check that the destination is a structured type'] ] 
condition: [false] after: [ :gXi | nil ] ].
(gXinit) ifTrue: [ 
GXcallout run line: 0 proc: self method: false; gXfacility:'intPeak'; gXfunction:1; condEvaluate ] ifFalse: [ (gXgauss) ifTrue: [ 
GXprintin run line: 0 proc: self method: false; p1:'Fitting to Gaussian peak shape'; condEvaluate.
GXcallout run line: 0 proc: self method: false; gXfacility:'intPeak'; gXfunction:2; condEvaluate ] ifFalse: [ (gXgexp) ifTrue: [ 
GXprintin run line: 0 proc: self method: false; p1:'Fitting to Gaussian + exponential peak shape'; condEvaluate.
GXcallout run line: 0 proc: self method: false; gXfacility:'intPeak'; gXfunction:3; condEvaluate ] ifFalse: [ (gXlexp) ifTrue: [ 
GXprintin run line: 0 proc: self method: false; p1:'Fitting to Lorentzian + exponential peak shape'; condEvaluate.
GXcallout run line: 0 proc: self method: false; gXfacility:'intPeak'; gXfunction:4; condEvaluate ] ifFalse: [ (gXloren) ifTrue: [ 
GXprintin run line: 0 proc: self method: false; p1:'Fitting to Lorentzian peak shape'; condEvaluate.
GXcallout run line: 0 proc: self method: false; gXfacility:'intPeak'; gXfunction:5; condEvaluate ] ifFalse: [ (gXvoigt) ifTrue: [ 
GXprintin run line: 0 proc: self method: false; p1:'Fitting to Voigt peak shape'; condEvaluate.
GXcallout run line: 0 proc: self method: false; gXfacility:'intPeak'; gXfunction:6; condEvaluate ] ifFalse: [ (gXvexp) ifTrue: [ 
GXprintin run line: 0 proc: self method: false; p1:'Fitting to Voigt + exponential peak shape'; condEvaluate.
GXcallout run line: 0 proc: self method: false; gXfacility:'intPeak'; gXfunction:14; condEvaluate ] ifFalse: [ (gXpoly) ifTrue: [ 
GXprintin run line: 0 proc: self method: false; p1:'Fitting to Polynomial (degree '; p2:((GXlength run line: 0 proc: self method: true; p1:(gXpars); condEvaluate) - 1); p3:')'; condEvaluate.
GXcallout run line: 0 proc: self method: false; gXfacility:'intPeak'; gXfunction:12; condEvaluate ] ifFalse: [ 
GXprinten run line: 0 proc: self method: false; p1:'PEAKFIT: Invalid or non-existent qualifier'; condEvaluate ] ]]]]]]].
(GXgUerrorUhandler run line: 0 proc: self method: true; condEvaluate) ifTrue: [ 
gXres _ (gXU) deepCopy ] ifFalse: [ 
(gXres isKindOf: GXArray) | (gXres isKindOf: GXWorkspace)
    ifTrue:  [ ((gXres) at:#gXpars put: (gXpars) deepCopy) ]
    ifFalse: [ self error: 'Assignment to array or workspace failed, check that the destination is a structured type'].
(gXres isKindOf: GXArray) | (gXres isKindOf: GXWorkspace)
    ifTrue:  [ ((gXres) at:#gXigood put: (gXigood) deepCopy) ]
    ifFalse: [ self error: 'Assignment to array or workspace failed, check that the destination is a structured type'].
(gXres isKindOf: GXArray) | (gXres isKindOf: GXWorkspace)
    ifTrue:  [ ((gXres) at:#gXsigpar put: (gXsigpar) deepCopy) ]
    ifFalse: [ self error: 'Assignment to array or workspace failed, check that the destination is a structured type'] ] .
self up.
^gXres
!!
nil! "(fix for smalltalk bug, this line is ignored if there is a variable declaration error)"

"End of procedure"

GXcomment run line: 0 proc: self method: false; p1:'peakfit'; p2:'Fit a peak shape to user data'; condEvaluate!
"Procedure class definition:
 ==========================="
((Smalltalk at: #GXpeakgen ifAbsent: [ nil ]) isKindOf: Function class) ifTrue: [ ('Warning: Overriding previously defined procedure ', 'GXpeakgen' gtype) prinl ].
Smalltalk at: #GXpeakgen put: nil!
Function subclass: 'GXpeakgen'
       instanceVariableNames: ''
       classVariableNames: ''
       poolDictionaries: ''
       category: 'Genie Procedure'
!

"HEADER
 ------"

"qualifiers
 ----------"

GXpeakgen

addInstVarName: #gXgauss;
compile:
'gXgauss
	gXgauss _ true';
compile:
'gXgauss: aBoolean
	gXgauss _ aBoolean';

addInstVarName: #gXgexp;
compile:
'gXgexp
	gXgexp _ true';
compile:
'gXgexp: aBoolean
	gXgexp _ aBoolean';

addInstVarName: #gXloren;
compile:
'gXloren
	gXloren _ true';
compile:
'gXloren: aBoolean
	gXloren _ aBoolean';

addInstVarName: #gXlexp;
compile:
'gXlexp
	gXlexp _ true';
compile:
'gXlexp: aBoolean
	gXlexp _ aBoolean';

addInstVarName: #gXvoigt;
compile:
'gXvoigt
	gXvoigt _ true';
compile:
'gXvoigt: aBoolean
	gXvoigt _ aBoolean';

addInstVarName: #gXvexp;
compile:
'gXvexp
	gXvexp _ true';
compile:
'gXvexp: aBoolean
	gXvexp _ aBoolean';

addInstVarName: #gXpoly;
compile:
'gXpoly
	gXpoly _ true';
compile:
'gXpoly: aBoolean
	gXpoly _ aBoolean'!

"parameters
 ----------"

GXpeakgen

addInstVarName: #gXx;
compile:
'gXx: aVar
	aVar notNil ifTrue:[gXx _ self typeCheck: aVar shouldBe: GXRealarray paramName: ''gXx'']';
compile:
'p1: aVar
	aVar notNil ifTrue:[gXx _ self typeCheck: aVar shouldBe: GXRealarray paramName: ''gXx'']';

addInstVarName: #gXpars;
compile:
'gXpars: aVar
	aVar notNil ifTrue:[gXpars _ self typeCheck: aVar shouldBe: GXRealarray paramName: ''gXpars'']';
compile:
'p2: aVar
	aVar notNil ifTrue:[gXpars _ self typeCheck: aVar shouldBe: GXRealarray paramName: ''gXpars'']'!

"result
 ------"

GXpeakgen

addInstVarName: #gXy
!

"Local variables
 ---------------"

GXpeakgen

addInstVarName: #gXe;

addInstVarName: #gXsigpar;

addInstVarName: #gXipropt;

addInstVarName: #gXigood
!

!GXpeakgen methodsFor: 'initializing'!

initVars
	gXgauss _ false.
	gXgexp _ false.
	gXloren _ false.
	gXlexp _ false.
	gXvoigt _ false.
	gXvexp _ false.
	gXpoly _ false.
	gXe _ (GXdimensions run line: 0 proc: self method: true; p1:1; condEvaluate).
	gXsigpar _ (GXdimensions run line: 0 proc: self method: true; p1:6; condEvaluate).
	gXipropt _ (GXdimensions run line: 0 proc: self method: true; p1:6; condEvaluate).
	gXigood _ 0
!!

"BODY
 ----"

!GXpeakgen methodsFor: 'running'!

evaluate
((GXdefined run line: 0 proc: self method: true; p1:(gXx); condEvaluate) not | (GXdefined run line: 0 proc: self method: true; p1:(gXpars); condEvaluate) not) ifTrue: [ 
GXprinten run line: 0 proc: self method: false; p1:'PEAKGEN: Not all required parameters given'; condEvaluate.
[nil.
self up.
^gXy] value ].
gXy _ (GXdimensions run line: 0 proc: self method: true; p1:(GXlength run line: 0 proc: self method: true; p1:(gXx); condEvaluate); condEvaluate) deepCopy.
(gXe isKindOf: GXArray) | (gXe isKindOf: GXWorkspace)
    ifTrue:  [ ((gXe) at:((Array new:1) at: 1 put: 1; yourself) put: 0.000000000000000e000 deepCopy) ]
    ifFalse: [ self error: 'Assignment to array or workspace failed, check that the destination is a structured type'].
(gXipropt isKindOf: GXArray) | (gXipropt isKindOf: GXWorkspace)
    ifTrue:  [ ((gXipropt) at:((Array new:1) at: 1 put: 1; yourself) put: 0 deepCopy) ]
    ifFalse: [ self error: 'Assignment to array or workspace failed, check that the destination is a structured type'].
(gXsigpar isKindOf: GXArray) | (gXsigpar isKindOf: GXWorkspace)
    ifTrue:  [ ((gXsigpar) at:((Array new:1) at: 1 put: 1; yourself) put: 0.000000000000000e000 deepCopy) ]
    ifFalse: [ self error: 'Assignment to array or workspace failed, check that the destination is a structured type'].
(gXy isKindOf: GXArray) | (gXy isKindOf: GXWorkspace)
    ifTrue:  [ ((gXy) at:((Array new:1) at: 1 put: 1; yourself) put: 0.000000000000000e000 deepCopy) ]
    ifFalse: [ self error: 'Assignment to array or workspace failed, check that the destination is a structured type'].
(gXgauss) ifTrue: [ 
GXcallout run line: 0 proc: self method: false; gXfacility:'intPeak'; gXfunction:7; condEvaluate ] ifFalse: [ (gXgexp) ifTrue: [ 
GXcallout run line: 0 proc: self method: false; gXfacility:'intPeak'; gXfunction:8; condEvaluate ] ifFalse: [ (gXloren) ifTrue: [ 
GXcallout run line: 0 proc: self method: false; gXfacility:'intPeak'; gXfunction:9; condEvaluate ] ifFalse: [ (gXlexp) ifTrue: [ 
GXcallout run line: 0 proc: self method: false; gXfacility:'intPeak'; gXfunction:10; condEvaluate ] ifFalse: [ (gXvoigt) ifTrue: [ 
GXcallout run line: 0 proc: self method: false; gXfacility:'intPeak'; gXfunction:11; condEvaluate ] ifFalse: [ (gXvexp) ifTrue: [ 
GXcallout run line: 0 proc: self method: false; gXfacility:'intPeak'; gXfunction:15; condEvaluate ] ifFalse: [ (gXpoly) ifTrue: [ 
GXcallout run line: 0 proc: self method: false; gXfacility:'intPeak'; gXfunction:13; condEvaluate ] ifFalse: [ 
GXprinten run line: 0 proc: self method: false; p1:'PEAKGEN: Invalid or non-existent qualifier'; condEvaluate ] ]]]]]].
(GXgUerrorUhandler run line: 0 proc: self method: true; condEvaluate) ifTrue: [ 
gXy _ (gXU) deepCopy ].
self up.
^gXy
!!
nil! "(fix for smalltalk bug, this line is ignored if there is a variable declaration error)"

"End of procedure"

GXcomment run line: 0 proc: self method: false; p1:'peakgen'; p2:'Generate Y points for a peak from supplied parameters and X values'; condEvaluate!
"Procedure class definition:
 ==========================="
((Smalltalk at: #GXUUxyechunk ifAbsent: [ nil ]) isKindOf: Function class) ifTrue: [ ('Warning: Overriding previously defined procedure ', 'GXUUxyechunk' gtype) prinl ].
Smalltalk at: #GXUUxyechunk put: nil!
Function subclass: 'GXUUxyechunk'
       instanceVariableNames: ''
       classVariableNames: ''
       poolDictionaries: ''
       category: 'Genie Procedure'
!

"HEADER
 ------"

"parameters
 ----------"

GXUUxyechunk

addInstVarName: #gXw;
compile:
'gXw: aVar
	aVar notNil ifTrue:[gXw _ self typeCheck: aVar shouldBe: GXWorkspace paramName: ''gXw'']';
compile:
'p1: aVar
	aVar notNil ifTrue:[gXw _ self typeCheck: aVar shouldBe: GXWorkspace paramName: ''gXw'']';

addInstVarName: #gXxmin;
compile:
'gXxmin: aVar
	aVar notNil ifTrue:[gXxmin _ self typeCheck: aVar shouldBe: GXReal paramName: ''gXxmin'']';
compile:
'p2: aVar
	aVar notNil ifTrue:[gXxmin _ self typeCheck: aVar shouldBe: GXReal paramName: ''gXxmin'']';

addInstVarName: #gXxmax;
compile:
'gXxmax: aVar
	aVar notNil ifTrue:[gXxmax _ self typeCheck: aVar shouldBe: GXReal paramName: ''gXxmax'']';
compile:
'p3: aVar
	aVar notNil ifTrue:[gXxmax _ self typeCheck: aVar shouldBe: GXReal paramName: ''gXxmax'']'!

"result
 ------"

GXUUxyechunk

addInstVarName: #gXxyearrays
!

"Local variables
 ---------------"

GXUUxyechunk

addInstVarName: #gXxyearrays;

addInstVarName: #gXimin;

addInstVarName: #gXimax;

addInstVarName: #gXxtmp
!

!GXUUxyechunk methodsFor: 'initializing'!

initVars
	gXxyearrays _ (GXfields run line: 0 proc: self method: true; condEvaluate)
!!

"BODY
 ----"

!GXUUxyechunk methodsFor: 'running'!

evaluate

    |x y|
    x _ gXw at: #gXx.
    y _ gXw at: #gXy.
    ( (x size) = (y size) )
	ifTrue: [ gXxtmp _ x ]
	ifFalse: [ gXxtmp _ x centre ]
.
gXimin _ (GXbracket run line: 0 proc: self method: true; p1:(gXxtmp); p2:(gXxmin); condEvaluate) deepCopy.
(((gXimin) = 1 negated)) ifTrue: [ 
gXimin _ 1 deepCopy ].
gXimax _ (GXbracket run line: 0 proc: self method: true; p1:(gXxtmp); p2:(gXxmax); condEvaluate) deepCopy.
(((gXimax) = 1 negated)) ifTrue: [ 
gXimax _ (GXlength run line: 0 proc: self method: true; p1:(gXxtmp); condEvaluate) deepCopy ].
(gXxyearrays isKindOf: GXArray) | (gXxyearrays isKindOf: GXWorkspace)
    ifTrue:  [ ((gXxyearrays) at:#gXx put: ((gXxtmp) at:((Array new:1) at: 1 put: (((gXimin)) to: ((gXimax))); yourself)) ) ]
    ifFalse: [ self error: 'Assignment to array or workspace failed, check that the destination is a structured type'].
(gXxyearrays isKindOf: GXArray) | (gXxyearrays isKindOf: GXWorkspace)
    ifTrue:  [ ((gXxyearrays) at:#gXy put: (((gXw) at:#gXy) at:((Array new:1) at: 1 put: (((gXimin)) to: ((gXimax))); yourself)) ) ]
    ifFalse: [ self error: 'Assignment to array or workspace failed, check that the destination is a structured type'].
(gXxyearrays isKindOf: GXArray) | (gXxyearrays isKindOf: GXWorkspace)
    ifTrue:  [ ((gXxyearrays) at:#gXe put: (((gXw) at:#gXe) at:((Array new:1) at: 1 put: (((gXimin)) to: ((gXimax))); yourself)) ) ]
    ifFalse: [ self error: 'Assignment to array or workspace failed, check that the destination is a structured type'].
self up.
^gXxyearrays
!!
nil! "(fix for smalltalk bug, this line is ignored if there is a variable declaration error)"

"End of procedure"

"Procedure class definition:
 ==========================="
((Smalltalk at: #GXUUxminmax ifAbsent: [ nil ]) isKindOf: Function class) ifTrue: [ ('Warning: Overriding previously defined procedure ', 'GXUUxminmax' gtype) prinl ].
Smalltalk at: #GXUUxminmax put: nil!
Function subclass: 'GXUUxminmax'
       instanceVariableNames: ''
       classVariableNames: ''
       poolDictionaries: ''
       category: 'Genie Procedure'
!

"HEADER
 ------"

"result
 ------"

GXUUxminmax

addInstVarName: #gXchar!

"parameters
 ----------"

GXUUxminmax

addInstVarName: #gXxlimits;
compile:
'gXxlimits: aVar
	aVar notNil ifTrue:[gXxlimits _ self typeCheck: aVar shouldBe: GXRealarray paramName: ''gXxlimits'']';
compile:
'p1: aVar
	aVar notNil ifTrue:[gXxlimits _ self typeCheck: aVar shouldBe: GXRealarray paramName: ''gXxlimits'']'
!

"Local variables
 ---------------"

GXUUxminmax

addInstVarName: #gXcursor
!

!GXUUxminmax methodsFor: 'initializing'!

initVars

!!

"BODY
 ----"

!GXUUxminmax methodsFor: 'running'!

evaluate
gXcursor _ (GXgetUcursor run line: 0 proc: self method: true; p1:0.000000000000000e000; p2:0.000000000000000e000; condEvaluate) deepCopy.
gXchar _ ((gXcursor) at:#gXchar) deepCopy.
((gXchar) = 'A') ifTrue: [ 
(gXxlimits isKindOf: GXArray) | (gXxlimits isKindOf: GXWorkspace)
    ifTrue:  [ ((gXxlimits) at:((Array new:1) at: 1 put: 1; yourself) put: ((gXcursor) at:#gXwUx) deepCopy) ]
    ifFalse: [ self error: 'Assignment to array or workspace failed, check that the destination is a structured type'].
gXcursor _ (GXgetUcursor run line: 0 proc: self method: true; p1:((gXcursor) at:#gXwUx); p2:((gXcursor) at:#gXwUy); condEvaluate) deepCopy.
gXchar _ ((gXcursor) at:#gXchar) deepCopy.
((gXchar) = 'A') ifTrue: [ 
(gXxlimits isKindOf: GXArray) | (gXxlimits isKindOf: GXWorkspace)
    ifTrue:  [ ((gXxlimits) at:((Array new:1) at: 1 put: 2; yourself) put: ((gXcursor) at:#gXwUx) deepCopy) ]
    ifFalse: [ self error: 'Assignment to array or workspace failed, check that the destination is a structured type'] ] ].
self up.
^gXchar
!!
nil! "(fix for smalltalk bug, this line is ignored if there is a variable declaration error)"

"End of procedure"

"Procedure class definition:
 ==========================="
((Smalltalk at: #GXUUshowfit ifAbsent: [ nil ]) isKindOf: Function class) ifTrue: [ ('Warning: Overriding previously defined procedure ', 'GXUUshowfit' gtype) prinl ].
Smalltalk at: #GXUUshowfit put: nil!
Function subclass: 'GXUUshowfit'
       instanceVariableNames: ''
       classVariableNames: ''
       poolDictionaries: ''
       category: 'Genie Procedure'
!

"HEADER
 ------"

"parameters
 ----------"

GXUUshowfit

addInstVarName: #gXw;
compile:
'gXw: aVar
	aVar notNil ifTrue:[gXw _ self typeCheck: aVar shouldBe: GXWorkspace paramName: ''gXw'']';
compile:
'p1: aVar
	aVar notNil ifTrue:[gXw _ self typeCheck: aVar shouldBe: GXWorkspace paramName: ''gXw'']'
!

"Local variables
 ---------------"

GXUUshowfit

addInstVarName: #gXi
!

!GXUUshowfit methodsFor: 'initializing'!

initVars

!!

"BODY
 ----"

!GXUUshowfit methodsFor: 'running'!

evaluate
GXprintn run line: 0 proc: self method: false; p1:'Function : '; p2:((gXw) at:#gXtype); condEvaluate.
GXprintn run line: 0 proc: self method: false; p1:'Description : '; p2:((gXw) at:#gXdesc); condEvaluate.
Function new loop: 1 to: (GXlength run line: 0 proc: self method: true; p1:((gXw) at:#gXpars); condEvaluate) by: 1 before: [ :gXi | 
GXprintn run line: 0 proc: self method: false; p1:'Parameter P'; p2:(gXi); p3:': '; p4:(((gXw) at:#gXpars) at:((Array new:1) at: 1 put: (gXi); yourself)); p5:' +- '; p6:(((gXw) at:#gXsigpar) at:((Array new:1) at: 1 put: (gXi); yourself)); condEvaluate ] 
condition: [false] after: [ :gXi | nil ].
self up.
^nil
!!
nil! "(fix for smalltalk bug, this line is ignored if there is a variable declaration error)"

"End of procedure"

"Procedure class definition:
 ==========================="
((Smalltalk at: #GXpeak ifAbsent: [ nil ]) isKindOf: Function class) ifTrue: [ ('Warning: Overriding previously defined procedure ', 'GXpeak' gtype) prinl ].
Smalltalk at: #GXpeak put: nil!
Function subclass: 'GXpeak'
       instanceVariableNames: ''
       classVariableNames: ''
       poolDictionaries: ''
       category: 'Genie Procedure'
!

"HEADER
 ------"

"parameters
 ----------"

GXpeak

addInstVarName: #gXw;
compile:
'gXw: aVar
	aVar notNil ifTrue:[gXw _ self typeCheck: aVar shouldBe: GXWorkspace paramName: ''gXw'']';
compile:
'p1: aVar
	aVar notNil ifTrue:[gXw _ self typeCheck: aVar shouldBe: GXWorkspace paramName: ''gXw'']'!

"result
 ------"

GXpeak

addInstVarName: #gXres
!

"Local variables
 ---------------"

GXpeak

addInstVarName: #gXans;

addInstVarName: #gXi;

addInstVarName: #gXxlimits;

addInstVarName: #gXxvals;

addInstVarName: #gXyvals;

addInstVarName: #gXtmp;

addInstVarName: #gXpars;

addInstVarName: #gXfittedy!

"Local variables
 ---------------"

GXpeak

addInstVarName: #gXnfunc;

addInstVarName: #gXchar;

addInstVarName: #gXx1;

addInstVarName: #gXy1;

addInstVarName: #gXx2;

addInstVarName: #gXy2;

addInstVarName: #gXxyechunk;

addInstVarName: #gXxyearrays
!

!GXpeak methodsFor: 'initializing'!

initVars

!!

"BODY
 ----"

!GXpeak methodsFor: 'running'!

evaluate
(GXdefined run line: 0 proc: self method: true; p1:(gXw); condEvaluate) not ifTrue: [ 
GXprinten run line: 0 proc: self method: false; p1:'PEAK: Must supply a workspace'; condEvaluate.
[nil.
self up.
^gXres] value ].
gXnfunc _ 7 deepCopy.
gXxlimits _ (GXdimensions run line: 0 proc: self method: true; p1:2; condEvaluate) deepCopy.
(gXxlimits isKindOf: GXArray) | (gXxlimits isKindOf: GXWorkspace)
    ifTrue:  [ ((gXxlimits) at:((Array new:1) at: 1 put: 1; yourself) put: (((gXw) at:#gXx) at:((Array new:1) at: 1 put: 1; yourself)) deepCopy) ]
    ifFalse: [ self error: 'Assignment to array or workspace failed, check that the destination is a structured type'].
(gXxlimits isKindOf: GXArray) | (gXxlimits isKindOf: GXWorkspace)
    ifTrue:  [ ((gXxlimits) at:((Array new:1) at: 1 put: 2; yourself) put: (((gXw) at:#gXx) at:((Array new:1) at: 1 put: (GXlength run line: 0 proc: self method: true; p1:((gXw) at:#gXx); condEvaluate); yourself)) deepCopy) ]
    ifFalse: [ self error: 'Assignment to array or workspace failed, check that the destination is a structured type'].
GXprintn run line: 0 proc: self method: false; p1:'Click left to set x-bounds for region to fit, right to continue'; condEvaluate.
Function new loop: 1 to: 100 by: 1 before: [ :gXi | 
GXdisplay run line: 0 proc: self method: false; gXmarkers; p1:(gXw); p2:((gXxlimits) at:((Array new:1) at: 1 put: 1; yourself)); p3:((gXxlimits) at:((Array new:1) at: 1 put: 2; yourself)); condEvaluate.
gXchar _ (GXUUxminmax run line: 0 proc: self method: true; p1:(gXxlimits); condEvaluate) deepCopy ] 
condition: [ :gXi | (((gXchar) = 'X')) ] after: [ :gXi | nil ].
gXxyearrays _ (GXUUxyechunk run line: 0 proc: self method: true; p1:(gXUgrUwksp); p2:((gXxlimits) at:((Array new:1) at: 1 put: 1; yourself)); p3:((gXxlimits) at:((Array new:1) at: 1 put: 2; yourself)); condEvaluate) .
GXprint run line: 0 proc: self method: false; p1:'
Fit 	1. Gaussian
	2. Gaussian convolved with exponential
  	3. Lorentzian
	4. Lorentzian convolved with exponential
  	5. Voigt
	6. Voigt convolved with exponential
	7. Polynomial
	8. Do nothing

	Which ? '; condEvaluate.
gXans _ (GXinquire run line: 0 proc: self method: true; condEvaluate) deepCopy.
Function new case: (gXans) body: [ :caseTempVar :break |
(caseTempVar = 1) ifTrue: [
 gXres _ (GXpeakfit run line: 0 proc: self method: true; gXgauss; p1:((gXxyearrays) at:#gXx); p2:((gXxyearrays) at:#gXy); p3:((gXxyearrays) at:#gXe); condEvaluate) deepCopy.
gXfittedy _ (GXpeakgen run line: 0 proc: self method: true; gXgauss; p1:((gXxyearrays) at:#gXx); p2:((gXres) at:#gXpars); condEvaluate) deepCopy.
(gXres isKindOf: GXArray) | (gXres isKindOf: GXWorkspace)
    ifTrue:  [ ((gXres) at:#gXtype put: 'Gaussian' deepCopy) ]
    ifFalse: [ self error: 'Assignment to array or workspace failed, check that the destination is a structured type'].
(gXres isKindOf: GXArray) | (gXres isKindOf: GXWorkspace)
    ifTrue:  [ ((gXres) at:#gXdesc put: 'Y = P1x + P2 + P3*Gauss(P4,P5)' deepCopy) ]
    ifFalse: [ self error: 'Assignment to array or workspace failed, check that the destination is a structured type']. break value ].
(caseTempVar = 2) ifTrue: [
 gXres _ (GXpeakfit run line: 0 proc: self method: true; gXgexp; p1:((gXxyearrays) at:#gXx); p2:((gXxyearrays) at:#gXy); p3:((gXxyearrays) at:#gXe); condEvaluate) deepCopy.
gXfittedy _ (GXpeakgen run line: 0 proc: self method: true; gXgexp; p1:((gXxyearrays) at:#gXx); p2:((gXres) at:#gXpars); condEvaluate) deepCopy.
(gXres isKindOf: GXArray) | (gXres isKindOf: GXWorkspace)
    ifTrue:  [ ((gXres) at:#gXtype put: 'Gaussian convolved with an exponential' deepCopy) ]
    ifFalse: [ self error: 'Assignment to array or workspace failed, check that the destination is a structured type'].
(gXres isKindOf: GXArray) | (gXres isKindOf: GXWorkspace)
    ifTrue:  [ ((gXres) at:#gXdesc put: 'Y = P1x + P2 + P3*Gsexp(P4,P5,P6)' deepCopy) ]
    ifFalse: [ self error: 'Assignment to array or workspace failed, check that the destination is a structured type']. break value ].
(caseTempVar = 3) ifTrue: [
 gXres _ (GXpeakfit run line: 0 proc: self method: true; gXloren; p1:((gXxyearrays) at:#gXx); p2:((gXxyearrays) at:#gXy); p3:((gXxyearrays) at:#gXe); condEvaluate) deepCopy.
gXfittedy _ (GXpeakgen run line: 0 proc: self method: true; gXloren; p1:((gXxyearrays) at:#gXx); p2:((gXres) at:#gXpars); condEvaluate) deepCopy.
(gXres isKindOf: GXArray) | (gXres isKindOf: GXWorkspace)
    ifTrue:  [ ((gXres) at:#gXtype put: 'Lorentzian' deepCopy) ]
    ifFalse: [ self error: 'Assignment to array or workspace failed, check that the destination is a structured type'].
(gXres isKindOf: GXArray) | (gXres isKindOf: GXWorkspace)
    ifTrue:  [ ((gXres) at:#gXdesc put: 'Y = P1x + P2 + P3*Lorentz(P4,P5)' deepCopy) ]
    ifFalse: [ self error: 'Assignment to array or workspace failed, check that the destination is a structured type']. break value ].
(caseTempVar = 4) ifTrue: [
 gXres _ (GXpeakfit run line: 0 proc: self method: true; gXlexp; p1:((gXxyearrays) at:#gXx); p2:((gXxyearrays) at:#gXy); p3:((gXxyearrays) at:#gXe); condEvaluate) deepCopy.
gXfittedy _ (GXpeakgen run line: 0 proc: self method: true; gXlexp; p1:((gXxyearrays) at:#gXx); p2:((gXres) at:#gXpars); condEvaluate) deepCopy.
(gXres isKindOf: GXArray) | (gXres isKindOf: GXWorkspace)
    ifTrue:  [ ((gXres) at:#gXtype put: 'Lorentzian convolved with an exponential' deepCopy) ]
    ifFalse: [ self error: 'Assignment to array or workspace failed, check that the destination is a structured type'].
(gXres isKindOf: GXArray) | (gXres isKindOf: GXWorkspace)
    ifTrue:  [ ((gXres) at:#gXdesc put: 'Y = P1x + P2 + P3*Lzexp(P4,P5,P6)' deepCopy) ]
    ifFalse: [ self error: 'Assignment to array or workspace failed, check that the destination is a structured type']. break value ] ].
Function new case: (gXans) body: [ :caseTempVar :break |
(caseTempVar = 5) ifTrue: [
 gXres _ (GXpeakfit run line: 0 proc: self method: true; gXvoigt; p1:((gXxyearrays) at:#gXx); p2:((gXxyearrays) at:#gXy); p3:((gXxyearrays) at:#gXe); condEvaluate) deepCopy.
gXfittedy _ (GXpeakgen run line: 0 proc: self method: true; gXvoigt; p1:((gXxyearrays) at:#gXx); p2:((gXres) at:#gXpars); condEvaluate) deepCopy.
(gXres isKindOf: GXArray) | (gXres isKindOf: GXWorkspace)
    ifTrue:  [ ((gXres) at:#gXtype put: 'Voigt' deepCopy) ]
    ifFalse: [ self error: 'Assignment to array or workspace failed, check that the destination is a structured type'].
(gXres isKindOf: GXArray) | (gXres isKindOf: GXWorkspace)
    ifTrue:  [ ((gXres) at:#gXdesc put: 'Y = P1x + P2 + P3*Voigt(P4,P5,P6)' deepCopy) ]
    ifFalse: [ self error: 'Assignment to array or workspace failed, check that the destination is a structured type']. break value ].
(caseTempVar = 6) ifTrue: [
 gXres _ (GXpeakfit run line: 0 proc: self method: true; gXvexp; p1:((gXxyearrays) at:#gXx); p2:((gXxyearrays) at:#gXy); p3:((gXxyearrays) at:#gXe); condEvaluate) deepCopy.
gXfittedy _ (GXpeakgen run line: 0 proc: self method: true; gXvexp; p1:((gXxyearrays) at:#gXx); p2:((gXres) at:#gXpars); condEvaluate) deepCopy.
(gXres isKindOf: GXArray) | (gXres isKindOf: GXWorkspace)
    ifTrue:  [ ((gXres) at:#gXtype put: 'Voigt convolved with an exponential' deepCopy) ]
    ifFalse: [ self error: 'Assignment to array or workspace failed, check that the destination is a structured type'].
(gXres isKindOf: GXArray) | (gXres isKindOf: GXWorkspace)
    ifTrue:  [ ((gXres) at:#gXdesc put: 'Y = P1x + P2 + P3*Vtexp(P4,P5,P6,P7)' deepCopy) ]
    ifFalse: [ self error: 'Assignment to array or workspace failed, check that the destination is a structured type']. break value ].
(caseTempVar = 7) ifTrue: [
 gXtmp _ (GXinquire run line: 0 proc: self method: true; p1:'Degree of polynomial'; condEvaluate) deepCopy.
gXpars _ (GXdimensions run line: 0 proc: self method: true; p1:((gXtmp) + 1); condEvaluate) deepCopy.
GXfill run line: 0 proc: self method: false; p1:(gXpars); p2:0.000000000000000e000; condEvaluate.
gXres _ (GXpeakfit run line: 0 proc: self method: true; gXpoly; p1:((gXxyearrays) at:#gXx); p2:((gXxyearrays) at:#gXy); p3:((gXxyearrays) at:#gXe); p4:(gXpars); condEvaluate) deepCopy.
gXfittedy _ (GXpeakgen run line: 0 proc: self method: true; gXpoly; p1:((gXxyearrays) at:#gXx); p2:((gXres) at:#gXpars); condEvaluate) deepCopy.
(gXres isKindOf: GXArray) | (gXres isKindOf: GXWorkspace)
    ifTrue:  [ ((gXres) at:#gXtype put: (('Polynomial (degree' + (GXasUstring run line: 0 proc: self method: true; p1:(gXtmp); condEvaluate)) + ')') deepCopy) ]
    ifFalse: [ self error: 'Assignment to array or workspace failed, check that the destination is a structured type'].
(gXres isKindOf: GXArray) | (gXres isKindOf: GXWorkspace)
    ifTrue:  [ ((gXres) at:#gXdesc put: 'Y = P1 + P2*x + p3*x^2 + ... ' deepCopy) ]
    ifFalse: [ self error: 'Assignment to array or workspace failed, check that the destination is a structured type']. break value ].
(caseTempVar = 8) ifTrue: [
 GXprintn run line: 0 proc: self method: false; p1:'Peak fit cancelled'; condEvaluate. break value ] ].
((((gXans) >= 1)) & (((gXans) <= (gXnfunc)))) ifTrue: [ 
GXdraw run line: 0 proc: self method: false; gXplot; p1:((gXxyearrays) at:#gXx); p2:(gXfittedy); p3:(gXDgreen); p4:(gXDfull); p5:2.000000000000000e000; condEvaluate.
GXUUshowfit run line: 0 proc: self method: false; p1:(gXres); condEvaluate ].
(((((gXans) >= 1)) & (((gXans) <= (gXnfunc)))) & (((gXans) ~= 7))) ifTrue: [ 
gXx1 _ (GXmin run line: 0 proc: self method: true; p1:((gXxyearrays) at:#gXx); condEvaluate) deepCopy.
gXy1 _ (((((gXres) at:#gXpars) at:((Array new:1) at: 1 put: 1; yourself)) * (gXx1)) + (((gXres) at:#gXpars) at:((Array new:1) at: 1 put: 2; yourself))) deepCopy.
gXx2 _ (GXmax run line: 0 proc: self method: true; p1:((gXxyearrays) at:#gXx); condEvaluate) deepCopy.
gXy2 _ (((((gXres) at:#gXpars) at:((Array new:1) at: 1 put: 1; yourself)) * (gXx2)) + (((gXres) at:#gXpars) at:((Array new:1) at: 1 put: 2; yourself))) deepCopy.
GXdraw run line: 0 proc: self method: false; gXline; p1:(gXx1); p2:(gXy1); p3:(gXx2); p4:(gXy2); p5:(gXDyellow); p6:(gXDdot); condEvaluate ].
GXlimits run line: 0 proc: self method: false; gXdefault; condEvaluate.
self up.
^gXres
!!
nil! "(fix for smalltalk bug, this line is ignored if there is a variable declaration error)"

"End of procedure"

"Procedure class definition:
 ==========================="
((Smalltalk at: #GXUmoduleUcompile ifAbsent: [ nil ]) isKindOf: Function class) ifTrue: [ ('Warning: Overriding previously defined procedure ', 'GXUmoduleUcompile' gtype) prinl ].
Smalltalk at: #GXUmoduleUcompile put: nil!
Function subclass: 'GXUmoduleUcompile'
       instanceVariableNames: ''
       classVariableNames: ''
       poolDictionaries: ''
       category: 'Genie Procedure'
!

"HEADER
 ------"

"parameters
 ----------"

GXUmoduleUcompile

addInstVarName: #gXfile;
compile:
'gXfile: aVar
	aVar notNil ifTrue:[gXfile _ self typeCheck: aVar shouldBe: GXString paramName: ''gXfile'']';
compile:
'p1: aVar
	aVar notNil ifTrue:[gXfile _ self typeCheck: aVar shouldBe: GXString paramName: ''gXfile'']';

addInstVarName: #gXcompUflags;
compile:
'gXcompUflags: aVar
	aVar notNil ifTrue:[gXcompUflags _ self typeCheck: aVar shouldBe: GXString paramName: ''gXcompUflags'']';
compile:
'p2: aVar
	aVar notNil ifTrue:[gXcompUflags _ self typeCheck: aVar shouldBe: GXString paramName: ''gXcompUflags'']';

addInstVarName: #gXlinkUflags;
compile:
'gXlinkUflags: aVar
	aVar notNil ifTrue:[gXlinkUflags _ self typeCheck: aVar shouldBe: GXString paramName: ''gXlinkUflags'']';
compile:
'p3: aVar
	aVar notNil ifTrue:[gXlinkUflags _ self typeCheck: aVar shouldBe: GXString paramName: ''gXlinkUflags'']';

addInstVarName: #gXsymbols;
compile:
'gXsymbols: aVar
	aVar notNil ifTrue:[gXsymbols _ self typeCheck: aVar shouldBe: GXString paramName: ''gXsymbols'']';
compile:
'p4: aVar
	aVar notNil ifTrue:[gXsymbols _ self typeCheck: aVar shouldBe: GXString paramName: ''gXsymbols'']';

addInstVarName: #gXtype;
compile:
'gXtype: aVar
	aVar notNil ifTrue:[gXtype _ self typeCheck: aVar shouldBe: GXString paramName: ''gXtype'']';
compile:
'p5: aVar
	aVar notNil ifTrue:[gXtype _ self typeCheck: aVar shouldBe: GXString paramName: ''gXtype'']';

addInstVarName: #gXoutput;
compile:
'gXoutput: aVar
	aVar notNil ifTrue:[gXoutput _ self typeCheck: aVar shouldBe: GXString paramName: ''gXoutput'']';
compile:
'p6: aVar
	aVar notNil ifTrue:[gXoutput _ self typeCheck: aVar shouldBe: GXString paramName: ''gXoutput'']'
!

"Local variables
 ---------------"

GXUmoduleUcompile

addInstVarName: #gXcompUflags;

addInstVarName: #gXlinkUflags;

addInstVarName: #gXtype;

addInstVarName: #gXoutput
!

!GXUmoduleUcompile methodsFor: 'initializing'!

initVars
	gXcompUflags _ ''.
	gXlinkUflags _ ''.
	gXtype _ 'FORTRAN'.
	gXoutput _ ''
!!

"BODY
 ----"

!GXUmoduleUcompile methodsFor: 'running'!

evaluate

    gXfile isNil ifTrue: ['MODULE: No file to compile given' prenl. ^nil ].
    gXsymbols isNil ifTrue: ['MODULE: No symbols supplied for export' prenl. 
			'MODULE: Rerun command with e.g. SYMBOLS="myfunc1,myfunc2"' prinl.
			'        where myfunc1,myfunc are the functions/subroutines you wish to call with MODULE:EXECUTE' prinl.
			^nil ]
.
GXUsystem run line: 0 proc: self method: false; p1:(((((((((((((('gmodule "' + (gXfile)) + '" "') + 'GENIE') + '" "') + (gXcompUflags)) + '" "') + (gXlinkUflags)) + '" "') + (gXsymbols)) + '" "') + (gXtype)) + '" "') + (gXoutput)) + '"'); condEvaluate.
self up.
^nil
!!
nil! "(fix for smalltalk bug, this line is ignored if there is a variable declaration error)"

"End of procedure"

GXcomment run line: 0 proc: self method: false; p1:'_module_compile'; p2:'Compile a FORTRAN program into a module'; condEvaluate!
"Procedure class definition:
 ==========================="
((Smalltalk at: #GXUmoduleUlist ifAbsent: [ nil ]) isKindOf: Function class) ifTrue: [ ('Warning: Overriding previously defined procedure ', 'GXUmoduleUlist' gtype) prinl ].
Smalltalk at: #GXUmoduleUlist put: nil!
Function subclass: 'GXUmoduleUlist'
       instanceVariableNames: ''
       classVariableNames: ''
       poolDictionaries: ''
       category: 'Genie Procedure'
!

"BODY
 ----"

!GXUmoduleUlist methodsFor: 'running'!

evaluate
GXprintin run line: 0 proc: self method: false; p1:'List of loaded modules'; condEvaluate.
GXcallout run line: 0 proc: self method: false; gXfacility:'extModule'; gXfunction:3; condEvaluate.
GXgUerrorUhandler run line: 0 proc: self method: false; condEvaluate.
self up.
^nil
!!
nil! "(fix for smalltalk bug, this line is ignored if there is a variable declaration error)"

"End of procedure"

GXcomment run line: 0 proc: self method: false; p1:'_module_list'; p2:'List currently loaded modules'; condEvaluate!
"Procedure class definition:
 ==========================="
((Smalltalk at: #GXUmoduleUunload ifAbsent: [ nil ]) isKindOf: Function class) ifTrue: [ ('Warning: Overriding previously defined procedure ', 'GXUmoduleUunload' gtype) prinl ].
Smalltalk at: #GXUmoduleUunload put: nil!
Function subclass: 'GXUmoduleUunload'
       instanceVariableNames: ''
       classVariableNames: ''
       poolDictionaries: ''
       category: 'Genie Procedure'
!

"HEADER
 ------"

"parameters
 ----------"

GXUmoduleUunload

addInstVarName: #gXfile;
compile:
'gXfile: aVar
	aVar notNil ifTrue:[gXfile _ self typeCheck: aVar shouldBe: GXString paramName: ''gXfile'']';
compile:
'p1: aVar
	aVar notNil ifTrue:[gXfile _ self typeCheck: aVar shouldBe: GXString paramName: ''gXfile'']'!

"result
 ------"

GXUmoduleUunload

addInstVarName: #gXres
!

!GXUmoduleUunload methodsFor: 'initializing'!

initVars

!!

"BODY
 ----"

!GXUmoduleUunload methodsFor: 'running'!

evaluate
(GXdefined run line: 0 proc: self method: true; p1:(gXfile); condEvaluate) ifTrue: [ 
GXcallout run line: 0 proc: self method: false; gXfacility:'extModule'; gXfunction:4; condEvaluate ] ifFalse: [ 
GXprinten run line: 0 proc: self method: false; p1:'MODULE: No file to unload given'; condEvaluate ] .
self up.
^gXres
!!
nil! "(fix for smalltalk bug, this line is ignored if there is a variable declaration error)"

"End of procedure"

"Procedure class definition:
 ==========================="
((Smalltalk at: #GXUmoduleUload ifAbsent: [ nil ]) isKindOf: Function class) ifTrue: [ ('Warning: Overriding previously defined procedure ', 'GXUmoduleUload' gtype) prinl ].
Smalltalk at: #GXUmoduleUload put: nil!
Function subclass: 'GXUmoduleUload'
       instanceVariableNames: ''
       classVariableNames: ''
       poolDictionaries: ''
       category: 'Genie Procedure'
!

"HEADER
 ------"

"parameters
 ----------"

GXUmoduleUload

addInstVarName: #gXfile;
compile:
'gXfile: aVar
	aVar notNil ifTrue:[gXfile _ self typeCheck: aVar shouldBe: GXString paramName: ''gXfile'']';
compile:
'p1: aVar
	aVar notNil ifTrue:[gXfile _ self typeCheck: aVar shouldBe: GXString paramName: ''gXfile'']';

addInstVarName: #gXdesc;
compile:
'gXdesc: aVar
	aVar notNil ifTrue:[gXdesc _ self typeCheck: aVar shouldBe: GXString paramName: ''gXdesc'']';
compile:
'p2: aVar
	aVar notNil ifTrue:[gXdesc _ self typeCheck: aVar shouldBe: GXString paramName: ''gXdesc'']'!

"result
 ------"

GXUmoduleUload

addInstVarName: #gXres
!

"Local variables
 ---------------"

GXUmoduleUload

addInstVarName: #gXdesc
!

!GXUmoduleUload methodsFor: 'initializing'!

initVars
	gXdesc _ '<no description>'
!!

"BODY
 ----"

!GXUmoduleUload methodsFor: 'running'!

evaluate
(GXdefined run line: 0 proc: self method: true; p1:(gXfile); condEvaluate) ifTrue: [ 
GXcallout run line: 0 proc: self method: false; gXfacility:'extModule'; gXfunction:1; condEvaluate ] ifFalse: [ 
GXprinten run line: 0 proc: self method: false; p1:'MODULE: No file to load given'; condEvaluate ] .
GXgUerrorUhandler run line: 0 proc: self method: false; condEvaluate.
self up.
^gXres
!!
nil! "(fix for smalltalk bug, this line is ignored if there is a variable declaration error)"

"End of procedure"

GXcomment run line: 0 proc: self method: false; p1:'_module_load'; p2:'Load a compiled user program into a module'; condEvaluate!
"Procedure class definition:
 ==========================="
((Smalltalk at: #GXUmoduleUexecute ifAbsent: [ nil ]) isKindOf: Function class) ifTrue: [ ('Warning: Overriding previously defined procedure ', 'GXUmoduleUexecute' gtype) prinl ].
Smalltalk at: #GXUmoduleUexecute put: nil!
Function subclass: 'GXUmoduleUexecute'
       instanceVariableNames: ''
       classVariableNames: ''
       poolDictionaries: ''
       category: 'Genie Procedure'
!

"HEADER
 ------"

"parameters
 ----------"

GXUmoduleUexecute

addInstVarName: #gXfunc;
compile:
'gXfunc: aVar
	aVar notNil ifTrue:[gXfunc _ self typeCheck: aVar shouldBe: GXString paramName: ''gXfunc'']';
compile:
'p1: aVar
	aVar notNil ifTrue:[gXfunc _ self typeCheck: aVar shouldBe: GXString paramName: ''gXfunc'']';

addInstVarName: #gXpars;
compile:
'gXpars: aVar
	aVar notNil ifTrue:[gXpars _ self typeCheck: aVar shouldBe: GXWorkspace paramName: ''gXpars'']';
compile:
'p2: aVar
	aVar notNil ifTrue:[gXpars _ self typeCheck: aVar shouldBe: GXWorkspace paramName: ''gXpars'']';

addInstVarName: #gXtype;
compile:
'gXtype: aVar
	aVar notNil ifTrue:[gXtype _ self typeCheck: aVar shouldBe: GXString paramName: ''gXtype'']';
compile:
'p3: aVar
	aVar notNil ifTrue:[gXtype _ self typeCheck: aVar shouldBe: GXString paramName: ''gXtype'']'!

"result
 ------"

GXUmoduleUexecute

addInstVarName: #gXres
!

"Local variables
 ---------------"

GXUmoduleUexecute

addInstVarName: #gXtype;

addInstVarName: #gXpars
!

!GXUmoduleUexecute methodsFor: 'initializing'!

initVars
	gXtype _ 'FORTRAN'.
	gXpars _ (GXfields run line: 0 proc: self method: true; condEvaluate)
!!

"BODY
 ----"

!GXUmoduleUexecute methodsFor: 'running'!

evaluate
(GXdefined run line: 0 proc: self method: true; p1:(gXfunc); condEvaluate) not ifTrue: [ 
GXprinten run line: 0 proc: self method: false; p1:'MODULE: No function to execute given'; condEvaluate.
[nil.
self up.
^gXres] value ].
gXres _ (GXfields run line: 0 proc: self method: true; p1:((GXlength run line: 0 proc: self method: true; p1:(gXpars); condEvaluate) + 1); condEvaluate) deepCopy.
GXcallout run line: 0 proc: self method: false; gXfacility:'extModule'; gXfunction:2; condEvaluate.
(GXgUerrorUhandler run line: 0 proc: self method: true; condEvaluate) ifTrue: [ 
gXres _ (gXU) deepCopy ].
self up.
^gXres
!!
nil! "(fix for smalltalk bug, this line is ignored if there is a variable declaration error)"

"End of procedure"

GXcomment run line: 0 proc: self method: false; p1:'_module_execute'; p2:'Run a function contained in a previously loaded module'; condEvaluate!
"Procedure class definition:
 ==========================="
((Smalltalk at: #GXmodule ifAbsent: [ nil ]) isKindOf: Function class) ifTrue: [ ('Warning: Overriding previously defined procedure ', 'GXmodule' gtype) prinl ].
Smalltalk at: #GXmodule put: nil!
Function subclass: 'GXmodule'
       instanceVariableNames: ''
       classVariableNames: ''
       poolDictionaries: ''
       category: 'Genie Procedure'
!

"HEADER
 ------"

"parameters
 ----------"

GXmodule

addInstVarName: #gXp1;
compile:
'gXp1: aVar
	aVar notNil ifTrue:[ gXp1 _ aVar ]';
compile:
'p1: aVar
	aVar notNil ifTrue:[ gXp1 _ aVar ]';

addInstVarName: #gXp2;
compile:
'gXp2: aVar
	aVar notNil ifTrue:[ gXp2 _ aVar ]';
compile:
'p2: aVar
	aVar notNil ifTrue:[ gXp2 _ aVar ]';

addInstVarName: #gXp3;
compile:
'gXp3: aVar
	aVar notNil ifTrue:[ gXp3 _ aVar ]';
compile:
'p3: aVar
	aVar notNil ifTrue:[ gXp3 _ aVar ]';

addInstVarName: #gXp4;
compile:
'gXp4: aVar
	aVar notNil ifTrue:[ gXp4 _ aVar ]';
compile:
'p4: aVar
	aVar notNil ifTrue:[ gXp4 _ aVar ]';

addInstVarName: #gXp5;
compile:
'gXp5: aVar
	aVar notNil ifTrue:[ gXp5 _ aVar ]';
compile:
'p5: aVar
	aVar notNil ifTrue:[ gXp5 _ aVar ]';

addInstVarName: #gXfile;
compile:
'gXfile: aVar
	aVar notNil ifTrue:[gXfile _ self typeCheck: aVar shouldBe: GXString paramName: ''gXfile'']';
compile:
'p6: aVar
	aVar notNil ifTrue:[gXfile _ self typeCheck: aVar shouldBe: GXString paramName: ''gXfile'']';

addInstVarName: #gXfunc;
compile:
'gXfunc: aVar
	aVar notNil ifTrue:[gXfunc _ self typeCheck: aVar shouldBe: GXString paramName: ''gXfunc'']';
compile:
'p7: aVar
	aVar notNil ifTrue:[gXfunc _ self typeCheck: aVar shouldBe: GXString paramName: ''gXfunc'']';

addInstVarName: #gXpars;
compile:
'gXpars: aVar
	aVar notNil ifTrue:[gXpars _ self typeCheck: aVar shouldBe: GXWorkspace paramName: ''gXpars'']';
compile:
'p8: aVar
	aVar notNil ifTrue:[gXpars _ self typeCheck: aVar shouldBe: GXWorkspace paramName: ''gXpars'']';

addInstVarName: #gXdesc;
compile:
'gXdesc: aVar
	aVar notNil ifTrue:[gXdesc _ self typeCheck: aVar shouldBe: GXString paramName: ''gXdesc'']';
compile:
'p9: aVar
	aVar notNil ifTrue:[gXdesc _ self typeCheck: aVar shouldBe: GXString paramName: ''gXdesc'']';

addInstVarName: #gXcompUflags;
compile:
'gXcompUflags: aVar
	aVar notNil ifTrue:[gXcompUflags _ self typeCheck: aVar shouldBe: GXString paramName: ''gXcompUflags'']';
compile:
'p10: aVar
	aVar notNil ifTrue:[gXcompUflags _ self typeCheck: aVar shouldBe: GXString paramName: ''gXcompUflags'']';

addInstVarName: #gXlinkUflags;
compile:
'gXlinkUflags: aVar
	aVar notNil ifTrue:[gXlinkUflags _ self typeCheck: aVar shouldBe: GXString paramName: ''gXlinkUflags'']';
compile:
'p11: aVar
	aVar notNil ifTrue:[gXlinkUflags _ self typeCheck: aVar shouldBe: GXString paramName: ''gXlinkUflags'']';

addInstVarName: #gXsymbols;
compile:
'gXsymbols: aVar
	aVar notNil ifTrue:[gXsymbols _ self typeCheck: aVar shouldBe: GXString paramName: ''gXsymbols'']';
compile:
'p12: aVar
	aVar notNil ifTrue:[gXsymbols _ self typeCheck: aVar shouldBe: GXString paramName: ''gXsymbols'']';

addInstVarName: #gXoutput;
compile:
'gXoutput: aVar
	aVar notNil ifTrue:[gXoutput _ self typeCheck: aVar shouldBe: GXString paramName: ''gXoutput'']';
compile:
'p13: aVar
	aVar notNil ifTrue:[gXoutput _ self typeCheck: aVar shouldBe: GXString paramName: ''gXoutput'']'!

"qualifiers
 ----------"

GXmodule

addInstVarName: #gXload;
compile:
'gXload
	gXload _ true';
compile:
'gXload: aBoolean
	gXload _ aBoolean';

addInstVarName: #gXexecute;
compile:
'gXexecute
	gXexecute _ true';
compile:
'gXexecute: aBoolean
	gXexecute _ aBoolean';

addInstVarName: #gXlist;
compile:
'gXlist
	gXlist _ true';
compile:
'gXlist: aBoolean
	gXlist _ aBoolean';

addInstVarName: #gXcompile;
compile:
'gXcompile
	gXcompile _ true';
compile:
'gXcompile: aBoolean
	gXcompile _ aBoolean';

addInstVarName: #gXc;
compile:
'gXc
	gXc _ true';
compile:
'gXc: aBoolean
	gXc _ aBoolean';

addInstVarName: #gXunload;
compile:
'gXunload
	gXunload _ true';
compile:
'gXunload: aBoolean
	gXunload _ aBoolean'!

"result
 ------"

GXmodule

addInstVarName: #gXres
!

"Local variables
 ---------------"

GXmodule

addInstVarName: #gXtype
!

!GXmodule methodsFor: 'initializing'!

initVars
	gXload _ false.
	gXexecute _ false.
	gXlist _ false.
	gXcompile _ false.
	gXc _ false.
	gXunload _ false
!!

"BODY
 ----"

!GXmodule methodsFor: 'running'!

evaluate
 gXc ifTrue: [ gXtype _ 'C' ] .
(gXload) ifTrue: [ 
 
	gXfile isNil ifTrue: [ gXfile _ gXp1 ].
	gXdesc isNil ifTrue: [ gXdesc _ gXp2 ]
    .
gXres _ (GXUmoduleUload run line: 0 proc: self method: true; p1:(gXfile); p2:(gXdesc); condEvaluate) deepCopy ] ifFalse: [ (gXexecute) ifTrue: [ 

        gXfunc isNil ifTrue: [ gXfunc _ gXp1 ].
        gXpars isNil ifTrue: [ gXpars _ gXp2 ].
        gXres _ (GXUmoduleUexecute run line: 0 proc: self method: true; p1:(gXfunc); p2:(gXpars); p3:(gXtype); condEvaluate)
     ] ifFalse: [ (gXlist) ifTrue: [ 
GXUmoduleUlist run line: 0 proc: self method: false; condEvaluate ] ifFalse: [ (gXcompile) ifTrue: [ 
 
	gXfile isNil ifTrue: [ gXfile _ gXp1 ]. 
	gXcompUflags isNil ifTrue: [ gXcompUflags _ gXp2 ]. 
	gXlinkUflags isNil ifTrue: [ gXlinkUflags _ gXp3 ].
	gXsymbols isNil ifTrue: [ gXsymbols _ gXp4 ].
	gXoutput isNil ifTrue: [ gXoutput _ gXp5 ]
    .
GXUmoduleUcompile run line: 0 proc: self method: false; p1:(gXfile); p2:(gXcompUflags); p3:(gXlinkUflags); p4:(gXsymbols); p5:(gXtype); p6:(gXoutput); condEvaluate ] ifFalse: [ (gXunload) ifTrue: [ 
 
	gXfile isNil ifTrue: [ gXfile _ gXp1 ]
    .
gXres _ (GXUmoduleUunload run line: 0 proc: self method: true; p1:(gXfile); condEvaluate) deepCopy ] ifFalse: [ 
GXprinten run line: 0 proc: self method: false; p1:'MODULE: Invalid or non-existant qualifier'; condEvaluate ] ]]]].
self up.
^gXres
!!
nil! "(fix for smalltalk bug, this line is ignored if there is a variable declaration error)"

"End of procedure"

GXcomment run line: 0 proc: self method: false; p1:'module'; p2:'Load and run a compiled user program (module)'; condEvaluate!
"Procedure class definition:
 ==========================="
((Smalltalk at: #GXhelp ifAbsent: [ nil ]) isKindOf: Function class) ifTrue: [ ('Warning: Overriding previously defined procedure ', 'GXhelp' gtype) prinl ].
Smalltalk at: #GXhelp put: nil!
Function subclass: 'GXhelp'
       instanceVariableNames: ''
       classVariableNames: ''
       poolDictionaries: ''
       category: 'Genie Procedure'
!

"BODY
 ----"

!GXhelp methodsFor: 'running'!

evaluate
GXprintn run line: 0 proc: self method: false; p1:'For help on a specific command, type the command name followed by /HELP'; condEvaluate.
GXprintn run line: 0 proc: self method: false; p1:'e.g. DISPLAY/HELP'; condEvaluate.
GXprintn run line: 0 proc: self method: false; condEvaluate.
GXprintn run line: 0 proc: self method: false; p1:'To search for a command, press <Ctrl>-K and input a keyword'; condEvaluate.
GXprintn run line: 0 proc: self method: false; condEvaluate.
GXprintn run line: 0 proc: self method: false; p1:'To view all commands starting with a specific combination of letters, type'; condEvaluate.
GXprintn run line: 0 proc: self method: false; p1:'those letters and then press TAB twice (note that pressing TAB twice on a'; condEvaluate.
GXprintn run line: 0 proc: self method: false; p1:'blank command line will list all GENIE command)'; condEvaluate.
GXprintn run line: 0 proc: self method: false; condEvaluate.
GXprintn run line: 0 proc: self method: false; p1:'To halt execution of a command press <Ctrl>-C'; condEvaluate.
GXprintn run line: 0 proc: self method: false; condEvaluate.
GXprintn run line: 0 proc: self method: false; p1:'To leave GENIE, type EXIT or QUIT'; condEvaluate.
GXprintn run line: 0 proc: self method: false; condEvaluate.
GXprintn run line: 0 proc: self method: false; p1:'GENIE example programs are located in the examples subdirectory'; condEvaluate.
GXprintn run line: 0 proc: self method: false; p1:'of '; p2:(GXUgetenv run line: 0 proc: self method: true; p1:'GENIE_DIR'; condEvaluate); condEvaluate.
self up.
^nil
!!
nil! "(fix for smalltalk bug, this line is ignored if there is a variable declaration error)"

"End of procedure"

GXalias run line: 0 proc: self method: false; p1:'info'; p2:'help'; condEvaluate!

GXabs comment: 'Calculate the absolute values.
   
     [1mSQRT([0mx) [4mx=Generic[0m Calculate |x|.
     [1m[0m
     example:
     
        # Print the absolute value
        printn Abs(-3.3)
        3.3

     Note: This function can be applied to all numeric types (including
     arrays & workspaces).'.
!

GXalias comment: 'Allows the definition of aliases for command names and qualifiers
   
     [1mALIAS[0m [[4malias_name=String[0m] [[4mfull_name=String[0m] Define one or more
     [1m[0maliases for commands and qualifiers.
     [1m[0m
     example:
     
        # Some aliases defined for the display command
        >> alias "d/e/m/l" "display/errors/markers/line"
        >> d/e/l
        
        # can have more than one alias defined for a command
        >> alias "showit/Ebars/stars/wigglything" "display/errors/markers/line"
        
        >> showit/e_bars/wigglything

     Note: Open GENIE already defines some common aliases for commands
     (e.g. d/e for display/errors).'.
!

GXalter comment: 'Sets various defaults for the subsequent Display(), Plot() and
   Multiplot() commands
   [1m[0m
     [1mALTER/STATUS[0m   Reports the values of all settings.
     [1mALTER/BINNING[0m [4mp1=Integer[0m Sets bin grouping for plotting
     [1mALTER/DEVICE[0m [4mp1=String[0m Change interactive device
     [1mALTER/FONT[0m [4mp1=Font[0m Set text font
     [1mALTER/HARDCOPY[0m [4mp1=String[0m Hardcopy device
     [1mALTER/LINECOLOUR[0m [4mp1=Colour[0m Axes/box colour
     [1mALTER/LINETYPE[0m [4mp1=LineStyle[0m Line style for line plots
     [1mALTER/LINEWIDTH[0m [4mp1=Real[0m Overall line thickness
     [1mALTER/MARKERS[0m [4mp1=Marker[0m Sets the marker plot symbol
     [1mALTER/MARKERSIZE[0m [4mp1=Real[0m Size of markers on a Display()
     [1mALTER/PLOT[0m [4mxmin=Real[0m [4mxmax=Real[0m [4mymin=Real[0m [4mymax=Real[0m Change plot size
     [1mALTER/PLOTCOLOUR[0m [4mp1=Colour[0m Data plot colour
     [1mALTER/SIZE[0m [4mp1=Real[0m X window display size (scale factor)
     [1mALTER/TEXTHEIGHT[0m [4mp1=Real[0m Text height
     [1mALTER/TEXTCOLOUR[0m [4mp1=Colour[0m Label/title colour
     [1m[0m
     example:
     
        # Change the axis colour to blue
        >> Alter/linecolour $BLUE

     Note: See also the Toggle() command for changing plot parameters'.
!

GXarccos comment: 'Generic trig functions.
   
     [1mARCCOS([0mx) [4mx=Generic[0m Arccosine in radians.
     [1m[0m
     example:
     
        # print result in degrees
        y = Arccos(0.5) * 180 / $pi
        #take the arccos of all elements in an array
        a = Dimensions(10)      # create a 10 element array
        fill a 0.5              # Set all elements to 0.5
        y = Arccos(a)

     Note: These functions can be applied to all numeric types
     (including arrays & workspaces).'.
!

GXarcsin comment: 'Generic trig functions.
   
     [1mARCSIN([0mx) [4mx=Generic[0m Arcsine in radians.
     [1m[0m
     example:
     
        # print result in degrees
        y = Arcsin(0.5) * 180 / $pi
        #take the arcsine of all elements in an array
        a = Dimensions(10)      # create a 10 element array
        fill a 0.5              # Set all elements to 0.5
        y = Arcsin(a)

     Note: These functions can be applied to all numeric types
     (including arrays & workspaces).'.
!

GXarctan comment: 'Generic trig functions.
   
     [1mARCTAN([0mx) [4mx=Generic[0m Arctangent in radians.
     [1m[0m
     example:
     
        # print result in degrees
        y = Arctan(1000.0) * 180 / $pi
        #take the arctan of all elements in an array
        a = Dimensions(10)      # create a 10 element array
        fill a 0.5              # Set all elements to 0.5
        y = Arctan(a)

     Note: These functions can be applied to all numeric types
     (including arrays & workspaces).'.
!

GXasUinteger comment: 'Truncates an Open GENIE real number to an integer.
   
     [1mAS_INTEGER[0m [4mvar=Real[0m Convert to integer
     [1m[0m
     example:
     
        # Convert 2.5 to an integer
        >> printn As_integer(2.5)
        2

     Note: Add 0.5 before truncating to get the nearest integer instead'.
!

GXasUnew comment: 'Converts a GENIE-V2 compatible workspace/data file to a new format
   workspace
   [1m[0m
   [1m([0mdescription awaiting ratification of NeXus)'.
!

GXasUold comment: 'Converts a new format workspace/data file to a GENIE-V2 compatible
   workspace.
   [1m[0m
   [1m([0mdescription awaiting ratification of NeXus)'.
!

GXasUstring comment: 'Converts any genie variable into a string.
   
     [1mAS_STRING[0m [4mvar=Generic[0m Print a variable into an internal string
     [1m[0m
     example:
     
        # Format a real to two decimal places
        >> printn myval
        34.234567
        >> str = as_string(myval)
        >> prec = Substring( str, _, locate(str,".") + 2 )
        >> printn prec
        34.23
        # useful for graphics
        >> Draw/text xcoord=0.5 ycoord=0.5 prec

     Note: Has a similar purpose to the FORTRAN internal write.'.
!

GXasUvariable comment: 'Converts a string into an internal Open GENIE variable type.
   
     [1mAS_VARIABLE[0m [4mastring=String[0m Read a string into a variable
     [1m[0m
     example:
     
        # Obtain a number from an awkward string
        >> printn w.title
        BaCuO2 Sprogget & Sylvester t=.589E+1K
        >> tstr = Substring(w.title, locate(w.title,"=")+1)
        >> printn as_variable(tstr)
        5.89

     Note: Only reads valid numbers otherwise returns the string.'.
!

GXasciifile comment: 'Read or write any format of ASCII data.
   
     [1mASCIIFILE:OPEN()[0m [4mfile=String[0m [[4mcomment=String[0m] [[4mdelimiter=String[0m]
     [1mO[0mpen an ASCII file
     [1mASCIIFILE/CLOSE[0m [4mhandle=AsciiFile[0m Close an open file
     [1mASCIIFILE:DATA()[0m
     [1m[0m[:RESET] [4mhandle=AsciiFile[0m Return all data currently accumulated in
     [1m[0ma workspace.
     [1mASCIIFILE:LINES()[0m [4mhandle=AsciiFile[0m Return the number of data lines
     [1m[0mleft to read
     [1mASCIIFILE/READFIXED[0m [4mhandle=AsciiFile[0m [[4mfields=String[0m] [4mformat=String[0m
     [1m[0m[[4mcount=Integer[0m] Read fixed format data
     [1mASCIIFILE/READFREE[0m [4mhandle=AsciiFile[0m [[4mfields=String[0m]
     [1m[0m[[4mseparator=String[0m] [[4mcount=Integer[0m] Read free format data
     [1mASCIIFILE/SKIP[0m [4mhandle=AsciiFile[0m [[4mnlines=Integer[0m] Skip lines on
     [1m[0minput
     [1mASCIIFILE/WRITEFREE[0m [4mhandle=AsciiFile[0m [4mcomment=String[0m
     [1m[0m[4mseparator=String[0m [4mgv1=Any[0m [[4mgv2=Any[0m] [[4mgv3=Any[0m] [[4mgv4=Any[0m] Write free
     [1m[0mformat data
     [1mASCIIFILE/REWIND[0m [4mhandle=AsciiFile[0m Close and re-open a file at the
     [1m[0mstart
     [1m[0m
     example:
     
        # Read three data arrays (in columns separated by whitespace)
        # and assign to arrays in fields X, Y, and E in the new workspace.
        Handle = Asciifile:Open("myilldata.dat")
        wk1 = Asciifile:Readfree(Handle, "X,Y,E", $WHITESPACE, 8)
        printin wk1
          Workspace []
          (
            x = [1.0 1.1 1.2 1.3 1.4 ...] Array(8 )
            y = [2.1 2.1 2.2 2.0 2.0 ...] Array(8 )
            e = [0.0 0.1 0.2 0.0 0.1 ...] Array(8 )
          )'.
!

GXassign comment: 'Select a run for input by giving the ISIS run number
   
     [1mASSIGN[0m [4mp1=Integer[0m Selects the default run for input with the S()
     [1m[0mcommand.
     [1m[0m
     example:
     
        # Select run 2004 and read in a spectrum
        >> Assign 2004
        >> w=s(2)               # 2nd Spectrum

     Note: Assign nnn is equivalent to Set/File/Input Cfn(nnn)'.
!

GXaxes comment: 'Displays an axes on the graphics device
   
     [1mAXES/DRAW[0m [/XLOG] [/YLOG] [/YHORIZ] [/NOXNUM] [/NOYNUM] [/XLINEAR]
     [1m[0m[/YLINEAR] [/YVERTICAL] [/XNUM] [/YNUM] [[4mcolour=Colour[0m] [[4msize=Real[0m]
     [1m[0m[[4mfont=Font[0m] [[4mline_thickness=Real[0m] [[4mline_type=LineStyle[0m] Draws an
     [1mA[0mxis on a graphics device
     [1mAXES/ALTER[0m [/XLOG] [/YLOG] [/YHORIZ] [/NOXNUM] [/NOYNUM] [/XLINEAR]
     [1m[0m[/YLINEAR] [/YVERTICAL] [/XNUM] [/YNUM] [[4mcolour=Colour[0m] [[4msize=Real[0m]
     [1m[0m[[4mfont=Font[0m] [[4mline_thickness=Real[0m] [[4mline_type=LineStyle[0m]
     [1m[0m[4mobject=Gobject[0m Alters an axis already drawn on a graphics device
     [1m[0m
     example:
     
        # Draw green Axes on a device with a window already
        # displayed, then alters the colour.
        >> a = axes:draw(colour=$green)
        >> axes/alter colour=$red object=a

     Note: By default the axes are linear, x is labeled horizontally and
     y is labeled vertically'.
!

GXbracket comment: 'Finds the index position of a given value in an array.
   
     [1mBRACKET[0m [4mxarray=Realarray[0m [4mxval=Real[0m Allows bracketing of X values.
     [1m[0m
     example:
     
        # Rebin data to a smaller range manually
        >> lower = bracket(w.x, 1000.0)
        >> upper = bracket(w.x, 4000.0)
        >> w.x = w.x[(lower):(upper)]
        >> w.y = w.y[(lower):(upper-1)]
        >> w.e = w.e[(lower):(upper-1)]

     Note: Normally this would be done with the Rebin() command.'.
!

GXbuffer comment: 'Sorry, no help is available for this command'
!

GXcalledUasUfunction comment: 'Test if a procedure was called as a function, i.e. x = command(y)
   
     [1mCALLED_AS_FUNCTION[0m Test whether a procedure should alter its
     [1m[0mparameters
     [1m[0m
     example:
     
        # Procedure which checks how it is called
        PROCEDURE double
        PARAMETERS w=workspace
        RESULT wres
                IF Called_as_function()         # leave the parameter untouched
                        wres = w
                        wres.y = w.y * 2.0
                ELSE                            # change the parameter
                        w.y = w.y * 2.0
                ENDIF
        ENDPROCEDURE'.
!

GXcallout comment: 'Sorry, no help is available for this command'
!

GXcd comment: 'Change the default directory from within Open GENIE
   
     [1mCD[0m [[4mpath=String[0m] Change the working directory
     [1m[0m
     example:
     
        # Change to the examples directory
        >> cd "/usr/local/genie/examples"
        >> dir'.
!

GXcell comment: 'Colour cell plot for gridded data.
   
     [1mCELL/DRAW[0m [/SMOOTH] [/LOG] [/SQRT] [4mvalues=RealArray[0m
     [1m[0m[4mtable=ColourTable[0m [[4mtr=RealArray[0m] [[4mvalmax=Real[0m] [[4mvalmin=Real[0m] Plots
     [1m[0ma cell on a graphics device
     [1mCELL/ALTER[0m [/SMOOTH] [/LOG] [/SQRT] [4mvalues=RealArray[0m
     [1m[0m[4mtable=ColourTable[0m [[4mtr=RealArray[0m] [[4mvalmax=Real[0m] [[4mvalmin=Real[0m]
     [1m[0m[4mobject=Gobject[0m Alters a cell already drawn on a graphics device
     [1m[0m
     example:
     
        # Produce a cell plot of 2-D data
        >> yindex=dimensions(spec_count)
        >> fill yindex 1.0 1.0
        >> win_twod x=w.x y=yindex
        >> Cell/Draw values=w.y table=$heat()'.
!

GXcellUarray comment: 'Colour cell plot for gridded data with explicit gridding.
   
     [1mCELL_ARRAY/DRAW[0m [/SMOOTH] [/LOG] [/SQRT] [4mvalues=RealArray[0m
     [1m[0m[4mtable=ColourTable[0m [4mxarray=RealArray[0m [4myarray=RealArray[0m [[4mvalmax=Real[0m]
     [1m[0m[[4mvalmin=Real[0m] Plots a cell_array on a graphics device
     [1mCELL_ARRAY/ALTER[0m [/SMOOTH] [/LOG] [/SQRT] [4mvalues=RealArray[0m
     [1m[0m[4mtable=ColourTable[0m [4mxarray=RealArray[0m [4myarray=RealArray[0m [[4mvalmax=Real[0m]
     [1m[0m[[4mvalmin=Real[0m] [4mobject=Gobject[0m Alters a cell_array already drawn on a
     [1m[0mgraphics device
     [1m[0m
     example:
     
        # Produce a cell plot of 2-D data on a non-linear grid
        >> Cell_array/Draw values=my_points               xarray=non_linear_x yarray=non_linear_y               table=$rainbow()'.
!

GXcellUfunction comment: 'Coloured cell plot for ungridded data.
   
     [1mCELL_FUNCTION/DRAW[0m [/SMOOTH] [/LOG] [/SQRT] [4mvalues=RealArray[0m
     [1m[0m[4mtable=ColourTable[0m [4mfunction_name=String[0m [[4mvalmax=Real[0m] [[4mvalmin=Real[0m]
     [1mP[0mlots a cell_function on a graphics device
     [1mCELL_FUNCTION/ALTER[0m [/SMOOTH] [/LOG] [/SQRT] [4mvalues=RealArray[0m
     [1m[0m[4mtable=ColourTable[0m [4mfunction_name=String[0m [[4mvalmax=Real[0m] [[4mvalmin=Real[0m]
     [1m[0m[4mobject=Gobject[0m Alters a cell_function already drawn on a graphics
     [1m[0mdevice
     [1m[0m
     example:
     
        # Plot some ungridded data with a
        # a user specified module function
        >> Cell_function/Draw values=my_points               function_name="my_gridding_function"

     Note: Need to load user function with the Module/Load command
     before use.'.
!

GXcellUwedge comment: 'Displays a cell_wedge (colour key) on the graphics device.
   
     [1mCELL_WEDGE/DRAW[0m [/HORIZONTAL] [/VERTICAL] [4mcell=Gobject[0m Draws a
     [1m[0mcell_wedge onto a graphics device.
     [1mCELL_WEDGE/ALTER[0m [/HORIZONTAL] [/VERTICAL] [4mobject=Gobject[0m Alters a
     [1m[0mcell_wedge already drawn on a graphics device.
     [1m[0m
     example:
     
        # Annotate an already drawn cell plot
        >> my_cell = Cell:Draw(w.y, $heat())
        >> Cell_Wedge/horizontal  my_cell

     Note: Must be in a separate window to the cell plot. By default the
     wedge is vertical.'.
!

GXcentreUbins comment: 'Centre the bins of a real array (i.e. convert from histogram to point
   mode).
   [1m[0m
     [1mCENTRE_BINS[0m [4marray=Array[0m Centre the bins of a real array (i.e.
     [1m[0mconvert from histogram to point mode).
     [1m[0m
     example:
     
        # Do a rough conversion from a histogram workspace to
        # to a new point mode workspace
        point_mode = my_work
        point_mode.x = centre_bins(my_work.x)
        
        # convert an array to point mode
        centre_bins my_array'.
!

GXcfn comment: 'Construct the full filename from run number and defaults
   
     [1mCFN[0m [[4mnumber=Integer[0m] [[4mname=String[0m] Construct a full path name for
     [1m[0mthe given run or file number.
     [1m[0m
     example:
     
        >> Set/Disk "IRIS$DISK0:"
        >> Set/Dir "[IRSMGR.DATA]"
        >> Set/Inst "IRS"
        >> Set/Ext ".RAW"
        >> printn Cfn(345)
        IRIS$DISK0:[IRSMGR.DATA]IRS00345.RAW
        >> printn Cfn("special.dat")
        IRIS$DISK0:[IRSMGR.DATA]SPECIAL.DAT

     Note: Cfn() adds the appropriate number of 0s for ISIS raw file
     names.'.
!

GXcolour comment: 'Define a new graphics colour based on one of several models
   
     [1mCOLOUR:RGB()[0m [4mr=Real[0m [4mg=Real[0m [4mb=Real[0m Create new colour using RGB model
     [1mCOLOUR:HLS()[0m [4mh=Real[0m [4ml=Real[0m [4ms=Real[0m Create new colour using HLS model
     [1mCOLOUR:NAMED()[0m [4mname=String[0m Access a named palette colour
     [1m[0m
     example:
     
        # Draw lines in colours
        # created from two different models
        >> Draw/line 0.0 0.0 1.0 1.0 colour:rgb(0.5, 0.6, 0.7)
        >> Draw/line 0.0 0.0 1.0 1.0 colour:named("SteelBlue")

     Note: The number of simultaneous colours available depends on the
     display hardware.'.
!

GXcolourtable comment: 'Manipulate graphics colour tables (used with Cell() commands).
   
     [1mCOLOURTABLE:CREATE()[0m [4mred=RealArray[0m [4mgreen=RealArray[0m [4mblue=RealArray[0m
     [1m[0m[4mncolours=Integer[0m [4mcontrast=Real[0m [4mbrightness=Real[0m Creates and returns
     [1m[0ma new colour table.
     [1m[0m[/NOSHARE] Avoid sharing colours
     [1mCOLOURTABLE/DELETE[0m [4mtable=ColourTable[0m Deletes and frees up colours
     [1m[0mfrom an existing colourtable.
     [1m[0m
     example:
     
        # Create, use in a cell plot and then
        # delete a colourtable.
        >> ct1 = Colourtable:create(ra,ga,ba,80,1.0,1.0)
        >> Cell/alter object=obj(1,2,3) table=ct1
        >> Colourtable/delete ct1

     Note: The data in the real arrays will be interpolated if ncolours
     exceeds the number of elements in the arrays.'.
!

GXcomment comment: 'Sorry, no help is available for this command'
!

GXcontour comment: 'Makes a contour plot on the graphics device.
   
     [1mCONTOUR/DRAW[0m [/LOG] [/SQRT] [4mvalues=RealArray[0m [[4mcontours=RealArray[0m]
     [1m[0m[[4mncont=Integer[0m] [[4mtr=RealArray[0m] [[4mcolour=Colour[0m]
     [1m[0m[[4mline_type=LineStyle[0m] [[4mline_thickness=Real[0m] Draw a contour plot
     [1m[0monto a graphics device.
     [1mCONTOUR/ALTER[0m [/LOG] [/SQRT] [[4mvalues=RealArray[0m]
     [1m[0m[[4mcontours=RealArray[0m] [[4mncont=Integer[0m] [[4mtr=RealArray[0m] [[4mcolour=Colour[0m]
     [1m[0m[[4mline_type=LineStyle[0m] [[4mline_thickness=Real[0m] [4mobject=Contour[0m Alters a
     [1m[0mpreviously drawn contour plot on a graphics device.
     [1m[0m
     example:
     
        # Produce a contour plot of 2-D data
        >> cp = Contour:Draw(my_points)

     Note: By default the contour is linear. Use contours or ncont, not
     both.'.
!

GXcontourUarray comment: 'Plots a contour_array on the graphics device
   
     [1mCONTOUR_ARRAY/DRAW[0m [/LOG] [/SQRT] [4mvalues=RealArray[0m
     [1m[0m[4mcontours=RealArray[0m [4mncont=Integer[0m [4mxarray=RealArray[0m [4myarray=RealArray[0m
     [1m[0m[[4mcolour=Colour[0m] [[4mline_type=LineStyle[0m] [[4mline_thickness=Real[0m] Draw a
     [1m[0mcontour array onto a graphics device.
     [1mCONTOUR_ARRAY/ALTER[0m [/LOG] [/SQRT] [[4mvalues=RealArray[0m]
     [1m[0m[[4mcontours=RealArray[0m] [[4mncont=Integer[0m] [[4mxarray=RealArray[0m]
     [1m[0m[[4myarray=RealArray[0m] [[4mcolour=Colour[0m] [[4mline_type=LineStyle[0m]
     [1m[0m[[4mline_thickness=Real[0m] [4mobject=Gobject[0m Alters a previously drawn
     [1m[0mcontour array on a graphics device.
     [1m[0m
     example:
     
        # Produce a contour plot of 2-D data on a non-linear grid
        >> Contour_array/Draw values=my_points               xarray=non_linear_x yarray=non_linear_y               table=$heat()

     Note: Use contours or ncont, not both.'.
!

GXcontourUfunction comment: 'Plots contours for ungridded data.
   
     [1mCONTOUR_FUNCTION/DRAW[0m [/LOG] [/SQRT] [4mvalues=RealArray[0m
     [1m[0m[4mcontours=RealArray[0m [4mncont=Integer[0m [4mfunction_name=String[0m
     [1m[0m[[4mcolour=Colour[0m] [[4mline_type=LineStyle[0m] [[4mline_thickness=Real[0m] Draw a
     [1m[0mcontour function onto a graphics device.
     [1mCONTOUR_FUNCTION/ALTER[0m [/LOG] [/SQRT] [[4mvalues=RealArray[0m]
     [1m[0m[[4mcontours=RealArray[0m] [[4mncont=Integer[0m] [[4mfunction_name=String[0m]
     [1m[0m[[4mcolour=Colour[0m] [[4mline_type=LineStyle[0m] [[4mline_thickness=Real[0m]
     [1m[0m[4mobject=Integer[0m Alter a previously drawn contour function on a
     [1m[0mgraphics device.
     [1m[0m
     example:
     
        # Plot some ungridded data with a
        # a user specified module function
        >> Contour_function/Draw values=my_points               function_name="my_gridding_function"

     Note: Use contours or ncont, not both.'.
!

GXcontourUlabel comment: 'Labels the contours in the contour plot
   
     [1mCONTOUR_LABEL/DRAW[0m [[4mtext=String[0m] [[4mstart=Integer[0m] [[4mend=Integer[0m]
     [1m[0m[[4mst=Integer[0m] [[4mintval=Integer[0m] [[4mminint=Integer[0m] [[4mfont=Font[0m]
     [1m[0m[[4msize=Real[0m] [[4mcolour=Colour[0m] [4mcontour=Gobject[0m Draw labels on a
     [1m[0mcontour plot.
     [1mCONTOUR_LABEL/ALTER[0m [[4mtext=String[0m] [[4mstart=Integer[0m] [[4mend=Integer[0m]
     [1m[0m[[4mst=Integer[0m] [[4mintval=Integer[0m] [[4mminint=Integer[0m] [[4mfont=Font[0m]
     [1m[0m[[4msize=Real[0m] [[4mcolour=Colour[0m] [4mobject=Gobject[0m Alter labels on a
     [1m[0mcontour plot.
     [1m[0m
     example:
     
        # put labels on a contour plot, use
        # label formatting to add units.
        >> my_cont = Contour:Draw(w.y)
        >> Contour_label contour=my_cont              text="%gmEV"

     Note: Default for text is just to give the values.'.
!

GXcopying comment: 'Prints the GNU licensing conditions for Open GENIE
   
     [1mCOPYING[0m Print the distribution license'.
!

GXcos comment: 'Generic trig function.
   
     [1mCOS([0mx) [4mx=Generic[0m Cosine in radians.
     [1m[0m
     example:
     
        # take cosine of angle in degrees
        y = Cos(90.0 * 180.0/$PI)
        #take the cosine of all elements in an array
        a = Dimensions(10)      # create a 10 element array
        fill a $pi*2.0          # Set all elements to pi*2
        y = Cos(a)

     Note: This function can be applied to all numeric types (including
     arrays & workspaces).'.
!

GXcursor comment: 'Displays a cursor on the graphics screen to allow interactive
   annotation.
   [1m[0m
     [1mCURSOR[0m [[4mxval=Real[0m] [[4myval=Real[0m] Displays a cursor and waits for key
     [1m[0mpress:
     [1m[0m
     X-display X coordinate"
     Y-display Y coordinate"
     P-display both coordinates"
     T-add text"
     L- lower left corner of box"
     U-draw box to upper right point"
     E-exit
     
     [/HORIZONTAL]   Annotation text horizontal
     [/VERTICAL]   Annotation text vertical
     
     example:
     
        # Display a cursor near the expected position
        # of a peak in world coordinates.
        >> Cursor/Horizontal 10000.0 50.0'.
!

GXcut comment: 'Takes a one-D cut out of a two-D array
   
     [1mCUT[0m [4marray=Realarray[0m [4mdimension=Integer[0m [4mindex=Real[0m Takes a slice out
     [1m[0mof an array.
     [1m[0m
     example:
     
        # Take a cuts at 1 and 1.5 along the 2nd dimension
        printn y
        >> y = Dimensions(2,3)
        >> fill y 1.0 1.0
        >> printn cut(y, 1, 1.0)
        [1.0 2.0 3.0 ] Array(3 )
        >> printn cut(y, 1, 1.5)
        [2.5 3.5 4.5 ] Array(3 )

     Note: Returns a one-D array of one element if a one-D array is
     supplied.'.
!

GXdata comment: 'Constructor for Data information
   
   [1m([0mdescription awaiting ratification of NeXus)'.
!

GXdebug comment: 'Enables simple procedure call debugging
   
     [1mDEBUG/ON[0m   Enable debugging of procedure calls
     [1mDEBUG/OFF[0m   Switch off debugging messages
     [1m[0m
     example:
     
        # Switch debugging on for next statement
        >> DEBUG/ON
        >> printn "HI"
        0:PRINTN
        hi'.
!

GXdefined comment: 'Tests whether a variable has a defined value (not undefined).
   
     [1mDEFINED[0m [4mvar=Any[0m Returns $TRUE if var was defined
     [1m[0m
     example:
     
        # Check for arithmetic undefined values
        >> x = 5.5 / 0.0
        >> printn Defined(x)
        $FALSE

     Note: Useful for testing the presence or absence of procedure
     parameters.'.
!

GXdelete comment: 'Deletes unwanted graphics objects from memory
   
     [1mDELETE[0m [4mitem=Gobject[0m Deletes an object, window or a picture.
     [1m[0m
     example:
     
        # Delete object 10 in window 2, picture 4.
        >> delete obj(4,2,10)
        # Create and delete a picture
        >> mypict = picture()
        >> delete item=mypict

     Note: Once something is deleted then there is no going back.'.
!

GXdev comment: 'Locates and returns device object given its index.
   
     [1mDEV()[0m [4mdevnum=Integer[0m Absolute index of the object.
     [1m[0m
     example:
     
        # Close the third device
        # and the current device
        >> device/close dev(3)
        >> device/close dev(devnum=0)'.
!

GXdevice comment: 'Selects and controls graphics devices
   
     [1mDEVICE/OPEN[0m [[4mdevtype=String[0m] [[4mwidth=Real[0m] [[4mheight=Real[0m] Opens a
     [1m[0mgraphics device for plotting on
     [1mDEVICE/CLEAR[0m   Clears the current graphics device
     [1mDEVICE/CLOSE[0m   Close the current graphics device
     [1mDEVICE:STATUS()[0m   Returns $TRUE if there is a device open
     [1m[0m
     example:
     
        # Open two X-Windows display devices
        # and close one.
        >> device/open "XW" height=5.0
        >> dev2 = device:open("XWINDOW",height=3.0)
        >> select dev2
        >> device/close

     Note: For windowing systems, multiple device may be open at once.'.
!

GXdimensionality comment: 'Return an integer giving the dimensionality of the array
   
     [1mDIMENSIONALITY[0m [[4marray=Array[0m] Returns the number of dimensions
     [1m[0m
     example:
     
        # create an array and print its dimensionality
        >> a = dimensions(2,3,4)
        >> printn dimensionality(a)
        3'.
!

GXdimensions comment: 'Create and size an Open GENIE array
   
     [1mDIMENSIONS()[0m [4mdim1=Integer[0m [[4mdim2=Integer[0m] ... [[4mdim10=Integer[0m] Create
     [1m[0man empty array
     [1m[0m
     example:
     
        # Dimension a 1 x 2 x 3 array
        >> a = dimensions(1, 2, 3)
        >> printn is_a(a, "Realarray")
        $FALSE
        >> a[1,2,1] = 4.0
        >> printn is_a(a, "Realarray")
        $TRUE
        >> printn a
        [_ _ _ 4.0 _ ...] Array(1 2 3 )

     Note: Arrays print as if they are one dimensional (in row major
     order).'.
!

GXdir comment: 'List the files in the current directory or the directory specified
   
     [1mDIR[0m [[4mpath=String[0m] List the directory contents
     [1m[0m
     example:
     
        # List the examples directory
        >> dir "[.examples]"'.
!

GXdisplay comment: 'Display a graphical plot of spectra in a workspace.
   
     [1mDISPLAY[0m [[4mwksp=Workspace[0m] [[4mxmin=Real[0m] [[4mxmax=Real[0m] [[4mymin=Real[0m]
     [1m[0m[[4mymax=Real[0m] [[4mlinecolour=Colour[0m] [[4mlinewidth=Real[0m]
     [1m[0m[[4mtextcolour=Colour[0m] [[4mtextheight=Real[0m] Display the workspace within
     [1m[0mthe given range.
     [1m[0m[/HISTOGRAM] Display as a histogram
     [1m[0m[/LINE] Display as a line plot
     [1m[0m[/MARKERS] Display as markers
     [1m[0m[/ERRORS] Display as error bars
     [1m[0m
     example:
     
        # Display a spectrum but change the title first
        >> wm = s(10)
        >> wm.title = "This is my Nobel plot!"
        >> Display wm xmin=1.0e4 xmax=7.0e4

     Note: Other display parameters may be altered using the Alter() and
     Toggle() commands.'.
!

GXdraw comment: 'Obsolete command
   
     [1mDRAW/AXES[0m Displays axes on the graphics device, use AXES
     [1mDRAW/ERRORS[0m Displays error-bars on a plot, use ERRORS
     [1mDRAW/GRAPH[0m Plots a Graph on the graphics device, use GRAPH
     [1mDRAW/GRATICULE[0m Displays a Graticule on the graphics device, use
     [1mGRATICULE[0m
     [1mDRAW/HISTOGRAM[0m Plots a histogram on the graphics device, use
     [1mHISTOGRAM[0m
     [1mDRAW/LABELS[0m Displays x/y labels on the graphics device, use LABELS
     [1mDRAW/LINE[0m Displays a line on the graphics device, use LINE
     [1mDRAW/MARKERS[0m Displays markers on the graphics device, use MARKERS
     [1mDRAW/POLYGON[0m Displays a polygon on the graphics device, use POLYGON
     [1mDRAW/TEXT[0m Displays some text on the graphics device, use TEXT
     [1mDRAW/TITLE[0m Displays a Title on the graphics device, use TITLE'.
!

GXerrors comment: 'Displays error-bars on a plot
   
     [1mERRORS/DRAW[0m [/HORIZONTAL] [/VERTICAL] [4mxarray=RealArray[0m
     [1m[0m[4myarray=RealArray[0m [4mearray=RealArray[0m [[4mcolour=Colour[0m]
     [1m[0m[[4mline_thickness=Real[0m] Draw error-bars at the x/yarray points, with
     [1m[0mthe size of the earray values
     [1mERRORS/ALTER[0m [/HORIZONTAL] [/VERTICAL] [[4mxarray=RealArray[0m]
     [1m[0m[[4myarray=RealArray[0m] [[4mearray=RealArray[0m] [[4mcolour=Colour[0m]
     [1m[0m[[4mline_thickness=Real[0m] [4mobject=Gobject[0m Alter the error-bars
     [1m[0m
     example:
     
        # Add error bars to a plot
        >> e = Errors:draw(colour=$blue)
        >> Errors/alter colour=$red object=e

     Note: By default the error-bars are drawn vertically.'.
!

GXexit comment: 'Exits from Open GENIE
   
     [1mEXIT[0m   Stops Open GENIE
     [1m[0m
     example:
     
        # Finish an open genie session
        >> Exit
        GENIE exiting
        $

     Note: Can also use quit to stop Open GENIE'.
!

GXexp comment: 'Generic transcendental functions.
   
     [1mEXP([0mx) [4mx=Generic[0m Exponentiate
     [1m[0m
     example:
     
        # antilog a data array
        # which is in logs to base 10
        data = Exp( w.y * ln(10.0) )

     Note: These functions can be applied to all numeric types
     (including arrays & workspaces).'.
!

GXexperiment comment: 'Constructor for Experiment information
   
   [1m([0mdescription awaiting ratification of NeXus)'.
!

GXfields comment: 'Create a workspace
   
     [1mFIELDS()[0m   Create a new empty workspace.
     [1m[0m
     example:
     
        # Example in preformatted font
        >> w = fields()
        >> w.new = 5
        >> printn w
          Workspace []
          (
            new = 5
          )'.
!

GXfiletype comment: 'Finds out the type of a data file (if recognized by Open GENIE).
   
     [1mFILETYPE[0m [[4mfile=String[0m] Return a string giving the data file type.
     [1m[0m
     example:
     
        # Check the type of a file
        printn Filetype("HRP00389.RAW")
        raw

     Note: "file" defaults to the type of the default input file if
     there is one.'.
!

GXfill comment: 'Fill an array with values
   
     [1mFILL[0m [4mvalue=Any[0m [[4mstep=Number[0m] Fill and array with values
     [1m[0m[/GEOMETRIC]   Fill array geometrically
     [1m[0m
     example:
     
        # Create some geometric data
        >> a = dimensions(200)
        >> Fill/Geometric a 1.0 0.9
        >> printn a
        [1.0 0.9 0.81 0.729 0.6561 ...] Array(200 )

     Note: step values only work with integer and real arrays.'.
!

GXfix comment: 'Fix any undefined values in an array
   
     [1mFIX[0m [4marray=Array[0m [4mvalue=Any[0m except Array Replaces any instances of
     [1m[0m"undefined" in an array.
     [1m[0m
     example:
     
        # Fix undefined results to 999
        >> fill a -4.0 1.0
        >> printn 10/a
        [-2.5 -3.33333333333333 -5.0 -10.0  _  ...] Array(6 )
        >> fix a 999.0
        >> printn a
        [-2.5 -3.33333333333333 -5.0 -10.0 999.0 ...] Array(6 )

     Note: See Unfix() for replacing sentinel values with "_" on data
     import.'.
!

GXfocus comment: 'Focus a range of spectra with given parameters
   
     [1mFOCUS()[0m [4mspectra=Range[0m or IntegerArray [[4mfile=String[0m]
     [1m[0m[[4mdetpars=Workspace[0m] [[4mspecpars=Workspace[0m] Focus a set of TOF spectra
     [1m[0mcorrecting for detector efficiency.
     [1m[0m[:D] Convert to D-spacing before summing
     [1m[0m[:Q] Convert to momentum transfer before summing
     [1m[0m[:T] Sum in time of flight
     [1m[0m[:VERBOSE] print out TOF parameters used in conversions
     [1m[0m
     example:
     
        # Focus a time of flight spectrum in D-spacing
        # get the TOF parameters from the raw data file
        >> w = focus( 1:17, "irs12838.raw" )
        # Focus default file, every third period
        >> assign 3045
        >> refl = focus( 3:903, _, mydet )

     Note: Focussing is done in D-spacing by default.'.
!

GXfree comment: 'Deletes a variable or workspace field and frees the storage.
   
     [1mFREE[0m [4mvariable=String[0m [[4msubvariable=String[0m] Delete a variable
     [1m[0m[/PROC]   Delete a procedure definition
     [1m[0m[/TYPE]   Delete a user defined workspace type
     [1m[0m
     example:
     
        # Delete a workspace field "test" and then an array "arr"
        >> printn w
          Workspace []
          (
            test = 45.0
          )
        >> free "w" "test"
          Workspace []
          (
          )
        >> free "arr"'.
!

GXgUerrorUhandler comment: 'Sorry, no help is available for this command'
!

GXgctorture comment: 'Sorry, no help is available for this command'
!

GXget comment: 'Read binary spectrum data into GENIE from any valid data source (e.g.
   ISIS Raw files, DAE, intermediate files etc.)
   [1m[0m
     [1mGET()[0m [4mitem=String[0m [[4mfile=String[0m] Read binary data by name
     [1mGET()[0m [4mitem=Integer[0m [[4mfile=String[0m] Read binary data by number
     [1mGET()[0m [4mitem=Interval[0m [[4mfile=String[0m] Read binary data by interval
     [1mGET()[0m [4mitem=IntegerArray[0m [[4mfile=String[0m] Read binary data by range
     [1m[0m
     example:
     
        # print the user name
        # then read in all the spectra in the file,
        # finally read every third spectrum from a file
        >> printn get("USER", "Isisfile.raw")
        Professor G. Offar
        >> 2d = get( 1 : get("NSP1") )
        >> spin_up = get( 1:503 , "CSP4589.RAW" )

     Note: Use the Dir() command to find out what data can be read from
     the file.'.
!

GXgetUcursor comment: 'Sorry, no help is available for this command'
!

GXgetcursor comment: 'Cursor input routine for a window or a picture.
   
     [1mGETCURSOR()[0m [[4mx=Real[0m] [[4my=Real[0m] Returns co-ordinates of the point the
     [1m[0mmouse was clicked
     [1m[0m
     example:
     
        # Shows how to use the getcursor command
        >> printn getcursor(0.5, 0.5)

     Note: If there is more than one window where the pointer clicked,
     it will return the last window drawn.'.
!

GXgraph comment: 'Displays a graph (or line plot) on a graphics device.
   
     [1mGRAPH/DRAW[0m [4mxarray=RealArray[0m [4myarray=RealArray[0m [[4mcolour=Colour[0m]
     [1m[0m[[4mline_type=LineStyle[0m] [[4mline_thickness=Real[0m] Draw a graph at the
     [1m[0mx/yarray points.
     [1mGRAPH/ALTER[0m [[4mxarray=RealArray[0m] [[4myarray=RealArray[0m] [[4mcolour=Colour[0m]
     [1m[0m[[4mline_type=LineStyle[0m] [[4mline_thickness=Real[0m] [4mobject=Gobject[0m Alter
     [1m[0mthe graph
     [1m[0m
     example:
     
        # Add error bars to a plot
        >> g = Graph:draw(colour=$blue)
        >> Graph/alter colour=$red object=g

     Note: For graph/alter yarray and xarray are not yet implemented.'.
!

GXgraticule comment: 'Displays a graticule axis on the graphics device
   
     [1mGRATICULE/DRAW[0m [/XLOG] [/YLOG] [/YHORIZ] [/NOXNUM] [/NOYNUM]
     [1m[0m[/XLINEAR] [/YLINEAR] [/YVERTICAL] [/XNUM] [/NUM] [[4mcolour=Colour[0m]
     [1m[0m[[4msize=Real[0m] [[4mfont=Font[0m] [[4mline_thickness=Real[0m] [[4mline_type=LineStyle[0m]
     [1mD[0mraws a Graticule on a graphics device
     [1mGRATICULE/ALTER[0m [/XLOG] [/YLOG] [/YHORIZ] [/NOXNUM] [/NOYNUM]
     [1m[0m[/XLINEAR] [/YLINEAR] [/YVERTICAL] [/XNUM] [/NUM] [[4mcolour=Colour[0m]
     [1m[0m[[4msize=Real[0m] [[4mfont=Font[0m] [[4mline_thickness=Real[0m] [[4mline_type=LineStyle[0m]
     [1m[0m[4mobject=Gobject[0m Alters a graticule already drawn on a graphics
     [1m[0mdevice
     [1m[0m
     example:
     
        # Draws a green Graticule on a device
        # then alters the colour.
        >> g = graticule:draw(colour=$green)
        >> graticule/alter colour=$red object=g

     Note: By default the graticule is linear.'.
!

GXgripe comment: 'Report a problem, solution or idea about Open GENIE
   
     [1mGRIPE[0m [[4msubject=String[0m] [[4mmessage=String[0m] [[4memail=String[0m] Send a
     [1m[0mmessage to the Open GENIE development team.
     [1m[0m
     example:
     
        # Type in a gripe interactively
        >> Gripe
          ... fill in details as requested ...

     Note: email defaults to genieisise.rl.ac.uk, the normal support
     mail address for Open GENIE.'.
!

GXgroupbins comment: 'Groups histogram bins by averaging the bin contents.
   
     [1mGROUPBINS[0m [4mwksp=Workspace[0m [4mnbin=Integer[0m [[4mundef=Real[0m] Group bins in a
     [1mGENIE[0m-V2 format workspace.
     [1mGROUPBINS()[0m [4mxarray=Realarray[0m [4myarray=Realarray[0m [4mearray=Realarray[0m
     [1m[0m[4mnbin=Integer[0m [[4mundef=Real[0m] Group bins in a spectrum given X, X, and
     [1mE[0m arrays.
     [1m[0m
     example:
     
        # Group the bins in a workspace by tens.
        >> work = s(1)
        >> Groupbins work 10 undef=0.0

     Note: Undef gives a value to assume when undefined elements are met
     during the averaging process.'.
!

GXhardcopy comment: 'Makes a copy of picture on a hardcopy device.
   
     [1mHARDCOPY[0m [[4mfilename=String[0m] [[4mdevtype=String[0m] [[4mpicture=Gobject[0m]
     [1mC[0mreates a hardcopy of a previously drawn picture
     [1m[0m
     example:
     
        # Save a postscript copy of the screen
        Hardcopy
        # Save a copy of the last but one picture as
        # colour postscript (found with Redraw/Info)
        Hardcopy filename="test.cps" picture=my_pict'.
!

GXhistogram comment: 'Displays a histogram on a graphics device.
   
     [1mHISTOGRAM/DRAW[0m [/CENTRE] [/NOTCENTRED] [4mxarray=RealArray[0m
     [1m[0m[4myarray=RealArray[0m [[4mcolour=Colour[0m] [[4mline_type=LineStyle[0m]
     [1m[0m[[4mline_thickness=Real[0m] Draw a Histogram at the x/yarray points.
     [1mHISTOGRAM/ALTER[0m [/CENTRE] [/NOTCENTRED] [[4mxarray=RealArray[0m]
     [1m[0m[[4myarray=RealArray[0m] [[4mcolour=Colour[0m] [[4mline_type=LineStyle[0m]
     [1m[0m[[4mline_thickness=Real[0m] [4mobject=Gobject[0m Alter the Histogram
     [1m[0m
     example:
     
        # Draws a histogram
        # then alters the colour.
        >> h = histogram:draw(colour=$green)
        >> histogram/alter colour=$red object=h

     Note: By default the points are not centred.'.
!

GXhistory comment: 'Constructor for History information
   
   [1m([0mdescription awaiting ratification of NeXus)'.
!

GXinquire comment: 'Prompts the user for terminal input and reads back an input variable.
   
     [1mINQUIRE[0m [[4maprompt=String[0m] Inquire information from the user
     [1m[0minteractively
     [1m[0m
     example:
     
        # Read in a lower limit from the user
        >> llimit = inquire("Enter lower limit")
        Enter lower limit: 45.678
        >> printn llimit
        45.6779999999999
        >> printn Is_a(llimit, "Real")
        $TRUE

     Note: To read a string without any conversions or prompting use
     Read_terminal()'.
!

GXinspect comment: 'Inspect in some detail, the type and contents of Open GENIE variables.
   
     [1mINSPECT[0m [4mp1=Any[0m Print the internal type or representation of a
     [1m[0mvariable
     [1m[0m
     example:
     
        # Example in preformatted font
        >> inspect 45
        An instance of Integer'.
!

GXinstrument comment: 'Constructor for Instrument information
   
   [1m([0mdescription awaiting ratification of NeXus)'.
!

GXintegrate comment: 'Integrate one or more spectra
   
     [1mINTEGRATE()[0m [4mwksp=Workspace[0m [[4mxmin=Real[0m] [[4mxmax=Real[0m] Integrate a
     [1m[0mworkspace (1d or 2d)
     [1mINTEGRATE()[0m [4mspectra=Interval[0m or IntegerArray [[4mxmin=Real[0m]
     [1m[0m[[4mxmax=Real[0m] [[4mfile=String[0m] Integrate spectra from a file
     [1m[0m
     example:
     
        # print the integrals of two spectra
        >> printn integrate(1:2, "tfx00345.raw")
        INTEGRATE: Integrating between 38000 and 78000
          Workspace []
          (
            error = [2341.4700 1027.1312 ] Array(2 )
            sum = [5482585.0 1054968.0 ] Array(2 )
          )

     Note: The sum is always in total counts'.
!

GXinteractive comment: 'Tests whether code is running as part of a procedure or not.
   
     [1mINTERACTIVE()[0m   Returns $TRUE if it is run within a procedure
     [1m[0m
     example:
     
        # Only print date if this is code is running
        # outside of a procedure (in a .gcl file for example)
        IF Interactive()
                printn "Running as batch file at " Now()
        ENDIF

     Note: Useful for code fragments which may be used in different ways'.
!

GXisUa comment: 'Tests the type of an Open GENIE variable.
   
     [1mIS_A()[0m [4mvar=Any[0m [4mtypestring=String[0m Checks to see what type an Open
     [1mGENIE[0m variable is
     [1m[0m
     example:
     
        # Convert a real if necessary
        IF Is_a(x, "real")
            x = As_integer(x)
        ENDIF

     Note: Important when using user defined workspace types.'.
!

GXjump comment: 'Similar to the System() command.
   
     [1mJUMP/S[0m [4macommand=String[0m Execute one operating system command
     [1mJUMP/P[0m   Create an operating system session within Open GENIE
     [1m[0m
     example:
     
        # Use Jump/s to print a file
        >> Jump/S "print/q=sys$lsr0 genie.ps"

     Note: For new programs use the System() command.'.
!

GXkeep comment: 'The keep command is no longer supported. Use Hardcopy() for saving
   graphics and the Getcursor() and Asciifile() commands for retrieving
   [1m[0mand storing peak location data.'.
!

GXlabels comment: 'Displays x/y labels on the graphics device
   
     [1mLABELS/DRAW[0m [/XHORIZONTAL] [/YVERTICAL] [/XVERTICAL] [/YHORIZONTAL]
     [1m[0m[[4mxlabel=String[0m] [[4mylabel=String[0m] [[4msize=Real[0m] [[4mcolour=Colour[0m]
     [1m[0m[[4mfont=Font[0m] Draws x and y labels.
     [1mLABELS/ALTER[0m [/XHORIZONTAL] [/YVERTICAL] [/XVERTICAL]
     [1m[0m[/YHORIZONTAL] [[4mxlabel=String[0m] [[4mylabel=String[0m] [[4msize=Real[0m]
     [1m[0m[[4mcolour=Colour[0m] [[4mfont=Font[0m] [4mobject=Gobject[0m Alters the x and y
     [1m[0mlabels
     [1m[0m
     example:
     
        # Draw some labels
        >> a = labels:draw("X Axis", "Y Axis")
        # Wanted capitals
        >> labels/alter "X AXIS" "Y AXIS" object=a

     Note: By default xlabel is labeled horizontally, and ylabel is
     labeled vertically.'.
!

GXlength comment: 'Generic command for returning the length of something
   
     [1mLENGTH[0m [4mitem=Generic[0m Returns the length of a variable
     [1m[0m
     example:
     
        # a String
        >> printn length("abcd")
        4
        # an array
        >> a = dimensions(2,3)
        >> printn length(a)
        6

     Note: for workspaces, Length() returns the number of defined
     elements.'.
!

GXlimits comment: 'Sets axis limits for the Display() and Plot() commands.
   
     [1mLIMITS[0m [/DEFAULT]   Sets X and Y axes to autoscale
     [1mLIMITS/X[0m [[4mxmin=Real[0m] [[4mxmax=Real[0m] Sets X axis limits
     [1mLIMITS/Y[0m [[4mymin=Real[0m] [[4mymax=Real[0m] Sets Y axis limits
     [1mLIMITS/NO_AUTO[0m [[4mxmin=Real[0m] [[4mxmax=Real[0m] [[4mymin=Real[0m] [[4mymax=Real[0m] Sets
     [1mX[0m and Y axis limits
     [1m[0m
     example:
     
        # Set the limits before doing a display
        >> Limits/X xmin=10.0E4 xmax=50.0E4
        >> Display s(1)

     Note: If no data appears on a Display(), typing "Limits" may help
     find it!'.
!

GXline comment: 'Displays a line on the graphics device
   
     [1mLINE/DRAW[0m [4mxstart=Real[0m [4mystart=Real[0m [4mxend=Real[0m [4myend=Real[0m
     [1m[0m[[4mcolour=Colour[0m] [[4mline_type=LineStyle[0m] [[4mline_thickness=Real[0m] Draws a
     [1m[0mline on a graphics device.
     [1mLINE/ALTER[0m [[4mxstart=Real[0m] [[4mystart=Real[0m] [[4mxend=Real[0m] [[4myend=Real[0m]
     [1m[0m[[4mcolour=Colour[0m] [[4mline_type=LineStyle[0m] [[4mline_thickness=Real[0m]
     [1m[0m[4mobject=Gobject[0m Alters a previously drawn line.
     [1m[0m
     example:
     
        #Draw a line
        >> aline = line:draw(0.0, 0.0, 1.0, 1.0, line_type=$dash)
        # Want the line type to be dot-dash
        >> line/alter line_type=$dot-dash object=aline'.
!

GXlist comment: 'Lists the items available within any datafile with a format Open GENIE
   understands.
   [1m[0m
     [1mLIST[0m [4mfile=String[0m Catalogues the contents of a data file
     [1mLIST/IN[0m   Lists the contents of the default input file
     [1mLIST/OUT[0m   Lists the contents of the default output file
     [1m[0m[/FULL]   List the file contents with full details
     [1m[0m
     example:
     
        >> List "dat.out"
        Genie intermediate file version 1.0
        Block    Label           Type
        1       vanadium        GXWorkspace
        2       Run 1   GXWorkspace
        3       Run 2   GXWorkspace

     Note: To see comments use the /Full qualifier.'.
!

GXln comment: 'Generic transcendental functions.
   
     [1mLN([0mx) [4mx=Generic[0m Log to the base e
     [1m[0m
     example:
     
        # Take logs to base e of a data array
        lndata = Ln(w.y)

     Note: These functions can be applied to all numeric types
     (including arrays & workspaces).'.
!

GXload comment: 'Loads a file containing GCL procedures into GENIE
   
     [1mLOAD[0m [4mfilename=String[0m Load a command file
     [1m[0m
     example:
     
        # Load a genie program
        >> Load "myprog.gcl"
        Loading file myprog.gcl ........
                56 lines scanned/compiled
        >>

     Note: Cannot always load one command file from within another.'.
!

GXlocate comment: 'Finds the position (or positions) of a given sub-string within a
   string.
   [1m[0m
     [1mLOCATE[0m [4mastring=String[0m [4msubstring=String[0m [[4mcount=Integer[0m] Returns the
     [1m[0mindex position of a sub-string within the string
     [1m[0m
     example:
     
        # Look for all the positions of the string "quick"
        >> position = 0
        >> words = "The quick brown quick fox            jumped quick over the lazy quick dog"
        LOOP i FROM 1 TO 10
            position = locate(words,"quick",i)
            EXITIF position = 0
            print position
        ENDLOOP
        5 17 33 53>>

     Note: Locate() is most useful when used in conjunction with the
     Substring() function.'.
!

GXlog comment: 'Generic transcendental functions.
   
     [1mLOG([0mx) [4mx=Generic[0m Log to the base 10
     [1m[0m
     example:
     
        # Take logs of a data array
        log10data = Log(w.y)

     Note: These functions can be applied to all numeric types
     (including arrays & workspaces).'.
!

GXmarkers comment: 'Displays markers on a plot
   
     [1mMARKERS/DRAW[0m [4mxarray=RealArray[0m [4myarray=RealArray[0m [[4mcolour=Colour[0m]
     [1m[0m[[4msymbol=Marker[0m] [[4msize=Real[0m] Draw markers at the x/yarray points.
     [1mMARKERS/ALTER[0m [[4mxarray=RealArray[0m] [[4myarray=RealArray[0m] [[4mcolour=Colour[0m]
     [1m[0m[[4msymbol=Marker[0m] [[4msize=Real[0m] [4mobject=Gobject[0m Alter the markers.
     [1m[0m
     example:
     
        # Draws a marker plot with crosses
        # then alters the marker symbol.
        >> mp = Markers:draw(symbol=$cross)
        >> Markers/alter object=mp symbol=$box

     Note: For markers/alter yarray, xarray are not implemented.'.
!

GXmax comment: 'Find the maximum value in an array
   
     [1mMAX()[0m [4marray=Array[0m Find the value of the highest element
     [1m[0m
     example:
     
        # Find the index of the highest data value
        >> print Bracket(w.y, Max(w.y))
        12456'.
!

GXmin comment: 'Find the minimum value in an array
   
     [1mMIN()[0m [4marray=Array[0m Find the value of the lowest element
     [1m[0m
     example:
     
        # Find the index of the lowest data value
        >> print Bracket(w.y, Min(w.y))
        245'.
!

GXmodule comment: 'Manage the dynamic loading and execution of user written C or FORTRAN
   code.
   [1m[0m
     [1mMODULE/COMPILE[0m
     [1m[0m[/C] [4mfile=String[0m [[4mcomp_flags=String[0m] [[4mlink_flags=String[0m]
     [1m[0m[[4msymbols=String[0m] Compile a user written module
     [1mMODULE/LOAD[0m [4mfile=String[0m [[4mdesc=String[0m] (Re-) load the routine into
     [1m[0mmemory
     [1mMODULE/EXECUTE[0m
     [1m[0m[/C] [4mfunc=String[0m [4mpars=Workspace[0m Run a loaded module
     [1mMODULE/LIST[0m   Show loaded modules
     [1m[0m
     example:
     
        # Compile a convolution module, load then run it on a 2-D workspace
        >> Module/Compile "conv.for" comp_flags="/check=all" symbols="conv_sub"
        >> Module/Load "conv.so"
        >> w1 = get(1:30)
        >> w2 = Module:Execute("conv_sub", w1)

     Note: A compiled module only needs loading once per Open GENIE
     session.'.
!

GXmultiUplot comment: 'Controls multiplot creation and plotting
   
     [1mMULTI_PLOT/CREATE[0m [/BINCENTRE] [/NOTCENTRED] [4mdata=RealArray[0m
     [1m[0m[[4mnumhist=Integer[0m] [[4mchmin=Integer[0m] [[4mchmax=Integer[0m] [[4mygap=Real[0m]
     [1m[0m[[4mcolour=Colour[0m] [[4mline_type=LineStyle[0m] [[4mline_thickness=Real[0m] Creates
     [1m[0mthe initial parameters of the multiplot.
     [1mMULTI_PLOT/SPECTRA[0m [4mdata=Range[0m or RealArray Adds the final parameter
     [1m([0mthe y array''s) needed to draw a multiplot.
     [1mMULTI_PLOT/DRAW[0m Draws a multiplot onto a graphics device.
     [1mMULTI_PLOT/ALTER[0m [/BINCENTRE] [/NOTCENTRED] [[4mnumhist=Integer[0m]
     [1m[0m[[4mchmin=Integer[0m] [[4mchmax=Integer[0m] [[4mygap=Real[0m] [[4mcolour=Colour[0m]
     [1m[0m[[4mline_type=LineStyle[0m] [[4mline_thickness=Real[0m] [4mobject=Gobject[0m Alter
     [1m[0mthe multiplot previously drawn on a graphics device.
     [1m[0m
     example:
     
        # Create a multiplot and display it
        >> w=get(1:10)
        >> multi_plot/create data=w.x
        >> multi_plot/spectra data=w.y
        >> win_multiplot 0.1 0.9 0.1 0.9
        >> mp = multi_plot:draw()

     Note: Multiplot window must be created after the multi_plot but
     before drawing.'.
!

GXmultiplot comment: 'Plots multiple spectra on one set of axes using a hidden line removal
   
     [1mMULTIPLOT[0m [4mspectra=Interval[0m or IntegerArray
     [1m[0m[[4mygap=Real[0m] [[4mfile=String[0m] [[4mlinecolour=Colour[0m] [[4mlinetype=LineStyle[0m]
     [1m[0m[[4mlinewidth=Real[0m] [[4mtextcolour=Colour[0m] [[4mtextheight=Real[0m] plot the
     [1m[0mspectra specified from a file.
     [1mMULTIPLOT[0m [4mwa=WorkspaceArray[0m
     [1m[0m[[4mygap=Real[0m] [[4mfile=String[0m] [[4mlinecolour=Colour[0m] [[4mlinetype=LineStyle[0m]
     [1m[0m[[4mlinewidth=Real[0m] [[4mtextcolour=Colour[0m] [[4mtextheight=Real[0m] plot the
     [1m[0mspectra in the workspace array.
     [1m[0m
     example:
     
        # Do multiplots from a file
        >> multiplot 1:20 file="mydata.raw" ygap=10.0
        >> multiplot 1:3003    # plot every third spectrum
        # Make and plot a workspace array
        >> LOOP i FROM 1 TO 20; ww=s(i); ENDLOOP
        >> multiplot wa=ww linecolour=$green

     Note: multiplot from a file is more efficient in terms of memory'.
!

GXnblocks comment: 'Returns the number of readable blocks in an Open GENIE recognized
   file.
   [1m[0m
     [1mNBLOCKS()[0m [[4mfile=String[0m] Return number of blocks in an intermediate
     [1m[0mfile, or number of spectra in a raw file
     [1m[0m
     example:
     
        # Check the number of spectra in a raw
        # file then do a multiplot
        >> Set/File/Input "hrp00345.raw"
        >> n = nblocks()
        >> multiplot 1:(n)

     Note: For an ISIS raw file the above use of Nblocks() is equivalent
     to Get("NSP1").'.
!

GXnewUzoom comment: 'Zooms in on a graphics device
   
     [1mNEW_ZOOM[0m   Magnifies specified areas of the current picture
     [1m[0m
     example:
     
        # Using the zoom command
        new_zoom

     Note: This is only partially implemented.'.
!

GXnow comment: 'Returns the current date and time as a string.
   
     [1mNOW()[0m   Returns date and time
     [1m[0m
     example:
     
        # Printn the date and time
        >> printn Now()
        11-Jul-97 8:47:50

     Note: Use Substring() and Locate() to select time or date
     individually'.
!

GXobj comment: 'Locates and returns a graphics primitive object given its location.
   
     [1mOBJ()[0m [4mpicnum=Integer[0m [4mwinnum=Integer[0m [4mobjnum=Integer[0m Absolute index
     [1m[0mof the object.
     [1m[0m
     example:
     
        # Undraw the third object in the
        # current picture and window
        >> undraw obj(0,0,3)

     Note: Picture and window indices of 0 for current values.'.
!

GXos comment: 'Returns a string giving the operating system Open GENIE is running on
   
     [1mOS()[0m Returns the name of the operating system
     [1m[0m
     example:
     
        # Print the current operating system
        >> printn Os()
        OSF'.
!

GXpeak comment: 'Performs interactive peak fitting
   
     [1mPEAK[0m [[4mw=Workspace[0m] Fit peaks in a workspace
     [1m[0m
     example:
     
        # Fit peaks from a genie intermediate file
        >> fit = peak( get(3,"mydata.in3") )
        >> printn fit

     Note: See the Peakfit() and Peakgen() commands for more detail.'.
!

GXpeakfit comment: 'Fits various peaks to data supplied in arrays.
   
     [1mPEAKFIT()[0m [4mx=Realarray[0m [4my=Realarray[0m [4me=Realarray[0m
     [1m[0m[[4mpars=Realarray[0m] [[4mipropt=IntegerArray[0m] Fits a selection of peaks to
     [1m[0msupplied data.
     [1m[0m[:GAUSS] Gaussian
     [1m[0m[:GEXP] Gaussian convolved with exponential
     [1m[0m[:LOREN] Lorentzian
     [1m[0m[:LEXP] Lorentzian convolved with exponential
     [1m[0m[:VOIGT] Voigt
     [1m[0m[:VEXP] Voigt convolved with exponential
     [1m[0m[:POLY] Polynomial of a specified degree
     [1m[0m
     example:
     
        # Read some data and fit a peak to it
        # remembering to convert from histogram
        # to point data.
        >> w=s(1)
        >> res = Peakfit:Lexp(centre_bins(w.x), w.y, w.e)

     Note: Centre_bins() reduces X-array length by one and picks bin
     centres.'.
!

GXpeakgen comment: 'Generates peak data from an X-array and peak parameters.
   
     [1mPEAKGEN()[0m [4mx=Realarray[0m [4mpars=Realarray[0m Fits a selection of peaks to
     [1m[0msupplied data.
     [1m[0m[:GAUSS] Gaussian
     [1m[0m[:GEXP] Gaussian convolved with exponential
     [1m[0m[:LOREN] Lorentzian
     [1m[0m[:LEXP] Lorentzian convolved with exponential
     [1m[0m[:VOIGT] Voigt
     [1m[0m[:VEXP] Voigt convolved with exponential
     [1m[0m[:POLY] Polynomial
     [1m[0m
     example:
     
        # Produce data to show goodness of a previous fit
        >> y_fit = Peakgen:Lexp(centre_bins(w.x), res.pars)
        >> y_goodness = w.y - y_fit  # difference for plotting

     Note: Centre_bins() reduces X-array length by one and picks bin
     centres.'.
!

GXpic comment: 'Locates and returns a picture object given its index.
   
     [1mPIC()[0m [4mpicnum=Integer[0m Absolute index of the object.
     [1m[0m
     example:
     
        # Redraw the third picture
        >> redraw pic(3) device=dev(2)'.
!

GXpicUadd comment: 'Copy graphical objects from picture to picture.
   
     [1mPIC_ADD[0m [4mitem=Gobject[0m [[4mdevice=Gobject[0m] Copy item to current
     [1mPIC_ADD[0m [4mitem=Gobject[0m [[4mdest=Gobject[0m] [[4mdevice=Gobject[0m] Copy item to
     [1m[0mpicture or window
     [1m[0m
     example:
     
        # Draw a line object, then copy it into
        # the last open window on picture 4
        # redraw the updated picture on device 3.
        >> my_line = Line:Draw(0.0,1.0,0.0,1.0,$green)
        >> Pic_add item=my_line dest=pic(4) device=dev(3)

     Note: item can only ever be a primitive object or a window.'.
!

GXpicture comment: 'Creates a picture
   
     [1mPICTURE[0m Creates and displays a picture on the graphics device
     [1m[0m
     example:
     
        # Opening a device then
        # creating a picture
        >> device/open
        >> picture

     Note: To create a pictures without opening a real device, open a
     "/NULL" device.'.
!

GXplot comment: 'Overplots an existing graph produced by Display()
   
     [1mPLOT[0m [[4mwksp=Workspace[0m] [[4mbinning=Integer[0m] Overplots with new data
     [1m[0m[/HISTOGRAM]   Plots as a histogram (default)
     [1m[0m[/LINE]   Plots as a line
     [1m[0m[/MARKERS]   Plots as markers
     [1m[0m[/ERRORS]   Plots as error bars
     [1m[0m
     example:
     
        # Plot a vanadium spectrum on top of
        # the data as a comparison
        >> Display s(5) xmin=20000.0
        >> Plot/Line get("vanadium1","normal.in3")

     Note: Plot does not allow any change of scaling.'.
!

GXpolygon comment: 'Displays a polygon on the graphics device
   
     [1mPOLYGON/DRAW[0m [/FILLED] [/OUTLINE] [4mxarray=RealArray[0m [4myarray=RealArray[0m
     [1m[0m[[4mcolour=Colour[0m] [[4mline_thickness=Real[0m] Draws a polygon onto a
     [1m[0mgraphics device.
     [1mPOLYGON/ALTER[0m [/FILLED] [/OUTLINE] [[4mxarray=RealArray[0m]
     [1m[0m[[4myarray=RealArray[0m] [[4mcolour=Colour[0m] [[4mline_thickness=Real[0m]
     [1m[0m[4mobject=Gobject[0m Alters a polygon on a graphics device
     [1m[0m
     example:
     
        # Draws a polygon plot with crosses
        # then alters the marker symbol.
        >> Polygon/draw x_points y_points
        >> Markers/alter line_thickness=10.0 object=obj(0,0,0)

     Note: By default the polygon is outlined.'.
!

GXprint comment: 'Print to the users terminal (no newline).
   
     [1mPRINT[0m [p1-[4mp20=Any[0m] Converts all parameters for printing to the
     [1m[0mterminal screen
     [1m[0m
     example:
     
        # print out a variable with some text
        >> print "Value of variable $PI=" $PI
        Value of variable $PI=3.14159265358979>>

     Note: See also the other print commands in the group ( "print" -
     "n, i, in, e, en, d, dn" e.g. "printdn" )'.
!

GXprintd comment: 'Print an debugging message to the users terminal (no newline).
   
     [1mPRINTD[0m [p1-[4mp20=Any[0m] Converts all parameters for printing to the
     [1m[0mterminal screen
     [1m[0m
     example:
     
        # print out a variable with some text
        >> printd "Value of variable $PI=" $PI
        Value of variable $PI=3.14159265358979>>

     Note: See also the other print commands in the group ( "print" -
     "n, i, in, e, en, d, dn" e.g. "printdn" )'.
!

GXprintdn comment: 'Print a debugging message to the users terminal with a terminating
   newline.
   [1m[0m
     [1mPRINTDN[0m [p1-[4mp20=Any[0m] Converts all parameters for printing to the
     [1m[0mterminal screen
     [1m[0m
     example:
     
        # print out a variable with some text
        >> printdn "Value of variable $PI=" $PI
        Value of variable $PI=3.14159265358979
        >>

     Note: See also the other print commands in the group ( "print" -
     "n, i, in, e, en, d, dn" e.g. "printdn" )'.
!

GXprinte comment: 'Print an error message to the users terminal (no newline).
   
     [1mPRINTE[0m [p1-[4mp20=Any[0m] Converts all parameters for printing to the
     [1m[0mterminal screen
     [1m[0m
     example:
     
        # print out a variable with some text
        >> printe "Value of variable $PI=" $PI
        0/(none)/Value of variable $PI=3.14159265358979>>

     Note: See also the other print commands in the group ( "print" -
     "n, i, in, e, en, d, dn" e.g. "printdn" )'.
!

GXprinten comment: 'Print an error message to the users terminal with a terminating
   newline.
   [1m[0m
     [1mPRINTEN[0m [p1-[4mp20=Any[0m] Converts all parameters for printing to the
     [1m[0mterminal screen
     [1m[0m
     example:
     
        # print out a variable with some text
        >> printen "Value of variable $PI=" $PI
        0/(none)/Value of variable $PI=3.14159265358979
        >>

     Note: See also the other print commands in the group ( "print" -
     "n, i, in, e, en, d, dn" e.g. "printdn" )'.
!

GXprinti comment: 'Print an informational message to the users terminal (no newline).
   
     [1mPRINTI[0m [p1-[4mp20=Any[0m] Converts all parameters for printing to the
     [1m[0mterminal screen
     [1m[0m
     example:
     
        # print out a variable with some text
        >> printi "Value of variable $PI=" $PI
        Value of variable $PI=3.14159265358979>>

     Note: See also the other print commands in the group ( "print" -
     "n, i, in, e, en, d, dn" e.g. "printdn" )'.
!

GXprintin comment: 'Print an informational message to the users terminal with a
   terminating newline.
   [1m[0m
     [1mPRINTIN[0m [p1-[4mp20=Any[0m] Converts all parameters for printing to the
     [1m[0mterminal screen
     [1m[0m
     example:
     
        # print out a variable with some text
        >> printin "Value of variable $PI=" $PI
        Value of variable $PI=3.14159265358979
        >>

     Note: See also the other print commands in the group ( "print" -
     "n, i, in, e, en, d, dn" e.g. "printdn" )'.
!

GXprintn comment: 'Print to the users terminal with a terminating newline.
   
     [1mPRINTN[0m [p1-[4mp20=Any[0m] Converts all parameters for printing to the
     [1m[0mterminal screen
     [1m[0m
     example:
     
        # print out a variable with some text
        >> printn "Value of variable $PI=" $PI
        Value of variable $PI=3.14159265358979
        >>

     Note: See also the other print commands in the group ( "print" -
     "n, i, in, e, en, d, dn" e.g. "printdn" )'.
!

GXput comment: 'Outputs Open GENIE data to a known file type.
   
     [1mPUT[0m [4mgv=Any[0m [[4mlabel=String[0m] [[4mcomment=String[0m] [[4mfile=String[0m] Appends
     [1m[0mthe specified variable to the end of the data file, by default in
     [1mO[0mpen GENIE intermediate file format..
     [1m[0m[/NEW] Create a new data file or overwrite existing file.
     [1m[0m[/ASCII] Write the data file in formatted ASCII.
     [1m[0m[/HDF] Write in HDF format.
     [1m[0m
     example:
     
        # Copy a whole multidimensional spectrum from a raw
        # file into a single item in an intermediate file
        >> Set/file/Input "hrp00273.raw"
        >> Set/file/Output "example.in3"
        >> put get(1:20) label="bigspec" comment="demo"'.
!

GXpwd comment: 'Print the current working directory
   
     [1mPWD[0m   Show the working directory
     [1m[0m
     example:
     
        # Change to the examples directory
        >> cd "/usr/local/genie/examples"
        >> pwd
        /usr/local/genie/examples'.
!

GXquit comment: 'Sorry, no help is available for this command'
!

GXread comment: 'Sorry, no help is available for this command'
!

GXreadUterminal comment: 'Read a string back from the input terminal into a variable
   
     [1mREAD_TERMINAL()[0m   Return a string typed in at the terminal
     [1m[0m
     example:
     
        # Read a string
        >> printn "(" read_terminal() ")"
        abcd efg
        (abcd efg)
        >>

     Note: To read numbers as well as strings you may wish to use the
     Inquire() command.'.
!

GXrebin comment: 'Rebin a workspace given the specified binning scheme.
   
     [1mREBIN[0m
     [1m[0m[/LIN]
     [1m[0m[/LOG] [4mwksp=Workspace[0m [4mbound1=Real[0m [4mstep1=Real[0m [4mbound2=Real[0m
     [1m[0m[[4mstep2=Real[0m] [[4mbound3=Real[0m] [[4mstep3=Real[0m] [[4mbound4=Real[0m] Rebin a
     [1m[0mworkspace in linear or logarithmic steps
     [1mREBIN[0m [4mwksp=Workspace[0m [4mxmin=Real[0m [4mxmax=Real[0m Rebin over a range
     [1mREBIN[0m [4mwksp=Workspace[0m [4mxarray=Realarray[0m Rebin to the given X values.
     [1m[0m
     example:
     
        # Read a spectrum and re-bin the same as an
        # already loaded vanadium workspace
        >> w = s(1)
        >> w = rebin( w, vanadium.x )

     Note: You can combine rebins by concatenating the results with the
     & operator.'.
!

GXredim comment: 'Re-dimensions an array whilst keeping the size and contents identical
   
     [1mREDIM[0m [[4marray=Array[0m] [4mi=Integer[0m [[4mj=Integer[0m] [[4mk=Integer[0m][[4ml=Integer[0m]
     [1mR[0me-dimensions an array whilst keeping the contents identical
     [1m[0m
     example:
     
        >> printn a
        [1.0 2.0 3.0 4.0 5.0 ...] Array[400 ] Array(1 )
        >> printn redim(a,20,10,2)
        [1.0 2.0 3.0 4.0 5.0 ...] Array[20 10 2 ] Array(3 )
        >> printn a[1,2,1]
        3.0

     Note: Redim will return an error if an attempt is made to change
     the total size of the array'.
!

GXredraw comment: 'Redisplay current or previous pictures or objects.
   
     [1mREDRAW[0m [[4mitem=Gobject[0m] [[4mdevice=Gobject[0m] Redraws all or part of the
     [1m[0mspecified picture and optionally resets it to be the current
     [1m[0mpicture.
     [1m[0m
     example:
     
        # Store an HPGL copy of the last but one picture
        >> device/close
        >> device/open "HPGL" file="OUT.HPGL"
        >> redraw pic(-2)

     Note: The default is to redraw the current picture on the current
     device'.
!

GXs comment: 'Get data, now identical to the Get() command. See Get/Help'.
!

GXsample comment: 'Constructor for Sample information
   
   [1m([0mdescription awaiting ratification of NeXus)'.
!

GXsave comment: 'Saves a complete Open GENIE image file with all variables and
   procedures.
   [1m[0m
     [1mSAVE[0m [4mfilename=String[0m Save a frozen image of the current Open GENIE
     [1m[0m
     example:
     
        # Save the current session
        >> Save "mygenie.im"
        Saving GENIE in file mygenie.im ...
        >>

     Note: Saving an image after loading a large number of procedures
     can give a quicker startup.'.
!

GXscatmode comment: 'Return GENIE-V2 Energy mode description
   
     [1mSCATMODE[0m [[4memode=Integer[0m] Returns a string giving the meaning of the
     [1m[0memode value.
     [1m[0m
     example:
     
        # Get a description from an emode value of 1
        >> printn Scatmode(1)
        inelastic (direct geometry)'.
!

GXsearch comment: 'Sorry, no help is available for this command'
!

GXselect comment: 'Select a new current device, window or picture
   
     [1mSELECT[0m [4mitem=Gobject[0m Sets this to be the current object
     [1m[0m
     example:
     
        # Select an old window to draw in
        >> Select win(picnum=3,winnum=4)
        >> keep_dev = Select(dev(3))
        >> draw/text 0.2 0.3 "Fudge this data"
        # back to previous device & current window
        >> select keep_dev
        >> select win(0,0)

     Note: Drawing is always to the current window.'.
!

GXset comment: 'Set user defined defaults.
   
     [1mSET/DISK[0m [4mvalue=String[0m Select the default disk
     [1mSET/DIR[0m [4mvalue=String[0m Select the default directory
     [1mSET/INST[0m [4mvalue=String[0m Select the default instrument short name
     [1mSET/EXT[0m [4mvalue=String[0m Select the default file extension
     [1mSET/FILE[0m
     [1m[0m[/INPUT]
     [1m[0m[/OUTPUT] [4mvalue=String[0m Set the default input or output files
     [1m[0m
     example:
     
        # Change the file for saving data to
        >> Set/file/output "Mydat.in3"

     Note: Set/File/Input is the default for Set/File'.
!

GXsetpar comment: 'GENIE-V2 style T-O-F parameter setting for the Units() command.
   
     [1mSETPAR[0m [4mwksp=Workspace[0m [[4ml1=Real[0m] [[4ml2=Real[0m] [[4mtt=Real[0m] [[4mem=Integer[0m]
     [1m[0m[[4mef=Real[0m] [[4md=Real[0m] [[4mt=String[0m] Set time of flight parameters
     [1m[0m[/R]   Take angles to be values in radians
     [1m[0m
     example:
     
        # Set a value of two-theta that was not in the raw data file
        >> w = s(1)
        >> Setpar w tt=50.0
        >> wd = Units:d(w)

     Note: Could probably be done more easily by assignment (i.e.
     w.twotheta = 50.0)'.
!

GXshow comment: 'Set user defined defaults and system variables.
   
     [1mSHOW/DEFAULTS[0m   Show all I/O defaults
     [1mSHOW/PAR[0m [4mwksp=Workspace[0m Show parameters set by Setpar()
     [1mSHOW/DATA[0m [4mwksp=Workspace[0m Emulate GENIE-V2 show data command
     [1mSHOW/CONST[0m   Show all the constants in Open GENIE
     [1mSHOW/PROC[0m   Show all the procedures in Open GENIE
     [1mSHOW/TYPE[0m   Show all the types in Open GENIE
     [1mSHOW/VAR[0m   Show all the variables in Open GENIE
     [1m[0m[/SYS]   Show system variables as well
     [1m[0m
     example:
     
        # Check the I/O defaults
        >> Show/defaults
        Current default disk = EVS$DISK0:
        Current default directory = [EVSDATA]
        Current default instrument = EVS
        Current default extension = .RAW
        Current default input = EVS$DISK0:[EVSDATA]EVS00456.RAW
        Current default output =

     Note: The current default input file can be altered by Assign() or
     Set/File/Input'.
!

GXsin comment: 'Generic trig function.
   
     [1mSIN([0mx) [4mx=Generic[0m Sin in radians.
     [1m[0m
     example:
     
        # take sin of angle in degrees
        y = sin(40.0 * 180.0/$PI)
        #take the sin of all elements in an array
        a = Dimensions(10)      # create a 10 element array
        fill a 0.5              # Set all elements to 0.5
        y = Sin(a)

     Note: This function can be applied to all numeric types (including
     arrays & workspaces).'.
!

GXsqrt comment: 'Calculate the square root of a value.
   
     [1mSQRT([0mx) [4mx=Generic[0m Calculate the square root.
     [1m[0m
     example:
     
        # Calculate the square root of Pi
        printn Sqrt($PI)
        1.77245310234149

     Note: This function can be applied to all numeric types (including
     arrays & workspaces).'.
!

GXsubstring comment: 'Extracts a smaller string out of a larger string using the given
   indices.
   [1m[0m
     [1mSUBSTRING()[0m [4mastring=String[0m [[4mstart=Integer[0m] [[4mlength=Integer[0m] Select
     [1m[0ma string of a specified length from a given starting position.
     [1m[0m
     example:
     
        # pick the user name from this string
        >> printn substring("Run: no, User: A Neutron", 16)
        A Neutron
        
        # Select just the initial
        >> printn substring("Run: no, User: A Neutron", 16, 1)
        A

     Note: if the length value is null, the rest of the string is
     returned.'.
!

GXsum comment: 'Sums all the elements in an array
   
     [1mSUM()[0m [4marray=Array[0m Sum all the elements
     [1m[0m
     example:
     
        # Integrate some data
        >> printn sum(w.y)
        12456890.789'.
!

GXsumspec comment: 'Sum spectra in a given range from the specified file. (See also
   Focus/T)
   [1m[0m
     [1mSUMSPEC[0m [[4mspectra=Range[0m or IntegerArray] [[4mfile=String[0m] Sum spectra
     [1m[0min Time-of-Flight
     [1m[0m
     Note: Obsolete command, use Focus:T()'.
!

GXsystem comment: 'Executes a single command or command session from within Open GENIE.
   
     [1mSYSTEM[0m [[4mcommand=String[0m] Execute system commands from within Open
     [1mGENIE[0m
     [1m[0m
     example:
     
        # find some user details on VMS
        >> system "$ SEARCH journal.txt "flux" "
        IRS14142ZAB/NJR Flux tests  18-FEB-1997 12:00:01 163.2
        IRS14143ZAB/NJR Flux tests  18-FEB-1997 13:43:40   6.1
        >>

     Note: The Dir() command in Open GENIE is simply a procedure written
     using the System() command.'.
!

GXtan comment: 'Generic trig function.
   
     [1mTAN([0mx) [4mx=Generic[0m Tangent in radians.
     [1m[0m
     example:
     
        # take tangent of angle in degrees
        y = Tan(90.0 * 180.0/$PI)
        #take the tangent of all elements in an array
        a = Dimensions(10)      # create a 10 element array
        fill a 0.5              # Set all elements to 0.5
        y = Tan(a)

     Note: This function can be applied to all numeric types (including
     arrays & workspaces).'.
!

GXtext comment: 'Displays some text on the graphics device
   
     [1mTEXT/DRAW[0m [4mxcoord=Real[0m [4mycoord=Real[0m [4mtext=String[0m [[4mcolour=Colour[0m]
     [1m[0m[[4msize=Real[0m] [[4mangle=Real[0m] [[4mfont=Font[0m] Draws some text onto a
     [1m[0mgraphics device
     [1m[0m[/CENTRE] Centres the text on the given coordinate
     [1mTEXT/ALTER[0m [[4mxcoord=Real[0m] [[4mycoord=Real[0m] [[4mtext=String[0m]
     [1m[0m[[4mcolour=Colour[0m] [[4msize=Real[0m] [[4mangle=Real[0m] [[4mfont=Font[0m] [4mobject=Gobject[0m
     [1mA[0mlters some text, on a graphics device.
     [1m[0m
     example:
     
        >> text/draw 0.0 0.0 "some text" size=3.0
        # The text is a bit to big
        >> text/alter size=1.0 colour=$yellow object=obj(0,0,0)'.
!

GXtitle comment: 'Displays a Title on the graphics device
   
     [1mTITLE/DRAW[0m [4mtext=String[0m [[4mcolour=Colour[0m] [[4msize=Real[0m] [[4mfont=Font[0m] Draw
     [1m[0ma title onto a graphics device
     [1mTITLE/ALTER[0m [[4mtext=String[0m] [[4mcolour=Colour[0m] [[4msize=Real[0m] [[4mfont=Font[0m]
     [1m[0m[4mobject=Gobject[0m Alters a title on a graphics device
     [1m[0m
     example:
     
        >> title/draw "A Title"
        # Want the font to be italic
        >> title/alter font=$italic'.
!

GXtoggle comment: 'Toggles between common settings, mainly graphics.
   
     [1mTOGGLE/STATUS[0m Reports the values of all Toggled settings.
     [1mTOGGLE/LOGX[0m X axis log/linear
     [1mTOGGLE/LOGY[0m Y axis log/linear
     [1mTOGGLE/CLEAR[0m Screen clearing between plots
     [1mTOGGLE/GRATICULE[0m Toggle graticule off/on
     [1mTOGGLE/HEADER[0m Header plotted on Display() command.
     [1mTOGGLE/RX[0m X axis rounding off/on
     [1mTOGGLE/RY[0m Y axis rounding off/on
     [1mTOGGLE/INFO[0m Informational message printing on/off
     [1m[0m[/ON] Toggles item on
     [1m[0m[/OFF] Toggles item off
     [1m[0m
     example:
     
        # Switch off informational messages and display
        # a data plot with no banner heading.
        >> Toggle/info
        >> Toggle/header
        >> Display Spectrum(22)

     Note: See the Alter() command for changing other graphics settings.'.
!

GXundraw comment: 'Undraw a primitive graphics operation.
   
     [1mUNDRAW[0m [[4mitem=Gobject[0m] Temporarily remove a graphics primitive
     [1m[0m
     example:
     
        >> Draw/Text 0.5 0.4 "Special data point"
        # oops, too far to the right!
        >> Undraw
        >> Draw/Text 0.4 0.4 "Special data point"

     Note: See Obj(), Win() and Pic() commands for specifying objects.'.
!

GXunfix comment: 'Replace sentinel values with "undefined"
   
     [1mUNFIX[0m [4marray=Array[0m [4mvalue=Any[0m except Array Replace a specific value
     [1m[0mwith undefined
     [1m[0m
     example:
     
        # Fix undefined results to 999
        >> scan = get_dat("Old_data_file")
        >> Unfix scan 999.0    # 999.0 represents unknown values

     Note: See Fix() for replacing undefined values with a sentinel
     value.'.
!

GXunit comment: 'Constructor for Unit information
   
   [1m([0mdescription awaiting ratification of NeXus)'.
!

GXunits comment: 'Convert the units of a Time of Flight workspace as specified
   
     [1mUNITS()[0m [4mwksp=Workspace[0m [[4mxmin=Real[0m] [[4mxmax=Real[0m] Convert workspace
     [1m[0munits
     [1m[0m[/C] or [/Channel]   To Channel numbers (one way)
     [1m[0m[/D]   To D-Spacing (A)
     [1m[0m[/E]   To Energy (meV)
     [1m[0m[/LAM]   To Wavelength (A)
     [1m[0m[/Q]   To Momentum Transfer (A-1)
     [1m[0m[/SQ]   To (Momentum Transfer)2 (A-2)
     [1m[0m[/T]   To Time of Flight (us)
     [1m[0m[/LA1]   To primary flight path wavelength (A)
     [1m[0m[/W]   To energy transfer E1-E2 (meV)
     [1m[0m[/WN]   To energy transfer E1-E2 (cm-1)
     [1m[0m[/TAU]   To reciprocal time-of-flight (us/m)
     [1m[0m
     example:
     
        # Read in a spectrum whilst converting to Wavelength
        >> w = units:Lam( s(1) )

     Note: TOF Parameters must be set correctly in the workspace (or set
     Set/Par).'.
!

GXuser comment: 'Constructor for User information
   
   [1m([0mdescription awaiting ratification of NeXus)'.
!

GXversion comment: 'Prints the Open GENIE Version.
   
     [1mVERSION[0m   Print out the version string
     [1m[0m
     example:
     
        # print out the genie version
        >> Version
        (#)Open GENIE V1.1 BUILD-30
        [Linked] Thu Jul 10 13:47:02 BST 1997 [library version] 1.1

     Note: Please quote this when reporting problems with the Gripe()
     command.'.
!

GXwarranty comment: 'Prints the warranty disclaimer for Open GENIE
   
     [1mWARRANTY[0m   Print the warranty disclaimer'.
!

GXwin comment: 'Locates and returns a graphics window object given its location.
   
     [1mWIN()[0m [4mpicnum=Integer[0m [4mwinnum=Integer[0m Absolute index of the object.
     [1m[0m
     example:
     
        # Copy the 2nd window in the
        # last but one picture into a
        # new picture
        >> pic_add item=win(-2,2) picture=picture()

     Note: A picture value of 0 specifies the current picture.'.
!

GXwinUautoscaled comment: 'Creates a scaled window automatically from supplied data.
   
     [1mWIN_AUTOSCALED[0m [4mdxmin=Real[0m [4mdxmax=Real[0m [4mdymin=Real[0m [4mdymax=Real[0m
     [1m[0m[4mx=RealArray[0m [4my=RealArray[0m [4me=RealArray[0m [[4mminmode=Integer[0m]
     [1m[0m[[4mmaxmode=Integer[0m] [[4mcolour=Colour[0m] Creates an autoscaled window.
     [1m[0m
     example:
     
        # Creating and displaying an autoscaled
        # window, where yarray, xarray and earray
        # have already been defined
        >> win_autoscaled 0.1 0.9 0.1 0.9 xarray yarray earray

     Note: d means the device co-ordinate.'.
!

GXwinUmultiplot comment: 'Controls and creates multi_plot windows for plotting
   
     [1mWIN_MULTIPLOT[0m [4mdxmin=Real[0m [4mdxmax=Real[0m [4mdymin=Real[0m [4mdymax=Real[0m
     [1m[0m[4mobject=Gobject[0m [[4mminmode=Integer[0m] [[4mmaxmode=Integer[0m] [[4mcolour=Colour[0m]
     [1mC[0mreates a multiplot window.
     [1m[0m
     example:
     
        # Creating and displaying a multiplot
        # window
        >> win_multiplot 0.1 0.9 0.1 0.9

     Note: d means the device co-ordinate.'.
!

GXwinUscaled comment: 'Controls and creates pre-scaled windows for plotting data.
   
     [1mWIN_SCALED[0m [4mdxmin=Real[0m [4mdxmax=Real[0m [4mdymin=Real[0m [4mdymax=Real[0m [4mwxmin=Real[0m
     [1m[0m[4mwxmax=Real[0m [4mwymin=Real[0m [4mwymax=Real[0m [[4mcolour=Colour[0m] [[4mminmode=Integer[0m]
     [1m[0m[[4mmaxmode=Integer[0m] Creates a scaled window.
     [1m[0m
     example:
     
        # Creating and displaying a scaled
        # window
        >> win_scaled 0.1 0.9 0.1 0.9 -100.0 10.0 0.0 10.0
        >>

     Note: d means the device co-ordinate, w means the world
     co-ordinate.'.
!

GXwinUtwod comment: 'Controls and creates two dimensional windows for plotting
   
     [1mWIN_TWOD[0m [4mdxmin=Real[0m [4mdxmax=Real[0m [4mdymin=Real[0m [4mdymax=Real[0m [4mx=RealArray[0m
     [1m[0m[4my=RealArray[0m [[4mcolour=Colour[0m] Creates a Two dimensional window.
     [1m[0m
     example:
     
        # Creating and displaying a Twod window,
        # where yarray and xarray have already
        # been defined
        >> win_twod 0.1 0.9 0.1 0.9 y=yarray x=xarray

     Note: d means the device co-ordinate. Giving the x/yarray the
     window is scaled automatically.'.
!

GXwinUunscaled comment: 'Controls and creates unscaled windows for plotting.
   
     [1mWIN_UNSCALED[0m [4mxmin=Real[0m [4mxmax=Real[0m [4mymin=Real[0m [4mymax=Real[0m
     [1m[0m[[4mcolour=Colour[0m] Sets the device co-ordinates and creates an
     [1m[0munscaled window.
     [1m[0m
     example:
     
        # Creating and displaying an
        # unscaled window
        >> win_unscaled 0.1 0.9 0.1 0.9

     Note: d means the device co-ordinate.'.
!

GXwindow comment: 'Obsolete command
   
     [1mWINDOW/AUTOSCALE[0m   Autoscaled window, use WIN_AUTOSCALED
     [1mWINDOW/MULTI_PLOT[0m   multiplot window, use WIN_MULTIPLOT
     [1mWINDOW/SCALED[0m   Scaled window, use WIN_SCALED
     [1mWINDOW/TWOD[0m   2-D window, use WIN_TWOD
     [1mWINDOW/UNSCALED[0m   Unscaled window, use WIN_UNSCALED'.
!

GXworkspaceUabs comment: 'Called by the generic function Abs(x) when x is of type Workspace.
   
     [1mWORKSPACE_ABS()[0m [4mw1=Workspace[0m |w|'.
!

GXworkspaceUadd comment: 'Called by the generic function x + y when x and y are of type
   Workspace.
   [1m[0m
     [1mWORKSPACE_ADD()[0m [4mw1=Workspace[0m [4mw2=Workspace[0m w1 + w2'.
!

GXworkspaceUand comment: 'Called by the generic function x AND y when x and y are of type
   Workspace.
   [1m[0m
     [1mWORKSPACE_AND()[0m [4mw1=Workspace[0m [4mw2=Workspace[0m w1 AND w2'.
!

GXworkspaceUappend comment: 'Called by the generic function x & y (append) when x and y are of type
   Workspace.
   [1m[0m
     [1mWORKSPACE_APPEND()[0m [4mw1=Workspace[0m [4mw2=Workspace[0m w1 & w2'.
!

GXworkspaceUarccos comment: 'Called by the generic function Acos(x) when x is of type Workspace.
   
     [1mWORKSPACE_ARCCOS()[0m [4mw1=Workspace[0m acos(w)'.
!

GXworkspaceUarcsin comment: 'Called by the generic function Asin(x) when x is of type Workspace.
   
     [1mWORKSPACE_ARCSIN()[0m [4mw1=Workspace[0m asin(w)'.
!

GXworkspaceUarctan comment: 'Called by the generic function Atan(x) when x is of type Workspace.
   
     [1mWORKSPACE_ARCTAN()[0m [4mw1=Workspace[0m atan(w)'.
!

GXworkspaceUcoerce comment: 'Called to convert numbers and arrays automatically in to workspaces.
   
     [1mWORKSPACE_COERCE()[0m [4msomething=Any[0m convert "something" into a
     [1m[0mworkspace (if possible)'.
!

GXworkspaceUcos comment: 'Called by the generic function Cos(x) when x is of type Workspace.
   
     [1mWORKSPACE_COS()[0m [4mw1=Workspace[0m cos(w)'.
!

GXworkspaceUdivide comment: 'Called by the generic function x / y when x and y are of type
   Workspace.
   [1m[0m
     [1mWORKSPACE_DIVIDE()[0m [4mw1=Workspace[0m [4mw2=Workspace[0m w1 / w2'.
!

GXworkspaceUequal comment: 'Called by the generic function x = y when x and y are of type
   Workspace.
   [1m[0m
     [1mWORKSPACE_EQUAL()[0m [4mw1=Workspace[0m [4mw2=Workspace[0m w1 [4m=[0m w2'.
!

GXworkspaceUexp comment: 'Called by the generic function Exp(x) when x is of type Workspace.
   
     [1mWORKSPACE_EXP()[0m [4mw1=Workspace[0m exp(w)'.
!

GXworkspaceUgreaterUthan comment: 'Called by the generic function x > y when x and y are of type
   Workspace.
   [1m[0m
     [1mWORKSPACE_GREATER_THAN()[0m [4mw1=Workspace[0m [4mw2=Workspace[0m w1 > w2'.
!

GXworkspaceUgreaterUthanUorUequal comment: 'Called by the generic function x >= y when x and y are of type
   Workspace.
   [1m[0m
     [1mWORKSPACE_GREATER_THAN_OR_EQUAL()[0m [4mw1=Workspace[0m [4mw2=Workspace[0m w1 >[4m=[0m
     [1m[0mw2'.
!

GXworkspaceUlessUthan comment: 'Called by the generic function x < y when x and y are of type
   Workspace.
   [1m[0m
     [1mWORKSPACE_LESS_THAN()[0m [4mw1=Workspace[0m [4mw2=Workspace[0m w1 < w2'.
!

GXworkspaceUlessUthanUorUequal comment: 'Called by the generic function x <= y when x and y are of type
   Workspace.
   [1m[0m
     [1mWORKSPACE_LESS_THAN_OR_EQUAL()[0m [4mw1=Workspace[0m [4mw2=Workspace[0m w1 <[4m=[0m w2'.
!

GXworkspaceUln comment: 'Called by the generic function ln(x) when x is of type Workspace.
   
     [1mWORKSPACE_LN()[0m [4mw1=Workspace[0m ln(w)'.
!

GXworkspaceUlog comment: 'Called by the generic function log(x) when x is of type Workspace.
   
     [1mWORKSPACE_LOG()[0m [4mw1=Workspace[0m log(w)'.
!

GXworkspaceUmodulo comment: 'Called by the generic function x | y when x and y are of type
   Workspace.
   [1m[0m
     [1mWORKSPACE_MODULO()[0m [4mw1=Workspace[0m [4mw2=Workspace[0m w1 | w2'.
!

GXworkspaceUmultiply comment: 'Called by the generic function x * y when x and y are of type
   Workspace.
   [1m[0m
     [1mWORKSPACE_MULTIPLY()[0m [4mw1=Workspace[0m [4mw2=Workspace[0m w1 * w2'.
!

GXworkspaceUnegated comment: 'Called by the generic function -x when x is of type Workspace.
   
     [1mWORKSPACE_NEGATED()[0m [4mw1=Workspace[0m -w'.
!

GXworkspaceUnot comment: 'Called by the generic function NOT x when x is of type Workspace.
   
     [1mWORKSPACE_NOT()[0m [4mw1=Workspace[0m NOT w'.
!

GXworkspaceUnotUequal comment: 'Called by the generic function x != y when x and y are of type
   Workspace.
   [1m[0m
     [1mWORKSPACE_NOT_EQUAL()[0m [4mw1=Workspace[0m [4mw2=Workspace[0m w1 ![4m=[0m w2'.
!

GXworkspaceUor comment: 'Called by the generic function x OR y when x and y are of type
   Workspace.
   [1m[0m
     [1mWORKSPACE_OR()[0m [4mw1=Workspace[0m [4mw2=Workspace[0m w1 OR w2'.
!

GXworkspaceUraisedUto comment: 'Called by the generic function x ^ y when x and y are of type
   Workspace.
   [1m[0m
     [1mWORKSPACE_RAISED_TO()[0m [4mw1=Workspace[0m [4mw2=Workspace[0m w1 ^ w2'.
!

GXworkspaceUsin comment: 'Called by the generic function Sin(x) when x is of type Workspace.
   
     [1mWORKSPACE_SIN()[0m [4mw1=Workspace[0m sin(w)'.
!

GXworkspaceUsqrt comment: 'Called by the generic function Sqrt(x) when x is of type Workspace.
   
     [1mWORKSPACE_SQRT()[0m [4mw1=Workspace[0m sqrt(w)'.
!

GXworkspaceUsubtract comment: 'Called by the generic function x - y when x and y are of type
   Workspace.
   [1m[0m
     [1mWORKSPACE_SUBTRACT()[0m [4mw1=Workspace[0m [4mw2=Workspace[0m w1 - w2'.
!

GXworkspaceUtan comment: 'Called by the generic function Tan(x) when x is of type Workspace.
   
     [1mWORKSPACE_TAN()[0m [4mw1=Workspace[0m tan(w)'.
!

GXwrite comment: 'Sorry, no help is available for this command'
!

GXzoom comment: 'Interactively choose an area of a plot to magnify.
   
     [1mZOOM[0m Redraw a plot scaling to the cursor selected box.
     [1m[0m[/ERRORS] Redraw using error bars
     [1m[0m[/HISTOGRAM] Redraw as a histogram
     [1m[0m[/LINE] Redraw as a line plot
     [1m[0m[/MARKERS] Redraw as a marker plot
     [1m[0m
     example:
     
        # Zoom in on the current plot
        # and re-display with error bars
        >> zoom/errors

     Note: A zoom can be undone by a Limits/Default command with no
     arguments'.
!
GXUUinitgraphics run line: 0 proc: self method: false; condEvaluate!
GXUUinitio run line: 0 proc: self method: false; condEvaluate!
