C   Open GENIE, a program for neutron scattering data analysis & display
C   ISIS facility, Rutherford Appleton Laboratory, Chilton, Didcot, OXON UK
C   Copyright (C) 1996,1997
C
C   This program is free software; you can redistribute it and/or modify
C   it under the terms of the GNU General Public License as published by
C   the Free Software Foundation; either version 2 of the License, or
C   (at your option) any later version.
C
C   This program is distributed in the hope that it will be useful,
C   but WITHOUT ANY WARRANTY; without even the implied warranty of
C   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
C   GNU General Public License for more details.
C
C   You should have received a copy of the GNU General Public License
C   along with this program; if not, write to the Free Software
C   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
C
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C
      SUBROUTINE FNLORZ(X,Y,N,PARS)
C     -----------------------------
C
C+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
C
C Purpose
C      Calculates the Y-values for a Lorentzian-peak sitting on a 
C    straight-line background for a given set of X-points:
C
C       y(x) = Mx + C + A*lorentz(X0,GAMMA) ,
C
C                                          GAMMA  
C    where:  lorentz(X0,GAMMA) = ---------------------------  .
C                                pi [ (x-X0)**2 + GAMMA**2 ]
C
C    PARS(1) = M ;  PARS(2) = C  ;
C    PARS(3) = A ;  PARS(4) = X0 ; PARS(5) = GAMMA .
C
C Parameters
C   ARGUMENT  TYPE  I/O  DIMENSION  DESCRIPTION
C    N        I*4    I       -      Number of points.
C    X        R*4    I       N      Array of X-coordinates.
C    Y        R*4    O       N      Array of Y-coordinates.
C    PARS     R*4    I       5      Functional parameters.
C
C History
C   D. S. Sivia      28 Nov 1995  Initial release.
C-----------------------------------------------------------------------
C
      REAL X(*),Y(*),PARS(*)
      DATA SMALL,PI /1.0E-20,3.141592654/
C
      XM=PARS(1)
      C=PARS(2)
      A=PARS(3)
      X0=PARS(4)
      GAM=MAX(ABS(PARS(5)),SMALL)
      A=A*GAM/PI
      GAM2=GAM**2
      DO 10 I=1,N
  10    Y(I)=XM*X(I)+C+A/((X(I)-X0)**2+GAM2)
      END
C
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
