C   Open GENIE, a program for neutron scattering data analysis & display
C   ISIS facility, Rutherford Appleton Laboratory, Chilton, Didcot, OXON UK
C   Copyright (C) 1996,1997
C
C   This program is free software; you can redistribute it and/or modify
C   it under the terms of the GNU General Public License as published by
C   the Free Software Foundation; either version 2 of the License, or
C   (at your option) any later version.
C
C   This program is distributed in the hope that it will be useful,
C   but WITHOUT ANY WARRANTY; without even the implied warranty of
C   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
C   GNU General Public License for more details.
C
C   You should have received a copy of the GNU General Public License
C   along with this program; if not, write to the Free Software
C   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
C
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C
      SUBROUTINE PKGEXP(X,Y,E,N,XW,YW,EW,PARS,SIGPAR,IPROPT,IGOOD)
C     ------------------------------------------------------------
C
C+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
C
C Purpose
C      Estimates the parameters of a peak, consisting of a Gaussian 
C    convolved with a sharp-edged exponential, sitting on a straight-
C    line background, given a pertinent set of data:
C
C       y(x) = Mx + C + A*gsexp(X0,SIG,TAU) ,
C
C
C                                       1           [ - (x-X0)**2 ]  
C    where:  gsexp(X0,SIG,TAU) =  -------------- exp|-------------| 
C                                 SIG*sqrt(2*pi)    [  2 * SIG**2 ]
C
C
C                                   {        0           if x/TAU < 0
C                  Convolved with   {                                  .
C                                   { exp(-x/TAU)/|TAU|  if x/TAU > 0 
C
C
C    PARS(1) = M  ;  PARS(2) = C   ;  PARS(3) = A ;  
C    PARS(4) = X0 ;  PARS(5) = SIG ;  PARS(6) = TAU .
C
C Parameters
C   ARGUMENT  TYPE  I/O  DIMENSION  DESCRIPTION
C    N        I*4    I       -      Number of data.
C    X        R*4    I       N      X-coordinate of data.
C    Y        R*4    I       N      Y-coordinate of data.
C    E        R*4    I       N      Error-bar for data.
C    XW,YW,EW R*4    -       N      Work arrays.
C    PARS     R*4    I       6      Parameter-values that are to be 
C                                   fixed or used for the initial guess.
C    PARS     R*4    O       6      Best estimates of the parameters.
C    SIGPAR   R*4    O       6      1-sigma error-bars for PARS; can be
C                                   negative if IGOOD < 0!
C    IPROPT   I*4    I       6      Option for treatment of parameters:
C                                     0 = no initial estimate;
C                                     1 = use as "      "    ;
C                                     2 = fix at "      "    .
C    IGOOD    I*4    O       -      Negative if a problem occurs:
C                                     < -1 = serious; -1 = may be OK.
C
C Globals
C    PKCOM1
C    PKCOM2
C
C External Calls
C   SUBROUTINE   DESCRIPTION
C     PKINIT     Gives crude non-parametric estimate of peak properties.
C     CHGEXP     Calculates "chi-squared" misfit.
C
C History
C   D. S. Sivia      24 Nov 1995  Initial release.
C-----------------------------------------------------------------------
C
      EXTERNAL        CHGEXP
      REAL            CHGEXP
      REAL            X(*),Y(*),E(*),XW(*),YW(*),EW(*),PARS(*),SIGPAR(*)
      INTEGER         IPROPT(*)
      COMMON /PKCOM1/ XSCALE,XMIDL,YSCALE
      INCLUDE 'pkdefs.inc'
      DATA            SMALL /1.0E-20/
C
      IGOOD=-10
      NPARS=0
      DO 10 I=1,6
        IF (IPROPT(I).EQ.2) THEN
          PKPARS(I)=PARS(I)
          SIGPAR(I)=0.0
        ELSE
   1      NPARS=NPARS+1
          IPARS(NPARS)=I
          IF (IPROPT(I).EQ.1) PKPARS(I)=PARS(I)
        ENDIF
  10  CONTINUE
      IF (NPARS.EQ.0 .OR. N.LE.NPARS) RETURN
      CALL PKINIT(X,Y,E,N,XW,YW,EW,PARS(1),PARS(2),XYMAX,XAVG,PARS(3),
     *            VAR,IGOOD)
      IF (IGOOD.LT.-2) THEN
        CALL VCOPY(PKPARS,PARS,6)
        RETURN
      ENDIF
      PARS(4)=XYMAX
      PARS(5)=0.5*SQRT(ABS(VAR)+SMALL)
      PARS(6)=1.73205*PARS(5)
      IF (VAR.LT.SMALL) PARS(5)=-PARS(5)
      IF (XAVG.LT.XYMAX) PARS(6)=-PARS(6)
      DO 20 I=1,NPARS
        J=IPARS(I)
        IF (IPROPT(J).EQ.0) PKPARS(J)=PARS(J)
  20  CONTINUE
      IF (PKPARS(3).LT.SMALL .OR. PKPARS(5).LE.SMALL) THEN
        CALL VCOPY(PKPARS,PARS,6)
        PARS(3)=5.0*SMALL
        PARS(5)=0.1*ABS(X(N)-X(1))
        PARS(6)=PARS(5)
        RETURN
      ENDIF
      XSCL1=1.0/XSCALE
      YSCL1=1.0/YSCALE
      PKPARS(2)=(PKPARS(1)*XMIDL+PKPARS(2))*YSCALE
      PKPARS(1)=PKPARS(1)*YSCALE*XSCL1
      PKPARS(3)=PKPARS(3)*XSCALE*YSCALE
      PKPARS(4)=(PKPARS(4)-XMIDL)*XSCALE
      PKPARS(5)=PKPARS(5)*XSCALE
      PKPARS(6)=PKPARS(6)*XSCALE
      DPAR(1)=0.1*((MIN(ABS(PKPARS(1)),1.0E10))**2+1.0)
      DPAR(2)=0.1*MAX(ABS(PKPARS(2)),MAX(SMALL,E(2),E(N-1)))
      DPAR(3)=0.1*PKPARS(3)
      DPAR(4)=0.1*SQRT(PKPARS(5)**2+PKPARS(6)**2)
      DPAR(5)=0.15*PKPARS(5)
      DPAR(6)=0.15*PKPARS(6)
      DO 30 I=1,NPARS
        J=IPARS(I)
        PARS(I)=PKPARS(J)
        DPAR(I)=DPAR(J)
  30  CONTINUE
      IGOOD=ISMPLX(PARS,DPAR,COVAR,NPARS,.FALSE.,CHGEXP,XW,YW,EW,N)
      DO 40 I=1,NPARS
        J=IPARS(I)
        PKPARS(J)=PARS(I)
        SIG2=COVAR((I-1)*NPARS+I)
        SIGPAR(J)=SQRT(ABS(SIG2)+SMALL)
        IF (SIG2.LT.SMALL) THEN
          SIGPAR(J)=-SIGPAR(J)
          IGOOD=-1
        ENDIF
  40  CONTINUE
      PARS(1)=PKPARS(1)*XSCALE*YSCL1
      PARS(2)=PKPARS(2)*YSCL1-PARS(1)*XMIDL
      PARS(3)=PKPARS(3)*XSCL1*YSCL1
      PARS(4)=PKPARS(4)*XSCL1+XMIDL
      PARS(5)=PKPARS(5)*XSCL1
      PARS(6)=PKPARS(6)*XSCL1
      SIGPAR(1)=SIGPAR(1)*XSCALE*YSCL1
      IF (IPROPT(2).NE.2) THEN
        IF (IPROPT(1).EQ.2) THEN
          SIGPAR(2)=SIGPAR(2)*YSCL1
        ELSE
          XSM=XSCALE*XMIDL
          SIGC2=COVAR(NPARS+2)+COVAR(1)*XSM**2-2.0*XSM*COVAR(2)
          SIGPAR(2)=SQRT(ABS(SIGC2)+SMALL)*YSCL1
          IF (SIGC2.LT.SMALL) THEN
            SIGPAR(2)=-SIGPAR(2)
            IGOOD=-1
          ENDIF
        ENDIF
      ENDIF
      SIGPAR(3)=SIGPAR(3)*XSCL1*YSCL1
      SIGPAR(4)=SIGPAR(4)*XSCL1
      SIGPAR(5)=SIGPAR(5)*XSCL1
      SIGPAR(6)=SIGPAR(6)*XSCL1
      END
C
      FUNCTION CHGEXP(PARS,NPARS,X,D,EW,N)
C     ------------------------------------
C
      EXTERNAL        GSEXP
      REAL            PARS(*),X(*),D(*),EW(*)
      INCLUDE 'pkdefs.inc'
      DATA            SMALL /1.0E-20/
C
      CHGEXP=1.0E10
      DO 10 I=1,NPARS
  10    PKPARS(IPARS(I))=PARS(I)
      XM=PKPARS(1)
      C=PKPARS(2)
      A=PKPARS(3)
      X0=PKPARS(4)
      SIG=PKPARS(5)
      TAU=MAX(ABS(PKPARS(6)),SMALL)
      TSIGN=1.0
      IF (PKPARS(6).LT.0.0) TSIGN=-1.0 
      IF (A.LE.SMALL .OR. SIG.LE.SMALL) RETURN
      CHI=0.0
      DO 20 I=1,N
        Y=XM*X(I)+C+A*GSEXP(TSIGN*(X(I)-X0),SIG,TAU)
        CHI=CHI+((Y-D(I))*EW(I))**2
  20  CONTINUE
      CHGEXP=CHI
      END
C
C***<Bill David's routines (renamed)>***********************************
C
	FUNCTION GSEXP(xdel,sigma,tau)
C       ------------------------------
c
c..  Author:  W.I.F.David 	Date:  17-MAY-1985
c..  The normalised KROP function otherwise known as the GEC function
c..  is a convolution of the Gaussian and exponential functions.
C
	bet= 1./tau
	delsig= xdel/sigma
	betexp= bet*exp(-0.5*delsig*delsig)
	bs= bet*sigma
	ztem= 0.707106781*(bs-delsig)
	betdel= bet*xdel
	bssq= bs*bs
	arg= 0.5*bssq-betdel
	if (ztem.lt.0.0) then
 	  GSEXP= 0.5*bet*exp(arg)*ZGEXP2(ztem)
        else
          GSEXP= 0.5*betexp*ZGEXP1(ztem)
        endif
	end
C
	function ZGEXP1(y)
C       ------------------
C
c	W.I.F.David 6-June-84
c..	gives	exp(y*y)erfc(y) for +ve y only
C
	implicit real*8		(a-h,o-v)
	real*8			lambda
	logical			b
C
	vy=dble(y)
	if (vy .lt. 4.29 ) go to 1
	h= 0.
	nc= 0
	n= 9
	lambda= 0.
	b= .true.
	go to 2
 1	s=(1.0-vy/4.29)
	h=1.6*s
	h2=2.0*h
	nc=6+idint(23.0*s)
	n=10+idint(21.0*s)
	lambda=h2**nc
	b= .false.
	if (lambda .eq. 0.) b= .true.
 2	r1=0.
	s1=0.
 3	n=n-1
	fn=n+1
	t1=vy+h+fn*r1
	r1=0.5/t1
	if (h .le. 0.0 .or. n .gt. nc) go to 4
	s1=r1*(lambda+s1)
	lambda=lambda/h2
 4	if (n .gt. 0) go to 3
	if (b) go to 6
	vr= s1
	go to 7
 6	vr= r1
 7	vr= 1.12837916709551*vr
	if (vy .eq. 0.0) vr= 1.
	ZGEXP1= vr
	end
C
	function ZGEXP2(x)
C       ------------------
C
c  These coefficients were taken from Abramovitz and Stegun p.299
c  and give erfc accurate to 3E-07 for both +ve and -ve x.
c
	data a1,a2,a3/0.0705230784,0.0422820123,0.0092705272/
	data a4,a5,a6/0.0001520143,0.0002765672,0.0000430638/
C
	z= abs(x)
	zz=z*z
	zzz=z*z*z
	erfc= 1.0 +a1*z +a2*zz +a3*zzz +a4*zz*zz +a5*zz*zzz +a6*zzz*zzz
	if (erfc.ge.10.0) then
	  erfc=0.0
	else
	  erfc=1.0/erfc**16
	end if
	if(erfc.le.0.0) erfc=0.0
	if(x.ge.0.0) erfc=erfc
	if(x.lt.0.0) erfc=2.0-erfc
        ZGEXP2=erfc
	end
C
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
