C   Open GENIE, a program for neutron scattering data analysis & display
C   ISIS facility, Rutherford Appleton Laboratory, Chilton, Didcot, OXON UK
C   Copyright (C) 1996,1997
C
C   This program is free software; you can redistribute it and/or modify
C   it under the terms of the GNU General Public License as published by
C   the Free Software Foundation; either version 2 of the License, or
C   (at your option) any later version.
C
C   This program is distributed in the hope that it will be useful,
C   but WITHOUT ANY WARRANTY; without even the implied warranty of
C   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
C   GNU General Public License for more details.
C
C   You should have received a copy of the GNU General Public License
C   along with this program; if not, write to the Free Software
C   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
C
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C
      SUBROUTINE PKPOLY(X,Y,E,N,XW,YW,EW,PARS,SIGPAR,IPROPT,INPAR,IGOOD)
C     ------------------------------------------------------------
C
C+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
C
C Parameters
C   ARGUMENT  TYPE  I/O  DIMENSION  DESCRIPTION
C    N        I*4    I       -      Number of data.
C    X        R*4    I       N      X-coordinate of data.
C    Y        R*4    I       N      Y-coordinate of data.
C    E        R*4    I       N      Error-bar for data.
C    XW,YW,EW R*4    -       N      Work arrays.
C    PARS     R*4    I       5      Parameter-values that are to be 
C                                   fixed or used for the initial guess.
C    PARS     R*4    O       5      Best estimates of the parameters.
C    SIGPAR   R*4    O       5      1-sigma error-bars for PARS; can be
C                                   negative if IGOOD < 0!
C    IPROPT   I*4    I       5      Option for treatment of parameters:
C                                     0 = no initial estimate;
C                                     1 = use as "      "    ;
C                                     2 = fix at "      "    .
C    IGOOD    I*4    O       -      Negative if a problem occurs:
C                                     < -1 = serious; -1 = may be OK.
C
C Globals
C    PKCOM1
C    PKCOM2
C
C External Calls
C   SUBROUTINE   DESCRIPTION
C     PKINIT     Gives crude non-parametric estimate of peak properties.
C     CHGAUS     Calculates "chi-squared" misfit for Gaussian peak.
C
C History
C   Modifed by F. A. Akeroyd from original by D. S. Sivia
C-----------------------------------------------------------------------
C
      EXTERNAL        CHPOLY
      REAL            CHPOLY
      REAL            X(*),Y(*),E(*),XW(*),YW(*),EW(*),PARS(*),SIGPAR(*)
      INTEGER         IPROPT(*)
      INCLUDE 'pkdefs.inc'
      DATA            SMALL /1.0E-20/
C
      IGOOD=-10
      NPARS=0
      NPKPARS=INPAR
      IF (INPAR .GT. MAXPKPARS) THEN
          WRITE(6,*) 'PEAK: Too many parameters - max allowed is ', MAXPKPARS
          RETURN
      ENDIF
      DO 10 I=1,INPAR
        IF (IPROPT(I).EQ.2) THEN
          PKPARS(I)=PARS(I)
          SIGPAR(I)=0.0
        ELSE
          NPARS=NPARS+1
          IPARS(NPARS)=I
          IF (IPROPT(I).EQ.1) PKPARS(I)=PARS(I)
        ENDIF
  10  CONTINUE
      IF (NPARS.EQ.0 .OR. N.LE.NPARS) THEN
          WRITE(6,*) 'PEAK: Invalid number of parameters: ', npars
          RETURN
      ENDIF
      YMAX = Y(1)
      YMIN = Y(1)
      XAV=0.0
      DO I=1,N
          YMAX=MAX(YMAX,Y(I))
          YMIN=MIN(YMIN,Y(I))
          XAV=XAV+X(I)
      ENDDO
      XAV=XAV/FLOAT(N)
      DO I=1,N
          XW(I) = X(I)
          YW(I) = Y(I)
          IF (E(I) .GT. SMALL) THEN
              EW(I) = 1.0 / E(I)
          ELSE
              EW(I) = 20.0 / YMAX
          ENDIF
      ENDDO
C Estimate PARS
      CALL POLYGUESS(X, Y, E, N, PARS, IPROPT, INPAR, 
     +               YMIN, YMAX, XAV, IGOOD)
      IF (IGOOD.LT.-2) THEN
        CALL VCOPY(PKPARS,PARS,INPAR)
        RETURN
      ENDIF
      DO 20 I=1,NPARS
        J=IPARS(I)
        IF (IPROPT(J).EQ.0) PKPARS(J)=PARS(J)
  20  CONTINUE
      DO I=1,INPAR
          DPAR(I) = MAX(ABS(PKPARS(I)),1.0)
      ENDDO
      DO 30 I=1,NPARS
        J=IPARS(I)
        PARS(I)=PKPARS(J)
        DPAR(I)=DPAR(J)
  30  CONTINUE
      IGOOD=ISMPLX(PARS,DPAR,COVAR,NPARS,.FALSE.,CHPOLY,XW,YW,EW,N)
      DO 40 I=1,NPARS
        J=IPARS(I)
        PKPARS(J)=PARS(I)
        SIG2=COVAR((I-1)*NPARS+I)
        SIGPAR(J)=SQRT(ABS(SIG2)+SMALL)
        IF (SIG2.LT.SMALL) THEN
          SIGPAR(J)=-SIGPAR(J)
          IF (IGOOD .GE. 0) IGOOD=-1
        ENDIF
  40  CONTINUE
      DO I=1,INPAR
          PARS(I)=PKPARS(I)
      ENDDO
      END
C
      REAL FUNCTION CHPOLY(PARS,NPARS,X,D,EW,N)
C     ------------------------------------
C
      REAL      PARS(*),X(*),D(*),EW(*)
      INCLUDE 'pkdefs.inc'
C
      CHPOLY=1.0E10
      DO 10 I=1,NPARS
        PKPARS(IPARS(I))=PARS(I)
  10  CONTINUE
      CHI=0.0
      DO 20 I=1,N
        Y=0.0
        XI=1.0
        DO 30 J=1,NPKPARS
          Y=Y+XI*PKPARS(J)
          XI=XI*X(I)
  30    CONTINUE
        CHI=CHI+((Y-D(I))*EW(I))**2
  20  CONTINUE
      CHPOLY=CHI
      END
C
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
      SUBROUTINE POLYGUESS(X, Y, E, N, PARS, IPROPT, INPAR, YMIN, YMAX, 
     1                     XAV, IGOOD)
      REAL X(*),Y(*),E(*),PARS(*)
      INTEGER IPROPT(*)
      IGOOD = 0
      YDIFF = YMAX - YMIN
      XI=1.0
      DO I=1,INPAR
          IF (IPROPT(I) .EQ. 0) THEN
              PARS(I) = YDIFF / XI
          ENDIF
          XI=XI*XAV
      ENDDO
      END
C
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
