C   Open GENIE, a program for neutron scattering data analysis & display
C   ISIS facility, Rutherford Appleton Laboratory, Chilton, Didcot, OXON UK
C   Copyright (C) 1996,1997
C
C   This program is free software; you can redistribute it and/or modify
C   it under the terms of the GNU General Public License as published by
C   the Free Software Foundation; either version 2 of the License, or
C   (at your option) any later version.
C
C   This program is distributed in the hope that it will be useful,
C   but WITHOUT ANY WARRANTY; without even the implied warranty of
C   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
C   GNU General Public License for more details.
C
C   You should have received a copy of the GNU General Public License
C   along with this program; if not, write to the Free Software
C   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
C
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C
      SUBROUTINE PKVEXP(X,Y,E,N,XW,YW,EW,PARS,SIGPAR,IPROPT,IGOOD)
C     ------------------------------------------------------------
C
C+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
C
C Purpose
C      Estimates the parameters of a peak, consisting of a Voigt
C    convolved with a sharp-edged exponential, sitting on a straight-
C    line background, given a pertinent set of data:
C
C       y(x) = Mx + C + A*vtexp(X0,SIG,GAM,TAU) ,
C
C
C                                       1           [ - (x-X0)**2 ]  
C    where:  vtexp(X0,SIG,TAU) =  -------------- exp|-------------| 
C                                 SIG*sqrt(2*pi)    [  2 * SIG**2 ]
C
C                                                     GAM  
C                           Convolved with    ---------------------  .
C                                              pi [ x**2 + GAM**2 ]
C
C
C                                   {        0           if x/TAU < 0
C                  Convolved with   {                                  .
C                                   { exp(-x/TAU)/|TAU|  if x/TAU > 0 
C
C
C    PARS(1) = M  ;  PARS(2) = C   ;  PARS(3) = A ;  
C    PARS(4) = X0 ;  PARS(5) = SIG ;  PARS(6) = GAM ;  PARS(7) = TAU .
C
C Parameters
C   ARGUMENT  TYPE  I/O  DIMENSION  DESCRIPTION
C    N        I*4    I       -      Number of data.
C    X        R*4    I       N      X-coordinate of data.
C    Y        R*4    I       N      Y-coordinate of data.
C    E        R*4    I       N      Error-bar for data.
C    XW,YW,EW R*4    -       N      Work arrays.
C    PARS     R*4    I       7      Parameter-values that are to be 
C                                   fixed or used for the initial guess.
C    PARS     R*4    O       7      Best estimates of the parameters.
C    SIGPAR   R*4    O       7      1-sigma error-bars for PARS; can be
C                                   negative if IGOOD < 0!
C    IPROPT   I*4    I       7      Option for treatment of parameters:
C                                     0 = no initial estimate;
C                                     1 = use as "      "    ;
C                                     2 = fix at "      "    .
C    IGOOD    I*4    O       -      Negative if a problem occurs:
C                                     < -1 = serious; -1 = may be OK.
C
C Globals
C    PKCOM1
C    PKCOM2
C
C External Calls
C   SUBROUTINE   DESCRIPTION
C     PKINIT     Gives crude non-parametric estimate of peak properties.
C     CHVEXP     Calculates "chi-squared" misfit.
C
C History
C   D. S. Sivia      30 Jan 1996  Initial release.
C-----------------------------------------------------------------------
C
      EXTERNAL        CHVEXP
      REAL            X(*),Y(*),E(*),XW(*),YW(*),EW(*),PARS(*),SIGPAR(*)
      REAL 	      CHVEXP
      INTEGER         IPROPT(*)
      COMMON /PKCOM1/ XSCALE,XMIDL,YSCALE
      INCLUDE 'pkdefs.inc'
      DATA            SMALL /1.0E-20/
C
      IGOOD=-10
      NPARS=0
      DO 10 I=1,7
        IF (IPROPT(I).EQ.2) THEN
          PKPARS(I)=PARS(I)
          SIGPAR(I)=0.0
        ELSE
   1      NPARS=NPARS+1
          IPARS(NPARS)=I
          IF (IPROPT(I).EQ.1) PKPARS(I)=PARS(I)
        ENDIF
  10  CONTINUE
      IF (NPARS.EQ.0 .OR. N.LE.NPARS) RETURN
      CALL PKINIT(X,Y,E,N,XW,YW,EW,PARS(1),PARS(2),XYMAX,XAVG,PARS(3),
     *            VAR,IGOOD)
      IF (IGOOD.LT.-2) THEN
        CALL VCOPY(PKPARS,PARS,7)
        RETURN
      ENDIF
      PARS(4)=XYMAX
      PARS(5)=0.35*SQRT(ABS(VAR)+SMALL)
      PARS(6)=PARS(5)
      PARS(7)=1.73205*PARS(5)*1.414
      IF (VAR.LT.SMALL) THEN
        PARS(5)=-PARS(5)
        PARS(6)=-PARS(6)
      ENDIF
      IF (XAVG.LT.XYMAX) PARS(7)=-PARS(7)
      DO 20 I=1,NPARS
        J=IPARS(I)
        IF (IPROPT(J).EQ.0) PKPARS(J)=PARS(J)
  20  CONTINUE
      IF (PKPARS(3).LT.SMALL .OR. PKPARS(5).LE.SMALL 
     *                       .OR. PKPARS(6).LE.SMALL) THEN
        CALL VCOPY(PKPARS,PARS,7)
        PARS(3)=5.0*SMALL
        PARS(5)=0.1*ABS(X(N)-X(1))
        PARS(6)=PARS(5)
        PARS(7)=PARS(5)
        RETURN
      ENDIF
      XSCL1=1.0/XSCALE
      YSCL1=1.0/YSCALE
      PKPARS(2)=(PKPARS(1)*XMIDL+PKPARS(2))*YSCALE
      PKPARS(1)=PKPARS(1)*YSCALE*XSCL1
      PKPARS(3)=PKPARS(3)*XSCALE*YSCALE
      PKPARS(4)=(PKPARS(4)-XMIDL)*XSCALE
      PKPARS(5)=PKPARS(5)*XSCALE
      PKPARS(6)=PKPARS(6)*XSCALE
      PKPARS(7)=PKPARS(7)*XSCALE
      DPAR(1)=0.1*((MIN(ABS(PKPARS(1)),1.0E10))**2+1.0)
      DPAR(2)=0.1*MAX(ABS(PKPARS(2)),MAX(SMALL,E(2),E(N-1)))
      DPAR(3)=0.1*PKPARS(3)
      DPAR(4)=0.1*SQRT(PKPARS(5)**2+PKPARS(6)**2+PKPARS(7)**2)
      DPAR(5)=0.15*PKPARS(5)
      DPAR(6)=0.15*PKPARS(6)
      DPAR(7)=0.15*PKPARS(7)
      DO 30 I=1,NPARS
        J=IPARS(I)
        PARS(I)=PKPARS(J)
        DPAR(I)=DPAR(J)
  30  CONTINUE
      IGOOD=ISMPLX(PARS,DPAR,COVAR,NPARS,.FALSE.,CHVEXP,XW,YW,EW,N)
      DO 40 I=1,NPARS
        J=IPARS(I)
        PKPARS(J)=PARS(I)
        SIG2=COVAR((I-1)*NPARS+I)
        SIGPAR(J)=SQRT(ABS(SIG2)+SMALL)
        IF (SIG2.LT.SMALL) THEN
          SIGPAR(J)=-SIGPAR(J)
          IGOOD=-1
        ENDIF
  40  CONTINUE
      PARS(1)=PKPARS(1)*XSCALE*YSCL1
      PARS(2)=PKPARS(2)*YSCL1-PARS(1)*XMIDL
      PARS(3)=PKPARS(3)*XSCL1*YSCL1
      PARS(4)=PKPARS(4)*XSCL1+XMIDL
      PARS(5)=PKPARS(5)*XSCL1
      PARS(6)=PKPARS(6)*XSCL1
      PARS(7)=PKPARS(7)*XSCL1
      SIGPAR(1)=SIGPAR(1)*XSCALE*YSCL1
      IF (IPROPT(2).NE.2) THEN
        IF (IPROPT(1).EQ.2) THEN
          SIGPAR(2)=SIGPAR(2)*YSCL1
        ELSE
          XSM=XSCALE*XMIDL
          SIGC2=COVAR(NPARS+2)+COVAR(1)*XSM**2-2.0*XSM*COVAR(2)
          SIGPAR(2)=SQRT(ABS(SIGC2)+SMALL)*YSCL1
          IF (SIGC2.LT.SMALL) THEN
            SIGPAR(2)=-SIGPAR(2)
            IGOOD=-1
          ENDIF
        ENDIF
      ENDIF
      SIGPAR(3)=SIGPAR(3)*XSCL1*YSCL1
      SIGPAR(4)=SIGPAR(4)*XSCL1
      SIGPAR(5)=SIGPAR(5)*XSCL1
      SIGPAR(6)=SIGPAR(6)*XSCL1
      SIGPAR(7)=SIGPAR(7)*XSCL1
      END
C
      FUNCTION CHVEXP(PARS,NPARS,X,D,EW,N)
C     ------------------------------------
C
      EXTERNAL        VTEXP
      REAL            PARS(*),X(*),D(*),EW(*)
      INCLUDE 'pkdefs.inc'
      DATA            SMALL /1.0E-20/
C
      CHVEXP=1.0E10
      DO 10 I=1,NPARS
  10    PKPARS(IPARS(I))=PARS(I)
      XM=PKPARS(1)
      C=PKPARS(2)
      A=PKPARS(3)
      X0=PKPARS(4)
      SIG=PKPARS(5)
      GAM=2.0*PKPARS(6)
      TAU=MAX(ABS(PKPARS(7)),SMALL)
      TSIGN=1.0
      IF (PKPARS(7).LT.0.0) TSIGN=-1.0 
      IF (A.LE.SMALL .OR. SIG.LE.SMALL.OR. GAM.LE.SMALL) RETURN
      CHI=0.0
      DO 20 I=1,N
        Y=XM*X(I)+C+A*VTEXP(TSIGN*(X(I)-X0),SIG,GAM,TAU)
        CHI=CHI+((Y-D(I))*EW(I))**2
  20  CONTINUE
      CHVEXP=CHI
      END
C
C***<Bill David's routines (renamed)>***********************************
C
	FUNCTION VTEXP(x,sig,fwhml,tau)
C       -------------------------------
C
c..	This function corresponds to a normalised convolution of
c	the exponential function     (1/tau)*exp(-x/tau)  ,
c	the Gaussian function  (1/2pi*sig)*exp(0.5{x/sig}**2) and 
c	the Lorentzian function  (fwhm/2pi)/[(fwhm/2)**2 + x**2].
c	The formulation uses the fact that a linear combination
c	of Gaussian and Lorentzian functions (pseudo-Voigt) is an
c	excellent approximation to the convolution of these two
c	functions (Voigt).
c	W.I.F.David		23-Jan-86
	data	con1/2.35482005/,con2/0.4246609/
c
c	convert from second moment of Gaussian to FWHM
	fwhmg= con1*sig
	fwhm= fwhmg+fwhml
	fg= fwhmg/fwhm
	fg2= fg*fg
	fg3= fg*fg2
	fg4= fg*fg3
	fl= 1.-fg
	fl2= fl*fl
	fl3= fl*fl2
	fl4= fl*fl3
	sumhl= (1.-0.9972540*fg+1.1459370*fg2+2.5615029*fg3
     +	    -6.5208845*fg4+5.8264747*fg2*fg3-1.9108598*fg3*fg3)
	pvgaml= fwhm*sumhl
	sumhg= (1.-0.5073386*fl-0.2274422*fl2+1.6380388*fl3
     +	    -2.2853246*fl4+1.3194250*fl2*fl3)
	pvgamg= fwhm*sumhg
	al= 1.3524760*fl+0.4116874*fl2-2.1873136*fl3+6.4245215*fl4
     +	   -10.2903624*fl*fl4+6.8809338*fl2*fl4-1.5919422*fl3*fl4
	ag= 0.0026800*fg+0.7545807*fg2+2.8889816*fg3-3.8514416*fg4
     +	    -0.5576528*fg*fg4+3.0382416*fg2*fg4-1.2753890*fg3*fg4
c
c	convert back from FWHM to 2nd moment for Gaussian component
	pvsig= con2*pvgamg
c
	call VEXP1(x,pvsig,tau,yg)
	call VEXP2(x,pvgaml,tau,yl)
	VTEXP= ag*yg + al*yl
	end
c
	subroutine vexp1(x,sig,tau,ynorm)
C       ---------------------------------
C
c..	This function correspond to a normalised convolution of
c	exponential exp(-x/tau) and gaussian exp[-0.5*(x/sig)**2).
c
c.. Coding for the KROP function
	bet= 1./tau
	xsig= x/sig
	betexp= bet*exp(-0.5*xsig*xsig)
	bs= bet*sig
	ztem= 0.707106781*(bs-xsig)
	betx= bet*x
	bssq= bs*bs
	arg= 0.5*bssq-betx
	if (ztem) 2,1,1
 1	ynorm= 0.5*betexp*VEXP1A(ztem)
	return
 2	ynorm= 0.5*bet*exp(arg)*VEXP1B(ztem)
	end
C
	function VEXP1A(y)
C       ------------------
C
c	W.I.F.David 6-June-84
c..	gives	exp(y*y)erfc(y) for +ve y only
C
	implicit real*8		(a-h,o-v)
	real*8			lambda
        real*4                  vexp1a
	logical			b
C
	vy=dble(y)
	if (vy .lt. 4.29 ) go to 1
	h= 0.
	nc= 0
	n= 9
	lambda= 0.
	b= .true.
	go to 2
 1	s=(1.0-vy/4.29)
	h=1.6*s
	h2=2.0*h
	nc=6+idint(23.0*s)
	n=10+idint(21.0*s)
	lambda=h2**nc
	b= .false.
	if (lambda .eq. 0.) b= .true.
 2	r1=0.
	s1=0.
 3	n=n-1
	fn=n+1
	t1=vy+h+fn*r1
	r1=0.5/t1
	if (h .le. 0.0 .or. n .gt. nc) go to 4
	s1=r1*(lambda+s1)
	lambda=lambda/h2
 4	if (n .gt. 0) go to 3
	if (b) go to 6
	vr= s1
	go to 7
 6	vr= r1
 7	vr= 1.12837916709551*vr
	if (vy .eq. 0.0) vr= 1.
	VEXP1A= vr
	end
C
	function VEXP1B(x)
C       ------------------
C
c  These coefficients were taken from Abramovitz and Stegun p.299
c  and give erfc accurate to 3E-07 for both +ve and -ve x.
c
	data a1,a2,a3/0.0705230784,0.0422820123,0.0092705272/
	data a4,a5,a6/0.0001520143,0.0002765672,0.0000430638/
C
	z= abs(x)
	zz=z*z
	zzz=z*z*z
	erfc= 1.0 +a1*z +a2*zz +a3*zzz +a4*zz*zz +a5*zz*zzz +a6*zzz*zzz
	if (erfc.ge.10.0) then
	  erfc=0.0
	else
	  erfc=1.0/erfc**16
	end if
	if(erfc.le.0.0) erfc=0.0
	if(x.ge.0.0) erfc=erfc
	if(x.lt.0.0) erfc=2.0-erfc
        VEXP1B=erfc
	end
c
	subroutine VEXP2(x,fwhm,tau,ynorm)
C       ---------------------------------
C
c..	This function corresponds to a normalised convolution of
c	the exponential function     (1/tau)*exp(-x/tau)    and 
c	the Lorentzian function	  (fwhm/2pi)/[(fwhm/2)**2 + x**2]
c..	W.I.F.David		23-Jan-86
c
	beta= 1./tau
	ax= -x*beta
	ay= 0.5*fwhm*beta
	axy= ax*ax+ay*ay
	anorm=  0.31830988*beta
	btem= beta*anorm
	call VEXP2A(ax,ay,eimag)
	ynorm= -anorm*eimag
	end
c
	subroutine VEXP2A(ax,ay,ansy)
C       -----------------------------
c
c..	ansx and ansy are the real and imaginary components
c..	of the function   exp(z)E(z)       where   E(z)  is
c..	the exponential integral function.  The complex
c..	argument z  equals   ax + i ay.
c..	W.I.F.David	1-SEP-84
c
	implicit complex*8	(b-h,o-z)
c
	z= cmplx(ax,ay)
	aych= -1.
	if (ax .lt. 0.) aych= 0.125*ax+4.
        a2= ax**2+ay**2
	if (a2 .lt. 16. .or. abs(ay) .lt. aych) then
          a= sqrt(a2)
          go to 50
        endif
	if (ax .lt. 0.) then
          a=sqrt(0.111111*(ax+29.)*(ax+29.)+ay*ay)
        else
          a= sqrt(a2)
        endif
	iter= 4 + nint(128./a)
	tem= float(iter)
	add= tem
	do 10 i=1,iter-1
	tem= tem - 1.
	sum= z + add
	rat= tem/sum
 10	add= tem/(1.+rat)
	zans= 1./(z+add)
	go to 999
c
 50	iter= 10 + 4*nint(a)
	euler= 0.5772156649
	cln= clog(z)
c
	tem= float(iter)*z/float((iter+1)*(iter+1))
	sum= tem
	do 40 j= iter,2,-1
	t= 1./float(j)
	sub= 1.-sum
	tem= tem*(1.+t)*(1-t*t)
 40	sum= tem*sub
	sum= -euler-cln+z*(1.-sum)
	zans= sum*cexp(z)
c
 999    ansy= aimag(zans)
	end
C
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
